/* config.h - Motorola MVME1600/1300 and ULTRA board configuration header */

/* Copyright 1984-1996 Wind River Systems, Inc. */

/*
modification history
--------------------
02d,21feb97,mas  defined PCI_DEV_IDSEL_MAX as 6 (SPR 8026).
02c,12nov96,tam  added DEFAULT_POWER_MGT_MODE to select the default power
		 management mode.
02b,30oct96,tam  set SM_MEM_ADRS and SM_OBJ_MEM_ADRS back below RAM_LOW_ADRS;
		 changed RAM_LOW_ADRS to 0x30000 for board with VME (spr #7416)
02a,22oct96,dat  added #undef INCLUDE_TIMESTAMP, ppcDecTimer has timestamp
		 removed #include STANDALONE_NET
01z,15oct96,tam  set DC_POOL_ADRS, IF_USR_ARG4, SM_MEM_ADRS and SM_OBJ_MEM_ADRS
		 to NONE and changed RAM_LOW_ADRS to 0x10000, SPR #7347.
01y,02oct96,tam  Set Rev to 4.
01x,15sep96,tam  added support for flash memory devices.
01w,08aug96,dat  merged with mv1300. Set Rev to 3.
01v,31jul96,dat  added USER_RESERVED_MEM, see sysMemTop()
01u,30jul96,jds  added USER_D_CACHE_MODE and fixed SCSI-2 cacheing
01t,29jul96,tam  changed ROM_BASE_ADRS from ff000000 to fff00000 (spr 6935).
01s,18jul96,tam	 changed BSP_REV to /2. removed macro CPU_SPEED
01r,21apr96,jds  modified for SCSI-2 support and changed BSP_REV to /1
01q,06mar96,tpr  added #define INCLUDE_Z8536_AUXCLK.
01p,20feb96,tpr  cleaned up following PPC603 - PPC604 split.
01o,20oct95,kvk  rearranged the timer clocks.
01n,20oct95,vin	 added VME_A24_SLV_SIZE.
01m,10oct95,tpr  removed INCLUDE_NFS, INCLUDE_NFS_MOUNT_ALL, INCLUDE_DOSFS,
		 INCLUDE_RAMDRV, INCLUDE_POSIX_TIMERS.
01l,09oct95,kvk  added macros to define the amount of memory the MPC 105
		 memory controller has to handle.
01k,28jun95,caf  increased RAM_HIGH_ADRS, modified ROM_TEXT_ADRS.
01j,22jun95,kvk  further tuning of the memory map, cleanup.
01i,16jun95,kvk  fine tuned the memory map.
01h,24may95,caf  enabled the data cache.
01g,02may95,caf  include floating point I/O and (for now) NFS.
01f,28apr95,kvk  defined a new macro for specifying LOCAL_MEM_MAX.
01e,05apr95,caf  included ping, for now.
01d,31mar95,caf  enabled the instruction cache.
01c,21mar95,caf  cleanup.
01b,15mar95,vin  added DEC21040 and AMD 79c970 ethernet support.
01a,11nov94,vin  written.
*/

/*
 * This file contains the configuration parameters for the
 * Motorola MVME1600/1300 and ULTRA board.
 */

#ifndef	INCconfigh
#define	INCconfigh

/* BSP version/revision identification, before configAll.h */

#define BSP_VER_1_1	1
#define BSP_VERSION	"1.1"
#define BSP_REV		"/4"		/* 0 for first revision */

/* SCSI-2 inclusion macro before configAll.h */

#define  INCLUDE_SCSI2
#include "configAll.h"
#include "mv1600.h"

/* Make adjustments to configAll.h defaults here */

/* select (define) or unselect (undef) network drivers */

#define	INCLUDE_DC		/* Dec 21040 driver */
#define	INCLUDE_LN_PCI		/* Lance (PCI) driver */
#define	INCLUDE_SM_NET		/* shared memory backplane */
#define INCLUDE_SM_SEQ_ADDR	/* shared mem auto address setup */
#undef	INCLUDE_EX		/* include Excelan Ethernet interface */
#undef	INCLUDE_ENP		/* include CMC Ethernet interface*/

/* MMU and Cache options */

#define INCLUDE_MMU_BASIC	/* bundled mmu support */
#define USER_D_CACHE_ENABLE	/* Enable data cache, copyback w/snooping */
#define USER_I_CACHE_ENABLE	/* Enable instruction cache */
#undef  USER_D_CACHE_MODE
#define USER_D_CACHE_MODE	(CACHE_COPYBACK | CACHE_SNOOP_ENABLE)
#define INCLUDE_CACHE_L2	/* L2 cache */
#define INCLUDE_CACHE_SUPPORT

/* Optional timestamp support */

#undef INCLUDE_TIMESTAMP	/* to include timestamp driver */

/* SCSI Options, set FALSE to TRUE to turn on SCSI */

#if FALSE
#  define INCLUDE_SCSI		/* include SCSI driver */
#  define INCLUDE_SCSI_BOOT	/* include ability to boot from SCSI */
#  define INCLUDE_DOSFS		/* file system to be used */
#endif	/*FALSE*/

/* default boot line for standalone and rommed images */

#define DEFAULT_BOOT_LINE \
	"dc(0,0)host:/usr/vw/config/" TARGET_DIR "/vxWorks h=90.0.0.3" \
	" e=90.0.0.50 u=target"

/* Other options */

#define INCLUDE_VME		/* Include VME interface */
#define INCLUDE_VMECHIP2	/* Include vmeChip2 driver */
#define INCLUDE_Z8536_AUXCLK
#define INCLUDE_FLASH		/* Include flash device support */

/* Ultra boards have no VME interface */

#ifdef ULTRA
#   undef INCLUDE_VME		/* No VME bus interface */
#   undef INCLUDE_VMECHIP2	/* No VME Chip2 support */
#   undef INCLUDE_SM_NET	/* No VME on ultra boards */
#endif /*ULTRA*/

/* mv1300 can't use L2 cache and has no aux clock */

#ifdef MV1300
#   undef  INCLUDE_CACHE_L2	/* Level 2 cache support */
#   undef  INCLUDE_Z8536_AUXCLK	/* aux clock for boards w/z8536 */
#   undef  INCLUDE_SPY		/* no auxClk, no spy task */
#endif /*MV1300*/

/* Check for L2 caching */

#if defined(INCLUDE_CACHE_SUPPORT) && defined(USER_D_CACHE_ENABLE) \
	   && defined(USER_I_CACHE_ENABLE)
#   define CACHE_L2_CAPABLE
#else
#   undef  CACHE_L2_CAPABLE
#endif  /*INCLUDE_CACHE_SUPPORT && ... */

/* Set maximum number of PCI device idsel lines */

#define	PCI_DEV_IDSEL_MAX	6

/* Miscellaneous definitions */

#if (CPU == PPC603)
#   define WRONG_CPU_MSG "A PPC603 VxWorks image cannot run on a PPC604!\n";
#elif (CPU == PPC604)
#   define WRONG_CPU_MSG "A PPC604 VxWorks image cannot run on a PPC603!\n";
#endif	/* (CPU == PPC604) */

/* non-volatile ram parameters */

#define NV_RAM_SIZE 		BBRAM_SIZE
#define NV_RAM_ADRS		((char *) BBRAM_ADRS)
#define NV_RAM_INTRVL		1

#undef	NV_BOOT_OFFSET		/* first 256 bytes are used by Mot Bug roms */
#define NV_BOOT_OFFSET		256

/* serial parameters */

#undef	NUM_TTY
#define	NUM_TTY			N_SIO_CHANNELS

/* clock rates */

#define SYS_CLK_RATE_MIN  	10		/* minimum system clock rate */
#define SYS_CLK_RATE_MAX  	5000		/* maximum system clock rate */
#define AUX_CLK_RATE_MIN  	40		/* min auxiliary clock */
#define AUX_CLK_RATE_MAX	5000		/* max auxiliary clock rate */

/* parameters for network device drivers, if included */

#ifdef	INCLUDE_DC
#   define INT_VEC_DC	LN_INT_VEC 		/* interrupt vector */
#   define INT_LVL_DC	LN_INT_LVL		/* interrupt level */
#   define DC_POOL_ADRS	(NONE)			/* NONE = allocate from mem */
#   define DC_POOL_SIZE	0x0000C000		/* memory pool size */
#   define DC_DATA_WIDTH (NONE)			/* all data widths */
#   define IO_ADRS_DC	 (PCI_IO_LN_ADRS | CPU_PCI_IO_ADRS)/* I/O adrs */
#   define DC_RAM_PCI_ADRS (PCI2DRAM_BASE_ADRS)	/* RAM seen from PCI */
#   define DC_MODE	 (NONE)			/* default mode */
#endif	/* INCLUDE_DC */

#ifdef	INCLUDE_LN_PCI
#   define INCLUDE_IF_USR
#   define IF_USR_NAME	("lnPci")		/* name */
#   define IF_USR_ATTACH (lnPciattach)		/* <(attach) v(adrs) */
#   define IF_USR_ARG1	((char *)((PCI_IO_LN_ADRS | CPU_PCI_IO_ADRS) + 0x10))
#   define IF_USR_ARG2	LN_INT_VEC 		/* interrupt vector */
#   define IF_USR_ARG3	LN_INT_LVL		/* interrupt level */
#   define IF_USR_ARG4	(NONE)			/* mememory pool adrs */
#						/* NONE = allocated from mem */
#   define IF_USR_ARG5	0xC000			/* memory pool size */
#   define IF_USR_ARG6	(NONE)			/* all data widths */
#   define IF_USR_ARG7	(PCI2DRAM_BASE_ADRS)	/* RAM seen from PCI */
#   define IF_USR_ARG8	(NONE)			/* default mode */
#endif	/* INCLUDE_LN_PCI */

#ifdef	INCLUDE_VME

    /* VME and shared memory parameters */


    /* Backplane network parameters */

    /*
     * the backplane network and shared memory objects memory pools, 
     * if on-board, are not allocated from the system memory pool ( = NONE) 
     * because of what appears to be a cache coherency problem with 603 
     * processor.
     * To resolve this problem we have to make sure that SM_MEM_ADRS and 
     * SM_OBJ_MEM_ADRS are aligned to a cache ligne size (32 bytes).
     * This problem does not occur with 603e or 604 processor (cf spr #7416).
     */

#   define SM_OFF_BOARD		FALSE

#   if	SM_OFF_BOARD
#	undef  SM_ANCHOR_ADRS
#	define SM_ANCHOR_ADRS	((char *)0xd8004100) /* off-brd anchor addr */
#	define SM_MEM_ADRS	SM_ANCHOR_ADRS	/* off-brd shared mem addr */
#	define SM_MEM_SIZE	0x00010000	/* 512K */
#	define SM_OBJ_MEM_ADRS 	(SM_MEM_ADRS+SM_MEM_SIZE) /* SM Obj pool adrs */
#	define SM_OBJ_MEM_SIZE 	0x00010000	/* SM Obj pool size 512K */
#   else

#	define SM_MEM_ADRS 	(RAM_LOW_ADRS - (SM_MEM_SIZE + SM_OBJ_MEM_SIZE))
#	define SM_MEM_SIZE	0x00010000	/* 64K */
#	define SM_OBJ_MEM_ADRS  (SM_MEM_ADRS + SM_MEM_SIZE)
#	define SM_OBJ_MEM_SIZE 	0x00010000	/* SM Objects pool size 64K */
#   endif	/* SM_OFF_BOARD */

#   define GCSR_GROUP_ADDR 	(0xc2) 		/* A15-A8 for brd mbox addr */
#   define SM_INT_TYPE		SM_INT_MAILBOX_1 /* 1-byte write mbox int */
#   define SM_INT_ARG1 		VME_AM_SUP_SHORT_IO /* bus address space */
		   				/* bus address */
#   define SM_INT_ARG2 		((GCSR_GROUP_ADDR << 8) + \
				 (sysProcNumGet () << 4) + 2)
#   define SM_INT_ARG3		0x08		/* value (SIG3 bit) */

    /*
     * Local memory mapped to A24 VME space.
     * The only valid values of VME_A24_SLV_SIZE are
     * 64Kb, 128kb, 256kb, 512Kb, 1Mb, 2Mb, 4Mb, 8Mb, 16Mb.
     */

#   define VME_A24_SLV_SIZE	0x00400000	/* 4 Meg */
#   define VME_A24_SLV_ADRS	0x00400000	/* Bus address in A24 range */

    /*
     * Local Memory mapped to VME A32 bus.
     * The only valid values of VME_A32_SLV_SIZE are:
     * 64Kb, 128kb, 256kb, 512Kb, 1Mb, 2Mb, 4Mb, 8Mb, 16Mb, 32Mb, 64Mb, etc.
     */

#   define VME_A32_SLV_SIZE	0x01000000	/* 16 Meg */
#   define VME_A32_SLV_ADRS	0x08000000	/* Bus address in A32 range */

    /*
     * VME A32 mapped to local cpu space, only a portion of A32 is
     * visible.  MASTER_MEM_BUS_A32 + MASTER_MEM_A32_SIZE must not
     * exceed 0x10000000.
     */

#   define MASTER_MEM_BUS_A32	0x08000000	/* VME A32 address */
#   define MASTER_MEM_A32_SIZE	0x01000000	/* 16MB */

#endif	/*INCLUDE_VME*/

/* Local Memory addresses */

#define LOCAL_MEM_LOCAL_ADRS	0x00000000	/* fixed at zero */
/* #define LOCAL_MEM_SIZE	0x00800000 */	/* memory: 8  Mbytes */
/* PCI 8/11/97 */
#define LOCAL_MEM_SIZE		0x02000000	/* memory: 32  Mbytes */

/* user reserved memory, see sysMemTop() */

#define USER_RESERVED_MEM	(0)		/* number of reserved bytes */

/*
 * The constants ROM_TEXT_ADRS, ROM_SIZE, RAM_LOW_ADRS and RAM_HIGH_ADRS
 * are defined in config.h, Makefile, and Makefile.PPC603<tool>.
 * All definitions for these constants must be identical.
 */

#undef	INCLUDE_MOT_BUG_ROM 	/* define this to use Motorola BUG Roms */

#ifdef INCLUDE_MOT_BUG_ROM
#   define ROM_BASE_ADRS 	(LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE)
#   define ROM_TEXT_ADRS 	ROM_BASE_ADRS
#else
#   define ROM_BASE_ADRS 	0xfff00000	/* base addr of ROM */
#   define ROM_TEXT_ADRS 	(ROM_BASE_ADRS + 0x100)
#endif /* INCLUDE_MOT_BUG_ROM */

#define	ROM_SIZE		0x00080000	/* 512K ROM space */

/* 
 * RAM_HIGH_ADRS is text addr for bootrom, RAM_LOW_ADRS is for VxWorks.
 * RAM_LOW_ADRS is set to 0x10000 for the ULTRA boards because there no need 
 * to reserve space below RAM_LOW_ADRS for the shared memory network and 
 * objects memory pools (refer to spr #7416).
 */

#define RAM_HIGH_ADRS		(LOCAL_MEM_LOCAL_ADRS + 0x00200000)
#ifdef	INCLUDE_VME				/* SM and SM objects included */
#    define RAM_LOW_ADRS	(LOCAL_MEM_LOCAL_ADRS + 0x00030000)
#else						/* no space for SM needed */
#    define RAM_LOW_ADRS	(LOCAL_MEM_LOCAL_ADRS + 0x00010000)
#endif

/* flash support */

#ifdef  INCLUDE_FLASH
#   define FLASH_ADRS		0xff080000	/* Flash device on Bank 1 */
#   define FLASH_SIZE		0x00080000	/* 512 KBytes */
#   define FLASH_WIDTH		1		/* 1 byte wide */
#   define FLASH_SIZE_WRITEABLE	FLASH_SIZE	/* write all of flash */
#   define SYS_FLASH_TYPE	FLASH_29F010	/* same algorythm as 29F040 */
#   define SYS_FLASH_WRITE			/* enable writting to flash */
#   define FLASH_NO_OVERLAY
#endif  /* INCLUDE_FLASH */

/*
 * Default power management mode - selected via vxPowerModeSet() in
 * sysHwInit().
 */

#define DEFAULT_POWER_MGT_MODE  VX_POWER_MODE_DISABLE

#endif	/* INCconfigh */

/* The following has been added by WindCfg */
#include "configdb.h"
