/* sysLib.c - Motorola MVME1300, MVME1600 and Ultra board system-dependent library */

/* Copyright 1984-1997 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
02t,20feb97,mas  made ppcZ8536Timer.c inclusion only for MV1600 (SPR 7801);
		 fixed error checking in sysLocalToBusAdrs() (SPR 7995).
02s,18nov96,mas  deleted CPU_PCI_MEM_SIZE comment in sysPhysMemDesc (SPR 7406).
02r,11nov96,tam  added power management mode initialization. 
02q,30oct96,tam  added support for the PPC603EV processor (spr #7412). 
		 optimized sysInByte() and sysOutByte() (spr #7415).
02p,23oct96,wlf  doc: cleanup.
02o,15oct96,tam  renamed IO_SYNC to EIEIO_SYNC, SPR #7346; modified 
		 sysPhysMemDesc[], SPR #7347
02n,14oct96,dat  added LOCAL_MEM_AUTOSIZE, SPR #7288
02m,02oct96,mas  removed conditional compilation regarding SPR #7213. 
02l,26sep96,mas  removed _MMU_LBAT_GUARDED from sysBatDesc[] (SPR 7111);
		 added sysPicrGet/Set() from sysCache.c; added MPC105 macros.
02k,15sep96,tam  fix for spr # 7195: read enet address from NV-RAM.
		 added support for flash memory. added code to boot using
		 Motorola PPC BUG ROMs. fix for spr 6371.
02j,23aug96,tam  added code to disable mailbox in sysHwInit.
02i,15aug96,dat  fixed sysInByte, some macros, some comments
02h,14aug96,tam  added definition of PCI_READ and PCI_WRITE to use latest
		 version of pciLocal.c.
02g,13aug96,tam  changed VME set up to use new macro definition for MASTER
		 and SLAVE VME windows. Added i/o synchronization to sysInByte
		 and sysOutByte.
02f,08aug96,mas  fix for SPR 6824&6889: 1305 -> 1300; merge w/1603 code.
		 BSP-conditional compilation changed to component-conditional
		 compilation (see component list in config.h).
		 L2 cache support now dependent upon:
		 CACHE_L2_CAPABLE && INCLUDE_CACHE_L2 (see config.h)
02e,05aug96,mas  new sysBusTas, renamed sysBusClearTas to sysBusTasClear; added
		 reference to smUtilTasClearRtn and set it to sysBusTasClear.
02d,01aug96,tam  added support for the PPC604E processor type.
02c,31jul96,tam  set the serial baud rate in sysCpuCheck (spr #6808).
02b,31jul96,dds  SPR 6821: #include ncr810.h. is now in sysScsi.c
02a,31jul96,dat  added sysPhysMemTop and new sysMemTop, some coding std fixes.
01z,22jul96,tam  added code to calculate the external cpu bus clk speed for
		 ULTRA boards.
		 fixed spr #5940: cleared BRDFAIL signal.
01y,19jun96,tam  fixed calculation of VMECHIP2_TIMEOUT (spr #6134).
01x,24apr96,jds  moved inclusion of sysScsi.c to the correct place
01w,01apr96,jds  added support for both NCR810 and NCR825 SCSI controllers
01v,04apr96,tpr  added PowerPC603e support (SPR 6293).
01u,18mar96,dat	 removed dependence upon bspVersion.h
01t,07mar96,tpr	 added #ifdef INCLUDE_Z8536_AUXCLK.
01s,06mar96,tpr  changed i8259Pic.c by i82378Ibc.c
01r,29oct95,tpr  added #include "sysSerial.c".
01q,17jan96,tpr  put the first Mega of DRAM Not Cacheable.
01p,16jan96,tpr  changed sysBusTas() code and added comment.
01o,01dec95,kvk  provided a spurious interrupt handler for level 7 of the PIC,
		 refer to page 3-188 of the Intel 8259A manual for details.
01n,29nov95,kvk  disabled the I and D caches for clean reboot with CTRL-x
		 after the boot time was improved with the I cache ON.
01m,22oct95,kvk  adjusted default VME bus mappings and disabled spurious
		 interrupt 0x07 (parallel port).
01l,22sep95,kvk  made mv1600 BSP to work for the Ultra board.
01k,20jul95,kvk  added vxMsrSet (0) to sysToMonitor for correct reboot.
01j,28jun95,caf  removed usrBootLineInit() call.
01i,26jun95,kvk  modified sysMemTop to return only the configured memory.
01h,16jun95,kvk  memory fine tuning and use of LN_INT_LVL macro.
01g,24may95,caf  replaced sysPhysMemDesc[] with sysBatDesc[] (for data cache),
		 removed dbgTrace().
01f,28apr95,caf  renamed timer driver.
01e,28apr95,kvk  modified sysMemTop to return the min of probed memory/32MB.
01d,05apr95,caf  initialize boot line in sysHwInit2() until boot ROM supported.
01c,04apr95,vin  fixed VMEchip2 initialization.
01b,29mar95,caf  added cacheLib support.
01a,14nov94,vin  written.
*/

/*
DESCRIPTION
This library provides board-specific routines for the MVME1300, MVME1600, and
Ultra series of target boards.  The chip drivers included are:

    For the MVME1300, MVME1600, and Ultra series:

    i8250Sio.c - Intel i8250 UART driver 
    ppcDecTimer.c - PowerPC decrementer timer library
    i82378Ibc.c - Intel i82378 interrupt controller library
    pciLocalBus.c - PCI configuration library
    byteNvRam.c	- byte-oriented generic non-volatile RAM library
    flashMem.c - flash memory device library

    For the MVME1300 and MVME1600 series only:
    
    vmeChip2Vme.c - VMEchip2 library
    vme2Pci.c - VME2PCI chip library

    For the MVME1600 and Ultra series only:
    
    ncr810Lib - NCR 53C810 or NCR 53C825 SCSI controller library
    if_lnPci - AMD Am7990 LANCE Ethernet driver (`lnPci')
    if_dc - DEC 21040 PCI Ethernet LAN network-interface driver (`dc')

    For the MVME1600 series only:
    
    ppcZ8536Timer.c - Zilog Z8536 timer library

INCLUDE FILES: sysLib.h

SEE ALSO:
.pG "Configuration"
*/

/* includes */

#include "vxWorks.h"
#include "memLib.h"
#include "string.h"
#include "intLib.h"
#include "logLib.h"
#include "taskLib.h"
#include "vxLib.h"
#include "tyLib.h"

/* Architecture-dependent includes */
#include "sysLib.h"
#include "config.h"

#include "arch/ppc/vxPpcLib.h"
#include "cacheLib.h"
#include "private/vmLibP.h"
#include "arch/ppc/mmu603Lib.h"

#include "vme.h"
#include "drv/vme/vme2Pci.h"
#include "drv/pci/mpc105.h"
#include "drv/scsi/ncr810.h"


/* defines */

#define ZERO		0
#define CPU_TYPE	((vxPvrGet() >> 16) & 0xffff)


/* define macros for pciLocalBus.c to read/write a longword on PCI bus */

#define PCI_READ(base,index,addr) \
	    {*(ULONG *)addr = PSWAP(*PCI_ADDR(base,index)); EIEIO_SYNC;}

#define PCI_WRITE(base,index,data) \
		{*PCI_ADDR(base,index) = PSWAP(data); EIEIO_SYNC;}

/* defines for sysBusTas() and sysBusTasClear() */

#define VMEBUS_OWNER		(*VMECHIP2_DMACR1 & DMACR1_DHB)
#define CPU_CLOCKS_PER_LOOP	10
#define LOCK_TIMEOUT		10	/* uSec */
#define UNLOCK_TIMEOUT		10000	/* uSec */

/* defines for the vmeChip2 */

#define VMECHIP2_LBSAR_RESET_VAL	0x0
#define VMECHIP2_LBTVCR_RESET_VAL 	0x00f00000

/* defines for the MPC105 PCI Bridge and Memory Controller chip */

    /*
     * Access presumes:
     *   PCI byte ordering,
     *   32-bit MPC105 registers,
     *   memory mapped access,
     *   requires sync for ordered I/O.
     */

#define MPC105_REF(reg) ((UINT32)(reg) | CPU_PCI_ISA_IO_ADRS)

#ifndef MPC_WRITE_LONG
#   define MPC_WRITE_LONG(reg,value) \
	*((UINT32 *)(reg)) = PSWAP(value)
#endif

#ifndef MPC_READ_LONG
#   define MPC_READ_LONG(reg,pValue) \
	*((UINT32 *)(pValue)) = PSWAP(*((UINT32 *)(reg)))
#endif

/* defines for the Z8536 timer driver */

#define ZCIO_CNTRL_ADRS                 (UINT8 *)0x80000847
#define ZCIO_IACK_ADRS                  (UINT8 *)0x8000084F
#define CIO_RESET_DELAY                 5000
#define ZCIO_HZ                         2500000 /* 2.5 MHz clock */
#define CIO_INT_VEC                     9
#define Z8536_TC                        ZCIO_HZ


/* external declarations */

IMPORT void		sysClkIntCIO (void);
IMPORT UINT32		sysBusSpeed  (int);
IMPORT VOIDFUNCPTR	smUtilTasClearRtn;


/* globals */

/*
 * sysBatDesc[] is used to initialize the block address translation (BAT)
 * registers within the PowerPC 603/604 MMU.  BAT hits take precedence
 * over Page Table Entry (PTE) hits and are faster.  Overlap of memory
 * coverage by BATs and PTEs is permitted in cases where either the IBATs
 * or the DBATs do not provide the necessary mapping (PTEs apply to both
 * instruction AND data space, without distinction).
 *
 * The primary means of memory control for VxWorks is the MMU PTE support
 * provided by vmLib and cacheLib.  Use of BAT registers will conflict
 * with vmLib support.  User's may use BAT registers for i/o mapping and
 * other purposes but are cautioned that conflicts with caching and mapping
 * through vmLib may arise.  Be aware that memory spaces mapped through a BAT
 * are not mapped by a PTE and any vmLib() or cacheLib() operations on such
 * areas will not be effective, nor will they report any error conditions.
 *
 * Note: BAT registers CANNOT be disabled - they are always active.
 * For example, setting them all to zero will yield four identical data
 * and instruction memory spaces starting at local address zero, each 128KB
 * in size, and each set as write-back and cache-enabled.  Hence, the BAT regs
 * MUST be configured carefully.
 *
 * With this in mind, it is recommended that the BAT registers be used
 * to map LARGE memory areas external to the processor if possible.
 * If not possible, map sections of high RAM and/or PROM space where
 * fine grained control of memory access is not needed.  This has the
 * beneficial effects of reducing PTE table size (8 bytes per 4k page)
 * and increasing the speed of access to the largest possible memory space.
 * Use the PTE table only for memory which needs fine grained (4KB pages)
 * control or which is too small to be mapped by the BAT regs.
 *
 * The BAT configuration for 4xx/6xx-based PPC boards is as follows:
 * All BATs point to PROM/FLASH memory so that end customer may configure
 * them as required.
 *
 * [Ref: chapter 7, PowerPC Microprocessor Family: The Programming Environments]
 */

UINT32 sysBatDesc [2 * (_MMU_NUM_IBAT + _MMU_NUM_DBAT)] =
    {
    /* I BAT 0 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* I BAT 1 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* I BAT 2 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* I BAT 3 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* D BAT 0 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* D BAT 1 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* D BAT 2 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT),

    /* D BAT 3 */
    ((ROM_BASE_ADRS & _MMU_UBAT_BEPI_MASK) | _MMU_UBAT_BL_1M |
    _MMU_UBAT_VS | _MMU_UBAT_VP),
    ((ROM_BASE_ADRS & _MMU_LBAT_BRPN_MASK) | _MMU_LBAT_PP_RW |
    _MMU_LBAT_CACHE_INHIBIT)
    };


/*
 * sysPhysMemDesc[] is used to initialize the Page Table Entry (PTE) array
 * used by the MMU to translate addresses with single page (4k) granularity.
 * PTE memory space should not, in general, overlap BAT memory space but
 * may be allowed if only Data or Instruction access is mapped via BAT.
 *
 * Address translations for local RAM, memory mapped PCI bus, memory mapped
 * VME A16 space and local PROM/FLASH are set here.
 *
 * PTEs are held, strangely enough, in a Page Table.  Page Table sizes are
 * integer powers of two based on amount of memory to be mapped and a
 * minimum size of 64KB.  The MINIMUM recommended Page Table sizes
 * for 32-bit PowerPCs are:
 *
 *	Total mapped memory		Page Table size
 *	-------------------		---------------
 *	       8 Meg			     64 K
 *	      16 Meg			    128 K
 *	      32 Meg			    256 K
 *	      64 Meg			    512 K
 *	     128 Meg			      1 Meg
 *		.				.
 *		.				.
 *		.				.
 *
 * [Ref: chapter 7, PowerPC Microprocessor Family: The Programming Environments]
 */

PHYS_MEM_DESC sysPhysMemDesc [] =
    {
    {
    (void *) LOCAL_MEM_LOCAL_ADRS,
    (void *) LOCAL_MEM_LOCAL_ADRS,
    (LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE),
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE
    },

    {
    (void *) CPU_PCI_ISA_IO_ADRS,
    (void *) CPU_PCI_ISA_IO_ADRS,
    CPU_PCI_ISA_IO_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) CPU_PCI_CNFG_ADRS,
    (void *) CPU_PCI_CNFG_ADRS,
    CPU_PCI_CNFG_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) CPU_PCI_IO_ADRS,
    (void *) CPU_PCI_IO_ADRS,
    CPU_PCI_IO_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) CPU_PCI_IACK_ADRS,
    (void *) CPU_PCI_IACK_ADRS,
    CPU_PCI_IACK_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) CPU_PCI_MEM_ADRS,
    (void *) CPU_PCI_MEM_ADRS,
    CPU_PCI_MEM_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) CPU_PCI_ISA_MEM_ADRS,
    (void *) CPU_PCI_ISA_MEM_ADRS,
    CPU_PCI_ISA_MEM_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

#if defined(MV1600) || defined(MV1300)
    {
    (void *) VME_WINDOW_A24_BASE,
    (void *) VME_WINDOW_A24_BASE,
    VME_WINDOW_A24_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) VME_WINDOW_A16_BASE,
    (void *) VME_WINDOW_A16_BASE,
    VME_WINDOW_A16_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    {
    (void *) VME_WINDOW_A32_BASE,
    (void *) VME_WINDOW_A32_BASE,
    VME_WINDOW_A32_SIZE,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },
#endif	/* MV1600 || MV1300 */

    {
    (void *) 0xFF000000,
    (void *) 0xFF000000,
    0x01000000,					/* 16 meg */
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    }
    };

int sysPhysMemDescNumEnt = NELEMENTS (sysPhysMemDesc);

int   sysBus      = BUS;                /* system bus type (VME_BUS, etc) */
int   sysCpu      = CPU;                /* system CPU type (MC680x0) */
char *sysBootLine = BOOT_LINE_ADRS;	/* address of boot line */
char *sysExcMsg   = EXC_MSG_ADRS;	/* catastrophic message area */
int   sysProcNum;			/* processor number of this CPU */
int   sysFlags;				/* boot flags */
char  sysBootHost [BOOT_FIELD_LEN];	/* name of host from which we booted */
char  sysBootFile [BOOT_FIELD_LEN];	/* name of file from which we booted */
UINT  sysVectorIRQ0 = INT_VEC_IRQ0;	/* vector for IRQ0 */
BOOL  sysVmeEnable = FALSE;		/* by default no VME */
int   sysMemoryBusSpeed = 33333333;	/* external cpu bus clock speed */

/* last 5 nibbles are board specific, initialized in sysHwInit */

unsigned char lnEnetAddr [6] = { 0x08, 0x00, 0x3e, 0x00, 0x00, 0x00 };

/* forward declarations */

void	sysSpuriousIntHandler (void);
UCHAR	sysInByte (ULONG);
void	sysOutByte (ULONG, char);
void	sysCpuCheck (void);
char *  sysPhysMemTop (void);
void	sysBusTasClear (volatile char * address);
UCHAR	sysNvRead (ULONG);
void	sysNvWrite (ULONG,UCHAR);
int	sysPicrGet (int);
STATUS	sysPicrSet (int, int);

/* locals */

LOCAL char sysModelStr[80];
LOCAL char wrongCpuMsg[] = WRONG_CPU_MSG; 

/*
 * COMPONENT CONFIGURATION
 */

#ifdef DOC
#define INCLUDE_TIMESTAMP
#define INCLUDE_SHOW_ROUTINES
#endif
    
#include "sysSerial.c"
#include "intrCtl/i82378Ibc.c"
#include "pci/pciLocalBus.c"
#include "mem/byteNvRam.c"

#ifdef	ULTRA			/* redefine MEMORY_BUS_SPEED for ULTRA boards */

#undef	MEMORY_BUS_SPEED
#define	MEMORY_BUS_SPEED 	sysMemoryBusSpeed

#else

/* vme2Pci config header */

VME2PCI_CNFG_HDR vme2PciCnfgHdr	=
    {
    MSW(PCI2VME_SLV1_END_ADRS),		/* vme A24 PCI ending address */
    MSW(PCI2VME_SLV1_STRT_ADRS),	/* vme A24 PCI starting address */
    VME2PCI_SLAVE_ENABLE,		/* slave 1 enable */
    MSW(PCI2VME_SLV1_OFFSET),		/* vme A24 PCI offset */
    MSW(PCI2VME_SLV2_END_ADRS),		/* vme A32 PCI ending address */
    MSW(PCI2VME_SLV2_STRT_ADRS),	/* vme A32 PCI ending address */
    VME2PCI_SLAVE_ENABLE,		/* slave 2 enable */
    MSW(PCI2VME_SLV2_OFFSET),		/* vme A32 PCI offset */
    VME2PCI_INT_ENABLE			/* vme2Pci interrupts enable */
    };

#include "vme/vme2Pci.c"
#include "vme/vmeChip2Vme.c"

#endif	/* ULTRA */

#include "timer/ppcDecTimer.c"		/* PPC603 & 604 have on chip timers  */
					/* need to be after the definition of */
					/* MEMORY_BUS_SPEED for ultra boards */

#ifndef MV1300

#include "sysScsi.c"			/* MV1600 and ULTRA series */
#include "sysCache.c"

#endif  /* MV1300 */

#ifdef	MV1600

#include "timer/ppcZ8536Timer.c"	/* MV1600 series only */

#endif	/* MV1600 */

#ifdef  INCLUDE_FLASH
#include "mem/flashMem.c"
#endif	/* INCLUDE_FLASH */


/******************************************************************************
*
* sysModel - return the model name of the CPU board
*
* This routine returns the model name of the CPU board.  The returned string
* depends on the board model and CPU version being used, for example,
* "Motorola ULTRA - PowerPC 603e".
*
* RETURNS: A pointer to the string.
*/

char * sysModel (void)
    {
    if (CPU_TYPE == CPU_TYPE_603E)
#ifdef	ULTRA
    	sprintf (sysModelStr, "Motorola ULTRA - PowerPC 603e");
    else if (CPU_TYPE == CPU_TYPE_603EV)
	sprintf (sysModelStr, "Motorola ULTRA - PowerPC 603ev");
    else if (CPU_TYPE == CPU_TYPE_604E)
	sprintf (sysModelStr, "Motorola ULTRA - PowerPC 604e");
    else
	sprintf (sysModelStr, "Motorola ULTRA - PowerPC 60%d", CPU_TYPE);
#endif  /* ULTRA */
#ifdef  MV1600
    	sprintf (sysModelStr, "Motorola MVME1600 - PowerPC 603e");
    else if (CPU_TYPE == CPU_TYPE_603EV)
	sprintf (sysModelStr, "Motorola MVME1600 - PowerPC 603ev");
    else if (CPU_TYPE == CPU_TYPE_604E)
	sprintf (sysModelStr, "Motorola MVME1600 - PowerPC 604e");
    else
	sprintf (sysModelStr, "Motorola MVME1600 - PowerPC 60%d", CPU_TYPE);
#endif	/* MV1600 */
#ifdef MV1300
        sprintf (sysModelStr, "Motorola MVME1300 - PowerPC 603e");
    else if (CPU_TYPE == CPU_TYPE_603EV)
	sprintf (sysModelStr, "Motorola MVME1300 - PowerPC 603ev");
    else if (CPU_TYPE == CPU_TYPE_604E)
	sprintf (sysModelStr, "Motorola MVME1300 - PowerPC 604e");
    else
	sprintf (sysModelStr, "Motorola MVME1300 - PowerPC 60%d", CPU_TYPE);
#endif  /* MV1300 */

    return (sysModelStr);
    }


/******************************************************************************
*
* sysBspRev - return the BSP version and revision number
*
* This routine returns a pointer to a BSP version and revision number, for
* example, 1.1/0. BSP_REV is concatenated to BSP_VERSION and returned.
*
* RETURNS: A pointer to the BSP version/revision string.
*/

char * sysBspRev (void)
    {
    return (BSP_VERSION BSP_REV);
    }


/******************************************************************************
*
* sysHwInit - initialize the system hardware
*
* This routine initializes various features of the CPU board.  It is called
* by usrInit() in usrConfig.c.  This routine sets up the control registers
* and initializes various devices if they are present.
*
* NOTE: This routine should not be called directly by the user application.  It
* cannot be used to initialize interrupt vectors.
*
* RETURNS: N/A
*/

void sysHwInit (void)
    {
    ULONG	pciDevCnfgAdrs;
    int         pciBusSpeed;

    /* check CPU type */

    sysCpuCheck ();

    EIEIO_SYNC;

    /* update boot line */
#ifdef	INCLUDE_MOT_BUG_ROM
    sysNvRamGet (BOOT_LINE_ADRS, BOOT_LINE_SIZE, 0);
#endif	/* INCLUDE_MOT_BUG_ROM */

    /* configure the IBC (ISA Bridge Controller 82378) */

    sysPciDevConfig (
		    CNFG_IBC_ADRS,
		    PCI_IO_IBC_ADRS,
		    NULL,
		    (CSR_BM_EN | CSR_IO_EN),
		    sysIbcInit,
		    NONE,
		    NONE
		    );

    /* configure PCI LANCE device if present */

    if (DEVICE_PRESENT(MV1600_BCR_LANP) == TRUE)
	{
	EIEIO_SYNC;

	sysPciDevConfig (
			CNFG_LN_ADRS,
			PCI_IO_LN_ADRS,
			NULL,
			(CSR_BM_EN | CSR_IO_EN),
			NULL,
			NONE,
			NONE
			);
	}

    /*
     * Configure SCSI device if present. The SCSI device could be either
     * an NCR825 or an NCR810. Therefore, try to find either one of them.
     */

    if ((DEVICE_PRESENT(MV1600_BCR_SCIP) == TRUE) &&
	(sysPciDevProbe (CPU_PCI_CNFG_ADRS, PCI_ID_NCR825, &pciDevCnfgAdrs)
	 == OK))
	{
	EIEIO_SYNC;

	sysPciDevConfig (
			pciDevCnfgAdrs,
			PCI_IO_SCSI_ADRS,
			NULL,
			(CSR_BM_EN | CSR_IO_EN),
			NULL,
			NONE,
			NONE
			);
	}
    else if ((DEVICE_PRESENT(MV1600_BCR_SCIP) == TRUE) &&
      (sysPciDevProbe (CPU_PCI_CNFG_ADRS, PCI_ID_NCR810, &pciDevCnfgAdrs)
	 == OK))
	{
	EIEIO_SYNC;

	sysPciDevConfig (
			pciDevCnfgAdrs,
			PCI_IO_SCSI_ADRS,
			NULL,
			(CSR_BM_EN | CSR_IO_EN),
			NULL,
			NONE,
			NONE
			);
	}

#ifdef	ULTRA
    /* calculate the external cpu bus clock speed in hertz */

    sysMemoryBusSpeed = sysBusSpeed (0);

    if ((sysMemoryBusSpeed < 28) && (sysMemoryBusSpeed > 22))
	sysMemoryBusSpeed = 25000000;
    else if ((sysMemoryBusSpeed < 36) && (sysMemoryBusSpeed > 30))
	sysMemoryBusSpeed = 33333333;
    else if ((sysMemoryBusSpeed < 43) && (sysMemoryBusSpeed > 37))
	sysMemoryBusSpeed = 40000000;
    else if ((sysMemoryBusSpeed < 53) && (sysMemoryBusSpeed > 47))
	sysMemoryBusSpeed = 50000000;
    else if ((sysMemoryBusSpeed < 63) && (sysMemoryBusSpeed > 57))
	sysMemoryBusSpeed = 60000000;
    else
	sysMemoryBusSpeed = 66666666;	/* 66.66 MHz */
#endif	/* ULTRA */

    /* calculate the PCI bus clock speed in mega hertz */

    switch (MEMORY_BUS_SPEED)
	{
	case 50000000:
	    pciBusSpeed = 25;
	    break;
	case 40000000:
	    pciBusSpeed = 20;
	    break;
	default:
	    pciBusSpeed = 33;
	    break;
	}

#if defined(MV1600) || defined(MV1300)
    /* configure VME2PCI device if present */

    EIEIO_SYNC;

    if ((DEVICE_PRESENT(MV1600_BCR_VMEP) == TRUE) &&
	(sysPciDevProbe (CPU_PCI_CNFG_ADRS, PCI_ID_VME2PCI, &pciDevCnfgAdrs)
	 == OK))
	{
	EIEIO_SYNC;

	sysPciDevConfig (
			pciDevCnfgAdrs,
			PCI_IO_VME2PCI_ADRS,
			NULL,
			(CSR_BM_EN | CSR_IO_EN),
			sysVme2PciInit,
			(ULONG)(&vme2PciCnfgHdr),
			pciDevCnfgAdrs
			);

	}

    /* initialize  VMECHIP2 if VME present */

    if (DEVICE_PRESENT(MV1600_BCR_VMEP) == TRUE)
	{
	/*
	 * Disable VME access to on-board ram and VMEchip slave write posting.
	 * (VME access is later enabled for processor 0 in sysProcNumSet.)
	 * no VME access now, maybe later
	 * Set-up VMEbus access as follows:
	 *
	 * addresses generated on the PCI bus are added by an offset and
         * translated to the VME addresses. Seeing from the processor bus
         * the VME window starts from 0xe0000000 for A24 and 0xd0000000
         * for A32. The A24/A16 window is mapped on the F page of the VME
	 * and the A32 window is mapped at 0 of the VME address space.
	 *
	 *   SHORT_IO		0xffff0000 - 0xffffffff
	 *   STD (A24/D16)	0xf0000000 - 0xf0ffffff
	 *   EXT (A32)		0x00000000 - 0x7fffffff
	 */

	/* set VMEbus short access and enable map */

	*VMECHIP2_LBSAR1	= 0xffffffff;
        *VMECHIP2_LBSAR         = VMECHIP2_LBSAR_RESET_VAL;
	*VMECHIP2_LBSAR		|= (LBSAR1_AM_SUP_SHORT_IO |
				   LBSAR1_D16);

        *VMECHIP2_LBTVCR        = VMECHIP2_LBTVCR_RESET_VAL;
	*VMECHIP2_LBTVCR	|= LBTVCR_EN1;

	/* set VMEbus std access and enable map */

	*VMECHIP2_LBSAR2	= 0xf0fff000;
	/* *VMECHIP2_LBSAR	|= (LBSAR2_AM_STD_USR_DATA | LBSAR2_D16); */
        /* Change to D32 - BEH 8/25/97 */
	*VMECHIP2_LBSAR		|= (LBSAR2_AM_STD_USR_DATA | LBSAR2_D32);
	*VMECHIP2_LBTVCR	|= LBTVCR_EN2;

	/* set VMEbus ext access and enable map */

	*VMECHIP2_LBSAR3	= (0x7fff0000 | 0x0);
	*VMECHIP2_LBSAR		|= (LBSAR3_AM_EXT_USR_DATA | LBSAR3_D32);
	*VMECHIP2_LBTVCR	|= LBTVCR_EN3;

	/*
	 * set VMEbus global timeout to 64us
	 * set VMEbus access timeout to 1ms
	 * set local bus timeout to 64us
	 * set watchdog timeout to 16ms
	 * and set prescaler register
	 */

	*VMECHIP2_TIMEOUTCR	= (TIMEOUTCR_VGTO_256US	|
				   TIMEOUTCR_VATO_1MS	|
				   TIMEOUTCR_LBTO_64US  |
				   TIMEOUTCR_WDTO_16MS) |
				   ((256 - pciBusSpeed) & 0xff);

	/* all VMEbus vectors with 1:1 mapping */
	
	*VMECHIP2_ILR4		= (ILR4_VIRQ7_LEVEL7	|
				   ILR4_VIRQ6_LEVEL6	|
				   ILR4_VIRQ5_LEVEL5	|
				   ILR4_VIRQ4_LEVEL4	|
				   ILR4_VIRQ3_LEVEL3	|
				   ILR4_VIRQ2_LEVEL2	|
				   ILR4_VIRQ1_LEVEL1);
	
	/* setup VMEbus requester control */
	
	*VMECHIP2_DMACR1	= (DMACR1_SINK_DATA	|
				   DMACR1_LVRWD		|
				   DMACR1_LVREQ_L3);
	
	/* master enable chip interrupts */
	
	*VMECHIP2_IOCR		= (IOCR_MEIN			|
				   (UTIL_INT_VEC_BASE0 << 24)	|
				   (UTIL_INT_VEC_BASE1 << 20));

	/* drive SYSFAIL line down if asserted */

	if ((*VMECHIP2_TIMERCR) & TIMERCR_SFFL)
	    *VMECHIP2_TIMERCR &= ~TIMERCR_BDFLO;

	/* set shared memory TAS Clear routine pointer */

	smUtilTasClearRtn = (VOIDFUNCPTR)sysBusTasClear;

	/*
	 * disable mailbox interrupts and clear a possible mailbox 
	 * interrupt condition
	 */

    	*VMECHIP2_LBTVCR &= ~(sysProcNumGet() << 20);
    	*VMECHIP2_ILR2  &= ~ILR2_SIG3_LEVEL3;	/* unset level 3 */
    	*VMECHIP2_LBIER &= ~LBIER_ESIG3;	/* disable GCSR SIG3 intr */

    	*VC2GCSR_LM_SIG = LM_SIG_LM3;		/* clear signal */
    	*VMECHIP2_ICLR  = ICLR_CSIG3;		/* clear IRQ */

	}
#endif	/* MV1600 || MV1300 */

    /* Reset serial channels */

    sysSerialHwInit ();

    /*
     * Extract the Ethernet address out of non-volatile RAM.
     * The Motorola convention for the Ethernet address is that they only
     * save the low 3 bytes in BBRAM.  The high three bytes are the
     * manufacturers code, and Motorola software knows its own.
     * The Motorola code is 0x08003Exxx.
     */

    lnEnetAddr [3] = sysNvRead ((int)BB_ENET + 3);
    lnEnetAddr [4] = sysNvRead ((int)BB_ENET + 4);
    lnEnetAddr [5] = sysNvRead ((int)BB_ENET + 5);

    /*
     * If MMU tables are used, this is where we would dynamically
     * update the entry describing main memory, using sysPhysMemTop().
     * We must call sysPhysMemTop () at sysHwInit() time to do
     * the memory autosizing if available.
     */

    sysPhysMemTop ();

    /*
     * The power management mode is initialized here. Reduced power mode
     * is activated only when the kernel is iddle (cf vxPowerDown).
     * Power management mode is selected via vxPowerModeSet().
     * DEFAULT_POWER_MGT_MODE is defined in config.h.
     */

    vxPowerModeSet (DEFAULT_POWER_MGT_MODE);
    }


/*******************************************************************************
*
* sysPhysMemTop - get the address of the top of physical memory
*
* This routine returns the address of the first missing byte of memory,
* which indicates the top of memory.
*
* Normally, the user specifies the amount of physical memory with the
* macro LOCAL_MEM_SIZE in config.h.  BSPs that support run-time
* memory sizing do so only if the macro LOCAL_MEM_AUTOSIZE is defined.
* If not defined, then LOCAL_MEM_SIZE is assumed to be, and must be, the
* true size of physical memory.
*
* NOTE: Do no adjust LOCAL_MEM_SIZE to reserve memory for application
* use.  See sysMemTop() for more information on reserving memory.
*
* RETURNS: The address of the top of physical memory.
*
* SEE ALSO: sysMemTop()
*/

char * sysPhysMemTop (void)
    {
    static char * physTop = NULL;

    if (physTop == NULL)
	{
#ifdef LOCAL_MEM_AUTOSIZE

	/*
	 * If auto-sizing is possible, this would be the spot.
	 * For mv1600/ultra/mv1300 autosizing is not supported
	 */

#	error	"LOCAL_MEM_AUTOSIZE: dynamic memory sizing not supported"

#else /*LOCAL_MEM_AUTOSIZE*/

	/* Don't do autosizing, size is given */

	physTop = (char *)(LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE);

#endif /*LOCAL_MEM_AUTOSIZE*/
	}

    return physTop;
    }


/*******************************************************************************
*
* sysMemTop - get the address of the top of VxWorks memory
*
* This routine returns a pointer to the first byte of memory not
* controlled or used by VxWorks.
*
* The user can reserve memory space by defining the macro USER_RESERVED_MEM
* in config.h.  This routine returns the address of the reserved memory
* area.  The value of USER_RESERVED_MEM is in bytes.
*
* RETURNS: The address of the top of VxWorks memory.
*/

char * sysMemTop (void)
    {
    static char * memTop = NULL;

    if (memTop == NULL)
	{
	memTop = sysPhysMemTop () - USER_RESERVED_MEM;
	}

    return memTop;
    }


/******************************************************************************
*
* sysToMonitor - transfer control to the ROM monitor
*
* This routine transfers control to the ROM monitor.  Normally, it is called
* only by reboot()--which services ^X--and by bus errors at interrupt level.
* However, in some circumstances, the user may wish to introduce a
* <startType> to enable special boot ROM facilities.
*
* RETURNS: Does not return.
*/

STATUS sysToMonitor
    (
    int startType	/* parameter passed to ROM to tell it how to boot */
    )
    {
#ifdef	INCLUDE_MOT_BUG_ROM
    IMPORT  STATUS sysToRom(void);
#else
    FUNCPTR pRom = (FUNCPTR) (ROM_TEXT_ADRS + 4); /* Warm reboot */
#endif	/* INCLUDE_MOT_BUG_ROM */

    cacheDisable (0);		/* Disable the Instruction Cache */
    cacheDisable (1);		/* Disable the Data Cache */

#if     (CPU == PPC604)
    vxHid0Set (vxHid0Get () & ~_PPC_HID0_SIED);	/* Enable Serial Instr Exec */
#endif  /* (CPU == PPC604) */

    vxMsrSet (0);		/* Clear the MSR */

#ifdef	INCLUDE_MOT_BUG_ROM
    sysToRom ();		/* jump to Motorola PPC BUG ROM */
#else
    (*pRom) (startType);
#endif	/* INCLUDE_MOT_BUG_ROM */

    return (OK);		/* in case we ever continue from ROM monitor */
    }


/******************************************************************************
*
* sysHwInit2 - initialize additional system hardware
*
* This routine connects system interrupt vectors and configures any 
* required features not configured by sysHwInit().
*
* RETURNS: N/A
*/

void sysHwInit2 (void)
    {
    static BOOL configured = FALSE;

    if (!configured)
	{
#if defined(MV1600) || defined(MV1300)
	/* intconnects for various devices */
	
	/* connect VME2PCI interrupt */

	intConnect (INUM_TO_IVEC(VME2PCI_INT_VEC), vme2PciIntr, 0);

	/* interrupts can only be turned on ultimately by the PIC int ctrlr */

	intEnable (VME2PCI_INT_LVL);

#ifdef MV1600
	sysZ8536Init ();
	intConnect (INUM_TO_IVEC(Z8536_INT_VEC), sysClkIntCIO, 0);
	intEnable (Z8536_INT_LVL);
#endif  /* MV1600 */
#endif  /* MV1600 || MV1300 */

        /* connect a dummy routine for the spurious interrupt (0x07) */

        intConnect (INUM_TO_IVEC(PP_INT_LVL), sysSpuriousIntHandler, 0);

	/* initialize serial interrupts */

	sysSerialHwInit2 ();

#if	defined(CACHE_L2_CAPABLE) && defined(INCLUDE_CACHE_L2)
	/* initialize the L2 cache if available */

	sysL2CacheInit ();

#endif  /* L2 cache */
	configured = TRUE;
	}
    }


/******************************************************************************
*
* sysProcNumGet - get the processor number
*
* This routine returns the processor number for the CPU board, which is
* set with sysProcNumSet().
*
* RETURNS: The processor number for the CPU board.
*
* SEE ALSO: sysProcNumSet()
*/

int sysProcNumGet (void)
    {
    return (sysProcNum);
    }


/******************************************************************************
*
* sysProcNumSet - set the processor number
*
* This routine sets the processor number for the CPU board.  Processor numbers
* should be unique on a single backplane.
*
* RETURNS: N/A
*
* SEE ALSO: sysProcNumGet()
*
*/

void sysProcNumSet
    (
    int 	procNum			/* processor number */
    )
    {
#if defined(MV1600) || defined(MV1300)
    int 	temp;

    /* dual port the on board memory for vme accesses */


    if (DEVICE_PRESENT(MV1600_BCR_VMEP) == TRUE)
	{    
	if (procNum == 0)
	    {

	    /* map initial VME_A24_SLV_SIZE of DRAM space to VME A24 */

	    *VMECHIP2_VSAR2   =  (((VME_A24_SLV_ADRS + VME_A24_SLV_SIZE 
				    - 1) & 0xffff0000) |
				  (VME_A24_SLV_ADRS >> 16));
	    /* 
	     * 0x80000000 to 0xffffffff of PCI is mapped to DRAM
	     * The valid segment sizes for the VME to DRAM are 
	     * 64Kb, 128kb, 256kb, 512Kb, 1Mb, 2Mb, 4Mb, 8Mb, 16Mb, 32Mb,
	     * 64Mb, 128Mb, 256Mb, 512Mb, 1GB, 2 Gb, 4Gb.
	     */ 

	    *VMECHIP2_VSATR2 = (PCI2CPU_TRANSLATE (LOCAL_MEM_LOCAL_ADRS) &
				0xffff0000) |
			       ((~(VME_A24_SLV_SIZE - 1) >> 16) & 
				0x0000ffff);

	    *VMECHIP2_VSAMSR |= (VSAMSR2_SNP_INHIBIT	| 
				 VSAMSR2_WP		|
				 VSAMSR2_SUP		|
				 VSAMSR2_USR		|
				 VSAMSR2_A24		|
				 VSAMSR2_D64		|
				 VSAMSR2_BLK		|
				 VSAMSR2_PGM		|
				 VSAMSR2_DAT); /* all but A32 */

	    /* map VME_A32_SLV_SIZE of DRAM to VME A32 space */

	    *VMECHIP2_VSAR1   = (((VME_A32_SLV_ADRS + VME_A32_SLV_SIZE - 1)
				   & 0xffff0000)
				| (VME_A32_SLV_ADRS >> 16));

	    *VMECHIP2_VSATR1  = ((PCI2CPU_TRANSLATE (LOCAL_MEM_LOCAL_ADRS)) &
				 0xffff0000) |
				((~(VME_A32_SLV_SIZE - 1) >> 16) & 0x0000ffff);

	    *VMECHIP2_VSAMSR |= (VSAMSR1_SNP_INHIBIT	|
				 VSAMSR1_WP		|
				 VSAMSR1_SUP		|
				 VSAMSR1_USR		|
				 VSAMSR1_A32		|
				 VSAMSR1_D64		|
				 VSAMSR1_BLK		|
				 VSAMSR1_PGM		|
				 VSAMSR1_DAT); /* all but A24 */
	    }
	
	else
	    *VMECHIP2_VSAMSR = 0x00;	/* turn off all bits */

	if (sysProcNum < 15)
	    {
	    /* VME address of GCSR is function of processor number */

	    temp = *VMECHIP2_LBTVCR;
	    temp &= ~(0xfff00000); /* clear bits */
	    *VMECHIP2_LBTVCR = (temp | (sysProcNum << 20) |
				(GCSR_GROUP_ADDR << 24));
	    }
	}
#endif	/* MV1600 || MV1300 */

    sysProcNum = procNum;
    }


/* miscellaneous support routines */

/******************************************************************************
*
* sysLocalToBusAdrs - convert a local address to a bus address
*
* This routine gets the VMEbus address that accesses a specified local
* memory address.
*
* RETURNS: OK, or ERROR if the address space is unknown or the mapping is not
* possible.
*
* SEE ALSO: sysBusToLocalAdrs()
*/
 
STATUS sysLocalToBusAdrs
    (
    int 	adrsSpace,	/* bus address space where busAdrs resides */
    char *	localAdrs,	/* local address to convert */ 
    char **	pBusAdrs	/* where to return bus address */ 
    )
    {
#ifdef	ULTRA
    return (ERROR);
#else	/* MV1600 || MV1300 */

    if ((int)localAdrs < LOCAL_MEM_LOCAL_ADRS || localAdrs >= sysPhysMemTop ())
        {
        /* This routine pertains to on-board memory only - just return ERROR */

        return (ERROR);
        }

    /* 
     *   - this is on-board memory - map to VME bus address space;
     *   - the following memory mapping is established in sysProcNumSet():
     *   - only processor 0 has memory on bus,
     *   - the memory is placed in STD space at address VME_A24_SLV_ADRS.
     *	 - the memory is placed in EXT space at address VME_A32_SLV_ADRS   
     *   - the memory is not mapped to short I/O space.
     */
 
    switch (adrsSpace)
        {
        case VME_AM_SUP_SHORT_IO:
        case VME_AM_USR_SHORT_IO: /* no local map */
            return (ERROR);
 
        case VME_AM_STD_SUP_PGM:
        case VME_AM_STD_SUP_DATA:
        case VME_AM_STD_USR_PGM:
        case VME_AM_STD_USR_DATA:

	    /* Check if exceeding A24 */

	    if ((ULONG)localAdrs < (VME_A24_SLV_SIZE + 
				    LOCAL_MEM_LOCAL_ADRS))
		{
                *pBusAdrs = localAdrs - LOCAL_MEM_LOCAL_ADRS + VME_A24_SLV_ADRS;
                return (OK);
                }
            else
                return (ERROR);

        case VME_AM_EXT_SUP_PGM:
        case VME_AM_EXT_SUP_DATA:
        case VME_AM_EXT_USR_PGM:
        case VME_AM_EXT_USR_DATA:

	    /* Check if exceeding A32 */

	    if ((ULONG)localAdrs < (VME_A32_SLV_SIZE + 
				    LOCAL_MEM_LOCAL_ADRS))
		{
		*pBusAdrs = localAdrs - LOCAL_MEM_LOCAL_ADRS + VME_A32_SLV_ADRS;
		return (OK);
                }
            else
                return (ERROR);
	    
        default:
            return (ERROR);
        }
#endif	/* ULTRA */
    }


/******************************************************************************
*
* sysBusToLocalAdrs - convert a bus address to a local address
*
* This routine gets the local address that accesses a specified VMEbus
* address.
*
* RETURNS: OK, or ERROR if the address is unknown or the mapping is not
* possible.
*
* SEE ALSO: sysLocalToBusAdrs()
*/

STATUS sysBusToLocalAdrs
    (
    int  	adrsSpace, 	/* bus address space where busAdrs resides */
    char *	busAdrs,   	/* bus address to convert */
    char **	pLocalAdrs 	/* where to return local address */
    )
    {
#ifdef	ULTRA
    return (ERROR);
#else	/* MV1600 || MV1300 */

    switch (adrsSpace)
	{
        case VME_AM_SUP_SHORT_IO:
        case VME_AM_USR_SHORT_IO:
            if (busAdrs > (char *)0x0000ffff)
                return (ERROR);

            *pLocalAdrs = (char *) busAdrs + VME_WINDOW_A16_BASE;
            return (OK);

	case VME_AM_EXT_SUP_PGM:
	case VME_AM_EXT_USR_PGM:
	case VME_AM_EXT_SUP_DATA:
	case VME_AM_EXT_USR_DATA:
	    if ((ULONG) busAdrs < MASTER_MEM_BUS_A32
	     || (ULONG) busAdrs > (MASTER_MEM_BUS_A32 + MASTER_MEM_A32_SIZE))
	    	return (ERROR);

	    *pLocalAdrs = (char *)busAdrs + VME_WINDOW_A32_BASE
				- MASTER_MEM_BUS_A32;
	    return (OK);
	    break;

	case VME_AM_STD_SUP_PGM:
	case VME_AM_STD_USR_PGM:
	case VME_AM_STD_SUP_DATA:
	case VME_AM_STD_USR_DATA:
	    if ((ULONG) busAdrs > 0x00ffffff)
		return (ERROR);
	    *pLocalAdrs = (char *) busAdrs + VME_WINDOW_A24_BASE;

	    return (OK);
	    break;

	default:
	    return (ERROR);
	}
#endif	/* ULTRA */
    }


/*******************************************************************************
*
* sysBusTas - test and set a location across the bus
*
* This routine performs a test-and-set (TAS) instruction on the specified
* address.  To prevent deadlocks, interrupts are disabled and the VMEbus is
* locked during the test-and-set operation.
*
* NOTE: This routine is dependent upon, not equivalent to, vxTas().
* 
* RETURNS: TRUE if the value had not been set but is now, or FALSE if the 
* VMEbus cannot be locked or the value was set already.
*
* SEE ALSO: vxTas(), sysBusTasClear()
*/

BOOL sysBusTas
    (
    char * adrs          /* address to be tested and set */
    )
    {
#ifdef	ULTRA
    return (FALSE);
#else	/* !ULTRA */
    BOOL state = FALSE;  /* semaphore state */
    int  lockKey;        /* interrupt lock key */

    /* NotTimeOut(loops) = timeout(usec) / clks/loop / CPU_SPEED(clks/usec) */

    int  NotTimeOut = ((MEMORY_BUS_SPEED/1000000) * LOCK_TIMEOUT)
		      / CPU_CLOCKS_PER_LOOP;

    /* lock interrupts so there will be no TAS interference */

    lockKey = intLock ();

    /* Grab the bus to ensure atomicity */

    *VMECHIP2_DMACR1 |= DMACR1_DWB;
    EIEIO_SYNC;

    /* wait for the VME controller to give you the BUS */

    while (!VMEBUS_OWNER && NotTimeOut)
	{
	--NotTimeOut;
	}

    /* perform the TAS */

    if (VMEBUS_OWNER)
	{
	state =  vxTas (adrs);
	EIEIO_SYNC;
	}

    /* release the VME BUS */

    *VMECHIP2_DMACR1 &= ~DMACR1_DWB;

    /* unlock the interrupt */

    intUnlock (lockKey);

    /* return TAS test result */

    return (state);
#endif	/* ULTRA */
    }


/******************************************************************************
*
* sysBusTasClear - clear a location set by sysBusTas()
*
* This routine clears the specified location if it has been set by a
* call to sysBusTas().  It attempts to own the VMEbus and waits to do so
* up to 10 milliseconds.  If the bus can be owned, the location is cleared.
* If, after 10 milliseconds, the bus is not owned, the location is cleared 
* anyway.
* 
* To prevent deadlocks, interrupts are disabled and the VMEbus is
* locked during the clear operation.
*
* RETURNS: N/A
*
* SEE ALSO: sysBusTas()
*/

void sysBusTasClear
    (
    volatile char * address	/* address to be tested-and-cleared */
    )
    {
#ifndef	ULTRA
    int  lockKey;        /* interrupt lock key */

    /* NotTimeOut(loops) = timeout(usec) / clks/loop / CPU_SPEED(clks/usec) */

    int  NotTimeOut = ((MEMORY_BUS_SPEED/1000000) * UNLOCK_TIMEOUT)
		      / CPU_CLOCKS_PER_LOOP;

    lockKey = intLock ();	/* Do clearing with no interference */

    *VMECHIP2_DMACR1 |= DMACR1_DWB;             /* ask to have the VME BUS */
    EIEIO_SYNC;

    /* wait for the VME controller to give you the BUS */

    while (!VMEBUS_OWNER && NotTimeOut)
	{
	--NotTimeOut;
	}

    *address = 0;		/* Clear the location */
    EIEIO_SYNC;

    *VMECHIP2_DMACR1 &= ~DMACR1_DWB;            /* release the VME BUS */

    intUnlock (lockKey);	/* Enable normal operation */
#endif	/* ULTRA */
    } 


/******************************************************************************
*
* sysLanIntEnable - enable a LAN interrupt
*
* This routine enables interrupts at a specified level for the on-board LAN
* chip.  LAN interrupts are controlled by the ISA Bridge Control (IBC)
* chip.  The LANCE chip on this board is on the Peripheral Component
* Interconnect (PCI) bus.  The PCI interrupts should be routed through the
* IBC to the processor.  The LANCE chip asserts PCI IRQ0 which is routed
* to the IBC.  The IBC must be programmed to generate an ISA IRQ10.
*
* RETURNS: OK, always.
*
* SEE ALSO: sysLanIntDisable()
*/

STATUS sysLanIntEnable
    (
    int intLevel 		/* interrupt level to enable */
    )
    {
    intEnable (LN_INT_LVL);
    return (OK);
    }


/******************************************************************************
*
* sysLanIntDisable - disable the LAN interrupt
*
* This routine disables interrupts for the on-board LAN chip. 
*
* RETURNS: OK, always.
*
* SEE ALSO: sysLanIntEnable()
*/

STATUS sysLanIntDisable (void)
    {

    /* 
     *  disable the ISA IRQ10 for MV1600 and 
     * 		   ISA IRQ9 for Ultra in the 8259 PIC
     */

    intDisable (LN_INT_LVL);

    return (OK);
    }


/******************************************************************************
*
* sysSpuriousIntHandler - spurious interrupt handler
*
* This is the entry point for spurious interrupts.
*
* NOTE: This routine has no effect.
*
* RETURNS: N/A
*
* NOMANUAL
*/

void sysSpuriousIntHandler (void)
    {
    }


/*****************************************************************************
*
* sysInByte - read one byte from an I/O address
*
* This routine reads a single byte from a specified I/O address.
*
* RETURNS: The byte from the specified address.
*/

UCHAR sysInByte
    (
    ULONG	address		/* I/O address to read the byte from */
    )
    {
    return (*(UCHAR *)address);
    }


/******************************************************************************
*
* sysOutByte - write one byte to an I/O address
*
* This routine writes a single byte to a specified I/O address.
*
* RETURNS: N/A
*/

void sysOutByte
    (
    ULONG	address,	/* I/O address to write the byte to */
    char	data		/* byte to write */
    )
    {
    *(UCHAR *)address = data;
    EIEIO;
    }


/******************************************************************************
*
* sysNvRead - read one byte from NVRAM
*
* This routine reads a single byte from a specified offset in NVRAM.
*
* RETURNS: The byte from the specified NVRAM offset.
*/

UCHAR sysNvRead
    (
    ULONG	offset	/* NVRAM offset to read the byte from */
    )
    {
    sysOutByte (NV_RAM_LSB_REG, LSB(offset));
    sysOutByte (NV_RAM_MSB_REG, MSB(offset));
    return sysInByte (NV_RAM_DAT_REG);
    }


/******************************************************************************
*
* sysNvWrite - write one byte to NVRAM
*
* This routine writes a single byte to a specified offset in NVRAM.  The
* MVME1600 and Ultra series use a 48T18 device with fast write times.  The
* MVME1300 series uses a 28C64 device with slow write times.
*
* RETURNS: N/A
*/

void sysNvWrite
    (
    ULONG	offset,	/* NVRAM offset to write the byte to */
    UCHAR	data	/* datum byte */
    )
    {
    sysOutByte (NV_RAM_LSB_REG, LSB(offset));
    sysOutByte (NV_RAM_MSB_REG, MSB(offset));
    sysOutByte (NV_RAM_DAT_REG, data);

#ifdef MV1300
    {
    int delay = NV_RAM_DELAY;

    while (delay && data != sysInByte (NV_RAM_DAT_REG))
	--delay;
    }
#endif  /* MV1300 */
    }


/*******************************************************************************
*
* sysCpuCheck - confirm the CPU type
*
* This routine checks for the appropriate CPU type.
*
* RETURNS: N/A
*/

void sysCpuCheck (void)
    {
    int msgSize;
    int msgIx;
    SIO_CHAN * pSioChan;        /* serial I/O channel */

#if	(CPU == PPC603)
    if	((CPU_TYPE != CPU_TYPE_603) && (CPU_TYPE != CPU_TYPE_603E) && 
	 (CPU_TYPE != CPU_TYPE_603EV))
#else	/* (CPU == PPC604) */
    if	((CPU_TYPE != CPU_TYPE_604) && (CPU_TYPE != CPU_TYPE_604E))
#endif	/* (CPU == PPC604) */
	{

	msgSize = strlen (wrongCpuMsg);

	sysSerialHwInit ();

	pSioChan = sysSerialChanGet (0);

	sioIoctl (pSioChan, SIO_BAUD_SET, (void *)CONSOLE_BAUD_RATE);

	sioIoctl (pSioChan, SIO_MODE_SET, (void *) SIO_MODE_POLL);

	for (msgIx = 0; msgIx < msgSize; msgIx++)
	    {
	    while (sioPollOutput (pSioChan, wrongCpuMsg[msgIx]) == EAGAIN);
	    }

	sysToMonitor (BOOT_NO_AUTOBOOT);
	}
    }


/******************************************************************************
*
* sysPicrGet - get the value of a specified PICR 
*
* This routine reads the value of the specified Processor Interface
* Configuration Register (PICR) on the MPC105.
*
* RETURNS: The value of the specified PICR.
*
* SEE ALSO: sysPicrSet()
*/ 

int sysPicrGet
    (
    int picrReg		/* Processor Interface Config. Register to get */
    )
    {
    int val;		/* value from register */

    EIEIO;
    MPC_WRITE_LONG (MPC105_REG_SELECT, MPC105_REF(picrReg));
    EIEIO;
    MPC_READ_LONG (MPC105_REG_DATA, &val);

    return (val);
    }

/******************************************************************************
*
* sysPicrSet - set the value of a specified PICR 
*
* This routine writes the value of the specified Processor Interface
* Configuration Register (PICR) on the MPC105.
*
* RETURNS: OK, always.
*
* SEE ALSO: sysPicrGet()
*/ 

STATUS sysPicrSet
    (
    int picrReg,	/* Processor Interface Config. Register to set */
    int val		/* value to place in register */
    )
    {
    EIEIO;
    MPC_WRITE_LONG (MPC105_REG_SELECT, MPC105_REF(picrReg));
    EIEIO;
    MPC_WRITE_LONG (MPC105_REG_DATA, val);

    return (OK);
    }

#ifdef	INCLUDE_FLASH
/******************************************************************************
*
* sysFlashWriteEnable - enable writing to flash memory
*
* This routine sets the flash write-enable bit in Processor Interface
* Configuration Register 1 of the MPC105.
*
* RETURNS: N/A 
*
* SEE ALSO: sysFlashWriteDisable()
*/

void sysFlashWriteEnable (void)
    {
    /* enable r/w */
    sysPicrSet (MPC105_PICR1, sysPicrGet(MPC105_PICR1) | PICR1_FLASH_WR_EN);
    }


/******************************************************************************
*
* sysFlashWriteDisable - disable writing to flash memory
*
* This routine disables the flash write-enable bit in Processor Interface
* Configuration Register 1 of the MPC105.
*
* RETURNS: N/A 
*
* SEE ALSO: sysFlashWriteEnable()
*/

void sysFlashWriteDisable (void)
    {
    /* enable read only */
    sysPicrSet (MPC105_PICR1, sysPicrGet(MPC105_PICR1) & ~PICR1_FLASH_WR_EN);
    }
#endif  /* INCLUDE_FLASH */

