/* $RCSfile: matrix.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/algebra/matrix.cc,v 1.1.1.1 1998/06/11 21:36:19 chara Exp $
*/

#ifndef	INC_MATRIXHPP
#define	INC_MATRIXHPP

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>

#include "isit.h"
#include "tVector.h"
#include "matrix.h"

#define PYTHAG(a,b) ((at=fabs(a)) > (bt=fabs(b)) ? \
(ct=bt/at,at*sqrt(1.0+ct*ct)) : (bt ? (ct=at/bt,bt*sqrt(1.0+ct*ct)): 0.0))

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))


//this function is called whenever an error is detected
inline void error(char* message)
{
	TraceMsg(tmt_ERROR,S_vector_BAD_MATRIX_OPERATION,message);
}


tMatrix::tMatrix(void) {
	size=0;
	pointer=NULL;
}

tMatrix::tMatrix(const int numRows)	{
	size = numRows;
	if (size !=	0)
		pointer	=	new	tVector[numRows];
	else
		pointer	=	NULL;
}


tMatrix::tMatrix(const int numRows,	const	int	numCols) {
	int	i;
	size = numRows;
	colSize	=	numCols;
	if (size !=	0) {
		pointer	=	new	tVector[numRows];
		for	(i=0;	i<size;	i++)
			pointer[i] = tVector(numCols);
	}
	else
		pointer	=	NULL;
}


tMatrix::tMatrix(const tMatrix	&Matrix);


tMatrix::tMatrix(const tVector	&Vector) {
	int	i;
	size = Vector.Dimension();
	colSize	=	1;
	if (size !=	0) {
		pointer	=	new	tVector[size];
		for	(i=0;	i<size;	i++) {
			pointer[i] = tVector(colSize);
			pointer[i][0]	=	Vector[i];
		}
	}
	else
		pointer	=	NULL;
}


tMatrix& tMatrix:: addrow(void)	{
	register int i;
	// This	is really	painful, but realloc doesn't seem	to work
	// Copy	all	the	old	stuff	into a new array
	tVector *newPointer = new tVector[size+1];
	for	(i=0;	i<size;	i++)
		newPointer[i]	=	pointer[i];
	// ... purge the old stuff
	delete[] pointer;
	// and initialize	the	new	member
	size++;
	pointer	=	newPointer;
	pointer[size-1]	=	tVector(colSize);

	return *this;
}



void tMatrix:: print(void) const {
	int	i;
	printf("[\n");
	for	(i=0;i<size;i++)
		pointer[i].print();
	printf("]\n");
}


void tMatrix::input(void) {
	int	i;
	tVector d;
	if (size ==	0) {
		printf("Enter	number of	rows of	tMatrix:	");
		scanf("%d",&size);
	}
	for	(i=0;	i<size;	i++) {
		printf("****\nEnter	row	%d out of	%d:\n",i+1,size);
		pointer[i].input();
	}
}


void tMatrix::clear(void) {
	int	i;
	for	(i=0;	i<size;	i++)
		pointer[i].clear();
}


tVector& tMatrix::operator[] (const int element) {
	if (element	>	size-1 ||	element<0) {
		error("Error - attempted access to non-present tMatrix row\n");
		return(pointer[0]);
	}
	return (pointer[element]);
}

const tVector& tMatrix::operator[] (const int element) const {
	if (element	>	size-1 ||	element<0) {
		error("Error - attempted access to non-present tMatrix row\n");
		return(pointer[0]);
	}
	return (pointer[element]);
}

tVector tMatrix::make_tVector(void) const	{
	if (rows() ==	1) {
		return pointer[0];
	}
	else if	(columns() ==	1) {
		return transpose()[0];
	}
	else {
		error("Error	-	Cannot cast	tMatrix to	tVector");
		return tVector();
	}
}



tMatrix tMatrix::operator* (const	tMatrix& Matrix) const {
	if (size ==	0	|| Matrix.size ==	0) {
		error("Error	-	multiplication with	empty	tMatrix\n");
		return tMatrix();
	}
	if (columns()	!= Matrix.rows())	{
		error("Error	-	multiplication of	incompatible matrices\n");
		return tMatrix();
	}
	tMatrix result(rows(),Matrix.columns());
	int	i,j,k;
	NUMBER sum;
	for	(i=0;	i<rows();	i++)
		for	(j=0;	j<(Matrix.columns());	j++) {
			sum	=	0;
			for	(k=0;	k<(Matrix.rows()); k++)
				sum	+= pointer[i][k] * Matrix[k][j];
			result[i][j] = sum;
		}
	return result;
}


tVector tMatrix::operator* (const	tVector& Vector) const {
	if (size ==	0	|| Vector.Dimension()	== 0)	{
		error("Error	-	multiplication with	empty	tMatrix\n");
		return tVector();
	}
	if (columns()	!= Vector.Dimension()) {
		error("Error	-	multiplication of	incompatible matrices\n");
		return tVector();
	}
	tVector result(rows());
	int	i,k;
	NUMBER sum;
	for	(i=0;	i<rows();	i++) {
		sum	=	0;
		for	(k=0;	k<(columns()); k++)
			sum	+= pointer[i][k] * Vector[k];
		result[i]	=	sum;
	}
	return result;
}


tMatrix tMatrix::operator+ (const	tMatrix& Matrix) const {
	return tMatrix(*this) += Matrix;	
}

tMatrix& tMatrix::operator+=	(const tMatrix	&	Matrix)	{
	if (rows() !=	Matrix.rows()	|| columns() !=	Matrix.columns())	{
		error("Error	-	tMatrix sum size	mismatch\n");
		return(*this);
	}
	if (size ==	0) return	*this;
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] +=	Matrix[i];
	return(*this);
}

tMatrix tMatrix::operator- (const	tMatrix& Matrix) const {
	return tMatrix(*this) -= Matrix;	
}

tMatrix& tMatrix:: operator-=	(const tMatrix	&	Matrix)	{
	if (rows() !=	Matrix.rows()	|| columns() !=	Matrix.columns())	{
		error("Error	-	tMatrix difference	size mismatch\n");
		return(*this);
	}
	if (size ==	0) return	*this;
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] -=	Matrix[i];
	return(*this);
}

tMatrix tMatrix:: operator- ()	{
	int	i;
	tMatrix sum(size);
	for	(i=0;i<size;i++)
		sum[i] = -pointer[i];
	return(sum);
}

tMatrix tMatrix:: operator+(const NUMBER	r) const {
	return tMatrix(*this) += r;
}

tMatrix tMatrix:: operator-(const NUMBER	r) const {
	return tMatrix(*this) -= r;
}

tMatrix tMatrix:: operator*(const NUMBER	r) const {
	return tMatrix(*this) *= r;
}

tMatrix tMatrix:: operator/(const NUMBER	r) const {
	return tMatrix(*this) /= r;
}

tMatrix& tMatrix:: operator+=(const	NUMBER r)	{
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] +=	r;
	return(*this);
}

tMatrix& tMatrix:: operator-=(const	NUMBER r)	{
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] -=	r;
	return(*this);
}

tMatrix& tMatrix:: operator*=(const	NUMBER r)	{
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] *=	r;
	return(*this);
}

tMatrix& tMatrix:: operator/=(const	NUMBER r)	{
	int	i;
	for	(i=0;i<size;i++)
		pointer[i] /=	r;
	return(*this);
}

tMatrix::~tMatrix()	{
	 if	(size	>	0)
		 delete[]	pointer;
}


//LU back substitution
void tMatrix::lubksb(tVector& indx, tVector& b)
{
   int n=size;
	int i,ii=-1,ip,j;
	float sum;

	for (i=0;i<n;i++) {
		ip=(int)indx[i];
		sum=b[ip];
		b[ip]=b[i];
		if (ii>=0)
			for (j=ii;j<=i-1;j++) sum -= pointer[i][j]*b[j];
		else if (sum) ii=i;
		b[i]=sum;
	}
	for (i=n-1;i>=0;i--) {
		sum=b[i];
		for (j=i+1;j<n;j++) sum -= pointer[i][j]*b[j];
		b[i]=sum/pointer[i][i];
	}
}


// returns the lu decomposition of the tMatrix in place of the tMatrix.
// The old tMatrix is lost.
// The tVector indx should be of the same dimension as the tMatrix.  It contains
// the row exchange permutation on return.
void tMatrix::ludcmp(tVector & indx,NUMBER & d)
{
   int n=size;
	int i,imax,j,k;
	NUMBER big,dum,sum,temp;

	tVector vv(n);
	d=1.0;
	for (i=0;i<n;i++) {
		big=0.0;
		for (j=0;j<n;j++)
			if ((temp=fabs(pointer[i][j])) > big) big=temp;
		if (big == 0.0) error("Singular tMatrix in routine LUDCMP");
		vv[i]=1.0/big;
	}
	for (j=0;j<n;j++) {
		for (i=0;i<j;i++) {
			sum=pointer[i][j];
			for (k=0;k<i;k++) sum -= pointer[i][k]*pointer[k][j];
			pointer[i][j]=sum;
		}
		big=0.0;
		for (i=j;i<n;i++) {
			sum=pointer[i][j];
			for (k=0;k<j;k++)
				sum -= pointer[i][k]*pointer[k][j];
			pointer[i][j]=sum;
			if ( (dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=0;k<n;k++) {
				dum=pointer[imax][k];
				pointer[imax][k]=pointer[j][k];
				pointer[j][k]=dum;
			}
			d = -d;
			vv[imax]=vv[j];
		}
		indx[j]=imax;
		if (pointer[j][j] == 0.0) pointer[j][j] = DBL_EPSILON;
		if (j != n) {
			dum=1.0/(pointer[j][j]);
			for (i=j+1;i<n;i++) pointer[i][j] *= dum;
		}
	}
}


//singular value back substitution
tVector & tMatrix::svbksb(const tVector & w, const tMatrix & v, const tVector & b) const
{
	int m,n;
	int jj,j,i;
	NUMBER s;

	m = rows();
	n = columns();
	
	tVector & x = *(new tVector(n));   // Allocate space for return
	const tMatrix& u = *this;          // Because u is prettier than "this"
	tVector tmp(n);

	for (j=0;j<n;j++) {
		s=0.0;
		if (w[j]) {
			for (i=0;i<m;i++) s += u[i][j]*b[i];
			s /= w[j];
		}
		tmp[j]=s;
	}
	for (j=0;j<n;j++) {
		s=0.0;
		for (jj=0;jj<n;jj++) s += v[j][jj]*tmp[jj];
		x[j]=s;
	}
	return x;
}

//singular value decomposition
void tMatrix::svdcmp(tVector & w, tMatrix & v)
{
   static NUMBER at,bt,ct;
   static NUMBER maxarg1,maxarg2;
	int flag,i,its,j,jj,k,l,nm, m, n;
	NUMBER c,f,h,s,x,y,z;
	NUMBER anorm=0.0,g=0.0,scale=0.0;

	m = rows();
	n = columns();
	if (m < n) error("SVDCMP: You must augment A with extra zero rows");

	tMatrix & a = *this;        // Because "a" is prettier than "this"
	tVector rv1(n);
	w = tVector(n);
	v = tMatrix(n,n);
	
	for (i=0;i<n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i < m) {
			for (k=i;k<m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				if (i != n) {
					for (j=l;j<n;j++) {
						for (s=0.0,k=i;k<m;k++) s += a[k][i]*a[k][j];
						f=s/h;
						for (k=i;k<m;k++) a[k][j] += f*a[k][i];
					}
				}
				for (k=i;k<m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale*g;
		g=s=scale=0.0;
		if (i < m && i != n-1) {
			for (k=l;k<n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<n;k++) rv1[k]=a[i][k]/h;
				if (i != m-1) {
					for (j=l;j<m;j++) {
						for (s=0.0,k=l;k<n;k++) s += a[j][k]*a[i][k];
						for (k=l;k<n;k++) a[j][k] += s*rv1[k];
					}
				}
				for (k=l;k<n;k++) a[i][k] *= scale;
			}
		}
		anorm=MAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n-1;i>=0;i--) {
		if (i < n-1) {
			if (g) {
				for (j=l;j<n;j++)
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<n;j++) {
					for (s=0.0,k=l;k<n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=n-1;i>=0;i--) {
		l=i+1;
		g=w[i];
		if (i < n-1)
			for (j=l;j<n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			if (i != n-1) {
				for (j=l;j<n;j++) {
					for (s=0.0,k=l;k<m;k++) s += a[k][i]*a[k][j];
					f=(s/a[i][i])*g;
					for (k=i;k<m;k++) a[k][j] += f*a[k][i];
				}
			}
			for (j=i;j<m;j++) a[j][i] *= g;
		} else {
			for (j=i;j<m;j++) a[j][i]=0.0;
		}
		++a[i][i];
	}
	for (k=n-1;k>=0;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=0;l--) {
				nm=l-1;
				if ((NUMBER)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((NUMBER)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((NUMBER)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=PYTHAG(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s=(-f*h);
					for (j=0;j<m;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=0;j<n;j++) v[j][k]=(-v[j][k]);
				}
				break;
			}
			if (its == 30) error("No convergence in 30 SVDCMP iterations");
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=PYTHAG(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=PYTHAG(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g=g*c-x*s;
				h=y*s;
				y=y*c;
				for (jj=0;jj<n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=PYTHAG(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=(c*g)+(s*y);
				x=(c*y)-(s*g);
				for (jj=0;jj<m;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
}


tMatrix::tMatrix(const tMatrix &Matrix) {
  int i;
  size = Matrix.size;
  colSize = Matrix.colSize;
  pointer = new tVector[size];
  for (i=0; i<size; i++)
    pointer[i] = Matrix.pointer[i];   
}

tMatrix & tMatrix::operator= (const tMatrix &Matrix) {
  int i;

  if (this == &Matrix) return (*this);
  
  if (pointer && size != Matrix.size) {
    delete[] pointer;
    size = Matrix.size;
    colSize = Matrix.colSize;
    pointer = new tVector[size];
  }
  else if (pointer == NULL) {
    size = Matrix.size;
    colSize = Matrix.colSize;
    pointer = new tVector[size];
  }
      
  for (i=0; i<size; i++)
    pointer[i] = Matrix.pointer[i];
  return (*this);
}


tVector & tMatrix::LeastSquares(tVector & b, tVector & errorVec) {
  int m, n;
  
  m = rows();
  n = columns();
  tVector & result = *(new tVector(n));
  
  if (m != b.Dimension()) {
    fprintf(stderr, "Error: Incompatible dimensions for LeastSquares\n");
    return result;
  }
  
  tMatrix A = *this;
  tVector w;
  tMatrix v;
  
  A.svdcmp(w, v);
  result = A.svbksb(w, v, b);
  
  errorVec = b - (*this)*result;
  
  return result;
}
  

NUMBER tMatrix::det() const {
  if (rows() != columns()) {
    printf("Error - detetminant of non-square tMatrix\n");
    return 0.0;
  }
  if (rows()<1) {
    printf("Error - determinant of zero-dimension tMatrix\n");
    return 0.0;
  }
  NUMBER d;
  tMatrix this_copy = *this;
  tVector Index(rows());
  this_copy.ludcmp(Index,d);
  int i;
  for (i=0; i<rows(); i++)
    d *= this_copy[i][i];
  return d;
}

tMatrix tMatrix::transpose(void) const
{
  if (size == 0) return tMatrix();
  int i,j;
  tMatrix trn(colSize,size);
  for (i=0;i<size;i++)
    for (j=0;j<colSize;j++)
      trn[j][i] = pointer[i][j];
  return trn;
}

tMatrix tMatrix::inverse(void) const
{
  if (rows() != columns()) {
    printf("Error - inverse of non-square tMatrix\n");
    return *this;
  }
  if (rows()<1) {
    printf("Error - inverse of zero-dimension tMatrix\n");
    return *this;
  }
  // Perform the inversion using the LU method on page 38 of Num. Rec.
  int j;
  NUMBER d;
  tMatrix this_copy = *this;
  tVector Index(rows());
  this_copy.ludcmp(Index,d);
  tMatrix y=Identity(rows());   //Identity tMatrix for use in back substitution
  for (j=0; j<rows(); j++)
    this_copy.lubksb(Index,y[j]);
  return(y.transpose());   // lubksb returns result in rows of y rather than columns
}




tMatrix operator*(const NUMBER r, const tMatrix & Matrix) {
  return (Matrix*r);
}

// returns an identity tMatrix of the specified size
tMatrix Identity(const int dimension) {
  int i,j;
  tMatrix identity(dimension,dimension);
  for (i=0; i<dimension; i++)
    for (j=0; j<dimension; j++)
      identity[i][j] = ( i==j ? 1 : 0 );
  return identity;
}


#endif
