/* $RCSfile: tVector.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/algebra/tVector.cc,v 1.1.1.1 1998/06/11 21:36:19 chara Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "tVector.h"
#include "trace.h"
#include "vtError.h"

typedef  NUMBER type;

//this function is called whenever an error is detected
inline void error(char* message)
{
   TraceMsg(tmt_ERROR,S_vector_BAD_VECTOR_OPERATION,message);
}


// Initialization of statics
const char *const tVector::defaultFormat = "%10.6lf ";
const char *tVector::pFmt = "%10.6lf ";


tVector::tVector(void) {
   size=0;
   pointer=NULL;
}


tVector::tVector(const int numElements)   {
   size = numElements;
   pointer  =  new   type[size];
}


tVector::tVector(const tVector& Vector) {
   int   i;
   size = Vector.size;
   pointer  =  new   type[size];
   for   (i=0; i<size;  i++)
      pointer[i] = Vector.pointer[i];     
}

tVector& tVector::addelt(void)   {
   register int i;
   // This  is really   painful, but realloc doesn't seem   to work
   // Copy  all   the   old   stuff into a new array
   type *newPointer = new type[size+1];
   for   (i=0; i<size;  i++)
      newPointer[i]  =  pointer[i];
   // ... purge the old stuff
   delete[] pointer;
   // and initialize the   new   member
   size++;
   pointer  =  newPointer;
   pointer[size-1]   =  0.0;

   return *this;
}

tVector& tVector::addelt(type x) {
   addelt();
   pointer[size-1]   =  x;
   return *this;
}

tVector& tVector::operator= (const  tVector& Vector)  {
   int   i;
   
   if (this == &Vector) return   (*this);
   
   if (pointer && size != Vector.size) {
      delete[] pointer;
      size = Vector.size;
      pointer  =  new  type[size];
   }
   else if (pointer == NULL) {
      size = Vector.size;
      pointer  =  new  type[size];
   }
   for   (i=0; i<size;  i++)
      pointer[i] = Vector.pointer[i];      
   return (*this);
}

void tVector::print(void) const {
   int   i;
   printf("[   ");
   for   (i=0;i<size;i++)
      printf(pFmt,pointer[i]);
   printf("]\n");
}

void tVector::input(void) {
   int   i;
   type d;
   if (size == 0) {
      printf("Enter  size of  tVector: ");
      scanf("%d",&size);
      pointer  =  new   type[size];
   }
   for   (i=0;i<size;i++) {
      printf("Enter  element  %d out of   %d:   ",i+1,size);
      scanf("%lf",&pointer[i]);
   }
}

void tVector::clear(void) {
   int   i;
   for   (i=0; i<size;  i++)
      pointer[i] = 0.0;
}


type& tVector::operator[] (const int element) {
   if (element >  size-1 ||   element<0) {
      error("Error   -  attempted   access to   non-present tVector element\n");
      return(pointer[0]);
   }
   return (pointer[element]);
}

const type& tVector::operator[] (const int element) const {
   if (element >  size-1 ||   element<0) {
      error("Error   -  attempted   access to   non-present tVector element\n");
      return(pointer[0]);
   }
   return (pointer[element]);
}


// Magnitude
NUMBER tVector::mag(void) const {
   if (size < 1)  {
      error("Error   -  magnitude   of zero-dimension tVector\n");
      return(0);
   }
   int   i;
   NUMBER result=0.0;
   for   (i=0; i<size;  i++)
      result +=   pointer[i]*pointer[i];
   result = sqrt(result);
   return result;
}


// Sum
NUMBER tVector::sum(void) const {
   if (size < 1)  {
      error("Error   -  sum   of zero-dimension tVector\n");
      return(0);
   }
   int   i;
   NUMBER result=0.0;
   for   (i=0; i<size;  i++)
      result +=   pointer[i];
   return result;
}


// Average
NUMBER tVector::avg(void) const {
   if (size < 1)  {
      error("Error   -  average  of zero-dimension tVector\n");
      return(0);
   }
   int   i;
   NUMBER result=0.0;
   result = sum()/size;
   return result;
}


// Absolute value
tVector tVector::abs(void) const {
   if (size < 1)  {
      error("Error - sum of zero-dimension tVector\n");
      return(tVector(0));
   }
   int   i;
   tVector result(size);
   for   (i=0;i<size;i++)
      result[i]   =  fabs(result[i]);
   return result;
}


// Sort  (low-to-high)
// Bubble   sort for now.
// Returns a tVector of permuted indices
tVector tVector::sort(void)   {
   if (size < 1)  {
      error("Error   -  sort of  zero-dimension tVector\n");
      return(tVector(0));
   }
   register int i,   j, itmp;
   NUMBER tmp;
   tVector indices(size);
   
   for   (i=0; i<size;  i++)
      indices[i] = i;
   
   for   (i=0; i<size-1;   i++)
      for(j=i+1; j<size; j++)
         if (pointer[j] < pointer[i]) {
            tmp = indices[j];          tmp   =  pointer[j];
            indices[j] = indices[i];   pointer[j] = pointer[i];
            indices[i] = tmp;          pointer[i] = tmp;
         }
   return indices;
}


/* Dot product */
NUMBER tVector::operator* (const tVector& Vector) const {
    if (size != Vector.size) {
      error("Error - vector dot product size mismatch\n");
      return(0);
    }
    int i;
    NUMBER sum=0;
    for (i=0; i<size; i++)
      sum += pointer[i] * Vector[i];
    return sum;
}


tVector tVector::operator+ (const tVector& Vector) const {
   return tVector(*this) += Vector; 
}


tVector& tVector::operator+= (const tVector& Vector)  {
   if (size != Vector.size) {
      error("Error   -  tVector sum size  mismatch\n");
      return(*this);
   }
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] +=  Vector[i];
   return *this;
}


tVector tVector::operator- (const tVector& Vector) const {
   return tVector(*this) -= Vector; 
}


tVector& tVector::operator-= (const tVector& Vector)  {
   if (size != Vector.size) {
      error("Error   -  tVector difference   size mismatch\n");
      return *this;
   }
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] -=  Vector[i];
   return *this;
}


tVector tVector::operator-()  {
   int   i;
   tVector sum(size);
   for   (i=0;i<size;i++)
      sum[i] = -pointer[i];
   return sum;
}

tVector tVector::operator+(const NUMBER   r) const {
   return tVector(*this) += r;   
}

tVector tVector::operator-(const NUMBER   r) const {
   return tVector(*this) -= r;   
}

tVector tVector::operator*(const NUMBER   r) const {
   return tVector(*this) *= r;   
}

tVector tVector::operator/(const NUMBER   r) const {
   return tVector(*this) /= r;   
}

tVector& tVector::operator+=(const NUMBER r) {
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] +=  r;
   return *this;
}

tVector& tVector::operator-=(const NUMBER r) {
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] -=  r;
   return *this;
}

tVector& tVector::operator*=(const NUMBER r) {
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] *=  r;
   return *this;
}

tVector& tVector::operator/=(const NUMBER r) {
   int   i;
   for   (i=0;i<size;i++)
      pointer[i] /=  r;
   return *this;
}

tVector::~tVector()  {
    delete[] pointer;
}

tVector operator*(const NUMBER r, const tVector& Vector) {
  return (Vector*r);
}

tVector cross(const tVector& v1, const tVector & v2) {
  if (v1.Dimension() != 3 || v2.Dimension() != 3) {
    printf("Error - Operands to a cross product must have dimension 3\n");
    return tVector();
  }
  tVector prod(3);
  prod[0] = v1[1]*v2[2] - v2[1]*v1[2];
  prod[1] = v2[0]*v1[2] - v1[0]*v2[2];
  prod[2] = v1[0]*v2[1] - v2[0]*v1[1];
  return prod;
}

NUMBER angle(const tVector & v1, const tVector & v2) {
  NUMBER magprod = v1.mag() * v2.mag();

  NUMBER sin_theta = cross(v1,v2).mag() / magprod;
  NUMBER cos_theta = v1*v2 / magprod;
  
  return atan2(sin_theta, cos_theta);
}


tPoint::tPoint(const tVector& v) : tVector(2)
{
   if (v.Dimension() != 2) {
      error("Error - tVector to tPoint conversion, size mismatch");
   } 
   else {
      x() = v[0];
      y() = v[1];
   }
}

tVelocity::tVelocity(const tVector& v) : tVector(2)
{
   if (v.Dimension() != 2) {
      error("Error - tVector to tVelocity conversion, size mismatch");
   } 
   else {
      xDot() = v[0];
      yDot() = v[1];
   }
}

