/* $RCSfile*/

/*
$Header: /home/chara/cvsroot/ople/sys/comm/image/image.cc,v 1.1.1.1 1998/06/11 21:36:19 chara Exp $
*/

#include "image.h"
#include "hwError.h"


/****************************************************************************

class tImage

DESCRIPTION:
   tImage holds images that are stored by the Image Channels.  tImage is an 
abstract data type that holds and allows manipulation of images.

State Box
----------
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
this->pImage - a pointer to a signle dimensional array that holds the image
this->szBufferSize - the size of the array pointed to by pImage (in bytes)

MAPPING
State Box                       Black Box
-----------------------------------------------------
this->iHSize                    this->iHSize
this->iVSize                    this->iVSize
this->pImage			this->Image
this->szBufferSize		size_of(this->Image)

wild_pointer (this->pImage)     invalid Image
this->pImage = NULL             NULL Image

****************************************************************************/

/****************************************************************************
  tImage::tImage();
     Constructor

[[U tImage::tImage () ]]
*/
tImage::tImage() {
  /****
  Base Class Constructor
  [[U tBuffer::tBuffer() ]]
  ****/
  this->iVSize = 0;
  this->iHSize = 0;
  this->szBufferSize = 0;
  this->pImage = NULL;

  TraceMark(S_tImage_DONE_CONSTRUCTING);
  }
  
/****************************************************************************
  tImage::tImage(tImage &Image);
     Copy Constructor

[[U tImage::tImage_t (Image) ]]
*/
tImage::tImage(tImage &Image) {
  /****
  Base Class Constructor
  [[U tBuffer::tBuffer() ]]
  ****/

  /****
  [[U tImage::Copy(Image) ]]
  ****/
  this->Copy(Image);

  TraceMark(S_tImage_DONE_CONSTRUCTING);
  }

/****************************************************************************
  tImage::tImage (int iHSize, int iVSize);
     Constructor for setting size of the image.

[[U tImage::tImage_ii (iHSize, iVSize) ]]
*/
tImage::tImage(int iHSize, int iVSize) {
  /****
  Base Class Constructor
  [[U tBuffer::tBuffer() ]]
  ****/
  this->iVSize = iVSize;
  this->iHSize = iHSize;
  this->szBufferSize = iHSize * iVSize * sizeof(PIXEL);
  this->pImage = new PIXEL[iHSize * iVSize];

  TraceMark(S_tImage_DONE_CONSTRUCTING);
  }

/****************************************************************************
  tImage::~tImage();
     Destructor - Frees any memory used by tImage

[[U tImage::~tImage () ]]
*/
tImage::~tImage() {
  /****
  [[U tImage::Release() ]]
  ****/
  this->Release();  

  TraceMark(S_tImage_DONE_CONSTRUCTING);
  }

/****************************************************************************
  PIXEL &tImage::GetPixel(int x, int y);
     Method to get access to image held by tImage, one pixel at a time.
    
[[U tImage::GetPixel (x, y) ]]
*/
PIXEL &tImage::GetPixel(int x, int y) {
  static PIXEL BadPixel;
  int pos;

  BadPixel = 0;
  if (this->pImage == NULL) {
    Traceback(S_tImage_IMAGE_NOT_YET_LOADED, "tImage::GetPixel");
    return BadPixel;
    }
  if ((x < 0) || (x >= this->iHSize)) {
    Traceback(S_tImage_INVALID_POINT, "tImage::GetPixel", this->iHSize, this->iVSize, x, y);
    return BadPixel;
    }
  if ((y < 0) || (y >= this->iVSize)) {
    Traceback(S_tImage_INVALID_POINT, "tImage::GetPixel", this->iHSize, this->iVSize, x, y);
    return BadPixel;
    }
  pos = y * this->iHSize + x;
  return this->pImage[pos]; 
  }
  
  
/****************************************************************************
  PIXEL &tImage::GetPixel(int x, int y);
     Method to get access to image held by tImage, one pixel at a time.
    
[[U tImage::GetPixel (x, y) ]]
*/
const PIXEL &tImage::GetPixel(int x, int y) const {
  static PIXEL BadPixel;
  int pos;
  
  BadPixel = 0;
  if (this->pImage == NULL) {
    Traceback(S_tImage_IMAGE_NOT_YET_LOADED, "tImage::GetPixel");
    return BadPixel;
    }
  if ((x < 0) || (x >= this->iHSize)) {
    Traceback(S_tImage_INVALID_POINT, "tImage::GetPixel", this->iHSize, this->iVSize, x, y);
    return BadPixel;
    }
  if ((y < 0) || (y >= this->iVSize)) {
    Traceback(S_tImage_INVALID_POINT, "tImage::GetPixel", this->iHSize, this->iVSize, x, y);
    return BadPixel;
    }

  pos = y * this->iHSize + x;
  return this->pImage[pos]; 
  }


/****************************************************************************
  ERROR_CODE tImage::GetDimensions (int &iHSize, int &iVSize);
     Method for getting size of the image.
     
[[U tImage::GetDimensions (iHSize, iVSize) ]]
*/
ERROR_CODE tImage::GetDimensions (int &iHSize, int &iVSize) const {
  iHSize = this->iHSize;
  iVSize = this->iVSize;
  return OK;
  }

/****************************************************************************
  ERROR_CODE tImage::StoreImage(int iHSize, int iVSize, const PIXEL *pImageArray);
     Creates a image used by tImage.  This makes a copy of the data 
     in pImageArray.  pImageArray is assumed to be an array of pointers 
     to rows of pixels of the image.

[[U tImage::StoreImage (iHSize, iVSize, pImageArray) ]]
*/
ERROR_CODE tImage::StoreImage(int iHSize, int iVSize, const PIXEL *pImageArray[]) {
  ERROR_CODE ec1, ec2;
  int iSize;
  int x, y;
  int pos;
  
  /****
  [[U this->tImage::IsSameSize (iHSize, iVSize) ]]
  ****/
  if (this->IsSameSize(iHSize, iVSize))
    ec1 = OK;
  else
    ec1 = S_tImage_RESIZING_IMAGE;

  /****
  [[U ec2 := this->tImage::ResizeImage (iHSize, iVSize) ]]
  ****/
  ec2 = this->ResizeImage(iHSize, iVSize);
  if (ec2 != OK) {
    Traceback (ec2, "tImage::StoreImage", iHSize, iVSize);
    return ec2;
    }
  for (y = 0; y < iVSize; y++) {
    for (x = 0; x < iHSize; x++) {
      pos = y * iHSize + x;
      this->pImage[pos] = pImageArray[y][x];
      }
    }
  TraceMark(S_tImage_IMAGE_LOADED);
  return ec1;
  }

/****************************************************************************
  ERROR_CODE tImage::ResizeImage (int iHSize, int iVSize);
     Creates an image of correct size.  The image that will 
     be held by tImage is unknown but storage will exist to
     hold an image.

[[U tImage::ResizeImage (int iHSize, int iVSize) ]]
*/
ERROR_CODE tImage::ResizeImage(int iHSize, int iVSize) {
  if ((this->iVSize != iVSize) || (this->iHSize != iHSize)) {
    TraceMark(S_tImage_RESIZING_IMAGE);
    /****
    [[U tImage::Release() ]]
    ****/
    this->Release();
    
    this->iVSize = iVSize;
    this->iHSize = iHSize;
    }
  /****
  [[U tImage::EnsureBuffer () ]]
  ****/
  return this->EnsureBuffer();
  }

/****************************************************************************
  BOOL IsSameSize (const tImage &Image) const;
     Method for coparing image sizes

[[U tImage::IsSameSize_t (Image) ]]
*/
BOOL tImage::IsSameSize(const tImage &Image) const {
  if (this->iVSize != Image.iVSize)
    return FALSE;
  if (this->iHSize != Image.iHSize)
    return FALSE;
  return TRUE;
  }

/****************************************************************************
  BOOL IsSameSize (int iHSize, int iVSize) const;
     Method for coparing image sizes

[[U tImage::IsSameSize_t (iHSize, iVSize) ]]
*/
BOOL tImage::IsSameSize(int iHSize, int iVSize) const {
  if (this->iVSize != iVSize)
    return FALSE;
  if (this->iHSize != iHSize)
    return FALSE;
  return TRUE;
  }

/****************************************************************************

  ERROR_CODE tImage::Put (size_t szSize, const BYTE *pInBuffer);
     This method is main for receiving an image accross IPC.  This
     method converts the Image to a one dimensional array of bytes.  
     This method takes in an array of byte and puts it into the image.

[[U tImage::Put (szSize, pInBuffer) ]]
*/
ERROR_CODE tImage::Put (size_t szSize, const BYTE *pInBuffer) {
  ERROR_CODE ec;
  
  if (this->szBufferSize != szSize)
    return Traceback (S_tImage_BUFFER_SIZE_DOES_NOT_MATCH, "tImage::Put");
  
  /****
  [[U tImage::EnsureBuffer () ]]
  ****/
  ec = this->EnsureBuffer();
  if (ec != OK)
    return Traceback (ec, "tImage::Put");
  
  bcopy((char *)pInBuffer, (char *)this->pImage, this->szBufferSize);
  return OK;
  }

/****************************************************************************
  ERROR_CODE tImage::Get (size_t szSize, BYTE *pOutBuffer);
     This method is main for sending an image accross IPC.  This
     method converts the image to a one dimensional array of bytes. 
     This method takes a pointer to an array of bytes and fills it 
     with the image.

[[U tImage::Get (szSize, pOutBuffer) ]]
*/
ERROR_CODE tImage::Get (size_t szSize, BYTE *pOutBuffer) const {
  if (szBufferSize != szSize)
    return Traceback (S_tImage_BUFFER_SIZE_DOES_NOT_MATCH, "tImage::Get");
  if (this->pImage == NULL)
    return Traceback (S_tImage_IMAGE_NOT_YET_LOADED, "tImage::Get");  
  bcopy((char *)this->pImage, (char *)pOutBuffer, this->szBufferSize);
  return OK;
  }

/****************************************************************************
  void tImage::operator= (const tImage &Image);
     tImage assignment operator.

[[U tImage::operator_equals (Image) ]]
*/
void tImage::operator = (const tImage &Image) {
  /****
  [[U tImage::Copy(Image) ]]
  ****/
  this->Copy(Image);  
  }

/****************************************************************************
  void tImage::Release();
     Method to accually release an image.  All other method that need to 
     release.

[[U tImage::Release() ]]
*/
void tImage::Release() {
  if (this->pImage != NULL) {
    delete this->pImage;
    this->pImage = NULL;
    }
  }
  
/****************************************************************************
  void tImage::Copy(const tImage &Image);
     This method accually does all copying of images.  All other methods
     that need to preform copy, call this method.  

[[U tImage::Copy (Image) ]]
*/
void tImage::Copy (const tImage &Image) {
  ERROR_CODE ec;
  
  /****
  [[U tImage::ResizeImage (Image.iHSize, Image.iVSize) ]]
  ****/
  ec = this->ResizeImage(Image.iHSize, Image.iVSize);
  if (ec != OK)
    Traceback (ec, "tBuffer::Copy");
  else
    bcopy((char *)Image.pImage, (char *)this->pImage, this->szBufferSize);
  TraceMark(S_tImage_IMAGE_COPIED);
  }

/****************************************************************************
  ERROR_CODE tImage::EnsureBuffer();
     This method makes sure a buffer exist so that an Image can by stored
     
[[U tImage::EnsureBuffer () ]]
*/
ERROR_CODE tImage::EnsureBuffer() {
  int iSize = this->iVSize * this->iHSize;
  
  TraceMark(S_tImage_ENSURING_IMAGE_BUFFER_EXIST);
  if ((iHSize <= 0) || (iVSize <= 0))
    return S_tImage_INVALID_DIMENSIONS;
  this->szBufferSize = iSize * sizeof(PIXEL);
  if (this->pImage == NULL) {
    this->pImage = new PIXEL [iSize];
    if (this->pImage == NULL)
      return S_tImage_IMAGE_MEMORY_ALLOCATION_FAILED;
    }
  return OK;
  }
