 /* $RCSfile: imageSensor.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/comm/io/imageSensor.cc,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/* Derek Slye - Jet Propulsion Laboratory */


#include "ftGizmoInfo.h"
#include "imageSensor.h"
#include "imageChannel.h"
#include "ftTypes.h"



/*******************************************************************
*
* tImageSensor::tImageSensor -- tImageSensor constructor
*
  [[UD tImageSensor::tImageSensor (pFullImageChannel, iXStart, iYStart,
                                   iHSizeToUse, iHSizeToUse) ]]
*
* SPEC:
  [[U3 return this->tImageSensor::tImageSensor (pFullImageChannel, iXStart, iYStart,
                                   iHSizeToUse, iHSizeToUse) ]]
*
* RETURNS:
*    nothing
*
****/
tImageSensor::tImageSensor (tFullImageChannel* pFullImageChannel,
                  int iXStart, int iYStart, int iHSizeToUse, int iVSizeToUse) :
  
  /***
  [[U this->tSubImageChannel::tSubImageChannel_tiiii (pFullImageChannel,iXStart,iYStart,iHSizeToUse,iVSizeToUse) ]]
  ***/
  ImageChannel (pFullImageChannel,iXStart,iYStart,iHSizeToUse,iVSizeToUse) 
{
  this->iHSize = iHSizeToUse;
  this->iVSize = iVSizeToUse;
}


/*******************************************************************
*
* tImageSensor::SetRegionOfInterest -- set image region
*
  [[UD tImageSensor::SetRegionOfInterest (iXStart, iYStart) ]]
*
* SPEC:
  [[U3 return this->tImageSensor::SetRegionOfInterest (iXStart, iYStart) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tImageSensor::SetRegionOfInterest (int iXStart, int iYStart)
{
                            
  ERROR_CODE ec;
             
  /***
  [[U ec := this->ImageChannel.tSubImageChannel::SetRegionOfInterest(iXStart, iYStart, this->iHSize, this->iVSize) ]]
  ***/
  ec = this->ImageChannel.SetRegionOfInterest (iXStart, iYStart,
                                               this->iHSize, this->iVSize);
                                                   
  if (ec != OK) return Traceback (ec,"tImageSensor::SetRegionOfInterest");
  
  return OK;
}
  

/*******************************************************************
*
* tImageSensor::GetImage -- fetch video sub image
*
   [[UD tImageSensor::GetImage (StarImage) ]]
*
* SPEC:
   [[U3 return this->tImageSensor::GetImage (StarImage) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/  
ERROR_CODE tImageSensor::GetImage (tImage& StarImage) {
   
  ERROR_CODE ec;
                                                           
  /***
  [[U ec := this->ImageChannel.tSubImageChannel::GetImage (StarImage) ]]
  ***/
  ec = this->ImageChannel.GetImage (StarImage);
               
  if (ec != OK) return Traceback (ec,"tImageSensor::GetImage");
  
  return OK;
  
}  

/*******************************************************************
*
* tImageSensor::ImageParameters
*
   [[UD tImageSensor::ImageParameters (iXStart, iYStart, iHSize, iVSize) ]]
*
* SPEC:
   [[U3 return this->tImageSensor::ImageParameters (iXStart, iYStart, iHSize, iVSize) ]]
*
* RETURNS:
*    nothing
*
****/  
void tImageSensor::ImageParameters (int& iXStart, int& iYStart, 
      int& iHSize, int& iVSize) const
{
  /****
  [[U this->ImageChannel.tSubImageChannel::GetRegionOfInterest (iXStart, 
        iYStart, iHSize, iVSize) ]]
  ****/
  this->ImageChannel.GetRegionOfInterest (iXStart, iYStart, iHSize, iVSize);
}



/*******************************************************************
*
* tFullFrameSensor::tFullFrameSensor -- Full Frame sensor constructor
*
  [[UD tFullFrameSensor::tFullFrameSensor (pFullImageChannelToUse) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::tFullFrameSensor (pFullImageChannelToUse) ]]
*
* RETURNS:
*    nothing
*
****/  
tFullFrameSensor::tFullFrameSensor (tFullImageChannel* pFullImageChannelToUse) 
{

  this->pFullImageChannel = pFullImageChannelToUse;

}


/*******************************************************************
*
* tFullFrameSensor::StartCapture -- start video board
*
  [[UD tFullFrameSensor::tFullFrameSensor (pFullImageChannelToUse) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::tFullFrameSensor (pFullImageChannelToUse) ]]
*
* RETURNS:
*    nothing
*
****/ 
ERROR_CODE tFullFrameSensor::StartCapture () {
  
  /****
  [[U ec := *(this->pFullImageChannel).tFullImageChannel::StartCapture (void) ]]
  ****/
  ERROR_CODE ec = this->pFullImageChannel->StartCapture ();

  if (ec != OK) return Traceback (ec,"tFullFrameSensor::StartCapture");
  
  return OK;
}


/*******************************************************************
*
* tFullFrameSensor::GetImage -- fetch full video frame non-atomically
*
  [[UD tFullFrameSensor::GetImage (FullFrameImage) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::GetImage (FullFrameImage) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFullFrameSensor::GetImage (tImage& FullFrameImage) 
{ 
  /****
  [[U ec := *(this->pFullImageChannel).tFullImageChannel::GetImage (FullFrameImage) ]]
  ****/
  ERROR_CODE ec = this->pFullImageChannel->GetImage (FullFrameImage);
  
  if (ec != OK) return Traceback (ec,"tFullFrameSensor::GetImage");
  
  return OK;
}


/*******************************************************************
*
* tFullFrameSensor::GetImage -- fetch full video frame atomically
*
  [[UD tFullFrameSensor::AtomicGetImage (FullFrameImage) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::AtomicGetImage (FullFrameImage) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFullFrameSensor::AtomicGetImage (tImage& FullFrameImage) 
{
  /****
  [[U ec := *(this->pFullImageChannel).tFullImageChannel::AtomicGetImage (FullFrameImage) ]]
  ****/
  ERROR_CODE ec = this->pFullImageChannel->AtomicGetImage (FullFrameImage);
  
  if (ec != OK) return Traceback (ec,"tFullFrameSensor::GetAtomicImage");
  
  return OK;
}

/*******************************************************************
*
* tFullFrameSensor::Update 
*
  [[UD tFullFrameSensor::Update (void) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::Update (void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFullFrameSensor::Update () 
{
  /****
  [[U ec := *(this->pFullImageChannel).tFullImageChannel::Update (void) ]]
  ****/
  ERROR_CODE ec = this->pFullImageChannel->Update ();
  
  if (ec != OK) return Traceback (ec,"tFullFrameSensor::Update");
  
  return OK;
}

/*******************************************************************
*
* tFullFrameSensor::ImageParameters 
*
  [[UD tFullFrameSensor::ImageParameters (iHSize, iVSize) ]]
*
* SPEC:
  [[U3 return this->tFullFrameSensor::ImageParameters (iHSize, iVSize) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
void tFullFrameSensor::ImageParameters (int& iHSize, int& iVSize) 
{
  /****
  [[U this->pFullImageChannel->tFullImageChannel::GetDimensions (
        iHSize, iVSize) ]]
  ****/
  this->pFullImageChannel->GetDimensions (iHSize, iVSize);
}

