/* $RCSfile: inputSource.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/inputSource.cc,v 1.3 1998/08/27 18:34:14 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
05a,rlj,30Jan98    updated architecture and specification
01a,beh,20Apr97    written
*/
 
/*
DESCRIPTION

*/

#include "inputSource.h"

/******************************************************************
* tInputSource::tInputSource (void)
* 
* DESCRIPTION: 
[[U2 tInputSource::tInputSource (void)]]
*
[[U3 tInputSource::tInputSource (void)]]
*/
tInputSource::tInputSource () 
{ 
  dValue = 0.0; 
  dOffset = 0.0; 
}

/******************************************************************
* tInputSource::tInputSource_d (double dOffsetIn)
* 
* DESCRIPTION: 
*
[[U2 tInputSource::tInputSource_d (double dOffsetIn) ]]
*
[[U3 tInputSource::tInputSource_d (double dOffsetIn) ]]
*/
tInputSource::tInputSource (double dOffsetIn) 
{ 
  dValue = 0.0; 
  dOffset = dOffsetIn ;
}


/******************************************************************
* tInputSource::Value (void)
*
* DESCRIPTION: 
[[U2 tInputSource::Value (void)]]
*
[[U3 tInputSource::Value (void)]]
*/
double tInputSource::Value () const
{
  return this->dValue - this->dOffset;
}

/******************************************************************
* tInputSource::Offset (void)
*
* DESCRIPTION: 
[[U2 tInputSource::Offset (void)]]
*
[[U3 tInputSource::Offset (void)]]
*/
double tInputSource::Offset () const
{
  return this->dOffset;
}

/******************************************************************
* tInputSource::SetValue 
*
* DESCRIPTION: 
[[U2 tInputSource::SetValue (dNewValue)]]
*
[[U3 tInputSource::SetValue (dNewValue)]]
*/
void  tInputSource::SetValue (double dNewValue) 
{
   this->dValue = dNewValue;
}

/******************************************************************
* tInputSource::SetOffset 
*
* DESCRIPTION: 
[[U2 tInputSource::SetOffset (dNewOffset)]]
*
[[U3 tInputSource::SetOffset (dNewOffset)]]
*/
void tInputSource::SetOffset (double dNewOffset) 
{
  this->dOffset = dNewOffset;
}


/******************************************************************
* tInputSource::RawValue
*
* DESCRIPTION: 
[[U2 tInputSource::RawValue ()]]
*
[[U3 tInputSource::RawValue ()]]
*/
double tInputSource::RawValue () const
{
  return this->dValue;
}




/***************************  tSensor  ***************************/

/*****************************************************************
* tSensor::tSensor
*
* DESCRIPTION: 
[[U2 tSensor::tSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit) ]]
*
[[U3 tSensor::tSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit) ]]
*/
tSensor::tSensor (const tInputChannel &ChannelIn,
                  double dPhysicalUnitsPerElectricalUnit) :

  /****
  [[U tInputSource::tInputSource (void)]]
  ****/
  tInputSource (),

  /****
  [ this->Channel := reference to ChannelIn ]
  ****/
  Channel(ChannelIn),

  
  /****
  [[U dElectricalUnitsPerCount := ChannelIn.GetChannelUnitsPerCount () ]]
  
  [ this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
       dElectricalUnitsPerCount ]
  ****/
  dPhysicalUnitsPerCount (dPhysicalUnitsPerElectricalUnit *
     ChannelIn.GetChannelUnitsPerCount () )

{
  TraceMark(S_inputSource_TSENSOR_CONSTRUCTOR);
}



/*****************************************************************
* tSensor::tSensor
*
* DESCRIPTION: 
[[U2 tSensor::tSensor_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit,
         dOffsetIn) ]]
*
[[U3 tSensor::tSensor_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit,
         dOffsetIn) ]]
*/
tSensor::tSensor (const tInputChannel &ChannelIn,
                  double dPhysicalUnitsPerElectricalUnit,
                  double dOffsetIn) :

  /****
  [[U tInputSource::tInputSource_d (dOffsetIn)]]
  ****/
  tInputSource (dOffsetIn),

  /****
  [ this->Channel := reference to ChannelIn  ]
  ****/
  Channel (ChannelIn),
  
  /****
  [[U dElectricalUnitsPerCount := ChannelIn.GetChannelUnitsPerCount () ]]
  
  [ this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
       dElectricalUnitsPerCount ]
  ****/
  dPhysicalUnitsPerCount (dPhysicalUnitsPerElectricalUnit *
     ChannelIn.GetChannelUnitsPerCount () )

{
  TraceMark(S_inputSource_TSENSOR_CONSTRUCTOR);
}








/******************************************************************
*
* tSensor::Reset ()
*
* DESCRIPTION:
[[U2 tSensor::Reset (void)]]
*
[[U3 tSensor::Reset (void)]]
*/
ERROR_CODE tSensor::Reset(void)
{
  TraceMark(S_inputSource_TSENSOR_RESET);
  
  
  /****
  [[U ec := this->Channel.tInputChannel::Reset(void) ]]
  ****/
  FAST ERROR_CODE ec = this->Channel.Reset();
  
  /****
  [[U tInputSource::SetValue (0.0) ]]
  ****/
  this->SetValue (0.0);
  
  
  return ec;
}


/******************************************************************
* tSensor::Update
*
* DESCRIPTION: 
[[U2 tSensor::Update (tUpdateTime)]]
*
[[U3 tSensor::Update (tUpdateTime)]]
*/
double tSensor::Update (const TIME tUpdateTime)
{
  double dTemp;
  
  /****
  [[U dTemp := tSensor::ChannelInput () ]]
  ****/
  dTemp = this->ChannelInput ();

  /****
  [[U this->tSensor::SetValue (dTemp)]]
  ****/
  this->SetValue (dTemp);
  
  /****
  [[U this->tSensor::Value (void)]]
  ****/
  return this->Value ();
}


/******************************************************************
* tSensor::ChannelInput
*
[[U tSensor::ChannelInput (tUpdateTime)]]
*/
double tSensor::ChannelInput ()
{
  double dTemp;
  
  /****
  [[U this->Channel.tInputChannel::Input_double (&dTemp) ]]
  ****/
  this->Channel.Input (&dTemp);

  return dTemp;
}



/******************************************************************
*
* tModulationSource::tModulationSource
*
* DESCRIPTION: 
[[U2 tModulationSource::tModulationSource (void) ]]
*
[[U3 tModulationSource::tModulationSource()]]
*/
tModulationSource::tModulationSource (void) :

  /****
  [[U tInputSource::tInputSource (void) ]]
  ****/
  tInputSource (),

  /****
  [[U Modulation.tModulation::tModulation (void)]]
  ****/
  Modulation ()
  
{
  TraceMark(S_inputSource_TMODULATIONSOURCE_CONSTRUCTOR);
}

/******************************************************************
*
* tModulationSource::tModulationSource (const tModulationParams& InitialParams) 
*
* DESCRIPTION: 
[[U2 tModulationSource::tModulationSource_t (InitialParams) ]]
*
[[U3 tModulationSource::tModulationSource_t (InitialParams) ]]
*/
tModulationSource::tModulationSource (const tModulationParams& InitialParams) :

  /****
  [[U tInputSource::tInputSource (void) ]]
  ****/
  tInputSource (),

  /****
  [[U Modulation.tModulation::tModulation_t (InitialParams)]]
  ****/
  Modulation (InitialParams)
  
{
  TraceMark(S_inputSource_TMODULATIONSOURCE_CONSTRUCTOR);
}


/******************************************************************
*
* tModulationSource::tModulationSource (const tModulationParams& InitialParams,
                                        double dOffsetIn) 
*
* DESCRIPTION: 
[[U2 tModulationSource::tModulationSource_td (InitialParams, dOffsetIn) ]]
*
[[U3 tModulationSource::tModulationSource_td (InitialParams, dOffsetIn) ]]
*/
tModulationSource::tModulationSource (const tModulationParams& InitialParams,
  double dOffsetIn) :

  /****
  [[U tInputSource::tInputSource_d (dOffsetIn) ]]
  ****/
  tInputSource (dOffsetIn),

  /****
  [[U Modulation.tModulation::tModulation_t (InitialParams)]]
  ****/
  Modulation (InitialParams)
  
{
  TraceMark(S_inputSource_TMODULATIONSOURCE_CONSTRUCTOR);
}


/*******************************************************************
*
* tModulationSource::~tModulationSource
*
* DESCRIPTION: Compiler-supplied destructor
[[U2 tModulationSource::~tModulationSource (void)]]
*
[[U3 tModulationSource::~tModulationSource (void)]]

*/

/***
[[U this->Modulator.tModulation::~tModulation (void)]]
***/
/***
[[U tInputSource::~tInputSource (void)]]
***/


/******************************************************************
*
* tModulationSource::SetModulationParameters
*
[[U5 tModulationSource::SetModulationParameters (NewParameters) ]]
*/
ERROR_CODE tModulationSource::SetModulationParameters 
           (const tModulationParams& NewParameters) 

{
  /****
  [[U Modulation.tModulation::SetParameters_t (NewParameters)]]
  ****/
  return Modulation.SetParameters (NewParameters);
  
}

/******************************************************************
*
* tModulationSource::Reset
*
* DESCRIPTION: 
[[U2 tModulationSource::Reset ()]]
*
[[U3 tModulationSource::Reset ()]]
*/
ERROR_CODE tModulationSource::Reset (void)
{
  TraceMark(S_inputSource_TMODULATIONSOURCE_RESET);
  
  /****
  [[U this->tModulationSource::SetValue (0.0) ]]
  ****/
  this->SetValue (0.0);
  return OK;
}


/******************************************************************
*
* tModulationSource::Update
*
* DESCRIPTION: Computes new modulation value using tModulate class
*
* In increment 2, tModulationSource's ignore the TIME input, but
* this may change in a later increment.
*
* RETURNS: double
*
[[U tModulationSource::Update (tUpdateTime)]]

*
*/

double tModulationSource::Update(const TIME tUpdateTime)
{
  /****
  [[U dTemp := this->Modulation.tModulation::ComputeModulation (tUpdateTime) ]]
  
  [[U return := tModulationSource::SetValue (dTemp) ]]
  ****/  
  this->SetValue (this->Modulation.ComputeModulation (tUpdateTime));
  
  /****
  [[U tModulationSource::Value () ]]
  ****/
  return this->Value ();
}


/***************************  tModulatedSensor  ***************************/

/*****************************************************************
* tModulatedSensor::tModulatedSensor
*
* DESCRIPTION: 
[[U2 tModulatedSensor::tModulatedSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit) ]]
*
[[U3 tModulatedSensor::tModulatedSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit) ]]
*/
tModulatedSensor::tModulatedSensor (const tInputChannel &ChannelIn,
                  double dPhysicalUnitsPerElectricalUnit) :

  /****
  [[U tSensor::tSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit)]]
  ****/
  tSensor (ChannelIn, dPhysicalUnitsPerElectricalUnit),

  dModulationValue (0.0),
  
  /****
  [[U this->Modulation.tModulation::tModulation () ]]
  ****/
  Modulation ()
  
{
  TraceMark(S_inputSource_TSENSOR_CONSTRUCTOR);
}

/*****************************************************************
* tModulatedSensor::tModulatedSensor
*
* DESCRIPTION: 
[[U2 tModulatedSensor::tModulatedSensor_tdt (ChannelIn, dPhysicalUnitsPerElectricalUnit,
        InitialParameters) ]]
*
[[U3 tModulatedSensor::tModulatedSensor_tdt (ChannelIn, dPhysicalUnitsPerElectricalUnit,
        InitialParameters) ]]
*/
tModulatedSensor::tModulatedSensor (const tInputChannel& ChannelIn,
                  double dPhysicalUnitsPerElectricalUnit,
                  const tModulationParams& InitialParameters) :

  /****
  [[U tSensor::tSensor_td (ChannelIn, dPhysicalUnitsPerElectricalUnit)]]
  ****/
  tSensor (ChannelIn, dPhysicalUnitsPerElectricalUnit),

  dModulationValue (0.0),
  
  /****
  [[U this->Modulation.tModulation::tModulation_t (InitialParameters) ]]
  ****/
  Modulation ()
  
{
  TraceMark(S_inputSource_TSENSOR_CONSTRUCTOR);
}



/*****************************************************************
* tModulatedSensor::tSensorModulated
*
* DESCRIPTION: 
[[U2 tModulatedSensor::tModulatedSensor_tdtd (ChannelIn, dPhysicalUnitsPerElectricalUnit,
        InitialParameters, dOffSetIn) ]]
*
[[U3 tModulatedSensor::tModulatedSensor_tdtd (ChannelIn, dPhysicalUnitsPerElectricalUnit,
        InitialParameters, dOffsetIn) ]]
*/
tModulatedSensor::tModulatedSensor (const tInputChannel &ChannelIn,
                  double dPhysicalUnitsPerElectricalUnit,
                  const tModulationParams& InitialParameters, double dOffsetIn) :

  /****
  [[U tSensor::tSensor_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                        dOffsetIn)]]
  ****/
  tSensor (ChannelIn, dPhysicalUnitsPerElectricalUnit, dOffsetIn),

  dModulationValue (0.0),
  
  /****
  [[U this->Modulation.tModulation::tModulation_t (InitialParameters) ]]
  ****/
  Modulation ()
  
{
  TraceMark(S_inputSource_TSENSOR_CONSTRUCTOR);
}


/*****************************************************************
* tModulatedSensor::SetModulationParameters
*
* DESCRIPTION: 
[[U2 tModulatedSensor::SetModulationParameters (NewParameters) ]]
*
[[U3 tModulatedSensor::SetModulationParameters (NewParameters) ]]
*/
ERROR_CODE tModulatedSensor::SetModulationParameters (const tModulationParams& NewParameters)

{
  /****
  [[U this->Modulation.tModulation::SetParameters_t (NewParameters) ]]
  ****/
  return this->Modulation.SetParameters (NewParameters);
}


/*****************************************************************
* tModulatedSensor::ModulationParameters
*
* DESCRIPTION: 
[[U2 tModulatedSensor::ModulationParameters () ]]
*
[[U3 tModulatedSensor::ModulationParameters () ]]
*/
const tModulationParams& tModulatedSensor::ModulationParameters () const
{
  /****
  [[U this->Modulation.tModulation::GetParameters () ]]
  ****/
  return this->Modulation.GetParameters ();
}



/*****************************************************************
* tModulatedSensor::EnableModulation
*
* DESCRIPTION: 
[[U2 tModulatedSensor::EnableModulation () ]]
*
[[U3 tModulatedSensor::EnableModulation () ]]
*/
ERROR_CODE tModulatedSensor::EnableModulation () 

{
  /****
  [[U this->Modulation.tModulation::Enable () ]]
  ****/
  this->Modulation.Enable ();
  
  return OK;
}




/*****************************************************************
* tModulatedSensor::DisableModulation
*
* DESCRIPTION: 
[[U2 tModulatedSensor::DisableModulation () ]]
*
[[U3 tModulatedSensor::DisableModulation () ]]
*/
ERROR_CODE tModulatedSensor::DisableModulation () 

{
  /****
  [[U this->Modulation.tModulation::Disable () ]]
  ****/
  this->Modulation.Disable ();
  
  return OK;
}


/*****************************************************************
* tModulatedSensor::IsModulationEnabled
*
* DESCRIPTION: 
[[U2 tModulatedSensor::IsModulationEnabled () ]]
*
[[U3 tModulatedSensor::IsModulationEnabled () ]]
*/
BOOL tModulatedSensor::IsModulationEnabled ()  const

{
  /****
  [[U this->Modulation.tModulation::IsEnabled () ]]
  ****/
  return this->Modulation.IsEnabled ();
 
}

/*****************************************************************
* tModulatedSensor::ModulationValue 
*
* DESCRIPTION: 
[[U2 tModulatedSensor::ModulationValue () ]]
*
[[U3 tModulatedSensor::ModulationValue ()  ]]
*/
double tModulatedSensor::ModulationValue () const
{
  return this->dModulationValue;
   
}

/*****************************************************************
* tModulatedSensor::UnmodulatedValue 
*
* DESCRIPTION: 
[[U2 tModulatedSensor::UnmodulatedValue () ]]
*
[[U3 tModulatedSensor::UnmodulatedValue ()  ]]
*/
double tModulatedSensor::UnmodulatedValue () const
{
  /****
  [[U dTemp := tModulatedSensor::Value () ]]
  
  [ return := dTemp - this->dModulationValue ]
  ****/  
  return this->Value () - this->dModulationValue;
}


/*****************************************************************
* tModulatedSensor::Reset
*
* DESCRIPTION: 
[[U2 tModulatedSensor::Reset () ]]
*
[[U3 tModulatedSensor::Reset () ]]
*/
ERROR_CODE tModulatedSensor::Reset ()  

{
  this->dModulationValue = 0.0; 
  
  /****
  [[U tSensor::Reset () ]]
  ***/
  return this->tSensor::Reset ();
  
 
}


/*****************************************************************
* tModulatedSensor::Update
*
* DESCRIPTION: 
[[U2 tModulatedSensor::Update (tUpdateTime) ]]
*
[[U3 tModulatedSensor::Update (tUpdateTime) ]]
*/
double tModulatedSensor::Update (TIME tUpdateTime) 

{
  double dSensorValue, dModulatedValue;
  
  /*****
  [[U tSensor::Update (tUpdateTime) ]]
  ****/ 
  this->tSensor::Update (tUpdateTime);
  
  /*****
  [[U this->dModulationValue := this->Modulation.tModulation::ComputeModulation (tUpdateTime) ]]
  ****/ 
  this->dModulationValue = this->Modulation.ComputeModulation (tUpdateTime);

  /*****
  [[U dTemp := tModulatedSensor::RawValue () ]]
  
  [ dModulatedValue := dTemp + this->dModulatedValue ]
  ****/ 
  dModulatedValue = this->RawValue () + this->dModulationValue;
  
  /*****
  [[U tModulatedSensor::SetValue (dModulatedValue) ]]
  ****/ 
  this->SetValue (dModulatedValue);
   
  return this->Value ();
}




