/* $RCSfile: integratorSensor.cc,v $ - Implements abstract hardware and software input classes */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/integratorSensor.cc,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/* Ken Starr - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,kms,11Nov97    written
*/
 
/*
DESCRIPTION

*/

#include "integratorSensor.h"
#include "hwTypes.h"
#include "channel.h"




/*******************************************************************/
/***********************  tIntegratorSensor  ************************/
/*******************************************************************/

/******************************************************************
*
* tIntegratorSensor::tIntegratorSensor
*
* DESCRIPTION: Prepares the class for use by telling it the
* modulation parameters to use and the channel to read sensor
* input from.
*

  [[U tIntegratorSensor::tIntegratorSensor()]]

*
*/

tIntegratorSensor::tIntegratorSensor(const tInputChannel &ChannelToUse,
                        const double dElectricalToPhysicalScaleFactorToUse) :
  /****
    [[U tSensor::tSensor(this=this, ChannelToUse=ChannelToUse,
                         dElectricalToPhysicalScaleFactorToUse=
                             dElectricalToPhysicalScaleFactorToUse)]]
  ****/
  tSensor(ChannelToUse, dElectricalToPhysicalScaleFactorToUse)
{
  double dLastIntegratorCount;

  this->dLastIntegratorCount = 0.0;
  
}


/*******************************************************************
*
* tIntegratorSensor::~tIntegratorSensor
*
* DESCRIPTION: Compiler-supplied destructor
*
* RETURNS: Nothing
*

[[U tIntegratorSensor::~tIntegratorSensor()]]

*/

/***
  [[U tSensor::~tSensor()]]
***/



/*******************************************************************
*
* tIntegratorSensor::Update
*
* DESCRIPTION: Computes new Integrated value using current and last
*		   values.
*
* Currently, tIntegratorSensor's ignore the TIME input.
*
* Note that the TIME input is ignored by a base tSensor class.
*
* ASSUMPTIONS:  There is a bottom-up condition that comes up from
* tInputChannel::Input that says that the board must be receiving
* input properly.  For now, this condition is considered to be handled at
* the system level via a statement like "All hardware must be connected
* and receiving input properly".  Therefore, we'll use that assumption
* here to make the bottom-up condition disappear.
*
* RETURNS: double
*

  [[U dIntegratorCount := tIntegratorSensor::Update()]]

*
*/
double tIntegratorSensor::Update(const TIME tUpdateTime)
{
  FAST double dRawSensorValue;
  
  /****
    [[U tSensor::Update(this=this,tUpdateTime=tUpdateTime,
                        return=dRawSensorValue) ]]
  ****/
  this->tSensor::Update(tUpdateTime);
  
  /****
  [[U dRawSensorValue := tIntegratingSensor::RawValue ()]]
  ****/
  dRawSensorValue = this->RawValue ();
  
  if (dRawSensorValue < this->dLastIntegratorCount)
    /* wraparuond has occured */
    /****
    [[U tIntegratingSensor::SetValue (MAX_INTEGRATOR_COUNT + dRawSensorValue - dLastIntegratorCount ) ]]
    ****/
    this->SetValue (MAX_INTEGRATOR_COUNT + dRawSensorValue - dLastIntegratorCount );
  else	
    /****
    [[U tIntegratingSensor::SetValue (dRawSensorValue - this->dLastIntegratorCount ) ]]
    ****/
    this->SetValue (dRawSensorValue - this->dLastIntegratorCount);

  this->dLastIntegratorCount = dRawSensorValue;

  /****
  [[U tIntegratingSensor::Value () ]]
  ****/
  return this->Value ();
}





/******************************************************************
*
* tIntegratorSensor::ResetIntegrator
*
* DESCRIPTION: Resets the Integrator state on the board
*
* RETURNS: ERROR_CODE
*

  [[U tIntegratorSensor::ResetIntegrator ()]]

*
*/

ERROR_CODE tIntegratorSensor::ResetIntegrator (void)
{
  ERROR_CODE ec; 
  /****
    [[U ec := tSensor::Reset(void) ]]
  ****/
  ec = this->Reset();

  this->dLastIntegratorCount = 0.0;
    
  return ec;
}

