/* $RCSfile: laserMetrologySensor.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/laserMetrologySensor.cc,v 1.2 1998/08/27 18:34:15 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$Log: laserMetrologySensor.cc,v $
Revision 1.2  1998/08/27 18:34:15  chara
Seems to track for negative numbers now, but not positive

Revision 1.1.1.1  1998/06/11 21:36:18  chara
First import of RICST Code

Revision 1.2.2.3  1998/05/05 15:28:05  rich
Delay line full function

Revision 1.2.2.2  1998/04/30 20:35:39  rbrill
commit before build_5a_008

Revision 1.2.2.1  1998/04/04 00:31:38  phil
commit before build_5a_002

Revision 1.2  1998/04/01 15:08:31  rich
i/o overhaul

Revision 1.1  1998/03/12 01:07:29  rich
delay line overhaul cont.

*/

#include "laserMetrologySensor.h"


/*****************************************************************
* tLaserMetrologySensor::tLaserMetrologySensor
*
* SPECIFICATION:
[[U5 tLaserMetrologySensor::tLaserMetrologySensor_tditTTd (ChannelIn,
       dLaserOpdPerElectricalUnit, iNumLaserPassesIn, iLaserSignIn, 
       DitherPztChannel, dDitherPhysicalUnitsPerElectricalUnits,
       tServoPeriodIn, tDitherPeriod, dDitherAmplitude) ]]
*/
tLaserMetrologySensor::tLaserMetrologySensor (
         tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave, int iNumLaserPassesIn, int iLaserSignIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, 
         TIME tServoPeriodIn, TIME tDitherPeriod, double dDitherAmplitude) :

   /****
   [[U tModulatedSensor::tModulatedSensor_td (ChannelIn, dLaserOpdPerWave ) ]]
   ****/
   tModulatedSensor (ChannelIn, dLaserOpdPerWave),
   
   iNumLaserPasses (MAX (1,iNumLaserPassesIn) ),
   
   iLaserSign (iLaserSignIn), 

   dDitherValue (0.0),
   
   /***
   [[U this->WaveformActuator.tActuator::tActuator (DitherPztChannel, dDitherPhysicalUnitsPerElectricalUnits) ]]
   ***/
   WaveformActuator (DitherPztChannel, dDitherPhysicalUnitsPerElectricalUnits),

   /****
   [[U dLaserPhysicalUnitsPerCount := this->tLaserMetrologySensor::PhysicalUnitsToCounts (1) ]]
 
   [[U dDitherPztPhysicalUnitsPerCount := this->WaveformActuator.tActuator::PhysicalUnitsToCounts (1) ]]
   
   [[U dAmplitudeLU := this->LaserOpdToLaserCounts (dDitherAmplitude) ]]

   [[U Dither.tDlDither::tDlDither (DitherPztChannel, (dDitherPztPhysicalUnitsPerCount / dLaserPhysicalUnitsPerCount /
             this->iNumLaserPasses), 
         tServoPeriodIn, tDitherPeriod, dAmplitudeLU) ]]
   ****/
   Dither (DitherPztChannel, this->WaveformActuator.PhysicalUnitsToCounts (1) / 
           this->PhysicalUnitsToCounts (1) / this->iNumLaserPasses,
           tServoPeriodIn, 
           tDitherPeriod, this->LaserOpdToLaserCounts (dDitherAmplitude)) ,

   /****
   [ this->LaserChannel := reference to ChannelIn ]
   ****/
   LaserChannel(ChannelIn)
{}





/*****************************************************************
* tLaserMetrologySensor::tLaserMetrologySensor
*
* SPECIFICATION:
[[U5 tLaserMetrologySensor::tLaserMetrologySensor_tditTTdt (ChannelIn,
       dLaserOpdPerWave, iNumLaserPassesIn, iLaserSignIn,
       DitherPztChannel, dDitherPhysicalUnitsPerElectricalUnits,
       tServoPeriodIn, tDitherPeriod, dDitherAmplitude, InitialParameters) ]]
*/
tLaserMetrologySensor::tLaserMetrologySensor (
         tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave, int iNumLaserPassesIn, int iLaserSignIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, TIME tServoPeriodIn, 
         TIME tDitherPeriod, double dDitherAmplitude, 
         const tModulationParams& InitialParameters) :

   /****
   [[U tModulatedSensor::tModulatedSensor_tdt (ChannelIn, 
         dElectricalToOpdScaleFactorIn / MAX (1,iNumPassesIn), InitialParameters )]]
   ****/
   tModulatedSensor (ChannelIn, dLaserOpdPerWave, InitialParameters ),
   
   iNumLaserPasses (MAX (1,iNumLaserPassesIn) ),
   
   iLaserSign (iLaserSignIn),
   
   dDitherValue (0.0),
   
   /***
   [[U this->WaveformActuator.tActuator::tActuator (DitherPztChannel, 
                dDitherPhysicalUnitsPerElectricalUnits) ]]
   ***/
   WaveformActuator (DitherPztChannel, dDitherPhysicalUnitsPerElectricalUnits),

   /****
   [[U dLaserPhysicalUnitsPerCount := this->tLaserMetrologySensor::PhysicalUnitsToCounts (1) ]]
 
   [[U dDitherPztPhysicalUnitsPerCount := this->WaveformActuator.tActuator::PhysicalUnitsToCounts (1) ]]
   
   [[U dAmplitudeLU := this->LaserOpdToLaserCounts (dDitherAmplitude) ]]

   [[U Dither.tDlDither::tDlDither (DitherPztChannel, (dDitherPztPhysicalUnitsPerCount / dLaserPhysicalUnitsPerCount /
             this->iNumLaserPasses), 
         tServoPeriodIn, tDitherPeriod, dAmplitudeLU) ]]
   ****/
   Dither (DitherPztChannel, this->WaveformActuator.PhysicalUnitsToCounts (1) / 
           this->PhysicalUnitsToCounts (1) / this->iNumLaserPasses,
           tServoPeriodIn, 
           tDitherPeriod, this->LaserOpdToLaserCounts (dDitherAmplitude)) ,

   /****
   [ this->LaserChannel := reference to ChannelIn ]
   ****/
   LaserChannel(ChannelIn)
{}



/*****************************************************************
* tLaserMetrologySensor::EnableDither
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::EnableDither () ]]
*
[[U3 tLaserMetrologySensor::EnableDither () ]]
*/
ERROR_CODE tLaserMetrologySensor::EnableDither ()
{
  ERROR_CODE ec;
  
  /****
  [[U ec := this->Dither.tDlDither::DitherOn () ]]
  ****/
  ec = this->Dither.DitherOn (); 
 
  return Traceback (ec, "tLaserMetrologySensor::EnableDither" ); 
}


/*****************************************************************
* tLaserMetrologySensor::DisableDither
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::DisableDither () ]]
*
[[U3 tLaserMetrologySensor::DisableDither () ]]
*/
ERROR_CODE tLaserMetrologySensor::DisableDither ()
{
  ERROR_CODE ec;

  /****
  [[U ec := this->Dither.tDlDither::DitherOff () ]]
  ****/
  ec = this->Dither.DitherOff (); 
 
  return Traceback (ec, "tLaserMetrologySensor::DisableDither" ); 
}


/*****************************************************************
* tLaserMetrologySensor::IsDitherEnabled
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::IsDitherEnabled () ]]
*
[[U3 tLaserMetrologySensor::IsDitherEnabled () ]]
*/
BOOL tLaserMetrologySensor::IsDitherEnabled () const
{
  /****
  [[U this->Dither.tDlDither::IsDitherOn () ]]
  ****/
  return this->Dither.IsDitherOn (); 
 
}




/*****************************************************************
* tLaserMetrologySensor::SetDitherParameters
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::SetDitherParameters (dNewAmplitude, tNewPeriod) ]]
*
[[U3 tLaserMetrologySensor::SetDitherParameters (dNewAmplitude, tNewPeriod) ]]
*/
ERROR_CODE tLaserMetrologySensor::SetDitherParameters (double dNewAmplitude, TIME tNewPeriod)
{
  ERROR_CODE ec;
  
  // dNewAmplitude is in Laser OPD...

  /****
  [[U dAmplitudeLU := tLaserMetrologySensor::LaserOpdToLaserCounts (dNewAmplitude)]]
  
  [[U ec := this->Dither.tDlDither::SetDitherParameters (dAmplitudeLU, tNewPeriod) ]]
  ****/
  ec = this->Dither.SetDitherParameters (this->LaserOpdToLaserCounts (dNewAmplitude), tNewPeriod);
  
  return Traceback (ec, "tLaserMetrologySensor::SetDitherParameters" ); 
}


/*****************************************************************
* tLaserMetrologySensor::DitherAmplitude 
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::DitherAmplitude () ]]
*
[[U3 tLaserMetrologySensor::DitherAmplitude () ]]
*/
double tLaserMetrologySensor::DitherAmplitude () const
{
  /****
  [[U this->Dither.tDlDither::DitherAmplitude () ]]
  ****/
  return this->Dither.DitherAmplitude ();
}

/*****************************************************************
* tLaserMetrologySensor::DitherPeriod
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::DitherPeriod () ]]
*
[[U3 tLaserMetrologySensor::DitherPeriod () ]]
*/
TIME tLaserMetrologySensor::DitherPeriod () const
{
  /****
  [[U this->Dither.tDlDither::DitherPeriod () ]]
  ****/
  return this->Dither.DitherPeriod ();
}


/*****************************************************************
* tLaserMetrologySensor::CalibrateDither
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::CalibrateDither () ]]
*
[[U3 tLaserMetrologySensor::CalibrateDither () ]]
*/
ERROR_CODE tLaserMetrologySensor::CalibrateDither () 
{
  ERROR_CODE ec;

  /****
  [[U ec := this->Dither.tDlDither::CalibrateDither () ]]
  ****/
  ec = this->Dither.CalibrateDither ();
  
  return Traceback (ec, "tLaserMetrologySensor::CalibrateDither" ); 
}

/*****************************************************************
* tLaserMetrologySensor::CheckDither
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::CheckDither () ]]
*
[[U3 tLaserMetrologySensor::CheckDither () ]]
*/
ERROR_CODE tLaserMetrologySensor::CheckDither () 
{
  ERROR_CODE ec;

  /****
  [[U ec := this->Dither.tDlDither::CheckDither () ]]
  ****/
  ec = this->Dither.CheckDither ();
  
  return Traceback (ec, "tLaserMetrologySensor::CheckDither" ); 
}


/*****************************************************************
* tLaserMetrologySensor::DoingCheckOrCalib
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::DoingCheckOrCalib () ]]
*
[[U3 tLaserMetrologySensor::DoingCheckOrCalib () ]]
*/
BOOL tLaserMetrologySensor::DoingCheckOrCalib () const 
{
  /****
  [[U this->Dither.tDlDither::DoingCheckOrCalib () ]]
  ****/
  return this->Dither.DoingCheckOrCalib ();
  
}



/*****************************************************************
* tLaserMetrologySensor::DitherCalibrationInfo 
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::DitherCalibrationInfo () ]]
*
[[U3 tLaserMetrologySensor::DitherCalibrationInfo () ]]
*/
tDitherCalibInfo tLaserMetrologySensor::DitherCalibrationInfo () const 
{

  // Convert calibration amplitude from laser counts to laser opd

  /****
  [[U Temp := this->Dither.tDlDither::GetCalibInfo () ]]
  ****/
  tDitherCalibInfo Temp = this->Dither.GetCalibInfo ();
 
  Temp.dAmplitude = this->LaserCountsToLaserOpd (Temp.dAmplitude);
  
  return Temp;
}



/*****************************************************************
* tLaserMetrologySensor::UnditherCompensatedValue
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::UnditherCompensatedValue () ]]
*
[[U3 tLaserMetrologySensor::UnditherCompensatedValue () ]]
*/
double tLaserMetrologySensor::UnditherCompensatedValue () const 
{
  /*****
  [[U dValue := tLaserMetrologySensor::Value () ]]
  
  [ return := dValue - dDitherValue ]
  ****/
  return this->Value () - this->dDitherValue;
}


/*****************************************************************
* tLaserMetrologySensor::UnmodulatedUnditherCompensatedValue ()
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::UnmodulatedUnditherCompensatedValue () ]]
*
[[U3 tLaserMetrologySensor::UnmodulatedUnditherCompensatedValue () ]]
*/
double tLaserMetrologySensor::UnmodulatedUnditherCompensatedValue () const 
{
  /*****
  [[U dValue := tLaserMetrologySensor::Value () ]]
  
  [[U dModulationValue := tLaserMetrologySensor::ModulationValue () ]]
  
  [ return := dValue - dModulationValue - dDitherValue ]
  ****/
  return this->Value () - this->ModulationValue () - this->dDitherValue;
}


/*****************************************************************
* tLaserMetrologySensor::Update
*
* DESCRIPTION: 
[[U2 tLaserMetrologySensor::Update (TIME tUpdateTime) ]]
*
[[U3 tLaserMetrologySensor::Update (TIME tUpdateTime) ]]
*/
double tLaserMetrologySensor::Update (TIME tUpdateTime)
{
  double dLaserValue, dRawValue;
  
  // Updates value from input channel and add modulation if enabled.
  // Update returns the laser value with offset added which will be used to calibrate or 
  // check the dither waveform if the sensor is calibrating or checking.
 
  /****
  [[U tModulatedSensor::Update (tUpdateTime) ]]
  ****/
  dLaserValue = this->tModulatedSensor::Update (tUpdateTime);
  
  // Get the non-offsetted value just updated.
  /****
  [[U dRawValue := tLaserMetrologySensor::RawValue () ]]
  ****/
  dRawValue = this->RawValue ();
  
  // Compute the dither value to compensate for harware pzt dithering / calibrate or check
  // dither waveform.
  /****
  [[U this->dDitherValue := this->Dither.tDlDither::DoDither (tUpdateTime, dLaserValue) ]]
  ****/
  this->dDitherValue = this->Dither.DoDither (tUpdateTime, dLaserValue);
  
   
  if (this->iLaserSign < 0) { // invert value for negative laser sign
  
    // Set the value of the sensor to the sum of the raw value, modulation value, and - the dither value.
    /****
    [[U dMod := tLaserMetrologySensor::ModulationValue () ]]
    
    [[U tLaserMetrologySensor::SetValue (- (dRawValue + dMod - this->dDitherValue)) ]]
    ****/
/* THIS SEEMS TO BE WRONG... anding modulation twice! 
    this->SetValue ( -(dRawValue + this->ModulationValue () - this->dDitherValue) );
*/
    this->SetValue ( -(dRawValue - this->dDitherValue) );
  }
  
  else {
  
    // Set the value of the sensor to the sum of the raw value, modulation value and - the dither value.
    /****
    [[U dMod := tLaserMetrologySensor::ModulationValue () ]]
    
    [[U tLaserMetrologySensor::SetValue (dRawValue + dMod - this->dDitherValue) ]]
    ****/

/* THIS SEEMS TO BE WRONG... anding modulation twice! 
    this->SetValue (dRawValue + this->ModulationValue () - this->dDitherValue);
*/
    this->SetValue (dRawValue - this->dDitherValue);
  }

  // returns the offset compensated sensor value. 
  /****
  [[U tLaserMetrologySensor::Value () ]]
  ****/
  return this->Value ();
  
}

