/* $RCSfile: modLib.cc,v $ - modulation class implementation */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/modLib.cc,v 1.2 1998/08/17 21:00:47 phil Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */

/*
modification history
--------------------
05a,02Feb98,rlj     cleaned up specification and code
01b,08Jul97,eamck   changed ints to doubles
01a,22Oct96,eamck   written
*/
      
/*
DESCRIPTION
  ISIT modulation library definitions
*/

#include <stdio.h>
#include <math.h>

#include "isit.h"
#include "modLib.h"
#include "errorStream.h"





/******************************************************************************
* tModulation::tModulation (void)
*
* DESCRIPTION:
[[U2 tModulation::tModulation (void) ]]
*
* SPECIFICATION: 
[[U3 tModulation::tModulation (void) ]]
*/
tModulation::tModulation () :
   
   /****
   [[U mParams.tModulationParams::tModulationParams (void) ]]
   ****/
   mpParams (),
   bActive (FALSE),
   tPeriod (1) 
{}


/******************************************************************************
* 
* tModulation::tModulation (const tModulationParams& InitalParams)
* 
* DESCRIPTION:
[[U2 tModulation::tModulation_t (InitialParams) ]]
*
[[U3 tModulation::tModulation_t (InitialParams) ]]
*/
tModulation::tModulation (const tModulationParams& InitialParameters) :

  mpParams (InitialParameters),
  bActive (FALSE) 
{

  /****
  [[U tModulation::ComputePeriod (this->mpParams.dFrequency) ]]
  ****/
  this->ComputePeriod (this->mpParams.dFrequency);
  
  /****
  [[U tModulation::ComputeTaus (void) ]]
  ****/
  this->ComputeTaus ();
}



/******************************************************************************
* 
* tModulation::tModulation (MOD_TYPE mtType, double dFrequency
                 double dAmplitude double dMeanVal, TIME tSyncTime,
                 double dDutyCycle)
*    
* DESCRIPTION:
[[U2 tModulation::tModulation_MTdddtd (mtType, dFrequency,
                 dAmplitude, dMeanVal, tSyncTime, dDutyCycle)]]
*
[[U3 tModulation::tModulation_MTdddtd (mtType, dFrequency,
                 dAmplitude, dMeanVal, tSyncTime, dDutyCycle)]]
*/
tModulation::tModulation (MOD_TYPE mtType, double dFrequency,
                 double dAmplitude, double dMeanVal, TIME tSyncTime,
                 double dDutyCycle) :
   mpParams (mtType, dFrequency, dAmplitude, dMeanVal, tSyncTime, dDutyCycle),
   bActive (FALSE)
   
{
  /****
  [[U tModulation::ComputePeriod (dFrequency) ]]
  ****/
  this->ComputePeriod (dFrequency);
  
  // Set up duty cycle tau's
  /****
  [[U tModulation::ComputeTaus (void) ]]
  ****/
  this->ComputeTaus (); 
}


/******************************************************************************
*
* tModulation::SetParameters (MOD_TYPE mtType,
             double dFrequency, double dAmplitude, 
             double dMeanVal, tSyncTime, double dDutyCycle)
*    
* DESCRIPTION:
[[U2 tModulation::SetParameters_MTdddtd (mtType, dFrequency, dAmplitude, 
              dMeanVal, tSyncTime, dDutyCycle) ]]
*
[[U3 tModulation::SetParameters_MTdddtd (mtType, dFrequency, dAmplitude, 
              dMeanVal, tSyncTime, dDutyCycle) ]]
*/
ERROR_CODE tModulation::SetParameters (MOD_TYPE mtType,
             double dFrequency, double dAmplitude, 
             double dMeanVal, TIME tSyncTime, double dDutyCycle)
{
   /*****
   [[U this->mpParams.tModulationParams::tModulationParams_MTdddtd (mtType, 
         dFrequency, dAmplitude, dMeanVal, tSyncTime, dDutyCycle) ]]
   ****/
   this->mpParams = tModulationParams (mtType, dFrequency,
     dAmplitude, dMeanVal, tSyncTime, dDutyCycle);

  /****
  [[U tModulation::ComputePeriod (dFrequency) ]]
  ****/
  this->ComputePeriod (dFrequency);
  
  // Set up duty cycle tau's
  /****
  [[U tModulation::ComputeTaus (void) ]]
  ****/
  this->ComputeTaus (); 

  return OK;
   
}   /* tModulation::SetParameters() */


/******************************************************************************
*
* tModulation::SetParameters (const tModulationParams& NewModParams)
*    
* DESCRIPTION:
[[U2 tModulation::SetParameters_t (NewModParams) ]]
*
[[U3 tModulation::SetParameters_t (NewModParams) ]]
*/
ERROR_CODE tModulation::SetParameters (const tModulationParams &NewModParams)
{
  this->mpParams = NewModParams;

  /****
  [[U tModulation::ComputePeriod (this->mpParams.dFrequency) ]]
  ****/
  this->ComputePeriod (this->mpParams.dFrequency);
  
  // Set up duty cycle tau's
  /****
  [[U tModulation::ComputeTaus (void) ]]
  ****/
  this->ComputeTaus (); 

  return OK;

}   




/******************************************************************************
*
* tModulation::GetParameters ()
*
* DESCRIPTION
[[U2 tModulation::GetParameters (void) ]]
*
[[U3 tModulation::GetParameters (void) ]]
*/
inline const tModulationParams& tModulation::GetParameters () const
{
  return this->mpParams;
}

/******************************************************************************
*
* tModulation::Reset ()
*
* DESCRIPTION:
[[U2 tModulation::Reset (void) ]]
*
[[U3 tModulation::Reset (void) ]]
*/
void tModulation::Reset( void )
{
  this->mpParams = tModulationParams ();
  this->bActive = FALSE;
}
 

/******************************************************************************
*
* tModulation::Enable ()
*
* DESCRIPTION:
[[U2 tModulation::Enable (void) ]]
*
[[U3 tModulation::Enable (void) ]]
*/
inline void tModulation::Enable ()
{
  this->bActive = TRUE;
}

/******************************************************************************
*
* tModulation::Disable ()
*
* DESCRIPTION:
[[U2 tModulation::Disable (void) ]]
*
[[U3 tModulation::Disable (void) ]]
*/
inline void tModulation::Disable ()
{
  this->bActive = FALSE;
}

/******************************************************************************
*
* tModulation::IsEnabled ()
*
[[U tModulation::IsEnabled () ]]
*/
inline BOOL tModulation::IsEnabled () const
{
  return this->bActive;
}
/******************************************************************************
*
* tModulation::SyncNow ()
*
* DESCRIPTION
[[U2 tModulation::SyncNow (void) ]]
*
[[U3 tModulation::SyncNow (void) ]]
*/
inline void tModulation::SyncNow () 
{
  /****
  [[U this->mpParams.tSyncTime := ::GetTime (void) ]]
  ****/
  this->mpParams.tSyncTime = ::GetTime ();
}


/******************************************************************************
*
* tModulation::SyncAt (TIME tNewSyncTime)
*
* DESCRIPTION
[[U2 tModulation::SyncAt (tNewSyncTime) ]]
*
[[U3 tModulation::SyncAt (tNewSyncTime) ]]
*/
inline void tModulation::SyncAt (TIME tNewSyncTime) 
{
  this->mpParams.tSyncTime = tNewSyncTime;
}




/******************************************************************************
*
* tModulation::ComputeModulation (TIME tUpdateTime)
*    
* DESCRIPTION
[[U2 tModulation::ComputeModulation (tUpdateTime) ]]
*
[[U3 tModulation::ComputeModulation (tUpdateTime) ]]
*/
double tModulation::ComputeModulation( TIME tUpdateTime )
{
  TIME tCycleTime;

  if (!this->bActive) return 0;
  
       /* first, calculate where we are in the waveform cycle */

  if (tUpdateTime < this->mpParams.tSyncTime)  // catch early morning bug
     tCycleTime = tPeriod - 
		 ((tUpdateTime - this->mpParams.tSyncTime) % this->tPeriod);
  else
     tCycleTime = (tUpdateTime - this->mpParams.tSyncTime) % this->tPeriod;


       /* here, output waveform value for that point in cycle */

  switch ( this->mpParams.mtModulationType )
    {
    case MT_NONE: return 0;
      
    case MT_SQUARE:
      if (tCycleTime < this->tTau0)
        return ( this->mpParams.dMeanVal + this->mpParams.dAmplitude );
      else   
        return ( this->mpParams.dMeanVal - this->mpParams.dAmplitude );

    case MT_SINE:
      return this->mpParams.dMeanVal + this->mpParams.dAmplitude *
        sin (2 * PI * (double) tCycleTime / (double)this->tPeriod);
      
    case MT_TRIANGLE:
      if ( tCycleTime < this->tTau0 )
        return this->mpParams.dMeanVal + 
          this->mpParams.dAmplitude * tCycleTime / (double) this->tTau0;
      else if ( (this->tTau0 <= tCycleTime) && (tCycleTime < (this->tTau0 + 2 * this->tTau1)) )
        return this->mpParams.dMeanVal -
          this->mpParams.dAmplitude * 
            ((double)tCycleTime - (double)this->tPeriod / 2.0) / this->tTau1;
      else              
        return this->mpParams.dMeanVal + 
          this->mpParams.dAmplitude * ((double) tCycleTime - (double)this->tPeriod) / (double) this->tTau0;
            

    case MT_WOBBLE:
      if ( this->tTau0 <= tCycleTime &&
                tCycleTime < this->tTau0 + tTau1 ) 
                  return this->mpParams.dMeanVal + this->mpParams.dAmplitude;
      else if ( (2 * this->tTau0 + this->tTau1 <= tCycleTime) &&
                (tCycleTime < 2 * (this->tTau0 + tTau1) ) )
                  return this->mpParams.dMeanVal - this->mpParams.dAmplitude;
      else return this->mpParams.dMeanVal;      

    default:
      TraceMsg (tmt_ERROR, S_modLib_INVALID_MOD_TYPE,
		"tModulation::ComputeModulation -- Invalid Modulation Type");

      return 0;     /* 0 output should get someone's attention eventually */

    }  /* switch */

}   /* tModulation::ComputeModulation() */



/******************************************************************************
*
* tModulation::ComputePeriod (double dFreq)
*
* DESCRIPTION:
[[U2 tModulation::ComputePeriod (dFreq) ]]
*
[[U3 tModulation::ComputePeriod (dFreq) ]]
*/
void tModulation::ComputePeriod (double dFreq) 
{
  if (dFreq == 0) {
    ::GetErrorStream().Put(S_modLib_INVALID_MOD_FREQ);
    this->tPeriod = 1;
  }
  else this->tPeriod = (TIME) (TIME_TICKS_PER_SECOND / dFreq);
}


/******************************************************************************
*
* tModulation::ComputeTaus () 
* 
* DESCRIPTION:
[[U2 tModulation::ComputeTaus (void) ]]
*
[[U3 tModulation::ComputeTaus (void) ]]
*****/
void tModulation::ComputeTaus () 
{
   switch (this->mpParams.mtModulationType) {
     case MT_SQUARE :
       this->tTau0 = (TIME) (this->mpParams.dDutyCycle * this->tPeriod);
       this->tTau1 = this->tPeriod - this->tTau0;
       break;
     case MT_TRIANGLE :
       this->tTau0 = (TIME) (this->mpParams.dDutyCycle * this->tPeriod / 2);
       this->tTau1 = this->tPeriod / 2 - this->tTau0;
       break;
     case MT_WOBBLE :
       this->tTau0 = (TIME) (this->mpParams.dDutyCycle * this->tPeriod / 2);
       this->tTau1 = (this->tPeriod - 2 * this->tTau0) / 2;
       break;
     
     // duty cycle is not meaningful for other modulation types
     default :
       this->tTau1 = 0;
       this->tTau0 = this->tPeriod;
   }
   
}
