/* $RCSfile: outputDest.cc,v $ - Implements abstract hardware and software Output classes */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/outputDest.cc,v 1.1.1.1 1998/06/11 21:36:19 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,20Apr97    written
*/
 
/*
DESCRIPTION

*/

#include "outputDest.h"
#include "hwTypes.h"
#include "outputChannel.h"

#include "logLib.h"


/*******************************************************************
*
* tOutputDestination::LimitValue 
*
* DESCRIPTION:
[[U2 tOutputDestination::LimitValue_dd (dNewValue, dLimit) ]]
*
[[U3 tOutputDestination::LimitValue_dd (dNewValue, dLimit) ]]
*/
double tOutputDestination::LimitValue (double dNewValue, double dLimit) const
{

  if (dNewValue > dLimit) return dLimit;
  else if (dNewValue < -dLimit ) return -dLimit;
  else return dNewValue;
}


/*******************************************************************
*
* tOutputDestination::LimitValue 
*
* DESCRIPTION:
[[U2 tOutputDestination::LimitValue_ddd (dNewValue, dLowerLimit, dUpperLimit) ]]
*
[[U3 tOutputDestination::LimitValue_ddd (dNewValue, dLowerLimit, dUpperLimit) ]]
*/
double tOutputDestination::LimitValue (double dNewValue, double dLowerLimit, 
        double dUpperLimit) const
{

  if (dNewValue > dUpperLimit) return dUpperLimit;
  else if (dNewValue < dLowerLimit ) return dLowerLimit;
  else return dNewValue;
}


/***************************  tActuator  ***************************/


/******************************************************************
*
* tActuator::tActuator (tOutputChannel &ChannelIn,
    double dPhysicalUnitsPerElectricalUnit)
*
* DESCRIPTION: 
[[U2 tActuator::tActuator_td (ChannelIn, dPhysicalUnitsPerElectricalUnit) ]]
*
[[U3 tActuator::tActuator_td (ChannelIn, dPhysicalUnitsPerElectricalUnit)  ]]
*/
tActuator::tActuator (tOutputChannel &ChannelIn,
     double dPhysicalUnitsPerElectricalUnit) :

  /****
  [[U tOutputDestination::tOutputDestination () ]]
  ****/
  tOutputDestination (),

  /****
  [ Channel := reference to ChannelIn ]
  ****/
  Channel (ChannelIn),
  
  /*****
  [[U dElectricalUnitsPerCount := ChannelIn.tOutputChannel::GetChannelUnitsPerCount () ]]
  
  [ this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnits * 
      dElectricalUnitsPerCount ]
  ****/
  dPhysicalUnitsPerCount (dPhysicalUnitsPerElectricalUnit *
     ChannelIn.GetChannelUnitsPerCount () )
  
{
  TraceMark(S_outputDest_TACTUATOR_CONSTRUCTOR);
}


/******************************************************************
*
* tActuator::tActuator (tOutputChannel &ChannelIn,
    double dPhysicalUnitsPerElectricalUnit, double dOffsetIn)
*
* DESCRIPTION: 
[[U2 tActuator::tActuator_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit, dOffsetIn) ]]
*
[[U3 tActuator::tActuator_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit, dOffsetIn)  ]]
*/
tActuator::tActuator (tOutputChannel &ChannelIn,
     double dPhysicalUnitsPerElectricalUnit, double dOffsetIn) :

  /****
  [[U tOutputDestination::tOutputDestination_d (dOffsetIn)]]
  ****/
  tOutputDestination (dOffsetIn),

  /****
  [ Channel := reference to ChannelIn ]
  ****/
  Channel (ChannelIn),
  
  /*****
  [[U dElectricalUnitsPerCount := ChannelIn.tOutputChannel::GetChannelUnitsPerCount () ]]
  
  [ this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnits * 
      dElectricalUnitsPerCount ]
  ****/
  dPhysicalUnitsPerCount (dPhysicalUnitsPerElectricalUnit *
     ChannelIn.GetChannelUnitsPerCount () )
  
{
  TraceMark(S_outputDest_TACTUATOR_CONSTRUCTOR);
}



/******************************************************************
*
* tActuator::MaxValueInChannelUnits
*
[[U2 tActuator::MaxValueInChannelUnits () ]]
*
[[U3 tActuator::MaxValueInChannelUnits () ]]
*/
double tActuator::MaxValueInChannelUnits () const
{
  /****
  [[U this->Channel.tChannel::GetChannelMaxValue () ]]
  ****/
  return this->Channel.GetChannelMaxValue();
}

/******************************************************************
*
* tActuator::MinValueInChannelUnits
*
[[U2 tActuator::MinValueInChannelUnits () ]]
*
[[U3 tActuator::MinValueInChannelUnits () ]]
*/
double tActuator::MinValueInChannelUnits () const
{
  /****
  [[U this->Channel.tChannel::GetChannelMaxValue () ]]
  ****/
  return -this->Channel.GetChannelMaxValue();
}


/******************************************************************
*
* tActuator::Reset
*
[[U2 tActuator::Reset () ]]
*
[[U3 tActuator::Reset () ]]
*/

ERROR_CODE tActuator::Reset ()
{ 
  /****
  [[U tOutputDestination::Reset()]]
  ****/
  this->tOutputDestination::Reset();

  /****
  [[U tActuator::ChannelOutput (0.0) ]]
  ****/
  this->ChannelOutput (0.0);
  
 
  /****
  [[U this->Channel.tOutputChannel::Reset()]]
  ****/
  return this->Channel.Reset();
}


/******************************************************************
*
* tActuator::Output (TIME tUpdateTime)
*
[[U2 tActuator::Output (tUpdateTime) ]]
*
[[U3 tActuator::Output (tUpdateTime) ]]
*/
ERROR_CODE tActuator::Output (TIME tUpdateTime)
{
  
  /****
  [[U dValue := tActuator::Value () ]]
  
  [[U dOffset := tActuator::Offset () ]]
  
  [[U tActuator::ChannelOutput ( (dValue + dOffset)) ]]
  ****/
  this->ChannelOutput (this->Value () + this->Offset () );

  return OK;
}


/******************************************************************
*
* tActuator::ChannelOutput (dOutputValue)
*
[[U2 tActuator::ChannelOutput (dOutputValue) ]]
*
[[U3 tActuator::ChannelOutput (dOutputValue) ]]
*/
ERROR_CODE tActuator::ChannelOutput (double dOutputValue)
{
  
  /****
 [[U this->Channel.tOutputChannel::Output_double (dOutputValue) ]]
  ****/
  this->Channel.Output (dOutputValue);

  return OK;
}


/***********************  tModulatedActuator  ************************/


/******************************************************************
*
* tModulatedActuator::tModulatedActuator
*
[[U2 tModulatedActuator::tModulatedActuator_td (ChannelIn,
      dPhysicalUnitsPerElectricalUnit) ]]
*
[[U3 tModulatedActuator::tModulatedActuator_td (ChannelIn,
      dPhysicalUnitsPerElectricalUnit) ]]
*/

tModulatedActuator::tModulatedActuator (tOutputChannel &ChannelIn,
         double dPhysicalUnitsPerElectricalUnit) :
  /****
  [[U tActuator::tActuator_td (ChannelIn, dPhysicalUnitsPerElectricalUnit)]]
  ****/
  tActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit),

  /****
  [[U tModulation::tModulation () ]]
  ****/
  Modulation ()
  
{
  TraceMark(S_outputDest_TMODULATEDACTUATOR_CONSTRUCTOR);
}


/******************************************************************
*
* tModulatedActuator::tModulatedActuator
*
[[U2 tModulatedActuator::tModulatedActuator_tdt (ChannelIn,
      dPhysicalUnitsPerElectricalUnit, InitialParameters) ]]
*
[[U3 tModulatedActuator::tModulatedActuator_tdt (ChannelIn,
      dPhysicalUnitsPerElectricalUnit, InitialParameters) ]]
*/

tModulatedActuator::tModulatedActuator (tOutputChannel &ChannelIn,
         double dPhysicalUnitsPerElectricalUnit,
         const tModulationParams& InitialParameters) :
  /****
  [[U tActuator::tActuator_td (ChannelIn, dPhysicalUnitsPerElectricalUnit)]]
  ****/
  tActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit),

  /****
  [[U tModulation::tModulation_t (InitialParameters) ]]
  ****/
  Modulation (InitialParameters)

{
  TraceMark(S_outputDest_TMODULATEDACTUATOR_CONSTRUCTOR);
}


/******************************************************************
*
* tModulatedActuator::tModulatedActuator
*
[[U2 tModulatedActuator::tModulatedActuator_tdtd (ChannelIn,
      dPhysicalUnitsPerElectricalUnit, InitialParameters, dOffsetIn) ]]
*
[[U3 tModulatedActuator::tModulatedActuator_tdtd (ChannelIn,
      dPhysicalUnitsPerElectricalUnit, InitialParameters, dOffsetIn) ]]
*/

tModulatedActuator::tModulatedActuator (tOutputChannel &ChannelIn,
         double dPhysicalUnitsPerElectricalUnit,
         const tModulationParams& InitialParameters, double dOffsetIn) :
  /****
  [[U tActuator::tActuator_tdd (ChannelIn, dPhysicalUnitsPerElectricalUnit, dOffsetIn) ]]
  ****/
  tActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit),

  /****
  [[U tModulation::tModulation_t (InitialParameters) ]]
  ****/
  Modulation (InitialParameters)

{
  TraceMark(S_outputDest_TMODULATEDACTUATOR_CONSTRUCTOR);
}


/******************************************************************
*
* tModulatedActuator::SetModulationParameters
*
* DESCRIPTION
*
[[U2 tModulatedActuator::SetModulationParameters (NewParameters) ]]
*
[[U3 tModulatedActuator::SetModulationParameters (NewParameters) ]]
*/
ERROR_CODE tModulatedActuator::SetModulationParameters (const tModulationParams& NewParameters)
{
  ERROR_CODE ec;
  
  TraceMark(S_outputDest_TMODULATEDACTUATOR_SET_MODULATION);
  
  /****
  [[U ec := tModulation::SetParameters_t (NewParameters) ]]
  ****/
  ec = this->Modulation.SetParameters (NewParameters);
  
  if (ec  != OK ) return Traceback (ec, "tModulatedActuator::SetModualtion");
  return OK;

}



/******************************************************************
*
* tModulatedActuator::ModulationParameters
*
* DESCRIPTION
*
[[U2 tModulatedActuator::ModulationParameters () ]]
*
[[U3 tModulatedActuator::ModulationParameters () ]]
*/
const tModulationParams& tModulatedActuator::ModulationParameters () const
{
 
  /****
  [[U tModulation::GetParameters () ]]
  ****/
  return this->Modulation.GetParameters ();
    
}


/******************************************************************
*
* tModulatedActuator::ModulationValue
*
* DESCRIPTION
*
[[U2 tModulatedActuator::ModulationValue () ]]
*
[[U3 tModulatedActuator::ModulationValue () ]]
*/
double tModulatedActuator::ModulationValue () const
{
 
  return this->dModulationValue;
    
}




/******************************************************************
*
* tModulatedActuator::EnableModulation
*
* DESCRIPTION
*
[[U2 tModulatedActuator::EnableModulation () ]]
*
[[U3 tModulatedActuator::EnableModulation () ]]
*/
ERROR_CODE tModulatedActuator::EnableModulation ()
{
 ERROR_CODE ec;
 
 this->Modulation.Enable ();
 return  OK;
    
}


/******************************************************************
*
* tModulatedActuator::DisableModulation
*
* DESCRIPTION
*
[[U2 tModulatedActuator::DisableModulation () ]]
*
[[U3 tModulatedActuator::DisableModulation () ]]
*/
ERROR_CODE tModulatedActuator::DisableModulation ()
{
 
  this->Modulation.Disable ();
  return OK;
    
}


/******************************************************************
*
* tModulatedActuator::IsModulationEnabled
*
* DESCRIPTION
*
[[U2 tModulatedActuator::IsModulationEnabled () ]]
*
[[U3 tModulatedActuator::IsModulationEnabled () ]]
*/
BOOL tModulatedActuator::IsModulationEnabled () const
{
 return this->Modulation.IsEnabled ();
}



/******************************************************************
*
* tModulatedActuator::Reset
*
* DESCRIPTION
[[U2 tModulatedActuator::Reset () ]]
*
[[U3 tModulatedActuator::Reset () ]]
*/
ERROR_CODE tModulatedActuator::Reset ()
{
  ERROR_CODE ec;
  
  this->dModulationValue = 0.0;
  
  /****
  [[U ec := tActuator::Reset () ]]
  ****/
  ec = tActuator::Reset ();
  
  
    
  if (ec != OK) return Traceback (ec, "tModulatedActuator::Reset");
  return OK;

}



/*******************************************************************
*
* tModulatedActuator::Output
*
* DESCRIPTION:
[[U2 tModulatedActuator::Output (tUpdateTime) ]]
*
[[U3 tModulatedActuator::Output (tUpdateTime) ]]
*/
ERROR_CODE tModulatedActuator::Output (TIME tUpdateTime)
{

  ERROR_CODE ec;

  /****
  [[U this->dModulationValue := tModulation::ComputeModulation (tUpdateTime) ]]
  ****/
  this->dModulationValue = this->Modulation.ComputeModulation (tUpdateTime);
  

  /****
  [[U dTemp1 := this->tModulatedActuator::Value () ]]

  [[U dTemp2 := this->tModulatedActuator::Offset () ]]
  
  [[U tActuator::ChannelOutput ((dTemp1 + dTemp2 + this->dModulationValue)) ]] 
  ****/
  ec = tActuator::ChannelOutput (this->Value () + this->Offset () + this->dModulationValue);

  if (ec != OK) return Traceback (ec, "tModulatedActuator::Output");
  
  return OK;
}



/******************************************************************
*
* tModulatedActuator::UnmodulatedOutput 
*
* DESCRIPTION: 
[[U2 tModulatedActuator::UnmodulatedOutput (tUpdateTime) ]]
*
[[U3 tModulatedActuator::UnmodulatedOutput (tUpdateTime) ]]
*/

ERROR_CODE tModulatedActuator::UnmodulatedOutput (TIME tUpdateTime)
{
  ERROR_CODE ec;
  
  
  
  /****
  [[U dTemp1 := tModulatedActuator::Value () ]]

  [[U dTemp2 := tModulatedActuator::Offset () ]]
  
  [[U ec := tModulatedActuator::ChannelOutput (dTemp1 + dTemp2) ]]
  ****/
  ec = this->ChannelOutput (this->Value () + this->Offset ());

  if (ec != OK) return Traceback (ec, "tModulatedActutator::UnmodulatedOutput");
  return OK;
}

/******************************************************************
*
* tModulatedActuator::UnmodulatedOutputValue 
*
* DESCRIPTION: 
[[U2 tModulatedActuator::UnmodulatedOutputValue (tUpdateTime, dNewValue) ]]
*
[[U3 tModulatedActuator::UnmodulatedOutputValue (tUpdateTime, dNewValue) ]]
*/

ERROR_CODE tModulatedActuator::UnmodulatedOutputValue (TIME tUpdateTime, double dNewValue)
{
  ERROR_CODE ec;
  
  /****
  [[U ec := tModulatedActuator::SetValue (dNewValue) ]]
  ****/
  this->SetValue (dNewValue);
  
  /****
  [[U ec := tModulatedActuator::UnmodulatedOutput (tUpdateTime) ]]
  ****/
  ec = this->UnmodulatedOutput (tUpdateTime);
  
  if (ec != OK) return Traceback (ec, "tModulatedActutator::UnmodulatedOutputValue");
  return OK;
}





/****************  tModulatedAccelerationActuator  *****************/

/******************************************************************
*
* tModulatedAccelerationActuator::tModulatedAccelerationActuator
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tdd (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn) ]]
*
[[U3 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tdd (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn) ]]
*/
tModulatedAccelerationActuator::tModulatedAccelerationActuator (
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dOutputRateIn) :
  
  /****
  [[U tModulatedActuator::tModulatedActuator_td (ChannelIn, 
       dPhysicalUnitsPerElectricalUnit) ]]
  ****/
  tModulatedActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit),
  
  /**** 
  Compute maximum acceleration as the minimum of the maximum physical accel converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedAccelerationActuator::PhysicalUnitsToCounts (dMaxAccelerationIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxAcceleration (MIN (this->PhysicalUnitsToCounts (dMaxAccelerationIn), this->tActuator::MaxValueInChannelUnits ())),
  
  dOutputRate (dOutputRateIn)
  

{
  TraceMark(S_outputDest_TMOD_ACCEL_ACTUATOR_CONSTRUCTOR);
}



/******************************************************************
*
* tModulatedAccelerationActuator::tModulatedAccelerationActuator
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddt (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn, InitialParameters) ]]
*
[[U3 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddt (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn, InitialParameters) ]]
*/
tModulatedAccelerationActuator::tModulatedAccelerationActuator (
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dOutputRateIn, const tModulationParams& InitialParameters) :
  
  /****
  [[U tModulatedActuator::tModulatedActuator_tdt (ChannelIn, 
       dPhysicalUnitsPerElectricalUnit, InitialParameters) ]]
  ****/
  tModulatedActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit, InitialParameters),

  /**** 
  Compute maximum acceleration as the minimum of the maximum physical accel converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedAccelerationActuator::PhysicalUnitsToCounts (dMaxAccelerationIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxAcceleration (MIN (this->PhysicalUnitsToCounts (dMaxAccelerationIn), this->tActuator::MaxValueInChannelUnits ())),
  
  dOutputRate (dOutputRateIn)
  

{
  TraceMark(S_outputDest_TMOD_ACCEL_ACTUATOR_CONSTRUCTOR);
}


/******************************************************************
*
* tModulatedAccelerationActuator::tModulatedAccelerationActuator
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddtd (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn, InitialParameters, dOffsetIn) ]]
*
[[U3 tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddtd (ChannelIn, 
      dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn, InitialParameters, dOffsetIn) ]]
*/
tModulatedAccelerationActuator::tModulatedAccelerationActuator (
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dOutputRateIn, const tModulationParams& InitialParameters,
   double dOffsetIn) :
  
  /****
  [[U tModulatedActuator::tModulatedActuator_tdtd (ChannelIn, 
       dPhysicalUnitsPerElectricalUnit, InitialParameters, dOffsetIn) ]]
  ****/
  tModulatedActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit, 
                      InitialParameters, dOffsetIn),

  /**** 
  Compute maximum acceleration as the minimum of the maximum physical accel converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedAccelerationActuator::PhysicalUnitsToCounts (dMaxAccelerationIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxAcceleration (MIN (this->PhysicalUnitsToCounts (dMaxAccelerationIn), this->tActuator::MaxValueInChannelUnits ())),
  
  dOutputRate (dOutputRateIn)
  

{
  TraceMark(S_outputDest_TMOD_ACCEL_ACTUATOR_CONSTRUCTOR);
}



/*******************************************************************
*
* tModulatedAccelerationActuator::MaxAcceleration
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::MaxAcceleration () ]]
*
[[U3 tModulatedAccelerationActuator::MaxAcceleration () ]]
*/
double tModulatedAccelerationActuator::MaxAcceleration () const
{
  TraceMark(S_outputDest_GET_MAX_ACCEL);

  return this->CountsToPhysicalUnits (dMaxAcceleration);
}


/*******************************************************************
*
* tModulatedAccelerationActuator::MaxAccelerationInChannelUnits
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::MaxAccelerationInChannelUnits ()]]
*
[[U3 tModulatedAccelerationActuator::MaxAccelerationInChannelUnits ()]]
*/

double tModulatedAccelerationActuator::MaxAccelerationInChannelUnits () const
{
  
  return this->dMaxAcceleration;
}


/*******************************************************************
*
* tModulatedAccelerationActuator::SetValue
*
[[U5 tModulatedAccelerationActuator::SetValue (dNewValue) ]]
*/

void tModulatedAccelerationActuator::SetValue (double dNewValue)  
{
  
  /****
  [[U dLimittedAccel := tModulatedAccelerationActuator::LimitAcceleration (dNewValue) ]]
  ****/
  double dLimittedAccel = this->LimitAcceleration (dNewValue);
  
  /****
  [[U tModulatedActuator::SetValue (dLimittedAccel) ]]
  ****/
  this->tModulatedActuator::SetValue (dLimittedAccel);
}



/*******************************************************************
*
* tModulatedAccelerationActuator::LimitAcceleration
*
* DESCRIPTION:
[[U2 tModulatedAccelerationActuator::LimitAcceleration (dAcceleration) ]]
*
[[U3 tModulatedAccelerationActuator::LimitAcceleration (dAcceleration) ]]
*/
double tModulatedAccelerationActuator::LimitAcceleration (double dAcceleration) const
{
  /****
  [[U tModulatedAccelerationActuator::LimitValue_dd (dAcceleration, this->dMaxAcceleration) ]]
  ****/ 
  return this->LimitValue (dAcceleration, this->dMaxAcceleration);
}



/******************  tModulatedVelocityActuator  *******************/

/******************************************************************
*
* tModulatedVelocityActuator::tModulatedVelocityActuator
*
* DESCRIPTION
[[U2 tModulatedVelocityActuator::tModulatedVelocityActuator_tddd (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn)]]
*
[[U3 tModulatedVelocityActuator::tModulatedVelocityActuator_tddd (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn)]]
*/
tModulatedVelocityActuator::tModulatedVelocityActuator(
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn,
   double dMaxVelocityIn, double dOutputRateIn) :
  
  /****
    [[U tModulatedAccelerationActuator::tModulatedAccelerationActuator_tdd (
       ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn) ]]
  ****/
  tModulatedAccelerationActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                                  dMaxAccelerationIn, dOutputRateIn),

  /**** 
  Compute maximum velocity as the minimum of the maximum physical velocity converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedVelocityActuator::PhysicalUnitsToCounts (dMaxVelocityIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxVelocity (MIN (this->PhysicalUnitsToCounts (dMaxVelocityIn), this->tActuator::MaxValueInChannelUnits ())),
  
  dLastVelocity (0.0)
 
{
  /****
  Overide maximum acceleration set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxAcceleration (dMaxAccelerationIn)
  ****/
  this->SetMaxAcceleration (dMaxAccelerationIn);
  

}

/******************************************************************
*
* tModulatedVelocityActuator::tModulatedVelocityActuator
*
* DESCRIPTION
[[U2 tModulatedVelocityActuator::tModulatedVelocityActuator_tdddt (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn,
   InitialParameters)]]
*
[[U3 tModulatedVelocityActuator::tModulatedVelocityActuator_tdddt (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn,
   InitialParameters)]]
*/
tModulatedVelocityActuator::tModulatedVelocityActuator(
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dOutputRateIn,
   double dMaxVelocityIn, const tModulationParams& InitialParameters) :
  
  /****
  [[U tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddt (
       ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn,
       InitialParameters) ]]
  ****/
  tModulatedAccelerationActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                                  dMaxAccelerationIn, dOutputRateIn, InitialParameters),

  /**** 
  Compute maximum velocity as the minimum of the maximum physical velocity converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedVelocityActuator::PhysicalUnitsToCounts (dMaxVelocityIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxVelocity (MIN (this->PhysicalUnitsToCounts (dMaxVelocityIn), this->tActuator::MaxValueInChannelUnits ())),
  
  dLastVelocity (0.0)

{
  /****
  Overide maximum acceleration set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxAcceleration (dMaxAccelerationIn)
  ****/
  this->SetMaxAcceleration (dMaxAccelerationIn);
}


/******************************************************************
*
* tModulatedVelocityActuator::tModulatedVelocityActuator
*
* DESCRIPTION
[[U2 tModulatedVelocityActuator::tModulatedVelocityActuator_tdddtd (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn,
   InitialParameters, dOffsetIn)]]
*
[[U3 tModulatedVelocityActuator::tModulatedVelocityActuator_tdddtd (ChannelIn,
   dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn,
   InitialParameters, dOffsetIn)]]
*/
tModulatedVelocityActuator::tModulatedVelocityActuator(
   tOutputChannel &ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn,
   double dMaxVelocityIn, double dOutputRateIn, const tModulationParams& InitialParameters,
   double dOffsetIn) :
  
  /****
  [[U tModulatedAccelerationActuator::tModulatedAccelerationActuator_tddtd (
       ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, dOutputRateIn,
       InitialParameters, dOffsetIn) ]]
  ****/
  tModulatedAccelerationActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                                  dMaxAccelerationIn, dOutputRateIn, InitialParameters, dOffsetIn),

  /**** 
  Compute maximum velocity as the minimum of the maximum physical velocity converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedVelocityActuator::PhysicalUnitsToCounts (dMaxVelocityIn) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  dMaxVelocity (MIN (this->PhysicalUnitsToCounts (dMaxVelocityIn), this->tActuator::MaxValueInChannelUnits ())),
   
  dLastVelocity (0.0)

{
  /****
  Overide maximum acceleration set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxAcceleration (dMaxAccelerationIn)
  ****/
  this->SetMaxAcceleration (dMaxAccelerationIn);
}




/*******************************************************************
*
* tModulatedVelocityActuator::MaxVelocity
*
* DESCRIPTION: Returns max velocity in channel units
[[U2 tModulatedVelocityActuator::MaxVelocity ()]]
*
[[U3 tModulatedVelocityActuator::MaxVelocity ()]]
*/
double tModulatedVelocityActuator::MaxVelocity () const
{
  TraceMark(S_outputDest_GET_MAX_VEL);

  return this->CountsToPhysicalUnits (dMaxVelocity);
}


/*******************************************************************
*
* tModulatedVelocityActuator::MaxVelocityInChannelUnits
*
* DESCRIPTION
[[U2 tModulatedVelocityActuator::MaxVelocityInChannelUnits ()]]
*
[[U2 tModulatedVelocityActuator::MaxVelocityInChannelUnits ()]]
*/
double tModulatedVelocityActuator::MaxVelocityInChannelUnits () const
{
  
  return this->dMaxVelocity;
}


/*******************************************************************
*
* tModulatedVelocityActuator::SetValue
*
[[U5 tModulatedVelocityActuator::SetValue (dNewValue)]]
*/
void tModulatedVelocityActuator::SetValue (double dNewValue)
{

  double dLimittedVelocity;
  
  /****
  [[U this->dLastVelocity := tModulatedVelocityActuator::Velocity () ]]
  ****/
  this->dLastVelocity = this->Velocity ();


  /****
  [[U dLimittedVelocity := tModulatedVelocityActuator::
        LimitVelocity (this->dLastVelocity, dNewVelocity) ]]
  ****/
  dLimittedVelocity = this->LimitVelocity (this->dLastVelocity, dNewValue);
  
  
   // Do not want to use tModulatedAccelerationActuator version because it applies different limitting 
  /****
  [[U tModulatedActuator::SetValue (dLimittedVelocity) ]]
  ****/
  this->tModulatedActuator::SetValue (dLimittedVelocity);
      
 
}


/*******************************************************************
*
* tModulatedVelocityActuator::Acceleration
*
[[U2 tModulatedVelocityActuator::Acceleration ()]]
*
[[U3 tModulatedVelocityActuator::Acceleration ()]]
*/
double tModulatedVelocityActuator::Acceleration () const 
{
  /****
  [[U dCurrentVelocity := tModulatedVelocityActuator::Velocity () ]]
  
  [[U dRate := tModulatedVelocityActutator::OutputRate () ]]
  
  [ return := (dCurrentVelocity - this->dLastVelocity) * dRate ]
  ****/
  return (this->Velocity () - this->dLastVelocity) * this->OutputRate ();
}


/*******************************************************************
*
* tModulatedVelocityActuator::Reset
*
[[U5 tModulatedVelocityActuator::Reset ()]]
*/
ERROR_CODE tModulatedVelocityActuator::Reset ()
{

  this->dLastVelocity = 0.0;
  
  
  /****
  [[U tModulatedAccelerationActuator::Reset () ]]
  ****/
  return this->tModulatedAccelerationActuator::Reset () ;

}


/*******************************************************************
*
* tModulatedVelocityActuator::LimitVelocity
*
* DESCRIPTION:
[[U2 tModulatedVelocityActuator::LimitVelocity (dOldVelocity, dNewVelocity) ]]
*
[[U3 tModulatedVelocityActuator::LimitVelocity (dOldVelocity, dNewVelocity) ]]
*/
double tModulatedVelocityActuator::LimitVelocity (double dOldVelocity, 
         double dNewVelocity) const
{
  double dLimittedAcc, dLimittedVel;
  
  /****
  [[U dRate := tModulatedVelocityActuator::OutputRate () ]]
  ****/
  double dRate = this->OutputRate ();

  /****
  [[U dLimittedAcc := tModulatedVelocityActuator::
      LimitAcceleration (((dNewVelocity - dOldVelocity) * dRate) ) ]]
  ****/      
  dLimittedAcc = this->LimitAcceleration ( (dNewVelocity - dOldVelocity) * dRate );
  
  /****
  [[U tModulatedVelocityActuator::
        LimitValue_dd ((dOldVelocity + dLimittedAcc / dRate), this->dMaxVelocity) ]]
  ****/      
  return this->LimitValue (dOldVelocity + dLimittedAcc / dRate, this->dMaxVelocity);
  
}





/*******************************************************************/
/******************  tModulatedPositionActuator  *******************/
/*******************************************************************/

/******************************************************************
*
* tModulatedPositionActuator::tModulatedPositionActuator
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::tModulatedPositionActuator_tddddd (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn)]]
*
[[U3 tModulatedPositionActuator::tModulatedPositionActuator_tddddd (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn)]]
*
*/
tModulatedPositionActuator::tModulatedPositionActuator (
   tOutputChannel& ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dMaxVelocityIn,
   double dMinPositionAllowed, double dMaxPositionAllowed, double dOutputRateIn):
  
  /****
  [[U tModulatedVelocityActuator::tModulatedVelocityActuator_tddd (
         ChannelIn, dPhysicalUnitsPerElectricalUnit,dMaxAccelerationIn,
         dMaxVelocityIn, dOutputRateIn)]]
  ****/
  tModulatedVelocityActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                             dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn),


  /**** 
  Compute minimum position as the maximum of the minimum physical position converted to channel units, 
  and the minimum value of the channel
  
  [[U dPhysLimit := tModulatedPositionActuator::PhysicalUnitsToCounts (dMinPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MinValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MAX (dPhysLimit, dChanLimit) ]
  ****/
  CdMinPosition (MAX (this->PhysicalUnitsToCounts (dMinPositionAllowed), this->tActuator::MinValueInChannelUnits ())),

  /**** 
  Compute maximum position as the minimum of the maximum physical position converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedPositionyActuator::PhysicalUnitsToCounts (dMaxPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  CdMaxPosition (MIN (this->PhysicalUnitsToCounts (dMaxPositionAllowed), this->tActuator::MaxValueInChannelUnits ())),
 
  dLastPosition (0.0)

{
  /****
  Overide maximum velocity set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxVeloctiy (dMaxVelocityIn)
  ****/
  this->SetMaxVelocity (dMaxVelocityIn);
}


/******************************************************************
*
* tModulatedPositionActuator::tModulatedPositionActuator
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::tModulatedPositionActuator_tdddddt (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn, InitialParameters)]]
*
[[U3 tModulatedPositionActuator::tModulatedPositionActuator_tdddddt (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn, InitialParameter)]]
*
*/
tModulatedPositionActuator::tModulatedPositionActuator (
   tOutputChannel& ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dMaxVelocityIn,
   double dMinPositionAllowed, double dMaxPositionAllowed, double dOutputRateIn,
   const tModulationParams& InitialParameters) :
  
  /****
  [[U tModulatedVelocityActuator::tModulatedVelocityActuator_tdddt (
         ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn,
         dMaxVelocityIn, dOutputRate, InitialParameters) ]]
  ****/
  tModulatedVelocityActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                             dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn, InitialParameters),

  /**** 
  Compute minimum position as the maximum of the minimum physical position converted to channel units, 
  and the minimum value of the channel
  
  [[U dPhysLimit := tModulatedPositionActuator::PhysicalUnitsToCounts (dMinPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MinValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MAX (dPhysLimit, dChanLimit) ]
  ****/
  CdMinPosition (MAX (this->PhysicalUnitsToCounts (dMinPositionAllowed), this->tActuator::MinValueInChannelUnits ())),

  /**** 
  Compute maximum position as the minimum of the maximum physical position converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedPositionyActuator::PhysicalUnitsToCounts (dMaxPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  CdMaxPosition (MIN (this->PhysicalUnitsToCounts (dMaxPositionAllowed), this->tActuator::MaxValueInChannelUnits ())),
 
  dLastPosition (0.0)

{
  /****
  Overide maximum velocity set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxVeloctiy (dMaxVelocityIn)
  ****/
  this->SetMaxVelocity (dMaxVelocityIn);
}

/******************************************************************
*
* tModulatedPositionActuator::tModulatedPositionActuator
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::tModulatedPositionActuator_tdddddtd (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn, InitialParameters,
   dOffsetIn)]]
*
[[U3 tModulatedPositionActuator::tModulatedPositionActuator_tdddddtd (
   ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn, 
   dMaxVelocityIn, dMinPositionAllowed, dMaxPositionAllowed, dOutputRateIn, InitialParameters,
   dOffsetIn)]]
*
*/
tModulatedPositionActuator::tModulatedPositionActuator (
   tOutputChannel& ChannelIn,
   double dPhysicalUnitsPerElectricalUnit,
   double dMaxAccelerationIn, double dMaxVelocityIn,
   double dMinPositionAllowed, double dMaxPositionAllowed, double dOutputRateIn,
   const tModulationParams& InitialParameters, double dOffsetIn) :
  
  /****
  [[U tModulatedVelocityActuator::tModulatedVelocityActuator_tdddtd (
         ChannelIn, dPhysicalUnitsPerElectricalUnit, dMaxAccelerationIn,
         dMaxVelocityIn, dOutputRate, InitialParameters, dOffsetIn) ]]
  ****/
  tModulatedVelocityActuator (ChannelIn, dPhysicalUnitsPerElectricalUnit,
                             dMaxAccelerationIn, dMaxVelocityIn, dOutputRateIn, InitialParameters,
                             dOffsetIn),

  /**** 
  Compute minimum position as the maximum of the minimum physical position converted to channel units, 
  and the minimum value of the channel
  
  [[U dPhysLimit := tModulatedPositionActuator::PhysicalUnitsToCounts (dMinPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MinValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MAX (dPhysLimit, dChanLimit) ]
  ****/
  CdMinPosition (MAX (this->PhysicalUnitsToCounts (dMinPositionAllowed), this->tActuator::MinValueInChannelUnits ())),

  /**** 
  Compute maximum position as the minimum of the maximum physical position converted to channel units, 
  and the maximum value of the channel
  
  [[U dPhysLimit := tModulatedPositionyActuator::PhysicalUnitsToCounts (dMaxPositionAllowed) ]]
  
  [[U dChanLimit := tActuator::MaxValueInChannelUnits () ]]
  
  [ this->dMaxAcceleration := MIN (dPhysLimit, dChanLimit) ]
  ****/
  CdMaxPosition (MIN (this->PhysicalUnitsToCounts (dMaxPositionAllowed), this->tActuator::MaxValueInChannelUnits ())),
 
  dLastPosition (0.0)

{
  /****
  Overide maximum velocity set by base class since it is not limited by the maximun channel value.
  
  [[U tModulatedVelocityActuator::SetMaxVeloctiy (dMaxVelocityIn)
  ****/
  this->SetMaxVelocity (dMaxVelocityIn);
}



/*******************************************************************
*
* tModulatedPositionActuator::MinPosition
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::MinPosition ()]]
*
[[U3 tModulatedPositionActuator::MinPosition ()]]
*/
double tModulatedPositionActuator::MinPosition () const
{
  TraceMark(S_outputDest_GET_MIN_POSITION);
  
  return this->CountsToPhysicalUnits (this->CdMinPosition);
}




/*******************************************************************
*
* tModulatedPositionActuator::GetMinPositionInChannelUnits
*
* DESCRIPTION: Returns min position in channel units
*
* RETURNS: double
*
[[U tModulatedPositionActuator::MinPositionInChannelUnits()]]

*/

double tModulatedPositionActuator::MinPositionInChannelUnits () const
{

  return this->CdMinPosition;
}


/*******************************************************************
*
* tModulatedPositionActuator::MaxPosition
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::MaxPosition ()]]
*
[[U3 tModulatedPositionActuator::MaxPosition ()]]
*/
double tModulatedPositionActuator::MaxPosition(void) const
{
  TraceMark(S_outputDest_GET_MAX_POSITION);
  
  return this->CountsToPhysicalUnits (this->CdMaxPosition);
}



/*******************************************************************
*
* tModulatedPositionActuator::MaxPositionInChannelUnits
*
* DESCRIPTION
[[U2 tModulatedPositionActuator::MaxPositionInChannelUnits ()]]
*
[[U2 tModulatedPositionActuator::MaxPositionInChannelUnits ()]]
*/
double tModulatedPositionActuator::MaxPositionInChannelUnits () const
{
  return this->CdMaxPosition;
}


/*******************************************************************
*
* tModulatedPositionActuator::SetValue
*
[[U5 tModulatedPositionActuator::SetValue (dNewValue)]]
*/
void tModulatedPositionActuator::SetValue (double dNewValue)
{

  double dLimittedPosition;
  double dLastVelocity;


  // First compute velocity from current and previous position and store it as the last velocity
  
  /****
  [[U dPos := tModulatedPositionActuator::Position () ]]
  
  [[U dRate := tModulatedPositionActuator::OutputRate () ]]
  
  [ dLastVelocity = (dPos - this->dLastPosition) * dRate ]
  ****/
  dLastVelocity = (this->Position () - this->dLastPosition) * this->OutputRate ();    

  /****
  [[U tModulatedPositionActuator::SetLastVelocity (dLastVelocity) ]]
  ****/
  this->SetLastVelocity (dLastVelocity);
  

  // Now update last position with current position

  /****
  [[U this->dLastPosition := tModulatedPositionActuator::Position () ]]
  ****/
  this->dLastPosition = this->Position ();
  
  // apply position limits to new position
  
  /****
  [[U dLimittedPosition := tModulatedPositionActuator::LimitPosition (this->dLastPosition, dNewValue, dLastVelocity) ]]
  ****/ 
  dLimittedPosition = this->LimitPosition (this->dLastPosition, dNewValue, dLastVelocity);
  
  // set value to limitted position
  
  /****
  [[U tModulatedActuator::SetValue (dLimitedPosition) ]]
  ****/
  this->tModulatedActuator::SetValue (dLimittedPosition);
  
}
  

/*******************************************************************
*
* tModulatedPositionActuator::Reset
*
[[U5 tModulatedPositionActuator::Reset ()]]
*/
ERROR_CODE tModulatedPositionActuator::Reset ()
{

  this->dLastPosition = 0.0;
  
  /****
  [[U tModulatedVelocityActuator::Reset () ]]
  ****/
  return this->tModulatedVelocityActuator::Reset () ;

}

/*******************************************************************
*
* tModulatedPositionActuator::Acceleration
*
[[U5 tModulatedPositionActuator::Acceleration ()]]
*/
double tModulatedPositionActuator::Acceleration () const
{

  /****
  [[U dRate := tModulatedPositionActuator::OutputRate () ]]
  ****/
  double dRate = this->OutputRate ();
  
  /****
  [[U dTemp1 := tModulatedPositionActuator::Value () ]]
  
  [[U dTemp2 := tModulatedPositionActuator::LastVelocity () ]]
  
  [ return := ( (dTemp1 - this->dLastPosition ) * dRate - dTemp2 ) * dRate ]
  ****/  
  return  ( (this->Value () - this->dLastPosition) * dRate  - this->LastVelocity () ) * dRate;
}



/*******************************************************************
*
* tModulatedPositionActuator::Velocity
*
[[U5 tModulatedPositionActuator::Velocity ()]]
*/
double tModulatedPositionActuator::Velocity () const
{

  /****
  [[U dTemp1 := tModulatedPositionActuator::Value () ]]
  
  [[U dRate := tModulatedPositionActuator::OutputRate () ]]
  
  [ return := (dTemp - this->dLastPosition ) * dRate ]
  ****/  
  return (this->Position () - this->dLastPosition) * this->OutputRate ();
}



/*******************************************************************
*
* tModulatedPositionActuator::LimitPosition
*
* DESCRIPTION:
[[U2 tModulatedPositionActuator::LimitPosition (dOldPosition, dNewPosition, dOldVelocity) ]]
*
[[U3 tModulatedPositionActuator::LimitPosition (dOldPosition, dNewPosition, dOldVelocity) ]]
*/
double tModulatedPositionActuator::LimitPosition (double dOldPosition, double dNewPosition, 
    double dOldVelocity) const
{
  double dLimittedAccel, dLimittedVel, dLimittedPos;
  
  /****
  [[U dRate := tModulatedVelocityActuator::OutputRate () ]]
  ****/
  double dRate = this->OutputRate ();
  
  /****
  [[U dLimittedVel := tModulatedPositionActuator::LimitVelocity (
       dOldVelocity, ((dNewPosition - dOldPosition) * dRate)) ]]
  ****/      
  dLimittedVel = this->LimitVelocity (dOldVelocity, (dNewPosition - dOldPosition) * dRate);

  /****
  [[U tModulatedVelocityActuator::LimitValue_ddd ((dOldPosition + dLimittedVel / dRate), 
        this->CdMinPosition, this->CdMaxPosition,) ]]
  ****/      
  
  return this->LimitValue (dOldPosition + dLimittedVel / dRate, this->CdMinPosition, 
           this->CdMaxPosition);
  
}

