/* $RCSfile: targetSource.cc,v $ - Implements abstract hardware and software input classes */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/io/targetSource.cc,v 1.1.1.1 1998/06/11 21:36:19 chara Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,20Apr97    written
*/
 
/*
DESCRIPTION

*/
#include "isit.h"
#include "targetSource.h"
#include "dlTargGen.h"

/*******************************************************************/
/*******************************************************************/
/********************                      *************************/
/********************  Class Methods       *************************/
/********************                      *************************/
/*******************************************************************/
/*******************************************************************/

/******************************************************************
*
* tPositionTargetSource::tPositionTargetSource
*
* DESCRIPTION: Constructor
*
* RETURNS: Nothing
*
*
[[D tPositionTargetSource::tPositionTargetSource (in: TheTargetGenerator;
                                                  inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
]]
*
*****/
 
tPositionTargetSource::tPositionTargetSource(tTargGen &TheTargetGenerator) :

   /***
   [ &TargetGenerator := &TheTargetGenerator ]
   ***/
   TargetGenerator( TheTargetGenerator )

{
  TraceMark(S_dlTargetSet_POS_TARG_SRC_CONSTRUCTOR);
} 


/******************************************************************
*
* tPositionTargetSource::Update
*
* DESCRIPTION: Updates dValue from target generator based on time tTargetTime
*
* RETURNS: new value of dValue (double)
*
*
[[D tPositionTargetSource::Update (in: tTargetTime; inout: this; 
				   out: return;)]
  [ this invalid => UNDEFINED
  | true =>
      [[U tTargGen::GetCurrentTargetPosition(this=*this.TargetGenerator,
                                            tCurrTime=tTargetTime)]]
      : return := *this.dValue
]]
*
*****/
 
double tPositionTargetSource::Update(TIME tTargetTime) 
{
   this->SetValue (TargetGenerator.GetCurrentTargetPosition(tTargetTime)); 
   return this->Value ();
}


/******************************************************************
*
* tVelocityTargetSource::tVelocityTargetSource
*
* DESCRIPTION: Constructor
*
* RETURNS: Nothing
*
*
[[D tVelocityTargetSource::tVelocityTargetSource (in: TheTargetGenerator;
                                                  inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
]]
*
*****/

tVelocityTargetSource::tVelocityTargetSource(tTargGen &TheTargetGenerator) :

   /***
   [ &TargetGenerator := &TheTargetGenerator ]
   ***/
   TargetGenerator( TheTargetGenerator )

{
  TraceMark(S_dlTargetSet_VEL_TARG_SRC_CONSTRUCTOR);
}


/******************************************************************
*
* tVelocityTargetSource::Update
*
* DESCRIPTION: Updates dValue from target generator based on time tTargetTime
*
* RETURNS: new value of dValue (double)
*
*
[[D tVelocityTargetSource::Update (in: tTargetTime;
                                   inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
      [[U tTargGen::GetCurrentTargetVelocity(this=*this.TargetGenerator,
                                            tCurrTime=tTargetTime)]]
      : return := *this.dValue
]]
*
*****/
 
double tVelocityTargetSource::Update(TIME tTargetTime) 
{
   this->SetValue (TargetGenerator.GetCurrentTargetVelocity(tTargetTime)); 
   return this->Value ();
}

