/* $RCSfile: schedShow.cc,v $ - Scheduler routines */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/comm/scheduler/schedShow.cc,v 1.1.1.1 1998/06/11 21:36:20 chara Exp $
*/
/* Art Sims - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01c,26Nov96,eamck    wrote schedShow routines to generate task info
01b,29Oct96,eamck    all sched routines collected into single file
01a,23Oct96,asims    routines written
*/
 
/*
DESCRIPTION -- routines:
 *
 * tScheduler::Show - show task info (some fields)
 * tScheduler::ShowAll - show task info (all fields)
 *
 */
 
#include <vxWorks.h>
#include <stdioLib.h>
#include "isit.h"
#include "rte.h"
#include "Scheduler.h"

extern tRte *pThisRteG;


/**************************************************************************
*
* tScheduler::Show -- show task info (some fields)
*
* IN :
*   *this.TaskSet[]
* INOUT :
* OUT :
*
* SPEC: 
*
   [ this invalid -> UNDEFINED
   | *this.iNumtasks < 0 -> UNDEFINED
   | *this.bInit == FALSE ->
      print message
      : return OK
   | true -> 
      print selected fields of task control block for each task in task set
      : return OK
   ]
*
* RETURNS: 
*    OK 
*
**************************************************************************/
ERROR_CODE  tScheduler::Show( void )
{
   ERROR_CODE ec;
   int iTask;
 
   if (!this->bInit)
   {
      printf("Scheduler not initialized.\n");
      return OK;
   }

   printf("Scheduler Rate: %d\tLevel: %d\tNum Tasks: %d\tCurrent Time: %ld\n",
                this->iRate, this->Level, this->iNumTasks, *(this->pRteTime));
 
   printf("Name\t\tTaskId\t\tStrvng\tEState\tMode\tSchedT\t\tPrio\tPeriod\n");

   for (iTask = 0; iTask < this->iNumTasks; iTask++)
      printf("%-16s0x%6x\t%d\t%d\t%d\t%12lu\t%d\t%ld\n",
		taskName(this->TaskSet[iTask].Id),
                this->TaskSet[iTask].Id,
                this->TaskSet[iTask].iNumStarvingCycles,
                this->TaskSet[iTask].ExecState,
                this->TaskSet[iTask].Mode,
                this->TaskSet[iTask].tScheduledTime,
                this->TaskSet[iTask].iPriority,
                this->TaskSet[iTask].tPeriod );

   return OK; 
 
}  /* tScheduler::Show */
 
 
/**************************************************************************
*
* tScheduler::ShowAll -- show task info (all fields)
*
* IN :
*   *this.TaskSet[]
* INOUT :
* OUT :
*
* SPEC: 
*
   [ this invalid -> UNDEFINED
   | *this.iNumtasks < 0 -> UNDEFINED
   | *this.bInit == FALSE ->
      print message
      : return OK
   | true -> 
      print all fields of task control block for each task in task set
      : return OK
   ]
*
* RETURNS: 
*    OK 
*
**************************************************************************/
ERROR_CODE  tScheduler::ShowAll( void )
{
   ERROR_CODE ec;
   int iTask;

   if (!this->bInit)
   {
      printf("Scheduler not initialized.\n");
      return OK;
   }
 
   printf("Scheduler Rate: %d\tLevel: %d\tNum Tasks: %d\tCurrent Time: %ld\n",
                this->iRate, this->Level, this->iNumTasks, *(this->pRteTime));
 
   printf("Name\t\tTaskId\t\tStrvng\tNoPnic\tBehErr\tPendOp\tEState\tMode\tSchedT\t\tPrio\tPeriod\tOffset\tErrCode\n");
 
   for (iTask = 0; iTask < this->iNumTasks; iTask++)
     printf("%-16s0x%6x\t%d\t%d\t%d\t%d\t%d\t%d\t%12lu\t%d\t%lu\t%lu\t0x%08x\n",
		taskName(this->TaskSet[iTask].Id),
                this->TaskSet[iTask].Id,
                this->TaskSet[iTask].bStarving,
                this->TaskSet[iTask].bNoPanic,
                this->TaskSet[iTask].BehaviorOnError,
                this->TaskSet[iTask].PendingOp,
                this->TaskSet[iTask].ExecState,
                this->TaskSet[iTask].Mode,
                this->TaskSet[iTask].tScheduledTime,
                this->TaskSet[iTask].iPriority,
                this->TaskSet[iTask].tPeriod, 
                this->TaskSet[iTask].tOffset, 
                this->TaskSet[iTask].ErrorCode 
);
 
   return OK;

}  /* tScheduler::ShowAll */



ERROR_CODE schedShow( void )
{
   GetScheduler().Show();
   return OK;
}

ERROR_CODE schedShowAll( void )
{
   GetScheduler().ShowAll();
   return OK;
}

ERROR_CODE schedEnable( void )
{
   ERROR_CODE ec = OK;

   ec = GetScheduler().Enable();
   return ec;
}

ERROR_CODE schedDisable( void )
{
   ERROR_CODE ec = OK;

   ec = GetScheduler().Disable();
   return ec;
}

ERROR_CODE taskEnable( int iTaskId )
{
   ERROR_CODE ec = OK;

   ec = ::GetScheduler().EnableTask(iTaskId);
   return ec;
}

ERROR_CODE taskDisable( int iTaskId )
{
   ERROR_CODE ec = OK;

   ec = ::GetScheduler().DisableTask(iTaskId);
   return ec;
}
