/* $RCSfile: controllerSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/comm/servo/controllerSet.cc,v 1.1.1.1 1998/06/11 21:36:20 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a28Aug97,rlj    written
*/

#include "controller.h"
#include "controllerSet.h"
#include "servo.h"
#include "Scheduler.h"


/****
[[U tControllerGroup::tControllerGroup (iNumberOfPartitionsToUse) ]]
****/
tControllerGroup::tControllerGroup (int iNumberOfPartitionsToUse) :

  iNumberOfPartitions (MAX (0,iNumberOfPartitionsToUse) ),

  /****
  [[U this->NullController.tController::tController (void) ]]
  ****/
  NullController ()
{}




/****
[[U tControllerGroup::~tControllerGroup (void) ]]
****/
tControllerGroup::~tControllerGroup () {
}


/****
[[U tControllerGroup::GetNumberOfPartitions (void) ]]
****/
int tControllerGroup::GetNumberOfPartitions () const {

  return this->iNumberOfPartitions;
}




/****
[[U tSimpleControllerGroup::tSimpleControllerGroup (iNumberOfPartitionsToUse) ]]
****/
tSimpleControllerGroup::tSimpleControllerGroup (int iNumberOfPartitionsToUse) :
  tControllerGroup (iNumberOfPartitionsToUse),
  pControllerArray (new (tController*) [this->iNumberOfPartitions] )
{
 
  int iPartition;
 
  for (iPartition = 0; iPartition < this->iNumberOfPartitions; iPartition++) 
    
    /****
    [[U tSimpleControllerGroup::AssignController (iPartition,
        this->NullController) ]]
    ****/
    this->AssignController (iPartition,this->NullController);

}

/****
[[U tSimpleControllerGroup::~tSimpleControllerGroup (void) ]]
****/
tSimpleControllerGroup::~tSimpleControllerGroup () {

  delete[] this->pControllerArray;
}

/****
[[U tSimpleControllerGroup::AssignController (iWhichPartition, Controller) ]]
****/
ERROR_CODE tSimpleControllerGroup::AssignController 
             (int iWhichPartition,tController& Controller) {
  
  if ( iWhichPartition < 0 ||
       iWhichPartition >= this->iNumberOfPartitions ) 
         return Traceback (S_controllergroup_INVALID_PARTITION,
                           "tSimpleControllerGroup::AssignController");

  this->pControllerArray[iWhichPartition] = &Controller;
  return OK;
}

      


/****
[[U tSimpleControllerGroup::Calculate (tServoTime, iWhichPartition) ]]
****/
ERROR_CODE tSimpleControllerGroup::Calculate (TIME tServoTime, 
                                              int iWhichPartition) {

  if ( iWhichPartition < 0 || 
       iWhichPartition >= this->iNumberOfPartitions ) 
          return Traceback (S_controllergroup_INVALID_PARTITION,
                            "tSimpleControllerGroup::Calculate");
  
 
  /****
  [[U return := this->pControllerArray[iWhichPartition]->
        tController::Calculate (tServoTime) ]]
  ****/
  return this->pControllerArray[iWhichPartition]->Calculate (tServoTime);
} 


/****
[[U tSimpleControllerGroup::Init_i (tServoTime, iWhichPartition) ]]
****/
ERROR_CODE tSimpleControllerGroup::Init (TIME tServoTime, int iWhichPartition) {

  if ( iWhichPartition < 0 || 
       iWhichPartition >= this->iNumberOfPartitions ) 
          return Traceback (S_controllergroup_INVALID_PARTITION,
                            "tSimpleControllerGroup::Init");
  
 
  /****
  [[U return := this->pControllerArray[iWhichPartition]->
        tController::Init (tServoTime) ]]
  ****/
  return this->pControllerArray[iWhichPartition]->Init (tServoTime);
} 


/****
[[U tSimpleControllerGroup::Reset_i (tServoTime, iWhichPartition) ]]
****/
ERROR_CODE tSimpleControllerGroup::Reset (TIME tServoTime, int iWhichPartition) {

  if ( iWhichPartition < 0 || 
       iWhichPartition >= this->iNumberOfPartitions ) 
          return Traceback (S_controllergroup_INVALID_PARTITION,
                            "tSimpleControllerGroup::Init");
  
 
  /****
  [[U return := this->pControllerArray[iWhichPartition]->
        tController::Reset (tServoTime) ]]
  ****/
  return this->pControllerArray[iWhichPartition]->Reset (tServoTime);
} 




/****
[[U tModedControllerGroup::tModedControllerGroup (iNumberOfPartitionsToUse, 
                               iNumberOfControllerModesToUse,
                               UpdateModeDesisionToUseFunction, iTaskPriority,
                               tTaskPeriod, tTaskOffset) ]]
****/
tModedControllerGroup::tModedControllerGroup (
                           int iNumberOfPartitionsToUse, 
                           int iNumberOfControllerModesToUse,
                           tUpdateModeDecision& UpdateModeDecisionToUse,
                           int iTaskPriority,
                           TIME tTaskPeriod, TIME tTaskOffset) :

  tControllerGroup (MAX (0,iNumberOfPartitionsToUse) ),
  iNumberOfControllerModes (MAX (0,iNumberOfControllerModesToUse) ),
  iCurrentControllerModeArray (new int [this->iNumberOfPartitions] ),
  iNewControllerMode (0),
  pController2DArray (new (tController**) [this->iNumberOfPartitions] ),

  /****
  [ this->UpdateModeDecision := UpdateModeDecisionToUse ]
  ****/
  UpdateModeDecision (UpdateModeDecisionToUse)

{
 
  ERROR_CODE ec;
  int iPartition,iControllerMode;
  TIME tPeriod = tTaskPeriod;
  TIME tOffset = tTaskOffset;


  
  for (iPartition = 0; iPartition < this->iNumberOfPartitions; iPartition++) {
 
    this->iCurrentControllerModeArray [iPartition] = 0;
    this->pController2DArray[iPartition] = new (tController *)  [this->iNumberOfControllerModes];
   
    if (this->pController2DArray [iPartition] == NULL) {
      this->iNumberOfPartitions = 0;
      this->iNumberOfControllerModes = 0;
      TraceMsg (tmt_ERROR,S_controllergroup_UNABLE_TO_ALLOCATE_CONTROLLER_GROUP,
              "tModedControllerGroup::tModedControllerGroup");
      return;
    }

  }

  for (iPartition = 0; iPartition < this->iNumberOfPartitions; iPartition++) {
    for (iControllerMode = 0; iControllerMode < this->iNumberOfControllerModes;
         iControllerMode++) {
      /****
      [[U tModedControllerGroup::AssignController (iPartition,iControllerMode,
          this->NullController) ]]
      ****/
      this->AssignController (iPartition,iControllerMode,this->NullController);  
    }
  }


  ec = ::GetScheduler ().AddTask (&this->iUpdateModeTaskId,"ControllerModeUpdate",
            (FUNCPTR) this->PeriodicControllerModeUpdate, NULL, NULL,
            this, iTaskPriority, &tPeriod, &tOffset,
            TEB_DISABLE, TRUE);

  if (ec != OK) 
    TraceMsg (tmt_ERROR,ec,"tModedControllerGroup::tModedControllerGroup");
  
  ec = ::GetScheduler ().DisableTask (this->iUpdateModeTaskId); 
  
  if (ec != OK) 
    TraceMsg (tmt_ERROR,ec,"tModedControllerGroup::tModedControllerGroup");
  
}



/****
[[U tModedControllerGroup::~tModedControllerGroup (void) ]]
****/
tModedControllerGroup::~tModedControllerGroup () {
 
  ERROR_CODE ec;

  ec = ::GetScheduler ().DeleteTask (this->iUpdateModeTaskId);
 
  if (ec != OK) TraceMsg (tmt_ERROR,ec,
      "tModedControllerGroup::~tModedControllerGroup");

  delete[] this->pController2DArray;
}


/****
[[U tModedControllerGroup::AssignController (iWhichPartition,
                                             iWhichControllerMode,
                                             Controller) ]]
****/
ERROR_CODE tModedControllerGroup::AssignController (
           int iWhichPartition, int iWhichControllerMode,
           tController& Controller) {
  
  if ( iWhichPartition < 0 ||
       iWhichPartition >= this->iNumberOfPartitions )
     
     return Traceback (S_controllergroup_INVALID_PARTITION,
                           "tModedControllerGroup::AssignController");

  if ( iWhichControllerMode < 0 ||
       iWhichControllerMode >= this->iNumberOfControllerModes ) 
  
     return Traceback (S_controllergroup_INVALID_CONTROLLER_MODE,
                           "tModedControllerGroup::AssignController");

  this->pController2DArray [iWhichPartition][iWhichControllerMode] = &Controller;

  return OK;
}

      

/****
[[U tModedControllerGroup::Calculate (tServoTime, iWhichController) ]]
****/
ERROR_CODE tModedControllerGroup::Calculate (TIME tServoTime,
                                             int iWhichPartition) {

  ERROR_CODE ec;

  if ( this->iNewControllerMode != this->iCurrentControllerModeArray [iWhichPartition] ) {

    /****
    [[U ec := this->pController2DArray[iWhichPartition]
          [this->iCurrentControllerModeArray [iWhichPartition] ]->tController::Reset (tServoTime) ]]
    ****/
    ec = this->pController2DArray [iWhichPartition]
       [this->iCurrentControllerModeArray [iWhichPartition] ]->Reset (tServoTime);
    
    if (ec != OK) return Traceback (ec,"tModedControllerGroup::Calculate");

    /****
    [[U ec := this->pController2DArray [iWhichPartition]
          [this->iNewControllerMode]->tController::Init (tServoTime) ]]
    ****/
    ec = this->pController2DArray[iWhichPartition][this->iNewControllerMode]->Init (tServoTime);

    if (ec != OK) return Traceback (ec,"tModedControllerGroup::Calculate");

    this->iCurrentControllerModeArray [iWhichPartition] = this->iNewControllerMode;
  }

  /****
  [[U return := this->pController2DArray[iWhichPartition]
        [this->iCurrentControllerMode]->tController::Calculate (tServoTime) ]]
  ****/
  return this->pController2DArray[iWhichPartition][this->iCurrentControllerModeArray [iWhichPartition] ]->
     Calculate (tServoTime);
} 

ERROR_CODE tModedControllerGroup::Init () {

  ERROR_CODE ec;
 
  /****
  [[U ec := this->UpdateModeDecision.tUpdateModeDecision::Init (void) ]]
  ****/
  ec = this->UpdateModeDecision.Init ();
  if (ec != OK) return Traceback (ec,"tModedControllerGroup::Init");

  ec = ::GetScheduler ().EnableTask (this->iUpdateModeTaskId);
 
  if (ec != OK) 
    return Traceback (ec,"tModedControllerGroup::Init");

  return OK;
}  

ERROR_CODE tModedControllerGroup::Init (TIME tServoTime, int iWhichPartition) {

  ERROR_CODE ec;
 
  ec = this->pController2DArray[iWhichPartition]
        [this->iCurrentControllerModeArray [iWhichPartition] ]->Init (tServoTime);

  if (ec != OK) 
    return Traceback (ec,"tModedControllerGroup::Init");

  return OK;
}  
  

ERROR_CODE tModedControllerGroup::Reset () {

  ERROR_CODE ec;

  ec = ::GetScheduler ().DisableTask (this->iUpdateModeTaskId);
 
  if (ec != OK) 
    return Traceback (ec,"tModedControllerGroup::Init");

  /****
  [[U ec := this->UpdateModeDecision.tUpdateModeDecision::Reset (void) ]]
  ****/
  ec = this->UpdateModeDecision.Reset ();
  if (ec != OK) return Traceback (ec,"tModedControllerGroup::Reset");

  return OK;
}  
  

ERROR_CODE tModedControllerGroup::Reset (TIME tServoTime, int iWhichPartition) {

  ERROR_CODE ec;

  ec = this->pController2DArray[iWhichPartition]
        [this->iCurrentControllerModeArray [iWhichPartition] ]->Reset (tServoTime);

  if (ec != OK) 
    return Traceback (ec,"tModedControllerGroup::Reset");

  return OK;
}  
  

ERROR_CODE tModedControllerGroup::PeriodicControllerModeUpdate (
  TIME tUpdateTime) {

 
  int iTempMode = this->UpdateModeDecision.GetNewMode (tUpdateTime);

  if (iTempMode < 0 ||
      iTempMode >= this->iNumberOfControllerModes)

    return Traceback (S_controllergroup_INVALID_CONTROLLER_MODE_FROM_UPDATE,
      "tModedControllerGroup::PeriodicControllerModeUpdate");


  this->iNewControllerMode = iTempMode;

  return OK;
}


/****
[[U tControllerSet::tControllerSet (iNumberOfControllerGroupsToUse,
                                    iNumberOfParitionsPerGroupToUse) ]]
****/
tControllerSet::tControllerSet (int iNumberOfControllerGroupsToUse,
                                int iNumberOfPartitionsPerGroupToUse) :

  iNumberOfControllerGroups (MAX (0,iNumberOfControllerGroupsToUse) ),
  iNumberOfPartitionsPerGroup (MAX (0,iNumberOfPartitionsPerGroupToUse) ),
  iCurrentControllerGroupArray (new int [iNumberOfPartitionsPerGroup] ),
  iNewControllerGroup (0),

  /****
  [[U this->NullControllerGroup.tSimpleControllerGroup::tSimpleControllerGroup (
        this->iNumberOfPartitionsPerGroup) ]]
  ****/
  NullControllerGroup (iNumberOfPartitionsPerGroup),
  pControllerGroupArray (new (tControllerGroup*) [iNumberOfControllerGroups] ),
  pPreCalculateControllerGroup (&NullControllerGroup),
  pPostCalculateControllerGroup (&NullControllerGroup)
 

{

  int i;
 
  for (i = 0; i < this->iNumberOfControllerGroups; i++) {
    this->pControllerGroupArray[i] = &this->NullControllerGroup;
  }

  for (i = 0; i < this->iNumberOfPartitionsPerGroup; i++)
    this->iCurrentControllerGroupArray[i] = 0;

}


/****
[[U tControllerSet::~tControllerSet (void) ]]
****/
tControllerSet::~tControllerSet () {

  delete[] this->pControllerGroupArray;

}

 
/****
[[U tControllerSet::SetPreCalculateControllerGroup (tControllerGroup& ControllerGroup) ]]
****/
ERROR_CODE tControllerSet::SetPreCalculateControllerGroup (tControllerGroup& ControllerGroup) {

  /****
  [[U iNumberOfPartitionsInGroup := 
        ControllerGroup.tControllerGroup::GetNumberOfPartitions (void) ]]
  ****/
  int iNumberOfPartitionsInGroup = ControllerGroup.GetNumberOfPartitions ();

  if (iNumberOfPartitionsInGroup != this->iNumberOfPartitionsPerGroup)
         return Traceback (S_controllergroup_GROUP_NOT_SAME_SIZE,
                           "tControllerSet::AssignController");

  this->pPreCalculateControllerGroup = &ControllerGroup;
  
  return OK;
}

/****
[[U tControllerSet::SetPostCalculateControllerGroup (tControllerGroup& ControllerGroup)  ]]
****/
ERROR_CODE tControllerSet::SetPostCalculateControllerGroup (tControllerGroup& ControllerGroup)  {
 
  /****
  [[U iNumberOfPartitionsInGroup := 
        ControllerGroup.tControllerGroup::GetNumberOfPartitions (void) ]]
  ****/
  int iNumberOfPartitionsInGroup = ControllerGroup.GetNumberOfPartitions ();

  if (iNumberOfPartitionsInGroup != this->iNumberOfPartitionsPerGroup)
         return Traceback (S_controllergroup_GROUP_NOT_SAME_SIZE,
                           "tControllerSet::AssignController");
 
  this->pPostCalculateControllerGroup = &ControllerGroup;
  return OK;
}


/****
[[U tControllerSet::AssignControllerGroup (iWhichControllerGroup, 
		      	                   ControllerGroup) ]]
****/
ERROR_CODE tControllerSet::AssignControllerGroup (int iWhichControllerGroup, 
		      	                 tControllerGroup& ControllerGroup) { 
		      	                   
  /****
  [[U iNumberOfPartitionsInGroup := 
        ControllerGroup.tControllerGroup::GetNumberOfPartitions (void) ]]
  ****/
  int iNumberOfPartitionsInGroup = ControllerGroup.GetNumberOfPartitions ();
  
  if (iNumberOfPartitionsInGroup != this->iNumberOfPartitionsPerGroup)
         return Traceback (S_controllergroup_GROUP_NOT_SAME_SIZE,
                           "tControllerSet::AssignControllerGroup");
  		      	                   
  if ( iWhichControllerGroup < 0 ||
       iWhichControllerGroup >= this->iNumberOfControllerGroups ) 
         return Traceback (S_controllergroup_INVALID_CONTROLLER_GROUP,
                           "tControllerSet::AssignControllerGroup");

  
  this->pControllerGroupArray[iWhichControllerGroup] = &ControllerGroup;
  return OK;
}

/****
[[U tControllerSet::SelectControllerGroup (iWhichControllerGroup)  ]]
****/
ERROR_CODE tControllerSet::SelectControllerGroup (int iWhichControllerGroup) {


  ERROR_CODE ec;

  if ( iWhichControllerGroup < 0 ||
       iWhichControllerGroup >= this->iNumberOfControllerGroups ) 
         return Traceback (S_controller_INVALID_CONTROLLER,
                           "tControllerSet::SelectController");

  if (this->pControllerGroupArray[iWhichControllerGroup] == NULL)
    return Traceback (S_controllergroup_UNASSIGNED_CONTROLLER_GROUP,
                      "tControllerGroup::SelectControllerGroup");

  this->iNewControllerGroup = iWhichControllerGroup;
  
  // Perform Reset/Init for the Controller Group as a whole if the controller group is
  // changed.
  
  if (this->iNewControllerGroup != this->iCurrentControllerGroupArray [0] ) {
    /****
    [[U ec := this->pControllerGroupArray [this->iCurrentControllerGroupArray [0] ]->
        tControllerGroup::Reset (tServoTime) ]]
    ****/
    ec = this->pControllerGroupArray [this->iCurrentControllerGroupArray [0] ]->Reset ();
    
    if (ec != OK) return Traceback (ec, "tControllerGroup::SelectControllerGroup");

    /****
    [[U ec := this->pControllerGroupArray [this->iNewControllerGroup]->
        tControllerGroup::Init (void) ]]
    ****/
    ec = this->pControllerGroupArray [this->iNewControllerGroup]->Init ();

    if (ec != OK) return Traceback (ec, "tControllerGroup::SelectControllerGroup");
  }
  
  return OK;

}


/****
[[U tControllerSet::GetCurrentControllerGroup (void)  ]]
****/
ERROR_CODE tControllerSet::GetCurrentControllerGroup () const 

{
 
  return this->iCurrentControllerGroupArray [0]; // is this what we want???
 
}

/****
[[U tControllerSet::ControllerMode (void) ]]
****/                          
int tControllerSet::ControllerMode () const {

  /****
  [[U this->ControllerSet.pControllerGroupArray 
    [this->iCurrentControllerGroup]->tControllerGroup::ControllerMode (void) ]]
  ****/                          
  return this->pControllerGroupArray [this->iCurrentControllerGroupArray [0] ]->
    ControllerMode ();
}

      
/****
[[U tControllerSet::Calculate (tServoTime, iWhichPartition) ]]
****/
ERROR_CODE tControllerSet::Calculate (TIME tServoTime, 
                                      int iWhichPartition) {
 
  ERROR_CODE ec;

  /****
  [[U ec := this->pPreCalculateControllerGroup->tControllerGroup::
            Calculate (tServoTime,iWhichPartiton) ]]
  ****/
  ec = this->pPreCalculateControllerGroup->Calculate (tServoTime,iWhichPartition);
  
  if (ec != OK) return Traceback (ec,"tControllerSet::Calculate");


  if (this->iNewControllerGroup != this->iCurrentControllerGroupArray [iWhichPartition] ) {
  
    /****
    [[U ec := this->pControllerGroupArray[this->iCurrentControllerGroupArray [iWhichPartition] ]->
        tControllerGroup::Reset_i (tServoTime, iWhichPartition) ]]
    ****/
    ec = this->pControllerGroupArray [this->iCurrentControllerGroupArray [iWhichPartition] ]->
          Reset (tServoTime, iWhichPartition);

    if (ec != OK) return Traceback (ec,"tControllerSet::Calculate");

    /****
    [[U ec := this->pControllerGroupArray[this->iNewControllerGroup]->
        tControllerGroup::Init_i (tServoTime, iWhichPartition) ]]
    ****/
    ec = this->pControllerGroupArray[this->iNewControllerGroup]->Init (tServoTime, iWhichPartition);

    if (ec != OK) return Traceback (ec,"tControllerSet::Calculate, PreGroup");

    this->iCurrentControllerGroupArray [iWhichPartition] = this->iNewControllerGroup;

  }

  /****
  [[U ec := this->pControllerGroupArray[this->iCurrentControllerGroup [iWhichPartiton] ]->
      tControllerGroup::Calculate (tServoTime, iWhichController) ]]
  ****/
  ec = this->pControllerGroupArray[this->iCurrentControllerGroupArray [iWhichPartition] ]->
        Calculate (tServoTime,iWhichPartition);

  if (ec != OK) return Traceback (ec,"tControllerSet::Calculate, ServoModeGroup");


  /****
  [[U ec := this->pPostCalculateControllerGroup->tControllerGroup::
            Calculate (tServoTime,iWhichPartition) ]]
  ****/
  ec = this->pPostCalculateControllerGroup->Calculate (tServoTime,iWhichPartition);
  
  if (ec != OK) return Traceback (ec,"tControllerSet::Calculate, PostGroup");

  return OK;
} 





