/* $RCSFile$ -- simple floating-point filters */

/*
$Header: /home/chara/cvsroot/ople/sys/comm/servo/filtLibF.cc,v 1.1.1.1 1998/06/11 21:36:20 chara Exp $
*/

/*
modification history
--------------------
01a,25Jun97,eamck               adapted from Palomar filtLibF
*/


/*
DESCRIPTION
This module contains simple floating-point low-pass 
and hi-pass filters.
A setup routine precalculates the filter coefficients for
a given filter memory.  All constants and memory
are in a structure local to the calling subroutine.
*/


#include <float.h>
#include <math.h>
#include "filtLibF.h"


/**********************************************
*
* tFilter::tFilter
*
* DESCRIPTION:
[[U2 tFilter::tFilter_d (dMemoryInSamples) ]]
*
[[U3 tFilter::tFilter_d (dMemoryInSamples) ]]
*/
tFilter::tFilter (double dMemoryInSamples)
{
   /****
   [[U tFilter::Initialize (dMemoryInSamples) ]]
   ****/
   this->Initialize (dMemoryInSamples);
}


/**********************************************
*
* tFilter::Initialize 
*
* DESCRIPTION:
[[U2 tFilter::Initialize (dMemoryInSamples) ]]
*
[[U3 tFilter::Initialize (dMemoryInSamples) ]]
*/
void tFilter::Initialize (double dMemoryInSamples)
{
   this->dDecayingMemory = 0.0;
   this->dSampleWeight = 1.0 / dMemoryInSamples;
   this->dMemoryWeight = 1.0 - 1.0 / dMemoryInSamples;
}


/***********************************************
*
* tFilter::Reset - reset filter memory 
*
* requires reset value input
*
* RETURNS: Nothing
*/

void tFilter::Reset(double dResetVal)
{
   this->dDecayingMemory = dResetVal;
}

/**********************************************
*
* tFilter::LowPass 
*
* DESCRIPTION:
[[U2 tFilter::LowPass (dInputVal) ]]
*
[[U3 tFilter::LowPass (dInputVal) ]]
*/
double tFilter::LowPass(double dInputVal)
{
   this->dDecayingMemory =   this->dMemoryWeight * this->dDecayingMemory
                           + this->dSampleWeight * dInputVal;

   /* Prevent underflow */
  if ( fabs(this->dDecayingMemory) < (1.0e20 * DBL_MIN) )
    this->dDecayingMemory = 0.0;

   return this->dDecayingMemory;
}



/**********************************************
*
* tFilter::HighPass 
*
* DESCRIPTION:
[[U2 tFilter::HighPass (dInputVal) ]]
*
[[U3 tFilter::HighPass (dInputVal) ]]
*/
double tFilter::HighPass(double dInputVal)
{
   this->dDecayingMemory =   this->dMemoryWeight * this->dDecayingMemory
                           + this->dSampleWeight * dInputVal;
   
  /* Prevent underflow */
  if ( fabs(this->dDecayingMemory) < (1.0e20 * DBL_MIN) )
    this->dDecayingMemory = 0.0;

  return dInputVal - this->dDecayingMemory;
}
 



