/* $RCSfile: pidLibF.cc,v $  -- structure header for pidLibF.c */
/*
$Header: /home/chara/cvsroot/ople/sys/comm/servo/pidLibF.cc,v 1.1.1.1 1998/06/11 21:36:21 chara Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,22Oct96,beh   Converted from old pidLibF.c
03c,12jun90,mmc
*/

/*
DESCRIPTION
This module contains a PID (proportional,integral,derivative)
compensator, plus routines to calculate the compensator
coefficients.
*/

#include <vxWorks.h>
#include <stdioLib.h>
#include <float.h>
#include <math.h>
#include "isit.h"
#include "pidLibF.h"
#include "dlErrors.h"



/***********************************************
*
* tPidDouble::tPidDouble 
*
* DESCRIPTION:
[[U2 tPidDouble::tPidDouble_ddddd (gain, tau1, tau2, taud, tsamp)]]
*
[[U3 tPidDouble::tPidDouble_ddddd (gain, tau1, tau2, taud, tsamp)]]
*/
tPidDouble::tPidDouble (double gain, double tau1,
                                  double tau2, double taud, double tsamp)
{
   /****
    [[U tPidDouble::InitializeUsingPolesAndZeros (gain, tau1, tau2, taud, tsamp) ]]
    ****/ 
    this->InitializeUsingPolesAndZeros (gain, tau1, tau2, taud, tsamp);

}

/***********************************************
*
* tPidDouble::tPidDouble 
*
* DESCRIPTION:
[[U2 tPidDouble::tPidDouble_dddddd (gain, p, i, d, taud, tsamp)]]
*
[[U3 tPidDouble::tPidDouble_dddddd (gain, p, i, d, taud, tsamp)]]
*/
tPidDouble::tPidDouble (double gain, double p, double i,
                        double d, double taud, double tsamp)
{
   /****
    [[U tPidDouble::InitializeUsingPids (gain, p, i, d, taud, tsamp) ]]
    ****/ 
    this->InitializeUsingPids (gain, p, i, d, taud, tsamp);

}


/***********************************************
*
* InitializeUsingPolesAndZeros 
*
* DESCRIPTION:
[[U2 tPidDouble::InitializeUsingPolesAndZeros (gain, tau1, tau2, taud, tsamp)]]
*
[[U3 tPidDouble::InitializeUsingPolesAndZeros (gain, tau1, tau2, taud, tsamp)]]
*/
void tPidDouble::InitializeUsingPolesAndZeros (double gain, double tau1,
                                  double tau2, double taud, double tsamp)
{
  TraceMark(S_tPidDouble_INIT_USING_POLES_AND_ZEROS_START);
  
  if ((tau1 == 0.0) && (tau2 == 0.0)) 
    /****
    [[U tPidDouble::InitializeUsingPids (gain, 1.0, 0.0, 0.0, 0.0, tsamp) ]]
    ****/ 
    this->InitializeUsingPids (gain, 1.0, 0.0, 0.0, 0.0, tsamp);
  else if (tau1 == 0.0)
    /****
    [[U tPidDouble::InitializeUsingPids (gain, 1.0, 0.0, tau2, taud, tsamp) ]]
    ****/ 
    this->InitializeUsingPids (gain, 1.0, 0.0, tau2, taud, tsamp);
  else if (tau2 == 0.0) {
    /****
    [[U tPidDouble::InitializeUsingPids (gain, tau1, 1.0, 0.0, 0.0, tsamp) ]]
    ****/ 
    this->InitializeUsingPids (gain, tau1, 1.0, 0.0, 0.0, tsamp);
    }
  else
    /****
    [[U tPidDouble::InitializeUsingPids (gain, (tau1 + tau2), 1.0, (tau1 * tau2), taud, tsamp) ]]
    ****/ 
    this->InitializeUsingPids (gain, tau1 + tau2, 1.0, tau1 * tau2, taud, tsamp);

  
  TraceMark(S_tPidDouble_INIT_USING_POLES_AND_ZEROS_EXIT);
}


/***********************************************
*
* InitializeUsingPids 
*
* DESCRIPTION:
[[U2 tPidDouble::InitializeUsingPids (gain, p, i, d, taud, tsamp)]]
*
[[U3 tPidDouble::InitializeUsingPids (gain, p, i, d, taud, tsamp)]]
*/
void tPidDouble::InitializeUsingPids (double gain, double p, double i,
                           double d, double taud, double tsamp)
{
  TraceMark(S_tPidDouble_INIT_USING_PIDS_START);
  
  this->p      = gain * p;
  this->i      = gain * i * tsamp;
  this->d      = gain * d / (tsamp + taud);
  this->dpole  = taud / (tsamp + taud);

  this->dLimit = 32766.99;

  /*****
  [[U tPidDouble::Reset (0.0) ]]
  *****/
  this->Reset (0.0);

  TraceMark(S_tPidDouble_INIT_USING_PIDS_EXIT);
}


/***********************************************
*
* Reset
*
* DESCRIPTION:
[[U2 tPidDouble::Reset (dInitialX)]]
*
[[U3 tPidDouble::Reset (dInitialX)]]
*/
void tPidDouble::Reset (double dInitialX)
{
  TraceMark(S_tPidDouble_INIT_PID_RESET_START);

  this->imem = 0.0;
  this->dmem = 0.0;
  this->xmem = dInitialX; 

  TraceMark(S_tPidDouble_INIT_PID_RESET_EXIT);
}

/***********************************************
*
* Calculate
*
* DESCRIPTION:
[[U2 tPidDouble::Calculate (x)]]
*
[[U3 tPidDouble::Calculate (x)]]
*/
double tPidDouble::Calculate(double x)
{
  FAST double ytemp, itemp, dtemp, output;

  itemp = this->i * x + this->imem;
  dtemp = this->d * (x - this->xmem) + this->dpole * this->dmem;
  
  ytemp = this->p * x + itemp + dtemp;
  
  this->xmem = x;
  this->imem = limit(itemp, this->dLimit);
  this->dmem = limit(dtemp, this->dLimit);

  /* Prevent underflow */
  if ( fabs(this->dmem)  <  (1.0e20 * DBL_MIN) )
    this->dmem = 0.0;
  
  /****
  [[U ::limit (ytemp, this->dLimit) ]]
  ****/
  output = limit (ytemp, this->dLimit);
  
  return output;
}


/***********************************************
*
* Display
*
* DESCRIPTION:
[[U2 tPidDouble::Display () ]]
*
[[U3 tPidDouble::Display () ]]
*/
void tPidDouble::Display (void) {
  printf ("xmem: %f \n"
          "imem: %f \n"
          "dmem: %f \n"
          "p:    %f \n"
          "i:    %f \n"
          "d:    %f \n"
          "dpole:%f \n",
          xmem, imem, dmem, p, i, d, dpole);
}


/***********************************************
*
* CopyPidMemory
*
* DESCRIPTION:
[[U2 ::CopyPidMemory (Destination, Source) ]]
*
[[U3 ::CopyPidMemory (Destination, Source) ]]
*/
void CopyPidMemory (tPidDouble& Destination, const tPidDouble& Source) 
{
  Destination.xmem = Source.xmem;
  Destination.dmem = Source.dmem;
  Destination.imem =  Source.imem;
}  
