 /* $RCSfile: servo.cc,v $ - implements generic servo and controller functionality */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/servo/servo.cc,v 1.2 1998/08/17 21:00:50 phil Exp $
*/
/* Richard Johnson - Jet Propulstion Laboratory */
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,23Apr97    written
*/
 
/*
DESCRIPTION

*/


#include "isit.h"
#include "servo.h"
#include "servoInfo.h"

#include "controllerSet.h"

#include "Scheduler.h"






static const BOOL bParanoid = FALSE;

/****
[[U tServoTask::tServoTask (sTaskName,TaskInfoToUse,
                            iPartitionNumberToUse,ControllerSetToUse) ]]
****/                          
tServoTask::tServoTask (char *sTaskName, const tServoTaskInfo& TaskInfoToUse,
                        int iPartitionNumberToUse,
                        tControllerSet& ControllerSetToUse) :

  tBaseObject (),
  ServoTaskInfo (TaskInfoToUse),
  ControllerSet (ControllerSetToUse) {

  ERROR_CODE ec;
  TIME tOffset = this->ServoTaskInfo.tOffset;
  TIME tPeriod = this->ServoTaskInfo.tTaskPeriod;
  
  TraceMark(S_servotask_CONSTRUCTOR);

  this->iPartitionNumber = iPartitionNumberToUse;


  /***
  [[U ec := (::pThisRteG)->pScheduler->tScheduler::AddTask (&this->iServoTaskId, 
             sTaskName,&this->DoServo, NULL, NULL,
             this, iPriority, &tPeriod, &tOffset,
             TEB_DISABLE, TRUE)]]
  ****/
  ec = ::GetScheduler ().AddTask(&this->iServoTaskId,sTaskName,
            (FUNCPTR) &this->DoServo, NULL, NULL,
            this, ServoTaskInfo.iTaskPriority, &tPeriod, &tOffset,
            TEB_DISABLE, TRUE);

  TraceMark(S_servotask_ADDED_SERVO_TASK);
  
  /****
    [ tPeriod was adjusted by scheduler =>
        this->tServoPeriod := Adjusted period
        : [ traceback working =>
             console := console + message ]
    ]
  ****/
  if (this->ServoTaskInfo.tTaskPeriod != tPeriod || tOffset != 0) {
    this->ServoTaskInfo.tTaskPeriod = tPeriod;
    TraceMsg(tmt_WARN,S_servo_REQUESTED_TASK_PERIOD_MODIFIED,
             "tServo::tServo");
  }

  if (ec != OK)
    TraceMsg(tmt_ERROR,ec, "tServoTask::tServoTask");

  else tBaseObject::SetInit (TRUE);

  tBaseObject::SetInitError (ec);
  
}

/****
[[U tServoTask::~tServoTask (void) ]]
****/                          
tServoTask::~tServoTask () {

  ERROR_CODE ec;

  TraceMark(S_servotask_DESTRUCTOR);


  /****
  [[U ec := (::pThisRteG)->pScheduler->tScheduler::DeleteTask(
                this->iServoTaskId) ]]
  ****/
  ec = ::GetScheduler().DeleteTask(this->iServoTaskId);
  
  if (ec != OK) TraceMsg(tmt_ERROR,ec, "tServoTask::~tServoTask");

  TraceMark(S_servotask_DELETED_SERVO_TASK);
}


ERROR_CODE tServoTask::Restart () {

  ERROR_CODE ec;

  if (tBaseObject::IsInit () == TRUE) {
    TraceMsg (tmt_WARN, S_tservotask_TASK_ALREADY_RUNNING, "tServoTask::Restart");
    return OK;
  }

  /****
  [[U ec := (::pThisRteG)->pScheduler->tScheduler::EnableTask 
            (this->iServoTaskId) ]]
  ****/
  ec = ::GetScheduler ().EnableTask (this->iServoTaskId);
  if (ec != OK) return Traceback (ec,"tServoTask::Shutdown");

  return tBaseObject::Restart ();

}



ERROR_CODE tServoTask::Shutdown () {

  ERROR_CODE ec;

  if (tBaseObject::IsInit () == FALSE) {
    TraceMsg (tmt_WARN, S_tservotask_TASK_ALREADY_SHUTDOWN, "tServoTask::Shutdown");
    return OK;
  }

  /****
  [[U ec := (::pThisRteG)->pScheduler->tScheduler::DisableTask 
            (this->iServoTaskId) ]]
  ****/
   ec = ::GetScheduler ().DisableTask (this->iServoTaskId);
   if (ec != OK) return Traceback (ec,"tServoTask::Shutdown");
   
   return tBaseObject::Shutdown ();
   
}    

BOOL tServoTask::IsOk () {
 
  return tBaseObject::IsOk ();
}


/****
[[U tServoTask::DoServo (tServoTime) ]]
****/                          
ERROR_CODE tServoTask::DoServo(TIME tServoTime) {

  ERROR_CODE ec;

  if (bParanoid) TraceMark(S_servotask_DOSERVO);
  
 
  TraceMark(S_servotask_DOSERVO);
   
  /****
  [[U ec := this->ControllerSet.tControllerSet::Calculate (
                tServoTime,this->iPartitionNumber)]]
  ****/
  ec = this->ControllerSet.Calculate (tServoTime, this->iPartitionNumber);
  
  if (ec != OK) Traceback (ec, "tServoTask::DoServo");  

  if (bParanoid) TraceMark(S_servotask_EXITING_DOSERVO);

  return OK;
}



/****
[[U tServo::tServo (ServoInfoToUse,ControllerSetToUse) ]]
****/                          
tServo::tServo (const tServoInfo& ServoInfoToUse,
                tControllerSet& ControllerSetToUse) :

  tBaseObject (),
  ServoInfo (ServoInfoToUse),
  ControllerSet (ControllerSetToUse) { 

  tBaseObject::SetInit (FALSE);
}


/****
[[U tServo::~tServo (void) ]]
****/                          
tServo::~tServo () {

  int iTotalPartitions, iPartitionNumber;
  
  iTotalPartitions = this->ServoInfo.GetNumberOfPartitions ();
  
  for (iPartitionNumber = 0; iPartitionNumber < iTotalPartitions; 
       iPartitionNumber++ ) {
       
      /****
      [[U this->pServoTaskArray [iPartitionNumber].
            tServoTask::~tServoTask (void) ]]
      ****/                          
    delete this->pServoTaskArray [iPartitionNumber];
  }
  
  delete[] this->pServoTaskArray;
}


ERROR_CODE tServo::Restart () {

  ERROR_CODE ec;
  int iPartitionNumber;
  int iNumberOfPartitions;

  iNumberOfPartitions = ServoInfo.GetNumberOfPartitions ();

  if (tBaseObject::IsInit () == TRUE) {
    TraceMsg (tmt_WARN, S_tservo_ALREADY_RUNNING, "tServo::Restart");
    return OK;
  }

  for (iPartitionNumber = 0; iPartitionNumber < iNumberOfPartitions; iPartitionNumber++) {
 
    /****
    [[U ec := this->pServoTaskArray [iPartitionNumber].tServoTask::Restart (void) ]]
    ****/
    ec = this->pServoTaskArray [iPartitionNumber]->Restart ();
    if (ec != OK) return Traceback (ec,"tServo::Restart");
  }
 
  return tBaseObject::Shutdown ();
}



ERROR_CODE tServo::Shutdown () {

  ERROR_CODE ec;
  int iPartitionNumber;
  int iNumberOfPartitions;

  iNumberOfPartitions = ServoInfo.GetNumberOfPartitions ();

  if (tBaseObject::IsInit () == FALSE) {
    TraceMsg (tmt_WARN, S_tservo_ALREADY_SHUTDOWN, "tServo::Shutdown");
    return OK;
  }

  for (iPartitionNumber = 0; iPartitionNumber < iNumberOfPartitions; iPartitionNumber++) {
 
    /****
    [[U ec := this->pServoTaskArray [iPartitionNumber].tServoTask::Shutdown (void) ]]
    ****/
    ec = this->pServoTaskArray [iPartitionNumber]->Shutdown ();
    if (ec != OK) return Traceback (ec,"tServo::Shutdown");
  }
 
  return tBaseObject::Shutdown ();
}    


BOOL tServo::IsOk () {
 
  return tBaseObject::IsOk ();
}


/****
[[U tServo::StartServo (void) ]]
****/                          
ERROR_CODE tServo::StartServo () {

  int iTotalPartitions,iPartitionNumber;
  ERROR_CODE ec;
  char sServoTaskName[25];

 
 TraceMark(S_servo_CONSTRUCTOR_BEGIN);

  /****
  [[U iTotalPartitions := this->ServoInfo.tServoInfo::
        GetNumberOfPartitions (void) ]]
  ****/                          
  iTotalPartitions = this->ServoInfo.GetNumberOfPartitions (); 

  this->pServoTaskArray = new (tServoTask*) [iTotalPartitions];
  
  if (this->pServoTaskArray == NULL)
    return Traceback (S_servo_COULD_NOT_ALLOCATE_SERVO_TASK_ARRAY,
                      "tServo::StartServo");


  TraceMark(S_servo_TASK_ARRAY_ALLOCATED);


  for ( iPartitionNumber = 0; iPartitionNumber < iTotalPartitions ; 
          iPartitionNumber++) {
    
    sprintf (sServoTaskName,"%d_",iPartitionNumber);
    strcat (sServoTaskName,ServoInfo.GetServoName ());

    /****
    [[U this->pServoTaskArray[iPartitionNumber]->tServoTask::tServoTask (
            sServoTaskName, *this, 
            ServoInfo.GetServoTaskInfo (iPartitionNumber),
            iPartitionNumber)]]
    ****/
    this->pServoTaskArray[iPartitionNumber] = new tServoTask (
            sServoTaskName,
            ServoInfo.GetServoTaskInfo (iPartitionNumber),
            iPartitionNumber, this->ControllerSet);
    
    if (this->pServoTaskArray[iPartitionNumber] == NULL) 
    
      return Traceback (S_servo_COULD_NOT_ALLOCATE_SERVO_TASK_ARRAY,
                 "tServo::StartServo");

    //    taskDelay (1); /// TEMPORARY diagnostic only  
     
  } // for
  
  TraceMark(S_servo_TASK_ARRAY_CONSTRUCTOR_COMPLETE); 

  tBaseObject::SetInit (TRUE);
  return OK;
}




/****
[[U tServo::SelectMode (int iServoMode) ]]
****/                          
ERROR_CODE tServo::SelectMode(int iServoMode) {

  TraceMark(S_servo_SELECT_MODE);

  /****
  [[U return := this->ControllerSet.tControllerSet::
        SelectControllerGroup (iServoMode) ]]
  ****/                          
  return this->ControllerSet.SelectControllerGroup (iServoMode);
}

/****
[[U tServo::GetMode (void) ]]
****/                          
int tServo::GetMode () const {


  /****
  [[U return := this->ControllerSet.tControllerSet::
        GetCurrentControllerGroup (iServoMode) ]]
  ****/                          
  return this->ControllerSet.GetCurrentControllerGroup ();
}


/****
[[U tServo::ControllerMode (void) ]]
****/                          
int tServo::ControllerMode () const {

  /****
  [[U this->ControllerSet.tControllerSet::ControllerMode (void) ]]
  ****/                          
  return this->ControllerSet.ControllerMode ();
}
