/* $RCSfile: servoInfo.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/servo/servoInfo.cc,v 1.2 1998/08/17 21:00:50 phil Exp $
*/
/* Richard Johnson - Jet Propulstion Laboratory */
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,2Sept97    written
*/
 
/*
DESCRIPTION

*/


#include "servoInfo.h"

tServoTaskInfo::tServoTaskInfo () {}

tServoTaskInfo::tServoTaskInfo (TIME tPeriod, TIME tOffset, int iPriority) {

 this->tTaskPeriod = tPeriod;
 this->tOffset     = tOffset;
 this->iTaskPriority = iPriority;
}



tServoInfo::tServoInfo (const char* sServoNameToUse, int iNumberOfPartitonsToUse) {

  
  this->sServoName = new char [strlen (sServoNameToUse) + 1];
  strcpy (this->sServoName, sServoNameToUse);
  
  this->iNumberOfPartitions = iNumberOfPartitonsToUse;
  
  this->ServoTaskInfoArray = new tServoTaskInfo [iNumberOfPartitonsToUse];
  
  if (this->ServoTaskInfoArray == NULL) 
    TraceMsg (tmt_ERROR, S_servoinfo_UNABLE_TO_ALLOCATE_SERVO_TASK_INFO_ARRAY,
              "tServoInfo::tServoInfo");
}


tServoInfo::tServoInfo (const tServoInfo& ServoInfo) 
{
 
  int i;

  this->iNumberOfPartitions = ServoInfo.iNumberOfPartitions;
  this->sServoName = new char [strlen (ServoInfo.sServoName) + 1];
  strcpy (this->sServoName,ServoInfo.sServoName);
  
  this->ServoTaskInfoArray = new tServoTaskInfo [ServoInfo.iNumberOfPartitions];

  if (this->ServoTaskInfoArray == NULL)

    TraceMsg (tmt_ERROR, S_servoinfo_UNABLE_TO_ALLOCATE_SERVO_TASK_INFO_ARRAY,
              "tServoInfo::tServoInfo");

  for (i = 0; i < ServoInfo.iNumberOfPartitions; i++) 
    this->ServoTaskInfoArray [i] = ServoInfo.ServoTaskInfoArray [i];

}


tServoInfo::~tServoInfo () {

   delete[] this->ServoTaskInfoArray;
}


ERROR_CODE tServoInfo::AssignServoTaskInfo (int iWhichPartition,
                                TIME tPeriod, int iPriority ) {

  return this->AssignServoTaskInfo(iWhichPartition, tPeriod, 0, iPriority);
}


ERROR_CODE tServoInfo::AssignServoTaskInfo (int iWhichPartition,
                                TIME tPeriod, TIME tOffset, int iPriority ) {
                                
  if (iWhichPartition < 0 || iWhichPartition >= this->iNumberOfPartitions)
    return Traceback (S_servoinfo_INVALID_PARTITION, 
                      "tServoInfo::AssignServoTaskInfo");     

  this->ServoTaskInfoArray[iWhichPartition] = 
               tServoTaskInfo (tPeriod, tOffset, iPriority);
    
  return OK;
}


const tServoTaskInfo& tServoInfo::GetServoTaskInfo (int iWhichPartition) const {

  if (iWhichPartition < 0 || iWhichPartition >= this->iNumberOfPartitions)
    TraceMsg (tmt_ERROR,S_servoinfo_INVALID_PARTITION, 
                      "tServoInfo::GetServoTaskInfo");     
  return this->ServoTaskInfoArray[iWhichPartition];
}


int tServoInfo::GetNumberOfPartitions () const { return this->iNumberOfPartitions; }


char* tServoInfo::GetServoName () const { return this->sServoName; }


