/* $RCSfile: trace.cc,v $ - Traceback header file */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/traceback/trace.cc,v 1.1.1.1 1998/06/11 21:36:22 chara Exp $
*/
/* Michael Deck - Cleanroom Software Engineering, Inc. */

/* 
modification history
--------------------
01a,mdd,14Apr97    written
02a,mdd,17Apr97    rewritten
*/

/*
DESCRIPTION

*/

#define DIAGS_NOINFO

/****************************************************************
*
* tTrace Clear Box
*
****************************************************************/

#include "errLookup.h"

#ifndef SPECTOOL
#include <stdio.h>
#include <logLib.h>
#include <stdarg.h>
#include "isit.h"
#endif

#include "trace.h"
#include "tracesb.h"
#include "vxworks/taskLib.spc"
#include "vxworks/msgQLib.spc"

/* definitions of a static instance */
static tTrace theTrace;

/**** [[T ::GetZeroTime(void)] [ return := 0 ]] ****/
static TIME GetZeroTime(void) { return 0; }


/***
[[U this->tTrace::tTrace_SB(void)]]
***/
tTrace::tTrace(void) {

  /***
  [[U this->msgQId := msgQLib::msgQCreate(CiMaxTraceStrings,sizeof(char *),MSG_Q_FIFO)]]
  ***/
  this->msgQId = msgQCreate(CiMaxTraceStrings,sizeof(char *),MSG_Q_FIFO);
  
  /***
  [[U this->iTraceTaskId := taskLib::taskSpawn("TraceBk",
                                 TRACEBACK_TASK_PRIO,
                                 TRACEBACK_TASK_OPTIONS,
                                 TRACEBACK_TASK_STACKSIZE,
                                 (FUNCPTR) &(this->TraceMsgOut),
                                 (int) this, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0)]]
  ***/
  this->iTraceTaskId = taskSpawn("TraceBk",
                                 TRACEBACK_TASK_PRIO,
                                 TRACEBACK_TASK_OPTIONS,
                                 TRACEBACK_TASK_STACKSIZE,
                                 (FUNCPTR) &(this->TraceMsgOut),
                                 (int) this, 0,
                                 0, 0, 0, 0, 0, 0, 0, 0);
                                 
  this->bTraceMsgUsesLogMsg = FALSE;
  this->bFrozen = FALSE;
  this->imarkbufferlength = 0;
  this->imarkbuffernewest = 0;

  this->pGetTimeFunc = &GetZeroTime;
}
                                 
/***
[[U this->tTrace::~tTrace_SB(void) ]]
***/
tTrace::~tTrace(void) {
  /***
  [[U msgQLib::msgQDelete(this->msgQId)]]
  ***/
  msgQDelete(this->msgQId);
  
  /***
  [[U taskLib::taskDelete(this->iTraceTaskId)]]
  ***/
  taskDelete(this->iTraceTaskId);
}

/***
[[U this->tTrace::TraceMark_SB(iStatus)]]
***/
ERROR_CODE tTrace::TraceMark (const int iStatus){
  if (!this->bFrozen) {
    this->imarkbuffernewest++;
    if (this->imarkbuffernewest >= CiMaxTraceMarks) this->imarkbuffernewest=0;
    if (this->imarkbufferlength < CiMaxTraceMarks) this->imarkbufferlength++;
    this->imarkbufferarray[this->imarkbuffernewest]=iStatus;
  }
  return OK;
}

/***
[[U this->tTrace::TraceFreeze(void)]]
***/
void tTrace::TraceFreeze(void) {
  this->bFrozen = TRUE;
}

/***
[[U this->tTrace::TraceUnfreeze(void)]]
***/
void tTrace::TraceUnfreeze(void) {
  this->bFrozen = FALSE;
}

/***
[[U this->tTrace::TraceDump_SB(iCount,iFromNewest) ]]
***/
ERROR_CODE tTrace::TraceDump (const int iCount, const int iFromNewest) { 

  int iDumped = 0;
  int iPosition;
  
  static const char *fmt = "%d : 0x%x";
  int ec = OK;

  int iToDump = iCount;
  if (iFromNewest < 0) iToDump = 0;
  if (iToDump > imarkbufferlength - iFromNewest) 
    iToDump = imarkbufferlength - iFromNewest;
  
  while ((ec==OK) && (iDumped < iToDump)) {
    /*[iPosition := 
         (imarkbuffernewest-iFromNewest-iDumped) mod CiMaxTraceMarks ]*/
    {         
      iPosition = imarkbuffernewest-iFromNewest-iDumped;
      if (iPosition < 0) iPosition += CiMaxTraceMarks;
    }
    char *psbuf = new char[::strlen(fmt)+1+17+17];
    sprintf 
     (psbuf,
      fmt,
      iFromNewest-iDumped, 
      imarkbufferarray[iPosition]);
    /***
    [[U ec := msgQLib::msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),
         TRACEBACK_QUEUE_TIMEOUT_SECS*sysClkRateGet(), MSG_PRI_NORMAL)]]
    ***/
    ec=msgQSend(
         this->msgQId,
         (char *) &psbuf,
         sizeof(char *),
         TRACEBACK_QUEUE_TIMEOUT_SECS*sysClkRateGet(),
         MSG_PRI_NORMAL);
    if (ec != OK) {
      delete [] psbuf;
      logMsg("ERROR: Tracedump message queue error %d\n",ec,0,0,0,0,0);
    }
    iDumped++;
  }
  return ec;
}
/***
[[U this->tTrace::TraceDump_SB(iCount,iFromNewest) ]]
***/
ERROR_CODE tTrace::TraceDumpText (const int iCount, const int iFromNewest) {

  int iDumped = 0;
  int iPosition;
  char *str;

  static const char *fmt = "%d : 0x%x => %s";
  int ec = OK;

  int iToDump = iCount;
  if (iFromNewest < 0) iToDump = 0;
  if (iToDump > imarkbufferlength - iFromNewest)
    iToDump = imarkbufferlength - iFromNewest;

  while ((ec==OK) && (iDumped < iToDump)) {
    /*[iPosition :=
         (imarkbuffernewest-iFromNewest-iDumped) mod CiMaxTraceMarks ]*/
    {
      iPosition = imarkbuffernewest-iFromNewest-iDumped;
      if (iPosition < 0) iPosition += CiMaxTraceMarks;
    }

    /***
    [[U str := ::errLookup(imarkbufferarray[iPosition]) ]] 
    ***/
    str =  errLookup ( imarkbufferarray[iPosition] );
    char *psbuf = new char[::strlen(fmt)+1+17+17+5+strlen(str)];
    sprintf
     (psbuf,
      fmt,
      iFromNewest-iDumped,
      imarkbufferarray[iPosition],
      str);
      
    /***
    [[U ec := msgQLib::msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),
         TRACEBACK_QUEUE_TIMEOUT_SECS*sysClkRateGet(), MSG_PRI_NORMAL)]]
    ***/
    ec=msgQSend(
         this->msgQId,
         (char *) &psbuf,
         sizeof(char *),
         TRACEBACK_QUEUE_TIMEOUT_SECS*sysClkRateGet(),
         MSG_PRI_NORMAL);
    if (ec != OK) {
      delete [] psbuf;
      logMsg("ERROR: Tracedump message queue error %d\n",ec,0,0,0,0,0);
    }
    iDumped++;
  }
  return ec;
}


/***
[[U this->tTrace::TraceClear_SB(void)]]
***/
void tTrace::TraceClear() {    
  this->imarkbufferlength = 0;
}
  
/***
[[U this->tTrace::TraceMsg(tmtMsgType, ec, *fmt, va_list arglist)]]
***/
ERROR_CODE tTrace::TraceMsgCommon (const TRACE_MSG_TYPE tmtMsgType, 
                      const int ec, const char *fmt, va_list arglist)  {    

  int msgQec = OK;

  /* [ this->pGetTimeFunc not a valid pointer to a 
         function that returns TIME => UNDEFINED
     | true => tTimeNow := TIME returned by this->pGetTimeFunc ]] */
  TIME tTimeNow = (*(this->pGetTimeFunc))();

  static const char *stdfmt = "TRC(%s)  @%u : ID 0x%x, ec=0x%x  (%s)";  
  char *argbuf = new char[1024];
  int numbytes = vsprintf(argbuf,fmt,arglist);
  if ((numbytes > 1024) || (numbytes==EOF)) {
    logMsg("ERROR: Traceback message overflow\n",0,0,0,0,0,0);
    printf("ERROR: Traceback message overflow\n");
    delete [] argbuf;
    return ec;
  }
  
  char *psbuf = new char[::strlen(stdfmt)+1+7+17+20+20+::strlen(argbuf)];

  switch (tmtMsgType) {
    case tmt_INFO: 
#if !defined(DIAGS_NOWARN) && !defined(DIAGS_NOINFO)
      if (bTraceMsgUsesLogMsg)
        logMsg("TRC(%s)  @%u : ID 0x%x, ec=0x%x  (%s)\n",
               (int)"INFO", tTimeNow,(int)(taskIdSelf()),ec,(int)argbuf,0);
      else {
        sprintf(
           psbuf,stdfmt,"INFO",tTimeNow,taskIdSelf(),ec,argbuf);

        /***
        [[U msgQec:=msgQLib::msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL) ]]
        ***/
        msgQec=msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL);
      }
#else
   delete[] psbuf;

#endif
      break;
case tmt_WARN:
#if !defined(DIAGS_NOWARN) 
      if (bTraceMsgUsesLogMsg)
        logMsg("TRC(%s)  @%u : ID 0x%x, ec=0x%x  (%s)\n",
               (int)"WARN",tTimeNow,(int)(taskIdSelf()),ec,(int)argbuf,0);
      else {
        sprintf(
          psbuf,stdfmt,"WARN",tTimeNow,taskIdSelf(),ec,argbuf);

        /***
        [[U msgQec:=msgQLib::msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL) ]]
        ***/
        msgQec=msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL);
      }
#else
  delete[] psbuf;

#endif
      break;                                                                
    case tmt_ERROR:
      if (bTraceMsgUsesLogMsg)
        logMsg("TRC(%s)  @%u : ID 0x%x, ec=0x%x  (%s)\n",
               (int)"ERR ",tTimeNow,(int)(taskIdSelf()),ec,(int)argbuf,0);
      else {
        sprintf(
          psbuf,stdfmt,"ERR ",tTimeNow,taskIdSelf(),ec,argbuf);

        /***
        [[U msgQec:=msgQLib::msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL) ]]
        ***/
        msgQec=msgQSend(this->msgQId,(char *) &psbuf,sizeof(char *),NO_WAIT,MSG_PRI_NORMAL);
      }
      break;
    default: 
      logMsg("Invalid tmtMsgType %d in tTrace::TraceMsgCommon\n",
             tmtMsgType,0,0,0,0,0);
  }                              
  if (msgQec!=OK) {
    logMsg("ERROR: Traceback message queue error %d\n",msgQec,0,0,0,0,0);
    printf("ERROR: Traceback message queue error on message=%s\n",psbuf);
    delete [] psbuf;
  }
  delete [] argbuf;
  return ec;
}

/***
[[U this->tTrace::TraceMsg(tmtMsgType, ec, fmt, args)]]
***/
ERROR_CODE tTrace::TraceMsg (const TRACE_MSG_TYPE tmtMsgType, 
                      const int ec, const char *fmt, ...)  {    

  ERROR_CODE ecLocal; 
  
  va_list ap;
  va_start (ap,fmt);
  ecLocal = theTrace.TraceMsgCommon(tmtMsgType,ec,fmt,ap);
  va_end(ap);
  return ecLocal;
}

/***
[[U this->tTrace::Traceback(ec, fmt, args)]]
***/
ERROR_CODE tTrace::Traceback(const int ec, const char *fmt, ...)  {    

  ERROR_CODE ecLocal;
  
  va_list ap;
  va_start (ap,fmt);
  if (ec==0)
    ecLocal = this->TraceMsgCommon(tmt_INFO,ec,fmt,ap);
  else
    ecLocal = this->TraceMsgCommon(tmt_ERROR,ec,fmt,ap);
  va_end(ap);
  return ec;
}

/***
[[U this->tTrace::TraceMsgOut(void)]]
***/
void tTrace::TraceMsgOut (void) {
  char *msgptr;

  FOREVER {
    /***
    [[U msgQLib::msgQReceive (this->msgQId, (char *) &msgptr, sizeof(char *),
                    WAIT_FOREVER) ]] 
    ***/
    if (msgQReceive(this->msgQId, (char *) &msgptr, sizeof(char *),
                    WAIT_FOREVER) != ERROR) {
      puts(msgptr);
      delete [] msgptr;
    }
    else {
      msgQDelete(this->msgQId);
      this->msgQId = NULL;
      return;
    }
  }
}

/***
[[U this->tTrace::SetGetTimeFuncSB(pNewGetTimeFunc)]]
***/
void tTrace::SetGetTimeFunc(const PTIMEFUNC pNewGetTimeFunc)
{
  this->pGetTimeFunc = pNewGetTimeFunc;
} 

/***
[[U this->tTrace::Okay(void)]]
***/
ERROR_CODE tTrace::Okay(void){ return OK; } 

/***
[[U theTrace.main::Traceback(ec, fmt, args)]]
***/
ERROR_CODE Traceback (const int ec, const char *fmt, ...) {
  ERROR_CODE ecLocal;
  
  va_list ap;
  va_start (ap,fmt);
  if (ec==0)
    ecLocal = theTrace.TraceMsgCommon(tmt_INFO,ec,fmt,ap);
  else
    ecLocal = theTrace.TraceMsgCommon(tmt_ERROR,ec,fmt,ap);
  va_end(ap);
  return ec;
}

void TraceSetGetTimeFunc(PTIMEFUNC pNewGetTimeFunc) {
  theTrace.SetGetTimeFunc(pNewGetTimeFunc);
}


/***
[[U theTrace.main::TraceMsg(tmtMsgType,ec,fmt,args)]]
***/
ERROR_CODE TraceMsg (const TRACE_MSG_TYPE tmtMsgType, 
                     const int ec, const char *fmt, ... ) {

  ERROR_CODE ecLocal; 
  
  va_list ap;
  va_start (ap,fmt);
  ecLocal = theTrace.TraceMsgCommon (tmtMsgType,ec,fmt,ap);
  va_end(ap);
  return ecLocal;
}

/***
[[U theTrace.main::TraceMark(iStatus) ]]
***/
ERROR_CODE TraceMark (const int iStatus) {
  return theTrace.TraceMark (iStatus);
}

/***
[[U theTrace.main::TraceDump(iCount,iFromNewest)]]
***/
ERROR_CODE TraceDump (const int iCount, const int iFromNewest) {
  return theTrace.TraceDump(iCount,iFromNewest);
}                             

/***
[[U theTrace.main::TraceDumpText(iCount,iFromNewest)]]
***/
ERROR_CODE TraceDumpText (const int iCount, const int iFromNewest) {
  return theTrace.TraceDumpText(iCount,iFromNewest);
}                             

/***
[[U theTrace.main::TraceClear(void)]]
***/
void TraceClear(void) {    
  theTrace.TraceClear();
}

/***
[[U theTrace.main::TraceFreeze(void)]]
***/
void TraceFreeze(void) {
  theTrace.TraceFreeze();
}

/***
[[U theTrace.main::TraceUnfreeze(void)]]
***/
void TraceUnfreeze(void) {
  theTrace.TraceUnfreeze();;
}

void TraceUseLogMsg(const BOOL bNewValue) {
  theTrace.bTraceMsgUsesLogMsg = bNewValue;
}

