/* $RCSfile: tracesb.h,v $ - Traceback header file */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/comm/traceback/tracesb.h,v 1.1.1.1 1998/06/11 21:36:22 chara Exp $
*/
/* Michael Deck - Cleanroom Software Engineering, Inc. */

/* 
modification history
--------------------
01a,mdd,14Apr97    written
02a,mdd,17Apr97    rewritten
*/

/*
DESCRIPTION

*/
#include "trace.h"
/****************************************************************
*
* tTrace State Box
*
* SB State Variables
* ------------------
*   imarkbufferlength : integer 
*     !tTraceOK(this) => UNDEFINED
*     | true => RM(imarkbufferlength) = fastbuf.length
*   imarkbufferarray : array [0..CiMaxTraceMarks] of int
*     !tTraceOK(this) = UNDEFINED
*     | imarkbufferlength == 0 => RM(markbuffer) == empty fastbuf
*     | true => RM(markbuffer[imarkbuffernewest] == 
*               most recent item in fastbuf, 
*               RM(markbuffer[(imarkbuffernewest-1) mod 
*                 CiMaxTraceMarks]) == 2nd most recent item in fastbuf
*               ...
  Specification Functions
  -----------------------
  
  tracedump_logsb(int iCount, int iFromNewest, 
                  imarkbufferarray,imarkbufferlength,imarkbuffernewest) ==
    iFromNewest < 0 => UNDEFINED
                  
  | true => 
    returns a string containing max(iCount,imarbufferlength-iFromNewest)
    messages, nicely formatted. The messages represent elements of 
    imarkbufferarray beginning with the iFromNewest'th from imarkbuffernewest

* 
*   BOOL tTraceOK (tTrace* this) == 
*     this->imarkbufferlength < 0 ||
*     this->imarkbufferlength > CiMaxTraceMarks ||
*     this->imarkbuffernewest < 0 ||
*     this->imarkbuffernewest > CiMaxTraceMarks => FALSE
*   | true => TRUE
*   end
* 
****************************************************************/

/*****************************************************************
*
* tTrace::tTrace(void) - constructor
*
* DESCRIPTION: see black box 
*
[[U tTrace::tTrace() ]]
[[D tTrace::tTrace_SB() ]
 [ ! ((FUNCPTR) &(this->TraceMsgOut) is a pointer to valid C function || 
        ((FUNCPTR) &(this->TraceMsgOut) is a valid pointer to a C++ method && 
        (int) this is a valid pointer to a C++ object with method *(FUNCPTR) 
        &(this->TraceMsgOut) )) => UNDEFINED
 | a message queue cannot be created to contain CiMaxTraceMarks 
   elements each sizeof(char*) => 
      OS.Task.errno := error code appropriate to failure mode
    : this->msgQId := NULL
    : this->iTraceTaskId := anything 
    : this->imarkbufferlength := 0
    : this->imarkbuffernewest := 0
    : this->bFrozen := FALSE
 | a new task cannot be created having the appropriate characteristics =>    
      OS.Task.errno := error code appropriate to failure mode
    : this->msgQId := id of a new message queue that contains 
        CiMaxTraceMarks elements each sizeof(char*) 
    : this->iTraceTaskId := anything
    : this->imarkbufferlength := 0
    : this->imarkbuffernewest := 0
    : this->bFrozen := FALSE
 | true => 
      this->msgQId := id of a new message queue that contains 
        CiMaxTraceMarks elements each sizeof(char*) 
    : this->iTraceTaskId := id of a new task w/appropriate characteristics 
    : this->imarkbufferlength := 0
    : this->imarkbuffernewest := 0
    : this->bFrozen := FALSE
]]
* 
* RETURNS: void
*
*
*****************************************************************
*
* tTrace::~tTrace(void) - destructor
*
* DESCRIPTION: see black box 
*
[[U tTrace::~tTrace() ]]
[[D tTrace::~tTrace_SB () ]
 [ this invalid => UNDEFINED
 | cannot delete message queue or delete task => 
      *this := anything
    : OS.Task.errno := appropriate error code 
 | true => return space of *this to heap
    : free message queue this->msgQId
    : delete task this->iTraceTaskId
    : *this := anything
]]       
* 
* RETURNS: void
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceMark(int iStatus)
*
* DESCRIPTION: fast adds a trace mark to circular buffer for later display
*
[[U tTrace::TraceMark() ]]
[[D tTrace::TraceMark_SB (in: this, iStatus, imarkbufferlength, imarkbuffernewest;
                          out: markbuffer,imarkbufferlength,imarkbuffernewest;)]
 [ !tTraceOK(this) => UNDEFINED
 | imarkbufferlength == CiMaxTraceMarks =>
     imarkbuffernewest := (imarkbuffernewest+1) mod CiMaxTraceMarks
   : imarkbufferarray[(imarkbuffernewest+1) mod CiMaxTraceMarks] := iStatus
   : return := OK
 | true => 
     imarkbufferlength := imarkbufferlength+1
   : imarkbuffernewest := (imarkbuffernewest+1) mod CiMaxTraceMarks
   : imarkbufferarray[(imarkbuffernewest+1) mod CiMaxTraceMarks] := iStatus
   : return := OK
]]
* 
* RETURNS: OK
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceDump(int iCount)      
*
* DESCRIPTION: copies up to iCount most recent elements in fastbuf to 
*              console via message queue with appropriate accompanying text
*
[[U tTrace::TraceDump() ]]
[[D tTrace::TraceDump_SB(in: iCount, markbuffer, imarkbufferlength, 
                           imarkbuffernewest, this->iMsgQId; 
                       out: console, logmsg, return;)]
[ !tTraceOk(this) => UNDEFINED
| task that drains msgQ this->iMsgQId is not running or cannot write
    msgQ items to console => UNDEFINED
| insufficient memory to allocate space for message => UNDEFINED
| msgQ this->iMsgQId returns error on send => 
     return := S_tracedump_QUEUE_ERROR
   : OS.Task.errno := appropriate to failure mode    
| msgQ this->iMsgQId is full => return S_tracedump_QUEUE_FULL
    : logmsg "ERROR: Tracedump message queue is full"
| imarkbufferlength == 0 => 
   return := OK
| true => append tracedump_logsb(iCount,imarkbufferarray,
                  imarkbufferlength,imarkbuffernewest) to msgQ this->iMsgQId
 : return := OK
]]
*
* 
* RETURNS: OK, S_tracedump_QUEUE_FULL 
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceClear(void)    
*
* DESCRIPTION: clears tTrace::fastbuf 
*
[[U tTrace::TraceClear() ]]
[[D tTrace::TraceClear_SB(in: this; out: imarkbufferlength;)]
[ !tTraceOK(this) => UNDEFINED
| true => imarkbufferlength := 0
    : return := OK
]]    
*
* 
* RETURNS: OK
*
*
*****************************************************************
*
* ERROR_CODE tTrace::Okay(void)    
*
* DESCRIPTION: returns OK
*
[[U tTrace::Okay() ]]
[[D tTrace::Okay_SB() ]
[ !tTraceOk(this) => UNDEFINED
| true => return := OK
]]
*
* 
* RETURNS: OK
*
*
*****************************************************************/

