/* $RCSfile: dlCommandEngine.cc,v $ */
/* CM Info:
$Header $
*/
/* Richard L Johnson Jr - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,24Sep97	   addapted to new gizmo architecture
01a,beh,24Sep96    written
*/
 
/*
DESCRIPTION
This file contains the implementation of the delay line
command engines.
*/

#include "isit.h"
#include "inputChannel.h"
#include "dlGizmo.h"
#include "dlCommands.h"
#include "dl/dlCommandEngine.h"
#include "dl/limits.h"

//----------------------------------DlCommandEngine--------------------------

/****************************************************************************
[[U tDlCommandEngine::tDlCommandEngine_t (GizmoToUse) ]]
****/
tDlCommandEngine::tDlCommandEngine (tDlGizmo& GizmoToUse) : 

  /****
  [ this->Gizmo := reference to GizmoToUse ]
  ****/
  Gizmo (GizmoToUse) {}



/****************************************************************************
[[U tDlCommandEngine::tDlCommandEngine_tts (MutexToUse,GizmoToUse,
         sTaskNameToUse)]]
****/
tDlCommandEngine::tDlCommandEngine (tMutex& MutexToUse, tDlGizmo& GizmoToUse,
                                    char *sTaskNameToUse) :

  /****
  [[U tMutexUser::tMutexUser (MutexToUse,GizmoToUse,sTaskNameToUse) ]]
  ****/
  tMutexUser (&MutexToUse, &GizmoToUse, sTaskNameToUse), 

  /****
  [ this->Gizmo := reference to GizmoToUse ]
  ****/
  Gizmo (GizmoToUse) 
{}

/****************************************************************************
[[U tDlCommandEngine::UpdateState (dcsState) ]]
****/
void tDlCommandEngine::UpdateState (DL_COMMAND_STATUS dcsState) 
{

  /****
  [[U StatusTemp := this->Gizmo.tDlGizmo::GetStatus (void) ]]
  
  [[U StatusTemp.tDlStatus::UpdateCommandStatus (dcsState) ]]
  ****/
  this->Gizmo.GetStatus ().UpdateCommandStatus(dcsState);
}

/****************************************************************************
[[U tDlCommandEngine::ReportError (ec) ]] 
****/
void tDlCommandEngine::ReportError(ERROR_CODE ec) 
{
  /****
  [[U this->Gizmo.tGizmo::SendError_ec (ec) ]]
  ****/
  this->Gizmo.SendError (ec); 
}

/****************************************************************************
[[U tDlCommandEngine::SetServoMode (iServoMode) ]]
****/
ERROR_CODE tDlCommandEngine::SetServoMode (int iServoMode) 
{
  /****
  [[U this->Gizmo.Servo.tServo::SelectMode (iServoMode) ]]
  ****/
  return this->Gizmo.Servo.SelectMode (iServoMode);
}
 

//----------------------------------DlOffCommandEngine----------------------


/****************************************************************************
*
* tDlOffCommandEngine::ExecuteMethod - do an off command
*
* SPEC:

  [[U tDlCommandOffEngine::ExecuteMethod (void) ]]

*/

ERROR_CODE tDlOffCommandEngine::ExecuteMethod (void)
{
  ERROR_CODE ec;
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_OFF_START) ]]
  ****/
  this->UpdateState (DLCS_OFF_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_offCommand_OFF_START) ]]
  ****/
  this->ReportError(S_offCommand_OFF_START);

  /****
  [[U ec := tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode(DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_OFF_DONE) ]]
  ****/
  this->UpdateState (DLCS_OFF_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_offCommand_OFF_DONE) ]]
  ****/
  this->ReportError(S_offCommand_OFF_DONE);

  return ec;
}




/******************************************************************
*
* tDlFrontCommandEngine::ExecuteMethod - do a front command

  [[U tDlCommandFrontEngine::ExecuteMethod (void) ]]

*/

ERROR_CODE tDlFrontCommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_FRONT_START) ]]
  ****/
  this->UpdateState (DLCS_FRONT_START);
  
  /****
  [[U this->tDlCommandEngine::ReportError (S_frontCommand_FRONT_START) ]]
  ****/
  this->ReportError(S_frontCommand_FRONT_START);


  /****
  [[U ec := tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
  ****/
  ec = this->SetServoMode (DSM_FRONT);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_FRONT_MOVING) ]]
  ****/
  this->UpdateState (DLCS_FRONT_MOVING);

  while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit() == FALSE)
    taskDelay(1);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_FRONT_DONE) ]]
  ****/
  this->UpdateState (DLCS_FRONT_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_frontCommand_FRONT_DONE) ]]
  ****/
  this->ReportError(S_frontCommand_FRONT_DONE);

  return ec;  
}


/******************************************************************
*
* tDlFrontCommandEngine::AbortMethod - terminate a front command
*
  [[U tFrontCommandEngine::AbortMethod (void) ]]
*/

ERROR_CODE tDlFrontCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;
  
  /*****
  [[U this->tDlCommandEngine::SetServoMode(DSM_OFF)]]
  *****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_FRONT_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_FRONT_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_frontCommand_FRONT_ABORTED) ]]
  ****/
  this->ReportError(S_frontCommand_FRONT_ABORTED);

  return ec;  
}


/****************************************************************
   [[U  tDlHomeSetCommandEngine::ExecuteMethod (void)]]
****************************************************************/

ERROR_CODE tDlHomeSetCommandEngine::ExecuteMethod(void)
{
  BOOL       bOnBackSensor;   // these variables handle case #9 (see 
			      // Home Set/Check document. Fig. xxx). 
			      // That is, there exist the possibility
			      // that the value returned by the method 
			      // IsOnAnyXXXLimit could change from True
			      // to False before we get a chance to 
  BOOL       bOnFrontSensor;  //check it.

  ERROR_CODE         ec;
/* CHANGED BY THEO
  DL_DIRECTION  ddDesiredDirection = DD_POSITIVE;
*/
  DL_DIRECTION  ddDesiredDirection = DD_NEGATIVE;

  /****
  [[U this->tDlCommandEngine::UpdateState (DDLCS_HOMESET_START) ]]
  ****/
  this->UpdateState (DLCS_HOMESET_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_START) ]]
  ****/
  this->ReportError (S_homesetCommand_HOMESET_START);


  if (ddDesiredDirection == DD_POSITIVE)
  {
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_BACK_START) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_BACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_BACK_START) ]]
      ****/
      this->ReportError (S_homesetCommand_HOMESET_BACK_START);


      /****
      [[U ec := tDlCommandEngine::SetServoMode (DSM_BACK) ]]
      ****/
      ec = this->SetServoMode (DSM_BACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_BACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_BACK_MOVING);

      // loop until the delay line reaches the Home Sensor or a Back limit.
      while ( (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit() == FALSE) &&
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
	taskDelay(1);

      // Command Delay line to halt.  Give it time to do so.

      /****
      [[U ec := tDlCommandEngine::SetServoMode (DSM_OFF) ]]
      ****/
      ec = this->SetServoMode (DSM_OFF);
      taskDelay(2*sysClkRateGet());
      
      /*
       * STATUS: At this point the delay line is in one of the following 
       * positions 1, 2, 3, 4 or 5. see Figure 2.1 of Home Set/Check document.
       */

      // reverse direction
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_FRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_FRONT_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_FRONT_START) ]]
      ****/
      this->ReportError (S_homesetCommand_HOMESET_FRONT_START);

      /****
      [[U ec := tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
      ****/
      ec = this->SetServoMode(DSM_FRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_FRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_FRONT_MOVING);

      bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      while ( (!bOnFrontSensor) && 
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
      {
	taskDelay(1);
	bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      }

      // If we didn't find home sensor at all
      if (bOnFrontSensor)
      {
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homesetCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError(S_homesetCommand_CANNOT_FIND_HOME_SENSOR);
        return Traceback(S_homesetCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeSetCommandEngine::ExecuteMethod");
      }

      //continue delay line movement until no longer on home sensor.
      while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == TRUE);
	taskDelay(1);                                    

      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 6, 7, 8, 9 or 10. see Figure 2.1 of Home Set/Check document.
       */

      // reverse direction and head towards home sensor for the last time
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_CREEPBACK_START) ]]
      ****/
      this->UpdateState(DLCS_HOMESET_CREEPBACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_CREEPBACK_START) ]]
      ****/
      this->ReportError (S_homesetCommand_HOMESET_CREEPBACK_START);

      /****
      [[U this->tDlCommandEngine::SetServoMode (DSM_CREEPBACK) ]]
      ****/
      ec = this->SetServoMode (DSM_CREEPBACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_CREEPBACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_CREEPBACK_MOVING);

      // this method will cause the CPR and FPR values to be cleared next 
      // time the fiducial is crossed.
      taskDelay(sysClkRateGet()/5);         /* wait in case of noise */
      ec = this->Gizmo.GetCompleteIoSet ().Laser.ClearCounterOnFiducial (); 

      bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      while ( (!bOnBackSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() 
               == FALSE) )
      {
	taskDelay(1);
	bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      }

      /*
       * STATUS: At this point the delay line is either at the Home Sensor 
       * (positions 11 or 12 Fig 2.1) or the Back limits (positions 13, 14, or
       *  15)
       */

      // Check for error condition
      if (bOnBackSensor)
      {
	// We hit a Back limit (positions 13, 14 or 15)
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homesetCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError(S_homesetCommand_CANNOT_FIND_HOME_SENSOR);
	return Traceback(S_homesetCommand_CANNOT_FIND_HOME_SENSOR,
                         "tDlHomeSetCommandEngine::ExecuteMethod");
      }
  }
  else
  {
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_FRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_FRONT_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_FRONT_START) ]]
      ****/
      this->ReportError (S_homesetCommand_HOMESET_FRONT_START);

      /****
      [[U this->tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
      ****/
       ec = this->SetServoMode (DSM_FRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_FRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_FRONT_MOVING);

      while ( (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit() == FALSE) &&
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) ) 
	taskDelay(1);

      // Command Delay line to halt.  Give it time to do so.

      /****
      [[U this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
      ****/
      ec = this->SetServoMode (DSM_OFF);
      taskDelay(2*sysClkRateGet());

      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 1, 2, 3, 4 or 5. see Figure 2.2 of Home Set/Check document.
       */

      // reverse direction
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_BACK_START) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_BACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_BACK_START) ]]
      ****/
      this->ReportError(S_homesetCommand_HOMESET_BACK_START);

      /****
      [[U this->tDlCommandEngine::SetServoMode (DSM_BACK) ]]
      ****/
      ec = this->SetServoMode (DSM_BACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_BACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_BACK_MOVING);

      bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      while ( (!bOnBackSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() 
               == FALSE) )
      {
	taskDelay(1);
	bOnBackSensor =  this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      }

      
      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 6, 7, 8, 9 or 10. see Figure 2.2 of Home Set/Check document.
       */

      // If we somehow managed to miss the home sensor
      if (bOnBackSensor)
      {
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homesetCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError(S_homesetCommand_CANNOT_FIND_HOME_SENSOR);
	return Traceback(S_homesetCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeSetCommandEngine::ExecuteMethod");
      }

      //continue delay line movement until no longer on home sensor.
      while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == TRUE); 
	taskDelay(1);                                 


      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions  7, 8, 9 or 10. see Figure 2.2 of Home Set/Check document.
       * That is, the delay line has gone just past the Home Sensor.
       */

      // reverse direction and head towards home sensor for the last time
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_CREEPFRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_CREEPFRONT_START);

     /****
     [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_CREEPFRONT_START) ]]
     ****/
     this->ReportError(S_homesetCommand_HOMESET_CREEPFRONT_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_CREEPFRONT) ]]
      ****/
      ec = this->SetServoMode(DSM_CREEPFRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_CREEPFRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMESET_CREEPFRONT_MOVING);

      // this method will cause the CPR and FPR values to be cleared next 
      // time the fiducial is crossed.
      taskDelay(sysClkRateGet()/5);         /* wait in case of noise */
      ec = this->Gizmo.GetCompleteIoSet ().Laser.ClearCounterOnFiducial ();
      
      bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      while ( (!bOnFrontSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() 
               == FALSE))
      {
	taskDelay(1);
	bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      }

      /*
       * STATUS: At this point the delay line is at the Home Sensor (i.e.
       * positions  11 or 12 of Fig. 2.2) or the Front limits (positions 13, 14,
       * or 15)
       */

      // Check for error condition
      if (bOnFrontSensor)
      {
	// We hit a Front limit (position 13, 14, 15, on figure 2.2)
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homesetCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError(S_homesetCommand_CANNOT_FIND_HOME_SENSOR);
        return Traceback(S_homesetCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeSetCommandEngine::ExecuteMethod");
      }
		    
  }
    

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_DONE) ]]
  ****/
  this->UpdateState (DLCS_HOMESET_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_DONE) ]]
  ****/
  this->ReportError (S_homesetCommand_HOMESET_DONE);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  return ec;

} /* tDlHomeSetCommandEngine */

// xxxxx


/****************************************************************
   [[U  tDlHomeSetCommandEngine::CleanUp (void)]]
****************************************************************/

ERROR_CODE tDlHomeSetCommandEngine::CleanUp(void)
{
  ERROR_CODE ec1, ec2;

  /****
  [[U ec1 := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec1 = this->SetServoMode(DSM_OFF);

  ec2 = Gizmo.GetCompleteIoSet ().Laser.ResetFiducialWait();


  return (ec1 != OK ? ec1: ec2);
}


/****************************************************************
   [[U  tDlHomeSetCommandEngine::AbortMethod (void)]]
****************************************************************/

ERROR_CODE tDlHomeSetCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  ec = this->CleanUp();

  /****
  [[U this->tDlCommandEngine::ReportError (S_homesetCommand_HOMESET_ABORTED) ]]
  ****/
  this->ReportError(S_homesetCommand_HOMESET_ABORTED);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_HOMESET_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_HOMESET_ABORTED);

  return ec;
}


/****************************************************************
   [[U tDlHomeCheckCommandEngine::ExecuteMethod (void)]]
**/

ERROR_CODE tDlHomeCheckCommandEngine::ExecuteMethod(void)
{
  BOOL       bOnBackSensor;   // these variables handle case #9 (see 
			      // Home Set/Check document. Fig. xxx). 
			      // That is, there exist the possibility
			      // that the value returned by the method 
			      // IsOnAnyXXXLimit could change from True
			      // to False before we get a chance to 
  BOOL       bOnFrontSensor;  //check it.

  ERROR_CODE         ec;
/* CHANGED BY THEO
  DL_DIRECTION  ddDesiredDirection = DD_POSITIVE;
*/
  DL_DIRECTION  ddDesiredDirection = DD_NEGATIVE;

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_START) ]]
  ****/
  this->UpdateState (DLCS_HOMECHECK_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_START) ]]
  ****/
  this->ReportError(S_homecheckCommand_HOMECHECK_START);

  if (ddDesiredDirection == DD_POSITIVE)
  {
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_BACK_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_BACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_BACK_START) ]]
      ****/
      this->ReportError(S_homecheckCommand_HOMECHECK_BACK_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_BACK) ]]
      ****/
      ec = this->SetServoMode(DSM_BACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_BACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_BACK_MOVING);

      // loop until the delay line reaches the Home Sensor or a Back limit.
      while ( (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit() == FALSE) &&
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
	taskDelay(1);

      // Command Delay line to halt.  Give it time to do so.

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
      ****/
      ec = this->SetServoMode (DSM_OFF);
      taskDelay(2*sysClkRateGet());
      
      /*
       * STATUS: At this point the delay line is in one of the following 
       * positions 1, 2, 3, 4 or 5. see Figure 2.1 of Home Set/Check document.
       */

      // reverse direction

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_FRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_FRONT_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_FRONT_START) ]]
      ****/
      this->ReportError (S_homecheckCommand_HOMECHECK_FRONT_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
      ****/
      ec = this->SetServoMode (DSM_FRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_FRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_FRONT_MOVING);

      bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      while ( (!bOnFrontSensor) && 
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
      {
	taskDelay(1);
	bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      }

      // If we didn't find home sensor at all
      if (bOnFrontSensor)
      {
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR);
        return Traceback(S_homecheckCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeCheckCommandEngine::ExecuteMethod");
      }

      //continue delay line movement until no longer on home sensor.
      while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == TRUE);
	taskDelay(1);                                    

      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 6, 7, 8, 9 or 10. see Figure 2.1 of Home Set/Check document.
       */

      // load the CPR value into the FPR the next time the fiducial is crossed.
      // Effectively storing the fiducial position in the FPR.
      taskDelay(sysClkRateGet()/5);         /* wait in case of noise ? */
      ec = this->Gizmo.GetCompleteIoSet ().Laser.LoadOnFiducial();

      // reverse direction and head towards home sensor for the last time

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_CREEPBACK_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_CREEPBACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_CREEPBACK_START) ]]
      ****/
      this->ReportError (S_homecheckCommand_HOMECHECK_CREEPBACK_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_CREEPBACK) ]]
      ****/
      ec = this->SetServoMode (DSM_CREEPBACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_CREEPBACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_CREEPBACK_MOVING);

      bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      while ( (!bOnBackSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
      {
	taskDelay(1);
	bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      }

      /*
       * STATUS: At this point the delay line is either at the Home Sensor 
       * (positions 11 or 12 Fig 2.1) or the Back limits (positions 13, 14, or
       *  15)
       */

      // Check for error condition
      if (bOnBackSensor)
      {
	// We hit a Back limit (positions 13, 14 or 15)
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR);
	return Traceback(S_homecheckCommand_CANNOT_FIND_HOME_SENSOR,
                         "tDlHomeCheckCommandEngine::ExecuteMethod");
      }
  }
  else
  {
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_FRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_FRONT_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_FRONT_START) ]]
      ****/
      this->ReportError (S_homecheckCommand_HOMECHECK_FRONT_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
      ****/
      ec = this->SetServoMode (DSM_FRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_FRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_FRONT_MOVING);

      while ( (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit() == FALSE) &&
	      (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) ) 
	taskDelay(1);

      // Command Delay line to halt.  Give it time to do so.

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
      ****/
      ec = this->SetServoMode (DSM_OFF);
      taskDelay(2*sysClkRateGet());

      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 1, 2, 3, 4 or 5. see Figure 2.2 of Home Set/Check document.
       */

      // reverse direction
      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_BACK_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_BACK_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_BACK_START) ]]
      ****/
      this->ReportError (S_homecheckCommand_HOMECHECK_BACK_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_BACK) ]]
      ****/
      ec = this->SetServoMode (DSM_BACK);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_BACK_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_BACK_MOVING);

      bOnBackSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      while ( (!bOnBackSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE) )
      {
	taskDelay(1);
	bOnBackSensor =  this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit();
      }

      
      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions 6, 7, 8, 9 or 10. see Figure 2.2 of Home Set/Check document.
       */

      // If we somehow managed to miss the home sensor
      if (bOnBackSensor)
      {
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR);
	return Traceback(S_homecheckCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeCheckCommandEngine::ExecuteMethod");
      }

      //continue delay line movement until no longer on home sensor.
      while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == TRUE); 
	taskDelay(1);                                 


      /*
       * STATUS: At this point the delay line is at one of the following 
       * positions  7, 8, 9 or 10. see Figure 2.2 of Home Set/Check document.
       * That is, the delay line has gone just past the Home Sensor.
       */

      // load the CPR value into the FPR the next time the fiducial is crossed.
      // Effectively storing the fiducial position in the FPR.
      taskDelay(sysClkRateGet()/5);         /* wait in case of noise ? */
      ec = this->Gizmo.GetCompleteIoSet ().Laser.LoadOnFiducial();

      // reverse direction and head towards home sensor for the last time

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_CREEPFRONT_START) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_CREEPFRONT_START);

      /****
      [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_FRONT_START) ]]
      ****/
      this->ReportError (S_homecheckCommand_HOMECHECK_FRONT_START);

      /****
      [[U ec := this->tDlCommandEngine::SetServoMode (DSM_CREEPFRONT) ]]
      ****/
      ec = this->SetServoMode (DSM_CREEPFRONT);

      /****
      [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_CREEPFRONT_MOVING) ]]
      ****/
      this->UpdateState (DLCS_HOMECHECK_CREEPFRONT_MOVING);

      bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      while ( (!bOnFrontSensor) && (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnHomeSensor() == FALSE))
      {
	taskDelay(1);
	bOnFrontSensor = this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit();
      }

      /*
       * STATUS: At this point the delay line is at the Home Sensor (i.e.
       * positions  11 or 12 of Fig. 2.2) or the Front limits (positions 13, 14,
       * or 15)
       */

      // Check for error condition
      if (bOnFrontSensor)
      {
	// We hit a Front limit (position 13, 14, 15, on figure 2.2)
	this->CleanUp();

        /****
        [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR) ]]
        ****/
	this->ReportError (S_homecheckCommand_CANNOT_FIND_HOME_SENSOR);
        return Traceback(S_homecheckCommand_CANNOT_FIND_HOME_SENSOR,
			 "tDlHomeCheckCommandEngine::ExecuteMethod");
      }
		    
  }
    
   /****
   [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_DONE) ]]
   ****/
   this->UpdateState  (DLCS_HOMECHECK_DONE);

   /****
   [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_DONE) ]]
   ****/
   this->ReportError  (S_homecheckCommand_HOMECHECK_DONE);

   /****
   [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
   ****/
   ec = this->SetServoMode (DSM_OFF);

  return ec;

} /* tDlHomeCheckCommandEngine */


/****************************************************************
  [[U tDlHomeCheckCommandEngine::AbortMethod (void)]]
****************************************************************/
ERROR_CODE tDlHomeCheckCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  ec = this->CleanUp();

  /****
  [[U this->tDlCommandEngine::ReportError (S_homecheckCommand_HOMECHECK_ABORTED) ]]
  ****/
  this->ReportError (S_homecheckCommand_HOMECHECK_ABORTED);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_HOMECHECK_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_HOMECHECK_ABORTED);

  return ec;
}



/****************************************************************
  [[U tDlHomeCheckCommandEngine::CleanUp (void)]]
****************************************************************/

ERROR_CODE tDlHomeCheckCommandEngine::CleanUp(void)
{
  ERROR_CODE ec1, ec2;

   /****
   [[U ec1 := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
   ****/
  ec1 = this->SetServoMode (DSM_OFF);

  ec2 = this->Gizmo.GetCompleteIoSet ().Laser.ResetFiducialWait();


  return (ec1 != OK ? ec1 : ec2);
}

/******************************************************************
*
* tDlBackCommandEngine::ExecuteMethod - do a back command
*
*
*
* ARGUMENTS:
*
*
* SPEC:
  [[U tDlBackCommandEngine::ExecuteMethod (void) ]]
  
*/

ERROR_CODE tDlBackCommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_BACK_START) ]]
  ****/
  this->UpdateState (DLCS_BACK_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_backCommand_BACK_START) ]]
  ****/
  this->ReportError (S_backCommand_BACK_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_BACK) ]]
  ****/
  ec = this->SetServoMode (DSM_BACK);


  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_BACK_MOVING) ]]
  ****/
  this->UpdateState (DLCS_BACK_MOVING);

  while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyBackLimit() == FALSE)
    taskDelay(1);
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_BACK_DONE) ]]
  ****/
  this->UpdateState (DLCS_BACK_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_backCommand_BACK_DONE) ]]
  ****/
  this->ReportError (S_backCommand_BACK_DONE);
  return ec;  
}


/******************************************************************
*
* tDlBackCommandEngine::AbortMethod - terminate a back command
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlBackCommandEngine::AbortMethod (void) ]]
*/

ERROR_CODE tDlBackCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;
  
  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_BACK_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_BACK_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_backCommand_BACK_ABORTED) ]]
  ****/
  this->ReportError (S_backCommand_BACK_ABORTED);

  return ec;  
}


/******************************************************************
*
* tDlTrackCommandEngine::ExecuteMethod - do a track command
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlTrackCommandEngine::ExecuteMethod (void) ]]
*/

ERROR_CODE tDlTrackCommandEngine::ExecuteMethod (void)
{
  ERROR_CODE ec;
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_TRACK_START) ]]
  ****/
  this->UpdateState (DLCS_TRACK_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_trackCommand_TRACK_START) ]]
  ****/
  this->ReportError (S_trackCommand_TRACK_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_TRACK) ]]
  ****/
  ec = this->SetServoMode (DSM_TRACK);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_TRACK_ACTIVE) ]]
  ****/
  this->UpdateState (DLCS_TRACK_ACTIVE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_trackCommand_TRACK_IN_PROGRESS) ]]
  ****/
  this->ReportError (S_trackCommand_TRACK_IN_PROGRESS);

  return ec;  
}


/******************************************************************
*
* tDlTrackCommandEngine::AbortMethod - terminate a track command
*
*
* SPEC:

  [[U tDlTrackCommandEngine::AbortMethod (void) ]]
*
* SEE ALSO:
*
*
* RETURNS:
*   Nothing
*/

ERROR_CODE tDlTrackCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;
  
  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_TRACK_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_TRACK_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_trackCommand_TRACK_ABORTED) ]]
  ****/
  this->ReportError (S_trackCommand_TRACK_ABORTED);

  return ec;  
}


/******************************************************************
*
* tDlUser1CommandEngine::ExecuteMethod - do a User1 command (track)
*
*
*
* ARGUMENTS:
*
*
* SPEC:
  [[U tDlUser1CommandEngine::ExecuteMethod (void) ]]
*/

ERROR_CODE tDlUser1CommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER1_START) ]]
  ****/
  this->UpdateState (DLCS_USER1_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user1Command_USER1_START) ]]
  ****/
  this->ReportError (S_user1Command_USER1_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_USER1) ]]
  ****/
  ec = this->SetServoMode (DSM_USER1);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER1_ACTIVE) ]]
  ****/
  this->UpdateState (DLCS_USER1_ACTIVE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user1Command_USER1_IN_PROGRESS) ]]
  ****/
  this->ReportError (S_user1Command_USER1_IN_PROGRESS);

  return ec;
}


/******************************************************************
*
* tDlUser1CommandEngine::AbortMethod - terminate a User1 command
*
*
*
* ARGUMENTS:
*
*
* SPEC:
  [[U tDlUser1CommandEngine::AbortMethod (void) ]]
*/

ERROR_CODE tDlUser1CommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER1_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_USER1_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user1Command_USER1_ABORTED) ]]
  ****/
  this->ReportError (S_user1Command_USER1_ABORTED);

  return ec;
}


/******************************************************************
*
* tDlUser2CommandEngine::ExecuteMethod - do a User2 command (track)
*
*
*
* SPEC:

  [[U tDlUser2CommandEngine::ExecuteMethod (void) ]]
*/

ERROR_CODE tDlUser2CommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER2_START) ]]
  ****/
  this->UpdateState (DLCS_USER2_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user2Command_USER2_START) ]]
  ****/
  this->ReportError (S_user2Command_USER2_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_USER2) ]]
  ****/
  ec = this->SetServoMode (DSM_USER2);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER2_ACTIVE) ]]
  ****/
  this->UpdateState (DLCS_USER2_ACTIVE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user2Command_USER2_IN_PROGRESS) ]]
  ****/
  this->ReportError (S_user2Command_USER2_IN_PROGRESS);

  return ec;
}


/******************************************************************
*
* tDlUser2CommandEngine::AbortMethod - terminate a User2 command
*
*
* SPEC:

  [[U tDlUser2CommandEngine::AbortMethod (void) ]]
*/

ERROR_CODE tDlUser2CommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_USER2_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_USER2_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_user2Command_USER2_ABORTED) ]]
  ****/
  this->ReportError (S_user2Command_USER2_ABORTED);

  return ec;
}



 

/******************************************************************
*
* tDlDitherCalibrateCommandEngine::ExecuteMethod - do a DitherCalibrate
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlDitherCalibrateCommandEngine::ExecuteMethod (void) ]]

*/

ERROR_CODE tDlDitherCalibrateCommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CALIB_DITHER_START) ]]
  ****/
  this->UpdateState (DLCS_CALIB_DITHER_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCalibCommand_CALIB_DITHER_START) ]]
  ****/
  this->ReportError (S_ditherCalibCommand_CALIB_DITHER_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
  ****/
  ec = this->SetServoMode (DSM_FRONT);

  while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit() == FALSE)
    taskDelay(1);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);
  taskDelay(5*sysClkRateGet());

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CALIB_DITHER_ACTIVE) ]]
  ****/
  this->UpdateState (DLCS_CALIB_DITHER_ACTIVE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCalibCommand_CALIB_DITHER_IN_PROGRESS) ]]
  ****/
  this->ReportError (S_ditherCalibCommand_CALIB_DITHER_IN_PROGRESS);

  ec = this->Gizmo.GetCompleteIoSet ().Laser.CalibrateDither();
  if (ec!=OK)
      /****
      [[U this->tDlCommandEngine::ReportError (ec) ]]
      ****/
     this->ReportError (ec);

  this->Gizmo.GetCompleteIoSet ().Laser.CleanUpDither (ec);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CALIB_DITHER_DONE) ]]
  ****/
  this->UpdateState (DLCS_CALIB_DITHER_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCalibCommand_CALIB_DITHER_DONE) ]]
  ****/
  this->ReportError (S_ditherCalibCommand_CALIB_DITHER_DONE);

  return ec;

}   


/******************************************************************
*
* tDlDitherCalibrateCommandEngine::AbortMethod - terminate a DitherCalibrate
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlDitherCalibrateCommandEngine::AbortMethod (void) ]]

*/

ERROR_CODE tDlDitherCalibrateCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  this->Gizmo.GetCompleteIoSet ().Laser.CleanUpDither (ERROR);  /* pass anything but OK */

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CALIB_DITHER_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_CALIB_DITHER_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCalibCommand_CALIB_DITHER_ABORTED) ]]
  ****/
  this->ReportError (S_ditherCalibCommand_CALIB_DITHER_ABORTED);

  return ec;

}   /* tDlDitherCalibrateCommandEngine::AbortMethod() */


/******************************************************************
*
* tDlDitherCheckCommandEngine::ExecuteMethod - do a DitherCheck command
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlDitherCheckCommandEngine::ExecuteMethod (void) ]]

*/

ERROR_CODE tDlDitherCheckCommandEngine::ExecuteMethod(void)
{
  ERROR_CODE ec;

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CHECK_DITHER_START) ]]
  ****/
  this->UpdateState (DLCS_CHECK_DITHER_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCheckCommand_CHECK_DITHER_START) ]]
  ****/
  this->ReportError (S_ditherCheckCommand_CHECK_DITHER_START);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_FRONT) ]]
  ****/
  ec = this->SetServoMode (DSM_FRONT);

  while (this->Gizmo.GetCompleteIoSet ().DlLimits.IsOnAnyFrontLimit() == FALSE)
    taskDelay(1);

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  // Wait 5 seconds for DL to settle out
  taskDelay(5*sysClkRateGet());

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CHECK_DITHER_ACTIVE) ]]
  ****/
  this->UpdateState (DLCS_CHECK_DITHER_ACTIVE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCheckCommand_CHECK_DITHER_IN_PROGRESS) ]]
  ****/
  this->ReportError (S_ditherCheckCommand_CHECK_DITHER_IN_PROGRESS);

  ec = this->Gizmo.GetCompleteIoSet ().Laser.CheckDither();
  if (ec!=OK)
      /****
      [[U this->tDlCommandEngine::ReportError (ec) ]]
      ****/
     this->ReportError (ec);

  this->Gizmo.GetCompleteIoSet ().Laser.CleanUpDither (ec);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CHECK_DITHER_DONE) ]]
  ****/
  this->UpdateState (DLCS_CHECK_DITHER_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCheckCommand_CHECK_DITHER_DONE) ]]
  ****/
  this->ReportError (S_ditherCheckCommand_CHECK_DITHER_DONE);

  return ec;

}   /* tDlDitherCheckCommandEngine::ExecuteMethod() */


/******************************************************************
*
* tDlDitherCheckCommandEngine::AbortMethod - terminate a DitherCheck command
*
*
*
* ARGUMENTS:
*
*
* SPEC:

  [[U tDlDitherCheckCommandEngine::AbortMethod (void) ]]
*/

ERROR_CODE tDlDitherCheckCommandEngine::AbortMethod(void)
{
  ERROR_CODE ec;

  this->Gizmo.GetCompleteIoSet ().Laser.CleanUpDither (ERROR);  /* pass anything but OK */

  /****
  [[U ec := this->tDlCommandEngine::SetServoMode (DSM_OFF) ]]
  ****/
  ec = this->SetServoMode (DSM_OFF);

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CHECK_DITHER_ABORTED) ]]
  ****/
  this->UpdateState (DLCS_CHECK_DITHER_ABORTED);

  /****
  [[U this->tDlCommandEngine::ReportError (S_ditherCheckCommand_CHECK_DITHER_ABORTED) ]]
  ****/
  this->ReportError (S_ditherCheckCommand_CHECK_DITHER_ABORTED);

  return ec;

}   



/******************************************************************
*
* tDlClearLaserCounterCommandEngine::ExecuteMethod
*
* Executes a clear laser counter command
*
*
*
* SPEC:

  [[U tDlClearLaserCounterCommandEngine::ExecuteMethod (void) ]]
*/

ERROR_CODE tDlClearLaserCounterCommandEngine::ExecuteMethod (void)
{
  ERROR_CODE ec;
  
  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CLEAR_LASER_COUNTER_START) ]]
  ****/
  this->UpdateState (DLCS_CLEAR_LASER_COUNTER_START);

  /****
  [[U this->tDlCommandEngine::ReportError (S_clearLaserCounterCommand_CLEAR_LASER_COUNTER_START) ]]
  ****/
  this->ReportError (S_clearLaserCounterCommand_CLEAR_LASER_COUNTER_START);

  /****
  [[U ec := this->Gizmo.GetCompleteIoSet ().Laser.tFiducialChannel::ClearCounter (void) ]]
  ****/
  ec = this->Gizmo.GetCompleteIoSet ().Laser.ClearCounter ();

  /****
  [[U this->tDlCommandEngine::UpdateState (DLCS_CLEAR_LASER_COUNTER_DONE) ]]
  ****/
  this->UpdateState (DLCS_CLEAR_LASER_COUNTER_DONE);

  /****
  [[U this->tDlCommandEngine::ReportError (
     S_clearLaserCounterCommand_CLEAR_LASER_COUNTER_COMPLETE) ]]
  ****/
  this->ReportError (S_clearLaserCounterCommand_CLEAR_LASER_COUNTER_COMPLETE);

  return ec;  
}















