/* $RCSfile: dlDataStore.cc,v $ - DL-Data */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlDataStore.cc,v 1.4 1999/03/30 20:23:19 chara Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,17Oct96,eamck    written (based on Relayer.cc)
*/
      
/*
DESCRIPTION
  ISIT Relayer code
*/

#include "isit.h"
#include "typeTag.h"
#include "dlTypes.h"
#include <errnoLib.h>
#include "dl/dlDataStore.h"
#include <math.h>



/****
[[U tDlDataStore::tDlDataStore (iDataDumpTaskPriority,iBufferSize, iHpBlockSizeToUse,
                  TypeTagToUse, StatusToUse) ]]
****/                 
tDlDataStore::tDlDataStore (int iDataDumpTaskPriority, int iBufferSize, int iHpBlockSizeToUse,
                  const tTypeTag& TypeTagToUse, tDlStatus& StatusToUse) :
                  
  /****
  [[U tDataStore::tDataStore (iDataDumpTaskPriority,iBufferSize, 
                  TypeTagToUse) ]]
  ****/                 
  tDataStore<tDlData> (iDataDumpTaskPriority,iBufferSize,TypeTagToUse),


  /****
  [[ this->Status := reference to StatusToUse ]
  ****/
  Status (StatusToUse),
  
  bSuspendData (TRUE)
{

  this->iHpBlockSize = iHpBlockSizeToUse;
}


/****
[[U tDlDataStore::~tDlDataStore (void) ]]
****/                 
tDlDataStore::~tDlDataStore () {}

                  
/****
[[U tDlDataStore::DumpToDataStream (void) ]]
****/                 
void tDlDataStore::DumpToDataStream () 
{

  double dDlJitter, dDlHpJitter;


  /****
  [[U dDlJitter := this->tDlDataStore::CalcDlJitter (void) ]]
  ****/                 
  dDlJitter = this->CalcDlJitter ();

  /****
  [[U dDlHpJitter := this->tDlDataStore::CalcDlHpJitter (void) ]]
  ****/                 
  dDlHpJitter = this->CalcDlHpJitter ();

  /****
  [[U this->tDlStatus::UpdateJitter (dDlJitter, dDlHpJitter) ]]
  ****/                 
  this->Status.UpdateJitter (dDlJitter, dDlHpJitter);

  if(!this->bSuspendData) {
    /****
    [[U this->tDataStore::DumpToDataStream (void) ]]
    ****/                 
    this->tDataStore<tDlData>::DumpToDataStream ();
  }
}

/****
[[U tDlDataStore::CalcDlJitter (void) ]]
****/                 

double tDlDataStore::CalcDlJitter ()  
{
    int i;
    double dTmp;
    double dVar;
    double dPmean;
    double dSumx=0.0;
    double dSumx2=0.0;
   
        
    for (i=0; i < DL_DATA_BUF_SIZE; i++)
    {
      dTmp =  (double) this->DataBufferArrayArray [this->iCurrentReadBuffer][0].
                                  ServoDataArray[i].laError * (double) JITTER_SCALE;
      dSumx  += dTmp; 
      dSumx2 += dTmp  * dTmp ;
    }

    /* dPmean = dSumx / (double) DL_DATA_BUF_SIZE;  <---- not currently used */
    
    dVar = (dSumx2 - (dSumx*dSumx) / (double) DL_DATA_BUF_SIZE) / 
                    (double) DL_DATA_BUF_SIZE ;

    //printf("CalcDlJitter: Sum=%f, Sum^2=%f, dVar=%f, NumEl=%d\n",
    //       dSumx, dSumx2, dVar, DL_DATA_BUF_SIZE);

    if (dVar<0)
      return ERROR;
    else
      return sqrt(dVar);
}  /* calcDlJitter() */



/****
[[U tDlDataStore::CalcDlHpJitter (void) ]]
****/                 
double tDlDataStore::CalcDlHpJitter ()
{
     int i, j;
     double dTmp;
     double dSumx;
     double dSumx2;
     double dSumVar=0.0;
     int iVarCounter=0;

     if (this->iHpBlockSize > DL_DATA_BUF_SIZE) return ERROR;

     for (i=0; i < (DL_DATA_BUF_SIZE - this->iHpBlockSize); i += this->iHpBlockSize)
     {
         dSumx = 0;
         dSumx2 = 0;
         for (j=0; j < this->iHpBlockSize; j++)
         {
             dSumx  += dTmp = 
               this->DataBufferArrayArray [this->iCurrentReadBuffer][0].
	       ServoDataArray[i+j].laError * JITTER_SCALE;
	     dSumx2 += dTmp * dTmp;
         }
         dSumVar += (dSumx2-(dSumx * dSumx)/ (double) this->iHpBlockSize) / 
              (double) this->iHpBlockSize;
         iVarCounter++;
    }

    if (dSumVar<0)
      return ERROR;
    else
      return sqrt(dSumVar/iVarCounter);
}  /* calcDlHpJitter() */






