/* $RCSfile: dlGizmoInfo.cc,v $ - tDlGizmoInfo class definition */

/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlGizmoInfo.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/

/* Phil Irwin - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,26Sep97,pci      written
*/

#include "isit.h"
#include "configMgr.h"
#include "dlGizmoInfo.h"
#include "inputChannel.h"
#include "outputChannel.h"
#include "dlTypes.h"


tDlGizmoInfo::tDlGizmoInfo(int iWhichRteToUse,int iGizmoIdToUse,
       const int iTargetSourceArray[NUM_DELAY_LINE_TARGETS],
       TIME tDitherPeriodToUse,
       DlPhysicalConstants PhysConstToUse, int iNumServoModesToUse,
       const tServoInfo& ServoInfoToUse,
       BOARD_NAME bnVc1BoardToUse, int iVc1ChannelToUse,
       BOARD_NAME bnServoPztBoardToUse, int iServoPztChannelToUse,
       BOARD_NAME bnMotorBoardToUse, int iMotorChannelToUse,
       BOARD_NAME bnDitherPztBoardToUse, int iDitherPztChannelToUse,
       BOARD_NAME bnFrontSoftLimitBoardToUse, int iFrontSoftLimitChannelToUse,
       BOARD_NAME bnFrontHardLimitBoardToUse, int iFrontHardLimitChannelToUse,
       BOARD_NAME bnBackSoftLimitBoardToUse, int iBackSoftLimitChannelToUse,
       BOARD_NAME bnBackHardLimitBoardToUse, int iBackHardLimitChannelToUse,
       BOARD_NAME bnLaserBoardToUse , int iLaserChannelToUse,
       BOARD_NAME bnHomeSensorBoardToUse , int iHomeSensorChannelToUse,
       BOARD_NAME bnServoPztReadbackBoardToUse, int iServoPztReadbackChannelToUse,
       BOARD_NAME bnMotorReadbackBoardToUse, int iMotorReadbackChannelToUse,
       BOARD_NAME bnDiagLaserInputBoardToUse, int iDiagLaserInputChannelToUse,
       BOARD_NAME bnDiagPztPositionInputBoardToUse, int iDiagPztPositionChannelToUse
,
       BOARD_NAME bnDiagLaserOutputBoardToUse, int iDiagLaserOutputChannelToUse,
       BOARD_NAME bnDiagLaserServoErrorOutputBoardToUse,
              int iDiagLaserServoErrorOutputChannelToUse,
       BOARD_NAME bnDiagVc1ServoErrorOutputBoardToUse,
              int iDiagVc1ServoErrorOutputChannelToUse,
       BOARD_NAME bnDiagMotorServoErrorOutputBoardToUse,
              int iDiagMotorServoErrorOutputChannelToUse) :

  /****
  [[U tRealtimeGizmoInfo::tRealtimeGizmoInfo(iWhichRteToUse, iGizmoIdToUse,
                      iNumServoModesToUse, ServoInfoToUse) ]]
  ****/
  tRealtimeGizmoInfo(iWhichRteToUse, iGizmoIdToUse,
                      iNumServoModesToUse, ServoInfoToUse)
  {
    int i;
    IPC_NAME inName;
    MEM_TYPE mtMemType;

    for (i=0; i < NUM_DELAY_LINE_TARGETS; i++) {
      /****
      [[U inName := this->tGizmoInfo::NewIpcName (void) ]]
      ****/
      inName = this->NewIpcName ();

      /****
      [[U mtMemType := this->tGizmoInfo::TypeOfMemory (this->iRteId,
            iTargetSourceArray[i]) ]]
      ****/
      mtMemType = this->TypeOfMemory (this->iRteId,iTargetSourceArray[i]) ;

      /****
      [[U this->iiTargetIdArray[i].tIpcDblBufferId (inName,mtMemType) ]]
      ****/
      this->iiTargetIdArray[i] = tIpcDblBufferId (inName,mtMemType);
    }

    tDitherPeriod = tDitherPeriodToUse;

    pVc1Channel = D_CAST(tOutputChannel *)
                              ConfigManager.Hardware.CreateChannel
                                          (bnVc1BoardToUse, iVc1ChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
             "pVc1Channel is %ld\n",(int)pVc1Channel);

    pServoPztChannel = D_CAST(tOutputChannel *)
                              ConfigManager.Hardware.CreateChannel
                                          (bnServoPztBoardToUse,
                                           iServoPztChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pServoPztChannel is %ld\n",(int)pServoPztChannel);

    pMotorChannel = D_CAST(tOutputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                          (bnMotorBoardToUse, iMotorChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pMotorChannel is %ld\n",(int)pMotorChannel);

    pDitherPztChannel = D_CAST(tWaveformChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                          (bnDitherPztBoardToUse,
                                           iDitherPztChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pDitherPztChannel is %ld\n",(int)pDitherPztChannel);

    pFrontSoftLimitChannel =D_CAST(tInputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                          (bnFrontSoftLimitBoardToUse,
                                           iFrontSoftLimitChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pFrontSoftLimitChannel is %ld\n",(int)pFrontSoftLimitChannel);

    pFrontHardLimitChannel = D_CAST(tInputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                          (bnFrontHardLimitBoardToUse,
                                           iFrontHardLimitChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pFrontHardLimitChannel is %ld\n",(int)pFrontHardLimitChannel);

    pBackSoftLimitChannel = D_CAST(tInputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                          (bnBackSoftLimitBoardToUse,
                                           iBackSoftLimitChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pBackSoftLimitChannel is %ld\n",(int)pBackSoftLimitChannel);

    pBackHardLimitChannel = D_CAST(tInputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                        (bnBackHardLimitBoardToUse,
                                         iBackHardLimitChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pBackHardLimitChannel is %ld\n",(int)pBackHardLimitChannel);

    pLaserChannel = D_CAST(tLaserMetrologyChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                        (bnLaserBoardToUse,
                                         iLaserChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
             "pLaserChannel is %ld\n",(int)pLaserChannel);

    pHomeSensorChannel = D_CAST(tInputChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                        (bnHomeSensorBoardToUse,
                                         iHomeSensorChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
             "pHomeSensorChannel is %ld\n",(int)pHomeSensorChannel);

    pServoPztReadbackChannel = D_CAST(tReadbackChannel *)
                                   ConfigManager.Hardware.CreateChannel
                                        (bnServoPztReadbackBoardToUse,
                                         iServoPztReadbackChannelToUse);
    TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pServoPztReadbackChannel is %ld\n",(int)pServoPztReadbackChannel);

    if(!pVc1Channel || !pServoPztChannel || !pMotorChannel || !pDitherPztChannel ||
       !pFrontSoftLimitChannel || !pFrontHardLimitChannel ||
       !pBackSoftLimitChannel || !pBackHardLimitChannel ||
       !pLaserChannel || !pHomeSensorChannel  || !pServoPztReadbackChannel)
      TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
               "tDlGizmoInfo::tDlGizmoInfo");

    // Initialize Physical constants
    this->PhysConst = PhysConstToUse;


    // A number of diagnostic features aren't being used at present.
    // Set them to point to Null channels.
    pMotorReadbackChannel = D_CAST(tReadbackChannel *)
                              ConfigManager.Hardware.CreateChannel(
                                  bnMotorReadbackBoardToUse,
                                  iMotorReadbackChannelToUse);

    pDiagLaserInputChannel = D_CAST(tInputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                  bnDiagLaserInputBoardToUse,
                                  iDiagLaserInputChannelToUse);


    pDiagPztPositionInputChannel = D_CAST(tInputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                  bnDiagPztPositionInputBoardToUse,
                                  iDiagPztPositionChannelToUse);

    pDiagLaserOutputChannel = D_CAST(tOutputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                       bnDiagLaserOutputBoardToUse,
                                       iDiagLaserOutputChannelToUse);

    pDiagLaserServoErrorOutputChannel = D_CAST(tOutputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                 bnDiagLaserServoErrorOutputBoardToUse,
                                 iDiagLaserServoErrorOutputChannelToUse);

    pDiagVc1ServoErrorOutputChannel = D_CAST(tOutputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                        bnDiagVc1ServoErrorOutputBoardToUse,
                                        iDiagVc1ServoErrorOutputChannelToUse);

    pDiagMotorServoErrorOutputChannel = D_CAST(tOutputChannel *)
                               ConfigManager.Hardware.CreateChannel(
                                     bnDiagMotorServoErrorOutputBoardToUse,
                                     iDiagMotorServoErrorOutputChannelToUse);

    if(!pMotorReadbackChannel || !pDiagLaserInputChannel ||
          !pDiagPztPositionInputChannel || !pDiagLaserOutputChannel ||
          !pDiagLaserServoErrorOutputChannel || !pDiagVc1ServoErrorOutputChannel ||
          !pDiagMotorServoErrorOutputChannel)
      TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
               "tDlGizmoInfo::tDlGizmoInfo");
}

tDlGizmoInfo::~tDlGizmoInfo () {}


const tIpcDblBufferId& tDlGizmoInfo::GetTargetId (int iTargetNum) const {

  return this->iiTargetIdArray[iTargetNum];
}


const tIpcDblBufferId& tDlGizmoInfo::GetFtTargetId () const {
  return this->iiTargetIdArray[TT_FT];
}

const tIpcDblBufferId& tDlGizmoInfo::GetEmTargetId () const {
  return this->iiTargetIdArray [TT_EM];
}

const tIpcDblBufferId& tDlGizmoInfo::GetManualTargetId () const {
  return this->iiTargetIdArray [TT_MANUAL];
}

const tIpcDblBufferId& tDlGizmoInfo::GetOpenLoopTargetId () const {
  return this->iiTargetIdArray [TT_OPEN_LOOP];
}

TIME tDlGizmoInfo::FastServoPeriod () const{

  return this->ServoInfo.GetServoTaskInfo (DSP_FAST).tTaskPeriod;
}

TIME tDlGizmoInfo::MediumServoPeriod () const {

  return this->ServoInfo.GetServoTaskInfo (DSP_MEDIUM).tTaskPeriod;
}

TIME tDlGizmoInfo::SlowServoPeriod () const {

  return this->ServoInfo.GetServoTaskInfo (DSP_SLOW).tTaskPeriod;
}
