/* $RCSfile: dlIoSet.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlIoSet.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,eamck,11May97    written (based on dlIoSet.h)
*/
 
/*
DESCRIPTION

*/

#include "dl/dlIoSet.h"



/******************************************************************
*
* tDlCompleteIoSet::tDlCompleteIoSet
*
* SPECIFICATION
[[U5 tDlCompleteIoSet::tDlCompleteIoSet (GizmoInfo) ]]
****/
tDlCompleteIoSet::tDlCompleteIoSet (const tDlGizmoInfo& GizmoInfo) :

   /***
   [[U tPeriod := GizmoInfo.tDlGizmoInfo::FastServoPeriod () ]]
   
   [[U dAmp := Gizmo.tDlGizmoInfo::DitherAmplitude () ]]
   
   [[U this->Laser.tLaserMetrologySensor::tLaserMetrologySensor (
          *(GizmoInfo.pLaserChannel),
          LaserDither,  GizmoInfo.PhysConst.dLaserScale,
          GizmoInfo.PhysConst.iLaserNumPasses, GizmoInfo.PhysConst.iLaserSign,
          *(GizmoInfo.pDitherPztChannel),
          GizmoInfo.PhysConst.dDitherPztScale, GizmoInfo.tPeriod, (dAmp * 
            GizmoInfo.PhysConst.iLaserNumPasses / GizmoInfo.PhysConst.iStellarNumPasses) )]]
   ***/
   Laser (*GizmoInfo.pLaserChannel,
          GizmoInfo.PhysConst.dLaserScale,
          GizmoInfo.PhysConst.iLaserNumPasses, 
          GizmoInfo.PhysConst.iLaserSign, 
          *GizmoInfo.pDitherPztChannel, GizmoInfo.PhysConst.dDitherPztScale,
          GizmoInfo.FastServoPeriod (),
          GizmoInfo.DitherPeriod (), GizmoInfo.DitherAmplitude () * 
            GizmoInfo.PhysConst.iLaserNumPasses / GizmoInfo.PhysConst.iStellarNumPasses ),
   
   /***
   [[U this->PztPosition.tModulatedSensor::tModulatedSensor (
         *(GizmoInfo.pServoPztReadbackChannel), 
         GizmoInfo.PhysConst.dPztScale)]]
   ***/
   PztPosition (*(GizmoInfo.pServoPztReadbackChannel), 
                GizmoInfo.PhysConst.dPztScale ),
   
   /****
   [[U this->MotorPosition.tModulatedSensor::tModulatedSensor (
         *(GizmoInfo.pMotorReadbackChannel) , 
         GizmoInfo.PhysConst.dMotorPosScale )]]
   ****/
   MotorPosition (*(GizmoInfo.pMotorReadbackChannel) , 
                  GizmoInfo.PhysConst.dMotorPosScale ),
   
   /***
   [[U this->LaserDiagInput.tSensor::tSensor (
         *(GizmoInfo.pDiagLaserInputChannel), 
	 GizmoInfo.PhysConst.dDiagLaserScale)]]
   ***/
   LaserDiagInput (*(GizmoInfo.pDiagLaserInputChannel), 
			 GizmoInfo.PhysConst.dDiagLaserScale ),
   
   /***
   [[U this->PztPositionDiagInput.tSensor::tSensor (
         *(GizmoInfo.pDiagPztPositionInputChannel),
	 GizmoInfo.PhysConst.dDiagPztPosScale)]]
   ***/
   PztPositionDiagInput (*GizmoInfo.pDiagPztPositionInputChannel,
				    GizmoInfo.PhysConst.dDiagPztPosScale ),
   
   /****
   [[U this->DlLimits.tDlLimits::tDlLimits (
             *(GizmoInfo.pFrontHardLimitChannel), GizmoInfo.PhysConst.dFrontHLimitScale, 
             *(GizmoInfo.pFrontSoftLimitChannel), GizmoInfo.PhysConst.dFrontSLimitScale,
             *(GizmoInfo.pBackHardLimitChannel), GizmoInfo.PhysConst.dBackHLimitScale,
             *(GizmoInfo.pBackSoftLimitChannel), GizmoInfo.PhysConst.dBackSLimitScale,
             *(GizmoInfo.pHomeSensorChannel), GizmoInfo.PhysConst.dHomeScale)]]
   ****/
   DlLimits (*GizmoInfo.pFrontHardLimitChannel, GizmoInfo.PhysConst.dFrontHLimitScale, 
             *GizmoInfo.pFrontSoftLimitChannel, GizmoInfo.PhysConst.dFrontSLimitScale,
             *GizmoInfo.pBackHardLimitChannel, GizmoInfo.PhysConst.dBackHLimitScale,
             *GizmoInfo.pBackSoftLimitChannel, GizmoInfo.PhysConst.dBackSLimitScale,
             *GizmoInfo.pHomeSensorChannel, GizmoInfo.PhysConst.dHomeScale),
             

   /***
   [[U this->Pzt.tModulatedVelocityActuator::tModulatedVelocityActuator (
        *(GizmoInfo.pServoPztChannel),
        GizmoInfo.PhysConst.dPztScale,
        GizmoInfo.PhysConst.dPztMaxAccel,
        GizmoInfo.PhysConst.dPztMaxVelocity)]]
   ***/
   Pzt (*(GizmoInfo.pServoPztChannel),
        GizmoInfo.PhysConst.dPztScale,
        GizmoInfo.PhysConst.dPztMaxAccel,
        GizmoInfo.PhysConst.dPztMaxVelocity, (double)TIME_TICKS_PER_SECOND / (double)GizmoInfo.FastServoPeriod () ),
   
   /***
   [[U this->Vc1.tModulatedPositionActuator::tModulatedPositionActuator (
        *(GizmoInfo.pVc1Channel),
        GizmoInfo.PhysConst.dVc1Scale,
        GizmoInfo.PhysConst.dVc1MaxAccel,
        GizmoInfo.PhysConst.dVc1MaxVelocity,
        GizmoInfo.PhysConst.dVc1MinPosition,
        GizmoInfo.PhysConst.dVc1MaxPosition,
        ZERO_MODULATION, GizmoInfo.PhysConst.dVc1Offset)]]
   ***/
   Vc1 (*(GizmoInfo.pVc1Channel),
        GizmoInfo.PhysConst.dVc1Scale,
        GizmoInfo.PhysConst.dVc1MaxAccel,
        GizmoInfo.PhysConst.dVc1MaxVelocity,
        GizmoInfo.PhysConst.dVc1MinPosition,
        GizmoInfo.PhysConst.dVc1MaxPosition, 
        (double)TIME_TICKS_PER_SECOND / (double)GizmoInfo.MediumServoPeriod (),
        tModulationParams (), GizmoInfo.PhysConst.dVc1Offset),
   
   /***
   [[U this->Motor.tModulatedVelocityActuator::tModulatedVelocityActuator (
          *(GizmoInfo.pMotorChannel),
          GizmoInfo.PhysConst.dMotorScale,
          GizmoInfo.PhysConst.dMotorMaxAccel,
          GizmoInfo.PhysConst.dMotorMaxVelocity)]]
   ***/
   Motor (*(GizmoInfo.pMotorChannel),
          GizmoInfo.PhysConst.dMotorScale,
          GizmoInfo.PhysConst.dMotorMaxAccel,
          GizmoInfo.PhysConst.dMotorMaxVelocity,
          (double)TIME_TICKS_PER_SECOND / (double)GizmoInfo.SlowServoPeriod () ),


   /***
   [[U this->LaserDiagOutput.tActuator::tActuator (
         *(GizmoInfo.pDiagLaserOutputChannel), 
         GizmoInfo.PhysConst.dDiagLaserScale)]]
   ***/
   LaserDiagOutput (*(GizmoInfo.pDiagLaserOutputChannel), 
                    GizmoInfo.PhysConst.dDiagLaserScale ),
   
   /***
   [[U this->ServoErrorDiagOutput.tActuator::tActuator (
         *(GizmoInfo.pDiagLaserServoErrorOutputChannel), 
         GizmoInfo.PhysConst.dDiagLaserServoErrorScale)]]
   ***/
   ServoErrorDiagOutput (*(GizmoInfo.pDiagLaserServoErrorOutputChannel), 
                              GizmoInfo.PhysConst.dDiagLaserServoErrorScale ),
   
   /***
   [[U this->Vc1ServoErrorDiagOutput.tActuator::tActuator (
         *(GizmoInfo.pDiagVc1ServoErrorOutputChannel), 
         GizmoInfo.PhysConst.dDiagVc1ServoErrorScale)]]
   ***/
   Vc1ServoErrorDiagOutput (*(GizmoInfo.pDiagVc1ServoErrorOutputChannel), 
                            GizmoInfo.PhysConst.dDiagVc1ServoErrorScale ),
   
   /***
   [[U this->MotorServoErrorDiagOutput.tActuator::tActuator (
         *(GizmoInfo.pDiagMotorServoErrorOutputChannel), 
         GizmoInfo.PhysConst.dDiagMotorServoErrorScale)]]
   ***/
   MotorServoErrorDiagOutput (*(GizmoInfo.pDiagMotorServoErrorOutputChannel), 
                              GizmoInfo.PhysConst.dDiagMotorServoErrorScale ),
   
   /****
   [[U this->TargetGenerator.tDlTargGen::tDlTargGen (GizmoInfo) ]]
   ****/
   TargetGenerator (GizmoInfo),
   
   /***
   [[U this->LaserTarget.tPositionTargetSource::
       tPositionTargetSource (this->TargetGenerator) ]]
   ***/
   LaserTarget (this->TargetGenerator),

   /***
   [[U this->PztPositionTarget.tZeroTargetSource::tZeroTargetSource (void)]]
   ***/
   PztPositionTarget (),

   /***
   [[U this->MotorTarget.tVelocityTargetSource::
         tVelocityTargetSource (this->TargetGenerator)]]
   ***/
   MotorTarget (TargetGenerator)


{}



