/* $RCSfile: dlMotorController.cc,v $ -*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlMotorController.cc,v 1.2 1998/09/13 19:43:40 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
DESCRIPTION

*/

#include "dl/dlMotorController.h"
#include <math.h>

/***************************************************************
*
* tDlMotorController::tDlMotorController
*
* SPECIFICATION:
[[U tDlMotorController::tDlMotorController (Data, IoSet, InitialParameters, 
    dLaserPhysicalPerCount, iLaserPasses) ]]
****/
tDlMotorController::tDlMotorController (tDlControllerData& Data, tDlCompleteIoSet& IoSet, 
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses) :
     
  /****
  [[U tController::tController () ]]
  ****/
  tController (),
  
  MotorFeedforward (InitialParameters.dMotorFeedforwardGain),

  /****
  [ this->MotorTarget := reference to IoSet.MotorTarget
    : this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->dMotorVelocity := reference to Data.dMotorVelocity
    : this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := InitialParameters.dMotorSamplePeriod
  ]
  ****/
  MotorTarget(IoSet.MotorTarget),
  Limits (IoSet.DlLimits),
  dMotorSamplePeriod (InitialParameters.dMotorSamplePeriod),
  Motor (IoSet.Motor),
  dMotorVelocity (Data.MotorVelocity ()),
  bEnabled (InitialParameters.bEnabled),

  /****
  Convert meters/wave to meters/(count pass)
  
  [[U dTemp := this->Motor.tModulatedVelocityActuator::PhysicalUnitsToCounts (1) ]]
  
  [ dMotorErrorPerLaserError := dLaserScale / (dTemp * iLaserPasses) ]
  ****/
  
  CdMotorErrorPerLaserError (dLaserPhysicalPerCount / (double) iLaserPasses),
  
  /****
  [[U this->CdLaserServoError := Data.dServoError ]]
  ***/
  CdLaserError (Data.ServoError () )


{} 
  
      
/***************************************************************
*
* tDlMotorController::SetControlParameters 
*
* SPECIFICATION:
[[U tDlMotorController::SetControlParameters (NewParameters) ]]
****/
ERROR_CODE tDlMotorController::SetControlParameters (const tDlMotorParameters& NewParameters)
{
  this->bEnabled = NewParameters.bEnabled;
  this->dMotorSamplePeriod = NewParameters.dMotorSamplePeriod;
  this->MotorFeedforward.Initialize(NewParameters.dMotorFeedforwardGain);
  
  return OK;
}    


/***************************************************************
*
* tDlMotorController::MotorVelocity 
*
* SPECIFICATION:
[[U tDlMotorController::MotorVelocity () ]]
****/
double tDlMotorController::MotorVelocity () const
{
  return this->dMotorVelocity;
}

/***************************************************************
*
* tDlMotorController::Init 
*
* SPECIFICATION:
[[U tDlMotorController::Init (tServoTime) ]]
****/
ERROR_CODE tDlMotorController::Init (TIME tServoTime)
{
  return OK;
}


/***************************************************************
*
* tDlMotorController::Reset 
*
* SPECIFICATION:
[[U tDlMotorController::Reset (tServoTime) ]]
****/
ERROR_CODE tDlMotorController::Reset (TIME tServoTime)
{
  /****
  [[U tDlMotorController::Init (tServoTime) ]]
  ****/
  return this->Init (tServoTime);
}



/***************************************************************
*
* tDlMotorController::DecelerationDistance 
*
* SPECIFICATION:
[[U tDlMotorController::DecelerationDistance () ]]
****/
double tDlMotorController::DecelerationDistance () const
{
  double dVel = this->Motor.MaxVelocity ();
  double dAcc = this->Motor.MaxAcceleration ();
  
  return dVel*dVel / (2.0 * dAcc) + dVel * this->dMotorSamplePeriod / 2.0;
}


/***************************************************************
*
* tDlMotorController::Actuate 
*
* SPECIFICATION:
[[U tDlMotorController::Actuate (dMotorVelocity, tServoTime) ]]
****/
ERROR_CODE tDlMotorController::Actuate (double dMotorVelocity, TIME tServoTime)
{
  double dVelocity = dMotorVelocity;
  
  /****
  [[U bTemp1 := this->Limits.tDlLimits::IsOnAnyFrontLimit () ]]
  
  [[U bTemp2 := this->Limits.tDlLimits::IsOnAnyBackLimit () ]]
  
  [ (dMotorVelocity < 0) && bTemp1 => dVelocity := 0
  | (dMotorVelocity > 0) && bTemp2 => dVelocity := 0
  | true => no response
  ]
  ****/
  if ( ((dMotorVelocity < 0) && this->Limits.IsOnAnyFrontLimit () )  ||
       ((dMotorVelocity > 0) && this->Limits.IsOnAnyBackLimit () ) ) dVelocity = 0;

  
  /****
  [[U this->Motor.tModulatedVelocityActuator::OutputValue (tServoTime, dVelocity) ]]
  ****/
  this->Motor.OutputValue (tServoTime, dVelocity);

  /****
  [[U this->dMotorVelocity := this->Motor.tModulatedVelocityActuator::CountsToPhysicalUnits (dVelocity) ]]
  ****/
  this->dMotorVelocity = this->Motor.CountsToPhysicalUnits (this->Motor.Velocity ());
  
  
  return OK;
} 



/***************************************************************
*
* tDlMotorController::MaxMotorAcceleration ()
*
* SPECIFICATION:
[[U tDlMotorController::MaxMotorAcceleration () ]]
****/
double tDlMotorController::MaxMotorAcceleration () const
{
  /****
  [[U this->Motor.tModulatedVelocityActuator::MaxAcceleration () ]]
  ****/
  return this->Motor.MaxAcceleration ();
}



/***************************************************************
*
* tDlMotorSlewController::tDlMotorSlewController
*
* SPECIFICATION:
[[U tDlMotorSlewController::tDlMotorSlewController (Data, IoSet, InitialParameters, 
    dLaserPhysicalPerCount, iLaserPasses) ]]
****/
tDlMotorSlewController::tDlMotorSlewController ( tDlControllerData& Data, tDlCompleteIoSet& IoSet,
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses) :
     
  /****
  [[U tDlMotorController::tDlMotorController (Data, IoSet, InitialParameters) ]]
  ****/
  tDlMotorController (Data, IoSet, InitialParameters, dLaserPhysicalPerCount, iLaserPasses),
  
  dMotorSlewVelocity (this->MotorPhysicalUnitsToMotorCounts (InitialParameters.dMotorSlewVelocity) )
{}
  
      
/***************************************************************
*
* tDlMotorSlewController::SetControlParameters 
*
* SPECIFICATION:
[[U tDlMotorSlewController::SetControlParameters (NewParameters) ]]
****/
ERROR_CODE tDlMotorSlewController::SetControlParameters (const tDlMotorParameters& NewParameters)
{
  this->dMotorSlewVelocity = this->MotorPhysicalUnitsToMotorCounts (NewParameters.dMotorSlewVelocity);
  
  /****
  [[U tDlMotorController::SetControlParameters (NewParameters) ]]
  ****/
  return this->tDlMotorController::SetControlParameters (NewParameters);
}    


/***************************************************************
*
* tDlMotorSlewController::Calculate
*
* SPECIFICATION:
[[U tDlMotorSlewController::Calculate (tServoTime) ]]
****/
ERROR_CODE tDlMotorSlewController::Calculate (TIME tServoTime)
{
  double dVelocity = 0;
  
  /****
  [[U bTemp := tDlMotorSlewController::Enabled () ]]
  [ bTemp => dVelocity := this->dMotorSlewVelocity ]]
  ****/  
  if (this->Enabled ()) {
     dVelocity = this->dMotorSlewVelocity;
  }

  /****
  [[U tDlMotorSlewController::Actuate (dVelocity, tServoTime) ]]
  ****/
  return this->Actuate (dVelocity, tServoTime);
}
 
  
/***************************************************************
*
* tDlMotorTrackSlewController::tDlMotorTrackSlewController
*
* SPECIFICATION:
[[U tDlMotorTrackSlewController::tDlMotorTrackSlewController (
      ControllerData,IoSet, InitialParameters, dLaserPhysicalPerCount, iLaserPasses) ]]
****/
tDlMotorTrackSlewController::tDlMotorTrackSlewController (
      tDlControllerData& ControllerData, tDlCompleteIoSet& IoSet,
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses) :
     
  /****
  [[U tDlMotorSlewController::tDlMotorSlewController (ControllerData, ConIoSet, InitialParameters) ]]
  ****/
  tDlMotorSlewController (ControllerData, IoSet, InitialParameters, dLaserPhysicalPerCount, iLaserPasses),

  /****
  [ this->dDecelerationAggressiveness := InitialParameters.dDecelerationAggressiveness
    : this->dLaserError := reference to ControllerData.dLaserError    
    : this->dMotorServoError := reference to ControllerData.dMotorServoError
  ]
  ****/
  dDecelerationAggressiveness (InitialParameters.dDecelerationAggressiveness)
{}
  
      
/***************************************************************
*
* tDlMotorTrackSlewController::SetControlParameters 
*
* SPECIFICATION:
[[U tDlMotorTrackSlewController::SetControlParameters (NewParameters) ]]
****/
ERROR_CODE tDlMotorTrackSlewController::SetControlParameters (const tDlMotorParameters& NewParameters)
{
  this->dDecelerationAggressiveness = NewParameters.dDecelerationAggressiveness;
  
  /****
  [[U tDlMotorSlewController::SetControlParameters (NewParameters) ]]
  ****/
  return this->tDlMotorSlewController::SetControlParameters (NewParameters);
}    


/***************************************************************
*
* tDlMotorSlewController::Init 
*
* SPECIFICATION:
[[U tDlMotorSlewController::Init (tServoTime) ]]
****/
ERROR_CODE tDlMotorSlewController::Init (TIME tServoTime)
{

  return OK;
}


/***************************************************************
*
* tDlMotorTrackSlewController::Calculate
*
* SPECIFICATION:
[[U tDlMotorTrackSlewController::Calculate (tServoTime) ]]
****/
ERROR_CODE tDlMotorTrackSlewController::Calculate (TIME tServoTime)
{
  double dMotorFeedforward, dVelocity = 0;
  
  
  /****
  [[U bTemp := tDlMotorTrackSlewController::Enabled () ]]
  [ bTemp => dVelocity := tDlMotorTrackSlewController::ProfiledVelocity () ]]
  ****/  
  if (this->Enabled ()) {

     dVelocity = this->ProfiledVelocity ();

     // Add in Feedforward.  It is important to do this in slew in
     // order to get a smooth transition to track at high target speeds

     dMotorFeedforward = this->MotorFeedforward.Calculate (
       this->MotorPhysicalUnitsToMotorCounts(
         this->LaserCountsToMotorPhysical (
           ((double) TIME_TICKS_PER_SECOND) * this->MotorTarget.Update (tServoTime)
       )));

     dVelocity += dMotorFeedforward;
  }

    
  /****
  [[U tDlMotorTrackSlewController::Actuate (dVelocity, tServoTime) ]]
  ****/
  return this->Actuate (dVelocity, tServoTime);
}
 
  

/***************************************************************
*
* tDlMotorTrackSlewController::ProfiledVelocity
*
* SPECIFICATION:
[[U tDlMotorTrackSlewController::ProfiledVelocity () ]]
****/
double tDlMotorTrackSlewController::ProfiledVelocity () const
{
  double dVelocity, dError, dAdjustedError;


  /****
  [[U dError  := tDlMotorTrackSlewController::LinearCartError () ]]
  ****/
  dError = this->LinearCartError ();
  
  /****
  dAdjustedError := |dError| * dDecelerationAggressiveness
  ****/
  dAdjustedError = fabs (dError) * dDecelerationAggressiveness;


  /****
  [[U dStoppingDistance := tDlMotorTrackSlewController::DecelerationDistance ()  ]]
  ****/
  double dStoppingDistance = this->DecelerationDistance ();


  /****
  [[U dSamplePeriod := tDlMotorTrackSlewController::MotorSamplePeriod () ]]
  ****/
  double dSamplePeriod = (double) this->MotorSamplePeriod ();

  /****
  [[U dMaxAcc := tDlMotorTrackSlewController::MaxMotorAcceleration ()   ]]
  ****/
  double dMaxAcc = this->MaxMotorAcceleration ();
   
  if (dAdjustedError > dStoppingDistance) dVelocity = this->MaxMotorVelocity ();
  else dVelocity = -1 * dMaxAcc * dSamplePeriod / 2.0 +
                    sqrt (dMaxAcc * dMaxAcc * dSamplePeriod * dSamplePeriod + 
                          8 * dMaxAcc * dAdjustedError) / 2.0;
                       
  // go in correct direction                          
  if (dError < 0)  dVelocity = - dVelocity;                         
                          
  return this->MotorPhysicalUnitsToMotorCounts (dVelocity);                         
}
 


