/* $RCSfile: dlPztController.cc,v $*/
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlPztController.cc,v 1.5 1998/09/23 23:57:07 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
05a,29Jan98,rlj   written
*/
 
/*
DESCRIPTION
  Delay Line PZT Controllers.
*/
  

#include "dl/dlPztController.h"
#include "recLibB.h"



/*******************************************************************
*
* tDlPztController::tDlPztController
*
* DESCRIPTION:
[[U2 tDlPztController::tDlPztController (ControllerData, CompleteIoSet) ]]
*
[[U3 tDlPztController::tDlPztController (ControllerData, CompleteIoSet) ]]
*/
tDlPztController::tDlPztController (tDlControllerData& ControllerData,
                                    tDlCompleteIoSet& CompleteIoSet) :

  /****
  Initialize all references...
  
  [ this->LaserInput := reference to CompleteIoSet.Laser 
    : this->LaserInput := reference to CompleteIoSet.LaserDiagInput 
    : this->Pzt := reference to CompleteIoSet.Pzt 
    : this->LaserDiagOutput := reference to CompleteIoSet.LaserDiagOutput
    : this->ServoErrorDiagOutput := reference to CompleteIoSet.ServoErrorDiagOutput
    : this->LaserTarget := reference to CompleteIoSet.LaserTarget  
    : this->dServoError := reference to ControllerData.dServoError
  ]
  ****/
  iCurrentWriteBuffer(0),

  bEnabled (TRUE),
  
  LaserInput (CompleteIoSet.Laser),

  LaserDiagInput (CompleteIoSet.LaserDiagInput),

  Pzt (CompleteIoSet.Pzt),

  LaserDiagOutput (CompleteIoSet.LaserDiagOutput),

  ServoErrorDiagOutput (CompleteIoSet.ServoErrorDiagOutput),

  LaserTarget  (CompleteIoSet.LaserTarget),

  dServoError (ControllerData.ServoError ())
  
{}




/*******************************************************************
*
* tDlPztController::Init
*
* DESCRIPTION:
[[U2 tDlPztController::Init (tUpdateTime) ]]
*
[[U3 tDlPztController::Init (tUpdateTime)  ]]
*/
ERROR_CODE tDlPztController::Init (TIME tUpdateTime) 
{
  /****
  [[U this->Pzt.tModulatedVelocityActuator::Reset () ]]
  
  ****/
  return this->Pzt.Reset ();
}

/*******************************************************************
*
* tDlPztController::Reset
*
* DESCRIPTION:
[[U2 tDlPztController::Reset (tUpdateTime) ]]
*
[[U3 tDlPztController::Reset (tUpdateTime)  ]]
*/
ERROR_CODE tDlPztController::Reset (TIME tUpdateTime) 
{
  /****
  [[U this->Pzt.tModulatedVelocityActuator::Reset () ]]
  ****/
  return this->Pzt.Reset ();
}


/*******************************************************************
*
* tDlPztController::Calculate
*
* DESCRIPTION:
[[U2 tDlPztController::Calculate (tServoTime) ]]
*
[[U3 tDlPztController::Calculate (tServoTime)  ]]
*/
ERROR_CODE tDlPztController::Calculate (TIME tServoTime) 
{

  /****
  Base class controller only computes the servo error 

  [[U tDlPztController::ServoError (tServoTime) ]]
  ****/
  this->ServoError (tServoTime);
  
  return OK;
}


/******* Boolean for ignoring laser value *************
********  used for servo tuning of VC's & Motor ******/
BOOL bIgnoreLaser = FALSE;

/*******************************************************************
*
* tDlPztController::ServoError
*
* DESCRIPTION:
[[U2 tDlPztController::ServoError (tServoTime) ]]
*
[[U3 tDlPztController::ServoError (tServoTime)  ]]
*/
double tDlPztController::ServoError (TIME tServoTime) 
{

  double dLaserValue;
  double dUnmodulatedLaserValue;
  
  /****
  Read current delay line position from internal metrology
  
  [[U dLaserValue :=  this->LaserInput.tLaserMetrologySensor::Update (tServoTime)  ]]
  ****/
  dLaserValue = this->LaserInput.Update (tServoTime);
  dUnmodulatedLaserValue = this->LaserInput.UnmodulatedValue ();
  if(bIgnoreLaser) dLaserValue = 0.0;


  /****
  Compute servo error from position, diagnostic, and target values
  
  [[U Diag :=  this->LaserDiagInput.tSensor::Update (tServoTime)  ]]
  
  [[U Target :=  this->LaserTarget.tTargetSource::Update (tServoTime)  ]]
  
  [ this->dServoError := Input  - Target + Diag]
  ****/
  this->dServoError = dLaserValue - this->LaserTarget.Update (tServoTime) +
       this->LaserDiagInput.Update (tServoTime);
  
  
  #ifdef SERVO_TUNING_ON

  recInt((int)dLaserValue);

  /****
  Send servo error to diagnostic output
  [[U this->LaserDiagOutput.tActuator::OutputValue (tServoTime, dLaserValue) ]]
  ****/
  this->LaserDiagOutput.OutputValue (tServoTime,16.0*dUnmodulatedLaserValue);

  /****
  [[U this->ServoErrorDiagOutput.tActuator::OutputValue (tServoTime, this->ServoError) ]]
  ****/
  this->ServoErrorDiagOutput.OutputValue (tServoTime, 16.0*this->dServoError);

  #endif /* SERVO_TUNING_ON */

  return this->dServoError;

}


/*******************************************************************
*
* tDlPztController::Actuate
*
* DESCRIPTION:
[[U2 tDlPztController::Actuate (tUpdateTime, dPztVelocity) ]]
*
[[U3 tDlPztController::Actuate (tUpdateTime, dPztVelocity) ]]
*/
ERROR_CODE tDlPztController::Actuate (TIME tUpdateTime, double dPztVelocity) 
{

  /****
  [[U this->Pzt.tModulatedVelocityActuator::OutputValue (tUpdateTime, dPztVelocity) ]]
  ****/
  return this->Pzt.OutputValue (tUpdateTime, dPztVelocity);
}


/*******************************************************************
*
* tDlPztController::LaserData
*
* DESCRIPTION:
*
[[U5 tDlPztController::LaserData (tServoTime) ]]
*/
void tDlPztController::LaserData (TIME tServoTime)
{
  FAST int iSampleNumber = (tServoTime / (TIME_TICKS_PER_SECOND/DL_SERVO_RATE)) %
                        DL_DATA_BUF_SIZE;
  
  this->DlData[this->iCurrentWriteBuffer].ServoDataArray[iSampleNumber].laLaser =
        (LASER_VALUE) this->LaserInput.Value ();
  this->DlData[this->iCurrentWriteBuffer].ServoDataArray[iSampleNumber].laError =
        (LASER_VALUE) this->dServoError;
}


/*******************************************************************
*
* tDlPztTrackController::tDlPztTrackController
*
* DESCRIPTION:
[[U2 tDlPztTrackController::tDlPztTrackController (ControllerData, CompleteIoSet, 
         InitialParameters) ]]
*
[[U3 tDlPztTrackController::tDlPztTrackController (ControllerData,CompleteIoSet, InitialParameters) ]]
*/
tDlPztTrackController::tDlPztTrackController (tDlControllerData& ControllerData,
     tDlCompleteIoSet& IoSet, const tDlPztParameters& InitialParameters) :

  /****
  Construct base class
  
  [[U tDlPztController::tDlPztController (ControllerData, IoSet) ]]
  ****/
  tDlPztController (ControllerData, IoSet),
  
  // Initialize additional variables
  
  bLagEnabled (InitialParameters.bLagEnabled),
  
  dPztGain (InitialParameters.dPztGain),
  
  /****
  [[U PztLag.tLagCompensator::tLagCompensator_ddd (InitialParameters.dLagTauP, InitialParameters.dLagTauZ, 
        InitialParameters.dSamplePeriod) ]]
  ****/
  PztLag (InitialParameters.dLagTauP, InitialParameters.dLagTauZ, InitialParameters.dSamplePeriod),
  
  LaserDataStore (ControllerData.LaserDataStore)
  
{

  this->Enabled () = InitialParameters.bEnabled;
}


/*******************************************************************
*
* tDlPztTrackController::SetControlParameters
*
* DESCRIPTION:
[[U2 tDlPztTrackController::SetControlParameters (NewParameters) ]]
*
[[U3 tDlPztTrackController::SetControlParameters (NewParameters) ]]
*/
ERROR_CODE tDlPztTrackController::SetControlParameters (const tDlPztParameters& NewParameters) 
{

  this->Enabled () = NewParameters.bEnabled;
  this->bLagEnabled = NewParameters.bLagEnabled;
  this->dPztGain = NewParameters.dPztGain;
    
  /****
  [[U PztLag.tLagCompensator::tLagCompensator_ddd (InitialParameters.dLagTauP, InitialParameters.dLagTauZ, 
        InitialParameters.dSamplePeriod) ]]
  ****/
  this->PztLag.Initialize (NewParameters.dLagTauP, NewParameters.dLagTauZ, NewParameters.dSamplePeriod);
  
  return OK;
}


/*******************************************************************
*
* tDlPztTrackController::ControlParameters
*
* DESCRIPTION:
[[U2 tDlPztTrackController::ControlParameters () ]]
*
[[U3 tDlPztTrackController::ControlParameters () ]]
*/
tDlPztParameters tDlPztTrackController::ControlParameters () const
{
  /****
  [[U taup := this->PztLag.tLagCompensator::TauP () ]]
  
  [[U taup := this->PztLag.tLagCompensator::TauZ () ]]
  
  [[U taup := this->PztLag.tLagCompensator::SamplePeriod () ]]
  
  [[U Temp.tDlPztParameters::tDlPztParameters_bbdddd (this->bEnabled, 
        this->bLagEnabled,this->dPztGain, taup, tauz, sampleperiod) ]]
  ****/
  return tDlPztParameters (this->Enabled (), this->bLagEnabled, this->dPztGain, this->PztLag.TauP (), 
                           this->PztLag.TauZ (), this->PztLag.SamplePeriod () );
}








/*******************************************************************
*
* tDlPztTrackController::LagCompensator
*
* DESCRIPTION:
[[U2 tDlPztTrackController::LagCompensator () ]]
*
[[U3 tDlPztTrackController::LagCompensator ()  ]]
*/
const tLagCompensator& tDlPztTrackController::LagCompensator () const
{
  return this->PztLag;
}


/*******************************************************************
*
* tDlPztTrackController::Init
*
* DESCRIPTION:
[[U2 tDlPztTrackController::Init (tUpdateTime) ]]
*
[[U3 tDlPztTrackController::Init (tUpdateTime)  ]]
*/
ERROR_CODE tDlPztTrackController::Init (TIME tUpdateTime) 
{
  /****
  [[U this->PztLag.tLagCompensator::Reset (0.0) ]]
  ****/  
  this->PztLag.Reset (0.0);
  
  /****
  [[U tDlPztController::Init (tUpdateTime) ]]
  ****/  
  return this->tDlPztController::Init (tUpdateTime);
}
  
/*******************************************************************
*
* tDlPztTrackController::Reset
*
* DESCRIPTION:
[[U2 tDlPztTrackController::Reset (tUpdateTime) ]]
*
[[U3 tDlPztTrackController::Reset (tUpdateTime)  ]]
*/
ERROR_CODE tDlPztTrackController::Reset (TIME tUpdateTime) 
{
  /****
  [[U this->PztLag.tLagCompensator::Reset (0.0) ]]
  ****/  
  this->PztLag.Reset (0.0);
  
  /****
  [[U tDlPztController::Init (tUpdateTime) ]]
  ****/  
  return this->tDlPztController::Reset (tUpdateTime);
}

  
 /*******************************************************************
*
* tDlPztTrackController::Calculate
*
* DESCRIPTION:
[[U2 tDlPztTrackController::Calculate (tServoTime) ]]
*
[[U3 tDlPztTrackController::Calculate (tServoTime)  ]]
*/
ERROR_CODE tDlPztTrackController::Calculate (TIME tServoTime) 
{
  double dServoError, dPztVelocity;
  
  /****
  Compute current delay line position error
  
  [[U dServoError := tDlPztController::ServoError (tServoTime) ]]
  ****/
  dServoError = this->ServoError (tServoTime);
  
  // depending on the enable states compute the compensation for the PZT
 

  if (this->Enabled () ) {
    if (this->bLagEnabled) {
  
      /****
      Pzt velocity using gain and lag compensator 
      
      [[U dLag := this->PztLag.tLagCompensator::Calculate (dServoError) ]]
    
      [ dPztVelocity := this->dPztGain * dLag ]
      ****/
      dPztVelocity = this->dPztGain * this->PztLag.Calculate (dServoError);
    }
  
    // no lag compensation 
        
    else dPztVelocity = this->dPztGain * dServoError;
  }
  // no control
  
  else dPztVelocity = 0.0;
  
  /****
  Actuate PZT using computed compensation
  
  [[U tDlPztTrackController::Actuate (tServoTime, dPztVelocity) ]]
  ****/
  this->Actuate (tServoTime, dPztVelocity);
  
   
  /****
  [[U Data := tDlPztTrackController::LaserData (tServoTime) ]]
  ****/
  this->LaserData(tServoTime);

  /****
  [[U this->LaserDataStore.tDlDataStore::StoreData (Data) ]]
  ****/
  if (((tServoTime / (TIME_TICKS_PER_SECOND/DL_SERVO_RATE)) %
            DL_DATA_BUF_SIZE) == 0) {
    this->DlData[this->iCurrentWriteBuffer].tTimeStamp = tServoTime;
    this->LaserDataStore.StoreData (this->DlData[this->iCurrentWriteBuffer]);
    if (this->iCurrentWriteBuffer == 0)  this->iCurrentWriteBuffer = 1;
    else                                 this->iCurrentWriteBuffer = 0;
  }
 
  return OK;   
}
