/* $RCSfile: dlStatus.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlStatus.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,24Sep97,rlj      
01a,21Jun97,beh    adapted form dlTypes.h
*/
      
/*
DESCRIPTION
  Routines for manipulating tDlStatus structures.
*/

#include "isit.h"
#include "dlTypes.h"
#include "dlDither.h"


/**************************************************************************
*
* tDlStatus::tDlStatus
*
  [[U tDlStatus::tDlStatus (iDlNumber) ]]
*/

tDlStatus::tDlStatus (int iDlNumber)
{
  tModulationParams ModulationParams;
  tDitherCalibInfo CalibInfo;
  
  /* updated after this by various DL cmd engines */
  this->UpdateCommandStatus(DLCS_SYSTEM_READY);
  this->UpdateServoState(DSM_OFF);

  CalibInfo.dAmplitude  = 0.0;
  CalibInfo.dPhaseShift = 0.0;
  CalibInfo.dMetric     = 0.0;
  
  this->UpdateDitherCalibration(CalibInfo);
  
  this->bDoingDitherCheckOrCalib = FALSE; 
  this->UpdateModulation(ModulationParams);
  // this->UpdateTrackServoParams(...);     // do init in Gizmo constructor
  
  /* updated after this by DL data relayer */
  this->UpdateJitter(0.0, 0.0);
  
  /* updated after this by DL status relayer */
  this->UpdateLimits(FALSE,FALSE,FALSE,FALSE,FALSE);
  this->UpdateServoInfo(0.0,0.0,0,0,0,0.0,0.0,0);
  this->UpdateAdditionalGsuServoInfo(0,0,0);
  
  this->tDlStatusUnix::Initialize (iDlNumber); 
}




/**************************************************************************
*
* tDlStatus::UpdateCommandStatus
*
 [[U tDlStatus::UpdateCommandStatus (dcsNewCommandStatus) ]]
*/

void tDlStatus::UpdateCommandStatus(DL_COMMAND_STATUS dcsNewCommandStatus)
{
  this->DlCommandStatus = dcsNewCommandStatus;
}


/**************************************************************************
*
* tDlStatus::UpdateServoState
*
  [[U tDlStatus::UpdateServoState (dsmNewServoMode) ]]
*/

void tDlStatus::UpdateServoState(DL_SERVO_MODE dsmNewServoMode)
{
  this->dsmServoMode = dsmNewServoMode;
}


/**************************************************************************
*
* tDlStatus::UpdateLimits
*
  [[U tDlStatus::UpdateLimits (bNewLimitFrontSoft,bNewLimitFrontHard,
                  bNewLimitBackSoft,bNewLimitBackHard,bNowOnHomeSensor) ]]
*/

void tDlStatus::UpdateLimits (BOOL bNewLimitFrontSoft, BOOL bNewLimitFrontHard,
                  BOOL bNewLimitBackSoft,  BOOL bNewLimitBackHard,
                  BOOL bNowOnHomeSensor)
{
  this->bLimitFrontSoft = bNewLimitFrontSoft;
  this->bLimitFrontHard = bNewLimitFrontHard;
  this->bLimitBackSoft  = bNewLimitBackSoft;
  this->bLimitBackHard  = bNewLimitBackHard;  
  this->bOnHomeSensor   = bNowOnHomeSensor;
}


/**************************************************************************
*
* tDlStatus::UpdateJitter
*
  [[U tDlStatus::UpdateJitter (dJitter, dHighPassJitter) ]]
*/

void tDlStatus::UpdateJitter(double dJitter, double dHighPassJitter)
{
  this->dJitter = dJitter;
  this->dHighPassJitter = dHighPassJitter;
}


/**************************************************************************
*
* tDlStatus::UpdateModulation
*
  [[U tDlStatus::UpdateModulation (Params) ]]
*/

void tDlStatus::UpdateModulation(const tModulationParams &Params)
{
  this->SensorModulation = Params;
}


/**************************************************************************
*
* tDlStatus::UpdateServoInfo
*
  [[U tDlStatus::UpdateServoInfo (laLaserValue,pdPztPos,pvPztVelocity,
                     vvVc1,mvMotorVelocity,laTargetValue, laErrorValue,
                     iPztClip) ]]
*/

void tDlStatus::UpdateServoInfo(LASER_VALUE laLaserValue,
                     PZT_DOUBLE pdPztPos, PZT_VALUE pvPztVelocity,
                     VC_VALUE vvVc1,
                     MOTOR_VALUE mvMotorVelocity,
                     LASER_VALUE laTargetValue, LASER_VALUE laErrorValue,
                     int iPztClip)
{
  this->laLaser         = laLaserValue;
  this->pdPztPos        = pdPztPos;
  this->pvPztVelocity   = pvPztVelocity;
  this->vvVc1           = vvVc1;
  this->mvMotorVelocity = mvMotorVelocity;
  this->laTarget        = laTargetValue;
  this->laError         = laErrorValue;
  this->iPztClip        = iPztClip;
}


/**************************************************************************
*
* tDlStatus::UpdateAdditionalGsuServoInfo
*
  [[U tDlStatus::UpdateAdditionalGsuServoInfo (evEddy1, evEddy2, vvVc2) ]]
*/

void tDlStatus::UpdateAdditionalGsuServoInfo(EDDY_VALUE evEddy1,
                                             EDDY_VALUE evEddy2,
                                             VC_VALUE vvVc2)
{
  this->evEddy1         = evEddy1;
  this->evEddy2         = evEddy2;
  this->vvVc2           = vvVc2;
}


/**************************************************************************
*
* tDlStatus::UpdateDitherCalibration
*
  [[U tDlStatus::UpdateDitherCalibration (CalibInfo) ]]
*/

void tDlStatus::UpdateDitherCalibration(const tDitherCalibInfo &CalibInfo)
{
  this->dDitherAmplitude  = CalibInfo.dAmplitude;
  this->dDitherPhase      = CalibInfo.dPhaseShift;
  this->dDitherDistortion = CalibInfo.dMetric;
}


/**************************************************************************
*
* tDlStatus::SetDitherCheckOrCalib
*
  [[U tDlStatus::UpdateDitherCheckOrCalib (bNowDoingCheckOrCalib) ]]
*/

void tDlStatus::UpdateDitherCheckOrCalib(BOOL bNowDoingCheckOrCalib) 
{
  this->bDoingDitherCheckOrCalib = bNowDoingCheckOrCalib;
}


