/* $RCSfile: dlTargGen.cc,v $ - Target Generator source file */
/* CM Info:

$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/dlTargGen.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,eamck,22Apr97    written (Incr 2 version)
*/
 
/*
DESCRIPTION

*/



#include "isit.h"
#include "rte.h"
#include "dlGizmoInfo.h"
#include "dlGizmo.h"
#include "dlTargGen.h"
#include "Scheduler.h"

/******************************************************************
*
* tDlTarget::tDlTarget(void)
*
******/
tDlTarget::tDlTarget(void)
{
    this->dPosition = 0.0;
    this->dRate = 0.0;
    this->tTime = 0;
}

/******************************************************************
*
* tDlTarget::tDlTarget(...)
*
******/
tDlTarget::tDlTarget(double dInitPos, double dInitRate, TIME tInitTime)
{
    this->dPosition = dInitPos;
    this->dRate = dInitRate;
    this->tTime = tInitTime;
}


/******************************************************************
*
* tDlTargGen::tDlTargGen
*
* SPECIFICATION:
[[U tDlTargGen::tDlTargGen (GizmoInfo) ]]
*****/
tDlTargGen::tDlTargGen (const tDlGizmoInfo& GizmoInfo)
{
   int         i;
   ERROR_CODE  ec;


   /****
   [[U iPriority := GizmoInfo.tGizmoInfo::TargetGeneratorPriority () ]]
   ****/
   int iPriority = GizmoInfo.TargetGeneratorPriority ();
   
   /****
   [[U tPeriod := GizmoInfo.tDlGizmoInfo::TargetGeneratorPeriod () ]]
   ****/
   TIME tPeriod = GizmoInfo.TargetGeneratorPeriod ();

   /****
   [[U tOffset := GizmoInfo.tDlGizmoInfo::TargetGeneratorOffset () ]]
   ****/
   TIME	tOffset = GizmoInfo.TargetGeneratorOffset ();  

   const tDlTarget CdtZeroTarget;


   TraceMark (S_dlTargGen_ENTERING_DL_TARGGEN_INIT);

   this->dtCurrentTarget.dPosition = CdtZeroTarget.dPosition;
   this->dtCurrentTarget.dRate = CdtZeroTarget.dRate;
   this->dtCurrentTarget.tTime = CdtZeroTarget.tTime;

   this->pTargetStreamArray = 
     new (tIpcDblBufferAccessor<tDlTarget> *) [NUM_DELAY_LINE_TARGETS];
   
   if (this->pTargetStreamArray == NULL) {
     TraceMsg (tmt_ERROR, S_dlTargGen_CANT_ALLOCATE_TARGETS_STREAMS,
           "tDlTargGen::Initialize()");
     return; 
   }
            
   for (i=0; i<NUM_DELAY_LINE_TARGETS; i++)
   {
      this->bTargetSelectionArray[i] = FALSE;
      
      this->pTargetStreamArray[i] = 
        new tIpcDblBufferAccessor<tDlTarget> (GizmoInfo.GetTargetId (i));
        
    if (this->pTargetStreamArray[i] == NULL) {
      TraceMsg (tmt_ERROR, S_dlTargGen_CANT_ALLOCATE_TARGETS_STREAMS,
                "tDlTargGen::Initialize()");
      return;
    }

    ec = this->pTargetStreamArray[i]->GetInitError ();
    
    if (ec != OK) {
      TraceMsg (tmt_ERROR, ec,"tDlTargGen::Initialize");
      return;
    }

      /* put something in buffer so it won't be empty */
     ec = this->pTargetStreamArray[i]->Put(this->dtCurrentTarget);

   }

   TraceMark(S_dlTargGen_ALL_TARGET_SOURCES_INITED);

   ec = GetScheduler ().AddTask( &(this->iTargGenTaskId),
	  "TargGenTask", (FUNCPTR) &(this->ComputeTotalTarget),
	  NULL, NULL, (void *)this, iPriority, &tPeriod, &tOffset, 
	  TEB_CONTINUE, TRUE );

   if (ec!=OK) TraceMsg (tmt_ERROR, ec,"tDlTargGen::Initialize");

   TraceMark(S_dlTargGen_LEAVING_DL_TARGGEN_INIT);


}




/******************************************************************
*
* tDlTargGen::~tDlTargGen(void) - destructor
*
* Removes the targ gen task from the scheduler
*       (prob. not used in Incr 2)
* Note that there is no warning if DeleteTask() doesn't succeed
*

*****/
tDlTargGen::~tDlTargGen(void) 
{
   tScheduler &Scheduler = GetScheduler();

   Scheduler.DeleteTask(this->iTargGenTaskId);

}   /* tDlTargGen::~tDlTargGen() */




/******************************************************************
*
* ERROR_CODE tDlTargGen::GetCurrentTargetTriple(tDlTarget *pCurrentTarget)
*
* SPEC:
[[U tDlTargGen::GetCurrentTargetTriple() ]]
*****/
ERROR_CODE tDlTargGen::GetCurrentTargetTriple(tDlTarget *pCurrentTarget)
{
   pCurrentTarget->dPosition = dtCurrentTarget.dPosition;
   pCurrentTarget->dRate = dtCurrentTarget.dRate;
   pCurrentTarget->tTime = dtCurrentTarget.tTime;
   
   return OK;

}   /* tDlTargGen::GetCurrentTargetTriple() */


/******************************************************************
*
* double tDlTargGen::GetCurrentTargetPosition(TIME tCurrTime) 
*
* SPEC:
*
* RETURNS:
*    current target position based on tCurrTime
*
*****/

double tDlTargGen::GetCurrentTargetPosition(TIME tCurrTime) 
{
   return ( this->dtCurrentTarget.dPosition + (this->dtCurrentTarget.dRate * 
                                  (tCurrTime - this->dtCurrentTarget.tTime)) );

}   /* tDlTargGen::GetCurrentTargetPosition() */


/******************************************************************
*
* double tDlTargGen::GetCurrentTargetVelocity(void)
*
* SPEC:
[[U tDlTargGen::GetCurrentTargetVelocity() ]]
*
* RETURNS:
*    current target rate
*
*****/
double tDlTargGen::GetCurrentTargetVelocity(void)
{
   return this->dtCurrentTarget.dRate;

}   /* tDlTargGen::GetCurrentTargetVelocity() */



/******************************************************************
*
* ERROR_CODE tDlTargGen::SetAllActiveTargets(BOOL *pbTargetSelections)
*
* SPEC:
*
* RETURNS:
*
*
*****/
ERROR_CODE tDlTargGen::SetAllActiveTargets(BOOL *pbTargetSelections)
{
   int i;

   for (i=0; i<NUM_DELAY_LINE_TARGETS; i++)
      this->bTargetSelectionArray[i] = pbTargetSelections[i];

   return OK;      

}   /* tDlTargGen::SetAllActiveTargets() */


/******************************************************************
*
* ERROR_CODE tDlTargGen::SetActiveTarget(int iWhichTarget, BOOL bTurnOn)
*
* SPEC:
*****/
ERROR_CODE tDlTargGen::SetActiveTarget(int iWhichTarget, BOOL bTurnOn)
{
   this->bTargetSelectionArray[iWhichTarget] = bTurnOn;

   return OK;

}   /* tDlTargGen::SetActiveTarget() */


/******************************************************************
*
* ERROR_CODE tDlTargGen::ComputeTotalTarget(void)
*
* SPEC:
*****/
ERROR_CODE tDlTargGen::ComputeTotalTarget(void)
{
   int        i;
   ERROR_CODE ec;
   tDlTarget  dtNewTarget;
   double     dPositionSum;
   double     dRateSum;
   double dRatePerTick;
   TIME       tCurrTime;
   tIpc       &Ipc = GetIpc();

   TraceMark(S_dlTargGen_COMPUTE_TOTAL_TARGET_TASK);

   dPositionSum = 0.0;
   dRateSum = 0.0;
   tCurrTime = GetTime();

   for (i=0; i<NUM_DELAY_LINE_TARGETS; i++)
      if (this->bTargetSelectionArray[i])
      {
	 ec = this->pTargetStreamArray[i]->Get(dtNewTarget);
	 if (ec != OK)
	    return Traceback(ec,"tDlTargGen::ComputeTotalTarget");
 
 

         // Convert rate from per-second units to per-timetick units
         dRatePerTick = dtNewTarget.dRate / ((double) TIME_TICKS_PER_SECOND);

         /* project position to time tCurrTime */
	 dPositionSum += dtNewTarget.dPosition + 
                         (dRatePerTick * (tCurrTime - dtNewTarget.tTime));

	 dRateSum += dRatePerTick;
      }

   #ifndef INCR1
     #error THESE OPERATIONS NEED TO BE ATOMIC TO ASSURE GOOD SERVO PERFORMANCE  --BEH
   #endif
   this->dtCurrentTarget.dPosition = dPositionSum; 
   this->dtCurrentTarget.dRate = dRateSum; 
   this->dtCurrentTarget.tTime = tCurrTime;

   return OK;

}   /* tDlTargGen::ComputeTotalTarget() */

