/* $RCSfile: limits.cc,v $ - dlServo inmplementation */
/* CM Info:

$Header: /home/chara/cvsroot/ople/sys/gizmo/dl/limits.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $

*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,20Oct96    written
*/
 
/*
DESCRIPTION

*/

#include "dl/limits.h"
#include "Scheduler.h"
#include "errorStream.h"
#include "dlErrors.h"


/******************************************************************
*
* tDlLimits::tDlLimits
*
* SPECIFICATION:
[[U tDlLimits::tDlLimits (FrontHard, dFrontHardScale,
               FrontSoft, dFrontSoftScale, BackHard, dBackHardScale,
               BackSoft, dBackSoftScale, Home, dHomeScale) ]]
****/
tDlLimits::tDlLimits (tInputChannel& FrontHard, double dFrontHardScale,
               tInputChannel& FrontSoft, double dFrontSoftScale,
               tInputChannel& BackHard, double dBackHardScale,
               tInputChannel& BackSoft, double dBackSoftScale,
               tInputChannel& Home, double dHomeScale) :
               
  /****
  [[U this->FrontHardLimit.tSensor::tSensor_td (FrontHard, dFrontHardScale) ]]
  ****/               
  FrontHardLimit (FrontHard, dFrontHardScale),
               
  /****
  [[U this->FrontSoftdLimit.tSensor::tSensor_td (SoftHard, dFrontSoftScale) ]]
  ****/               
  FrontSoftLimit (FrontSoft, dFrontSoftScale),
               
  /****
  [[U this->BackHardLimit.tSensor::tSensor_td (BackHard, dBacktHardScale) ]]
  ****/               
  BackHardLimit (BackHard, dBackHardScale),
               
  /****
  [[U this->BackSoftLimit.tSensor::tSensor_td (BackSoft, dBackSoftScale) ]]
  ****/               
  BackSoftLimit (BackSoft, dBackSoftScale),
               
  /****
  [[U this->HomeSensor.tSensor::tSensor_td (Home, dHomeScale) ]]
  ****/               
  HomeSensor (Home, dHomeScale),
  
  bFrontHardLimit (FALSE),
  bFrontSoftLimit (FALSE),
  bBackHardLimit (FALSE),
  bBackSoftLimit (FALSE),
  bHome (FALSE)
{
  ERROR_CODE ec;
  

  TIME tTaskPeriod = DL_READ_LIMIT_TASK_PERIOD;
  TIME tTaskOffset = DL_READ_LIMIT_TASK_OFFSET;
  

  /****
  [[U Scheduler.tScheduler::AddTask (&(this->iUpdateCachedLimitsTaskId),
         "ReadLimitsTask", (FUNCPTR) &(this->UpdateCachedLimitValues),
          NULL, NULL, (void *) this, DL_READ_LIMIT_TASK_PRIORITY,
          &tTaskPeriod, &tTaskOffset, TEB_DISABLE, TRUE) ]]
  ****/          
  ec = GetScheduler().AddTask (&(this->iUpdateCachedLimitsTaskId),
         "ReadLimitsTask", (FUNCPTR) &(this->UpdateCachedLimitValues),
          NULL, NULL, (void *) this, DL_READ_LIMIT_TASK_PRIORITY,
          &tTaskPeriod, &tTaskOffset, TEB_DISABLE, TRUE);


}



  
/******************************************************************
*
* tDlLimits::~tDlLimits
*
* SPECIFICATION
[[U tDlLimits::~tDlLimits () ]]
*/
tDlLimits::~tDlLimits ()
{
  if (this->iUpdateCachedLimitsTaskId != NULL) {
    /****
    [[U Scheduler.tScheduler::DeleteTask (this->iUpdateCachedLimitsTaskId) ]]
    ****/
    GetScheduler().DeleteTask (this->iUpdateCachedLimitsTaskId);
  }

  // CONCURRENCY ISSUE!!  We should wait until the task has actually been
  // deleted before allowing the object to be deleted.
}


/******************************************************************
*
* tDlLimits::UpdateCachedLimitValue 
*
* SPECIFICATION:
[[U tDlLimits::UpdateCachedLimitValues (tTimeNow) ]]
****/
ERROR_CODE tDlLimits::UpdateCachedLimitValues (TIME tTimeNow)
{
  

  BOOL bOnALimitNow;
  
  /****
  [[U WasOnAnyLimit := tDlLimits::IsOnAnyLimit () ]]
  ****/
  BOOL WasOnAnyLimit = this->IsOnAnyLimit();

  /****
  [[U this->bFrontHardLimit := this->FrontHardLimit.tSensor::Update (tTimeNow) ]]
  ****/
  this->bFrontHardLimit = (BOOL) this->FrontHardLimit.Update(tTimeNow);

  /****
  [[U this->bFrontSoftLimit := this->FrontSoftLimit.tSensor::Update (tTimeNow) ]]
  ****/
  this->bFrontSoftLimit = (BOOL) this->FrontSoftLimit.Update(tTimeNow);

  /****
  [[U this->bBackSoftLimit := this->BackSoftLimit.tSensor::Update (tTimeNow) ]]
  ****/
  this->bBackSoftLimit  = (BOOL) this->BackSoftLimit.Update(tTimeNow);

  /****
  [[U this->bbBackHardLimit := this->BackHardLimit.tSensor::Update (tTimeNow) ]]
  ****/
  this->bBackHardLimit  = (BOOL) this->BackHardLimit.Update(tTimeNow);

  /****
  [[U this->bHome := this->HomeSensor.tSensor::Update (tTimeNow) ]]
  ****/
  this->bHome = (BOOL) this->HomeSensor.Update(tTimeNow);

  /****
  [[U OnALimitNow := tDlLimits::IsOnAnyLimit () ]]
  ****/
  bOnALimitNow = this->IsOnAnyLimit ();
   
  if ( ! WasOnAnyLimit && bOnALimitNow ) {
  
    /****
    [[U ErrorStream.tErrorStreamSender::Put_ec (S_dlLimits_DELAY_LINE_REACHED_LIMIT) ]]
    ****/    
    GetErrorStream().Put (S_dlLimits_DELAY_LINE_REACHED_LIMIT);
  }
      

  return OK;
}



