/* $RCSfile: emController.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/em/emController.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#include "em/emController.h"
#include "em/emIoSet.h"
#include "periodicTargetGen.h"
#include "dlActuator.h"
#include "ipcid.h"
#include "emGizmoInfo.h"
#include "emTypes.h"
#include "servo.h"


// Fix to real values;

const int CiEmDataDumpPriority = 78;
const int CiEmDataBufferSize = 100;

/****
[[U tEmControllerData::tEmControllerData (DataTag) ]]
*****/
tEmControllerData::tEmControllerData (const tTypeTag& DataTag) :

  /**** 
  [[U this->EmDataStore.tDataStore::tDataStore (CiEmDataDumpPriority, 
                                         CiEmDataBufferSize, DataTag) ]]
  *****/
  EmDataStore (CiEmDataDumpPriority, CiEmDataBufferSize, DataTag) {
}


/****
[[U tEmController::tEmController (ControllerDataToUse) ]]
****/
tEmController::tEmController (tEmControllerData& ControllerDataToUse) :

  /****
  [ this->ControllerData := ControllerDataToUse ]
  ****/
  ControllerData (ControllerDataToUse) {
}

/****
[[U tEmOffController::tEmOffController (ControllerDataToUse) ]]
****/
tEmOffController::tEmOffController (tEmControllerData& ControllerData) :

  /****
  [[U tEmController::tEmController (ControllerData) ]]
  ****/
  tEmController (ControllerData) {
}


/****
[[U tEmTrackController::tEmTrackController (CompleteIoSet,ControllerData) ]]
****/
tEmTrackController::tEmTrackController (tEmCompleteIoSet& CompleteIoSet,
                                        tEmControllerData& ControllerData) :
 
  /****
  [[U tEmController::tEmController (ControllerData) ]]
  ****/
  tEmController (ControllerData),

  /****
  [ this->TargetGenerator := CompleteIoSet.TargetGenerator
    : this->DelayLineActuator := CompleteIoSet.DelayLineActuator
  ]
  ****/
  TargetGenerator (CompleteIoSet.TargetGenerator),
  DelayLineActuator (CompleteIoSet.DelayLineActuator) {
}

/****
[[U tEmTrackController::~tEmTrackController (void) ]]
****/
tEmTrackController::~tEmTrackController () {}




/******************** DEBUG KLUDGE 12/18/97 *****************************/
/**** EAMCK This will give a "kick" of 25000 laser units every second ***/
/************************************************************************/
BOOL   bKickit = FALSE;
double dKickTargetG = 0.0;


/****
[[U tEmTrackController::Calculate (tServoTime) ]]
****/
ERROR_CODE tEmTrackController::Calculate (TIME tServoTime) {

  
  tEmData Data;
  
  /****
  [[U NewTarget.tDlTarget::tDlTarget (void) ]]
  ****/
  tDlTarget NewTarget;
   
  /****
  [[U this->TargetGenerator.tPeriodicTargetGenerator::Update (tServoTime) ]]
  ****/
  this->TargetGenerator.Update (tServoTime);
  
  /****
  [[U NewTarget := this->TargetGenerator.tPeriodicTargetGenerator::
                   GetTarget (void) ]]
  ****/
  NewTarget = this->TargetGenerator.GetTarget ();



/******************** DEBUG KLUDGE 12/18/97 *****************************/
/**** EAMCK This will give a "kick" of 25000 laser units every second ***/
/************************************************************************/

if (bKickit)
{
   if ( (tServoTime % TIME_TICKS_PER_SECOND) == 0 )
      dKickTargetG += 25000.0;
   NewTarget = tDlTarget(dKickTargetG, 0.0, tServoTime);
}

  
  /****
  [[U this->DelayLineActuator.tDlActuator::SetTargetValue (NewTarget) ]]
  ****/
  this->DelayLineActuator.SetTargetValue (NewTarget);
  
  /****
  [[U this->DelayLineActuator.tDlActuator::Output (tServoTime) ]]
  ****/
  this->DelayLineActuator.Output (tServoTime);
  
  Data.tTime = tServoTime;
  Data.Target = NewTarget;
  
  /****
  [[U this->ControllerData.EmDataStore.tDataStore::StoreData (Data) ]]
  ****/
  this->ControllerData.EmDataStore.StoreData (Data);

  return OK;
}


ERROR_CODE tEmTrackController::Reset (TIME tServoTime) {

  ERROR_CODE ec;

  ec = this->DelayLineActuator.Reset ();

  if (ec != OK) return Traceback (ec,"tEmTrackController::Reset");

  return OK;
}


/****
[[U tEmCompleteControllerSet::tEmCompleteControllerSet (CompleteIoSetToUse,
                                                        DataTag) ]]
****/
tEmCompleteControllerSet::tEmCompleteControllerSet (
     tEmCompleteIoSet& CompleteIoSetToUse,
     const tTypeTag& DataTag) :

  /****
  [[U this->OffController.tEmOffController::
         tEmOffController (this->ControllerData) ]]
  ****/
  OffController (ControllerData),

  /****
  [[U this->TrackController.tEmTrackController::
      tEmTrackController (CompleteIoSetToUse, this->ControllerData) ]]
  ****/
  TrackController (CompleteIoSetToUse,ControllerData),

  /****
  [ this->CompleteIoSet := CompleteIoSetToUse ]
  ****/
  CompleteIoSet (CompleteIoSetToUse),
  
  /****
  [[U this->ControllerData.tEmControllerData::
      tEmControllerData (DataTag) ]]
  ****/
  ControllerData (DataTag) {
}

/****
[[U tEmCompleteControllerSet::~tEmCompleteControllerSet (void) ]]
****/
tEmCompleteControllerSet::~tEmCompleteControllerSet () {}



/****
[[U tEmCompleteControllerSet::GetOffController (void) ]]
****/
tEmOffController& tEmCompleteControllerSet::GetOffController () {

   return this->OffController;
}

/****
[[U tEmCompleteControllerSet::GetTrackController (void) ]]
****/
tEmTrackController& tEmCompleteControllerSet::GetTrackController () {

   return this->TrackController;
}
 

/****
[[U tEmCompleteControllerSet::SetServoParameters (
       const tEmServoParameters& ServoParameters) ]]
****/
ERROR_CODE tEmCompleteControllerSet::SetServoParameters (
  const tEmServoParameters& ServoParameters) {
  
  /****
  [[U ModulationParameters := ServoParameters.tEmServoParameters::
        GetModulationParameters (void) ]]
  ****/
  const tModulationParams& ModulationParameters = 
     ServoParameters.GetModulationParameters ();

  /****
  [[U this->CompleteIoSet.TargetGenerator.tPeriodicTargetGenerator::
        SetModulationParameters(ModulationParameters)]]
  ****/
  return this->CompleteIoSet.TargetGenerator.
     SetModulationParameters(ModulationParameters);

}
