/* $RCSfile: emGizmo.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/em/emGizmo.cc,v 1.2 1998/09/23 23:57:08 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#include "iostream.h"
#include "isit.h"
#include "baseObject.h"
#include "emError.h"
#include "emGizmo.h"
#include "servo.h"
#include "emCommands.h"



/****
[[U tEmGizmo::tEmGizmo (GizmoInfo) ]]
****/
tEmGizmo::tEmGizmo (const tEmGizmoInfo& GizmoInfo) :
 
  /****
  [[U tRealtimeGizmo::tRealtimeGizmo (GizmoInfo) ]]
  ****/ 
  tRealtimeGizmo (GizmoInfo),
  
  /****
  [[U this->CEmStatusTag.tTypeTag::tTypeTag_STTNisz (ST_STATUS, TN_EM_STATUS, 
                GizmoInfo.iGizmoId, sizeof (tEmStatus) ) ]]
  ****/ 
  CEmStatusTag (ST_STATUS, TN_EM_STATUS, GizmoInfo.GetGizmoId(),
                sizeof (tEmStatus) ) ,
  /****
  [[U this->CEmDataTag.tTypeTag::tTypeTag_STTNisz (ST_DATA, TN_EM_DATA, 
                GizmoInfo.iGizmoId, sizeof (tEmData) ) ]]
  ****/ 
  CEmDataTag (ST_DATA, TN_EM_DATA, GizmoInfo.GetGizmoId (), sizeof (tEmData) ),  

  /****
  [[U this->CompleteIoSet.tEmCompleteIoSet::tEmCompleteIoSet (GizmoInfo) ]]
  ****/ 
  CompleteIoSet (GizmoInfo),
  
  /****
  [[U this->CompleteControllerSet.tEmCompleteControllerSet::
        tEmCompleteControllerSet (this->CompleteIoSet, this->CEmDataTag) ]]
  ****/ 
  CompleteControllerSet (CompleteIoSet, CEmDataTag),

  /****
  [[U this->Status.tEmStatus::tEmStatus (GizmoInfo.iGizmoId) ]]
  ****/ 
  Status (GizmoInfo.GetGizmoId () ),
  
                              
  /****
  [[U this->EmOffControllerGroup.tSimpleControllerGroup::
        tSimpleControllerGroup_i (GizmoInfo.ServoInfo.iNumberOfPartitions) ]]
  ****/ 
  EmOffControllerGroup (GizmoInfo.GetNumberOfServoPartitions () ),
  
  /****
  [[U this->EmTrackControllerGroup.tSimpleControllerGroup::
        tSimpleControllerGroup_i (GizmoInfo.ServoInfo.iNumberOfServoPartitions) ]]
  ****/ 
  EmTrackControllerGroup (GizmoInfo.GetNumberOfServoPartitions () ) {
  
  ERROR_CODE ec;
  
    if (this->GetInitError() != OK) {
      TraceMsg (tmt_ERROR,this->GetInitError(),"tEmGizmo::tEmGizmo");
      return;
    }

    if ( GizmoInfo.GetGizmoType () != GT_EXTERNAL_METROLOGY ) {

      this->SetInitError (S_emgizmo_GIZMO_INFO_NOT_EM);
      TraceMsg (tmt_ERROR,S_emgizmo_GIZMO_INFO_NOT_EM,
                "tEmGizmo::tEmGizmo" );
      return;
    }
  

    // Sanity Checks until we can better automate the controller setup.

    if ( GizmoInfo.GetNumberOfServoModes () != 2) {

      this->SetInitError (S_emgizmo_INVALID_NUMBER_OF_SERVO_MODES);
      TraceMsg (tmt_ERROR,S_emgizmo_INVALID_NUMBER_OF_SERVO_MODES,
                "tEmGizmo::tEmGizmo" );
      return;
    }
    
    if (GizmoInfo.GetNumberOfServoPartitions () != 1 ) {

      this->SetInitError (S_emgizmo_INVALID_NUMBER_OF_SERVO_PARTITIONS);
      TraceMsg (tmt_ERROR,S_emgizmo_INVALID_NUMBER_OF_SERVO_PARTITIONS,
                "tEmGizmo::tEmGizmo" );
      return;
    }


    /****
    [[U OffController := this->CompleteControllerSet.tEmCompleteControllerSet::
           GetOffController (void) ]]
    ****/ 
    tEmOffController& OffController = this->CompleteControllerSet.GetOffController ();
       
    /****
    [[U TrackController := this->CompleteControllerSet.tEmCompleteControllerSet::
           GetTrackController (void) ]]
    ****/ 
    tEmTrackController& TrackController = 
                     this->CompleteControllerSet.GetTrackController ();

  
    // Assign Controller Groups to Servo Modes in Controller Set
  
    /****
    [[U ec := this->ControllerSet.tControllerSet::AssignControllerGroup 
               (ESM_OFF,this->EmOffControllerGroup) ]] 
    ****/ 
    ec = this->ControllerSet.AssignControllerGroup (ESM_OFF,this->EmOffControllerGroup);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }


    /****
    [[U ec := this->ControllerSet.tControllerSet::AssignControllerGroup 
               (ESM_TRACK,this->EmTrackControllerGroup) ]]
    ****/ 
    ec = this->ControllerSet.AssignControllerGroup (ESM_TRACK,
              this->EmTrackControllerGroup);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }
  
    // Assign Controllers to Servo Partitions in Controller Group
       
    /****
    [[U ec := this->EmOffControllerGroup.tSimpleControllerGroup::
                 AssignController (0,OffController) ]]
    ****/ 
    ec = this->EmOffControllerGroup.AssignController (0,OffController);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }

    /****
    [[U ec := this->EmTrackControllerGroup.tSimpleControllerGroup::
                 AssignController (0,TrackController) ]]
    ****/ 
    ec = this->EmTrackControllerGroup.AssignController (0,TrackController);
 
    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }

    // Select Off Mode
    /****
    [[U ec := this->Servo.tServo::SelectMode (ESM_OFF) ]] 
    ****/ 
    ec = this->Servo.SelectMode (ESM_OFF);
    
    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }

    // Enable Base class tasks ...

    /****
    [[U ec := tRealtimeGizmo::StartTasks (GizmoInfo) ]] 
    ****/ 
    ec = tRealtimeGizmo::StartTasks (GizmoInfo);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tEmGizmo::tEmGizmo" );
      this->SetInitError (ec);
      return;
    }

}

/****
[[U tEmGizmo::~tEmGizmo (void) ]]
****/
tEmGizmo::~tEmGizmo () { 

  /****
  [[U this->EmTrackControllerGroup.tSimpleControllerGroup::
        ~tSimpleControllerGroup (void) ]]

  [[U this->EmOffControllerGroup.tSimpleControllerGroup::
        ~tSimpleControllerGroup (void) ]]

  [[U this->CEmStatusTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->CEmStatusTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->Status.tEmStatus::~tEmStatus (void) ]]

  [[U this->CompleteControllerSet.tEmCompleteControllerSet::
        ~tEmCompleteControllerSet (void) ]]

  [[U this->CompleteIoSet.tEmCompleteIoSet::~tEmCompleteIoSet (void) ]]

  [[U tRealtimeGizmo::~tRealtimeGizmo (void) ]]
  ****/ 
 
}

/****
[[U tEmGizmo::Restart (void) ]]
****/
ERROR_CODE tEmGizmo::Restart () {

  /****
  [[U tRealtimeGizmo::Restart (void) ]]
  ****/
  return tRealtimeGizmo::Restart ();
}

/****
[[U tEmGizmo::Shutdown (void) ]]
****/
ERROR_CODE tEmGizmo::Shutdown () {

  /****
  [[U tRealtimeGizmo::Shutdown (void) ]]
  ****/
  return tRealtimeGizmo::Shutdown ();
}
 
/****
[[U tEmGizmo::IsOk (void) ]]
****/
BOOL tEmGizmo::IsOk () {

  /****
  [[U tRealtimeGizmo::IsOk (void) ]]
  ****/
  return tRealtimeGizmo::IsOk ();
}

/****
[[U tEmGizmo::operator_extractor (ostream& stream, tEmGizmo& Gizmo) ]]
****/
ostream& operator<< (ostream& stream, tEmGizmo& Gizmo) {

  stream << "External Metrology Gizmo" << endl;

  /****
  [[U tRealtimeGizmo::operator_extractor (stream,Gizmo) ]]
  ****/  stream << (tRealtimeGizmo &) Gizmo;
  return stream;
}


/****
[[U tEmGizmo::CollectStatus (void) ]]
****/
ERROR_CODE tEmGizmo::CollectStatus () {

  ERROR_CODE ec;
  
  /****
  [[U this->Status.tTime := ::GetTime (void) ]]
  ****/
  this->Status.tTime = ::GetTime ();

  /****
  [[U ec := tGizmo::SendStatus_tp (CEmStatusTag,&this->Status) ]]
  ****/
  ec = this->SendStatus (CEmStatusTag,(BYTE *) &this->Status);
  
    /****
    [[U ec := tGizmo::SendError_ec (S_emgizmo_UNABLE_TO_SEND_STATUS) ]]
    ****/
  if (ec != OK) this->SendError (S_emgizmo_UNABLE_TO_SEND_STATUS);

  return OK;
}
  


/****
[[U  tEmGizmo::Dispatch (Command) ]]
****/
ERROR_CODE tEmGizmo::Dispatch (const tIsitCmd& Command) {

  switch (Command.Command) {

      /****
      [[U  tEmGizmo::Ping (void) ]]
      ****/
    case EM_CMD_PING : return this->Ping ();

      /****
      [[U  tEmGizmo::Off (void) ]]
      ****/
    case EM_CMD_OFF : return this->Off ();

      /****
      [[U  tEmGizmo::Track (void) ]]
      ****/
    case EM_CMD_TRACK : return this->Track ();

      /****
      [[U  tEmGizmo::SetServoParameters (Command.ServoParametes) ]]
      ****/
    case EM_CMD_CHNG_PARAM : return this->SetServoParameters ( 
                              (tEmSetServoParametersCmd&) Command);

    default : return S_emgizmo_UNRECOGNIZED_COMMAND;
  };
  return OK;
}
  
    
/****
[[U  tEmGizmo::Ping (void) ]]
****/
ERROR_CODE tEmGizmo::Ping () {
 
   /****
   [[U  tGizmo::SendError_ec (S_emgizmo_EXTERNAL_METROLOGY_ALIVE) ]]
   ****/
   return SendError (S_emgizmo_EXTERNAL_METROLOGY_ALIVE);
}

   
/****
[[U  tEmGizmo::Off (void) ]]
****/
ERROR_CODE tEmGizmo::Off () {

  ERROR_CODE ec;
  
  /****
  [[U this->Servo.tServo::SelectMode (ESM_OFF) ]]
  ****/
  ec = this->Servo.SelectMode (ESM_OFF);

  if (ec == OK) {

    /****
    [[U this->Status.tEmStatus::UpdateCommandStatus (EMCS_OFF) ]]
    ****/
    this->Status.UpdateCommandStatus (EMCS_OFF);
  
    /****
    [[U this->Status.tEmStatus::UpdateServoState (ESS_OFF) ]]
    ****/
    this->Status.UpdateServoState (ESS_OFF);

    /****
    [[U this->tGizmo::SendError_ec (S_em_OFF) ]]
    ****/
    this->SendError (S_em_OFF_COMPLETE);
  }
   
  return ec;
}

  
/****
[[U  tEmGizmo::Track (void) ]]
****/
ERROR_CODE tEmGizmo::Track () {

  ERROR_CODE ec;


  /****
  [[U ec := this->Servo.tServo::SelectMode (ESM_TRACK) ]]
  ****/
  ec = this->Servo.SelectMode (ESM_TRACK);
  
  if (ec == OK) {

    /****
    [[U this->Status.tEmStatus::UpdateCommandStatus (EMCS_TRACK_ACTIVE) ]]
    ****/
    this->Status.UpdateCommandStatus (EMCS_TRACK_ACTIVE);

    /****
    [[U this->Status.tEmStatus::UpdateServoState (ESS_TRACK) ]]
    ****/
    this->Status.UpdateServoState (ESS_TRACK);

    /****
    [[U this->tGizmo::SendError_ec (S_em_TRACK_IN_PROGRESS) ]]
    ****/
    this->SendError (S_em_TRACK_IN_PROGRESS);
  }
  return ec;
}

/****
[[U  tEmGizmo::SetServoParameters (Command) ]]
****/
ERROR_CODE tEmGizmo::SetServoParameters (tEmSetServoParametersCmd& Command) {

  ERROR_CODE ec;
  tEmServoParameters ServoParameters;  

  /****
  [[U ServoParameters.tEmServoParameters::tEmServoParameters (
        Command.mtModulationType, Command.dFrequency, Command.dAmplitude 
         Command.MeanVal, Command.tSyncTime) ]]
  ****/
  ServoParameters.tEmServoParameters (Command.mtModulationType, 
    Command.dFrequency,  Command.dAmplitude, Command.dMeanVal,
    Command.tSyncTime );

  /****
  [[U ec := this->CompleteControllerSet.tEmCompleteControllerSet::
              SetServoParameters (ServoParameters) ]]
  ****/
  ec = this->CompleteControllerSet.SetServoParameters (ServoParameters);


  if (ec == OK) {

    /****
    [[U this->Status.tEmStatus::UpdateCommandStatus (
          EMCS_SERVO_PARAMETERS_SET) ]]
    ****/
    this->Status.UpdateCommandStatus (EMCS_SERVO_PARAMETERS_SET);

    /****
    [[U this->Status.tEmStatus::UpdateServoParams (ServoParameters) ]]
    ****/
    this->Status.UpdateServoParameters (ServoParameters);

    /****
    [[U tGizmo::SendError_ec (S_emgizmo_SERVO_PARAMETERS_SET) ]]
    ****/
    this->SendError (S_emgizmo_SERVO_PARAMETERS_SET);
  }
  
  return ec;  
} 









