/* $RCSfile: ftControllerSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftControllerSet.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

#include "ft/ftControllerSet.h"
#include "ft/ftIoSet.h"
#include "ftGizmoInfo.h"
#include "ftTypes.h"

/******************************************************************
*
* tFtCompleteControllerSet::tFtCompleteControllerSet (tFtCompleteIoSet& CompleteIoSet,
                              const tFtGizmoInfo& GizmoInfo)
*
*  DESCRIPTION:
   Constructs complete controller set instantiating a single controller for each type of 
   controller needed for the fringe tracker servo.  These controllers are mapped into 
   the appropriate servo location by (ServoMode,ServoPartition,ControllerMode). 
*
[[U tFtCompleteControllerSet::tFtCompleteControllerSet (CompleteIoSet, 
      GizmoInfo) ]]
****/ 
tFtCompleteControllerSet::tFtCompleteControllerSet (tFtCompleteIoSet& CompleteIoSet,
                              const tFtGizmoInfo& GizmoInfo) :
 
  /****
  [[U this->ControllerParameters := GizmoInfo.tFtGizmoInfo::ControllerParameters (void) ]]
  ****/
  ControllerParameters (GizmoInfo.ControllerParameters () ),
  
  /****
  [[U this->Thresholds := GizmoInfo.tFtGizmoInfo::Thresholds (void) ]]
  ****/
  Thresholds (GizmoInfo.Thresholds () ),

  /****
  [[U this->FastOff.tFtFastOffController::tFtFastOffController (CompleteIoSet) ]]
  ****/
  FastOff (CompleteIoSet),

  /****
  [[U this->FastTrackSearch.tFtFastTrackSearchController::tFtFastTrackSearchController (CompleteIoSet) ]]
  ****/
  FastTrackSearch (CompleteIoSet),

  /****
  [[U this->FastTrackSemiLock.tFtFastTrackController::tFtFastTrackController (CompleteIoSet) ]]
  ****/
  FastTrackSemiLock (CompleteIoSet),

  /****
  [[U this->FastTrackLock.tFtFastTrackLockController::tFtFastTrackLockController (CompleteIoSet) ]]
  ****/
  FastTrackLock (CompleteIoSet),

  /****
   [[U this->MediumOff.tFtMediumOffController::tFtMediumOffController (CompleteIoSet,
     this->ControllerParameters) ]]
  ****/
  MediumOff (CompleteIoSet, ControllerParameters ),

  /****
  [[U this->MediumTrackSearch.tFtMediumTrackSearchController::
        tFtMediumTrackSearchController (CompleteIoSet, this->ControllerParameters) ]]
  ****/
  MediumTrackSearch (CompleteIoSet,ControllerParameters),

  /****
  [[U this->MediumTrackSemiLock.tFtMediumTrackSemiLockController::
        tFtMediumTrackSemiLockController (CompleteIoSet, this->ControllerParameters) ]]
  ****/
  MediumTrackSemiLock (CompleteIoSet, ControllerParameters ),

  /****
  [[U this->MediumTrackLock.tFtMediumTrackLockController::
        tFtMediumTrackLockController (CompleteIoSet, this->ControllerParameters) ]]
  ****/
  MediumTrackLock (CompleteIoSet, ControllerParameters),

  /****
  [[U this->SlowOffSearchSemiLock.tFtSlowController::tFtSlowController (
        CompleteIoSetm this->ControllerParameters) ]]
  ****/
  SlowOffSearchSemiLock (CompleteIoSet, ControllerParameters),

  /****
  [[U this->SlowTrackLock.tFtSlowTrackLockController::tFtSlowTrackLockController (
        CompleteIoSet, this->ControllerParameters) ]]
  ****/
  SlowTrackLock (CompleteIoSet,ControllerParameters),

  /****
  [[U this->ImagingOff.tFtImagingController::tFtImagingController (
        CompleteIoSet) ]]
  ****/
  ImagingOff (CompleteIoSet),

  /****
  [[U this->ImagingTrack.tFtImagingTrackController::tFtImagingTrackController (
        CompleteIoSet) ]]
  ****/
  ImagingTrack (CompleteIoSet),

  /****
  [[U ServoPeriod := GizmoInfo.tFtGizmoInfo::ServoPeriod (FSP_MEDIUM) ]]
  
  [[U this->TrackUpdate.tFtTrackGroupUpdate::tFtTrackGroupUpdate (CompleteIoSet.Fringe,
        this->Thresholds, ServoPeriod) ]]
  ****/      
  TrackUpdate (CompleteIoSet.Fringe, Thresholds, 
      GizmoInfo.ServoPeriod (FSP_MEDIUM)),

  /****
  [[U this->OffGroup.tSimpleControllerGroup::tSimpleControllerGroup (
        FSP_NUM_SERVO_PARTITIONS) ]]
  ****/
  OffGroup (FSP_NUM_SERVO_PARTITIONS),

  /****
  [[U TrackUpdatePeriod := GizmoInfo.tFtGizmoInfo::TrackUpdatePeriod (void) ]]
  
  [[U TrackUpdatePriority := GizmoInfo.tFtGizmoInfo::TrackUpdatePriority (void) ]]
  
  [[U TrackUpdateOffset := GizmoInfo.tFtGizmoInfo::TrackUpdateOffset (void) ]]

  [[U this->TrackGroup.tModedControllerGroup::tModedControllerGroup_iititt (
        FSP_NUM_SERVO_PARTITIONS, FTM_NUM_OF_MODES,
        this->TrackUpdate,  TrackUpdatePriority, TrackUpdatePeriod,
        TrackUpateOffset) ]]
  ****/
  TrackGroup (FSP_NUM_SERVO_PARTITIONS, FTM_NUM_MODES,
    TrackUpdate, GizmoInfo.TrackUpdatePriority (),
    GizmoInfo.TrackUpdatePeriod (),
    GizmoInfo.TrackUpateOffset () ) 
{

  /****
  [[U this->OffGroup.tSimpleControllerGroup::AssignController (FSP_FAST, 
        this->FastOff) ]]
  
  [[U   this->OffGroup.tSimpleControllerGroup::AssignController (FSP_MEDIUM, 
        this->MediumOff) ]]
  
  [[U this->OffGroup.tSimpleControllerGroup::AssignController (FSP_SLOW, 
        this->SlowOffSearchSemilLock) ]]
  
  [[U this->OffGroup.tSimpleControllerGroup::AssignController (FSP_IMAGING, 
        this->ImagingOff) ]]
  ****/
  { 
  this->OffGroup.AssignController (FSP_FAST, this->FastOff);
  this->OffGroup.AssignController (FSP_MEDIUM, this->MediumOff);
  this->OffGroup.AssignController (FSP_SLOW, this->SlowOffSearchSemiLock);
  this->OffGroup.AssignController (FSP_IMAGING, this->ImagingOff);
  }
  
  /****
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_FAST, 
        FTM_SEARCH, this->FastTrackSearch) ]]

  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_FAST, 
        FTM_SEMILOCK, this->FastTrackSemiLock) ]]
  
  [[U  this->TrackGroup.tModedControllerGroup::AssignController (FSP_FAST, 
        FTM_LOCK, this->FastTrackLock) ]]
  ****/ 
  {
  this->TrackGroup.AssignController (FSP_FAST, FTM_SEARCH,   this->FastTrackSearch);
  this->TrackGroup.AssignController (FSP_FAST, FTM_SEMI_LOCK, this->FastTrackSemiLock);
  this->TrackGroup.AssignController (FSP_FAST, FTM_LOCK,     this->FastTrackLock);
  }
  
  /****
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_MEDIUM, 
        FTM_SEARCH, this->MediumTrackSearch) ]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_MEDIUM, 
        FTM_SEMI_LOCK, this->MediumTrackSemiLock)]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_MEDIUM, 
        FTM_LOCK, this->MediumTrackLock) ]]
  ****/
  {
  this->TrackGroup.AssignController (FSP_MEDIUM, FTM_SEARCH,   this->MediumTrackSearch);
  this->TrackGroup.AssignController (FSP_MEDIUM, FTM_SEMI_LOCK, this->MediumTrackSemiLock);
  this->TrackGroup.AssignController (FSP_MEDIUM, FTM_LOCK,     this->MediumTrackLock);
  }
  
  /****
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_SLOW, 
        FTM_SEARCH, this->SlowOffSearchSemiLock) ]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_SLOW, 
        FTM_SEMI_LOCK, this->SlowOffSearchSemiLock) ]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_SLOW, 
        FTM_LOCK, this->SlowTrackLock) ]]
  ****/
  {
  this->TrackGroup.AssignController (FSP_SLOW, FTM_SEARCH,   this->SlowOffSearchSemiLock);
  this->TrackGroup.AssignController (FSP_SLOW, FTM_SEMI_LOCK, this->SlowOffSearchSemiLock);
  this->TrackGroup.AssignController (FSP_SLOW, FTM_LOCK,     this->SlowTrackLock);
  }


  /****
  [[U this->TrackGroup.tModedController::AssignController (FSP_IMAGING, 
        FTM_SEARCH, this->ImagingTrack) ]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_IMAGING, 
         FTM_SEMI_LOCK, this->ImagingTrack) ]]
  
  [[U this->TrackGroup.tModedControllerGroup::AssignController (FSP_IMAGING, 
         FTM_LOCK, this->ImagingTrack) ]]
  ****/
  {
  this->TrackGroup.AssignController (FSP_IMAGING, FTM_SEARCH,   this->ImagingTrack);
  this->TrackGroup.AssignController (FSP_IMAGING, FTM_SEMI_LOCK, this->ImagingTrack);
  this->TrackGroup.AssignController (FSP_IMAGING, FTM_LOCK,     this->ImagingTrack);
  }
  
}



/******************************************************************
*
* tFtCompleteControllerSet::~tFtCompleteControllerSet ()
*
*  DESCRIPTION:
   Destructs complete controller set destroying all members. 
*
[[U tFtCompleteControllerSet::~tFtCompleteControllerSet (void) ]]
****/ 
tFtCompleteControllerSet::~tFtCompleteControllerSet () 
{

  /****
  [[U this->TrackGroup.tModedControllerGroup::~tModedControllerGroup (void) ]]

  [[U this->OffGroup.tSimpleControllerGroup::~tSimpleControllerGroup (void) ]]

  [[U this->TrackUpdate.tFtTrackGroupUpdate::~tFtTrackGroupUpdate (void) ]]

  [[U this->ImagingTrack.tFtImagingTrackController::~tFtImagingTrackController (void) ]]

  [[U this->ImagingOff.tFtImagingController~::tFtImagingController (void) ]]

  [[U this->SlowTrackLock.tFtSlowTrackLockController::~tFtSlowTrackLockController (void) ]]

  [[U this->SlowOffSearchSemiLock.tFtSlowController::~tFtSlowController (void) ]]

  [[U this->MediumTrackLock.tFtMediumTrackLockController::
        ~tFtMediumTrackLockController (void) ]]

  [[U this->MediumTrackSemiLock.tFtMediumTrackSemiLockController::
        ~tFtMediumTrackSemiLockController (void) ]]

  [[U this->MediumTrackSearch.tFtMediumTrackSearchController::
        ~tFtMediumTrackSearchController (void) ]]

  [[U this->MediumOff.tFtMediumOffController::~tFtMediumOffController (void) ]]

  [[U this->FastTrack.tFtFastTrackController::~tFtFastTrackController (void) ]]

  [[U this->FastOff.tFtFastController::~tFtFastController (void) ]]

  [[U this->Thresholds.tFtThresholdParameters::~tFtThresholdParameters (void) ]]

  [[U this->ControllerParameters.tFtControllerParameters::~tFtControllerParameters (void) ]]
  ****/
}


   
/******************************************************************
*
* tFtCompleteControllerSet::OffControllerGroup ()
*
*  DESCRIPTION:
   Returns a reference to a controller group for fringe tracker off servo mode
*
[[U tFtCompleteControllerSet::OffControllerGroup (void) ]]
****/    
tControllerGroup& tFtCompleteControllerSet::OffControllerGroup ()
{
  return this->OffGroup;
}



/******************************************************************
*
* tFtCompleteControllerSet::TrackControllerGroup ()
*
*  DESCRIPTION:
   Returns a reference to a controller group for fringe tracker track servo mode
*
[[U tFtCompleteControllerSet::TrackControllerGroup (void) ]]
****/    
tControllerGroup& tFtCompleteControllerSet::TrackControllerGroup ()
{
  return this->TrackGroup;
}



/******************************************************************
*
* tFtCompleteControllerSet::SetControllerParameters (const tFtControllerParameters& 
      ControllerParametersToUse)
*
*  DESCRIPTION:
   Sets controller parameters to user defined values contained in ControllerParametersToUse
*
[[U tFtCompleteControllerSet::SetControllerParameters (ControllerParametersToUse) ]]
****/    
ERROR_CODE tFtCompleteControllerSet::SetControllerParameters (const tFtControllerParameters& 
      ControllerParametersToUse)
{
  this->ControllerParameters = ControllerParametersToUse;
  return OK;
}



/******************************************************************
*
* tFtCompleteControllerSet::SetThresholds (const tFtThresholdParameters& ThresholdsToUse)
*
*  DESCRIPTION:
   Sets controller parameters to user defined values contained in ControllerParametersToUse
*
[[U tFtCompleteControllerSet::SetThresholds (ThresholdsToUse) ]]
****/    
ERROR_CODE tFtCompleteControllerSet::SetThresholds (const tFtThresholdParameters& 
      ThresholdsToUse)
{
  this->Thresholds = ThresholdsToUse;
  return OK;
}          


/******************************************************************
*
* tFtCompleteControllerSet::GetControllerParameters ()
*
*  DESCRIPTION:
   Returns current Controller Parameters used by controllers.
*
[[U tFtCompleteControllerSet::GetControllerParameters (void) ]]
****/    
const tFtControllerParameters& tFtCompleteControllerSet::
  GetControllerParameters () const
{ 
  return this->ControllerParameters;
}

/******************************************************************
*
* tFtCompleteControllerSet::GetThresholds ()
*
*  DESCRIPTION:
   Returns current Thresholds used by fringe tracker controllers.
*
[[U tFtCompleteControllerSet::GetThresholds (void) ]]
****/    
const tFtThresholdParameters& tFtCompleteControllerSet::GetThresholds () const
{ 
  return this->Thresholds;
}



