/* $RCSfile: ftFastController.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftFastController.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a7Nov97,rlj    written
*/

#include "ft/ftFastController.h"
#include "ft/ftIoSet.h"
#include "integratorSensor.h"
#include "ft/fringe.h"

BOOL bPhotonCount = FALSE;

 
/************* BEH ************/
//FILE *fp = NULL;
//BOOL bRecordFringe=FALSE;
//#include "rngLib.h"
//RING_ID riData;
//int rngTask(void)
//{
//  int iBytes;
//  char cBuf[40000];

//  printf("Hello from rngTask\n");
  
//  FOREVER {
//    iBytes = rngBufGet(riData,cBuf,40000);
//    if (iBytes > 0)
//      fwrite(cBuf,1,iBytes,fp);
 //   if (iBytes < 40000)
//      taskDelay(1);
//  }
//}
/******************************/


/*******************************************************************
*
* tFtFastController::tFtFastController (tFtCompleteIoSet& CompleteIoSet)
*
* DESCRIPTION:
  Constructs base class for all fringe tracker fast controllers assigning
  references to a tApdSensor and tFringe from the FT CompleteIoSet.
*
[[U tFtFastController::tFtFastController (CompleteIoSet) ]]
****/
tFtFastController::tFtFastController (tFtCompleteIoSet& CompleteIoSet) :
  
  /****
  [[U tFtController::tFtController (void) ]]
  ****/
  tFtController (),
  
  /****
  [ this->Apd := reference to CompleteIoSet.Apd ]
  ****/
  Apd (CompleteIoSet.Apd),

  /****
  [ this->ApdDiagOutput := reference to CompleteIoSet.ApdDiagOutput ]
  ****/
  ApdDiagOutput (CompleteIoSet.ApdDiagOutput),

  /****
  [ this->Fringe := reference to CompleteIoSet.Fringe ]
  ****/
  Fringe (CompleteIoSet.Fringe),

  bAudioFringe (FALSE),
 
  PhotonDiag (CompleteIoSet.PhotonDiag)

{  //}
  
/************* BEH ************/
//if (fp==NULL) {
//  fp=fopen ("/home2/realtime/data/fringe/fringe.dat","wb");
//  if (fp==NULL)
//    perror("Error opening fringe.dat\n");
 // else {
//    printf("File opened: fp=%d\n", (int)fp);
//    riData = rngCreate(80000);
 //   taskSpawn("rngTask",99,0,100000,(FUNCPTR)rngTask,0,0,0,0,0,0,0,0,0,0);
 // }
//}
/******************************/

}


/*******************************************************************
*
* tFtFastController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
  Reads the photon count from the Apd and stores it in Fringe.
*
[[U tFtFastController::Calculate (tServoTime) ]]
****/
ERROR_CODE tFtFastController::Calculate (TIME tServoTime)
{

  ERROR_CODE ec;
  double dPhotonCount;
  
  /****
  [[U dPhotonCount := this->Apd.tIntegratorSensor::Update (tServoTime) ]]
  ****/
  dPhotonCount = this->Apd.Update (tServoTime);

  //if (this->bAudioFringe) {
    /****
    [[U  this->ApdDiagOutput.tActuator::OutputValue (tServoTime,dPhotonCount) ]]
    ****/
    this->ApdDiagOutput.OutputValue (tServoTime,dPhotonCount*128.0); // scale up output
  //}

  /****
  [[U ec :=  this->Fringe.tFringe::AcceptCount (tServoTime, iPhotonCount) ]]
  ****/
  ec = this->Fringe.AcceptCount (tServoTime, (int) dPhotonCount);

  PhotonDiag.Append (tCount (dPhotonCount,tServoTime));

  if (bPhotonCount) {
    for (int i=-12;i < 0; i++)  cout << "Time = " << PhotonDiag[i].time << " Count = " << PhotonDiag[i].count << endl;
    bPhotonCount = FALSE;
  }
  
  
/************* BEH - KMS************/
//if (bRecordFringe == 1) {

//double dTime = (double)tServoTime;

//   rngBufPut(riData,(char *) &dPhotonCount,sizeof(dPhotonCount));

//   rngBufPut(riData,(char *) &dTime,sizeof(double));      
//}

//if (bRecordFringe == 2) {

//double dTime = (double)tServoTime;
//double dphase = Fringe.Phase();
//static int bStart = FALSE;

//if (bStart) 
//   rngBufPut(riData,(char *) &dPhotonCount,sizeof(dPhotonCount));

  
//if ((tServoTime % 40) == 0) {
//      rngBufPut(riData,(char *) &dTime,sizeof(double)); 
//      rngBufPut(riData,(char *) &dphase,sizeof(double));
//      bStart = TRUE;
//  }
     
//}
/******************************/


  return ec;
}



/*******************************************************************
*
* tFtFastOffController::tFtFastOffController (tFtCompleteIoSet& CompleteIoSet)
*
* DESCRIPTION:
*
[[U tFtFastOffController::tFtFastOffController (CompleteIoSet) ]]
****/
tFtFastOffController::tFtFastOffController (
    tFtCompleteIoSet& CompleteIoSet) :
  
  /****
  [[U tFtFastController::tFtFastController (CompleteIoSet) ]]
  ****/
  tFtFastController (CompleteIoSet)
  
{}



/*******************************************************************
*
* tFtFastOffController::Init
*
* DESCRIPTION:
  Reads the photon count from the Apd and stores it in Fringe.
*
[[U tFtFastOffController::Init (tServoTime) ]]
****/
ERROR_CODE tFtFastOffController::Init (TIME tServoTime)
{

  this->bAudioFringe = FALSE;

  /****
  [[U this->Fringe.tFringe::StoreOnly (void) ]]
  ****/
  this->Fringe.StoreOnly ();
 
  return OK;
}



/*******************************************************************
*
* tFtFastTrackController::tFtFastTrackController (tFtCompleteIoSet& CompleteIoSet)
*
* DESCRIPTION:
  Construct controller used for all for all fast track controllers modes.
  The controller reads from the ApdSensor and stores the photon count in the
  Fringe object.
*
[[U tFtFastTrackController::tFtFastTrackController (CompleteIoSet) ]]
****/
tFtFastTrackController::tFtFastTrackController (
    tFtCompleteIoSet& CompleteIoSet) :
  
  /****
  [[U tFtFastController::tFtFastController (CompleteIoSet) ]]
  ****/
  tFtFastController (CompleteIoSet)

  
{}



/*******************************************************************
*
* tFtFastTrackSearchController::tFtFastTrackSearchController (tFtCompleteIoSet& CompleteIoSet)
*
* DESCRIPTION:
*
[[U tFtFastTrackController::tFtFastTrackController (CompleteIoSet) ]]
****/
tFtFastTrackSearchController::tFtFastTrackSearchController (
    tFtCompleteIoSet& CompleteIoSet) :
  
  /****
  [[U tFtFastTrackController::tFtFastTrackController (CompleteIoSet) ]]
  ****/
  tFtFastTrackController (CompleteIoSet)
  
{}




/*******************************************************************
*
* tFtFastTrackSearchController::Init (tServoTime)
*
* DESCRIPTION:
  Reads the photon count from the Apd and stores it in Fringe.
*
[[U tFtFastTrackSearchController::Init (tServoTime) ]]
****/
ERROR_CODE tFtFastTrackSearchController::Init (TIME tServoTime)
{

  this->bAudioFringe = TRUE;

  this->Fringe.Reset ();

  /****
  [[U this->Fringe.tFringe::CheckCounts (void) ]]
  ****/
  this->Fringe.CheckCounts ();

  return OK;
 
}



/*******************************************************************
*
* tFtFastTrackLockController::tFtFastTrackLockController (tFtCompleteIoSet& CompleteIoSet)
*
* DESCRIPTION:
*
[[U tFtFastTrackLockController::tFtFastTrackLockController (CompleteIoSet) ]]
****/
tFtFastTrackLockController::tFtFastTrackLockController (
    tFtCompleteIoSet& CompleteIoSet) :
  
  /****
  [[U tFtFastTrackController::tFtFastTrackController (CompleteIoSet) ]]
  ****/
  tFtFastTrackController (CompleteIoSet)
  
{}


/*******************************************************************
*
* tFtFastTrackLockController::Reset (tServoTime)
*
* DESCRIPTION:
*
[[U tFtFastTrackLockController::Reset (tServoTime) ]]
****/
ERROR_CODE tFtFastTrackLockController::Reset (TIME tServoTime)
{


  /****
  [[U this->ApdDiagOutput.tActuator::Reset (void) ]]
  ****/ 
  this->ApdDiagOutput.Reset ();


  /****
  [[U return :=  this->Fringe.tFringe::Reset (void) ]]
  ****/ 
  return this->Fringe.Reset ();

}












