/* $RCSfile: ftGizmo.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftGizmo.cc,v 1.1.1.1 1998/06/11 21:36:23 chara Exp $
*/
/* Derek Slye - Jet Propulsion Laboratory */



#include "iostream.h"
#include "isit.h"
#include "baseObject.h"
#include "ftError.h"
#include "ftGizmo.h"
#include "ftGizmoInfo.h"
#include "ftCommands.h"
#include "modLib.h"
#include "ftTypes.h"
//#include "ftCommandEngine.h"


/*******************************************************************
*
* tFtGizmo::tFtGizmo -- tFtGizmo constructor
*
*  1. Calls tRealtimeGizmo (base class) constructor, passing it FT's GizmoInfo
*  2. Constructs the standard status type tag and status block
*  3. Constructs the Complete I/O Set
*  4. Constructs the Complete Controller Set
*  5. Assigns Controller Groups to Servo modes
*  6. Initializes servo mode to OFF
*  7. Calls tRealtimeGizmo::StartTasks() to complete initialization
*  
*
* SPEC:
      [[U tFtGizmo::tFtGizmo(GizmoInfo) ]]
*
* RETURNS:
*    nothing
*
****/
tFtGizmo::tFtGizmo (const tFtGizmoInfo& GizmoInfo) :
 
  // First call the base class constructor 
  /****
  [[U tRealtimeGizmo::tRealtimeGizmo (GizmoInfo) ]]
  ****/ 
  tRealtimeGizmo (GizmoInfo),

  // Now construct the standard status block type tag
  /****
  [[U this->CFtStatusTag.tTypeTag::tTypeTag_FTTNisz (ST_STATUS, TN_FT_STATUS, 
                GizmoInfo.iGizmoId, sizeof (tFtStatus) ) ]]
  ****/ 
  CFtStatusTag (ST_STATUS, TN_FT_STATUS, GizmoInfo.GetGizmoId (), 
                sizeof (tFtStatus) ),


  // Now construct the standard status block
  /****
  [[U this->Status.tFtStatus::tFtStatus () ]]
  ****/ 
  Status (),
 

  // Now construct the I/O set
  /****
  [[U this->CompleteIoSet.tFtCompleteIoSet::tFtCompleteIoSet (GizmoInfo) ]]
  ****/ 
  CompleteIoSet (GizmoInfo),
  

  // Now construct the controller set
  /****
  [[U this->CompleteControllerSet.tFtCompleteControllerSet::
        tFtCompleteControllerSet (CompleteIoSet, GizmoInfo) ]]
  ****/ 
  CompleteControllerSet (CompleteIoSet, GizmoInfo)  

{
    ERROR_CODE ec;
    
    // Assign Controller Groups to Servo Modes in Controller Set
    
    /****
    [[U OffGroup := this->CompleteControllerSet.tFtCompleteControllerSet::
          OffControllerGroup (void) ]]
    ****/
    tControllerGroup& OffGroup = this->CompleteControllerSet.OffControllerGroup ();
    
    /****
    [[U ec := this->ControllerSet.tControllerSet::AssignControllerGroup 
               (FSM_OFF, OffGroup) ]] 
    ****/ 
    ec = this->ControllerSet.AssignControllerGroup (FSM_OFF, OffGroup);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tFtGizmo::tFtGizmo, assign Off Group" );
      tBaseObject::SetInitError (ec);
      return;
    }
    
    /****
    [[U TrackGroup := this->CompleteControllerSet.tFtCompleteControllerSet::
          TrackControllerGroup (void) ]]
    ****/
    tControllerGroup& TrackGroup = this->CompleteControllerSet.TrackControllerGroup ();
       
    /****
    [[U ec := this->ControllerSet.tControllerSet::AssignControllerGroup 
               (FSM_TRACK, TrackGroup) ]]
    ****/ 
    ec = this->ControllerSet.AssignControllerGroup (FSM_TRACK, TrackGroup);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tFtGizmo::tFtGizmo, assign Track Group" );
      tBaseObject::SetInitError (ec);
      return;
    }
    
    // Select Off Mode
    
    /****
    [[U ec := this->Servo.tServo::SelectMode (FSM_OFF) ]] 
    ****/ 
    ec = this->Servo.SelectMode (FSM_OFF);
    
    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tFtGizmo::tFtGizmo" );
      tBaseObject::SetInitError (ec);
      return;
    }
 
    // Enable Base class tasks

    /****
    [[U ec := this->tRealtimeGizmo::StartTasks (GizmoInfo) ]] 
    ****/ 
    this->tRealtimeGizmo::StartTasks (GizmoInfo);

    if (ec != OK) {
      TraceMsg (tmt_ERROR,ec,"tFtGizmo::tFtGizmo" );
      tBaseObject::SetInitError (ec);
      return;
    }  

}  /* tFtGizmo::tFtGizmo */


/******************************************************************
*
* tFtGizmo::~tFtGizmo -- tFtGizmo destructor
*
*  Destroys tFtGizmo object.  (calls base class destructor)
*
* SPEC:
      [[U tFtGizmo::~tFtGizmo (void) ]]
*
* RETURNS:
*    nothing
*
****/
tFtGizmo::~tFtGizmo () 

  /****
     [[U tRealtimeGizmo::~tRealtimeGizmo(void) ]]
  ****/
{  
  /****
  [[U this->FtTrackControllerGroup.tModedControllerGroup::
        ~tModedControllerGroup (void) ]]

  [[U this->FtOffControllerGroup.tSimpleControllerGroup::
        ~tSimpleControllerGroup (void) ]]

  [[U this->CFtStatusTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->CFtStatusTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->Status.tFtStatus::~tFtStatus (void) ]]

  [[U this->CompleteControllerSet.tFtCompleteControllerSet::
        ~tFtCompleteControllerSet (void) ]]

  [[U this->CompleteIoSet.tFtCompleteIoSet::~tFtCompleteIoSet (void) ]]

  [[U tRealtimeGizmo::~tRealtimeGizmo (void) ]]
  ****/ 
  
}  /* tFtGizmo::~tFtGizmo */


/******************************************************************
*
* tFtGizmo::Restart 
*
*  Calls base class Restart().
*
* SPEC:
      [[U tFtGizmo::Restart(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::Restart ()
{
  /****
  [[U tRealtimeGizmo::Restart (void) ]]
  ****/
  return tRealtimeGizmo::Restart ();
  
}  /* tFtGizmo::Restart */


/******************************************************************
*
* tFtGizmo::Shutdown 
*
*  Calls base class Shutdown().
*
* SPEC:
      [[U tFtGizmo::Shutdown(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::Shutdown ()
{
  /****
  [[U tRealtimeGizmo::Shutdown (void) ]]
  ****/
  return tRealtimeGizmo::Shutdown ();
  
}  /* tFtGizmo::Shutdown */

 
/******************************************************************
*
* tFtGizmo::IsOk 
*
*  Calls base class IsOk().
*
* SPEC:
      [[U tFtGizmo::IsOk(void) ]]
*
* RETURNS:
*    BOOLEAN
*
****/
BOOL tFtGizmo::IsOk ()
{
  /****
  [[U tRealtimeGizmo::IsOk (void) ]]
  ****/
  return tRealtimeGizmo::IsOk ();
  
}  /* tFtGizmo::IsOk */


/******************************************************************
*
* tFtGizmo::operator<< 
*
*  Prints out "Fringe Tracker Gizmo" followed by results of
*     calling base class operator<<
*
* SPEC:
      [[U tFtGizmo::operator_extractor(ostream& stream, tFtGizmo& Gizmo) ]]
*
* RETURNS:
*     stream
*
****/
ostream& operator<< (ostream& stream, tFtGizmo& Gizmo)
{
  stream << "Fringe Tracker Gizmo" << endl;
  /****
  [[U tRealtimeGizmo::operator_extractor (stream,Gizmo) ]]
  ****/
  stream << (tRealtimeGizmo &) Gizmo;
  return stream;
  
}  /* tFtGizmo::operator<< */


/******************************************************************
*
* tFtGizmo::CollectStatus 
*
* Gets status info(star position, image) from rest of Fringe Tracker 
*    Gizmo, then sends it to RTE status stream
*
* SPEC:
      [[U tFtGizmo::CollectStatus(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::CollectStatus ()
{
  ERROR_CODE                ec;
  tModulationParams         WobbleParameters;
  tFtSearchParameters       SearchParameters;
  tFtThresholdParameters    Thresholds;
  tFtControllerParameters   ControllerParameters;
  tFtImageParameters        ImageParameters;
  tFringeStatus             FringeStatus;
  tFtTargetStatus           TargetStatus;
  int                       iControllerMode;
  int                       iServoMode;
  FT_CONTROLLER_MODE        fcmControllerMode;

  // Update Time
  


  /****
  [[U this->Status.tFtStatus::UpdateTime (void) ]] 
  ****/
  this->Status.UpdateTime ();
  
  
  // Update ServoParameters
  
  /****
  [[U WobbleParameters := this->CompleteIoSet.
        tFtCompleteIoSet::WobbleParameters (void) ]]
  ****/
  WobbleParameters = this->CompleteIoSet.WobbleParameters ();
  
  /****
  [[U SearchParameters := this->CompleteIoSet.
        tFtCompleteIoSet::SearchParameters (void) ]]
  ****/
  SearchParameters = this->CompleteIoSet.SearchParameters ();
  
  /****
  [[U Thresholds := this->CompleteControllerSet.
        tFtCompleteControllerSet::GetThresholds (void) ]]
  ****/
  Thresholds = this->CompleteControllerSet.GetThresholds ();
  
  /****
  [[U ControllerParameters := this->CompleteControllerSet.
        tFtCompleteControllerSet::GetControllerParameters (void) ]]
  ****/
  ControllerParameters = this->CompleteControllerSet.
      GetControllerParameters ();

  /****
  [[U this->Status.tFtStatus::UpdateServoParameters (SearchParameters,
      WobbleParameters, ControllerParameters, Thresholds) ]] 
  ****/
  this->Status.UpdateServoParameters (SearchParameters,
      WobbleParameters, ControllerParameters, Thresholds);
      
      
  // Update Image Parameters
  
  /****
  [[U ImageParameters := this->CompleteIoSet.
        tFtCompleteIoSet::ImageParameters (void) ]]
  ****/
  ImageParameters = this->CompleteIoSet.ImageParameters ();
  
  /****
  [[U this->Status.tFtStatus::UpdateImageParameters (ImageParameters) ]]
  ****/
  this->Status.UpdateImageParameters (ImageParameters);
  
  
  // Update Fringe Status
  
  /****
  [[U FringeStatus := this->CompleteIoSet.
        tFtCompleteIoSet::FringeStatus (void) ]]
  ****/
  FringeStatus = this->CompleteIoSet.FringeStatus ();
  
  /****
  [[U this->Status.tFtStatus::UpdateFringeStatus (FringeStatus) ]]
  ****/
  this->Status.UpdateFringeStatus (FringeStatus);
  

  // Update Target Status
  
  /****
  [[U TargetStatus := this->CompleteIoSet.
        tFtCompleteIoSet::Targets (void) ]]
  ****/
  TargetStatus = this->CompleteIoSet.Targets ();
  
  /****
  [[U this->Status.tFtStatus::UpdateTargetStatus (TargetStatus) ]]
  ****/
  this->Status.UpdateTargetStatus (TargetStatus);

 
  // Update Servo Mode

  /****
  [[U iServoMode := this->Servo.tServo::GetMode (void) ]]
  ****/
  iServoMode = this->Servo.GetMode ();

  this->Status.UpdateServoMode ((FT_SERVO_MODE) iServoMode);
  
  /****
  [[U iControllerMode := this->Servo.tServo::ControllerMode (void) ]]
  ****/
  iControllerMode = this->Servo.ControllerMode ();

  
  switch (iServoMode) {
    case FSM_OFF : fcmControllerMode = FCM_NONE; break;
    case FSM_TRACK :
      switch (iControllerMode) {
        case FTM_SEARCH : fcmControllerMode = FCM_SEARCH; break;
	case FTM_SEMI_LOCK : fcmControllerMode = FCM_SEMI_LOCK; break;
        case FTM_LOCK : fcmControllerMode = FCM_LOCK; break;
        default : fcmControllerMode = FCM_UNKNOWN;
      }
      break;
    default : fcmControllerMode = FCM_UNKNOWN;
  }

  /****
  [[U this->Status.tFtStatus::UpdateControllerMode (fcmControllerMode) ]] 
  ****/
  this->Status.UpdateControllerMode (FT_CONTROLLER_MODE (fcmControllerMode));  

  // Send Status Type Tag
  
  /****
  [[U ec := tGizmo::SendStatus_tp (CFtStatusTag, &this->Status) ]]
  ****/
  ec = this->SendStatus (this->CFtStatusTag, (BYTE *) &this->Status);
  
  /****ControllerSet
  [[U ec := tGizmo::SendError_ec (S_ft_UNABLE_TO_SEND_STATUS) ]]
  ****/
  if (ec != OK)  this->SendError (S_ft_UNABLE_TO_SEND_STATUS);
  
  
  // Tell I/O to send Fringe Image Status Tag and Fringe Image
   

  /****
  [[U this->CompleteIoSet.tFtCompleteIoSet::SendDispersedFringeImage (void) ]]
  ****/
  ec = this->CompleteIoSet.SendDispersedFringeImage ();   

  /****
  [[U ec := tGizmo::SendError_ec (S_ft_UNABLE_TO_SEND_IMAGE) ]]
  ****/
  if (ec != OK)  this->SendError (S_ft_UNABLE_TO_SEND_IMAGE);
  
  return OK;
  
}  /* tFtGizmo::CollectStatus */

/******************************************************************
*
* tFtGizmo::Dispatch
*
* Calls the appropriate tFtGizmo method that corresponds to the
*    IsitCmd called Command.  
*
* SPEC:
     [[U tFtGizmo::Dispatch(Command) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::Dispatch (const tIsitCmd& Command)
{
  switch (Command.Command) {

    case FT_CMD_PING : 
      /****
      [[U  tFtGizmo::Ping (void) ]]
      ****/
      return this->Ping ();

    case FT_CMD_OFF : 
      /****
      [[U  tFtGizmo::Off (void) ]]
      ****/
      return this->Off ();

    case FT_CMD_TRACK : 
      /****
      [[U  tFtGizmo::Track (void) ]]
      ****/
      return this->Track ();
    
    case FT_CMD_CALIBRATE :
      /****
      [[U  tFtGizmo::Calibrate (void) ]]
      ****/
      return this->Calibrate ();
    
    case FT_CMD_SET_MT :
      /****
      [[U  tFtGizmo::SetManualTarget ((tFtSetManualTargetCmd&) Command) ]]
      ****/
      return this->SetManualTarget ( (const tFtSetManualTargetCmd&) Command);
      
    case FT_CMD_GET_IMAGE : 
      /****
      [[U  tFtGizmo::GetImage (void) ]]
      ****/
      return this->GetImage ();

    case FT_CMD_CHNG_PARAM :  
      /****
      [[U  tFtGizmo::SetServoParameters ((tFtSetServoParametersCmd&) Command) ]]
      ****/
      return this->SetServoParameters (
                              (const tFtSetServoParametersCmd&) Command);

    case FT_CMD_SET_ROI :
      /****
      [[U  tFtGizmo::SetRegionOfInterest ((tFtSetRegionOfInterestCmd&) Command) ]]
      ****/
      return this->SetRegionOfInterest ( 
                              (const tFtSetRegionOfInterestCmd&) Command);
    
   default : return S_ft_UNRECOGNIZED_COMMAND;
  };
  return OK;
  
}  /* tFtGizmo::Dispatch */
  
    
/******************************************************************
*
* tFtGizmo::Ping
*
*  Called in response to the FT receiving a PING command.  Sends a 
*  friendly message to the RTE error stream.
*
* SPEC:
     [[U  tFtGizmo::Ping(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::Ping ()
{ 
   /****
   [[U  tGizmo::SendError_ec (S_ft_FRINGE_TRACKER_ALIVE) ]]
   ****/
   this->SendError (S_ft_FRINGE_TRACKER_ALIVE);
   
   return OK;
   
}  /* tFtGizmo::Ping */

   
/******************************************************************
*
* tFtGizmo::Off
*
* Called in response to the FT receiving an OFF command.  Creates
* an Off Command Engine and calls its Execute method.
*
* SPEC:
     [[U tFtGizmo::Off(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::Off ()
{
  ERROR_CODE ec;
  
  /****
  [[U this->Servo.tServo::SelectMode (FSM_OFF) ]]
  ****/
  ec = this->Servo.SelectMode (FSM_OFF);
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::Off");

  /****
  [[U this->Status.tFtStatus::UpdateCommandStatus (FTCS_OFF) ]]
  ****/
  this->Status.UpdateCommandStatus (FTCS_OFF);
  
  /****
  [[U this->tGizmo::SendError (S_ft_OFF) ]]
  ****/
  this->SendError (S_ft_OFF);
  
  return ec;
  
}  /* tFtGizmo::Off */


/******************************************************************
*
* tFtGizmo::Track
*
* Called in response to the FT receiving a TRACK command.  Creates
* a TRACK Command Engine and calls its Execute method.
*
* SPEC:
     [[U  tFtGizmo::Track(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFtGizmo::Track ()
{
  ERROR_CODE ec;
  
  /****
  [[U this->Servo.tServo::SelectMode (FSM_TRACK) ]]
  ****/
  ec = this->Servo.SelectMode (FSM_TRACK);
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::Track");
  
  /****
  [[U this->Status.tFtStatus::UpdateCommandStatus (FTCS_TRACK_ACTIVE) ]]
  ****/
  this->Status.UpdateCommandStatus (FTCS_TRACK_ACTIVE);

  /****
  [[U this->tGizmo::SendError (S_ft_TRACK_IN_PROGRESS) ]]
  ****/
  this->SendError (S_ft_TRACK_IN_PROGRESS);
   
  return ec; 
  
}  /* tFtGizmo::Track */


/******************************************************************
*
* tFtGizmo::Calibrate
*
* Called in response to the FT receiving a CALIBRATE command.  Stubbed
* in Increment 4.
*
* SPEC:
     [[U  tFtGizmo::Calibrate(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFtGizmo::Calibrate ()
{
  return OK;

}  /* tFtGizmo::Calibrate */


/******************************************************************
*
* tFtGizmo::SetManualTarget 
*
* Called in response to the FT receiving a SET_MT command.  Extracts delay
* line position (a double) from the IsitCmd and passes it to 
* CompleteControllerSet.SetManualTarget().
*
* SPEC:
     [[U  tFtGizmo::SetManualTarget(Command) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFtGizmo::SetManualTarget (const tFtSetManualTargetCmd& Command)
{

  return this->CompleteIoSet.SetManualTarget (Command.ftdTargetDirection);

}  /* tFtGizmo::SetManualTarget */


/******************************************************************
*
* tFtGizmo::GetImage
*
* Called in response to the FT receiving an GET_IMAGE command.  Creates
* an GetImage Command Engine and calls its Execute method.
*
* SPEC:
      [[U tFtGizmo::GetImage(void) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/ 
ERROR_CODE tFtGizmo::GetImage () {

  ERROR_CODE ec;

  /****
  [[U this->Status.tFtStatus::UpdateCommandStatus (FTCS_GET_IMAGE) ]]
  ****/
  this->Status.UpdateCommandStatus (FTCS_GET_IMAGE);
  
  /****
  [[U ec := this->tGizmo::SendError (S_ft_GET_IMAGE) ]]
  ****/
  this->SendError (S_ft_GET_IMAGE);

  if (this->Servo.GetMode () != FSM_TRACK) {

    ec = this->CompleteIoSet.SendAtomicFullFrameImage ();
  }

  else ec = this->CompleteIoSet.SendFullFrameImage ();

  if (ec != OK) return Traceback (ec, "tFtGizmo::GetImage");

  return ec; 
  
}  /* tFtGizmo::GetImage */


/******************************************************************
*
* tFtGizmo::SetServoParameters
*
* Called in response to the FT receiving an SET_SERVO_PARAMETERS command. 
* Extracts wobble modulation parameters, spiral-search modulation parameters,
* controller mode switching thresholds, and controller parameters from 
* the IsitCmd and passes them to CompleteControllerSet.SetServoParameters().
*
* SPEC:
      [[U tFtGizmo::SetServoParameters(Command) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::SetServoParameters (const tFtSetServoParametersCmd& Command)
{
  ERROR_CODE ec;
  tModulationParams WobbleParametersToUse;
  tFtSearchParameters SearchParametersToUse;
  tFtThresholdParameters ThresholdsToUse;
  
  // We want to preserve the non-GUI-settable parameters, so copy them in first
  tFtControllerParameters ControllerParametersToUse (
           this->CompleteControllerSet.GetControllerParameters ());
  
  /****
  [[U tGizmo::SendError_ec (S_emgizmo_SETTING_SERVO_PARAMETERS) ]]
  ****/
  SendError (S_ft_SERVO_PARAMETERS_SET);
  
  // Set Wobble Parameters
    
  WobbleParametersToUse.mtModulationType = Command.mtModulationType; 
  WobbleParametersToUse.dFrequency = Command.dFrequency; 
  WobbleParametersToUse.dAmplitude = Command.dAmplitude; 
  WobbleParametersToUse.dMeanVal = Command.dMeanVal; 
  WobbleParametersToUse.tSyncTime = Command.tSyncTime; 
  WobbleParametersToUse.dDutyCycle = Command.dDutyCycle; 

  ec = this->CompleteIoSet.SetWobbleParameters (WobbleParametersToUse);
  
  if (ec == OK) {

    /****
    [[U tGizmo::SendError_ec (S_ftgizmo_SERVO_PARAMETERS_SET) ]]
    ****/
    SendError (S_ft_WOBBLE_PARAMETERS_SET);
  }
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::SetServoParameters");
  
  
  // Set Search Parameters

  SearchParametersToUse.dSpiralRadiusInitial = Command.dSpiralRadiusInitial;
  SearchParametersToUse.dSpiralRadiusLimit = Command.dSpiralRadiusLimit;
  SearchParametersToUse.dSpiralStep = Command.dSpiralStep;
  SearchParametersToUse.dProgressionConstant = Command.dProgressionConstant;
  SearchParametersToUse.stSearchType = Command.stSearchType;  

  ec = this->CompleteIoSet.SetSearchParameters (SearchParametersToUse);
  
  if (ec == OK) {

   /****
   [[U tGizmo::SendError_ec (S_ftgizmo_SERVO_PARAMETERS_SET) ]]
   ****/
    SendError (S_ft_SEARCH_PARAMETERS_SET);
  }
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::SetServoParameters");

  
  // Set Threshold Parameters
  
  ThresholdsToUse.tSemiLockToLockTimeout = Command.tSemiLockToLockTimeout;
  ThresholdsToUse.tTotalTimeToLockTimeout = Command.tTotalTimeToLockTimeout;
  ThresholdsToUse.dSNRThreshSearchToSemiLock = Command.dSNRThreshSearchToSemiLock;
  ThresholdsToUse.dSNRThreshSemiLockToLock = Command.dSNRThreshSemiLockToLock;
  ThresholdsToUse.dV2ThreshSemiLockToLock = Command.dV2ThreshSemiLockToLock;
  ThresholdsToUse.dSNRThreshLockToSearch = Command.dSNRThreshLockToSearch;
  ThresholdsToUse.dThreshForDrop = Command.dThreshForDrop;
  ThresholdsToUse.dThreshForSearch = Command.dThreshForSearch;

  ec = this->CompleteControllerSet.SetThresholds (ThresholdsToUse);
  
  if (ec == OK) {

   /****
   [[U tGizmo::SendError_ec (S_ftgizmo_SERVO_PARAMETERS_SET) ]]
   ****/
    SendError (S_ft_THRESHOLD_PARAMETERS_SET);
  }
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::SetServoParameters");

  // Set only those Controller Parameters settable from the GUI (there are others)
  
  ControllerParametersToUse.iMemBoxcar = Command.iMemBoxcar;
  ControllerParametersToUse.dFringeServoGain = Command.dFringeServoGain;
  ControllerParametersToUse.dDeltaVDeltaWobbleToHop = Command.dDeltaVDeltaWobbleToHop;
  ControllerParametersToUse.iPhaseSign = Command.iPhaseSign;
  ControllerParametersToUse.dPhaseOffset = Command.dPhaseOffset;
  
  ec = this->CompleteControllerSet.SetControllerParameters (ControllerParametersToUse);
   
  if (ec == OK) {

    /****
    [[U tGizmo::SendError_ec (S_ftgizmo_SERVO_PARAMETERS_SET) ]]
    ****/
    SendError (S_ft_CONTROLLER_PARAMETERS_SET);
  }
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::SetServoParameters");

  return OK;  
  
}  /* tFtGizmo::SetServoParameters */


/******************************************************************
*
* tFtGizmo::SetRegionOfInterest
*
* Called in response to the FT receiving an SET_REGION_OF_INTEREST command.  
* Extracts ImageParameters from the IsitCmd and passes them to
* CompleteControllerSet.SetRegionOfInterest().
*
* SPEC:
      [[U tFtGizmo::SetRegionOfInterest(Command) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtGizmo::SetRegionOfInterest (const tFtSetRegionOfInterestCmd& Command)
{
  ERROR_CODE ec;
  
  /****
  [[U ec := this->CompleteIoSet.tFtCompleteIoSet::
              SetRegionOfInterest (Command.ImageParameters) ]]
  ****/
  ec = this->CompleteIoSet.SetRegionOfInterest (Command.ImageParameters);


  if (ec == OK) {

    /****
    [[U tGizmo::SendError_ec (S_ft_REGION_OF_INTEREST_SET) ]]
    ****/
    SendError (S_ft_REGION_OF_INTEREST_SET);
  }
  
  if (ec != OK) return Traceback (ec, "tFtGizmo::SetRegionOfInterest");

  return ec;  
  
}  /* tFtGizmo::SetRegionOfInterest */

