/* $RCSfile: ftGizmoInfo.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftGizmoInfo.cc,v 1.2 1998/08/17 21:00:54 phil Exp $
*/

/* Derek Slye - Jet Propulsion Laboratory */



#include "ftGizmoInfo.h"
#include "dlGizmoInfo.h"
#include "configMgr.h"
#include "stGizmoInfo.h"

#define FULL_IMAGE_HSIZE   640
#define FULL_IMAGE_VSIZE   480



/*******************************************************************
*
* tFtGizmoInfo::tFtGizmoInfo -- tFtGizmoInfo constructor
*
*  Calls tRealtimeGizmoInfo (base class) constructor.  Initializes Physical Constants,
*  ServoParameters, and ImageParameters.  Assigns gtGizmoType (from base class) 
*  the value GT_FRINGE_TRACKER, and passed in id's to DL, ST0, and ST1.
*  Initializes elements of ImageParameters to zero.
*
[[U tFtGizmoInfo::tFtGizmoInfo (iRteId, iGizmoId, iNumberOfServoModes,
      ServoInfo, PhysicalConstantsToUse, ServoParametersToUse, iClosedLoopDlIdToUse, 
      iSt0ImageGizmoId, iSt1ImageGizmoId, ImageParametersToUse,
      bnApdBoard, iApdChannel, bnApdDiagBoard, iApdDiagChannel, 
      bnFrameGrabber, iFullImageChannel) ]]
****/                      
tFtGizmoInfo::tFtGizmoInfo (int iRteId, int iGizmoId, int iNumberOfServoModes,
      const tServoInfo& ServoInfo, const tFtPhysicalConstants& PhysicalConstantsToUse,
      const tFtServoParameters& ServoParametersToUse, int iClosedLoopDlIdToUse, 
      int iSt0ImageGizmoId, int iSt1ImageGizmoId,
      const tFtImageParameters& ImageParametersToUse,
      BOARD_NAME bnApdBoard, int iApdChannel,
      BOARD_NAME bnApdDiagBoard, int iApdDiagChannel,
      BOARD_NAME bnFrameGrabber, int iFullImageChannel) :

  /****
  [[U tRealtimeGizmoInfo::tRealtimeGizmoInfo (iRteId,iGizmoId,
       iNumberOfServoMode,ServoInfo) ]]
  ****/                            
  tRealtimeGizmoInfo (iRteId, iGizmoId, iNumberOfServoModes,
                      ServoInfo),
  
  // Initialize Physical Constants
  
  PhysicalConstants (PhysicalConstantsToUse),
  
  // Initialize Servo Parameters
  
  ServoParameters (ServoParametersToUse),
  
  // Initialize Image Parameters
  
  ImageParameters (ImageParametersToUse)

{  
 
  this->gtGizmoType = GT_FRINGE_TRACKER;
  this->iClosedLoopDlId = iClosedLoopDlIdToUse;
  this->iSt0Id = iSt0ImageGizmoId;
  this->iSt1Id = iSt1ImageGizmoId;
  this->pApdChannel = D_CAST(tInputChannel *)
                        ConfigManager.Hardware.CreateChannel
                          (bnApdBoard, iApdChannel); 

  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pApdChannel is %ld\n",(int) this->pApdChannel);

  this->pApdDiagChannel = D_CAST(tOutputChannel *)
                        ConfigManager.Hardware.CreateChannel
                          (bnApdDiagBoard, iApdDiagChannel); 

  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pApdDiagChannel is %ld\n",(int) this->pApdDiagChannel);

  this->pFullImageChannel =  D_CAST(tFullImageChannel *)
                        ConfigManager.Hardware.CreateChannel
                          (bnFrameGrabber, iFullImageChannel );  

  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pFullImageChannel is %ld\n",(int) this->pFullImageChannel);
            
//  this->iTrackUpdatePriority = CiFtTrackUpdatePriority;
//  this->tTrackUpdatePeriod = CtFtTrackUpdatePeriod;
  this->tTrackUpdateOffset = 0;

}


/*******************************************************************
*
* tFtGizmoInfo::WhiteLightLambda -- get White Light Lambda value
*
*  Get laWhiteLightLambda parameter from this->PhysicalConstants
*
* SPEC:
      [[U tFtGizmoInfo::WhiteLightLambda (void) ]]
*
* RETURNS:
*    LASER_VALUE
*
****/    
LASER_VALUE tFtGizmoInfo::WhiteLightLambda () const
{

	return this->PhysicalConstants.lvWhiteLightLambda;
	
}


/*******************************************************************
*
* tFtGizmoInfo::WobbleParameters -- get Wobble Parameters
*
*  Return WobbleParameters from this->ServoParameters
*
* SPEC:
      [[U tFtGizmoInfo::WobbleParameters (void) ]]
*
* RETURNS:
*    tModulationParams
*
****/    
tModulationParams tFtGizmoInfo::WobbleParameters () const
{

	return this->ServoParameters.WobbleParameters;
	
}


/*******************************************************************
*
* tFtGizmoInfo::SearchParameters -- get Search Parameters
*
*  Return SearchParameters from this->ServoParameters
*
* SPEC:
      [[U tFtGizmoInfo::SearchParameters (void) ]]
*
* RETURNS:
*    tFtSearchParameters
*
****/    
tFtSearchParameters tFtGizmoInfo::SearchParameters () const
{

	return this->ServoParameters.SearchParameters;
	
}


/*******************************************************************
*
* tFtGizmoInfo::ControllerParameters -- get Controller Parameters
*
*  Return ControllerParameters from this->ServoParameters
*
* SPEC:
      [[U tFtGizmoInfo::ControllerParameters (void) ]]
*
* RETURNS:
*   tFtControllerParameters
*
****/    
tFtControllerParameters tFtGizmoInfo::ControllerParameters () const
{

	return this->ServoParameters.ControllerParameters;
	
}


/*******************************************************************
*
* tFtGizmoInfo::ThresholdParameters -- get Threshold Parameters
*
*  Return ThresholdParameters from this->ServoParameters
*
* SPEC:
      [[U tFtGizmoInfo::ThresholdParameters (void) ]]
*
* RETURNS:
*    ThresholdParameters
*
****/    
tFtThresholdParameters tFtGizmoInfo::Thresholds () const
{

	return this->ServoParameters.ThresholdParameters;
	
}


/*******************************************************************
*
* tFtGizmoInfo::NumBoxcarSamples -- get sample size of the boxcar filter
*
*  Return the number of dither periods over which fringe SNR
*  data is calculated.
*
* SPEC:
      [[U tFtGizmoInfo::NumBoxcarSamples (void) ]]
*
* RETURNS:
*    int
*
****/    
int tFtGizmoInfo::NumBoxcarSamples () const
{

	return this->ServoParameters.ControllerParameters.iMemBoxcar;
	
}


/*******************************************************************
*
* tFtGizmoInfo::NumWobbleSamples -- What is this?????
*
*  
*
* SPEC:
      [[U tFtGizmoInfo::NumWobbleSamples (void) ]]
*
* RETURNS:
*    int
*
****/    
int tFtGizmoInfo::NumWobbleSamples () const
{

	return this->PhysicalConstants.iNumberOfWobbleSamples;
	
}


/*******************************************************************
*
* tFtGizmoInfo::ServoPeriod -- get period of a given Servo Partition
*
*  Given a servo partition in FT_SERVO_PARTITION, return its period
*
* SPEC:
      [[U tFtGizmoInfo::ServoPeriod (void) ]]
*
* RETURNS:
*    TIME
*
****/    
TIME tFtGizmoInfo::ServoPeriod (FT_SERVO_PARTITION fsmPartition) const
{

  /****
  [[U TaskInfo := this->ServoInfo.tServoInfo::GetServoTaskInfo (fsmPartition) ]]
  ****/
  const tServoTaskInfo& TaskInfo = this->ServoInfo.GetServoTaskInfo (fsmPartition);
  
  return TaskInfo.tTaskPeriod;
 	
}


/*******************************************************************
*
* tFtGizmoInfo::ServoPriority -- get priorty of a given Servo Partition
*
*  Given a servo priorty in FT_SERVO_PARTITION, return its period
*
* SPEC:
      [[U tFtGizmoInfo::ServoPriority (void) ]]
*
* RETURNS:
*    int
*
****/    
int tFtGizmoInfo::ServoPriority (FT_SERVO_PARTITION fsmPartition) const
{

  /****
  [[U TaskInfo := this->ServoInfo.tServoInfo::GetServoTaskInfo (fsmPartition) ]]
  ****/
  const tServoTaskInfo& TaskInfo = this->ServoInfo.GetServoTaskInfo (fsmPartition);
  
  return TaskInfo.iTaskPriority;
}


/*******************************************************************
*
* tFtGizmoInfo::ClosedLoopDlTargetId -- get target buffer ID from Delay Line
*
*  Gets Delay Line Gizmo ID from FT's GizmoInfo.  This ID is used to contact
*  the Delay Line (via ConfigManager) and access its GetFtTargetId.
*
* SPEC:
      [[U tFtGizmoInfo::ClosedLoopDlTargetId (void) ]]
*
* RETURNS:
*    tIpcDblBufferId
*
****/                           
tIpcDblBufferId tFtGizmoInfo::ClosedLoopDlTargetId () const
{


  /****
  [[U GizmoInfo := *ConfigManager.tConfig::GetGizmoInfo (this->iDlGizmoId) ]]
  ****/                            
  tDlGizmoInfo &GizmoInfo = 
     (tDlGizmoInfo &) *ConfigManager.GetGizmoInfo (this->iClosedLoopDlId);
  
  /****
  [[U GizmoInfo.tDlGizmoInfo::GetFtTargetId (void) ]]
  ****/                            
  return GizmoInfo.GetFtTargetId ();

}


/*******************************************************************
*
* tFtGizmoInfo::StarImageId -- get image buffer ID from Star Tracker
*
*  Takes Star Tracker index (0 or 1) and passes it to tFtGizmoInfo::GetStGizmoInfo.
*  The resulting Star Tracker GizmoInfo object is then asked for its IPC
*  Indirection Double Buffer ID.
*
* SPEC:
      [[U tFtGizmoInfo::StarImageId (iWhichSt) ]]
*
* RETURNS:
*    tIpcIndirectionDblBufferId
*
****/    
tIpcIndirectionDblBufferId tFtGizmoInfo::StarImageId (int iWhichSt) const
{

	return (GetStGizmoInfo (iWhichSt)).GetStarImageId ();
	
}


/*******************************************************************
*
* tFtGizmoInfo::StarXStart -- get fringe's horizontal starting position
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::StarXStart (void) ]]
*
* RETURNS:
*    int
*
****/  
int tFtGizmoInfo::StarXStart (int iWhichSt) const
{
 
  switch (iWhichSt) {
    case 0 : return this->ImageParameters.iStartXSt0;
    case 1 : return this->ImageParameters.iStartXSt1;
    default :
      { // fix so that return correct gizmo info for invalid st #
      TraceMsg (tmt_ERROR, iWhichSt, "tFtGizmoInfo::StarXStart");
      return this->ImageParameters.iStartXSt1;
      } 
  }

}
  

/*******************************************************************
*
* tFtGizmoInfo::StarYStart -- get fringe's vertical starting position
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::StarYStart (void) ]]
*
* RETURNS:
*    int
*
****/   
int tFtGizmoInfo::StarYStart (int iWhichSt) const
{  
  
   switch (iWhichSt) {
    case 0 : return this->ImageParameters.iStartYSt0;
    case 1 : return this->ImageParameters.iStartYSt1;
    default :
      { // fix so that return correct gizmo info for invalid st #
      TraceMsg (tmt_ERROR, iWhichSt, "tFtGizmoInfo::StarYStart");
      return this->ImageParameters.iStartYSt1;
      } 
   }

}


/*******************************************************************
*
* tFtGizmoInfo::StarHSize -- request star image width from Star Tracker
*
*  Takes Star Tracker index (0 or 1) and passes it to tFtGizmoInfo::GetStGizmoInfo.
*  The resulting Star Tracker GizmoInfo object is then asked for the 
*  horizontal size of the star image it will be receiving.
*
* SPEC:
      [[U tFtGizmoInfo::StarHSize (iWhichSt) ]]
*
* RETURNS:
*    int
*
****/   
int tFtGizmoInfo::StarHSize (int iWhichSt) const
{
  
  const tStGizmoInfo& TempInfo = this->GetStGizmoInfo (iWhichSt);
  
  return TempInfo.GetStarImageHSize ();
  
}


/*******************************************************************
*
* tFtGizmoInfo::StarVSize -- request star image height from Star Tracker
*
*  Takes Star Tracker index (0 or 1) and passes it to tFtGizmoInfo::GetStGizmoInfo.
*  The resulting Star Tracker GizmoInfo object is then asked for the 
*  vertical size of the star image it will be receiving.
*
* SPEC:
      [[U tFtGizmoInfo::StarVSize (iWhichSt) ]]
*
* RETURNS:
*    int
*
****/ 
int tFtGizmoInfo::StarVSize (int iWhichSt) const
{

  const tStGizmoInfo& TempInfo = this->GetStGizmoInfo (iWhichSt);
  
  return TempInfo.GetStarImageVSize ();
  
}


/*******************************************************************
*
* tFtGizmoInfo::GetStGizmoInfo -- request Star Tracker's GizmoInfo object
*
*  Takes Star Tracker index (0 or 1) and chooses appropriate GizmoId from
*  the FT GizmoInfo.  This ID is used to contact the corresponding
*  Star Tracker (via ConfigManager) and access its GetGizmoInfo method.
*  The resulting GizmoInfo object is then returned.
*
* SPEC:
      [[U tFtGizmoInfo::GetStGizmoInfo (iWhichSt) ]]
*
* RETURNS:
*    const tStGizmoInfo&
*
****/ 
const tStGizmoInfo& tFtGizmoInfo::GetStGizmoInfo (int iWhichSt) const
{
	
  int iStId;
	
  switch (iWhichSt) {
    case 0 : iStId = this->iSt0Id; break;
    case 1 : iStId = this->iSt1Id; break;
    default :
      { // fix so that return correct gizmo info for invalid st #
      TraceMsg (tmt_ERROR, iWhichSt, "tFtGizmoInfo::GetStGizmoInfo()");
      iStId = this->iSt1Id; // just pick #1 in error case
      }
  }
		
  tStGizmoInfo& GizmoInfo = (tStGizmoInfo&) *ConfigManager.GetGizmoInfo (iStId);
	
  return GizmoInfo;
}


/*******************************************************************
*
* tFtGizmoInfo::DispersedFringeXStart -- get fringe's horizontal starting position
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::DispersedFringeXStart (void) ]]
*
* RETURNS:
*    int
*
****/  
int tFtGizmoInfo::DispersedFringeXStart () const
{
  
  return this->ImageParameters.iStartXFringeImage;
  
}
  

/*******************************************************************
*
* tFtGizmoInfo::DispersedFringeYStart -- get fringe's vertical starting position
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::DispersedFringeYStart (void) ]]
*
* RETURNS:
*    int
*
****/   
int tFtGizmoInfo::DispersedFringeYStart () const
{  
  
  return this->ImageParameters.iStartYFringeImage;

}


/*******************************************************************
*
* tFtGizmoInfo::DispersedFringeHSize -- get fringe image width
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::DispersedFringeHSize (void) ]]
*
* RETURNS:
*    int
*
****/  
int tFtGizmoInfo::DispersedFringeHSize () const
{
  
  return this->ImageParameters.iFringeImageHSize;
  
} 
  

/*******************************************************************
*
* tFtGizmoInfo::DispersedFringeVSize -- get fringe image height
*
*  Returns the integer value stored in ImageParameters.
*
* SPEC:
      [[U tFtGizmoInfo::DispersedFringeVSize (void) ]]
*
* RETURNS:
*    int
*
****/   
int tFtGizmoInfo::DispersedFringeVSize () const
{  
  
  return this->ImageParameters.iFringeImageVSize;

}


/*******************************************************************
*
* tFtGizmoInfo::FullFrameHSize -- get full frame image width
*
*  Returns the integer value FULL_IMAGE_HSIZE.
*
* SPEC:
      [[U tFtGizmoInfo::FullFrameHSize (void) ]]
*
* RETURNS:
*    int
*
****/ 
int tFtGizmoInfo::FullFrameHSize () const
{
  
  return FULL_IMAGE_HSIZE;

}


/*******************************************************************
*
* tFtGizmoInfo::FullFrameVSize -- get full frame image height
*
*  Returns the integer value FULL_IMAGE_VSIZE.
*
* SPEC:
      [[U tFtGizmoInfo::FullFrameVSize (void) ]]
*
* RETURNS:
*    int
*
****/ 
int tFtGizmoInfo::FullFrameVSize () const
{

  return FULL_IMAGE_VSIZE;
  
}


/*******************************************************************
*
* tFtGizmoInfo::ApdScaleFactor -- get Avalanche PhotoDetector scale factor
*
*  Returns this->PhysicalConstants.dApdScaleFactor
*
* SPEC:
      [[U tFtGizmoInfo::ApdScaleFactor (void) ]]
*
* RETURNS:
*    double
*
****/ 
double tFtGizmoInfo::ApdScaleFactor () const
{

  return this->PhysicalConstants.dApdScaleFactor;
  
}


/*******************************************************************
*
* tFtGizmoInfo::ApdChannel -- get hardware channel of APD
*
*  Returns this->pApdChannel
*
* SPEC:
      [[U tFtGizmoInfo::ApdChannel (void) ]]
*
* RETURNS:
*    pointer to tInputChannel
*
****/ 
tInputChannel* tFtGizmoInfo::ApdChannel () const
{

  return this->pApdChannel;
  
}


/*******************************************************************
*
* tFtGizmoInfo::ApdDiagScaleFactor -- get Avalanche PhotoDetector scale factor
*
*  Returns this->PhysicalConstants.dApdScaleFactor
*
[[U tFtGizmoInfo::ApdDiagScaleFactor (void) ]]
****/ 
double tFtGizmoInfo::ApdDiagScaleFactor () const
{

  return this->PhysicalConstants.dApdDiagScaleFactor;
  
}


/*******************************************************************
*
* tFtGizmoInfo::ApdDiagChannel -- get hardware channel of APD
*
*  Returns this->pApdDiagChannel
*
[[U tFtGizmoInfo::ApdChannel (void) ]]
****/ 
tOutputChannel* tFtGizmoInfo::ApdDiagChannel () const
{

  return this->pApdDiagChannel;
  
}




/*******************************************************************
*
* tFtGizmoInfo::FullImageChannel -- get hardware channel of video subsystem
*
*  Returns this->pFullImageChannel
*
* SPEC:
      [[U tFtGizmoInfo::FullImageChannel (void) ]]
*
* RETURNS:
*    pointer to tFullImageChannel
*
****/ 
tFullImageChannel* tFtGizmoInfo::FullImageChannel () const
{

  return this->pFullImageChannel;
  
}


/*******************************************************************
*
* tFtGizmoInfo::TrackUpdatePeriod -- period of controller mode switching decision
*
*  Returns this->tTrackUpdatePeriod
*
* SPEC:
      [[U tFtGizmoInfo::TrackUpdatePeriod (void) ]]
*
* RETURNS:
*    TIME
*
****/ 
TIME tFtGizmoInfo::TrackUpdatePeriod () const
{

  return this->tTrackUpdatePeriod;
  
}


/*******************************************************************
*
* tFtGizmoInfo::TrackUpdatePriority -- task priority of controller mode switching decision
*
*  Returns this->tTrackUpdatePriority
*
* SPEC:
      [[U tFtGizmoInfo::TrackUpdatePriority (void) ]]
*
* RETURNS:
*    int
*
****/ 
int tFtGizmoInfo::TrackUpdatePriority () const
{

  return this->iTrackUpdatePriority;
  
}


/*******************************************************************
*
* tFtGizmoInfo::TrackUpateOffset -- What is this????
*
*  Returns this->tTrackUpateOffset
*
* SPEC:
      [[U tFtGizmoInfo::TrackUpateOffset (void) ]]
*
* RETURNS:
*    TIME
*
****/ 
TIME tFtGizmoInfo::TrackUpateOffset () const
{

  return this->tTrackUpdateOffset;
  
}
