/* $RCSfile: ftImagingController.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftImagingController.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Derek Slye - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#include "ft/ftImagingController.h"
#include "ft/ftIoSet.h"
#include "ipcid.h"
#include "ftGizmoInfo.h"
#include "ftTypes.h"
#include "imageStatus.h"
#include "image.h"
#include "stActuator.h"

/*******************************************************************
*
* tFtImagingController::tFtImagingController 
*
*
[[U tFtImagingController::tFtImagingController (CompleteIoSet) ]]
****/
tFtImagingController::tFtImagingController (tFtCompleteIoSet& CompleteIoSet) :

  /****
  [[U tFtController::tFtController (void) ]]
  ****/ 
  tFtController (),
  
  /****
  [ this->FullFrame := reference to CompleteIoSet.FullFrame
    : this->StarImage0 := reference to CompleteIoSet.StarImage0
    : this->StarImage1 := reference to CompleteIoSet.StarImage1
    : this->StarTracker0 := reference to CompleteIoSet.StarTracker0
    : this->StarTracker1 := reference to CompleteIoSet.StarTracker1
  ]
  ****/
  FullFrame (CompleteIoSet.FullFrame),
  Star0 (CompleteIoSet.Star0),
  Star1 (CompleteIoSet.Star1),
  DispersedFringe (CompleteIoSet.DispersedFringe),
  StarTracker0 (CompleteIoSet.StarTracker0),
  StarTracker1 (CompleteIoSet.StarTracker1) ,
  DispersedFringeImage (CompleteIoSet.DispersedFringeImage)
{  
                 
}
                                    

/*******************************************************************
*
* tFtImagingController::Calculate -- the heart of the ImagingController
*
*  Updates the video frame buffer via the FullFrameSensor.Update method.
*  Update is called on each of the star image sensors and the resulting
*  images are passed to the star tracker actuators; lastly, the actuator's 
*  Output methods are called to send the images on their way.
*
[[U tFtImagingController::Calculate (tServoTime) ]]
****/                                         
ERROR_CODE tFtImagingController::Calculate (TIME tServoTime)
{

  tImage St0Image, St1Image, FringeImage;
  ERROR_CODE ec;

  /****
  [[U ec := this->FullFrame.tFullFrameSensor::Update (void) ]]
  ****/
  ec = FullFrame.Update();
  
  if (ec != OK) return Traceback (ec, "tFtImagingController::Calculate");  
  
  /****
  [[U this->Star0.tImageSensor::GetImage (St0Image) ]]
  ****/
  ec = this->Star0.GetImage (St0Image);
  
  if (ec != OK) return Traceback (ec, "tFtImagingController::Calculate");  
   
  /****
  [[U this->Star1.tImageSensor::GetImage (St1Image) ]]
  ****/
  ec = this->Star1.GetImage (St1Image);
  
  if (ec != OK) return Traceback (ec, "tFtImagingController::Calculate");  
     
  /****
  [[U this->StarTracker0.tStActuator::SetImage (St0Image) ]]
  ****/
  this->StarTracker0.SetImage (St0Image);
  
  /****
  [[U this->StarTracker0.tStActuator::Ouput (tServoTime) ]]
  ****/
  ec = this->StarTracker0.Output (tServoTime);
  
  if (ec != OK) return Traceback (ec, "tFtImagingController::Calculate"); 
   
  /****
  [[U this->StarTracker1.tStActuator::SetImage (St1Image) ]]
  ****/
  this->StarTracker1.SetImage (St1Image);

  /****
  [[U this->StarTracker1.tStActuator::Ouput (tServoTime) ]]
  ****/
  ec = this->StarTracker1.Output (tServoTime);
  
  if (ec != OK) return Traceback (ec, "tFtImagingController::Calculate"); 
   
  return OK;

}


/*******************************************************************
*
* tFtImagingController::Init 
*
* DESCRIPTION:
  Reverts to default dispersed fringe image pattern  
*
[[U tFtImagingController::Init (tServoTime) ]]
*
*
****/
ERROR_CODE tFtImagingController::Init (TIME tServoTime) 
{

  /****
  [[U this->DispersedFringeImage.tImageStatus::Update (void) ]]
  ****/
  this->DispersedFringeImage.Update ();
  return OK;

}

/*******************************************************************
*
* tFtImagingTrackController::tFtImagingTrackController 
* DESCRIPTION

[[U tFtImagingTrackController::tFtImagingTrackController (CompleteIoSet) ]]
****/
tFtImagingTrackController::tFtImagingTrackController (
  tFtCompleteIoSet& CompleteIoSet) :
  
  /**** 
  [[U tFtImagingController::tFtImagingController (void) ]]
  *****/
  tFtImagingController (CompleteIoSet) 
  
{}


/*******************************************************************
*
* tFtImagingTrackController::~tFtImagingTrackController -- tFtImagingTrackController destructor
*
*  Does nothing.
*
* SPEC:
      [[U tFtImagingTrackController::~tFtImagingTrackController (void) ]]
*
* RETURNS:
*    nothing
*
****/
tFtImagingTrackController::~tFtImagingTrackController ()
{

}


/*******************************************************************
*
* tFtImagingTrackController::Calculate -- the heart of the tFtImagingTrackController
*
*  Gets an image from the FringeImageSensor; requests an updated target from the
*  target generator and passes it to the delay line actuator; calls DlActuator's 
*  Output method to send the target on its way; stores the just-acquired fringe
*  image in the FringeImageData object inside ControllerData.
*
* SPEC:
      [[U tFtImagingTrackController::Calculate (tServoTime) ]]
*
* RETURNS:
*    ERROR_CODE
*
****/
ERROR_CODE tFtImagingTrackController::Calculate (TIME tServoTime)
{

  ERROR_CODE ec;

  /****
  [[U ec := this->tFtImagingController::Calculate (tServoTime) ]]
  ****/
  ec = this->tFtImagingController::Calculate (tServoTime);

  if (ec != OK) return ec;
 
  /****
  [[U Image.tImage::tImage (void) ]]
  ****/
  tImage Image;
  
  /****
  [[U ec := this->DispersedFringe.tSubImageChannel::GetImage (Image) ]]
  ****/
  ec = this->DispersedFringe.GetImage (Image);
  
  if (ec != OK) return ec;
  
  /****
  [[U ec :=this->DispersedFringeImage.tImageStatus::Update_t (Image) ]]
  ****/
  ec = this->DispersedFringeImage.Update (Image);
  
  return ec;
}




