/* $RCSfile: ftIoSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftIoSet.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */


#include "ft/ftIoSet.h"
#include "ftGizmoInfo.h"
#include "gizmo.h"
#include "ftTypes.h" 

#ifndef MADE_UP 
#error Made up values
#endif
const int CiFtDataDumpPriority = 100;
const int CiFtDataStoreSize = 1000;
//end of made up values

#include "ft/ftMediumController.h"

/******************************************************************
*
* tFtCompleteIoSet::tFtCompleteIoSet - FtCompleteIoSet constructor 
*
* DESCRIPTION:
  Constructs tFtCompleteIoSet creating instances of all I/O available to fringe
  tracker controllers contained in tFtCompleteControllerSet.  I/O includes
  object to access specific hardware sensors and actuators, as well as objects 
  that provide targets, status, or data input and output.  The GizmoInfo used
  to construct the tFtCompleteIoSet contains configuration information such as
  hardware channels, and default values needed by the fringe tracker I/O objects.
*
[[U tFtCompleteIoSet::tFtCompleteIoSet (GizmoInfo) ]]
****/
tFtCompleteIoSet::tFtCompleteIoSet (const tFtGizmoInfo& GizmoInfo) :

  /****
  [[U iGizmoId := GizmoInfo.tGizmoInfo::GetGizmoId (void) ]]

  [[U this->CDataTag.tTypeTag::tTypeTag (ST_DATA, TN_FT_DATA,  iGizmoId, sizeof (tFtData) ) ]]
  ****/
  CDataTag (ST_DATA, TN_FT_DATA, GizmoInfo.GetGizmoId (), sizeof (tFtData) ),
  
  /****
  [[U iGizmoId := GizmoInfo.tGizmoInfo::GetGizmoId (void) ]]

  [[U iHSize := GizmoInfo.tFtGizmoInfo::DispersedFringeHSize (void) ]]

  [[U iVSize := GizmoInfo.tFtGizmoInfo::DispersedFringeVSize (void) ]]

  [[U this->CDispersedFringeTag.tTypeTag::tTypeTag (ST_STATUS, TN_FT_IMAGE, iGizmoId, 
      (iHSize * iVSize * sizeof (PIXEL) )) ]]
  ****/
  CDispersedFringeTag (ST_STATUS, TN_FT_IMAGE, GizmoInfo.GetGizmoId (),
    GizmoInfo.DispersedFringeHSize () * 
    GizmoInfo.DispersedFringeVSize () * sizeof (PIXEL) ),

  /****
  [[U iGizmoId := GizmoInfo.tGizmoInfo::GetGizmoId (void) ]]

  [[U iHSize := GizmoInfo.tFtGizmoInfo::FullFrameHSize (void) ]]

  [[U iVSize := GizmoInfo.tFtGizmoInfo::FullFrameVSize (void) ]]

  [[U this->CFullFrameTag.tTypeTag::tTypeTag (ST_STATUS, TN_FT_FULL_IMAGE, iGizmoId,
        (iHSize * iVSize * sizeof (PIXEL) ) ) ]]
  ****/
  CFullFrameTag (ST_STATUS, TN_FT_FULL_IMAGE,  GizmoInfo.GetGizmoId (), GizmoInfo.FullFrameHSize () * 
    GizmoInfo.FullFrameVSize () * sizeof (PIXEL) ), 
  
  /****
  [[U iHSize := GizmoInfo.tFtGizmoInfo::FullFrameHSize (void) ]]

  [[U iVSize := GizmoInfo.tFtGizmoInfo::FullFrameVSize (void) ]]

  [[U this->FullFrameImage.tImage::tImage (iHSize, iVSize) ]]
  ****/ 
  FullFrameImage (GizmoInfo.FullFrameHSize (), GizmoInfo.FullFrameVSize () ),
  
  /****
  [[U pApdChannel := GizmoInfo.tFtGizmoInfo::ApdChannel (void) ]]

  [[U dScaleFactor := GizmoInfo.tFtGizmoInfo::ApdScaleFactor (void) ]]

  [[U this->Apd.tIntegratorSensor::tIntegratorSensor (pApdChannel, dApdScaleFactor) ]]
  ****/
  Apd (*GizmoInfo.ApdChannel () ,GizmoInfo.ApdScaleFactor () ),

  /****
  [[U pFullImageChannel := GizmoInfo.tFtGizmoInfo::FullImageChannel (void) ]]

  [[U iXStart := GizmoInfo.tFtGizmoInfo::StarXStart (0) ]]

  [[U iYStart := GizmoInfo.tFtGizmoInfo::StarYStart (0) ]]

  [[U iHSize := GizmoInfo.tFtGizmoInfo::StarHSize (0) ]]
  
  [[U iVSize := GizmoInfo.tFtGizmoInfo::StarVSize (0) ]]

  [[U this->Star0.tImageSensor::tImageSensor (GizmoInfo.pFullImageChannel,
      iXStart, iYStart, iHSize,iVSize) ]]
  ****/
  Star0 (GizmoInfo.FullImageChannel (),
     GizmoInfo.StarXStart (0), GizmoInfo.StarYStart (0),
     GizmoInfo.StarHSize (0), GizmoInfo.StarVSize (0) ),
   
  /****
  [[U pFullImageChannel := GizmoInfo.tFtGizmoInfo::FullImageChannel (void) ]]

  [[U iXStart := GizmoInfo.tFtGizmoInfo::StarXStart (1) ]]

  [[U iYStart := GizmoInfo.tFtGizmoInfo::StarYStart (1) ]]

  [[U iHSize := GizmoInfo.tFtGizmoInfo::StarHSize (1) ]]
  
  [[U iVSize := GizmoInfo.tFtGizmoInfo::StarVSize (1) ]]

  [[U this->Star1.tImageSensor::tImageSensor (pFullImageChannel,
      iXStart, iYStart,iHSize,iVSize) ]]
  ****/
  Star1 (GizmoInfo.FullImageChannel (),
     GizmoInfo.StarXStart (1), GizmoInfo.StarYStart (1),
     GizmoInfo.StarHSize (1), GizmoInfo.StarVSize (1) ),
    
  /****
  [[U pFullImageChannel := GizmoInfo.tFtGizmoInfo::FullImageChannel (void) ]]

  [[U iXStart := GizmoInfo.tFtGizmoInfo::DispersedFringeXStart (void) ]]

  [[U iYStart := GizmoInfo.tFtGizmoInfo::DispersedFringeYStart (void) ]]

  [[U iHSize := GizmoInfo.tFtGizmoInfo::StarHSize (1) ]]
  
  [[U iVSize := GizmoInfo.tFtGizmoInfo::StarVSize (1) ]]

  [[U this->Star1.tImageSensor::tImageSensor (pFullImageChannel,
      iXStart, iYStart,iHSize,iVSize) ]]
  [[U this->DispersedFringe.tImageSensor::tImageSensor (
      pFullImageChannel, iXStart, iYStart, iHSize, iVSize ) ]]
  ****/
  DispersedFringe (GizmoInfo.FullImageChannel (),
     GizmoInfo.DispersedFringeXStart (),
     GizmoInfo.DispersedFringeYStart (),
     GizmoInfo.DispersedFringeHSize (),
     GizmoInfo.DispersedFringeVSize () ),

  /****
  [[U pFullImageChannel := GizmoInfo.tFtGizmoInfo::FullImageChannel (void) ]]

  [[U this->FullFrame.tFullFrameSensor::tFullFrameSensor (
        pFullImageChannel) ]]
  ****/
  FullFrame (GizmoInfo.FullImageChannel ()),


  /****
  [[U pApdDiagChannel := GizmoInfo.tFtGizmoInfo::ApdDiagChannel (void) ]]
  
  [[U dScaleFactor := GizmoInfo.tFtGizmoInfo::ApdDiagScaleFactor (void) ]]

  [[U this->ApdDiagOutput.tActuator::tActuator (*pApdDiagChannel, dScaleFactor ) ]]
  ****/
  ApdDiagOutput (*GizmoInfo.ApdDiagChannel (), 
                 GizmoInfo.ApdDiagScaleFactor () ),


  /****
  [[U iiDlTargetId := GizmoInfo.tFtGizmoInfo::ClosedLoopDlTargetId ()]]
  
  [[U this->ClosedLoopDl.tDlActuator::tDlActuator (iiDlTargetId) ]]
  ****/
  ClosedLoopDl (GizmoInfo.ClosedLoopDlTargetId () ),

  /****
  [[U iSt0ImageHSize := GizmoInfo.tFtGizmoInfo::StarHSize (0) ]]

  [[U iSt0ImageVSize := GizmoInfo.tFtGizmoInfo::StarVSize (0) ]]

  [[U iSt0ImageId := GizmoInfo.tFtGizmoInfo::StarImageId (0) ]]
  
  [[U this->StarTracker0.tStActuator::tStActuator (iSt0ImageHSize, 
        iSt0ImageVSize, iiSt0ImageId) ]]
  ****/
  StarTracker0 (GizmoInfo.StarHSize (0), 
                GizmoInfo.StarVSize (0),
                GizmoInfo.StarImageId (0) ),
    
  /****
  [[U iSt1ImageHSize := GizmoInfo.tFtGizmoInfo::StarHSize (1) ]]

  [[U iSt1ImageVSize := GizmoInfo.tFtGizmoInfo::StarVSize (1) ]]

  [[U iSt1ImageId := GizmoInfo.tFtGizmoInfo::StarImageId (1) ]]

  [[U this->StarTracker1.tStActuator::tStActuator (iSt1ImageHSize, 
        iSt1ImageVSize, iiSt1ImageId) ]]
  ****/
  StarTracker1 (GizmoInfo.StarHSize (1), 
                GizmoInfo.StarVSize (1),
                GizmoInfo.StarImageId (1) ) ,
                        
  /****
  [[U this->DataStore.tDataStore::tDataStore (CiFtDataDumpPriority,
     CiFtDataStoreSize, this->CiDataTag) ]]
  ****/
  DataStore (CiFtDataDumpPriority, CiFtDataStoreSize, this->CDataTag),
  
  /****
  [[U iNumBoxcarSamples := GizmoInfo.tFtGizmoInfo::NumBoxcarSamples (void) ]]

  [[U iNumWobbleSamples := GizmoInfo.tFtGizmoInfo::NumWobbleSamples (void) ]]

  [[U tServoPeriod := GizmoInfo.tFtGizmoInfo::ServoPeriod (FSP_MEDIUM) ]]

  [[U this->Fringe.tFringe::tFringe (iNumBoxcarSamples, iNumWobbleSample,
        tServoPeriod) ]]
  ****/
  Fringe (GizmoInfo.NumBoxcarSamples (),
          GizmoInfo.NumWobbleSamples (),
          GizmoInfo.ServoPeriod (FSP_MEDIUM) ),

  /****
  [[U iHSize := GizmoInfo.tFtGizmoInfo::DispersedFringeHSize (void) ]]
 
  [[U iVSize := GizmoInfo.tFtGizmoInfo::DispersedFringeVSize (void) ]]
 
  [[U this->DispersedFringeImage.tImageStatus::tImageStatus (
      this->CDispersedFringeTag, iHSize, iVSize) ]]
  ****/
  DispersedFringeImage (this->CDispersedFringeTag, GizmoInfo.DispersedFringeHSize (),
    GizmoInfo.DispersedFringeVSize () ),


  /****
  [[U this->PhaseTarget.tPhaseTarget::tPhaseTarget (void) ]]
  ****/
  PhaseTarget (),


  /****
  [[U SearchParameters := GizmoInfo.tFtGizmoInfo::SearchParameters (void) ]]

  [[U this->SearchTarget.tSearch::tSearch (SearchParameters) ]]
  ****/  
  SearchTarget (GizmoInfo.SearchParameters () ),
    
  /****
  [[U WobbleParameters := GizmoInfo.tFtGizmoInfo::WobbleParameters (void) ]]

  [[U this->WobbleTarget.tWobbleTarget::tWobbleTarget (3,WobbleParameters) ]]
  ****/
  WobbleTarget (3, GizmoInfo.WobbleParameters ()), 




  TestDump (10000,dum (0,0,0,0,0,0,0)),
  PhotonDiag (10000,tCount (0,0)),
  dUnwrappedPhase (0.0),
  dManualTarget (0.0),
  dLockTarget (0.0),
  dFringeCenterTarget (0.0)

{
  
  /****
  [[U this->lvWhiteLightLambda := GizmoInfo.tFtGizmoInfo::WhiteLightLambda (void) ]]
  ****/
  this->lvWhiteLightLambda = GizmoInfo.WhiteLightLambda ();

  /****
  [[U this->FullFrame.tFullFrameSensor::StartCapture () ]]
  ****/
  this->FullFrame.StartCapture();

}
  

/******************************************************************
*
* tFtCompleteIoSet::~tFtCompleteIoSet - FtCompleteIoSet destructor 
*
*  does nothing
*
[[U tFtCompleteIoSet::~tFtCompleteIoSet (void) ]]
****/ 
tFtCompleteIoSet::~tFtCompleteIoSet () {

  /****
  [[U this->WobbleTarget.tWobbleTarget::~tWobbleTarget (void) ]]

  [[U this->PhaseTarget.tPhaseTarget::~tPhaseTarget (void) ]]

  [[U this->SearchTarget.tSearch::~tSearch (void) ]]

  [[U this->DispersedFringeImage.tImageStatus::~tImageStatus (void) ]]

  [[U this->DataStore.tDataStore::~tDataStore (void) ]]

  [[U this->StarTracker1.tStActuator::~tStActuator (void) ]]

  [[U this->StarTracker0.tStActuator::~tStActuator (void) ]]

  [[U this->ClosedLoopDl.tDlActuator::~tDlActuator (void) ]]

  [[U this->FullFrame.tFullFrameSensor::~tFullFrameSensor (void) ]]

  [[U this->DispersedFringe.tImageSensor::~tImageSensor (void) ]]

  [[U this->Star1.tImageSensor::~tImageSensor (void) ]]

  [[U this->Star0.tImageSensor::~tImageSensor (void) ]]
 
  [[U this->Apd.tIntegratorSensor::~tIntegratorSensor (void) ]]

  [[U this->FullFrameImage.tImage::~tImage (void) ]]

  [[U this->CFullFrameTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->CDispersedFringeTag.tTypeTag::~tTypeTag (void) ]]

  [[U this->CDataTag.tTypeTag::~tTypeTag (void) ]]
  *****/ 
}


/******************************************************************
*
* tFtCompleteIoSet::SetSearchParameters (const tFtSearchParameters& 
     SearchParameters)
*
* DESCRIPTION:
  Updates the parameters that govern the spiral search algorythm that
  the Medium Track-Search controller uses.
*
[[U tFtCompleteIoSet::SetSearchParameters (SearchParameters) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SetSearchParameters (const tFtSearchParameters& 
  SearchParameters) 
{
  /****
  [[ return := this->SearchTarget.tSearch::SetParameters (SearchParameters) ]]
  ****/
  return this->SearchTarget.SetParameters (SearchParameters);
}

/******************************************************************
*
* tFtCompleteIoSet::SetWobbleParameters (const tModulationParams& 
     ModulationParameters)
*
* DESCRIPTION:
  Updates the parameters that govern the fringe wobble waveform that
  the Medium Track-Lock controller uses.
*
[[U tFtCompleteIoSet::SetWobbleParameters (WobbleParameters) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SetWobbleParameters (const tModulationParams& 
  WobbleParameters) 
{
  /****
  [[U return := this->WobbleTarget.tModulation::
      SetParameters (WobbleParameters) ]]
  ****/
  return this->WobbleTarget.SetParameters (WobbleParameters);
}


/******************************************************************
*
* tFtCompleteIoSet::SetRegionOfInterest (const tFtImageParameters& 
    ImageParameters)
*
* DESCRIPTION:
  Sets the region of the video frame for the image channels for 
  the two star image sensors, and the dispersed fringe sensor.
*      
[[U tFtCompleteIoSet::SetRegionOfInterest (const tFtImageParameters& 
      ImageParameters) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SetRegionOfInterest (
  const tFtImageParameters& ImageParameters) 
{
 
  ERROR_CODE ec, ec1,ec2,ec3; 

  ec1 = this->Star0.SetRegionOfInterest (
    ImageParameters.iStartXSt0, ImageParameters.iStartYSt0);

  ec2 = this->Star1.SetRegionOfInterest  (
    ImageParameters.iStartXSt1, ImageParameters.iStartYSt1);

  ec3 = this->DispersedFringe.SetRegionOfInterest (
    ImageParameters.iStartXFringeImage, ImageParameters.iStartYFringeImage);
  
  if (ec1 != OK) ec = ec1;
  else if (ec2 != OK) ec = ec2;
  else if (ec3 != OK) ec = ec3;
  else ec = OK;

  if (ec != OK) return Traceback (ec,"tFtIoSet::SetRegionOfInterest");

  return OK;
   
}


/******************************************************************
*
* tFtCompleteIoSet::SetManualTarget (FT_TARGET_DIRECTION ftdTargetDirection)
*
* DESCRIPTION:
  Sets the region of the video frame for the image channels for 
  the two star image sensors, and the dispersed fringe sensor.
*      
[[U tFtCompleteIoSet::SetManualTarget (ftdTargetDirection) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SetManualTarget (
    FT_TARGET_DIRECTION ftdTargetDirection) 
{

  if (ftdTargetDirection == FTD_POSITIVE) 
    this->dManualTarget += this->lvWhiteLightLambda;
  
  else if (ftdTargetDirection == FTD_NEGATIVE)
    this->dManualTarget -= this->lvWhiteLightLambda;

  else return Traceback (S_ft_INVALID_TARGET_DIRECTION, 
    "tFtCompleteIoSet::SetManualTarget");
   
  return OK;
}


/******************************************************************
*
* tFtCompleteIoSet::SendFullFrameImage ()
*
* DESCRIPTION:
  Grabs and sends the most recent image stored in the full frame image 
  sensor to the status stream. This method does not garantee an atomic image
*      
[[U tFtCompleteIoSet::SendFullFrameImage (void) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SendFullFrameImage () 
{
  ERROR_CODE ec;

  /****
  [[U ec := this->FullFrame.tFullFrameSensor::GetImage (
      this->FullFrameImage) ]]
  ****/
  ec = this->FullFrame.GetImage (this->FullFrameImage);

  if (ec != OK) return Traceback (ec,
    "tFtCompleteIoSet::SendFullFrameImage");

  /****
  [[U return := tGizmo::SendStatus_tt (this->CFullFrameTag, 
      this->FullFrameImage) ]]
  ****/
  return tGizmo::SendStatus (this->CFullFrameTag, this->FullFrameImage);

}


/******************************************************************
*
* tFtCompleteIoSet::SendAtomicFullFrameImage ()
*
* DESCRIPTION:
  Grabs and sends the most recent image stored in the full frame image 
  sensor to the status stream.  This method garantees an atomic image.
*      
[[U tFtCompleteIoSet::SendAtomicFullFrameImage (void) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SendAtomicFullFrameImage () 
{
  ERROR_CODE ec;

  /****
  [[U ec := this->FullFrame.tFullFrameSensor::AtomicGetImage (
      this->FullFrameImage) ]]
  ****/
  ec = this->FullFrame.AtomicGetImage (this->FullFrameImage);

  if (ec != OK) return Traceback (ec,
    "tFtCompleteIoSet::SendAtomicFullFrameImage");

  /****
  [[U return := tGizmo::SendStatus_tt (this->CFullFrameTag, 
      this->FullFrameImage) ]]
  ****/
  return tGizmo::SendStatus (this->CFullFrameTag, this->FullFrameImage);

}


/******************************************************************
*
* tFtCompleteIoSet::SendDispersedFringeImage ()
*
* DESCRIPTION:
  Sends the most recent image stored in the Dispersed Fringe status
  block to the status stream.
*      
[[U tFtCompleteIoSet::SendDispersedFringeImage (void) ]]
****/ 
ERROR_CODE tFtCompleteIoSet::SendDispersedFringeImage () 
{
  ERROR_CODE ec;

  /****
  [[U return := this->DispersedFringeImage.tImageStatus::
    SendToStatusStream (void) ]]
  ****/
  return this->DispersedFringeImage.SendToStatusStream ();

}

/******************************************************************
*
* tFtCompleteIoSet::Targets ()
*
* DESCRIPTION:
  Returns snapshot of current fringe tracker targets used to command the
  closed loop delay line
*      
[[U tFtCompleteIoSet::Targets (void) ]]
****/ 
tFtTargetStatus tFtCompleteIoSet::Targets () const
{
  /****
  [[U dSearch := this->SearchTarget.tSearch::GetValue (void) ]]
  ****/
  double dSearch = this->SearchTarget.GetValue ();

  /****
  [[U dWobble := this->WobbleTarget.tWobbleTarget::operator_index (0) ]]
  ****/
  double dWobble = this->WobbleTarget [0];

  /****
  [[U dPhase := this->PhaseTarget.tPhaseTarget::Position (void) ]]
  ****/
  double dPhase = this->PhaseTarget.Position ();
  
  /****
  [[U return := tFtTargetStatus::tFtTargetStatus (this->dManualTarget, dSearch, 
       dWobble, this->dLockTarget, this->dFringeCenterTarget) ]]
  ****/
  return tFtTargetStatus (this->dManualTarget, dSearch, dWobble, 
     this->dLockTarget, this->dFringeCenterTarget, dPhase);
     
}

/******************************************************************
*
* tFtCompleteIoSet::ImageParameters ()
*
* DESCRIPTION:
  Returns snapshot of current fringe tracker image parameters used to 
  grab star and dispersed fringe images from the frame grabber board 
*
[[U tFtCompleteIoSet::ImageParameters (void)  ]]
****/
tFtImageParameters tFtCompleteIoSet::ImageParameters () const 
{
  
  tFtImageParameters ImageParameters;
  
  int iHSize,iVSize; // dummy storage variables for method calls
  
  /****
  [[U this->Star0.tImageSensor::ImageParameters (ImageParameters.iStartXSt0, 
        ImageParameters.iStartYSt0, iHSize, iVSize) ]]
  ****/
  this->Star0.ImageParameters (ImageParameters.iStartXSt0, 
        ImageParameters.iStartYSt0, iHSize, iVSize);     

  /****
  [[U this->Star1.tImageSensor::ImageParameters (ImageParameters.iStartXSt1, 
        ImageParameters.iStartYSt1, iHSize, iVSize) ]]
  ****/
  this->Star1.ImageParameters (ImageParameters.iStartXSt1, 
        ImageParameters.iStartYSt1, iHSize, iVSize);     

  /****
  [[U this->DispersedFringe.tImageSensor::ImageParameters (
        ImageParameters.iStartXFringeImage, ImageParameters.iStartYFringeImage
        ImageParameters.iFringeImageHSize, ImageParameters.iFringeImageVSize) ]]
  ****/
  this->DispersedFringe.ImageParameters (ImageParameters.iStartXFringeImage, 
    ImageParameters.iStartYFringeImage, ImageParameters.iFringeImageHSize, 
    ImageParameters.iFringeImageVSize);     

  return ImageParameters;
}
 
/******************************************************************
*
* tFtCompleteIoSet::WobbleParameters ()
*
* DESCRIPTION:
  Returns snapshot of current wobble parameters used to calculate the 
  wobble targets.
*   
[[U tFtCompleteIoSet::WobbleParameters (void)  ]]
****/
const tModulationParams& tFtCompleteIoSet::WobbleParameters () const
{
  /****
  [[U return := this->WobbleTarget.tModulation::
        GetParameters (void) ]]
  ****/
  return this->WobbleTarget.GetParameters ();
}


/******************************************************************
*
* tFtCompleteIoSet::SearchParameters ()
*
* DESCRIPTION:
  Returns snapshot of current search parameters used to calculate the 
  search targets.
*   
[[U tFtCompleteIoSet::SearchParameters (void)  ]]
****/
tFtSearchParameters tFtCompleteIoSet::SearchParameters () const
{
  /****
  [[U return := this->SearchTarget.tSearch::SearchParmeters (void) ]]
  ****/
  return this->SearchTarget.SearchParameters ();
}


/******************************************************************
*
* tFtCompleteIoSet::FringeStatus ()
*
* DESCRIPTION:
  Returns snapshot of the current fringe data values.
*
[[U tFtCompleteIoSet::FringeStatus (void)  ]]
****/
tFringeStatus tFtCompleteIoSet::FringeStatus () const
{
  /****
  [[U dPhotons := this->Fringe.tFringe::Photons (void) ]]
  ****/
  double dPhotons = this->Fringe.Photons ();

  /****
  [[U dVisibility2 := this->Fringe.tFringe::EstVisibility2 (void) ]]
  ****/
  double dVisibility2 = this->Fringe.EstVisibility2 ();


  /****
  [[U dSnr2 := this->Fringe.tFringe::EstSnr2 (void) ]]
  ****/
  double dSnr2 = this->Fringe.EstSnr2 ();
  
  double dSnr;
  
  if (dSnr2 < 0) dSnr = 0; 
  else dSnr = sqrt (dSnr2);
  
  /****
  [[U return := tFringeStatus::tFringeStatus (dPhotons, dVisibility2, dSnr) ]]
  ****/
  return tFringeStatus (dPhotons, dVisibility2, dSnr);
}
  

