/* $RCSfile: ftMediumController.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftMediumController.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a10Nov97,rlj    written
*/

#include "ft/ftMediumController.h"
#include "ft/ftIoSet.h"
#include "ringBuffer.h"

#include <iostream.h>

BOOL bUnWrapPhaseEnable = TRUE; /* Enables UnWrap Phase Calulation       */
				/* When disabled (= FALSE) UnWrapedPhase */
				/* returns Raw phase                     */

BOOL bTestDump = FALSE;

/************* BEH ************/
FILE *fpC = NULL;
int bRecordCor=0;
#include "rngLib.h"
RING_ID riCorData;
int rngCorTask(void)
{
  int iBytes;
  char cBuf[40000];

  printf("Hello from rngCorTask\n");
  
  FOREVER {
    iBytes = rngBufGet(riCorData,cBuf,40000);
    if (iBytes > 0)
      fwrite(cBuf,1,iBytes,fpC);
    if (iBytes < 40000)
      taskDelay(1);
  }
}
/******************************/


/*******************************************************************
*
* tFtMediumController::tFtMediumController (tFtCompleteIoSet& CompleteIoSet,
    tFtMediumControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs base class for all fringe tracker medium controllers assigning
  references to a tFringe, approprate fringe tracker target sources, and closed loop tDlActuator
  from the FT CompleteIoSet.
*
[[U tFtMediumController::tFtMediumController (CompleteIoSet) ]]
****/
tFtMediumController::tFtMediumController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtController::tFtController (void) ]]
  ****/
  tFtController (),


  /****
  [ this->ControllerParameters := reference to ControllerParametersToUse ]
  ****/
  ControllerParameters (ControllerParametersToUse),
  
  /****
  [ this->Fringe := reference to CompleteIoSet.Fringe ]
  ****/
  Fringe (CompleteIoSet.Fringe),

  /****
  [ this->ClosedLoopDl := reference to CompleteIoSet.ClosedLoopDl ]
  ****/
  ClosedLoopDl (CompleteIoSet.ClosedLoopDl),

  /****
  [ this->DataStore := reference to CompleteIoSet.DataStore ]
  ****/
  DataStore (CompleteIoSet.DataStore),

  /****
  [ this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase ]
  ****/
  dUnwrappedPhase (CompleteIoSet.dUnwrappedPhase),

  /****
  [ this->ManualTarget := reference to CompleteIoSet.ManualTarget ]
  ****/
  dManualTarget (CompleteIoSet.dManualTarget),

  /****
  [ this->LockTarget := reference to CompleteIoSetd.LockTarget ]
  ****/
  dLockTarget (CompleteIoSet.dLockTarget),

  /****
  [ this->FringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget ]
  ****/
  dFringeCenterTarget (CompleteIoSet.dFringeCenterTarget),

  /****
  [ this->dPhaseTarget := reference to CompleteIoSet.PhaseTarget ]
  ****/
  PhaseTarget (CompleteIoSet.PhaseTarget),

  /****
  [ this->SearchTarget := reference to CompleteIoSet.SearchTarget ]
  ****/
  SearchTarget (CompleteIoSet.SearchTarget),

  /****
  [ this->WobbleTarget := reference to CompleteIoSet.WobbleTarget ]
  ****/
  WobbleTarget (CompleteIoSet.WobbleTarget),
 
  TestDump (CompleteIoSet.TestDump)

{
/************* BEH ************/
if (fpC==NULL) {
  fpC=fopen ("/home2/realtime/data/fringe/cor.dat","wb");
  if (fpC==NULL)
    perror("Error opening cor.dat\n");
  else {
    printf("File opened: fpC=%d\n", (int)fpC);
    riCorData = rngCreate(80000);
    taskSpawn("rngCorTask",99,0,100000,(FUNCPTR)rngCorTask,0,0,0,0,0,0,0,0,0,0);
  }
}
/******************************/


}


/*******************************************************************
*
* tFtMediumController::~tFtMediumController (void)
*
* DESCRIPTION:
  Destructs base class for all fringe tracker medium controllers.
*
[[U tFtMediumController::~tFtMediumController (void) ]]
****/
tFtMediumController::~tFtMediumController ()
{
  /****
  [[U tFtController::~tFtController (void) ]]
  ****/ 
}



/*******************************************************************
*
* tFtMediumController::StoreData (TIME tServoTime, 
    FT_CONTROLELR_MODE fcmControllerModeToUse)
*
* DESCRIPTION:
  Stores Fringe Tracker data for sending to data stream.
*
[[U tFtMediumController::StoreData (tServoTime, fcmControllerModeToUse) ]]

****/
void tFtMediumController::StoreData (TIME tServoTime, 
        FT_CONTROLLER_MODE fcmControllerModeToUse) 
{
  tFtData Data;
  
  Data.tTimeStamp = tServoTime;
  Data.fcmControllerMode = fcmControllerModeToUse;
  
  /****
  [[U Data.dPhotons := this->Fringe.tFringe::Photons () ]]
  ****/
  Data.dPhotons = this->Fringe.Photons ();

  /****
  [[U Data.dInPhaseComponent := this->Fringe.tFringe::InPhaseComponent () ]]
  ****/
  Data.dInPhaseComponent = this->Fringe.InPhaseComponent ();

  /****
  [[U Data.dQuadratureComponent := this->Fringe.tFringe::QuadratureComponent () ]]
  ****/
  Data.dQuadratureComponent = this->Fringe.QuadratureComponent ();

  Data.dUnwrappedPhase = this->dUnwrappedPhase;
 
  /****
  [[U this->DataStore.tDataStore::StoreData (Data) ]]
  ****/
  this->DataStore.StoreData (Data);

}


/*******************************************************************
*
* tFtMediumController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
   Compute Fringe Statistics
*
[[U tFtMediumController::Calculate (tServoTime) ]]
****/
ERROR_CODE tFtMediumController::Calculate (TIME tServoTime)
{
  
  ERROR_CODE ec;

  /****
  [[U this->Fringe.tFringe::Calculate (tServoTime) ]]
  ****/
  ec =  this->Fringe.Calculate (tServoTime);

  return ec;

}



/*******************************************************************
*
* tFtMediumOffController::tFtMediumOffController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constucts  fringe tracker medium off controller.
*
[[U tFtMediumOffController::tFtMediumOffController (CompleteIoSet,
    ControllerParametersToUse) ]]
****/
tFtMediumOffController::tFtMediumOffController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtMediumController::tFtMediumController (CompleteIoSet, 
      ControllerParametersToUse) ]]
  ****/ 
  tFtMediumController (CompleteIoSet, ControllerParametersToUse)

{}



/*******************************************************************
*
* tFtMediumOffController::~tFtMediumOffController (void)
*
* DESCRIPTION:
  Destructs fringe tracker medium off controller.
*
[[U tFtMediumOffController::~tFtMediumOffController (void) ]]
****/
tFtMediumOffController::~tFtMediumOffController ()
{
  /****
  [[U tFtMediumController::~tFtMediumController (void) ]]
  ****/ 
}


/*******************************************************************
*
* tFtMediumOffController::Init 
*
* DESCRIPTION:
  Initializes the FT Medium Off Controller by sending a ZERO_TARGET to closed loop delay line
*
[[U tFtMediumOffController::Init (tServoTime) ]]
****/
ERROR_CODE tFtMediumOffController::Init (TIME tServoTime )
{
 
 
  /****
  [[U this->SearchTarget.tSearch::Reset (void) ]]
  ****/
  this->SearchTarget.Reset ();

  /****
  [[U this->ClosedLoopDl.tDlActuator::Reset (void) ]]
  ****/
  this->ClosedLoopDl.Reset ();

  if (bTestDump) {
    for (int i = -49;i <= 0; i++) 
      cout << "N  = " << TestDump[i].d1 << " X  = " << TestDump[i].d2 << " Y = " << TestDump[i].d3 << 
          " SNR  = " << TestDump[i].d4 <<  endl << " TIME = " << TestDump[i].t << endl << endl;
    bTestDump = FALSE;
  }


  return OK;
}


/*******************************************************************
*
* tFtMediumTrackController::tFtMediumTrackController (tFtCompleteIoSe&t CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs base class for all fringe tracker medium track controllers. 
*
[[U tFtMediumTrackController::tFtMediumTrackController (CompleteIoSet,
    ControllerParametersToUse) ]]
****/
tFtMediumTrackController::tFtMediumTrackController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtMediumController::tFtMediumController (CompleteIoSet, 
      ControllerParametersToUse) ]]
  ****/
  tFtMediumController (CompleteIoSet, ControllerParametersToUse)

{}


/*******************************************************************
*
* tFtMediumTrackController::~tFtMediumTrackController (void)
*
* DESCRIPTION:
  Destructs base class for all fringe tracker medium track controllers.
*
[[U tFtMediumTrackController::~tFtMediumTrackController (void) ]]
****/
tFtMediumTrackController::~tFtMediumTrackController ()
{
  /****
  [[U tFtMediumController::~tFtMediumController (void) ]]
  ****/ 
}

  
/*******************************************************************
*
* tFtMediumTrackController::UnwrapPhase (double dOldUnwrappedPhase, 
                                         double dNewPhase)
*
* DESCRIPTION:
  Computes the new unwrapped phase value from the old unwrapped phase and 
  the new "wrapped" phase.
*
Global Input BOOL bUnWrapPhaseEnable = TRUE; 
                  Enables UnWrap Phase Calulation. When disabled (= FALSE) 
                  UnWrapedPhase returns Raw phase         
  
  
*
[[U tFtMediumTrackController::UnwrapPhase (dOldUnwrappedPhase, dNewPhase) ]]
****/
double tFtMediumTrackController::UnwrapPhase (double dOldUnwrappedPhase, 
    double dNewPhase) const
{
  if (bUnWrapPhaseEnable == FALSE) 
      return (dNewPhase);          /* Unwrapping is turned off */
  
  double dDeltaPiShiftedPhase,  dModuloPortion;
  
  dDeltaPiShiftedPhase = dNewPhase - dOldUnwrappedPhase + PI;
  
  // dDeltaPiShiftedPhase % (2 * PI) giving value between 0 and 2*PI
  
  dModuloPortion = dDeltaPiShiftedPhase -
     floor ( dDeltaPiShiftedPhase / (2*PI) ) * 2 * PI;  
   
  // remove PI shift from result and add to old unwrapped phase
  return dOldUnwrappedPhase + dModuloPortion - PI; 
}


/*******************************************************************
*
* tFtMediumTrackController::PhaseCorrection ()
*
* DESCRIPTION:
*
[[U tFtMediumTrackController::PhaseCorrection (tServoTime) ]]
****/
tDlTarget tFtMediumTrackController::PhaseCorrection (TIME tServoTime) 
{
  double dPhase, dPhaseError, dPhaseTarget;
  tDlTarget NewTarget;

  // store the wobble position 
  /****
  [[U this->Fringe.tFringe::AcceptWobble (tServoTime, current wobble position) ]]
  ****/
  this->Fringe.AcceptWobble (tServoTime, this->WobbleTarget [0] );

 
  /****
  [[U dPhase := this->Fringe.tFringe::Phase (void) ]]
  ****/
  dPhase = this->Fringe.Phase ();

  /****
  [[U this->dUnwrappedPhase := this->tFtMediumTrackController::
        UnwrapPhase (this->dUnWrappedPhase, dPhase) ]]
  ****/
  this->dUnwrappedPhase = this->UnwrapPhase (this->dUnwrappedPhase, dPhase);
  
 
  /****
  [[U dOldWobble := this->WobbleTarget.tWobbleTarget::operator_index (-1) ]]
  
  [ dPhaseError := this->dUnwrappedPhase - 
                      this->ControllerParameters.dPhaseTarget - 
                      dOldWobble ]
  ****/                     
  dPhaseError = this->dUnwrappedPhase - 
                      this->ControllerParameters.dPhaseOffset -
                      2 * PI * this->WobbleTarget [-1] /
                        (double) this->ControllerParameters.lvWhiteLightLambda;

     
  /****
  [[U NewTarget := this->PhaseTarget.tPhaseTarget::Calculate (this->ControllerParameters,
        tServoTime, dPhaseError) ]]
  ****/
  NewTarget = this->PhaseTarget.Calculate (this->ControllerParameters, tServoTime, dPhaseError);


  //TestDump.Append (dum (Fringe.Photons (), Fringe.InPhaseComponent (), Fringe.QuadratureComponent (), Fringe.Snr2 (), 0.0, 0.0, tServoTime));



  return NewTarget;


}

/*******************************************************************
*
* tFtMediumTrackSearchController::tFtMediumTrackSearchController (
    tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs FT medium track search controller. 
*
[[U tFtMediumTrackSearchController::tFtMediumTrackSearchController (CompleteIoSet,
    ControllerParametersToUse) ]]
****/
tFtMediumTrackSearchController::tFtMediumTrackSearchController (
    tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtMediumTrackController::tFtMediumTrackController (CompleteIoSet,
      ControllerParametersToUse) ]]
  ****/
  tFtMediumTrackController (CompleteIoSet, ControllerParametersToUse)

{}


/*******************************************************************
*
* tFtMediumTrackSearchController::~tFtMediumTrackSearchController (void)
*
* DESCRIPTION:
  Destructs FT medium track search controller. 
*
[[U tFtMediumTrackSearchController::~tFtMediumTrackSearchController (void) ]]
****/
tFtMediumTrackSearchController::~tFtMediumTrackSearchController ()
{
  /****
  [[U tFtMediumTrackController::~tFtMediumTrackController (void) ]]
  ****/ 
}




/*******************************************************************
*
* tFtMediumTrackSearchController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
  Sends target to Closed loop DL based on the current search, manual and lock targets
*
[[U tFtMediumTrackSearchController::Calculate (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackSearchController::Calculate (TIME tServoTime)
{
  
  double dSearchTarget;

  static int iSearchCounter = 0;

 
  /****
  [[U NewTarget.tDlTarget::tDlTarget (void) ]]
  ****/
  tDlTarget NewTarget;

  ERROR_CODE ec;

  ec = this->tFtMediumController::Calculate (tServoTime);

  if (ec != OK) return ec;


  // keep the wobble position up to date 
  /****
  [[U this->Fringe.tFringe::AcceptWobble (tServoTime, 0.0) ]]
  ****/
  this->Fringe.AcceptWobble (tServoTime, 0.0);

 
  if ((tServoTime/this->ControllerParameters.tFtServoPeriod) % 4 == 0) {
  //if (iSearchCounter++ % 4 == 0) {
  
    /****
    [[U dSearchTarget := this->SearchTarget.tSearch::UpdateTarget (void) ]]
    ****/
    dSearchTarget = this->SearchTarget.UpdateTarget ();
 


    NewTarget.dPosition = this->dManualTarget + this->dLockTarget + dSearchTarget;

    NewTarget.dRate = 0.0;
  
    NewTarget.tTime = tServoTime;

    // at some point we need to specify the time to activate the target

    /****
    [[U ec := this->ClosedLoopDl.tDlActuator::OutputValue (tServoTime, NewTarget) ]]
    ****/
    ec = this->ClosedLoopDl.OutputValue (tServoTime, NewTarget);
  }

  /****
  [[U this->tFtMediumController::StoreData (tServoTime, FCM_SEARCH) ]]
  ****/
  this->StoreData (tServoTime, FCM_SEARCH);
  
  /************* BEH -KMS************/
if (bRecordCor == 1) {

double dTemp = (double)tServoTime;

rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.A();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.B();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.C();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.D();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.Phase();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.Snr2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.Visibility2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));

}
/******************************/


  return ec; 
}





/*******************************************************************
*
* tFtMediumTrackSemiLockController::tFtMediumTrackSemiLockController (
    tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs FT medium track semi-lock controller. 
*
[[U tFtMediumTrackSemiLockController::tFtMediumTrackSemiLockController (
    CompleteIoSet, ControllerParametersToUse) ]]
****/
tFtMediumTrackSemiLockController::tFtMediumTrackSemiLockController (
    tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtMediumTrackController::tFtMediumTrackController (
    CompleteIoSet,ControllerParametersToUse) ]]
  ****/
  tFtMediumTrackController (CompleteIoSet, ControllerParametersToUse)

{}

/*******************************************************************
*
* tFtMediumTrackSemiLockController::~tFtMediumTrackSemiLockController ()
*
* DESCRIPTION:
  Destructs FT medium track semi-lock controller. 
*
[[U tFtMediumTrackSemiLockController::~tFtMediumTrackSemiLockController () ]]
****/
tFtMediumTrackSemiLockController::~tFtMediumTrackSemiLockController () 

  /****
  [[U tFtMediumTrackController::~tFtMediumTrackController (void) ]]
  ****/

{}


/*******************************************************************
*
* tFtMediumTrackSemiLockController::Init 
*
* DESCRIPTION:
*
[[U tFtMediumTrackSemiLockController::Init (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackSemiLockController::Init (TIME tServoTime)
{
 

  /****
  [[U this->PhaseTarget.tPhaseTarget::Reset (void) ]]
  ****/
  this->PhaseTarget.Reset ();
  
  /* Clear Unwrapped Phase befor starting control */
  this->dUnwrappedPhase = 0.0;  
  
cout <<  "SL Init dLockTarget = " << this->dLockTarget
	<< " SearchTarget = " << this->SearchTarget.GetValue () 
	<< " dManualTarget = " << this->dManualTarget
	<< " dFringeCenterTarget = " << this->dFringeCenterTarget 
	<< " PhaseTarget.Position = " << this->PhaseTarget.Position () << endl;  
  
  return OK;
}
 
 

/*******************************************************************
*
* tFtMediumTrackSemiLockController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
  Sends target to Closed loop DL based on the frozen search, manual and lock targets.
*
[[U tFtMediumTrackSearchController::Calculate (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackSemiLockController::Calculate (TIME tServoTime)
{

  ERROR_CODE ec;
  double dSearchTarget;
  tDlTarget NextDlTarget;
 

  ec = this->tFtMediumTrackController::Calculate (tServoTime);

  if (ec != OK) return ec;
 
  /****
  [[U NextDlTarget := this->tFtMediumTrackController::PhaseCorrection (tServoTime) ]]
  ****/
  NextDlTarget = this->PhaseCorrection (tServoTime);
  
  // use last search value...
  /****
  [[U dSearchTarget := this->SearchTarget.tSearch::GetValue (void) ]]
  ****/
  dSearchTarget = this->SearchTarget.GetValue (); 
 
  // add in discrete position offsets for SemiLock
  NextDlTarget.dPosition += this->dManualTarget + this->dLockTarget + dSearchTarget;

  /****
  [[U ec := this->ClosedLoopDl.tDlActuator::OutputValue (tServoTime, NewTarget) ]]
  ****/
  ec = this->ClosedLoopDl.OutputValue (tServoTime, NextDlTarget);

  /****
  [[U this->tFtMediumController::StoreData (tServoTime, FCM_SEARCH) ]]
  ****/
  this->StoreData (tServoTime, FCM_SEMI_LOCK);

    /************* BEH -KMS************/
if (bRecordCor == 2) {

double dTemp = (double)tServoTime;

rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.A();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.B();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.C();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.D();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.Phase();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));

dTemp = dUnwrappedPhase;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.dPosition;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.dRate;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.tTime;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
//dTemp = NextDlTarget.dErrorSum;
//rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));

dTemp = Fringe.Snr2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.Visibility2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));

}
/******************************/
  return ec; 

}



/*******************************************************************
*
* tFtMediumTrackLockController::tFtMediumTrackLockController (
    tFtCompleteIoSet& CompleteIoSet, 
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs FT medium track lock controller. 
*
[[U tFtMediumTrackLockController::tFtMediumTrackLockController (
    CompleteIoSet, ControllerParametersToUse) ]]
****/
tFtMediumTrackLockController::tFtMediumTrackLockController (
    tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtMediumTrackController::tFtMediumTrackController (
    tFtCompleteIoSet CompleteIoSet,
    tFtMediumControllerParameters& ControllerParametersToUse) ]]
  ****/
  tFtMediumTrackController (CompleteIoSet, ControllerParametersToUse)

{}

/*******************************************************************
*
* tFtMediumTrackLockController::~tFtMediumTrackLockController ()
*
* DESCRIPTION:
  Destructs FT medium track lock controller. 
*
[[U tFtMediumTrackLockController::~tFtMediumTrackiLockController () ]]
****/
tFtMediumTrackLockController::~tFtMediumTrackLockController () 

  /****
  [[U tFtMediumTrackController::~tFtMediumTrackController (void) ]]
  ****/

{}


/*******************************************************************
*
* tFtMediumTrackLockController::Init 
*
* DESCRIPTION:
  Initializes the FT Medium Track Lock Controller, setting the spiral object to freeze 
*
[[U tFtMediumTrackLockController::Init (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackLockController::Init (TIME tServoTime)
{

  /****
  [[U dTemp := this->SearchTarget.tSearch::GetValue (void) ]]
  [ this->dLockTarget := this->dLockTarget + dTemp1 + this->dManualTarget ]
  ****/
cout <<  "Track Lock Init dLockTarget = " << this->dLockTarget
	<< " SearchTarget = " << this->SearchTarget.GetValue () 
	<< " dManualTarget = " << this->dManualTarget
	<< "dFringeCenterTarget = " << this->dFringeCenterTarget  
	<< " PhaseTarget.Position = " << this->PhaseTarget.Position () << endl;
  
  this->dLockTarget += this->SearchTarget.GetValue () + this->dManualTarget;

  this->dManualTarget = 0.0;
  
  /****
  [[U this->SearchTarget.tSearch::Reset (void) ]]
  ****/
  this->SearchTarget.Reset ();


  return OK; 
}


/*******************************************************************
*
* tFtMediumTrackLockController::Reset 
*
* DESCRIPTION:
  Resets the FT Medium Track Lock Controller, setting the spiral object to Freeze
*
[[U tFtMediumTrackLockController::Reset (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackLockController::Reset (TIME tServoTime)
{

  this->dLockTarget += this->dManualTarget + this->dFringeCenterTarget;
  
  /****
  [[U this->WobbleTarget.tWobbleTarget::Reset (void) ]]
  ****/
  this->WobbleTarget.Reset ();

  this->PhaseTarget.Reset ();

  this->dFringeCenterTarget = 0.0;

  return OK; 
}


/*******************************************************************
*
* tFtMediumTrackLockController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
  Sends target to Closed loop DL based on the frozen search, manual, lock, fringe center targets, and
  phase offset.
*
[[U tFtMediumTrackLockController::Calculate (tServoTime) ]]
****/
ERROR_CODE tFtMediumTrackLockController::Calculate (TIME tServoTime)
{
  
  double dWobbleTarget;

  /****
  [[U NextDlTarget.tDlTarget::tDlTarget (void) ]]
  ****/
  tDlTarget NextDlTarget;

  ERROR_CODE ec;

  ec = this->tFtMediumTrackController::Calculate (tServoTime);

  if (ec != OK) return ec;
 
  /****
  [[U NextDlTarget := this->tFtMediumTrackController::PhaseCorrection (tServoTime) ]]
  ****/
  NextDlTarget = this->PhaseCorrection (tServoTime);
  
  // Calculate wobble for next servo period..
  /****
  [[U dWobbleTarget := this->WobbleTarget.tModulationSource::Update (tServoTime) ]]
  ****/
  dWobbleTarget = this->WobbleTarget.Calculate (tServoTime + 
    this->ControllerParameters.tFtServoPeriod);
  

  // add in discrete position pffsets for Lock
   
  NextDlTarget.dPosition += this->dManualTarget + this->dLockTarget + 
                         dWobbleTarget + this->dFringeCenterTarget;

  
  /****
  [[U ec := this->ClosedLoopDl.tDlActuator::OutputValue (tServoTime, NewTarget) ]]
  ****/
  ec = this->ClosedLoopDl.OutputValue (tServoTime, NextDlTarget);

  /****
  [[U this->tFtMediumController::StoreData (tServoTime, FCM_SEARCH) ]]
  ****/
  this->StoreData (tServoTime, FCM_LOCK);
  
      /************* BEH -KMS************/
if (bRecordCor == 2) {

double dTemp = (double)tServoTime;

rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.A();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.B();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.C();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.D();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = Fringe.Phase();
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));

dTemp = dUnwrappedPhase;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.dPosition;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.dRate;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
dTemp = NextDlTarget.tTime;
rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));
//dTemp = NextDlTarget.dErrorSum;
//rngBufPut(riCorData,(char *) &dTemp ,sizeof(double));

dTemp = Fringe.Snr2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));
dTemp = Fringe.Visibility2();
rngBufPut(riCorData,(char *) &dTemp,sizeof(double));

}
/******************************/
  
  return ec; 
}


