/* $RCSfile: ftSlowController.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftSlowController.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a10Nov97,rlj    written
*/

#include "ft/ftSlowController.h"
#include "ft/ftIoSet.h"
#include "ftGizmoInfo.h"

/*******************************************************************
*
* tFtSlowController::tFtSlowController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs base class for all fringe tracker Slow controllers assigning
  references to a tFringe, and the fringe center target.
*
[[U tFtSlowController::tFtSlowController (CompleteIoSet) ]]
****/
tFtSlowController::tFtSlowController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtController::tFtController (void) ]]
  ****/
  tFtController (),
 
  /****
  [ this->ControllerParameters := reference to ControllerParametersToUse ]
  ****/
  ControllerParameters (ControllerParametersToUse),
  
  /****
  [ this->Fringe := reference to CompleteIoSet.Fringe ]
  ****/
  Fringe (CompleteIoSet.Fringe),

  /****
  [ this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget ]
  ****/
  dFringeCenterTarget (CompleteIoSet.dFringeCenterTarget) 

{}

/*******************************************************************
*
* tFtSlowController::~tFtSlowController ()
*
* DESCRIPTION:
  Destructs base class for all fringe tracker Slow controllers .
*
[[U tFtSlowController::~tFtSlowController () ]]
****/
tFtSlowController::~tFtSlowController () 
{

  /****
  [[U tFtController::~tFtController (void) ]]
  ****/
}
 
/*******************************************************************
*
* tFtSlowController::Init
*
* DESCRIPTION:
  Destructs base class for all fringe tracker Slow controllers .
*
[[U tFtSlowController::Init (tServoTime) ]]
****/
ERROR_CODE tFtSlowController::Init (TIME tServoTime) {

  this->dFringeCenterTarget = 0.0;
  return OK;

}

/*******************************************************************
*
* tFtSlowController::Reset 
*
* DESCRIPTION:
  Destructs base class for all fringe tracker Slow controllers .
*
[[U tFtSlowController::Reset (tServoTime) ]]
****/
ERROR_CODE tFtSlowController::Reset (TIME tServoTime) {

  this->dFringeCenterTarget = 0.0;
  return OK;

}


/*******************************************************************
*
* tFtSlowTrackLockController::tFtSlowTrackLockController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse)
*
* DESCRIPTION:
  Constructs FT Slow track controller.
*
[[U tFtSlowTrackLockController::tFtSlowTrackLockController (CompleteIoSet,
    ControllerParametersToUse) ]]
****/
tFtSlowTrackLockController::tFtSlowTrackLockController (tFtCompleteIoSet& CompleteIoSet,
    tFtControllerParameters& ControllerParametersToUse) :

  /****
  [[U tFtSlowController::tFtSlowController (CompleteIoSet,
      ControllerParametersToUse) ]]
  ****/
  tFtSlowController ( CompleteIoSet, ControllerParametersToUse)

{}

/*******************************************************************
*
* tFtSlowTrackLockController::~tFtSlowTrackLockController ()
*
* DESCRIPTION:
  Destructs FT Slow track controller.
*
[[U tFtSlowTrackLockController::~tFtSlowTrackLockController () ]]
****/
tFtSlowTrackLockController::~tFtSlowTrackLockController () 
{

  /****
  [[U tFtSlowController::~tFtSlowController (void) ]]
  ****/
}
 
/*******************************************************************
*
* tFtSlowTrackLockController::Calculate (TIME tServoTime)
*
* DESCRIPTION:
  Determines the direction of the center fringe based on average visibility2
  measuments, and updates the fringe center target appropriately assuming a 3 
  3 position wobble waveform  0 +x 0 -x 0 +x 0 -x ...
  The visibility averages are stored in 4 bins (0-3) since this waveform 
  has a period of 4 samples.  Therefore this method uses bins 1 and 3 
  corresponding to  +x and -x and caclutes the slope of the line  which 
  approximates the dV2/dx at x = 0 (Mean Value Theorym) If this slope is
  greater than a specified threshold, the fringe center target is updated
  appropriately based on the sign of the slope.
  
  This algorithm need to be modified in the future to do more sophisticated
  decision making but it will do for now.
*
[[U tFtSlowTrackLockController::Calculate (TIME tServoTime) ]]
****/
ERROR_CODE tFtSlowTrackLockController::Calculate (TIME tServoTime) {

  double dWobblePositionArray [this->ControllerParameters.iNumWobbleSamples];
  double dVisibilitySqrArray [this->ControllerParameters.iNumWobbleSamples];
  double dDeltaWobblePosition, dDeltaVisibilitySqr;
  double dVisibilitySlope;
  double dSnr2; // not used but needed for call to fringe.
  /****
  [[U this->Fringe.tFringe::WobbleVisibility2 (dWobblePositionArray, 
        dVisibilitySqrArray, this->ControllerParameters.iNumWobbleSamples)]]
  ****/
  this->Fringe.WobbleVisibility2 (dVisibilitySqrArray, dWobblePositionArray, 
    this->ControllerParameters.iNumWobbleSamples, &dSnr2);
  
  dDeltaWobblePosition = dWobblePositionArray [1] - 
    dWobblePositionArray [3];
   
  dDeltaVisibilitySqr =  dVisibilitySqrArray [1] -
     dVisibilitySqrArray [3];
    
  if (dDeltaWobblePosition == 0)
    dVisibilitySlope = 0;
  else 
    dVisibilitySlope = dDeltaVisibilitySqr/dDeltaWobblePosition;
   
  if (dVisibilitySlope > this->ControllerParameters.dDeltaVDeltaWobbleToHop )
    this->dFringeCenterTarget += this->ControllerParameters.lvWhiteLightLambda;
   
  else if (dVisibilitySlope < -1.0 * this->ControllerParameters.dDeltaVDeltaWobbleToHop )
    this->dFringeCenterTarget -= this->ControllerParameters.lvWhiteLightLambda;


  //  cout << " SNR = " << Fringe.Snr2 () << " SNR EST = " << Fringe.EstSnr2 () << endl;

    
  return OK;

}

