

/* $RCSfile: ftTypes.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftTypes.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard L Johnson - Jet Propulsion Laboratory */
 
#include "ftTypes.h"

/******************************************************************
*
* tFtSearchParameters::tFtSearchParameters (double dSpiralRadiusInitialToUse,
      double dSpiralRadiusLimitToUse, double dSpiralStepToUse,
      double dProgressionConstantToUse, SEARCH_TYPE stSearchTypeToUse)
*
* DESCRIPTION: 
    Constructs FT search parameters using initial values passed in.
*
[[U tFtSearchParameters::tFtSearchParameters_dddST (dSpiralRadiusInitialToUse,
      dSpiralRadiusLimitToUse, dSpiralStepToUse,
      dProgressionConstantToUse, stSearchTypeToUse) ]]
****/
tFtSearchParameters::tFtSearchParameters (double dSpiralRadiusInitialToUse,
      double dSpiralRadiusLimitToUse, double dSpiralStepToUse,
      double dProgressionConstantToUse, SEARCH_TYPE stSearchTypeToUse)
{
  this->dSpiralRadiusInitial = dSpiralRadiusInitialToUse;
  this->dSpiralRadiusLimit = dSpiralRadiusLimitToUse;
  this->dSpiralStep = dSpiralStepToUse;
  this->dProgressionConstant = dProgressionConstantToUse;
  this->stSearchType = stSearchTypeToUse;
}
 
/******************************************************************
*
* tFtThresholdParameters::tFtThresholdParameters (
      TIME tSemiLockToLockTimeoutToUse, 
      TIME tTotalTimeToLockTimeoutToUse, double dSNRThreshSearchToSemiLockToUse,
      double dSNRThreshSemiLockToLockToUse, double dV2ThreshSemiLockToLockToUse,
      double dSNRThreshLockToSearchToUse, double dThreshForDropToUse, 
      double dThreshForSearchToUse) 
*
* DESCRIPTION: 
    Constructs FT threshold parameters using initial values passed in.
*
[[U tFtThresholdParameters::tFtThresholdParameters_ttdddddd (
      tSemiLockToLockTimeoutToUse, 
      tTotalTimeToLockTimeoutToUse, dSNRThreshSearchToSemiLockToUse,
      dSNRThreshSemiLockToLockToUse, dV2ThreshSemiLockToLockToUse,
      dSNRThreshLockToSearchToUse, dThreshForDropToUse, 
      dThreshForSearchToUse) ]]
****/
tFtThresholdParameters::tFtThresholdParameters (
      TIME tSemiLockToLockTimeoutToUse, 
      TIME tTotalTimeToLockTimeoutToUse, double dSNRThreshSearchToSemiLockToUse,
      double dSNRThreshSemiLockToLockToUse, double dV2ThreshSemiLockToLockToUse,
      double dSNRThreshLockToSearchToUse, double dThreshForDropToUse, 
      double dThreshForSearchToUse) 
      
{
  this->tSemiLockToLockTimeout = tSemiLockToLockTimeoutToUse;
  this->tTotalTimeToLockTimeout = tTotalTimeToLockTimeoutToUse;
  this->dSNRThreshSearchToSemiLock = dSNRThreshSearchToSemiLockToUse;
  this->dSNRThreshSemiLockToLock = dSNRThreshSemiLockToLockToUse;
  this->dV2ThreshSemiLockToLock = dV2ThreshSemiLockToLockToUse;
  this->dSNRThreshLockToSearch = dSNRThreshLockToSearchToUse;
  this->dThreshForDrop = dThreshForDropToUse;
  this->dThreshForSearch = dThreshForSearchToUse;
}       

/******************************************************************
*
* tFtControllerParameters::tFtControllerParameters (TIME tFtServoPeriodToUse, 
      LASER_VALUE lvWhiteLightLambdaToUse,
      int iNumWobbleSamplesToUse,
      int iMemBoxcarToUse, double dFringeServoGainToUse, 
      double dDeltaVDeltaWobbleToHopToUse, int iPhaseSignToUse,
      double dPhaseOffsetToUse)
*
* DESCRIPTION: 
    Constructs FT controller parameters using initial values passed in.
*
[[U tFtControllerParameters::tFtControllerParameters_tiiiddid (
      tFtServoPeriodToUse, 
      lvWhiteLightLambdaToUse, iNumWobbleSamplesToUse, 
      iMemBoxcarToUse, dFringeServoGainToUse, 
      dDeltaVDeltaWobbleToHopToUse, iPhaseSignToUse,
      dPhaseOffsetToUse) ]]
****/
tFtControllerParameters::tFtControllerParameters (TIME tFtServoPeriodToUse, 
      LASER_VALUE lvWhiteLightLambdaToUse,
      int iNumWobbleSamplesToUse,
      int iMemBoxcarToUse, double dFringeServoGainToUse, 
      double dDeltaVDeltaWobbleToHopToUse, int iPhaseSignToUse,
      double dPhaseOffsetToUse)
{
  this->tFtServoPeriod = tFtServoPeriodToUse;
  this->lvWhiteLightLambda = lvWhiteLightLambdaToUse;
  this->iNumWobbleSamples = iNumWobbleSamplesToUse;
  this->iMemBoxcar = iMemBoxcarToUse;
  this->dFringeServoGain = dFringeServoGainToUse;
  this->dDeltaVDeltaWobbleToHop = dDeltaVDeltaWobbleToHopToUse;
  this->iPhaseSign = iPhaseSignToUse;
  this->dPhaseOffset = dPhaseOffsetToUse;
}

/******************************************************************
*
* tFtServoParameters::tFtServoParameters (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse)
*
* DESCRIPTION: 
    Constructs FT servo parameters using initial values passed in.
*
[[U tFtServoParameters::tFtServoParameters_tttt (
      SearchParametersToUse,WobbleParametersToUse,
      ControllerParametersToUse, ThresholdsToUse) ]]
****/
tFtServoParameters::tFtServoParameters (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse)
{
  this->SearchParameters = SearchParametersToUse;
  this->WobbleParameters = WobbleParametersToUse;
  this->ControllerParameters = ControllerParametersToUse;
  this->ThresholdParameters = ThresholdsToUse;
}


  
/******************************************************************
*
* tFtTargetStatus::tFtTargetStatus (double dManualToUse, double dSearchToUse, 
                     double dWobbleToUse, double dLockToUse,
                     double dFringeCenterToUse, double dPhaseToUse)
*
* DESCRIPTION: 
    Constructs target status using initial values passed in.
*
[[U tFtTargetStatus::tFtTargetStatus_ddddd (dManualToUse, dSearchToUse, 
                     dWobbleToUse, dLockToUse, dFringeCenterToUse, 
                     dPhaseToUse)]]
****/
tFtTargetStatus::tFtTargetStatus (double dManualToUse, double dSearchToUse, 
                     double dWobbleToUse, double dLockToUse,
                     double dFringeCenterToUse, double dPhaseToUse) 
{
  this->dManual = dManualToUse;
  this->dSearch = dSearchToUse;
  this->dWobble = dWobbleToUse;
  this->dLock = dLockToUse;
  this->dFringeCenter = dFringeCenterToUse;
  this->dPhase = dPhaseToUse;
}
 
/******************************************************************
*
* tFtTargetStatus::Manual ()
*
* DESCRIPTION: 
    Returns manual target.
*
[[U tFtTargetStatus::Manual (void) ]]
****/
double tFtTargetStatus::Manual () const 
{
  return this->dManual;
}

/******************************************************************
*
* tFtTargetStatus::Search ()
*
* DESCRIPTION: 
    Returns search target.
*
[[U tFtTargetStatus::Search (void) ]]
****/
double tFtTargetStatus::Search () const 
{
  return this->dSearch;
}


/******************************************************************
*
* tFtTargetStatus::Wobble ()
*
* DESCRIPTION: 
    Returns wobble target.
*
[[U tFtTargetStatus::Wobble (void) ]]
****/
double tFtTargetStatus::Wobble () const 
{
  return this->dWobble;
}


/******************************************************************
*
* tFtTargetStatus::Lock ()
*
* DESCRIPTION: 
    Returns lock target.
*
[[U tFtTargetStatus::Lock (void) ]]
****/
double tFtTargetStatus::Lock () const 
{
  return this->dLock;
}


/******************************************************************
*
* tFtTargetStatus::FringeCenter ()
*
* DESCRIPTION: 
    Returns fringe center target.
*
[[U tFtTargetStatus::FringeCenter (void) ]]
****/
double tFtTargetStatus::FringeCenter () const 
{
  return this->dFringeCenter;
} 

/******************************************************************
*
* tFtTargetStatus::Phase ()
*
* DESCRIPTION: 
    Returns phase target.
*
[[U tFtTargetStatus::Phase (void) ]]
****/
double tFtTargetStatus::Phase () const 
{
  return this->dPhase;
} 

/******************************************************************
*
* tFtTargetStatus::Total ()
*
* DESCRIPTION: 
    Returns sum of all FT targets.
*
[[U tFtTargetStatus::Total (void) ]]
****/
double tFtTargetStatus::Total () const
{
  return this->dManual + this->dSearch + this->dWobble + this->dLock +
    this->dFringeCenter;
}


/******************************************************************
*
* tFringeStatus::tFringeStatus (double dPhotonsToUse, double dVisibility2ToUse,
       double dSnr2ToUse)
*
* DESCRIPTION: 
    Constructs fringe status with initial values passed in.
*
[[U tFringeStatus::tFringeStatus_ddd (dPhotonsToUse, dVisibility2ToUse,
       dSnr2ToUse) ]]
****/
tFringeStatus::tFringeStatus (double dPhotonsToUse, double dVisibility2ToUse,
       double dSnr2ToUse) 
{
  this->dPhotons = dPhotonsToUse;
  this->dVisibility2 = dVisibility2ToUse;
  this->dSnr = dSnr2ToUse;
}

/******************************************************************
*
* tFringeStatus::Photons ()
*
* DESCRIPTION: 
    returns number of photon counted in a single FT period.
*
[[U tFringeStatus::Photons (void) ]]
****/
double tFringeStatus::Photons () const
{
  return this->dPhotons;
}

/******************************************************************
*
* tFringeStatus::Visibility2  ()
*
* DESCRIPTION: 
    returns visibility squared value.
*
[[U tFringeStatus::Visibility2 (void) ]]
****/
double tFringeStatus::Visibility2 () const
{
  return this->dVisibility2;
}

/******************************************************************
*
* tFringeStatus::Snr ()
*
* DESCRIPTION: 
    returns SNR value.
*
[[U tFringeStatus::Snr (void) ]]
****/
double tFringeStatus::Snr () const 
{
  return this->dSnr;
}


/******************************************************************
*
* tFtStatus::UpdateTime () 
*
* DESCRIPTION: 
    Updates status time stamp using current system time
*
[[U tFtStatus::UpdateTime (void) ]]
****/
void tFtStatus::UpdateTime () 
{
  /****
  [[U this->tTimeStamp := ::GetTime (void) ]]
  ****/
  this->tTimeStamp = ::GetTime ();
}

/******************************************************************
*
* tFtStatus::UpdateServoParameters (
    const tFtSearchParameters& SearchParametersToUse,
    const tModulationParams& WobbleParametersToUse,
    const tFtControllerParameters& ControllerParametersToUse,
    const tFtThresholdParameters& ThresholdsToUse)
*
* DESCRIPTION: 
    Updates servo parameters in status block with the values passed.
*
[[U tFtStatus::UpdateServoParameters_tttt (SearchParametersToUse,
    WobbleParametersToUse,ControllerParametersToUse, ThresholdsToUse)]]
****/
void tFtStatus::UpdateServoParameters (
    const tFtSearchParameters& SearchParametersToUse,
    const tModulationParams& WobbleParametersToUse,
    const tFtControllerParameters& ControllerParametersToUse,
    const tFtThresholdParameters& ThresholdsToUse)
{
  this->ServoParameters.SearchParameters = SearchParametersToUse;
  this->ServoParameters.WobbleParameters = WobbleParametersToUse;
  this->ServoParameters.ControllerParameters = ControllerParametersToUse;
  this->ServoParameters.ThresholdParameters = ThresholdsToUse;
}
  
/******************************************************************
*
* tFtStatus::UpdateServoParameters (const tFtServoParameters& ServoParameters)
*
* DESCRIPTION: 
    Updates servo parameters in status block.
*
[[U tFtStatus::UpdateServoParameters_t (ServoParametersToUse)]]
****/
void tFtStatus::UpdateServoParameters (const tFtServoParameters& ServoParametersToUse)
{
  this->ServoParameters = ServoParametersToUse;
}

/******************************************************************
*
* tFtStatus::UpdateImageParameters (const tFtImageParameters& ImageParametersToUse)
*
* DESCRIPTION: 
    Updates image parameters in status block.
*
[[U tFtStatus::UpdateImageParameters_t (ImageParametersToUse)]]
****/
void tFtStatus::UpdateImageParameters (const tFtImageParameters& ImageParametersToUse)
{
  this->ImageParameters = ImageParametersToUse;
}

/******************************************************************
*
* tFtStatus::UpdateFringeStatus (const tFringeStatus& FringeStatusToUse)
*
* DESCRIPTION: 
    Updates fringe status values.
*
[[U tFtStatus::UpdateFringeStatus (FringeStatusToUse)]]
****/
void tFtStatus::UpdateFringeStatus (const tFringeStatus& FringeStatusToUse)
{ 
  this->FringeStatus = FringeStatusToUse;
}


/******************************************************************
*
* tFtStatus::UpdateTargetStatus (const tTargetStatus& TargetStatusToUse)
*
* DESCRIPTION: 
    Updates target status values.
*
[[U tFtStatus::UpdateTargetStatus (TargetStatusToUse)]]
****/
void tFtStatus::UpdateTargetStatus (const tFtTargetStatus& TargetStatusToUse)
{ 
  this->Targets = TargetStatusToUse;
}

/******************************************************************
*
* tFtStatus::UpdateCommandStatus (FT_COMMAND_STATUS fcsCommandStatusToUse)
*
* DESCRIPTION: 
    Updates command status.
*
[[U tFtStatus::UpdateCommandStatus (fcsCommandStatusToUse)]]
****/
void tFtStatus::UpdateCommandStatus (FT_COMMAND_STATUS fcsCommandStatusToUse)
{
  this->fcsCommandStatus = fcsCommandStatusToUse;
}



/******************************************************************
*
* tFtStatus::UpdateServoMode (FT_SERVO_MODE fsmServoModeToUse)
*
* DESCRIPTION: 
    Updates servo mode status.
*
[[U tFtStatus::UpdateServoMode (fsmServoModeToUse)]]
****/
void tFtStatus::UpdateServoMode (FT_SERVO_MODE fsmServoModeToUse)
{
  this->fsmServoMode = fsmServoModeToUse;
}


/******************************************************************
*
* tFtStatus::UpdateControllerMode (FT_CONTROLLER_MODE fcmControllerModeToUse)
*
* DESCRIPTION: 
    Updates controller mode status.
*
[[U tFtStatus::UpdateControllerMode (fcmControllerModeToUse)]]
****/
void tFtStatus::UpdateControllerMode (FT_CONTROLLER_MODE fcmControllerModeToUse)
{
  this->fcmControllerMode = fcmControllerModeToUse;
}