/* $RCSfile: ftUpdateMode.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/ft/ftUpdateMode.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a17Nov97,rlj    written
*/

#include "ftTypes.h"
#include "ft/ftUpdateMode.h"
#include "ft/fringe.h"
#include "gizmo.h"
#include "ftError.h"

extern BOOL bTestDump;

/*******************************************************************
*
* tFtTrackGroupUpdate::tFtTrackGroupUpdate (tFringe& FringeToUse,
     tFtThresholdParameters& ThresholdsToUse)
*
* DESCRIPTION:
  Constructs FT track mode update class to use a particular fringe object,
  and set of thresholds.
*
[[U tFtTrackGroupUpdate::tFtTrackGroupUpdate (FringeToUse, ThresholdsToUse) ]]
****/
tFtTrackGroupUpdate::tFtTrackGroupUpdate (tFringe& FringeToUse,
     tFtThresholdParameters& ThresholdsToUse, TIME tFtMediumServoPeriodIn) :
     
  /****
  [[U tUpdateModeDecision::tUpdateModeDecision (void) ]]
  ****/
  tUpdateModeDecision (),

  /****
  [ this->Fringe := reference to FringeToUse ]
  ****/  
  Fringe (FringeToUse),
  
  /****
  [ this->Thresholds := reference to ThresholdsToUse ]
  ****/  
  Thresholds (ThresholdsToUse),
  
  tFtMediumServoPeriod (tFtMediumServoPeriodIn), 

  SnrDiag (10000,0.0)
  
{
  this->iCurrentControllerMode = FTM_SEARCH;
  iTrigger = 0;

}     


/*******************************************************************
*
* tFtTrackGroupUpdate::~tFtTrackGroupUpdate ()
*
* DESCRIPTION:
  Destructs FT track mode update class.
*
[[U tFtTrackGroupUpdate::~tFtTrackGroupUpdate (void) ]]
****/
tFtTrackGroupUpdate::~tFtTrackGroupUpdate () 

{
  /****
  [[U tUpdateModeDecision::~tUpdateModeDesicion (void) ]]
  ****/
}

/*******************************************************************
*
* tFtTrackGroupUpdate::Init ()
*
* DESCRIPTION:
  Initializes track group update.
*
[[U tFtTrackGroupUpdate::Init (void) ]]
****/
ERROR_CODE tFtTrackGroupUpdate::Init () 

{

  /****
  [[U iPhotons := this->Fringe.tFringe::Photons (void) ]]
  [ iPhotons < this->Thresholds.dThreshForSearch => 
      ::ErrorStream := ::ErrorStream + appropriate error message
  ]
  ****/
  if (Fringe.Photons () < this->Thresholds.dThreshForSearch) 
  
    /****
    [[U tGizmo::SendError_ec (S_ft_TOO_FEW_PHOTONS_FOR_SEARCH) ]]
    ****/
    tGizmo::SendError (S_ft_TOO_FEW_PHOTONS_FOR_SEARCH);
     
  this->iCurrentControllerMode = FTM_SEARCH;
  
  /****
  [[U this->tSearchStart := ::GetTime (void) ]]
  ****/
  this->tSearchStart = ::GetTime ();

  return OK;
}

ERROR_CODE tFtTrackGroupUpdate::Reset () 

{

  return OK;
}


/*******************************************************************
*
* tFtTrackGroupUpdate::GetNewMode  ()
*
* DESCRIPTION:
  Calcualtes new track mode based on current fringe measurements.
*
[[U tFtTrackGroupUpdate::GetNewMode (tUpdateTime) ]]
****/
int tFtTrackGroupUpdate::GetNewMode (TIME tUpdateTime)
{


 TIME tTimeInSearch, tTimeInSemiLock; 
 
 double dPhotons, dSnr2, dEstSnr2, dEstVisibility2;
 
 
 
  switch (this->iCurrentControllerMode) {
  
    case FTM_SEARCH :

      /****
      [[U dPhotons := this->Fringe.tFringe::Photons (void) ]]
      ****/
      dPhotons = this->Fringe.Photons ();

      tTimeInSearch = tUpdateTime - this->tSearchStart;
      
      /****
      [[U dSnr2 := this->Fringe.tFringe::Snr2 (void) ]]
      ****/
      dSnr2 = this->Fringe.Snr2 ();



      if (dPhotons < this->Thresholds.dThreshForDrop ) {
        /****
        [[U tGizmo::SendError_ec (S_ft_TOO_FEW_PHOTONS_TO_CONTINUE_SEARCH) ]]
        ****/
        tGizmo::SendError (S_ft_TOO_FEW_PHOTONS_TO_CONTINUE_SEARCH);
      } //if
      
      else if (tTimeInSearch >= this->Thresholds.tTotalTimeToLockTimeout) {
          /****
          [[U tGizmo::SendError_ec (S_ft_SEARCH_TIME_OUT) ]]
          ****/
          tGizmo::SendError (S_ft_SEARCH_TIME_OUT);
        } //else if
          
      else if ((tUpdateTime/this->tFtMediumServoPeriod) % 4 > 1){
         /* Allow data to settle (two  peroids) after moving search target */
            
         if (dSnr2 >= this->Thresholds.dSNRThreshSearchToSemiLock)  {
           this->iCurrentControllerMode = FTM_SEMI_LOCK;
           this->tSemiLockStart = tUpdateTime;

           cout << "SNR2 = " << dSnr2 << " V2 = " << this->Fringe.Visibility2 () 
                <<" TIME = " << tUpdateTime << endl;

         } //if
       } //else if  
        
     break;
   
    case FTM_SEMI_LOCK :  
      
      tTimeInSemiLock = tUpdateTime - this->tSemiLockStart;
      
      /****
      [[U dEstSnr2 := this->Fringe.tFringe::EstSnr2 (void) ]]
      ****/
      dEstSnr2 = this->Fringe.EstSnr2 (); 
             
      /****
      [[U dEstVisibility2 := this->Fringe.tFringe::EstVisibility2 (void) ]]
      ****/
      dEstVisibility2 = this->Fringe.EstVisibility2 (); 
      
      if (tTimeInSemiLock >= this->Thresholds.tSemiLockToLockTimeout) {
             this->iCurrentControllerMode = FTM_SEARCH;  
             cout << "Exit SL Est SNR2 = " << dEstSnr2 << " Est V2 = " 
             << dEstVisibility2 << " tTimeInSemiLock =" << tTimeInSemiLock
             << " TIME = " << tUpdateTime << endl;
            }
       
      else if ( 
        dEstSnr2 >= this->Thresholds.dSNRThreshSemiLockToLock &&
        dEstVisibility2 >= this->Thresholds.dV2ThreshSemiLockToLock ) {

          this->iCurrentControllerMode = FTM_LOCK;
          cout << "Enter Lock Est SNR2 = " << dEstSnr2 << " Est V2 = " << dEstVisibility2 << " TIME = " << 
            tUpdateTime << endl;
      }
      break;
      
    case FTM_LOCK :

      /****
      [[U dEstSnr2 := this->Fringe.tFringe::EstSnr2 (void) ]]
      ****/
      dEstSnr2 = this->Fringe.EstSnr2 ();
      
      if (dEstSnr2 <  this->Thresholds.dSNRThreshLockToSearch ) {
        this->iCurrentControllerMode = FTM_SEARCH;
        this->tSearchStart = tUpdateTime;  // reset the search time
         cout << "Exit Lock Est SNR2 = " << dEstSnr2 << " Est V2 = " << dEstVisibility2 << " TIME = " << 
            tUpdateTime << endl <<  endl;
       } //if
      
      break;
    
    default : 
      this->iCurrentControllerMode = FTM_SEARCH;

      /****
      [[U tGizmo::SendError_ec (S_ft_INVALID_TRACK_CONTROLLER_MODE) ]]
      ****/
      tGizmo::SendError (S_ft_INVALID_TRACK_CONTROLLER_MODE);  
    
    } // switch
    
  return this->iCurrentControllerMode;
}

           
