/* $RCSfile: phaseTarget.cc,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/gizmo/ft/phaseTarget.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $ */

/*
modification history
--------------------
04c,4Dec97, rlj    written
*/

#include "ft/phaseTarget.h"
#include "isit.h"
  
BOOL bOne = TRUE;


/******************************************************************
*
* tPhaseTarget::tPhaseTarget () 
*
* DESCRIPTION:
[[U tPhaseTarget::tPhaseTarget (void) ]]
*/
tPhaseTarget::tPhaseTarget () :

  dPosition  (0.0),
  dVelocity (0.0),
  tTime (0),
  dErrorSum (0.0)
{}


/******************************************************************
*
* tPhaseTarget::Reset () 
*
* DESCRIPTION:
[[U tPhaseTarget::Reset (void)  ]]
*/
ERROR_CODE tPhaseTarget::Reset () 
{
  this->dPosition = 0.0;
  this->dVelocity = 0.0;
  this->tTime = 0;
  this->dErrorSum = 0.0;
  return OK;
}


/******************************************************************
*
* tPhaseTarget::Calculate (const tFtControllerParameters& ControllerParameters,
                           TIME tServoTime, double dPhaseError)  
*
* DESCRIPTION:
[[U tPhaseTarget::Calculate (ControllerParameters, tServoTime, dPhaseError)   ]]
*/
tDlTarget tPhaseTarget::Calculate (const tFtControllerParameters& ControllerParameters,
     TIME tServoTime, double dPhaseError) 
{

  double dGain = -1.0 * ControllerParameters.iPhaseSign * ControllerParameters.dFringeServoGain * 
    ControllerParameters.lvWhiteLightLambda / PI;

  double dPeriodInSeconds = (double) ControllerParameters.tFtServoPeriod / (double) TIME_TICKS_PER_SECOND;
  
  if (dPeriodInSeconds == 0.0) return tDlTarget (0.0, 0.0, 0);
  
  this->dPosition += this->dVelocity * dPeriodInSeconds;
  this->dVelocity = dGain * this->dErrorSum / dPeriodInSeconds;
  this->tTime = tServoTime + ControllerParameters.tFtServoPeriod;
  this->dErrorSum += dPhaseError;

  // Diagnostics
  /**
  if (bOne) {
    printf ("Pos = %f, Vel = %f, dGain = %f, ErrorSum = %f, ServoPeriod = %ld, Time = %ld\n",
             dPosition,dVelocity,dGain,dErrorSum,ControllerParameters.tFtServoPeriod,tTime);
    bOne = FALSE;
  }
  ****/

  /****
  [[U tDlTarget::tDlTarget (this->dPosition, this->dVelocity, this->tTime) ]]
  ****/
  return tDlTarget (this->dPosition, this->dVelocity, this->tTime);
  
} 

/******************************************************************
*
* tPhaseTarget::Position ()  
*
* DESCRIPTION:
[[U tPhaseTarget::Position (void)  ]]
*/
double tPhaseTarget::Position () const
{
  return this->dPosition;
}

/******************************************************************
*
* tPhaseTarget::Velocity ()
*
* DESCRIPTION:
[[U tPhaseTarget::Velocity (void)  ]]
*/
double tPhaseTarget::Velocity () const
{
  return this->dVelocity;
}

/******************************************************************
*
* tPhaseTarget::Time ()
*
* DESCRIPTION:
[[U tPhaseTarget::Time (void)  ]]
*/
TIME tPhaseTarget::Time () const 
{
  return this->tTime;
}

/******************************************************************
*
* tPhaseTarget::ErrorSum ()
*
* DESCRIPTION:
[[U tPhaseTarget::ErrorSum (void)  ]]
*/
double tPhaseTarget::ErrorSum () const 
{
  return this->dErrorSum;
}



        
