/* $RCSfile: gizmo.cc,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/gizmo/genericGizmo/gizmo.cc,v 1.4 1998/09/23 23:57:09 phil Exp $ */

/*
modification history
--------------------
01a,23Jun97,rlj    written
*/


#include "isit.h"
#include "ipcid.h"
#include "gizmoInfo.h"
#include "gizmo.h"
#include "Scheduler.h"
#include "relayStream.h"
#include "errorStream.h"



/****
[[U tGizmo::tGizmo (GizmoInfo) ]]
****/
tGizmo::tGizmo (const tGizmoInfo& GizmoInfo) :
  
  /****
  [[U tBaseObject::tBaseObject () ]]
  ****/
  tBaseObject (),
  
  /****
  [[U this->CommandStream.tCommandStreamReceiver::tCommandStreamReceiver (
       GizmoInfo.CommandStreamId)  ]]
  ****/
  CommandStream (GizmoInfo.GetCommandStreamId() ) 

{
  ERROR_CODE ecTemp;
  
  TraceMark (S_gizmo_INIT_START);

  ecTemp = this->CommandStream.GetInitError ();
  
  if (ecTemp != OK )  {
    TraceMsg (tmt_ERROR,ecTemp,"tGizmo::tGizmo()");
    
    tBaseObject::SetInitError (ecTemp);
  }
    
    /****
    [[U tBaseObject::SetInit (BOOL bInitToUse)  ]]
    ****/
    tBaseObject::SetInit (FALSE);

    return;



}


/****
[[U tGizmo::StartTasks (GizmoInfo) ]]
****/
ERROR_CODE tGizmo::StartTasks (const tGizmoInfo& GizmoInfo) {

  ERROR_CODE ecTemp;

  char sCommandTaskName [25];
  char sStatusTaskName [25];

  sprintf (sCommandTaskName,"%d_",GizmoInfo.GetGizmoId ());
  strcat (sCommandTaskName,"Command Processor");
 
  sprintf (sStatusTaskName,"%d_",GizmoInfo.GetGizmoId ());
  strcat (sStatusTaskName,"Status Relay");
  
  int iStatusTaskPriority = GizmoInfo.GetStatusRelayPriority ();
  TIME tStatusTaskPeriod = GizmoInfo.GetStatusRelayPeriod();
  TIME tStatusTaskOffset = GizmoInfo.GetStatusRelayOffset();
  
  int iCommandTaskPriority = GizmoInfo.GetCommandRelayPriority ();
  TIME tCommandTaskPeriod = GizmoInfo.GetCommandRelayPeriod();
  TIME tCommandTaskOffset = GizmoInfo.GetCommandRelayOffset();


if ((ecTemp = tBaseObject::GetInitError () ) != OK)  {
    TraceMsg (tmt_WARN, ecTemp,"tGizmo::StartTasks()");
    return ecTemp;
  }
  
 if (tBaseObject::IsInit() == TRUE)  {
    TraceMsg (tmt_WARN, S_gizmo_ALREADY_INITIALIZED,"tGizmo::StartTasks()");
    return OK;
  }

 

  /****
  [[U tScheduler::AddTask (&this->iStatusTaskRelayTaskId, 
                           sStatusTaskName, 
                           this->ProcessStatus, 
                           NULL, NULL, this , 
                           iStatusTaskPriority,
                           &tStatusTaskPeriod,
                           &tStatusTaskOffset, 
                           TEB_CONTINUE, TRUE ) ]]
  ****/
  ecTemp = ::GetScheduler().AddTask (&this->iStatusRelayTaskId, 
                           sStatusTaskName, 
                           (FUNCPTR) this->ProcessStatus, 
                           NULL, NULL, (void *) this , 
                           iStatusTaskPriority,
                           &tStatusTaskPeriod,
                           &tStatusTaskOffset, 
                           TEB_CONTINUE, TRUE );

  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::StartTasks");



    
  /****
  [[U tScheduler::AddTask (&this->iProcessCommandTaskId, 
                           sCommandTaskName, 
                           this->ProcessCommand, 
                           NULL, NULL, this, 
                           iCommandTaskPriority, 
                           &tCommandTaskPeriod,
                           &tCommandTaskOffset, 
                           TEB_CONTINUE, TRUE ) ]]
  ****/
  ecTemp = ::GetScheduler().AddTask (&this->iProcessCommandTaskId, 
                           sCommandTaskName, 
                           (FUNCPTR) this->ProcessCommand, 
                           NULL, NULL, (void *) this, 
                           iCommandTaskPriority, 
                           &tCommandTaskPeriod,
                           &tCommandTaskOffset, 
                           TEB_CONTINUE, TRUE );
   
  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::StartTasks");  

  /****
  [[U tBaseObject::SetInit (BOOL bInitToUse)  ]]
  ****/
  this->SetInit (TRUE);

  TraceMark (S_gizmo_INIT_COMPLETE);
    
  return OK;
    
}

/****
[[U tGizmo::~tGizmo (void) ]]
****/
tGizmo::~tGizmo () {
  
  ERROR_CODE ecTemp;
 
 
  TraceMark (S_gizmo_DESTRUCT_START);
    

  if (tBaseObject::IsInit() ) {
     TraceMsg (tmt_WARN, S_gizmo_SHUTTING_DOWN_GIZMO,"tGizmo::~tGizmo()");
     
     /****
     [[U tGizmo::Shutdown() ]]
     ****/
     ecTemp = this->Shutdown ();
    
     if (ecTemp != OK)
       TraceMsg (tmt_ERROR,S_gizmo_SHUTDOWN_ERROR,"tGizmo::~tGizmo() ");
   }
 
  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::DeleteTask (
                    this->iProcessCommandTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().DeleteTask (this->iProcessCommandTaskId);
 
  if (ecTemp != OK) 
    TraceMsg (tmt_ERROR,ecTemp ,"tGizmo::~tGizmo() ");

  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::DeleteTask (
                    this->iStatusRelayTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().DeleteTask (this->iStatusRelayTaskId);
 
  if (ecTemp != OK) 
    TraceMsg (tmt_ERROR,ecTemp ,"tGizmo::~tGizmo() ");

  TraceMark (S_gizmo_DESTRUCT_COMPLETE);
 
  /****
  [[U tRelayStreamReceiver::~tRelayStreamReceiver (void) ]]
  [[U tBaseObject::~tBaseObject() ]]
  ****/

}


/****
[[U tGizmo::Restart (void) ]]
****/
ERROR_CODE tGizmo::Restart () {
  
  ERROR_CODE ecTemp;
  
  TraceMark (S_gizmo_RESTART_START);
  
  if (tBaseObject::IsInit() == TRUE)  {
    TraceMsg (tmt_WARN, S_gizmo_ALREADY_INITIALIZED,"tGizmo::Restart()");
    return OK;
  }
  
  /****
  [[U ecTemp := this->CommmandStream.tCommandStreamReceiver::Restart (void) ]]
  ****/
  ecTemp = this->CommandStream.Restart();
  
  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Restart()");

  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::EnableTask (
                 this->iProcessCommandTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().EnableTask (this->iProcessCommandTaskId);
  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Restart()");
  
  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::EnableTask (
                 this->iStatusRelayTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().EnableTask (this->iStatusRelayTaskId);
  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Restart()");
 
  TraceMark (S_gizmo_RESTART_COMPLETE);
  return OK;
}



/****
[[U tGizmo::Shutdown (void) ]]
****/
ERROR_CODE tGizmo::Shutdown () {
  
  ERROR_CODE ecTemp;
  
  TraceMark (S_gizmo_SHUTDOWN_START);
  
  if (tBaseObject::IsInit() == FALSE )  {
    TraceMsg (tmt_WARN, S_gizmo_ALREADY_SHUTDOWN,"tGizmo::Shutdown()");
    return OK;
  }
  
  /****
  [[U ecTemp := this->CommandStream.tCommandStreamReceiver::Shutdown(void) ]]
  ****/
  ecTemp = this->CommandStream.Shutdown();
  
  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Shutdown()");

  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::DisableTask (
                  this->iProcessCommandTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().DisableTask (this->iProcessCommandTaskId);

  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Shutdown()");
  
  /****
  [[U ecTemp := (::pThisRteG)->pScheduler->tScheduler::DisableTask (
                  this->iStatusRelayTaskId) ]]
  ****/
  ecTemp = ::GetScheduler().DisableTask (this->iStatusRelayTaskId);

  if (ecTemp != OK) return Traceback (ecTemp,"tGizmo::Shutdown()");
 
  TraceMark (S_gizmo_SHUTDOWN_COMPLETE);

  return OK;
}



/****
[[U tGizmo::IsOk (void) ]]
****/
BOOL tGizmo::IsOk () {
  
      /****
      [[U this->CommandStream.tCommandStreamReceiver::IsOk (void)]]
      ****/
  return ( this->CommandStream.IsOk () && tBaseObject::IsOk () );
 
 
}



/****
[[U tGizmo::SendData_tp (Tag,pData) ]]
****/
ERROR_CODE tGizmo::SendData (const tTypeTag& Tag, const BYTE* pData) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).DataStream.tRelayStreamSender::Put (Tag,pData) ]]
  ****/
  ec = ::GetDataStream ().Put (Tag,pData);
  
  if (ec != OK)
 
    /****
    [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ec (ec) ]]
    ****/
    ec = ::GetErrorStream ().Put (ec);

  return ec;
}




/****
[[U tGizmo::SendData_tt (Tag,BufferIn) ]]
****/
ERROR_CODE tGizmo::SendData (const tTypeTag& Tag, const tBuffer& BufferIn) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).DataStream.tRelayStreamSender::Put (Tag,BufferIn) ]]
  ****/
  ec = ::GetDataStream ().Put (Tag,BufferIn);
  
  if (ec != OK)
 
    /****
    [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ec (ec) ]]
    ****/
    ec = ::GetErrorStream ().Put (ec);

  return ec;
}


/****
[[U tGizmo::SendStatus_tp (Tag,pStatus) ]]
****/
ERROR_CODE tGizmo::SendStatus (const tTypeTag& Tag, const BYTE* pStatus) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).StatusStream.tRelayStreamSender::Put (Tag,pData) ]]
  ****/
  ec = ::GetStatusStream ().Put (Tag,pStatus);
  
  if (ec != OK)

    /****
    [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ec (ec) ]]
    ****/
    ec = ::GetErrorStream ().Put (ec);
  
  return ec;
}



/****
[[U tGizmo::SendStatus_tt (Tag,BufferIn) ]]
****/
ERROR_CODE tGizmo::SendStatus (const tTypeTag& Tag, const tBuffer& BufferIn) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).StatusStream.tRelayStreamSender::Put (Tag,BufferIn) ]]
  ****/
  ec = ::GetStatusStream ().Put (Tag,BufferIn);
  
  if (ec != OK)

    /****
    [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ec (ec) ]]
    ****/
    ec = ::GetErrorStream ().Put (ec);
  
  return ec;
}



/****
[[U tGizmo::SendError_ec (ecErrCode) ]]
****/
ERROR_CODE tGizmo::SendError (ERROR_CODE ecErrCode) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ec (ecErrCode) ]]
  ****/
  ec = ::GetErrorStream ().Put (ecErrCode);

  return ec;
}



/****
[[U tGizmo::SendError_ect (ecErrCode, tErrTime) ]]
****/
ERROR_CODE tGizmo::SendError (ERROR_CODE ecErrCode, TIME tErrTime) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::Put_ect (ecErrCode,tErrTime) ]]
  ****/
  ec = ::GetErrorStream ().Put (ecErrCode,tErrTime);


  return ec;
}
  
/****
[[U tGizmo::SendErrorNoWait (ecErrCode, tErrTime) ]]
****/
ERROR_CODE tGizmo::SendErrorNoWait (ERROR_CODE ecErrCode, TIME tErrTime) {

  ERROR_CODE ec;
  
  /****
  [[U ec := (::pThisRteG).ErrorStream.tErrorStreamSender::PutNoWait_ect (
						  ecErrCode, tErrTime) ]]
  ****/
  ec = ::GetErrorStream ().PutNoWait (ecErrCode,tErrTime);


  return ec;
}  
  
  
/****
[[U tGizmo::ProcessCommand () ]]
****/
ERROR_CODE tGizmo::ProcessCommand () {

  ERROR_CODE ec;
  ISIT_CMD Command;
  

  /****
  [[U ec := this->CommandStream.tCommandStreamReceiver::Get (ISIT_CMD &DataOut) ]]
  ****/
  ec = this->CommandStream.Get(Command);

  if (ec == S_ipc_FIFO_EMPTY) return OK;
  
  if (ec != OK) {
  	Traceback (ec, "tGizmo::ProcessCommand");
  	return ec;
    }	 
 
  /****
  [[U ec := tGizmo::Dispatch (Command.genericCmd) ]]
  ****/
  
  ec = this->Dispatch(Command.genericCmd);
  
  return ec;
}  
  
/****
[[U tGizmo::ProcessStatus (void) ]]
****/
ERROR_CODE tGizmo::ProcessStatus () {

  
  /****
  [[U tGizmo::CollectStatus (void) ]]
  ****/
  
  return this->CollectStatus ();

}  
  

