/* $RCSfile: gizmoInfo.cc,v $ */
/* CM Info: */   

/*$Header: /home/chara/cvsroot/ople/sys/gizmo/genericGizmo/gizmoInfo.cc,v 1.3 1999/05/17 23:32:42 theo Exp $ */

/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,29Aug97,rlj    written 

*/

#include "configMgr.h"
#include "gizmoInfo.h"

  
const int CiStatusTaskPriority  = 95;
const TIME CtStatusTaskPeriod    = 20000;
const int CiCommandTaskPriority = 80;
// changed to 12000 to see if it would fix starving probs - PcI 5/17/99
//const TIME CtCommandTaskPeriod    = 8000;
const TIME CtCommandTaskPeriod    = 12000;


tGizmoInfo::tGizmoInfo (int iRteIdToUse, int iGizmoIdToUse) {
         
  IPC_NAME inName;
  MEM_TYPE mtMemType;
  
  this->iRteId = iRteIdToUse;
  this->iGizmoId = iGizmoIdToUse;
  this->gtGizmoType = GT_UNKNOWN;

  // All status, data, and command streams go between the gizmo 
  //   and the instrument gizmo
  mtMemType = this->TypeOfMemory (iRteId, CiInst);

  /****
  [[U inName := tGizmoInfo::NewIpcName (void) ]]
  ****/ 
  inName = NewIpcName ();
  
  TraceMsg(tmt_INFO, OK, "Stream Memory Type is %d for Name 0x%x on RTE #%d",
           (int)mtMemType, (int)inName, this->iRteId);

  /****
  [[U this->iiCommandStreamId.tIpcFifoId::tIpcFifoId_INMTi (inName,mtMemType,
             CiCommandFifoDepth)]]
  ****/
  this->iiCommandStreamId = tIpcFifoId (inName,mtMemType,CiCommandFifoDepth);

  // for now all command and status tasks have same characteristics unless
  // overidden by a derived info class

  this->iStatusTaskPriority = CiStatusTaskPriority;
  this->tStatusTaskPeriod   = CtStatusTaskPeriod;
  this->tStatusTaskOffset   = 0;
  this->iCommandTaskPriority = CiCommandTaskPriority;
  this->tCommandTaskPeriod = CtCommandTaskPeriod;
  this->tCommandTaskOffset = 0;
  
}




/****
[[U tGizmoInfo::NewIpcName (void) ]]
****/
IPC_NAME tGizmoInfo::NewIpcName () {

  static iNameOffset = 0;
  
  iNameOffset++;
  return ((this->iGizmoId << 16) | iNameOffset);
}


/****
[[U tGizmoInfo::TypeOfMemory (iMyRteNum, int iOtherGizmoId) ]]
****/
MEM_TYPE tGizmoInfo::TypeOfMemory (int iMyRteNum, int iOtherGizmoId) const {
  
  int iOtherGizmoRteNum = ConfigManager.GetGizmoRteNumber (iOtherGizmoId);
  
  if (iOtherGizmoRteNum == iMyRteNum) return MT_LOCAL;

  else return MT_SHARED;
}
  



