/* $RCSfile: imageStatus.cc,v $ */
/* CM Info:   
$Header $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#include "imageStatus.h"
#include <semLib.h>
#include "gizmo.h"

/****
[[U tImageStatus::tImageStatus (ImageTagToUse, iHSize,iVSize) ]]
****/
tImageStatus::tImageStatus (const tTypeTag& ImageTagToUse,
       int iHSize, int iVSize) :

  /****
  [U this->CImageTag := ImageTagToUse ]
  ****/
  CImageTag (ImageTagToUse),
  
  /****
  [[U this->Image.tImage::tImage_ii (iHSize,iVSize) ]]
  ****/
  Image (iHSize,iVSize),

  /****
  [[U this->DefaultImage.tImage::tImage_ii (iHSize,iVSize) ]]
  ****/
  DefaultImage (iHSize,iVSize) {

  int x,y;

  this->siImageMutexSemaphore = semMCreate (SEM_Q_PRIORITY);

 
  //  Default CheckerBoard Pattern with four squares

  for (x = 0; x < iHSize/2; x++) 
    for (y = 0; y < iVSize/2; y++ ) 
 
      /****
      [[U this->DefaultImage.tImage::GetPixel (x,y) ]]
      ****/
      this->DefaultImage.GetPixel (x,y) = 255; 

  for (x = iHSize/2 + 1; x < iHSize; x++) 
    for (y = iVSize/2; y < iVSize;  y++ ) 

      /****
      [[U this->DefaultImage.tImage::GetPixel (x,y) ]]
      ****/
      this->DefaultImage.GetPixel (x,y) = 255;
 
  for (x = 0; x < iHSize/2; x++) 
    for (y = iVSize/2; y < iVSize;  y++ ) 

      /****
      [[U this->DefaultImage.tImage::GetPixel (x,y) ]]
      ****/
      this->DefaultImage.GetPixel (x,y) = 0; 

  for (x = iHSize/2 + 1; x < iHSize; x++) 
    for (y = 0; y < iVSize/2; y++ ) 

      /****
      [[U this->DefaultImage.tImage::GetPixel (x,y) ]]
      ****/
      this->DefaultImage.GetPixel (x,y) = 0;
 
  this->Image = this->DefaultImage;
}


/****
[[U tImageStatus::~tImageStatus (void) ]]
****/
tImageStatus::~tImageStatus () {

  /****
  [[U ::semDelete (this->siImageMutexSemaphore)]]
  ****/
  semDelete (this->siImageMutexSemaphore);

}


/****
[[U tImageStatus::SetDefaultImage (Image) ]]
****/
ERROR_CODE tImageStatus::SetDefaultImage (const tImage& Image) {


    /****
    [[U this->DefaultImage.tImage::IsSameSize_t (Image) ]]
    ****/
  if (this->DefaultImage.IsSameSize (Image) == FALSE) 
    return S_imagestatus_INCORRECT_IMAGE_SIZE;

  
  this->DefaultImage = Image;
  return OK;
}


/****
[[U tImageStatus::Update_t (Image) ]]
****/
ERROR_CODE tImageStatus::Update (tImage& Image) {

    /****
    [[U this->Image.tImage::IsSameSize_t (Image) ]]
    ****/
  if (this->Image.IsSameSize (Image) == FALSE)
    return S_imagestatus_INCORRECT_IMAGE_SIZE;
  
  /****
  [[U ::semTake (this->siImageMutexSemaphore,WAIT_FOREVER) ]]
  ****/
  semTake (this->siImageMutexSemaphore,WAIT_FOREVER);

  this->Image = Image;
 
  /****
  [[U ::semGive (this->siImageMutexSemaphore) ]]
  ****/
  semGive (this->siImageMutexSemaphore);

  return OK;
}


/****
[[U tImageStatus::Update (void) ]]
****/
void tImageStatus::Update () {

  /****
  [[U ::semTake (this->siImageMutexSemaphore,WAIT_FOREVER) ]]
  ****/
  semTake (this->siImageMutexSemaphore,WAIT_FOREVER);

  this->Image = this->DefaultImage;
 
  /****
  [[U ::semGive (this->siImageMutexSemaphore) ]]
  ****/
  semGive (this->siImageMutexSemaphore);
}


/****
[[U tImageStatus::GetImage (OutImage) ]]
****/
void tImageStatus::GetImage (tImage& OutImage) {

  /****
  [[U ::semTake (this->siImageMutexSemaphore,WAIT_FOREVER) ]]
  ****/
  semTake (this->siImageMutexSemaphore,WAIT_FOREVER);

  OutImage = this->Image;

  /****
  [[U ::semGive (this->siImageMutexSemaphore) ]]
  ****/
  semGive (this->siImageMutexSemaphore);
   
}


/****
[[U tImageStatus::GetImage (void) ]]
****/
size_t tImageStatus::GetSize () { 

  /****
  [[U this->Image.tBuffer::GetSize (void) ]]
  ****/
  return this->Image.GetSize ();
}


/****
[[U tImageStatus::SendToStatusStream (void) ]]
****/
ERROR_CODE tImageStatus::SendToStatusStream () 
{
  ERROR_CODE ec;
  
  /****
  [[U ec = tGizmo::SendStatus_tt (this->CImageTag,this->Image) ]]
  ****/
  ec = tGizmo::SendStatus (this->CImageTag,this->Image);
  
  if (ec != OK) return Traceback (ec, "tImageStatus::SendToStatusStream");

  return OK;
}
 











