/* $RCSfile: realTimeGizmo.cc,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/gizmo/genericGizmo/realTimeGizmo.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $ */

/*
modification history
--------------------
01a,4sept97,kms    written
*/


#include "isit.h"
#include "realTimeGizmoInfo.h"
#include "realTimeGizmo.h"
#include "servoInfo.h"


/****
[[U tRealtimeGizmo::tRealtimeGizmo (GizmoInfo) ]]
****/
tRealtimeGizmo::tRealtimeGizmo (const tRealtimeGizmoInfo& GizmoInfo) :
  
  /****
  [[U tGizmo::tGizmo (GizmoInfo) ]]
  ****/
  tGizmo (GizmoInfo),
  
  /****
  [[U this->ControllerSet.tControllerSet::
         tControllerSet_ii (GizmoInfo.iNumberOfServoModes,
                         GizmoInfo.iNumberOfServoPartitions) ]]
  ****/  
  ControllerSet (GizmoInfo.GetNumberOfServoModes (),
                 GizmoInfo.GetNumberOfServoPartitions () ),
  
  /****
  [[U this->Servo.tServo::tServo (GizmoInfo.ServoInfo, this->ControllerSet) ]]
  ****/  
  Servo (*new tServoInfo (GizmoInfo.GetServoInfo ()), ControllerSet) {
  
  ERROR_CODE ec;
  
  TraceMark (S_realtime_gizmo_CONSTRUCTOR_START);

  ec = this->Servo.GetInitError ();
  if (ec != OK) {
    TraceMsg (tmt_ERROR, ec, "tRealtimeGizmo::tRealtimeGizmo" );
    tBaseObject::SetInitError (ec);
  }
 
  TraceMark (S_realtime_gizmo_CONTRUCTOR_COMPLETE);
}  


/****
[[U tRealtimeGizmo::StartTasks (GizmoInfo) ]]
****/
ERROR_CODE tRealtimeGizmo::StartTasks (const tRealtimeGizmoInfo& GizmoInfo)  {

  ERROR_CODE ecTemp;
  
  /****
  [[U ecTemp := this->Servo.tServo::StartServo (void) ]]
  ****/
  ecTemp = this->Servo.StartServo ();
                           
  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::StartTasks");
                                               
  ecTemp = tGizmo::StartTasks (GizmoInfo);
  
  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::StartTasks");
                                               
  TraceMark (S_realtime_gizmo_START_TASKS_COMPLETE);
    
  return OK;
    
}

/****
[[U tRealtimeGizmo::~tRealtimeGizmo (void) ]]
****/
tRealtimeGizmo::~tRealtimeGizmo () {
  
  ERROR_CODE ecTemp;
  
  TraceMark (S_realtime_gizmo_DESTRUCTOR_START);
    

  if (tGizmo::IsInit() ) {
     TraceMsg (tmt_WARN, S_realtime_gizmo_SHUTTING_DOWN_GIZMO,
     	"tRealtimeGizmo::~tRealtimeGizmo()");
     
     /****
     [[U ecTemp := tRealtimeGizmo::Shutdown (void) ]]
     ****/
     ecTemp = this->Shutdown ();
    
     if (ecTemp != OK)
       TraceMsg (tmt_ERROR,ecTemp,"tRealtimeGizmo::~tRealTimeGizmo() ");
   }
 
  
  TraceMark (S_realtime_gizmo_DESTRUCTOR_COMPLETE);
 
  /****
  [[U this->Servo.tServo::~tServo (void) ]]
  [[U this->ControllerSet.tControllerSet::~tControllerSet (void) ]]
  [[U tGizmo::~tGizmo() ]]
  ****/

}


/****
[[U tRealtimeGizmo::Restart (void) ]]
****/
ERROR_CODE tRealtimeGizmo::Restart () {
  
  ERROR_CODE ecTemp;
  
  TraceMark (S_realtime_gizmo_RESTART_START);
  
  if (tGizmo::IsInit() == TRUE)  {
    TraceMsg (tmt_WARN, S_realtime_gizmo_ALREADY_RUNNING,"tGizmo::Restart()");
    return OK;
  }
  
  
  /****
  [[U ecTemp := this->Servo.tServo::Restart (void) ]]
  ****/
  ecTemp = this->Servo.Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::Restart()");
 
  /****
  [[U ecTemp := tGizmo::Restart (void) ]]
  ****/
  ecTemp = tGizmo::Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::Restart()");

  TraceMark (S_realtime_gizmo_RESTART_COMPLETE);

  return OK;
}


/****
[[U tRealtimeGizmo::Shutdown (void) ]]
****/
ERROR_CODE tRealtimeGizmo::Shutdown () {
  
  ERROR_CODE ecTemp;
  
  TraceMark (S_realtime_gizmo_SHUTDOWN_START);
  
  if (tGizmo::IsInit() == FALSE ) {
    TraceMsg (tmt_WARN, S_realtime_gizmo_ALREADY_SHUTDOWN,"tGizmo::Shutdown");
    return OK;
  }

  /****
  [[U ecTemp := tGizmo::Shutdown (void) ]]
  ****/
  ecTemp = tGizmo::Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::Shutdown");

  
  /****
  [[U ecTemp := this->Servo.tServo::Shutdown (void) ]]
  ****/
  ecTemp = this->Servo.Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tRealtimeGizmo::Shutdown");
 
  TraceMark (S_realtime_gizmo_SHUTDOWN_COMPLETE);

  return OK;

}
  
/****
[[U tRealtimeGizmo::IsOk (void) ]]
****/
BOOL tRealtimeGizmo::IsOk () {
  
  /****
  [[U tGizmo::IsOk (void)]]

  [[U this->Servo.tBaseObject::IsOk (void) ]]
  ****/
  return tGizmo::IsOk () && this->Servo.IsOk ();
   
}

