/* $RCSfile: stGizmoInfo.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/st/stGizmoInfo.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  28Aug97,rlj    written
*/

#include "stGizmoInfo.h"
#include "stError.h"
#include "stTypes.h"
#include "instError.h"
#include "image.h"
#include "hardware.h"
#include "configMgr.h"

/******************************************************************
*
* tStGizmoInfo::tStGizmoInfo -- tStGizmoInfo constructor
*
*  Calls tRealtimeGizmoInfo(base class) constructor, sets up IPC
*  IDs for the indirection double buffer, inits image size
*
* SPEC:
      [[U tStGizmoInfo::tStGizmoInfo(iRteId, iGizmoId, 
                                     const tStPhysicalConstants& PhysicalConstants,
                                     const tStServoParams& ServoParams,
                                     iNumberOfServoModes,
                                     ServoInfo, iFtGizmoId,iWapGizmoId, 
                                     iImageHSize, iImageVSize,
                                     bnFsmBoardArray, iFsmChannelArray) ]]
* RETURNS:
*    nothing
*
****/                            

tStGizmoInfo::tStGizmoInfo(int iRteId, int iGizmoId, 
             const tStPhysicalConstants& PhysicalConstants,
             const tStServoParams& ServoParams,
             int iNumberOfServoModes,
             const tServoInfo& ServoInfo, 
             int iFtGizmoId,int iWapGizmoId, int iImageHSize, int iImageVSize,
             const BOARD_NAME bnFsmBoardArray[NUM_FSM_CHANNELS],
             const int iFsmChannelArray[NUM_FSM_CHANNELS]) :

  // First, call base class constructor
  /****  
     [[U tRealtimeGizmoInfo::tRealtimeGizmoInfo(iRteId,iGizmoId,
                                           iNumberOfServoModes,ServoInfo) ]]
   ****/                            
  tRealtimeGizmoInfo(iRteId, iGizmoId, iNumberOfServoModes, ServoInfo),
  
  PhysicalConstants_(PhysicalConstants),
  ServoParams_(ServoParams)

{
  int iActuator;

  TraceMark(S_stgizmo_ENTERING_ST_GIZMOINFO_CONSTRUCTOR);
   
   //get Fsm output channels
   for (iActuator = 0; iActuator < NUM_FSM_CHANNELS; iActuator++) {     
      pFsmChannelArray[iActuator] = D_CAST(tOutputChannel*)
         ConfigManager.Hardware.CreateChannel(bnFsmBoardArray[iActuator],
                                              iFsmChannelArray[iActuator]);

      if(!pFsmChannelArray[iActuator])
         TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
                  "tStGizmoInfo::tStGizmoInfo");
                  
      }
            
  // Set up IPC IDs for getting images from FT
  /****     
     [[U mtType := this->tGizmoInfo::TypeOfMemory(iRteId,iFtGizmoId)]]
   ****/                            
  MEM_TYPE mtType = this->TypeOfMemory(iRteId,iFtGizmoId);


  // Make three new IPC names (spec only for one)
  /****     
     [[U inStarIndBuffer := this->tGizmoInfo::NewIpcName()]]
   ****/                            
  IPC_NAME inStarIndBuffer = this->NewIpcName();
  IPC_NAME inStarBuffer1 = this->NewIpcName();
  IPC_NAME inStarBuffer2 = this->NewIpcName();

  size_t   szImageSize = iImageHSize * iImageVSize * sizeof (PIXEL);

  /****     
     [[U this->iiStarImageId.tIpcIndirectionDblBufferId::
             tIpcIndirectionDblBufferId_InInInMtSz(
                                       inStarIndBuffer, inStarBuffer1, 
                   inStarBuffer2, mtType, szImageSize ) ]]
   ****/                            
  this->iiStarImageId = tIpcIndirectionDblBufferId(inStarIndBuffer,
                           inStarBuffer1, inStarBuffer2, mtType, szImageSize);
  
  this->iStarImageHSize = iImageHSize;
  this->iStarImageVSize = iImageVSize;
  this->iFtGizmoId = iFtGizmoId;
  this->gtGizmoType = GT_STAR_TRACKER;

  TraceMark(S_stgizmo_LEAVING_ST_GIZMOINFO_CONSTRUCTOR);
}



/******************************************************************
*
* tStGizmoInfo::GetStarImageId -- star image ID accessor function
*
* SPEC:
*     [[U tStGizmoInfo::GetStarImageId(void) ]]
*
* RETURNS:
*    reference to tIpcIndirectionDblBufferId
*
****/                            
const tIpcIndirectionDblBufferId& tStGizmoInfo::GetStarImageId(void) const 
{
  return this->iiStarImageId;
}


int tStGizmoInfo::GetStarImageHSize () const 
{
 return this->iStarImageHSize;
}

int tStGizmoInfo::GetStarImageVSize () const 
{
 return this->iStarImageVSize;
}

