/* $RCSfile: stIoSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/st/stIoSet.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  28Aug97,rlj    written
*/

#include "isit.h"
#include "ipcid.h"
#include "stError.h"
#include "matrix.h"
#include "configMgr.h"
#include "ipcDblBufferAccessor.h"
#include "stGizmoInfo.h"
#include "st/stIoSet.h"
#include "st/stController.h"


/******************************************************************
*
* tStCompleteIoSet::tStCompleteIoSet 
*
*  Constructs a tStCompleteIoSet.  Currently it consists of only
*  one item: a tStarSensor object which receives images from
*       the Fringe Tracker Gizmo.
*
* SPEC:
      [[U tStCompleteIoSet::tStCompleteIoSet(GizmoInfo) ]]
*
* RETURNS:
*    nothing
*
****/

tStCompleteIoSet::tStCompleteIoSet(const tStGizmoInfo& GizmoInfo) :

  /****
     [[U this->StarSensor.tStarSensor::tStarSensor(GizmoInfo.iiStarImageId, 
                     GizmoInfo.iStarImageHSize, 
                     GizmoInfo.iStarImageVSize) ]]
  ****/
  StarSensor(GizmoInfo.GetStarImageId(), GizmoInfo.GetStarImageHSize(),
              GizmoInfo.GetStarImageVSize() ),

  //Construct the FSM actuator
  /****
     [[U this->Fsm.tMultiActuator::tMultiActuator(GizmoInfo.FsmChannels(),NUM_FSM_CHANNELS,
                                   GizmoInfo.PhysicalConstants.dActuatorScaleFactor) ]]
   ****/  
   Fsm(GizmoInfo.FsmChannels(),NUM_FSM_CHANNELS,
       GizmoInfo.PhysicalConstants().dActuatorScaleFactor)///,
   
   //Contruct the WAP double buffer, with buffer id from the WAP
   /****
     [[U iWapId := GizmoInfo.tStGizmoInfo::WapGizmoId() ]]
     [[U WapGizmoInfo := ConfigManager.tConfig::GetGizmoInfo_i(iWapId) ]]
     [[U WapBufferId := WapGizmoInfo.tWapGizmoInfo::GetStBufferId() ]]
     [[U Wap.tIpcDblBufferAccessor::tIpcDblBufferAccessor(WapBufferId) ]]
   ****/  
   ///Wap(ConfigManager.tConfig::GetGizmoInfo(GizmoInfo.WapGizmoId()).GetStBufferId())
   
{
   TraceMark(S_stgizmo_ST_COMPLETE_IO_SET_CONSTRUCTOR);
}
  
 
/******************************************************************
*
* tStCompleteIoSet::~tStCompleteIoSet 
*
*  Destroys a tStCompleteIoSet.  
*
* SPEC:
      [[U tStCompleteIoSet::~tStCompleteIoSet(void) ]]
*
* RETURNS:
*    nothing
*
****/

tStCompleteIoSet::~tStCompleteIoSet(void) 
{
   TraceMark(S_stgizmo_ST_COMPLETE_IO_SET_DESTRUCTOR);
}

/******************************************************************
*
* tStCompleteIoSet::GetStarImage 
*
*  tStarSensor image accessor function
*
* SPEC:
      [[U tStCompleteIoSet::GetStarImage(void) ]]
*
* RETURNS:
*    reference to a tImage object
*
****/

tImage& tStCompleteIoSet::GetStarImage(void) 
{
  /****
     [[U this->StarSensor.tStarSensor::GetStarImage(void) ]]
  ****/
  return this->StarSensor.GetStarImage();
}


/******************************************************************
*
* tStCompleteIoSet::SetXformParams 
*
*  Sets actuator transformation  
*
* SPEC:
      [[U tStCompleteIoSet::SetXformParams(const tMatrix& Xform,
         const tVector& Home) ]]
*
* RETURNS:
*    nothing
*
****/

void tStCompleteIoSet::SetXformParams(const tMatrix& Xform,const tVector& Home)
{
   
  /****
     [[U this->Fsm.tMultiActuator::SetXform(Xform) ]]
     [[U this->Fsm.tMultiActuator::SetHome(Home) ]]
  ****/
   this->Fsm.SetXform(Xform);   
   this->Fsm.SetHome(Home);   
}


/******************************************************************
*
* tStCompleteIoSet::SetCalParams 
*
*  Sets calibration parameters 
*
* SPEC:
      [[U tStCompleteIoSet::SetCalParams(const tStCalibrateCmd& Params) ]]
*
* RETURNS:
*    nothing
*
****/

void tStCompleteIoSet::SetCalParams(const tStCalibrateCmd& Params)
{
   /****
   [[U ModulationParams.tModulationParam::tModulationParams(MT_TRIANGLE,
                                      Params.CalFrequency,Params.CalAmplitude) ]]
   ****/
   tModulationParams ModulationParams(MT_TRIANGLE,
                                      Params.CalFrequency,Params.CalAmplitude);
      
   /****
   [[U this->Fsm.tMultiActuator::SetActuatorModulation(Params.CalActuator,
                                                       ModulationParams) ]]
   ****/
   Fsm.SetActuatorModulation(Params.CalActuator,ModulationParams);   
}
   
