/* $RCSfile: starSensor.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/st/starSensor.cc,v 1.1.1.1 1998/06/11 21:36:24 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  04Sep97,rlj    written
*/

#include "st/starSensor.h"
#include "ipcid.h"
#include "stError.h"

/******************************************************************
*
* tStarSensor::tStarSensor 
*
*  Constructs a tStarSensor.
*
* SPEC:
     [[U tStarSensor::tStarSensor(iiId,iHSize,iVSize) ]]
*
* RETURNS:
*    nothing
*
****/

tStarSensor::tStarSensor(const tIpcIndirectionDblBufferId& iiId,
			  int iHSize, int iVSize) :

  // First call base class constructor
  /****
    [[U tInputSource::tInputSource(void) ]]
  ****/
  tInputSource(),


  // Construct the IPC double buffer accessor 
  /****
    [[U this->StarImageBuffer.tIpcIndirectionDblBufferAccessor::
                                  tIpcIndirectionDblBufferAccessor(iiId) ]]
  ****/
  StarImageBuffer(iiId),


  // Construct the tImage object (used to cache image)
  /****
    [[U this->StarImage.tImage::tImage_ii(iHSize,iVSize) ]]
  ****/
  StarImage(iHSize,iVSize) 

{
   TraceMark(S_stgizmo_STAR_SENSOR_CONSTRUCTOR);
}
  

/******************************************************************
*
* tStarSensor::GetStarImage 
*
*  Returns reference to cached image.
*
* SPEC:
     [[U tStarSensor::GetStarImage(void) ]]
*
* RETURNS:
*    reference to cached image
*
****/

tImage& tStarSensor::GetStarImage(void) 
{
  return this->StarImage;
}


/******************************************************************
*
* tStarSensor::Update 
*
*  Gets an image from the Fringe Tracker via IPC.  If the Fringe
*      Tracker has not sent a new image since the last time the
*      tStarSensor read one, the tStarSensor reads the same one
*      again.  Update() returns a double to conform to the overloaded
*      tSensor::Update() function definition.
*
* SPEC:
     [[U tStarSensor::Update(tUpdateTime) ]]
*
* RETURNS:
*    double (to match overloaded tSensor::Update fn)
*
****/

double tStarSensor::Update(const TIME tUpdateTime) 
{
  ERROR_CODE ec;

   TraceMark(S_stgizmo_STAR_SENSOR_UPDATE);

  /****
    [[U ec := this->StarImageBuffer.
		 tIpcIndirectionDblBufferAccessor::Get(this->StarImage) ]]
  ****/
  ec = this->StarImageBuffer.Get(this->StarImage);
  
  if((ec != OK) && (ec != S_ipc_BUFFER_EMPTY))
  {
     TraceMsg(tmt_ERROR,ec,"tStarSensor::Update");
     return -1.0;
  }

  return 0.0;
}
  
