/* $RCSfile: wapControllerSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/wap/wapControllerSet.cc,v 1.1.1.1 1998/06/11 21:36:25 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */

#include "wap/wapControllerSet.h"
#include "wapError.h"

const int CiWapDataDumpPriority = 78;
const int CiWapDataBufferSize = 100;     // #elements in data bag


/*******************************************************************
*
* tWapControllerData::tWapControllerData 
*
*  Constructs tWapControllerData object.
*
* SPEC:
     [[U tWapControllerData::tWapControllerData(
                       const tTypeTag& StarPosDataTag, 
                       const tTypeTag& CollectorPosDataTag) ]]
*
* RETURNS:
*    nothing
*
****/

tWapControllerData::tWapControllerData(const tTypeTag& StarPosDataTag, 
                                       const tTypeTag& CollectorPosDataTag) :
  /**** 
    [[U this->StarPositionData.tDataStore::tDataStore(CiWapDataDumpPriority,
              CiWapDataBufferSize,StarPosDataTag) ]]
  *****/
  StarPositionData(CiWapDataDumpPriority,CiWapDataBufferSize,StarPosDataTag),

  /**** 
    [[U this->FsmPositionData.tDataStore::tDataStore(CiWapDataDumpPriority,
              CiWapDataBufferSize,FsmPosDataTag) ]]
  *****/
  CollectorPositionData(CiWapDataDumpPriority,CiWapDataBufferSize,
                        CollectorPosDataTag)

{
   TraceMark(S_wapgizmo_WAP_CONTROLLER_DATA_CONSTRUCTOR);
}

  
/*******************************************************************
*
* tWapCompleteControllerSet::tWapCompleteControllerSet
*
*  Constructs tWapCompleteControllerSet.  This includes constructing
*     the controller data block, the OFF and TRACK controllers, and
*     storing a reference to the complete I/O set.
*
* SPEC:
*    [[U tWapCompleteControllerSet::tWapCompleteControllerSet(CompleteIoSetToUse,
                                                         WapStatusDataTag, 
*                                                        StarPosDataTag,
*                                                        CollectorPosDataTag,
*                                                        tServoPeriod) ]]
*
* RETURNS:
*    nothing
*
****/

tWapCompleteControllerSet::tWapCompleteControllerSet(
                                         tWapCompleteIoSet& CompleteIoSetToUse,
                                         const tTypeTag&   WapStatusDataTag, 
                                         const tTypeTag&   StarPosDataTag, 
                                         const tTypeTag&   CollectorPosDataTag, 
                                         TIME tServoPeriod) :

  /****
    [[U this->ControllerData.tWapControllerData::tWapControllerData(
           StarPosDataTag,CollectorPosDataTag) ]]
  ****/
  ControllerData(StarPosDataTag,CollectorPosDataTag),

  /****
    [[U this->OffController.tWapController::tWapController(
    CompleteIoSetToUse,ControllerData) ]]
  ****/
  OffController(CompleteIoSetToUse,ControllerData),

  /****
    [[U this->tWapSlewController.tWapSlewController::tWapSlewController(
    CompleteIoSetToUse,ControllerData) ]]
  ****/
  SlewController(CompleteIoSetToUse,ControllerData),

  /****
    [[U this->TrackController.tWapTrackController::tWapTrackController(
                 CompleteIoSetToUse,ControllerData,tServoPeriod) ]]
  ****/
  TrackController(CompleteIoSetToUse,ControllerData,tServoPeriod),

  /****
    [[U this->CalibrateController.tWapCalibrateController::tWapCalibrateController(
                 CompleteIoSetToUse,ControllerData) ]]
  ****/
  CalibrateController(CompleteIoSetToUse,ControllerData),

  /****
  [[U this->WapOffControllerGroup.tSimpleControllerGroup::tSimpleControllerGroup (1) ]]
  [[U this->WapTrackControllerGroup.tSimpleControllerGroup::tSimpleControllerGroup (1) ]]
  [[U this->WapCalibrateControllerGroup.tSimpleControllerGroup::tSimpleControllerGroup (1) ]]
  ****/
  WapOffControllerGroup (1),
  WapSlewControllerGroup (1),
  WapTrackControllerGroup (1),
  WapCalibrateControllerGroup (1)
  
{
   ERROR_CODE ec;

    TraceMark(S_wapgizmo_WAP_COMPLETE_CONTROLLER_SET_CONSTRUCTOR);
    TraceMark(S_wapgizmo_ASSIGNING_CTLRS_TO_PARTITIONS);


    // Assign Controllers to Servo Partitions in Controller Group
    /****
       [[U ec := this->WapOffControllerGroup.tSimpleControllerGroup::
                 AssignController(0,OffController) ]]
     ****/ 
     ec = this->WapOffControllerGroup.AssignController(0,OffController);

    if (ec != OK) {
      TraceMsg(tmt_ERROR,ec,"tWapGizmo::tWapGizmo" );
      return;
    }

    /****
       [[U ec := this->WapSlewControllerGroup.tSimpleControllerGroup::
                 AssignController(0,SlewController) ]]
     ****/ 
     ec = this->WapOffControllerGroup.AssignController(0,OffController);

    if (ec != OK) {
      TraceMsg(tmt_ERROR,ec,"tWapGizmo::tWapGizmo" );
      return;
    }

    /****
       [[U ec := this->WapTrackControllerGroup.tSimpleControllerGroup::
                 AssignController(0,TrackController) ]]
     ****/ 
    ec = this->WapTrackControllerGroup.AssignController(0,TrackController);
 
    if (ec != OK) {
      TraceMsg(tmt_ERROR,ec,"tWapGizmo::tWapGizmo" );
      return;
    }
}


/*******************************************************************
*
* tWapCompleteControllerSet::~tWapCompleteControllerSet
*
*  Destroys the tWapCompleteControllerSet.
*
* SPEC:
    [[U tWapCompleteControllerSet::~tWapCompleteControllerSet(void) ]]
*
* RETURNS:
*    nothing
*
****/

tWapCompleteControllerSet::~tWapCompleteControllerSet(void) 
{
   TraceMark(S_wapgizmo_WAP_COMPLETE_CONTROLLER_SET_DESTRUCTOR);
}
