/* $RCSfile: wapGizmoInfo.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/wap/wapGizmoInfo.cc,v 1.1.1.1 1998/06/11 21:36:25 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */

#include "wapGizmoInfo.h"
#include "wapError.h"
#include "wapTypes.h"
#include "instError.h"
#include "image.h"
#include "hardware.h"
#include "configMgr.h"

/******************************************************************
*
* tWapGizmoInfo::tWapGizmoInfo -- tWapGizmoInfo constructor
*
*  Calls tRealtimeGizmoInfo(base class) constructor, sets up IPC
*  IDs for the indirection double buffer, inits image size
*
* SPEC:
      [[U tWapGizmoInfo::tWapGizmoInfo(int iRteId, int iGizmoId, 
      const tWapPhysicalConstants PhysicalConstants,
      const tWapServoParams& ServoParams,
      int iNumberOfServoModes, const tServoInfo& ServoInfo, 
      int iFtGizmoId,int iWapGizmoId, int iImageHSize, int iImageVSize,
      BOARD_NAME bnInputBoardArray[COLLECTOR_CHANNELS],
      int iInputChannelArray[COLLECTOR_CHANNELS],
      BOARD_NAME bnOutputBoardArray[COLLECTOR_CHANNELS],
      int iOutputChannelArray[COLLECTOR_CHANNELS],
      BOARD_NAME bnPaddleBoardArray[PADDLE_CHANNELS],
      int iPaddleChannelArray[PADDLE_CHANNELS]) ]]
      BOARD_NAME bnPaddleOutputBoard,
      int iPaddleOutputChannel) ]]
* RETURNS:
*    nothing
*
****/                            

tWapGizmoInfo::tWapGizmoInfo(int iRteId, int iGizmoId,
        const tWapPhysicalConstants& PhysicalConstants, 
        const tWapServoParams& ServoParams, int iNumberOfServoModes,
        const tServoInfo& ServoInfo, int iFtGizmoId,int iStGizmoId, 
        int iImageHSize, int iImageVSize,
        BOARD_NAME bnInputBoardArray[COLLECTOR_CHANNELS],
        int iInputChannelArray[COLLECTOR_CHANNELS],
        BOARD_NAME bnOutputBoardArray[COLLECTOR_CHANNELS],
        int iOutputChannelArray[COLLECTOR_CHANNELS],
        BOARD_NAME bnPaddleBoardArray[PADDLE_CHANNELS],
        int iPaddleChannelArray[PADDLE_CHANNELS],
        BOARD_NAME bnPaddleOutputBoard,
        int iPaddleOutputChannel) :

  // First, call base class constructor
  /****  
     [[U tRealtimeGizmoInfo::tRealtimeGizmoInfo(iRteId,iGizmoId,
                                           iNumberOfServoModes,ServoInfo) ]]
   ****/                            
  tRealtimeGizmoInfo(iRteId, iGizmoId, iNumberOfServoModes, ServoInfo),
  
   //get id for star tracker buffer
   /****  
   [[U mtMemType := this->tGizmoInfo::TypeOfMemory(iRteId, CiInst) ]]
   [[U inName := tGizmoInfo::NewIpcName(void) ]]
   [[U this->StBufferId.tIpcDblBufferId::tIpcDblBufferId_INMT(inName,mtMemType)]]
   ****/                            
   StBufferId(this->NewIpcName(),this->TypeOfMemory (iRteId, CiInst)),

  PhysicalConstants_(PhysicalConstants),
  ServoParams_(ServoParams)

{
  int iActuator;

  TraceMark(S_wapgizmo_ENTERING_WAP_GIZMOINFO_CONSTRUCTOR);
   
   //get Collector output channels
   for (iActuator = 0; iActuator < COLLECTOR_CHANNELS; iActuator++) {     
      pOutputChannelArray[iActuator] = D_CAST(tOutputChannel*)
         ConfigManager.Hardware.CreateChannel(bnOutputBoardArray[iActuator],
                                              iOutputChannelArray[iActuator]);

      if(!pOutputChannelArray[iActuator])
         TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
                  "tWapGizmoInfo::tWapGizmoInfo");
   }
            
   //get Collector input channels
   for (iActuator = 0; iActuator < COLLECTOR_CHANNELS; iActuator++) {     
      pInputChannelArray[iActuator] = D_CAST(tInputChannel*)
         ConfigManager.Hardware.CreateChannel(bnInputBoardArray[iActuator],
                                              iInputChannelArray[iActuator]);

      if(!pOutputChannelArray[iActuator])
         TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
                  "tWapGizmoInfo::tWapGizmoInfo");
   }
            
   //get Paddle input channels
   for (iActuator = 0; iActuator < COLLECTOR_CHANNELS; iActuator++) {     
      pPaddleChannelArray[iActuator] = D_CAST(tInputChannel*)
         ConfigManager.Hardware.CreateChannel(bnPaddleBoardArray[iActuator],
                                              iPaddleChannelArray[iActuator]);

      if(!pPaddleChannelArray[iActuator])
         TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
                  "tWapGizmoInfo::tWapGizmoInfo");
   }
            
   //get Paddle output channel
   pPaddleOutputChannel = D_CAST(tOutputChannel*)
      ConfigManager.Hardware.CreateChannel(bnPaddleOutputBoard,
                                             iPaddleOutputChannel);

   if(!pPaddleOutputChannel)
      TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
               "tWapGizmoInfo::tWapGizmoInfo");
            
            
  // Set up IPC IDs for getting images from FT
  /****     
     [[U mtType := this->tGizmoInfo::TypeOfMemory(iRteId,iFtGizmoId)]]
   ****/                            
  MEM_TYPE mtType = this->TypeOfMemory(iRteId,iFtGizmoId);


  // Make three new IPC names (spec only for one)
  /****     
     [[U inStarIndBuffer := this->tGizmoInfo::NewIpcName()]]
   ****/                            
  IPC_NAME inStarIndBuffer = this->NewIpcName();
  IPC_NAME inStarBuffer1 = this->NewIpcName();
  IPC_NAME inStarBuffer2 = this->NewIpcName();

  size_t   szImageSize = iImageHSize * iImageVSize * sizeof (PIXEL);

  /****     
     [[U this->iiStarImageId.tIpcIndirectionDblBufferId::
             tIpcIndirectionDblBufferId_InInInMtSz(
                                       inStarIndBuffer, inStarBuffer1, 
                   inStarBuffer2, mtType, szImageSize ) ]]
   ****/                            
  this->iiStarImageId = tIpcIndirectionDblBufferId(inStarIndBuffer,
                           inStarBuffer1, inStarBuffer2, mtType, szImageSize);
  
  this->iStarImageHSize = iImageHSize;
  this->iStarImageVSize = iImageVSize;
  this->iFtGizmoId = iFtGizmoId;
  this->gtGizmoType = GT_STAR_TRACKER;

  TraceMark(S_wapgizmo_LEAVING_WAP_GIZMOINFO_CONSTRUCTOR);
}



/******************************************************************
*
* tWapGizmoInfo::GetStarImageId -- star image ID accessor function
*
* SPEC:
*     [[U tWapGizmoInfo::GetStarImageId(void) ]]
*
* RETURNS:
*    reference to tIpcIndirectionDblBufferId
*
****/                            
const tIpcIndirectionDblBufferId& tWapGizmoInfo::GetStarImageId(void) const 
{
  return this->iiStarImageId;
}


int tWapGizmoInfo::GetStarImageHSize () const 
{
 return this->iStarImageHSize;
}

int tWapGizmoInfo::GetStarImageVSize () const 
{
 return this->iStarImageVSize;
}

