/* $RCSfile: wapIoSet.cc,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gizmo/wap/wapIoSet.cc,v 1.1.1.1 1998/06/11 21:36:25 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */

#include <float.h>

#include "isit.h"
#include "ipcid.h"
#include "stError.h"
#include "matrix.h"
#include "configMgr.h"
#include "ipcDblBufferAccessor.h"
#include "wapGizmoInfo.h"
#include "wap/wapIoSet.h"
#include "wap/wapController.h"
#include "wapError.h"
#include "paddle.h"

/******************************************************************
*
* tWapCompleteIoSet::tWapCompleteIoSet 
*
*  Constructs a tWapCompleteIoSet.
*
* SPEC:
      [[U tWapCompleteIoSet::tWapCompleteIoSet(GizmoInfo) ]]
*
* RETURNS:
*    nothing
*
****/

tWapCompleteIoSet::tWapCompleteIoSet(const tWapGizmoInfo& GizmoInfo) :

  /****
     [[U this->StarSensor.tStarSensor::tStarSensor(GizmoInfo.iiStarImageId, 
                     GizmoInfo.iStarImageHSize, 
                     GizmoInfo.iStarImageVSize) ]]
  ****/
  AcquisitionSensor(GizmoInfo.GetStarImageId(), GizmoInfo.GetStarImageHSize(),
                    GizmoInfo.GetStarImageVSize() ),

  //Construct the paddle (compiler bug: cast should not be necessary)
  /****
  [[U this->Paddle.tPaddle::tPaddle(GizmoInfo.PaddleChannels(),
                                    GizmoInfo.PaddleOutputChannel(),
                                    GizmoInfo.PhysicalConstants().dPaddleScaleFactor,
                                    GizmoInfo.PhysicalConstants().dPaddleSpeeds) ]]
  ****/  
  Paddle(GizmoInfo.PaddleChannels(),
         GizmoInfo.PaddleOutputChannel(),
         GizmoInfo.PhysicalConstants().dPaddleScaleFactor,
         (double*)GizmoInfo.PhysicalConstants().dPaddleSpeeds),

   //Contruct the ST double buffer
   /****
   [[U StBufferId := this->GizmoInfo.tWapGizmoInfo::GetStBufferId() ]]
   [[U this->Fsm.tIpcDblBufferAccessor::tIpcDblBufferAccessor(StBufferId) ]]
   ****/  
   Fsm(GizmoInfo.GetStBufferId()),

  //Construct the Collector actuator
  /****
  [[U this->Collector.tStepperMultiActuator::tStepperMultiActuator(
                     COLLECTOR_CHANNELS,
                     GizmoInfo.InputChannels(),
                     GizmoInfo.PhysicalConstants().dInputScaleFactor,
                     GizmoInfo.OutputChannels(),
                     GizmoInfo.PhysicalConstants().dOutputScaleFactor) ]]
   ****/  
   Collector(COLLECTOR_CHANNELS,
            GizmoInfo.InputChannels(),
            GizmoInfo.PhysicalConstants().dInputScaleFactor,
            GizmoInfo.OutputChannels(),
            GizmoInfo.PhysicalConstants().dOutputScaleFactor)         
  
{
   TraceMark(S_stgizmo_ST_COMPLETE_IO_SET_CONSTRUCTOR);
}
  
 
/******************************************************************
*
* tWapCompleteIoSet::~tWapCompleteIoSet 
*
*  Destroys a tWapCompleteIoSet.  
*
* SPEC:
      [[U tWapCompleteIoSet::~tWapCompleteIoSet(void) ]]
*
* RETURNS:
*    nothing
*
****/

tWapCompleteIoSet::~tWapCompleteIoSet(void) 
{
   TraceMark(S_wapgizmo_WAP_COMPLETE_IO_SET_DESTRUCTOR);
}

/******************************************************************
*
* tWapCompleteIoSet::GetStarImage 
*
*  tStarSensor image accessor function
*
* SPEC:
      [[U tWapCompleteIoSet::GetStarImage(void) ]]
*
* RETURNS:
*    reference to a tImage object
*
****/

tImage& tWapCompleteIoSet::GetStarImage(void) 
{
  /****
     [[U this->AcquisitionSensor.tStarSensor::GetStarImage(void) ]]
  ****/
  return this->AcquisitionSensor.GetStarImage();
}


/******************************************************************
*
* tWapCompleteIoSet::SetXformParams 
*
*  Sets actuator transformation  
*
* SPEC:
      [[U tWapCompleteIoSet::SetXformParams(const tMatrix& Xform,
         const tVector& Home) ]]
*
* RETURNS:
*    nothing
*
****/

void tWapCompleteIoSet::SetXformParams(const tMatrix& Xform,const tVector& Home)
{
   
  /****
     [[U this->Collector.tMultiActuator::SetXform(Xform) ]]
     [[U this->Collector.tStepperMultiActuator::SetHome(Home) ]]
  ****/
   this->Collector.SetXform(Xform);   
   this->Collector.SetHome(Home);   
}


/******************************************************************
*
* tWapCompleteIoSet::SetCalParams 
*
*  Sets calibration parameters 
*
* SPEC:
      [[U tWapCompleteIoSet::SetCalParams(const tWapCalibrateCmd& Params) ]]
*
* RETURNS:
*    nothing
*
****/

void tWapCompleteIoSet::SetCalParams(const tWapCalibrateCmd& icNewCmd)
{
   tModulationParams VelocityParams;
   
   if (icNewCmd.dCalFrequency < DBL_EPSILON) return;

   //velocity modulation is derivative of triangle wave
   VelocityParams.mtModulationType = MT_SQUARE;
   VelocityParams.dAmplitude = 
      4 * icNewCmd.dCalFrequency * icNewCmd.dCalAmplitude; 
   VelocityParams.dFrequency = icNewCmd.dCalFrequency;

   //start modulation in middle of range
   VelocityParams.tSyncTime = icNewCmd.tTimeStamp + 
      (TIME)(TIME_TICKS_PER_SECOND / (4 * icNewCmd.dCalFrequency));
   
   /****
   [[U this->Collector.tMultiActuator::
      SetActuatorModulation(icNewCmd.iCalActuator,VelocityParams) ]]
   ****/
   Collector.SetActuatorModulation(icNewCmd.iCalActuator,VelocityParams);   
}
   
