/* $RCSfile: gsuDlControllerSet.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gsu/gsuDl/gsuDlControllerSet.cc,v 1.4 1998/09/23 23:57:10 phil Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,14May97    written
*/
 
/*
DESCRIPTION

*/


#include "isit.h"
#include "gsu/gsuDlTypes.h"
#include "gsu/gsuDlGizmoInfo.h"
#include "gsu/gsuDlControllerSet.h"
#include "gsu/gsuDlIoSet.h"


const int CiGsuDlUpdateOffModePriority = 30;
const TIME CtGsuDlUpdateOffModePeriod = 400;
const TIME CtGsuDlUpdateOffModeOffset = 0;
const int CiGsuDlUpdateTrackModePriority = 30;
const TIME CtGsuDlUpdateTrackModePeriod = 400;
const TIME CtGsuDlUpdateTrackModeOffset = 0;




/*******************************************************************
*
* tGsuDlCompleteControllerSet::tGsuDlCompleteControllerSet
*
* SPECIFICATION:
[[U5 tGsuDlCompleteControllerSet::tGsuDlCompleteControllerSet (
    GizmoInfo,CompleteIoSet)]]
*
*/

tGsuDlCompleteControllerSet::tGsuDlCompleteControllerSet (
    const tGsuDlGizmoInfo& GizmoInfo, tGsuDlCompleteIoSet& IoSet,
    const tTypeTag& DlDataTag, tGsuDlStatus& DlStatus) :

  /****
  [[U ControllerData_.tGsuDlControllerData::tGsuDlControllerData () ]]
  ****/
  ControllerData_ (DlDataTag, DlStatus, IoSet.Laser.CountsToPhysicalUnits (1) / GizmoInfo.PhysConst.iLaserNumPasses ),
  
  /****
  [ this->ServoParameters_ := GizmoInfo.ServoParameters_ ]
  ****/
  ServoParameters_ (GizmoInfo.ServoParameters),
  

  /****
  [[U this->OffGroupUpdate.tGsuDlOffGroupUpdate::
        tGsuDlOffGroupUpdate (GizmoInfo.Thresholds_, ControllerData__) ]]
  ****/
  OffGroupUpdate (GizmoInfo.Thresholds (), ControllerData_),
 
  /****
  [[U this->TrackGroupUpdate.tGsuDlTrackGroupUpdate::
        tGsuDlTrackGroupUpdate (GizmoInfo.Thresholds_, ControllerData__) ]]
  ****/
  TrackGroupUpdate (GizmoInfo.Thresholds (),ControllerData_),
  
  /****
  [[U this->OffControllerGroup_.tModedControllerGroup::tModedControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions, DOCM_NUM_OFF_MODES,
        OffGroupUpdate, CiGsuDlUpdateOffModePriority,CtGsuDlUpdateOffModePeriod,
        CtGsuDlUpdateOffModeOffset) ]]
  ****/  
  OffControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions (),
                      DOCM_NUM_OFF_MODES, this->OffGroupUpdate,
                      CiGsuDlUpdateOffModePriority, CtGsuDlUpdateOffModePeriod,
                      CtGsuDlUpdateOffModeOffset),
  
  /****
  [[U this->FrontControllerGroup_.tSimpleControllerGroup::tSimpleControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions) ]]
  ****/
  FrontControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions () ), 

  /****
  [[U this->BackControllerGroup_.tSimpleControllerGroup::tSimpleControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions) ]]
  ****/
  BackControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions () ),

  /****
  [[U this->FrontCreepControllerGroup_.tSimpleControllerGroup::tSimpleControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions) ]]
  ****/
  FrontCreepControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions () ),

  /****
  [[U this->BackCreepControllerGroup_.tSimpleControllerGroup::tSimpleControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions) ]]
  ****/
  BackCreepControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions () ),

  /****
  [[U this->TrackControllerGroup_.tModedControllerGroup::tModedControllerGroup (
        GizmoInfo.ServoInfo.iNumberOfPartions, DTCM_NUM_TRACKING_MODES,
        this->TrackGroupUpdate, CiGsuDlUpdateTrackModePriority,
        CtGsuDlUpdateTrackModePeriod, CtGsuDlUpdateTrackModeOffset) ]]
  ****/  
  TrackControllerGroup_ (GizmoInfo.GetNumberOfServoPartitions (),
                        DTM_NUM_TRACKING_MODES,
                        TrackGroupUpdate ,
                        CiGsuDlUpdateTrackModePriority,
                        CtGsuDlUpdateTrackModePeriod,
                        CtGsuDlUpdateTrackModeOffset),


  /****
  [[U this->PztIdle.tDlPztController::tDlPztController (ControllerData__, IoSet) ]]
  ****/
  PztIdle (ControllerData_, IoSet),
  
  /****
  [[U this->VcIdle.tGsuDlVcStabilizeController::tGsuDlVcStabilizeController (
      ControllerData_, IoSet, GizmoInfo.VcIdleParameters_) ]]
  ****/
  VcIdle (ControllerData_, IoSet, GizmoInfo.VcIdleParameters ()),
        
  /****
  [[U this->VcStablize.tGsuDlVcStabilizeController::tGsuDlVcStabilizeController (
      ControllerData__, IoSet, GizmoInfoVcStablilzeParameters_) ]]
  ****/
  VcStabilize (ControllerData_, IoSet, GizmoInfo.VcStabilizeParameters ()),


  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorDecelerate.tDlMotorSlewController::tDlMotorSlewController (
      ControllerData_, IoSet, GizmoInfo.MotorIdleParameters_,
       dPhysicalUnits, GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorDecelerate (ControllerData_, IoSet, GizmoInfo.MotorIdleParameters (), IoSet.Laser.CountsToPhysicalUnits (1),
             GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorFront.tDlMotorSlewController::tDlMotorSlewController (
      ControllerData_, IoSet, GizmoInfo.MotorFrontParameters_,
       dPhysicalUnits, GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorFront (ControllerData_, IoSet, GizmoInfo.MotorFrontParameters (), IoSet.Laser.CountsToPhysicalUnits (1),
             GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorBack.tDlMotorSlewController::tDlMotorSlewController (
      ControllerData_, IoSet, GizmoInfo.MotorBackParameters_,
       dPhysicalUnits, GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorBack (ControllerData_, IoSet, GizmoInfo.MotorBackParameters (), IoSet.Laser.CountsToPhysicalUnits (1),
             GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorFrontCreep.tDlMotorSlewController::tDlMotorSlewController (
      ControllerData_, IoSet, GizmoInfo.MotorFrontCreepParameters_,
       dPhysicalUnits,  GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorFrontCreep (ControllerData_, IoSet, GizmoInfo.MotorFrontCreepParameters (),
       IoSet.Laser.CountsToPhysicalUnits (1),  GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorBackCreep.tDlMotorSlewController::tDlMotorSlewController (
      IControllerData_, oSet, GizmoInfo.MotorBacktCreepParameters_,
       dPhysicalUnits, GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorBackCreep (ControllerData_, IoSet, GizmoInfo.MotorBackCreepParameters (),
       IoSet.Laser.CountsToPhysicalUnits (1),  GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorTrackSlew.tDlMotorTrackSlewController::tDlMotorTrackSlewController (
      ControllerData_, IoSet, GizmoInfo.MotorTrackSlewParameters_,
       dPhysicalUnits,  GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorTrackSlew (ControllerData_, IoSet, GizmoInfo.MotorTrackSlewParameters (),
       IoSet.Laser.CountsToPhysicalUnits (1),  GizmoInfo.PhysConst.iLaserNumPasses),

  /****
  [[U this->PztTrack.tDlPztTrackController::tDlPztTrackController (ControllerData__, 
        IoSet, GizmoInfo.PztTrackParameters_) ]]
  ****/
  PztTrack (ControllerData_, IoSet, GizmoInfo.PztTrackParameters ()),

  /****
  [[U this->VcTrack.tGsuDlVcTrackController::tGsuDlVcTrackController (
      ControllerData_, IoSet, GizmoInfoVcTrackParameters_) ]]
  ****/
  VcTrack (ControllerData_, IoSet, GizmoInfo.VcTrackParameters ()),
  
  /****
  [[U dPhyicalUnits := IoSet.Laser.tLaserMetrologySensor::CountsToPhysicalUnits (1) ]]
  
  [[U this->MotorTrack.tGsuDlMotorTrackController::tGsuDlMotorTrackController (
      ControllerData_, IoSet, GizmoInfo.MotorTrackParameters_, dPhyicalUnits, GizmoInfo.PhysConst.iLaserNumPasses) ]]
  ****/
  MotorTrack (ControllerData_, IoSet, GizmoInfo.MotorTrackParameters (),
       IoSet.Laser.CountsToPhysicalUnits (1),  GizmoInfo.PhysConst.iLaserNumPasses)
  

  
{
  // Assign Off Controllers...

  /****
  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_FAST, DOCM_IDLE, this->PztIdle) ]]

  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_FAST, DOCM_DECELERATE, this->PztIdle) ]]

  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_MEDIUM, DOCM_IDLE, this->VcIdle) ]]

  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_MEDIUM, DOCM_DECELERATE, this->VcStabilize) ]]

  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_SLOW, DOCM_IDLE, this->MotorIdle) ]]

  [[U this->OffControllerGroup_.tModedControllerGroup::AssignController (
        DSP_SLOW, DOCM_DECELERATE, this->MotorDecelerate) ]]
  ****/
  {
    this->OffControllerGroup_.AssignController (
        DSP_FAST, DOCM_IDLE, this->PztIdle);
  
    this->OffControllerGroup_.AssignController (
        DSP_FAST, DOCM_DECELERATE, this->PztIdle);

    this->OffControllerGroup_.AssignController (
        DSP_MEDIUM, DOCM_IDLE, this->VcIdle);
          
    this->OffControllerGroup_.AssignController (
        DSP_MEDIUM, DOCM_DECELERATE, this->VcStabilize);

    this->OffControllerGroup_.AssignController (
        DSP_SLOW, DOCM_IDLE, this->MotorDecelerate);
          
    this->OffControllerGroup_.AssignController (
        DSP_SLOW, DOCM_DECELERATE, this->MotorDecelerate);
  }     

  // Assign Front Controllers...

  /****
  [[U this->FrontControllerGroup_.tSimpleControllerGroup::AssignController (DSP_FAST,
      this->PztIdle) ]]

  [[U this->FrontControllerGroup_.tSimpleControllerGroup::AssignController (DSP_MEDIUM,
      this->VcStabilize) ]]

  [[U this->FrontControllerGroup_.tSimpleControllerGroup::AssignController (DSP_SLOW,
       this->MotorFront) ]]
  ****/
  {
    this->FrontControllerGroup_.AssignController (DSP_FAST,
      this->PztIdle);
    this->FrontControllerGroup_.AssignController (DSP_MEDIUM,
      this->VcStabilize);
    this->FrontControllerGroup_.AssignController (DSP_SLOW,
       this->MotorFront);
  }
  
  // Assign Back Controllers...

  /****
  [[U this->BackControllerGroup_.tSimpleControllerGroup::AssignController (DSP_FAST,
      this->PztIdle) ]]

  [[U this->BackControllerGroup_.tSimpleControllerGroup::AssignController (DSP_MEDIUM,
      this->VcStabilize) ]]

  [[U this->BackControllerGroup_.tSimpleControllerGroup::AssignController (DSP_SLOW,
       this->MotorBack) ]]
  ****/
  {  
    this->BackControllerGroup_.AssignController (DSP_FAST,
      this->PztIdle);
    this->BackControllerGroup_.AssignController (DSP_MEDIUM,
      this->VcStabilize);
    this->BackControllerGroup_.AssignController (DSP_SLOW,
       this->MotorBack);
  }
  
  // Assign Front Creep Controllers...
  
  /****
  [[U this->FrontCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_FAST,
      this->PztIdle) ]]

  [[U this->FrontCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_MEDIUM,
      this->VcStabilize) ]]

  [[U this->FrontCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_SLOW,
       this->MotorFrontCreep) ]]
  ****/
  {
    this->FrontCreepControllerGroup_.AssignController (DSP_FAST,
        this->PztIdle);
    this->FrontCreepControllerGroup_.AssignController (DSP_MEDIUM,
        this->VcStabilize);
    this->FrontCreepControllerGroup_.AssignController (DSP_SLOW,
        this->MotorFrontCreep);
  }
  
  // Assign Back Creep Controllers...

  /****
  [[U this->BackCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_FAST,
      this->PztIdle) ]]

  [[U this->BackCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_MEDIUM,
      this->VcStabilize) ]]

  [[U this->BackCreepControllerGroup_.tSimpleControllerGroup::AssignController (DSP_SLOW,
       this->MotorBackCreep) ]]
  ****/
  {
    this->BackCreepControllerGroup_.AssignController (DSP_FAST,        
        this->PztIdle);
    this->BackCreepControllerGroup_.AssignController (DSP_MEDIUM,
        this->VcStabilize);
    this->BackCreepControllerGroup_.AssignController (DSP_SLOW,
        this->MotorBackCreep);
  }
  
  // Assign Track Controllers...
  
  /****
  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_FAST, DTCM_TRACK, this->PztTrack) ]]

  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_FAST, DTCM_SLEW, this->PztIdle) ]]

  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_MEDIUM, DTCM_TRACK, this->VcTrack) ]]

  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_MEDIUM, DTM_SLEW, this->VcStabilize) ]]

  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_SLOW, DTCM_TRACK, this->MotorTrack) ]]

  [[U this->TrackControllerGroup_.tModedControllerGroup::AssignController (
      DSP_SLOW, DTCM_SLEW, this->MotorTrackSlew) ]]
  ****/
  {
    this->TrackControllerGroup_.AssignController (
        DSP_FAST, DTCM_TRACK, this->PztTrack);
    this->TrackControllerGroup_.AssignController (
        DSP_FAST, DTCM_SLEW, this->PztIdle);
    this->TrackControllerGroup_.AssignController (
        DSP_MEDIUM, DTCM_TRACK, this->VcTrack);
    this->TrackControllerGroup_.AssignController (
        DSP_MEDIUM, DTCM_SLEW, this->VcStabilize);
    this->TrackControllerGroup_.AssignController (
        DSP_SLOW, DTCM_TRACK, this->MotorTrack);
    this->TrackControllerGroup_.AssignController (
        DSP_SLOW, DTCM_SLEW, this->MotorTrackSlew);
  }

}


/*******************************************************************
*
* tGsuDlCompleteControllerSet::SetServoParameteres
*
* SPECIFICATION:
[[U tGsuDlCompleteControllerSet::SetServoParameters (NewServoParameters)]]
*/
ERROR_CODE tGsuDlCompleteControllerSet::SetServoParameters (
  const tGsuDlServoParameters& NewServoParameters)
{
  this->PztTrack.SetControlParameters (NewServoParameters.PztParameters () );
  this->VcStabilize.SetControlParameters (NewServoParameters.VcStabilizeParameters () );
  this->VcTrack.SetControlParameters (NewServoParameters.VcTrackParameters () );
  this->MotorFront.SetControlParameters (NewServoParameters.MotorFrontParameters () );
  this->MotorBack.SetControlParameters (NewServoParameters.MotorBackParameters () );
  this->MotorFrontCreep.SetControlParameters (NewServoParameters.MotorFrontCreepParameters () );
  this->MotorBackCreep.SetControlParameters (NewServoParameters.MotorBackCreepParameters () );
  this->MotorTrackSlew.SetControlParameters (NewServoParameters.MotorTrackSlewParameters () );
  this->MotorTrack.SetControlParameters (NewServoParameters.MotorTrackParameters () );
  
  this->OffGroupUpdate.SetThresholds (NewServoParameters.Thresholds () );
  this->TrackGroupUpdate.SetThresholds (NewServoParameters.Thresholds () );
  
  this->ServoParameters_ = NewServoParameters;
  return OK;

}


/*******************************************************************
*
* tGsuDlCompleteControllerSet::ServoParameters
* 
* [[U tGsuDlCompleteControllerSet::ServoParameters ()]]
*/
const tGsuDlServoParameters& tGsuDlCompleteControllerSet::ServoParameters() const
{
  return this->ServoParameters_;
}

/*******************************************************************
*
* tGsuDlCompleteControllerSet::ControllerData
*
* DESCRIPTION: Returns a reference to the controller data for this
* controller set
*
  [[U tGsuDlCompleteControllerSet::ControllerData()]]
*
*/

tGsuDlControllerData& tGsuDlCompleteControllerSet::ControllerData () 
{
  return this->ControllerData_;
}


