/* $RCSfile: gsuDlGizmo.cc,v $ - Type definitions for Delay Line gizmo stuff  */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gsu/gsuDl/gsuDlGizmo.cc,v 1.2 1998/09/23 23:57:11 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
$Log: gsuDlGizmo.cc,v $
Revision 1.2  1998/09/23 23:57:11  phil
Changes to data recording so FIFOs dont fill up

Revision 1.1.1.1  1998/06/11 21:36:39  chara
First import of RICST Code

Revision 1.6  1998/04/01 01:35:56  phil
include mods after moving quic and gsu header files around

Revision 1.5  1998/03/30 19:34:38  rich
*** empty log message ***

Revision 1.4  1998/03/12 17:58:53  rich
*** empty log message ***

*/
 

#include "isit.h"
#include "gsu/gsuDlGizmoInfo.h"
#include "gsu/gsuDlGizmo.h"
#include "dlTypes.h"
#include "dlDither.h"
#include "dlTargGen.h"


/******************************************************************************
*
* tGsuDlGizmo::tGsuDlGizmo
*
* SPECIFICATION:
[[U5 tGsuDlGizmo::tGsuDlGizmo (GizmoInfo) ]]
*****/
tGsuDlGizmo::tGsuDlGizmo(const tGsuDlGizmoInfo &GizmoInfo) :
  
  /****
  [[U tDlGizmo::tDlGizmo (GizmoInfo) ]]
  ****/
  tDlGizmo (GizmoInfo),

  /****
  [[U this->CGsuDlStatusTag.tTypeTag::tTypeTag_STTNisz (ST_STATUS,TN_GSU_DL_STATUS,
                GizmoInfo.iGizmoId, sizeof (this->Status) ) ]]
  ****/
  CGsuDlStatusTag (ST_STATUS,TN_GSU_DL_STATUS,
                GizmoInfo.GetGizmoId (), sizeof (this->Status) ),
                                        
  /****
  [[U this->CGsuDlDataTag.tTypeTag::tTypeTag_STTNisz (ST_STATUS,TN_GSU_DL_STATUS,
                GizmoInfo.iGizmoId, sizeof (this->Status) ) ]]
  ****/
  CGsuDlDataTag (ST_DATA,TN_DL_DATA,
                GizmoInfo.GetGizmoId (), sizeof(tDlData)),
                                           

  /****
  [[U this->Status.tDlGsuStatus::tDlGsuStatus (GizmoInfo.iGizmoId) ]]
  ****/
  Status (GizmoInfo.GetGizmoId () ),

  /****
  [[U this->CompleteIoSet.tGsuDlCompleteIoSet::tGsuDlCompleteIoSet (GizmoInfo) ]]
  ****/
  CompleteIoSet (GizmoInfo),
  
  /****
  [[U this->CompleteControllerSet.tGsuDlCompleteControllerSet::
          tGsuDlCompleteControllerSet (GizmoInfo, this->CompleteIoSet) ]]
  ****/
  CompleteControllerSet (GizmoInfo, CompleteIoSet, CGsuDlDataTag, Status)

{
  ERROR_CODE ec;

  /****
  [[U ec := tDlGizmo::SetOffGroup (this->CompleteControllerSet.OffControllerGroup_) ]]
  ****/
  ec = this->SetOffGroup (this->CompleteControllerSet.OffControllerGroup () );
 
  if (ec != OK) {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo Off Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetFrontGroup (this->CompleteControllerSet.FrontControllerGroup_) ]]
  ****/
  ec = this->SetFrontGroup (this->CompleteControllerSet.FrontControllerGroup () );
  if (ec != OK) {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo Front Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetBackGroup (this->CompleteControllerSet.BackControllerGroup_) ]]
  ****/
  ec = this->SetBackGroup (this->CompleteControllerSet.BackControllerGroup () );
  if (ec != OK) {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo Back Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetFrontCreepGroup (this->CompleteControllerSet.  
       FrontCreepControllerGroup_) ]]
  ****/
  ec = this->SetFrontCreepGroup (this->CompleteControllerSet.FrontCreepControllerGroup () );
  if (ec != OK)  {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo FrontCreep Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetBackCreepGroup (this->CompleteControllerSet.
       BackCreepControllerGroup_) ]]
  ****/
  ec = this->SetBackCreepGroup (this->CompleteControllerSet.BackCreepControllerGroup () );
  if (ec != OK)  {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo BackCreep Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetTrackGroup (this->CompleteControllerSet.TrackControllerGroup_) ]]
  ****/
  ec = this->SetTrackGroup (this->CompleteControllerSet.TrackControllerGroup () );
  if (ec != OK)  {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo Track Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetUser1Group (this->CompleteControllerSet.User1TrackControllerGroup_) ]]
  ****/
  ec = this->SetUser1Group (this->CompleteControllerSet.User1ControllerGroup () );
  if (ec != OK)  {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo User1 Group");
    return;
  }

  /****
  [[U ec := tDlGizmo::SetUser2Group (this->CompleteControllerSet.User2ControllerGroup_) ]]
  ****/
  ec = this->SetUser2Group (this->CompleteControllerSet.User2ControllerGroup () );
  if (ec != OK)  {
    this->SetInitError (ec);
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmoInfo User2 Group");
    return;
  }


  // GsuDl components contructed now start tasks that use them. 
  /****
  [[U ec :=tDlGizmo::StartTasks (GizmoInfo, this->CompleteIoSet) ]]
  ****/
  ec = tDlGizmo::StartTasks (GizmoInfo, this->CompleteIoSet);
  if (ec != OK) {
    TraceMsg (tmt_ERROR,ec,"tGsuDlGizmo::tGsuDlGizmo");
    this->SetInitError (ec);
    return;
   }      
  
  TraceMsg(tmt_INFO, S_gsuDlGizmo_GIZMO_INIT_SUCCEEDED,
           "tGsuDlGizmo::tGsuDlGizmo");
  
}   /* tGsuDlGizmo::tGsuDlGizmo(...) */



/******************************************************************************
*
* tGsuDlGizmo::Dispatch
*
* SPECIFICATION:
[[U5 tGsuDlGizmo::Dispatch (Command) ]]
****/
ERROR_CODE tGsuDlGizmo::Dispatch (const tIsitCmd& Command) {

  switch (Command.Command) {

    /****
    [[U  tGsuDlGizmo::SetServoParameters (Command) ]]
    ****/
    case GSUDL_CMD_CHNG_SERVO_PARAM : return this->SetServoParameters (
                                    (tGsuDlSetServoParametersCmd&) Command);

    default : return this->tDlGizmo::Dispatch (Command);
  };

  return OK;
} 


/******************************************************************************
*
* tGsuDlGizmo::CollectStatus
*
* SPECIFICATION:
[[U5 tGsuDlGizmo::CollectStatus () ]]
****/
ERROR_CODE tGsuDlGizmo::CollectStatus () 
{

  ERROR_CODE ec;
  EDDY_VALUE evEddy1, evEddy2;
  VC_VALUE   vvVc2;
 
  // Get common delay line status... (In future send as two chunks ???)
  
  /****
  [[U ec := this->tDlGizmo::CollectStatus (void) ]]
  ****/
  ec = this->tDlGizmo::CollectStatus ();


  // Get GSU status ....
  
  /****
  [[U tempParams := this->CompleteControllerSet.tGsuDlCompleteControllerSet::ServoParameters () ]]
  
  [[U Status.tGsuDlStatus::UpdateTrackServoParams (tempParams)  ]] 
  ****/
  Status.UpdateTrackServoParams (this->CompleteControllerSet.ServoParameters ().GuiParameters () );
  
  
  /****
  [[U evEddy1 := this->CompleteIoSet.Eddy1.tModulatedSensor::Value (void) ]]
  ****/
  evEddy1 = (EDDY_VALUE) this->CompleteIoSet.Eddy1.Value ();

  /****
  [[U evEddy2 := tthis->CompleteIoSet.Eddy2.tModulatedSensor::Value (void) ]]
  ****/
  evEddy2 = (EDDY_VALUE) this->CompleteIoSet.Eddy2.Value ();

  /****
  [[U vvVc2 := this->CompleteIoSet.Vc2.tModulatedAccelerationActuator::
        Value (void) ]]
  ****/
  vvVc2   = (VC_VALUE)   this->CompleteIoSet.Vc2.Value ();
   
  /****
  [[U this->Status.tGsuDlStatus::UpdateAdditionalGsuServoInfo (
        evEddy1, evEddy2, vvVc2) ]]
  ****/
  this->Status.UpdateAdditionalGsuServoInfo (evEddy1, evEddy2, vvVc2);
  
  // Send status to stream......

  /****
  [U[ ec := this->tGizmo::SendStatus (this->CGsuDlStatusTag,&this->Status) ]]
  ****/
  ec = this->SendStatus (this->CGsuDlStatusTag, (BYTE*) &this->Status);
  
  return ec;

}





/******************************************************************************
*
* tGsuDlGizmo::SetServoParameters
*
* SPECIFICATION:
[[U tGsuDlGizmo::SetServoParameters (Command) ]]
****/
  
ERROR_CODE tGsuDlGizmo::SetServoParameters (const tGsuDlSetServoParametersCmd& Command)

{
  ERROR_CODE ec;
  

  /****
  [[U this->CompleteControllerSet.tGsuCompleteControllerSet::
      SetServoParameters (Command.ServoParams) ]]
  ****/

  ec =  this->CompleteControllerSet.SetServoParameters (Command.ServoParams);

  if (ec == OK) {

    /****
    [[U Status := this->tGsuDlGizmo::GetStatus (void) ]]
     
    [[U Status.tGsuDlStatus::UpdateCommandStatus (
          DLCS_SET_SERVO_PARAMETERS_COMPLETE) ]]
    ****/
    this->GetStatus ().UpdateCommandStatus (
            DLCS_SET_SERVO_PARAMETERS_COMPLETE);


    /**** 
    [[U this->tGizmo::SendError_ec (
         S_setServoParametersCommand_SET_SERVO_PARAMETERS_COMPLETE) ]]
    ****/
    this->SendError (S_setServoParametersCommand_SET_SERVO_PARAMETERS_COMPLETE);
  }

  return ec;  
}


