/* $RCSfile: gsuDlGizmoInfo.cc,v $ - Type definitions for Delay Line gizmo stuff  */
/* CM Info:
$Header: /home/isit/cvsroot/isit/sys/dl/gsu/gsuDlGizmo.cc,v 1.2 1997/09/26 17:08
:23 rich Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */

/*
modification history
--------------------
02a,07May97,pci      changed Initialize to constructor
01a,29Oct96,eamck    written
*/

/*
DESCRIPTION
  DL Gizmo init code, etc.
*/


#include "isit.h"
#include "gsu/gsuDlGizmoInfo.h"

tGsuDlGizmoInfo::tGsuDlGizmoInfo(int iWhichRteToUse, int iGizmoIdToUse,
       const int iTargetSourceArray[NUM_DELAY_LINE_TARGETS],
       TIME tDitherPeriodToUse,
       DlPhysicalConstants PhysConstToUse, int iNumServoModesToUse,
       const tServoInfo& ServoInfoToUse,
       const tGsuDlServoParameters& ServoParametersToUse,
       BOARD_NAME bnVc1BoardToUse, int iVc1ChannelToUse,
       BOARD_NAME bnVc2BoardToUse, int iVc2ChannelToUse,
       BOARD_NAME bnServoPztBoardToUse, int iServoPztChannelToUse,
       BOARD_NAME bnMotorBoardToUse, int iMotorChannelToUse,
       BOARD_NAME bnDitherPztBoardToUse, int iDitherPztChannelToUse,
       BOARD_NAME bnEddy1BoardToUse, int iEddy1ChannelToUse,
       BOARD_NAME bnEddy2BoardToUse, int iEddy2ChannelToUse,
       BOARD_NAME bnFrontSoftLimitBoardToUse, int iFrontSoftLimitChannelToUse,
       BOARD_NAME bnFrontHardLimitBoardToUse, int iFrontHardLimitChannelToUse,
       BOARD_NAME bnBackSoftLimitBoardToUse, int iBackSoftLimitChannelToUse,
       BOARD_NAME bnBackHardLimitBoardToUse, int iBackHardLimitChannelToUse,
       BOARD_NAME bnLaserBoardToUse, int iLaserChannelToUse,
       BOARD_NAME bnHomeSensorBoardToUse, int iHomeSensorChannelToUse,
       BOARD_NAME bnServoPztReadbackBoardToUse, int iServoPztReadbackChannelToUse,
       BOARD_NAME bnMotorReadbackBoardToUse, int iMotorReadbackChannelToUse,
       BOARD_NAME bnDiagLaserInputBoardToUse, int iDiagLaserInputChannelToUse,
       BOARD_NAME bnDiagPztPositionInputBoardToUse, int iDiagPztPositionChannelToUse
,
       BOARD_NAME bnDiagEddy1BoardToUse, int iDiagEddy1ChannelToUse,
       BOARD_NAME bnDiagEddy2BoardToUse, int iDiagEddy2ChannelToUse,
       BOARD_NAME bnDiagLaserOutputBoardToUse, int iDiagLaserOutputChannelToUse,
       BOARD_NAME bnDiagLaserServoErrorOutputBoardToUse,
              int iDiagLaserServoErrorOutputChannelToUse,
       BOARD_NAME bnDiagVc1ServoErrorOutputBoardToUse,
              int iDiagVc1ServoErrorOutputChannelToUse,
       BOARD_NAME bnDiagVc2ServoErrorOutputBoardToUse,
              int iDiagVc2ServoErrorOutputChannelToUse,
       BOARD_NAME bnDiagMotorServoErrorOutputBoardToUse,
              int iDiagMotorServoErrorOutputChannelToUse) :

         tDlGizmoInfo(iWhichRteToUse,iGizmoIdToUse,
             iTargetSourceArray, tDitherPeriodToUse,
             PhysConstToUse, iNumServoModesToUse,
             ServoInfoToUse,
             bnVc1BoardToUse, iVc1ChannelToUse,
             bnServoPztBoardToUse, iServoPztChannelToUse,
             bnMotorBoardToUse, iMotorChannelToUse,
             bnDitherPztBoardToUse, iDitherPztChannelToUse,
             bnFrontSoftLimitBoardToUse, iFrontSoftLimitChannelToUse,
             bnFrontHardLimitBoardToUse, iFrontHardLimitChannelToUse,
             bnBackSoftLimitBoardToUse, iBackSoftLimitChannelToUse,
             bnBackHardLimitBoardToUse, iBackHardLimitChannelToUse,
             bnLaserBoardToUse , iLaserChannelToUse,
             bnHomeSensorBoardToUse , iHomeSensorChannelToUse,
             bnServoPztReadbackBoardToUse, iServoPztReadbackChannelToUse,
             bnMotorReadbackBoardToUse, iMotorReadbackChannelToUse,
             bnDiagLaserInputBoardToUse, iDiagLaserInputChannelToUse,
             bnDiagPztPositionInputBoardToUse, iDiagPztPositionChannelToUse,
             bnDiagLaserOutputBoardToUse, iDiagLaserOutputChannelToUse,
             bnDiagLaserServoErrorOutputBoardToUse,
             iDiagLaserServoErrorOutputChannelToUse,
             bnDiagVc1ServoErrorOutputBoardToUse,
             iDiagVc1ServoErrorOutputChannelToUse,
             bnDiagMotorServoErrorOutputBoardToUse,
             iDiagMotorServoErrorOutputChannelToUse)
{
  
  this->gtGizmoType = GT_DL_GSU;
  ServoParameters = ServoParametersToUse;
  pVc2Channel = D_CAST(tOutputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnVc2BoardToUse, iVc2ChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pVc2Channel is %ld\n",(int)pBackHardLimitChannel);

  pEddy1Channel = D_CAST(tInputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnEddy1BoardToUse, iEddy1ChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pEddy1Channel is %ld\n",(int)pBackHardLimitChannel);

  pEddy2Channel = D_CAST(tInputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnEddy2BoardToUse, iEddy2ChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pEddy2Channel is %ld\n",(int)pBackHardLimitChannel);

  pDiagVc2ServoErrorOutputChannel = D_CAST(tOutputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnDiagVc2ServoErrorOutputBoardToUse,
                                         iDiagVc2ServoErrorOutputChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pDiagVc2ServoErrorOutputChannel is %ld\n",(int)pBackHardLimitChannel);

  pDiagEddy1InputChannel = D_CAST(tInputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnDiagEddy1BoardToUse,
                                         iDiagEddy1ChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pDiagEddy1InputChannel is %ld\n",(int)pBackHardLimitChannel);

  pDiagEddy2InputChannel = D_CAST(tInputChannel *)
                                 ConfigManager.Hardware.CreateChannel
                                        (bnDiagEddy2BoardToUse,
                                         iDiagEddy2ChannelToUse);
  TraceMsg(tmt_INFO, S_configMgr_CHANNEL_INFO,
            "pDiagEddy2InputChannel is %ld\n",(int)pBackHardLimitChannel);
  if(!pVc2Channel || !pEddy1Channel || !pEddy2Channel ||
     !pDiagVc2ServoErrorOutputChannel || !pDiagEddy1InputChannel ||
     !pDiagEddy2InputChannel)
    TraceMsg(tmt_WARN, S_gizmoInfo_BAD_DYNAMIC_DOWNCAST,
             "tGsuDlGizmoInfo::tGsuDlGizmoInfo");
}



tGsuDlGizmoInfo::~tGsuDlGizmoInfo(void)
{ }




