/* $RCSfile: gsuDlModeUpdate.cc,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gsu/gsuDl/gsuDlModeUpdate.cc,v 1.1.1.1 1998/06/11 21:36:39 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
/*
$Log: gsuDlModeUpdate.cc,v $
Revision 1.1.1.1  1998/06/11 21:36:39  chara
First import of RICST Code

Revision 1.2.2.2  1998/05/05 15:28:14  rich
Delay line full function

Revision 1.2.2.1  1998/04/30 20:35:50  rbrill
commit before build_5a_008

Revision 1.2  1998/03/12 17:58:53  rich
*** empty log message ***

Revision 1.1  1998/03/11 01:00:14  rich
dl and gsuDl overhaul

*/

#include "gsu/gsuDlModeUpdate.h"
#include <math.h>


/**************************************************************************
*
* tGsuDlOffGroupUpdate:: tGsuDlOffGroupUpdate 
*
* SPECIFICATION:
[[U5 tGsuDlOffGroupUpdate::tGsuDlOffGroupUpdate (Thresholds, ControllerDataToUse) ]]
****/
tGsuDlOffGroupUpdate::tGsuDlOffGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse) :
 
   dDecelToIdleVelocityThreshold (Thresholds.dDecelToIdleVelocityThreshold),
   ControllerData (ControllerDataToUse) 
{

  this->iCurrentControllerMode = DOCM_DECELERATE;
}

/**************************************************************************
*
* tGsuDlOffGroupUpdate::GetNewMode 
*
* SPECIFICATION:
[[U5 tGsuDlOffGroupUpdate::GetNewMode (tUpdateTime) ]]
****/
int tGsuDlOffGroupUpdate::GetNewMode (TIME tUpdateTime)
{
  double dMotorVelocity = ControllerData.MotorVelocity ();
  
  // Check whether we should switch to idle from decelerating.
  
  if (this->iCurrentControllerMode == DOCM_DECELERATE &&
      fabs (dMotorVelocity) <= this->dDecelToIdleVelocityThreshold) 
        this->iCurrentControllerMode = DOCM_IDLE;
       
        
  return this->iCurrentControllerMode;
}
  

/**************************************************************************
*
* tGsuDlOffGroupUpdate::SetThresholds 
*
* SPECIFICATION:
[[U5 tGsuDlOffGroupUpdate::SetThresholds (Thresholds)  ]]
****/
ERROR_CODE tGsuDlOffGroupUpdate::SetThresholds (const tGsuDlThresholds& Thresholds) 
{

   this->dDecelToIdleVelocityThreshold = Thresholds.dDecelToIdleVelocityThreshold;
   return OK;
}



/**************************************************************************
*
* tGsuDlTrackGroupUpdate::tGsuDlTrackGroupUpdate
*
* SPECIFICATION:
[[U5 tGsuDlTrackGroupUpdate::tGsuDlTrackGroupUpdate (Thresholds, ControllerDataToUse) ]]
****/
tGsuDlTrackGroupUpdate::tGsuDlTrackGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse) :
 
   dTrackToSlewErrorThreshold (Thresholds.dTrackToSlewErrorThreshold),
   dSlewToTrackErrorThreshold (Thresholds.dSlewToTrackErrorThreshold),
   ControllerData (ControllerDataToUse) 
{

  this->iCurrentControllerMode = DTCM_SLEW;
}

/**************************************************************************
*
* tGsuDlTrackGroupUpdate::GetNewMode
*
* SPECIFICATION:
[[U5 tGsuDlTrackGroupUpdate::GetNewMode (tUpdateTime) ]]
****/
int tGsuDlTrackGroupUpdate::GetNewMode (TIME tUpdateTime)
{
  // convert to milli-meter from meter
  double dCartError = fabs (ControllerData.LinearCartError ()) * 1000.0;
  
  
  // Check whether we should switch to/from tracking/slewing
  switch (this->iCurrentControllerMode) {
  case DTCM_TRACK:
    if (dCartError >  this->dTrackToSlewErrorThreshold) {
      this->iCurrentControllerMode =  DTCM_SLEW;
    } 
    else
      this->iCurrentControllerMode =  DTCM_TRACK;
    break;
  
  case DTCM_SLEW:
    if (dCartError <  this->dSlewToTrackErrorThreshold) {
      this->iCurrentControllerMode =  DTCM_TRACK;
      
    }
    else this->iCurrentControllerMode =  DTCM_SLEW;
          
    break;
    
  default:
    this->iCurrentControllerMode =  DTCM_SLEW;
    break;
  }

  return this->iCurrentControllerMode;
}

/**************************************************************************
*
* tGsuDlTrackGroupUpdate::SetThresholds
*
* SPECIFICATION:
[[U5 tGsuDlTrackGroupUpdate::SetThresholds (Thresholds)  ]]
****/
ERROR_CODE tGsuDlTrackGroupUpdate::SetThresholds (const tGsuDlThresholds& Thresholds) 
{

   this->dTrackToSlewErrorThreshold = Thresholds.dTrackToSlewErrorThreshold;
   this->dSlewToTrackErrorThreshold = Thresholds.dSlewToTrackErrorThreshold;
   return OK;
}
