/* $RCSfile: gsuDlMotorController.cc,v $ -*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gsu/gsuDl/gsuDlMotorController.cc,v 1.3 1998/09/13 19:43:43 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
DESCRIPTION

*/

#include "gsu/gsuDlMotorController.h"
#include <math.h>


#ifdef SERVO_TUNING_ON
  BOOL     bSetEddy2ModParams = FALSE;
  MOD_TYPE mtEddy2ModType = MT_SQUARE;
  double   dEddy2ModFreq  = 2.0;
  double   dEddy2ModAmpl  = 50.0;
#endif /* SERVO_TUNING_ON */


/***************************************************************
*
* tGsuDlMotorTrackController::tGsuDlMotorTrackController
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::tGsuDlMotorTrackController (ControllerData, IoSet, 
      InitialParameters, dLaserPhysicalPerCount, iLaserPasses) ]]
****/
tGsuDlMotorTrackController::tGsuDlMotorTrackController (
      tGsuDlControllerData& ControllerData,
      tGsuDlCompleteIoSet& IoSet, const tGsuDlMotorParameters& InitialParameters,
      double dLaserPhysicalPerCount, int iLaserPasses) :
     
  /****
  [[U tDlMotorController::tDlMotorController (ControllerData, IoSet, InitialParameters, dLaserPhysicalPerCount, iLaserPasses) ]]
  ****/
  tDlMotorController (ControllerData, IoSet, InitialParameters, dLaserPhysicalPerCount, iLaserPasses),
  
  /****
  [ this->Eddy := reference to IoSet.Eddy2
    : this->EddyDiagInput := reference to IoSet.Eddy2DiagInput
    : this->MotorServoDiagOutput := reference to IoSet.MotorServoErrorDiagOutput
    : this->EddyTarget := reference to IoSet.Eddy2Target
  ]
  ****/
  Eddy (IoSet.Eddy2),
  EddyDiagInput (IoSet.Eddy2DiagInput),
  MotorServoErrorDiagOutput (IoSet.MotorServoErrorDiagOutput),
  EddyTarget (IoSet.Eddy2Target),
  
  /****
  [ this->MotorPid.tPidDouble::tPidDouble (InitialParameters.dMotorGain, 
      InitialParameters.dMotorPidTau1, InitialParameters.dMotorPidTau2, 
      InitialParameters.dMotorPidTaud, 
      InitialParameters.dMotorSamplePeriod) ) ]]
  ****/
  MotorPid (tPidDouble (InitialParameters.dMotorGain, InitialParameters.dMotorPidTau1, 
                        InitialParameters.dMotorPidTau2, InitialParameters.dMotorPidTaud,
                        InitialParameters.dMotorSamplePeriod) ),
  dMotorServoError (ControllerData.MotorServoError ())                        

{} 
  
/***************************************************************
*
* tGsuDlMotorTrackController::Init 
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::Init (TIME) ]]
****/
ERROR_CODE tGsuDlMotorTrackController::Init (TIME tServoTime)
{
  #ifdef SERVO_TUNING_ON
    if (bSetEddy2ModParams) {
      tModulationParams Eddy2ModParams(mtEddy2ModType, dEddy2ModFreq, dEddy2ModAmpl);
      this->EddyDiagInput.SetModulationParameters(Eddy2ModParams);
      bSetEddy2ModParams = FALSE;
    }
  #endif /* SERVO_TUNING_ON */
  return OK;
}


/***************************************************************
*
* tGsuDlMotorTrackController::Reset 
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::Reset (TIME) ]]
****/
ERROR_CODE tGsuDlMotorTrackController::Reset (TIME tServoTime)
{
  this->MotorPid.Reset(0.0);

  return OK;
}


/***************************************************************
*
* tGsuDlMotorTrackController::SetControlParameters 
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::SetControlParameters (NewParameters) ]]
****/
ERROR_CODE tGsuDlMotorTrackController::SetControlParameters (const tGsuDlMotorParameters& NewParameters)
{
  /****
  [ this->MotorPid.tPidDouble::tPidDouble (NewParameters.dMotorGain, 
      NewParameters.dMotorPidTau1, InitialParameters.dMotorPidTau2, 
      NewParameters.dMotorPidTaud, 
      NewParameters.dMotorSamplePeriod) ) ]]
  ****/
  this->MotorPid = tPidDouble (NewParameters.dMotorGain, NewParameters.dMotorPidTau1, 
                      NewParameters.dMotorPidTau2, NewParameters.dMotorPidTaud,
                      NewParameters.dMotorSamplePeriod);


  /****
  [[U tDlMotorController::SetControlParameters (NewParameters) ]]
  ****/
  return this->tDlMotorController::SetControlParameters (NewParameters);
}    


/***************************************************************
*
* tGsuDlMotorTrackController::Calculate 
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::Calculate (tServoTime) ]]
****/
ERROR_CODE tGsuDlMotorTrackController::Calculate (TIME tServoTime)
{

  double dServoError, dMotorFeedforward, dMotorVelocity;
  
  /****
  [[U dServoError := tGsuDlMotorTrackController::ServoError (tServoTime) ]]
  ****/
  dServoError = this->ServoError (tServoTime);
  
  /****
  [[U tGsuDlMotorTrackController::Enabled () ]]
  ****/
  if (this->Enabled ()) {

    /****
    [[U dMotorFeedforward := tGsuDlMotorTrackController::MotorFeedforward (tServoTime) ]]
    ****/
    
    /****
      Compute motor feedforward velocity as follows:
        Get the target velocity, which is in units of laser counts/time tick.
        Convert to laser counts per second.
        Convert to Motor physical motion per second
        Convert to Motor counts per second
        Multiply by feedforward gain.
    ****/
    dMotorFeedforward = this->MotorFeedforward.Calculate (
      this->MotorPhysicalUnitsToMotorCounts(
        this->LaserCountsToMotorPhysical (
          ((double) TIME_TICKS_PER_SECOND) * this->MotorTarget.Update (tServoTime)
      )));
  
    /****
    [[U dMotorVelocity := tGsuDlMotorTrackController::Correction (this->dMotorServoError + dMotorFeedforward) ]]
    ****/
    dMotorVelocity = this->Correction (dServoError) + dMotorFeedforward;
    
    /****
    [[U dMotorVelocity := tGsuDlMotorTrackController::Actuate (dMotorVelocity, tServoTime) ]]
    ****/
  }
  else 
    dMotorVelocity = 0.0;

  return this->Actuate (dMotorVelocity, tServoTime);
  
}

/***************************************************************
*
* tGsuDlMotorTrackController::ServoError  
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::ServoError (tServoTime) ]]
****/
double tGsuDlMotorTrackController::ServoError (TIME tServoTime)
{
  FAST double dRawDiagValue;

  /****
  [[U dInput := this->Eddy.tSensor::Update (tServoTime) ]]

  [[U dDiag := this->EddyDiag.tSensor::Update (tServoTime) ]]

  [[U dTarget := this->EddyTarget.tTargetSource::Update (tServoTime) ]]
  
  [ this->dMotorServoError := dInput + dDiag - dTarget ]
  ****/
  this->dMotorServoError = this->Eddy.Update (tServoTime)
                           - this->EddyTarget.Update (tServoTime)
                           + (dRawDiagValue = this->EddyDiagInput.Update (tServoTime));
         

  /****
  [[U this->MotorServoErrorDiagOutput.tActuator::OutputValue (tServoTime, 
      this->dMotorServoError) ]]
  ****/
  this->MotorServoErrorDiagOutput.OutputValue (tServoTime, dRawDiagValue);
  
  return this->dMotorServoError;         
}



/***************************************************************
*
* tGsuDlMotorTrackController::Correction 
*
* SPECIFICATION:
[[U5 tGsuDlMotorTrackController::Correction (dError) ]]
****/
double tGsuDlMotorTrackController::Correction (double dError)
{
  /****
  [[U this->MotorPid.tPidDouble::Calculate (dError) ]]
  ****/
  return this->MotorPid.Calculate (dError);
}



