/* $RCSfile: gsuDlVcController.cc,v $*/
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/gsu/gsuDl/gsuDlVcController.cc,v 1.5 1998/09/23 23:57:12 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
05a,29Jan98,rlj   written
*/
 
/*
DESCRIPTION
  GSU Delay Line Voice coil Controllers.
*/

#include "isit.h"
#include "recLibB.h"  
#include "gsu/gsuDlVcController.h"

#ifdef SERVO_TUNING_ON
  BOOL     bSetPztPosModParams = FALSE;
  MOD_TYPE mtPztPosModType = MT_SQUARE;
  double   dPztPosModFreq  = 2.0;
  double   dPztPosModAmpl  = 100.0;

  double dVc1DiagErrorScale = 100.0;

  BOOL     bSetEddy1ModParams = FALSE;
  MOD_TYPE mtEddy1ModType = MT_SQUARE;
  double   dEddy1ModFreq  = 0.2;
  double   dEddy1ModAmpl  = 550.0;
#endif /* SERVO_TUNING_ON */


/********************************************************************
* 
* tGsuDlVcController::tGsuDlVcController
*
* SPECIFICATION:
[[U5 tGsuDlVcController::tGsuDlVcController (ControllerData, IoSet, Vc1InputIn, Vc1DiagInputIn, 
        Vc2InputIn, Vc2DiagInputIn, Vc1TargetIn, Vc2TargetIn) ]]
*/
tGsuDlVcController::tGsuDlVcController (tGsuDlControllerData& ControllerData, tGsuDlCompleteIoSet& IoSet,
        tSensor& Vc1InputIn, tModulatedSensor& Vc1DiagInputIn, 
        tSensor& Vc2InputIn, tModulatedSensor& Vc2DiagInputIn, 
        tTargetSource& Vc1TargetIn, tTargetSource& Vc2TargetIn) :

    // Initialize all references
    
    Vc1Input (Vc1InputIn),
    Vc1DiagInput (Vc1DiagInputIn),
    Vc2Input (Vc2InputIn),
    Vc2DiagInput (Vc2DiagInputIn),
    
    Vc1 (IoSet.Vc1),
    Vc1ServoErrorDiagOutput (IoSet.Vc1ServoErrorDiagOutput),
    Vc2 (IoSet.Vc2),
    Vc2ServoErrorDiagOutput (IoSet.Vc2ServoErrorDiagOutput),
    
    Vc1Target (Vc1TargetIn),
    Vc2Target (Vc2TargetIn),
    

    // Iniitialize additional members...

    dVc1Error (ControllerData.Vc1Error ()),
    dVc2Error (ControllerData.Vc2Error ()),
    
    dVc1Output (ControllerData.Vc1Output ()),
    dVc2Output (ControllerData.Vc2Output ()),

    bVc1Enabled (TRUE),
    bVc2Enabled (TRUE)
{}



/********************************************************************
* 
* tGsuDlVcController::Vc1ServoError  
*
* SPECIFICATION:
[[U5 tGsuDlVcController::Vc1ServoError (tTime) ]]
*/
double tGsuDlVcController::Vc1ServoError (TIME tTime) 
{
  /****
  Update each servo error component and compute servo error.
  
  [[U dInput := this->Vc1Input.tSensor::Update (tTime) ]]
  [[U dDiag := this->Vc1Diag.tSensor::Update (tTime) ]]
  [[U dTarget := this->Vc1Target.tTargetSource::Update (tTime) ]]
  [ this->dVc1Error := dInput + dDiag - dTarget ]
  ****/ 
  this->dVc1Error = this->Vc1Input.Update (tTime)
                    + this->Vc1DiagInput.Update (tTime)
                    - this->Vc1Target.Update (tTime);

  return this->dVc1Error;
}



/********************************************************************
* 
* tGsuDlVcController::Vc2ServoError  
*
* SPECIFICATION:
[[U5 tGsuDlVcController::Vc2ServoError (TIME tTime) ]]
*/
double tGsuDlVcController::Vc2ServoError (TIME tTime) 
{
  /****
  Update each servo error component and compute servo error.
  
  [[U dInput := this->Vc2Input.tSensor::Update (tTime) ]]
  [[U dDiag := this->Vc2Diag.tSensor::Update (tTime) ]]
  [[U dTarget := this->Vc2Target.tTargetSource::Update (tTime) ]]
  [ this->dVc2Error := dInput + dDiag - dTarget ]
  ****/ 
  this->dVc2Error = (this->Vc2Input.Update (tTime))
                    + this->Vc2DiagInput.Update (tTime)
                    - this->Vc2Target.Update (tTime);

  return this->dVc2Error;
}





/********************************************************************
* 
* tGsuDlVcController::Actuate  
*
* SPECIFICATION:
[[U5 tGsuDlVcController::Actuate (tTime, dVc1Output, dVc2Output) ]]
*/
ERROR_CODE tGsuDlVcController::Actuate (TIME tTime, double dVc1Out, double dVc2Out)
{

  this->dVc1Output = dVc1Out;
  
  this->dVc2Output = dVc2Out;

  // Acutate voice coils with specified output values

  /****
  [[U this->Vc1.tModulatedPositionActuator::OutputValue (tTime, dVc1Output) ]]
  ****/
  this->Vc1.OutputValue (tTime, dVc1Output);
 
  /****
  [[U this->Vc2.tModulatedAccelerationActuator::OutputValue (tTime, dVc2Output) ]]
  ****/
  this->Vc2.OutputValue (tTime, dVc2Output);


  return OK;

}

/********************************************************************
* 
* tGsuDlVcController::ResetVc1 
*
* SPECIFICATION:
[[U5 tGsuDlVcController::ResetVc1 () ]]
*/
ERROR_CODE tGsuDlVcController::ResetVc1 ()
{

  /****
  [[U this->Vc1.tActuator::Reset () ]]
  ****/
  return this->Vc1.Reset ();
  

}

/********************************************************************
* 
* tGsuDlVcController::ResetVc2
*
* SPECIFICATION:
[[U5 tGsuDlVcController::ResetVc2 () ]]
*/
ERROR_CODE tGsuDlVcController::ResetVc2 ()
{

  /****
  [[U this->Vc2.tActuator::Reset () ]]
  ****/
  return this->Vc2.Reset ();
  

}

/********************************************************************
* 
* tGsuDlVcStablizeController::tGsuDlVcStablizeController
*
* SPECIFICATION:
[[U5 tGsuDlVcStabilizeController::tGsuDlVcStabilizeController (ControllerData,
    IoSet,InitialParameters) ]]
*/
tGsuDlVcStabilizeController::tGsuDlVcStabilizeController (
        tGsuDlControllerData& ControllerData, tGsuDlCompleteIoSet& IoSet, 
        const tGsuDlVcStabilizeParameters& InitialParameters) :

    /****
    [[U tGsuDlVcController::tGsuDlVcController (ControllerData, IoSet, 
      IoSet.Eddy1, IoSet.Eddy1DiagInput, 
      IoSet.Eddy2, IoSet.Eddy2DiagInput,
      IoSet.Eddy1Target, IoSet.Eddy2Target) ]]
    ****/
    tGsuDlVcController (ControllerData, IoSet,
      IoSet.Eddy1, IoSet.Eddy1DiagInput, 
      IoSet.Eddy2, IoSet.Eddy2DiagInput,
      IoSet.Eddy1Target, IoSet.Eddy2Target),
      
    /****
    [[U this->Vc1Pid.tPidDouble::tPidDouble_ddddd (InitialParameters.dVc1PidGain,
            InitialParameters.dVc1PidTau1, InitialParameters.dVc1PidTau2, 
            InitialParameters.dVc1PidTaud, InitialParameters.dVcSamplePeriod) ]]
    ****/            
    Vc1Pid (InitialParameters.dVc1PidGain, InitialParameters.dVc1PidTau1, 
            InitialParameters.dVc1PidTau2, InitialParameters.dVc1PidTaud, 
            InitialParameters.dVcSamplePeriod), 
              
    /****
    [[U this->Vc2Pid.tPidDouble::tPidDouble_ddddd (InitialParameters.dVc2PidGain,
            InitialParameters.dVc2PidTau1, InitialParameters.dVc2PidTau2, 
            InitialParameters.dVc2PidTaud, InitialParameters.dVcSamplePeriod) ]]
    ****/            
    Vc2Pid (InitialParameters.dVc2PidGain, InitialParameters.dVc2PidTau1, 
            InitialParameters.dVc2PidTau2, InitialParameters.dVc2PidTaud, 
            InitialParameters.dVcSamplePeriod),

    /****
    [[U this->Vc2Feedforward.tFeedforward::tFeedforward_d (
          InitialParameters.dVc2FeedforwardGain) ]]
    ****/            
    Vc2Feedforward (InitialParameters.dVc2FeedforwardGain),
      
    CdMotorAcceleration (ControllerData.MotorAcceleration ())
{
    /****
    [[ this->bVc1Enabled := InitialParameters.bVc1Enabled ]]
    [[ this->bVc2Enabled := InitialParameters.bVc2Enabled ]]
    ****/
    this->Vc1Enabled () = InitialParameters.bVc1Enabled;
    this->Vc2Enabled () = InitialParameters.bVc2Enabled;
}


/********************************************************************
* 
* tGsuDlVcStabilizeController::SetControlParameters 
*
* SPECIFICATION:
[[U5 tGsuDlVcStabilizeController::SetControlParameters (NewParameters) ]]
*/
ERROR_CODE tGsuDlVcStabilizeController::SetControlParameters (
     const tGsuDlVcStabilizeParameters& NewParameters) 
{
  // Set enable state of both voice coil controllers.
  
    /****
    [[ this->bVc1Enabled := NewParameters.bVc1Enabled ]]
    [[ this->bVc2Enabled := NewParameters.bVc2Enabled ]]
    ****/
    this->Vc1Enabled () = NewParameters.bVc1Enabled;
    this->Vc2Enabled () = NewParameters.bVc2Enabled;
    
  /****
  Update PID parameters for VC1 controller
  
  [[U this->Vc1Pid.tPidDouble::tPidDouble_ddddd (NewParameters.dVc1PidGain, 
      NewParameters.dVc1PidTau1, NewParameters.dVc1PidTau2, 
      NewParameters.dVc1PidTaud, NewParameters.dVcSamplePeriod) ]]
  ****/      
  this->Vc1Pid = tPidDouble (NewParameters.dVc1PidGain, NewParameters.dVc1PidTau1, 
                        NewParameters.dVc1PidTau2, NewParameters.dVc1PidTaud, 
                        NewParameters.dVcSamplePeriod);

  /****
  Update PID parameters for VC2 controller
  
  [[U this->Vc2Pid.tPidDouble::tPidDouble_ddddd (NewParameters.dVc2PidGain, 
      NewParameters.dVc2PidTau1, NewParameters.dVc2PidTau2, 
      NewParameters.dVc2PidTaud, NewParameters.dVcSamplePeriod) ]]
  ****/      
  this->Vc2Pid = tPidDouble (NewParameters.dVc2PidGain, NewParameters.dVc2PidTau1, 
                        NewParameters.dVc2PidTau2, NewParameters.dVc2PidTaud, 
                        NewParameters.dVcSamplePeriod);


  /****
  Update Feedforward parameters for VC2 controller

  [[U this->Vc2Feedforward.tFeedforward::tFeedforward_d (NewParameters.dVc2FeedforwardGain) ]]
  ****/      
  this->Vc2Feedforward = tFeedforward (NewParameters.dVc2FeedforwardGain);
  

  return OK;   
}
 


/********************************************************************
* 
* tGsuDlVcStabilizeController::Init
* 
* SPECIFICATION:
[[U5 tGsuDlVcStabilizeController::Init (tServoTime) ]]
*/
ERROR_CODE tGsuDlVcStabilizeController::Init (TIME tServoTime) 
{
  /****
  Reset PID to current servo errors
  
  [[U dError := tGsuDlVcStabilizeController::Vc1ServoError (tServoTime) ]]
  
  [[U this->Vc1Pid.tPidDouble::Reset (dError) ]]
  ****/
  this->Vc1Pid.Reset (this->Vc1ServoError (tServoTime)); //
  
  /****
  Reset PID to current servo errors
  
  [[U dError := tGsuDlVcStabilizeController::Vc2ServoError (tServoTime) ]]
  
  [[U this->Vc2Pid.tPidDouble::Reset (dError) ]]
  ****/
  this->Vc2Pid.Reset (this->Vc2ServoError (tServoTime)); //
  
  return OK;
} 


/********************************************************************
* 
* tGsuDlVcStabilizeController::Calculate
*
* SPECIFICATION:
[[U5 tGsuDlVcStabilizeController::Calculate (tServoTime) ]]
*/
ERROR_CODE tGsuDlVcStabilizeController::Calculate (TIME tServoTime)
{
 
 
  double dVc1Error, dVc2Error, dVc2FFPortion, dVc1Output, dVc2Output;

  
  // Compute servo errors for both voice coils and feed forward from motor accel.
  
  /****
  [[U dVc1Error := tGsuDlVcStabilizeController::Vc1ServoError (tServoTime) ]]
  ****/                                        
  dVc1Error = this->Vc1ServoError (tServoTime);
                                        
  /****
  [[U dVc2Error := tGsuDlVcStabilizeController::Vc2ServoError (tServoTime) ]]
  ****/                                        
  dVc2Error = this->Vc2ServoError (tServoTime);
  
  
  /****
  [[U dVc2FFPortion := this->Vc2Feedforward.tFeedForward::Calculate (this->CdMotorAcceleration) ]]
  ****/                                        
  dVc2FFPortion = this->Vc2Feedforward.Calculate (this->CdMotorAcceleration);
  
  // Run control laws for enabled voice coils only
  
  /****
  [[U tGsuDlVcStabilizeController::Vc1Enabled () ]]
  ****/
  if (this->Vc1Enabled () )  {
  
    
    /*****
    Compute PID compensation value using the servo error.
  
    [[U dVc1Output := this->Vc1Pid.tPidDouble::Calculate (dVc1Error) ]]
    ****/
    dVc1Output = this->Vc1Pid.Calculate (dVc1Error);

  }
  else dVc1Output = 0.0;
 
  /****
  [[U tGsuDlVcStabilizeController::Vc2Enabled () ]]
  ****/
  if (this->Vc2Enabled () )  {
  
    
    /*****
    Compute PID compensation value using the servo error and adde it to the filtered feed forward value
  
    [[U dTemp := this->Vc2Pid.tPidDouble::Calculate (dVc2Error) ]]
    
    [ dVc2Output := dTemp + dVc2FFPortion ]
    ****/
    dVc2Output = this->Vc2Pid.Calculate (dVc2Error) + dVc2FFPortion;

  }
  else  dVc2Output = 0.0;
  
  /****
  [[U tGsuDlVcStabilizeController::Actuate (tServoTime, dVc1Output, dVc2Output) ]]
  ****/   
  return this->Actuate (tServoTime, dVc1Output, dVc2Output);
}



 


/********************************************************************
* 
* tGsuDlVcTrackController::tGsuDlVcTrackController
*
* SPECIFICATION:
[[U5 tGsuDlVcTrackController::tGsuDlVcTrackController (ControllerData,
    IoSet, InitialParameters) ]]
*/
tGsuDlVcTrackController::tGsuDlVcTrackController (
        tGsuDlControllerData& ControllerData, tGsuDlCompleteIoSet& IoSet, 
        const tGsuDlVcTrackParameters& InitialParameters) :

    /****
    [[U tGsuDlVcController::tGsuDlVcController (ControllerData, IoSet,
      IoSet.PztPosition, IoSet.PztPositionDiagInput, 
      IoSet.Eddy1, IoSet.Eddy1DiagInput,
      IoSet.PztPositionTarget, IoSet.Eddy1Target) ]]
    ****/
    tGsuDlVcController (ControllerData, IoSet,
      IoSet.PztPosition, IoSet.PztPositionDiagInput, 
      IoSet.Eddy1, IoSet.Eddy1DiagInput,
      IoSet.PztPositionTarget, IoSet.Eddy1Target),

    Eddy2Input(IoSet.Eddy2),
      
    /****
    [[U this->Vc1Pid.tPidDouble::tPidDouble_ddddd (InitialParameters.dVc1PidGain,
            InitialParameters.dVc1PidTau1, InitialParameters.dVc1PidTau2, 
            InitialParameters.dVc1PidTaud, InitialParameters.dVcSamplePeriod) ]]
    ****/            
    Vc1Pid (InitialParameters.dVc1PidGain, InitialParameters.dVc1PidTau1, 
            InitialParameters.dVc1PidTau2, InitialParameters.dVc1PidTaud, 
            InitialParameters.dVcSamplePeriod), 
              
    /****
    [[U this->Vc2Pid.tPidDouble::tPidDouble_ddddd (InitialParameters.dVc2PidGain,
            InitialParameters.dVc2PidTau1, InitialParameters.dVc2PidTau2, 
            InitialParameters.dVc2PidTaud, InitialParameters.dVcSamplePeriod) ]]
    ****/            
    Vc2Pid (InitialParameters.dVc2PidGain, InitialParameters.dVc2PidTau1, 
            InitialParameters.dVc2PidTau2, InitialParameters.dVc2PidTaud, 
            InitialParameters.dVcSamplePeriod),

    /****
    [[U this->Vc1Feedforward.tFeedforward::tFeedforward_d (
          InitialParameters.dVc1FeedforwardGain) ]]
    ****/            
    Vc1Feedforward (InitialParameters.dVc1FeedforwardGain),

    /****
    [[U this->Vc2Feedforward.tFeedforward::tFeedforward_d (
          InitialParameters.dVc2FeedforwardGain) ]]
    ****/            
    Vc2Feedforward (InitialParameters.dVc2FeedforwardGain),
      
    /****
    [[U this->Vc1FeedforwardFilter.tFilter::tFilter_d (InitialParameters.dVc1FilterMemory) ]]
    ****/            
    Vc1FeedforwardFilter (InitialParameters.dVc1FilterMemory),

    CdLaserError (ControllerData.ServoError ()),
    
    /****
    Limit the upper range of VC1 output to half the output of VC2 but convert to Vc1 counts in case
    actuators have different scales 
    
    [[U dMaxVc2 := tGsuDlVcTrackController::Vc2UpperLimit () ]]
    
    [[U CdVc1ControlUpperLimit := tGsuDlVcTrackController::Vc2CountsToVc1Counts ((dMaxVc2/ 2.0) )]]
    ****/
    CdVc1ControlUpperLimit (this->Vc2CountsToVc1Counts (this->Vc2UpperLimit ()) / 2.0),

    /****
    Limit the lower range of VC1 output to half the output of VC2 but convert to Vc1 counts in case
    actuators have different scales 
    
    [[U dMinVc2 := tGsuDlVcTrackController::Vc2LowerLimit () ]]
    
    [[U CdVc1ControlLowerLimit := tGsuDlVcTrackController::Vc2CountsToVc1Counts ((dMinVc2 / 2.0) ]]
    ****/
    CdVc1ControlLowerLimit (this->Vc2CountsToVc1Counts (this->Vc2LowerLimit ()) / 2.0 )
    
    
    {
    /****
    [[ this->bVc1Enabled := InitialParameters.bVc1Enabled ]]
    [[ this->bVc2Enabled := InitialParameters.bVc2Enabled ]]
    ****/
    this->Vc1Enabled () = InitialParameters.bVc1Enabled;
    this->Vc2Enabled () = InitialParameters.bVc2Enabled;
}




 /********************************************************************
* 
* tGsuDlVcTrackController::SetControlParameters 
*
* SPECIFICATION:
[[U5 tGsuDlVcTrackController::SetControlParameters (NewParameters) ]]
*/
ERROR_CODE tGsuDlVcTrackController::SetControlParameters (
     const tGsuDlVcTrackParameters& NewParameters) 
{
  // Set enable state of both voice coil controllers.
  
    /****
    [[ this->bVc1Enabled := NewParameters.bVc1Enabled ]]
    [[ this->bVc2Enabled := NewParameters.bVc2Enabled ]]
    ****/
    this->Vc1Enabled () = NewParameters.bVc1Enabled;
    this->Vc2Enabled () = NewParameters.bVc2Enabled;
    
  /****
  Update PID parameters for VC1 controller
  
  [[U this->Vc1Pid.tPidDouble::tPidDouble_ddddd (NewParameters.dVc1PidGain, 
      NewParameters.dVc1PidTau1, NewParameters.dVc1PidTau2, 
      NewParameters.dVc1PidTaud, NewParameters.dVcSamplePeriod) ]]
  ****/      
  this->Vc1Pid = tPidDouble (NewParameters.dVc1PidGain, NewParameters.dVc1PidTau1, 
                        NewParameters.dVc1PidTau2, NewParameters.dVc1PidTaud, 
                        NewParameters.dVcSamplePeriod);

  /****
  Update PID parameters for VC2 controller
  
  [[U this->Vc2Pid.tPidDouble::tPidDouble_ddddd (NewParameters.dVc2PidGain, 
      NewParameters.dVc2PidTau1, NewParameters.dVc2PidTau2, 
      NewParameters.dVc2PidTaud, NewParameters.dVcSamplePeriod) ]]
  ****/      
  this->Vc2Pid = tPidDouble (NewParameters.dVc2PidGain, NewParameters.dVc2PidTau1, 
                        NewParameters.dVc2PidTau2, NewParameters.dVc2PidTaud, 
                        NewParameters.dVcSamplePeriod);


  /****
  Update Feedforward parameters for VC1 controller

  [[U this->Vc1Feedforward.tFeedforward::tFeedforward_d (NewParameters.dVc1FeedforwardGain) ]]
  ****/      
  this->Vc1Feedforward = tFeedforward (NewParameters.dVc1FeedforwardGain);

  /****
  Update Feedforward parameters for VC2 controller

  [[U this->Vc2Feedforward.tFeedforward::tFeedforward_d (NewParameters.dVc2FeedforwardGain) ]]
  ****/      
  this->Vc2Feedforward = tFeedforward (NewParameters.dVc2FeedforwardGain);
  
  /****
  Update Feedforward filter parameters for VC1 controller

  [[U this->Vc1FeedforwardFilter.tFilter::tFilter_d (NewParameters.dVc1FilterMemory) ]]
  ****/      
  this->Vc1FeedforwardFilter = tFilter (NewParameters.dVc1FilterMemory);

  return OK;   
}
  
  
  
/********************************************************************
* 
* tGsuDlVcTrackController::Init
* 
* SPECIFICATION:
[[U5 tGsuDlVcTrackController::Init (tServoTime) ]]
*/
ERROR_CODE tGsuDlVcTrackController::Init (TIME tServoTime) 
{
  tModulationParams tempModParams;

  #ifdef SERVO_TUNING_ON
    if (bSetPztPosModParams) {
      tModulationParams PztPosModParams(mtPztPosModType, dPztPosModFreq,
                                        dPztPosModAmpl);
      this->Vc1DiagInput.SetModulationParameters(PztPosModParams);
      this->Vc1DiagInput.EnableModulation();
      bSetPztPosModParams = FALSE;

      tempModParams = this->Vc1DiagInput.ModulationParameters();
      printf("PztPos Mod Params: Type=%d, Freq=%f, Ampl=%f\n",
              tempModParams.mtModulationType, tempModParams.dFrequency,
              tempModParams.dAmplitude);
    } /* Set PztPos Mod Params */

    if (bSetEddy1ModParams) {
      tModulationParams Eddy1ModParams(mtEddy1ModType, dEddy1ModFreq, dEddy1ModAmpl);
      this->Vc2DiagInput.SetModulationParameters(Eddy1ModParams);
      this->Vc2DiagInput.EnableModulation();
      bSetEddy1ModParams = FALSE;

      tempModParams = this->Vc2DiagInput.ModulationParameters();
      printf("Eddy1 Mod Params: Type=%d, Freq=%f, Ampl=%f\n",
              tempModParams.mtModulationType, tempModParams.dFrequency,
              tempModParams.dAmplitude);
    } /* Set Eddy1 Mod Params */
  #endif /* SERVO_TUNING_ON */

  /****
  Reset PID to current servo errors
  
  [[U dError := tGsuDlVcTrackController::Vc1ServoError (tServoTime) ]]
  
  [[U this->Vc1Pid.tPidDouble::Reset (dError) ]]
  ****/
  this->Vc1Pid.Reset (0.0);//this->Vc1ServoError (tServoTime));
  
  /****
  Reset PID to current servo errors
  
  [[U dError := tGsuDlVcTrackController::Vc2ServoError (tServoTime) ]]
  
  [[U this->Vc2Pid.tPidDouble::Reset (dError) ]]
  ****/
  this->Vc2Pid.Reset (0.0); //this->Vc2ServoError (tServoTime));
    
  /****
  Reset filter to current feedforward value
  
  [[U dFFValue := this->Vc1Feedforward.tFeedforward::Calculate (this->CdLaserError) ]]
  
  [[U this->Vc1FeedforwardFilter.tFilter::Reset (dFFValue) ]]
  ****/
  this->Vc1FeedforwardFilter.Reset (this->Vc1Feedforward.Calculate (this->CdLaserError));
  
  return OK;
} 


/********************************************************************
* 
* tGsuDlVcTrackController::Reset
* 
* SPECIFICATION:
[[U5 tGsuDlVcTrackController::Reset (tServoTime) ]]
*/
ERROR_CODE tGsuDlVcTrackController::Reset (TIME tServoTime) 
{

  /****
  [[U tGsuDlVcTrackController::ResetVc1 () ]]
  ****/
//  this->ResetVc1 ();
  
  /****
  [[U tGsuDlVcTrackController::ResetVc2 () ]]
  ****/
//  this->ResetVc2 ();
  
  return OK;
 
}


/********************************************************************
* 
* tGsuDlVcTrackController::Calculate
*
* SPECIFICATION:
[[U5 tGsuDlVcTrackController::Calculate (tServoTime) ]]
*/
ERROR_CODE tGsuDlVcTrackController::Calculate (TIME tServoTime)
{
  double dVc1Error, dVc1FFPortion, dVc2Error, dVc2FFPortion, dVc1Output, dVc2Output;
  
  // Compute servo errors for both voice coils.
  
  /****
  [[U dVc2Error := tGsuDlVcTrackController::Vc2ServoError (tServoTime) ]]
  ****/                                        
  dVc2Error = this->Vc2ServoError (tServoTime);
  
  /* These are too small to see so we multiply by 16. */
  this->Vc2ServoErrorDiagOutput.OutputValue (tServoTime,
                                      32.0*this->Eddy2Input.Update(tServoTime));
  
  /****
  [[U dVc1Error := tGsuDlVcTrackController::Vc1ServoError (tServoTime) ]]
  ****/                                        
  dVc1Error = this->Vc1ServoError (tServoTime);
   
  #ifdef SERVO_TUNING_ON 
  /******* This line is for Eddy Current Sensor Position Output *******/
  this->Vc1ServoErrorDiagOutput.OutputValue (tServoTime,
                                      dVc1DiagErrorScale*this->Vc2Input.Value());
  /******* This line is for Voice Coil Error Output *******/
  //this->Vc1ServoErrorDiagOutput.OutputValue (tServoTime,
  //                                  dVc1DiagErrorScale*dVc1Error);
  #endif /* SERVO_TUNING_ON */

  /****
  [[U tGsuDlVcTrackController::Vc1Enabled () ]]
  ****/
  if (this->Vc1Enabled () ) {

    /****
    Compute filtered feedforward value from laser error
  
    [[U  dPreFilter := this->Vc1Feedforward.tFeedforward::Calculate (this->CdLaserError) ]]
  
    [[U dFFPortion :=Vc1FeedforwardFilter.tFilter::LowPass (dPreFilter) ]]
    ****/
    dVc1FFPortion = this->Vc1FeedforwardFilter.LowPass (this->Vc1Feedforward.Calculate (this->CdLaserError));
   

    /*****
    Compute PID compensation value using the servo error and filtered feedforward.
  
    [[U dVc1Output := Vc1Pid.tPidDouble::Calculate ( (dVc1Error + dVc1FFPortion) ) ]]
    ****/
    dVc1Output = Vc1Pid.Calculate (dVc1Error + dVc1FFPortion);
  
    //  Limit Vc1 compensation to half the full range of the voice coil 2.  This is to allow voice coil 2 to apply an
    //  equal and opposite reaction force without saturating it's own range
  
    dVc1Output = this->CdVc1ControlUpperLimit < dVc1Output ? this->CdVc1ControlUpperLimit : dVc1Output;
    dVc1Output = this->CdVc1ControlLowerLimit > dVc1Output ? this->CdVc1ControlLowerLimit : dVc1Output;

  }
  
  else dVc1Output = 0.0;
  
  if (this->Vc2Enabled () )  {
  
    /****
    Compute feedforward value to compensate for force from VC1
  
    [[U  dFFPortion := this->Vc2Feedforward.tFeedforward::Calculate (dVc1Output) ]]
    ****/
    dVc2FFPortion = this->Vc2Feedforward.Calculate (dVc1Output);
    
    
    /*****
    Compute compensation value using the PID and Feedforward components.
  
    [[U dVc2Output := Vc2Pid.tPidDouble::Calculate (dVc2Error) ]]
    ****/
    dVc2Output = Vc2Pid.Calculate (dVc2Error) + dVc2FFPortion;
  
  }
  else dVc2Output = 0.0;

  // Actuate voice coils with appropriate values
  
  /****
  [[U tGsuDlVcTrackController::Actuate (tServoTime, this->dVc1Output, this->dVc2Output) ]]
  ****/                                        
  return this->Actuate (tServoTime, dVc1Output, dVc2Output);

}



 
