#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <X11/cursorfont.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>


#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "clientUtils.h"

#include "share/configShare.h"
#include "client.h"
#include "typeTag.h"
#include "clientData.h"


/*****************************************************************************
*
* ::processDlData(int iGizmoId, size_t szBlockSize)
*
* Description: This function processes delay line data from the embedded 
*    system by reading it over the network work and write it to disk
*
*
  [[U ::processDlData(iGizmoId, szBlockSize) ]]
*
*****************************************************************************/
void processDlData(int iGizmoId, size_t szBlockSize)
{
   tDlData buf;
   int nbytes;
   int dl_no = GizmoMap[iGizmoId].iGuiIndex;
   char defaultFilename[256];

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processDlData\n");
   else if (nbytes) {
      if (dl_shell[dl_no]->dataRecordOn && (dl_shell[dl_no]->data_fp != NULL) )
         fwrite((char *)&buf, szBlockSize, 1, dl_shell[dl_no]->data_fp);
   }

}

/*****************************************************************************
*
* ::processFtData(int iGizmoId, size_t szBlockSize)
*
* Description: This function processes Fringe Tracker data from the embedded 
*    system by reading it over the network work and write it to disk
*
*
  [[U ::processFtData(iGizmoId, szBlockSize) ]]
*
*****************************************************************************/
void processFtData(int iGizmoId, size_t szBlockSize)
{

   tFtData buf;
   int nbytes;
   int ft_no = GizmoMap[iGizmoId].iGuiIndex;

   /*
      [[U ::clientReadn(&buf, szBlockSize) ]]
   */
   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processFtData\n");
   else if (nbytes)
      /*
         [[U ::writeData(&buf, szBlockSize, ft_shell[ft_no]->ft_rec_data_text,
                      &(ft_shell[ft_no]->data_fp), 
                      ft_shell[ft_no]->dataRecordOn, DEFAULT_FT_DATA_NAME) ]]
      */
      writeData((char *)&buf, szBlockSize, ft_shell[ft_no]->ft_rec_data_text,
                  &(ft_shell[ft_no]->data_fp),
                  ft_shell[ft_no]->dataRecordOn, DEFAULT_FT_DATA_NAME);


}

/*****************************************************************************
*
* ::processEmData(int iGizmoId, size_t szBlockSize)
*
* Description: This function processes External Metrology data from the 
*    embedded system by reading it over the network work and write it to disk
*
*
  [[U ::processEmData(iGizmoId, szBlockSize) ]]
*
*****************************************************************************/
void processEmData(int iGizmoId, size_t szBlockSize)
{
   tEmData buf;
   int nbytes;
   int em_no = GizmoMap[iGizmoId].iGuiIndex;

   /*
      [[U ::clientReadn(&buf, szBlockSize) ]]
   */
   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processEmData\n");
   else if (nbytes)
      /*
         [[U ::writeData(&buf, szBlockSize, em_shell[em_no]->em_rec_data_text,
                         &(em_shell[em_no]->data_fp),
                         em_shell[em_no]->dataRecordOn, 
                         DEFAULT_EM_DATA_NAME) ]]
      */
      writeData((char *)&buf, szBlockSize, em_shell[em_no]->em_rec_data_text,
                  &(em_shell[em_no]->data_fp),
                  em_shell[em_no]->dataRecordOn, DEFAULT_EM_DATA_NAME);

}


/*****************************************************************************
*
* ::writeData(char *buf, int nbytes, Widget text_w, FILE **fp,
                 int toggle_flag, char *default_file_name )
*
* Description: This function write data to disk depending upon whether the
*    write data switch is turned on or off
*
*
  [[U ::writeData(char *buf, int nbytes, Widget text_w, FILE **fp,
                 int toggle_flag, char *default_file_name ) ]]
*
*****************************************************************************/

void writeData(char *buf, int nbytes, Widget text_w, FILE **fp,
                 int toggle_flag, char *default_file_name )
{
   char *string;
   char filename[256];


   if ( !toggle_flag )
   {
      if (*fp != NULL)
      {
         fclose(*fp);
         *fp = NULL;
      }
      return;
   }

   memset(filename,0,256);
   string  = XmTextGetString (text_w);
   if ( (string == NULL) || !(strcmp(string,"") ) )
   {
      sprintf(filename,"%s", default_file_name);
      XmTextSetString (text_w, default_file_name);
   }
   else
   {
      sprintf(filename,"%s", string);
      XmTextSetString (text_w, string);
   }

   XtFree(string);

   if (*fp == NULL) {
      if ( (*fp = fopen(filename,"a")) == NULL)
      {
         perror("Error opening file \n");
         fclose(*fp);
         return;
      }

   }
   fwrite(buf, nbytes, 1, *fp);
}
