#include <sys/types.h>
#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientUtils.h"
#include "clientTextUtils.h"
#include "clientDlCb.h"


/****************************************************************
*
* ::TssOpenLoop(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
*
* Description: This function will send target source select command
*    for open loop to the embedded system
*
   [[U ::TssOpenLoop(iGizmoId, iGuiGizmoIndex, bTargetSetActive) ]]
*
*****************************************************************/

void
TssOpenLoop(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"TssOpenLoop(<iGuiGizmoIndex=%d>)\n",iGuiGizmoIndex);
    data_send.dlsatCmd.Command = DL_CMD_SELECT_TARGET;
    data_send.dlsatCmd.bTargetSetActive = bTargetSetActive;
    data_send.dlsatCmd.ttWhichTarget = TT_OPEN_LOOP;
    data_send.dlsatCmd.iGizmoId = iGizmoId;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

}


/****************************************************************
*
* ::TssManualTarget(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
*
* Description: This function will send target source select command
*    for manual target to the embedded system
*
   [[U ::TssManualTarget(iGizmoId, iGuiGizmoIndex, bTargetSetActive) ]]
*
*****************************************************************/

void
TssManualTarget(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"TssManualTarget(<iGuiGizmoIndex=%d>)\n",iGuiGizmoIndex);
    data_send.dlsatCmd.Command = DL_CMD_SELECT_TARGET;
    data_send.dlsatCmd.bTargetSetActive = bTargetSetActive;
    data_send.dlsatCmd.ttWhichTarget = TT_MANUAL;
    data_send.dlsatCmd.iGizmoId = iGizmoId;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::TssFringeTracker(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
*
* Description: This function will send target source select command
*    for fringe tracker to the embedded system
*
   [[U ::TssFringeTracker(iGizmoId, iGuiGizmoIndex, bTargetSetActive) ]]
*
*****************************************************************/

void
TssFringeTracker(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"TssFringeTracker(<iGuiGizmoIndex=%d>)\n",iGuiGizmoIndex);
    data_send.dlsatCmd.Command = DL_CMD_SELECT_TARGET;
    data_send.dlsatCmd.ttWhichTarget = TT_FT;
    data_send.dlsatCmd.bTargetSetActive = bTargetSetActive;
    data_send.dlsatCmd.iGizmoId = iGizmoId;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::TssExternalMetrology(int iGizmoId, int iGuiGizmoIndex, 
                         BOOL bTargetSetActive)
*
* Description: This function will send target source select command
*    for external metrology to the embedded system
*
   [[U ::TssExternalMetrology(iGizmoId, iGuiGizmoIndex, bTargetSetActive) ]]
*
*****************************************************************/

void
TssExternalMetrology(int iGizmoId, int iGuiGizmoIndex, BOOL bTargetSetActive)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"TssExternalMetrology(<iGuiGizmoIndex=%d>)\n",iGuiGizmoIndex);
    data_send.dlsatCmd.Command = DL_CMD_SELECT_TARGET;
    data_send.dlsatCmd.bTargetSetActive = bTargetSetActive;
    data_send.dlsatCmd.ttWhichTarget = TT_EM;
    data_send.dlsatCmd.iGizmoId = iGizmoId;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::SetOpenLoopTarget(int iGizmoId, int iGuiGizmoIndex, tDlTargetBase Target)
*
* Description: This function will send a set open loop target command
*    to the embedded system
*
   [[U ::SetOpenLoopTarget(iGizmoId, iGuiGizmoIndex, Target) ]]
*
*****************************************************************/

void 
SetOpenLoopTarget(int iGizmoId, int iGuiGizmoIndex, tDlTargetBase Target)
{
    ISIT_CMD data_send;
    GIZMO_TYPE gtSpecificType;
    char cmd[256];

     data_send.instsoltCmd.Command = INST_CMD_SET_OPEN_LOOP_TARGET;
     data_send.instsoltCmd.iGizmoId = iGizmoId;
     data_send.instsoltCmd.Target.dPosition = Target.dPosition;
     data_send.instsoltCmd.Target.tTime =  Target.tTime;
     data_send.instsoltCmd.Target.dRate =  Target.dRate;
     GetEmbeddedSystemGizmoInfo (GT_DELAY_LINE, iGuiGizmoIndex, gtSpecificType,
     data_send.instsoltCmd.iDlGizmoId);

     memset(cmd,0,256);
     sprintf(cmd,"SetOpenLoopTarget(<iWhichDl=%d>, <dPosition=%lf>, 
                 <dRate=%lf>, <tTime=%lu>, <iDlGizmoId=%d>)\n", iGuiGizmoIndex, 
                 Target.dPosition, Target.dRate, Target.tTime,
                 data_send.instsoltCmd.iDlGizmoId);



    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

     /****
     [[U ::compTotalTarget(iGuiGizmoIndex, data_send.instsoltCmd.Target) ]]
     ****/
     compTotalTarget(iGuiGizmoIndex, data_send.instsoltCmd.Target);

}

/****************************************************************
*
* ::SetManualTarget(int iGizmoId, int iGuiGizmoIndex, tDlTargetBase Target)
*
* Description: This function will send a set manual target command
*    to the embedded system
*
   [[U ::SetManualTarget(iGizmoId, iGuiGizmoIndex, Target) ]]
*
*****************************************************************/
void 
SetManualTarget(int iGizmoId, int iGuiGizmoIndex, tDlTargetBase Target)
{
    ISIT_CMD data_send;
    char cmd[256];
    GIZMO_TYPE gtSpecificType;

     data_send.instsmtCmd.Command = INST_CMD_SET_MANUAL_TARGET;
     data_send.instsmtCmd.iGizmoId = iGizmoId;
     data_send.instsmtCmd.Target.dPosition =  Target.dPosition;
     data_send.instsmtCmd.Target.tTime=  Target.tTime;
     data_send.instsmtCmd.Target.dRate =  Target.dRate;
     GetEmbeddedSystemGizmoInfo (GT_DELAY_LINE, iGuiGizmoIndex, gtSpecificType,
     data_send.instsmtCmd.iDlGizmoId);

     memset(cmd,0,256);
     sprintf(cmd,"SetManualTarget(<iWhichDl=%d>, <dPosition=%lf>, 
                 <dRate=%lf>, <tTime=%lu>, <iDlGizmoId=%d>)\n", iGuiGizmoIndex, 
                 Target.dPosition, Target.dRate, Target.tTime,
                 data_send.instsmtCmd.iDlGizmoId);

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

     /****
     [[U ::compTotalTarget(iGuiGizmoIndex, data_send.instsmtCmd.Target) ]]
     ****/
     compTotalTarget(iGuiGizmoIndex, data_send.instsmtCmd.Target);

}

void DataRecord(int iGizmoId, int iGuiGizmoIndex, BOOL bTurnRecordOn)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"DlDataRecordCmd(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);
    data_send.dldrCmd.Command = DL_CMD_DATA_RECORD_ON_OFF;
    data_send.dldrCmd.bSuspendData = !bTurnRecordOn;
    data_send.dldrCmd.iGizmoId = iGizmoId;

    write_to_log(cmd, strlen(cmd) );

    write_to_error_file(cmd, strlen(cmd) );

    do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DitherOn(int iGizmoId, int iGuiGizmoIndex, BOOL bTurnDitherOn)
*
* Description: This function will send a dither on command to the 
*    embedded system
*
   [[U ::DitherOn(iGizmoId, iGuiGizmoIndex, bTurnDitherOn) ]]
*
*****************************************************************/

void DitherOn(int iGizmoId, int iGuiGizmoIndex, BOOL bTurnDitherOn)
{
    ISIT_CMD data_send;
    char cmd[256];

    memset(cmd,0,256);
    sprintf(cmd,"DitherOn(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);
    data_send.dldoCmd.Command = DL_CMD_DITHER_ON;
    data_send.dldoCmd.bTurnDitherOn = bTurnDitherOn;
    data_send.dldoCmd.iGizmoId = iGizmoId;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::SetOpenLoopAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send an open loop target command with
*    auto reasonable values to the embedded system
*
   [[U ::SetOpenLoopAutoReasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/
void 
SetOpenLoopAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
{
     tDlTargetBase Target;


     Target.dPosition =  ((double)(rand() %2) + 
                ( (rand() %2) ? 1 : -1) * (0.5) ) * (4*1000000000*256/633);
     Target.dRate = ( (double) (rand() % 21) +
                ( (rand() %2) ? 1 : -1) * (10) ) * (4*1000000*256/633);

     Target.tTime = DlTime[iGuiGizmoIndex];

     setTextForTarget( dl_solt_shell[iGuiGizmoIndex]->dl_solt_pos_text,
                       dl_solt_shell[iGuiGizmoIndex]->dl_solt_rate_text,
                       dl_solt_shell[iGuiGizmoIndex]->dl_solt_time_text,
                       Target);

}

/****************************************************************
*
* ::SetOpenLoopAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send an open loop target command with
*    auto unreasonable values to the embedded system
*
   [[U ::SetOpenLoopAutoUnreasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void 
SetOpenLoopAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     tDlTargetBase Target;


     Target.dPosition = (double)rand();
     Target.dRate = (double) rand();
     Target.tTime = (unsigned long)rand() * TIME_TICKS_PER_SECOND;

     setTextForTarget( dl_solt_shell[iGuiGizmoIndex]->dl_solt_pos_text,
                       dl_solt_shell[iGuiGizmoIndex]->dl_solt_rate_text,
                       dl_solt_shell[iGuiGizmoIndex]->dl_solt_time_text,
                       Target);

}

/****************************************************************
*
* ::SetManualAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a set manual target command with
*   auto reasonable values to the embedded system
*
   [[U ::SetManualAutoReasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void 
SetManualAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
{
     tDlTargetBase Target;


     Target.dPosition =  ((double)(rand() %2) +
                ( (rand() %2) ? 1 : -1) * (0.5) ) * (4*1000000000*256/633);
     Target.dRate = ( (double) (rand() % 21) +
                ( (rand() %2) ? 1 : -1) * (10) ) * (4*1000000*256/633);

     Target.tTime = DlTime[iGuiGizmoIndex];

     setTextForTarget( dl_smt_shell[iGuiGizmoIndex]->dl_smt_pos_text,
                       dl_smt_shell[iGuiGizmoIndex]->dl_smt_rate_text,
                       dl_smt_shell[iGuiGizmoIndex]->dl_smt_time_text,
                       Target);
}

/****************************************************************
*
* ::SetManualAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a set manual target command with
*   auto unreasonable values to the embedded system
*
   [[U ::SetManualAutoUnreasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void 
SetManualAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     tDlTargetBase Target;


     Target.dPosition = (double)rand();
     Target.dRate = (double) rand();
     Target.tTime = (unsigned long)rand() * TIME_TICKS_PER_SECOND;

     setTextForTarget( dl_smt_shell[iGuiGizmoIndex]->dl_smt_pos_text,
                       dl_smt_shell[iGuiGizmoIndex]->dl_smt_rate_text,
                       dl_smt_shell[iGuiGizmoIndex]->dl_smt_time_text,
                       Target);


}

/****************************************************************
*
* ::SetDither(int iGizmoId, int iGuiGizmoIndex, double dAmplitude,
              double dPeriod)
*
* Description: This function will send a set dither command 
*   to the embedded system
*
   [[U ::SetDither(iGizmoId, iGuiGizmoIndex, dAmplitude, dPeriod) ]]
*
*****************************************************************/

void 
SetDither(int iGizmoId, int iGuiGizmoIndex, double dAmplitude, double dPeriod)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"SetDither(<iGuiGizmoIndex=%d>,<dAmplitude=%lf>,
                  <dPeriod=%lf>)\n",iGuiGizmoIndex, dAmplitude, dPeriod);

     data_send.dlsdCmd.Command = DL_CMD_SET_DITHER;
     data_send.dlsdCmd.iGizmoId = iGizmoId;
     data_send.dlsdCmd.dAmplitude =  dAmplitude;
     data_send.dlsdCmd.dPeriod =  dPeriod;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );

     
}

void
SetDitherAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     double dAmplitude;
     double dPeriod;


     dAmplitude = (double)rand();
     dPeriod = (double) rand();

     setTextForSetDither(dl_set_dither_shell[iGuiGizmoIndex],
                         dAmplitude, dPeriod);

}

/****************************************************************
*
* ::DLHomeCheck(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line home check command with
*   to the embedded system
*
   [[U ::DLHomeCheck(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLHomeCheck(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLHomeCheck(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlhcCmd.iGizmoId = iGizmoId;
     data_send.dlhcCmd.Command = DL_CMD_HOME_CHECK;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::DLOff(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line off command 
*   to the embedded system
*
   [[U ::DLOff(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLOff(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLOff(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dloCmd.iGizmoId = iGizmoId;
     data_send.dloCmd.Command = DL_CMD_OFF;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::DLFront(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line front command
*   to the embedded system
*
   [[U ::DLFront(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLFront(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLFront(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlfCmd.iGizmoId = iGizmoId;
     data_send.dlfCmd.Command = DL_CMD_FRONT;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::DLBack(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line back command 
*   to the embedded system
*
   [[U ::DLBack(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLBack(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLBack(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlbCmd.iGizmoId = iGizmoId;
     data_send.dlbCmd.Command = DL_CMD_BACK;

    /****
    [[U ::write_to_log(cmd, strlen(cmd)) ]]
    ****/
    write_to_log(cmd, strlen(cmd) );

    /****
    [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
    ****/
    write_to_error_file(cmd, strlen(cmd) );

    /****
    [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
    ****/
    do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::DLTrack(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line track command 
*   to the embedded system
*
   [[U ::DLTrack(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLTrack(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLTrack(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dltCmd.iGizmoId = iGizmoId;
     data_send.dltCmd.Command = DL_CMD_TRACK;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DlUser1(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line User1 command 
*   to the embedded system
*
   [[U ::DLUser1(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLUser1(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLUser1(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlu1Cmd.iGizmoId = iGizmoId;
     data_send.dlu1Cmd.Command = DL_CMD_USER1;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLUser2(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line User2 command 
*   to the embedded system
*
   [[U ::DLUser2(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLUser2(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLUser2(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlu2Cmd.iGizmoId = iGizmoId;
     data_send.dlu2Cmd.Command = DL_CMD_USER2;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLSetModulation(int iGizmoId, int iGuiGizmoIndex, MOD_TYPE mtModulationType,
                double dAmplitude, double dFrequency, double dMeanVal,
                TIME tSyncTime)
*
* Description: This function will send a delay line set modulation command 
*   to the embedded system
*
   [[U ::DLSetModulation(iGizmoId, iGuiGizmoIndex, mtModulationType,
                         dAmplitude, dFrequency, dMeanVal, tSyncTime) ]]
*
*****************************************************************/

void 
DLSetModulation(int iGizmoId, int iGuiGizmoIndex, MOD_TYPE mtModulationType,
                double dAmplitude, double dFrequency, double dMeanVal,
                TIME tSyncTime)
{
     ISIT_CMD data_send;
     char cmd[256];

     sprintf(cmd,"DLSetModulation(<iGuiGizmoIndex=%d>,<dAmplitude=%lf>,
                 <dFrequency=%lf>,<mtModulationType=%d>, <dMeanVal=%lf>, 
                 <tSyncTime=%lu>)\n",iGuiGizmoIndex, dAmplitude, dFrequency, 
                 mtModulationType, dMeanVal, tSyncTime);

     data_send.dlsmCmd.Command = DL_CMD_MODULATE;
     data_send.dlsmCmd.iGizmoId = iGizmoId;
     data_send.dlsmCmd.dAmplitude =  dAmplitude;
     data_send.dlsmCmd.dFrequency =  dFrequency;
     data_send.dlsmCmd.mtModulationType = mtModulationType; 
     data_send.dlsmCmd.dMeanVal =  dMeanVal;
     data_send.dlsmCmd.tSyncTime =  tSyncTime;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

     /* set the current Modulation - for simulation */
     DLParams[iGuiGizmoIndex].dFrequency = dFrequency; 
     DLParams[iGuiGizmoIndex].dAmplitude = dAmplitude;
     DLParams[iGuiGizmoIndex].dMeanVal = 0.0;
     DLParams[iGuiGizmoIndex].tTime  = 
                  DLCurrentTarget[iGuiGizmoIndex].tTime;

}

void 
DLSetModulationAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
{
     MOD_TYPE mtModulationType;
     double   dFrequency;
     double   dAmplitude;
     double   dMeanVal;
     TIME     tSyncTime;


     dFrequency = rand()  % 10001;
     dAmplitude = (rand()  % 11) * (4*1000*256/633);
     dMeanVal = ( ( (rand() %2) ? 1 : -1) * (10) ) * (4*1000*256/633);
     mtModulationType = (MOD_TYPE) (rand() % 5);
     tSyncTime = DlTime[iGuiGizmoIndex];

     setTextForDlModulation(dl_set_modulation_shell[iGuiGizmoIndex],
                  mtModulationType,
                  dAmplitude,
                  dFrequency,
                  dMeanVal,
                  tSyncTime);

}

void
DLSetModulationAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     MOD_TYPE mtModulationType;
     double   dFrequency;
     double   dAmplitude;
     double   dMeanVal;
     TIME     tSyncTime;


     dFrequency = (double)rand();
     dAmplitude = (double)rand();
     dMeanVal = (double)rand();
     mtModulationType = (MOD_TYPE)(rand() % 5);
     tSyncTime = (unsigned long)rand() * TIME_TICKS_PER_SECOND;

     setTextForDlModulation(dl_set_modulation_shell[iGuiGizmoIndex],
                  mtModulationType,
                  dAmplitude,
                  dFrequency,
                  dMeanVal,
                  tSyncTime);

}


/****************************************************************
*
* ::DLHomeSet(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line home set command 
*   to the embedded system
*
   [[U ::DLHomeSet(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLHomeSet(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLHomeSet(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlhsCmd.Command = DL_CMD_HOME_SET;
     data_send.dlhsCmd.iGizmoId = iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::DLDitherCalibrate(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line dither calibrate command 
*   to the embedded system
*
   [[U ::DLDitherCalibrate(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLDitherCalibrate(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLDitherCalibdRate(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dldcCmd.Command = DL_CMD_DITHER_CALIBRATE;
     data_send.dldcCmd.iGizmoId = iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLDitherCheck(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line dither check command 
*   to the embedded system
*
   [[U ::DLDitherCheck(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLDitherCheck(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLDitherCheck(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dldchkCmd.Command = DL_CMD_DITHER_CHECK;
     data_send.dldchkCmd.iGizmoId = iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLPing(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line ping command 
*   to the embedded system
*
   [[U ::DLPing(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLPing(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLPing(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.dlpCmd.Command = DL_CMD_PING;
     data_send.dlpCmd.iGizmoId = iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLClrLaserCtr(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a delay line clear laser counter command 
*   to the embedded system
*
   [[U ::DLClrLaserCtr(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void DLClrLaserCtr(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"DLClrLaserCtr(<iGuiGizmoIndex=%d>)\n",iGuiGizmoIndex);
     data_send.dlclcCmd.Command = DL_CMD_CLR_LASER_CTR;
     data_send.dlclcCmd.iGizmoId =  iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::DLSetServoParam(int iGizmoId, int iGuiGizmoIndex, 
                    TRACK_SERVO_PARAMS& servoCmd)
*
* Description: This function will send a delay line set servo parameter command 
*   to the embedded system
*
   [[U ::DLSetServoParam(iGizmoId, iGuiGizmoIndex, servoCmd) ]]
*
*****************************************************************/

void
DLSetServoParam(int iGizmoId, int iGuiGizmoIndex, tGuiGsuDlServoParameters& servoCmd)
{

   ISIT_CMD data_send;
   char cmd[BUFSIZ*4];
   char val[BUFSIZ*2];

   memset(val,0,sizeof(val));
   memset(cmd,0,sizeof(cmd));

   strcat(val,"DLSetServoParam(<iGuiGizmoIndex=%d>,\n");

   strcat(val,"\t<bPztServoEnabled=%s>,\n");
   strcat(val,"\t<bPztLagEnabled=%s>,\n");
   strcat(val,"\t<bVc1TrackServoEnabled=%s>,\n");
   strcat(val,"\t<bVc1StabilizeServoEnabled=%s>,\n");
   strcat(val,"\t<bVc2TrackServoEnabled=%s>,\n");
   strcat(val,"\t<bVc2StabilizeServoEnabled=%s>,\n");
   strcat(val,"\t<bMotorServoEnabled=%s>,\n");
   strcat(val,"\t<dPztServoRate=%lf>,\n");
   strcat(val,"\t<dVcServoRate=%lf>,\n");
   strcat(val,"\t<dMotorServoRate=%lf>,\n");
   strcat(val,"\t<dPztGain=%lf>,\n");
   strcat(val,"\t<dPztLagPole=%lf>,\n");
   strcat(val,"\t<dPztLagZero=%lf>,\n");
   strcat(val,"\t<dVc1StabilizePidGain=%lf>,\n");
   strcat(val,"\t<dVc1StabilizePidZero1=%lf>,\n");
   strcat(val,"\t<dVc1StabilizePidZero2=%lf>,\n");
   strcat(val,"\t<dVc1StabilizePidPole=%lf>,\n");
   strcat(val,"\t<dVc1TrackPidGain=%lf>,\n");
   strcat(val,"\t<dVc1TrackPidZero1=%lf>,\n");
   strcat(val,"\t<dVc1TrackPidZero2=%lf>,\n");
   strcat(val,"\t<dVc1TrackPidPole=%lf>,\n");
   strcat(val,"\t<dVc1TrackLaserFeedforwardGain=%lf>,\n");
   strcat(val,"\t<dVc1TrackFilterMemory=%lf>,\n");
   strcat(val,"\t<dVc2StabilizePidGain=%lf>,\n");
   strcat(val,"\t<dVc2StabilizePidZero1=%lf>,\n");
   strcat(val,"\t<dVc2StabilizePidZero2=%lf>,\n");
   strcat(val,"\t<dVc2StabilizePidPole=%lf>,\n");
   strcat(val,"\t<dVc2StabilizeMotorAccelerationFeedforwardGain=%lf>,\n");
   strcat(val,"\t<dVc2TrackPidGain=%lf>,\n");
   strcat(val,"\t<dVc2TrackPidZero1=%lf>,\n");
   strcat(val,"\t<dVc2TrackPidZero2=%lf>,\n");
   strcat(val,"\t<dVc2TrackPidPole=%lf>,\n");
   strcat(val,"\t<dVc2TrackVc1ForceFeedforwardGain=%lf>,\n");
   strcat(val,"\t<dMotorSlewVelocity=%lf>,\n");
   strcat(val,"\t<dMotorCreepVelocity=%lf>,\n");
   strcat(val,"\t<dDecelerationAggressiveness=%lf>,\n");
   strcat(val,"\t<dMotorPidGain=%lf>,\n");
   strcat(val,"\t<dMotorPidZero1=%lf>,\n");
   strcat(val,"\t<dMotorPidZero2=%lf>,\n");
   strcat(val,"\t<dMotorPidPole=%lf>,\n");
   strcat(val,"\t<dMotorFeedForwardGain=%lf>,\n");
   strcat(val,"\t<dDecelToIdleVelocityThreshold=%lf>,\n");
   strcat(val,"\t<dSlewToTrackThresholdInMm=%lf>,\n");
   strcat(val,"\t<dTrackToSlewThresholdInMm=%lf>\n");

   sprintf(cmd, val, iGuiGizmoIndex,

   servoCmd.bPztServoEnabled?"True":"False",
   servoCmd.bPztLagEnabled?"True":"False",
   servoCmd.bVc1TrackServoEnabled?"True":"False",
   servoCmd.bVc1StabilizeServoEnabled?"True":"False",
   servoCmd.bVc2TrackServoEnabled?"True":"False",
   servoCmd.bVc2StabilizeServoEnabled?"True":"False",
   servoCmd.bMotorServoEnabled?"True":"False",
   servoCmd.dPztServoRate,
   servoCmd.dVcServoRate,
   servoCmd.dMotorServoRate,
   servoCmd.dPztGain,
   servoCmd.dPztLagPole,
   servoCmd.dPztLagZero,
   servoCmd.dVc1StabilizePidGain,
   servoCmd.dVc1StabilizePidZero1,
   servoCmd.dVc1StabilizePidZero2,
   servoCmd.dVc1StabilizePidPole,
   servoCmd.dVc1TrackPidGain,
   servoCmd.dVc1TrackPidZero1,
   servoCmd.dVc1TrackPidZero2,
   servoCmd.dVc1TrackPidPole,
   servoCmd.dVc1TrackLaserFeedforwardGain,
   servoCmd.dVc1TrackFilterMemory,
   servoCmd.dVc2StabilizePidGain,
   servoCmd.dVc2StabilizePidZero1,
   servoCmd.dVc2StabilizePidZero2,
   servoCmd.dVc2StabilizePidPole,
   servoCmd.dVc2StabilizeMotorAccelerationFeedforwardGain,
   servoCmd.dVc2TrackPidGain,
   servoCmd.dVc2TrackPidZero1,
   servoCmd.dVc2TrackPidZero2,
   servoCmd.dVc2TrackPidPole,
   servoCmd.dVc2TrackVc1ForceFeedforwardGain,
   servoCmd.dMotorSlewVelocity,
   servoCmd.dMotorCreepVelocity,
   servoCmd.dDecelerationAggressiveness,
   servoCmd.dMotorPidGain,
   servoCmd.dMotorPidZero1,
   servoCmd.dMotorPidZero2,
   servoCmd.dMotorPidPole,
   servoCmd.dMotorFeedForwardGain,
   servoCmd.dDecelToIdleVelocityThreshold,
   servoCmd.dSlewToTrackThresholdInMm,
   servoCmd.dTrackToSlewThresholdInMm
   );

   data_send.gsudlsspCmd.Command = GSUDL_CMD_CHNG_SERVO_PARAM;
   data_send.gsudlsspCmd.ServoParams = servoCmd;
   data_send.gsudlsspCmd.iGizmoId =  iGizmoId;


   /****
   [[U ::write_to_log(cmd, strlen(cmd)) ]]
   ****/
   write_to_log(cmd, strlen(cmd) );

   /****
   [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
   ****/
   write_to_error_file(cmd, strlen(cmd) );

   /****
   [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
   ****/
   do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

#ifdef TBD
void
SetBb1ServoParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex,
                                TRACK_SERVO_PARAMS& servoParam)

{
   servoParam.DlQuicServoParams.dPztGain = 1.0;
   servoParam.DlQuicServoParams.dPztLagPole = 0.25;;
   servoParam.DlQuicServoParams.dPztLagZero = 0.0025;
   servoParam.DlQuicServoParams.dPztServoRate = 5000.0;
   servoParam.DlQuicServoParams.bLagEnabled = (double)TRUE;
   servoParam.DlQuicServoParams.bPztServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dSlewToTrackThresholdInMm = 1.0;
   servoParam.DlQuicServoParams.dTrackToSlewThresholdInMm = 7.5;
                                     
   servoParam.DlQuicServoParams.dVc1TrackPidGain = 0.008;
   servoParam.DlQuicServoParams.dVc1TrackPidZero1 = 0.5;
   servoParam.DlQuicServoParams.dVc1TrackPidZero2 = 0.05;
   servoParam.DlQuicServoParams.dVc1TrackPidPole = 0.005;
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardGain = -1.65;
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardFilterMemory = 1.0;
   servoParam.DlQuicServoParams.dVc1TrackServoRate = 1000.0;
   servoParam.DlQuicServoParams.bVc1TrackServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dVc1SlewPidGain = 0.035;
   servoParam.DlQuicServoParams.dVc1SlewPidZero1  = 0.0;
   servoParam.DlQuicServoParams.dVc1SlewPidZero2 = 0.05;
   servoParam.DlQuicServoParams.dVc1SlewPidPole = 0.005;
   servoParam.DlQuicServoParams.dVc1SlewServoRate = 1000.0;
   servoParam.DlQuicServoParams.bVc1SlewServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dMotorGain = 0.0000975;
   servoParam.DlQuicServoParams.dMotorFeedForwardGain = 1.0;
   servoParam.DlQuicServoParams.dMotorServoRate = 100.0;
   servoParam.DlQuicServoParams.bMotorServoEnabled = (double)TRUE;
                                     
   setTextForQuicServo(dl_servo_shell[iGuiGizmoIndex],
                         servoParam);

}

void
SetQuicServoParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex,
                                TRACK_SERVO_PARAMS& servoParam)
{
   servoParam.DlQuicServoParams.dPztGain = (double)rand();
   servoParam.DlQuicServoParams.dPztLagPole =(double)rand();
   servoParam.DlQuicServoParams.dPztLagZero = (double)rand();
   servoParam.DlQuicServoParams.dPztServoRate =(double)rand();
   servoParam.DlQuicServoParams.bLagEnabled = (double)(rand() %2);
   servoParam.DlQuicServoParams.bPztServoEnabled = (double)(rand() %2);

   servoParam.DlQuicServoParams.dSlewToTrackThresholdInMm =(double)rand();
   servoParam.DlQuicServoParams.dTrackToSlewThresholdInMm =(double)rand();

   servoParam.DlQuicServoParams.dVc1TrackPidGain =(double)rand();
   servoParam.DlQuicServoParams.dVc1TrackPidZero1 =(double)rand();
   servoParam.DlQuicServoParams.dVc1TrackPidZero2 =(double)rand();
   servoParam.DlQuicServoParams.dVc1TrackPidPole =(double)rand();
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardGain =(double)rand();
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardFilterMemory = 
                                                          (double)rand();
   servoParam.DlQuicServoParams.dVc1TrackServoRate = (double)rand();
   servoParam.DlQuicServoParams.bVc1TrackServoEnabled = (double)(rand() %2);

   servoParam.DlQuicServoParams.dVc1SlewPidGain = (double)rand();
   servoParam.DlQuicServoParams.dVc1SlewPidZero1  = (double)rand();
   servoParam.DlQuicServoParams.dVc1SlewPidZero2 = (double)rand();
   servoParam.DlQuicServoParams.dVc1SlewPidPole = (double)rand();
   servoParam.DlQuicServoParams.dVc1SlewServoRate = (double)rand();
   servoParam.DlQuicServoParams.bVc1SlewServoEnabled = (double)(rand() %2);

   servoParam.DlQuicServoParams.dMotorGain = (double)rand();
   servoParam.DlQuicServoParams.dMotorFeedForwardGain = (double)rand();
   servoParam.DlQuicServoParams.dMotorServoRate = (double)rand();
   servoParam.DlQuicServoParams.bMotorServoEnabled = (double)(rand() %2);

   setTextForQuicServo(dl_servo_shell[iGuiGizmoIndex],
                         servoParam);

}
      

void
SetBb2ServoParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex,
                                TRACK_SERVO_PARAMS& servoParam)

{
   servoParam.DlQuicServoParams.dPztGain = 1.0;
   servoParam.DlQuicServoParams.dPztLagPole = 1.0;
   servoParam.DlQuicServoParams.dPztLagZero = 1.0;
   servoParam.DlQuicServoParams.dPztServoRate = 5000.0;
   servoParam.DlQuicServoParams.bLagEnabled = (double)TRUE;
   servoParam.DlQuicServoParams.bPztServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dSlewToTrackThresholdInMm = 1.0;
   servoParam.DlQuicServoParams.dTrackToSlewThresholdInMm = 7.5;
                                     
   servoParam.DlQuicServoParams.dVc1TrackPidGain = 1.0;
   servoParam.DlQuicServoParams.dVc1TrackPidZero1 = 1.0;
   servoParam.DlQuicServoParams.dVc1TrackPidZero2 = 1.0;
   servoParam.DlQuicServoParams.dVc1TrackPidPole = 1.0;
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardGain = 1.0;
   servoParam.DlQuicServoParams.dVc1LaserFeedforwardFilterMemory = 1.0;
   servoParam.DlQuicServoParams.dVc1TrackServoRate = 1000.0;
   servoParam.DlQuicServoParams.bVc1TrackServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dVc1SlewPidGain = 1.0;
   servoParam.DlQuicServoParams.dVc1SlewPidZero1  = 1.0;
   servoParam.DlQuicServoParams.dVc1SlewPidZero2 = 1.0;
   servoParam.DlQuicServoParams.dVc1SlewPidPole = 1.0;
   servoParam.DlQuicServoParams.dVc1SlewServoRate = 1000.0;
   servoParam.DlQuicServoParams.bVc1SlewServoEnabled = (double)TRUE;
                                     
   servoParam.DlQuicServoParams.dMotorGain = 1.0;
   servoParam.DlQuicServoParams.dMotorFeedForwardGain = 1.0;
   servoParam.DlQuicServoParams.dMotorServoRate = 100.0;
   servoParam.DlQuicServoParams.bMotorServoEnabled = (double)TRUE;
                                     
   setTextForQuicServo(dl_servo_shell[iGuiGizmoIndex],
                         servoParam);

}


void
SetGsuServoParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex,
                                tGuiGsuDlServoParameters& servoParam)

{
   servoParam.DlGsuServoParams.dPztGain = 1.0;
   servoParam.DlGsuServoParams.dPztLagPole = 0.25;;
   servoParam.DlGsuServoParams.dPztLagZero = 0.0025;
   servoParam.DlGsuServoParams.dPztServoRate = 5000.0;
   servoParam.DlGsuServoParams.bLagEnabled = (double)TRUE;
   servoParam.DlGsuServoParams.bPztServoEnabled = (double)TRUE;
                                     
   servoParam.DlGsuServoParams.dSlewToTrackThresholdInMm = 1.0;
   servoParam.DlGsuServoParams.dTrackToSlewThresholdInMm = 7.5;
                                     
   servoParam.DlGsuServoParams.dVc1TrackPidGain = 0.008;
   servoParam.DlGsuServoParams.dVc1TrackPidZero1 = 0.5;
   servoParam.DlGsuServoParams.dVc1TrackPidZero2 = 0.05;
   servoParam.DlGsuServoParams.dVc1TrackPidPole = 0.005;
   servoParam.DlGsuServoParams.dVc1LaserFeedforwardGain = -1.65;
   servoParam.DlGsuServoParams.dVc1LaserFeedforwardFilterMemory = 1.0;
   servoParam.DlGsuServoParams.dVc1TrackServoRate = 1000.0;
   servoParam.DlGsuServoParams.bVc1TrackServoEnabled = (double)TRUE;
                                     
   servoParam.DlGsuServoParams.dVc1SlewPidGain = 0.035;
   servoParam.DlGsuServoParams.dVc1SlewPidZero1  = 0.0;
   servoParam.DlGsuServoParams.dVc1SlewPidZero2 = 0.05;
   servoParam.DlGsuServoParams.dVc1SlewPidPole = 0.005;
   servoParam.DlGsuServoParams.dVc1SlewServoRate = 1000.0;
   servoParam.DlGsuServoParams.bVc1SlewServoEnabled = (double)TRUE;
                                     
   servoParam.DlGsuServoParams.dMotorGain = 0.0000975;
   servoParam.DlGsuServoParams.dMotorFeedForwardGain = 1.0;
   servoParam.DlGsuServoParams.dMotorServoRate = 100.0;
   servoParam.DlGsuServoParams.bMotorServoEnabled = (double)TRUE;
                                     
   servoParam.DlGsuServoParams.dVc2TrackPidGain = 20.0;
   servoParam.DlGsuServoParams.dVc2TrackPidZero1 = 0.2;
   servoParam.DlGsuServoParams.dVc2TrackPidZero2 = 0.2;
   servoParam.DlGsuServoParams.dVc2TrackPidPole = 0.01;
   servoParam.DlGsuServoParams.dVc2Vc1ForceFeedforwardGain = 0.0;
   servoParam.DlGsuServoParams.dVc2TrackServoRate = 1000.0;
   servoParam.DlGsuServoParams.bVc2TrackServoEnabled = (double)TRUE;

   servoParam.DlGsuServoParams.dVc2SlewPidGain = 5.0;
   servoParam.DlGsuServoParams.dVc2SlewPidZero1 = 0.0;
   servoParam.DlGsuServoParams.dVc2SlewPidZero2 = 0.25;
   servoParam.DlGsuServoParams.dVc2SlewPidPole = 0.02;
   servoParam.DlGsuServoParams.dVc2SlewMotorAccelerationFeedforwardGain = 5.0;
   servoParam.DlGsuServoParams.dVc2SlewServoRate = 1000.0;
   servoParam.DlGsuServoParams.bVc2SlewServoEnabled= (double)TRUE;

   setTextForGsuServo(dl_gsu_servo_shell[iGuiGizmoIndex],
                         servoParam);

}

void
SetGsuServoParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex,
                                TRACK_SERVO_PARAMS& servoParam)
{
   servoParam.DlGsuServoParams.dPztGain = (double)rand();
   servoParam.DlGsuServoParams.dPztLagPole =(double)rand();
   servoParam.DlGsuServoParams.dPztLagZero = (double)rand();
   servoParam.DlGsuServoParams.dPztServoRate =(double)rand();
   servoParam.DlGsuServoParams.bLagEnabled = (double)(rand() %2);
   servoParam.DlGsuServoParams.bPztServoEnabled = (double)(rand() %2);

   servoParam.DlGsuServoParams.dSlewToTrackThresholdInMm =(double)rand();
   servoParam.DlGsuServoParams.dTrackToSlewThresholdInMm =(double)rand();

   servoParam.DlGsuServoParams.dVc1TrackPidGain =(double)rand();
   servoParam.DlGsuServoParams.dVc1TrackPidZero1 =(double)rand();
   servoParam.DlGsuServoParams.dVc1TrackPidZero2 =(double)rand();
   servoParam.DlGsuServoParams.dVc1TrackPidPole =(double)rand();
   servoParam.DlGsuServoParams.dVc1LaserFeedforwardGain =(double)rand();
   servoParam.DlGsuServoParams.dVc1LaserFeedforwardFilterMemory = 
                                                          (double)rand();
   servoParam.DlGsuServoParams.dVc1TrackServoRate = (double)rand();
   servoParam.DlGsuServoParams.bVc1TrackServoEnabled = (double)(rand() %2);

   servoParam.DlGsuServoParams.dVc1SlewPidGain = (double)rand();
   servoParam.DlGsuServoParams.dVc1SlewPidZero1  = (double)rand();
   servoParam.DlGsuServoParams.dVc1SlewPidZero2 = (double)rand();
   servoParam.DlGsuServoParams.dVc1SlewPidPole = (double)rand();
   servoParam.DlGsuServoParams.dVc1SlewServoRate = (double)rand();
   servoParam.DlGsuServoParams.bVc1SlewServoEnabled = (double)(rand() %2);

   servoParam.DlGsuServoParams.dMotorGain = (double)rand();
   servoParam.DlGsuServoParams.dMotorFeedForwardGain = (double)rand();
   servoParam.DlGsuServoParams.dMotorServoRate = (double)rand();
   servoParam.DlGsuServoParams.bMotorServoEnabled = (double)(rand() %2);

   servoParam.DlGsuServoParams.dVc2TrackPidGain = (double)rand();
   servoParam.DlGsuServoParams.dVc2TrackPidZero1 = (double)rand();
   servoParam.DlGsuServoParams.dVc2TrackPidZero2 = (double)rand();
   servoParam.DlGsuServoParams.dVc2TrackPidPole = (double)rand();
   servoParam.DlGsuServoParams.dVc2Vc1ForceFeedforwardGain = (double)rand();
   servoParam.DlGsuServoParams.dVc2TrackServoRate = (double)rand();
   servoParam.DlGsuServoParams.bVc2TrackServoEnabled = (double)(rand()%2);

   servoParam.DlGsuServoParams.dVc2SlewPidGain = (double)rand();
   servoParam.DlGsuServoParams.dVc2SlewPidZero1 = (double)rand();
   servoParam.DlGsuServoParams.dVc2SlewPidZero2 = (double)rand();
   servoParam.DlGsuServoParams.dVc2SlewPidPole = (double)rand();
   servoParam.DlGsuServoParams.dVc2SlewMotorAccelerationFeedforwardGain
                                                 = (double)rand();
   servoParam.DlGsuServoParams.dVc2SlewServoRate = (double)rand();
   servoParam.DlGsuServoParams.bVc2SlewServoEnabled = (double)(rand()%2);

   setTextForGsuServo(dl_gsu_servo_shell[iGuiGizmoIndex],
                         servoParam);

}
#endif TBD
      
