#include <sys/types.h>
#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientUtils.h"
#include "clientTextUtils.h"
#include "clientEmCb.h"

/****************************************************************
*
* ::EMOff(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a external metrology off command 
*   to the embedded system
*
   [[U ::EMOff(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void EMOff(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"EMOff(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.emoCmd.Command = EM_CMD_OFF;
     data_send.emoCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::EMTrack(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a external metrology track command 
*   to the embedded system
*
   [[U ::EMTrack(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void EMTrack(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"EMTrack(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.emtCmd.Command = EM_CMD_TRACK;
     data_send.emtCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::EMPing(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a external metrology ping command 
*   to the embedded system
*
   [[U ::EMPing(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void EMPing (int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"EMPing(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.empCmd.Command = EM_CMD_PING;
     data_send.empCmd.iGizmoId =  iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}


/****************************************************************
*
* ::EMSetParams(int iGizmoId, int iGuiGizmoIndex
            MOD_TYPE  mtModulationType,
            double    dAmplitude,
            double    dFrequency,
            double    dMeanVal,
            TIME      tSyncTime)
*
* Description: This function will send a external metrology set params command 
*   to the embedded system
*
   [[U ::EMSetParams(iGizmoId, iGuiGizmoIndex, mtModulationType,
                     dAmplitude, dAmplitude, dMeanVal, tSyncTime) ]]
*
*****************************************************************/

void EMSetParams(int iGizmoId, int iGuiGizmoIndex,
            MOD_TYPE  mtModulationType,
            double    dAmplitude,
            double    dFrequency,
            double    dMeanVal,
            TIME      tSyncTime)

{
     ISIT_CMD data_send;
     char cmd[256];

     sprintf(cmd,"EMSetParams(<iGuiGizmoIndex=%d>,<dFrequency=%lf>,
                 <iAmplitude=%lf>,<dMeanVal=%lf>,<mtModulationType=%d>,
                 <tSyncTime=%lu>)\n",iGuiGizmoIndex, dFrequency,dAmplitude,
                 dMeanVal,mtModulationType,tSyncTime);

     data_send.emsspCmd.Command = EM_CMD_CHNG_PARAM;
     data_send.emsspCmd.iGizmoId =  iGizmoId;

     data_send.emsspCmd.dAmplitude = dAmplitude;
     data_send.emsspCmd.dFrequency = dFrequency;
     data_send.emsspCmd.dMeanVal = dMeanVal;
     data_send.emsspCmd.mtModulationType = mtModulationType;
     data_send.emsspCmd.tSyncTime = tSyncTime;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

     /* for simulation */
     EMParams[iGuiGizmoIndex].dAmplitude = dAmplitude;
     EMParams[iGuiGizmoIndex].dFrequency = dFrequency;
     EMParams[iGuiGizmoIndex].dMeanVal = dMeanVal;
     EMParams[iGuiGizmoIndex].tTime = DLCurrentTarget[iGuiGizmoIndex].tTime;

}

/****************************************************************
*
* ::EMSetParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex
            double    dAmplitude,
            double    dFrequency,
            double    dMeanVal,
            MOD_TYPE  mtModulationType,
            TIME      tSyncTime)
*
* Description: This function will send a external metrology set params command
*   to the embedded system
*
   [[U ::EMSetParamsAutoReasonable(iGizmoId, iGuiGizmoIndex, mtModulationType,
                     dAmplitude, dAmplitude, dMeanVal, tSyncTime) ]]
*
*****************************************************************/

void EMSetParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
{
     MOD_TYPE mtModulationType;
     double   dFrequency;
     double   dAmplitude;
     double   dMeanVal;
     TIME     tSyncTime;


     dFrequency = rand()  % 10001;
     dAmplitude = (rand()  % 11) * (4*1000*256/633);
     dMeanVal = ( ( (rand() %2) ? 1 : -1) * (10) ) * (4*1000*256/633);
     mtModulationType = (MOD_TYPE) (rand() % 5);
     tSyncTime = DlTime[iGuiGizmoIndex];

     setTextForEmModulation(em_set_param_shell[iGuiGizmoIndex],
                  mtModulationType,
                  dAmplitude,
                  dFrequency,
                  dMeanVal,
                  tSyncTime);

}

/****************************************************************
*
* ::EMSetParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex
            double    dAmplitude,
            double    dFrequency,
            double    dMeanVal,
            MOD_TYPE  mtModulationType,
            TIME      tSyncTime)
*
* Description: This function will send a external metrology set params command
*   to the embedded system
*
   [[U ::EMSetParamsAutoUnreasonable(iGizmoId, iGuiGizmoIndex, mtModulationType,
                     dAmplitude, dAmplitude, dMeanVal, tSyncTime) ]]
*
*****************************************************************/

void EMSetParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     MOD_TYPE mtModulationType;
     double   dFrequency;
     double   dAmplitude;
     double   dMeanVal;
     TIME     tSyncTime;


     dFrequency = (double)rand();
     dAmplitude = (double)rand();
     dMeanVal = (double)rand();
     mtModulationType = (MOD_TYPE)(rand() % 5);
     tSyncTime = (unsigned long)rand() * TIME_TICKS_PER_SECOND;

     setTextForEmModulation(em_set_param_shell[iGuiGizmoIndex],
                  mtModulationType,
                  dAmplitude,
                  dFrequency,
                  dMeanVal,
                  tSyncTime);

}

