#include <sys/types.h>
#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientUtils.h"
#include "clientTextUtils.h"
#include "clientFtCb.h"

/****************************************************************
*
* ::FTOff(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker off command 
*   to the embedded system
*
   [[U ::FTOff(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTOff(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"FTOff(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.ftoCmd.Command = FT_CMD_OFF;
     data_send.ftoCmd.iGizmoId = iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}


/****************************************************************
*
* ::FTTrack(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker track command 
*   to the embedded system
*
   [[U ::FTTrack(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTTrack(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];
     int dl_no;
     int haveDitherOn = 0;

     memset(cmd,0,256);
     sprintf(cmd,"FTTrack(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.fttCmd.Command = FT_CMD_TRACK;
     data_send.fttCmd.iGizmoId =  iGizmoId;

     for (dl_no=0; dl_no < NUM_DELAY_LINES; dl_no++)
     {
        if (dl_shell[dl_no]->ditherOn)
        {
           haveDitherOn = 1;
           break;
        }
     }

     if (!haveDitherOn)
     {
        if (AskUser( ft_shell[iGuiGizmoIndex]->ft_shell, QUESTION) == NO)
           return;
     }

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::FTPing(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker ping command 
*   to the embedded system
*
   [[U ::FTPing(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTPing(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"FTPing(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.ftpCmd.Command = FT_CMD_PING;
     data_send.ftpCmd.iGizmoId =  iGizmoId;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

void FTSetManualTarget(int iGizmoId, int iGuiGizmoIndex, 
                       FT_TARGET_DIRECTION targetDirection)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"FTSetManualTarget(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.ftsmtCmd.Command = FT_CMD_SET_MT;
     data_send.ftsmtCmd.iGizmoId =  iGizmoId;
     data_send.ftsmtCmd.ftdTargetDirection =  targetDirection;


     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::FTSetParams(int iGizmoId, int iGuiGizmoIndex
            MOD_TYPE  mtModulationType,
            double    dAmplitude,
            double    dFrequency,
            double    dMeanVal,
            TIME      tSyncTime) ]
*
* Description: This function will send a fringe tracker set params command 
*   to the embedded system
*
   [[U ::FTSetParams(iGizmoId, iGuiGizmoIndex, mtModulationType, 
                     dAmplitude, dFrequency, dMeanVal, tSyncTime) ]]
*
*****************************************************************/

void FTSetParams(int iGizmoId, int iGuiGizmoIndex,
            tFtSetServoParametersCmd& FtServoParams)

{
     ISIT_CMD data_send;
     char cmd[BUFSIZ];
     char val[BUFSIZ];

     strcat(val,"FTSetParams(<iGuiGizmoIndex=%d>,<dFrequency=%lf>,\n");
     strcat(val,"\t<iAmplitude=%lf>,<dMeanVal=%lf>,<mtModulationType=%d>,\n");
     strcat(val,"\t<tSyncTime=%lu>)\n");
     sprintf(cmd, val, iGuiGizmoIndex, FtServoParams.dFrequency,
                  FtServoParams.dAmplitude, FtServoParams.dMeanVal,
                  FtServoParams.mtModulationType, FtServoParams.tSyncTime);

     data_send.ftcpCmd = FtServoParams;

     data_send.ftcpCmd.Command = FT_CMD_CHNG_PARAM;
     data_send.ftcpCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


     /* for simulation */
     FTParams[iGuiGizmoIndex].dAmplitude = FtServoParams.dAmplitude;
     FTParams[iGuiGizmoIndex].dFrequency = FtServoParams.dFrequency;
     FTParams[iGuiGizmoIndex].dMeanVal = FtServoParams.dMeanVal;
     FTParams[iGuiGizmoIndex].tTime = DLCurrentTarget[iGuiGizmoIndex].tTime;

}


/****************************************************************
*
* ::FTSetParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker set params command 
*   to the embedded system
*
   [[U ::FTSetParamsAutoReasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTSetParamsAutoReasonable(int iGizmoId, int iGuiGizmoIndex)
{

     FTServoCmd[iGuiGizmoIndex].dFrequency = rand()  % 10001; 
     FTServoCmd[iGuiGizmoIndex].dAmplitude = (rand()  % 11) * (4*1000*256/633);
     FTServoCmd[iGuiGizmoIndex].dMeanVal = 
         ( ( (rand() %2) ? 1 : -1) * (10) ) * (4*1000*256/633);
     FTServoCmd[iGuiGizmoIndex].mtModulationType = (MOD_TYPE) (rand() % 5);
     FTServoCmd[iGuiGizmoIndex].tSyncTime = DlTime[iGuiGizmoIndex];

     setTextForFtModulation(ft_set_param_shell[iGuiGizmoIndex],
                  FTServoCmd[iGuiGizmoIndex]);

}

/****************************************************************
*
* ::FTSetParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker set params command 
*   to the embedded system
*
   [[U ::FTSetParamsAutoUnreasonable(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTSetParamsAutoUnreasonable(int iGizmoId, int iGuiGizmoIndex)
{
     FTServoCmd[iGuiGizmoIndex].dFrequency = (double)rand(); 
     FTServoCmd[iGuiGizmoIndex].dAmplitude = (double)rand();
     FTServoCmd[iGuiGizmoIndex].dMeanVal = (double)rand();
     FTServoCmd[iGuiGizmoIndex].mtModulationType = (MOD_TYPE)(rand() % 5);
     FTServoCmd[iGuiGizmoIndex].tSyncTime = 
       (unsigned long)rand() * TIME_TICKS_PER_SECOND;

     setTextForFtModulation(ft_set_param_shell[iGuiGizmoIndex],
                  FTServoCmd[iGuiGizmoIndex]);

}

/****************************************************************
*
* ::FTGetImg(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a fringe tracker Get Image command 
*   to the embedded system
*
   [[U ::FTGetImg(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void FTGetImg(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"FTGetImg(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.ftgiCmd.Command = FT_CMD_GET_IMAGE;
     data_send.ftgiCmd.iGizmoId = iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );



}

/****************************************************************
*
* ::FTSetROI(int iGizmoId, int iGuiGizmoIndex
              int iStartXSt0, int iStartYSt0,
              int iStartXSt1, int iStartYSt1,
              int iStartXFringeImage, int iStartYFringeImage)
*
* Description: This function will send a fringe tracker set region of interest
*   command to the embedded system
*
   [[U ::FTSetROI(iGizmoId, iGuiGizmoIndex,
              iStartXSt0, iStartYSt0,
              iStartXSt1, iStartYSt1,
              iStartXFringeImage, iStartYFringeImage) ]]
*
*****************************************************************/

void FTSetROI(int iGizmoId, int iGuiGizmoIndex, 
              int iStartXSt0, int iStartYSt0,
              int iStartXSt1, int iStartYSt1,
              int iStartXFringeImage, int iStartYFringeImage)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"FTSetROI(<iGuiGizmoIndex=%d>,<iStartXSt0=%d>,<iStartYSt0=%d>,<iStartXSt1=%d>,<iStartYSt1=%d>,<iStartXFringeImage=%d>,<iStartYFringeImage=%d>)\n", iGuiGizmoIndex,iStartXSt0,iStartYSt0,iStartXSt1,iStartYSt1,iStartXFringeImage,iStartYFringeImage);

/*
     printf("FTSetROI(<iGuiGizmoIndex=%d>,<iStartXSt0=%d>,<iStartYSt0=%d>,<iStartXSt1=%d>,<iStartYSt1=%d>,<iStartXFringeImage=%d>,<iStartYFringeImage=%d>)\n", iGuiGizmoIndex,iStartXSt0,iStartYSt0,iStartXSt1,iStartYSt1,iStartXFringeImage,iStartYFringeImage);
*/
     data_send.ftsroiCmd.Command = FT_CMD_SET_ROI;
     data_send.ftsroiCmd.iGizmoId = iGizmoId;
     data_send.ftsroiCmd.ImageParameters.iStartXSt0 = iStartXSt0;
     data_send.ftsroiCmd.ImageParameters.iStartYSt0 = iStartYSt0;
     data_send.ftsroiCmd.ImageParameters.iStartXSt1 = iStartXSt1;
     data_send.ftsroiCmd.ImageParameters.iStartYSt1 = iStartYSt1;
     data_send.ftsroiCmd.ImageParameters.iStartXFringeImage = 
                                              iStartXFringeImage;
     data_send.ftsroiCmd.ImageParameters.iStartYFringeImage = 
                                              iStartYFringeImage;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

