/* $RCSfile: clientImg.cc,v $ - GUI routine -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gui/client/clientImg.cc,v 1.1.1.1 1998/06/11 21:36:31 chara Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,17Oct97,ky       Initial creation
*/

/*
DESCRIPTION
  GUI code
*/
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#include "isitUnix.h"
#include "share/configShare.h"
#include "clientImg.h"
#include "xvUtils.h"

/****************************************************************
*
* ::draw_figure(Display *d, Window w, GC gc, int curfig, int posX, int posY)
*
* Description: This function will draw a rectangle on the fringe tracker
     get image display with the top left corner of x coordinate posX and
     y coordinate posY.  the width and height of the rectangle are determined 
     from the image type indicator curfig
*
   [[U ::draw_figure(d, w, gc, curfig, posX, posY) ]]
*
*****************************************************************/

void draw_figure(Display *d, Window w, GC gc, int whichImg, int posX, int posY)
{
   switch(whichImg)
   {
      case STAR0_IMAGE: 
         XDrawRectangle(d, w, gc, posX,  FT_WHOLE_HEIGHT-posY, CiStarImageHSize,
                        CiStarImageVSize);
         break;
      case STAR1_IMAGE:
         XDrawRectangle(d, w, gc, posX,  FT_WHOLE_HEIGHT-posY, CiStarImageHSize,
                        CiStarImageVSize);
         break;
      case FT_IMAGE:
         XDrawRectangle(d, w, gc, posX, FT_WHOLE_HEIGHT-posY, CiFringeImageHSize, 
                        CiFringeImageVSize);
         break;
      default:
         break;
   }

}


/* Description: validate the new point by ensuring that the point is
      within the drawing area boundary.  If the new point exceeds the
      boundary, the new points will be adjusted to fit within the 
      drawing area
*/
int validateNewPoint(int& x, int& y, int whichImg)
{

   int rval = 1;

   switch(whichImg)
   {
      case STAR0_IMAGE:
         if ( x < 0 ) 
            x = 0;
         else if ( x > (FT_WHOLE_WIDTH - CiStarImageHSize - 3) )
            x = FT_WHOLE_WIDTH - CiStarImageHSize - 3;

         if ( y > FT_WHOLE_HEIGHT ) 
            y = FT_WHOLE_HEIGHT;
         else if ( y < (CiStarImageVSize + 3) )
            y = CiStarImageVSize + 3;
         break;
      case STAR1_IMAGE:
         if ( x < 0 ) 
            x = 0;
         else if ( x > (FT_WHOLE_WIDTH - CiStarImageHSize - 3) )
            x = FT_WHOLE_WIDTH - CiStarImageHSize - 3;

         if ( y > FT_WHOLE_HEIGHT ) 
            y = FT_WHOLE_HEIGHT;
         else if ( y < (CiStarImageVSize + 3) )
            y = CiStarImageVSize + 3;
         break;
      case FT_IMAGE:
         if ( x < 0 ) 
            x = 0;
         else if ( x > (FT_WHOLE_WIDTH - CiFringeImageHSize - 3) )
            x = FT_WHOLE_WIDTH - CiFringeImageHSize - 3;

         if ( y > FT_WHOLE_HEIGHT ) 
            y = FT_WHOLE_HEIGHT;
         else if ( y < (CiFringeImageVSize + 3) )
            y = CiFringeImageVSize + 3;
         break;
      default :
         rval = -1;
         break;
   }
   return (rval);

}

void setImgPosition(Widget draw_area, GC gc, int whichImg,
                      int& oldPosX, int& oldPosY, int newPosX, int newPosY,
                      Widget& x_text, Widget& y_text ,
                      Widget& x_scale, Widget& y_scale)
{
   char buf[80];
   int newX = newPosX;
   int newY = newPosY;

   /* check for out of bound coordinates in the drawing area */
   if (validateNewPoint(newX, newY, whichImg) == -1)
      return;
   
   memset(buf,0,80);
   draw_figure(XtDisplay(draw_area),
               XtWindow(draw_area),
               gc, whichImg, oldPosX, oldPosY);

   draw_figure(XtDisplay(draw_area),
               XtWindow(draw_area),
               gc, whichImg, newX, newY);

   sprintf(buf,"%d",newX);
   XmTextSetString (x_text, buf);
   sprintf(buf,"%d",newY);
   XmTextSetString (y_text, buf);
   XtVaSetValues (x_scale, XmNvalue, newX, NULL);
   XtVaSetValues (y_scale, XmNvalue, newY, NULL);

   oldPosX = newX;
   oldPosY = newY;

}

void getNSetImgPosition(Widget draw_area, GC gc, int whichImg,
                      int& oldPosX, int& oldPosY, 
                      Widget& x_text, Widget& y_text ,
                      Widget& x_scale, Widget& y_scale)
{
   char *string;
   char buf[80];
   int newX;
   int newY;

   string = XmTextGetString (x_text);
   if ( (string== NULL) || !(strcmp(string, "") ) )
   {
      newX = oldPosX;
   }
   else
   {
      newX = atoi(string);
   }
   XtFree(string);

   string = XmTextGetString (y_text);
   if ( (string== NULL) || !(strcmp(string, "") ) )
   {
      newY = oldPosY;
   }
   else
   {
      newY = atoi(string);
   }
   XtFree(string);

   /* check for out of bound coordinates in the drawing area */
   if (validateNewPoint(newX, newY, whichImg) == -1)
      return;

   draw_figure(XtDisplay(draw_area),
               XtWindow(draw_area),
               gc, whichImg, oldPosX, oldPosY);

   draw_figure(XtDisplay(draw_area),
               XtWindow(draw_area),
               gc, whichImg, newX, newY);

   memset(buf,0,80);
   sprintf(buf,"%d",newX);
   XmTextSetString (x_text, buf);
   sprintf(buf,"%d",newY);
   XmTextSetString (y_text, buf);
   XtVaSetValues (x_scale, XmNvalue, newX, NULL);
   XtVaSetValues (y_scale, XmNvalue, newY, NULL);

   oldPosX = newX;
   oldPosY = newY;

}

void setTextSlide(int x, int y,
                  Widget& x_text, Widget& y_text ,
                  Widget& x_scale, Widget& y_scale)

{
   char buf[80];

   memset(buf,0,80);
   sprintf(buf,"%d",x);
   XmTextSetString (x_text, buf);
   sprintf(buf,"%d",y);
   XmTextSetString (y_text, buf);
   XtVaSetValues (x_scale, XmNvalue, x, NULL);
   XtVaSetValues (y_scale, XmNvalue, y, NULL);

}

