/* $RCSfile: clientInit.cc,v $ - GUI routines -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/gui/client/clientInit.cc,v 1.2 1998/08/17 21:01:03 phil Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,17Oct97,ky       Initial creation
*/

/*
DESCRIPTION
  Initialization code for GUI
*/
#include "commonIncludes.h"

#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"

#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientImg.h"
#include "xvUtils.h"

/****************************************************************
*
* ::dl_shell_init(int dl_no, int iGizmoId, GIZMO_TYPE gtSpecificType)
*
* Description: This function will create and initialize a delay line
*   shell window
*
   [[U ::dl_shell_init(dl_no, iGizmoId, gtSpecificType) ]]
*
*****************************************************************/

void dl_shell_init (int dl_no, int iGizmoId, GIZMO_TYPE gtSpecificType)

{
           char title_str[80];
	   int  i;

           dl_shell[dl_no] = new dl_shell_c;
           dl_shell[dl_no]->create ( ricst_shell->xd_rootwidget(), "dl_shell" );
           dl_shell[dl_no]->dl_num = dl_no;
           dl_shell[dl_no]->iGizmoId = iGizmoId;
           dl_shell[dl_no]->dataRecordOn = 0;
           dl_shell[dl_no]->data_fp = NULL;
           dl_shell[dl_no]->simulationOn = 0;
           dl_shell[dl_no]->simu_fp = NULL;

           dl_shell[dl_no]->targetSelectFT = 0;
           dl_shell[dl_no]->targetSelectEM = 0;
           dl_shell[dl_no]->targetSelectOL = 0;
           dl_shell[dl_no]->targetSelectMT = 0;
           dl_shell[dl_no]->ditherOn = 0;
           if (gtSpecificType == GT_DL_GSU)
           {
              sprintf(title_str,"DL #%d(GSU)",dl_no);
           }
           else if (gtSpecificType == GT_DL_BB1)
           {
              sprintf(title_str,"DL #%d(BB1)",dl_no);
           }
           else if (gtSpecificType == GT_DL_BB2)
           {
              sprintf(title_str,"DL #%d(BB2)",dl_no);
           }

           XtVaSetValues (dl_shell[dl_no]->dl_shell,
               XmNtitle, title_str,
               NULL);

           /***
              [[U ::set_dl_pixmap(dl_shell[dl_no]) ]]
           ***/
           set_dl_pixmap(dl_shell[dl_no]);

           sprintf(title_str,"%d",dl_no);
           dl_bt_ptr[dl_no] = XtVaCreateManagedWidget (title_str,
               xmPushButtonWidgetClass, ricst_shell->dl_menu, NULL);
           XtAddCallback (dl_bt_ptr[dl_no], XmNactivateCallback, popup_dialog,
                      (XtPointer) dl_shell[dl_no]->dl_dialog);


#ifdef TBD
	   /* initialize global servo params for status and cmd */
           for (i=0; i< sizeof(TRACK_SERVO_PARAMS)/sizeof(double); i++)
              DLServoCmd[dl_no].dGsuServoParams[i] = 0.0;
           for (i=0; i< sizeof(TRACK_SERVO_PARAMS)/sizeof(double); i++)
              DLServoStatus[dl_no].dGsuServoParams[i] = 0.0;

#endif TBD
	   /* initialize global target */
           DLCurrentTarget[dl_no].dPosition = 0.0;
           DLCurrentTarget[dl_no].dRate = 0.0;
           DLCurrentTarget[dl_no].tTime = 0;

	   /* initialize global modulatiion */
           DLParams[dl_no].dFrequency = 0.0;
           DLParams[dl_no].dAmplitude = 0.0;
           DLParams[dl_no].dMeanVal = 0.0;
           DLParams[dl_no].tTime = 0;

           DlTime[dl_no] = 0;
}

/****************************************************************
*
* ::dl_gsu_servo_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a GSU delay line
*   servo parameters shell window
*
   [[U ::dl_gsu_servo_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_gsu_servo_shell_init (int dl_no, int iGizmoId)
{
           char title_str[80];

           dl_gsu_servo_shell[dl_no] = new dl_gsu_servo_shell_c;
           dl_gsu_servo_shell[dl_no]->create ( ricst_shell->xd_rootwidget(), "dl_gsu_servo_shell" );
           sprintf(title_str,"DL #%d(GSU) Set Servo Parameters",dl_no);
           XtVaSetValues (dl_gsu_servo_shell[dl_no]->dl_gsu_servo_shell,
               XmNtitle, title_str,
               NULL);

           dl_gsu_servo_shell[dl_no]->dl_num = dl_no;
           dl_gsu_servo_shell[dl_no]->iGizmoId= iGizmoId;

}

/****************************************************************
*
* ::dl_servo_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a delay line
*   servo parameters shell window
*
   [[U ::dl_servo_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_servo_shell_init (int dl_no, int iGizmoId) 
{
           char title_str[80];

           dl_servo_shell[dl_no] = new dl_servo_shell_c;
           dl_servo_shell[dl_no]->create ( ricst_shell->xd_rootwidget(), "dl_servo_shell" );
           sprintf(title_str,"DL #%d(Quic) Set Servo Parameters",dl_no);
           XtVaSetValues (dl_servo_shell[dl_no]->dl_servo_shell,
               XmNtitle, title_str,
               NULL);
           dl_servo_shell[dl_no]->dl_num = dl_no;
           dl_servo_shell[dl_no]->iGizmoId = iGizmoId;

}

/****************************************************************
*
* ::dl_smt_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a delay line
*   set manual target shell window
*
   [[U ::dl_smt_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_smt_shell_init (int dl_no, int iGizmoId)
{
           char title_str[80];

           dl_smt_shell[dl_no] = new dl_smt_shell_c;
           dl_smt_shell[dl_no]->create ( ricst_shell->xd_rootwidget(),
               "dl_smt_shell" );
           sprintf(title_str,"DL #%d Set Manual Target",dl_no);
           XtVaSetValues (dl_smt_shell[dl_no]->dl_smt_shell,
               XmNtitle, title_str,
               NULL);

           dl_smt_shell[dl_no]->dl_num = dl_no;
           dl_smt_shell[dl_no]->iGizmoId = iGizmoId;
}

/****************************************************************
*
* ::dl_solt_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a delay line
*   set open loop target shell window
*
   [[U ::dl_solt_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_solt_shell_init (int dl_no, int iGizmoId)
{
           char title_str[80];

           dl_solt_shell[dl_no] = new dl_solt_shell_c;
           dl_solt_shell[dl_no]->create ( ricst_shell->xd_rootwidget(),
               "dl_solt_shell" );
           sprintf(title_str,"DL #%d Set Open Loop Target",dl_no);
           XtVaSetValues (dl_solt_shell[dl_no]->dl_solt_shell,
               XmNtitle, title_str,
               NULL);
           dl_solt_shell[dl_no]->dl_num = dl_no;
           dl_solt_shell[dl_no]->iGizmoId = iGizmoId;

}

/****************************************************************
*
* ::dl_set_modulation_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a delay line
*   set modulation shell window
*
   [[U ::dl_set_modulation_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_set_modulation_shell_init (int dl_no, int iGizmoId)
{
           char title_str[80];

           dl_set_modulation_shell[dl_no] = new dl_set_modulation_shell_c;
           dl_set_modulation_shell[dl_no]->create (
                ricst_shell->xd_rootwidget(),
               "dl_set_modulation_shell" );
           sprintf(title_str,"DL #%d Set Modulation",dl_no);
           XtVaSetValues (dl_set_modulation_shell[dl_no]->dl_set_modulation_shell,
               XmNtitle, title_str,
               NULL);
           dl_set_modulation_shell[dl_no]->dl_num = dl_no;
           dl_set_modulation_shell[dl_no]->iGizmoId = iGizmoId;
           dl_set_modulation_shell[dl_no]->mod_type = MT_NONE;

}

/****************************************************************
*
* ::dl_set_dither_shell_init (int dl_no, int iGizmoId)
*
* Description: This function will create and initialize a delay line
*   set dither shell window
*
   [[U ::dl_set_dither_shell_init (dl_no, iGizmoId) ]]
*
*****************************************************************/

void dl_set_dither_shell_init (int dl_no, int iGizmoId)
{
           char title_str[80];

           dl_set_dither_shell[dl_no]  = new dl_set_dither_shell_c;
           dl_set_dither_shell[dl_no] ->create ( ricst_shell->xd_rootwidget(),
                  "dl_set_dither_shell" );

           sprintf(title_str,"DL #%d Set Dither",dl_no);
           XtVaSetValues (dl_set_dither_shell[dl_no]->dl_set_dither_shell,
               XmNtitle, title_str,
               NULL);
           dl_set_dither_shell[dl_no]->dl_num = dl_no;
           dl_set_dither_shell[dl_no]->iGizmoId = iGizmoId;

}

/****************************************************************
*
* ::ft_shell_init (int ft_no, int iGizmoId)
*
* Description: This function will create and initialize a fringe
*   tracker shell window
*
   [[U ::ft_shell_init (ft_no, iGizmoId) ]]
*
*****************************************************************/

void ft_shell_init (int ft_no, int iGizmoId)
{
           char title_str[80];
           Display *theDisplay;
           Screen  *theScreen;
           Window   theWindow;


           ft_shell[ft_no] = new ft_shell_c;
           ft_shell[ft_no]->create ( ricst_shell->xd_rootwidget(), "ft_shell" );
           ft_shell[ft_no]->ft_num = ft_no;
           ft_shell[ft_no]->iGizmoId = iGizmoId;
           ft_shell[ft_no]->data_fp = NULL;
           ft_shell[ft_no]->dataRecordOn = 0;
           sprintf(title_str,"FT #%d",ft_no);
           XtVaSetValues (ft_shell[ft_no]->ft_shell,
               XmNtitle, title_str,
               NULL);


           sprintf(title_str,"%d",ft_no);
           ft_bt_ptr[ft_no] = XtVaCreateManagedWidget (title_str,
               xmPushButtonWidgetClass, ricst_shell->ft_menu, NULL);
           XtAddCallback (ft_bt_ptr[ft_no], XmNactivateCallback, popup_dialog,
                      (XtPointer) ft_shell[ft_no]->ft_dialog);

           theDisplay = XtDisplay(ft_shell[ft_no]->ft_fringe_image);
           theScreen = XtScreen(ft_shell[ft_no]->ft_fringe_image);
           theWindow = XtWindow(ft_shell[ft_no]->ft_fringe_image);

           ft_shell[ft_no]->thePixmap =
                   XCreatePixmap (theDisplay,
                                  RootWindowOfScreen( theScreen ),
                                  CiFringeImageHSize,
                                  CiFringeImageVSize,
                                  8 );

   	   XSetForeground ( theDisplay,theGC,
                    BlackPixelOfScreen(theScreen));
    	   XFillRectangle( theDisplay, ft_shell[ft_no]->thePixmap, theGC, 0, 0,
                   CiFringeImageHSize, CiFringeImageVSize);

	   XtVaSetValues (ft_shell[ft_no]->ftCenterFringePix,
              XmNlabelType,    XmPIXMAP,
              XmNlabelPixmap,  off,
              NULL);

           FTParams[ft_no].dFrequency = 0.0;
           FTParams[ft_no].dAmplitude = 0.0;
           FTParams[ft_no].dMeanVal = 0.0;
           FTParams[ft_no].tTime = 0;

}

/****************************************************************
*
* ::ft_get_image_shell_init (int ft_no, int iGizmoId)
*
* Description: This function will create and initialize a fringe
*   tracker get imgage shell window
*
   [[U ::ft_get_image_shell_init (ft_no, iGizmoId) ]]
*
*****************************************************************/

void ft_get_image_shell_init (int ft_no, int iGizmoId)
{
           char title_str[80];
           Display *theDisplay;
           Screen  *theScreen;
           Window   theWindow;
           int          fg, bg;
           XGCValues    xgcv;
           int          i;



           ft_get_image_shell[ft_no] = new ft_get_image_shell_c;
           ft_get_image_shell[ft_no]->create ( ricst_shell->xd_rootwidget(),
               "ft_get_image_shell" );

           sprintf(title_str,"FT #%d Get Image",ft_no);
           XtVaSetValues (ft_get_image_shell[ft_no]->ft_get_image_shell,
               XmNtitle, title_str,
               NULL);
           ft_get_image_shell[ft_no]->ft_num = ft_no;
           ft_get_image_shell[ft_no]->iGizmoId = iGizmoId;

           theDisplay = XtDisplay(ft_get_image_shell[ft_no]->ft_whole_image);
           theScreen = XtScreen(ft_get_image_shell[ft_no]->ft_whole_image);
           theWindow = XtWindow(ft_get_image_shell[ft_no]->ft_whole_image);


           ft_get_image_shell[ft_no]->thePixmap = 
                   XCreatePixmap (theDisplay,
                                  RootWindowOfScreen( theScreen ),
                                  FT_WHOLE_WIDTH,
                                  FT_WHOLE_HEIGHT,
                                  8 );
   	   XSetForeground ( theDisplay,theGC,
                    BlackPixelOfScreen(theScreen));
   	   XFillRectangle( theDisplay, ft_get_image_shell[ft_no]->thePixmap, 
                           theGC, 0, 0, FT_WHOLE_WIDTH, FT_WHOLE_HEIGHT);

           /* Set up a GC with exclusive-or */

           bg = BlackPixelOfScreen(theScreen);
           fg = WhitePixelOfScreen(theScreen);
           xgcv.foreground = fg ^ bg;
           xgcv.background = bg;
           xgcv.function = GXxor;
           ft_get_image_shell[ft_no]->xorGC = 
              XtGetGC(ft_get_image_shell[ft_no]->ft_whole_image,
                      GCForeground | GCBackground | GCFunction, &xgcv);


           for (i=0; i<3; i++)
           {
              ft_get_image_shell[ft_no]->posX[i] = i*4;
              ft_get_image_shell[ft_no]->posY[i] = FT_WHOLE_HEIGHT/2 +  i*4;
           }

	   XmToggleButtonGadgetSetState(
              ft_get_image_shell[ft_no]->ft_gi_star0_tog,TRUE,TRUE);
}

/****************************************************************
*
* ::ft_set_param_shell_init (int ft_no, int iGizmoId)
*
* Description: This function will create and initialize a fringe
*   tracker set parameters shell window
*
   [[U ::ft_set_param_shell_init (ft_no, iGizmoId) ]]
*
*****************************************************************/

void ft_set_param_shell_init (int ft_no, int iGizmoId)
{
           char title_str[80];

           ft_set_param_shell[ft_no] = new ft_set_param_shell_c;
           ft_set_param_shell[ft_no]->create ( ricst_shell->xd_rootwidget(), "ft_set_param_shell" );

           sprintf(title_str,"FT #%d Set Parameters",ft_no);
           XtVaSetValues (ft_set_param_shell[ft_no]->ft_set_param_shell,
               XmNtitle, title_str,
               NULL);
           ft_set_param_shell[ft_no]->ft_num = ft_no;
           ft_set_param_shell[ft_no]->iGizmoId = iGizmoId;
           ft_set_param_shell[ft_no]->mod_type = MT_NONE;
           ft_set_param_shell[ft_no]->search_type = SPIRAL_GEOMETRIC;
}

/****************************************************************
*
* ::em_shell_init (int em_no, int iGizmoId)
*
* Description: This function will create and initialize an external
*   metrology shell window
*
   [[U ::em_shell_init (em_no, iGizmoId) ]]
*
*****************************************************************/

void em_shell_init (int em_no, int iGizmoId)
{
           char title_str[80];

           em_shell[em_no] = new em_shell_c;
           em_shell[em_no]->create ( ricst_shell->xd_rootwidget(), "em_shell" );
           em_shell[em_no]->em_num = em_no;
           em_shell[em_no]->iGizmoId = iGizmoId;
           em_shell[em_no]->dataRecordOn = 0;
           em_shell[em_no]->data_fp = NULL;
           sprintf(title_str,"EM #%d",em_no);
           XtVaSetValues (em_shell[em_no]->em_shell,
               XmNtitle, title_str,
               NULL);


           sprintf(title_str,"%d",em_no);
           em_bt_ptr[em_no] = XtVaCreateManagedWidget (title_str,
               xmPushButtonWidgetClass, ricst_shell->em_menu, NULL);
           XtAddCallback (em_bt_ptr[em_no], XmNactivateCallback, popup_dialog,
                      (XtPointer) em_shell[em_no]->em_dialog);



           EMParams[em_no].dFrequency = 0.0;
           EMParams[em_no].dAmplitude = 0.0;
           EMParams[em_no].dMeanVal = 0.0;
           EMParams[em_no].tTime = 0;

}

/****************************************************************
*
* ::em_set_param_shell_init (int em_no, int iGizmoId)
*
* Description: This function will create and initialize an external
*   metrology set parameters shell window
*
   [[U ::em_set_param_shell_init (em_no, iGizmoId) ]]
*
*****************************************************************/

void em_set_param_shell_init (int em_no, int iGizmoId)
{
           char title_str[80];

           em_set_param_shell[em_no] = new em_set_param_shell_c;
           em_set_param_shell[em_no]->create ( ricst_shell->xd_rootwidget(), "em_set_param_shell" );

           sprintf(title_str,"EM #%d Set Parameters",em_no);
           XtVaSetValues (em_set_param_shell[em_no]->em_set_param_shell,
               XmNtitle, title_str,
               NULL);
           em_set_param_shell[em_no]->em_num = em_no;
           em_set_param_shell[em_no]->iGizmoId = iGizmoId;
           em_set_param_shell[em_no]->mod_type = MT_NONE;
}

/****************************************************************
*
* ::st_shell_init (int st_no, int iGizmoId)

* Description: This function will create and initialize a star
*   tracker shell window
*
   [[U ::st_shell_init (st_no, iGizmoId) ]]
*
*****************************************************************/

void st_shell_init (int st_no, int iGizmoId)
{
           char title_str[80];
           Display *theDisplay;
           Screen  *theScreen;
           Window   theWindow;

           st_shell[st_no] = new st_shell_c;
           st_shell[st_no]->create ( ricst_shell->xd_rootwidget(), "st_shell" );
           st_shell[st_no]->st_num = st_no;
           st_shell[st_no]->iGizmoId = iGizmoId;
           st_shell[st_no]->dataRecordOn = 0;
           st_shell[st_no]->data_fp = NULL;
           sprintf(title_str,"ST #%d",st_no);
           XtVaSetValues (st_shell[st_no]->st_shell,
               XmNtitle, title_str,
               NULL);


           sprintf(title_str,"%d",st_no);
           st_bt_ptr[st_no] = XtVaCreateManagedWidget (title_str,
               xmPushButtonWidgetClass, ricst_shell->st_menu, NULL);
           XtAddCallback (st_bt_ptr[st_no], XmNactivateCallback, popup_dialog,
                      (XtPointer) st_shell[st_no]->st_dialog);

           theDisplay = XtDisplay(st_shell[st_no]->st_star_image);
           theScreen = XtScreen(st_shell[st_no]->st_star_image);
           theWindow = XtWindow(st_shell[st_no]->st_star_image);

           st_shell[st_no]->thePixmap =
                   XCreatePixmap (theDisplay,
                                  RootWindowOfScreen( theScreen ),
                                  CiFringeImageHSize,
                                  CiFringeImageVSize,
                                  8 );

   	   XSetForeground ( theDisplay,theGC,
                    BlackPixelOfScreen(theScreen));
   	   XFillRectangle( theDisplay, st_shell[st_no]->thePixmap, theGC, 0, 0,
                   CiFringeImageHSize, CiFringeImageVSize);

}

void st_calibrate_shell_init(int st_no, int iGizmoId)
{
           char title_str[80];

           st_calibrate_shell[st_no] = new st_calibrate_shell_c;
           st_calibrate_shell[st_no]->create ( ricst_shell->xd_rootwidget(), 
                               "st_calibrate_shell" );

           sprintf(title_str,"ST #%d Calibrate",st_no);
           XtVaSetValues (st_calibrate_shell[st_no]->st_calibrate_shell,
               XmNtitle, title_str,
               NULL);
           st_calibrate_shell[st_no]->st_num = st_no;
           st_calibrate_shell[st_no]->iGizmoId = iGizmoId;
}

void st_set_servo_shell_init(int st_no, int iGizmoId)
{
           char title_str[80];

           st_set_servo_shell[st_no] = new st_set_servo_shell_c;
           st_set_servo_shell[st_no]->create ( ricst_shell->xd_rootwidget(), 
                               "st_set_servo_shell" );

           sprintf(title_str,"ST #%d Set Servo Parameters",st_no);
           XtVaSetValues (st_set_servo_shell[st_no]->st_set_servo_shell,
               XmNtitle, title_str,
               NULL);
           st_set_servo_shell[st_no]->st_num = st_no;
           st_set_servo_shell[st_no]->iGizmoId = iGizmoId;
           XmToggleButtonSetState(st_set_servo_shell[st_no]->lag_enabled_tog,
                                  FALSE, TRUE);
}

/****************************************************************
*
* ::set_dl_pixmap ( dl_shell_p dl_shell_ptr)
*
* Description: This function will create the delay line pixmap 
*    and set the default pixmap values 
*
   [[U ::set_dl_pixmap(dl_shell_ptr) ]]
*
*****************************************************************/

void set_dl_pixmap( dl_shell_p dl_shell_ptr)
{

   Pixel  fg, bg;

   XtVaGetValues (dl_shell_ptr->dl_dialog,
        XmNforeground, &fg,
        XmNbackground, &bg,
        NULL);


   on = XmGetPixmap (XtScreen (dl_shell_ptr->dl_dialog), ON_XBM, fg, bg);
   off = XmGetPixmap (XtScreen (dl_shell_ptr->dl_dialog), OFF_XBM, fg, bg);
   delay = XmGetPixmap (XtScreen (dl_shell_ptr->dl_dialog), DELAY_XBM, fg, bg);
   home = XmGetPixmap (XtScreen (dl_shell_ptr->dl_dialog), HOME_XBM, fg, bg);
   if (on == XmUNSPECIFIED_PIXMAP ||
       off == XmUNSPECIFIED_PIXMAP ||
       home == XmUNSPECIFIED_PIXMAP ||
       delay == XmUNSPECIFIED_PIXMAP) 
   {
       puts ("Couldn't load pixmaps");
       exit (1);
   }

   XtVaSetValues (dl_shell_ptr->FHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

   XtVaSetValues (dl_shell_ptr->FSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

   XtVaSetValues (dl_shell_ptr->HomePix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  delay,
           NULL);

   XtVaSetValues (dl_shell_ptr->BHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

   XtVaSetValues (dl_shell_ptr->BSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

}

/****************************************************************
*
* ::popup_dialog(Widget widget, XtPointer client_data, XtPointer call_data)
*
* Description: This function will popup a dialog 
*
   [[U ::popup_dialog(widget, client_data, call_data) ]]
*
*****************************************************************/

void popup_dialog(Widget widget, XtPointer client_data, XtPointer call_data)

{
    Widget dialog = (Widget)client_data;
    if (dialog)
    {
       XtManageChild(dialog);
       XMapRaised (XtDisplay (dialog), XtWindow (XtParent (dialog)));
    }
}

/****************************************************************
*
* ::show_dialog(Widget dialog)
*
* Description: This function will display a dialog to the screen
*
   [[U ::show_dialog(dialog) ]]
*
*****************************************************************/

void show_dialog(Widget dialog)
{
    if (dialog)
    {
       XtManageChild(dialog);
       XMapRaised (XtDisplay (dialog), XtWindow (XtParent (dialog)));
    }
}

/****************************************************************
*
* ::hide_dialog(Widget dialog)
*
* Description: This function will close a displayed dialog 
*
   [[U ::hide_dialog(dialog) ]]
*
*****************************************************************/

void hide_dialog(Widget dialog)
{
    if (dialog)
       XtUnmanageChild (dialog);
}

/****************************************************************
*
* ::GizmoIntoGuiIndex( GizmoIntoGuiIndex(GIZMO_TYPE gtGenericType, 
                       int iGuiIndex, GIZMO_TYPE gtSpecificType, int iGizmoId) 
*
* Description: This function will map a given iGizmoId to its corresponding
*    GUI index and gizmo type
   [[U ::GizmoIntoGuiIndex(gtGenericType, iGuiIndex, gtSpecificType
                           iGizmoId) ]]
*
*****************************************************************/

void GizmoIntoGuiIndex(GIZMO_TYPE gtGenericType, int iGuiIndex,
                                GIZMO_TYPE gtSpecificType, int iGizmoId) 
{
   GizmoMap[iGizmoId].gtGenericType = gtGenericType;
   GizmoMap[iGizmoId].gtSpecificType = gtSpecificType;
   GizmoMap[iGizmoId].iGuiIndex = iGuiIndex;

   /* comment out debug statements
   printf("GizmoMap[%d].gtGenericType = %d\n",iGizmoId,
            GizmoMap[iGizmoId].gtGenericType);
   printf("GizmoMap[%d].gtSpecificType = %d\n", iGizmoId,
            GizmoMap[iGizmoId].gtSpecificType);
   printf("GizmoMap[%d].iGuiIndex = %d\n",iGizmoId,
            GizmoMap[iGizmoId].iGuiIndex);

   */
}

