#include "clientNet.h"
#include "isitUnix.h"
#include "isitCommand.h"
#include "clientClass.h"
#include "client.h"

Boolean CheckForInterrupt();


/****************************************************************
*
* tClientNet::tClientNet()
*
* Description: This is constructor method for tClientNet 
*
   [[U tClientNet::tClientNet() ]]
*
*****************************************************************/

tClientNet::tClientNet()
{
   this->bInit = FALSE;
   this->iPortInUse = SERVER_PORT_NUM;
}

/****************************************************************
*
* tClientNet::~tClientNet()
*
* Description: This is destructor method for tClientNet 
*
   [[U tClientNet::~tClientNet() ]]
*
*****************************************************************/

tClientNet::~tClientNet()
{
   close(this->iSocketInUse);
}

/****************************************************************
*
* tClientNet::makeConnection(char *hostname)
*
* Description: This method creates and establishes connection over
     the network with socket
*
   [[U tClientNet::makeConnection(hostname) ]]
*
*****************************************************************/

int tClientNet::makeConnection(char *hostname)
{

   struct sockaddr_in server;
   struct hostent *hp;
   int flag;
   int buff_size;
   int incr=0;
   int sendbuff, optlen;

   if (hostname == NULL)
      return (FAILED);

   strcpy(this->sHostName, hostname);
   hp = gethostbyname(this->sHostName);
   if (hp == 0)
   {
      fprintf(stderr, "%s: unknown host", sHostName);
      return(FAILED);
   }

   this->iPortInUse = SERVER_PORT_NUM;
   buff_size = 16384; 

   flag = 1;

   do {

      if (networkTimeOut)
      {
         return(TIMEOUT);
      }

      this->iSocketInUse = socket(AF_INET, SOCK_STREAM, 0);
      if (this->iSocketInUse < 0)
      {
         perror("opening stream socket");
         return(FAILED);
      }

      /* set socket option send and receive buffer size to 16384 */
      if (setsockopt(this->iSocketInUse, SOL_SOCKET, SO_SNDBUF,
                     (char *) &buff_size, sizeof(buff_size)) < 0)
                perror("SO_SNDBUF setsockopt error");

      if (setsockopt(this->iSocketInUse, SOL_SOCKET, SO_RCVBUF, 
                     (char *) &buff_size, sizeof(buff_size)) < 0)
                perror("SO_RCVBUF setsockopt error");

      server.sin_family = AF_INET;

      strncpy( (char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length );

      server.sin_port = htons(this->iPortInUse);

      if (connect(this->iSocketInUse, (struct sockaddr *)&server, 
                  sizeof (server) ) < 0)
      {
         if (errno == ECONNREFUSED)
         {
            close(this->iSocketInUse);

            if ( (incr % 200) == 0)
            {
               incr = 0;
               if (CheckForInterrupt ()) 
               {
                  return(STOPPED);
               }

            }

            if (this->iPortInUse > MAXIMUM_PORT_NUM)
            {
               // printf("port is %d\n",this->iPortInUse);
               this->iPortInUse = SERVER_PORT_NUM;
            }
            else
            {
               this->iPortInUse++;
               incr++;
            }
            continue;
         }
         else
            return (FAILED);

      }
      else
         flag = 0;


   } while (flag);

   // printf("port is %d\n",this->iPortInUse);
   this->bInit = TRUE;
   return(SUCCESS);
}

/****************************************************************
*
* tClientNet::terminateConnection(void)
*
* Description: This method closes down network connection 
*
   [[U tClientNet::terminateConnection() ]]
*
*****************************************************************/

int tClientNet::terminateConnection(void)
{
   this->bInit = FALSE;
   return (close(this->iSocketInUse));
}
/****************************************************************
*
* tClientNet::setInit(BOOL bInitFlag)
*
* Description: This method set connection flag 
*
   [[U tClientNet::setInit(bInitFlag) ]]
*
*****************************************************************/

void tClientNet::setInit(BOOL bInitFlag)
{
   this->bInit = bInitFlag;
}

/****************************************************************
*
*  tClientNet::readn(char *ptr,size_t size)
*
* Description: This method read over the network into buffer ptr
     with size bytes
*
   [[U tClientNet::readn(ptr, size) ]]
*
*****************************************************************/

int tClientNet::readn(char *ptr,size_t size)
{   
   int no_left,no_read;

   no_left = size;
   while (no_left > 0)
   {
      no_read = read(this->iSocketInUse,ptr,no_left);
      if(no_read <0)
         return(no_read);
      if (no_read == 0)
         break;
      no_left -= no_read;
      ptr += no_read;
   }
   return(size - no_left);
}

/****************************************************************
*
*  tClientNet::readOnce(char *ptr,size_t size)
*
* Description: This method read over the network into buffer ptr
     with size bytes
*
   [[U tClientNet::read(ptr, size) ]]
*
*****************************************************************/

int tClientNet::readOnce(char *ptr,size_t size)
{  
   int no_read;

   no_read = read(this->iSocketInUse,ptr,size);
   return(no_read);
}

/****************************************************************
*
*  tClientNet::writen(char *ptr,size_t size)
*
* Description: This method write buffer ptr with size bytes over
     the network
*
   [[U tClientNet::writen(ptr, size) ]]
*
*****************************************************************/

int tClientNet::writen(char *ptr,size_t  size)
{        
   int no_left,no_written;

   if (this->bInit == FALSE)
       return (0);

   no_left = size;
   while (no_left > 0)
   {
      no_written = write(this->iSocketInUse,ptr,no_left);
      if(no_written <=0)
         return(no_written);
      no_left -= no_written;
      ptr += no_written;
   }
   return(size - no_left);
}
