#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>


#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "clientUtils.h"

#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "typeTag.h"
#include "clientReadInput.h"
#include "errLookup.h"
#include "winutil.h"
#include "clientStatus.h"
#include "clientData.h"

/****************************************************************
*
* ::readInput(XtPointer client_data, int *fid, XtInputId *id)
*
* Description: This function will read information from the embedded
     system over the network.  The type of read information is determined
     and processed accordingly
*
   [[U ::readInput(client_data, fid, id) ]]
*
*****************************************************************/
// BOOL read_header = TRUE;

void readInput(XtPointer client_data, int *fid, XtInputId *id)
{
   tTypeTag header;
   int nbytes;

   static size_t szBlockSize = 0;
   static STREAM_TYPE stType = ST_UNKNOWN;
   static TAG_NAME tnName = TN_UNKNOWN;
   static int iGizmoId = -1;
   static BOOL read_header = TRUE;

   
   if (read_header)
   {

      if ((nbytes = clientReadn((char *)&header, sizeof(tTypeTag) )) < 0)
      {
         return;
      }
     

      else if (nbytes == 0)
      {
         szBlockSize = 0;
         stType = ST_UNKNOWN;
         tnName = TN_UNKNOWN;
         iGizmoId = -1;
      }
      else if (nbytes > 0)
      {
         szBlockSize = header.Size();
         stType = header.StreamType();
         tnName = header.TagName();
         iGizmoId = header.GizmoId();
         read_header = FALSE;
         // printf("szBlockSize = %d\n", szBlockSize);
         // printf("stType = %d\n", stType);
         // printf("tnName = %d\n", tnName);
         // printf("iGizmoId = %d\n", iGizmoId);
      }
   } /* if (read_header) */

   read_header = TRUE;
   switch (stType)
   {
      case ST_ERROR:
         if (tnName == TN_ERROR_INFO)
         {
            processError(szBlockSize);
         }
         break;

      case ST_DATA:
         switch (tnName)
         {
            case TN_DL_DATA:
               /* printf("dl_data\n"); */
               processDlData(iGizmoId, szBlockSize);
               break;

            case TN_FT_DATA:
               /* printf("ft_data\n"); */
               processFtData(iGizmoId, szBlockSize);
               break;

            case TN_EM_DATA:
               /* printf("em_data\n"); */
               processEmData(iGizmoId, szBlockSize);
               break;
            default:
               break;
         } /* switch (tnName) */
         break;

      case ST_STATUS:
         switch (tnName)
         {
            case TN_GSU_DL_STATUS:
               processDlStatus(iGizmoId, szBlockSize);
               break;

            case TN_FT_STATUS:
               processFtStatus(iGizmoId, szBlockSize);
               break;

            case TN_FT_IMAGE:
               processFtImageStatus(iGizmoId, szBlockSize, read_header);
               break;

            case TN_FT_FULL_IMAGE:
               processFtFullImageStatus(iGizmoId, szBlockSize, read_header);
               break;

            case TN_EM_STATUS:
               processEmStatus(iGizmoId, szBlockSize);
               break;

            case TN_ST_STATUS:
               processStStatus(iGizmoId, szBlockSize);
               break;

            case TN_ST_IMAGE:
               processStImageStatus(iGizmoId, szBlockSize);
               break;

            case TN_ST_STAR_POSITION:
               processStStarPositionStatus(iGizmoId, szBlockSize);
               break;
            case TN_UNKNOWN:
               break;
            default:
               break;
         } /* switch (tnName) */
         break;
      case ST_UNKNOWN:
         break;
      default:
         break;
   } /* switch (stType) */

   XmUpdateDisplay(ricst_shell->xd_rootwidget());

}

/****************************************************************
*
* :: processError(size_t szBlockSize)
*
* Description: This function will read error information from the embedded
     system over the network.  The error information is then written
     to the log message panel and to disk when write to disk option is on
*
   [[U ::processError(szBlockSize) ]]
*
*****************************************************************/

void   processError(size_t szBlockSize)
{
	char buf[BUFSIZ];
        int nbytes;
        int i;
        ERROR_INFO error_code;
        char *str;
        char tmp[80];

        memset(buf,0,BUFSIZ);
        memset(tmp,0,sizeof(tmp));

        if ((nbytes = clientReadn((char *)&error_code, szBlockSize )) < 0)
        {
                perror("read_error_input");
                return;
        }
        else if (nbytes)
        {

           formatTimeToHMS(error_code.tTimeStamp, TIME_TICKS_PER_SECOND,
                            tmp, "");
           str = errLookup( error_code.ecErrCode);
           sprintf(buf,"%s:%s:[0x%x]:<Rte=%d>:<Cpu=%d>\n",tmp,str,
                           error_code.ecErrCode,
                           error_code.iWhichRte, error_code.iWhichCpu);
           if (write_interface_count > read_interface_count)
           {
              write_to_interface( buf,strlen(buf) );
              read_interface_count++;
           }
           write_to_error_file(buf,strlen(buf) );
           write_to_log(buf, strlen(buf) );
        }

}

