#include <sys/types.h>
#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientUtils.h"
#include "clientStCb.h"

/****************************************************************
*
* ::STOff(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a star tracker off command 
*   to the embedded system
*
   [[U ::STOff(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void STOff(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"STOff(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.stoCmd.Command = ST_CMD_OFF;
     data_send.stoCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

/****************************************************************
*
* ::STTrack(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a star tracker track command 
*   to the embedded system
*
   [[U ::STTrack(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void STTrack(int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"STTrack(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.sttCmd.Command = ST_CMD_TRACK;
     data_send.sttCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

/****************************************************************
*
* ::STPing(int iGizmoId, int iGuiGizmoIndex)
*
* Description: This function will send a star tracker ping command 
*   to the embedded system
*
   [[U ::STPing(iGizmoId, iGuiGizmoIndex) ]]
*
*****************************************************************/

void STPing (int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"STPing(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.stpCmd.Command = ST_CMD_PING;
     data_send.stpCmd.iGizmoId =  iGizmoId;

     /****
     [[U ::write_to_log(cmd, strlen(cmd)) ]]
     ****/
     write_to_log(cmd, strlen(cmd) );

     /****
     [[U ::write_to_error_file(cmd, strlen(cmd)) ]]
     ****/
     write_to_error_file(cmd, strlen(cmd) );

     /****
     [[U ::do_command(&data_send, sizeof(ISIT_CMD)) ]]
     ****/
     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

void STHome (int iGizmoId, int iGuiGizmoIndex)
{
     ISIT_CMD data_send;
     char cmd[256];

     memset(cmd,0,256);
     sprintf(cmd,"STHome(<iGuiGizmoIndex=%d>)\n", iGuiGizmoIndex);

     data_send.sthCmd.Command = ST_CMD_HOME;
     data_send.sthCmd.iGizmoId =  iGizmoId;

     write_to_log(cmd, strlen(cmd) );

     write_to_error_file(cmd, strlen(cmd) );

     do_command((char *)&data_send, sizeof(ISIT_CMD) );

}

void STCalibrate(int iGizmoId, int iGuiGizmoIndex,
            tStCalibrateCmd& StCalibrate)

{
     ISIT_CMD data_send;
     char cmd[BUFSIZ];
     char val[BUFSIZ];

     strcat(val,"STCalibrate(<iGuiGizmoIndex=%d>,<CalFrequency=%lf>,\n");
     strcat(val,"\t<CalAmplitude=%lf>,<CalActuator=%d>,\n");
     sprintf(cmd, val, iGuiGizmoIndex, StCalibrate.CalFrequency,
                  StCalibrate.CalAmplitude, 
                  StCalibrate.CalActuator);

     data_send.stcCmd = StCalibrate;

     data_send.stcCmd.Command = ST_CMD_CALIBRATE;
     data_send.stcCmd.iGizmoId =  iGizmoId;

     write_to_log(cmd, strlen(cmd) );

     write_to_error_file(cmd, strlen(cmd) );

     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

void STSetParams(int iGizmoId, int iGuiGizmoIndex,
            tStSetServoParamsCmd& StServoParams)

{
     ISIT_CMD data_send;
     char cmd[BUFSIZ];
     char val[BUFSIZ];

     strcat(val,"STSetParams(<iGuiGizmoIndex=%d>,<StarTarget[0]=%lf>,");
     strcat(val,"<StarTarget[1]=%lf>,\n\t<LagTauPole=%lf>,<LagTauZero=%lf>,\n");
     strcat(val,"\t<LagEnabled=%s>,<PidTauZero1=%lf>,<PidTauZero2=%lf>,\n");
     strcat(val,"\t<PidTauPole=%lf>,<Gain=%lf>,\n");
     strcat(val,"\t<XformDetToFsm[0][0]=%lf>,<XformDetToFsm[0][1]=%lf>,");
     strcat(val,"<XformDetToFsm[0][2]=%lf>,\n\t<XformDetToFsm[1][0]=%lf>,");
     strcat(val,"<XformDetToFsm[1][1]=%lf>,<XformDetToFsm[1][2]=%lf>,\n");
     strcat(val,"\t<FsmHome[0]=%lf>,<FsmHome[1]=%lf>,<FsmHome[2]=%lf>)\n");
     sprintf(cmd, val, iGuiGizmoIndex, 
             StServoParams.Params.ControllerParams.StarTarget[0],
             StServoParams.Params.ControllerParams.StarTarget[1],
             StServoParams.Params.ControllerParams.LagTauPole,
             StServoParams.Params.ControllerParams.LagTauZero,
             StServoParams.Params.ControllerParams.LagEnabled?"True":"False",
             StServoParams.Params.ControllerParams.PidTauZero1,
             StServoParams.Params.ControllerParams.PidTauZero2,
             StServoParams.Params.ControllerParams.PidTauPole,
             StServoParams.Params.ControllerParams.Gain,
             StServoParams.Params.XformDetToFsm[0][0],
             StServoParams.Params.XformDetToFsm[0][1],
             StServoParams.Params.XformDetToFsm[0][2],
             StServoParams.Params.XformDetToFsm[1][0],
             StServoParams.Params.XformDetToFsm[1][1],
             StServoParams.Params.XformDetToFsm[1][2],
             StServoParams.Params.FsmHome[0],
             StServoParams.Params.FsmHome[1],
             StServoParams.Params.FsmHome[2]
             );

     data_send.stsCmd = StServoParams;

     data_send.stsCmd.Command = ST_CMD_SETSS;
     data_send.stsCmd.iGizmoId =  iGizmoId;

     write_to_log(cmd, strlen(cmd) );

     write_to_error_file(cmd, strlen(cmd) );

     do_command((char *)&data_send, sizeof(ISIT_CMD) );


}

