#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>

#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "share/configShare.h"
#include "client.h"
#include "clientInit.h"
#include "clientUtils.h"
#include "winutil.h"
#include "typeTag.h"
#include "clientStatus.h"
#include "clientTextUtils.h"
#include "clientImg.h"
#include "xvUtils.h"

/****************************************************************
*
* ::process_dl_status(dl_shell_p dlShellPtr,int iGizmoId,tDlStatus status_data)
*
* Description: This function will update status information for dl display
*
   [[U ::process_dl_status(dlShellPtr, iGizmoId, status_data) ]]
*
*****************************************************************/

void process_dl_status(dl_shell_p dlShellPtr,int iGizmoId, tGsuDlStatus status_data)
{
    XmString string;
    char buf[BUFSIZ];
    char timeb[80];
    int dl_no = GizmoMap[iGizmoId].iGuiIndex;

    DlTime[dl_no] = status_data.tTime;

    formatTimeToHMS(status_data.tTime, TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (dlShellPtr->lTime,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.laLaser, FIELD_WIDTH, 0, 
                        buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lLaser,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.pdPztPos, FIELD_WIDTH, DEC_PLACES, 
                        buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lPztPos,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    if (status_data.bLimitFrontSoft == TRUE)
       XtVaSetValues (dlShellPtr->FSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  on,
           NULL);
    else
       XtVaSetValues (dlShellPtr->FSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

    if (status_data.bLimitFrontHard == TRUE)
       XtVaSetValues (dlShellPtr->FHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  on,
           NULL);
    else
       XtVaSetValues (dlShellPtr->FHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

    if (status_data.bLimitBackSoft == TRUE)
       XtVaSetValues (dlShellPtr->BSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  on,
           NULL);
    else
       XtVaSetValues (dlShellPtr->BSPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

    if (status_data.bLimitBackHard == TRUE)
       XtVaSetValues (dlShellPtr->BHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  on,
           NULL);
    else
       XtVaSetValues (dlShellPtr->BHPix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

    if (status_data.bOnHomeSensor == TRUE)
       XtVaSetValues (dlShellPtr->HomePix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  home,
           NULL);
    else
       XtVaSetValues (dlShellPtr->HomePix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  delay,
           NULL);


    memset(buf,0,BUFSIZ);
    switch (status_data.SensorModulation.mtModulationType)
    {

       case MT_SQUARE:
          strcpy (buf, "SQUARE");
	  break;
       case MT_TRIANGLE:
          strcpy (buf, "TRIANGLE");
	  break;
       case MT_SINE:
          strcpy (buf, "SINE");
	  break;
       case MT_RANDOM:
          strcpy (buf, "RANDOM");
	  break;
       case MT_NONE:
          strcpy (buf, "NONE");
	  break;
       default:
          strcpy (buf, "UNKNOWN");
	  break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lModType,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);


    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.SensorModulation.dFrequency, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lModPer,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.SensorModulation.dAmplitude, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lModAmpl,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    /* set dl modulation window with new values coming from dl status */
    DLModParams[dl_no] = status_data.SensorModulation;

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.pvPztVelocity, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lPztVel,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.dDitherAmplitude,
                        FIELD_WIDTH, DEC_PLACES, buf, "");

    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lDAmplitude,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.dDitherPhase,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lDPhase,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.dDitherDistortion,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lDDistortion,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.vvVc1, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lVc1,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    if (GizmoMap[iGizmoId].gtSpecificType == GT_DL_GSU)
    {
       memset(buf,0,BUFSIZ);
       formatDblWithCommas(status_data.vvVc2,
                           FIELD_WIDTH, DEC_PLACES, buf, "");
       string = XmStringCreateLocalized (buf);
       XtVaSetValues (dlShellPtr->lVc2,
           XmNlabelString,    string,
           NULL);
       XmStringFree (string);
   
       memset(buf,0,BUFSIZ);
       formatDblWithCommas(status_data.evEddy1,
                           FIELD_WIDTH, DEC_PLACES, buf, "");
       string = XmStringCreateLocalized (buf);
       XtVaSetValues (dlShellPtr->lEddy1,
           XmNlabelString,    string,
           NULL);
       XmStringFree (string);
   
       memset(buf,0,BUFSIZ);
       formatDblWithCommas(status_data.evEddy2,
                           FIELD_WIDTH, DEC_PLACES, buf, "");
       string = XmStringCreateLocalized (buf);
       XtVaSetValues (dlShellPtr->lEddy2,
           XmNlabelString,    string,
           NULL);
       XmStringFree (string);

    }
    else
    {
       memset(buf,0,BUFSIZ);
       string = XmStringCreateLocalized ("None");
       XtVaSetValues (dlShellPtr->lVc2,
           XmNlabelString,    string,
           NULL);

       XtVaSetValues (dlShellPtr->lEddy1,
           XmNlabelString,    string,
           NULL);

       XtVaSetValues (dlShellPtr->lEddy2,
           XmNlabelString,    string,
           NULL);
       XmStringFree (string);

    }

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.mvMotorVelocity, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");

    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lMotorVel,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    DLServoStatus[dl_no] = status_data.ServoParameters;
 
    memset(buf,0,BUFSIZ);
    switch (status_data.dsmServoMode)
    {
       case DSM_OFF:
          strcpy (buf, "OFF");
          break;
       case DSM_FRONT:
          strcpy (buf, "FRONT");
          break;
       case DSM_BACK:
          strcpy (buf, "BACK");
          break;
       case DSM_TRACK:
          strcpy (buf, "TRACK");
          break;
       case DSM_CREEPFRONT:
          strcpy (buf, "CREEPFRONT");
          break;
       case DSM_CREEPBACK:
          strcpy (buf, "CREEPBACK");
          break;
       case DSM_USER1:
          strcpy (buf, "USER1");
          break;
       case DSM_USER2:
          strcpy (buf, "USER2");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lServoState,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);


    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.laTarget, 
                        FIELD_WIDTH, 0, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lTarget,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);


    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.laError, 
                        FIELD_WIDTH, 0, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lError,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.dJitter, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lJitter,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.dHighPassJitter, 
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lHPJitter,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatIntWithCommas(status_data.iPztClip, INT_FIELD_WIDTH, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lPztClip,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.DlCommandStatus)
    {
       case DLCS_SYSTEM_READY:
          strcpy (buf, "SYSTEM_READY");
          break;
       case DLCS_OFF_START:
          strcpy (buf, "OFF_START");
          break;
       case DLCS_OFF_DONE:
          strcpy (buf, "OFF_DONE");
          break;
       case DLCS_FRONT_START:
          strcpy (buf, "FRONT_START");
          break;
       case DLCS_FRONT_MOVING:
          strcpy (buf, "FRONT_MOVING");
          break;
       case DLCS_FRONT_DONE:
          strcpy (buf, "FRONT_DONE");
          break;
       case DLCS_FRONT_ABORTED:
          strcpy (buf, "FRONT_ABORTED");
          break;
       case DLCS_BACK_START:
          strcpy (buf, "BACK_START");
          break;
       case DLCS_BACK_MOVING:
          strcpy (buf, "BACK_MOVING");
          break;
       case DLCS_BACK_DONE:
          strcpy (buf, "BACK_DONE");
          break;
       case DLCS_BACK_ABORTED:
          strcpy (buf, "BACK_ABORTED");
          break;
       case DLCS_TRACK_START:
          strcpy (buf, "TRACK_START");
          break;
       case DLCS_TRACK_ACTIVE:
          strcpy (buf, "TRACK_ACTIVE");
          break;
       case DLCS_TRACK_ABORTED:
          strcpy (buf, "TRACK_ABORTED");
          break;
       case DLCS_CLEAR_LASER_COUNTER_START:
          strcpy (buf, "CLC_START");
          break;
       case DLCS_CLEAR_LASER_COUNTER_DONE:
          strcpy (buf, "CLC_DONE");
          break;
       case DLCS_SET_MODULATION_COMPLETE:
          strcpy (buf, "SET_MOD_COMP");
          break;
       case DLCS_SET_SERVO_PARAMETERS_COMPLETE:
          strcpy (buf, "SET_SERVO_COMP");
          break;
       case DLCS_HOMESET_BACK_START:
          strcpy (buf, "HS_BACK_START");
          break;
       case DLCS_HOMESET_BACK_MOVING:
          strcpy (buf, "HS_BACK_MOVING");
          break;
       case DLCS_HOMESET_FRONT_START:
          strcpy (buf, "HS_FRONT_START");
          break;
       case DLCS_HOMESET_FRONT_MOVING:
          strcpy (buf, "HS_FRONT_MOVING");
          break;
       case DLCS_HOMESET_CREEPBACK_START:
          strcpy (buf, "HS_CB_START");
          break;
       case DLCS_HOMESET_CREEPBACK_MOVING:
          strcpy (buf, "HS_CB_MOVING");
          break;
       case DLCS_HOMESET_CREEPFRONT_START:
          strcpy (buf, "HS_CF_START");
          break;
       case DLCS_HOMESET_CREEPFRONT_MOVING:
          strcpy (buf, "HS_CF_MOVING");
          break;
       case DLCS_HOMESET_START:
          strcpy (buf, "HOMESET_START");
          break;
       case DLCS_HOMESET_DONE:
          strcpy (buf, "HOMESET_DONE");
          break;
       case DLCS_HOMESET_ABORTED:
          strcpy (buf, "HOMESET_ABORTED");
          break;
       case DLCS_HOMECHECK_BACK_START:
          strcpy (buf, "HC_BACK_START");
          break;
       case DLCS_HOMECHECK_BACK_MOVING:
          strcpy (buf, "HC_BACK_MOVING");
          break;
       case DLCS_HOMECHECK_FRONT_START:
          strcpy (buf, "HC_FRONT_START");
          break;
       case DLCS_HOMECHECK_FRONT_MOVING:
          strcpy (buf, "HC_FRONT_MOVING");
          break;
       case DLCS_HOMECHECK_CREEPBACK_START:
          strcpy (buf, "HC_CB_START");
          break;
       case DLCS_HOMECHECK_CREEPBACK_MOVING:
          strcpy (buf, "HC_CB_MOVING");
          break;
       case DLCS_HOMECHECK_CREEPFRONT_START:
          strcpy (buf, "HC_CF_START");
          break;
       case DLCS_HOMECHECK_CREEPFRONT_MOVING:
          strcpy (buf, "HC_CF_MOVING");
          break;
       case DLCS_HOMECHECK_START:
          strcpy (buf, "HC_START");
          break;
       case DLCS_HOMECHECK_DONE:
          strcpy (buf, "HC_DONE");
          break;
       case DLCS_HOMECHECK_ABORTED:
          strcpy (buf, "HC_ABORTED");
          break;
       case DLCS_USER1_START:
          strcpy (buf, "USER1_START");
          break;
       case DLCS_USER1_ACTIVE:
          strcpy (buf, "USER1_ACTIVE");
          break;
       case DLCS_USER1_ABORTED:
          strcpy (buf, "USER1_ABORTED");
          break;
       case DLCS_USER2_START:
          strcpy (buf, "USER2_START");
          break;
       case DLCS_USER2_ACTIVE:
          strcpy (buf, "USER2_ACTIVE");
          break;
       case DLCS_USER2_ABORTED:
          strcpy (buf, "USER2_ABORTED");
          break;
       case DLCS_CALIB_DITHER_START:
          strcpy (buf, "CAL_DITH_START");
          break;
       case DLCS_CALIB_DITHER_ACTIVE:
          strcpy (buf, "CAL_DITH_ACTIVE");
          break;
       case DLCS_CALIB_DITHER_DONE:
          strcpy (buf, "CAL_DITH_DONE");
          break;
       case DLCS_CALIB_DITHER_ABORTED:
          strcpy (buf, "CAL_DITH_ABORTED");
          break;
       case DLCS_CHECK_DITHER_START:
          strcpy (buf, "CHCK_DITH_START");
          break;
       case DLCS_CHECK_DITHER_ACTIVE:
          strcpy (buf, "CHCK_DITH_ACTIVE");
          break;
       case DLCS_CHECK_DITHER_DONE:
          strcpy (buf, "CHCK_DITH_DONE");
          break;
       case DLCS_CHECK_DITHER_ABORTED:
          strcpy (buf, "CHCK_DITH_ABORT");
          break;
       case DLCS_SELECT_TARGET_DONE:
          strcpy (buf, "SEL_TARGET_DONE");
          break;
       case DLCS_TURN_DITHER_ON_DONE:
          strcpy (buf, "DITHER_ON_DONE");
          break;
       case DLCS_TURN_DITHER_OFF_DONE:
          strcpy (buf, "DITHER_OFF_DONE");
          break;
       case DLCS_SET_DITHER_COMPLETE:
          strcpy (buf, "SET_DITH_COMP");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (dlShellPtr->lCmdStatus,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    /* for simulation */
    DLCurrentTarget[dl_no].dPosition += (DLCurrentTarget[dl_no].dRate * 
               (status_data.tTime - DLCurrentTarget[dl_no].tTime));
    DLCurrentTarget[dl_no].tTime = status_data.tTime;
    if (dlShellPtr->simulationOn)
         record_simulation(dl_no);

}

/****************************************************************
*
* ::process_em_status(em_shell_p emShellPtr,int iGizmoId,tEmStatus status_data)
*
* Description: This function will update status information for em display
*
   [[U ::process_em_status(emShellPtr, iGizmoId, status_data) ]]
*
*****************************************************************/


void process_em_status(em_shell_p emShellPtr,int iGizmoId,tEmStatus status_data)
{
    XmString string;
    char buf[BUFSIZ];
    char timeb[80];
    int em_no = GizmoMap[iGizmoId].iGuiIndex;

    formatTimeToHMS(status_data.tTime, TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (emShellPtr->emTime,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.TrackParameters.dFrequency, 
                        FIELD_WIDTH, 0, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emFrequency,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.TrackParameters.dAmplitude, 
                        FIELD_WIDTH, 0, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emAmplitude,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.TrackParameters.dMeanVal, 
                        FIELD_WIDTH, 0, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emMeanVal,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.TrackParameters.mtModulationType)
    {
       case MT_SQUARE:
          strcpy (buf, "SQUARE");
          break;
       case MT_TRIANGLE:
          strcpy (buf, "TRIANGLE");
          break;
       case MT_SINE:
          strcpy (buf, "SINE");
          break;
       case MT_RANDOM:
          strcpy (buf, "RANDOM");
          break;
       case MT_NONE:
          strcpy (buf, "NONE");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emModType,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    formatTimeToHMS(status_data.TrackParameters.tSyncTime, 
                    TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (emShellPtr->emSyncTime,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.ecsCommandStatus)
    {
       case EMCS_OFF:
          strcpy (buf, "OFF");
          break;
       case EMCS_TRACK_ACTIVE:
          strcpy (buf, "TRACK_ACTIVE");
          break;
       case EMCS_SERVO_PARAMETERS_SET:
          strcpy (buf, "SERVO_PARM_SET");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emCmdStatus,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.essServoState)
    {
       case ESS_OFF:
          strcpy (buf, "ESS_OFF");
          break;
       case ESS_TRACK:
          strcpy (buf, "ESS_TRACK");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (emShellPtr->emServoState,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);


}

/****************************************************************
*
* ::process_ft_status(ft_shell_p ftShellPtr,int iGizmoId,tFtStatus status_data)
*
* Description: This function will update status information for ft display
*
   [[U ::process_ft_status(ftShellPtr, iGizmoId, status_data) ]]
*
*****************************************************************/


void process_ft_status(ft_shell_p ftShellPtr,int iGizmoId,tFtStatus status_data)
{
    XmString string;
    char buf[BUFSIZ];
    char timeb[80];
    int ft_no = GizmoMap[iGizmoId].iGuiIndex;
    int curImg = ft_get_image_shell[ft_no]->curImage;

    /* status for Servo Parameters - controller parameters */
    
    formatTimeToHMS(
        status_data.ServoParameters.ControllerParameters.tFtServoPeriod, 
        TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (ft_set_param_shell[ft_no]->ft_sp_servo_period,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(
       status_data.ServoParameters.ControllerParameters.lvWhiteLightLambda,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ft_set_param_shell[ft_no]->ft_sp_white_light_lambda,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatIntWithCommas(
        status_data.ServoParameters.ControllerParameters.iNumWobbleSamples, 
        INT_FIELD_WIDTH, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ft_set_param_shell[ft_no]->ft_sp_num_wobble_samples,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    /* FT status for Target */

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dManual,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftManual,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dSearch,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftSearch,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dWobble,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftWobble,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dLock,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftLock,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dFringeCenter,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftFringeCenter,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.Targets.dPhase,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftPhase,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    /* FT status for Fringe */

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.FringeStatus.dPhotons,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftPhotons,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.FringeStatus.dVisibility2,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftVisibility2,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas(status_data.FringeStatus.dSnr,
                        FIELD_WIDTH, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftSnr,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    /* FT general status */

    formatTimeToHMS(status_data.tTimeStamp, TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (ftShellPtr->ftTime,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.fcsCommandStatus)
    {
       case FTCS_OFF:
          strcpy (buf, "OFF");
          break;
       case FTCS_TRACK_START:
          strcpy (buf, "TRACK_START");
          break;
       case FTCS_TRACK_ACTIVE:
          strcpy (buf, "TRACK_ACTIVE");
          break;
       case FTCS_TRACK_ABORTED:
          strcpy (buf, "TRACK_ABORTED");
          break;
       case FTCS_GET_IMAGE:
          strcpy (buf, "GET_IMAGE");
          break;
       case FTCS_SET_SERVO_PARAMETERS:
          strcpy (buf, "SET_SERVO");
          break;
       case FTCS_SET_ROI:
          strcpy (buf, "SET_ROI");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftCmdStatus,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.fsmServoMode)
    {
       case FSM_OFF:
          strcpy (buf, "FSM_OFF");
          break;
       case FSM_TRACK:
          strcpy (buf, "FSM_TRACK");
          break;
       case FSM_NUM_SERVO_MODES:
          strcpy (buf, "FSM_NUM_SERVO");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftServoState,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.fcmControllerMode)
    {
       case FCM_NONE:
          strcpy (buf, "FCM_NONE");
          break;
       case FCM_SEARCH:
          strcpy (buf, "FCM_SEARCH");
          break;
       case FCM_SEMI_LOCK:
          strcpy (buf, "SEMI_LOCK");
          break;
       case FCM_LOCK:
          strcpy (buf, "FCM_LOCK");
          break;
       case FCM_UNKNOWN:
          strcpy (buf, "FCM_UNKNOWN");
          break;
       case FCM_NUM_MODES:
          strcpy (buf, "FCM_NUM_MODES");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (ftShellPtr->ftControllerMode,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    if (status_data.bOnCenterFringe == TRUE)
       XtVaSetValues (ftShellPtr->ftCenterFringePix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  on,
           NULL);
    else
       XtVaSetValues (ftShellPtr->ftCenterFringePix,
           XmNlabelType,    XmPIXMAP,
           XmNlabelPixmap,  off,
           NULL);

    /* update servo parameters values from status */

    FTServoCmd[ft_no].dAmplitude =
        status_data.ServoParameters.WobbleParameters.dAmplitude; 
    FTServoCmd[ft_no].dFrequency=
        status_data.ServoParameters.WobbleParameters.dFrequency; 
    FTServoCmd[ft_no].dMeanVal=
        status_data.ServoParameters.WobbleParameters.dMeanVal; 
    FTServoCmd[ft_no].dDutyCycle=
        status_data.ServoParameters.WobbleParameters.dDutyCycle; 
    FTServoCmd[ft_no].mtModulationType=
        status_data.ServoParameters.WobbleParameters.mtModulationType; 
    FTServoCmd[ft_no].tSyncTime=
        status_data.ServoParameters.WobbleParameters.tSyncTime; 

    FTServoCmd[ft_no].dSpiralRadiusInitial =
        status_data.ServoParameters.SearchParameters.dSpiralRadiusInitial;
    FTServoCmd[ft_no].dSpiralRadiusLimit =
        status_data.ServoParameters.SearchParameters.dSpiralRadiusLimit;
    FTServoCmd[ft_no].dSpiralStep =
        status_data.ServoParameters.SearchParameters.dSpiralStep;
    FTServoCmd[ft_no].dProgressionConstant =
        status_data.ServoParameters.SearchParameters.dProgressionConstant;
    FTServoCmd[ft_no].stSearchType =
        status_data.ServoParameters.SearchParameters.stSearchType;

    FTServoCmd[ft_no].tSemiLockToLockTimeout =
        status_data.ServoParameters.ThresholdParameters.tSemiLockToLockTimeout;
    FTServoCmd[ft_no].tTotalTimeToLockTimeout =
        status_data.ServoParameters.ThresholdParameters.tTotalTimeToLockTimeout;
    FTServoCmd[ft_no].dSNRThreshSearchToSemiLock =
     status_data.ServoParameters.ThresholdParameters.dSNRThreshSearchToSemiLock;
    FTServoCmd[ft_no].dSNRThreshSemiLockToLock =
     status_data.ServoParameters.ThresholdParameters.dSNRThreshSemiLockToLock;
    FTServoCmd[ft_no].dV2ThreshSemiLockToLock=
        status_data.ServoParameters.ThresholdParameters.dV2ThreshSemiLockToLock;
    FTServoCmd[ft_no].dSNRThreshLockToSearch =
        status_data.ServoParameters.ThresholdParameters.dSNRThreshLockToSearch;
    FTServoCmd[ft_no].dThreshForDrop=
        status_data.ServoParameters.ThresholdParameters.dThreshForDrop;
    FTServoCmd[ft_no].dThreshForSearch =
        status_data.ServoParameters.ThresholdParameters.dThreshForSearch;

    FTServoCmd[ft_no].iMemBoxcar=
        status_data.ServoParameters.ControllerParameters.iMemBoxcar;
    FTServoCmd[ft_no].dFringeServoGain=
        status_data.ServoParameters.ControllerParameters.dFringeServoGain;
    FTServoCmd[ft_no].dDeltaVDeltaWobbleToHop =
      status_data.ServoParameters.ControllerParameters.dDeltaVDeltaWobbleToHop;
    FTServoCmd[ft_no].iPhaseSign =
        status_data.ServoParameters.ControllerParameters.iPhaseSign;
    FTServoCmd[ft_no]. dPhaseOffset =
        status_data.ServoParameters.ControllerParameters.dPhaseOffset;

    /* update Image parameters */
    if ( !XtIsManaged(ft_get_image_shell[ft_no]->ft_get_image_dialog) )
    {
       ft_get_image_shell[ft_no]->posX[0] = 
                             status_data.ImageParameters.iStartXSt0;

       // memset(buf,0,sizeof(buf));
       // sprintf(buf,"status_data.ImageParameters.iStartYSt0 = %d\n",
       //     status_data.ImageParameters.iStartYSt0);
       // write_to_log(buf, strlen(buf) );

       ft_get_image_shell[ft_no]->posY[0] = FT_WHOLE_HEIGHT -
                             status_data.ImageParameters.iStartYSt0;
       ft_get_image_shell[ft_no]->posX[1] = 
                             status_data.ImageParameters.iStartXSt1;

       // memset(buf,0,sizeof(buf));
       // sprintf(buf,"status_data.ImageParameters.iStartYSt1 = %d\n",
       //     status_data.ImageParameters.iStartYSt1);
       // write_to_log(buf, strlen(buf) );

       ft_get_image_shell[ft_no]->posY[1] = FT_WHOLE_HEIGHT -
                             status_data.ImageParameters.iStartYSt1;
       ft_get_image_shell[ft_no]->posX[2] = 
                             status_data.ImageParameters.iStartXFringeImage;

       // memset(buf,0,sizeof(buf));
       // sprintf(buf,"status_data.ImageParameters.iStartYFringeImage = %d\n",
       //     status_data.ImageParameters.iStartYFringeImage);
       // write_to_log(buf, strlen(buf) );

       ft_get_image_shell[ft_no]->posY[2] = FT_WHOLE_HEIGHT -
                             status_data.ImageParameters.iStartYFringeImage;
       setTextSlide(ft_get_image_shell[ft_no]->posX[curImg], 
                 ft_get_image_shell[ft_no]->posY[curImg],
                 ft_get_image_shell[ft_no]->ft_gi_x_text, 
                 ft_get_image_shell[ft_no]->ft_gi_y_text,
                 ft_get_image_shell[ft_no]->ft_x_scale, 
                 ft_get_image_shell[ft_no]->ft_y_scale);


    }

    /* ignore fringe dimension from status */
    // iFringeImageHSize = status_data.ImageParameters.iFringeImageHSize;
    // iFringeImageVSize = status_data.ImageParameters.iFringeImageVSize;

}

/****************************************************************
*
* ::process_st_status(st_shell_p stShellPtr,int iGizmoId,tStStatus status_data)
*
* Description: This function will update status information for st display
*
   [[U ::process_st_status(stShellPtr, iGizmoId, status_data) ]]
*
*****************************************************************/


void process_st_status(st_shell_p stShellPtr,int iGizmoId,tStStatus status_data)
{
    XmString string;
    char buf[BUFSIZ];
    char timeb[80];
    int st_no = GizmoMap[iGizmoId].iGuiIndex;

    formatTimeToHMS(status_data.tTime, TIME_TICKS_PER_SECOND, timeb, "");
    string = XmStringCreateLocalized (timeb);
    XtVaSetValues (stShellPtr->stTime,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.scsCommandStatus)
    {
       case STCS_OFF:
          strcpy (buf, "OFF");
          break;
       case STCS_TRACK_ACTIVE:
          strcpy (buf, "TRACK_ACTIVE");
          break;
       case STCS_CALIBRATE_ACTIVE:
          strcpy (buf, "CALIBRATE_ACTIVE");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stCmdStatus,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    switch (status_data.sssServoState)
    {
       case SSS_OFF:
          strcpy (buf, "SSS_OFF");
          break;
       case SSS_TRACK:
          strcpy (buf, "SSS_TRACK");
          break;
       case SSS_CALIBRATE:
          strcpy (buf, "SSS_CALIBRATE");
          break;
       default:
          strcpy (buf, "UNKNOWN");
          break;
    }
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stServoState,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);


    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.StarPosition.dXPosition,
                        7, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stXPosition,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    memset(buf,0,BUFSIZ);
    formatDblWithCommas((double)status_data.StarPosition.dYPosition,
                        7, DEC_PLACES, buf, "");
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stYPosition,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

}

void process_st_star_position(st_shell_p stShellPtr,int iGizmoId,
                              tStarPosition status_data)
{

    XmString string;
    char buf[BUFSIZ];
    int st_no = GizmoMap[iGizmoId].iGuiIndex;

    sprintf(buf,"%lf",status_data.dXPosition); 
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stXPosition,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

    sprintf(buf,"%lf",status_data.dYPosition); 
    string = XmStringCreateLocalized (buf);
    XtVaSetValues (stShellPtr->stYPosition,
        XmNlabelString,    string,
        NULL);
    XmStringFree (string);

}


/****************************************************************
*
* ::processDlStatus(int iGizmoId, size_t szBlockSize)
*
* Description: This function will read dl status information over 
     the network.  If read successfully, update dl display with new
     status information
*
   [[U ::processDlStatus(iGizmoId, szBlockSize) ]]
*
*****************************************************************/

void   processDlStatus(int iGizmoId, size_t szBlockSize)
{
   tGsuDlStatus buf;
   int nbytes;
   int dl_no = GizmoMap[iGizmoId].iGuiIndex;

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processDlStatus\n");
   else if (nbytes)
      process_dl_status(dl_shell[dl_no], iGizmoId, buf);
}

/****************************************************************
*
* ::processFtStatus(int iGizmoId, size_t szBlockSize)
*
* Description: This function will read ft status information over 
     the network.  If read successfully, update ft display with new
     status information
*
   [[U ::processFtStatus(iGizmoId, szBlockSize) ]]
*
*****************************************************************/

void   processFtStatus(int iGizmoId, size_t szBlockSize)
{
   tFtStatus buf;
   int nbytes;
   int ft_no = GizmoMap[iGizmoId].iGuiIndex;

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processFtStatus\n");
   else if (nbytes)
      process_ft_status(ft_shell[ft_no], iGizmoId, buf);
}

/****************************************************************
*
* ::processEmStatus(int iGizmoId, size_t szBlockSize)
*
* Description: This function will read em status information over 
     the network.  If read successfully, update em display with new
     status information
*
   [[U ::processEmStatus(iGizmoId, szBlockSize) ]]
*
*****************************************************************/

void   processEmStatus(int iGizmoId, size_t szBlockSize)
{
   tEmStatus buf;
   int nbytes;
   int em_no = GizmoMap[iGizmoId].iGuiIndex;

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processEmStatus\n");
   else if (nbytes)
      process_em_status(em_shell[em_no], iGizmoId, buf);
}

void   processFtImageStatus(int iGizmoId,size_t szBlockSize, BOOL& read_header)
{
   static unsigned char *img_data;
   static int first = 0;
   static int no_read = 0;
   int nbytes;
   int ft_no = GizmoMap[iGizmoId].iGuiIndex;

   if (!first)
      img_data = (unsigned char *)malloc(CiFringeImageHSize*CiFringeImageVSize);
   if (img_data == NULL)
   {
      printf("Cannot malloc\n");
   }

   first = 1;
   if ( (nbytes = clientRead((char *)&img_data[no_read], 
                             szBlockSize-no_read)) < 0)
   {
      perror("error reading in processFtImageStatus\n");
      read_header = TRUE;
   }
   else if (nbytes)
   {
      no_read += nbytes;
   }
   

   if ((nbytes == 0) || (no_read == szBlockSize))
   {
      no_read = 0;
      read_header = TRUE;
      // hide_dialog ( ft_shell[ft_no]->ft_dialog );

      setPixmapImage(img_data, 
                     CiFringeImageVSize, CiFringeImageHSize,
                     &(ft_shell[ft_no]->thePixmap),
                     ft_shell[ft_no]->ft_fringe_image,
                     XtIsManaged(ft_shell[ft_no]->ft_dialog));
      // show_dialog ( ft_shell[ft_no]->ft_dialog );
   }
   else
      read_header = FALSE;
          
}

void   processFtFullImageStatus(int iGizmoId,size_t szBlockSize, 
                                BOOL& read_header)
{

   static unsigned char *img_data;
   static int first = 0;
   static int no_read = 0;
   int nbytes;
   int ft_no = GizmoMap[iGizmoId].iGuiIndex;

   if (!first)
      img_data = (unsigned char *)malloc(FT_WHOLE_WIDTH*FT_WHOLE_HEIGHT);
   first = 1;
   if ( (nbytes = clientRead((char *)&img_data[no_read], 
                             szBlockSize-no_read)) < 0)
   {
      perror("error reading in processFtImageStatus\n");
      read_header = TRUE;
   }
   else if (nbytes)
   {
      no_read += nbytes;
           
   }
   if ((nbytes == 0) || (no_read == szBlockSize))
   {
      no_read = 0;
      read_header = TRUE;
      show_dialog(ft_get_image_shell[ft_no]->ft_get_image_dialog);
      setPixmapImage(img_data, 
                      FT_WHOLE_HEIGHT, FT_WHOLE_WIDTH,
                     &(ft_get_image_shell[ft_no]->thePixmap),
                     ft_get_image_shell[ft_no]->ft_whole_image, FALSE);
   }
   else 
      read_header = FALSE;
}

/****************************************************************
*
* ::processStStatus(int iGizmoId, size_t szBlockSize)
*
* Description: This function will read st status information over 
     the network.  If read successfully, update st display with new
     status information
*
   [[U ::processStStatus(iGizmoId, szBlockSize) ]]
*
*****************************************************************/

void  processStStatus(int iGizmoId, size_t szBlockSize)
{
   tStStatus buf;
   int nbytes;
   int st_no = GizmoMap[iGizmoId].iGuiIndex;

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processStStatus\n");
   else if (nbytes)
      process_st_status(st_shell[st_no], iGizmoId, buf);
}


void   processStImageStatus(int iGizmoId, size_t szBlockSize)
{

   static unsigned char *img_data;
   static int first = 0;
   int nbytes;
   int st_no = GizmoMap[iGizmoId].iGuiIndex;

   if (!first)
      img_data = (unsigned char *)malloc(CiStarImageHSize * CiStarImageVSize);
   first = 1;
   if ( (nbytes = clientReadn((char *)img_data, szBlockSize)) < 0)
      perror("error reading in processStImageStatus\n");
   else if (nbytes)
   {
      // hide_dialog ( st_shell[st_no]->st_dialog );

      setPixmapImage(img_data, 
                      CiStarImageVSize, CiStarImageHSize,
                     &(st_shell[st_no]->thePixmap),
                     st_shell[st_no]->st_star_image,
                     XtIsManaged(st_shell[st_no]->st_dialog));


      // show_dialog ( st_shell[st_no]->st_dialog );
   }

}

void   processStStarPositionStatus(int iGizmoId, size_t szBlockSize)
{

   tStarPosition buf;
   int nbytes;
   int st_no = GizmoMap[iGizmoId].iGuiIndex;

   if ( (nbytes = clientReadn((char *)&buf, szBlockSize)) < 0)
      perror("error reading in processStImageStatus\n");
   else if (nbytes)
      process_st_star_position(st_shell[st_no], iGizmoId, buf);


}



