#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>

#include <Xm/Xm.h>
#include <X11/StringDefs.h>
#include <Xm/ArrowB.h>
#include <Xm/CascadeB.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/ScrollBar.h>
#include <Xm/Separator.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/CascadeBG.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


#include "isitUnix.h"
#include "clientClass.h"
#include "isitCommand.h"
#include "clientNet.h"
#include "clientUtils.h"

#include "share/configShare.h"
#include "client.h"
#include "common.h"
#include "clientInit.h"
#include "clientReadInput.h"
#include "clientData.h"

#include <X11/cursorfont.h>
#include <Xm/Protocols.h>

#include "mesg.h"
#include "msgq.h"

static Boolean stopped;  /* True when user wants to stop task */
static Widget dialog;    
void process_interface( Data_buf buf);
/****************************************************************
*
* ::write_to_log(char *msg, int len)
*
* Description: This function will write message msg of with length
     len into the log message panel 
*
   [[U ::write_to_log(msg, len) ]]
*
*****************************************************************/

void write_to_log(char *msg, int len)
{
   static int pos=0;

   XmTextReplace (ricst_log_shell->ricst_log_text, pos, pos, msg);
   pos += len;
}

/****************************************************************
*
* ::write_to_error_file(char *buf, int nbytes)
*
* Description: This function will write data buf with length nbytes
     to disk if the write option is on
*
   [[U ::write_to_error_file(buf, nbytes) ]]
*
*****************************************************************/

void write_to_error_file(char *buf, int nbytes)
{
   char *string;
   char filename[256];


   if ( ricst_shell->ricst_error_rec_flag == FALSE)
   {
      if (ricst_shell->error_fp != NULL)
      {
         fclose(ricst_shell->error_fp);
         ricst_shell->error_fp = NULL;
      }
      return;
   }

   memset(filename,0,256);
   string  = XmTextGetString (ricst_error_shell->ricst_error_text);
   if ( (string == NULL) || !(strcmp(string,"") ) )
   {
      sprintf(filename,"%s%s", rootdir, defaultname);
      XmTextSetString (ricst_error_shell->ricst_error_text, defaultname);
   }
   else
   {
      sprintf(filename,"%s%s", rootdir,string);
      XmTextSetString (ricst_error_shell->ricst_error_text, string);
   }

   XtFree(string);

   if (ricst_shell->error_fp == NULL)
      if ( (ricst_shell->error_fp = fopen(filename,"a")) == NULL)
      {
         perror("Error opening file\n");
         fclose(ricst_shell->error_fp);
         return;
      }

   fwrite(buf, nbytes, 1, ricst_shell->error_fp);
}


/****************************************************************
*
* ::do_command(char *pStuff, size_t sSize)
*
* Description: This function will send data pStuff of size sSize over
     the network
*
   [[U ::do_command(pStuff, sSize) ]]
*
*****************************************************************/

void do_command(char *pStuff, size_t sSize)
{
   int n;
   XtAppContext  app = XtWidgetToApplicationContext (ricst_shell->ricst_shell);

   if (clientIsConnected())
      /****
      [[U tClientNet::writen(pStuff, sSize) ]]
      ****/
      n = netObj->writen(pStuff,sSize);
   else
      XtAppAddTimeOut (app, 1000L, connectServer, NULL);

}

/****************************************************************
*
* ::clientGetSocketHandle(void)
*
* Description: This function will return the socket handle of the
     current network connection
*
   [[U ::clientGetSocketHandle() ]]
*
*****************************************************************/

int clientGetSocketHandle(void)
{
   return(netObj->getSocketHandle());
}

/****************************************************************
*
* ::clientCreateConnection(void)
*
* Description: This function will create a new network connection
     object
*
   [[U ::clientCreateConnection() ]]
*
*****************************************************************/

int clientCreateConnection(void)
{
   if ( (netObj = new tClientNet()) != NULL)
      return 1;
   else
      return -1;
}

/****************************************************************
*
* ::clientDestroyConnection(void)
*
* Description: This function will destroy and delete the current
     network connection object 
*
   [[U ::clientDestroyConnection() ]]
*
*****************************************************************/

void clientDestroyConnection(void)
{
   delete netObj;
   netObj = NULL;
}

int clientMakeConnection(char *hostname)
{
   return(netObj->makeConnection(hostname));
   
}

/****************************************************************
*
* ::clientTerminateConnection(void)
*
* Description: This function will close the current network connection
*
   [[U ::clientTerminateConnection() ]]
*
*****************************************************************/

int clientTerminateConnection(void)
{
   return (netObj->terminateConnection());
}

/****************************************************************
*
* ::clientRead(void)
*
* Description: This function will read over the network connection
*
   [[U ::clientRead() ]]
*
*****************************************************************/

int clientRead(char *pStuff, size_t sSize)
{
   return(netObj->readOnce(pStuff,sSize));
}

/****************************************************************
*
* ::clientReadn(void)
*
* Description: This function will read over the network connection
*
   [[U ::clientReadn() ]]
*
*****************************************************************/

int clientReadn(char *pStuff, size_t sSize)
{
   return(netObj->readn(pStuff,sSize));
}

/****************************************************************
*
* ::void compTotalTarget(int iGuiGizmoIndex, tDlTargetBase dtNewTarget)
*
* Description: This function will update the target position
*
   [[U ::compTotalTarget(iGuiGizmoIndex, dtNewTarget) ]]
*
*****************************************************************/

void compTotalTarget(int iGuiGizmoIndex, tDlTargetBase dtNewTarget)
{
   DLCurrentTarget[iGuiGizmoIndex].dPosition += dtNewTarget.dPosition +
         (dtNewTarget.dRate *
          (DLCurrentTarget[iGuiGizmoIndex].tTime - dtNewTarget.tTime));

   DLCurrentTarget[iGuiGizmoIndex].dRate += dtNewTarget.dRate;

}

/****************************************************************
*
* ::write_to_interface(char *buf, int nbytes)
*
* Description: This function will write buf data of length nbytes
     to the command line interface
*
   [[U ::write_to_interface(buf, nbytes) ]]
*
*****************************************************************/

void write_to_interface(char *buf, int nbytes)
{

   int qid;
   Mesg data;

   if ( (qid = msgget(MKEY1, 0)) == -1)
   {
      printf("error in msgget()\n");
   }

   data.mesg_type = 1L;
   strcpy(data.mesg_data, buf);
   data.mesg_len = nbytes + 1;
   if (msgsnd(qid, &(data.mesg_type), data.mesg_len,  0) == -1)
   {
      printf("error in msgsnd()\n");
   }

}

/****************************************************************
*
* ::read_interface_input(XtPointer client_data, int *fid, XtInputId *id)
*
* Description: This function will read from the command line interface
*
   [[U ::read_interface_input(client_data, fid, id) ]]
*
*****************************************************************/
void read_interface_input(XtPointer client_data, int *fid, XtInputId *id)
{

        Data_buf buf;
        int nbytes;
        int i;


        if ((nbytes = read(*fid, (char *)&buf, sizeof(Data_buf) )) < 0 )
                perror("read_interface_input");
        else if (nbytes)
        {
           process_interface(buf);
        }
}

void Quit(void)
{
   ISIT_CMD data_send;

   if (clientIsConnected())
   {
      data_send.genericCmd.Command = DISCONNECT;
      do_command((char *)&data_send, sizeof(ISIT_CMD) );

      /* Close network connections */
      if ( clientTerminateConnection() == 0 )
         XtRemoveInput(socket_id);
   }

   interfaceDestroy();
   exit(0);
}


void Reset(Widget w)
{
   ISIT_CMD data_send;
   int rc;
   int socket_handle;

   if (clientIsConnected())
   {
      data_send.genericCmd.Command = RESET;
      do_command((char *)&data_send, sizeof(ISIT_CMD) );

      /* Close network connections */
      if ( clientTerminateConnection() == 0 )
         XtRemoveInput(socket_id);
   }

   XtAppAddTimeOut (XtWidgetToApplicationContext(w), 1000L, connectServer, NULL);




}

double calculateModulation(TIME tTime, int dl_no)
{
   double sum = 0.0;

   if ( ( ((tTime - FTParams[dl_no].tTime) % 
          ((TIME) (1/FTParams[dl_no].dFrequency)) )
      >= 0) &&
      ( ((tTime - FTParams[dl_no].tTime) % 
          ((TIME) (1/FTParams[dl_no].dFrequency)) )
      < (TIME) (1/(2*FTParams[dl_no].dFrequency)) ) )

      sum += (FTParams[dl_no].dMeanVal + FTParams[dl_no].dAmplitude);
   else
      sum += (FTParams[dl_no].dMeanVal - FTParams[dl_no].dAmplitude);

   if ( ( ((tTime - EMParams[dl_no].tTime) % 
          ((TIME) (1/EMParams[dl_no].dFrequency)) )
      >= 0) &&
      ( ((tTime - EMParams[dl_no].tTime) % 
          ((TIME) (1/EMParams[dl_no].dFrequency)) )
      < (TIME) (1/(2*EMParams[dl_no].dFrequency)) ) )

      sum += (EMParams[dl_no].dMeanVal + EMParams[dl_no].dAmplitude);
   else
      sum += (EMParams[dl_no].dMeanVal - EMParams[dl_no].dAmplitude);

   if ( ( ((tTime - DLParams[dl_no].tTime) % 
          ((TIME) (1/DLParams[dl_no].dFrequency)) )
      >= 0) &&
      ( ((tTime - DLParams[dl_no].tTime) % 
          ((TIME) (1/DLParams[dl_no].dFrequency)) )
      < (TIME) (1/(2*DLParams[dl_no].dFrequency)) ) )

      sum += (DLParams[dl_no].dMeanVal + DLParams[dl_no].dAmplitude);
   else
      sum += (DLParams[dl_no].dMeanVal - DLParams[dl_no].dAmplitude);

   return(sum);
}

void record_simulation(int dl_no)
{
   int increment = TIME_TICKS_PER_SECOND / DL_SERVO_RATE;
   int i;
   tDlData simulation_data;

   simulation_data.iVersion = CURRENT_DL_DATA_VERSION;
   simulation_data.iWhichDl = dl_no;
   simulation_data.tTimeStamp = DLCurrentTarget[dl_no].tTime;

   for (i=0; i < DL_DATA_BUF_SIZE; i++)
   {
      simulation_data.ServoDataArray[i].laLaser  = DLCurrentTarget[dl_no].dPosition +
          (DLCurrentTarget[dl_no].dRate * i * increment) +
          calculateModulation( (DLCurrentTarget[dl_no].tTime + 
                                        (i * increment)), dl_no);
      simulation_data.ServoDataArray[i].laError = 0;
   }

   writeData((char *)&simulation_data, sizeof(simulation_data), 
             dl_shell[dl_no]->dl_simulation_file_text,
             &(dl_shell[dl_no]->simu_fp),
             dl_shell[dl_no]->simulationOn, DEFAULT_SIMU_DATA_NAME);
}

int interfaceCreate(void)
{
   int qid;
   int rc = 1;

   if ( (qid = msgget(MKEY1, PERMS | IPC_CREAT)) == -1)
   {
      // printf("Cannot create message queue id with key %d\n",MKEY1);
      rc = -1;
   }
   return (rc);

}

int interfaceDestroy(void)
{
   int qid;
   int rc = 1;

   if ( (qid = msgget(MKEY1, 0)) == -1)
   {
      // printf("error in getting message queue id with key %d\n",MKEY1);
      rc = -1;
   }
   else if (msgctl(qid, IPC_RMID, (struct msqid_ds *) 0) < 0)
   {
      // printf("cannot delete message queue id with key %d\n",MKEY1);
      rc = -1;
   }
   return (rc);
}

/* stop() -- user pressed the "Stop" button in dialog. */
void
stop( Widget dialog, XtPointer client_data, XtPointer call_data)
{
    stopped = True;
}

/* CheckForInterrupt() -- check events in event queue and process
 * the interesting ones.
 */
Boolean
CheckForInterrupt()
{
    Display *dpy = XtDisplay (ricst_shell->ricst_shell);
    Window win = XtWindow (dialog);
    XEvent event;
    static int timeoutAlready = 0;
    char buf[80];
    XmString str;

    /* Make sure all our requests get to the server */
    XFlush (dpy);

    /* Let Motif process all pending exposure events for us. */
    XmUpdateDisplay (ricst_shell->ricst_shell);

    /* Check the event loop for events in the dialog ("Stop"?) */
    while (XCheckMaskEvent (dpy,
           ButtonPressMask | ButtonReleaseMask | ButtonMotionMask |
           PointerMotionMask | KeyPressMask, &event)) {
        /* got an "interesting" event. */
        if (event.xany.window == win)
            XtDispatchEvent (&event); /* it's in our dialog.. */
        else /* uninteresting event--throw it away and sound bell */
            XBell (dpy, 50);
    }
    if ((!timeoutAlready) && networkTimeOut)
    {
       strcpy(buf,"Server connection timeout expires.\n");
       strcat(buf,"      Press Ok to continue.");
       str = XmStringCreateLocalized (buf);
       XtVaSetValues (dialog, XmNmessageString, str, NULL);
       XmStringFree (str);
       str = XmStringCreateLocalized ("Ok");
       XtVaSetValues (dialog, XmNcancelLabelString, str, NULL);
       XmStringFree (str);
       timeoutAlready = 1; 
    }

    timeoutAlready = timeoutAlready & networkTimeOut;
    return stopped;
}

void
TimeoutCursors( Boolean on, Boolean interruptable)
{
    static int locked;
    static Cursor cursor;
    XSetWindowAttributes attrs;
    XSetWindowAttributes group_attrs;
    Display *dpy = XtDisplay (ricst_shell->ricst_shell);
    XEvent event;
    Window win = XtWindow (ricst_shell->ricst_dialog);
    int dl_no, ft_no, em_no, st_no;

    Arg args[5];
    int n;
    XmString str;
    XmString title_str;
    char msg_buf[BUFSIZ];


    /* "locked" keeps track if we've already called the function.
     * This allows recursion and is necessary for most situations.
     */
    if (on)
        locked++;
    else
        locked--;
    if (locked > 1 || locked == 1 && on == 0)
        return; /* already locked and we're not unlocking */

    stopped = False;
    if (!cursor)
        cursor = XCreateFontCursor (dpy, XC_watch);

    /* if on is true, then turn on watch cursor, otherwise, return
     * the shell's cursor to normal.
     */
    attrs.cursor = on ? cursor : None;

    /* change the main application shell's cursor to be the timeout
     * cursor or to reset it to normal.  If other shells exist in
     * this application, they will have to be listed here in order
     * for them to have timeout cursors too.
     */
    XChangeWindowAttributes (dpy, XtWindow (ricst_shell->ricst_shell), 
                             CWCursor, &attrs);
    for (dl_no = 0; dl_no < NUM_DELAY_LINES; dl_no++)
    {

        group_attrs.cursor = 
           XtIsManaged(dl_set_dither_shell[dl_no]->dl_set_dither_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_set_dither_shell[dl_no]->dl_set_dither_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_set_modulation_shell[dl_no]->dl_set_modulation_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_set_modulation_shell[dl_no]->dl_set_modulation_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_solt_shell[dl_no]->dl_solt_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_solt_shell[dl_no]->dl_solt_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_smt_shell[dl_no]->dl_smt_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_smt_shell[dl_no]->dl_smt_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_servo_shell[dl_no]->dl_servo_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_servo_shell[dl_no]->dl_servo_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_gsu_servo_shell[dl_no]->dl_gsu_servo_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_gsu_servo_shell[dl_no]->dl_gsu_servo_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(dl_shell[dl_no]->dl_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (dl_shell[dl_no]->dl_shell), 
                                 CWCursor, &group_attrs);

    }
    for (ft_no = 0; ft_no < CiNumFringeTrackers; ft_no++)
    {

        group_attrs.cursor = 
           XtIsManaged(ft_set_param_shell[ft_no]->ft_set_param_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (ft_set_param_shell[ft_no]->ft_set_param_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(ft_get_image_shell[ft_no]->ft_get_image_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (ft_get_image_shell[ft_no]->ft_get_image_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(ft_shell[ft_no]->ft_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (ft_shell[ft_no]->ft_shell), 
                                 CWCursor, &group_attrs);

    }
    for (em_no = 0; em_no < CiNumExtMets; em_no++)
    {

        group_attrs.cursor = 
           XtIsManaged(em_set_param_shell[em_no]->em_set_param_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (em_set_param_shell[em_no]->em_set_param_shell), 
                                 CWCursor, &group_attrs);

        group_attrs.cursor = 
           XtIsManaged(em_shell[em_no]->em_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (em_shell[em_no]->em_shell), 
                                 CWCursor, &group_attrs);


    }
    for (st_no = 0; st_no < CiNumStarTrackers; st_no++)
    {
        group_attrs.cursor = 
           XtIsManaged(st_shell[st_no]->st_dialog)  && on ? 
                       cursor : None;
        XChangeWindowAttributes (dpy, 
           XtWindow (st_shell[st_no]->st_shell), 
                                 CWCursor, &group_attrs);


    }

    group_attrs.cursor = 
           XtIsManaged(ricst_log_shell->ricst_log_dialog)  && on ? 
                       cursor : None;
    XChangeWindowAttributes (dpy, 
           XtWindow (ricst_log_shell->ricst_log_shell), 
                                 CWCursor, &group_attrs);

    group_attrs.cursor = 
           XtIsManaged(ricst_error_shell->ricst_error_dialog)  && on ? 
                       cursor : None;
    XChangeWindowAttributes (dpy, 
           XtWindow (ricst_error_shell->ricst_error_shell), 
                                 CWCursor, &group_attrs);



    XFlush (dpy);


    if (on){
        /* we're timing out, put up a WorkingDialog.  If the process
         * is interruptable, allow a "Stop" button.  Otherwise, remove
         * all actions so the user can't stop the processing.
         */
        n = 0;
        strcpy(msg_buf, "   Busy connecting to server... \n");
        strcat(msg_buf, "Please wait or press Stop to interrupt.");
        title_str = XmStringCreateLocalized ("Message Dialog");
        str = XmStringCreateLocalized (msg_buf);
        XtSetArg (args[n], XmNdialogTitle, title_str); n++;
        XtSetArg (args[n], XmNmessageString, str); n++;
        XtSetArg (args[n], XmNalignment, XmALIGNMENT_CENTER); n++;
        // dialog = XmCreateWorkingDialog (ricst_shell->ricst_shell, 
        dialog = XmCreateMessageDialog(ricst_shell->ricst_shell, 
                                        "busy", args, n);
        XmStringFree (str);
        XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_OK_BUTTON));
        XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));
        if (interruptable) {
            str = XmStringCreateLocalized ("Stop");
            XtVaSetValues (dialog, XmNcancelLabelString, str, NULL);
            XmStringFree (str);
            XtAddCallback (dialog, XmNcancelCallback, stop, NULL);
        }
        else
            XtUnmanageChild (XmMessageBoxGetChild
                (dialog, XmDIALOG_CANCEL_BUTTON));
        XtManageChild (dialog);

    }

    else {
        /* get rid of all button and keyboard events that occured
         * during the time out.  The user shouldn't have done anything
         * during this time, so flush for button and keypress events.
         * KeyRelease events are not discarded because accelerators
         * require the corresponding release event before normal input
         * can continue.
         */
        while (XCheckMaskEvent (dpy,
                ButtonPressMask | ButtonReleaseMask | ButtonMotionMask
                | PointerMotionMask | KeyPressMask, &event)) {
            /* do nothing */;
        }
        XtDestroyWidget (dialog);
   }

}


void connectServer( XtPointer client_data, XtIntervalId *id)
{

   XtAppContext  app = XtWidgetToApplicationContext (ricst_shell->ricst_shell);
   int socket_handle;
   int rc;

   alarm(NETWORK_CONNECTION_TIMEOUT);
   networkTimeOut = FALSE;

   TimeoutCursors( True, True);

   if ( (rc = clientMakeConnection(hostname) ) == FAILED)
   {
      perror("connection error\n");
      exit(1);
   }
   else if (rc == STOPPED)
   {
      alarm(0);
      
   }
   else if (rc == TIMEOUT)
   {
      for(;;)
      {
         if (CheckForInterrupt ())
         {
            break;
         }
      }
   }
   else if (SUCCESS)
   {
      alarm(0);
      socket_handle = clientGetSocketHandle();
      socket_id = XtAppAddInput(app, socket_handle,
                  (XtPointer)XtInputReadMask, readInput, NULL);

   }

   TimeoutCursors( False, False);

}

void delete_window_handler(Widget w)
{

   Atom WM_DELETE_WINDOW;

   WM_DELETE_WINDOW = XmInternAtom (XtDisplay (w), "WM_DELETE_WINDOW", False);
   XmAddWMProtocolCallback (w, WM_DELETE_WINDOW, del_window_cb, NULL);

}

void del_window_cb( Widget widget, XtPointer client_data, XtPointer call_data)
{
        Quit();
}

BOOL clientIsConnected(void)
{
   return(netObj->GetInit());
}

/* response() --The user made some sort of response to the
 * question posed in AskUser().  Set the answer (client_data)
 * accordingly and destroy the dialog.
 */
void response( Widget widget, XtPointer client_data, XtPointer call_data)
{
    int *answer = (int *) client_data;
    XmAnyCallbackStruct *cbs = (XmAnyCallbackStruct *) call_data;

    switch (cbs->reason) {
        case XmCR_OK:
            *answer = YES;
            break;
        case XmCR_HELP:
            *answer = NO;
            break;
        default:
            return;
    }
}

/*
 * AskUser() -- a generalized routine that asks the user a question
 * and returns the Yes/No response.
 */
int AskUser(Widget parent, char *question)
{
    static Widget dialog;
    XmString text, yes, no, title_str;
    static int answer;
    Arg args[5];
    int n;
    XtAppContext  app = XtWidgetToApplicationContext (ricst_shell->ricst_shell);

    n = 0;
    title_str = XmStringCreateLocalized ("FT Track Dialog");
    XtSetArg (args[n], XmNdialogTitle, title_str); n++;

    if (!dialog) {
        dialog = XmCreateQuestionDialog (parent, "dialog", args, n);
        yes = XmStringCreateLocalized ("Yes");
        no = XmStringCreateLocalized ("No");
        XtVaSetValues (dialog,
            XmNdialogStyle,        XmDIALOG_FULL_APPLICATION_MODAL,
            XmNokLabelString,      yes,
            XmNhelpLabelString,    no,
            NULL);
        XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));

        XtAddCallback (dialog, XmNokCallback, response, &answer);
        XtAddCallback (dialog, XmNhelpCallback, response, &answer);
        XmStringFree (yes);
        XmStringFree (no);
    }
    answer = 0;
    text = XmStringCreateLocalized (question);
    XtVaSetValues (dialog,
        XmNmessageString,      text,
        NULL);
    XmStringFree (text);
    XmStringFree (title_str);
    XtManageChild (dialog);
    XtPopup (XtParent (dialog), XtGrabNone);

    /* while the user hasn't provided an answer, simulate main loop.
     * The answer changes as soon as the user selects one of the
     * buttons and the callback routine changes its value.
     */
    while (answer == 0)
        XtAppProcessEvent (app, XtIMXEvent);

    XtPopdown (XtParent (dialog));
    return answer;
}

