/* $RCSfile: winutil.cc,v $ -- useful routines for assisting in window update */
/*
$Header: /home/chara/cvsroot/ople/sys/gui/client/winutil.cc,v 1.1.1.1 1998/06/11 21:36:31 chara Exp $
*/

/* winutil.c - useful routines for assisting in window update */
/*       sccs info       @(#)winutil.c	1.9  01 Nov 1995  */
/*
modification history
--------------------
01b,14Oct93,beh       added support for double-precision numbers
01a,18Aug93,beh       adapted from statUpdate.c
*/

/*
DESCRIPTION
Contains routines that do useful work in the conversion of data from internal
representations into the forms needed for display in windows.
*/

#include <stdio.h>
#include <string.h>

#include "winutil.h"



/********************************************************
*
* formatIntWithCommas - format an integer with commas every three digits
*
* Arguments are the number, the width of the field to format it into,
* and a pointer to a character buffer where the result will be returned.
*
* RETURNS:
* The formatted string is returned in the buffer pointed to by sText.
*/

void formatIntWithCommas(int num, int fieldwidth, char *sText, char *sPrefix)
{
  int len, commas, fromPos, toPos, iLastFrom;

  sprintf(sText, "%d", num);
  len = strlen(sText);

  if (num < 0)
    commas = (len - 2) / 3;
  else
    commas = (len - 1) / 3;

  if (len + commas > fieldwidth) {
    strncpy(sText, "*OVRFLW*", fieldwidth);
    sText[fieldwidth-1] = 0;   /* In case trailing null didn't make it */
    return;
  }

  fromPos = len;
  toPos = fieldwidth;    /* len + commas; */

  iLastFrom = 0;
  if (num < 0) iLastFrom = 1;
  
  sText[toPos--] = sText[fromPos--];
  
  while (fromPos >= iLastFrom) {
    if (toPos < 0) {        /* Oops! Overflowed the field!! */
      strncpy(sText, "*OVRFLW*", fieldwidth);
      sText[fieldwidth-1] = 0;   /* In case trailing null didn't make it */
      return;
    }
      
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = ',';
  }

  /* Tack on the minus sign if it is present and we can */
  if (toPos >= 0 && fromPos >= 0) sText[toPos--] = sText[fromPos--];

  /* Put prefix on the front if it will fit */
  len = strlen(sPrefix);
  if (len > 0)
    for (fromPos = 0; fromPos < len && fromPos < toPos; fromPos++)
      sText[fromPos] = sPrefix[fromPos];
  
  /* Front-fill with spaces */
  while (fromPos <= toPos)
    sText[fromPos++] = ' ';
  
  /* while (toPos >= 0)
   * sText[toPos--] = ' ';
   */
}


/********************************************************
*
* formatDblWithCommas - format a double with commas every three digits
*
* Arguments are the number, the width of the field to format it into,
* and a pointer to a character buffer where the result will be returned.
*
* RETURNS:
* The formatted string is returned in the buffer pointed to by sText.
*/

void formatDblWithCommas(double num, int fieldwidth, int decplaces,
                         char *sText, char *sPrefix)
{
  int len, intplaces, commas, fromPos, toPos, iLastFrom, i;
  char format[20];

  sprintf(format, "%%.%df", decplaces);
  sprintf(sText, format, num);
  len = strlen(sText);

  if (decplaces > 0)
    intplaces = len - decplaces - 1;
  else
    intplaces = len;
  
  if (num < 0.0)
    commas = (intplaces - 2) / 3;
  else
    commas = (intplaces - 1) / 3;

  if (len + commas > fieldwidth) {
    strncpy(sText, "*OVRFLW*", fieldwidth);
    sText[fieldwidth-1] = 0;   /* In case trailing null didn't make it */
    return;
  }

  fromPos = len;
  toPos = fieldwidth;    /* len + commas; */

  iLastFrom = 0;
  if (num < 0.0) iLastFrom = 1;
  
  /* Copy the trailing NULL */
  sText[toPos--] = sText[fromPos--];

  /* Copy the decimal places and decimal point */
  if (decplaces > 0)
    for (i=0; i<=decplaces; i++)
      if (toPos >= 0 && fromPos >= iLastFrom)
        sText[toPos--] = sText[fromPos--];

  /* Copy the integer part, inserting commas */
  while (fromPos >= iLastFrom) {
    if (toPos < 0) {        /* Oops! Overflowed the field!! */
      strncpy(sText, "*OVRFLW*", fieldwidth);
      sText[fieldwidth-1] = 0;   /* In case trailing null didn't make it */
      return;
    }
      
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = sText[fromPos--];
    if (toPos >= 0 && fromPos >= iLastFrom) sText[toPos--] = ',';
  }

  /* Tack on the minus sign if it is present and we can */
  if (toPos >= 0 && fromPos >= 0) sText[toPos--] = sText[fromPos--];
  
  /* Put prefix on the front if it will fit */
  len = strlen(sPrefix);
  if (len > 0)
    for (fromPos = 0; fromPos < len && fromPos < toPos; fromPos++)
      sText[fromPos] = sPrefix[fromPos];
  
  /* Front-fill with spaces */
  while (fromPos <= toPos)
    sText[fromPos++] = ' ';
  
  /* while (toPos >= 0)
   *  sText[toPos--] = ' ';
   */
}


/********************************************************
*
* formatTimeToSec - format an integer time into seconds
*
* Arguments are the time in ticks, the number of ticks per second, and
* a pointer to a character buffer where the result will be returned.
*
* RETURNS:
* The formatted string is returned in the buffer pointed to by sText.
*/

void formatTimeToSec(unsigned long iTime, unsigned long iTicksPerSecond, 
                     char *sText, char *sPrefix)
{
  int value;
  
  value = iTime/iTicksPerSecond;     /* Convert to seconds */
  sprintf(sText, "%s%d", sPrefix, value);
}


/********************************************************
*
* formatTimeToHMS - format an integer time into hh:mm:ss
*
* Arguments are the time in ticks, the number of ticks per second, and
* a pointer to a character buffer where the result will be returned.
*
* RETURNS:
* The formatted string in hours:minutes:seconds is returned in the
* buffer pointed to by sText.
*/

void formatTimeToHMS(unsigned long iTime, unsigned long iTicksPerSecond, 
                     char *sText, char *sPrefix)
{
  int hours, minutes, seconds, value;
  
  value = iTime/iTicksPerSecond;     /* Convert to seconds */

  hours = value / 3600;
  value -= hours*3600;
  minutes = value / 60;
  seconds = value - minutes*60;
  sprintf(sText,"%s%02d:%02d:%02d", sPrefix, hours, minutes, seconds);
}


