/* $RCSfile: simser.cc,v $ */

/* 
$Header: /home/chara/cvsroot/ople/sys/gui/simser/simser.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $
*/


#include "isit.h"
#include "simser.h"
#include "share/configShare.h"
#include "CmdHdlr.h"
#include <signal.h>

static int writen(int sd,char *ptr,int size);
static void sig_handler(int signo);

/* global socket descriptors */
int msgsock[4];   
BOOL bUIAttached;

/* This is the main routine that kicks off */
/**************************************************************************
Function Name: tcpServer( )
*
* IN :
   int        msgsock[0..3]
*
* INOUT :
   system error stream
* OUT :
*
* SPEC:
*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 4 -> Undefined
   |  ptCommandHdlr invalid -> Undefined
   |  socket descriptor failed to open ->
         print error message
         : return ERROR
   |  cannot associate a network address with the socket
         print error message
         : close socket connection
         : return ERROR
   |  socket descriptor invalid or cannot enable connections to the socket
         print error message
         : close socket connection
         : return ERROR
   |  true ->
         socket is opened and has a socket descriptor
         : associate a network address with the socket
         : enables connections to the socket
         : listen on each connection
         : FOREVER
             [ number of socket stream connection is none ->
                 establish 4 socket connection streams
                 : set bUIAattched flag to TRUE
             | true ->
                 read command packet from client UI command stream
                 : put the command packet onto a command stream
             ]
   ]
*
* RETURNS:
*  OK if success
*  ERROR otherwise
*
**************************************************************************/

int num_of_connect;
STATUS tcpServer( tCmdHdlr *ptCommandHdlr)
// STATUS tcpServer(void)
{
   int sock;
   struct sockaddr_in server;
   int rval;
   int i;
   ERROR_CODE ec;
   ISIT_CMD Msg;
   

   TraceMark(S_gui_STARTING_TCPSERVER);

   signal(SIGPIPE, sig_handler);
   num_of_connect = 0;
   bUIAttached = FALSE;

   /* create a TCP-based socket */

   /*
   NAME
     socket() - open a socket

   SYNOPSIS
     int socket
         (
         int  domain,   -- address family (e.g. AF_INET)       
         int  type,     -- SOCK_STREAM, SOCK_DGRAM, or SOCK_RAW 
         int  protocol  -- socket protocol (usually 0)        
         )

   DESCRIPTION
     This routine opens a socket and returns a socket descriptor.
     The socket descriptor is passed to the other socket routines
     to identify the socket.  The socket descriptor is a standard
     I/O  system  file  descriptor  (fd) and can be used with the
     close(), read(), write(), and ioctl() routines.

   RETURNS
     A socket descriptor, or ERROR.
   */

   /*
   [  socket descriptor failed to open ->
         print error message
         : return ERROR
   |  true ->
         socket is opened has a socket descriptor
   ]
   */
   sock = socket(AF_INET, SOCK_STREAM, 0);
   if (sock  == ERROR)
   {
      perror("opening stream socket");
      return(ERROR);
   }
   
   /*
   SYNOPSIS
     #include <sys/types.h>
     #include <netinet/in.h>

     ulong htonl(u_long hostlong);
     u_short htons(u_short hostshort);

   DESCRIPTION
     These routines convert 16 and 32 bit quantities between net-
     work  byte order and host byte order. 
   */

   /*
   [ server doesn't exists -> Undefined
   | true ->
        server.sin_family = AF_INET;
        server.sin_addr.s_addr = htonl(INADDR_ANY);
        server.sin_port = htons(SERVER_PORT_NUM);
   ]
   */

   /* set up the local address */
   server.sin_family = AF_INET;
   server.sin_addr.s_addr = htonl(INADDR_ANY);
   server.sin_port = htons(SERVER_PORT_NUM);


   /*
   NAME
     bind() - bind a name to a socket

   SYNOPSIS
     STATUS bind
         (
         int              s,       -- socket descriptor
         struct sockaddr  *name,   -- name to be bound
         int              namelen  -- length of name  
         )

   DESCRIPTION
     This routine associates a network address (also referred  to
     as  its  "name")  with  a  specified  socket  so  that other
     processes can connect or send  to  it.   When  a  socket  is
     created  with  socket(), it belongs to an address family but
     has no assigned name.

   RETURNS
     OK, or ERROR if there is an invalid socket, the  address  is
     either  unavailable  or  in  use,  or  the socket is already
     bound.
   */

   /*
   [ &server invalid || socket bound to network address failed ->
       print error message
       : close(socket)
       : return ERROR
   | true ->
       bind the socket to the network address
   ]
   */
   /* bind socket to local address */
   if (bind(sock, (struct sockaddr *)&server, sizeof server )  == ERROR )
   { 
      perror("binding stream socket");
      close(sock);
      return(ERROR);
   }


   /*
   NAME
     listen() - enable connections to a socket

   SYNOPSIS
     STATUS listen
         (
         int  s,       -- socket descriptor              
         int  backlog  -- number of connections to queue 
         )

   DESCRIPTION
     This routine enables  connections  to  a  socket.   It  also
     specifies  the maximum number of unaccepted connections that
     can be pending at one time (<backlog>).  After enabling con-
     nections with listen(), connections are actually accepted by
     accept().

   RETURNS
     OK, or ERROR if the socket is invalid or unable to listen.
   */

   /*
   [ socket invalid or unable to listen ->
         print error message
         : close(socket)
         : return ERROR
   | true ->
        enables connection to the socket
   ]
   */
   /* create queue for client connection requests */
   if ( listen(sock, SERVER_MAX_CONNECTIONS)  == ERROR)
   {
      perror("listen");
      close(sock);
      return(ERROR);
   }

   /* We are going to accept 4 stream connections.  Loop until all four
      connections are established.  The descriptions for 4 stream connections
      are as follows:

      msgsock[0] --> stream for command sent from Sun GUI to VxWorks.
      msgsock[1] --> stream for error sent from Vx to Sun GUI.
      msgsock[2] --> stream for data sent from Vx to Sun GUI.
      msgsock[3] --> stream for status sent from Vx to Sun GUI.
   */

   /*
   [ if num_of_connect < 4 ->
        establish 4 socket connection streams
   | true ->
        read command packet from UI client
        : put command packet on a command stream
   ]
   */
   do 
   {

      if (num_of_connect < 4)
      {
         /*
         NAME
            accept() - accept a connection from a socket

         SYNOPSIS
              int accept
                  (
                  int              s,        -- socket descriptor   
                  struct sockaddr  *addr,    -- peer address       
                  int              *addrlen  -- peer address length 
                  )
         
         DESCRIPTION
              This routine accepts a connection on a socket, and returns a
              new  socket  created for the connection.  The socket must be
              bound to an address with bind(), and enabled for connections
              by  a  call  to listen().  The accept() routine dequeues the
              first connection and creates a new socket with the same pro-
              perties  as <s>.  It blocks the caller until a connection is
              present, unless the socket is marked as non-blocking.
         
         RETURNS
              A socket descriptor, or ERROR if the call fails.
         */

         /* Start to accept connection */

         msgsock[num_of_connect] = accept(sock, (struct sockaddr *)0, (int *)0 );
         if (msgsock[num_of_connect]  == ERROR)
         { 
	    perror("accept");
            close(sock);
            return(ERROR);
         }
         else  
         {
            num_of_connect++;
         } 

         if (num_of_connect == 4)
            bUIAttached = TRUE;
      }
      else
      {
         rval = read(msgsock[0], (char *)&Msg,
                      sizeof(ISIT_CMD) );
         if ( (rval == ERROR) || (rval < 0) )
         {
            num_of_connect = 0;
            bUIAttached = FALSE;
            close(msgsock[0]);
            close(msgsock[1]);
            close(msgsock[2]);
            close(msgsock[3]);

            perror("reading stream socket");
         }
         else
         {
            TraceMark(S_gui_PROCESSING_CMD);

            if ( (Msg.genericCmd.Command == DISCONNECT) ||
                 (Msg.genericCmd.Command == RESET) )
            {
               close(msgsock[0]);
               close(msgsock[1]);
               close(msgsock[2]);
               close(msgsock[3]);
               num_of_connect = 0;
               bUIAttached = FALSE;
            }
            else

               ec = ptCommandHdlr->Put( &Msg );
         }
      } 
   } while(1); 


   return(OK);
}

/**************************************************************************
Function Name: sendError( void   *pStuff,
                          size_t sSize )
*
* IN :
   void    *pStuff
   size_t  sSize
   int     *msgsock;
   int     msgsock[1]
* INOUT :
   system error stream
* OUT :
*
* SPEC:
*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 2 -> Undefined
   |  msgsock[1] invalid -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
*
* RETURNS: 
*  OK if success
*  ERROR otherwise
*
**************************************************************************/

STATUS sendError(void *pStuff, size_t sSize)
{
   int rval;


   /*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 2 -> Undefined
   |  msgsock[1] not connected socket -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
   */

   if (bUIAttached == FALSE)
      return(ERROR);

   if (msgsock[1] != ERROR) {
     if ( (rval = writen(msgsock[1], (char *)pStuff, sSize) )  == ERROR ) {
       perror("writing stream socket");
       msgsock[1] = ERROR;
       return (ERROR);
     }
     else
     {
       return (OK);
     }
   }

   return ERROR;
}

/**************************************************************************
Function Name: sendData( void   *pStuff,
                          size_t sSize )
*
* IN :
   void    *pStuff
   size_t  sSize
   int     *msgsock
   int     msgsock[2]
* INOUT :
   system error stream
* OUT :
*
* SPEC:
*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 3 -> Undefined
   |  msgsock[2] invalid -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
*
* RETURNS: 
*  OK if success
*  ERROR otherwise
*
**************************************************************************/

STATUS sendData(void *pStuff, size_t sSize)
{
   int rval;
   int i;

   /*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 3 -> Undefined
   |  msgsock[2] invalid -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
   */
   if (bUIAttached == FALSE)
      return(ERROR);

   /* return(OK); */
   /* To be used with recOpen(file) and recClose */
   /* recBytes((char *) pStuff, sSize ); */


   if (msgsock[2] != ERROR) {
      if ( (rval = writen(msgsock[2], (char *)pStuff, sSize) )  == ERROR ) {
         perror("writing stream socket");
         msgsock[2] = ERROR;
         return (ERROR);
      }
      else
      {
       return (OK);
      }
   }
   return ERROR;
}

/**************************************************************************
Function Name: sendStatus( void   *pStuff,
                          size_t sSize )
*
* IN :
   void    *pStuff
   size_t  sSize
   int     *msgsock
   int     msgsock[3]
* INOUT :
   system error stream
* OUT :
*
* SPEC:
*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 4 -> Undefined
   |  msgsock[3] invalid -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
*
* RETURNS: 
*  OK if success
*  ERROR otherwise
*
**************************************************************************/

STATUS sendStatus(void *pStuff, size_t sSize)
{
   int rval;

   /*
   [  msgsock invalid -> Undefined
   |  msgsock has dimensions < 4 -> Undefined
   |  msgsock[3] invalid -> Undefined
   |  pStuff invalid || sSize < 0 -> Undefined
   |  sSize is not the size of *pStuff -> Undefined
   |  OS cannot successfully perform write() ->
         print error message
         : return ERROR
   |  true ->
         write *pStuff to socket
         : return OK 
   ]
   */

   if (bUIAttached == FALSE)
      return(ERROR);

   if (msgsock[3] != ERROR) {
     if ( (rval = writen(msgsock[3], (char *)pStuff, sSize) )  == ERROR ) {
       perror("writing stream socket");
       msgsock[3] = ERROR;
       return (ERROR);
     }
     else
     {
       return (OK);
     }
   }

   return ERROR;
}

static void sig_handler(int signo)
{
   num_of_connect = 0;
   bUIAttached = FALSE;
   close(msgsock[0]);
   close(msgsock[1]);
   close(msgsock[2]);
   close(msgsock[3]);
   signal(SIGPIPE, sig_handler);
}

static int writen(int sd,char *ptr,int size)
{
   int no_left,no_written;

   no_left = size;
   while (no_left > 0)
   {
      no_written = write(sd,ptr,no_left);
      if(no_written <=0)
         return(no_written);
      no_left -= no_written;
      ptr += no_written;
   }
   return(size - no_left);
}


