/* $RCSfile: board.cc,v $ - Board class Methods*/
/* 
$Header: /home/chara/cvsroot/ople/sys/hw/board/board.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $
*/

/*
modification history
--------------------
01a,pci,08Oct96    written
*/

/*
DESCRIPTION
  ISIT Board Base class methods
*/

#include <vxWorks.h>
#include <sysLib.h>
#include "isit.h"
#include "hwTypesP.h"
#include "hardware.h"
#include "board.h"
#include "hwError.h"



/******************************************************************************
*
* tBoard::tBoard - Board Class Constructor
*
IN :
  this

OUT :
  BOOL        *this.bInit
*/
tBoard::tBoard(void)
  /****
  [[U tBaseObject::tBaseObject()]]
  ****/
{
  this->SetInit(FALSE);
  uiSize = 0;
  iNumActiveChannels = 0;
  iNumActiveInterrupts = 0;
}


/******************************************************************************
*
* tBoard::Initialize - Board Class Initialization Routine
*
IN :
  this
  tBoard *    pBoard

INOUT :
  BOOL        *this.bInit
  tBoard *    *this.pBoard

RETURNS : ERROR_CODE
  S_board_BOARD_ALREADY_INITIALIZED
  vxWorks error from sysBusToLocalAdrs
  OK
*
*/
ERROR_CODE tBoard::Initialize(void)
{
  /*
  [this is invalid -> UNDEFINED
  | *this.bInit == TRUE -> return S_board_BOARD_ALREADY_INITIALIZED
  | true -> NOP
  ] */
  if(this->IsInit()) {
    return S_board_BOARD_ALREADY_INITIALIZED;
  }

  if(this->vmeAddress.pu8Address == NULL) {
    return Traceback(S_board_INIT_ATTEMPTED_WITH_NULL_ADDRESS,
                     "tBoard::Initialize");
  }

  /*
  [ pvmeBoardAddress is invalid => UNDEFINED
  | *pvmeBoardAddress.modifier is invalid  => UNDEFINED
  | *pvmeBoardAddress.address is invalid  => UNDEFINED
  | *pvmeBoardAddress not reachable by CPU =>
      return VxWorks error code from sysBusToLocalAdrs
  | true -> *this.vAddress := local address corresponding to vmeBoardAddress
  ] */
  this->vu8LocalAddress = this->vmeAddress.GetLocalAddress();
  if(this->vu8LocalAddress == NULL) {
    return Traceback(S_hw_LOCAL_ADDRESS_SET_TO_ZERO, "tBoard::Initialize");
  }

  /*
  [ true -> *this.vmeAddress := vmeBoardAddress
  | : *this.boardName := boardName
    : *this.bInit := TRUE
    : return OK
  ] */
  this->SetInit(TRUE);
  return OK;
}


/******************************************************************************
*
* tBoard::GetChannel - returns channel pointer
*                      initialized, otherwise NULL.
*
[[U tBoard::GetChannel()]]
*
*/
tChannel *tBoard::GetChannel(int iChannelNum)
{
  if(this->IsInit() == FALSE || iChannelNum < 0 ||
                 iChannelNum >= this->iNumActiveChannels) {
    TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
          "Returning NULL channel pointer from tBoard::GetChannel, channel #%d\n",
           iChannelNum);
    return NULL;
  }

  return (this->CreateChannel(iChannelNum));
}


/******************************************************************************
*
* tBoard::CreateChannel - this function is near pure virtual.  It should never
*                         be called.
*
[[U tBoard::CreateChannel()]]
*
*/
tChannel *tBoard::CreateChannel(int iChannelNum)
{
  TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
        "Near Pure Virtual function called: tBoard::CreateChannel, channel=%d",
         iChannelNum);
  return NULL;
}


/******************************************************************************
*
* tBoard::CreateInterrupt - this function is near pure virtual.  It should never
*                         be called.
*
[[U tBoard::CreateInterrupt()]]
*
*/
tInterrupt *tBoard::CreateInterrupt(int iInterruptNum)
{
  TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
        "Near Pure Virtual function called: tBoard::CreateInterrupt, channel=%d",
         iInterruptNum);
  return NULL;
}


/******************************************************************************
*
* tBoard::GetLocalAddress - returns local address of board if board has been
*                           initialized, otherwise NULL.
*
[[U tBoard::GetLocalAddress()]]
*
*/
volatile uint8_t *tBoard::GetLocalAddress(void)
{
  if(!this->IsInitialized()) {
    return NULL;
  }

  return this->vu8LocalAddress;
}


/******************************************************************************
*
* tBoard::IsInitialized - Board Class Check for Initialization
*
IN :
  this
  *this.bInit

RETURNS : BOOL
*
*/
BOOL tBoard::IsInitialized( /* this */ )
{
  /*
  [ this is invalid -> UNDEFINED
  | true -> return *this.bInit
  ] */
  return this->IsInit();
}

/******************************************************************************
*
* tBoard::HasInterrupt - Board Class Check for onBoard Interrupt function
*
IN :
  this
  *this.bInit
  *this.bfFunctions

RETURNS : BOOL
*
*/
BOOL tBoard::HasInterrupt( /* this */ )
{
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED
  | *this.bfFunctions & BF_INTERRUPT == 0 -> return FALSE
  | true -> return TRUE
  ] */
  if(this->bfFunctions & BF_INTERRUPT == 0) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}

/******************************************************************************
*
* tBoard::HasClock - Board Class Check for onBoard Clock function
*
IN :
  this
  *this.bInit
  *this.bfFunctions

RETURNS : BOOL
*
*/
BOOL tBoard::HasClock( /* this */ )
{
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED
  | *this.bfFunctions & BF_CLOCK == 0 -> return FALSE
  | true -> return TRUE
  ] */
  if(this->bfFunctions & BF_CLOCK == 0) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}
  
