/* $RCSfile: channel.cc,v $ */
/*
$Header: /home/chara/cvsroot/ople/sys/hw/channel/channel.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $
*/

#include "isit.h"
#include "hwTypesP.h"
#include "inputChannel.h"
#include "outputChannel.h"


/*******************************************************************************
*
*  tOutputChannel::Reset
*
[[U tOutputChannel::Reset()]]
*
*/
ERROR_CODE tOutputChannel::Reset(void)
{
  this->Output(this->iResetValue);
  
  return OK;
}


/*******************************************************************************
*
*  tOutputChannel::SetResetValue
*
[[U tOutputChannel::SetResetValue()]]
*
*/
ERROR_CODE tOutputChannel::SetResetValue(int iNewResetValue)
{
  if(this->IsInit() == FALSE) {
    return (S_tOutputChannel_CHANNEL_NOT_INITIALIZED);
  }

  this->iResetValue = iNewResetValue;

  return (OK);
}


/*******************************************************************************
*
* tOutputChannel::GetOutputValue(...)
*
* 
* Description: Converts iValueToConvert into the appropriate value for the type
*              of output.  Return value depends on ccConvCode.
*
[[U tOutputChannel::GetOutputValue(iValueToConvert)]]
*
******/
int tOutputChannel::GetOutputValue(int iValueToConvert)
{
  uint32_t uiMaxValue       = (1 << this->iBitWidth) - 1;
  uint32_t uiOffsetMaxValue = (1 << (this->iBitWidth - 1));

  switch(this->ccConvCode) {
   case CC_NO_CONV_CODE:
   case CC_TWOS_COMPLEMENT:
     return iValueToConvert;

   case CC_COMPLEMENT_BINARY:
     return (int) (((uiMaxValue+1) >> 1) - iValueToConvert);

   case CC_STRAIGHT_BINARY:
     return (int) ((unsigned int) iValueToConvert);

   case CC_OFFSET_BINARY:
     return (iValueToConvert + uiOffsetMaxValue);

   default:
     return (iValueToConvert);
  }
}


/*******************************************************************************
*
* tInputChannel::Input(...)
*
[[U tInputChannel::Input_double()]]
*
******/
ERROR_CODE tInputChannel::Input(double *pdValue) const
{
  ERROR_CODE ec;
  int iTempValue;

  ec = this->Input(&iTempValue);
  if(ec != OK) {
    return Traceback(ec, "tInputChannel::Input", pdValue);
  }
 
  *pdValue = (double) iTempValue; 
  return OK;
}


/*******************************************************************************
*
* tInputChannel::GetInputValue(...)
*
* 
* Description: Converts iValueToConvert into the appropriate value for the type
*              of input.  Return value depends on this->ccConvCode.
*
[[U tInputChannel::GetInputValue(iValueToConvert)]]
*
******/
int tInputChannel::GetInputValue(int iValueToConvert) const
{
  uint32_t uiMaxValue       = (1 << this->iBitWidth) - 1;
  uint32_t uiOffsetMaxValue = (1 << (this->iBitWidth - 1));

  switch(this->ccConvCode) {
   case CC_NO_CONV_CODE:
   case CC_TWOS_COMPLEMENT:
   case CC_STRAIGHT_BINARY:
     return iValueToConvert;

   case CC_COMPLEMENT_BINARY:
     return (int) (uiMaxValue - iValueToConvert);

   case CC_OFFSET_BINARY:
     return (iValueToConvert - uiOffsetMaxValue);

   default:
     return iValueToConvert;
  }
}
