/* $RCSfile: imageChannel.cc,v $ - Code file for Image Channels */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/channel/imageChannel.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $
*/

#include "imageChannel.h"
#include "hwError.h"

/****************************************************************************
class tImageChannel

DESCRIPTION:
  tImageChannel is a base class that provides methods for getting a image
  from the frame graber.  This class is the base class for tFullImageChannels
  and tSubImageChannel hiding wether the Image Channel being used is 
  the full image or just a small portion.

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
****************************************************************************/


/****************************************************************************
  tImageChannel::tImageChannel();
  
[[U tImageChannel::tImageChannel(void) ]]
*/
tImageChannel::tImageChannel() {
  /****
  Base Class Constructor
  [[U tChannel::tChannel() ]]
  ****/
  this->SetInit(FALSE);
  this->iHSize = 0;
  this->iVSize = 0;
  }

/****************************************************************************
  ERROR_CODE tImageChannel::GetDimensions(int &iHSize, int &iVSize);
     Method for getting size of the image.
     
[[U tImageChannel::GetDimensions(iHSize, iVSize) ]]
*/
ERROR_CODE tImageChannel::GetDimensions(int &iHSize, int &iVSize) {
  ERROR_CODE ec = OK;

  if (!this->IsInit())
    return S_tImageChannel_CHANNEL_NOT_INITIALIZED;

  iHSize = this->iHSize;
  iVSize = this->iVSize;
  return OK;
  }

/****************************************************************************
tFullImageChannel

DESCRIPTION:

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
****************************************************************************/

/****************************************************************************
  ERROR_CODE tFullImageChannel::GetImage(tImage &Image);
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  

[[U tFullImageChannel::GetImage(Image) ]]
*/
ERROR_CODE tFullImageChannel::GetImage(tImage &Image) {
  ERROR_CODE ec = OK;
  
  if (!this->IsInit())
    return S_tFullImageChannel_CHANNEL_NOT_INITIALIZED;
  
  /****
  [[U ec := tFullImageChannel::GetImageRegion(0, 0, iHSize, iVSize, Image) ]]
  ****/
  ec = this->GetImageRegion(0, 0, iHSize, iVSize, Image);
  if (ec != OK)
    return Traceback(ec, "tFullImageChannel::GetImage");
  return OK;
  }

/****************************************************************************
  ERROR_CODE tFullImageChannel::AtomicGetImage(tImage &Image);
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  The image is guaranteed to be from a single frame.

[[U tFullImageChannel::AtomicGetImage(Image) ]]
*/
ERROR_CODE tFullImageChannel::AtomicGetImage(tImage &Image) {
  ERROR_CODE ec = OK;
  
  if (!this->IsInit())
    return S_tFullImageChannel_CHANNEL_NOT_INITIALIZED;
  
  /****
  [[U ec := tFullImageChannel::AtomicGetImageRegion(0, 0, iHSize, iVSize, Image) ]]
  ****/
  ec = this->AtomicGetImageRegion(0, 0, iHSize, iVSize, Image);
  if (ec != OK)
    return Traceback(ec, "tFullImageChannel::AtomicGetImage");
  return OK;
  }

/****************************************************************************
 
 tSubImageChannel

DESCRIPTION:

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iStartX - Starting X position
this->iStartY - Starting Y position
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
this->pficFullImage - pointer to the real image channel that this is a returns a part of

****************************************************************************/

/****************************************************************************
  tSubImageChannel::tSubImageChannel ();
     Base constructor.

[[U tSubImageChannel::tSubImageChannel () ]]
*/
tSubImageChannel::tSubImageChannel () {
  /****
  Base Class Constructor
  [[U tImageChannel::tImageChannel() ]]
  ****/
  this->iStartX = 0; 
  this->iStartY = 0; 
  this->pficFullImage = NULL;
  }
  
/****************************************************************************
  tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImage);
     Constructor.

[[U tSubImageChannel::tSubImageChannel_t (pficFullImage) ]]
*/
tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImageToUse) {
  int iFullHSize, iFullVSize;

  /****
  Base Class Constructor
  [[U tImageChannel::tImageChannel() ]]
  ****/

  this->iStartX = 0; 
  this->iStartY = 0; 
  this->pficFullImage = NULL;
  if (pficFullImageToUse != NULL) {
    this->SetInit(TRUE);
    this->pficFullImage = pficFullImageToUse;
    /****
    [[U this->pficFullImage->tImageChannel::GetDimensions(iFullHSize, iFullVSize) ]]
    ****/
    this->pficFullImage->GetDimensions(iFullHSize, iFullVSize);
    this->iHSize = iFullHSize;
    this->iVSize = iFullVSize;
    }
  else
    Traceback (S_tSubImageChannel_INVALID_FULL_IMAGE_CHANNEL, "tSubImageChannel::tSubImageChannel");
  }
  
/****************************************************************************
  tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImage, 
                        int iStartX, int iStartY, int iHSize, int iVSize);
     Constructor.

[[U tSubImageChannel::tSubImageChannel_tiiii (pficFullImageToUse, iStartX, iStartY, 
                                              iHSize, iVSize) ]]
*/
tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImageToUse, 
				    int iStartX, int iStartY, int iHSize, int iVSize) :
  /****
  [[U tImageChannel::tImageChannel() ]]
  ****/
  tImageChannel ()

{
  this->iStartX = 0; 
  this->iStartY = 0; 
  this->pficFullImage = NULL;
  if (pficFullImageToUse != NULL) {
    this->SetInit(TRUE);
    this->pficFullImage = pficFullImageToUse;
    }
  else
    Traceback (S_tSubImageChannel_INVALID_FULL_IMAGE_CHANNEL, 
               "tSubImageChannel::tSubImageChannel");

  /****
  [[U tSubImageChannel::SetRegionOfInterest(iStartX, iStartY, iHSize, iVSize) ]]
  ****/
  this->SetRegionOfInterest(iStartX, iStartY, iHSize, iVSize);
  }

/****************************************************************************
  ERROR_CODE tSubImageChannel::SetRegionOfInterest(int iStartX, int iStartY, 
                                                      int iHSize, int iVSize);
     This method sets the portion of the real image that it is to return.
     Returns an error code if points are out of range or if the virtual Image
     channel has not yet been initialized.  If points are out of range, the
     range of interest is unaffected.

[[U tSubImageChannel::SetRegionOfInterest(iStartX, iStartY, iHSize, iVSize) ]]
*/
ERROR_CODE tSubImageChannel::SetRegionOfInterest(int iStartX, int iStartY, int iHSize, int iVSize) {
  int iFullHSize, iFullVSize;
  ERROR_CODE ec = OK;
  
  if (!this->IsInit())
    return S_tSubImageChannel_CHANNEL_NOT_INITIALIZED;

  /****
  [[U ec := this->pficFullImage->tImageChannel::GetDimensions(iFullHSize, iFullVSize) ]]
  ****/
  ec = this->pficFullImage->GetDimensions(iFullHSize, iFullVSize);  
  if (ec != OK)
    return Traceback (ec, "tSubImageChannel::SetRegionOfinterest");

  if ((iStartX < 0) || (iStartX+iHSize > iFullHSize))
    return S_tSubImageChannel_CHANNEL_REGION_OUT_OF_RANGE;
  if ((iStartY < 0) || (iStartY+iVSize > iFullVSize))
    return S_tSubImageChannel_CHANNEL_REGION_OUT_OF_RANGE;

  this->iStartX = iStartX; 
  this->iStartY = iStartY; 
  this->iHSize = iHSize; 
  this->iVSize = iVSize; 
  return OK;
  }
  
/****************************************************************************
  ERROR_CODE tSubImageChannel::GetRegionOfInterest(int &iStartX, int &iStartY, 
                                                      int &iHSize, int &iVSize);
     This method returns the range of interest currently selected.  Returns error
     if the Virtual Image has not yet been initialized.

[[U tSubImageChannel::GetRegionOfInterest(iStartX, iStartY, iHSize, iVSize) ]]
*/
ERROR_CODE tSubImageChannel::GetRegionOfInterest(int &iStartX, int &iStartY, int &iHSize, int &iVSize) const {
  if (!this->IsInit())
    return S_tSubImageChannel_CHANNEL_NOT_INITIALIZED;
  
  iStartX = this->iStartX;
  iStartY = this->iStartY; 
  iHSize = this->iHSize; 
  iVSize = this->iVSize; 
  return OK;
  }

/****************************************************************************
  ERROR_CODE tSubImageChannel::GetImage(tImage &Image);
     Method to get a copy of the image.  It will return the most recent
     copy of the image. 

[[U tSubImageChannel::GetImage(Image) ]]
*/
ERROR_CODE tSubImageChannel::GetImage(tImage &Image) {
  ERROR_CODE ec = OK;
  
  if (!this->IsInit())
    return S_tSubImageChannel_CHANNEL_NOT_INITIALIZED;
  
  /****
  [[U ec := this->pficFullImage->tFullImageChannel::GetImageRegion(this->iStartX, this->iStartY, this->iHSize, this->iVSize, Image) ]]
  ****/
  ec = this->pficFullImage->GetImageRegion(this->iStartX, this->iStartY, iHSize, iVSize, Image);
  if (ec != OK)
    return Traceback (ec, "tSubImageChannel::GetImage");
  return OK;
  }
  
/****************************************************************************
  ERROR_CODE tSubImageChannel::AtomicGetImage(tImage &Image);
     Method to get a copy of the image.  It will return the most recent
     copy of the image.  The image is guaranteed to be from a single frame.

[[U tSubImageChannel::AtomicGetImage(Image) ]]
*/
ERROR_CODE tSubImageChannel::AtomicGetImage(tImage &Image) {
  ERROR_CODE ec = OK;
  
  if (!this->IsInit())
    return S_tSubImageChannel_CHANNEL_NOT_INITIALIZED;
  
  /****
  [[U ec := this->pficFullImage->tFullImageChannel::AtomicGetImageRegion(this->iStartX, this->iStartY, this->iHSize, this->iVSize, Image) ]]
  ****/
  ec = this->pficFullImage->AtomicGetImageRegion(this->iStartX, this->iStartY, iHSize, iVSize, Image);
  if (ec != OK)
    return Traceback (ec, "tSubImageChannel::AtomicGetImage");
  return OK;
  }
