/* $RCSfile: clock.cc,v $ - Clock class Methods*/
/* $Header: /home/chara/cvsroot/ople/sys/hw/clocks/clock.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $ */

/*
modification history
--------------------
01a,pci,08Oct96    written
*/

/*
DESCRIPTION
  ISIT Clock class methods
*/

#include <vxWorks.h>
#include "isit.h"
#include "hardware.h"
#include "board.h"
#include "clock.h"
#include "hwError.h"



/******************************************************************************
*
* tClock::tClock - Clock Class Constructor
*
IN :
  this

OUT :
  BOOL        *this.bInit
*/

tClock::tClock(void /* this */ )
{
  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.bInit := FALSE
  ] */
  this->SetInit(FALSE);
}


/******************************************************************************
*
* tClock::Initialize - Clock Class Initialization Routine
*
IN :
  this
  tBoard *    pBoardObject
  BOOL        *this.bInit

OUT :
  tBoard *    *this.pBoard
*/

ERROR_CODE tClock::Initialize(tBoard *pBoardObject /* this */ )
{
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == TRUE -> return S_clock_E_CLOCK_ALREADY_INITIALIZED
  | true -> NOP
  ] */
  if(this->IsInit()) {
    return S_clock_E_CLOCK_ALREADY_INITIALIZED;
  }

  /*
  [ this is invalid -> UNDEFINED
  | pBoard is invalid -> UNDEFINED
  | *this.*pBoard.bInit == FALSE -> return S_clock_E_BOARD_NOT_INITIALIZED
  | true -> NOP
  ] */
  if(!pBoardObject->IsInitialized()) {
    return S_clock_E_BOARD_NOT_INITIALIZED;
  }

  /*
  [ this is invalid -> UNDEFINED
  | *this.*pBoard does not have clock function ->
      return S_clock_E_BOARD_HAS_NO_CLOCK
  | true -> NOP
  ] */
  if(!pBoardObject->HasClock()) {
    return S_clock_E_BOARD_HAS_NO_CLOCK;
  }

  /*  
  [ this is invalid -> UNDEFINED
  | true -> *this.pBoard := pBoard
    : *this.bInit := TRUE
    : return OK
  ] */
  this->pBoard = pBoardObject;
  this->SetInit(TRUE);
  return OK;
} 


/******************************************************************************
*
* tClock::GetTimeLastInt - Gets time (in 25s of usec) since last interrupt
*
IN :
  this
  *this.bInit
  *this.*pBoard
*
*/

TIME tClock::GetTimeLastInt ( /* this */ )
{
  /*
  [ this OR *this.pBoard is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED
  | *this.*pBoard.bInit == FALSE -> UNDEFINED
  | *this.*pBoard does not have clock function -> UNDEFINED
  | manual capture (GetCurrentTime) is performed during this function -> UNDEFINED
  | true -> return time since last interrupt in 25us since midnight
  ] */
  return this->pBoard->GetTimeLastInt();
}
