/* $RCSfile: interrupt.cc,v $ - Interrupt Class Methods */

/* $Header: /home/chara/cvsroot/ople/sys/hw/clocks/interrupt.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $ */

/* Description:
 * Generic interrupt base classes.
 */

/* $Log: interrupt.cc,v $
 * Revision 1.1.1.1  1998/06/11 21:36:33  chara
 * First import of RICST Code
 *
 * Revision 1.12.2.2  1998/05/12 20:26:15  phil
 * Interrupt and tTimBoard changes for I5b
 *
 * Revision 1.12.2.1  1998/05/08 01:19:45  phil
 * interrupt functionality for I5b
 * */


#include <vxWorks.h>
#include <iv.h>
#include <intLib.h>
#include "isit.h"
#include "hardware.h"
#include "board.h"
#include "interrupt.h"
#include "hwError.h"


/******************************************************************************
*
* tInterrupt::tInterrupt - Interrupt Class Constructor
*
  [[U5 tInterrupt::tInterrupt()]]
*/
tInterrupt::tInterrupt(void) :

  /****
  [[U tBaseObject::tBaseObject()]]
  ****/

  bConnected(FALSE),
  bEnabled(FALSE)
{
  this->SetInit(FALSE);
}

/******************************************************************************
*
* tInterrupt::Initialize - Interrupt Class Initializer 
*
  [[U5 tInterrupt::Initialize()]]
*
*/
ERROR_CODE tInterrupt::Initialize(void)
{
  return OK;
}


/******************************************************************************
*
* tInterrupt::SetIdentity - Sets Priority/Vector
*
  [[U5 tInterrupt::SetIdentity(uiVMEPriorityToUse, uiVMEVectorToUse)]]
*
*/
ERROR_CODE tInterrupt::SetIdentity(uint8_t uiVMEPriorityToUse,
                                   uint8_t uiVMEVectorToUse)
{
  if(uiVMEPriorityToUse < MIN_BUS_INT_PRIORITY ||
     uiVMEPriorityToUse > MAX_BUS_INT_PRIORITY) {
    return Traceback(S_interrupt_INVALID_INTERRUPT_PRIORITY,
                     "tInterrupt::Initialize(Priority=%d, Vector=0x%x)",
                     uiVMEPriorityToUse, uiVMEVectorToUse);
  }

  if(uiVMEVectorToUse < MIN_BUS_INT_VECTOR ||
     uiVMEVectorToUse > MAX_BUS_INT_VECTOR) {
    return Traceback(S_interrupt_INVALID_INTERRUPT_VECTOR,
                     "tInterrupt::Initialize(Priority=%d, Vector=0x%x)",
                     uiVMEPriorityToUse, uiVMEVectorToUse);
  }

  this->uiVMEPriority = uiVMEPriorityToUse;
  this->uiVMEVector   = uiVMEVectorToUse;
  this->SetInit(TRUE);
  return OK;
}


/******************************************************************************
*
* tInterrupt::Connect - connect an interrupt to a function
*
  [[U5 tInterrupt::Connect(pFunc, iArg)]]
*/
ERROR_CODE tInterrupt::Connect(FUNCPTR pFunc, int iArg)
{
  ERROR_CODE ec;

  if (!this->IsInit()) {
    return Traceback(S_interrupt_INT_NOT_INITIALIZED,
                     "tInterrupt::Connect(pFunc=%d, iArg=%d)", (int)pFunc, iArg);
  }

  if(pFunc == NULL) {
    return Traceback(S_interrupt_NULL_FUNCTION_POINTER,
                     "tInterrupt::Connect(pFunc=%d, iArg=%d)", (int)pFunc, iArg);
  }

  if(this->bConnected) {
    TraceMsg(tmt_WARN, S_interrupt_INT_ALREADY_CONNECTED,
             "tInterrupt::Connect: Interrupt #%d", this->uiVMEPriority);
    return OK;
  }


  this->pFunc = pFunc;
  this->iArg  = iArg;

  /****
  [[U ec := main::intConnect(uiVMEVector, this->Isr, (int)this)]]
  ****/
  ec = intConnect((VOIDFUNCPTR *) INUM_TO_IVEC(uiVMEVector),
                  (VOIDFUNCPTR) this->Isr, (int) this);
  if(ec != OK) {
    return Traceback(ec, "tInterrupt::Connect(pFunc=%d, iArg=%d)", pFunc, iArg);
  }

  this->bConnected = TRUE;

  return OK;
}


/*****************************************************************************
*
* tInterrupt::Disconnect - disconnect an interrupt from a function
*
  [[U5 tInterrupt::Disconnect()]]
*/

ERROR_CODE tInterrupt::Disconnect(void) 
{
  ERROR_CODE ec;

  if(this->IsInit()) {
    return S_interrupt_INT_NOT_INITIALIZED;
  }

  if(!this->bConnected) {
    return OK;
  }

  /****
  [[U ec := tInterrupt::Disable()]]
  ****/
  ec = this->Disable();
  if(ec!=OK) {
    return Traceback(ec, "tInterrupt::Disconnect: interrupt #%d",
                     this->uiVMEPriority);
  }

  this->bConnected = FALSE;
  return OK;
}


/******************************************************************************
*
* tInterrupt::Enable - turn board interrupts on
*
  [[U5 tInterrupt::Enable()]]
*
*/

ERROR_CODE tInterrupt::Enable(void) 
{
  ERROR_CODE ec;

  if(!this->IsInit()) {
    return Traceback(S_interrupt_INT_NOT_INITIALIZED,
                     "tInterrupt::Enable: interrupt #%d", this->uiVMEPriority);
  }

  if(!this->bConnected) {
    return Traceback(S_interrupt_INT_NOT_CONNECTED,
                     "tInterrupt::Enable: interrupt #%d", this->uiVMEPriority);
  }

  if(this->bEnabled) {
    return OK;
  }

  /****
  [[U ec := main::sysIntEnable(this->uiVMEPriority)]]
  ****/
  ec = sysIntEnable(this->uiVMEPriority);
  if(ec != OK) {
    return Traceback(ec, "tInterrupt::Enable: interrupt #%d", this->uiVMEPriority);
  }

  this->bEnabled = TRUE;
  return OK;
}


/******************************************************************************
*
* tInterrupt::Disable - turn board interrupts off
*
  [[U5 tInterrupt::Disable()]]
*
*/
ERROR_CODE tInterrupt::Disable(void)
{
  ERROR_CODE ec;

  if(!this->IsInit()) {
    return Traceback(S_interrupt_INT_NOT_INITIALIZED,
                     "tInterrupt::Disable: interrupt #%d", this->uiVMEPriority);
  }

  if(!this->bEnabled) {
    return OK;
  }

  /****
  [[U ec := main::sysIntDisable(this->uiVMEPriority)]]
  ****/
  ec = sysIntDisable(this->uiVMEPriority);
  if(ec != OK) {
    return Traceback(ec, "tInterrupt::Disable: interrupt #%d", this->uiVMEPriority);
  }

  this->bEnabled = FALSE;
  return OK;
}


/******************************************************************************
*
* tInterrupt::Isr - interrupt service routine
*
  [[U5 tInterrupt::Isr()]]
*
*/
void tInterrupt::Isr(void)
{
  (this->pFunc)(this->iArg);
}


/******************************************************************************
*
* tPeriodicInterrupt::tPeriodicInterrupt - Interrupt Class Constructor
*
  [[U5 tPeriodicInterrupt::tPeriodicInterrupt()]]
*/
tPeriodicInterrupt::tPeriodicInterrupt(void) :
/****
[[U tInterrupt::tInterrupt()]]
****/
  bRateSetYet(FALSE),
  irRate(0),
  irMinRate(0),
  irMaxRate(0)
{
}


/******************************************************************************
*
* tPeriodicInterrupt::Initialize  - initialize an Interrupt Object
*
  [[U5 tPeriodicInterrupt::Initialize(irMinRateToUse, irMaxRateToUse)]]
*/
ERROR_CODE tPeriodicInterrupt::Initialize(INT_RATE irMinRateToUse,
                                          INT_RATE irMaxRateToUse)
{
  ERROR_CODE ec;
 
  /**** 
  [[U ec := tInterrupt::Initialize()]]
  ****/
  ec = tInterrupt::Initialize();
  if(ec!=OK)  return Traceback(ec, "tPeriodicInterrupt::Initialize");

  this->irMinRate = irMinRateToUse;
  this->irMaxRate = irMaxRateToUse;

  return OK;
}


/******************************************************************************
*
* tPeriodicInterrupt::RateSet -  Set interrupt rate on board
*
* The following spec shows the board-level behavior.  At this level,
* we simply set this->irRate to  *pirRateToUse.
  [[U5 tPeriodicInterrupt::RateSet(pirRateToUse)]]
*/
ERROR_CODE tPeriodicInterrupt::RateSet(INT_RATE* pirRateToUse)
{
  ERROR_CODE ec;

  if(this->IsInit() == FALSE) {
    return Traceback(S_interrupt_INT_NOT_INITIALIZED,
                     "tPeriodicInterrupt::RateSet: interrupt #%d, rate %d",
                     this->uiVMEPriority, *pirRateToUse);
  }

  TraceMsg(tmt_WARN, S_interrupt_RATE_SET_CALLED_USING_BASE_CLASS,
           "tPeriodicInterrupt::RateSet: Interrupt #%d, Rate %d",
           this->uiVMEPriority, *pirRateToUse);
 
  this->bRateSetYet = TRUE;
 
  if(*pirRateToUse < this->irMinRate) {
    this->irRate = this->irMinRate;
    return S_interrupt_W_INVALID_RATE;
  }
    
  if(*pirRateToUse > this->irMaxRate) {
    this->irRate = this->irMaxRate;
    return S_interrupt_W_INVALID_RATE;
  }

  this->irRate = *pirRateToUse;
  return OK;
}


/******************************************************************************
*
* tPeriodicInterrupt::Enable - enable the interrupt
*
  [[U5 tPeriodicInterrupt::Enable()]]
*/
ERROR_CODE tPeriodicInterrupt::Enable(void)
{
  ERROR_CODE ec;

  if(!this->bRateSetYet) {
    return Traceback(S_interrupt_INT_RATE_NOT_SET,
                     "tInterrupt::Enable: interrupt #%d", this->uiVMEPriority);
  }

  /****
  [[U ec := tInterrupt::Enable()]]
  ****/
  ec = tInterrupt::Enable();
  if(ec != OK) {
    return Traceback(ec, "tPeriodicInterrupt::Enable: interrupt #%d",
                     this->uiVMEPriority);
  }
 
  return OK;
}


/******************************************************************************
*
* tPeriodicTimeInterrupt::Initialize  - initialize an Interrupt Object
*
  [[U5 tPeriodicTimeInterrupt::Initialize(irMinRateToUse, irMaxRateToUse)]]
*/
ERROR_CODE tPeriodicTimeInterrupt::Initialize(INT_RATE irMinRateToUse,
                                              INT_RATE irMaxRateToUse)
{
  ERROR_CODE ec;
 
  /****
  [[U ec := tPeriodicInterrupt::Initialize(irMinRateToUse, irMaxRateToUse)]]
  ****/
  ec = tPeriodicInterrupt::Initialize(irMinRateToUse, irMaxRateToUse);
  if(ec!=OK)  return Traceback(ec, "tPeriodicTimeInterrupt::Initialize");

  return OK;
}

