/*$RCSfile: dt1401.cc,v $*/
/*
$Header: /home/chara/cvsroot/ople/sys/hw/drivers/dt1401.cc,v 1.1.1.1 1998/06/11 21:36:33 chara Exp $
*/

/*
modification history
--------------------
01a,22Oct96,pci           written
*/

/*
*  tDT1401 Class methods for ISIT project
*/

#include "isit.h"
#include "hardware.h"
#include "board.h"
#include "dt1401.h"
#include "hwError.h"

/*****************************************************************************
*
* tDT1401::tDT1401 - tDT1401 class constructor
*
*
IN :
  this

OUT :
  *this.bInit
  *this.bInit
  *this.vADSCR
  *this.vADCHANREG
  *this.vPACLK
  *this.vADDATA
  *this.vDADATA0
  *this.vDADATA1
  *this.vDIOREG

SPEC :
  same as line by line spec
*
*/

tDT1401::tDT1401(/*this*/ void)
{
  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.bInit := FALSE
    : *this.vADSCR := NULL
    : *this.vADCHANREG := NULL
    : *this.vPACLK := NULL
    : *this.vADDATA := NULL
    : *this.vDADATA0 := NULL
    : *this.vDADATA1 := NULL
    : *this.vDIOREG := NULL
  ] */
  this->bInit = FALSE;
  this->vADSCR = NULL;
  this->vADCHANREG = NULL;
  this->vPACLK = NULL;
  this->vADDATA = NULL;
  this->vDADATA0 = NULL;
  this->vDADATA1 = NULL;
  this->vDIOREG   = NULL;
}


/*****************************************************************************
*
* tDT1401::SoftInit - tDT1401 Initialization Method
*
*
IN :
  this
  tVmeAddress vmeBoardAddress
  BOARD_NAME boardName

INOUT :
  BOOL       *this.bInit

OUT :
  uint16_t *    *this.vADSCR
  uint16_t *    *this.vADCHANREG
  uint16_t *    *this.vPACLK
  int16_t *    *this.vADDATA
  int16_t *    *this.vDADATA0
  int16_t *    *this.vDADATA1
  uint16_t *    *this.vDIOREG
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.vu8LocalAddress
  BOARD_NAME *this.boardName
  BOARD_FUNC *this.bfFunctions

SPEC :
  [this OR *this.vu8LocalAddress is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_INTERRUPT |
                           BF_AD_CHAN | BF_DA_CHAN |
                           BF_DIGITAL_IN_CHAN | BF_DIGITAL_OUT_CHAN
    : *this.bInit := TRUE
    : *this.vu8LocalAddress := local address corresponding to vmeBoardAddress.address
    :: *this.vADSCR    := vu8LocalAddress + 0x0
    :: *this.vADCHANREG:= vu8LocalAddress + 0x2
    :: *this.vPACLK    := vu8LocalAddress + 0x4
    :: *this.vADDATA   := vu8LocalAddress + 0x6
    :: *this.vDADATA0  := vu8LocalAddress + 0x8
    :: *this.vDADATA1  := vu8LocalAddress + 0xA
    :: *this.vDIOREG  := vu8LocalAddress + 0xC
    : return OK
  ]

RETURNS : ERROR_CODE
  S_dt1401_BOARD_ALREADY_INITIALIZED
  OK
*
*/
ERROR_CODE tDT1401::SoftInit(/*this,*/
                             tVmeAddress vmeBoardAddress,
                             BOARD_NAME  boardName)
{
  ERROR_CODE ec;

  /*
  [this is invalid -> UNDEFINED
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bInit := TRUE
    : *this.vu8LocalAddress := local address corresponding to vmeBoardAddress.address
  ] */
  ec = tBoard::Initialize(&vmeBoardAddress, boardName);
  if(ec != OK) {
    return Traceback(ec, "tDT1401::SoftInit", vmeBoardAddress, boardName);
  }


  /*
  [ this is invalid -> UNDEFINED
  | true ->
    *this.bfFunctions := BF_INTERRUPT |
                         BF_AD_CHAN | BF_DA_CHAN |
                         BF_DIGITAL_IN_CHAN | BF_DIGITAL_OUT_CHAN
  ] */
  this->bfFunctions =    BF_INTERRUPT |
                         BF_AD_CHAN | BF_DA_CHAN |
                         BF_DIGITAL_IN_CHAN | BF_DIGITAL_OUT_CHAN;

  /*
  [ this OR *this.vu8LocalAddress is invalid -> UNDEFINED
  | true -> *this.vADSCR    := vu8LocalAddress + 0x0
    : *this.vADCHANREG:= vu8LocalAddress + 0x2
    : *this.vPACLK    := vu8LocalAddress + 0x4
    : *this.vADDATA   := vu8LocalAddress + 0x6
    : *this.vDADATA0  := vu8LocalAddress + 0x8
    : *this.vDADATA1  := vu8LocalAddress + 0xA
    : *this.vDIOREG  := vu8LocalAddress + 0xC
  ] */
  this->vADSCR    = (uint16_t *) (vu8LocalAddress + 0x0);
  this->vADCHANREG= (uint16_t *) (vu8LocalAddress + 0x2);
  this->vPACLK    = (uint16_t *) (vu8LocalAddress + 0x4);
  this->vADDATA   = (int16_t *) (vu8LocalAddress + 0x6);
  this->vDADATA0  = (int16_t *) (vu8LocalAddress + 0x8);
  this->vDADATA1  = (int16_t *) (vu8LocalAddress + 0xA);
  this->vDIOREG  = (uint16_t *) (vu8LocalAddress + 0xC);
  return OK;
}



/*****************************************************************************
*
* tDT1401::HardInit - tDT1401 Initialization Method
*
*
IN :
  this
  tVmeAddress vmeBoardAddress
  BOARD_NAME boardName

INOUT :
  BOOL       *this.bInit

OUT :
  uint16_t *    *this.vADSCR
  uint16_t *    *this.vADCHANREG
  uint16_t *    *this.vPACLK
  int16_t *    *this.vADDATA
  int16_t *    *this.vDADATA0
  int16_t *    *this.vDADATA1
  uint16_t *    *this.vDIOREG
 *** VALUES OF ABOVE LISTED POINTERS ***
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.vu8LocalAddress
  BOARD_NAME *this.boardName
  BOARD_FUNC *this.bfFunctions

SPEC :
  [this is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_INTERRUPT |
                           BF_AD_CHAN | BF_DA_CHAN |
                           BF_DIO
    : *this.bInit := TRUE
    : *this.vu8LocalAddress := local address corresponding to vmeBoardAddress.address
    :: *this.vADSCR    := vu8LocalAddress + 0x0
    :: *this.vADCHANREG:= vu8LocalAddress + 0x2
    :: *this.vPACLK    := vu8LocalAddress + 0x4
    :: *this.vADDATA   := vu8LocalAddress + 0x6
    :: *this.vDADATA0  := vu8LocalAddress + 0x8
    :: *this.vDADATA1  := vu8LocalAddress + 0xA
    :: *this.vDIOREG  := vu8LocalAddress + 0xC

    :: *this.*vADSCR    := 0x0000 
    :: *this.*vADCHANREG:= 0x0000
    :: *this.*vPACLK    := 0x0030
    :: *this.*vDADATA0  := 0x0000
    :: *this.*vDADATA1  := 0x0000
    :: *this.*vDIOREG  := 0xffff
    : return OK
  ]

RETURNS : ERROR_CODE
  S_dt1401_BOARD_ALREADY_INITIALIZED
  OK
*
*/
ERROR_CODE tDT1401::HardInit(/*this,*/
                             tVmeAddress vmeBoardAddress,
                             BOARD_NAME  boardName)
{
  ERROR_CODE ec;
  /*
  [this is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_INTERRUPT |
                           BF_AD_CHAN | BF_DA_CHAN |
                           BF_DIO
    : *this.bInit := TRUE
    : *this.vu8LocalAddress := local address corresponding to vmeBoardAddress.address
    :: *this.vADSCR    := vu8LocalAddress + 0x0
    :: *this.vADCHANREG:= vu8LocalAddress + 0x2
    :: *this.vPACLK    := vu8LocalAddress + 0x4
    :: *this.vADDATA   := vu8LocalAddress + 0x6
    :: *this.vDADATA0  := vu8LocalAddress + 0x8
    :: *this.vDADATA1  := vu8LocalAddress + 0xA
    :: *this.vDIOREG  := vu8LocalAddress + 0xC
  ] */
  ec = this->SoftInit(vmeBoardAddress, boardName);
  if(ec!=OK) {
    return Traceback(ec, "tDT1401::HardInit", 
                     vmeBoardAddress, boardName);
  }

  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.*vADSCR    := 0x0000 
    : *this.*vADCHANREG:= 0x0000
    : *this.*vPACLK    := 0x0030
    : *this.*vDADATA0  := 0x0000
    : *this.*vDADATA1  := 0x0000
    : *this.*vDIOREG  := 0xffff
  ] */
  *this->vADSCR    = 0x0000;
  *this->vADCHANREG= 0x0000;
  *this->vPACLK    = 0x0030;
  *this->vDADATA0  = 0x0000;
  *this->vDADATA1  = 0x0000;
  *this->vDIOREG  = 0xffff;
  return OK;
}


/*****************************************************************************
*
* tDT1401::D2A - tDT1401 D2A output Method
*
*
IN :
    this
    iChannel
    wValue

OUT :
    *this.*vDADATA0
    *this.*vDADATA1

SPEC :

  [ this is invalid OR board has not been initialized -> UNDEFINED
  | iChannel == 0 -> *this->vDADATA0 = wValue & 0x0fff;
    : return OK
  | iChannel == 1 -> *this->vDADATA1 = wValue & 0x0fff;
    : return OK
  | true -> return S_dt1401_INVALID_DA_CHANNEL
  ]

RETURNS : ERROR_CODE
  S_dt1401_INVALID_DA_CHANNEL
  OK
*
*/
ERROR_CODE tDT1401::D2A(int iChannel, int16_t wValue)
{
  /*
  [ this is invalid OR board has not been initialized -> UNDEFINED
  | iChannel == 0 -> *this.*vDADATA0 := wValue & 0x0fff
  | iChannel == 1 -> *this.*vDADATA1 := wValue & 0x0fff
  | true -> return S_dt1401_INVALID_DA_CHANNEL
  ] */
  if(iChannel == 0) {
    *this->vDADATA0 = wValue & 0x0fff;
  }
  else if(iChannel == 1) {
    *this->vDADATA1 = wValue & 0x0fff;
  }
  else {
    return S_dt1401_INVALID_DA_CHANNEL;
  }
  return OK;
}


/*****************************************************************************
*
* tDT1401::DioOut - tDT1401 Digital output Method
*
*
IN :
  this
  uwMask
  uwValue

INOUT :
  *this.*vDIOREG

SPEC :
  [ this is invalid OR board is not initialized -> UNDEFINED
  | true -> *this.*vDIOREG := *this.*vDIOREG | 
      (GetDioOutputMask() & uwMask & uwValue)
  ]

RETURNS : void
*
*/
void tDT1401::DioOut(uint16_t uwMask, uint16_t uwValue)
{
  /*
  [ this is invalid OR board is not initialized -> UNDEFINED
  | true -> *this.*vDIOREG := *this.*vDIOREG |
      (GetDioOutputMask() & uwMask & uwValue)
  ] */
  *this->vDIOREG |= this->GetDioOutputMask() & uwMask & uwValue;
}


/*****************************************************************************
*
* tDT1401::DioIn - tDT1401 Digital input Method
*
*
IN :
  this
  uwMask
  *this.*vDIOREG

INOUT :

SPEC :
  [ this is invalid OR board is not initialized -> UNDEFINED
  | true -> return (*this->vDIOREG & GetDioInputMask() & uwMask)
  ]

RETURNS : uint16_t
*
*/
uint16_t tDT1401::DioIn(uint16_t uwMask)
{
  /*
  [ this is invalid OR board is not initialized -> UNDEFINED
  | true -> return (*this.*vDIOREG & GetDioInputMask() & uwMask)
  ] */
  return (*this->vDIOREG & this->GetDioInputMask() & uwMask);
}


/*****************************************************************************
*
* tDT1401::GetDioInputMask - tDT1401 Method to get input filter for DIOREG
*
*
IN :
  this
  *this.*vADSCR
 
SPEC :
  [ this is invalid || board not initailized -> UNDEFINED
  | *this.*vADSCR & 0x0300 == 0x0000 -> return 0xffff
  | *this.*vADSCR & 0x0300 == 0x0100 -> return 0x00ff
  | *this.*vADSCR & 0x0300 == 0x0200 -> return 0xff00
  | true -> return 0x0000
  ]

LOCAL :
uint16_t uwMask

RETURNS : uint16_t
*
*/
uint16_t tDT1401::GetDioInputMask(void)
{
  uint16_t uwMask;

  uwMask = *this->vADSCR & 0x0300;

  if(uwMask == 0x0000) {
    return 0xffff;
  }
  if(uwMask == 0x0100) {
    return 0x00ff;
  }
  if(uwMask == 0x0200) {
    return 0xff00;
  }
  return (0);
}


/*****************************************************************************
*
* tDT1401::GetDioOutputMask - tDT1401 Method to get output filter for DIOREG
*
*
IN :
  this
  *this.*vADSCR
 
SPEC :
  [ this is invalid || board not initailized -> UNDEFINED
  | *this.*vADSCR & 0x0300 == 0x0300 -> return 0xffff
  | *this.*vADSCR & 0x0300 == 0x0100 -> return 0xff00
  | *this.*vADSCR & 0x0300 == 0x0200 -> return 0x00ff
  | true -> return 0x0000
  ]

LOCAL :
uint16_t uwMask

RETURNS : uint16_t
*
*/
uint16_t tDT1401::GetDioOutputMask(void)
{
  uint16_t uwMask;

  uwMask = *this->vADSCR & 0x0300;

  if(uwMask == 0x0300) {
    return 0xffff;
  }
  if(uwMask == 0x0100) {
    return 0xff00;
  }
  if(uwMask == 0x0200) {
    return 0x00ff;
  }
  return (0);
}
