/* RCSfile  - driver library for the MM6216D memory card */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/drivers/mm6216.cc,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

/*
modification history
--------------------
01b,pci,05Jun97      Added call to tBoard::Intiailize in SoftInit
01a,pci,27May97      written
*/

#include "isit.h"
#include "hwTypesP.h"
#include "mm6216.h"

/*******************************************************************************
*
* tMm6216Board::tMm6216Board - constructor for SHMEMory board
*
[[U tMm6216Board::tMm6216Board()]]
*
****/
tMm6216Board::tMm6216Board(int iBoardNum)
{

  TraceMark(S_mm6216_ENTERING_CONSTRUCTOR);

  switch(iBoardNum) {
   case 0:
    this->vmeAddress = tVmeAddress(VME_AM_EXT_SUP_DATA, (uint8_t *) 0x08000000);
    break;
   default:
    this->vmeAddress = tVmeAddress();
    break;
  }
  this->boardName  = MM6216_BOARD_NAME + iBoardNum;

  TraceMark(S_mm6216_DONE_CONSTRUCING);
}


/*******************************************************************************
*
* tMm6216Board::SoftInit - initializes class variables
*
[[U tMm6216Board::SoftInit()]]
*
****/
ERROR_CODE tMm6216Board::SoftInit(void)
{
  ERROR_CODE ec;

  TraceMark(S_mm6216_ENTERING_SOFTINIT);

  if((this->CuiDefaultMemSize % sizeof(int32_t)) != 0) {
    return Traceback(S_mm6216_MEMORY_SIZE_NOT_4_BYTE_ALIGNED,
                     "tMm6216Board::SoftInit");
  }

  /*****
  [[U tBoard::Initialize()]]
  *****/
  ec = tBoard::Initialize();
  if(ec!=OK) {
    return Traceback(ec, "tMm6216Board::SoftInit");
  }

  this->viLocalAddress    = (volatile int32_t *) this->vu8LocalAddress;
  this->uiSize            = this->CuiDefaultMemSize;
  this->uiMemSizeLongWords = this->CuiDefaultMemSize / sizeof(int32_t);

  this->SetInit(TRUE);

  TraceMark(S_mm6216_INITIALIZED_SOFTLY);

  return OK;
}


/*******************************************************************************
*
* tMm6216Board::HardInit - initializes class variables and all memory locations
*
[[U tMm6216Board::HardInit()]]
*
****/
ERROR_CODE tMm6216Board::HardInit(void)
{
  ERROR_CODE ec;
  unsigned int ui;
  volatile int32_t *viLoopAddress;

  TraceMark(S_mm6216_ENTERING_HARDINIT);

  /****
  [[U tMm6216Board::SoftInit()]]
  ****/
  ec = this->SoftInit();
  if(ec != OK) {
    return Traceback(ec, "tMm6216Board::HardInit");
  }

  for(ui=0; ui<this->uiMemSizeLongWords; ui++) {
    viLoopAddress = this->viLocalAddress + ui;
    *viLoopAddress = 0;
  }

  TraceMark(S_mm6216_INITIALZED_HARDLY);

  return OK;
}

/*********************************************************************
*
* tMm6216Board::CreateChannel -  written to conform with tBoard::CreateChannel
*                               Since This board has no channels, this function
*                               should never be called.
*
[[U tMm6216Board::CreateChannel()]]
*
*/
tChannel * tMm6216Board::CreateChannel(int iChannelNum)
{
  TraceMsg(tmt_ERROR, S_pc03v_CREATE_CHANNEL_ATTEMPTED,
           "Error: CreateChannel called for MM6216 Board\n");
  return NULL;
}
