/* $RCSfile: pc03v.cc,v $ */
/* 
$Header: /home/chara/cvsroot/ople/sys/hw/drivers/pc03v.cc,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

/*
* This file contains the PC03V class methods.  The PC03V is an IRIG Time
* code processor card which contains interrupt and clock capabilities 
* as well as other functions unused by ISIT.
*/

/*
modification history
--------------------
01a,pci,17Oct96               written
*/

#include "isit.h"

#ifndef  SPECTOOL
#include <sysLib.h>
#include <iv.h>
#include <intLib.h>
#endif

#include "hwTypesP.h"
#include "hardware.h"
#include "board.h"
#include "pc03v.h"
#include "hwError.h"

/*********************************************************************
*
* tPc03vBoard::tPc03vBoard - PC03V card constructor
*
*
IN :
  this

INOUT :
  *this.bInit
*
*/
tPc03vBoard::tPc03vBoard(int iBoardNum)
{
  TraceMark(S_pc03v_ENTERING_CONSTRUCTOR);

  switch(iBoardNum) {
   case 0:
    this->vmeAddress = tVmeAddress(VME_AM_SUP_SHORT_IO, (uint8_t *) 0xab00);
    break;
   case 1:
    this->vmeAddress = tVmeAddress(VME_AM_SUP_SHORT_IO, (uint8_t *) 0xaa00);
    break;
   default:
    this->vmeAddress = tVmeAddress();
    break;
  }
  this->boardName  = PC03V_BOARD_NAME + iBoardNum;
  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.bInit := FALSE
    : set all register address variables to NULL
  ]
  */
  this->vCAPTR = NULL;
  this->vTIME0 = NULL;
  this->vTIME1 = NULL;
  this->vTIME2 = NULL;
  this->vTIME3 = NULL;
  this->vCLRIG = NULL;
  this->vCR0 = NULL;
  this->vCR1 = NULL;
  this->vBIMCR0 = NULL;
  this->vBIMCR1 = NULL;
  this->vBIMCR2 = NULL;
  this->vBIMCR3 = NULL;
  this->vBIMV0 = NULL;
  this->vBIMV1 = NULL;
  this->vBIMV2 = NULL;
  this->vBIMV3 = NULL;
  this->vRELFR = NULL;
  this->vSTR1W1 = NULL;
  this->vSTR1W2 = NULL;
  this->vSTR1W3 = NULL;
  this->vSTR2W1 = NULL;
  this->vSTR2W2 = NULL;
  this->vSTR2W3 = NULL;
  this->vCTC0 = NULL;
  this->vCTCCR = NULL;

  TraceMark(S_pc03v_DONE_CONSTRUCTING);
}


/*********************************************************************
*
* tPc03vBoard::SoftInit - PC03V card Soft Initialize method
*
*
IN :
  this

INOUT :
  uint16_t *    *this.vCAPTR
  uint16_t *    *this.vTIME0
  uint16_t *    *this.vTIME1
  uint16_t *    *this.vTIME2
  uint16_t *    *this.vTIME3
  uint16_t *    *this.vCLRIG
  uint16_t *    *this.vCR0
  uint16_t *    *this.vCR1
  uint16_t *    *this.vBIMCR0
  uint16_t *    *this.vBIMCR1
  uint16_t *    *this.vBIMCR2
  uint16_t *    *this.vBIMCR3
  uint16_t *    *this.vBIMV0
  uint16_t *    *this.vBIMV1
  uint16_t *    *this.vBIMV2
  uint16_t *    *this.vBIMV3
  uint16_t *    *this.vRELFR
  uint16_t *    *this.vSTR1W1
  uint16_t *    *this.vSTR1W2
  uint16_t *    *this.vSTR1W3
  uint16_t *    *this.vSTR2W1
  uint16_t *    *this.vSTR2W2
  uint16_t *    *this.vSTR2W3
  uint16_t *    *this.vCTC0
  uint16_t *    *this.vCTCCR
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.vu8LocalAddress
  BOARD_NAME *this.boardName
  BOOL       *this.bInit
  BOARD_FUNC *this.bfFunctions 

LOCAL :
  int        i
  ERROR_CODE ec

SPEC :
  [this is invalid -> UNDEFINED
  | this->vmeAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit := TRUE -> return S_pc03v_BOARD_ALREADY_INITIALIZED
  | true ->
    : *this.boardName := boardName
    : *this.bfFunctions := BF_CLOCK | BF_INTERRUPT
    : *this.bInit := TRUE
    : *this.pRate[0..3] :=0
    : *this.ppFunc[0..3] := NULL
    : *this.piArg[0..3] := 0
    : *this.vu8LocalAddress := local address corresponding to vmeBoardAddress.address
    :: *this.vCAPTR= vu8LocalAddress + 0x0
    :: *this.vTIME0= vu8LocalAddress + 0x2
    :: *this.vTIME1= vu8LocalAddress + 0x4
    :: *this.vTIME2= vu8LocalAddress + 0x6
    :: *this.vTIME3= vu8LocalAddress + 0x8
    :: *this.vCLRIG= vu8LocalAddress + 0xA
    :: *this.vCR0= vu8LocalAddress + 0xC
    :: *this.vCR1= vu8LocalAddress + 0xE
    :: *this.vBIMCR0= vu8LocalAddress + 0x20
    :: *this.vBIMCR1= vu8LocalAddress + 0x22
    :: *this.vBIMCR2= vu8LocalAddress + 0x24
    :: *this.vBIMCR3= vu8LocalAddress + 0x26
    :: *this.vBIMV0= vu8LocalAddress + 0x28
    :: *this.vBIMV1= vu8LocalAddress + 0x2A
    :: *this.vBIMV2= vu8LocalAddress + 0x2C
    :: *this.vBIMV3= vu8LocalAddress + 0x2E
    :: *this.vRELFR= vu8LocalAddress + 0x30
    :: *this.vSTR1W1= vu8LocalAddress + 0x42
    :: *this.vSTR1W2= vu8LocalAddress + 0x44
    :: *this.vSTR1W3= vu8LocalAddress + 0x46
    :: *this.vSTR2W1= vu8LocalAddress + 0x62
    :: *this.vSTR2W2= vu8LocalAddress + 0x64
    :: *this.vSTR2W3= vu8LocalAddress + 0x66
    :: *this.vCTC0= vu8LocalAddress + 0x80
    :: *this.vCTCCR= vu8LocalAddress + 0x86
  ]

RETURNS : ERROR_CODE
  S_pc03v_BOARD_ALREADY_INITIALIZED
  OK
*
*/
ERROR_CODE tPc03vBoard::SoftInit(void)
{
  ERROR_CODE ec;
  int        i;

  TraceMark(S_pc03v_ENTERING_SOFTINIT);
  /*
  [ this is invalid -> UNDEFINED
  | this->vmeBoardAddress not reachable by CPU ->
      return vxWorks error for sysBusToLocalAdrs()
  | *this.bInit == TRUE -> return S_pc03v_E_BOARD_ALREADY_INITIALIZED
  | true ->
    : *this.vu8LocalAddress := local address corresponding to this->vmeBoardAddress
    : *this.boardName := boardName
    : *this.bInit := TRUE
  ] */
  ec = this->Initialize();
  if(ec != OK) {
    return Traceback(ec, "tPc03vBoard::SoftInit");
  }

  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.bfFunctions := BF_CLOCK | BF_INTERRUPT
    : *this.pRate[0..3] := 0
    : *this.ppFunc[0..3] := NULL
    : *this.piArg[0..3] := 0
  ] */
  this->bfFunctions = BF_CLOCK | BF_INTERRUPT;
  for(i=0 ; i<4 ; i++) {
    this->pRate[i] = 0;
    this->ppFunc[i] = NULL;
    this->piArg[i] = 0;
  }

  /*
  [ this is invalid -> UNDEFINED
  | *this.vu8LocalAddress is invalid -> UNDEFINED
  | true ->
      set all register address variables as shown in the SPEC section
  ] */
  this->vCAPTR = (uint16_t *) (vu8LocalAddress + 0x0);
  this->vTIME0 = (uint16_t *) (vu8LocalAddress + 0x2);
  this->vTIME1 = (uint16_t *) (vu8LocalAddress + 0x4);
  this->vTIME2 = (uint16_t *) (vu8LocalAddress + 0x6);
  this->vTIME3 = (uint16_t *) (vu8LocalAddress + 0x8);
  this->vCLRIG = (uint16_t *) (vu8LocalAddress + 0xA);
  this->vCR0 = (uint16_t *) (vu8LocalAddress + 0xC);
  this->vCR1 = (uint16_t *) (vu8LocalAddress + 0xE);
  this->vBIMCR0 = (uint16_t *) (vu8LocalAddress + 0x20);
  this->vBIMCR1 = (uint16_t *) (vu8LocalAddress + 0x22);
  this->vBIMCR2 = (uint16_t *) (vu8LocalAddress + 0x24);
  this->vBIMCR3 = (uint16_t *) (vu8LocalAddress + 0x26);
  this->vBIMV0 = (uint16_t *) (vu8LocalAddress + 0x28);
  this->vBIMV1 = (uint16_t *) (vu8LocalAddress + 0x2A);
  this->vBIMV2 = (uint16_t *) (vu8LocalAddress + 0x2C);
  this->vBIMV3 = (uint16_t *) (vu8LocalAddress + 0x2E);
  this->vRELFR = (uint16_t *) (vu8LocalAddress + 0x30);
  this->vSTR1W1 = (uint16_t *) (vu8LocalAddress + 0x42);
  this->vSTR1W2 = (uint16_t *) (vu8LocalAddress + 0x44);
  this->vSTR1W3 = (uint16_t *) (vu8LocalAddress + 0x46);
  this->vSTR2W1 = (uint16_t *) (vu8LocalAddress + 0x62);
  this->vSTR2W2 = (uint16_t *) (vu8LocalAddress + 0x64);
  this->vSTR2W3 = (uint16_t *) (vu8LocalAddress + 0x66);
  this->vCTC0 = (uint16_t *) (vu8LocalAddress + 0x80);
  this->vCTCCR = (uint16_t *) (vu8LocalAddress + 0x86);

  TraceMark(S_pc03v_INITIALIZED_SOFTLY);

  return OK;
}


/*********************************************************************
*
* tPc03vBoard::HardInit - PC03V card Hard Initialize method
*
*
IN :
  this

INOUT :
  uint16_t *    *this.vCAPTR
  uint16_t *    *this.vTIME0
  uint16_t *    *this.vTIME1
  uint16_t *    *this.vTIME2
  uint16_t *    *this.vTIME3
  uint16_t *    *this.vCLRIG
  uint16_t *    *this.vCR0
  uint16_t *    *this.vCR1
  uint16_t *    *this.vBIMCR0
  uint16_t *    *this.vBIMCR1
  uint16_t *    *this.vBIMCR2
  uint16_t *    *this.vBIMCR3
  uint16_t *    *this.vBIMV0
  uint16_t *    *this.vBIMV1
  uint16_t *    *this.vBIMV2
  uint16_t *    *this.vBIMV3
  uint16_t *    *this.vRELFR
  uint16_t *    *this.vSTR1W1
  uint16_t *    *this.vSTR1W2
  uint16_t *    *this.vSTR1W3
  uint16_t *    *this.vSTR2W1
  uint16_t *    *this.vSTR2W2
  uint16_t *    *this.vSTR2W3
  uint16_t *    *this.vCTC0
  uint16_t *    *this.vCTCCR
*** CONTENTS OF ALL ABOVE LISTED REGISTERS ***
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.vu8LocalAddress
  BOARD_NAME *this.boardName
  BOOL       *this.bInit
  INT_RATE   *this.pRate[4]
  BOARD_FUNC *this.bfFunctions 

LOCAL : 
  int        i
  ERROR_CODE ec

SPEC :
  [this is invalid -> UNDEFINED
  | this->vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit := TRUE -> return S_pc03v_BOARD_ALREADY_INITIALIZED
  | true ->
    : *this.boardName := boardName
    : *this.bfFunctions := BF_CLOCK | BF_INTERRUPT
    : *this.bInit := TRUE
    : *this.pRate[0..3] := 0
    : *this.vu8LocalAddress := local address corresponding to this->vmeBoardAddress
    :: *this.vCAPTR:= vu8LocalAddress + 0x0
    :: *this.vTIME0:= vu8LocalAddress + 0x2
    :: *this.vTIME1:= vu8LocalAddress + 0x4
    :: *this.vTIME2:= vu8LocalAddress + 0x6
    :: *this.vTIME3:= vu8LocalAddress + 0x8
    :: *this.vCLRIG:= vu8LocalAddress + 0xA
    :: *this.vCR0:= vu8LocalAddress + 0xC
    :: *this.vCR1:= vu8LocalAddress + 0xE
    :: *this.vBIMCR0:= vu8LocalAddress + 0x20
    :: *this.vBIMCR1:= vu8LocalAddress + 0x22
    :: *this.vBIMCR2:= vu8LocalAddress + 0x24
    :: *this.vBIMCR3:= vu8LocalAddress + 0x26
    :: *this.vBIMV0:= vu8LocalAddress + 0x28
    :: *this.vBIMV1:= vu8LocalAddress + 0x2A
    :: *this.vBIMV2:= vu8LocalAddress + 0x2C
    :: *this.vBIMV3:= vu8LocalAddress + 0x2E
    :: *this.vRELFR:= vu8LocalAddress + 0x30
    :: *this.vSTR1W1:= vu8LocalAddress + 0x42
    :: *this.vSTR1W2:= vu8LocalAddress + 0x44
    :: *this.vSTR1W3:= vu8LocalAddress + 0x46
    :: *this.vSTR2W1:= vu8LocalAddress + 0x62
    :: *this.vSTR2W2:= vu8LocalAddress + 0x64
    :: *this.vSTR2W3:= vu8LocalAddress + 0x66
    :: *this.vCTC0:= vu8LocalAddress + 0x80
    :: *this.vCTCCR:= vu8LocalAddress + 0x86

    :: *this.*vCR0:=   0x0000
    :: *this.*vCR1:=   0x2005
    :: *this.*BIMCR0:= 0x0000
    :: *this.*BIMCR1:= 0x0000
    :: *this.*BIMCR2:= 0x0000
    :: *this.*BIMCR3:= 0x0000
    :: *this.*BIMV0:=  0x000f
    :: *this.*BIMV1:=  0x000f
    :: *this.*BIMV2:=  0x000f
    :: *this.*BIMV3:=  0x000f
  ]

RETURNS : ERROR_CODE
  S_pc03v_BOARD_ALREADY_INITIALIZED
  OK
*
*/
ERROR_CODE tPc03vBoard::HardInit(void)
{
  ERROR_CODE ec;
  int        i;

  TraceMark(S_pc03v_ENTERING_HARDINIT);
  /*
  [ this is invalid -> UNDEFINED
  | this->vmeBoardAddress not reachable by CPU ->
      return vxWorks error for sysBusToLocalAdrs()
  | *this.bInit == TRUE -> return S_pc03v_E_BOARD_ALREADY_INITIALIZED
  | true ->
    : *this.vu8LocalAddress := local address corresponding to this->vmeBoardAddress
    : *this.boardName := boardName
    : *this.bInit := TRUE
    : *this.bfFunctions := BF_CLOCK | BF_INTERRUPT
    : *this.pRate[0..3] := 0
    : *this.ppFunc[0..3] := NULL
    : *this.piArg[0..3] := 0
    : set all register address variables as shown in the SPEC section
  ] */
  ec = this->SoftInit();
  if(ec != OK) {
    return Traceback(ec, "tPc03vBoard::HardInit");
  }

  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.*vCR0:=   0x0000
    : *this.*vCR1:=   0x2005
    : *this.*vBIMCR0:= 0x0000
    : *this.*vBIMCR1:= 0x0000
    : *this.*vBIMCR2:= 0x0000
    : *this.*vBIMCR3:= 0x0000
    : *this.*vBIMV0:=  0x000f
    : *this.*vBIMV1:=  0x000f
    : *this.*vBIMV2:=  0x000f
    : *this.*vBIMV3:=  0x000f
  ] */
  *this->vCR0 =   0x0000;
  *this->vCR1 =   0x2005;
  *this->vBIMCR0 = 0x0000;
  *this->vBIMCR1 = 0x0000;
  *this->vBIMCR2 = 0x0000;
  *this->vBIMCR3 = 0x0000;
  *this->vBIMV0 =  0x000f;
  *this->vBIMV1 =  0x000f;
  *this->vBIMV2 =  0x000f;
  *this->vBIMV3 =  0x000f;

  TraceMark(S_pc03v_INITIALIZED_HARDLY);
  return OK;
}


/*********************************************************************
*
* tPc03vBoard::CreateChannel -  written to conform with tBoard::CreateChannel
*                               Since This board has no channels, this function
*                               should never be called.
*
[[U tPc03vBoard::CreateChannel()]]
*
*/
tChannel *tPc03vBoard::CreateChannel(int iChannelNum)
{
  TraceMsg(tmt_ERROR, S_pc03v_CREATE_CHANNEL_ATTEMPTED,
           "Error: CreateChannel called for PC03V Board\n");
  return NULL;
}


/*********************************************************************
*
* tPc03vBoard::RateSet - set rate of a clock-driven interrupt source
*
IN :
  this
  int            iIntSource
  BOOL           *this.bInit

INOUT :
  INT_RATE       *pRate
  INT_RATE       *this.pRate[iIntSource]
  uint16_t          *this.*vCTC0
  
SPEC :
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | *this.vCTC0 is invalid -> UNDEFINED
  | *this.vCTC0 is unreachable by CPU -> UNDEFINED
  | *pRate < PC03V_MIN_HB_RATE ->
    *pRate := PC03V_MIN_HB_RATE
    : *this.pRate[iIntSource] := PC03V_MIN_HB_RATE
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE) >> 8)
    ]
    : return OK
  | PC03V_MINOR_CLOCK_SPEED modulo *pRate != 0 ->
    *this.pRate[iIntSource]:=
      PC03V_MINOR_CLOCK_SPEED/(PC03V_MINOR_CLOCK_SPEED/ *pRate) 
    : *pRate := PC03V_MINOR_CLOCK_SPEED/(PC03V_MINOR_CLOCK_SPEED/ *pRate)
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/ *pRate - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate) >> 8)
    ]
    : return OK
  | true -> *this.pRate[iIntSource]:= *pRate
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/ *pRate - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate) >> 8)
    ]
    : return OK
  ]
*
* RETURNS : ERROR_CODE
*  S_pc03v_E_BOARD_NOT_INITIALIZED
*  S_pc03v_E_INVALID_INT_SOURCE
*  OK
*
*/

ERROR_CODE tPc03vBoard::RateSet(/*this,*/ INT_RATE *pRate, int iIntSource)
{
  int iPeriod;
  
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  ] */
  if(!this->IsInit()) {
    return S_pc03v_E_BOARD_NOT_INITIALIZED;
  }

  /*
  [ iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  ] */
  if (iIntSource != 1) {
    return S_pc03v_E_INVALID_INT_SOURCE;
  }

  /*
  [ this is invalid -> UNDEFINED
  | *this.vCTC0 is invalid -> UNDEFINED
  | *this.vCTC0 is unreachable by CPU -> UNDEFINED
  | *pRate < PC03V_MIN_HB_RATE ->
    *pRate := PC03V_MIN_HB_RATE
    : *this.pRate[iIntSource] := PC03V_MIN_HB_RATE
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/PC03V_MIN_HB_RATE) >> 8)
    ]
    : return OK
  | PC03V_MINOR_CLOCK_SPEED modulo *pRate != 0 ->
    *this.pRate[iIntSource]:=
      PC03V_MINOR_CLOCK_SPEED/(PC03V_MINOR_CLOCK_SPEED/ *pRate) 
    : *pRate := PC03V_MINOR_CLOCK_SPEED/(PC03V_MINOR_CLOCK_SPEED/ *pRate)
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/ *pRate - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate) >> 8)
    ]
    : return OK
  | true -> *this.pRate[iIntSource]:= *pRate
    [ *this.*vCR1 & 0x2000 == 0x2000 -> *this.*vCTCCR = 0x003a
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate - 1)
      :: *this.*vCTC0 :=
          0x00ff & ((PC03V_MINOR_CLOCK_SPEED/ *pRate - 1) >> 8)
    | true -> *this.*vCTCCR := 0x0034
      : *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate)
      :: *this.*vCTC0 :=
          0x00ff & (PC03V_MINOR_CLOCK_SPEED/ *pRate) >> 8)
    ]
    : return OK
  ] */
  if(*pRate<PC03V_MIN_HB_RATE) {
    *pRate = PC03V_MIN_HB_RATE;
  }

  iPeriod = PC03V_MINOR_CLOCK_SPEED / *pRate;
  *pRate = this->pRate[iIntSource] = PC03V_MINOR_CLOCK_SPEED / iPeriod;
  
  if(this->IsHbSynced()) {
    *this->vCTCCR = 0x003a;
    *this->vCTC0  = 0x00ff & (iPeriod - 1);
    *this->vCTC0  = 0x00ff & ((iPeriod - 1) >> 8);
  }
  else {
    *this->vCTCCR = 0x0034;
    *this->vCTC0 = 0x00ff & (iPeriod);
    *this->vCTC0  = 0x00ff & (iPeriod >> 8);
  }

  return OK;
}

/*********************************************************************
*
* tPc03vBoard::RateGet - get current rate setting on PC03V board
*
IN :
  this
  int             iIntSource

INOUT :
  INT_RATE *this.pRate[iIntSource]

SPEC :
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED
  | iIntSource != 1 -> UNDEFINED
  | *this.*pRate is out of sync with actual rate -> incorrect response
  | true -> return (*this.pRate[iIntSource])
  ]

RETURNS : INT_RATE
*
*/
INT_RATE tPc03vBoard::RateGet(/*this*/  int iIntSource)
{
  return this->pRate[iIntSource];
}

/*********************************************************************
*
* tPc03vBoard::Connect - Connect a function to an interrupt
*
INOUT:
  this
  *this.*vmeBIMCR[iIntSource]
  *this.*vmeBIMV[iIntSource]
  FUNCPTR       ppFunc[iIntSource]
  int           iArg[iIntSource]

IN :
  FUNCPTR       pFunc
  int           iArg
  int           iIntSource
  unsigned int  uiIntNum
  unsigned int  uiIntVector

SPEC :
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | pFunc is invalid -> UNDEFINED
  | pFunc == NULL -> return S_pc03v_E_NULL_FUNCTION_POINTER
  | uiIntNum < MIN_BUS_INT_PRIORITY || uiIntNum > MAX_BUS_INT_PRIORITY ->
      return S_pc03v_E_INVALID_INTERRUPT_NUMBER
  | true -> *this.ppFunc[iIntSource] := pFunc
    : *this.piArg[iIntSource] := iArg
    : *this.vCR0 := *this.*vCR0 | 0x0010
    : *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] | uiIntNum
    : *this.*vBIMV[iIntSource] := uiIntVector
    : Attach pFunc to Interrupt uiIntNum at uiIntVector
    : return OK
  ]

RETURNS : ERROR_CODE
S_pc03v_E_BOARD_NOT_INITIALIZED
S_pc03v_E_INVALID_INT_SOURCE
S_pc03v_E_NULL_FUNCTION_POINTER
S_pc03v_E_INVALID_INTERRUPT_NUMBER
OK
*
*/
#ifndef INCR1
#error "iArg is not used because we need to pass in the this ptr to intConnect"
#endif
ERROR_CODE tPc03vBoard::Connect(/*this*/
                           FUNCPTR pFunc,
                           int iArg,
                           unsigned int uiIntNum,
                           unsigned int uiIntVector,
                           int iIntSource)
{
  ERROR_CODE ec;
  unsigned int mbi = MIN_BUS_INT_PRIORITY;
  
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED 
  | true -> NOP 
  ] */
  if(!this->IsInit()) {
    return S_pc03v_E_BOARD_NOT_INITIALIZED;
  }

  /*
  [ iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> NOP
  ] */
  if (iIntSource != 1) {
    return S_pc03v_E_INVALID_INT_SOURCE;
  }
  
  /*
  [ this is invalid -> UNDEFINED
  | pFunc is invalid -> UNDEFINED
  | pFunc == NULL -> return S_pc03v_E_NULL_FUNCTION_POINTER
  | true -> NOP
  ] */
  if(pFunc == NULL) {
    return S_pc03v_E_NULL_FUNCTION_POINTER;
  }

  /*
  [ uiIntNum < MIN_BUS_INT_PRIORITY || uiIntNum > MAX_BUS_INT_PRIORITY ->
      return S_pc03v_E_INVALID_INTERRUPT_NUMBER
  | true -> NOP
  ] */
  if (uiIntNum < mbi /* =MIN_BUS_INT_PRIORITY */  ||
      uiIntNum > MAX_BUS_INT_PRIORITY) {
    return S_pc03v_E_INVALID_INTERRUPT_NUMBER;
  }

  /*
  [ this is invalid -> UNDEFINED
  | true -> *this.ppFunc[iIntSource] := pFunc
    : *this.piArg[iIntSource] := iArg
  ] */
  this->ppFunc[iIntSource] = pFunc;
  this->piArg[iIntSource] = iArg;

  /*
  NAME 
       intConnect( ) - connect a C routine to a hardware interrupt 

  SYNOPSIS 
       STATUS intConnect
           (
           VOIDFUNCPTR *  vector,    // interrupt vector to attach to
           VOIDFUNCPTR    routine,   // routine to be called
           int            parameter  // parameter to be passed to routine
           )

  DESCRIPTION 
       This routine connects a specified C routine to a specified interrupt vector. The 
       address of routine is stored at vector so that routine is called with parameter when the 
       interrupt occurs. The routine is invoked in supervisor mode at interrupt level. A proper 
       C environment is established, the necessary registers saved, and the stack set up.
       The routine can be any normal C code, except that it must not invoke certain 
       operating system functions that may block or perform I/O operations.
       This routine simply calls intHandlerCreate( ) and intVecSet( ). The address of the 
       handler returned by intHandlerCreate( ) is what actually goes in the interrupt vector. 

RETURNS 
       OK, or ERROR if the interrupt handler cannot be built. 
  */
  /*
  NAME 
       sysIntEnable( ) - enable a bus interrupt level 

  SYNOPSIS 
       STATUS sysIntEnable
           (
           int  intLevel  // interrupt level to enable (1-7)
           )

  DESCRIPTION 
       This routine enables a specified bus interrupt level.
       NOTE: This is a generic man page for a BSP-specific routine; this description 
       contains general information only. To determine if this routine is supported by your 
       BSP, or for information specific to your BSPs version of this routine, see the man 
       pages for your BSP. 

  RETURNS 
       OK, or ERROR if intLevel is out of range. 

  */
  /*
  [true -> *this.vCR0 := *this.*vCR0 | 0x0010
    : *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] | uiIntNum
    : *this.*vBIMV[iIntSource] := uiIntVector
    : Attach pFunc to Interrupt uiIntNum at uiIntVector
    : return OK
  ] */
  #ifndef INCR1
  #error "Fix Spec to match bug fixes"
  #endif
  *this->vCR1    |= 0x0010;
  *this->vBIMCR1 = (*this->vBIMCR1 & 0xf8) | (uint16_t) (uiIntNum & 0x7);
  *this->vBIMV1   = (uint16_t) uiIntVector;
  ec = intConnect((VOIDFUNCPTR *)INUM_TO_IVEC(uiIntVector),
                 (VOIDFUNCPTR) this->ClkIsr, (int) this);
  if(ec!=OK) {
    return Traceback(ec, "tPc03vBoard::Connect",
                     pFunc, iArg, uiIntNum, uiIntVector, iIntSource);
  }

  ec = sysIntEnable((int) uiIntNum);
  if(ec!=OK) {

    return Traceback(ec, "tPc03vBoard::Connect",
                     pFunc, iArg, uiIntNum, uiIntVector, iIntSource);
  }

  return OK;
}

/*********************************************************************
*
* tPc03vBoard::Disconnect - Disconnect a function from an interrupt
*
IN :
  this
  int iIntSource
*
INOUT:
  *this.*vmeBIMCR[iIntSource]
  *this.*vmeBIMV[iIntSource]

  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> *this.puiIntNum[iIntSource] := uiIntNum
    : *this.vCR0 := *this.*vCR0 & 0xffef
    : *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] & 0xfff8
    : *this.*vBIMV[iIntSource] := 0x000f
    : Disable VME Interrupt stored in iIntSource
    : return OK
  ]
*
RETURNS : ERROR_CODE
S_pc03v_E_BOARD_NOT_INITIALIZED
S_pc03v_E_INVALID_INT_SOURCE
OK
*
*/
ERROR_CODE tPc03vBoard::Disconnect(/*this*/ int iIntSource)
{
  ERROR_CODE ec;

  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | true -> NOP
  ] */
  if(!this->IsInit()) {
    return S_pc03v_E_BOARD_NOT_INITIALIZED;
  }

  /*
  [ iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> NOP
  ] */
  if(iIntSource != 1) {
    return S_pc03v_E_INVALID_INT_SOURCE;
  }

  /* 
            man page for sysIntDisable goes here
  */
  /*
  [ true -> *this.puiIntNum[iIntSource] := uiIntNum
    : *this.vCR0 := *this.*vCR0 & 0xffef
    : *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] & 0xfff8
    : *this.*vBIMV[iIntSource] := 0x000f
    : Disable Interrupt stored in iIntSource
    : return OK
  ] */
  ec = sysIntDisable((int) *this->vBIMCR1 & 0x0007);
  if(ec!=OK) {
    return Traceback(ec, "tPc03vBoard::Disconnect", iIntSource);
  }

  *this->vCR1    &= ~0x0010;
  *this->vBIMCR1 &= ~MAX_BUS_INT_PRIORITY;
  *this->vBIMV1   = 0x000f;

  return OK;
}


/*********************************************************************
*
* tPc03vBoard::Enable - Enable an interrupt
*
INOUT:
  this
  *this.*vmeBIMCR[iIntSource]

IN :
  int iIntSource

[ this is invalid -> UNDEFINED
| *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
| iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
| true -> *this.pbActivated[iIntSource] := TRUE
  : *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] | 0x0010
  : Begin sending interrupt number *this.puiIntNum[iIntSource] to the
    VME bus.  The ISR on the CPU connected to *this.puiIntNum[iIntSource]
    will be called at a rate of *this.*ppRate[iIntSource] until a Disable
    command is given (see Below).
  : return OK
]

RETURNS : ERROR_CODE
S_pc03v_E_BOARD_NOT_INITIALIZED
S_pc03v_E_INVALID_INT_SOURCE
OK
*
*/

ERROR_CODE tPc03vBoard::Enable(/*this*/ int iIntSource)
{
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | true -> NOP
  ]  */
  if(!this->IsInit()) {
    return S_pc03v_E_BOARD_NOT_INITIALIZED;
  }

  /*
  [ iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> NOP
  ] */
  if(iIntSource != 1) {
    return S_pc03v_E_INVALID_INT_SOURCE;
  }

  /*
  [ true -> *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] | 0x0010
    : Begin sending interrupt number *this.puiIntNum[iIntSource] to the
      VME bus.  The ISR on the CPU connected to *this.puiIntNum[iIntSource]
      will be called at a rate of *this.*ppRate[iIntSource] until a Disable
      command is given (see Below).
    : return OK
  ] */
  *this->vBIMCR1 |= 0x0010;

  return OK;
}

 
/*********************************************************************
*
*tPc03vBoard::Disable - Disable an interrupt
*
INOUT:
  this, *this
  *this.*vmeBIMCR[iIntSource]

IN :
  int iIntSource

  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] & 0xffef
    : Stop sending interrupt number *this.puiIntNum[iIntSource] to the
      VME bus.  The ISR on the CPU connected to *this.puiIntNum[iIntSource]
      will stop being called periodically.
    : return OK 
  ]

RETURNS : ERROR_CODE
S_pc03v_E_BOARD_NOT_INITIALIZED
S_pc03v_E_INVALID_INT_SOURCE
OK
*
*/
ERROR_CODE tPc03vBoard::Disable(/*this,*/ int iIntSource)
{
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
  | true -> NOP
  ] */
  if(!this->IsInit()) {
    return S_pc03v_E_BOARD_NOT_INITIALIZED;
  }
  
  /*
  [ iIntSource != 1 -> return S_pc03v_E_INVALID_INT_SOURCE
  | true -> NOP
  ] */
  if(iIntSource != 1) {
    return S_pc03v_E_INVALID_INT_SOURCE;
  }

  /*
  [true -> *this.*vBIMCR[iIntSource] := *this.*vBIMCR[iIntSource] & 0xffef
    : Stop sending interrupt number *this.puiIntNum[iIntSource] to the
      VME bus.  The ISR on the CPU connected to *this.puiIntNum[iIntSource]
      will stop being called periodically.
    : return OK 
  ] */
  *this->vBIMCR1 &= 0xffef;

  return OK;
}


/*********************************************************************
*
*tPc03vBoard::GetTimeLastInt - Gets time of last interrupt in 25's of usec
*
IN :
  this
  *this.bInit
  *this.time registers

INOUT :

  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED 
  | *this.*pBoard.bInit == FALSE -> UNDEFINED
  | *this.*pBoard does not have clock function -> UNDEFINED
  | manual capture (GetTimeNow) is performed during this function -> UNDEFINED
  | true -> 
      return (TIME)
      ((UW_NIBBLE(*this.vTIME3, 1) * 10 + UW_NIBBLE(*this.vTIME3, 0)) / 25 +
      UW_NIBBLE(*this.vTIME3, 2) * 4 +
      UW_NIBBLE(*this.vTIME3, 3) * 40 +
      UW_NIBBLE(*this.vTIME2, 0) * 400 +
      UW_NIBBLE(*this.vTIME2, 1) * 4000 +
      UW_NIBBLE(*this.vTIME2, 2) * 40000 +
      UW_NIBBLE(*this.vTIME2, 3) * 400000 +
      UW_NIBBLE(*this.vTIME1, 0) * 2400000 +
      UW_NIBBLE(*this.vTIME1, 1) * 24000000 +
      UW_NIBBLE(*this.vTIME1, 2) * 144000000 +
      UW_NIBBLE(*this.vTIME1, 3) * 1440000000)]
  ]

RETURNS : TIME
*
*/

//#define DEBUG_INTERRUPTS

// ********************* kludgy debug stuff *********************************
#ifdef DEBUG_INTERRUPTS
const int CiIrigTimeCheckArrayLength = 10000;
int iInterruptPeriod = 200;
int iIrigTimeCheckIndex = 0;
int iIrigTimeCheckArray[CiIrigTimeCheckArrayLength];
BOOL bWrap = TRUE;
int iStopIndex = CiIrigTimeCheckArrayLength;

#ifdef __cplusplus
extern "C" {
void LoadIrigArray(void) {
  iIrigTimeCheckIndex = 0;
}
#endif /* __cplusplus */

void CheckIrigArray(void) {
  int i, j, iCurrent, iNext, iCurrentArray[4], iNextArray[4];
  BOOL bWackoTiming = FALSE;
  
  for(i=0; i < (CiIrigTimeCheckArrayLength-2); i++) {

    iCurrentArray[0] = UW_NIBBLE(iIrigTimeCheckArray[i], 0);
    iCurrentArray[1] = UW_NIBBLE(iIrigTimeCheckArray[i], 1);
    iCurrentArray[2] = UW_NIBBLE(iIrigTimeCheckArray[i], 2);
    iCurrentArray[3] = UW_NIBBLE(iIrigTimeCheckArray[i], 3);
    iCurrent = iCurrentArray[0] + iCurrentArray[1] * 10 +
               iCurrentArray[2] * 100 + iCurrentArray[3] * 1000;

    iNextArray[0]    = UW_NIBBLE(iIrigTimeCheckArray[i+1], 0);
    iNextArray[1]    = UW_NIBBLE(iIrigTimeCheckArray[i+1], 1);
    iNextArray[2]    = UW_NIBBLE(iIrigTimeCheckArray[i+1], 2);
    iNextArray[3]    = UW_NIBBLE(iIrigTimeCheckArray[i+1], 3);
    iNext = iNextArray[0] + iNextArray[1] * 10 +
            iNextArray[2] * 100 + iNextArray[3] * 1000;

    /* Check for invalid BCD values */
    for (j=0; j<4; j++) {
      if (iNextArray[j] > 9) {
        printf("Array[%d]: %d %d %d %d\n", i, iCurrentArray[3], iCurrentArray[2],
                 iCurrentArray[1], iCurrentArray[0]);
        printf("Array[%d]: %d %d %d %d\n", i+1, iNextArray[3], iNextArray[2],
                 iNextArray[1], iNextArray[0]);
      }
    }


    if((iCurrent % 25) != 0) {
      if(!bWackoTiming) {
        bWackoTiming = TRUE;
        printf("IRIG read error: time value: %d, index: %d\n",iCurrent, i);
        printf("Array[%d]: %d %d %d %d\n",i,iCurrentArray[3],iCurrentArray[2],
                 iCurrentArray[1], iCurrentArray[0]);
        printf("Array[%d]: %d %d %d %d\n", i+1, iNextArray[3], iNextArray[2],
                 iNextArray[1], iNextArray[0]);
      }
      continue;
    }

    if(bWackoTiming) {
      bWackoTiming = FALSE;
      printf("Read errors stopped at %d\n",i);
    }

    if(((iCurrent == 10000 - iInterruptPeriod) && (iNext != 0)) ||
       ((iCurrent != 10000 - iInterruptPeriod) &&
          (iCurrent != (iNext - iInterruptPeriod))))
      printf("Array[%d] = %d, Array[%d] = %d, diff is %d\n",
                     i, iCurrent,  i+1,  iNext, iNext-iCurrent);
  }
}
} /* extern C */

#endif /* DEBUG_INTERRUPTS */


int iGlobalSmallTime = 0;

// ********************* end kludgy debug stuff *****************************

TIME tPc03vBoard::GetTimeLastInt(/*this*/)
{
  FAST uint16_t uTime1, uTime2, uTime3;
  FAST uint32_t iSmallTime, iMicroTime;
  /*
  [ this is invalid -> UNDEFINED
  | *this.bInit == FALSE -> UNDEFINED 
  | *this.*pBoard.bInit == FALSE -> UNDEFINED
  | *this.*pBoard does not have clock function -> UNDEFINED
  | manual capture (GetTimeNow) is performed during this function -> UNDEFINED
  | true -> return (TIME) 
  ((UW_NIBBLE(*this.vTIME3, 1) * 10 + UW_NIBBLE(*this.vTIME3, 0)) / 25 +
   UW_NIBBLE(*this.vTIME3, 2) * 4 +
   UW_NIBBLE(*this.vTIME3, 3) * 40 +
   UW_NIBBLE(*this.vTIME2, 0) * 400 +
   UW_NIBBLE(*this.vTIME2, 1) * 4000 +
   UW_NIBBLE(*this.vTIME2, 2) * 40000 +
   UW_NIBBLE(*this.vTIME2, 3) * 400000 +
   UW_NIBBLE(*this.vTIME1, 0) * 2400000 +
   UW_NIBBLE(*this.vTIME1, 1) * 24000000 +
   UW_NIBBLE(*this.vTIME1, 2) * 144000000 +
   UW_NIBBLE(*this.vTIME1, 3) * 1440000000))
  ] */
  
  uTime1 = *this->vTIME1;
  uTime2 = *this->vTIME2;
  uTime3 = *this->vTIME3;


  /****
  * iSmallTime is the least two significant digits of the time register.
  * The time value returned is in 25's of microseconds.  We have observed some
  * timing errors that are occasionally as much as 40 microseconds off.
  * These errors
  * can cause this function to return a value that is one less than it should
  * be, because of the integer division truncate.  We avoid this problem by
  * adding 24 to the 3 LSDs before dividing by 25.  This actually decreases
  * our time resolution to 50's of microseconds, but the units are still
  * 25's of microseconds 
  ****/
  iGlobalSmallTime = 
               UW_NIBBLE(uTime3, 2)*100 + UW_NIBBLE(uTime3, 1)*10 +
               UW_NIBBLE(uTime3, 0);

  iSmallTime = (iGlobalSmallTime + 24) / 50 * 2;


#ifdef DEBUG_INTERRUPTS
  iMicroTime = iSmallTime + (UW_NIBBLE(uTime3, 2) * 4); 

  if(bWrap && (iIrigTimeCheckIndex >= CiIrigTimeCheckArrayLength)) {
    iIrigTimeCheckIndex = 0;
  }

  if (iIrigTimeCheckIndex < CiIrigTimeCheckArrayLength) {
    iIrigTimeCheckArray[iIrigTimeCheckIndex++] = uTime3;
  }

  if ((iIrigTimeCheckIndex == iStopIndex) && (!bWrap)) {
    iIrigTimeCheckIndex = CiIrigTimeCheckArrayLength;
  }

  if (bWrap && ((iMicroTime%(iInterruptPeriod/25)) != 0)) {  
    int i;
    bWrap = FALSE;
    iStopIndex = iIrigTimeCheckIndex - 1250;  // 1/4 array pretrigger
    logMsg("Timing Problem Occurred, iMicroTime = %d (CurrentIndex = %d)\n",
			iMicroTime,iIrigTimeCheckIndex,0,0,0,0);
    for (i=-10; i<20; i++) {
      logMsg("[%d]: %d\n", iIrigTimeCheckIndex + i, 
                         iIrigTimeCheckArray[iIrigTimeCheckIndex + i],
                         0,0,0,0);
    }
  }
#endif /* DEBUG_INTERRUPTS */

  return (TIME)
  (iSmallTime +
   UW_NIBBLE(uTime3, 3) * 40 +
   UW_NIBBLE(uTime2, 0) * 400 +
   UW_NIBBLE(uTime2, 1) * 4000 +
   UW_NIBBLE(uTime2, 2) * 40000 +
   UW_NIBBLE(uTime2, 3) * 400000 +
   UW_NIBBLE(uTime1, 0) * 2400000 +
   UW_NIBBLE(uTime1, 1) * 24000000 +
   UW_NIBBLE(uTime1, 2) * 144000000 +
   UW_NIBBLE(uTime1, 3) * 1440000000);
}

/*********************************************************************
*
* tPc03vBoard::IsHbSynced - Returns TRUE if the Heartbeat is synced to 1PPS
*
IN :
  this
  *this.*vCR1

SPEC :
  [ this is invalid -> UNDEFINED
  | *this.vCR1 is invalid -> UNDEFINED
  | *this.*vCR1 & 0x2000 == 0x2000 -> return TRUE
  | true -> return FALSE
  ]

RETURNS : BOOL
*
*/
BOOL tPc03vBoard::IsHbSynced(/*this*/ void)
{

  /*
  [ this is invalid -> UNDEFINED
  | *this.vCR1 is invalid -> UNDEFINED
  | *this.*vCR1 & 0x2000 == 0x2000 -> return TRUE
  | true -> return FALSE
  ] */
  if(*this->vCR1 & 0x2000 == 0x2000) {
    return TRUE;
  }
  else {
    return FALSE;
  }
}

/*********************************************************************
*
* tPc03vBoard::ClkIsr - ISR for heartbeat interrupt
*
IN :
  this
  int iIntSource  ==> hard-coded to 1
  *this.*vRELFR
  *this.ppFunc[iIntSource]
  *this.piArg[iIntSource]

SPEC :
  [ this is invalid -> UNDEFINED
  | board has not been initialized -> UNDEFINED
  | true -> release capture lockout
    : call *this.ppFunc[iIntSource] with argument *this.piArg[iIntSource]
  ]

LOCAL : 
  int iTmp

RETURNS : void
*
*/
#ifndef INCR1
#error "iIntSource hard-coded to 1 because we need to pass this pointer"
#endif
void tPc03vBoard::ClkIsr(/*this*/ int iArg)
{
  int iTmp;

  /*
  [ this is invalid -> UNDEFINED
  | board has not been initialized -> UNDEFINED
  | true -> release capture lockout
    : call *this.ppFunc[iIntSource] with argument *this.piArg[iIntSource]
  ] */
  (*this->ppFunc[1])(this->piArg[1]);
  iTmp = *this->vRELFR;  /* accessing *vRELFR causes capture lock release */
}

