/* $RCSfile: vip640a.cc,v $ - This module is an interface to the Matrox VIP 640a Video Capture Card  */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/drivers/vip640a.cc,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

#include <semLib.h>
#include <vme.h>
#include "isit.h"
#include "hwTypes.h"
#include "hwError.h"
#include "Scheduler.h"
#include "board.h"
#include "channel.h"
#include "vip640a.h"
#include "vip640aConfig.h"

/* Caution: DO NOT ALTER ANY OF THESE NUMBERS.  OTHER       */
/*          COMBINATIONS DO NOT WORK AT ALL SPEEDS.         */
const int iVip640aResX[NUM_HIGH_RES_PAGES] = { 0, 0 };
const int iVip640aResY[NUM_HIGH_RES_PAGES] = { 0, 30};

#define INCLUDE_IMAGE_CHANNEL_TEST

#ifdef INCLUDE_IMAGE_CHANNEL_TEST
tFullImageChannel *testImageChannel;
unsigned long int bufSwCnt = 0;
#endif

/*************************************************************
  tVip640aFullImageChannel::tVip640aFullImageChannel();
  
[[U tVip640aFullImageChannel::tVip640aFullImageChannel(void) ]]
*/
tVip640aFullImageChannel::tVip640aFullImageChannel() {
  this->SetInit(FALSE);
  this->iHSize = 0;
  this->iVSize = 0;
  this->pidUpdateSignaler = 0;
  this->bContinuousGrab = FALSE;
  }

/*************************************************************
  tVip640aFullImageChannel::~tVip640aFullImageChannel();
  
[[U tVip640aFullImageChannel::~tVip640aFullImageChannel(void) ]]
*/
tVip640aFullImageChannel::~tVip640aFullImageChannel() {
  //::GetScheduler().DeleteTask(this->pidUpdateSignaler);
  }

/*************************************************************
  ERROR_CODE tVip640aFullImageChannel::Initialize(tVmeAddress vmeRegAddr, tVmeAddress vmeFrBufAddr);
  
[[U tVip640aFullImageChannel::Initialize(vmeRegAddr, vmeFrBufAddr) ]]
*/
ERROR_CODE tVip640aFullImageChannel::Initialize(tVmeAddress vmeRegAddr, tVmeAddress vmeFrBufAddr) {
  volatile uint8_t *vRegAddr;

  if (this->IsInit() == TRUE)
    return S_tVip640aFullImageChannel_CHANNEL_ALREADY_INITIALIZED;

  this->vFrBuffAddr = (uint8_t *) vmeFrBufAddr.GetLocalAddress();
  if (this->vFrBuffAddr == NULL) {
    this->SetInit(FALSE);
    return Traceback(S_hw_LOCAL_ADDRESS_SET_TO_ZERO, "tVip640aFullImageChannel::Initialize");
    }

  vRegAddr = (uint8_t *) vmeRegAddr.GetLocalAddress();
  if (vRegAddr == NULL) {
    this->SetInit(FALSE);
    return Traceback(S_hw_LOCAL_ADDRESS_SET_TO_ZERO, "tVip640aFullImageChannel::Initialize");
    }

  this->vSnapShotReg  = (uint8_t *) (vRegAddr + SNAPSHOT_REG);  
  this->vStatReg      = (uint8_t *) (vRegAddr + STAT_REG);
  this->vCrtcAddrReg  = (uint8_t *) (vRegAddr + CRTC_ADDR_REG);
  this->vCrtcStatReg  = (uint8_t *) (vRegAddr + CRTC_STAT_REG);
  this->vCrtcDataReg  = (uint8_t *) (vRegAddr + CRTC_DATA_REG);

  this->bContinuousGrab = FALSE;

  this->iHSize = PAGE_X_DIM;
  this->iVSize = PAGE_Y_DIM;

  this->SelectBuf(0);
  this->iWhichBuffer = 1;
      
  /* Start first Grab */
  *this->vSnapShotReg = SNAPSHOT;

  this->SetInit(TRUE);
  return OK;
  }

/***********************************************************************
  ERROR_CODE tVip640aFullImageChannel::GetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image);
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.

[[U tVip640aFullImageChannel::GetImageRegion(iStartX, iStartY, iHSize, iVSize, Image) ]]
*/

ERROR_CODE tVip640aFullImageChannel::GetImageRegion (int iStartX, int iStartY, int iHSize, int iVSize, tImage &Image) {
  int xPos, yPos;
  int bits08, bits917, bit18, bit19;
  int xLoopStart, yLoopStart;
  int xPageSt, yPageSt;
  volatile UINT8 *vFrBuffRow;

  if (this->IsInit() == FALSE)
    return S_tVip640aFullImageChannel_CHANNEL_NOT_INITIALIZED;

  if (this->pidUpdateSignaler == 0)
    return S_tVip640aFullImageChannel_CHANNEL_CAPTURE_NOT_STARTED;

  /****
  [[U Image.tImage::ResizeImage(iHSize, iVSize) ]]
  ****/
  Image.ResizeImage(iHSize, iVSize);

  xPageSt = iVip640aResX[this->iWhichBuffer] * 8;
  yPageSt = iVip640aResY[this->iWhichBuffer] * 16;

  xLoopStart = xPageSt + iStartX;
  yLoopStart = yPageSt + iStartY;  //This line starts Y at top of img

  for (yPos = 0; yPos < iVSize; yPos++) {

    bits917 = (yLoopStart + yPos) & 0x1FF;   // To start from top of img, yPos
    bit19   = (yLoopStart + yPos) & 0x200;   // should be added to yStartLoop

    vFrBuffRow = this->vFrBuffAddr + ((bits917 << 9) | (bit19 << 10));

    for (xPos = 0; xPos < iHSize; xPos++) {

      bits08  = (xPos + xLoopStart) & 0x1FF;
      bit18   = (xPos + xLoopStart) & 0x200;

      /****
      [[U Image.tImage::GetPixel(xPos, yPos) ]]
      ****/
      Image.GetPixel(xPos, yPos) = *(vFrBuffRow + (bits08 | (bit18 << 9)));
      }
    }

  return OK;
  }

/***********************************************************************
  ERROR_CODE tVip640aFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image);
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.  The image is guaranteed to be from 
     a single frame.

[[U tVip640aFullImageChannel::AtomicGetImageRegion(iStartX, iStartY, iHSize, iVSize, Image) ]]
*/
ERROR_CODE tVip640aFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                                     int iHSize, int iVSize, tImage &Image) {
  ERROR_CODE ec;

  if (this->IsInit() == FALSE)
    return S_tVip640aFullImageChannel_CHANNEL_NOT_INITIALIZED;
                                       
  if (this->pidUpdateSignaler == 0)
    return S_tVip640aFullImageChannel_CHANNEL_CAPTURE_NOT_STARTED;

  this->bContinuousGrab = FALSE;
  /****
  [[U tVip640aFullImageChannel::GetImageRegion(iStartX, iStartY, iHSize, iVSize, Image) ]]
  ****/
  ec = this->GetImageRegion(iStartX, iStartY, iHSize, iVSize, Image);
  this->bContinuousGrab = TRUE;
  return ec;
  }                                     

/*************************************************************
  ERROR_CODE tVip640aFullImageChannel::Reset();
     This method puts restores the tVip640aFullImageChannel to a normal operating condition.
     
[[U tVip640aFullImageChannel::Reset(void) ]]
*/
ERROR_CODE tVip640aFullImageChannel::Reset() {
  if (this->IsInit() == FALSE) 
    return S_tVip640aFullImageChannel_CHANNEL_NOT_INITIALIZED;
    
  if (this->pidUpdateSignaler == 0)
    return S_tVip640aFullImageChannel_CHANNEL_CAPTURE_NOT_STARTED;

  this->bContinuousGrab = TRUE;
  return OK;
  }

/*************************************************************
  ERROR_CODE tVip640aFullImageChannel::Update();
     This method starts and waits for an update to occure.

[[U tVip640aFullImageChannel::Update(void) ]]
*/
ERROR_CODE tVip640aFullImageChannel::Update() {
  if (this->IsInit() == FALSE)
    return S_tVip640aFullImageChannel_CHANNEL_NOT_INITIALIZED;
    
  if (this->pidUpdateSignaler == 0)
    return S_tVip640aFullImageChannel_CHANNEL_CAPTURE_NOT_STARTED;

  if (semTake (this->semUpdateFinished, WAIT_FOREVER))
    return S_tVip640aFullImageChannel_CHANNEL_CAPTURE_TIMEOUT;
  return OK;
  }

/*************************************************************
  ERROR_CODE tVip640aFullImageChannel::StartCapture();
     This method initially puts the tFullImageChannel into a normal operating condition.
     This should only be called when images are going to be needed soon.  Depending on
     hardware, this method may need schedular to be initalize before this method is called.
     
[[U tVip640aFullImageChannel::StartCapture(void) ]]
*/
ERROR_CODE tVip640aFullImageChannel::StartCapture() {
  static unsigned long int period = VIP640A_INT_PERIOD;
  static unsigned long int offset = VIP640A_INT_OFFSET;
  if (this->IsInit() == FALSE)
    return S_tVip640aFullImageChannel_CHANNEL_NOT_INITIALIZED;

  this->semUpdateFinished = semBCreate(SEM_Q_FIFO, SEM_FULL);
  ::GetScheduler().AddTask(&this->pidUpdateSignaler, "Vip640aInterrupt", (FUNCPTR) this->UpdateSignaler,
                           NULL, NULL, this, VIP640A_INT_PRI, &period, &offset,
                           TEB_CONTINUE, TRUE);
  this->bContinuousGrab = TRUE;
  return OK;
  }

/*************************************************************
*
* tVip640aFullImageChannel::SelectBuf - Sets the CRTC to the start of the desired page
*
* RETURNS:
*
* OK if there are no errors, ERROR otherwise.
*
*/

ERROR_CODE tVip640aFullImageChannel::SelectBuf(int bufferNum) {
  /* Coordinates of the displayed screen:                     */
  int horizPos;
  int vertPos;

  if (bufferNum > NUM_HIGH_RES_PAGES - 1 || bufferNum < 0) {
    this->iWhichBuffer = 0;
    bufferNum = 0;
    }

  horizPos = iVip640aResX[bufferNum];
  vertPos = iVip640aResY[bufferNum];

  /* Set up the CRTC:              */
  *this->vCrtcAddrReg = DISP_START_ADDRH;
  *this->vCrtcDataReg = vertPos;
  *this->vCrtcAddrReg = DISP_START_ADDRL;
  *this->vCrtcDataReg = horizPos;
  return OK;
  }

/*************************************************************
*
* tVip640aFullImageChannel::UpdateSignaler - Gives semaphore when the grabbing ends.
*
*/

ERROR_CODE tVip640aFullImageChannel::UpdateSignaler (void) 
{
  static BOOL bWasVertBlnk = FALSE;

#ifdef SHOW_UPDATE_RATE
  static int cnt = 0;

  cnt++;
  cnt %= 40000/VIP640A_INT_PERIOD;
  if (cnt == 0) {
    logMsg("%i\tUpdate Signaller called %i times, %i Buffer Swaps have occured\n", tickGet(),
           40000/VIP640A_INT_PERIOD, bufSwCnt,0,0,0);
    bufSwCnt = 0;
    }
#endif
 
  /* Check if grab is done */
  if ((*this->vCrtcStatReg & VERT_BLANK_MASK) && (*this->vStatReg & ODD_FIELD_MASK)) {
    if (bContinuousGrab) {
      if (!bWasVertBlnk) {      
        /* Switch Buffers */
        this->SelectBuf(this->iWhichBuffer);
        if (this->iWhichBuffer == 0)
          this->iWhichBuffer = 1;
        else
          this->iWhichBuffer = 0;
      
        // /* Start next Grab */
        // *this->vSnapShotReg = SNAPSHOT;
    
        /* Indicate, Update Complete */
        semGive(this->semUpdateFinished);

        #ifdef SHOW_UPDATE_RATE
        bufSwCnt++;
        #endif        
      }
      bWasVertBlnk = TRUE;
    }
  }
  else bWasVertBlnk = FALSE;
  return OK;
}

/****************************************************************************
  tVip640aBoard::tVip640aBoard(int iBoardNum);

[[U tVip640aBoard::tVip640aBoard(iBoardNum) ]]
*/
tVip640aBoard::tVip640aBoard (int iBoardNum) : 
  
  /****
  [[U tBoard::tBoard (void) ]]
  ****/
  tBoard () 
{

  this->boardName  = VIP640A_BOARD_NAME + iBoardNum;
}
  
/****************************************************************************
  ERROR_CODE tVip640aBoard::SoftInit(void);

[[U tVip640aBoard::SoftInit() ]]
*/

ERROR_CODE tVip640aBoard::SoftInit() {
  ERROR_CODE ec;

  if (this->IsInit())
    return Traceback(S_board_BOARD_ALREADY_INITIALIZED, "tVip640aBoard::SoftInit");

  this->vmeAddress   = tVmeAddress(VIP640A_AM, (uint8_t *) VIP640A_REG_BASE_ADRS);
  this->vmeFrBufAddr = tVmeAddress(VIP640A_AM, (uint8_t *) VIP640A_FRAME_BUF_ADRS);

  ec = this->tBoard::Initialize();
  if (ec != OK)
    return Traceback(ec,"tVip640aBoard::SoftInit");

  /****
  [[U ec := this->ficImageChannel.tVip640aFullImageChannel::Initialize(this->vmeAddress, this->vmeFrBufAddr) ]]
  ****/
  ec = this->ficImageChannel.Initialize(this->vmeAddress, this->vmeFrBufAddr);
  if (ec != OK)
    return Traceback(ec,"tVip640aBoard::SoftInit");

  this->SetInit(TRUE);

#ifdef INCLUDE_IMAGE_CHANNEL_TEST
  testImageChannel = &this->ficImageChannel;
#endif
  return OK;
  }

/****************************************************************************
  ERROR_CODE tVip640aBoard::HardInit(void);

[[U tVip640aBoard::HardInit() ]]
*/

ERROR_CODE tVip640aBoard::HardInit() {
  ERROR_CODE ec;
  
  /* Pointers to various registers on Matrox VIP-640A card.   */
  volatile uint8_t *vCtrlReg0;
  volatile uint8_t *vCtrlReg1;
  volatile uint8_t *vMaskReg;
  volatile uint8_t *vSnapShotReg;
  volatile uint8_t *vStatReg;
  volatile uint8_t *vVidGainReg;
  volatile uint8_t *vVidOffsetReg;
  volatile uint8_t *vCrtcAddrReg;
  volatile uint8_t *vCrtcStatReg;
  volatile uint8_t *vCrtcDataReg;
  volatile uint8_t *vInpLkUpTbAddr;
  volatile uint8_t *vRedLkUpTbAddr;
  volatile uint8_t *vGrnLkUpTbAddr;
  volatile uint8_t *vBluLkUpTbAddr;

  volatile uint8_t *vFrBuffAddr;

  /* Shadow registers for the write only registers to be      */
  /* changed:                                                 */
  uint8_t control_reg_0;
  uint8_t control_reg_1;

  ec = this->SoftInit();
  if (ec != OK)
    return Traceback(ec,"tVip640aFullImageChannel::HardInit");

  vFrBuffAddr   = (uint8_t *) this->vmeFrBufAddr.GetLocalAddress();
  if (vFrBuffAddr == NULL) {
    this->SetInit(FALSE);
    Traceback(S_hw_LOCAL_ADDRESS_SET_TO_ZERO, "tVip640aFullImageChannel::SoftInit");
    }

  vInpLkUpTbAddr   = (uint8_t *) (this->vu8LocalAddress + INPUT_LUT);
  vRedLkUpTbAddr   = (uint8_t *) (this->vu8LocalAddress + RED_LUT);  
  vGrnLkUpTbAddr   = (uint8_t *) (this->vu8LocalAddress + GREEN_LUT);
  vBluLkUpTbAddr   = (uint8_t *) (this->vu8LocalAddress + BLUE_LUT); 

  vCtrlReg0     = (uint8_t *) (this->vu8LocalAddress + CNTRL_REG_0);  
  vCtrlReg1     = (uint8_t *) (this->vu8LocalAddress + CNTRL_REG_1);
  vMaskReg      = (uint8_t *) (this->vu8LocalAddress + MASK_REG);
  vSnapShotReg  = (uint8_t *) (this->vu8LocalAddress + SNAPSHOT_REG);  
  vStatReg      = (uint8_t *) (this->vu8LocalAddress + STAT_REG);
  vVidGainReg   = (uint8_t *) (this->vu8LocalAddress + VIDGAIN_REG);
  vVidOffsetReg = (uint8_t *) (this->vu8LocalAddress + VIDOFFSET_REG);
  vCrtcAddrReg  = (uint8_t *) (this->vu8LocalAddress + CRTC_ADDR_REG);
  vCrtcStatReg  = (uint8_t *) (this->vu8LocalAddress + CRTC_STAT_REG);
  vCrtcDataReg  = (uint8_t *) (this->vu8LocalAddress + CRTC_DATA_REG);
 
  //Setup Controll Registers
  control_reg_0 = (uint8_t) 0;
  *vCtrlReg0 = control_reg_0;
  control_reg_1 = (uint8_t) 0;
  *vCtrlReg1 = control_reg_1;
  
  /* Disable raster scan: */
  control_reg_0 |= REFR_DISABLE;
  *vCtrlReg0 = control_reg_0;

  /* Use external sync:   */
  control_reg_1 &= ~EXT_SYNC;
  *vCtrlReg1 = control_reg_1;

  /* Enable raster scan: */
  control_reg_0 &= ~REFR_DISABLE;
  *vCtrlReg0 = control_reg_0;

  /* set up the CRTC:     */
  *vCrtcAddrReg = HORIZ_TOTAL;
  *vCrtcDataReg = HORIZ_TOT_USA; 
   
  *vCrtcAddrReg = HORIZ_DISPLAYED; 
  *vCrtcDataReg = HORIZ_DISP_USA;

  *vCrtcAddrReg = HORIZ_SYNC_POS;
  *vCrtcDataReg = HORIZ_SPOS_USA;

  *vCrtcAddrReg = VERTAHOR_SYNC_WIDTH;;
  *vCrtcDataReg = VAH_SWID_USA; 
  
  *vCrtcAddrReg = VERT_TOTAL; 
  *vCrtcDataReg = VERT_TOT_USA; 

  *vCrtcAddrReg = VERT_TOT_ADJUST; 
  *vCrtcDataReg = VERT_TOTA_USA; 

  *vCrtcAddrReg = VERT_DISPLAYED;
  *vCrtcDataReg = VERT_DISP_USA;
    
  *vCrtcAddrReg = VERT_SYNC_POS; 
  *vCrtcDataReg = VERT_SPOS_USA;
    
  *vCrtcAddrReg = MODE_CONTROL; 
  *vCrtcDataReg = MODE_CONT_USA;
    
  *vCrtcAddrReg = SCAN_LINE;    
  *vCrtcDataReg = SCAN_LINE_USA;

  *vCrtcAddrReg = DISP_START_ADDRH;
  *vCrtcDataReg = DISP_STADH_USA;
  
  *vCrtcAddrReg = DISP_START_ADDRL;
  *vCrtcDataReg = DISP_STADL_USA;

  /* Set up the rest of the registers:    */
  
  /* Control register 0:                  */
  control_reg_0 &= ~LUT_SEL_MASK;
  control_reg_0 |=  LUT_MAP_0;        /* Choose the first map.     */
  control_reg_0 &= ~QUAD_BUFFER;      /* No quad buffer for 640a.  */
  control_reg_0 &= ~Y_QUADRANT;       /* No Y choice for quadrant. */
  control_reg_0 &= ~X_QUADRANT;       /* No X choice for quadrant. */
  control_reg_0 &= ~ZOOM_ENABLE;      /* Do not zoom.              */
  control_reg_0 &= ~REFR_DISABLE;     /* Enable refresh.           */
 
  /* Set the register:                */
  *vCtrlReg0 = control_reg_0;

  /* Control register 1:                  */
  control_reg_1 &= ~VIDINP_SRC_MASK;
  control_reg_1 |=  CHANNEL_0;         /* Choose the first input ch.    */
  control_reg_1 &= ~VIDOUT_SRC_MASK;
  control_reg_1 |=  FRAME_BUFFER;      /* Output is from frame buf.     */
  control_reg_1 &= ~VIDOUT_DIS;        /* Video output is enabled.      */
  control_reg_1 &=  ~CONT_GRAB;        /* Continuous grab is disabled.  */
  control_reg_1 &= ~EURO_STD;          /* Video is US standard.         */

  /* Set the register:                */
  *vCtrlReg1 = control_reg_1;

  /* Mask register:                       */
  *vMaskReg = TRANSPARENT;          /* All bit planes are visible.   */
 
  /* Video gain register:                 */
  *vVidGainReg =  DEFAULT_GAIN;     /* Use the default gain.         */
  
  /* Video offset register:               */
  *vVidOffsetReg =  DEFAULT_OFFSET; /* Use the default offset.       */
  
  /* Set up the input, red, green and blue output look up tables:      */ 
  /* All buffers are uniform gray scale:                               */
  for (int i = 0; i < TOT_LUT_LENGTH; i++) {
    *(vInpLkUpTbAddr + i) = (uint8_t) (~(i & LUT_ADDR_MASK));
    *(vRedLkUpTbAddr + i) = (uint8_t) (i & LUT_ADDR_MASK);
    *(vGrnLkUpTbAddr + i) = (uint8_t) (i & LUT_ADDR_MASK);
    *(vBluLkUpTbAddr + i) = (uint8_t) (i & LUT_ADDR_MASK);
    }
     
  /* Disable raster scan: */
  control_reg_0 |= REFR_DISABLE;
  *vCtrlReg0 = control_reg_0;

  /* Use external sync:   */
  control_reg_1 |= EXT_SYNC;
  *vCtrlReg1 = control_reg_1;

  /* Enable raster scan: */
  control_reg_0 &= ~REFR_DISABLE;
  *vCtrlReg0 = control_reg_0;

  /* Control register 1:                  */
  control_reg_1 |=  CONT_GRAB;        /* Continuous grab is disabled.  */

  /* Set the register:                */
  *vCtrlReg1 = control_reg_1;

  return OK;
  }

/****************************************************************************
  tChannel *tVip640aBoard::CreateChannel (int iChannelNum);

[[U tVip640aBoard::CreateChannel (iChannelNum) ]]
*/
tChannel *tVip640aBoard::CreateChannel (int iChannelNum) {

  if (this->IsInit() == TRUE || iChannelNum != 0) {
    Traceback (S_tVip640aBoard_RETURNING_NULL_CHANNEL_POINTER, 
               "tVip640aBoard::CreateChannel", iChannelNum);
    return NULL;
    }
    
  return &this->ficImageChannel;
  }
