/* $RCSfile: hardware.cc,v $ - Hardware class methods */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/hardware/hardware.cc,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

/*
modification history
--------------------

01a,96Nov05,pci     written
*/

#include "isit.h"
#include "configMgr.h"
#include "hwTypesP.h"
#include "hardware.h"
#include "board.h"
#include "laserLib.h"
#include "pc03v.h"
#include "pztMotor.h"
#include "laserLib.h"
#include "dvme626.h"
#include "mpv955.h"
#include "mpv906.h"
#include "mm6216.h"
#include "vip640a.h"
#include "ipQuad.h"
#include "ip16Dac.h"
#include "tim.h"
//#include "dt1401.h"

/***********************************************************************
*
* tHardware::tHardware - hardware class constructor
*
*/
tHardware::tHardware(const BOARD_TYPE *BoardTypeArray, int iArraySizeToUse)
{
  int i,
      iBoardNum,
      iNumBoardsArray[BT_NUM_BOARD_TYPES];

  TraceMark(S_hw_ENTERING_HARDWARE_CONSTRUCTOR);

  for(i=0; i<BT_NUM_BOARD_TYPES; i++) {
    iNumBoardsArray[i] = 0;
  }
 
  this->iArraySize  = iArraySizeToUse; 
  this->pBoardArray = new (tBoard *)[iArraySizeToUse];
    
  for(i=0; i<this->iArraySize; i++) {
    iBoardNum = iNumBoardsArray[BoardTypeArray[i]];
    switch(BoardTypeArray[i]) {
     case BT_DVME626:
       this->pBoardArray[i] = new tDvme626Board(iBoardNum);
       iNumBoardsArray[BT_DVME626]++;
       break;
     case BT_MPV955:
       this->pBoardArray[i] = new tMpv955Board(iBoardNum);
       iNumBoardsArray[BT_MPV955]++;
       break;
     case BT_MPV906:
       this->pBoardArray[i] = new tMpv906Board(iBoardNum);
       iNumBoardsArray[BT_MPV906]++;
       break;
     case BT_PZT_MOTOR:
       this->pBoardArray[i] = new tPztMotorBoard(iBoardNum);
       iNumBoardsArray[BT_PZT_MOTOR]++;
       break;
     case BT_FAST_LASER:
       this->pBoardArray[i] = new tFastLaserBoard(iBoardNum);
       iNumBoardsArray[BT_FAST_LASER]++;
       break;
     case BT_DT1401:
       this->pBoardArray[i] = NULL;
       iNumBoardsArray[BT_DT1401]++;
       break;
     case BT_PC03V:
       this->pBoardArray[i] = new tPc03vBoard(iBoardNum);
       iNumBoardsArray[BT_PC03V]++;
       break;
     case BT_MM6216:
       this->pBoardArray[i] = new tMm6216Board(iBoardNum);
       iNumBoardsArray[BT_MM6216]++;
       break;
     case BT_VIP640A:
       this->pBoardArray[i] = new tVip640aBoard(iBoardNum);
       iNumBoardsArray[BT_VIP640A]++;
       break;
     case BT_IPQUAD:
       this->pBoardArray[i] = new tIpQuadBoard(iBoardNum);
       iNumBoardsArray[BT_IPQUAD]++;
       break;
     case BT_IP16DAC:
       this->pBoardArray[i] = new tIp16DacBoard(iBoardNum);
       iNumBoardsArray[BT_IP16DAC]++;
       break;
     case BT_TIM:
       this->pBoardArray[i] = new tTimBoard(iBoardNum);
       iNumBoardsArray[BT_TIM]++;
       break;
     default:
       this->pBoardArray[i] = NULL;
       TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_BOARD_POINTER,
                "tHardware::tHardware Returning NULL board pointer\n");
       break;
    }
  }
  this->bSoftInit = FALSE;

  TraceMark(S_hw_DONE_CONSTRUCTING_HARDWARE);
}


/************************************************************************
*
* tHardware::~tHardware - class destructor. Deletes news done in constructor
*
*/
tHardware::~tHardware(void)
{
  int i;

  TraceMark(S_hw_ENTERING_HARDWARE_DESTRUCTOR);
  
  for(i=0; i<iArraySize; i++) {
    delete(pBoardArray[i]);
  }

  TraceMark(S_hw_DONE_DESTRUCTING_HARDWARE);
}


/************************************************************************
*
* tHardware::HardInit - method to initialize all boards in the system
*
*
* LOCAL : 
*   ERROR_CODE ec
*   VME_ADDRESS  vmeDt1401Address,
*                vmePc03v0Address,
*                vmePc03v1Address,
*                vmeFastLaserAddress,
*                vmePztMotorAddress
*/
ERROR_CODE tHardware::HardInit(void)
{
  ERROR_CODE ec;
  int i;

  TraceMark(S_hw_ENTERING_HARDWARE_HARDINIT);

  for(i=0; i<this->iArraySize; i++) {
    TraceMsg(tmt_INFO, S_hw_CALLING_HARDINIT,
             "Calling HardInit for board number %d\n", i);
    pBoardArray[i]->HardInit();
    TraceMsg(tmt_INFO, S_hw_BACK_FROM_HARDINIT,
             "Back from HardInit for board number %d\n", i);
  }

  TraceMark(S_hw_HARDWARE_INITIALIZED_HARDLY);

  return OK;
}


/************************************************************************
*
* tHardware::tSoftInit - method to setup pointers to all system boards
*
*
* LOCAL : 
*   ERROR_CODE ec
*   VME_ADDRESS  vmeDt1401Address,
*                vmePc03v0Address,
*                vmePc03v1Address,
*                vmeFastLaserAddress,
*                vmePztMotorAddress
*/
ERROR_CODE tHardware::SoftInit(void)
{
  ERROR_CODE ec;
  int i;

  TraceMark(S_hw_ENTERING_HARDWARE_SOFTINIT);

  for(i=0; i<this->iArraySize; i++) {
    TraceMsg(tmt_INFO, S_hw_CALLING_SOFTINIT,
             "Calling SoftInit for board number %d\n", i);
    pBoardArray[i]->SoftInit();
    TraceMsg(tmt_INFO, S_hw_BACK_FROM_SOFTINIT,
             "Back from SoftInit for board number %d\n", i);
  }
  
  TraceMark(S_hw_HARDWARE_INITIALIZED_SOFTLY);

  return OK;
}


/***********************************************************************
*
* tHardware::GetBoard - returns a pointer to the board object that
*                       is represented by boardName
*
* SPEC:
[[U tHardware::GetBoard()]]
*
******/
tBoard *tHardware::GetBoard(BOARD_NAME boardName)
{
  int i;


  for(i=0; i<this->iArraySize; i++) {
    if(pBoardArray[i]->boardName == boardName) {
      return pBoardArray[i];
    }
  }

  TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_BOARD_POINTER,
           "Returning NULL pointer from tHardware::GetBoard for board name 0x%x\n",
           boardName);
  return NULL;
}


/***********************************************************************
*
* tHardware::CreateChannel - returns a pointer to the channel object that
*                       is on board represented by boardName and numbered
*                       iChannelNum
*
[[U tHardware::CreateChannel()]]
*
******/
tChannel *tHardware::CreateChannel(BOARD_NAME boardName, int iChannelNum)
{
  tBoard *pBoard;
  tChannel *tmpChannel;
  int i;

  // Handle the case of a request for a NULL channel by returning a
  // reference to the appropriate Null channel
  if (boardName == NULL_BOARD_NAME) {
    if (iChannelNum == NCT_NULL_INPUT_CHANNEL)
      tmpChannel = &(this->NullInputChannel);
    else if (iChannelNum == NCT_NULL_OUTPUT_CHANNEL)
      tmpChannel = &(this->NullOutputChannel);
    else {
       tmpChannel = NULL;
       TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
           "Unknown channel request %d for NULL_BOARD_NAME", iChannelNum,
           boardName, iChannelNum);
    }
    return (tmpChannel);
  }

  /****
  [[U tHardware::GetBoard()]]
  ****/
  pBoard = this->GetBoard(boardName);

  if (pBoard == NULL) {
   TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_BOARD_POINTER,
            "tHardware::GetBoard returned NULL pointer, name=0x%x\n ", boardName);
    return NULL;
  }

  tmpChannel = pBoard->CreateChannel(iChannelNum);
  if (tmpChannel==NULL) {
       TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
           "pBoard->CreateChannel returned NULL pointer, name=0x%x, channel=%d\n ",
           boardName, iChannelNum);
  }
  return (tmpChannel);
}


/***********************************************************************
*
* tHardware::CreateInterrupt - returns a pointer to the channel object that
*                       is on board represented by boardName and numbered
*                       iChannelNum
*
[[U tHardware::CreateInterrupt()]]
*
******/
tInterrupt *tHardware::CreateInterrupt(BOARD_NAME boardName,
                                     int iInterruptPriority)
{
  tBoard *pBoard;
  tInterrupt *tmpInterrupt;
  int i;

  if (boardName == NULL_BOARD_NAME) {
    TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
             "tHardware::CreateInterrupt: boardName=0x%x, Priority=%d",
             boardName, iInterruptPriority);
    return NULL;
  }

  /****
  [[U tHardware::GetBoard()]]
  ****/
  pBoard = this->GetBoard(boardName);

  if (pBoard == NULL) {
   TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_BOARD_POINTER,
            "tHardware::GetBoard returned NULL pointer, name=0x%x\n ", boardName);
    return NULL;
  }

  tmpInterrupt = pBoard->CreateInterrupt(iInterruptPriority);
  if (tmpInterrupt==NULL) {
       TraceMsg(tmt_ERROR, S_hw_RETURNING_NULL_CHANNEL_POINTER,
           "pBoard->CreateInterrupt returned NULL pointer, name=0x%x, priority=%d",
           boardName, iInterruptPriority);
  }
  return (tmpInterrupt);
}

#if 0
/***********************************************************************
*
* tHardware::GetClockBoard - returns a pointer to the board object that
*                            represents the clock board
*
*/
tBoard * tHardware::GetClockBoard(int iRteNum)
{
  if(iRteNum<0 || iRteNum>=NRTES) {
    return NULL;
  }
  return this->GetBoard(PC03V_BOARD_NAME+iRteNum);
}
#endif

/***********************************************************************
*
* tHardware::GetShmemLocalAddress- returns a local pointer to the 
*                                  beginning of shared memory board
*
[[U tHardware::GetShmemLocalAddress(void) ]]
*
*/
volatile uint8_t  *tHardware::GetShmemLocalAddress(void)
{
  /****
  [[U pShmemBoard := tHardware::GetBoard(MM6216_BOARD_NAME) ]]
  ****/
  tBoard *pShmemBoard = this->GetBoard(MM6216_BOARD_NAME);

  if(pShmemBoard == NULL)
    return NULL;

  /****
  [[U return := tHardware::GetLocalAddress(void) ]]
  ****/
  return pShmemBoard->GetLocalAddress();
}

/***********************************************************************
*
* tHardware::GetShmemSize - returns total number of bytes on shared
*                           memory board
*
[[U tHardware::GetShmemSize(void) ]]
*
*/
unsigned int tHardware::GetShmemSize(void)
{
  /****
  [[U pShmemBoard := tHardware::GetBoard(MM6216_BOARD_NAME) ]]
  ****/
  tBoard *pShmemBoard = this->GetBoard(MM6216_BOARD_NAME);

  if(pShmemBoard == NULL)
    return 0;

  /****
  [[U return := tHardware::GetSize(void) ]]
  ****/
  return pShmemBoard->GetSize();
}

