/* $RCSfile: dt1401.h,v $ */
/*
$Header: /home/chara/cvsroot/ople/sys/hw/include/dt1401.h,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

/*
modification history
--------------------
01a,pci,21Oct96         written
*/

/*
* DT1401 driver header file.  Derived from board base class
*/




/****************************************
*
* Class: tDT1401
*
*
* SPEC:
tDT1401::tDT1401( //this )

IN :
  this

OUT :
  *this.bInit

  [ this is invalid -> UNDEFINED
  | true -> *this.bInit := FALSE
    : set register address variables to NULL
  ]


ERROR_CODE tDT1401::SoftInit(//this,
                             tVmeAddress vmeBoardAddress,
                             BOARD_NAME  boardName)
IN :
  this
  tVmeAddress vmeBoardAddress
  BOARD_NAME boardName

INOUT :
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.v8LocalAddress
  BOARD_NAME *this.boardName
  BOOL       *this.bInit
  BOARD_FUNC *this.bfFunctions 
  register address variables

  [this is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_INTERRUPT |
                           BF_AD_CHAN | BF_DA_CHAN |
                           BF_DIGITAL_IN_CHAN | BF_DIGITAL_OUT_CHAN
    : *this.bInit := TRUE
    : *this.v8LocalAddress := local address corresponding to vmeBoardAddress.address
    : set register variables to proper offset from base address
    : return OK
  ]



ERROR_CODE tDT1401::HardInit(//this,
                             tVmeAddress vmeBoardAddress,
                             BOARD_NAME  boardName)
IN :
  this
  tVmeAddress vmeBoardAddress
  BOARD_NAME boardName

INOUT :
  BOOL       *this.bInit

OUT :
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.v8LocalAddress
  BOARD_NAME *this.boardName
  BOARD_FUNC *this.bfFunctions 
  register address variables
  BOOL errIntEnable
  BOOL errClear
  BOOL initClear
  AD_MODE a2dMode
  DIO_MODE dioMode
  BOOL doneIntEnable
  BOOL eosIntEnable
  BOOL hardTrigEnable
  BOOL incEnable
  BOOL a2dStart
  unsigned int a2dGain
  unsigned int lastChannel
  unsigned int startChannel
  BOOL clkExt
  BOOL clkEnabled
  INT_RATE clkRate
  int16_t dioData

  [this is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit == TRUE -> return S_dt1401_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_INTERRUPT |
                           BF_AD_CHAN | BF_DA_CHAN |
                           BF_DIGITAL_IN_CHAN | BF_DIGITAL_OUT_CHAN
    : *this.bInit := TRUE
    : *this.rate := 0
    : *this.v8LocalAddress := local address corresponding to vmeBoardAddress.address
    : set register variables to proper offset from base address

    : BOOL errIntEnable  := FALSE	
    : BOOL errClear      := FALSE
    : BOOL initClear     := FALSE
    : AD_MODE a2dMode    := SINGLE
    : DIO_MODE dioMode   := ALL_IN
    : BOOL doneIntEnable := FALSE
    : BOOL eosIntEnable  := FALSE
    : BOOL hardTrigEnable:= FALSE
    : BOOL incEnable     := FALSE
    : BOOL a2dStart      := FALSE
    : unsigned int a2dGain     := 1
    : unsigned int lastChannel := 0
    : unsigned int startChannel:= 0
    : BOOL clkExt        := FALSE
    : BOOL clkEnabled    := FALSE
    : INT_RATE clkRate   := 1000
    : int16_t dioData       := 0xffff
    : return OK
  ]

  NOT IMPLEMENTED IN INC1
  ERROR_CODE A2d(int iChannel, int16_t wValue)
  NOT IMPLEMENTED IN INC1
  ERROR_CODE A2dModeSet(AD_MODE amMode)
  NOT IMPLEMENTED IN INC1
  AD_MODE    A2dModeGet()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE A2dGainSet(int iGain)
  NOT IMPLEMENTED IN INC1
  int        A2dGainGet()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE A2dStartChanSet(int iChannel)
  NOT IMPLEMENTED IN INC1
  int        A2dStartChanGet()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE A2dEndChanSet(int iChannel)
  NOT IMPLEMENTED IN INC1
  int        A2dEndChanGet()


  ERROR_CODE D2A(int iChannel, int16_t wValue)

  IN :
    this
    iChannel
    wValue

  OUT :
    *this.DA data registers

  [ this is invalid -> UNDEFINED
  | board has not been initialized -> UNDEFINED
  | iChannel not in range for *this ->
      return invalid channel error for *this
  | true -> output wValue on *this.DA[iChannel]
    : return OK
  ]


  void DioOut(uint16_t uwMask, uint16_t uwValue)
  
  IN :
    this
    uwMask
    uwValue
    *this.dioMode

  OUT :
    *this.digital output register value

  [ this is invalid || board has not been initialized -> UNDEFINED
  | uwMask has bits set that are not configured for output in
    *this.dioMode ->
    output bits of uwValue that are set in uwMask and
      configured for output in *this.dioMode
    : return OK
  | true -> output bits of uwValue on DIO channel that are set in uwMask
    : return OK
  ]


  uint16_t DioIn(uint16_t uwMask)

  IN :
    this
    uwMask
    *this.dioMode
    *this.Digital input data register value

  INOUT :

  SPEC :
  [ this is invalid || board has not been initialized -> UNDEFINED
  | uwMask has bits set that are not configured for input in
    *this.dioMode ->
      return bits of uwValue on DIO channel that are set in uwMask and
      configured for input in *this.dioMode, other bits of word are
      set to 0.
  | true -> return bits of uwValue on DIO channel that are set in uwMask
  ]


  NOT IMPLEMENTED IN INC1
  ERROR_CODE DioModeSet(DIO_MODE dmMode)
  NOT IMPLEMENTED IN INC1
  DIO_MODE   DioModeGet()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE RateSet(int *rate)
  NOT IMPLEMENTED IN INC1
  int        RateGet()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE Enable()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE Disable()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE Connect()
  NOT IMPLEMENTED IN INC1
  ERROR_CODE Disconnect()


*/


class tDT1401 : public tBoard {
  public:
    tDT1401(void);
    ERROR_CODE HardInit(tVmeAddress vmeBoardAddress,
                        BOARD_NAME boardName);
/*
    ERROR_CODE ReInit(tVmeAddress vmeBoardAddress,
                        BOARD_NAME boardName);
*/
    ERROR_CODE SoftInit(tVmeAddress vmeBoardAddress,
                        BOARD_NAME boardName);
/*
    ERROR_CODE A2D(int iChannel, int16_t wValue);
    ERROR_CODE A2DModeSet(AD_MODE amMode);
    AD_MODE    A2DModeGet();
    ERROR_CODE A2DGainSet(int iGain);
    int        A2DGainGet();
    ERROR_CODE A2DStartChanSet(int iChannel);
    int        A2DStartChanGet();
    ERROR_CODE A2DEndChanSet(int iChannel);
    int        A2DEndChanGet();
*/
    virtual ERROR_CODE D2A(int iChannel, int16_t wValue);
    virtual void DioOut(uint16_t uwMask, uint16_t uwValue);
    virtual uint16_t DioIn(uint16_t uwMask);
/*
    ERROR_CODE DioModeSet(DIO_MODE dmMode);
    DIO_MODE   DioModeGet();
    ERROR_CODE ClkRateSet(int *rate);
    int        ClkRateGet();
    ERROR_CODE ClkEnable();
    ERROR_CODE ClkDisable();
    ERROR_CODE ClkConnect();
    ERROR_CODE ClkDisconnect();
*/

  protected:
    uint16_t GetDioInputMask(void);
    uint16_t GetDioOutputMask(void);

    volatile uint16_t *vADSCR;
    volatile uint16_t *vADCHANREG;
    volatile uint16_t *vPACLK;
    volatile int16_t *vADDATA;
    volatile int16_t *vDADATA0;
    volatile int16_t *vDADATA1;
    volatile uint16_t *vDIOREG;
} ;
