/* $RCSfile: dvme626.h,v $ - Driver software for DVME626 D/A board*/
/* 
$Header: /home/chara/cvsroot/ople/sys/hw/include/dvme626.h,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/
/*
* This file contains the methods necessary to initialize and use the DVME626
* D/A board.  Includes tBoard and tAdChannel subclasses.
*/

/*
modification history
--------------------
01a,pci,15Apr97               written
*/

#ifndef INC_dvme626h
#define INC_dvme626h

/*******************************************************************************
*
* class tDvme626DaChannel
*
* State Variables:
*  BOOL          tBaseObject::bInit
*  int           tChannel::iBitWidth
*  CONV_CODE     tChannel::ccOutputCode
*  tInputChannel tOutputChannel::*pReadbackChannel
*  int           tOutputChannel::iResetValue
*  const int        CiBitWidth
*  volatile int16_t *vwOutput
*  int              iOutputRange
* 
* ERROR_CODE tDvme626DaChannel::Initialize(int              iResetValueToUse,
*                                          tInputChannel    *pReadbackChannelToUse
*                                          volatile int16_t *vwOutputToUse)
* Description:  Sets values for all state variables to parameter values and sets
*               bInit to TRUE
[[D tDvme626DaChannel::Initialize(int              iResetValueToUse,
                                  tInputChannel    *pReadbackChannelToUse,
                                  volatile int16_t *vwOutputToUse)]
[ this is invalid || pReadbackChannelToUse is invalid ||
  vwOutputToUse is invalid => UNDEFINED
| true =>
  this->iResetValue := iResetValueToUse
  : this->pReadbackChannel := pReadbackChannelToUse
  : this->iBitWidth := CiBitWidth
  : this->vwOutput  := vwOutputToUse
  : this->bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tDvme626DaChannel::Output(int iValue)
* Description:  Outputs a new voltage on this DVME626 channel.
[[D tDvme626DaChannel::Output(in: iValue; inout: this;) ]
[ this is invalid || this->vwOutput is invalid || this->bInit == FALSE =>
  UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  *(this->vwOutput) := low 16 bits of iValue
  : this->dActualChannelValue := any voltage (voltage will reflect relevant bits only)
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  *(this->vwOutput) := low 16 bits of iValue
  : this->dActualChannelValue := (2^this->iBitWidth - 1 -
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth))
  : return OK
| this->ccOutputCode == CC_STRAIGHT_BINARY =>
  *(this->vwOutput) := low 16 bits of iValue
  : this->dActualChannelValue :=
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth)
  : return OK
| this->ccOutputCode == CC_OFFSET_BINARY =>
  *(this->vwOutput) := low 16 bits of iValue
  : this->dActualChannelValue :=
                          Unsigned(iValue, this->iBitWidth) *
                          DaResolution(this->iOutputRange, this->iBitWidth) -
                          (2^this->iBitWidth/2)
  : return OK
| this->ccOutputCode == CC_TWOS_COMPLEMENT =>
  *(this->vwOutput) := low 16 bits of iValue
  : this->dActualChannelValue := iValue *
                               DaResolution(this->iOutputRange, this->iBitWidth)
| true => UNDEFINED
]]
*
*
******/
class tDvme626DaChannel : public tDaChannel {
public:
  ERROR_CODE Initialize(int              iResetValueToUse,
                        tInputChannel    *pReadbackChannelToUse,
                        volatile int16_t *vwOutputToUse,
                        CONV_CODE        ccConvCodeToUse);
  ERROR_CODE Output(int iValue);

protected:
  const int        CiBitWidth     = 16;
  const int        CiVoltageRange = 20;
  volatile int16_t *vwOutput;
};


/*******************************************************************************
*
* class tDvme626Board : public tBoard
*
* State Variables
*  @ tVmeAddress     vmeAddress
*  @ volatile int8_t *vu8LocalAddress
*  @ BOOL            bInit
*  const uint8_t const *Cpu8DefaultBaseAddress := 0xC900
*  const int         CiDefaultAddressMod  := VME_AM_SUP_SHORT_IO
*  const int         CiMaxChannels        := 6
*  const int         CiDefaultResetValue  := 0
*  int               iNumChannels
*  tDvme626DaChannel ChannelArray[CiMaxChannels]
*
* METHODS:
* tDvme626Board::tDvme626Board(void)
* Description: Constructor - sets all default values for variables.
[[D tDvme626Board::tDvme626Board(void)]
[ this is invalid => UNDEFINED
| true =>
  [[U tBoard::tBoard()]]
  : this->iNumChannels := this->CiMaxChannels
]]
*
*
* ERROR_CODE tDvme626Board::SoftInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables
[[D tDvme626Board::SoftInitParams(int iNumChannelsToUse, int iResetValueToUse)]
[ this is invalid => UNDEFINED
| true =>
  [[U tBoard::InitializeParams() ]]
  : [[U tDvme626DaChannel::Initialize(this=this->ChannelArray[0..iNumChannels],
                 pReadbackChannelToUse=NULL,
                 iResetValueToUse=iResetValueToUse,
                 vwOutputToUse=pvmeAddressToUse->
                      (volatile int16_t *) GetLocalAddress() + appropriate offset)]]
  : this->iNumChannels := iNumChannelsToUse
  : this->bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tDvme626Board::SoftInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables using the default values.
[[D tDvme626Board::SoftInitVoid(void)]
[ true =>
  [[U tBoard::InitializeVoid()]]
  : [[U tDvme626DaChannel::Initialize(this=this->ChannelArray[0..iNumChannels],
                pReadbackChannelToUse=NULL,
                iResetValueToUse=CiDefaultResetValue,
                vwOutputToUse=this->vmeAddress->
                     (volatile int16_t *)GetLocalAddress() + appropriate offset)]]
]]
*
*
* ERROR_CODE tDvme626Board::HardInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables, and sets channel outputs to reset value.
[[D tDvme626Board::HardInitParams(int iNumChannelsToUse,
                                  int iResetValueToUse)]
[ this is invalid => UNDEFINED
| true =>
  [[U tDvme626Board::SoftInitParams()]]
  : [[U tDvme626Board::ResetAllChannels()]]
  : return OK
]]
*
*
* ERROR_CODE tDvme626Board::HardInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables using the default values, and sets channel
*               outputs to default reset value.
[[D tDvme626Board::HardInitVoid(void)]
[ true =>
  [[U tDvme626Board::SoftInitVoid()]]
  : [[U tDvme626Board::ResetAllChannels()]]
]]
*
*
* ERROR_CODE tDvme626Board::GetChannel(iChannelNum)
* Description:  Gets channel specified by iChannelNum.
*        DVME626 Global board numbering:
*     D/A channels 0-5 ==> Global Channels 0-5
*
[[D tDvme626Board::GetChannel(int iChannelNum)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := NULL
| iChannelNum<0 || iChannelNum> this->iNumChannels =>
  return := NULL
| true =>
  return := D/A channel corresponding to iChannelNum
]]
*
*
* ERROR_CODE tDvme626Board::ResetAllChannels(void)
* Description:  Resets all active output channels to current value of
*               this->iResetValue
* Return ERROR_CODE
[[D tDvme626Board::ResetAllChannels(void)]
[ this is invalid => UNDEFINED
| (this->iNumChannels > this->CiMaxChannels) ||
  (this->iNumChannels < 0) =>
    return S_tDvme626Board_INVALID_NUMBER_OF_CHANNELS
| true=>
  [[U tOutputChannel::Reset(this=this->ChannelArray[0..iNumChannels-1]) ]]
]]
*
*
******/
class tDvme626Board : public tBoard {
public:
  tDvme626Board(int iBoardNum);
  ERROR_CODE SoftInit(int         iNumChannelsToUse,
                      int         iResetValueToUse,
                      CONV_CODE   ccConvCodeToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(int         iNumChannelsToUse,
                      int         iResetValueToUse,
                      CONV_CODE   ccConvCodeToUse);
  ERROR_CODE HardInit(void);
  tChannel *CreateChannel(int iChannelNum);
  ERROR_CODE ResetAllChannels(void);

protected:  
  const int         CiMaxChannels        = 6;
  const int         CiDefaultResetValue  = 0;
  const CONV_CODE   CccDefaultConvCode   = CC_TWOS_COMPLEMENT;
  tDvme626DaChannel ChannelArray[CiMaxChannels];
  
  void ShowId(void);

private:
  const int         CiChannelRegOffset  = 0xA0;
};


#endif
