/* $RCSfile: hwTypesP.h,v $ - Private Hardware type definitions */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/include/hwTypesP.h,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/

/*
modification history
--------------------

01b,pci,03Jun97     added LT_HOME to LIMIT_TYPE
01a,pci,29Apr97     written
*/

#ifndef INC_hwTypesPh
#define INC_hwTypesPh

#include <vme.h>
#include "hwError.h"
#include "hwTypes.h"

/****************************************************************************
* 
* Typedefs for Increment 2, needed by tBoard(board.h) & tChannel(channel.h)
*  Public hardware typedefs are in hwTypes.h 
*/
/*************************************** 
* SIGNAL_SENSE: Signal Sense, used for determining whether a digital signal
*                is active high or active low.
*    SS_ACTIVE_HI: TRUE = HI
*    SS_ACTIVE_LO: TRUE = LO
******/
typedef enum SIGNAL_SENSE {
  SS_ACTIVE_HI=0, SS_ACTIVE_LO
} SIGNAL_SENSE;

/*************************************** 
*  LASER_SYNC_MODE: Used for laser counters to determine the event on which
*                   the laser counter will clear.
*    SM_NO_SYNC:          Laser channel will update at native clock rate.
*    SM_SYNC_TO_REF:      Laser channel will update every reference edge.
*    SM_SYNC_TO_EXT:      Laser channel will update every external clock pulse.
*    SM_SYNC_TO_FIDUCIAL: Fiducial counter will load on next fiducial crossing
*                         then return to previous sync mode.
******/
typedef enum LASER_SYNC_MODE {
  SM_NO_SYNC=0, SM_SYNC_TO_REF, SM_SYNC_TO_EXT, SM_SYNC_TO_FIDUCIAL
} LASER_SYNC_MODE;

/*************************************** 
*  SCAN_MODE: Used for channels with scanning capability (A/D and
*             waveform channels) to determine whether the board scans
*             continuously or halts after one cycle.
******/
typedef enum SCAN_MODE {
  SM_CONTINUOUS, SM_ONE_SHOT
} SCAN_MODE;

/*************************************** 
*  TRIGGER_SELECT: Used for channels that have different possibilities
*                  for trigger sources.
******/
typedef enum TRIGGER_SELECT {
  TS_INTERNAL, TS_EXTERNAL, TS_SOFTWARE, TS_EVENT
} TRIGGER_SELECT;

/*************************************** 
*  AD_MODE: Used for A/D channels.  Single-ended or differential.
******/
typedef enum AD_MODE {
  AM_SINGLE_ENDED, AM_DIFFERENTIAL
} AD_MODE;

/*************************************** 
*  LIMIT_TYPE: Used for limit switch channels.
*              4 possibilities: hard/soft, cw/ccw.
******/
typedef enum LIMIT_TYPE {
  LT_CW_SOFT=0, LT_CCW_SOFT=1, LT_CW_HARD=2, LT_CCW_HARD=3, 
  LT_HOME=4
} LIMIT_TYPE;


#endif /* INC_hwTypesPh */
