/* $RCSfile: ip16Dac.h,v $ - Greenspring IP-16DAC digital-to-analog board */

/* $Header: /home/chara/cvsroot/ople/sys/hw/include/ip16Dac.h,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $ */

/*
 * This file contains the methods necessary to initialize and use
 *  the Greenspring IP-16DAC Digital-To-Analog board.
 * Includes tBoard and tDaChannel subclasses.
 */

/* $Log: ip16Dac.h,v $
 * Revision 1.1.1.1  1998/06/11 21:36:34  chara
 * First import of RICST Code
 *
 * Revision 1.8  1998/04/02 21:01:12  phil
 * Build I5a near complete; still some configMgr stuff left
 *
 * Revision 1.7  1998/03/31 20:39:38  tgl
 * Fix up a few lines to get within 80 column page.
 *
 * Revision 1.6  1998/03/31 20:09:41  tgl
 * Fix typo in Spec.
 *
 * Revision 1.5  1998/03/13 18:49:56  tgl
 * Match spec to code changes to decouple tBoard from tChannel interactions.
 * Add iResetValue to channel class.
 *
 * Revision 1.4  1998/03/12 00:41:57  tgl
 * Match description of ResetAllChannels method with actual behavior.
 *
 * Revision 1.3  1998/03/12 00:40:21  tgl
 * Update spec to allow clean pass through the spec tool.
 *
 * Revision 1.2  1998/03/11 23:40:38  tgl
 * Add offset and gain calibration registers.
 * Remove remnants of the IP-QUAD CSR and vector registers.
 * Fix method declarations in class.
 *
 * Revision 1.1  1998/03/11 18:18:23  tgl
 * Declarations for IP-16DAC digital to analog board.
 * First check-in; code not yet compiled and probably not quite ready.
 * */

#ifndef INC_IP16DACh
#define INC_IP16DACh

/*******************************************************************************
*
* class tIp16DacChannel
*
* State Variables:
*  BOOL      tBaseObject::bInit
*  int       tChannel::iBitWidth
*  int       tIp16DacChannel::CiBitWidth
*  const int CiBitWidth
*
********************************************************************************
* 
* ERROR_CODE tIp16DacChannel::Initialize(...)
* Description:  Sets values for all state variables to parameter values and sets
*               bInit to TRUE
[[D tIp16DacChannel::Initialize(volatile uint16_t *vuwDaRegToUse,
                                volatile uint8_t  *vu8OffsetCorrRegToUse,
                                volatile uint8_t  *vu8GainCorrRegToUse,
                                int                iResetValueToUse)]
[ invalid_pointer(this) ||
  invalid_pointer(vuwDaRegToUse) ||
  iResetValueToUse out of range ||
  vuwDaRegToUse does not point to valid dac register => UNDEFINED
| true =>
    this->iBitWidth           =  this->CiBitWidth;
    : tChannel::dChannelRange := 2^this->CiBitWidth
    : tChannel::ccConvCode    := CC_STRAIGHT_BINARY
    : this->vuwOutput         := vuwDaRegToUse
    : reset channel output value to iResetValueToUse rather than this->iResetValue
    : tBaseObject::ecInitError := OK
    : tBaseObject::bInit := TRUE
]]
*
*
* ERROR_CODE tIp16DacChannel::Output(int iValue) const
[[D tIp16DacChannel::Output(int iValue)]
[ invalid_pointer(this) ||
  this->vuwOutput is invalid
  tBaseObject::bInit == FALSE ||
  host tIp16DacBoard is not initialize properly => UNDEFINED
| true =>
    iValue := denormalized number between 0 and (2^16-1) for dac setting
    : write iValue to this dac register
    : return := OK
]]
*
*
* ERROR_CODE tIp16DacChannel::Reset() const
[[D tIp16DacChannel::ResetVoid(void)]
[ invalid_pointer(this) ||
  tBaseObject::bInit == FALSE => UNDEFINED
| true =>
    set dac register to midrange
    : return := OK
]]
*
*
* ERROR_CODE tIp16DacChannel::Reset() const
[[D tIp16DacChannel::ResetParams(int iResetValueToUse)]
[ invalid_pointer(this) ||
  tBaseObject::bInit == FALSE => UNDEFINED
| true =>
    set dac register to iResetValueToUse
    : return := OK
]]
*
*
********************************************************************************
********************************************************************************
*
* class tIp16DacBoard : public tBoard
*
* State Variables
*  tVmeAddress        tBoard::vmeAddress
*  volatile uint16_t* tBoard::vuwLocalAddress
*  BOOL               tBaseObject::bInit
*  ERROR_CODE         tBaseObject::ecInitError
*  int                iNumChannels
*  const int          CiMaxChannels
*  tIp16DacChannel    ChannelArray[this->CiMaxChannels]
*
* METHODS:
* tIp16DacBoard::tIp16DacBoard(int iBoardNum)
* Description: Constructor - sets all default values for variables.
[[D tIp16DacBoard::tIp16DacBoard(int iBoardNum)]
[ invalid_pointer(this) => UNDEFINED
| iBoardNum is out of range =>
  tBaseObject::ecInitError := S_Ip16Dac_INVALID_BOARD_NUM
  : tBaseObject::bInit       := FALSE
| true =>
  this->uiSize := 0x100
  : this->iNumActiveChannel := 0
  : tBaseObject::bInit := FALSE
  : tBaseObject::ecInitError := OK
  : this->vmeAddress := default vme address for this board
  : this->boardName  := IP16DAC_BOARD_NAME + iBoardNum
]]
*
*
* ERROR_CODE tIp16DacBoard::SoftInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables
[[D tIp16DacBoard::SoftInitParams(int iNumChannelsToUse)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize appropriate number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tIp16DacBoard::SoftInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables using the default values.
[[D tIp16DacBoard::SoftInitVoid(void)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize maximum number of channels
  : tBaseObject::bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tIp16DacBoard::HardInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables, and sets channel outputs to reset value.
[[D tIp16DacBoard::HardInitParams(int iNumChannelsToUse)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize appropriate number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : reset all channels using iResetValueToUse rather than this->iResetValue
  : return OK
]]
*
*
* ERROR_CODE tIp16DacBoard::HardInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables using the default values, and sets channel
*               outputs to default reset value.
[[D tIp16DacBoard::HardInitVoid(void)]
[ true =>
  initialize tBoard variables to appropriate values
  : initialize maximum number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : reset all channels
  : return OK
]]
*
*
* ERROR_CODE tIp16DacBoard::VerifyId(void)
* Description:  Prints the Board's ID on the console
[[D tIp16DacBoard::VerifyId(void)]
[ invalid_pointer(this) => UNDEFINED
| true=>
    boardId := PROM identification string
    : return := board matches driver?
]]
*
*
* ERROR_CODE tIp16DacBoard::CreateChannel(iChannelNum)
* Description:  Gets channel specified by iChannelNum.
*        IP-16DAC Global board numbering:
*          dacs 1-3 ==> Global Channels 0-2
*
[[D tIp16DacBoard::CreateChannel(int iChannelNum)]
[ invalid_pointer(this) => UNDEFINED
| iChannelNum<0 || iChannelNum> this->iNumChannels =>
  return := NULL
| true =>
  return := dac channel corresponding to iChannelNum
]]
*
*
* ERROR_CODE tIp16DacBoard::ResetAllChannels()
* Description:  Resets all active output channels to specified value
* Return ERROR_CODE
[[D tIp16DacBoard::ResetAllChannels()]
[ invalid_pointer(this) => UNDEFINED
| true=>
    reset all active channels on board to iResetValueToUse
    : return := OK
]]
*
*
* ERROR_CODE tIp16DacBoard::ShowId(void)
* Description:  Prints the Board's ID on the console
[[D tIp16DacBoard::ShowId(void)]
[ invalid_pointer(this) => UNDEFINED
| true=>
    console := console || formatted contents of board's ID EPROM
]]
*
*
* ERROR_CODE tIp16DacBoard::Reset(void)
* Description:  Set DAC output to nominal value
[[D tIp16DacBoard::Reset(void)]
[ invalid_pointer(this) => UNDEFINED
| true=>
    reset channel to nominal value
]]
*
*
******/



/*******************************************************************************
****************************** Class Definitions *******************************
*******************************************************************************/

class tIp16DacChannel : public tDaChannel {
public:
  ERROR_CODE Initialize(volatile uint16_t *vuwDaRegToUse,
                        volatile uint8_t  *vu8OffsetCorrRegToUse,
                        volatile uint8_t  *vu8GainCorrRegToUse,
                        int                iResetValueToUse);
  ERROR_CODE Output(int iValue);
  ERROR_CODE Reset(void);
  ERROR_CODE Reset(int iResetValueToUse);

protected:
  const int          CiBitWidth     = 16;
  const int          CiVoltageRange = 10;
  int                iResetValue;
  volatile uint16_t *vuwOutput;
};


class tIp16DacBoard : public tBoard {
public:
  tIp16DacBoard(int iBoardNum);
  ERROR_CODE SoftInit(int iNumChannelsToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(int iNumChannelsToUse);
  ERROR_CODE HardInit(void);
  ERROR_CODE VerifyId(void);
  ERROR_CODE ResetAllChannels(void);
  tChannel *CreateChannel(int iChannelNum);

protected:  
  const int             CiMaxChannels = 3;
  const int             CiIdPromOffset = 0x80;
  tIp16DacChannel ChannelArray[CiMaxChannels];
  
  void ShowId(void);
};


#endif
