/* $RCSfile: ipQuad.h,v $ */
/* 
$Header: /home/chara/cvsroot/ople/sys/hw/include/ipQuad.h,v 1.1.1.1 1998/06/11 21:36:34 chara Exp $
*/
/*
* This file contains the methods necessary to initialize and use the Greenspring 
* IP-QUADRATURE Counter board.  Includes tBoard and tCounterChannel subclasses.
*/

/*
modification history
--------------------
01a,pci,19Nov97               written
*/

#ifndef INC_ipQuadh
#define INC_ipQuadh

/*******************************************************************************
*
* class tIpQuadCounterChannel
*
* State Variables:
*  BOOL  tBaseObject::bInit
*  int   tChannel::iBitWidth
*  const int CiBitWidth
*
********************************************************************************
* 
* ERROR_CODE tIpQuadCounterChannel::Initialize(...)
* Description:  Sets values for all state variables to parameter values and sets
*               bInit to TRUE
[[D tIpQuadCounterChannel::Initialize(volatile uint8_t* vu8CounterRegToUse,
                                      volatile uint8_t* vu8ConfigRegToUse)]
[ invalid_pointer(this) ||
  invalid_pointer(vu8CounterRegToUse) ||
  invalid_pointer(vu8ConfigRegToUse) ||
  vu8CounterRegToUse does not point to valid counter register ||
  vu8ConfigRegToUse does not point to a valid configuration register => UNDEFINED
| true =>
    tChannel::iBitWidth := this->CiBitWidth
    : tChannel::dChannelRange := 2^this->CiBitWidth
    : tChannel::ccConvCode    := CC_STRAIGHT_BINARY
    : this->vu8Input          := vu8CounterRegToUse
    : this->vu8ChannelCsr     := vu8CounterRegToUse + 2
    : this->vu8ConfigReg      := vu8CounterRegToUse
    : set on-board Init flag (CR)
    : set interrupt on borrow (OCR)
    : set quadrature disable (QR)
    : enable channel
    : tBaseObject::ecInitError := OK
    : tBaseObject::bInit := TRUE
]]
*
*
* ERROR_CODE tIpQuadCounterChannel::Input(int* piValue) const
[[D tIpQuadCounterChannel::Input(int* piValue)]
[ invalid_pointer(this) ||
  this->vu8Input is invalid 
  tBaseObject::bInit == FALSE ||
  host tIpQuadBoard is not initialize properly => UNDEFINED
| true =>
    *piValue := number between 0 and (2^24-1) that corresponds to current
                counter value
    : return := OK
]]
*
*
* ERROR_CODE tIpQuadCounterChannel::Reset() const
[[D tIpQuadCounterChannel::Reset(void)]
[ invalid_pointer(this) ||
  this->vu8ChannelCsr is invalid || 
| tBaseObject::bInit == FALSE => UNDEFINED
| true =>
    set counter value to 0
    : return := OK
]]
*
*
* void tIpQuadCounterChannel::Enable(void)
[[D tIpQuadCounterChannel::Enable(void)]
[ invalid_pointer(this) ||
  this->vu8ChannelCsr is invalid ||
  tBaseObject::bInit == FALSE => UNDEFINED
| true =>
    this->vu8ChannelScr set to appropriate value to start counter
]]
*
*
* ERROR_CODE tIpQuadCounterChannel::Disable(int* piValue)
[[D tIpQuadCounterChannel::Disable(int* piValue)]
[ invalid_pointer(this) ||
  this->vu8ChannelCsr is invalid ||
  tBaseObject::bInit == FALSE => UNDEFINED
| true =>
    this->vu8ChannelScr set to appropriate value to stop counter
]]
*
*
********************************************************************************
********************************************************************************
*
* class tIpQuadBoard : public tBoard
*
* State Variables
*  tVmeAddress       tBoard::vmeAddress
*  volatile uint8_t* tBoard::vu8LocalAddress
*  BOOL              tBaseObject::bInit
*  ERROR_CODE        tBaseObject::ecInitError
*  int               iNumChannels
*  const int         CiMaxChannels
*  tIpQuadCounterChannel ChannelArray[this->CiMaxChannels]
*
* METHODS:
* tIpQuadBoard::tIpQuadBoard(int iBoardNum)
* Description: Constructor - sets all default values for variables.
[[D tIpQuadBoard::tIpQuadBoard(void)]
[ invalid_pointer(this) => UNDEFINED
| iBoardNum is out of range =>
  tBaseObject::ecInitError := S_ipQuad_INVALID_BOARD_NUM
  : tBaseObject::bInit       := FALSE
| true =>
  this->uiSize := 0
  : this->iNumActiveChannel := 0
  : tBaseObject::bInit := FALSE
  : tBaseObject::ecInitError := OK
  : this->vmeAddress := default vme address for this board (0x6000)
  : this->boardName  := IPQUAD_BOARD_NAME + iBoardNum
]]
*
*
* ERROR_CODE tIpQuadBoard::SoftInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables
[[D tIpQuadBoard::SoftInitParams(int iNumChannelsToUse)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize appropriate number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tIpQuadBoard::SoftInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               and sets class variables using the default values.
[[D tIpQuadBoard::SoftInitVoid(void)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize maximum number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : return OK
]]
*
*
* ERROR_CODE tIpQuadBoard::HardInit(...)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables, and sets channel outputs to reset value.
[[D tIpQuadBoard::HardInitParams(int iNumChannelsToUse,
                                  int iResetValueToUse)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  initialize tBoard variables to appropriate values
  : initialize appropriate number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : reset all channels
  : disable termination resistors (enable TTL mode)
  : return OK
]]
*
*
* ERROR_CODE tIpQuadBoard::HardInit(void)
* Description:  Sets board address, initializes the pointers to board variables,
*               sets class variables using the default values, and sets channel
*               outputs to default reset value.
[[D tIpQuadBoard::HardInitVoid(void)]
[ true =>
  initialize tBoard variables to appropriate values
  : initialize maximum number of channels
  : this->iNumActiveChannels := MAX(this->iNumActiveChannels, iNumChannelsToUse)
  : tBaseObject::bInit := TRUE
  : reset all channels
  : disable termination resistors (enable TTL mode)
  : return OK
]]
*
*
* ERROR_CODE tIpQuadBoard::CreateChannel(iChannelNum)
* Description:  Gets channel specified by iChannelNum.
*        IP-QUADRATURE Global board numbering:
*          counters 1-4 ==> Global Channels 0-3
*
[[D tIpQuadBoard::CreateChannel(int iChannelNum)]
[ invalid_pointer(this) => UNDEFINED
| tBaseObject::bInit == FALSE =>
  return := NULL
| iChannelNum<0 || iChannelNum> this->iNumChannels =>
  return := NULL
| true =>
  return := counter channel corresponding to iChannelNum
]]
*
*
* ERROR_CODE tIpQuadBoard::ResetAllChannels(void)
* Description:  Resets all active output channels to 0
* Return ERROR_CODE
[[D tIpQuadBoard::ResetAllChannels(void)]
[ invalid_pointer(this) => UNDEFINED
| true=>
    reset all active channels on board to 0
    : return := OK
]]
*
*
* ERROR_CODE tIpQuadBoard::ShowId(void)
* Description:  Prints the Board's ID on the console
[[D tIpQuadBoard::ShowId(void)]
[ invalid_pointer(this) => UNDEFINED
| true=>
    console := cosole || formatted contents of board's ID EPROM
]]
*
*
******/



/*******************************************************************************
****************************** Class Definitions *******************************
*******************************************************************************/

class tIpQuadCounterChannel : public tCounterChannel {
public:
  ERROR_CODE Initialize(volatile uint8_t *vu8CounterRegToUse,
                        volatile uint8_t *vu8ConfigRegToUse);
  virtual ERROR_CODE Input(int* piValue) const;
  virtual ERROR_CODE Reset(void) const;
  void Enable(void);
  void Disable(void);

protected:
  const int        CiBitWidth = 24;
  volatile uint8_t *vu8Input;
  volatile uint8_t *vu8ChannelCsr;
  volatile uint8_t *vu8ConfigReg;
};


class tIpQuadBoard : public tBoard {
public:
  tIpQuadBoard(int iBoardNum);
  ERROR_CODE SoftInit(int iNumChannelsToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(int iNumChannelsToUse);
  ERROR_CODE HardInit(void);
  ERROR_CODE ResetAllChannels(void);
  tChannel *CreateChannel(int iChannelNum);

protected:  
  volatile uint8_t      *vu8BoardCsr;
  volatile uint8_t      *vu8VectorReg;
  const int             CiMaxChannels = 4;
  const int             CiIdPromOffset = 0x80;
  tIpQuadCounterChannel ChannelArray[CiMaxChannels];
  
  void ShowId(void);

};


#endif
