/* $RCSfile: mpv906.h,v $ - Driver software for MPV906 A/D board */

/* 
$Header: /home/chara/cvsroot/ople/sys/hw/include/mpv906.h,v 1.1.1.1 1998/06/11 21:36:35 chara Exp $
*/

/*
* This file contains the black box spec and class definitions necessary to
*  initialize and use the MPV906 A/D board.  
*/

/*
modification history
--------------------
01a,pci,15Apr97               written
*/

#ifndef INC_mpv906h
#define INC_mpv906h

/*******************************************************************************
*
* Forward Declarations
*
******/
class tMpv906AdChannelBank;
class tMpv906Board;


/*******************************************************************************
*
* class tMpv906AdChannel
*
* State Variables:
*  
*
* ERROR_CODE tMpv906AdChannel::Initialize(...)
Description:  Sets values for state variables to parameter values and sets
              bInit to TRUE
[[D tMpv906AdChannel::Initialize(tMpv906AdChannelBank* pChannelBankToUse,
                                 volatile int16_t*     vwDataToUse,
                                 CONV_CODE             ccConvCodeToUse,
                                 int                   iChannelRangeToUse,
                                 int                   iGainToUse)]   
[ this is invalid || pChannelBankToUse is invalid || vwDataToUse is invalid =>
  UNDEFINED
  | true =>
  set class variables to associated parameter values
  : this->iBitWidth := this->CiBitWidth
  : this->bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906AdChannel::Input(...)
* Description: sets *piValue to most recent value converted for this channel
[[D tMpv906AdChannel::Input(in: this; inout: piValue;)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| this->IsBoardScanning() == FALSE =>
  *piValue := anything
  : return := S_mpv906_STALE_VALUE_-_BOARD_NOT_SCANNING
| this->ccInputCode == CC_COMPLEMENT_BINARY =>
  *piValue := 2^this->iBitWidth - 1 - ((uint) *this->vwData) 
  : return := OK
| this->ccInputCode == CC_NO_CONV_CODE    ||
  this->ccInputCode == CC_STRAIGHT_BINARY || 
  this->ccInputCode == CC_TWOS_COMPLEMENT =>
   *piValue :=  (int) *this->vwData
   : return := OK
| this->ccInputCode == CC_OFFSET_BINARY =>
  *piValue :=  ((uint) *this->vwData - 2^this->iBitWidth/2)
  : return := OK
| true =>
  return S_mpv906_INVALID_CONV_CODE
]]
*
******/
class tMpv906AdChannel : public tAdChannel {
public:
  tMpv906AdChannel(void);
  ERROR_CODE Initialize(tMpv906AdChannelBank *pChannelBankToUse,
                        volatile int16_t     *vwDataToUse,
                        int                  iDataOffsetToUse,
                        CONV_CODE            ccConvCodeToUse,
                        int                  iChannelRangeToUse);
  ERROR_CODE Input(int *piValue) const;
  ERROR_CODE SetGain(int iGainToUse);

protected:
    const int CiBitWidth = 12;
    const int CiScanListBuffer = 4;

    tMpv906AdChannelBank *pChannelBank;  
    volatile int16_t     *vwData;
    int                  iDataOffset;
};
  

/*******************************************************************************
*
* class tMpv906AdChannelBank
*
* State Variables:
*  
*
* ERROR_CODE tMpv906AdChannelBank::Initialize(...)
* Description:  Sets values for state variables to parameter values,
*               initializes all channels and sets bInit to TRUE.
[[D tMpv906AdChannelBank::Initialize(volatile int16_t*  vwDataToUse,
                                     volatile uint8_t*  vu8ScanListToUse,
                                     volatile uint16_t* vuwIoCtrlRegToUse,
                                     int                iNumChannelsToUse,
                                     CONV_CODE          ccConvCodeToUse,
                                     int                iVoltageRangeToUse,
                                     int                iGainToUse)]
[ this is invalid || pAdChannelBankToUse is invalid || vwInputData is invalid =>
  UNDEFINED
| iNumChannelsToUse > allowed by this->amAdMode =>
  return := S_mpv906_CHANNEL_NUMBER_TOO_HIGH_FOR_BANK
| true =>
  set class variables to associated parameter values
????  : this->amAdMode      := approprate register value  ????
  : this->iStartChannel := CiDefaultStartChannel
  : set scan list appropriately
  : set all channel gains to 1
  : this->bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906AdChannelBank::GetChannel(...)
* Description:  Gets channel specified by iNumChannel.
*        MPV906 Global board numbering:
*     A/D channels 0-64 ==> Global Channels 0-64
*
[[D tMpv906AdChannelBank::GetChannel(int iChannelNum)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := NULL
| iChannelNum<0 || iChannelNum > this->iNumChannels-1 =>
  return := NULL
| true =>
  return := A/D channel corresponding to iNumChannel
]]
*
*
* ERROR_CODE tMpv906AdChannelBank::StartScan(void)
* Description: Starts the MPV906 board scanning through prorammed channels.
[[D  tMpv906AdChannelBank::StartScan(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_mpv906_CHANNEL_BANK_NOT_INITIALIZED
| this->bScanning == TRUE =>
  return OK
| true =>
  Continually update board registers associated with this->iStartChannel
     to this->iStartChannel + this->iNumChannels - 1
  : this->bScanning := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906AdChannelBank::StopScan(void)
* Description: The MPV906 board does not have a stop scan capability.  This
*              method is present to conform to the tAdChannelBank virtual
*              method.
[[D tMpv906AdChannelBank::StopScan(void)]
[ this is invalid => UNDEFINED
| true =>
  return := S_mpv906_CANNOT_STOP_MPV906_SCAN
]]
*
*
* ERROR_CODE tMpv906AdChannelBank::LoadScanList(...)
* Description: Loads the MPV906s scan list for the appropriate number of 
*              channels. The list will be loaded repetitively with
*              channel values from this->iStartChannel to
*              this->iStartChannel + this->iNumChannels - 1.
*              If the number of channels is not an integer multiple of 64,
*              all channels will have slightly varying frequencies.
[[D tMpv906AdChannelBank::LoadScanList(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_mpv906_CHANNEL_BANK_NOT_INITIALIZED
| true =>
  load scan list repetitively with values from this->iStartChannel to
    this->iStartChannel + this->iNumChannels - 1
  : return := OK
]]
*
*
* ERROR_CODE tMpv906AdChannelBank::SetGain(int iChannel, int iNewGain)
* Description:  The MPV906 allows each channel to have its own gain in addition
*               to the board gain.  The channel gains are set to "1" upon
*               initialization, but each channel has a separate gain that can
*               be altered with this method.
*         NOTE:  User should be aware of the value of the board gain
[[D tMpv906AdChannel::SetGain(int iChannel, int iNewGain)]
[ this is invalid => UNDEFINED
| iChannel > allowed by this->amAdMode =>
  return := S_mpv906_CHANNEL_NUMBER_TOO_HIGH_FOR_BANK
| iNewGain!=1 && iNewGain!=10 && iNewGain!=100 =>
  return := S_mpv906_INVALID_GAIN
| true =>
  channel gain in all scan list entries for iChannel := iNewGain
  : return := OK
]]
*
******/
class tMpv906AdChannelBank : public tAdChannelBank {
protected:  //Protected Constants
  const int       CiMaxNumChannels = 64;
  const int       CiNumScanListLoc = 64;
  const uint16_t  CuwStartScanValue = 0;

public:
  tMpv906AdChannelBank(tMpv906Board *pBoardToUse);
  ERROR_CODE Initialize(volatile int16_t  *vwDataToUse,
                        volatile uint8_t  *vu8ScanListToUse,
                        volatile uint16_t *vuwIoCtrlRegToUse,
                        int               iNumChannelsToUse,
                        int               iStartChannelToUse,
                        CONV_CODE         ccConvCodeToUse,
                        int               iVoltageRangeToUse);
  tChannel   *CreateChannel(int iChannelNum);
  ERROR_CODE StartScan(void);
  ERROR_CODE StopScan(void) { return S_mpv906_CANNOT_STOP_MPV906_SCAN; }
  ERROR_CODE SetGain(int iChannel, int iNewGain);
  uint8_t    GetScanListPtr(void);
  volatile int16_t *GetCurrentChannelAddress(int iChannel);

protected:
  ERROR_CODE LoadScanList(BOOL bResetGain=FALSE);

  tMpv906Board        *pBoard;
  volatile uint8_t    *vu8ScanList;  
  volatile uint16_t   *vuwIoCtrlReg;
  volatile int16_t    *vwChannelData;
  tMpv906AdChannel    ChannelArray[CiMaxNumChannels];
};
  

/*******************************************************************************
*
* class tMpv906Board
* 
* ASSUMPTIONS: 
*   1. Board is set in sign extension mode to extend the MSB(11) of the
*      converted value to bit 15.
*   2. Jumper blocks are all set to legal values.
*
* State Variables:
*  
*
* ERROR_CODE tMpv906Board::SoftInit(...)
* Description:  Parameterized version - Sets values for state variables to
*               parameter values and sets bInit to TRUE.
[[D tMpv906Board::SoftInitParams(double       dSampleFreqToUse,
                                 int          iBaseSamplePeriodToUse,
                                 int          iStartChannelToUse,
                                 int          iNumChannelsToUse,
                                 CONV_CODE    ccConvCodeToUse,
                                 int          iVoltageRangeToUse)]
[ this is invalid || pvmeAddressToUse is invalid => UNDEFINED
| true =>
  set class variables to associated parameter values
  : initialize channel bank
  : this->bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906Board::SoftInit(void)
* Description:  No params version - Sets values for state variables to
*               default values and sets bInit to TRUE.
[[D tMpv906Board::SoftInitVoid(void)]   
[ this is invalid  => UNDEFINED
| true =>
  set class variables to associated parameter values
  : [[U tMpv906AdChannelBank::Initialize(this=this->ChannelBank)]]
  : this->bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906Board::HardInit(...)
* Description:  Parameterized version - Sets values for state variables to
*               parameter values, initializes board register values and sets
*               bInit to TRUE.
*[[D tMpv906Board::HardInitParams(double         dSampleFreqToUse,
                                  int            iBaseSamplePeriodToUse,
                                  int            iStartChannelToUse,
                                  int            iNumChannelsToUse,
                                  CONV_CODE      ccConvCodeToUse,
                                  int            iVoltageRangeToUse,
                                  int            iGainToUse,
                                  SCAN_MODE      smScanModeToUse,
                                  TRIGGER_SELECT tsTriggerSelectToUse)]
[ this is invalid || pvmeAddressToUse is invalid => UNDEFINED
| true =>
  set class variables to associated parameter values
  : initialize board registers
  : [[U tMpv906AdChannelBank::Initialize(this=this->ChannelBank)]]
  : this->bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv906Board::HardInit(void)
* Description:  No params version - Sets values for state variables to
*               default values, initializes board register values and sets
*               bInit to TRUE.
*
[[D tMpv906Board::HardInitParams(void)]
[ this is invalid => UNDEFINED
| true =>
  set class variables to associated parameter values
  : initialize board registers
  : [[U tMpv906AdChannelBank::Initialize(this=this->ChannelBank)]]
  : this->bInit := TRUE
  : return := OK
]]
*
* ERROR_CODE tMpv906Board::GetChannel(...)
* Description:  Gets channel specified by iNumChannel.
*        MPV906 Global board numbering:
           see tMpv906ChannelBank
*
[[D tMpv906Board::GetChannel(int iChannelNum)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := NULL
| true =>
  [[U tMpv906ChannelBank::GetChannel()]]
]]
*
*
* double tMpv906Board::SetSampleFrequency(double dSampleFreq)
* Description: Sets the sample frequency of the A/D channels.  This is the
*              frequency (in Hz) at which the board converts a single
*              channel, not the entire board (e.g. if there are two 
*              active channels and the user sets the sample frequency to
*              2000Hz, each channel will be read every 1ms (1/1000Hz)).
*              If the value passed in cannot be constructed exactly,
*              the frequency will be set to the nearest higher value.
[[D tMpv906Board::SetSampleFrequency(double dSampleFreq)]
[ this is invalid => UNDEFINED
| (dSampleFreq < CdMinSampleFreq) || (dSampleFreq > CdMaxSampleFreq) =>
  return := current sample frequency
| dSampleFreq cannot be constructed from
    (5MHz)*(1/(iBaseSamplePeriod+1))*(1/([0..255]+1)) =>
  set board period register to nearest lower value.
  : return := nearest higher possible frequency
| true =>
  set board period register appropriately
  : return := dSampleFreq
]]
*
*
* double tMpv906Board::GetSampleFrequency(void)
* Description: returns current sample frequency of the board.
[[D tMpv906Board::GetSampleFrequency(void)]
[ this is invalid => UNDEFINED
| true =>
  return := current sample frequency of the board
]]
*
*
* int tMpv906Board::GetMaxNumChannelsFromAdMode(void)
* Description: Returns number of channels possible based on current A/D mode
[[D tMpv906Board::GetMaxNumChannelsFromAdMode(void)]
[ this is invalid || pointer to IO Control Reg has not been set => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv906_BOARD_NOT_INITIALIZED
| this->amAdMode == AM_SINGLE_ENDED =>
  return := 64
| this->amAdMode == AM_DIFFERENTIAL =>
  return := 32
]]
******/
class tMpv906Board : public tBoard {
public:
  tMpv906Board(int iBoardNum);
  ERROR_CODE SoftInit(double       dSampleFreqToUse,
                      int          iBaseSamplePeriodToUse,
                      int          iStartChannelToUse,
                      CONV_CODE    ccConvCodeToUse,
                      int          iVoltageRangeToUse,
                      int          iGainToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(double         dSampleFreqToUse,
                      int            iBaseSamplePeriodToUse,
                      int            iStartChannelToUse,
                      CONV_CODE      ccConvCodeToUse,
                      int            iVoltageRangeToUse,
                      int            iGainToUse,
                      SCAN_MODE      smScanModeToUse,
                      TRIGGER_SELECT tsTriggerSelectToUse);
  ERROR_CODE HardInit(void);
  tChannel *CreateChannel(int iChannelNum);

  /* Set/Get Methods */
  double SetSampleFrequency(double dSampleFreq);
  double GetSampleFrequency(void) const
             { return (double) (5000000.0 * (1.0/(iBaseSamplePeriod+1)) *
                               (1.0/((*vuwPeriodReg & 0x00ff)+1))); }
  ERROR_CODE SetScanMode(SCAN_MODE smScanModeToUse);
  ERROR_CODE SetTriggerSelect(TRIGGER_SELECT tsTriggerSelectToUse);
  ERROR_CODE SetNumChannels(int iNumChannelsToUse);
  int        GetMaxNumChannelsFromAdMode(void);
  int        GetMaxChannels(void) const { return this->CiMaxChannels; }

protected:
  /* Defaults */
  const int             CiMaxChannels         = 64;
  const int             CiDefaultNumChannels  = 32;
  const double          CdMinSampleFreq       = 76.3;
  const double          CdMaxSampleFreq       = 33000.0;
  const int             CiDefaultBaseSamplePeriod = 0x18;
  const double          CdDefaultSampleFreq   = 33000.0;
  const CONV_CODE       CccDefaultConvCode    = CC_TWOS_COMPLEMENT;
  const SCAN_MODE       CsmDefaultScanMode    = SM_CONTINUOUS;
  const TRIGGER_SELECT  CtsDefaultTriggerSelect = TS_INTERNAL;
  const int             CiDefaultStartChannel = 0;
  const int             CiDefaultVoltageRange = 20;
  const int             CiDefaultBoardGain    = 1;

  tMpv906AdChannelBank  ChannelBank;
  int                   iBaseSamplePeriod;
  int                   iBoardGain;

private:
  /* Register variables and defaults */
  volatile uint16_t *vuwIntrCtrlReg;
  volatile uint16_t *vuwIoCtrlReg;
  volatile uint16_t *vuwTrigSourceReg;
  volatile uint16_t *vuwPeriodReg;

  const int CiDataOffset          = 0x0000;
  const int CiScanListOffset      = 0x0080;
  const int CiIntrCtrlRegOffset   = 0x0100;
  const int CiIoCtrlRegOffset     = 0x0102;
  const int CiTrigSourceRegOffset = 0x00fe;
  const int CiPeriodRegOffset     = 0x00fc;
};

  
#endif
