/* $RCSfile: mpv955.h,v $ - MPV955 Driver library */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/include/mpv955.h,v 1.1.1.1 1998/06/11 21:36:35 chara Exp $
*/

/*
modification history
--------------------
01a,21Apr97,pci    written
*/

/*
* DESCRIPTION
* This module contains class definitions for  tMpv955Board, tMpv955WaveformChannel,
* and tMpv955WaveformChannelBank
*/

#ifndef INC_mpv955h
#define INC_mpv955h

#include "channelBank.h"

//////////// From Palomar //////////////////////////////////////////////////
// STATUS strokeFullInit (void);
// STATUS strokeInit (void);
// STATUS strokeInitDataMemory (void);
// STATUS strokeOn (void);
// STATUS strokeOff (void);
// double triWaveGen(int frequency, double amplitude, double *sample_index, 
//                   int num_channels, int predistortion_type);
// STATUS strokeLoadTriWave(int frequency, double phase, double amplitude, 
//                          int channel_num, int predistortion_type);
// double strokeGetMaxAmp(int predistortion_type);
////////////////////////////////////////////////////////////////////////////

/*******************************************************************************
*
* Local type definitions
*
*****/
typedef enum TRIGGER_TYPE {
  TT_NORMAL, TT_EVENT
} TRIGGER_TYPE;

/*******************************************************************************
* Forward Declarations
******/
class tMpv955WaveformChannelBank;
class tMpv955Board;

/*******************************************************************************
*
* class tMpv955WaveformChannel
*
* State Variables:
*  int                        iResetValue
*  tMpv955WaveformChannelBank *pChannelBank
*  volatile int16_t           *vwData
*
* METHODS:
* ERROR_CODE tMpv955WaveformChannel::Initialize(...)
* Description:  Initialization routine for a single MPV955 channel.  Sets state
*               variables.
[[D tMpv955WaveformChannel::Initialize(tMpv955WaveformChannelBank* pChannelBankToUse,
                                       tInputChannel         pReadbackChannelToUse,
                                       int                   iResetValueToUse,
                                       volatile int16_t*     vwDataToUse,
                                       CONV_CODE             ccOutputCodeToUse)]
[ this is invalid || pChannelBank is invalid || vwDataToUse is invalid =>
    UNDEFINED
| true =>
  this->pChannelBank := pChannelBankToUse
  : this->pReadbackChannel := pReadbackChannelToUse
  : this->iResetValue  := iResetValueToUse
  : this->ccOutputCode := ccOutputCodeToUse
  : this->vwData       := vwDataToUse
  : this->bInit        := TRUE
  : return OK
]]
*
*
* ERROR_CODE tMpv955WaveformChannel::Output(int iValue)
* Description:  Simple Output routine for the MPV955 -- primarily written for
*               conformity to the tOutputChannel pure virtual methods.  This
*               method simply loads all values of the channel with iValue.
[[D tMpv955WaveformChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  this->vwData[all locations for this channel] := iValue
  : this->dActualChannelValue := (2^this->iBitWidth - 1 -
                                 Unsigned(iValue, this->iBitWidth) *
                                 DaResolution(this->iChannelRange, this->iBitWidth))
  : return OK
| this->ccOutputCode == CC_STRAIGHT_BINARY =>
  this->vwData[all locations for this channel] := iValue
  : this->dActualChannelValue := Unsigned(iValue, this->iBitWidth) *
                                 DaResolution(this->iChannelRange, this->iBitWidth)
  : return OK
| this->ccOutputCode == CC_OFFSET_BINARY =>
  this->vwData[all locations for this channel] := iValue
  : this->dActualChannelValue := Unsigned(iValue, this->iBitWidth) *
                                 DaResolution(this->iChannelRange, this->iBitWidth) -
                                 (2^this->iBitWidth/2)
  : return OK
| this->ccOutputCode == CC_TWOS_COMPLEMENT =>
  this->vwData[all locations for this channel] := iValue
  : this->dActualChannelValue := iValue * DaResolution(this->iChannelRange, this->iBitWidth)
| true =>
  return := S_mpv955_INVALID_CONV_CODE
]]
*
*
* ERROR_CODE tMpv955WaveformChannel::LoadMemory(int *piValues, iSize)
* Description:  Loads iSize values (piValues[0..iSize-1]) into the board's memory
*               for this channel.
[[D tMpv955WaveformChannel::LoadMemory(int* piValues, int iSize)]
[ this is invalid || piValues is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_CHANNEL_NOT_INITIALIZED
| ABS(piValue[0]..piValues[iSize-1]) >= 2^this->iBitWidth =>
  this->dActualChannelValues := anything (loaded values will reflect relevant bits)
| (iSize > this->iNumMemLoc) || (iSize < 0) =>
  return S_mpv955_INSUFFICIENT_MEMORY
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  this->vwData[appropriate board locations] := piValues[0]..piValues[iSize-1]
  : this->dActualChannelValue := (2^this->iBitWidth - 1 - 
                          Unsigned(piValues[0]..piValues[iSize-1], this->iBitWidth) *
                           DaResolution(this->iChannelRange, this->iBitWidth))
  : return OK
| this->ccOutputCode == CC_STRAIGHT_BINARY =>
  this->vwData[appropriate board locations] := piValues[0]..piValues[iSize-1]
  : this->dActualChannelValue := Unsigned(iValue, this->iBitWidth) *
                                 DaResolution(this->iChannelRange, this->iBitWidth)
  : return OK
| this->ccOutputCode == CC_OFFSET_BINARY =>
  this->vwData[appropriate board locations] := piValues[0]..piValues[iSize-1]
  : this->dActualChannelValue := Unsigned(iValue, this->iBitWidth) *
                                 DaResolution(this->iChannelRange, this->iBitWidth) -
                                 (2^this->iBitWidth/2)
  : return OK
| this->ccOutputCode == CC_TWOS_COMPLEMENT =>
  this->vwData[appropriate board locations] := piValues[0]..piValues[iSize-1]
  : this->dActualChannelValue :=
         iValue * DaResolution(this->iChannelRange, this->iBitWidth)
| true => 
  return := S_mpv955_INVALID_CONV_CODE
]]
*
*
* int tMpv955WaveformChannel::GetMemSize(void)
* Description:  Returns the number of memory locations available for this channel.
[[D tMpv955WaveformChannel::GetMemSize(void)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  return number of memory locations for this channel
]]
*
******/
class tMpv955WaveformChannel : public tWaveformChannel {
public:
  ERROR_CODE Initialize(tMpv955WaveformChannelBank  *pChannelBankToUse,
                        tInputChannel               *pReadbackChannelToUse,
                        int                         iResetValueToUse,
                        volatile int16_t            *vwDataToUse,
                        CONV_CODE                   ccOutputCodeToUse,
                        int                         iVoltageRangeToUse);
  ERROR_CODE Output(int iValue);
  ERROR_CODE LoadMemory(int *piValues);
  ERROR_CODE LoadMemory(double *pdValues);
  int        GetMemSize(void);
  tWaveformChannelBank *GetChannelBank(void);

protected:
  tMpv955WaveformChannelBank  *pChannelBank;
  volatile int16_t            *vwData;
  const int                   CiBitWidth = 16;
};

/*******************************************************************************
*
* class tMpv955WaveformChannelBank
*
* State Variables:
*  const int               CiMaxNumChannels := 8
*  volatile uint16_t        *vuwStartOutput
*  volatile uint16_t        *vuwStopOutput
*  volatile uint16_t        *vuwStartAddressReg
*  volatile uint16_t        *vuwStopAddressReg
*  volatile uint16_t        *vuwRateTimerCtrlReg
*  int                     iWaveformSize
*  
*  tMpv955WaveformChannel  ChannelArray[CiMaxNumChannels] 
*
* METHODS:
*
* tMpv955WaveformChannelBank::tMpv955WaveformChannelBank(...)
* Description: Sets pBoard to parameter passed in
*
[[D tMpv955WaveformChannelBank::tMpv955WaveformChannelBank(tMpv955Board* pBoardToUse)]
[ this is invalid => UNDEFINED
| true =>
  this->tBoard = pBoardToUse
]]
*
* ERROR_CODE tMpv955WaveformChannelBank::Initialize(...)
* Description: Initialization method for a bank of MPV955 channels.  Sets state
*              variables, and sets bInit to TRUE.
[[D tMpv955WaveformChannelBank::Initialize(volatile int16_t*   vwDataToUse,
                                     volatile uint16_t*  vuwStartOutputToUse,
                                     volatile uint16_t*  vuwStopOutputToUse,
                                     volatile uint16_t*  vuwStartAddressRegToUse,
                                     volatile uint16_t*  vuwStopAddressRegToUse,
                                     volatile uint16_t*  vuwRateTimerCtrlRegToUse,
                                     int          iResetValueToUse,
                                     CONV_CODE    ccOutputCodeToUse,
                                     uint16_t     uwNumChannelsToUse)]
[ this is invalid || vuwStartOutputToUse is invalid ||
   vuwStopOutputToUse => UNDEFINED
| this->bInit == TRUE =>
  return := S_mpv955_BANK_ALREADY_INITIALIZED
| true =>
  this->pBoard          := pBoardToUse
  : this->vuwStartOutput   := vuwStartOutputToUse 
  : this->vuwStopOutput    := vuwStopOutputToUse
  : this->vuwStartAddressReg := vuwStartAddressRegToUse
  : this->vuwStopAddressReg  := vuwStopAddressRegToUse
  : this->vuwRateTimerCtrlReg := vuwRateTimerCtrlRegToUse
  : this->bInit           := TRUE
  : this->ccOutputCode    := ccOutputCodeToUse
  : this->iTotalMemoryLoc := this->CiTotalMemoryLoc;
  : this->iWaveformSize   := this->CiTotalMemoryLoc / number of active channels
  : [[U tMpv955WaveformChannel::Initialize(
                             this=ChannelArray[0..this->pBoard->GetNumChannels()-1],
                             pChannelBankToUse=this,
                             pReadbackChannelToUse=NULL,
                             iResetValueToUse=iResetValueToUse,
                             vwDataToUse=vwDataToUse + appropriate offset)]]
  : return := OK
]]
*
*
* ERROR_CODE tMpv955WaveformChannelBank::GetChannel(...)
* Description:  Gets channel specified by iNumChannel.
*        MPV955 Global board numbering:
*     Waveform channels 0-7 ==> Global Channels 0-7
*
[[D tMpv955WaveformChannelBank::GetChannel(int iChannelNum)]
[ this is invalid || iChannelNum<0 ||this->pBoard->GetNumChannels()-1 => UNDEFINED
| this->bInit == FALSE =>
  return := NULL
| true =>
  return := Waveform channel corresponding to iNumChannel
]]
*
*
* ERROR_CODE SetStartStopAddresses(void);
* Description: Sets StartAddressReg and StopAddressReg so they correspond to
*              the beginning and end of the actual data.  The start and stop
*              registers are especially useful because if the number of
*              channels is not evenly divisible by the total number of memory
*              locations, there would be some garbage at the end of the 
*              waveform.
[[D tMpv955WaveformChannelBank::SetStartStopAddresses(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_CHANNEL_BANK_NOT_INITIALIZED
| true =>
  *this->vuwStartAddressReg := 0
  : *this->vuwStopAddressReg := Waveform Size * Number of Active Channels
  : return := OK
]]
*
*
* ERROR_CODE tMpv955WaveformChannelBank::StartOutput(void)
* Description: Start "pulse" for the board to begin outputting voltages to all
*              active channels.
[[D tMpv955WaveformChannelBank::StartOutput(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_BANK_NOT_INITIALIZED
| this->bOutputOn == TRUE =>
  return OK
| board is in continuous mode =>
  Start updating board output registers[0..this->GetNumChannels()-1] continuously
  : return := OK
| true =>
  update board output registers[0..this->GetNumChannels()-1]
  : this->bOutputOn := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tMpv955WaveformChannelBank::StopOutput(void)
* Description: Stop "pulse" for the board to stop outputting voltages to its
*              channels.
[[D tMpv955WaveformChannelBank::StopOutput(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_BANK_NOT_INITIALIZED
| this->bOutputOn == FALSE =>
  return := OK
| true =>
  Stop updating all board output registers
  : this->bOutputOn := FALSE
  : return := OK
]] 
*
*
* ERROR_CODE tMpv955WaveformChannelBank::ZeroMemory(void)
* Description: Zeros all data memory locations on the board.
[[D tMpv955WaveformChannelBank::ZeroMemory(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_BOARD_NOT_INITIALIZED
| true =>
  this->all data memory locations := 0
]]
*
*
*  ERROR_CODE tMpv955WaveformChannelBank::SetNumChannels(int iNumChannels);
*  Description: Sets the number of active channels on the board.
[[U tMpv955Board::SetNumChannels(this=this->pBoard, iNumChannels=iNumChannels)]]
*
*
* int tMpv955WaveformChannelBank::GetNumChannels(void)
* Description:    Returns the total number of active channels on the board.
[[D tMpv955WaveformChannelBank::GetNumChannels(void) ]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  return := number of channels in channel bank
]]
*
*
* ERROR_CODE tMpv955WaveformChannelBank::SetWaveformSize(int iSize)
* Description: Sets the size of the waveform (in number of samples)
[[D tMpv955WaveformChannelBank::SetWaveformSize(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return appropriate error code
| (iSize < 0) || (iSize > this->iTotalMemoryLoc/number of active channels) =>
  return := S_mpv955_INVALID_SIZE
| true =>
  this->iWaveformSize := iSize
]]
*
* ERROR_CODE tMpv955WaveformChannelBank::SetSamplePeriod(...)
* Description: Sets the absolute sample period of the board output waveform
*              (in nanoseconds)
*              Should be an integer multiple of CiSampleClockPeriod;
*              if not, set to nearest multiple.
[[D tMpv955WaveformChannelBank::SetSamplePeriod(int iNewPeriod)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return appropriate error code
| (iNewPeriod < 0) =>
  return := S_mpv955_NEGATIVE_PERIOD
| (iNewPeriod < this->CiSampleClockPeriod) =>
  display warning message on console
  : set board period to this->CiSampleClockPeriod
  : return := OK
| (iNewPeriod > this->CiSampleClockPeriod * this->CiRateTimerMaxValue) =>
  display warning message on console
  : set board period to this->CiSampleClockPeriod * this->CiRateTimerMaxValue
  : return := OK
| (iNewPeriod % this->CiSampleClockPeriod) != 0 =>
  display warning message on console
  : set board period to nearest, smaller multiple of this->CiSampleClockPeriod
  : return := OK
| true =>
  set board period to iNewPeriod
  : return := OK
]]
*
* int tMpv955WaveformChannelBank::GetWaveformSize(void)
* Description: Gets the size of the waveform (in number of samples)
[[D tMpv955WaveformChannelBank::GetWaveformSize(void)]
[ this is invalid => UNDEFINED
| true =>
  return := this->iWaveformSize
]]
*
*
* double tMpv955WaveformChannelBank::GetSampleFreq(void)
* Description: Gets the sample frequency (in kHz), that is the actual frequency
               that each channel will "see" at the output
[[D tMpv955WaveformChannelBank::GetSampleFreq(void)]
[ this is invalid => UNDEFINED
| true =>
  return := (1/(0.000001 * tMpv955WaveformChannelBank::GetSamplePeriod()))
]]
*
*
* ERROR_CODE tMpv955WaveformChannelBank::ResetAllChannels(void)
* Description:   Sets all channels to their respective reset values.
[[D tMpv955WaveformChannelBank::ResetAllChannels(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_mpv955_BANK_NOT_INITIALIZED
| true =>
  set all active memory locations to their respective reset values
]]
*
******/
class tMpv955WaveformChannelBank : public tWaveformChannelBank {
public:
  /* CiMaxNumChannels is public so tMpv955Board can get to it */
  const int CiMaxNumChannels = 8;

  tMpv955WaveformChannelBank::tMpv955WaveformChannelBank(tMpv955Board *pBoardToUse);
  ERROR_CODE Initialize(volatile int16_t  *vwDataToUse,
                        volatile uint16_t *vuwStartOutputToUse,
                        volatile uint16_t *vuwStopOutputToUse,
                        volatile uint16_t *vuwStartAddressToUse,
                        volatile uint16_t *vuwStopAddressToUse,
                        volatile uint16_t *vuwRateTimerCtrlRegToUse,
                        int       iResetValueToUse,
                        CONV_CODE ccOutputCodeToUse,
                        int       iVoltageRangeToUse,
                        uint16_t  uwNumChannelsToUse);
  void       DumpRegs(void);
  void       DumpBoardRegs(void);
  tChannel   *CreateChannel(int iChannelNum);
  ERROR_CODE SetStartStopAddresses(void);
  ERROR_CODE StartOutput(void);
  ERROR_CODE StopOutput(void);
  ERROR_CODE ZeroMemory(void);

  /* Set and Get Functions */
  int        GetTotalMemory(void) { return iTotalMemoryLoc; }
  int        GetNumChannels(void);
  ERROR_CODE SetWaveformSize(int iSize);
  int        GetWaveformSize(void) { return iWaveformSize; }
  double     GetSampleFreq(void)   { return (1.0/(1.0e-9 * GetSamplePeriod())); }
  /*********
  *NOTE:
  *  SetSamplePeriod sets sample period independent of number of channels
  *     (absolute sample period of board). So if there are 5 channels,
  *      sample period per channel will be iNewPeriod * 5).
  *  GetSamplePeriod returns period per channel
  *********/
  ERROR_CODE SetSamplePeriod(int iNewPeriod);
  int        GetSamplePeriod(void) {
                return (CiSampleClockPeriod *
                        (CiRateTimerMaxValue-(u8CachedRateTimerValue)) *
                        GetNumChannels()); }
  double     GetWaveformFreq(void) { return (1.0/(iWaveformSize * 1.0e-9 *
                                             GetSamplePeriod())); }
  int        GetWaveformPeriod(void) { return (iWaveformSize * GetSamplePeriod()); }

protected:
  const int               CiSampleClockPeriod = 500;  /* ns */
  const int               CiRateTimerMaxValue = 255;
  const int               CiBitWidth = 16;
  const int               CiTotalMemoryLoc = 0x4000; /* 32K bytes, 16K locs */
  volatile int16_t        *vwStartOfData;
  volatile uint16_t       *vuwStartOutput;
  volatile uint16_t       *vuwStopOutput;
  volatile uint16_t       *vuwStartAddressReg;
  volatile uint16_t       *vuwStopAddressReg;
  volatile uint16_t       *vuwRateTimerCtrlReg;
  tMpv955Board            *pBoard;
  int                     iWaveformSize;
  uint8_t                 u8CachedRateTimerValue; /* Rate Timer Reg is WRITE only */
  CONV_CODE               ccOutputCode;
  tMpv955WaveformChannel  ChannelArray[CiMaxNumChannels];
};


/*******************************************************************************
*
* class tMpv955Board
*
* State Variables:
*  tMpv955WaveformChannelBank ChannelBank
*  TRIGGER_SELECT   tsTriggerSelectR
*  TRIGGER_TYPE     ttTriggerTypeR
*  SCAN_MODE        smScanModeR
*  BOOL             bTimeoutEnableR
*  uint16_t         uwNumOutputChannelsR
*  BOOL             bHaltR                   (read-only)
*  BOOL             bTimeoutR                (read-only)
*  BOOL             bCycleFinishedR          (read-only)
*  BOOL             bOversampling            (read-only)
*
* METHODS:
* tMpv955Board::tMpv955Board(void)
* Description: Constructor - sets default values
[[D tMpv955Board::tMpv955Board(void)]
[ this is invalid => UNDEFINED

| true =>
  [[U tBoard::tBoard()]]
]]
*
*
* ERROR_CODE tMpv955Board::SoftInit(...)
* Description: Soft Init method for an MPV955 board.  Sets board address,
*              initializes pointers to board variables, and sets class
*              variables, and sets bInit to TRUE.
[[D tMpv955Board::SoftInitParams(int          iResetValueToUse,
                                 CONV_CODE    ccConvCodeToUse,
                                 uint32_t     uiNumChannelsToUse)]
[ this is invalid || pvmeAddressToUse is invalid => UNDEFINED
| true =>
  [[U tBoard::InitializeParams()]]
  : [[U tMpv955WaveformChannelBank::Initialize(
                pBoardToUse=this,
                vwDataToUse=(volatile uint16_t *)
                     (pvmeAddressToUse->GetLocalAddress() + CiDataOffset),
                vuwStartOutputToUse=(volatile uint16_t *)
                     (pvmeAddressToUse->GetLocalAddress() + CiStartOutputOffset),
                vuwStopOutputToUse=(volatile uint16_t)
                     (pvmeAddressToUse->GetLocalAddress() + CiStopOutputOffset),
                vuwRateTimerCtrlRegToUse=(volatile uint16_t)
                     (pvmeAddressToUse->GetLocalAddress() + CiRateTimerCtrlOffset),
                iResetValueToUse=iResetValueToUse,
                ccOutputCodeToUse=ccOutputCodeToUse,
                this=ChannelBank)]]
  : set pointers to board registers to appropriate values
  : return := OK
]]
*
*
* ERROR_CODE tMpv955Board::SoftInit(void)
* Description:  Default version of SoftInit. Sets board address, initializes
*               the pointers to board variables, and sets class variables using
*               the default values.
[[D tMpv955Board::SoftInitVoid(void)]
[ this is invalid => UNDEFINED
| true =>
  [[U tBoard::InitializeVoid()]]
  : [[U tMpv955WaveformChannelBank::Initialize(
                pBoardToUse=this,
                vwDataToUse=(volatile uint16_t *)
                     (this->vmeAddress.GetLocalAddress() + CiDataOffset),
                vuwStartOutputToUse=(volatile uint16_t *)
                     (this->vmeAddress.GetLocalAddress() + CiStartOutputOffset),
                vuwStopOutputToUse=(volatile uint16_t *)
                     (this->vmeAddress.GetLocalAddress() + CiStopOutputOffset),
                vuwRateTimerCtrlRegToUse=(volatile uint16_t)
                     (pvmeAddressToUse->GetLocalAddress() + CiRateTimerCtrlOffset),
                iResetValueToUse=iDefaultResetValue,
                ccOutputCodeToUse=ccDefaultOutputCode,
                this=ChannelBank)]]
  : set pointers to board registers to appropriate values
  : return := OK
]]
*
*
* ERROR_CODE tMpv955Board::HardInit(...)
* Description: Parameterized version of HardInit. Calls SoftInit with values passed
*              in and sets board registers with values passed in.
[[D tMpv955Board::HardInitParams(TRIGGER_SELECT  tsTriggerSelectToUse,
                                 TRIGGER_TYPE    ttTriggerTypeToUse,
                                 SCAN_MODE       smScanModeToUse,
                                 BOOL            bTimeoutEnableToUse,
                                 uint16_t        uwNumOutputChannelsToUse,
                                 int             iResetValueToUse,
                                 CONV_CODE       ccConvCodeToUse)]
[ true =>
  [[U tMpv955Board::SoftInitParams()]]
  : tsTriggerSelectR     := tsTriggerSelectToUse
  : ttTriggerTypeR       := ttTriggerTypeToUse
  : smScanModeR          := smScanModeToUse
  : bTimeoutEnableR      := bTimeoutEnableToUse
  : uwNumOutputChannelsR := uwNumOutputChannelsToUse
]]
*
*
* ERROR_CODE tMpv955Board::HardInit(void)
* Description: Default version of HardInit.  Calls default version of SoftInit and
*              sets default values in hardware.
[[D tMpv955Board::HardInitVoid(void)]
[ true =>
  [[U tMpv955Board::SoftInitVoid()]]
  : tsTriggerSelectR     := this->CtsDefaultTriggerSelect
  : ttTriggerTypeR       := this->CttDefaultTriggerType
  : smScanModeR          := this->CsmDefaultScanMode
  : bTimeoutEnableR      := this->CbDefaultTimeoutEnable
  : uwNumOutputChannelsR := this->CuwDefaultNumOutputChannels
]]
*
*
* ERROR_CODE tMpv955Board::GetChannel(...)
* Description:  Gets channel specified by iNumChannel.
*        MPV955 Global board numbering:
           see tMpv955ChannelBank
*
[[D tMpv955Board::GetChannel(int iChannelNum)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE || iChannelNum<0 || iChannelNum>this->GetNumChannels()-1 =>
  return := NULL
| true =>
  [[U tMpv955ChannelBank::GetChannel()]]
]]
*
*
* tVmeAddress tMpv955Board::GetDefaultVmeAddress(void)
* Description: Gets default vme address for the MPV955 board.
[[D tMpv955Board::GetDefaultVmeAddress(void)]
[ this is invalid => UNDEFINED
| true =>
  return := internal default vme address for this board
]]
*
*
* BOARD_NAME tMpv955Board::GetDefaultBoardName(void)
* Description: Gets default board name for the MPV955 board.
[[D tMpv955Board::GetDefaultBoardName(void)]
[ this is invalid => UNDEFINED
| true =>
  return := MPV955_BOARD_NAME
]]
*
*
* void tMpv955Board::DisableOutput(BOOL bOn)
* Description: Immediately disables all output channels if TRUE is passed in.
*              Immediately enables all output channels if FALSE is passed in.
[[D tMpv955Board::OutputEnable(BOOL bOn)]
[ this is invalid => UNDEFINED
| bOn == TRUE =>
  *vuwDisableOutputReg := 1
| bOn == FALSE =>
  *vuwDisableOutputReg := 0
| true => UNDEFINED
]]
*
*
* ERROR_CODE tMpv955Board::ZeroMemory(void)
* Description: Zeros all data memory locations on the board.
[[D tMpv955Board::ZeroMemory(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return S_mpv955_BOARD_NOT_INITIALIZED
| true =>
  this->all data memory locations := 0
]]
*
******/
class tMpv955Board : public tBoard {
public:
  tMpv955Board(int iBoardNum);
  ERROR_CODE SoftInit(int         iResetValueToUse,
                      int         iSamplePeriodToUse,
                      CONV_CODE   ccOutputCodeToUse,
                      int         iVoltageRangeToUse,
                      uint16_t    uwNumChannelsToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(TRIGGER_SELECT  tsTriggerSelectToUse,
                      TRIGGER_TYPE    ttTriggerTypeToUse,
                      SCAN_MODE       smScanModeToUse,
                      BOOL            bTimeoutEnableToUse,
                      int             iResetValueToUse,
                      int             iSamplePeriodToUse,
                      CONV_CODE       ccOutputCodeToUse,
                      int             iVoltageRangeToUse,
                      uint16_t        uwNumChannelsToUse);
  ERROR_CODE  HardInit(void);
  void        DumpRegs(void);
  tChannel    *CreateChannel(int iChannelNum);
  void        DisableOutput(BOOL bOn);
  int         GetNumChannels(void) { return iNumActiveChannels; }

  ERROR_CODE SetTriggerSelect(TRIGGER_SELECT tsTriggerSelectToUse);
  ERROR_CODE SetTriggerType(TRIGGER_TYPE ttTriggerTypeToUse);
  ERROR_CODE SetScanMode(SCAN_MODE smScanModeToUse);
  ERROR_CODE SetTimeoutEnable(BOOL bTimeoutToUse);
  ERROR_CODE SetNumChannels(void);

protected:

  tMpv955WaveformChannelBank ChannelBank;

///////////////////////
// BOARD DEFAULTS
//
  const TRIGGER_SELECT CtsDefaultTriggerSelect = TS_INTERNAL;
  const TRIGGER_TYPE   CttDefaultTriggerType   = TT_EVENT;
  const SCAN_MODE      CsmDefaultScanMode      = SM_CONTINUOUS;
  const BOOL           CbDefaultTimeoutEnable  = FALSE;
  const uint16_t       CuwDefaultNumChannels   = 5;
  const CONV_CODE      CccDefaultOutputCode    = CC_COMPLEMENT_BINARY;
  const int            CiDefaultVoltageRange   = 10;
  const int            CiDefaultSamplePeriod   = 2000;  /* ns */
  const int            CiDefaultResetValue     = 0;

 private:
  const uint16_t CiDataOffset = 0;
  const uint16_t CiCtrlStatusOffset1 = 0x8000;
  const uint16_t CiCtrlStatusOffset2 = 0x8010;
  const uint16_t CiStartAddressOffset1 = 0x8002;
  const uint16_t CiStartAddressOffset2 = 0x8012;
  const uint16_t CiStopAddressOffset1 = 0x8004;
  const uint16_t CiStopAddressOffset2 = 0x8014;
  const uint16_t CiInterruptCtrlOffset1 = 0x8006;
  const uint16_t CiInterruptCtrlOffset2 = 0x8016;
  const uint16_t CiRateTimerCtrlOffset1 = 0x8008;  
  const uint16_t CiRateTimerCtrlOffset2 = 0x8018;  
  const uint16_t CiTimeoutCtrlOffset1 = 0x800a;
  const uint16_t CiTimeoutCtrlOffset2 = 0x801a;
  const uint16_t CiDisableOutputOffset1 = 0x800c;
  const uint16_t CiDisableOutputOffset2 = 0x801c;
  const uint16_t CiStartOutputOffset = 0xc002;
  /* This MUST be an Area 1 register for StopOutput to work */
  /* It also MUST be a read/write reg so the proper contents */
  /*    will be written back */
  const uint16_t CiStopOutputOffset = CiCtrlStatusOffset1;
  volatile uint16_t *vuwCtrlStatusReg;
  volatile uint16_t *vuwInterruptCtrlReg;
  volatile uint16_t *vuwTimeoutCtrlReg;
  volatile uint16_t *vuwDisableOutputReg;
};
  
#endif /* INC_mpv955h */



