/* $RCSfile: pc03v.h,v $ - Hardware manager classes */
/* 
$Header: /home/chara/cvsroot/ople/sys/hw/include/pc03v.h,v 1.1.1.1 1998/06/11 21:36:35 chara Exp $
*/
/* 
* This file contains the PC03V card class definition
* The tPc03vBoard class is a subclass of tBoard and is used mostly
* by the hardware manager.
*/

/*
modification history
--------------------
01a,pci,17Oct96               written
*/

#ifndef PC03V_H
#define PC03V_H

#define PC03V_MIN_HB_RATE       16
#define PC03V_MINOR_CLOCK_SPEED 1000000
#define UW_NIBBLE(a,b)          ((uint32_t) (0x000f & ((a) >> (4*(b)))))


/****************************************
*
* Class: tPc03vBoard
*
*
* SPEC:
void tPc03vBoard::tPc03vBoard( this )

IN :
  this

OUT :
  *this.bInit
  *this.vCAPTR
  *this.vTIME0
  *this.vTIME1
  *this.vTIME2
  *this.vTIME3
  *this.vCLRIG
  *this.vCR0
  *this.vCR1
  *this.vBIMCR0
  *this.vBIMCR1
  *this.vBIMCR2
  *this.vBIMCR3
  *this.vBIMV0
  *this.vBIMV1
  *this.vBIMV2
  *this.vBIMV3
  *this.vRELFR
  *this.vSTR1W1
  *this.vSTR1W2
  *this.vSTR1W3
  *this.vSTR2W1
  *this.vSTR2W2
  *this.vSTR2W3
  *this.vCTC0
  *this.vCTCCR

  [ this is invalid -> UNDEFINED
  | true -> *this.bInit := FALSE
    : set all register address variables (everything in OUT
      section except *this.bInit) to NULL
  ]


ERROR_CODE tPc03vBoard::HardInit(this,
                            tVmeAddress vmeBoardAddress,
                            BOARD_NAME  boardName)
IN :
  this
  tVmeAddress vmeBoardAddress
  BOARD_NAME  boardName

INOUT :
  uint16_t *    *this.vCAPTR
  uint16_t *    *this.vTIME0
  uint16_t *    *this.vTIME1
  uint16_t *    *this.vTIME2
  uint16_t *    *this.vTIME3
  uint16_t *    *this.vCLRIG
  uint16_t *    *this.vCR0
  uint16_t *    *this.vCR1
  uint16_t *    *this.vBIMCR0
  uint16_t *    *this.vBIMCR1
  uint16_t *    *this.vBIMCR2
  uint16_t *    *this.vBIMCR3
  uint16_t *    *this.vBIMV0
  uint16_t *    *this.vBIMV1
  uint16_t *    *this.vBIMV2
  uint16_t *    *this.vBIMV3
  uint16_t *    *this.vRELFR
  uint16_t *    *this.vSTR1W1
  uint16_t *    *this.vSTR1W2
  uint16_t *    *this.vSTR1W3
  uint16_t *    *this.vSTR2W1
  uint16_t *    *this.vSTR2W2
  uint16_t *    *this.vSTR2W3
  uint16_t *    *this.vCTC0
  uint16_t *    *this.vCTCCR
  tVmeAddress *this.vmeAddress
  BYTE  *    *this.v8LocalAddress
  BOARD_NAME *this.boardName
  BOOL       *this.bInit
  INT_RATE   *this.pRate[4]
  BOARD_FUNC *this.bfFunctions 

  TIME_FORMAT *this.tfTimeFormat
  TIME_DIRECTION *this.tfTimeDirection
  MINOR_CLOCK *this.mcMinorClock
  CHAN_SELECT *this.csChannelSelect
  BOOL *this.bSecondaryCapture
  TRIGGER_SENSE *this.tsTriggerSense
  BOOL *this.bIntEnable[0..3]
  STROBE_MODE *this.smStrobe1Mode
  STROBE_MODE *this.smStrobe2Mode
  STROBE--_SENSE *this.ssStrobe1Sense
  STROBE_SENSE *this.ssStrobe2Sense
  STROBE_SENSE *this.ssIntervalGateSense
  BOOL *this.bHbSync
  BOOL * *this.pbIntActivate[0..3]
  BOOL * *this.pbIntFlags[0..3]
  BOOL * *this.pbIntFAC[0..3]
  BOOL * *this.pbIntIRAC[0..3]
  uint16_t * *this.puwIntNum[0..3]
  uint16_t * *this.puwVector[0..3]

  [this is invalid -> UNDEFINED
  | vmeBoardAddress is invalid  -> UNDEFINED
  | vmeBoardAddress not reachable by CPU ->
      return VxWorks error code from sysBusToLocalAdrs
  | *this.bInit := TRUE -> return S_pc03v_BOARD_ALREADY_INITIALIZED
  | true -> *this.vmeAddress := vmeBoardAddress
    : *this.boardName := boardName
    : *this.bfFunctions := BF_CLOCK | BF_INTERRUPT
    : *this.bInit := TRUE
    : *this.pRate[0..3] := 0
    : *this.v8LocalAddress := local address corresponding to vmeBoardAddress
    :: *this.vCAPTR:= v8LocalAddress + 0x0
    :: *this.vTIME0:= v8LocalAddress + 0x2
    :: *this.vTIME1:= v8LocalAddress + 0x4
    :: *this.vTIME2:= v8LocalAddress + 0x6
    :: *this.vTIME3:= v8LocalAddress + 0x8
    :: *this.vCLRIG:= v8LocalAddress + 0xA
    :: *this.vCR0:= v8LocalAddress + 0xC
    :: *this.vCR1:= v8LocalAddress + 0xE
    :: *this.vBIMCR0:= v8LocalAddress + 0x20
    :: *this.vBIMCR1:= v8LocalAddress + 0x22
    :: *this.vBIMCR2:= v8LocalAddress + 0x24
    :: *this.vBIMCR3:= v8LocalAddress + 0x26
    :: *this.vBIMV0:= v8LocalAddress + 0x28
    :: *this.vBIMV1:= v8LocalAddress + 0x2A
    :: *this.vBIMV2:= v8LocalAddress + 0x2C
    :: *this.vBIMV3:= v8LocalAddress + 0x2E
    :: *this.vRELFR:= v8LocalAddress + 0x30
    :: *this.vSTR1W1:= v8LocalAddress + 0x42
    :: *this.vSTR1W2:= v8LocalAddress + 0x44
    :: *this.vSTR1W3:= v8LocalAddress + 0x46
    :: *this.vSTR2W1:= v8LocalAddress + 0x62
    :: *this.vSTR2W2:= v8LocalAddress + 0x64
    :: *this.vSTR2W3:= v8LocalAddress + 0x66
    :: *this.vCTC0:= v8LocalAddress + 0x80
    :: *this.vCTCCR:= v8LocalAddress + 0x86
  *** CR0 ***
    : *this.tfTimeFormat:= TF_IRIG_B
    : *this.tdTimeDirection:= TD_FWD
    : *this.mcMinorClock:= MC_PLL
    : *this.csChannelSelect:= CS_CHAN_1
  *** CR1 ***
    : *this.bSecondaryCapture:= TRUE
    : *this.tsTriggerSense:= TS_RISING
    : *this.bIntEnable[0..3]:= FALSE
    : *this.smStrobe1Mode:= SM_MAJOR_MINOR
    : *this.smStrobe2Mode:= SM_MAJOR_MINOR
    : *this.ssStrobe1Sense:= SS_ACTIVE_HI
    : *this.ssStrobe2Sense:= SS_ACTIVE_HI
    : *this.ssIntervalGateSense:= SS_ACTIVE_HI
    : *this.bHbSync:= TRUE
  *** BIMCR0 - BIMCR3 ***
    : *this.pbIntActivate[0..3]:= FALSE
    : *this.pbIntFlags[0..3]:= FALSE
    : *this.pbIntFAC[0..3]:= FALSE
    : *this.pbIntIRAC[0..3]:= FALSE
    : *this.puwIntNum[0..3]:= 0
  *** BIMV0 - BIMV3 ***
    : *this.puwVector[0..3]:= 0x000F
    : return OK
  ]


ERROR_CODE tPc03vBoard::HardInit(this,
                            tVmeAddress vmeBoardAddress,
                            BOARD_NAME  boardName)
IN :
  this
  tBoard *pBoard
  int iWhich
  unsigned int uiIntNum

INOUT :
  *this.pBoard
  *this.bInit
  *this.iWhich
  *this.uiIntNum

[ this is invalid -> UNDEFINED
| pBoard is invalid -> UNDEFINED
| *this.bInit == TRUE -> return S_pc03v_E_INTERRUPT_ALREADY_INITIALIZED
| *pBoard does not have interrupt capabilities ->
    return S_pc03v_E_BOARD_HAS_NO_INTERRUPT
| pBoard.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
| true -> *this.pBoard := pBoard
  : *this.iWhich := iWhich
  : *this.uiIntNum := uiIntNum
  : *this.bInit := TRUE
  : return OK
]


ERROR_CODE tPc03vBoard::RateSet(this, INT_RATE *pRate, int iIntSource)

IN :
  this
  BOOL       *this.bInit
  int        iIntSource

INOUT :
  INT_RATE   *pRate
  tBoard     *this

[ this is invalid -> UNDEFINED
| pRate is invalid -> UNDEFINED
| this.bInit == FALSE ->
    return uninit error code for *this
| iIntSource is not a valid interrupt source on *this ||
  iIntSource is not a clock-driven interrupt source for *this ->
    return invalid interrupt source error for *this
| *pRate > highest possible rate for *this ->
    *pRate := highest possible rate for *this
    : *this.pRate[iIntSource] := highest possible rate for *this
    : return OK
| *pRate not valid for *this. -> 
    *pRate := valid rate nearest and higher to *pRate
    : *this.pRate[iIntSource] := valid rate nearest and higher to *pRate
    : return OK
| true -> *this.pRate[iIntSource] := *pRate
    :  return OK 
]


INT_RATE tPc03vBoard::RateGet(this, int iIntSource)

IN :
  this
  int             iIntSource

INOUT :
  INT_RATE        *this.pRate[iIntSource]

[ this is invalid -> UNDEFINED
| *this.bInit == FALSE -> UNDEFINED 
| iIntSource is not a valid interrupt source on *this ||
  iIntSource is not a clock-driven interrupt source for *this -> UNDEFINED
| *this.pRate[iIntSource] is out of sync with actual rate -> incorrect result
| true -> return currently set rate for the iIntSource source of *this
]

ERROR_CODE tPc03vBoard::Connect(this,
                           FUNCPTR pFunc,
                           int iArg,
                           unsigned int uiIntNum,
                           unsigned int uiIntVector,
                           int iIntSource)
 
IN :
  this
  FUNCPTR      pFunc
  BOOL         *this.bInit
  int          *this.iWhich
  unsigned int *this.uiIntNum
  BOOL         *this.*pBoard.bInit

INOUT :
  tBoard *     *this.pBoard

[ this is invalid -> UNDEFINED
| *this.bInit == FALSE -> return S_pc03v_E_BOARD_NOT_INITIALIZED
| pFunc is invalid -> UNDEFINED
| ( pFunc is not a pointer to valid C function ||
    ! ( pFunc is a pointer to valid C++ function &&
    (this == this))) ->UNDEFINED
| pFunc == NULL -> return S_pc03v_E_NULL_FUNCTION_POINTER
| iIntSource is out of range for *this -> 
        return appropriate error for *this
| uiIntNum < MIN_BUS_INT_PRIORITY || uiIntNum > MAX_BUS_INT_PRIORITY ->
        return error for *this
| true -> 
  : enable board interrupt source iintSource
  : connect board interrupt source iIntSource to
      VME interrupt uiIntNum at *this.uwIntVector
  : Attach pFunc to Interrupt uiIntNum at *this.uwIntVector
  : return OK
]

ERROR_CODE tPc03vBoard::Disconnect(this, iIntSource)

IN :
  this

INOUT :
    
[ this is invalid -> UNDEFINED
| *this.bInit == FALSE -> return uninit error for *this
| true ->
  [ iIntSource is out of range for board ->
      return appropriate error for *this
  | true -> Disable interrupt source iIntSource
    : *this.puiIntNum[iWhich] := 0
    : *this.puiIntVector[iWhich] := 0x000f
  ]
  : return OK
]

ERROR_CODE tPc03vBoard::Enable( this, iIntSource )

IN :
 this
 BOOL         *this.bInit
 int          iIntSource

INOUT :


[ this is invalid -> UNDEFINED 
| More than one CPU is connected to *this.uiIntNum -> UNDEFINED
| *this.bInit == FALSE -> return uninit error for *this
| iIntSource not valid range for *this ->
     return error for *this
| true -> enable interrupt iWhich on *this.*pBoard
    : Begin sending interrupt number *this.puwIntNum[iIntSource] to the
        VME bus.  The ISR on the CPU connected to
        *this interrupt source iWhich will be called at a rate of
        *this.*ppRate[iIntSource] until a Disable command is given.
  : *this.bEnabled := TRUE
  : return OK
]

ERROR_CODE tPc03vBoard::Disable(this, int iIntNum)

IN :
 this
 BOOL      *this.bInit

INOUT :

RETURNS : ERROR_CODE
S_pc03v_E_INT_NOT_INITIALIZED
S_pc03v_E_INT_NOT_ENABLED
 
[ this is invalid -> UNDEFINED 
| *this.bInit == FALSE -> return uninit error for *this
| iWhich is out of range for *this ->
      return appropriate error for *this
| true -> *this.bEnabled := FALSE
  : disable interrupt iIntNum on *this
  : Stop sending interrupt number *this.puwIntNum[iIntSource]
      to the VME bus. The ISR on the CPU connected to
      *this.puwIntNum[iSource] will stop being called periodically
  : return OK
]

TIME tClock::GetTimeLastInt ( this )

IN :
  this
  *this.bInit
  *this.time registers

INOUT :

[ this is invalid -> UNDEFINED
| *this.bInit == FALSE -> UNDEFINED
| *this.*pBoard.bInit == FALSE -> UNDEFINED
| *this.*pBoard does not have clock function -> UNDEFINED
| manual capture (GetTimeNow) is performed during this function -> UNDEFINED
| true -> return time of last interrupt in 25us since midnight
]


BOOL IsHbSynced(this)

IN :
  this
  *this.bHbSync

[ this is invalid -> UNDEFINED
| *this.bHbSync == TRUE -> return TRUE
| true -> return FALSE
]


void ClkIsr(this, iIntSource)

IN :
  this
  int iIntSource
  *this.ppFunc[iIntSource]
  *this.piArg[iIntSource]

  [ this is invalid -> UNDEFINED
  | board has not been initialized -> UNDEFINED
  | true -> release capture lockout
    : call *this.ppFunc[iIntSource] with argument *this.piArg[iIntSource]
  ]

*/

class tPc03vBoard : public tBoard {
  public:
    tPc03vBoard(int iBoardNum);
    ERROR_CODE SoftInit(void);
    ERROR_CODE HardInit(void);
    tChannel *CreateChannel(int iChannelNum);
    ERROR_CODE RateSet(INT_RATE *pRate, int iIntSource);
    INT_RATE   RateGet(int iIntSource);
    ERROR_CODE Connect(FUNCPTR   pFunc,
                       int       iArg,
                       unsigned int uiIntNum,
                       unsigned int uiIntVector,
                       int       iIntSource);
    ERROR_CODE Disconnect(int iIntSource);
    ERROR_CODE Enable(int iIntSource);
    ERROR_CODE Disable(int iIntSource);
    TIME       GetTimeLastInt();

  private:
    void ClkIsr(int iIntSource);
    BOOL IsHbSynced(void);       /* checks if HB is synced to 1PPS */

    INT_RATE pRate[4];           /* interrupt rate array */
    FUNCPTR  ppFunc[4];          /* function to call upon interrupt */
    int      piArg[4];           /* argument to pass to function */
    /* Board register address variables */
    volatile uint16_t *vCAPTR;
    volatile uint16_t *vTIME0;
    volatile uint16_t *vTIME1;
    volatile uint16_t *vTIME2;
    volatile uint16_t *vTIME3;
    volatile uint16_t *vCLRIG;
    volatile uint16_t *vCR0;
    volatile uint16_t *vCR1;
    volatile uint16_t *vBIMCR0;
    volatile uint16_t *vBIMCR1;
    volatile uint16_t *vBIMCR2;
    volatile uint16_t *vBIMCR3;
    volatile uint16_t *vBIMV0;
    volatile uint16_t *vBIMV1;
    volatile uint16_t *vBIMV2;
    volatile uint16_t *vBIMV3;
    volatile uint16_t *vRELFR;
    volatile uint16_t *vSTR1W1;
    volatile uint16_t *vSTR1W2;
    volatile uint16_t *vSTR1W3;
    volatile uint16_t *vSTR2W1;
    volatile uint16_t *vSTR2W2;
    volatile uint16_t *vSTR2W3;
    volatile uint16_t *vCTC0;
    volatile uint16_t *vCTCCR;
} ;


#endif
