/*$RCSfile: pztMotor.h,v $  - header file for pztMotor.cc library */
/*
$Header: /home/chara/cvsroot/ople/sys/hw/include/pztMotor.h,v 1.2 1998/08/17 21:01:10 phil Exp $
*/

/*
modification history
--------------------
03d,pci,18May97     Changed ramp channel to inherit from daChannel
03c,pci,16May97     Added GetChannel()
03b,pci,01May97     modified for new hardware architecture
03a,pci,22Oct96     modified for ISIT
02a,beh,18Nov93     Modified to work with both C and C++
01d,dlp,04Oct93     added stuff for pztPostionSet function
01c,beh,13Dec91     moved some stuff off to pztMotorConfig.h
01b,dlp,13Nov90     changed Lint16_t type in motorPositionRead to int type
01a,dlp,05Oct90     written, debugged
*/                                                

#ifndef INC_pztMotorh
#define INC_pztMotorh

#define PZT_MOTOR_MAX_NUM_PZTS   (6)
#define PZT_MOTOR_MAX_NUM_MOTORS (6)
#define PZT_MOTOR_MAX_NUM_LIMITS (30)

/****************************************************************
*
* Channel Class definitions
*
****************************************************************/

/****************************************************************
*
* class tPztMotorDaChannel
*
* State Variables:
* ----------------
*
* METHODS:
[[D tPztMotorDaChannel::Initialize(in: vwDataOutToUse, ccConvCodeToUse,
                                         dVoltageRangeToUse, iResetValueToUse,
                                         pReadbackChannelToUse;
                                  inout: this;)]
[ this is invalid || vwDataOutToUse is invalid ||
    pReadbackChannelToUse is invalid => UNDEFINED
| true =>
  set state variables to corresponding parameter values
  : return := OK
]]
*
*
[[D tPztMotorDaChannel::Output(in: this, iValue;)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
  : return := OK
| true =>
  this->dActualChannelValue := voltage ramp corresponding to iValue
  : return := OK
]]
*
******/
class tPztMotorDaChannel : public tDaChannel {
public:
  tPztMotorDaChannel(void);
  virtual ERROR_CODE Initialize(volatile int16_t *vwDataOutToUse,
                        CONV_CODE        ccConvCodeToUse,
                        double           dVoltageRangeToUse,
                        int              iResetValueToUse,
                        tInputChannel    *pReadbackChannelToUse);
  virtual ERROR_CODE Output(int iValue);

protected:
  const int        CiBitWidth = 16;
  volatile int16_t *vwDataOut;
};


/****************************************************************
*
* class tPztMotorRampChannel
*
* State Variables:
* ----------------
*
* METHODS:
[[D tPztMotorRampChannel::Initialize(in: vwDataOutToUse, ccConvCodeToUse,
                                         dVoltageRampRangeToUse, iResetValueToUse,
                                         pReadbackChannelToUse;
                                  inout: this;)]
[ this is invalid || vwDataOutToUse is invalid ||
    pReadbackChannelToUse is invalid => UNDEFINED
| true =>
  set state variables to corresponding parameter values
  : return := OK
]]
*
*
[[D tPztMotorRampChannel::Output(in: this, iValue;)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->dActualChannelValue := voltage ramp corresponding to iValue
  : return := OK
]]
*
*
[[D tPztMotorRampChannel::Reset(in: this;)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->dActualChannelValue := midrange voltage value
  : return := OK
]]
*
******/
class tPztMotorRampChannel : public tPztMotorDaChannel {
public:
  tPztMotorRampChannel(void);
  ERROR_CODE Initialize(volatile int16_t *vwDataOutToUse,
                        CONV_CODE        ccConvCodeToUse,
                        double           dVoltageRampRangeToUse,
                        int              iResetValueToUse,
                        tInputChannel    *pReadbackChannelToUse);
  ERROR_CODE Output(int iValue);
  ERROR_CODE Reset(void);

protected:
  ERROR_CODE       ClearCounter(void);
  const int        CiZeroVelocity = 0;
  volatile uint16_t *vuwClearCmd;
};



/****************************************************************
*
* class tPztMotorStepMotorChannel
*
* State Variables:
* ----------------
*
* METHODS:
[[D tPztMotorStepMotorChannel::Initialize(in: vwDataOutToUse, ccConvCodeToUse,
                                         dVoltageRampRangeToUse,
                                         pReadbackChannelToUse;
                                  inout: this;)]
[ this is invalid || vwDataOutToUse is invalid ||
    pReadbackChannelToUse is invalid => UNDEFINED
| true =>
  set state variables to corresponding parameter values
  : return := OK
]]
*
*
[[D tPztMotorStepMotorChannel::Output(in: this, iValue;)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->dActualChannelValue := voltage ramp corresponding to iValue
  : return := OK
]]
*
*
[[D tPztMotorStepMotorChannel::Reset(in: this;)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->dActualChannelValue := midrange voltage value
  : return := OK
]]
*
******/
class tPztMotorStepMotorChannel : public tStepMotorChannel {
public:
  tPztMotorStepMotorChannel(void);
  ERROR_CODE Initialize(volatile int16_t *vwDataOutToUse,
                        CONV_CODE        ccConvCodeToUse,
                        double           dPulseSpeedRangeToUse,
                        tInputChannel    *pReadbackChannelToUse);
  ERROR_CODE Output(int iValue);
  ERROR_CODE Reset(void);
  
protected:         
  ERROR_CODE       ClearCounter(void);
  const int        CiResetValue = 0;
  const int        CiBitWidth = 16;
  const int        CiZeroVelocity = 0;
  volatile int16_t  *vwDataOut;
  volatile uint16_t *vuwClearCmd;
};

/****************************************************************
*
* class tPztMotorReadbackChannel
*
* State Variables:
* ----------------
*
* METHODS:
[[D tPztMotorReadbackChannel::Initialize(in: vwDataInToUse, ccConvCodeToUse,
                                         dVoltageRampRangeToUse;
                                  inout: this;)]
[ this is invalid || vwDataOutToUse is invalid ||
    pReadbackChannelToUse is invalid => UNDEFINED
| true =>
  set state variables to corresponding parameter values
  : return := OK
]]
*
*
[[D tPztMotorReadbackChannel::Input(in: this; inout: *piValue;)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
  : return OK
| true =>
  *piValue := integer value corresponding to physical value at input
  : return OK
]]
*
******/
class tPztMotorReadbackChannel : public tReadbackChannel {
public:
  tPztMotorReadbackChannel(void);
  ERROR_CODE Initialize(volatile int16_t *vwDataInToUse,
                        CONV_CODE        ccConvCodeToUse,
                        double           dVoltageRangeToUse);
  ERROR_CODE Input(int *piValue) const;

protected:
  volatile int16_t *vwDataIn;
  const int        CiBitWidth  = 16;
};


/****************************************************************
*
* class tPztMotorDigitalInChannel
*
* State Variables:
* ----------------
*
* METHODS:
[[D tPztMotorDigitalInChannel::Initialize(in: vwDataInToUse,
                                         ltLimitTypeToUse;
                                  inout: this;)]
[ this is invalid || vwDataOutToUse is invalid ||
    pReadbackChannelToUse is invalid => UNDEFINED
| true =>
  set state variables to corresponding parameter values
  : return := OK
]]
*
*
[[D tPztMotorDigitalInChannel::Input(in: this; inout: *piValue;)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
  : return OK
| true =>
  *piValue := integer value corresponding to physical value at input
  : return OK
]]
*
******/
class tPztMotorDigitalInChannel : public tDigitalInChannel {
public:
  tPztMotorDigitalInChannel(void);
  ERROR_CODE Initialize(volatile uint16_t *vuwDataInToUse,
                        LIMIT_TYPE        ltLimitTypeToUse);
  ERROR_CODE Input(int *piValue) const;
  
protected:
  const int         CiBitWidth     = 1;
  const double      CdChannelRange = 2.0;
  const CONV_CODE   CccConvCode    = CC_NO_CONV_CODE;

  volatile uint16_t *vuwDataIn;
  int               iStatusBitNum;
};


/****************************************************************
*
* class tPztMotorBoard
*
* tPztMotorBoard(this)
* Description: Constructor - initializes class variables
[[D tPztMotorBoard::tPztMotorBoard(inout: this;)]
[ this is invalid => UNDEFINED 
| true => initialize numInstalled variables
  : *this.bInit := FALSE
]]
*
*  
[[D tPztMotorBoard::SoftInit(in: vmeBoardAddress, 
                                 boardName,
                                 numPzts;
                          inout: this;)]
[this is invalid || vmeBoardAddress is invalid => UNDEFINED
| vmeBoardAddress not reachable by CPU =>
    return VxWorks error code from sysBusToLocalAdrs
| numCounters is not in range of board =>
    return S_pztMotor_INIT_INVALID_ARGUMENT
| *this.bInit == TRUE => return S_pztMotor_BOARD_ALREADY_INITIALIZED
| true => *this.vmeAddress := vmeBoardAddress
  : *this.v8LocalAddress := local address corresponding to vmeBoardAddress
  : *this.boardName := boardName
  : *this.bInit := TRUE
  : initialize board register pointers and protected variables
  : return OK
]]
*
*
[[D tPztMotorBoard::HardInit(in: vmeBoardAddress,
                                 boardName,
                                 numPzts;
                          inout: this;)]
[this is invalid || vmeBoardAddress is invalid => UNDEFINED
| vmeBoardAddress not reachable by CPU =>
    return VxWorks error code from sysBusToLocalAdrs
| numCounters is not in range of board =>
    return S_pztMotor_INIT_INVALID_ARGUMENT
| *this.bInit == TRUE => return S_pztMotor_BOARD_ALREADY_INITIALIZED
| true => *this.vmeAddress := vmeBoardAddress
  : *this.v8LocalAddress := local address corresponding to vmeBoardAddress
  : *this.boardName := boardName
  : *this.bInit := TRUE
  : initialize board register pointers and protected variables
  : reset numPzts channels
  : return OK
]]
*
*
* ERROR_CODE tPztMotorBoard::GetChannel(iChannelNum)
* Description:  Gets channel specified by iChannelNum.
*        PztMotor Global board numbering:
*     PztChannels 0-5 ==> Global Channels 0-5
*     MotorChannels 0-5 ==> Global Channels 6-11
*     PztReadbackChannels 0-5 ==> Global Channels 12-17
*     MotorReadbackChannels 0-5 ==> Global Channels 18-23
*     LimitChannels 0-4 ==> Global Channels 24-28
*
[[D tPztMotorBoard::GetChannel(in: this, iChannelNum; out: return;)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := NULL
| iChannelNum<0 || iChannelNum> this->Max global channel number ||
    iChannelNum is not valid for current configuration =>
  return := NULL
| true =>
  return := Channel corresponding to iChannelNum
]]
*
******/
class tPztMotorBoard : public tBoard {
public:
  tPztMotorBoard(int iBoardNum);
  ERROR_CODE SoftInit(const int         iNumPztsToUse,
                      const BOOL        *bRampArray,
                      const CONV_CODE   *ccConvCodeArray,
                      const double      *dPztRangeArray,
                      const int         iNumMotorsToUse, 
                      const double      *dMotorRangeArray,
                      const int         iNumLimitsToUse);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(const int         iNumPztsToUse,
                      const BOOL        *bRampArrayToUse,
                      const CONV_CODE   *ccConvCodeArray,
                      const double      *dPztRangeArray,
                      const int         iNumMotorsToUse, 
                      const double      *dMotorRangeArray,
                      const int         iNumLimitsToUse);
  ERROR_CODE HardInit(void);
  tChannel *CreateChannel(int iChannelNum);

protected:
  uint8_t GetMotorOffset(int iWhichChannel)  
                          { return ((uint8_t) ((iWhichChannel+1) << 5)); }
  uint8_t GetMotorReadbackOffset(int iWhichChannel)
                          { return ((uint8_t) ((iWhichChannel+1) << 5)); }
  uint8_t GetPztOffset(int iWhichChannel)
                          { return ((uint8_t) (((iWhichChannel+1) << 5) + 0x10)); }
  uint8_t GetPztReadbackOffset(int iWhichChannel)
                          { return ((uint8_t) (((iWhichChannel+1) << 5) + 0x10)); }
  uint8_t GetLimitOffset(int iWhichChannel) {
    const int CiLimitsPerChannel = PZT_MOTOR_MAX_NUM_LIMITS / PZT_MOTOR_MAX_NUM_MOTORS;
    int iWhichMotorChannel = iWhichChannel / CiLimitsPerChannel;
    return ((uint8_t) (((iWhichMotorChannel+1) << 5) + 0x0c));
  }

  const int    CiDaResetValue   = 0;
  const int    CiRampResetValue = 0;
  const double CiPztInputFreq   = 8.0e6;
  const double CiMotorInputFreq = 2.0e6;

  BOOL bRampArray[PZT_MOTOR_MAX_NUM_PZTS];
  int iNumPztChannels;
  int iNumMotorChannels;
  int iNumLimitChannels;
  tPztMotorReadbackChannel  PztReadbackChannelArray[PZT_MOTOR_MAX_NUM_PZTS];
  tPztMotorReadbackChannel  MotorReadbackChannelArray[PZT_MOTOR_MAX_NUM_MOTORS];
  tPztMotorDaChannel        *pPztChannelArray[PZT_MOTOR_MAX_NUM_PZTS];
  tPztMotorStepMotorChannel MotorChannelArray[PZT_MOTOR_MAX_NUM_MOTORS];
  tPztMotorDigitalInChannel LimitChannelArray[PZT_MOTOR_MAX_NUM_LIMITS];

};

#endif  /* INC_pztMotorh */
