/* $RCSfile: vip640a.h,v $ - Header file for the VIP640A Frame Graber Card */

/*
$Header: /home/chara/cvsroot/ople/sys/hw/include/vip640a.h,v 1.1.1.1 1998/06/11 21:36:35 chara Exp $
*/

#ifndef INC_vip640ah
#define INC_vip640ah

#include "board.h"
#include "imageChannel.h"
#include "vip640aConst.h"

/****************************************************************************

tVip640aFullImageChannel

DESCRIPTION:

BlackBox State
--------------
this->bInit - Variable to indicate if tVip640aFullImageChannel has been initalized
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
this->bContinuousGrab - Is the Full Image Channel in Continous Grab mode
this->pidUpdateSignaler - the process id of the UpdateSignaler
this->vmeRegAddr - Register Address Space
this->vmeFrBufAddr - Frame Buffer Address Space

METHODS:
  tVip640aFullImageChannel::tVip640aFullImageChannel();
  
[[D tVip640aFullImageChannel::tVip640aFullImageChannel(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := FALSE
    : this->iHSize := 0
    : this->iVSize := 0
    : this->pidUpdateSignaler := 0
    : this->bContinuousGrab := FALSE
]]

  tVip640aFullImageChannel::~tVip640aFullImageChannel();
  
[[D tVip640aFullImageChannel::~tVip640aFullImageChannel(void) ]
[ invalid_pointer (this) ||
  this->pidUpdateSignaler is invalid => UNDEFINED
| this->pidUpdateSignaler == 0 => nothing
| true => UpdateSignaler is killed
]]

  ERROR_CODE tVip640aFullImageChannel::Initialize(tVmeAddress vmeRegAddr, tVmeAddress vmeFrBufAddr);
  
[[D tVip640aFullImageChannel::Initialize(tVmeAddress vmeRegAddr, tVmeAddress vmeFrBufAddr) ]
[ invalid_pointer (this) ||
  invalid_pointer (vmeRegAddr) ||
  invalid_pointer (vmeFrBufAddr) => UNDEFINED
| this->bInit == TRUE => ERROR
| true =>
    this->bInit := TRUE
    : this->iHSize := VIP640A_HSIZE
    : this->iVSize := VIP640A_VSIZE
    : this->vmeRegAddr := vmeRegAddr
    : this->vmeFrBufAddr := vmeFrBufAddr
    : this->bContinuousGrab := FALSE
    : return := OK
]]

  ERROR_CODE tVip640aFullImageChannel::GetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image);
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.

[[D tVip640aFullImageChannel::GetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := frame from frame grabber in the region define by a box starting at
             (iStartX, iStartY) and of size (iHSize, iVSize)
    : return := OK
]]

  ERROR_CODE tVip640aFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image);
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.  The image is guaranteed to be from 
     a single frame.

[[D tVip640aFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                              int iHSize, int iVSize, tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := an atomic frame from frame grabber  in the region define by a box 
             starting at (iStartX, iStartY) and of size (iHSize, iVSize)
    : return := OK
]]

  virtual ERROR_CODE tVip640aFullImageChannel::StartCapture() = 0;
     This method initially puts the tFullImageChannel into a normal operating condition.
     This should only be called when images are going to be needed soon.  Depending on
     hardware, this method may need schedular to be initalize before this method is called.
     
[[D tVip640aFullImageChannel::StartCapture(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true => 
    this->pidUpdateSignaler := schedual UpdateSignaler
    : this->bContinuousGrab := TRUE
    : return := OK
]]

  virtual ERROR_CODE tVip640aFullImageChannel::Reset() = 0;
     This method puts restores the tVip640aFullImageChannel to a normal operating condition.
     
[[D tVip640aFullImageChannel::Reset(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  capture not yet started => ERROR
| true => 
    this->bContinuousGrab := TRUE
    : return := OK
]]

  virtual ERROR_CODE tVip640aFullImageChannel::Update() = 0;
     This method starts and waits for an update to occure.

[[D tVip640aFullImageChannel::Update(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  capture not yet started => ERROR
| true => 
  next frame is ready
  : return := OK
]]

****************************************************************************/
class tVip640aFullImageChannel : public tFullImageChannel {
public:
  tVip640aFullImageChannel();
  virtual ~tVip640aFullImageChannel();
  ERROR_CODE Initialize(tVmeAddress vmeRegAddr, tVmeAddress vmeFrBufAddr);
  ERROR_CODE GetImageRegion (int iStartX, int iStartY, int iHSize, int iVSize, tImage &Image);
  ERROR_CODE AtomicGetImageRegion (int iStartX, int iStartY, int iHSize, int iVSize, tImage &Image);
  ERROR_CODE StartCapture();
  ERROR_CODE Reset();
  ERROR_CODE Update();
protected:
  ERROR_CODE UpdateSignaler (void);
  ERROR_CODE SelectBuf(int bufferNum);
  
  int iWhichBuffer;
  int pidUpdateSignaler;
  BOOL bContinuousGrab;
  SEM_ID semUpdateFinished;
  
  /* Pointers to various registers on Matrox VIP-640A card.   */
  /* Initialized by vip640aInit.                              */
  volatile uint8_t *vSnapShotReg;
  volatile uint8_t *vStatReg;
  volatile uint8_t *vCrtcAddrReg;
  volatile uint8_t *vCrtcStatReg;
  volatile uint8_t *vCrtcDataReg;

  volatile uint8_t *vFrBuffAddr;
  };
  
/*******************************************************************************

class tVip640aBoard : public tBoard

DESCRIPTION:

BlackBox State
--------------
ficImageChannel - Image Channel of the board

METHODS

  tVip640aBoard::tVip640aBoard()
     Compiler Default Constructor

[[D tBoard::tBoard(void)]
[ invalid_pointer (this) => UNDEFINED
| true =>
  this->bInit := FALSE
  : this->bInit := FALSE
]]

  tVip640aBoard::tVip640aBoard(int iBoardNum)

[[D tBoard::tBoard(void)]
[ invalid_pointer (this) => UNDEFINED
| true =>
  this->bInit := FALSE
  : this->bInit := FALSE
  : this->BoardName := VIP640A_BOARD_NAME + iBoardNum
]]

  ERROR_CODE tVip640aBoard::SoftInit(void);

[[D tVip640aBoard::SoftInit(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => initialize ficImageChannel
]]

  ERROR_CODE tVip640aBoard::HardInit(void);

[[D tVip640aBoard::HardInit(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    initialize Vip640a Card
    : initialize ficImageChannel
]]

  tChannel tVip640aBoard::*CreateChannel (int iChannelNum);

[[D tVip640aBoard::CreateChannel (int iChannelNum) ]
[ invalid_pointer(this) => UNDEFINED
| (iChannelNum != 0) ||
  (bInit == FALSE) => ERROR
| true => return := &ficImageChannel
]]
*******************************************************************************/

class tVip640aBoard : public tBoard {
public:
  tVip640aBoard(int iBoardNum);
  ERROR_CODE SoftInit(void);
  ERROR_CODE HardInit(void);
  tChannel   *CreateChannel (int iChannelNum);

protected:  
  tVip640aFullImageChannel ficImageChannel;
  tVmeAddress vmeFrBufAddr;
  };

#endif
