/* Frame grabber constants: */

/* Registers:                     */

/* Control Register 0 (write):    */
#define CNTRL_REG_0                     0x2800

/* Control Register 1:            */
#define CNTRL_REG_1                     0x2900

/* Mask Register:                 */
#define MASK_REG                        0x2A00

/* Status Register:               */
#define STAT_REG                        0x2B00

/* Snapshot Register:             */
#define SNAPSHOT_REG                    0x2C00

/* Video Gain Register:           */
#define VIDGAIN_REG                     0x3800

/* Video Offset Register:         */
#define VIDOFFSET_REG                   0x3C00

/* CRTC Address Register (write): */
#define CRTC_ADDR_REG                   0x3000

/* CRTC Status Register  (read):  */
#define CRTC_STAT_REG                   0x3000

/* CRTC Data Register:            */
#define CRTC_DATA_REG                   0x3400


/* Frame buffer constants:        */
#define FRAME_BUFFER_LENGTH           0x100000   /* 1 Mega-bytes  */

#define FRAME_BUFFER_X_SIZE               1024
#define FRAME_BUFFER_Y_SIZE               1024

/* Test pattern mask:             */
#define TEST_MASK                     0x00000FFF

/* Normal page dimensions:        */
#define PAGE_X_DIM                         640
#define PAGE_Y_DIM                         480

#define NORMAL_PAGE_SIZE   PAGE_X_DIM * PAGE_Y_DIM

/* Zoomed page dimensions:        */
#define PAGE_X_DIMZ                        320
#define PAGE_Y_DIMZ                        240

#define ZOOMED_PAGE_SIZE   PAGE_X_DIMZ * PAGE_Y_DIMZ

/* Multiple pages in the frame buffer:                      */

/* Caution: DO NOT ALTER ANY OF THESE NUMBERS.  OTHER       */
/*          COMBINATIONS DO NOT WORK AT ALL SPEEDS.         */

/* High resolution (640 x 480):                             */
#define NUM_HIGH_RES_PAGES                   2

/* Low resolution (320 x 240, zoom mode):                   */
#define NUM_LOW_RES_PAGES                   12

/* Bias and gain constants:       */
#define LOW_BIAS                           0x00
#define HIGH_BIAS                          0xFF

#define LOW_GAIN                           0x00
#define HIGH_GAIN                          0xFF

#define MID_SCREEN                       123130
#define TEST_LENGTH                          32

#define LOW_THRESHOLD                         0
#define HIGH_THRESHOLD                      250

/* Lookup table constants:        */
#define TOT_LUT_LENGTH                  0x0800
#define LUT_LENGTH                      0x0100
#define LUT_ADDR_MASK                   0x00FF

/* Lookup table types:            */
#define INPUT_LUT                       0x0000
#define RED_LUT                         0x0800
#define GREEN_LUT                       0x1000
#define BLUE_LUT                        0x1800

/* Lookup table maps:             */
#define LUT_MAP_0                       0x00
#define LUT_MAP_1                       0x01
#define LUT_MAP_2                       0x02
#define LUT_MAP_3                       0x03
#define LUT_MAP_4                       0x04
#define LUT_MAP_5                       0x05
#define LUT_MAP_6                       0x06
#define LUT_MAP_7                       0x07

/* Control Register 0 Bits:       */
#define REFR_DISABLE                    0x80
#define ZOOM_ENABLE                     0x40
#define X_QUADRANT                      0x20
#define Y_QUADRANT                      0x10
#define QUAD_BUFFER                     0x08
#define LUT_SEL_MASK                    0x07

/* Control Register 1 Bits:       */
#define EXT_SYNC                        0x80
#define EURO_STD                        0x40
#define CONT_GRAB                       0x20
#define VIDOUT_DIS                      0x10

#define INT_SYNC                        0x00
#define VIDOUT_SRC_MASK                 0x0C

/* Video output sources:          */
#define FINPUT_LUT                      0x00
#define FRAME_BUFFER                    0x04
#define SWITCH_LAFB                     0x0C

#define VIDINP_SRC_MASK                 0x03


/* Video input sources:           */
#define CHANNEL_0                       0x00
#define CHANNEL_1                       0x01
#define CHANNEL_2                       0x02
#define FEED_BACK                       0x03

/* Mask Register Bits:            */
#define MASK_REG_MASK                   0xFF

#define TRANSPARENT                     0x00

/* Bit planes to be masked:       */
#define BIT_PLANE_0                     0x01
#define BIT_PLANE_1                     0x02
#define BIT_PLANE_2                     0x04
#define BIT_PLANE_3                     0x08
#define BIT_PLANE_4                     0x10
#define BIT_PLANE_5                     0x20
#define BIT_PLANE_6                     0x40
#define BIT_PLANE_7                     0x80

/* Status Register Bits:          */
#define ODD_FIELD_MASK                  0x08
#define STRAP_ST_MASK                   0x04
#define FR_GR_ACT_MASK                  0x02
#define SLAVE_MASK                      0x01

/* Snapshot register bits:        */
#define SNAPSHOT                        0x00

/* Video Gain Register Bits:      */
#define VID_GAIN_MASK                   0x0FF
#define DEFAULT_GAIN                    0x080

/* Video Offset Register Bits:    */
#define VID_OFFSET_MASK                 0x0FF
#define DEFAULT_OFFSET                  0x080

/* CRTC Status Register Bits:     */
#define VERT_BLANK_MASK                 0x20

/* CRTC Register Addresses:       */
#define HORIZ_TOTAL                     0x00
#define HORIZ_DISPLAYED                 0x01
#define HORIZ_SYNC_POS                  0x02
#define VERTAHOR_SYNC_WIDTH             0x03
#define VERT_TOTAL                      0x04
#define VERT_TOT_ADJUST                 0x05
#define VERT_DISPLAYED                  0x06
#define VERT_SYNC_POS                   0x07
#define MODE_CONTROL                    0x08
#define SCAN_LINE                       0x09
#define CURSOR_START                    0x0A
#define CURSOR_END                      0x0B
#define DISP_START_ADDRH                0x0C
#define DISP_START_ADDRL                0x0D
#define CURSOR_POSH                     0x0E
#define CURSOR_POSL                     0x0F
#define LIGHT_PEN_REGH                  0x10
#define LIGHT_PEN_REGL                  0x11
#define UPDATE_LOCH                     0x12
#define UPDATE_LOCL                     0x13
#define DUMMY_LOC                       0x1F

/* CRTC Register Values:        */

/* Caution: DO NOT ALTER ANY OF THESE NUMBERS.  OTHER       */
/*          COMBINATIONS DO NOT WORK AT ALL SPEEDS.         */
#define HORIZ_TOT_USA                   0x61
#define HORIZ_TOT_EUR                   0x62
#define HORIZ_TOT_ZUSA                  0x30
#define HORIZ_TOT_ZEUR                  0x31

#define HORIZ_DISP_USA                  0x50
#define HORIZ_DISP_EUR                  0x50
#define HORIZ_DISP_ZUSA                 0x28
#define HORIZ_DISP_ZEUR                 0x28

#define HORIZ_SPOS_USA                  0x53
#define HORIZ_SPOS_EUR                  0x52
#define HORIZ_SPOS_ZUSA                 0x29
#define HORIZ_SPOS_ZEUR                 0x29

#define VAH_SWID_USA                    0x4d
#define VAH_SWID_EUR                    0x8f
#define VAH_SWID_ZUSA                   0x46
#define VAH_SWID_ZEUR                   0x87

#define VERT_TOT_USA                    0x1F
#define VERT_TOT_EUR                    0x25
#define VERT_TOT_ZUSA                   0x0F
#define VERT_TOT_ZEUR                   0x12

#define VERT_TOTA_USA                   0x06
#define VERT_TOTA_EUR                   0x08
#define VERT_TOTA_ZUSA                  0x06
#define VERT_TOTA_ZEUR                  0x08

#define VERT_DISP_USA                   0x1E
#define VERT_DISP_EUR                   0x20
#define VERT_DISP_ZUSA                  0x0F
#define VERT_DISP_ZEUR                  0x10

#define VERT_SPOS_USA                   0x1E
#define VERT_SPOS_EUR                   0x23
#define VERT_SPOS_ZUSA                  0x0F
#define VERT_SPOS_ZEUR                  0x11

#define MODE_CONT_USA                   0x07
#define MODE_CONT_EUR                   0x07
#define MODE_CONT_ZUSA                  0x05
#define MODE_CONT_ZEUR                  0x05

#define SCAN_LINE_USA                   0x0F
#define SCAN_LINE_EUR                   0x0F
#define SCAN_LINE_ZUSA                  0x0F
#define SCAN_LINE_ZEUR                  0x0F

#define DISP_STADH_USA                  0x00
#define DISP_STADH_EUR                  0x00
#define DISP_STADH_ZUSA                 0x00
#define DISP_STADH_ZEUR                 0x00

#define DISP_STADL_USA                  0x00
#define DISP_STADL_EUR                  0x00
#define DISP_STADL_ZUSA                 0x00
#define DISP_STADL_ZEUR                 0x00

#define VIP640A_INT_PRI			28
#define VIP640A_INT_PERIOD		20UL
#define VIP640A_INT_OFFSET		0UL

/* End of constant definitions. */
