/* $RCSfile: Scheduler.h,v $: Scheduler.h,v $ - Scheduler header file */

/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/Scheduler.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

/* Beth McKenney - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,21Oct96,eamck    based on SchedISR.h
*/
      
/*
DESCRIPTION
  ISIT Scheduler definitions
*/


#ifndef INC_schedulerh
#define INC_schedulerh

#ifndef SPECTOOL
 #include <stdio.h>

#include "hwError.h" 
#include "interrupt.h"
#include "clock.h"
#endif


#define MIN_SCHED_TASK_PRIO 1
#define MAX_SCHED_TASK_PRIO 254


/*-- Error code definitions --*/
#define  S_sched_SCHEDULER_NOT_INITIALIZED        (M_sched | 0x01)
#define  S_sched_NULL_TASK_EXEC_FUNCTION_POINTER  (M_sched | 0x02)
#define  S_sched_ILLEGAL_TASK_PERIOD              (M_sched | 0x03)
#define  S_sched_SCHEDULER_ALREADY_INITIALIZED    (M_sched | 0x04)
#define  S_sched_MAX_TASKS_EXCEEDED               (M_sched | 0x05)
#define  S_sched_INVALID_PRIORITY                 (M_sched | 0x06)
#define  S_sched_ILLEGAL_TASK_OFFSET              (M_sched | 0x07)
#define  S_sched_OUT_OF_TCB_POINTERS              (M_sched | 0x08)
#define  S_sched_INVALID_VALUE_FOR_MAXTASKS       (M_sched | 0x09)
#define  S_sched_INSUFFICIENT_MEMORY              (M_sched | 0x0a)
#define  S_sched_NOT_ENABLED                      (M_sched | 0x0b)
#define  S_sched_INVALID_TASK_MODE                (M_sched | 0x0c)
#define  S_sched_INVALID_TASK_ID                  (M_sched | 0x0d)
#define  S_sched_TASK_STARVING                    (M_sched | 0x0e)
#define  S_sched_UNEXPECTED_TASK_MODE             (M_sched | 0x0f)
#define  S_sched_TASK_HAS_PANICKED                (M_sched | 0x10)

#define S_sched_ENTERING_ADD_TASK                 (M_sched | 0x1001)
#define S_sched_PERIOD_AND_OFFSET_SET             (M_sched | 0x1002)
#define S_sched_JUST_ISSUED_TASK_LOCK             (M_sched | 0x1003)
#define S_sched_JUST_INSERTED_NEW_TCB             (M_sched | 0x1004)
#define S_sched_JUST_SPAWNED_NEW_TASKSKEL         (M_sched | 0x1005)
#define S_sched_LEAVING_ADD_TASK                  (M_sched | 0x1006)
#define S_sched_ENTERING_INIT                     (M_sched | 0x1007)
#define S_sched_ADJUSTED_VXWORKS_TASK_PRIOS       (M_sched | 0x1008)
#define S_sched_TASK_SET_INITIALIZED              (M_sched | 0x1009)
#define S_sched_CONNECTED_INTERRUPT_TO_SCHEDISR   (M_sched | 0x100a)
#define S_sched_LEAVING_INIT                      (M_sched | 0x100b)
#define S_sched_IN_TASKSKEL                       (M_sched | 0x100c)
#define S_sched_ENTERING_REMOVETASK               (M_sched | 0x100d)
#define S_sched_TASK_PERIOD_SET                   (M_sched | 0x100e)
#define S_sched_TASK_OFFSET_SET                   (M_sched | 0x100f)
#define S_sched_FOUND_EMPTY_TASK_PTR_TO_USE       (M_sched | 0x1010)
#define S_sched_DISABLED_SCHEDULER_IN_ADDTASK     (M_sched | 0x1011)
#define S_sched_FOUND_WHERE_TO_INSERT_TCB         (M_sched | 0x1012)
#define S_sched_MADE_SPACE_TO_INSERT_TCB          (M_sched | 0x1013)
#define S_sched_ABOUT_TO_DISABLE_SCHEDULER        (M_sched | 0x1014)
#define S_sched_ABOUT_TO_ENABLE_SCHEDULER         (M_sched | 0x1015)
#define S_sched_ABOUT_TO_DISABLE_A_TASK           (M_sched | 0x1016)
#define S_sched_ABOUT_TO_ENABLE_A_TASK            (M_sched | 0x1017)
#define S_sched_ABOUT_TO_DELETE_A_TASK            (M_sched | 0x1018)


typedef enum TASK_ERROR_BEHAVIOR {TEB_DISABLE, TEB_CONTINUE,
				  TEB_DELETE } TASK_ERROR_BEHAVIOR;

typedef enum TASK_PENDING_OPERATION {TPO_NONE, TPO_DISABLE_TASK,
				     TPO_DELETE_TASK}
					TASK_PENDING_OPERATION;

typedef enum TASK_STATE {TS_RUN, TS_READY, TS_DONE} TASK_STATE;

typedef enum TASK_MODE {TM_INIT, TM_ENABLED, TM_DELETE, TM_DISABLED, TM_ERROR, 
			TM_PANIC } TASK_MODE;


typedef enum SCHED_LEVEL { SL_DISABLED, SL_ENABLED, SL_SHUTDOWN } SCHED_LEVEL;


/* class tTask - based on Scheduler Black Box Spec document */

class tScheduler;
class tRte;

class tTask
{
   public:
      int        TaskId () { return this->Id; } 
      ERROR_CODE LastErrCode () { return this->ErrorCode; } 
      ERROR_CODE Initialize ( void );
      ERROR_CODE Initialize ( int aId,
                               BOOL abStarving,
                               BOOL abNoPanic,
                               TASK_ERROR_BEHAVIOR aBehaviorOnError,
                               TASK_PENDING_OPERATION aPendingOp,
                               TASK_STATE aExecState,
                               TASK_MODE aMode,
                               TIME atScheduledTime,
                               FUNCPTR apExecFunc,
                               FUNCPTR apInitFunc,
                               FUNCPTR apDeleteFunc,
                               int aiPriority,
                               TIME atPeriod,
                               TIME atOffset,
                               ERROR_CODE aErrorCode,
                               int aiTaskPosition,
                               tScheduler *aptScheduler);

      friend ERROR_CODE        taskSkel( void *pArgument, tScheduler *pSched,
                                         int iTaskPosition );
   protected:
      friend class tScheduler;

      int			Id;
      BOOL	  	        bStarving;
      int                       iNumStarvingCycles;
      BOOL		        bNoPanic;
      TASK_ERROR_BEHAVIOR	BehaviorOnError;
      TASK_PENDING_OPERATION	PendingOp;
      TASK_STATE	        ExecState;
      TASK_MODE	                Mode;
      TIME		        tScheduledTime;
      FUNCPTR		        pExecFunc;
      FUNCPTR		        pInitFunc;
      FUNCPTR		        pDeleteFunc;
      int			iPriority;
      TIME		        tPeriod;
      TIME		        tOffset;
      ERROR_CODE	        ErrorCode;
      int                       iTaskPosition;   /* index into hTaskSetPtr */

      tScheduler	       *ptScheduler;


};    /* tTask */


/* class tScheduler  - based on Scheduler Black Box Spec document */

/******************************************************************
*
* tScheduleer Black Box
*
* BB State Variables
* ------------------
*

[[D tScheduler::tScheduler (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->bInit == FALSE
]]

[[D tScheduler::AddTask (int *piTaskId, char *sName, FUNCPTR  pExec,
                         FUNCPTR pInit, FUNCPTR  pDelete,
                         void *pArgument, int iPriority,
                         TIME *ptPeriod, TIME *ptOffset, 
                         TASK_ERROR_BEHAVIOR tebErrorBehavior,
                         BOOL bNopanic) ]

[ SchedulerOkay( this ) == S_UNDEFINED => UNDEFINED
| SchedulerOkay( this ) != OK => return appropriate error msg
| true => 
    this->TaskSet := this->TaskSet U 
         PeriodicTask (piTaskId, sName, pExec, pInit, pDelete,
                       pArgument,iPriority, *ptPeriod, *ptOffset, 
                       tebErrorBehavior, bNopanic)
    : all due tasks in the Scheduler's task set are resumed
    : return OK
]]



[[D  tScheduler::Initialize (int iPriority, int iMaxTasks, INT_RATE iRate,
                             TIME *pRteTime, int iWhichRte,
                             tPeriodicTimeInterrupt *ptInterruptObject) ]
[ invalid_pointer (this) ||
  invalid_pointer (ptInterruptObject) ||
  invalid_pointer (ptInterruptObject->pBoard) is invalid ||
  invalid_pointer (ptInterruptObject->pBoard->pFunc) => UNDEFINED
| this->bInit == TRUE ||
  iMaxTasks < 1 ||
  unable to allocate memory for array of tTask of dimension iMaxTasks || 
  unable to allocate memory for array of (tTask *) of dim. iMaxTasks || 
  ptInterruptObject->bInit == FALSE ||
  ptInterruptObject->pBoard->bInit == FALSE ||
  ptInterruptObject->iWhich is not valid interrupt source on 
    *ptInterruptObject->pBoard || 
  ptInterruptObject->iWhich is not clock-driven interrupt source for 
    *ptInterruptObject->pBoard ||
  ptInterruptObject->bConnected == TRUE ||
  ptInterruptObject->bEnabled == TRUE ||
  &this->SchedIsr == NULL => return := appropriate error code
| task "tTelnetd" not found || 
  task "tRlogind" not found || 
  task "tShell" not found => 
          put VxWorks errno (taskPriorityGet or Set) on RTE error stream
| true =>
    this->schedTelnetdPriority := tTelnetd's priority
    : *this.schedRlogindPriority := tRlogind's priority
    : *this.schedShellPriority := tShell's priority
    : priority of tTelnetd task := SCHED_TELNETD_PRIORITY
    : priority of tRlogind task := SCHED_RLOGIND_PRIORITY
    : priority of tShell task := SCHED_SHELL_PRIORITY
    : *this.hTaskSetPtr[0..*this.iMaxTasks-1] := NULL
    : this->TaskSet[0..iMaxTasks-1].Id := ERROR
    : this->TaskSet[0..iMaxTasks-1].bStarving := FALSE
    : this->TaskSet[0..iMaxTasks-1].bNoPanic := FALSE
    : this->TaskSet[0..iMaxTasks-1].BehaviorOnError := TEB_DISABLE
    : this->TaskSet[0..iMaxTasks-1].PendingOp := TPO_NONE
    : this->TaskSet[0..iMaxTasks-1].Exec_State := TS_DONE
    : this->TaskSet[0..iMaxTasks-1].Mode := TM_INIT
    : this->TaskSet[0..iMaxTasks-1].tScheduledTime := 0
    : this->TaskSet[0..iMaxTasks-1].pExecFunc := NULL
    : this->TaskSet[0..iMaxTasks-1].pInitFunc := NULL
    : this->TaskSet[0..iMaxTasks-1].pDeleteFunc := NULL
    : this->TaskSet[0..iMaxTasks-1].iPriority := 255
    : this->TaskSet[0..iMaxTasks-1].tPeriod := 0
    : this->TaskSet[0..iMaxTasks-1].tOffset := 0
    : this->TaskSet[0..iMaxTasks-1].ErrorCode := OK
    : this->TaskSet[0..iMaxTasks-1].iTaskPosition := -1
    : this->TaskSet[0..iMaxTasks-1].ptScheduler := NULL
    : this->bInit := TRUE
    : this->Level := SL_ENABLED
    : this->iRate := iRate
    : this->pRteError := pRteErrorStream
    : this->pRteTime := pRteTime
    : this->ptInterrupt := ptInterruptObject
    : this->iMaxTasks := iMaxTasks
    : this->iNumTasks := 0 
    : this->pEndTaskSet := this->TaskSet
    : [ ptInterruptObject->uiIntNum < MIN_BUS_INTERRUPT ||
        ptInterruptObject->uiIntNum > MAX_BUS_INTERRUPT =>
         return := error for *ptInterruptObject.*pBoard
      | true => 
          enable board interrupt source ptInterruptObject->iWhich
          : connect board interrupt source ptInterruptObject->iWhich to
            VME interrupt ptInterruptObject->uiIntNum at 
            ptInterruptObject->uiIntVector
      ]
    : iRate := nearest value to iRate that is valid for 
      *ptInterruptObject->pBoard
    : ptInterruptObject->pBoard_.pRate [ptInterruptObject->iWhich] := 
      nearest value to iRate that is valid for *ptInterruptObject->pBoard
    : ptInterruptObject->bRateSetYet := TRUE
    : ptInterruptObject->bConnected := TRUE
    : ptInterruptObject->bEnabled := TRUE
    : ptInterruptObject->pBoard interrupt clock is enabled
    : ptInterruptObject->pBoard->pFunc starts getting called at 
      ptInterruptObject->rate
    : return := OK
]]

[[D tScheduler::DeleteTask (int iTaskId) ]
[ invalid_pointer (this) ||
  Number of elements in this->TaskSet != this->iNumTasks =>  UNDEFINED
| this->bInit != TRUE ||
  Task (iTaskId) in not element of this->TaskSet || 
  iTaskId not a valid VxWorks task ID ||
  Task (iTaskId) cannot be resumed =>  return := appropriate error code
| Task (iTaskId).Mode == TM_ENABLED && 
  Task (iTaskId).ExecState == TS_DONE =>
    Task (iTaskId).Mode := TM_DELETE
    : return :=  OK
| Task (iTaskId).Mode == TM_ENABLED && 
  Task (iTaskId).ExecState != TS_DONE =>
    Task (iTaskId).PendingOp := TPO_DELETE_TASK
    : return := OK
| Task (iTaskId).Mode == TM_INIT => 
    Task (iTaskId).PendingOp := TPO_DELETE_TASK
    : return := OK
| true => return := OK
]]

[[D tScheduler::DisableTask (int iTaskId) ]
[ invalid_pointer (this) ||
  Number of elements in this->TaskSet != this->iNumTasks =>  UNDEFINED
| this->bInit != TRUE ||
  Task (iTaskId) not element of this->TaskSet => 
    return := appropriate error code
| Task (iTaskId).Mode == TM_ENABLE && 
  Task(iTaskId).ExecState == TS_DONE =>
    Task(iTaskId).Mode := TM_DISABLED
    : return := OK
|  Task (iTaskId).Mode == TM_ENABLE && 
   Task (iTaskId).ExecState != TS_DONE =>
   Task (iTaskId).PendingOp := TPO_DISABLE_TASK
   : return := OK
| Task (iTaskId).Mode == TM_INIT => 
    Task (iTaskId).PendingOp := TPO_DISABLE_TASK
    : return := OK
| true => 
    this->Level := SL_ENABLED 
    : For all tasks i, s.t. Task (i).Mode == TM_ENABLED &&
      Task (i).ExecState == TS_READY || Task (i).ExecState == TS_RUN
      Task (i).ExecState := TS_RUN
    : For all tasks i, s.t. Task (i).Mode == TM_ENAMBELD &&
      Task (i).ExecState == TS_READY || Task (i).ExecState == TS_RUN 
      resume Task (i)  
    : return :=  OK
]]


[[D tScheduler::EnableTask (int iTaskId) ]
[ invalid_pointer (this) ||
  Number of elements in this->TaskSet != this->iNumTasks =>  UNDEFINED
| this->bInit != TRUE || 
  Task (iTaskId) not element of this->TaskSet => 
    return := appropriate error code
| Task (iTaskId).Mode == TM_DISABLED ||
  Task (iTaskId).Mode == TM_TM_PANIC ||
  Task (iTaskId).Mode == TM_ERROR =>
    Task (iTaskId).Mode := TM_ENABLED
    : Task (iTaskId).ExecState := TS_DONE
    : return := OK
| Task (iTaskId).Mode == TM_INIT ||
  Task (iTaskId).Mode == TM_ENABLED ||
  Task (iTaskId).Mode == TM_DISABLED &&
    Task (iTaskId).PendingOp == TP0_DISABLE_TASK => 
      Task (iTaskId).PendingOp := TPO_NONE
      : return := OK
| true => return := OK
]]


*****************************************************************************/
 

                                  
class tScheduler
{
   public:
      tScheduler(void) { bInit = FALSE; }
      ERROR_CODE  Initialize( int iPriority, 
			      int iMaxTasks, 
			      INT_RATE iRate, 
			      TIME *pRteTime,
			      tPeriodicTimeInterrupt *ptInterruptObject);
      ERROR_CODE  Enable( void );
      ERROR_CODE  Disable( void );
      ERROR_CODE  Shutdown( void ) { printf("Shutdown not implemented\n");
                                     return S_general_NOT_IMPLEMENTED; }
      ERROR_CODE  Show( void );
      ERROR_CODE  ShowAll( void );

      ERROR_CODE  AddTask( int                 *piTaskId, 
                           char                *sName, 
                           FUNCPTR             	pExec, 
                           FUNCPTR	     	pInit,
                           FUNCPTR 	     	pDelete, 
                           void 	       *pArgument, 
                           int 		       	iPriority, 
                           TIME		       	*ptPeriod,
                           TIME		       	*ptOffset, 
                           TASK_ERROR_BEHAVIOR  tebErrorBehavior, 
                           BOOL 	     	bNoPanic );

      ERROR_CODE  DeleteTask(int iTid);
      ERROR_CODE  DisableTask(int iTid);
      ERROR_CODE  EnableTask(int iTid);
      ERROR_CODE  RemoveTask(int iTid);

      tTask      *GetTaskPtr(int iTaskId);   /* get tTask ptr from task ID */
      tTask      *GetTaskPtrFromIndex(int iIndex);
                        /* get tTask ptr from index into hTaskSetPtr array */

      ERROR_CODE  Suspender(tTask *ptTask, ERROR_CODE ecTaskErr);

   protected:
      TIME        tSchedPeriod;
      ERROR_CODE  StartNextTask( void );	
      void	  SchedIsr( void );       /* called at each interrupt 
				           * from *ptInterrupt 
				           */
      BOOL	  bInit;
      SCHED_LEVEL Level;
      INT_RATE    iRate;
      int         iCountInterrupts;         /* for diagnostics */

      tPeriodicTimeInterrupt  *ptInterrupt;
      TIME        *pRteTime;                /* updated by schedIsr() */

      tTask       *TaskSet;                 /* array of tasks */
      tTask       **hTaskSetPtr;            /* array of tcb pointers */
      tTask       *pEndTaskSet;             /* == TaskSet + iNumTasks */

      int         iMaxTasks;
      int         iNumTasks; 

      int         schedTelnetdPriority;     /* save these to reset later? */
      int         schedRlogindPriority;
      int         schedShellPriority;

};	/* tScheduler */

extern tRte *pThisRteG;            /* global pointer to this RTE */

extern "C"
{
   ERROR_CODE schedShow( void );      /* calls pThisRteG->Scheduler.Show() */
   ERROR_CODE schedShowAll( void );   /* calls pThisRteG->Scheduler.ShowAll() */
   ERROR_CODE schedEnable( void );    /* calls pThisRteG->Scheduler.Enable() */
   ERROR_CODE schedDisable( void );   /* calls pThisRteG->Scheduler.Disable() */
   ERROR_CODE taskEnable( int iTaskId );    /* calls Scheduler.EnableTask() */
   ERROR_CODE taskDisable( int iTaskId );   /* calls Scheduler.DisableTask() */
}


/******************************************************************************
*
* Spec functions
 
SchedulerOkay(tScheduler *ptSched) :=
 
   [ this invalid -> return S_UNDEFINED
   | *this.iNumTasks <= 0 -> return S_UNDEFINED
   | *this.iNumTasks > *this.iMaxTasks -> return S_UNDEFINED
   | *this.iRate != scheduler interrupt rate in Hz -> return S_UNDEFINED
   | *this.hTaskSetPtr not pointer to an array of
             (tTask *) of dimension >= *this.iMaxTasks -> return S_UNDEFINED
   | *this.TaskSet is not a pointer to an array of tTasks of dimension >=
              (*this.pEndTaskSet - *this.TaskSet) / sizeof(tTask) -> return S_UNDEFINED
   | pExec, pDelete, or pInit not NULL and not a valid C/C++ fn -> UNDEFINED
   | VxWorks priority of calling task numerically <= MAX_SCHED_TASK_PRIO -> UNDEFINED

   | *this.bInit !=TRUE -> return S_sched_SCHEDULER_NOT_INITIALIZED
   | *this.iNumTasks >= *this.iMaxTasks -> return S_sched_MAX_TASKS_EXCEEDED
   | pExec == NULL -> return S_sched_NULL_TASK_EXEC_FUNCTION_POINTER
   | (iPriority < MIN_SCHED_TASK_PRIO) || (iPriority > MAX_SCHED_TASK_PRIO) -> 
          return S_sched_INVALID_PRIORITY
   | *ptPeriod <= 0 -> return S_sched_ILLEGAL_TASK_PERIOD
   | new task cannot be spawned -> return VxWorks errno (taskSpawn)
   | tOffset < 0 or tOffset >= *ptPeriod -> return S_sched_ILLEGAL_TASK_OFFSET
   | there does not exist an i in (0..*this.iMaxTasks-1) such that
             *this.hTaskSetPtr[i] == NULL -> return S_sched_OUT_OF_TCB_POINTERS
   | there exists a task in the TaskSet array whose .Mode is TM_ENABLED and
              whose .ExecState is TS_READY or TS_RUN that cannot be resumed ->
          return VxWorks errno (taskResume)
 
   | true -> return TRUE
   ]
 


PeriodicTask( Id, bStarving, bNoPanic, BehaviorOnError, PendingOp, ExecState, 
	      Mode, tScheduledTime, pExecFunc, pInitFunc, pDeleteFunc, iPriority, 
              tPeriod, tOffset, ErrorCode, iTaskPosition, ptScheduler ) :==

   [ a tTask object with the given parameters 
   ]

*/
 

/*******
[[D Scheduler::Suspend1 (in:ecTaskErr, *this;
                        out: *pThisTcb.ErrorCode,
                             *pThisTcb.ExecState,
                             RTE ErrorStream;) ]
[true =>
   *pThisTcb.ErrorCode := ecTaskErr
   : *pThisTcb.ExecState := TS_DONE
   : [[U Scheduler::StartNextTask()]]
   : RTE error stream := RTE error stream with error codes resulting
                from task activities to RTE error stream as appropriate
]]
****/

/******
[[D Scheduler::NewPendingOp (in: *ThisTcb.PendingOp;
                             out: *pThisTcb.PendingOp, VxWorks;)]
[true =>
    [ *pThisTcb.PendingOp == TPO_DISABLE_TASK =>
       *pThisTcb.PendingOp := TPO_NONE
    ]
    : [ *pThisTcb.PendingOp == TPO_DELETE_TASK =>
       *pThisTcb.PendingOp := TPO_NONE
       : VxWorks task *pThisTcb.Id priority := SCHED_INIT_PRIORITY
      ]
    : [ *pThisTcb.PendingOp == TPO_NONE =>
       do nothing
      ]
]]
******/

/******
[[D Scheduler::InitMode (in: *pThisTcb.Mode;
                         out: *pThisTcb.Mode, VxWorks, log;) ]
[ true =>
    [ *pThisTcb.Mode == TM_INIT =>
       *pThisTcb.Mode := TM_ENABLED
       : VxWorks task *pThisTcb.Id priority := SCHED_INIT_PRIORITY
       : log "changing priority" msg
    | true => do nothing
    ]
]]
*******/

/******
[[D Scheduler::SetPendingOpMode (in: pThisTcb.PendingOp;) ]
[ true =>
    [ *pThisTcb.PendingOp == TPO_DISABLE_TASK =>
       *pThisTcb.Mode := TM_DISABLED
       : *pThisTcb.PendingOp := TPO_NONE
       : delete this task from VxWorks
    ]
    : [ *pThisTcb.PendingOp == TPO_DELETE_TASK =>
       *pThisTcbOp.Mode := TM_DELETE
       : *pThisTcb.PendingOp := TPO_NONE
       : VxWorks task *pThisTcb.Id priority := SCHED_INIT_PRIORITY
      ]
    : [ *pThisTcb.PendingOp == TPO_NONE =>
       [[U Scheduler::InitMode() ]]
       : delete this task from VxWorks
      ]
]]
*******/


#endif       INC_schedulerh
