/* $RCSfile: board.h,v $ - board base class */

/* $Header: /home/chara/cvsroot/ople/sys/include/board.h,v 1.2 1998/07/13 22:07:13 phil Exp $ */

/* Description:
 * ISIT Board Class Definition
 */

/* $Log: board.h,v $
 * Revision 1.2  1998/07/13 22:07:13  phil
 * Added functionality for SetTime command
 *
 * Revision 1.1.1.1  1998/06/11 21:36:10  chara
 * First import of RICST Code
 *
 * Revision 1.33.4.3  1998/05/12 22:59:07  phil
 * changes before build_5b_001
 *
 * Revision 1.33.4.2  1998/05/12 20:26:19  phil
 * Interrupt and tTimBoard changes for I5b
 *
 * Revision 1.33.4.1  1998/05/08 01:20:14  phil
 * interrupt functionality for I5b
 * */

#ifndef INC_boardh
#define INC_boardh

#include "baseObject.h"
#include "hwTypes.h"
//#include "hardware.h"
//#include "interrupt.h"

class tChannel;
class tInterrupt;

/***************************************************************
*
* class tBoard - generic hardware Board class
*
* METHODS:
* tBoard::tBoard(void)
* Description: Constructor for tBoard.  Gets default vme address for the
*              board through the virtual GetDefaultVmeAddress().
[[D tBoard::tBoard(inout: this;)]
[ this is invalid => UNDEFINED
| true =>
  this->vmeAddress := this->GetDefaultVmeAddress()
  : this->boardName := this->GetDefaultBoardName()
  : tBaseObject::bInit := FALSE
]]
*
*
* ERROR_CODE tBoard::Initialize(tVmeAddress *pvmeAddressToUse,
*                              BOARD_NAME boardNameToUse)
* Description: Initializes the board with the vme address and board name,
*              overriding values set at construction.
[[D tBoard::InitializeParams(in: pvmeAddressToUse, boardNameToUse;
                          inout: this;)]
[this is invalid || pvmeAddressToUse is invalid  => UNDEFINED
| true =>
  this->vmeAddress := *pvmeAddressToUse
  : this->vu8LocalAddress := pvmeAddressToUse->GetLocalAddress()
  : this->boardName := boardNameToUse
  : tBaseObject::bInit := TRUE
  : return := OK
]]
*
*
* ERROR_CODE tBoard::Initialize(void)
* Description: Initializes board with defaults set at construction.
[[D tBoard::InitializeVoid(void)]
[ this is invalid => UNDEFINED
| true =>
  this->vu8LocalAddress := this->pvmeAddress->GetLocalAddress()
  : tBaseObject::bInit := TRUE
  : return := OK
]]
*
*
* uint8_t * tBoard::GetLocalAddress(void)
* Description: Returns local address of board if board has been
*              initialized, otherwise NULL.
[[D tBoard::GetLocalAddress(void)]
[ this is invalid => UNDEFINED
| tBaseObject::bInit == FALSE =>
  return NULL
| true =>
  return this->vu8LocalAddress
]]
*
*
* uint8_t * tBoard::GetSize(void)
* Description: Returns size of this board in bytes
[[D tBoard::GetSize(void)]
[ this is invalid => UNDEFINED
| true =>
  return this->uiSize
]]
*
*
* BOOL tBoard::HasInterrupt( void )
[[D tBoard::HasInterrupt(void)]
[ this is invalid ||
  tBaseObject::bInit == FALSE => UNDEFINED
| this->bfFunctions & BF_INTERRUPT == 0 =>
    return := FALSE
| true =>
    return := TRUE
]]
* 
* BOOL tBoard::HasClock( void )
[[D tBoard::HasClock(void)]
[ this is invalid ||
  tBaseOject::bInit == FALSE => UNDEFINED
| *this.bfFunctions & BF_CLOCK == 0 =>
    return := FALSE
| true =>
    return := TRUE
]]
*
******/
class tBoard : public tBaseObject {
  public:
    tBoard();
    ERROR_CODE         Initialize(void);
    volatile uint8_t   *GetLocalAddress(void);
    unsigned int       GetSize(void)  { return uiSize; }
    int                GetNumChannels(void)  { return iNumActiveChannels; }
    int                GetNumInterrupts(void)  { return iNumActiveInterrupts; }
    BOOL               IsInitialized(void);
    BOOL               HasInterrupt(void);
    BOOL               HasClock(void);

    virtual ERROR_CODE HardInit(void) = 0;
    virtual ERROR_CODE SoftInit(void) = 0;
    virtual tChannel   *GetChannel(int iChannelNum);
    virtual tChannel   *CreateChannel(int iChannelNum);
    //virtual tInterrupt *GetInterrupt(int iIntNum);
    virtual tInterrupt *CreateInterrupt(int iIntNum);

    BOARD_NAME boardName;

  protected:
    tVmeAddress      vmeAddress;
    volatile uint8_t *vu8LocalAddress;
    unsigned int     uiSize;
    BOARD_FUNC       bfFunctions;
    int              iNumActiveChannels;
    int              iNumActiveInterrupts;
};


class tClockBoard : public tBoard {
  public:
    // Methods for tBaseObject
    tClockBoard(void);

    // Other Methods
    virtual ERROR_CODE SetTime(int iSeconds)=0;
};


#endif  /* INC_boardh */
