/* $RCSfile: buffer.h,v $ - Header file for class tBuffer */

/*
$Header: /home/chara/cvsroot/ople/sys/include/buffer.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

#ifndef INC_bufferh
#define INC_bufferh

#include <stdlib.h>
#include "isit.h"

/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
*
* Buffer Header file for RICST.  This file define the following classes:
*
* tBuffer
*
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

/*****************************************************************************************

class tBuffer

DESCRIPTION:
   tBuffer is base class for any object that uses pointer to point to buffers of 
variable sizes that need to get accross IPC.
  
METHODS:
  tBuffer::tBuffer ()
      Compiler Default Constructor
      
[[D tBuffer::tBuffer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

  size_t tBuffer::GetSize () const {return szBufferSize;}
     This method is for getting the current size of the buffer needed to hold the
     object data.

[[D tBuffer::GetSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->szBufferSize
]]

  ERROR_CODE tBuffer::Put (size_t szSize, const BYTE *pInBuffer);
     This method is to take a buffer of data and load it into the class.  It is 
     assumed that the buffer being sent in via the pointer is of correct size.
     szSize should contain the size of the incoming buffer which will be compared to 
     the size of the buffer that is expected.  The data will be copied and changes 
     to the data after return will not effect on the copied data.

[[D tBuffer::Put (size_t szSize, const BYTE *pInBuffer) ]
[ invalid_pointer (this) ||
  invalid_pointer (pInBuffer) ||
  pInBuffer is incorrect size ||
  wild_pointer (this-> data pointer(s) ) => UNDEFINED
| (szSize != szBufferSize) => return := ERROR
| true => 
    data is correctly copied from buffer to internal structure
    : return := OK
]]

  ERROR_CODE tBuffer::Get (size_t szSize, BYTE *pOutBuffer);
     This method is to take an already allocated buffer of memory and fill it with 
     the class data.  This function assumes that the pointer is valid a points to 
     a large enough section of memeory for the data.  szSize should contain the 
     size of the memory allocated to the pointer for comparison with the size of 
     the data.

[[D tBuffer::Get (size_t szSize, const BYTE *pOutBuffer) ]
[ invalid_pointer (this) ||
  invalid_pointer (pOutBuffer) ||
  pOutBuffer is incorrect size ||
  wild_pointer (this-> data pointer(s) ) => UNDEFINED
| (szSize != szBufferSize) ||
  ( (this-> data pointer(s) ) == NULL) => return := ERROR
| true => 
    data is correctly copied from internal structure to pOutBuffer
    : return := OK
]]

****************************************************************************************/
class tBuffer {
public:
  size_t GetSize () const {return szBufferSize;}
  virtual ERROR_CODE Put (size_t szSize, const BYTE *pInBuffer) = 0;
  virtual ERROR_CODE Get (size_t szSize, BYTE *pOutBuffer) const = 0;
  
protected:
  size_t szBufferSize;
  };
  
#endif
