/*$RCSfile: channel.h,v $*/
/*

$Header: /home/chara/cvsroot/ople/sys/include/channel.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

/*
modification history
--------------------
01a,28Mar97,pci           written
*/

#ifndef INC_channelh
#define INC_channelh

#include <math.h>
#include "hwTypes.h"
#include "hwError.h"
#include "baseObject.h"


/****************************************************************************
*
* tChannel
* --------
* Black Box State Variables
*   tBaseObject::bInit         : BOOL
*   tBaseObject::ecInitError   : ERROR_CODE
*
*****************************************************************************
*
* Methods
*
[[D tChannel::IsInitialized(void) ]
[ this is invalid => UNDEFINED
| true =>
    return := this->tBaseObject::bInit
]]
*
*****************************************************************************
*
* Description:    Returns bit width of this channel
[[D tChannel::GetBitWidth(void) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE => 
    return := anything
| true =>
    return := this->iBitWidth
]]
*
*****************************************************************************
*
* Description: Returns value (in native channel units) per digital count (LSB)
[[D tChannel::GetChannelUnitsPerCount(void) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
    return := anything
| true =>
    return := conversion value from counts to channel units
]]
*
*****************************************************************************
*
* Description: Returns maximum value that will be accepted by this channel
[[D tChannel::GetChannelMaxValue(void)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
    return := anything
| true =>
    return := maximum value the channel is able to output
]]
***/
class tChannel : public tBaseObject {
public:
  int    GetBitWidth(void)    { return iBitWidth; }
  double GetChannelUnitsPerCount(void) const
               { return (dChannelRange / pow(2.0,(double)iBitWidth) ); }
  double GetChannelMaxValue(void) const
               { return ( ccConvCode==CC_TWOS_COMPLEMENT ? 
                          pow(2.0,(double)(iBitWidth-1)) - 1 :
                          pow(2.0,(double)iBitWidth) - 1 ); }
  CONV_CODE GetConvCode(void) const { return ccConvCode; }
  virtual ERROR_CODE Reset(void) { return S_channel_INVALID_CALL_TO_RESET; }

protected:
  int       iBitWidth;
  double    dChannelRange;
  CONV_CODE ccConvCode;
};

#endif /* INC_channelh */
