/*$RCSfile: channelBank.h,v $ - tChannelBank header file */

/*
$Header: /home/chara/cvsroot/ople/sys/include/channelBank.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

/*
modification history
--------------------
01a,7Nov97,pci           written, moved from channel.h
*/

#ifndef INC_channelBankh
#define INC_channelBankh

#include "channel.h"
#include <math.h>


/*******************************************************************************
*
* class tChannelBank
*
* BOOL tChannelBank::IsInitialized(void)
* Description:   Returns TRUE if the channel has been initialized,
*                FALSE otherwise
[[D tChannelBank::IsInitialized(void) ]
[ this is invalid => UNDEFINED
| true =>
  return := this->bInit
]]
******/
class tChannelBank : public tBaseObject {
};


/*******************************************************************************
*
* class tAdChannelBank
*
* StateVariables:
*  @ BOOL bInit
*  int  iStartChannel
*  int  iStopChannel
*  BOOL bScanning
*
* tAdChannelBank::tAdChannelBank(void)
* Description:    Constructor.  Sets bScanning to FALSE and iStartChannel and
*                 iStopChannel to 0.
[[D tAdChannelBank::tAdChannelBank(inout: this;) ]
[ Insufficient memory available for construction =>
  this is invalid
| true =>
  this is valid
  : this->bScanning := FALSE
  : this->iStartChannel := 0
  : this->iStopChannel := 0
]]
*
*
* pure virtual ERROR_CODE tAdChannelBank::StartScan(void)
* Description:    Starts converting channels from iStartChannel to iStopChannel,
*                 continuously, until a StopScan command is issued.
[[D tAdChannelBank::StartScan (inout: this;) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tAdChannelBank_BANK_NOT_INITIALIZED
| this->bScanning == TRUE =>
  return := OK
| true =>
  Continually update board registers associated with this->iStartChannel
     to this->iStopChannel
  : this->bScanning := TRUE
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tAdChannelBank::StopScan(void)
* Description:    Stops converting channels.  Subsequent Inputs will
*                 yield old data.
[[D tAdChannelBank::StopScan (void) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tAdChannelBank_BANK_NOT_INITIALIZED
| board does not have a stop capability =>
  return := appropriate error
| this->bScanning == FALSE =>
  return := OK
| true =>
  Stop updating board input registers
  : bScanning := FALSE
  : return := OK
]]
*
*
* ERROR_CODE tAdChannelBank::SetStartChannel(int iNewStartChannel)
* Description:    Sets the start channel number for this ChannelBank.  If
*                 this call is performed while bScanning is TRUE, the change
*                 will only take effect after StartScan is called again.
[[D tAdChannelBank::SetStartChannel(int iNewStartChannel) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tAdChannelBank_BANK_NOT_INITIALIZED
| true =>
  this->iStartChannel := iNewStartChannel
  : return := OK
]]
*
*
* ERROR_CODE tAdChannelBank::SetStopChannel(iNewStopChannel)
* Description:    Sets the stop channel number for this ChannelBank.  If
*                 this call is performed whild bScanning is TRUE, the change
*                 will only take effect after StartScan is called again.
[[D tAdChannelBank::SetStopChannel(int iNewStopChannel) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tAdChannelBank_BANK_NOT_INITIALIZED
| true =>
  this->iStopChannel := iNewStopChannel
  : return := OK
]]
*
*
******/
class tAdChannelBank : public tChannelBank {
public:
  virtual ERROR_CODE StartScan(void) = 0;
  virtual ERROR_CODE StopScan(void) = 0;
//  ERROR_CODE SetStartChannel(int iNewStartChannel);
//  ERROR_CODE SetStopChannel(int iNewStopChannel);

protected:
  int  iStartChannel;
  BOOL bScanning;
};


/*******************************************************************************
*
* class tWaveformChannelBank
*
* State Variables:
*  @ BOOL bInit
*  BOOL   bOutputOn
*  int    iTotalMemoryLoc
*
* pure virtual ERROR_CODE tWaveformChannelBank::StartOutput(void)
* Description:    Starts output on all active channels.
[[D tWaveformChannelBank::StartOutput(void) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tWaveformChannelBank_BANK_NOT_INITIALIZED
| this->bOutputOn == TRUE =>
  return := OK
| true =>
  Start updating board output registers[0..this->iNumActiveChannels-1]
   at a frequency of this->dSampleFreq
  : this->bOutputOn := TRUE
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tWaveformChannelBank::StopOutput(void)
* Description:    Stops output on all active channels.
[[D tWaveformChannelBank::StopOutput( void ) ]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tWaveformChannelBank_BANK_NOT_INITIALIZED
| this->bOutputOn == FALSE =>
  return := OK
| true =>
  Stop updating all board output registers
  : this->bOutputOn := FALSE
  : return := OK
]]
*
*
* int tWaveformChannelBank::GetTotalMemory(void)
* Descriptions:    Returns the total number of locations available on the board.
[[D tWaveformChannelBank::GetTotalMemory(void) ]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  return := this->iTotalMemoryLoc
]]
*
*
* pure virtual int tWaveformChannelBank::GetNumChannels(void)
* Description:    Returns the total number of active channels on the board.
[[D tWaveformChannelBank::GetNumChannels( void ) ]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  return := number of active channels on the board
]]
*
*
* pure virtual ERROR_CODE tWaveformChannelBank::SetWaveformSize(int iSize)
* Description: Sets the size of the waveform (in number of samples)
[[D tWaveformChannelBank::SetWaveformSize(int iSize)]
[ this is invalid => UNDEFINED
| iSize < 0 || (iSize > this->iTotalMemoryLoc) =>
  return := appropriate error
| true =>
  appropriate size variable := iSize
]]
*
*
* pure virtual int tWaveformChannelBank::GetWaveformSize(void)
* Description: Gets the size of the waveform (in number of samples)
[[D tWaveformChannelBank::GetWaveformSize( void )]
[ this is invalid => UNDEFINED
| true =>
  return := appropriate waveform size value
]]
*
*
* pure virtual double tWaveformChannelBank::GetSampleFreq(void)
* Description: Gets the sample frequency (in kHz), that is the actual frequency
               that each channel will "see" at the output
[[D tWaveformChannelBank::GetSampleFreq( void )]
[ this is invalid => UNDEFINED
| true =>
  return := (1/(0.000001 * sample period of each channel))
]]
*
*
* pure virtual int tWaveformChannelBank::GetSamplePeriod(void)
* Description: Gets the sample period (in ns), that each channel will "see"
*              at the output.
[[D tWaveformChannelBank::GetSamplePeriod( void )]
[ this is invalid => UNDEFINED
| true =>
  return := sample period of each channel
]]
*
*
* pure virtual double tWaveformChannelBank::GetWaveformFreq(void)
* Description: Gest the frequency (in kHz) that the board will exhibit if in
*              "continuous" mode.
[[D tWaveformChannelBank::GetWaveformFreq( void )]
[ this is invalid => UNDEFINED
| true =>
  return := (1/(0.000001 * period of the entire waveform))
]]
*
*
* pure virtual int tWaveformChannelBank::GetWaveformPeriod(void)
* Description: Gets the period (in ns) of the entire waveform
[[D tWaveformChannelBank::GetWaveformPeriod( void )]
[ this is invalid => UNDEFINED
| true =>
  return := period of entire waveform.
]]
*
******/
class tWaveformChannelBank : public tChannelBank {
public:
  virtual ERROR_CODE StartOutput(void) = 0;
  virtual ERROR_CODE StopOutput(void) = 0;
  int  GetTotalMemory(void) { return this->iTotalMemoryLoc; }
  virtual int        GetNumChannels(void) = 0;
  virtual ERROR_CODE SetWaveformSize(int iSize) = 0;
  virtual int        GetWaveformSize(void) = 0;
  virtual double     GetSampleFreq(void) = 0;
  virtual int        GetSamplePeriod(void) = 0;
  virtual double     GetWaveformFreq(void) = 0;
  virtual int        GetWaveformPeriod(void) = 0;

protected:
  BOOL   bOutputOn;
  int    iTotalMemoryLoc;
};

#endif /* INC_channelBankh */

