/* $RCSfile: clock.h,v $ - Clock Class Definition */
/*
   $Header: /home/chara/cvsroot/ople/sys/include/clock.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

/*
modification history
--------------------
01a,pci,10Oct96    written
*/

/*
DESCRIPTION
  Clock Class Definition, used for getting current time and
    time since last interrupt.
*/


#ifndef INC_clockh
#define INC_clockh

#include "board.h"

/*************************
[[D tClock::tClock (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->bInit := FALSE
]]

[[D tClock::Initialize (tBoard *pBoardObject) ]
[ invalid_pointer (this) ||
  invalid_pointer (pBoard) => UNDEFINED
| this->bInit == TRUE ||
  pBoard->bInit == FALSE ||
  *pBoard does not have clock function =>
    return := appropriate error code
| true => 
    this->pBoard := pBoard
    : this->bInit := TRUE
    : return OK
]]

[[D tClock::GetLastIntTime (void) ]
[ invalid_pointer (this) ||
  this->bInit == FALSE ||
  this->pBoard->bInit == FALSE ||
  this->*pBoard does not have clock function ||
  manual capture (GetCurrentTime) is performed on *this->pBoard during
     this function => UNDEFINED
| true => return := time since last interrupt in 25us since midnight
]]

NOT IMPLEMENTED IN INC 1
TIME tClock::GetCurrentTime ( this )
[ this is invalid -> UNDEFINED
| this->bInit == FALSE -> UNDEFINED
| this->pBoard->bInit == FALSE -> UNDEFINED
| this->*pBoard does not have clock function -> UNDEFINED
| true -> return current time in 25us of usec since midnight
]

NOT IMPLEMENTED IN INC 1
BOOL tClock::IsHealthy ( this )
[ this is invalid -> UNDEFINED
| this->bInit == FALSE -> UNDEFINED
| this->pBoard->bInit == FALSE -> UNDEFINED
| this->*pBoard does not have clock function -> UNDEFINED
| this->*pBoard time status flags are all in "normal" state ->
  return TRUE
| true -> return FALSE
]
*/

class tClock : public tBaseObject {
  public:
    tClock(void);
    ERROR_CODE Initialize (tBoard *pBoardObject);
    TIME GetTimeLastInt(void);
  /*
    TIME GetCurrentTime(void);
    BOOL IsHealthy(void);
  */
  protected:
    tBoard *pBoard;             /* clock's board object */
} ;

#endif  /* INC_clockh */
