/* $RCSfile: commandHandler.h,v $ - Instrument Command Handler */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/commandHandler.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/
/* Richard - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,20Aug97,rlj      udated for new instrument CDS
02a,25Apr97,rlj      added tCommandHandler class
01a,16Oct96,eamck    written 
02a,23Apr97,rlj	     spec modified for tool
*/
      
/*
DESCRIPTION
  ISIT Instrument Command Handler definitions
*/


#ifndef INC_commandHandlerh
#define INC_commandHandlerh

#include "isit.h"
#include "isitCommand.h"
#include "baseObject.h"



//ERRORS

#define S_cmdhdlr_CANT_ALLOCATE_COMMAND_STREAMS  (M_cmdHdlr | 0x1)
#define S_cmdhdlr_INVALID_CMD_STREAM             (M_cmdHdlr | 0x2)
#define S_cmdhdlr_INVALID_GIZMO_ID               (M_cmdHdlr | 0x3)


//WARNINGS
#define S_cmdhdlr_ALREADY_INITIALIZED            (M_cmdHdlr | 0x1001)
#define S_cmdhdlr_ALREADY_SHUTDOWN               (M_cmdHdlr | 0x1002)

//INFO
  
#define S_cmdHdlr_ENTERING_CMD_PROC_INIT             (M_cmdHdlr | 0x2001)
#define S_cmdHdlr_COMMAND_STREAM_INITIALIZED         (M_cmdHdlr | 0x2002)
#define S_cmdHdlr_LEAVING_CMD_PROC_INIT              (M_cmdHdlr | 0x2003)
#define S_cmdHdlr_GETTING_A_CMD_TO_PROCESS           (M_cmdHdlr | 0x2004)
#define S_cmdhdlr_ENTERING_CMDHDLR_CONSTRUCTOR       (M_cmdHdlr | 0x2005)
#define S_cmdHdlr_LEAVING_CMDHDLR_INIT               (M_cmdHdlr | 0x2006)
#define S_cmdHdlr_PUT                                (M_cmdHdlr | 0x2007)
#define S_cmdHdlr_CHECKING_FOR_NEW_CMD               (M_cmdHdlr | 0x2008)
#define S_cmdHdlr_LEAVING_CMDHDLR_DESTRUCTOR         (M_cmdHdlr | 0x2009)
#define S_cmdHdlr_ENTERING_CMDHDLR_DESTRUCTOR        (M_cmdHdlr | 0x200a)
#define S_cmdHdlr_RESTART                            (M_cmdHdlr | 0x2009)
#define S_cmdHdlr_SHUTDOWN                           (M_cmdHdlr | 0x200a)
#define S_cmdHdlr_ISOK                               (M_cmdHdlr | 0x2009)



/*********

[[D tCommandHandler::tCommandHandler (void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to allocate or initialize resources to handle commands =>	
    console := console + appropriate error messages
    : this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->GizmoSet := anything
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->GizmoSet := {ids of every gizmo}
]]

[[D tCommandHandler::~tCommandHandler (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tCommandHandler::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : return := OK
| Unable to resume command handling =>
    console := console + appropriate error messages
    : return := appropriate error code
| true => this->bInit := TRUE
]]

[[D tCommandHandler::Shutdown (void) ]	
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspend command handling =>
    console := console + appropriate error messages
    : return := appropriate error code
| true => this->bInit := FALSE
]]


[[D tCommandHandler::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED 
| T.E element id of this->GizmoSet s.t. command handler cannot send command to 
  Gizmo (id) => return := FALSE
| true => return := TRUE
]]


[[D tCommandHandler::Put (const ISIT_CMD& Command) ]
[ invalid_pointer (this)  ||
  invalid_reference (Command) => UNDEFINED
| this->bInit == FALSE => no response
| Command.GizmoId not an element of this->GizmoSet =>
    appropriate error sent to ::ThisRteG error queue
| true => Command sent to Gizmo(Command.GizmoId)
]]



******************/

class tCommandStreamSender;


class tCommandHandler : public tBaseObject {
  public:
    tCommandHandler ();
    ~tCommandHandler ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk();
    ERROR_CODE Put (const ISIT_CMD& Command);
  protected:
    int iNumberOfCommandStreams;
    tCommandStreamSender **pCommandStreamArray;
};


#endif   INC_commandHandlerh









