/* $RCSfile: commandStream.h,v $  */
/* $Header*/

/*
modification history
--------------------
01a,24Jun97,rlj    written
*/

#include "isit.h"
#include "isitCommand.h"
#include "ipcFifoAccessor.h"

#ifndef INC_commandStreamh
#define INC_commandStreamh

//Error

//Warning


//Info 
#define S_commandStream_SENDER_CONSTRUCTOR     (M_commandStream | 0x2001)
#define S_commandStream_SENDER_DESTRUCTOR      (M_commandStream | 0x2002)
#define S_commandStream_RECEIVER_CONSTRUCTOR   (M_commandStream | 0x2003)
#define S_commandStream_RECEIVER_DESTRUCTOR    (M_commandStream | 0x2004)

/*******

[[D tCommandStreamSender::tCommandStreamSender (const tIpcFifoId &iiId) ]
[ invalid_pointer (this) || 
  invalid_pointer (::IpcMemoryG) || 
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiFifoId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiFifoId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG U 
      NewObjectSet (::IpcMemoryG, iiId)
]]

[[D tCommandStreamSender::~tCommandStreamSender (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tCommandStreamSender::Put (const ISIT_CMD &DataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (DataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG,
    this->iiObjectId) == FALSE ||
  simultaneous Put or PutSeveral on 
    GetObject (::IpcMemoryG, this->iiFifoId)=> UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 
  GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => 
    return := S_ipc_FIFO_FULL
| true => 
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue := 
      GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue 
      with DataIn appended to back
    : return := OK 
]]
 

[[D tCommandStreamSender::IsFull (void) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiObjectId) == FALSE => UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 
  GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => return := TRUE
| true => return := FALSE
]]

 
[[D tCommandStreamReceiver::Flush (void) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiObjectId) == FALSE ||
  simultaneous Put or PutSeveral on 
    GetObject (::IpcMemoryG, this->iiFifoId)=> UNDEFINED
| true =>
    GetObject (::IpcMemoryG, this->ObjectId)->dtQueue := Empty queue
    : return OK
]]

[[D tCommandStreamReceiver::tCommandStreamReceiver (const tIpcFifoId &iiId) ]
[ invalid_pointer (this) || 
  invalid_pointer (::IpcMemoryG) || 
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiFifoId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiFifoId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG U 
      NewObjectSet (::IpcMemoryG, iiId)
]]

[[D tCommandStreamReceiver::~tCommandStreamReceiver (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tCommandStreamReceiver::Get (ISIT_CMD &DataOut) ]
[ invalid_pointer (this) || 
  invalid_reference (DataOut) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiObjectId) == FALSE ||
  simultaneous Get or GetSeveral on GetObject (::IpcMemoryG, 
    this->iiFifoId) => UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 0 => 
    return := S_ipc_FIFO_EMPTY
| true =>
    DataOut := front element of 
      GetObject (::IpcMemoryG, this->ObjectId)->dtQueue
    : GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue := 
      GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue 
      with front element removed
    : return := OK 
]]

[[D tCommandStreamReceiver::IsEmpty (void) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiObjectId) == FALSE => UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 0 => 
    return := TRUE
| true => return := FALSE
]]


*****/

class tCommandStreamSender : protected tIpcFifoAccessor<ISIT_CMD> {
  public :
    tBaseObject::GetInitError;
    tIpcFifoAccessor<ISIT_CMD>::IsOk;
    tCommandStreamSender (const tIpcFifoId& iiId);
    virtual ~tCommandStreamSender ();
    tIpcFifoAccessor<ISIT_CMD>::Put; 
    tIpcFifoAccessor<ISIT_CMD>::IsFull; 
    tIpcFifoAccessor<ISIT_CMD>::Restart;
    tIpcFifoAccessor<ISIT_CMD>::Shutdown;
    tIpcFifoAccessor<ISIT_CMD>::IsOk;

};


class tCommandStreamReceiver : protected tIpcFifoAccessor<ISIT_CMD> {
  public :
    tCommandStreamReceiver (const tIpcFifoId& iiId);
    virtual ~tCommandStreamReceiver ();
    tBaseObject::GetInitError;
    tIpcFifoAccessor<ISIT_CMD>::IsOk;
    tIpcFifoAccessor<ISIT_CMD>::Get; 
    tIpcFifoAccessor<ISIT_CMD>::Flush;
    tIpcFifoAccessor<ISIT_CMD>::IsEmpty; 
    tIpcFifoAccessor<ISIT_CMD>::Restart;
    tIpcFifoAccessor<ISIT_CMD>::Shutdown;
    tIpcFifoAccessor<ISIT_CMD>::IsOk;
 
};

#endif INC_commandStreamh
