/* $RCSfile: configInfo.h,v $ - tConfigInfo header file */

/*
$Header: /home/chara/cvsroot/ople/sys/include/configInfo.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/

/* Phil Irwin - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,26Aug97,pci    written

*/

#ifndef INC_configInfoh
#define INC_configInfoh

#include <iostream.h>
#include <strstream.h>
#include "baseObject.h"

/****************************************************************************
*
* tConfigInfo
* -----------
* Black Box State Variables
*   tBaseObject::bInit         : BOOL
*   tBaseObject::ecInitError   : ERROR_CODE
*   sInfoFilename              : char*
*   sWhichSection              : char*
*   isInfoStream               : istream
*
* -----------
* Spec Functions
[[D ::trunc_string(char* s, int len)]
[ s == NULL =>
    return := NULL
| length(s) > len =>
    return := a well-formed character string containing a copy of the
              first len chars of *s
| true =>
    return := s
]]
*
*****************************************************************************
*  Methods
*
[[D tConfigInfo::tConfigInfo(void)]
Default Constructor.  This constructor simply sets bare
minimum  values.  The object will not be fully initialized after this
constructor executes.  Specifically, isInfoStream will not be valid.
The user will need to call SetFilename, SetSection(optional), and
ReadInfoFromFile before calling GetInfoStream.  Two more constructors
are supplied for more complete initialization.
[ invalid_pointer(this) => UNDEFINED
| true =>
    tBaseObject::bInit         := FALSE
    : tBaseObject::ecInitError := OK
    : this->sInfoFilename      := NULL
    : this->sWhichSection      := NULL
    : this->isInfoStream       := NULL istream
]]
*
*****************************************************************************
*
[[D tConfigInfo::tConfigInfo_s(const char* sFilename)]
Constructor.  This constructor sets the filename string and
reads from the file into the info stream.  It assumes the user
wants to read the entire file; the section string is set accordingly.
The object will be ready to use after this constructor executes.
[ invalid_pointer(this) ||
  invalid_pointer(sFilename) => UNDEFINED
| sFilname == NULL ||
  unable to open file sFilename ||
  error while reading file sFilename =>
    *this := anything
    : tBaseObject::ecInitError := appropriate error code
    : console := console || appropriate error message
| true =>
    tBaseObject::bInit := TRUE
    : tBaseObject::ecInitError := OK
    : *this->sInfoFilename := trunc_string(sFilename, CiMaxFileNameLength)
    : *this->sWhichSection := NULL
    : this->isInfoStream := formatted info read from file sFilename
]]
*
*****************************************************************************
*
[[D tConfigInfo::tConfigInfo_ss(const char* sFilename, const char* sSection)]
Constructor.  This constructor sets the filename and section
strings and reads information into the info stream.  The object will be
ready to use after this constructor executes.
[ invalid_pointer(this) ||
  invalid_pointer(sFilename) ||
  invalid_pointer(sSection) => UNDEFINED
| sFilname == NULL ||
  sSection == NULL ||
  unable to open file sFilename ||
  error while reading file sFilename ||
  section sSection does not exist in file =>
    *this := anything
    : tBaseObject::ecInitError := appropriate error code
    : console := console || appropriate error message
| true =>
    tBaseObject::bInit := TRUE
    : tBaseObject::ecInitError := OK
    : *this->sInfoFilename := trunc_string(sFilename, CiMaxFileNameLength)
    : *this->sWhichSection := trunc_string(sSection, CiMaxSectionNameLength)
    : this->isInfoStream := formatted info read from file sFilename,
                               section sSection
]]
*
*****************************************************************************
*
[[D tConfigInfo::~tConfigInfo(void)]
[ invalid_pointer(this) => UNDEFINED
| true =>
    *this := anything
]]
*
*****************************************************************************
*
[[D tConfigInfo::SetFilename(const char* sFilename)]
[ invalid_pointer(this) ||
  invalid_pointer(sFilename) => UNDEFINED
| true =>
    *this->sInfoFilename := trunc_string(sFilename, CiMaxFileNameLength)
    : return := OK
]]
*
*****************************************************************************
*
[[D tConfigInfo::SetSection(const char* sSection)]
Sets the section name to read.  The section string is used if
the specific configInfo class only uses one section of the input file.
If this method (or constructor with two args) is not called, the entire
file will be read in.
[ invalid_pointer(this) ||
  invalid_pointer(sSection) => UNDEFINED
| true =>
    *this->sWhichSection:= trunc_string(sSection, CiMaxSectionNameLength)
    : return := OK
]]
*
*****************************************************************************
*
[[D tConfigInfo::ReadInfoFromFile(void)]
Reads configuration information from appropriate location.
Uses internal state to determine file to open/section to read.
[ invalid_pointer(this) => UNDEFINED
| this->sInfoFilname == NULL ||
  unable to open file sFilename ||
  error while reading file sFilename ||
  (this->sWhichSection!=NULL && this->sWhichSection does not exist in file) =>
    console := console || appropriate error message
    : return appropriate error code
| true =>
    this->isInfoStream := formatted info read from file sFilename, section sSection
    : tBaseObject::bInit := TRUE
    : return := OK
]]
*
*****************************************************************************

    
  
/****************************************************************************
*
*                          Class Definitions
*
****************************************************************************/

class tConfigInfo : public tBaseObject {
  public:
    tConfigInfo(void);
    tConfigInfo(const char* sFilename);
    tConfigInfo(const char* sFilename, const char* sSection);
    ~tConfigInfo(void) { }
    ERROR_CODE SetFilename(const char* sFilename);
    ERROR_CODE SetSection(const char* sSection);

  private: // Put private before protected so isInfoStream knows about osInfoStream.
    ERROR_CODE FindSection(const char* s);
    ERROR_CODE LoadTagsToInfoStream(void);
    char* FindNextTag(void);
    char* ReadValues(void);
    BOOL IsComment(const char* s);
    BOOL IsSection(const char* s);
    BOOL IsTag(const char* s);
    ostrstream osInfoStream;

  protected:
    char* sInfoFilename;
    char* sWhichSection;
    istream isInfoStream(osInfoStream.rdbuf());
    ifstream isInfoFile;

};


#endif /* INC_configInfoh */

