/* $RCSfile: configMgr.h,v $ - Header file describing system configuration */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/configMgr.h,v 1.3 1998/08/17 21:01:12 phil Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b,24Oct96,eamck    added RteIntNum and RteErrFifoName arrays
01a,25Sep96,eamck    written
*/
  
/*
DESCRIPTION
  ISIT system configuration defines and types.  Ideally, if we change system 
  configuration this would be the only place we'd have to make changes. 
*/

#ifndef INC_configMgrh
#define INC_configMgrh

#include "isit.h"
#include "ipcid.h"
#include "share/configShare.h"
#include "configError.h"
#include "instError.h"
#include "hardware.h"
#include "servo.h"
#include "dlTypes.h"
#include "gsu/gsuDlTypes.h"
#include "gizmoInfo.h"
#include "stCommands.h"
#include "stTypes.h"
#include "telemetryManager.h"


#ifdef DYNAMIC_CASTING_OK
#include <typeinfo.h>
#define D_CAST(a) dynamic_cast<a>
#else
#define D_CAST
#endif


/****
[[D main::OnWhichRte (int iGizmoId) ]
[ CiNumGizmosOnRte0 < 0 ||
  CiNumGizmosOnRte1 < 0 => UNDEFINED
| CiNumGizmosOnRte0 incorrect ||
  CiNumGizmosOnRte1 incorrect => return := anything
| Gizmo iGizmoId not on Rte0 or Rte1 => return := -1
| Gizmo iGizmoId on Rte0 => return := 0
| Gizmo iGizmoId on Rte1 => return := 1
]]
******/

// searches RTE list for gizmo id (only RTE0 and RTE1)
int OnWhichRte (int iGizmoId);

/****
[[D main::GetGizmoSubindex (int iGizmoId) ]
[ CiGizmoSubindexArray not configured properly => UNDEFINED
| true =>
    return := sub-index corresponding to iGizmoId
]]
****/
int GetGizmoSubindex (int iGizmoId);


int RteIndexToGizmoId (int iRteId, int iWhichGizmo);



////////////////////////////
// Config Manager Constants
//
//



extern const IPC_NAME IPC_NULL_NAME;

// Fifo Depths

extern const int CiCommandFifoDepth;
extern const int CiDataFifoDepth;
extern const int CiStatusFifoDepth;
extern const int CiRteErrorFifoDepth;
extern const int ERROR_Q_MAXMSGS;



/************************ tConfig Spec **************************************

[[D tConfig::NumberOfRtes (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := CiNumRtes
]]

[[D tConfig::NumberOfSystemGizmos (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := CiNumSysGizmos
]]

[[D tConfig::NumberOfGizmosOnRte (int iWhichRte) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichRte == 0 => return := CiNumRte0Gizmos 
| iWhichRte == 1 => return := CiNumRte1Gizmos
| true => return := 0 
]]
[[D tConfig::NumberOfDelayLines (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := CiNumDelayLines
]]

[[D tConfig::GetGizmoRteNumber (int iGizmoId) ]
[ invalid_pointer (this) ||
  iGizmoId not valid gizmo id => UNDEFINED
| true => return := appropriate RTE number for Gizmo iGizmoId
]]


[[D tConfig::GetRteSchedPriority (int iWhichRte) ]
[ invalid_pointer (this) || 
  iWhichRte < 0 ||
  iWhichRte > CiNumRtes => UNDEFINED
| true => return := CiRteSchedPriority[iWhichRte]
]]

[[D tConfig::GetRteSchedMaxTasks (int iWhichRte) ]
[ invalid_pointer (this) || 
  iWhichRte < 0 ||
  iWhichRte > CiNumRtes => UNDEFINED
| true => return := CiRteSchedMaxTasks[iWhichRte]
]]

[[D tConfig::GetRteSchedRate (int iWhichRte) ]
[ invalid_pointer (this) || 
  iWhichRte < 0 ||
  iWhichRte > CiNumRtes => UNDEFINED
| true => return := CiRteSchedRate[iWhichRte]
]]

[[D tConfig::GetGizmoInfo_i (int iGizmoId) ]
[ invalid_pointer (this) => UNDEFINED
| iGizmoId < 0 || 
  iGizmoId >= CiNumSysGizmos => return := NULL
| true => return := CgiSystemGizmos[iGizmoId]
]]

[[D tConfig::GetGizmoId (int iRteId, int iRteGizmoIndex) ]
[ invalid_pointer (this) => UNDEFINED
| iRteId == 0 => return := CiRte0Gizmos[iWhichRteGizmo]
| iRteId == 1 => return := CiRte1Gizmos[iWhichRteGizmo] 
| true => return := NULL
]]

[[D tConfig::GetGizmoType (int iGizmoId)]
[ invalid_pointer (this) ||
  iGizmoId < 0 ||
  iGizmoId >= CgiSystemGizmos => UNDEFINED
| true => return := GIZMO_TYPE of CgiSystemGizmos [iGizmoId]
]]


[[D tConfig::GetErrorStreamId (int iWhichRte)]
[ invalid_pointer (this) => UNDEFINED
| iWhichRte < 0 ||
  iWhichRte >= CiNumRtes => return := anything
| true => return := IPC_ID for Rte iWhichRte Error stream
]]

[[D tConfig::GetDataStreamId (int iWhichRte)]
[ invalid_pointer (this) => UNDEFINED
| iWhichRte < 0 ||
  iWhichRte >= CiNumRtes => return := anything
| true => return := IPC_ID for Rte iWhichRte Data stream
]]

[[D tConfig::GetStatusStreamId (int iWhichRte)]
[ invalid_pointer (this) => UNDEFINED
| iWhichRte < 0 ||
  iWhichRte >= CiNumRtes => return := anything
| true => return := IPC_ID for Rte iWhichRte Status stream
]]
   
****/

   
#define NRTES           3       /* RTE0 , RTE1, RTE2 */

/* The following ifdefs are a temporary kludge.  When we get the
 * real config manager, we won't need to do this.
 */
#ifdef INC_initMgrcc
char *sRteExecFile [NRTES] = { "/home/chara/ople/bin/rte",
                               "/home/chara/ople/bin/rte",
                               "/home/chara/ople/bin/rte" };
#endif



#ifdef INC_rtecc
const BOARD_NAME CbnInterruptBoardArray[NRTES] = { TIM_BOARD_NAME, TIM_BOARD_NAME,
                                                   TIM_BOARD_NAME };
const int        CiBoardInterruptArray[NRTES] = { 5, 3, 1 }; // VME IRQ `n' maps to Tim Interrupt `n-1'
unsigned int RteIntNum[NRTES] = { 6, 4, 2 };       /* physical VMEBus IRQ numbers, unique to ea. RTE */
unsigned int RteIntVec[NRTES] = { 0xC0, 0xC1, 0xC2 }; /* Interrupt vectors */


#else
extern unsigned int RteIntNum[NRTES];   /* board interrupt numbers, unique to ea. RTE */
extern unsigned int RteIntVec[NRTES];

#endif

///////////////////////////////////////////////////////////
// System Configuration Constants

// RTEs
extern const int CiRte0SchedPriority;
extern const int CiRte0SchedMaxTasks;
extern const int CiRte0SchedRate;

extern const int CiRte1SchedPriority;
extern const int CiRte1SchedMaxTasks;
extern const int CiRte1SchedRate;

extern const int CiRteSchedPriority[];
extern const int CiRteSchedMaxTasks[];
extern const int CiRteSchedRate[];

		
// Delay Line		
extern const tQuicDlServoParams CspBb1ServoParams;
extern const tQuicDlServoParams CspBb2ServoParams;
extern const tGsuDlServoParameters CspGsuServoParams;
extern const TIME CtDlFastServoPeriod;
extern const int CiDlFastServoPriority;
extern const TIME CtDlMedServoPeriod;
extern const int CiDlMedServoPriority;
extern const TIME CtDlSlowServoPeriod;
extern const int CiDlSlowServoPriority;

extern const TIME CtDlTargetGeneratorPeriod;
extern const TIME CtDlTargetGeneratorOffset;
extern const int CiDlTargetGeneratorPriority;


// External Metrology

extern const TIME CtEmServoPeriod;
extern const int CiEmServoPriority;


// Fringe Tracker

extern const TIME CtFtFastServoPeriod;
extern const int CiFtFastServoPriority;
extern const TIME CtFtMediumServoPeriod;
extern const int CiFtMediumServoPriority;
extern const TIME CtFtSlowServoPeriod;
extern const int CiFtSlowServoPriority;
extern const TIME CtFtImagingServoPeriod;
extern const int CiFtImagingServoPriority;

extern const TIME CtFtTrackUpdatePeriod;
extern const int CiFtTrackUpdatePriority;


// Star Tracker Constants

extern const tStPhysicalConstants Cst0PhysConst;
extern const tStPhysicalConstants Cst1PhysConst;
extern const tStControllerParams Cst0ControllerParams;
extern const tStControllerParams Cst1ControllerParams;
extern const tStServoParams Cst0ServoParams;
extern const tStServoParams Cst1ServoParams;
extern const BOARD_NAME CbnSt0FsmBoardArray[NUM_FSM_CHANNELS];
extern const BOARD_NAME CbnSt1FsmBoardArray[NUM_FSM_CHANNELS];
extern const int CiSt0FsmChannelArray[NUM_FSM_CHANNELS];
extern const int CiSt1FsmChannelArray[NUM_FSM_CHANNELS];
extern const TIME CtStServoPeriod;
extern const int CiStServoPriority;

// End Configuration Constants
///////////////////////////////////////////////////////////////////////	    



class tIpcFifoId;
class tGizmoInfo;

class tConfig : public tBaseObject {
  public:
    tConfig(void);
    int NumberOfRtes () const { return NRTES;}
    int NumberOfSystemGizmos () const { return NUM_SYS_GIZMOS; }
    int NumberOfGizmosOnRte (int iWhichRte) const;
    int NumberOfDelayLines () const { return NUM_DELAY_LINES; }
    int GetGizmoRteNumber (int iGizmoId) const;
    int GetRteSchedPriority (int iWhichRte) const;
    int GetRteSchedMaxTasks (int iWhichRte) const;
    int GetRteSchedRate (int iWhichRte) const;
    int GetGizmoId (int iRteId, int iRteGizmoIndex) const;
    const tGizmoInfo* GetGizmoInfo (int iGizmoId) const;
    GIZMO_TYPE GetGizmoType (int iGizmoId) const;

    const tIpcFifoId& GetErrorStreamId (int iWhichRte) const;
    const tIpcFifoId& GetDataStreamId (int iWhichRte) const;
    const tIpcFifoId& GetStatusStreamId (int iWhichRte) const;
    size_t GetRelayBufferSize () const;

    const tTelemetryManagerInfo& TelemetryInfo() const ;  // blw -- Added

    tHardware Hardware;
   
  protected:
    tGizmoInfo* pCgiSystemGizmoInfoArray[NUM_SYS_GIZMOS];
    tIpcFifoId  iiRteErrorStreamIdArray[NRTES];    
    tIpcFifoId  iiRteDataStreamIdArray[NRTES];    
    tIpcFifoId  iiRteStatusStreamIdArray[NRTES];    

}; 


extern tConfig ConfigManager;


#endif INC_configMgrh


