/* $RCSfile: controller.h,v $ - implements generic servo and controller functionality */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/controller.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/
/* Richard Johnson - JetPropulsion Laboratory */
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,27Aug97    modified entire controller hierachy
01a,beh,17May97    broken off from servo.h
*/
 
/*
DESCRIPTION

*/


#ifndef INC_controllerh
#define INC_controllerh

#include "isit.h"

/**************  Error Codes that should  be moved to another file *********/

#define S_controller_UNABLE_TO_ALLOCATE_MODED_CONTROLLER   (M_controller | 0x1)
#define S_controller_INVALID_CONTROLLER_MODE               (M_controller | 0x2)
#define S_controller_UNASSIGNED_CONTROLLER_MODE            (M_controller | 0x3)
#define S_controller_INVALID_CONTROLLER                    (M_controller | 0x4)
#define S_controller_INVALID_MODED_CONTROLLER              (M_controller | 0x5)


/******  tController Black Box Specification **************************

tController is the base class for all controllers.  This class has no data and 
performs default (return OK) controller initialization, reset, and calculation.


[[D tController::tController (void) ]
Default constructor.
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]


[[D tController::~tController (void) ]
Default destructor.
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]


[[D tController::Init (TIME tServoTime) ]
Performs appropriate control law initialization.  
Behavior depends on specific type of controller.
[ invalid_pointer (this) => UNDEFINED 
| true =>  return := OK
]]


[[D tController::Reset (TIME tServoTime) ]
Performs appropriate control law reset.  
Behavior depends on specific type of controller.
[ invalid_pointer (this) => UNDEFINED 
| true =>  return := OK
]]


[[D tController::Calculate (TIME tServoTime) ]
Performs appropriate control law calculation.  
Behavior depends on specific type of controller.
[ invalid_pointer (this) => UNDEFINED 
| true =>  return := OK
]]


****/

// Controller data is a remnant of the old servo design and should be removed 
// as soon as the delay lines don't depend on it.

class tControllerData {};


class tController {
  public:
    tController (void);
    virtual ~tController (void);
    virtual ERROR_CODE Init (TIME tServoTime);
    virtual ERROR_CODE Reset (TIME tServoTime);
    virtual ERROR_CODE Calculate (TIME tServoTime);
};


#endif  INC_controllerh
