/* $RCSfile: controllerSet.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/controllerSet.h,v 1.1.1.1 1998/06/11 21:36:10 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written

DESCRIPTION:  
  This file contains class definitons for:
    tControllerGroup
    tSimpleControllerGroup
    tUpdateModeDecision
    tModedControllerGroup
    tControllerSet

  For a general overview of theses classes see Servo Specification document.

*/


#ifndef INC_controllerSeth
#define INC_controllerSeth

#include "isit.h"
#include "controller.h"

/**************  Error Codes that should  be moved to another file *********/


#define S_controllergroup_UNABLE_TO_ALLOCATE_CONTROLLER_GROUP   (M_controller | 0x1)
#define S_controllergroup_INVALID_PARTITION                     (M_controller | 0x2)
#define S_controllergroup_INVALID_CONTROLLER_MODE               (M_controller | 0x3)
#define S_controllergroup_INVALID_CONTROLLER_GROUP              (M_controller | 0x4)
#define S_controllergroup_INVALID_CONTROLLER_MODE_FROM_UPDATE   (M_controller | 0x5)
#define S_controllerset_UNABLE_TO_ALLOCATE_CONTROLLER_SET       (M_controller | 0x6)
#define S_controllergroup_INVALID_MODED_CONTROLLER              (M_controller | 0x7)
#define S_controllergroup_GROUP_NOT_SAME_SIZE                   (M_controller | 0x8)
#define S_controllergroup_UNASSIGNED_CONTROLLER_GROUP           (M_controller | 0x9) 

/************************Specification Funcitons **************************************
BOOL InvalidIndex (int iIndex, int iNumberOfElements) :==
[ 0 <= iIndex < iNumberOfElments => return := FALSE
| true => return := TRUE
]
*/

/******  tControllerGroup Black Box Specification **************************
tControllerGroup is the base class for simple and moded controller groups.  


[[D tControllerGroup::tControllerGroup (int iNumberOfPartitionsIn) ]
Constructs this to be a controller group with iNumberOfPartitionsIn partitions.
[ invalid_pointer (this) => UNDEFINED 
| true => this->iNumberOfPartitions := MAX (0, iNumberOfPartitionsIn)
]]
 

[[D tControllerGroup::~tControllerGroup (void) ]
Virtual destructor: Depending on the specific type of controller group,
destructor will free all reasources consumed by this.
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]


[[D tControllerGroup::GetNumberOfPartitions (void) ]
Returns number of servo partitions handled by this.
[ invalid_pointer (this) => UNDEFINED 
| true => return := this->iNumberOfPartitions
]]

[[D tControllerGroup::ControllerMode (void) ]
Virtual: Returns current controller mode of this if applicable.
[ invalid_pointer (this) => UNDEFINED 
| true => return := 0
]]


[[D tControllerGroup::Init (void) ]
Virtual:  Initializes entire controller group *this.
[ invalid_pointer (this) => UNDEFINED 
| true => return := OK
]]


[[D tControllerGroup::Init_i (int iWhichPartition) ]
Pure Virtual:  Initializes *this partition iWhichPartition
[ invalid_pointer (this) => UNDEFINED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) => 
    return := appropriate error code
| true => See specific controller group init.
]]


[[D tControllerGroup::Reset (void) ]
Virtual: Resets entire controller group *this.
[ invalid_pointer (this) => UNDEFINED 
| true => return := OK
]]


[[D tControllerGroup::Reset_i (int iWhichPartition) ]
Pure Virtual:  Resets *this partition iWhichPartition
[ invalid_pointer (this) => UNDEFINED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) => 
    return := appropriate error code
| true => See specific controller group reset.
]]



[[D tControllerGroup::Calculate (TIME tServoTime, int iWhichPartition) ]
Vitual: Performs control law calculation for *this partition iWhichPartition.
[ invalid_pointer (this) => UNDEFINED 
| InvalidIndex (iWhichController, this->iNumberOfPartitions) => 
    return := appropriate error code
| true => See specific controller group calculate
]]



******  tSimpleControllerGroup Black Box Specification ************************

tSimpleControllerGroup organizes controllers that belong to the same servo 
mode indexed by servo partition.  

[[D tSimpleControllerGroup::tSimpleControllerGroup (int iNumberOfPartitionsIn) ]
Constructs *this as a controller group with iNumberOfPartitionsIn partitions 
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iNumberOfPartitions := MAX (0, iNumberOfPartitionsIn)
    : this->ControllerPointerArray := ARRAY <tController*> with 
      MAX (0, iNumberOfPartitionsIn) elements initialized &this->NullController
]]


[[D tSimpleControllerGroup::~tSimpleControllerGroup (void) ]
Destructs *this freeing resources consumed by this->ControllerPointerArray.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointerArray) => UNDEFINED 
| true => 
    resources consumed by this->ControllerPointerArray freed
    : *this := anything
]]


[[D tSimpleControllerGroup::AssignController (int iWhichController, 
    tController& Controller) ]
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointerArray) ||
  invalid_reference (Controller) => UNDEFINDED 
| InvalidIndex (this->iNumberOfControllers, iWhichController) =>
    return := appropriate error code
| true =>
    this->ControllerPointerArray [iWhichController] := &Controller
    : return := OK
]]


[[D tSimpleControllerGroup::Init (void) ] = tControllerGroup::Init ]]


[[D tSimpleControllerGroup::Init_i (int iWhichPartition) ]
Initializes *this partition iWhichPartition. The behavior depends on the 
specific partition and controller initialized.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointerArray) => UNDEFINED 
| InvalidIndex (this->iNumberOfPartitions, iWhichPartition) ||
  Error initializing controller 
    *this->ControllerPointerArray [iWhichPartition] => 
      return := appropriate error code
| true =>
    Controller *this->ControllerPointerArray [iWhichPartition] initialized
    : return := OK
]]


[[D tSimpleControllerGroup::Reset (void) ] = tControllerGroup::Reset ]]



[[D tSimpleControllerGroup::Reset_i (int iWhichPartition) ]
Resets *this partition iWhichPartition. The behavior depends on the 
specific partition and controller reset
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointerArray) => UNDEFINED 
| InvalidIndex (this->iNumberOfPartitions, iWhichPartition) ||
  Error resetting controller 
    *this->ControllerPointerArray [iWhichPartition] => 
      return := appropriate error code
| true =>
    Controller *this->ControllerPointerArray [iWhichPartition] reset
    : return := OK
]]


[[D tSimpleControllerGroup::Calculate (TIME tServoTime, int iWhichController) ]
Performs control law calculation for *this partition iWhichPartition. 
The behavior depends on the specific partition and controller calculated.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointerArray) => UNDEFINED 
| InvalidIndex (this->iNumberOfPartitions, iWhichPartition) ||
  Error calculating control law for controller 
    *this->ControllerPointerArray [iWhichPartition] => 
      return := appropriate error code
| true =>
    Control law computed for controller 
      *this->ControllerPointerArray [iWhichPartition] 
    : return := OK
]]





******************* tUpdateModeDecision Black Box Specification ***************

This class is an abstract companion class to tModedControllerGroup.
It is intended to provide a common mechanism for updating controller modes.
Derived classes must overload GetNewMode which returns the new mode base on 
whatever criteria necessary to chose a controller mode for a particular moded
controller group.  See also tModedControllerGroup.


[[D tUpdateModeDecision::tUpdateModeDecision (void) ]
Constructs *this and initializes default state.
[ invalid_pointer (this) => UNDEFINED
| true => this->iCurrentMode := 0
]]


[[D tUpdateModeDecision::~tUpdateModeDecision (void) ]
Virtual destructor:  Frees any reasources used by *this.
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tUpdateModeDecision::Init (void) ]
Virtual: Initializes *this.
[ invalid_pointer (this) => UNDEFINED
| true => return := OK
]]


[[D tUpdateModeDecision::Reset (void) ]
Virtual: Resets *this.
[ invalid_pointer (this) => UNDEFINED
| true => return := OK
]]


[[D tUpdateModeDecision::GetNewMode (TIME tUpdateTime) ]
Pure virtual:  Computes and returns the new mode.
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iCurrentControllerMode := new mode
    : return := new mode
]]


******************* tModedControllerGroup Black Box Specification ************

tModedControllerGroup organizes controllers that belong to the same servo mode
indexed by servo partition and controller mode.  

[[D tModedControllerGroup::tModedControllerGroup (
      int iNumberOfPartitionsIn, 
      int iNumberOfControllerModesIn,
      tUpdateModeDecision& UpdateModeDecisionIn,
      int iTaskPriority, TIME tTaskPeriod, TIME tTaskOffset) ]
Constructs *this as a controller group with iNumberOfPartitionsIn partitions,
and iNumberOfControllerModesIn controller modes.  Schedules initially disabled
periodic task to update controller mode.
[ invalid_pointer (this) ||
  invalid_reference (UpdateModeDecisionToUse) ||
  SchedulerOk () == FALSE => UNDEFINED 
| Unable to add update mode task to ::PeriodicTaskSet with periodid task
    parameters iTaskPriority, tTaskPeriod, tTaskOffset => 
      console := console + appropriate error code
      : *this := anything
      : return := appropriate error code
| true => 
    this->iNumberOfPartitions := MAX (0,iNumberOfPartitionsIn)
    : this->iNumberOfControllerModes := MAX (0,iNumberOfControllerModesIn)
    : this->ControllerPointer2DArray := ARRAY2D <tController*> with 
      MAX (0,iNumberOfPartitionsIn) x MAX (0, iNumberOfControllerModesIn) 
      elements initialized to &this->NullController
    : this->UpdateModeDecision := reference to UpdateModeDecisionIn
    : this->ModeUpdateTask := disabled PeriodicTask ("UpdateControllerMode",
        tTaskPeriod, iTaskPriority, tTaskOffset, 
        tModedControllerGroup::PeriodicControllerModeUpdate)
    : this->iCurrentControllerModeArray := ARRAY <int> with 
      MAX (0,iNumberOfPartitons) elements initialized to 0
    : this->iNewControllerMode := 0
]]


[[D tModedControllerGroup::~tModedControllerGroup (void) ]
Destructs *this freeing reasources consumed by this->ControllerPointer2DArray,
this->iCurrentControllerModeArray, and periodic controller mode update task.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointer2DArray) ||
  invalid_array (this->iCurrentControllerMode) ||
  SchedulerOK () == FALSE => UNDEFINED 
| true => 
    Resources consumed by this->ControllerPointer2DArray freed.
    Resources consumed by this->iControllerModeArray freed.
    : this->ModeUpdateTask deleted
    : *this := anything
]]



[[D tModedControllerGroup::GetNumberOfPartitions (void) ] =
    tControllerGroup::GetNumberOfPartitions ]]


[[D tModedControllerGroup::ControllerMode (void) ]
Returns most recent controller mode.
[ invalid_pointer (this) => UNDEFINED 
| true => return := this->iNewControllerMode
]]


[[D tModedControllerGroup::Init (void) ]
Initializes *this by initializing the decision object and 
enabling controller mode update task.
[ invalid_pointer (this) ||
  invalid_reference (this->UpdateModeDecision) ||
  SchedulerOk () == FALSE => UNDEFINED 
| Error initializing this->UpdateModeDecision ||
  Error enabling this->UpdateModeTask => 
    *this := anything
    : return := appropriate error code
| true =>
    this->UpdateModeDecision initialized
    : this->UpdateModeTask enabled
    : return := OK
]]


[[D tModedControllerGroup::Init_i (int iWhichPartition) ]
Initializes controller for partition iWhichPartition and the current 
controller mode.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointer2DArray) ||
  invalid_array (this->iCurrentControllerModeArray) ||
  Invalid_Index (this->iCurrentControllerMode [iWhichPartiton], 
    this->iNumberOfControllerModes) => UNDEFINED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) ||
  Unable to initialize controller *this->ControllerPointer2DArray 
    [iWhichPartition, this->iCurrentControllerMode [iWhichPartiton] ] => 
      return := appropriate error code
| true =>
    Controller *this->ControllerPointer2DArray [iWhichPartition, 
      this->iCurrentControllerMode [iWhichPartition] ] initialized
    : return := OK
]]


[[D tModedControllerGroup::Reset (void) ]
Resets *this by resetting the decision object and 
disabling controller mode update task.
[ invalid_pointer (this) ||
  invalid_reference (this->UpdateModeDecision) ||
  SchedulerOk () == FALSE => UNDEFINED 
| Error resetting this->UpdateModeDecision ||
  Error disabling this->UpdateModeTask => 
    *this := anything
    : return := appropriate error code
| true =>
    this->UpdateModeDecision reset
    : this->UpdateModeTask disabled
    : return := OK
]]


[[D tModedControllerGroup::Reset_i (int iWhichPartition) ]
Resets controller for partition iWhichPartition and the current 
controller mode.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointer2DArray) ||
  invalid_array (this->iCurrentControllerModeArray) ||
  Invalid_Index (this->iCurrentControllerMode [iWhichPartiton], 
    this->iNumberOfControllerModes) => UNDEFINED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) ||
  Unable to reset controller *this->ControllerPointer2DArray 
    [iWhichPartition, this->iCurrentControllerMode [iWhichPartiton] ] => 
      return := appropriate error code
| true =>
    Controller *this->ControllerPointer2DArray [iWhichPartition, 
      this->iCurrentControllerMode [iWhichPartition] ] reset
    : return := OK
]]


[[D tModedControllerGroup::AssignController (int iWhichPartition,
      int iWhichControllerMode, tController& Controller) ]
Assigns element (iWhichPartiton, iWhichControllerMode) to point to &Controller.      
[ invalid_pointer (this) ||
  invalid_reference (Controller) ||
  invalid_array (this->ControllerPointer2DArray) => UNDEFINDED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) ||
  InvalidIndex (iWhichControllerMode, this->iNumberOfControllerModes) =>
    return := appropriate error code
| true =>
    this->ControllerArray [iWhichPartition,iWhichControllerMode] := ModedController
    : return := OK
]]



[[D tModedControllerGroup::Calculate (TIME tServoTime, int iWhichPartition) ]
[ invalid_pointer (this) ||
  invalid_array (this->ControllerPointer2DArray) ||
  invalid_array (this->iCurrentControllerModeArray) ||
  InvalidIndex (this->iCurrentControllerMode [iWhichPartiton], this->iNumberOfControllerModes) ||
  InvalidIndex (this->iNewControllerMode, this->iNumberOfControllerModes) => 
    UNDEFINED
| InvalidIndex (iWhichPartition, this->iNumberOfPartitions) ||
  Error calculating control law for controller 
    *this->ControllerPointer2DArray [iWhichPartition, this->iNewControllerMode] => 
       return := appropriate error code
| this->iCurrentControllerModeArray [iWhichPartition] != this->iNewControllerMode =>
    this->iCurrentControllerMode [iWhichPartiton] := this->iNewControllerMode
    : Controller *this->ControllerPointer2DArray [iWhichPartition, 
      this->iCurrentControllerMode [iWhichPartition] ] reset
    : Controller *this->ControllerPointer2DArray [iWhichPartition, 
      this->iNewControllerMode] initialize
    : Control law calculated for controller *this->ControllerPointer2DArray [
       iWhichPartition, this->iNewControllerMode] 
    : return := OK  
| true =>
    Control law calculated for controller *this->ControllerPointer2DArray [
      iWhichPartition, this->iNewControllerMode]   
    : return := OK
]]


************************tControllerSet Black Box Specification ***********************

[[D tControllerSet::tControllerSet (int iNumberOfControllerGroupsIn,
                                    int iNumberOfPartitionsIn) ]
Constructs *this with iNumberOfControllerGroupsIn controller groups each having
iNumberOfPartitonIn partitions.                                    
[ invalid_pointer (this) => UNDEFINED 
| iNumberOfControllerGroups < 0 ||
  iNumberOfParititionsToUse < 0 => *this := anything
| true => 
    this->iNumberOfControllerGroups := MAX (0, iNumberOfControllerGroupsIn)
    : this->iNumberOfPartitions := MAX (0, iNumberOfParitionsIn)
    : this->ControllerGroupPointerArray := ARRAY <tControllerGroup*> with
      MAX (0, iNumberOfControllerGroupsIn) initalized to &this->NullControllerGroup
    : this->pPreCalculateControllerGroup := &this->NullControllerGroup
    : this->pPostCalculateControllerGroup := &this->NullControllerGroup
    : this->iCurrentControllerGroupArray := ARRAY <int> with 
      MAX (0, iNumberOfControllerGroupsIn) elements initialized to 0
    : this->iNewControllerGroup := 0
]]


[[D tControllerSet::~tControllerSet (void) ]
Destructor.  Frees all resources consumed by *this.
[ invalid_pointer (this) ||
  invalid_array (this->ControllerGroupPointerArray) ||
  invalid_array (this->iCurrentControllerGroupArray) => UNDEFINED 
| true => 
    resources consumed by this->ControllerGroupPointerArray freed
    : resources consumed by this->iCurrentControllerGroupArray freed
    : *this := anything
]]


[[D tControllerSet::SetPreCalculateControllerGroup (tControllerGroup& ControllerGroup) ]
Assign controller group to the pre calculate group.
[ invalid_pointer (this) ||
  invalid_reference (ControllerGroup) ||
  invalid_array (this->ControllerGroupPointerArray) => UNDEFINDED 
| ControllerGroup.iNumberOfPartitions != this->iNumberOfPartitions =>
    return := appropriate error code
| true =>
    this->pPreCalculateControllerGroup := &ControllerGroup
    : return := OK
]]


[[D tControllerSet::SetPostCalculateControllerGroup (tControllerGroup& ControllerGroup) ]
Assign controller group to the post calculate group.
[ invalid_pointer (this) ||
  invalid_reference (ControllerGroup) ||
  invalid_array (this->ControllerGroupPointerArray) => UNDEFINDED 
| ControllerGroup.iNumberOfPartitions != this->iNumberOfPartitionsPerGroup =>
    return := appropriate error code
| true =>
    this->pPostCalculateControllerGroup := &ControllerGroup
    : return := OK
]]


[[D tControllerSet::AssignControllerGroup (int iWhichControllerGroup, 
                                           tControllerGroup& ControllerGroup) ]
Assign controller group to this->ControllerGroupPointerArray.
[ invalid_pointer (this) ||
  invalid_reference (ControllerGroup) ||
  invalid_array (this->ControllerGroupPointerArray) => UNDEFINDED 
| InvalidIndex (iWhichControllerGroup, this->iNumberOfControllerGroups) ||
  ControllerGroup.iNumberOfPartitions != this->iNumberOfPartitions =>
    return := appropriate error code
| true =>
    this->ControllerGroupPointerArray [iWhichControllerGroup] := &ControllerGroup
    : return := OK
]]


[[D tControllerSet::SelectControllerGroup (int iWhichControllerGroup) ]
Select controller group iWhichControllerGroup as the new group to use for calculation.
[ invalid_pointer (this) => UNDEFINED
| InvalidIndex (iWhichControllerGroup, this->iNumberOfControllerGroups) =>
    return := appropriate error code
| true =>
    this->iNewControllerGroup := iWhichControllerGroup
    : return := OK
]]


[[D tControllerSet::GetCurrentControllerGroup (void) ]
Return most recently activated controller group number.
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iNewControllerGroup 
]]


[[D tControllerSet::ControllerMode (void) ]
Return most recently activated controller mode of the most recently 
activated controller group.
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ControllerGroupPointerArray [this->iNewControllerGroup].
    iNewControllerMode if moded group else 0.
]]



[[D tControllerSet::Calculate (TIME tServoTime, int iWhichPartition) ]
Perform control law calculationfor partition iWhichPartition using controllers of the
most recently activated controller group and controller mode if applicable.
[ invalid_pointer (this) || 
  invalid_array (this->ControllerGroupPointerArray) ||
  invalid_array (this->iCurrentControllerGroupArray) ||
  InvalidIndex (this->iCurrentControllerGroupArray [iWhichPartiton], 
                this->iNumberOfControllerGroups) => UNDEFINED 
| InvalidIndex (iWhichPartition, this->iNumberOfPartitons) ||
  InvalidIndex (iWhichPartition, this->ControllerGroupPointerArray [
    this->iCurrentControllerGroupArray [iWhichPartition] ].iNumberOfPartitions) ||
  Error calculating control law for controller group
    *this->ControllerGroupPointerArray [iNewControllerGroup] => 
      return := appropriate error code
| this->iCurrentControllerGroupArray [iWhichPartition] != this->iNewControllerGroup =>
    this->iCurrentControllerGroupArray [iWhichPartition] := this->iNewControllerGroup
    : Control law calculated for partition iWhichPartition of controller group 
      *this->PreCalculateControllerGroup
    : Control law calculated for partition iWhichPartition of controller group 
      *this->PostCalculateControllerGroup
    : Control law calculated for partition iWhichPartiton of controller group 
      *this->ControllerGroupArray [iNewControllerGroup] 
    : Partition iWhichPartition of controller group 
      *this->ControllerGroupArray [this->iCurrentControllerGroupArray [iWhichPartition] ] reset
    : Partition iWhichPartition of controller group 
      *this->ControllerGroupArray [this->iNewControllerGroup] initialize
    : return := OK
| true => 
    Control law calculated for partition iWhichPartition of controller group 
      *this->PreCalculateControllerGroup
    : Control law calculated for partition iWhichPartition of controller group 
      *this->PostCalculateControllerGroup
    : Control law calculated for partition iWhichPartiton of controller group 
      *this->ControllerGroupArray [iNewControllerGroup] 
    : return := OK
]]


****/

class tServoTaskInfo;
class tServoInfo;
class tControllerData;

class tControllerGroup {
  public:
    tControllerGroup (int iNumberOfPartitionsToUse);
    virtual ~tControllerGroup ();
    int GetNumberOfPartitions () const;
    virtual int ControllerMode () { return 0;}
    virtual ERROR_CODE Init () { return OK;} 
    virtual ERROR_CODE Init (TIME tServoTime, int iWhichPartition) = 0;
    virtual ERROR_CODE Reset () { return OK;}
    virtual ERROR_CODE Reset (TIME tServoTime, int iWhichPartition) = 0;
    virtual ERROR_CODE Calculate (TIME tServoTime, int iWhichController) = 0;
  protected:
    int iNumberOfPartitions;
    tController NullController;
};


class tSimpleControllerGroup : public tControllerGroup {
  public:
    tSimpleControllerGroup (int iNumberOfControllers);
    virtual ~tSimpleControllerGroup ();
    ERROR_CODE AssignController (int iWhichPartition, 
                                 tController& Controller);
    virtual ERROR_CODE Calculate (TIME tServoTime, int iWhichPartition);
    virtual ERROR_CODE Init (TIME tServoTime, int iWhichPartition);
    virtual ERROR_CODE Reset (TIME tServoTime, int iWhichPartition);
  private:
    tController** pControllerArray;
};

class tUpdateModeDecision {
  public:
    virtual ERROR_CODE Init () { return OK;};
    virtual ERROR_CODE Reset () { return OK;};
    virtual int GetNewMode (TIME tUpdateTime) = 0; 
  protected:
    int iCurrentControllerMode;
};

class tModedControllerGroup : public tControllerGroup {
  public:
    tModedControllerGroup (int iNumberOfPartitionsToUse, 
                           int iNumberOfControllerModesToUse,
                           tUpdateModeDecision& UpdateModeDecisionToUse,
                           int iTaskPriority,
                           TIME tTaskPeriod, TIME tTaskOffset);
    virtual ~tModedControllerGroup ();
    ERROR_CODE AssignController (int iWhichPartition, 
       int iWhichControllerMode, tController& Controller);
    virtual ERROR_CODE Calculate (TIME tServoTime, int iWhichPartition);
    virtual ERROR_CODE Init ();
    virtual ERROR_CODE Init (TIME tServoTime, int iWhichPartition);
    virtual ERROR_CODE Reset ();
    virtual ERROR_CODE Reset (TIME tServoTime,int iWhichPartition);
    virtual int ControllerMode () { return this->iCurrentControllerModeArray [0]; } // is this what we want?
  private:
    ERROR_CODE PeriodicControllerModeUpdate (TIME tUpdateTime);

    int iNumberOfControllerModes;
    int* iCurrentControllerModeArray;
    int iNewControllerMode;
    tController*** pController2DArray;
    tUpdateModeDecision& UpdateModeDecision;
    int iUpdateModeTaskId;
};


class tControllerSet {
  public:
    tControllerSet (int iNumberOfControllerGroupsToUse, 
                    int iNumberOfControllersPerGroupToUse);
    virtual ~tControllerSet ();
    ERROR_CODE SetPreCalculateControllerGroup (tControllerGroup& ControllerGroup);
    ERROR_CODE SetPostCalculateControllerGroup (tControllerGroup& ControllerGroup);
    ERROR_CODE AssignControllerGroup (int iWhichControllerGroup, 
                              tControllerGroup& ControllerGroup);
    ERROR_CODE SelectControllerGroup (int iWhichControllerGroup);
    int GetCurrentControllerGroup () const;
    int ControllerMode () const;
    virtual ERROR_CODE Calculate (TIME tServoTime, int iWhichController);
  private:
    int iNumberOfControllerGroups;
    int iNumberOfPartitionsPerGroup;
    int* iCurrentControllerGroupArray;
    int iNewControllerGroup;
    tSimpleControllerGroup NullControllerGroup;
    tControllerGroup** pControllerGroupArray;
    tControllerGroup* pPreCalculateControllerGroup;
    tControllerGroup* pPostCalculateControllerGroup;
};

#endif INC_controllerSeth

