/* $RCSfil$ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dl/dlCommandEngine.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,24Sep96    written
*/
 
/*
DESCRIPTION
This file contains the class and type definitions for the delay line
command processor.
*/

#ifndef INC_dlCommandEngineh
#define INC_dlCommandEngineh

#include "mutex.h"

#define S_dlCommandProcessor_ENTERING_DL_CMD_PROC_INIT (M_dlCommandProcessor | 0x1001)
#define S_dlCommandProcessor_LEAVING_DL_CMD_PROC_INIT  (M_dlCommandProcessor | 0x1002)
#define S_dlCommandProcessor_ENTERING_DISPATCH_COMMAND (M_dlCommandProcessor | 0x1003)
#define S_dlCommandProcessor_GOT_CLEAR_LASER_CMD       (M_dlCommandProcessor | 0x1004)
#define S_dlCommandProcessor_GOT_SET_MOD_CMD           (M_dlCommandProcessor | 0x1005)
#define S_dlCommandProcessor_GOT_SELECT_TARGET_CMD     (M_dlCommandProcessor | 0x1006)
#define S_dlCommandProcessor_GOT_CHNG_SERVO_PARAM_CMD  (M_dlCommandProcessor | 0x1007)
#define S_dlCommandProcessor_GOT_HOME_SET_CMD          (M_dlCommandProcessor | 0x1008)
#define S_dlCommandProcessor_GOT_HOME_CHECK_CMD        (M_dlCommandProcessor | 0x1009)
#define S_dlCommandProcessor_GOT_DITHER_ON_CMD         (M_dlCommandProcessor | 0x100a)
#define S_dlCommandProcessor_GOT_DITHER_CALIB_CMD      (M_dlCommandProcessor | 0x100b)
#define S_dlCommandProcessor_GOT_DITHER_CHECK_CMD      (M_dlCommandProcessor | 0x100c)
#define S_dlCommandProcessor_GOT_OFF_CMD               (M_dlCommandProcessor | 0x100d)
#define S_dlCommandProcessor_GOT_PING_CMD              (M_dlCommandProcessor | 0x100e)
#define S_dlCommandProcessor_GOT_FRONT_CMD             (M_dlCommandProcessor | 0x100f)
#define S_dlCommandProcessor_GOT_BACK_CMD              (M_dlCommandProcessor | 0x1010)
#define S_dlCommandProcessor_GOT_TRACK_CMD             (M_dlCommandProcessor | 0x1011)
#define S_dlCommandProcessor_GOT_USER1_CMD             (M_dlCommandProcessor | 0x1012)
#define S_dlCommandProcessor_GOT_USER2_CMD             (M_dlCommandProcessor | 0x1013)
#define S_dlCommandProcessor_DIDNT_RECOGNIZE_COMMAND   (M_dlCommandProcessor | 0x1014)
#define S_dlCommandProcessor_LEAVING_DISPATCH_COMMAND  (M_dlCommandProcessor | 0x1015)


/******************************************************************
 
[[D tDlCommandEngine::tDlCommandEngine_t (tGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (GizmoToUse) => UNDEFINED
| true => 
    this->Gizmo := reference to GizmoToUse
    : this->sTaskName := NULL 
]]

[[D tDlCommandEngine::tDlCommandEngine_tts (tMutex& MutexToUse, 
        tDlGizmo& GizmoToUse, char *sTaskNameToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true => 
    this-Mutex := reference to MutexToUse
    : this->Gizmo := reference to GizmoToUse
    : *this->sTaskName := *sTaskNameToUse
    : this->iTaskOptions := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority := MUTEX_TASK_PRIORITY 
]]



[[D tDlCommandEngine::SetServoMode (int iServoMode)]
[ invalid_pointer (&this->Gizmo.Servo) => UNDEFINED
| InvalidIndex (this->Gizmo.Servo.Controller3DArray,iServoMode) => 
    return:= appropriate error code
| true => 
    this->Gizmo.Servo->ControllerSet.iCurrentControllerGroup:= iServoMode
    : return OK 
]]


[[D tDlCommandEngine::ReportError (ERROR_CODE ec) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send error to ErrorStream (::pThisRteG) => 
    console:= console + appropriate error messages
| true => 
    ::ErrorStream (::pThisRteG):= ErrorStream (::pThisRteG) + ErrorInfo with
        ecErrorCode, ::pThisRteG->tSchedTime, ::pThisRteG->iRteId
    : return:= OK 
]]


[[D tDlCommandEngine::UpdateState (DL_COMAND_STATUS dcsStatus) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->Gizmo.Status.dcsCommandStatus := dcsStatus
]]


--------------------tDlFrontCommandEngine-----------------------------------------------

[[D tDlCommandEngine::tDlCommandEngine (tMutex& MutexToUse,
					tDlGizmo& GizmoToUse,
					char* sTaskNameToUse) ]

[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := copy of *sTaskNameToUse if non-NULL, 
                                  else "MutexTask"
    : this->iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]] 

--------------------tDlOffCommandEngine-----------------------------------------------


[[D tDlOffCommandEngine::tDlOffCommandEngine (tMutex& MutexToUse,
					      tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlOff"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]] 


[[D tDlOffCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

--------------------tDlFrontCommandEngine-----------------------------------------------


[[D tDlFrontCommandEngine::tDlFrontCommandEngine (tMutex& MutexToUse,
					      tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlFront"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]


[[D tDlFrontCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_FRONT
    : Delay Line moves to front of track and gets there if not prematurely interupted
      by another command
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


[[D tDlFrontCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

--------------------tDlBackCommandEngine-----------------------------------------------

[[D tDlBackCommandEngine::tDlBackCommandEngine (tMutex& MutexToUse,
					      tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlBack"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]


[[D tDlBackCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_BACK
    : Delay Line moves to back of track and gets there if not prematurely interupted
      by another command
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


[[D tDlBackCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


--------------------tDlBackCommandEngine-----------------------------------------------


[[D tDlTrackCommandEngine::tDlTrackCommandEngine (tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlTrack"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]

[[D tDlTrackCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_TRACK
    : Delay Line moves to track appropriate target until a front or back soft limit
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


[[D tDlTrackCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]

--------------------tDlUser1CommandEngine-----------------------------------------------

[[D tDlUser1CommandEngine::tDlUser1CommandEngine (tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlUser1"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]

[[D tDlUser1CommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_USER1
    : Delay Line moves to track appropriate target until a front or back soft limit
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


[[D tDlUser1TrackCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]


--------------------tDlUser2CommandEngine-----------------------------------------------


[[D tDlUser2CommandEngine::tDlUser2CommandEngine (tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlUser2"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]


[[D tDlUser2CommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_USER2
    : Delay Line moves to track appropriate target until a front or back soft limit
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


[[D tDlUser2CommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]


--------------------tDlClearLaserCounterCommandEngine--------------------------------------


[[D tDlClearLaserCounterCommandEngine::tDlClearLaserCounterCommandEngine (
                                                  tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlClearLaserCounter"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]

[[D tDlClearLaserCounterCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line laser counter cleared 
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

--------------------tDlDitherCalibrateCommandEngine-----------------------------------


[[D tDlDitherCalibrateCommandEngine::tDlDitherCalibrateCommandEngine (
                                                  tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlDihterCalibrate"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]



[[D tDlDitherCalibrateCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line moves to front of track and gets there if not prematurely interupted by
      another command
      : dither waveform calibrated
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]

[[D tDlDitherCalibrateCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : dither waveform calibration := anything
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 


--------------------tDlDitherCheckCommandEngine-----------------------------------


[[D tDlDitherCheckCommandEngine::tDlDitherCheckCommandEngine (
                                                  tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlDitherCheck"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]

[[D tDlDitherCheckCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line moves to front of track and gets there if not prematurely interupted by
      another command
      : dither waveform measured and metrics calculated
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]

[[D tDlDitherCheckCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : dither waveform metrics := anything
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

--------------------tDlHomeSetCommandEngine-----------------------------------

[[D tDlHomeSetCommandEngine::tDlHomeSetCommandEngine (
                                                  tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlHomeSet"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]


[[D tDlHomeSetCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE ||
  Delay Line speed <= slew speed && monitor task cannot 
    detect limit switches ||
  hard limits don't lie outside the soft limits => UNDEFINED
| this->Gizmo.Limits.IsOnHomeSensor() never returns true =>
         return appropriate status and error message.
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line moves to home sensor and gets there if not prematurely interupted by
      another command
    : Delay Line laser counter cleared
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]

[[D tDlHomeSetCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

--------------------tDlHomeCheckCommandEngine-----------------------------------

[[D tDlHomeCheckCommandEngine::tDlHomeCheckCommandEngine (
                                                  tMutex& MutexToUse,
		                    	          tDlGizmo& GizmoToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (MutexToUse) ||
  invalid_reference (GizmoToUse) ||
  invalid_string (sTaskNameToUse) => UNDEFINED
| true =>
    this->Mutex := reference to MutexToUse
    : *this->sTaskName := "DlHomeCheck"
    : this_iTaskOptions     := MUTEX_TASK_OPTIONS
    : this->iTaskStackSize   := MUTEX_TASK_STACK_SIZE
    : this->iTaskPriority    := MUTEX_TASK_PRIORITY
    : this->Gizmo := GizmoToUse
]]

[[D tDlHomeCheckCommandEngine::ExecuteMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE ||
  Delay Line speed <= slew speed && monitor task cannot 
    detect limit switches ||
  hard limits don't lie outside the soft limits => UNDEFINED
| this->Gizmo.Limits.IsOnHomeSensor() never returns true =>
         return appropriate status and error message.
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line moves to home sensor and gets there if not prematurely interupted by
      another command
    : Delay Line laser counter cleared
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]]

[[D tDlHomeCheckCommandEngine::AbortMethod (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| true =>
    this->Gizmo.Servo.iCurrentMode := DL_OFF
    : Delay Line stops moving
    : this->Gizmo.Status := appropriate values
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate 
        messages
    : return := OK
]] 

****/


/*******************************************************************/
/****************                     ******************************/
/****************  Class Definitions  ******************************/
/****************                     ******************************/
/*******************************************************************/


class tDlGizmo;

class tDlCommandEngine : public tMutexUser {
  public:
    tDlCommandEngine (tDlGizmo& GizmoToUse);
    tDlCommandEngine (tMutex& MutexToUse, tDlGizmo& GizmoToUse,
                   char *sTaskNameToUse);

  protected:

    void UpdateState(DL_COMMAND_STATUS dcsState);
    void ReportError(ERROR_CODE ec); 
    ERROR_CODE SetServoMode (int iServoMode);
    
    tDlGizmo& Gizmo;
};


class tDlOffCommandEngine :  public tDlCommandEngine {
  public:
    tDlOffCommandEngine(tMutex& MutexToUse, tDlGizmo& Gizmo) :
                      tDlCommandEngine (MutexToUse, Gizmo, "DlOff") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
};

class tDlFrontCommandEngine :  public tDlCommandEngine {
  public:
    tDlFrontCommandEngine (tMutex& MutexToUse, tDlGizmo& Gizmo) :
                      tDlCommandEngine (MutexToUse, Gizmo, "DlFront") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlBackCommandEngine :  public tDlCommandEngine {
  public:
    tDlBackCommandEngine (tMutex& MutexToUse, tDlGizmo& Gizmo) :
                         tDlCommandEngine (MutexToUse, Gizmo, "DlBack") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlTrackCommandEngine :  public tDlCommandEngine {
  public:
    tDlTrackCommandEngine (tMutex& MutexToUse, tDlGizmo& Gizmo) :
                           tDlCommandEngine (MutexToUse, Gizmo, "DlTrack") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlUser1CommandEngine :  public tDlCommandEngine {
  public:
    tDlUser1CommandEngine(tMutex& MutexToUse, tDlGizmo& Gizmo) :
                        tDlCommandEngine (MutexToUse, Gizmo, "DlUser1") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlUser2CommandEngine :  public tDlCommandEngine {
  public:
    tDlUser2CommandEngine(tMutex& MutexToUse, tDlGizmo& Gizmo) :
                      tDlCommandEngine (MutexToUse, Gizmo, "DlUser2") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlClearLaserCounterCommandEngine :  public tDlCommandEngine {
  public:
    tDlClearLaserCounterCommandEngine(tMutex& MutexToUse, tDlGizmo &Gizmo) :
                        tDlCommandEngine(MutexToUse, Gizmo,
                                     "DlClearLaserCounter") {}
  protected:
    virtual ERROR_CODE ExecuteMethod(void);
};


class tDlDitherCalibrateCommandEngine :  public tDlCommandEngine {
  public:
    tDlDitherCalibrateCommandEngine (tMutex& MutexToUse, tDlGizmo& Gizmo) :
      tDlCommandEngine (MutexToUse, Gizmo, "DlDitherCalibrate") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};

class tDlDitherCheckCommandEngine :  public tDlCommandEngine {
  public:
    tDlDitherCheckCommandEngine (tMutex& MutexToUse, tDlGizmo& Gizmo) :
                 tDlCommandEngine (MutexToUse, Gizmo, "DlDitherCheck") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
};



class tDlHomeSetCommandEngine :  public tDlCommandEngine {
  public:
    tDlHomeSetCommandEngine(tMutex& MutexToUse, tDlGizmo& Gizmo) :
                        tDlCommandEngine (MutexToUse, Gizmo, "DlHomeSet") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
    ERROR_CODE CleanUp (void);
};



class tDlHomeCheckCommandEngine :  public tDlCommandEngine {
  public:
    tDlHomeCheckCommandEngine(tMutex& MutexToUse, tDlGizmo& Gizmo) :
         tDlCommandEngine (MutexToUse, Gizmo, "DlHomeCheck") {}
  protected:
    virtual ERROR_CODE ExecuteMethod (void);
    virtual ERROR_CODE AbortMethod (void);
    ERROR_CODE CleanUp (void);
};




#endif INC_dlCommandProcessorh


