/* $RCSfile: dlControllerData.h,v $ - DlController base class definitions */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dl/dlControllerData.h,v 1.2 1998/09/23 23:57:30 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$Log: dlControllerData.h,v $
Revision 1.2  1998/09/23 23:57:30  phil
Changes to data recording so FIFOs dont fill up

Revision 1.1.1.1  1998/06/11 21:36:15  chara
First import of RICST Code

Revision 1.3.2.1  1998/04/30 20:36:01  rbrill
commit before build_5a_008

Revision 1.3  1998/04/02 21:01:21  phil
Build I5a near complete; still some configMgr stuff left

Revision 1.2  1998/03/30 19:35:09  rich
*** empty log message ***

Revision 1.1  1998/03/11 00:59:31  rich
dl and gsuDl overhaul

*/

#ifndef INC_DlControllerDatah
#define INC_DlControllerDatah

#include "typeTag.h"
#include "dlTypes.h"
#include "dlDataStore.h"

/*******

[[D tDlControllerData::tDlControllerData (void) ]
Constructs this with default values.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dServoError := 0.0
    : this->dMotorVelocity := 0.0
]]

[[D tDlControllerData::~tDlControllerData (void) ]
Destructs this.
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tDlControllerData::ServoError (void) ]
Returns this->dServoError or reference to this->dServoError.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dServoError
| true => reference to this->dServoError
]]

[[D tDlControllerData::MotorVelocity (void) ]
Returns this->dMotorVelocity or reference to this->dMotorVelocity.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dMotorVelocity
| true => reference to this->dMotorVelocity
]]

[[D tDlControllerData::MotorServoError (void) ]
Returns this->MotorServoError or reference to this->dMotorServoError.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dMotorServoError
| true => reference to this->dMotorServoError
]]

[[D tDlControllerData::LaserJitter (void) ]
Returns this->dLaserJitter or reference to this->dLaserJitter.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dLaserJitter
| true => reference to this->dLaserJitter
]]

[[D tDlControllerData::LaserHpJitter (void) ]
Returns this->dLaserHpJitter or reference to this->dLaserHpJitter.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dLaserHpJitter
| true => reference to this->dLaserHpJitter
]]



[[D tDlControllerData::LinearCartError (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := return this->dServoError * this->CdLinearCartErrorPerLaserError
]]

**/


extern const int CiDlDataDumpPriority;
extern const int CiDlJitterHpBlockSize;


class tDlControllerData {
  public:
    tDlControllerData (const tTypeTag& DlDataTag, tDlStatus& DlStatus, double dLinearCartErrorPerLaserError);
    double ServoError () const { return this->dServoError;}
    double& ServoError () { return this->dServoError;}
    double MotorVelocity () const { return this->dMotorVelocity;}
    double& MotorVelocity () { return this->dMotorVelocity;}
    double& MotorServoError () { return this->dMotorServoError;}
    double MotorServoError () const { return this->dMotorServoError;}
    double LinearCartError () const { return this->dServoError * this->CdLinearCartErrorPerLaserError;}
    
  private:
    const  double CdLinearCartErrorPerLaserError;
    double dServoError;
    double dMotorVelocity;
    double dMotorServoError;
    
  public:   // temporary until new telemetry architecture
     tDlDataStore LaserDataStore;
   
 
};

#endif  /* INC_DlControllerDatah */
