/* $RCSfile: dlIoSet.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dl/dlIoSet.h,v 1.2 1998/08/17 21:01:18 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$Log: dlIoSet.h,v $
Revision 1.2  1998/08/17 21:01:18  phil
Laser Ignore, Laser board works, Modulation works

Revision 1.1.1.1  1998/06/11 21:36:15  chara
First import of RICST Code

Revision 1.8  1998/03/12 01:05:10  rich
delay line overhaul cont.

Revision 1.7  1998/03/11 00:58:11  rich
dl and gsuDl overhaul

*/
 
/*
DESCRIPTION

*/

#ifndef INC_dlIoSeth
#define INC_dlIoSeth


#include "inputSource.h"
#include "laserMetrologySensor.h"
#include "outputDest.h"
#include "targetSource.h"
#include "limits.h"
#include "dlTargGen.h"
#include "dlGizmoInfo.h"

/***********************

[[D tDlCompleteIoSet::tDlCompleteIoSet (const tDlGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) || 
  invalid_reference (GizmoInfo) ||
  invalid_pointer (GizmoInfo.pLaserChannel, GizmoInfo.pDitherPztChannel,
    GizmoInfo.pServoPztReadbackChannel, pMotorReadbackChannel,
    GizmoInfo.pDiagLaserInputChannel, pDiagPztPositionInputChannel,
    GizmoInfo.pFrontHardLimitChannel, GizmoInfo.pFrontSoftLimitChannel,
    GizmoInfo.pBackHardLimitChannel, GizmoInfo.pBackSoftLimitChannel,
    GizmoInfo.pHomeSensorChannel, GizmoInfo.pServoPztChannel, 
    GizmoInfo.pServoVc1Channel, GizmoInfo.pServoMotorChannel,
    GizmoInfo.pDiagLaserOutputChannel, GizmoInfo.pDiagVc1ServoErrorOutputChannel,
    GizmoInfo.pDiagMotorServoErrorOutputChannel) ||
  ::IpcMemory.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| Unabled to allocate necessary Ipc Objects or scheudule periodic tasks =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    this->Laser := laser metrology sensor connected to *GizmoInfo.pLaserChannel
      with disabled dither waveform connected to *GizmoInfo.pDitherPztChannel
      and with disabled modulation initialized to ZERO_MODULATION and other
      parameters specified by GizmoInfo
    : this->PztPosition := tModulatedSensor connected to 
      *GizmoInfo.pPztReadbackChannel with disabled modulation initialized to
      ZERO_MODULATION and other parameters specified by GizmoInfo
    : this->MotorPosition := tModulatedSensor connected to 
      *GizmoInfo.pMotorReadbackChannel with disabled modulation initialized to
      ZERO_MODULATION and other parameters specified by GizmoInfo
    : this->LaserDiagInput := tSensor connected to 
      *GizmoInfo.pDiagLaserInputChannel and initialized with other parameters 
      specified by GizmoInfo
    : this->PztPositionDiagInput := tSensor connected to 
      *GizmoInfo.pDiagPztPositionInputChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->DlLimits := tDlLimits connected to 
      *GizmoInfo.pFrontHardLimitChannel, *GizmoInfo.pFrontSoftLimitChannel,
      *GizmoInfo.pBackHardLimitChannel, *GizmoInfo.pBackSoftLimitChannel, and
      *GizmoInfo.pHomeSensorChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Pzt := tModulatedVelocityActuator connected to 
      *GizmoInfo.pServoPztChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Vc1 := tModulatedPositionActuator connected to 
      *GizmoInfo.pServoVc1Channel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Motor := tModulatedVelocityActuator connected to 
      *GizmoInfo.pServoMotorChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->LaserDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagLaserOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->ServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagLaserServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->Vc1ServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagVc1ServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->MotorServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagMotorServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->TargetGenerator := tDlTargGen connected to four IPC double buffers of 
      type tDlTarget reading these buffers with a period specified by GizmoInfo.
    : ::IpcMemory := ::IpcMemory U { 
        IpcObject (GizmoInfo.iiOpenLoopTargetBufferId),
        IpcObject (GizmoInfo.iiManualTargetBufferId),
        IpcObject (GizmoInfo.iiEmTargetBufferId),
        IpcObject (GizmoInfo.iiFtTargetBufferId) }
    : ::PeriodicTaskSet := ::PeriodicTaskSet {
         PeriodicTask ("Dl Target Generator",...),
         PeriodicTask ("Dl Limit Update", ...) }      
    : this->LaserTarget := tPositionTargetSource connected to tDlTargGen 
      connected to four IPC double buffers of type tDlTarget
    : this->PztPositionTarget := tZeroTargetSource with defalt characteristics
    : this->MotorTarget := tVelocityTargetSource connected to tDlTargGen 
      connected to four IPC double buffers of type tDlTarget
]]    
*/

 class tDlCompleteIoSet {
  public:
    tDlCompleteIoSet (const tDlGizmoInfo& GizmoInfo);
  
    // Sensors
    tLaserMetrologySensor    Laser;
    tModulatedSensor         PztPosition;
    tModulatedSensor         MotorPosition;
  
    // Diagnostic inputs for servo performance tuning
    tModulatedSensor         LaserDiagInput;
    tModulatedSensor         PztPositionDiagInput;
   
    // Limit and home sensors
    tDlLimits                DlLimits;
  
    // Actuators
    tModulatedVelocityActuator Pzt;
    tModulatedPositionActuator Vc1;
    tModulatedVelocityActuator Motor;

    // Diagnostic outputs for displaying sensor positions and servo errors
    tActuator LaserDiagOutput;
    tActuator ServoErrorDiagOutput;   
    tActuator Vc1ServoErrorDiagOutput;
    tActuator MotorServoErrorDiagOutput;

    // Target Sources
    tDlTargGen TargetGenerator;
    tPositionTargetSource LaserTarget;
    tZeroTargetSource PztPositionTarget;  
    tVelocityTargetSource MotorTarget;
};


#endif  /* INC_dlIoSeth */
