/* $RCSfile: dlMotorController.h,v $ -*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dl/dlMotorController.h,v 1.2 1998/09/13 19:43:48 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
05a,rlj,26Feb98    Updated Dl Controller Design
02a,beh,11May97    Completely new for increment 2
01a,beh,23Oct96    written
*/
 
/*
DESCRIPTION

*/



#ifndef INC_dlMotorControllerh
#define INC_dlMotorControllerh

#include "isit.h"
#include "dlTypes.h"
#include "controller.h"
#include "dlControllerData.h"
#include "dlIoSet.h"
#include "feedforward.h"


/*********** Specification Functions **************************************

BOOL MovePastLimit (tDlLimits Limits, double dMotorVelocity) :==
[ invalid_limits (Limits) => UNDEFINED
| (Limits.bFrontSoft == TRUE || Limits.bFrontHard == TRUE) &&
  dMotorVelocity > 0 => return := TRUE
| (Limits.bBackSoft == TRUE || Limits.bBackHard == TRUE) &&
  dMotorVelocity < 0 => return := TRUE
| true => return := FALSE
]]

double StoppingDistance (double dVelocity, double dAccleration, 
                         double dSamplePeriod) :==
Velocity in Unit/TimeUnit, Accel in Unit/TimeUnit^2, 
sample period in TimeUnit.  Distance in Unit.
[ return := Velocity^2 / (2 * ABS (dAccleration)) + 
            ABS (dVelocity) * dSamplePeriod / 2
] 





************************* tDlMotorController Specification ***********************  

[[D tDlMotorController::tDlMotorController (tDlControllerData& Data, tDlCompleteIoSet& IoSet, 
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses);
    ERROR_CODE SetControlParameters (const tDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->dMotorVelocity := reference to Data.dMotorVelocity
    : this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := InitialParameters.dMotorSamplePeriod
    : this->CdMotorErrorPerLaserError := dLaserPhysicalPerCount/ iLaserPasses
    : this->CdLaserError := reference to Data.dServoError
]]

[[D tDlMotorController::~tDlMotorController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tDlMotorController::SetControlParameters (const tDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true => 
    this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := NewParameters.dMotorSamplePeriod
    : return := OK
]]


[[D tDlMotorController::Enabled (void) ]
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bEnabled
| true => return := reference to this->bEnabled
]]

[[D tDlMotorController::MotorVelocity (void) ]
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) => UNDEFINED
| true => return := this->Motor.dValue
]]


[[D tDlMotorController::MotorSamplePeriod (void) ]
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) => UNDEFINED
| true => return := this->dMotorSamplePeriod
]]


[[D tDlMotorController::Init (TIME tServoTime) ]
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| true =>
    Motor device associated with this->Motor reset
    : this->Motor := ResetOutput (this->Motor)
    : return := OK
]]

[[D tDlMotorController::Reset (TIME tServoTime) ] = tDlMotorController::Init ]]

[[D tDlMotorController::Calculate (TIME tServoTime) ] = tController::Calculate ]]

******************* Protected Specification ********************************



[[D tDlMotorController::DecelerationDistance (void) ]
Returns the distance required to decelerate the motor moving at the current velocity
using the maximum deceleration rate allowed by the motor.  This calculation 
returns the stopping distance in motor units which are whatever units are specified in the
motor actuator.  It is assumed that the velocity (dValue) is given in motor units / second
and that the acceleration is in motor units / (second^2).
This calculation compensates for the fact that the motor velocity is sampled and commanded
at discrete intervals and not continuously.
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| true =>
    return := StoppingDistance (this->Motor.dValue, this->Motor.dMaxAcceleration, 
      this->dMotorSamplePeriod)
]]



[[D tDlMotorController::Actuate (double dMotorVelocity, TIME tServoTime) ]
Moves motor actuator at specified velocity in motor units / second.
If a limit switch is tripped and the desired velocity is in the same direction
as the limit, the motor velocity is set to zero.
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| MovePastLimit (this->Limits, dMotorVelocity) => 
    this->Motor.dValue := 0
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, 0, tServoTime)
    : this->dLinearCartError := this->CdLaserError * CdMotorErrorPerLaserError
    : return := OK
| true =>
    this->Motor := UpdateOutput (this->Motor, dMotorVelocity, tServoTime)
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, dMotorVelocity, tServoTime)
    : this->dLinearCartError := this->CdLaserError * CdMotorErrorPerLaserError
    : return := OK
]]



[[D tDlMotorController::MaxMotorAcceleration (void) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Motor) => UNDEFINED
| true => return := this->Motor.dMaxAcceleration
]]


[[D tDlMotorController::MotorPhysicalUnitsToMotorCounts (double dPhysicalUnit) ]
[ invalid_pointer (this) => UNDEFINED
| true =>  return := dPhysicalUnit / this->Motor.dPhysicalUnitsPerCount 
]]


[[D tDlMotorController::LaserErrorToMotorError (double dLaserError) ]
[ invalid_pointer (this) => UNDEFINED
| true =>  return := dLaserError * this->CdMotorErrorPerLaserError
]]

[[D tDlMotorController::LinearCartError (double dLaserError) ]
Returns current distance from target in linear cart meters
[ invalid_pointer (this) => UNDEFINED
| true =>  return := this->CdLaserError * this->CdMotorPhysicalPerLaserError
]]

********************* tDlMotorSlewController Specification *****************************

[[D tDlMotorSlewController::tDlMotorSlewController (tDlControllerData& Data, tDlCompleteIoSet& IoSet, 
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses) ]
[ invalid_pointer (this) ||
  invalid_reference (IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->dMotorVelocity := reference to Data.dMotorVelocity
    : this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := InitialParameters.dtMotorSamplePeriod
    : this->dMotorSlewVelocity := InitialParameters.dMotorSlewVelocity converted to channel counts
    : this->CdLaserError := reference to Data.dServoError
]]

[[D tDlMotorSlewController::~tDlMotorSlewController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tDlMotorSlewController::SetControlParameters (const tDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true => 
    this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := NewParameters.dMotorSamplePeriod
    : this->dMotorSlewVelocity := NewParameters.dMotorSlewVelocity converted to channel counts
    : return := OK
]]


[[D tDlMotorSlewController::Enabled (void) ] = tDlMotorController::Enabled ]]

[[D tDlMotorSlewController::Velocity (void) ] = tDlMotorController::Velocity ]]

[[D tDlMotorSlewController::Init (TIME tServoTime) ] = tDlMotorController::Init ]]

[[D tDlMotorSlewController::Reset (TIME tServoTime) ] = tDlMotorController::Reset ]]

[[D tDlMotorSlewController::DesiredVelocity (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dMotorSlewVelocity
]]

[[D tDlMotorSlewController::Calculate (TIME tServoTime) ]
Moves motor actuator at slew velocity in motor units / second.
If a limit switch is tripped and the desired velocity is in the same direction
as the limit, the motor velocity is set to zero.
Because the velocity of the motor may be limited it may take several servo periods for the
motor velocity to reach the slew velocity or the maximium motor velocity allowed.
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| MovePastLimit (this->Limits, this->dMotorSlewVelocity) => 
    this->Motor.dValue := 0
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, 0, tServoTime)
    : this->dLinearCartError := this->CdLaserError * CdMotorErrorPerLaserError
    : return := OK
| true =>
    this->Motor := UpdateOutput (this->Motor, this->dMotorSlewVelocity, tServoTime)
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, this->dMotorSlewVelocity, tServoTime)
    : return := OK
]]

***************** Protected Specification ***************************************

[[D tDlMotorSlewController::DecelerationDistance (void) ] = 
    tDlMotorController::DecelerationDistance ]]

[[D tDlMotorSlewController::Actuate (double dMotorVelocity, TIME tServoTime) ] = 
    tDlMotorController::Actuate ]]


[[D tDlMotorSlewController::LaserToMotorUnits (double dLaserUnits) ] = 
    tDlMotorController::LaserToMotorUnits ]]


[[D tDlMotorSlewController::MotorToPhysicalUnits (double dMotorUnits) ] = 
    tDlMotorController::MotorToPhysicalUnits ]]

[[D tDlMotorController::PhysicalToMotorUnits (double dPhysicalUnits) ]
[ invalid_pointer (this) => UNDEFINED
| true =>  return := dMotorUnits * MOTOR_UNIT_PER_PHYSICAL_UNIT
]]

[[D tDlMotorSlewController::MotorSamplePeriod (void) ] = 
    tDlMotorController::MotorSamplePeriod ]]

[[D tDlMotorSlewController::MaxMotorAcceleration (void) ] = 
    tDlMotorController::MaxMotorAcceleration ]]

******************* tDlMotorTrackSlewController Specification *************************

[[D tDlMotorTrackSlewController::tDlMotorTrackSlewController (tDlControllerData& Data,
      tDlCompleteIoSet& IoSet, const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses) ]
[ invalid_pointer (this) ||
  invalid_reference (ControllerData, IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->dMotorVelocity := referenct to Data.dMotorVelocity
    : this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := InitialParameters.dMotorSamplePeriod
    : this->dMotorSlewVelocity := InitialParameters.dMotorSlewVelocity
    : this->dDecelerationAggressiveness := InitialParameters.dDecelerationAggressiveness
    : this->CdLaserServoError := reference to Data.dServoError
]]

[[D tDlMotorTrackSlewController::~tDlMotorTrackSlewController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tDlMotorTrackSlewController::SetControlParameters (const tDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true => 
    this->bEnabled := InitialParameters.bEnabled
    : this->dMotorSamplePeriod := NewParameters.dMotorSamplePeriod
    : this->dMotorSlewVelocity := NewParameters.dMotorSlewVelocity
    : this->dDecelerationAggressiveness := NewParameters.dDecelerationAggressiveness
    : return := OK
]]


[[D tDlMotorTrackSlewController::Enabled (void) ] = tDlMotorSlewController::Enabled ]]

[[D tDlMotorTrackSlewController::Velocity (void) ] = tDlMotorSlewController::Velocity ]]

[[D tDlMotorTrackSlewController::Init (TIME tServoTime) ] = tDlMotorSlewController::Init ]]

[[D tDlMotorTrackSlewController::Reset (TIME tServoTime) ] = tDlMotorSlewController::Reset ]]

[[D tDlMotorTrackSlewController::DesiredVelocity (void) ] = tDlMotorSlewController::DesiredVelocity ]]


[[D tDlMotorTrackSlewController::Calculate (TIME tServoTime) ]
Moves motor actuator so that the OPD error goes to zero using a profiled velocity
based on the maximum motor velocity, the maximum motor acceleration and the 
decelearation aggressiveness specified. 
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| this->bEnabled == FALSE ||
  MovePastLimit (this->Limits, VelocityProfile (this->Motor, 
    this->dDecelerationAggressiveness, this->dLaserError) ) => 
    this->Motor.dValue := 0
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, 0, tServoTime)
    : return := OK
| true =>
    this->Motor := UpdateOutput (this->Motor, 
      VelocityProfile (this->Motor, this->dDecelerationAggressiveness, 
      this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT), 
      tServoTime)
    : motor actuator associated with this->Motor moves at velocity OutputValue (this->Motor, 
      VelocityProfile (this->Motor, this->dMotorSampleTime,
        this->dDecelerationAggressiveness, 
        this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT), 
      tServoTime)
    : return := OK
]]

********************* Protected Specification **************************************

[[D tDlMotorTrackSlewController::ProfiledVelocity (void) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Motor, this->dLaserError) ||
  invalid_actuator (this->Motor) => UNDEFINED 
| true => 
    return := VelocityProfile (this->Motor, this->dMotorSampleTime,
      this->dDecelerationAggressiveness, this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT)
]]


[[D tDlMotorTrackSlewController::DecelerationDistance (void) ] = 
    tDlMotorSlewController::DecelerationDistance ]]

[[D tDlMotorTrackSlewController::Actuate (double dMotorVelocity, TIME tServoTime) ] = 
    tDlMotorSlewController::Actuate ]]


[[D tDlMotorTrackSlewController::LaserToMotorUnits (double dLaserUnits) ] = 
    tDlMotorSlewController::LaserToMotorUnits ]]


[[D tDlMotorTrackSlewController::MotorToPhysicalUnits (double dMotorUnits) ] = 
    tDlMotorSlewController::MotorToPhysicalUnits ]]

[[D tDlMotorController::PhysicalToMotorUnits (double dPhysicalUnits) ]
[ invalid_pointer (this) => UNDEFINED
| true =>  return := dMotorUnits * MOTOR_UNIT_PER_PHYSICAL_UNIT
]]

[[D tDlMotorTrackSlewController::MotorSamplePeriod (void) ] = 
    tDlMotorSlewController::MotorSamplePeriod ]]

[[D tDlMotorTrackSlewController::MaxMotorAcceleration (void) ] = 
    tDlMotorSlewController::MaxMotorAcceleration ]]


**************/     
            

 
class tDlMotorController : public tController {
  public:
    tDlMotorController (tDlControllerData& Data, tDlCompleteIoSet& IoSet, 
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses);
    ERROR_CODE SetControlParameters (const tDlMotorParameters& NewParameters);
    BOOL Enabled () const { return this->bEnabled;}
    BOOL& Enabled () {  return this->bEnabled;}
    double MotorVelocity () const;
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Reset (TIME tServoTime);

  protected:
    double DecelerationDistance () const;
    ERROR_CODE Actuate (double dMotorVelocity, TIME tServoTime);
    double MotorSamplePeriod () const {return this->dMotorSamplePeriod;}
    double MaxMotorAcceleration () const;
    double MaxMotorVelocity () const { return this->Motor.MaxVelocity ();}
    double LinearCartError () const { return this->CdLaserError * this->CdMotorErrorPerLaserError; }
    double MotorPhysicalUnitsToMotorCounts (double dPhysicalUnit) const { return this->Motor.PhysicalUnitsToCounts (dPhysicalUnit); }
    double LaserCountsToMotorPhysical (double dCounts) const 
        { return dCounts * this->CdMotorErrorPerLaserError; }

    tFeedforward MotorFeedforward;
    tTargetSource &MotorTarget;

 private:
    tDlLimits& Limits;
    double dMotorSamplePeriod;
    tModulatedVelocityActuator &Motor;
    double& dMotorVelocity;
    BOOL bEnabled;
    const double CdMotorErrorPerLaserError;   
    const double& CdLaserError; // in laser counts
 };


class tDlMotorSlewController : public tDlMotorController {
  public:
    tDlMotorSlewController (tDlControllerData& Data, tDlCompleteIoSet& IoSet, 
      const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses);
    ERROR_CODE SetControlParameters (const tDlMotorParameters& NewParameters);
    double DesiredVelocity () const {return this->dMotorSlewVelocity;}
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Calculate (TIME tServoTime);
  private:
    double dMotorSlewVelocity;
};


class tDlMotorTrackSlewController : public tDlMotorSlewController {
  public:
    tDlMotorTrackSlewController (tDlControllerData& Data,
      tDlCompleteIoSet& IoSet, const tDlMotorParameters& InitialParameters, double dLaserPhysicalPerCount, int iLaserPasses);
    ERROR_CODE SetControlParameters (const tDlMotorParameters& NewParameters);
    ERROR_CODE Calculate (TIME tServoTime);
  private:
    double ProfiledVelocity () const;
    
    double dDecelerationAggressiveness;
};





#endif INC_dlMotorControllerh
