/* $RCSfile: dlPztController.h,v $*/
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/dl/dlPztController.h,v 1.3 1998/09/23 23:57:31 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
05a,29Jan98,rlj   written
*/
 
/*
DESCRIPTION
  Delay Line PZT Controllers.
*/
  
#ifndef INC_dlPztControllerh
#define INC_dlPztControllerh 

#include "isit.h"
#include "controller.h"
#include "lagLib.h"
#include "dlControllerData.h"
#include "dlIoSet.h"




/*********** Specification Functions ****************************************

ChannelValue (tChannel) - see channel.h (someday)
Target (tTargetSource) - see TargetSource.h (someday)

double MODULATION (tModulationParams,TIME) - see modLib.h 
double DITHER (tDitherParams,TIME) - see dither.h (someday)
double LAG (tLagCompensator,double) - see lagLib.h 
tLagCompensator UpdateLag (tLagCompensator, double ) - see lagLib.h 



 
double CurrentLaserPositionError (tSensor LaserInput, 
                                  tSensor LaserDiagInput,
                                  tTargetSource LaserTarget, 
                                  TIME tTime) :==
[ invalid_sensor (LaserInput, LaserDiagInput) ||
  invalid_target (LaserTarget) => UNDEFINED
| true => return := InputValue (LaserInput, tTime) - 
   TargetValue (LaserTarget, tTime) + InputValue (LaserDiagInput, tTime)
]

double PztTrackVelocity (BOOL bEnabled, BOOL bLagEnabled, double dGain,
                         tLagCompensator Lag, double dServoError) :==
Computes the pzt velocity to compensate for the given servo error using
a lag compensator if enabled.                        
[ bEnabled == FALSE => return := 0.0
| bLagEnabled = FALSE => return := dGain * dServoError]
| true => return := dGain * LAG (Lag, dServoError)
] 

tLagCompensator UpdatePztLag (tLagCompensator Lag, BOOL bEnabled, BOOL bLagEnabled, double dError) :==
[ bEnabled == FALSE || bLagEnabled == FALSE => return Lag
| true => return => UpdateLag (Lag, dError)
]




********************** tPztController Specification ***************************
 
[[D tDlPztController::tDlPztController (tDlControllerData& ControllerData, tDlCompleteIoSet& IoSet) ]
Constructs this assigning appropriate references to data in ControllerData and I/O in IoSet.
[ invalid_pointer (this) ||
  invalid_reference (ControllerData, IoSet) => UNDEFINED
| true =>
    this->bEnable := TRUE
    : this->LaserInput := reference to IoSet.LaserInput
    : this->LaserDiagInput := reference to IoSet.LaserDiagInput
    : this->Pzt := reference to IoSet.Pzt
    : this->LaserDiagOutput := reference to IoSet.LaserDiagOutput
    : this->LaserTarget := reference to IoSet.LaserTarget
    : this->dServoError := reference to ControllerData.dServoError
]]

[[D tDlPztController::~tDlPztController (void) ]
Destroys this.
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything    
]]

[[D tDlPztController::Enabled (void) ]
Returns copy or reference to the enabled state of the controller.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bEnabled
| true => return := reference to this->bEnabled
]]


[[D tDlPztController::Init (TIME tServoTime) ]
Resets pzt device.
[ invalid_pointer (this) ||
  invalid_reference (this->Pzt) ||
  invalid_actuator (this->Pzt) => UNDEFINED
| true => 
    Pzt device associated with this->Pzt reset
    : this->Pzt := ResetOutput (this->Pzt)
    : return := OK
]]

[[D tDlPztController::Reset (TIME tServoTime) ] = tDlPztController::Init ]]


[[D tDlPztController::Calculate (TIME tServoTime) ]
Computes and stores OPD servo error in laser units. Sends servo error to diagnostic output.
[ invalid_pointer (this) ||
  invalid_reference (this->LaserInput, this->LaserInput,
    this->LaserDiagInput, this->Pzt, this->ServoErrorDiagOutput, 
    this->LaserTarget, this->dServoError) ||
  invalid_sensor (this->LaserInput, this->LaserDiagInput) ||
  invalid_target (this->LaserTarget) ||
  invalid_actuator (this->ServoErrorDiagOutput) => UNDEFINED
| true =>
    this->LaserInput := UpdateInput (this->LaserInput, tServoTime)
    : this->LaserDiagInput := UpdateInput (this->LaserDiagInput, tServoTime)
    : this->LaserTarget := UpdateInput (this->LaserTarget, tServoTime)
    : this->ServoErrorDiagOutput := UpdateOutput (this->ServoErrorDiagOutput,
      CurrentLaserPositionError (this->LaserInput, this->LaserDiagInput, 
        this->LaserTarget, tServoTime), tServoTime)
    : Diagnostic device associated with this->ServoErrorDiagOutput receives  
      CurrentLaserPositionError (this->LaserInput, 
        this->LaserDiagInput, this->LaserTarget, tServoTime)
    : this->dServoError := CurrentLaserPositionError (this->LaserInput, 
      this->LaserDiagInput, this->LaserTarget, tServoTime)
    : return := OK
]]

*********************** Protected Stimuili ***********************************

[[D tDlPztController::ServoError (TIME tServoTime) ]
Computes and returns OPD servo error in laser units.
[ invalid_pointer (this) ||
  invalid_reference (this->LaserInput, this->LaserInput,
    this->LaserDiagInput, this->Pzt, this->ServoErrorDiagOutput, 
    this->LaserTarget, this->dServoError) ||
  invalid_sensor (this->LaserInput, this->LaserDiagInput) ||
  invalid_target (this->LaserTarget) => UNDEFINED
| true =>
    this->LaserInput := UpdateInput (this->LaserInput, tServoTime)
    : this->LaserDiagInput := UpdateInput (this->LaserDiagInput, tServoTime)
    : this->LaserTarget := UpdateInput (this->LaserTarget, tServoTime)
    : this->ServoErrorDiagOutput := UpdateOutput (this->ServoErrorDiagOutput,
      CurrentLaserPositionError (this->LaserInput, this->LaserDiagInput, 
        this->LaserTarget, tServoTime), tServoTime)
    : Diagnostic device associated with this->ServoErrorDiagOutput receives  
      CurrentLaserPositionError (this->LaserInput, 
        this->LaserDiagInput, this->LaserTarget, tServoTime)
    : this->dServoError := CurrentLaserPositionError (this->LaserInput, 
      this->LaserDiagInput, this->LaserTarget, tServoTime)
    : return := CurrentLaserPositionError (this->LaserInput, this->LaserDiagInput, 
      this->LaserTarget, tServoTime)
]]


[[D tDlPztController::Actuate (double dPztOutput, TIME tTime) ]
Moves pzt at a rate of dPztVelocity.
[ invalid_pointer (this) ||
  invalid_reference (this->Pzt) ||
  invalid_actuator (this->Pzt) => UNDEFINED
| true =>
    Pzt device associated with this->Pzt actuated with value
      OutputValue (this->Pzt, dPztOutput, tTime)
    : this->Pzt := UpdateOutput (this->Pzt, dPztOutput, tTime)
    : return := OK
]] 


[[D tDlPztController::LaserData (tServoTime) ]
Collects laser data.
[ invalid_pointer (this) ||
  invalid_reference (this->LaserInput) ||
  invalid_sensor (this->LaserInput) => UNDEFINED
| true =>
     return tDlData with laLaser set to this->LaserInput.dValue, laError set to this->dServoError, and
     tTimeStamp set to tServoTime
]]


********************** tDlPztTrackController Stimuli ***********************

[[D tDlPztTrackController::tDlPztTrackController (tDlControllerData& ControllerData,
      tDlCompleteIoSet& IoSet, const tDlPztParameters& InitialParameters) ]
Constructs this assigning appropriate references to I/O in IoSet.
[ invalid_pointer (this) ||
  invalid_reference (IoSet, InitialParameters) => UNDEFINED
| true =>
    this->bEnable := TRUE
    : this->bLagEnable := TRUE
    : this->LaserInput := reference to IoSet.LaserInput
    : this->LaserDiagInput := reference to IoSet.LaserDiagInput
    : this->Pzt := reference to IoSet.Pzt
    : this->LaserDiagOutput := reference to IoSet.LaserDiagOutput
    : this->LaserTarget := reference to IoSet.LaserTarget
    : this->dPztGain := InitialParameters.dPztGain
    : this->PztLag := lag compensator with appropriate zero's and poles 
      specified in InitialParameters
]]

[[D tDlPztTrackController::~tDlPztTrackController (void) ]
Destructs this.
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything    
]]


[[D tDlPztTrackController::Enabled (void) ] = tDlPztController::Enabled ]]

[[D tDlPztTrackController::ServoError (TIME tTime) ] =
    tDlPztController::ServoError ]]

[[D tDlPztTrackController::Actuate (double dPztVelocity, TIME tTime) ] =
    tDlPztController::Actuate ]]

[[D tDlPztTrackController::LaserData (tServoTime) ] = tDlPztController::LaserData ]]



[[D tDlPztTrackController::SetControlParameters (
      const tDlPztParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true =>
   this->bEnabled := NewParameters.bEnabled
   : this->bLagEnabled := NewParameters.bLagEnabled
   : this->dPztGain := NewParameters.dPztGain
   : this->LagCompensator := lag compensator with parameters specified in
     NewParameters
   : return := OK
]]   


[[D tDlPztTrackController::ControlParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return tDlPztParameters with current gain and lag parameters
   this->dPztGain := NewParameters.dPztGain
]]  


[[D tDlPztController::LagEnabled (void) ]
Returns copy or reference to the enabled state of the lag compensator.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bLagEnabled
| true => return := reference to this->bLagEnabled
]]



[[D tDlPztTrackController::Gain (void)]
Returns copy or reference to the gain of the controller.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->dGain
| true => return := reference to this->dGain
]]


[[D tDlPztTrackController::LagCompensator (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->PztLag
]]


[[D tDlPztTrackController::Init (TIME tServoTime) ]
Zeros Pzt velocity and postion and resets lag compensator memory.
[ invalid_pointer (this) ||
  invalid_reference (this->Pzt) ||
  invalid_actuator (this->Pzt) => UNDEFINED
| true => 
    Pzt device associated with this->Pzt reset
    : this->Pzt := ResetOutput (this->Pzt)
    : this->PztLag := lag compensator reset to 0.0
    : return := OK
]]

[[D tDlPztTrackController::Reset (TIME tServoTime) ] = tDlPztTrackController::Init ]]


[[D tDlPztTrackController::Calculate (TIME tServoTime) ]
Computes OPD servo error in laser units and compensates for error by
actuating a PZT to move at a computed velocity.
[ invalid_pointer (this) ||
  invalid_reference (this->LaserInput, this->LaserDiagInput, this->Pzt, 
    this->ServoErrorDiagOutput, this->LaserTarget, this->Pzt) ||
  invalid_sensor (this->LaserInput, this->LaserDiagInput)
  invalid_actuator (this->Pztl) => UNDEFINED
| true => 
    this->LaserInput := UpdateInput (this->LaserInput, tServoTime)
    : this->LaserDiagInput := UpdateInput (this->LaserDiagInput, tServoTime)
    : this->LaserTarget := UpdateInput (this->LaserTarget, tServoTime)
    : this->ServoErrorDiagOutput := UpdateOutput (this->ServoErrorDiagOutput,
      CurrentLaserPositionError (this->LaserInput, this->LaserDiagInput, 
        this->LaserTarget, tServoTime), tServoTime)
    : Diagnostic device associated with this->ServoErrorDiagOutput receives  
      CurrentLaserPositionError (this->LaserInput, 
        this->LaserDiagInput, this->LaserTarget, tServoTime)
    : this->dServoError := CurrentLaserPositionError (this->LaserInput, 
      this->LaserDiagInput, this->LaserTarget, tServoTime)
    : this->PztLag := UpdatePztLag (this->PztLag, this->bEnabled, this->bLagEnabled,
        CurrentLaserPositionError (this->LaserInput, this->LaserDiagInput, 
        this->LaserTarget, tServoTime) )
    : Pzt device associated with this->Pzt actuated with PztTrackVelocity (
        this->bEnabled, this->bLagEnabled, this->dPztGain, this->PztLag,
        CurrentLaserPositionError (this->LaserInput, 
          this->LaserDiagInput, this->LaserTarget, tServoTime))
    : return := OK
]]




******/





class tDlPztController : public tController {
  public:
    tDlPztController (tDlControllerData& ControllerData, tDlCompleteIoSet& IoSet);
    BOOL Enabled () const { return this->bEnabled;}
    BOOL& Enabled () { return this->bEnabled;}
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Reset (TIME tServoTime);
    ERROR_CODE Calculate (TIME tServoTime);

  protected:
    double ServoError (TIME tServoTime);
    ERROR_CODE Actuate (TIME tTime, double dPztVelocity);
    void LaserData (TIME tServoTime);
    tDlData DlData[2];
    int iCurrentWriteBuffer;

  private:
    BOOL bEnabled;
    tModulatedSensor& LaserInput;
    tModulatedSensor& LaserDiagInput; 
    tActuator& Pzt;
    tActuator& LaserDiagOutput;
    tActuator& ServoErrorDiagOutput; 
    tTargetSource& LaserTarget;
    double& dServoError;
};

class tDlPztTrackController : public tDlPztController {
  public:
    tDlPztTrackController (tDlControllerData& ControllerData, tDlCompleteIoSet& IoSet, 
                           const tDlPztParameters& InitialParameters);
    ERROR_CODE SetControlParameters (const tDlPztParameters& NewParameters);
    tDlPztParameters ControlParameters () const;
    BOOL LagEnabled () const { return this->bLagEnabled;}
    BOOL& LagEnabled () { return this->bLagEnabled;}
    double Gain () const { return this->dPztGain;}
    double& Gain () { return this->dPztGain;}
    const tLagCompensator& LagCompensator () const;
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Reset (TIME tServoTime);
    ERROR_CODE Calculate (TIME tServoTime);
  private:
    BOOL bLagEnabled;
    double dPztGain;
    tLagCompensator PztLag;
    
    tDlDataStore& LaserDataStore; // temporary until new telemetry architecture
    
};    

#endif INC_dlPztControllerh
