/* $RCSfile: limits.h,v $ - dlServo inmplementation */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dl/limits.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
02a,beh,12May97    incr 2 version
01a,beh,20Oct96    written
*/
 
/*
DESCRIPTION

*/

#ifndef INC_dlLimitsh
#define INC_dlLimitsh

#include "isit.h"
#include "inputChannel.h"
#include "inputSource.h"


/**************** Specification Functions *********************************

double InputValue (tInputSource, TIME) - see inputSource.h
double UpdateInput (tInputSource, TIME) - see inputSource.h

***************** tDlLimits Specification *********************************
[[D tDlLimits::tDlLimits (tInputChannel& FrontHard, double dFrontHardScale,
  tInputChannel& FrontSoft, double dFrontSoftScale,
  tInputChannel& BackHard, double dBackHardScale,
  tInputChannel& BackSoft, double dBackSoftScale,
  tInputChannel& Home, double dHomeScale)]
Contructs tDlLimits initializing FrontHard, FrontSoft, BackHard, BackSoft Limits
and the Home sensors with specified channels and scale factors.
Adds periodic task to monitor and update the limit switch booleans cached by
tDlLimits.
[ invalid_pointer (this) ||
  invalid_reference (FrontHard, FrontSoft, BackHard, BackSoft, Home) ||
  invalid_channel (FrontHard, FrontSoft, BackHard, BackSoft, Home) ||
  SchedulerOk (::Scheduler) == FALSE => UNDEFINED
| true =>
    this->bInit := TRUE
    : this->bFrontHardLimit := FALSE
    : this->bFrontSoftLimit := FALSE
    : this->bBackSoftLimit := FALSE
    : this->bBackHardLimit := FALSE
    : this->bHome := FALSE
    : this->FrontHardLimit := sensor connected to channel *pFrontHard with
      scale factor dFrontHardScale
    : this->FrontSoftLimit := sensor connected to channel *pFrontSoft with
      scale factor dFrontSoftScale
    : this->FrontHardLimit := sensor connected to channel *pBackHard with
      scale factor dBackHardScale
    : this->FrontHardLimit := sensor connected to channel *pBackSoft with
      scale factor dBackSoftScale
    : this->FrontHardLimit := sensor connected to channel *pHome with
      scale factor dHomeScale
    : ::PeriodicTaskSet := ::PeriodicTaskSet U { 
      PeriodicTask ("ReadLimitsTask", tDlLimits::UpdateCachedLimitValues,
                     NULL, NULL, DL_READ_LIMIT_TASK_PRIORITY,
                     DL_READ_LIMIT_TASK_PERIOD, DL_READ_LIMIT_TASK_OFFSET,
                     TEB_DISABLE, TRUE)
    : this->iUpdateCachedLimitsTaskId := new periodic task id
]]

[[D tDlLimits::~tDlLimits (void)]
Destroys tDlLimits and deletes perioid limit update task
[ invalid_pointer (this) ||
  SchedulerOkay (::Scheduler) == FALSE => UNDEFINED
| true =>
    ::PeriodicTaskSet := ::PeriodicTaskSet - { 
      PeriodicTask (this->iUpdateCachedLimitsTaskId)
    : *this := anything  
]]

[[D tDlLimits::IsOnFrontSoftLimit (void) ]
Checks if front soft limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bFrontSoftLimit
]]

[[D tDlLimits::IsOnFrontHardLimit (void) ]
Checks if front hard limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bFrontHardLimit
]]


[[D tDlLimits::IsOnBackSoftLimit (void) ]
Checks if back soft limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bBackSoftLimit
]]

[[D tDlLimits::IsOnBackHardLimit (void) ]
Checks if back hard limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bBackHardLimit
]]

[[D tDlLimits::IsOnAnyLimit (void) ]
Checks if any limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bFrontSoftLimit || this->bFrontHardLimit ||
                     this->bBackSoftLimit || this->bBackHardLimit
]]

[[D tDlLimits::IsOnAnyFrontLimit (void) ]
Checks if front soft or hard limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bFrontSoftLimit || this->bFrontHardLimit
]]

[[D tDlLimits::IsOnAnyBackLimit (void) ]
Checks if back soft or hard limit has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bBackSoftLimit || this->bBackHardLimit
]]

[[D tDlLimits::IsOnHomeSensor (void) ]
Checks if home sensor has been tripped assuming that the 
limit update task is enabled and running without error
[ invalid_pointer (this) => UNDEFINED
| true => return  := this->bHome
]]

***************** Private Specification ************************************

[[D tDlLimits::UpdateCachedLimitValues (TIME tTimeNow) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bFrontHardLimit := InputValue (this->FrontHardLimit, tTimeNow)
    : this->bFrontSoftLimit := InputValue (this->FrontSoftLimit, tTimeNow)
    : this->bBackHardLimit := InputValue (this->BackHardLimit, tTimeNow)
    : this->bBackSoftLimit := InputValue (this->BackSoftLimit, tTimeNow)
    : this->bHome := InputValue (this->HomeSensor, tTimeNow)
    : this->FrontHardLimit := UpdateInput (this->FrontHardLimit, tTimeNow) 
    : this->FrontSoftLimit := UpdateInput (this->FrontSoftLimit, tTimeNow) 
    : this->BackHardLimit := UpdateInput (this->BackHardLimit, tTimeNow) 
    : this->BacktSoftLimit := UpdateInput (this->BackSoftLimit, tTimeNow) 
    : this->HomeSensor := UpdateInput (this->HomeSensor, tTimeNow) 
    : return := OK
]]

********/

class tDlLimits {
  public:
    tDlLimits (tInputChannel& FrontHard, double dFrontHardScale,
               tInputChannel& FrontSoft, double dFrontSoftScale,
               tInputChannel& BackHard, double dBackHardScale,
               tInputChannel& BackSoft, double dBackSoftScale,
               tInputChannel& Home, double dHomeScale);
    ~tDlLimits ();
    BOOL IsOnFrontHardLimit () { return this->bFrontHardLimit;}
    BOOL IsOnFrontSoftLimit () { return this->bFrontSoftLimit;}
    BOOL IsOnBackHardLimit () { return this->bBackHardLimit;}
    BOOL IsOnBackSoftLimit () { return this->bBackSoftLimit;}
    BOOL IsOnAnyLimit () { 
      return ( this->bFrontHardLimit || this->bFrontSoftLimit ||
               this->bBackHardLimit || this->bBackSoftLimit);
    }
    BOOL IsOnAnyFrontLimit ()  { 
      return ( this->bFrontHardLimit || this->bFrontSoftLimit );
    }
    BOOL IsOnAnyBackLimit ()  { 
      return (this->bBackHardLimit || this->bBackSoftLimit);
    }
    BOOL IsOnHomeSensor () { return this->bHome;}

  private:
    ERROR_CODE UpdateCachedLimitValues (TIME tTimeNow);

    tSensor FrontHardLimit;
    tSensor FrontSoftLimit;
    tSensor BackHardLimit;
    tSensor BackSoftLimit;
    tSensor HomeSensor;

    BOOL bFrontHardLimit;
    BOOL bFrontSoftLimit;
    BOOL bBackHardLimit;
    BOOL bBackSoftLimit;
    BOOL bHome;

    int iUpdateCachedLimitsTaskId;
};

#endif  /* INC_dlLimitsh */
