/* $RCSfile: dlActuator.h,v $  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dlActuator.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,2Sept97      changed to dlActuator from dlTargetActuator
01a,rlj,18,Apr,97    written
*/

#ifndef INC_dlActuatorh
#define INC_dlActuatorh

#include "isit.h"
#include "outputDest.h"
#include "ipcDblBufferAccessor.h"

/*****
Black Box State Variables
______________________________

tDlTarget  Target
tIpcDblBufferAccessor<tDlTarget> TargetBuffer

______________________________________________________________-
Stimuli

[[D tDlActuator::tDlActuator (const tIpcDblBufferId& iiId) ]
[ invalid_pointer (this) || 
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->TargetBuffer.bObjectValid := FALSE
    : this->TargetBuffer.iiDblBufferId := anything
    : this->TargetBuffer.bInit := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->TargetBuffer.ecInitError := OK
    : this->TargetBuffer.bInit := TRUE
    : this->TargetBuffer.bObjectValid := TRUE
    : this->TargetBuffer.iiDblBufferId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]


[[D tDlActuator::~tDlActuator (void) ]
[ this is invalid => UNDEFINED
| true => * this := anything
]]

[[D tDlActuator::SetTargetValue (const tDlTarget& NewTarget) ]
[ invalid_pointer (this) || 
  invalid_reference (Target) => UNDEFINED
| true => this->Target := NewTarget
]] 


[[D tDlActuator::Output (TIME tUpdateTime) ]
[ invalid_pointer (this) || 
  this->TargetBuffer.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->TargetBuffer.iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->TargetBuffer.iiDblBufferId) => UNDEFINED
| true => 
    GetObject (::IpcMemoryG, this->TargetBuffer.iiBufferId)->dtBuffer := Target
    : return := OK 
]] 



[[D tDlActuator::OutputValue (TIME tUpdateTime, const tDlTarget& NewTarget) ]
[ invalid_pointer (this) || 
  this->TargetBuffer.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->TargetBuffer.iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->TargetBuffer.iiDblBufferId) => UNDEFINED
| true => 
    this->Target := NewTarget
    : GetObject (::IpcMemoryG, this->TargetBuffer.iiBufferId)->dtBuffer :=
      NewTarget
    : return := OK 
]] 

[[D tDlActuator::Reset (void) ]
[ invalid_pointer (this) || 
  this->TargetBuffer.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->TargetBuffer.iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->TargetBuffer.iiDblBufferId) => UNDEFINED
| true =>
    this->Target := ZERO_TARGET 
    : GetObject (::IpcMemoryG, this->TargetBuffer.iiBufferId)->dtBuffer := 
      ZERO_TARGET
    : return := OK 
]] 
*/   

class tDlActuator : public tOutputDestination {
  public:
    tDlActuator (const tIpcDblBufferId& iiId);
    virtual ~tDlActuator () {}
    void SetTargetValue (const tDlTarget& NewTarget);
    virtual ERROR_CODE Output (TIME tUpdateTime);
    virtual ERROR_CODE OutputValue (TIME tUpdateTime, const tDlTarget& NewTarget);
    virtual ERROR_CODE Reset();
  private:
    tDlTarget Target;
    tIpcDblBufferAccessor<tDlTarget> TargetBuffer;
};

#endif INC_dlActuatorh
