/* $RCSfile $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/dlGizmo.h,v 1.2 1998/09/23 23:57:22 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,24Sep97,rlj	     converted to new architecture
01a,31Oct96,beh    written
*/
 
/*
DESCRIPTION
  Delay line Gizmo defines and types
*/
   

#ifndef INC_dlGizmoh
#define INC_dlGizmoh


#include "isit.h"
#include "realTimeGizmo.h"
#include "typeTag.h"
#include "mutex.h"

#include "dlTypes.h"
#include "dlGizmoInfo.h"
#include "dl/dlIoSet.h"


/***************

[[D tDlGizmo::tDlGizmo (const tDlGizmoInfo& GizmoInfo)]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 8 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 3 ||
  CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, GizmoInfo.OpenLoopTargetId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, GizmoInfo.ManualTargetId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, GizmoInfo.FtTargetId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, GizmoInfo.EmTargetId) == FALSE ||
  Unable to add periodic task for target generation ||
  Unable to spawn data task =>
    this->ecInitError := appropriate error code
    : *this except this->ecInitError := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := FALSE
    : this->Servo.ControllerArrayArray := appropriate controllers from
       CompleteControllerSet
    : this->Servo.Mode := DL_OFF
    : this->Status := anything
    : this->Data := anything
    : this->pLimits = &CompleteIoSet.Limits
    : this->pFiducialChannel = &CompleteIoSet.Laser
    : this->DlDither.bInit := TRUE
    : this->DlTargGen.bInit := TRUE
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
       NewObjectSet (::IpcMemoryG,GizmoInfo.iiCommandStreamId) U
       NewObjectSet (::IpcMemoryG,GizmoInfo.OpenLoopTargetId) U
       NewObjectSet (::IpcMemoryG,GizmoInfo.ManualTargetId) U
       NewObjectSet (::IpcMemoryG,GizmoInfo.FtTargetId) U
       NewObjectSet (::IpcMemoryG,GizmoInfo.EmTargetId) 
    : ::PeriodicTaskSet := ::PeriodicTaskSet U PeriodicTask ("TargGenTask",
      TARG_GEN_TASK_PRIORITY,TARG_GEN_TASK_PERIOD)
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet U 
      EventDrivenTask ("DataStoreDump",CiDlDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, 
        tDlDataStore::SendToDataStream )
	: this->tidCommandTaskId := anything
	: this->tidStatusTaskId := anything
	: this->tidServoTaskIdArray [0..
          GizmoInfo.iNumberOfServoPartitions-1] := anything
]]  



[[D tDlGizmo::~tDlGizmo (void)]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tDlGizmo::Restart (void)]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tDlGizmo::Shutdown (void)]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tDlGizmo::IsOk (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := (this->ecInitError == OK)
]]

[[D tDlGizmo::operator_extractor (ostream& stream, cosnt tDlGizmo& Gizmo)]
[ true => UNDEFINED
]]

******************* Protected Specification **********************************

[[D tDlGizmo::CollectStatus (void)]
[ invalid_pointer (this) => UNDEFINED
| true => Common delay line status collected
]]

[[D tDlGizmo::Dispatch (const tIsitCmd& Command)]
[ invalid_pointer (this) ||
  invalid_reference (Command) => UNDEFINED
| Command.Command not in { DL_CMD_SELECT_TARGET, DL_CMD_DITHER_ON,
    INST_CMD_PING, INST_CMD_SET_OPEN_LOOP_TARGET, INST_CMD_SET_MANUAL_TARGET,
    DL_CMD_OFF, DL_CMD_FRONT, DL_CMD_BACK, DL_CMD_TRACK, DL_CMD_USER1,
    DL_CMD_USER2, DL_CMD_HOME_CHECK, DL_CMD_HOME_SET, DL_CMD_DITHER_CALIBRATE,
    DL_CMD_DITHER_CHECK, DL_CMD_PING, DL_CMD_CLR_LASER_CTR, DL_CMD_MODULATE,
    DL_CMD_SET_DITHER } => 
      return := appropriate error code
| true => 
    appropriate delay line command executed
    : return := OK
]]

[[D tDlGizmo::GetCompleteIoSet (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to appropriate delay line CompleteIoSet
]]

[[D tDlGizmo::GetCompleteControllerSet (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to appropriate delay line CompleteControllerSet
]]

[[D tDlGizmo::GetStatus (void)]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to appropriate delay line Status
]]

[[D tDlGizmo::StartTasks (const tDlGizmoInfo& GizmoInfo)]
[ invalid_pointer (this) ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| Unable to start command processing, status processing, or servo tasks =>
    return := appropriate error code
| true =>  
    ::PeriodicTaskSet := ::PeriodicTaskSet U 
      PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
        tGizmo::ProcessStatus) U
      PeriodicTask (GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[0].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[0].iTaskPriority, 0,
        tServo::DoServo) U
      appropriate dither task U
      appropriate target gen task
    : this->tidCommandTaskId := new unique periodic task id
    : this->tidStatusTaskId := new unique periodic task id
    : this->tidServoTaskId := new unique periodic task id
    : this->tidDataTaskId := new unique event driven task id
]]



[[D tDlGizmo::GetLaserModulation (void)]
[ invalid_pointer (this) ||
  invalid_reference (this->IoSet) => UNDEFINED
| true =>
    return := this->IoSet.Laser.ModulationSource.ModulationParams
]]

[[D tDlGizmo::SetLaserModulation (const tModulationParams& ModParams)]
[ invalid_pointer (this) ||
  invalid_reference (ModParams) ||
  invalid_reference (this->IoSet) => UNDEFINED
| true =>
    this->IoSet.Laser.ModulationSource :=
      this->IoSet.Laser.ModulationSource appropriately initialized
         with ModParams
    : return := OK
]]


[[D tDlGizmo::GetServoParams (void)]
[ invalid_pointer (this) ||
  invalid_reference (this->CompleteControllerSet) => UNDEFINED
| true =>
    return := this->CompleteControllerSet->ControllerData.ServoParams 
]]



*********************** Private Specification ***************************************

[[D tDlGizmo::Ping (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| ::ErrorStream (::pThisRteG).bInit == FALSE => no response
| true =>
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages
    : return := OK
]]


[[D tDlGizmo::Off (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::Front  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := FRONT_MODE
    : appropriate delay line moves to front of track
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::FrontLimitReached  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| this->Servo.iServoMode == FRONT_MODE =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
| this->Servo.iServoMode == TRACK_MODE =>
    appropriate delay line stops moving until target is within 
      trackable bounds
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
]]

[[D tDlGizmo::Back  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := BACK_MODE
    : appropriate delay line moves to back of track
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::BackLimitReached  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| this->Servo.iServoMode == BACK_MODE =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
| this->Servo.iServoMode == TRACK_MODE =>
    appropriate delay line stops moving until target is within 
      trackable bounds
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
]]


[[D tDlGizmo::Track (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := TRACK_MODE
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      this->DlTargGen.TargetArray using this->ServoParameters
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]



[[D tDlGizmo::User1 (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := TRACK_MODE
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      this->DlTargGen.TargetArray using this->ServoParameters
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::User2 (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := TRACK_MODE
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      this->DlTargGen.TargetArray using this->ServoParameters
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::ClearLaserCounter (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : this->LaserCounter cleared
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::HomeCheck (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := any of {FRONT_MODE,BACK_MODE,
      FRONT_CREEP_MODE,BACK_CREEP_MODE, OFF_MODE}
    : this->Status := appropiate status info
    : appropriate delay line moves to home sensor
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::HomeSet (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| prematurely interupted by another command =>
    this->Servo.iServoMode := anything
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return OK;
| true =>
    this->Servo.iServoMode := any of {FRONT_MODE,BACK_MODE,
      FRONT_CREEP_MODE,BACK_CREEP_MODE, OFF_MODE}
    : this->Status := appropiate status info
    : appropriate delay line moves to home sensor
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::HomeSensorReached  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Current command == CMD_DL_HOME_CHECK =>
    this->Servo.iServoMode == OFF_MODE 
    : appropriate delay line stops moving 
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
| Current command == CMD_DL_HOME_SET =>
    this->Servo.iServoMode == OFF_MODE 
    : appropriate delay line stops moving 
    : this->LaserCounter cleared
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
| true => no response
]]

[[D tDlGizmo::DitherOnOff (const tDlDitherOnCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (Command) => UNDEFINED
| Command.bTurnDitherOn == TRUE =>
    this->DlDither.bDitherOn := TRUE
    : appropriate delay line pzt actuator dithers with current dither parameters
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
| true =>
    this->DlDither.bDitherOn := FALSE
    : appropriate delay line pzt actuator ceases to dither
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::SetActiveTarget (const tDlSetActiveTargetCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (Command) => UNDEFINED
| true =>
    this->DlTargGen.TargetArray [Command.ttWhichTarget].bActive := 
        CommandDither.bTargetSetActive
    : this->Status := appropiate status info
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      messages if ::ErrorStream (::pThisRteG).bInit == TRUE
    : return := OK
]]

[[D tDlGizmo::CalibrateDither (void) ]
[ incomplete spec => FIX THIS
| invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      message
    : return := OK
]]

[[D tDlGizmo::CheckDither (void) ]
[ incomplete spec => FIX THIS
| invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true =>
    this->Servo.iServoMode := OFF_MODE
    : appropriate delay line stops moving
    : ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate
      message
    : return := OK
]]

[[D tDlGizmo::SetModulation (const tDlSetModulationCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (Command) => UNDEFINED
| true =>
    this->LaserModulation.ModulationParameters := Command.ModulationParameters
]]

[[D tDlGizmo::SetServoParameters (const tDlSetServoParametersCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (Command) => UNDEFINED
| true =>
    this->ServoParameters := Command.ServoParameters
]]

**********************/

/*******************************************************************/
/*******************************************************************/
/********************                     **************************/
/********************  Class Definitions  **************************/
/********************                     **************************/
/*******************************************************************/
/*******************************************************************/

/*******************************************************************/
/***************************  tDlGizmo  ****************************/
/*******************************************************************/


class tDlGizmoInfo;
class ostream;
class tIsitCmd;
class tDlCompleteIoSet;
class tDlCompleteControllerSet;

class tDlSetActiveTargetCmd;
class tDlDitherOnCmd;
class tDlSetModulationCmd;
class tDlSetServoParametersCmd;
class tDlSetDitherCmd;

class tDlLimits;
class tDlCompleteSetOfControllerSets;
class tDlControllerData;
class tDlControllerSet;

class tDlGizmo : public tRealtimeGizmo {
  public:  
    tDlGizmo (const tDlGizmoInfo& GizmoInfo);
    virtual ~tDlGizmo(void);
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();

    ERROR_CODE SpoutDitherDiags ();
    friend ostream& operator<< (ostream& stream,const tDlGizmo& Gizmo);

    friend class tDlDither;
    friend class tDlCommandEngine; 
    friend class tDlOffCommandEngine; 
    friend class tDlFrontCommandEngine;
    friend class tDlBackCommandEngine;
    friend class tDlTrackCommandEngine;
    friend class tDlUser1CommandEngine;
    friend class tDlUser2CommandEngine;
    friend class tDlClearLaserCounterCommandEngine;
    friend class tDlDitherCalibrateCommandEngine;
    friend class tDlDitherCheckCommandEngine;
    friend class tDlHomeSetCommandEngine;
    friend class tDlHomeCheckCommandEngine;

  protected:
    virtual ERROR_CODE CollectStatus ();
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
    virtual tDlCompleteIoSet& GetCompleteIoSet ()  = 0 ;
    virtual tDlCompleteControllerSet& GetCompleteControllerSet ()  = 0;
    virtual tDlControllerData& GetControllerData ()  = 0;
    virtual tDlStatus& GetStatus () = 0;
    virtual double ServoError () = 0;
    ERROR_CODE StartTasks (const tDlGizmoInfo& GizmoInfo,
              tDlCompleteIoSet& CompleteIoSet);
    
    const tModulationParams& GetLaserModulation ();

    ERROR_CODE SetOffGroup (tControllerGroup& OffGroup);
    ERROR_CODE SetFrontGroup (tControllerGroup& FrontGroup);
    ERROR_CODE SetBackGroup (tControllerGroup& BackGroup);
    ERROR_CODE SetFrontCreepGroup (tControllerGroup& FrontCreepGroup);
    ERROR_CODE SetBackCreepGroup (tControllerGroup& BackCreepGroup);
    ERROR_CODE SetTrackGroup (tControllerGroup& TrackGroup);
    ERROR_CODE SetUser1Group (tControllerGroup& User1Group);
    ERROR_CODE SetUser2Group (tControllerGroup& User2Group);

    double LaserOpdToStellarOpd (double dLaserOpd) const {
      return dLaserOpd * this->iNumStellarPasses / 
        (double) this->iNumLaserPasses; }
        
    double StellarOpdToLaserOpd (double dStellarOpd) const {
      return dStellarOpd * this->iNumLaserPasses / 
        (double) this->iNumStellarPasses; }

  private:
    ERROR_CODE Ping ();
    ERROR_CODE Off ();
    ERROR_CODE Track ();
    ERROR_CODE Front ();
    ERROR_CODE Back ();
    ERROR_CODE ClearLaserCounter ();
    ERROR_CODE User1 ();
    ERROR_CODE User2 ();
    ERROR_CODE HomeCheck ();
    ERROR_CODE HomeSet ();
    ERROR_CODE DataRecordOnOff (const tDlDataRecordCmd &Command);
    ERROR_CODE DitherOnOff (const tDlDitherOnCmd& Command);
    ERROR_CODE SetActiveTarget (const tDlSetActiveTargetCmd& Command);
    ERROR_CODE CalibrateDither ();
    ERROR_CODE CheckDither ();
    ERROR_CODE SetDither (const tDlSetDitherCmd& Command);
    ERROR_CODE SetModulation (const tDlSetModulationCmd& Command);

    tMutex MutexDelayLine;
    int iNumLaserPasses;
    int iNumStellarPasses;


};     /* class tDlGizmo */



#define S_dlGizmo_COULD_NOT_CREATE_DL_SERVO_OBJECT  (M_dlGizmo | 0x1)
#define S_dlGizmo_NUMBER_OF_CONTROLLERS_DOES_NOT_MATCH_NUMBER_OF_SERVO_MODES  (M_dlGizmo | 0x2)
#define S_dlGizmo_COULD_NOT_CREATE_DLLIMITS_OBJECT  (M_dlGizmo | 0x3)
#define  S_dlGizmo_CANT_ALLOCAT_COMMAND_PROCESSOR    (M_dlGizmo | 0x4)

#define S_dlGizmo_ENTERING_DL_GIZMO_CONSTRUCTOR       (M_dlGizmo | 0x1000)
#define S_dlGizmo_DATA_RELAYER_INITIALIZED            (M_dlGizmo | 0x1001)
#define S_dlGizmo_TARGET_GENERATOR_INITIALIZED        (M_dlGizmo | 0x1002)
#define S_dlGizmo_DL_DITHER_OBJECT_INITIALIZED        (M_dlGizmo | 0x1003)
#define S_dlGizmo_ENTERING_DL_GIZMO_INITIALIZE        (M_dlGizmo | 0x1004)
#define S_dlGizmo_DL_LIMITS_CONSTRUCTED               (M_dlGizmo | 0x1005)
#define S_dlGizmo_CONTROLLER_SET_ARRAY_INITIALIZED    (M_dlGizmo | 0x1006)
#define S_dlGizmo_DL_SERVO_INITIALIZED                (M_dlGizmo | 0x1007)
#define S_dlGizmo_DL_STATUS_INITIALIZED               (M_dlGizmo | 0x1008)
#define S_dlGizmo_DL_STATUS_RELAYER_INITIALIZED       (M_dlGizmo | 0x1009)
#define S_dlGizmo_DL_COMMAND_PROCESSOR_INITIALIZED    (M_dlGizmo | 0x100A)
#define S_dlGizmo_INITIALIZE_COMPLETE                 (M_dlGizmo | 0x100B)
#define S_dlGizmo_ENTERING_DL_GIZMO_DESTRUCTOR        (M_dlGizmo | 0x100C)
#define S_dlGizmo_DL_LIMITS_DESTROYED                 (M_dlGizmo | 0x100D)
#define S_dlGizmo_EXITING_DL_GIZMO_DESTRUCTOR         (M_dlGizmo | 0x100E)
#define S_dlGizmo_ENTERING_SET_SERVO_MODE             (M_dlGizmo | 0x100F)
#define S_dlGizmo_ENTERING_SET_LASER_MODULATION       (M_dlGizmo | 0x1010)
#define S_dlGizmo_ENTERING_SET_SERVO_PARAMS           (M_dlGizmo | 0x1011)

#define S_dlGizmo_ENTERING_GET_LASER_MODULATION       (M_dlGizmo | 0x1012)
#define S_dlGizmo_ENTERING_GET_SERVO_PARAMS           (M_dlGizmo | 0x1013)
#define S_dlGizmo_ENTERING_GET_CONTROLLER_DATA        (M_dlGizmo | 0x1014)
#define S_dlGizmo_NULL_COMPLETE_CTRL_SET_POINTER      (M_dlGizmo | 0x1015)
#define S_dlGizmo_ENTERING_GET_SERVO_MODE             (M_dlGizmo | 0x1016)


#endif INC_dlGizmoh
