/* $RCSfile: dlGizmoInfo.h,v $ */
/* CM Info: */   
/*$Header: /home/chara/cvsroot/ople/sys/include/dlGizmoInfo.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $ */

/* Richard Johnson - Jet Propulsion Laboratory */

/* modification history
--------------------
*/

#ifndef INC_dlGizmoInfoh
#define INC_dlGizmoInfoh

#include "realTimeGizmoInfo.h"
#include "dlTypes.h"
#include "configMgr.h"


/*****
[[D tDlGizmoInfo::tDlGizmoInfo (int iWhichRteToUse,int iGizmoIdToUse,
      int iGuiGizmoNumToUse, int InstGizmoIdToUse, GIZMO_TYPE gtType,
        const int iTargetSourceArray[NUM_DELAY_LINE_TARGETS],
        TIME tDitherPeriodToUse,
        DlPhysicalConstants PhysConstToUse, int iNumServoModesToUse,
        const tServoInfo& ServoInfoToUse,
        BOARD_NAME bnVc1BoardToUse, int iVc1ChannelToUse,
        BOARD_NAME bnServoPztBoardToUse, int iServoPztChannelToUse,
        BOARD_NAME bnMotorBoardToUse, int iMotorChannelToUse,
        BOARD_NAME bnDitherPztBoardToUse, int iDitherPztChannelToUse,
        BOARD_NAME bnFrontSoftLimitBoardToUse, int iFrontSoftLimitChannelToUse,
        BOARD_NAME bnFrontHardLimitBoardToUse, int iFrontHardLimitChannelToUse,
        BOARD_NAME bnBackSoftLimitBoardToUse, int iBackSoftLimitChannelToUse,
        BOARD_NAME bnBackHardLimitBoardToUse, int iBackHardLimitChannelToUse,
        BOARD_NAME bnLaserBoardToUse , int iLaserChannelToUse,
        BOARD_NAME bnHomeSensorBoardToUse , int iHomeSensorChannelToUse,
        BOARD_NAME bnServoPztReadbackBoardToUse, int iServoPztReadbackChannelToUse,
        BOARD_NAME bnMotorReadbackBoardToUse, int iMotorReadbackChannelToUse,
        BOARD_NAME bnDiagLaserInputBoardToUse, int iDiagLaserInputChannelToUse,
        BOARD_NAME bnDiagPztPositionInputBoardToUse, int iDiagPztPositionChannelToUse,
        BOARD_NAME bnDiagLaserOutputBoardToUse, int iDiagLaserOutputChannelToUse,
        BOARD_NAME bnDiagLaserServoErrorOutputBoardToUse,
               int iDiagLaserServoErrorOutputChannelToUse,
        BOARD_NAME bnDiagVc1ServoErrorOutputBoardToUse,
               int iDiagVc1ServoErrorOutputChannelToUse,
        BOARD_NAME bnDiagMotorServoErrorOutputBoardToUse,
               int iDiagMotorServoErrorOutputChannelToUse) ]
[ invalid_pointer (this) ||
  invalid_array (iTargetSourceArray) ||
  IsAInstrumentCDSGizmo (iTargetSourcArray [TT_MANUAL] ) == FALSE ||
  IsAInstrumentCDSGizmo (iTargetSourcArray [TT_OPEN_LOOP] ) == FALSE ||
  IsAExternalMetrology  (iTargetSourcArray [TT_EM] ) == FALSE ||
  IsAFringeTracker      (iTargetSourcArray [TT_FT] ) == FALSE ||
  other undefined condtions => UNDEFINED
| true =>
    this->iiDlTargetIdArray [TT_MANUAL] := appropriate tIpcId
    : this->iiDlTargetIdArray [TT_OPEN_LOOP] := appropriate tIpcId
    : this->iiDlTargetIdArray [TT_EM] := appropriate tIpcId
    : this->iiDlTargetIdArray [TT_FT] := appropriate tIpcId
      board, channel and other default values assigned appropriately
]]
***/                

/******************************************************************
* Delay line Gizmo config manager constants
*/

struct DlPhysicalConstants {
  double dLaserScale;
  int    iLaserNumPasses;
  int    iLaserSign;
  int    iStellarNumPasses;
  double dPztScale;
  double dDitherPztScale;
  double dMotorPosScale;
  double dDiagLaserScale;
  double dDiagPztPosScale;
  double dFrontHLimitScale;
  double dFrontSLimitScale;
  double dBackHLimitScale;
  double dBackSLimitScale;
  double dHomeScale;
  double dEddy1Scale;
  double dEddy1Offset;
  double dDiagEddy1Scale;
  double dEddy2Scale;
  double dEddy2Offset;
  double dDiagEddy2Scale;
  double dPztMaxAccel;
  double dPztMaxVelocity;
  double dVc1Scale;
  double dVc1Offset;
  double dVc1MaxAccel;
  double dVc1MaxVelocity;
  double dVc1MinPosition;
  double dVc1MaxPosition;
  double dVc2Scale;
  double dVc2Offset;
  double dVc2MaxAccel;
  double dVc2MaxVelocity;
  double dVc2MinPosition;
  double dVc2MaxPosition;
  double dMotorScale;
  double dMotorMaxAccel;
  double dMotorMaxVelocity;
  double dDiagLaserServoErrorScale;
  double dDiagVc1ServoErrorScale;
  double dDiagVc2ServoErrorScale;
  double dDiagMotorServoErrorScale;
  double dCreepVelocity;
  double dDecelerationAggressiveness;
};


/*********************************************************************************
* Delay line Gizmo information classes
*/

class tServoTaskInfo;

/****************************************************************
*
* tDlGizmoInfo class spec
*
* NOTE: This method is not really void, but I didn't want to copy
* all the parameters into the spec 
[[D tQuicDlGizmoInfo::tQuicDlGizmoInfo_params(void)]
[ invalid_pointer(this) => UNDEFINED
| true =>
  set state variables to corresponding parameters
  : FA BOARD_NAME/ChannelNum pair,
     corresponding this->channelPointer := appropriate channel pointer
]]
*
*/
class tDlGizmoInfo : public tRealtimeGizmoInfo {
public:
    tDlGizmoInfo(int iWhichRteToUse,int iGizmoIdToUse,
        const int iTargetSourceArray[NUM_DELAY_LINE_TARGETS],
        TIME tDitherPeriodToUse,
        DlPhysicalConstants PhysConstToUse, int iNumServoModesToUse,
        const tServoInfo& ServoInfoToUse,
        BOARD_NAME bnVc1BoardToUse, int iVc1ChannelToUse,
        BOARD_NAME bnServoPztBoardToUse, int iServoPztChannelToUse,
        BOARD_NAME bnMotorBoardToUse, int iMotorChannelToUse,
        BOARD_NAME bnDitherPztBoardToUse, int iDitherPztChannelToUse,
        BOARD_NAME bnFrontSoftLimitBoardToUse, int iFrontSoftLimitChannelToUse,
        BOARD_NAME bnFrontHardLimitBoardToUse, int iFrontHardLimitChannelToUse,
        BOARD_NAME bnBackSoftLimitBoardToUse, int iBackSoftLimitChannelToUse,
        BOARD_NAME bnBackHardLimitBoardToUse, int iBackHardLimitChannelToUse,
        BOARD_NAME bnLaserBoardToUse , int iLaserChannelToUse,
        BOARD_NAME bnHomeSensorBoardToUse , int iHomeSensorChannelToUse,
        BOARD_NAME bnServoPztReadbackBoardToUse, int iServoPztReadbackChannelToUse,
        BOARD_NAME bnMotorReadbackBoardToUse, int iMotorReadbackChannelToUse,
        BOARD_NAME bnDiagLaserInputBoardToUse, int iDiagLaserInputChannelToUse,
        BOARD_NAME bnDiagPztPositionInputBoardToUse, int iDiagPztPositionChannelToUse,
        BOARD_NAME bnDiagLaserOutputBoardToUse, int iDiagLaserOutputChannelToUse,
        BOARD_NAME bnDiagLaserServoErrorOutputBoardToUse,
               int iDiagLaserServoErrorOutputChannelToUse,
        BOARD_NAME bnDiagVc1ServoErrorOutputBoardToUse,
               int iDiagVc1ServoErrorOutputChannelToUse,
        BOARD_NAME bnDiagMotorServoErrorOutputBoardToUse,
               int iDiagMotorServoErrorOutputChannelToUse);
    virtual ~tDlGizmoInfo();
 
    const tIpcDblBufferId& GetFtTargetId () const;
    const tIpcDblBufferId& GetEmTargetId () const;
    const tIpcDblBufferId& GetManualTargetId () const;
    const tIpcDblBufferId& GetOpenLoopTargetId () const;
    const tIpcDblBufferId& GetTargetId (int iTargetNum) const;
    TIME FastServoPeriod () const;
    TIME MediumServoPeriod () const;
    TIME SlowServoPeriod () const;
    TIME TargetGeneratorPeriod () const { return CtDlTargetGeneratorPeriod;}
    TIME TargetGeneratorOffset () const { return CtDlTargetGeneratorOffset;}
    int TargetGeneratorPriority () const { return CiDlTargetGeneratorPriority;}
    double DitherAmplitude () const { return WHITE_LIGHT_LAMBDA_IN_M;}
    virtual TIME DitherPeriod () const = 0; 
    
    tIpcDblBufferId        iiTargetIdArray [TT_NUM_DL_TARGETS];
    TIME		   tDitherPeriod;
    DlPhysicalConstants    PhysConst;
    tOutputChannel         *pVc1Channel;
    tOutputChannel         *pServoPztChannel;
    tOutputChannel         *pMotorChannel;
    tWaveformChannel       *pDitherPztChannel;
    tInputChannel          *pFrontSoftLimitChannel;
    tInputChannel          *pFrontHardLimitChannel;
    tInputChannel          *pBackSoftLimitChannel;
    tInputChannel          *pBackHardLimitChannel;
    tLaserMetrologyChannel *pLaserChannel;
    tInputChannel          *pHomeSensorChannel;
    tReadbackChannel       *pServoPztReadbackChannel;
    tReadbackChannel       *pMotorReadbackChannel;
    tInputChannel          *pDiagLaserInputChannel;
    tInputChannel          *pDiagPztPositionInputChannel;
    tOutputChannel         *pDiagLaserOutputChannel;
    tOutputChannel         *pDiagLaserServoErrorOutputChannel;
    tOutputChannel         *pDiagVc1ServoErrorOutputChannel;
    tOutputChannel         *pDiagMotorServoErrorOutputChannel;
};


////////////////////////
// DL DITHER PERIODS 
//
extern const TIME CtBb1DitherPeriod;   /* defined in isit.h */
extern const TIME CtBb2DitherPeriod;
extern const TIME CtGsuDitherPeriod;


///////////////////////////////////
// DL PHYSICAL CONSTANTS
//
extern const DlPhysicalConstants CpcBb1PhysConst;
extern const DlPhysicalConstants CpcBb2PhysConst;
extern const DlPhysicalConstants CpcGsuPhysConst;


///////////////////////////////////


///////////////////////////////////
// DL SERVO MODE INFO
//
extern const int CiBb1NumServoModes;
extern const int CiBb2NumServoModes;
extern const int CiGsuNumServoModes;

#endif /* INC_dlGizmoInfoh */
