/* $RCSfile: dlTargGen.h,v $ - Target Generator header file */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dlTargGen.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/
/* Beth McKenney - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,eamck,16Apr97    written (Incr 2 version)
*/
 
/*
DESCRIPTION

*/

#ifndef INC_dlTargGenh
#define INC_dlTargGenh

#include "configMgr.h"
#include "ipcDblBufferAccessor.h"

/******* Stuff that goes in system header files later: *******/


#define S_dlTargGen_CANT_ALLOCATE_TARGETS_STREAMS (M_dlTargGen | 0x0001)

#define S_dlTargGen_ENTERING_DL_TARGGEN_INIT     (M_dlTargGen | 0x1001)
#define S_dlTargGen_ALL_TARGET_SOURCES_INITED    (M_dlTargGen | 0x1002)
#define S_dlTargGen_LEAVING_DL_TARGGEN_INIT      (M_dlTargGen | 0x1003)
#define S_dlTargGen_COMPUTE_TOTAL_TARGET_TASK    (M_dlTargGen | 0x1004)


/************** (end of system header file stuff) ************/
 



/******************************************************************
*
* tDlTargGen Black Box
*
* BB State Variables
* ------------------
*
*******************************************************************
*
* tDlTargGen::tDlTargGen(void) - constructor
*
* SPEC: Nuthin' now... I did consider passing the tDlTargGenPeriod arg 
	   in to the constructor, but that seemed more 'slick' than 
	   really useful.  (We can do that in Increment "Slick".)
* RETURNS:
*
*******************************************************************
*
* tDlTargGen::~tDlTargGen(void) - destructor
*
* Removes the targ gen task from the scheduler
*       (prob. not used in Incr 2)
*
* SPEC:
   [[D tDlTargGen::~tDlTargGen() ]
   [ this invalid => UNDEFINED
   | Scheduler on this RTE not working properly => nothing happens
   | true =>
       target generator stops reading and processing targets
   ]]
*
* RETURNS:
*
*******************************************************************
*
* ERROR_CODE tDlTargGen::Initialize(const tDlGizmoInfo &GizmoInfo,
*                                   TIME tDlTargGenPeriod)
*
* SPEC:
   [[D tDlTargGen::Initialize(in: this,
                                  GizmoInfo,
                                  tDlTargGenPeriod,
                                  CdtZeroTarget;
                            out: *this.dtCurrentTarget,
                                 *this.bTargetSelectionArray,
                                 *this.iiTargetIn,
                                 *pThisGizmo.*pRte.*pScheduler.TaskSet,
                                  return; )]
   [ this invalid => UNDEFINED
   | can't allocate some of the target source objects in IPC =>
			  return := appropriate IPC error message
   | Scheduler on this RTE not working properly => 
			  return := appropriate Scheduler error message
   | true =>
       tDlTargGen object initialized appropriately
       : tDlTargGen::ComputeTotalTarget() starts being scheduled
		                 with period tDlTargGenPeriod
       : return := OK
   ]]
* RETURNS:
*    OK
*    S_ipc_APPROPRIATE_ERROR_MESSAGE
*
*******************************************************************
*
* ERROR_CODE tDlTargGen::GetCurrentTargetTriple(tDlTarget *pCurrentTarget)
*
* SPEC:
   [[D tDlTargGen::GetCurrentTargetTriple(in:   this,
                                                pCurrentTarget,
                                               *this.dtCurrentTarget;
                                          out:  return, 
                                               *pCurrentTarget; )]

   [ this invalid || pCurrentTarget invalid => UNDEFINED
   | true => 
       *pCurrentTarget := *this.dtCurrentTarget
       : return := OK
   ]]

* RETURNS:
*    OK
*
*******************************************************************
*
* double tDlTargGen::GetCurrentTargetPosition(TIME tCurrTime) 
*
* SPEC:
   [[D tDlTargGen::GetCurrentTargetPosition(in:   this,
                                                  tCurrTime,
                                                 *this.dtCurrentTarget;
                                            out:  return; ) ]

   [ this invalid => UNDEFINED
   | true => 
       return := current target position, calculated from current
		     target triple evaluated at time tCurrTime
   ]]

* RETURNS:
*    current target position based on tCurrTime
*
*******************************************************************
*
* double tDlTargGen::GetCurrentTargetVelocity(void)
*
* SPEC:
   [[D tDlTargGen::GetCurrentTargetVelocity(in:   this,
                                                 *this.dtCurrentTarget;
                                            out:  return; )]

   [ this invalid => UNDEFINED
   | true =>
       return := *this.dtCurrentTarget.dRate 
   ]]

* RETURNS:
*    current target rate 
*
*******************************************************************
*
* ERROR_CODE tDlTargGen::SetAllActiveTargets(BOOL *pbTargetSelections)
*
* SPEC:
   [[D tDlTargGen::SetAllActiveTargets(in:   this, 
				             pbTargetSelections,
				             CiNumDelayLineTargets; 
				       out:  return, 
                                            *this.bTargetSelectionArray[]; )]
   [ this invalid || pbTargetSelections invalid => UNDEFINED
   | true => 
       target selection switches are set to values in pbTargetSelections[]
       : return := OK
   ]]
*
* RETURNS:
*
*
*******************************************************************
*
* ERROR_CODE tDlTargGen::SetActiveTarget(int iWhichTarget, BOOL bTurnOn)
*
* SPEC:
   [[D tDlTargGen::SetActiveTarget(in:   this,
                                         iWhichTarget,
                                         bTurnOn;
                                   out:  return,
                                        *this.bTargetSelectionArray[]; )]
   [ this invalid => UNDEFINED
   | true =>
        target switch #iWhichTarget is set to bTurnOn 
       : return := OK
   ]]
*
* RETURNS:
*
*
*******************************************************************
*
* ERROR_CODE tDlTargGen::ComputeTotalTarget(void)
*
* SPEC:
   [[D tDlTargGen::ComputeTotalTarget(in: this,
					 *this.*pGizmo.*pRte.tSchedTime,
                                          CiNumDelayLineTargets,
                                         *this.iiTargetIn[],
                                         *this.bTargetSelectionArray[]; 
				    out:  return, 
                                         *this.dtCurrentTarget; )]
   [ this invalid => UNDEFINED
   | any of the objects of *this.iiTargetIn[] is invalid => UNDEFINED
   | true => 
       *this.dtCurrentTarget := sum of targets from IPC objects in 
             *this.iiTargetIn[] corresponding to TRUE elements 
              of *this.bTargetSelectionArray[]
       : return := OK
   ]]

* RETURNS:
* 
*
*******************************************************************
*/

class tDlTargetActuator;
class tDlGizmoInfo;

class tTargGen {
   public:
      virtual ~tTargGen(void) { }
      virtual double GetCurrentTargetPosition(TIME tCurrTime) = 0;
      virtual double GetCurrentTargetVelocity(void) = 0;
      virtual double GetCurrentTargetVelocity(TIME tCurrTime) {
        return GetCurrentTargetVelocity();
      }
};

class tDlTargGen : public tTargGen {
   public:
      tDlTargGen (const tDlGizmoInfo& GizmoInfo);
      ~tDlTargGen(void);
      double GetCurrentTargetPosition(TIME tCurrTime);
      double GetCurrentTargetVelocity(void);
      ERROR_CODE GetCurrentTargetTriple(tDlTarget *pCurrentTarget);
      ERROR_CODE SetAllActiveTargets(BOOL *bTargetSelectionArray);
      ERROR_CODE SetActiveTarget(int iWhichTarget, BOOL bTurnTargetOn);

   protected:
      tDlTarget 	dtCurrentTarget;
      BOOL		bTargetSelectionArray[NUM_DELAY_LINE_TARGETS];
      tIpcDblBufferAccessor<tDlTarget> **pTargetStreamArray;
      int		iTargGenTaskId;
      ERROR_CODE 	ComputeTotalTarget(void);



};


#endif  /* INC_dlTargGenh */

