/* $RCSfile: dlTypes.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/dlTypes.h,v 1.3 1998/09/23 23:57:23 phil Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
02b,rlj,6May97     added constructor to tDlStatus, tDlData to set gizmo type and
                   size of structure
01a,beh,16Oct96    written
*/
 
/*
DESCRIPTION
This file contains type defintions of externally visible delay
line structures
*/

#ifndef INC_dlTypesh
#define INC_dlTypesh

#include "isit.h"
#include "modLib.h"
#include "dlErrors.h"

#if 0
#ifndef INCR1
#error PrefixTheseWithDL_
#endif
#endif
#define JITTER_SCALE 1
#define JITTER_HP_BLKSZ (DL_DATA_BUF_SIZE/2)  /* 2 high pass blks per buffer */

class tDlGizmo;
class tDitherCalibInfo;

/* diff types of DL targets; used by SetActiveTarget commands and dlTargGen */
/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/
/* diff types of DL targets; used by SetActiveTarget commands and dlTargGen */

typedef enum TARGET_TYPE { TT_MANUAL=0, TT_OPEN_LOOP=1, TT_FT=2, 
  TT_EM=3, TT_NUM_DL_TARGETS} TARGET_TYPE;


typedef enum {
  DLCS_SYSTEM_READY, DLCS_OFF_START, DLCS_OFF_DONE, 
  DLCS_FRONT_START, DLCS_FRONT_MOVING, DLCS_FRONT_DONE, DLCS_FRONT_ABORTED,
  DLCS_BACK_START, DLCS_BACK_MOVING, DLCS_BACK_DONE, DLCS_BACK_ABORTED,
  DLCS_TRACK_START, DLCS_TRACK_ACTIVE, DLCS_TRACK_ABORTED,
  DLCS_CLEAR_LASER_COUNTER_START, DLCS_CLEAR_LASER_COUNTER_DONE,
  DLCS_SET_MODULATION_COMPLETE,
  DLCS_SET_SERVO_PARAMETERS_COMPLETE,

  DLCS_HOMESET_BACK_START, DLCS_HOMESET_BACK_MOVING,
    DLCS_HOMESET_FRONT_START, DLCS_HOMESET_FRONT_MOVING,
    DLCS_HOMESET_CREEPBACK_START, DLCS_HOMESET_CREEPBACK_MOVING,
    DLCS_HOMESET_CREEPFRONT_START, DLCS_HOMESET_CREEPFRONT_MOVING,
    DLCS_HOMESET_START, DLCS_HOMESET_DONE, DLCS_HOMESET_ABORTED, 

  DLCS_HOMECHECK_BACK_START, DLCS_HOMECHECK_BACK_MOVING,
    DLCS_HOMECHECK_FRONT_START, DLCS_HOMECHECK_FRONT_MOVING,
    DLCS_HOMECHECK_CREEPBACK_START, DLCS_HOMECHECK_CREEPBACK_MOVING,
    DLCS_HOMECHECK_CREEPFRONT_START, DLCS_HOMECHECK_CREEPFRONT_MOVING,
    DLCS_HOMECHECK_START, DLCS_HOMECHECK_DONE, DLCS_HOMECHECK_ABORTED,

  DLCS_USER1_START, DLCS_USER1_ACTIVE, DLCS_USER1_ABORTED, 
  DLCS_USER2_START, DLCS_USER2_ACTIVE, DLCS_USER2_ABORTED, 
  DLCS_CALIB_DITHER_START, DLCS_CALIB_DITHER_ACTIVE, 
    DLCS_CALIB_DITHER_DONE, DLCS_CALIB_DITHER_ABORTED, 
  DLCS_CHECK_DITHER_START, DLCS_CHECK_DITHER_ACTIVE, 
    DLCS_CHECK_DITHER_DONE, DLCS_CHECK_DITHER_ABORTED, 
  DLCS_SELECT_TARGET_DONE,
  DLCS_TURN_DITHER_ON_DONE, DLCS_TURN_DITHER_OFF_DONE,
  DLCS_SET_DITHER_COMPLETE
  
} DL_COMMAND_STATUS;

typedef enum DL_MODE {
  DM_OFF, DM_FRONT, DM_BACK, DM_TRACK, DM_DITHERCALIB, 
  DM_DITHERCHECK, DM_HOMESET, DM_HOMECHECK, DM_USER1, DM_USER2
} DL_MODE;

/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/

typedef enum DL_SERVO_MODE {
  DSM_OFF, DSM_FRONT, DSM_BACK, DSM_TRACK, 
  DSM_CREEPFRONT, DSM_CREEPBACK, DSM_USER1, DSM_USER2, DSM_NUM_SERVO_MODES
} DL_SERVO_MODE;


/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo tracking mode.  It should always be kept at the last
* of these enums.
*/

enum DL_OFF_CONTROLLER_MODE {DOCM_IDLE, DOCM_DECELERATE, DOCM_NUM_OFF_MODES};
enum DL_TRACK_CONTROLLER_MODE {DTCM_TRACK, DTCM_SLEW, DTM_NUM_TRACKING_MODES};

/******************************************************************
* The final value here is used to size arrays, etc.  It is not an
* actual servo task speed.  It should always be kept at the last
* of these enums.
*
* THE PARITION INFO ENTRIES IN THE GIZMO INFO STRUCTURE MUST BE
* ORDERED THE SAME AS THESE ENTRIES!
*/

enum DL_SERVO_PARTITION { DSP_FAST, DSP_MEDIUM, DSP_SLOW, DSP_NUM_DL_SERVO_TASKS};

//
//AS
typedef enum DL_DIRECTION {DD_NEGATIVE, DD_POSITIVE} DL_DIRECTION;


/*************** tDlPztParameters Specification **************************

[[D tDlPztParameters::tDlPztParameters (void) ]
[ invalid_pointer (this)  => UNDEFINED
| true => *this := anything
]]


[[D tDlPztParameters::tDlPztParameters_bbdddd (BOOL bEnabledIn, BOOL bLagEnabledIn,
     double dGain, double dTauP, double dTauZ, double dSamplePeriodIn) ]
[ invalid_pointer (this)  => UNDEFINED
| true =>
    this->bEnabled := bEnabledIn
    : this->bLagEnabled := bLagEnabledIn
    : this->dPztGain := dGain
    : this->dLagTauP := dTauP
    : this->dLagTauZ := dTauZ
    : this->dSamplePeriod := dSamplePeriodIn
]]
*/

class tDlPztParameters {
  public:
    tDlPztParameters () {}
    tDlPztParameters (BOOL bEnabledIn, BOOL bLagEnabledIn, double dGain, 
                      double dTauP, double dTauZ, double dSamplePeriodIn) :
      bEnabled (bEnabledIn), bLagEnabled (bLagEnabledIn), dPztGain (dGain),
      dLagTauP (dTauP), dLagTauZ (dTauZ), dSamplePeriod (dSamplePeriodIn) {}                      
    BOOL bEnabled;
    BOOL bLagEnabled;
    double dPztGain;
    double dLagTauP;
    double dLagTauZ;
    double dSamplePeriod;
};

/*************** tDlMotorParameters Specification **************************

[[D DlMotorParameters::DlMotorParameters (void) ]
[ invalid_pointer (this)  => UNDEFINED
| true => *this := anything
]]


[[D DlMotorParameters::DlMotorParameters_bddd(BOOL bEnabledIn, 
      double dSlewVelocity, double dAggressiveness, double dSamplePeriod) ]
[ invalid_pointer (this)  => UNDEFINED
| true =>
    this->bEnabled := bEnabledIn
    : this->dMotorSlewVelocity := dSlewVelocity
    : this->dDecelerationAggressiveness := dAggressiveness
    : this->dMotorSamplePeriod := dSamplePeriod
]]
*/

class tDlMotorParameters {
  public:
    tDlMotorParameters () {}
    tDlMotorParameters (BOOL bEnabledIn, double dSlewVelocity, 
       double dAggressiveness, double dSamplePeriod, double dMotorFeedforwardGain) :
         bEnabled (bEnabledIn),
         dMotorSlewVelocity (dSlewVelocity), 
         dDecelerationAggressiveness (dAggressiveness),
         dMotorSamplePeriod (dSamplePeriod),
         dMotorFeedforwardGain (dMotorFeedforwardGain) {}
    BOOL bEnabled;
    double dMotorSlewVelocity;
    double dDecelerationAggressiveness;            
    double dMotorSamplePeriod;
    double dMotorFeedforwardGain;
};

 
/******************************************************************
*
* class tDlServoParams spec
*
  [[D tDlServoParams::tDlServoParams_void(void)]
  [ invalid_pointer(this) => UNDEFINED
  | true => *this = anything
  ]]
*
*
  [[D tDlServoParams::tDlServoParams_params(double dPztGain, 
					    double dPztLagPole,
                                            double dPztLagZero, 
					    double dPztServoRate,
                                            BOOL bLagEnabled, 
					    BOOL bPztServoEnabled,
                                            double dSlewToTrackThresholdInMm,
                                            double dTrackToSlewThresholdInMm,
                                            double dVc1TrackPidGain, 
					    double dVc1TrackPidZero1,
                                            double dVc1TrackPidZero2, 
					    double dVc1TrackPidPole,
                                            double dVc1LaserFeedforwardGain,
                                            double dVc1LaserFeedforwardFilterMemory,
                                            double dVc1TrackServoRate, 
					    BOOL bVc1TrackServoEnabled,
                                            double dVc1SlewPidGain, 
					    double dVc1SlewPidZero1,
                                            double dVc1SlewPidZero2, 
					    double dVc1SlewPidPole,
                                            double dVc1SlewServoRate, 
					    BOOL bVc1SlewServoEnabled,
                                            double dMotorGain, 
					    double dMotorFeedForwardGain,
                                            double dMotorServoRate, 
					    BOOL bMotorServoEnabled) ]
  [ invalid_pointer(this) => UNDEFINED
  | true => 
     set all state variables to corresponding parameter values
  ]]
*
*/
class tDlServoParams {
public:
  tDlServoParams(void) {};
  tDlServoParams(double dPztGain, double dPztLagPole,
                 double dPztLagZero, double dPztServoRate,
                 BOOL bLagEnabled, BOOL bPztServoEnabled,
                 
                 double dSlewToTrackThresholdInMm,
                 double dTrackToSlewThresholdInMm,
  
                 double dVc1TrackPidGain, double dVc1TrackPidZero1,
                 double dVc1TrackPidZero2, double dVc1TrackPidPole,
                 double dVc1LaserFeedforwardGain,
                 double dVc1LaserFeedforwardFilterMemory,
                 double dVc1TrackServoRate, BOOL bVc1TrackServoEnabled,

                 double dVc1SlewPidGain, double dVc1SlewPidZero1,
                 double dVc1SlewPidZero2, double dVc1SlewPidPole,
                 double dVc1SlewServoRate, BOOL bVc1SlewServoEnabled,
                 
                 double dMotorGain, double dMotorFeedForwardGain,
                 double dMotorServoRate, BOOL bMotorServoEnabled);

  double dPztGain;
  double dPztLagPole;
  double dPztLagZero;
  double dPztServoRate;
  double bLagEnabled;
  double bPztServoEnabled;
 
  double dSlewToTrackThresholdInMm;
  double dTrackToSlewThresholdInMm;

  double dVc1TrackPidGain;
  double dVc1TrackPidZero1;
  double dVc1TrackPidZero2;
  double dVc1TrackPidPole;
  double dVc1LaserFeedforwardGain;
  double dVc1LaserFeedforwardFilterMemory;
  double dVc1TrackServoRate;
  double bVc1TrackServoEnabled;

  double dVc1SlewPidGain;
  double dVc1SlewPidZero1;
  double dVc1SlewPidZero2;
  double dVc1SlewPidPole;
  double dVc1SlewServoRate;
  double bVc1SlewServoEnabled;

  double dMotorGain;
  double dMotorFeedForwardGain;
  double dMotorServoRate;
  double bMotorServoEnabled;
};



/******************************************************************
*
* class tQuicDlServoParams spec
*
  [[D tQuicDlServoParams::tQuicDlServoParams_void(void)]
  [ invalid_pointer(this) => UNDEFINED
  | true => *this = anything
  ]]
*
*
  [[D tQuicDlServoParams::tQuicDlServoParams_params(double dPztGain, 
						    double dPztLagPole,
                                                    double dPztLagZero, 
						    double dPztServoRate,
                                                    BOOL bLagEnabled, 
						    BOOL bPztServoEnabled,
                                                    double dSlewToTrackThresholdInMm,
                                                    double dTrackToSlewThresholdInMm,
                                                    double dVc1TrackPidGain, 
						    double dVc1TrackPidZero1,
                                                    double dVc1TrackPidZero2, 
						    double dVc1TrackPidPole,
                                                    double dVc1LaserFeedforwardGain,
                                                 double dVc1LaserFeedforwardFilterMemory,
                                                    double dVc1TrackServoRate, 
						    BOOL bVc1TrackServoEnabled,
                                                    double dVc1SlewPidGain, 
						    double dVc1SlewPidZero1,
                                                    double dVc1SlewPidZero2, 
						    double dVc1SlewPidPole,
                                                    double dVc1SlewServoRate, 
						    BOOL bVc1SlewServoEnabled,
                                                    double dMotorGain, 
						    double dMotorFeedForwardGain,
                                                    double dMotorServoRate, 
						    BOOL bMotorServoEnabled,
                                                    double dMotorErrorGain) ]
  [ invalid_pointer(this) => UNDEFINED
  | true =>
     set all state variables to corresponding parameter values
  ]]
*
*/
class tQuicDlServoParams : public tDlServoParams {
public:
  tQuicDlServoParams(void) {};
  tQuicDlServoParams(double dPztGain, double dPztLagPole,
                 double dPztLagZero, double dPztServoRate,
                 BOOL bLagEnabled, BOOL bPztServoEnabled,
                 
                 double dSlewToTrackThresholdInMm,
                 double dTrackToSlewThresholdInMm,
  
                 double dVc1TrackPidGain, double dVc1TrackPidZero1,
                 double dVc1TrackPidZero2, double dVc1TrackPidPole,
                 double dVc1LaserFeedforwardGain,
                 double dVc1LaserFeedforwardFilterMemory,
                 double dVc1TrackServoRate, BOOL bVc1TrackServoEnabled,

                 double dVc1SlewPidGain, double dVc1SlewPidZero1,
                 double dVc1SlewPidZero2, double dVc1SlewPidPole,
                 double dVc1SlewServoRate, BOOL bVc1SlewServoEnabled,
                 
                 double dMotorGain, double dMotorFeedForwardGain,
                 double dMotorServoRate, BOOL bMotorServoEnabled,
                 double dMotorErrorGain, double dDiagLaserScale);

  double bPztAdcOn;
  double bVcAdcOn;
  double bMotorAdcOn;
  double dMotorErrorGain;
};


typedef struct QuicServoParams {
  double dPztGain;
  double dPztLagPole;
  double dPztLagZero;
  double dPztServoRate;
  double bLagEnabled;
  double bPztServoEnabled;

  double dSlewToTrackThresholdInMm;
  double dTrackToSlewThresholdInMm;

  double dVc1TrackPidGain;
  double dVc1TrackPidZero1;
  double dVc1TrackPidZero2;
  double dVc1TrackPidPole;
  double dVc1LaserFeedforwardGain;
  double dVc1LaserFeedforwardFilterMemory;
  double dVc1TrackServoRate;
  double bVc1TrackServoEnabled;

  double dVc1SlewPidGain;
  double dVc1SlewPidZero1;
  double dVc1SlewPidZero2;
  double dVc1SlewPidPole;
  double dVc1SlewServoRate;
  double bVc1SlewServoEnabled;

  double dMotorGain;
  double dMotorFeedForwardGain;
  double dMotorServoRate;
  double bMotorServoEnabled;

  double bPztAdcOn;
  double bVcAdcOn;
  double bMotorAdcOn;
  double dMotorErrorGain;
} QuicServoParams;


#if 0
typedef union TRACK_SERVO_PARAMS {
  // tDlServoParams     GenericDlServoParams;
  // tQuicDlServoParams QuicServoParams;
  QuicServoParams    DlQuicServoParams;
  GsuServoParams     DlGsuServoParams;
  double             dQuicServoParams[(sizeof(tQuicDlServoParams)/sizeof(double))]; 
  // tGsuDlServoParams  GsuServoParams;
  double             dGsuServoParams[(sizeof(tGsuDlServoParams)/sizeof(double))]; 
} TRACK_SERVO_PARAMS;

#endif 0

class tDlStatus_Version_1  {
public:
  int iVersion;

  int iWhichDl;
  TIME tTime;
  
  // Sensors
  LASER_VALUE laLaser;
  PZT_DOUBLE  pdPztPos;        // PZT position
  EDDY_VALUE    evEddy1;        /* GSU delay line only */
  EDDY_VALUE    evEddy2;        /* GSU delay line only */

  BOOL bLimitFrontSoft;
  BOOL bLimitFrontHard;
  BOOL bLimitBackSoft;
  BOOL bLimitBackHard;
  BOOL bOnHomeSensor;

  // Sensor modulation info
  tModulationParams SensorModulation;
  
  // Dither calibration info
  BOOL       bDoingDitherCheckOrCalib;
  double     dDitherAmplitude;
  double     dDitherPhase;
  double     dDitherDistortion;

  // Actuators
  PZT_VALUE    pvPztVelocity;   // PZT Velocity
  VC_VALUE     vvVc1;            // VC1 Output
  VC_VALUE     vvVc2;            // VC2 Output (GSU only; ignored otherwise)
  MOTOR_VALUE  mvMotorVelocity; // Motor speed

  // Actuator modulation

  // Servo performance info
//  TRACK_SERVO_PARAMS ServoParams; moved to separate status types...
  DL_SERVO_MODE  dsmServoMode;
  LASER_VALUE    laTarget;
  LASER_VALUE    laError;
  double         dJitter;
  double         dHighPassJitter;
  int iPztClip;      // Pzt Clip counter

  // Status of current command
  DL_COMMAND_STATUS  DlCommandStatus;
};

typedef struct DL_DATUM {
  LASER_VALUE laLaser;
  LASER_VALUE laError;
} DL_DATUM;

class tDlData_Version_1  {
public:
  int iVersion;

  int iWhichDl;
  TIME tTimeStamp;
  DL_DATUM ServoDataArray[DL_DATA_BUF_SIZE];
};


#define CURRENT_DL_STATUS_VERSION 1
#define CURRENT_DL_DATA_VERSION   1

#define tDlStatusCurrentVersion \
   EXPAND_AND_CONCAT(tDlStatus_Version_, CURRENT_DL_STATUS_VERSION)
#define tDlDataCurrentVersion \
   EXPAND_AND_CONCAT(tDlData_Version_, CURRENT_DL_DATA_VERSION)

class tDlStatusUnix: public tDlStatusCurrentVersion {
public:
  ERROR_CODE Initialize(int iDlNumber) {
    iVersion = CURRENT_DL_STATUS_VERSION;
    iWhichDl = iDlNumber;
    return OK;
  }
};



class tDlDataUnix:   public tDlDataCurrentVersion {
public:
  ERROR_CODE Initialize(int iDlNumber) {
    iVersion = CURRENT_DL_DATA_VERSION;
    iWhichDl = iDlNumber;
    return OK;
  }
};

class tDlData: public tDlDataUnix {
public:
  ERROR_CODE Initialize(int iDlNumber)
    { return tDlDataUnix::Initialize(iDlNumber); }
  double calcDlJitter ( int iArraySize );
  double calcDlHpJitter ( int iArraySize );
};


/******************************************************************
*
* tDlStatus Black Box
*
*  State Variable Dictionary
*  Variable                   Initial Value
*  ----------------------------------------
*  int iVersion               Unknown
*  int iWhichDl               Unknown
*  *** Refer to tDlStatus_Version_n class above for other variables ***
*  
*******************************************************************
*
* ERROR_CODE Initialize(int iDlNumber)
*
* SPEC:
*
     [[D tDlStatus::Initialize(int iDlNumber)]
      [ this invalid => UNDEFINED
      | true =>
          *this.iVersion := CURRENT_DL_STATUS_VERSION
          : *this.iWhichDl := iDlNumber
          : return OK
     ]]
*
*  
*******************************************************************
*
  [[D tDlStatus::UpdateDitherCalibration(double dAmplitude, 
					 double dPhase, 
					 double dDistortion) ]
   [ this invalid => UNDEFINED
   | true =>
       *this.dDitherAmplitude    := dAmplitude
       : *this.dDitherPhase      := dPhase
       : *this.dDitherDistortion := dDistortion
  ]]
*
* The specs for these methods are all obvious enough that it's not worth
* wasting space to write them.
*/

class tDlStatus: public tDlStatusUnix {
public:
  tDlStatus () {}
  tDlStatus (int iDlNumber);
//  void UpdateTrackServoParams(const TRACK_SERVO_PARAMS &NewServoParams);
  void UpdateCommandStatus(DL_COMMAND_STATUS dcsNewCommandStatus);
  void UpdateServoState(DL_SERVO_MODE dsmNewServoMode);
  void UpdateLimits(BOOL bNewLimitFrontSoft, BOOL bNewLimitFrontHard,
                    BOOL bNewLimitBackSoft,  BOOL bNewLimitBackHard,
                    BOOL bNowOnHomeSensor);
  void UpdateJitter(double dJitter, double dHighPassJitter);
  void UpdateModulation(const tModulationParams &ModulationParams);
  void UpdateServoInfo(LASER_VALUE laLaserValue,
                       PZT_DOUBLE pdPztPos,
                       PZT_VALUE pvPztVelocity,
                       VC_VALUE vvVc1,
                       MOTOR_VALUE mvMotorVelocity,
                       LASER_VALUE laTargetValue,
                       LASER_VALUE laErrorValue,
                       int iPztClip);
  void UpdateAdditionalGsuServoInfo(
                       EDDY_VALUE evEddy1,
                       EDDY_VALUE evEddy2,
                       VC_VALUE vvVc2);
  void UpdateDitherCalibration(const tDitherCalibInfo &CalibInfo);
  void UpdateDitherCheckOrCalib(BOOL bNowDoingCheckOrCalib);
};



#endif  /* INC_dlTypesh */

