/* $RCSfile: emController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/em/emController.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#ifndef emControllerh
#define emControllerh

#include "isit.h"
#include "controller.h"
#include "typeTag.h"
#include "emTypes.h"
#include "dataStore.h"


/*******
Black Box State Variables
--------------------------

tEmServoParameters - see emTypes.h
--------------------------------------------

tEmControllerData 
 
tDataStore<tEmData> EmData
--------------------------------------------
tEmOffController

tEmControllerData&        ControllerData
--------------------------------------------
tEmTrackController

tPeriodicTargetGenerator& TargetGenerator
tDlActuator&              DelayLineActuator
tEmControllerData&        ControllerData
--------------------------------------------
tEmCompleteControllerSet

tPeriodicTargetGenerator TargetGenerator
tDlActuator              DelayLineActuator
tEmControllerData        ControllerData
------------------------------------------------
Stimuli


[[D tEmControllerData::tEmControllerData (const tTypeTag& TypeTagToUse) ]
[ invalid_pointer (this) ||
   invalid_reference (TypeTagToUse) => UNDEFINED
| true => this->EmData := DATA_STORE <TypeTagToUse+tEmData>
]]

[[D tEmController::tEmController (tEmControllerData& ControllerDataToUse) ]
[ invalid_pointer (this) ||
   invalid_reference (ControllerData) => UNDEFINED
| true => this->ControllerData := ControllerDataToUse
]]



[[D tEmOffController::tEmOffController (tEmControllerData& ControllerDataToUse) ]
[ invalid_pointer (this) ||
   invalid_reference (ControllerData) => UNDEFINED
| true => this->ControllerData := ControllerDataToUse
]]

[[D tEmTrackController::tEmTrackController (tEmCompleteIoSet& CompleteIoSet,
                                      tEmControllerData& ControllerDataToUse) ]
[ invalid_pointer (this) ||
   invalid_reference (CompleteIoSet) ||
   invalid_reference (ControllerData) => UNDEFINED
| true => 
  this->TargetGenerator := CompleteIoSet.TargetGenerator
    : this->DelayLineActuator := CompleteIoSet.DelayLineActuator
    : this->ControllerData := ControllerDataToUse
]]

[[D tEmTrackController::~tEmTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tEmTrackController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) ||
   invalid_reference (this->TargetGenerator) ||
   invalid_reference (this->DelayLineActuator)  => UNDEFINED
| true => 
    DlTarget (this->DelayLineActuator.iDlGizmoId, TT_EM) := 
      this->TargetGenerator.TargetValue
    : this->TargetGenerator.TargetValue := new value based on modulation parameters
    : appropriate data added to this->ControllerData.EmData
    : return := OK
]]

[[D tEmTrackController::Reset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->TargetGenerator := CompleteIoSet.TargetGenerator
    : this->DelayLineActuator.TargetValue := ZERO_TARGET
    : Gizmo (this->DelayLineActuator.iDlGizmoId).EmTarget := ZERO_TARGET
]]


[[D tEmCompleteControllerSet::tEmCompleteControllerSet (
          tEmCompleteIoSet& CompleteIoSetToUse,
          const tTypeTag& DataTag) ]
[ invalid_pointer (this) ||
  invalid_reference (CompleteIoSetToUse) ||
  invalid_reference (DataTag)=> UNDEFINED
| true => 
  this->TrackController.TargetGenerator := CompleteIoSet.TargetGenerator
  : this->TrackController.DelayLineActuator := CompleteIoSet.DelayLineActuator
  : this->ControllerData.EmData := Empty data store with tag DataTag
]]

[[D tEmCompleteControllerSet::~tEmCompleteControllerSet (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tEmCompleteControllerSet::GetOffController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->OffController
]]


[[D tEmCompleteControllerSet::GetTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->TrackController
]]

[[D tEmCompleteControllerSet::SetServoParameters (
       const tEmServoParameters& ServoParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (ServoParameters) => UNDEFINED
| true => 
    this->TargetGenerator.ModulationParameters := 
      ServoParameters.TrackParameters
    : return := OK
]]
*****/

class tPeriodicTargetGenerator;
class tDlActuator;
class tEmCompleteIoSet;
class tEmCompleteControllerSet;
class tEmServoParameters;
class tServoInfo;


class tEmControllerData : public tControllerData {
  public:
    tEmControllerData (const tTypeTag& DataTag);

    tDataStore<tEmData> EmDataStore;
};

class tEmController : public tController {
  public:
    tEmController (tEmControllerData& ControllerDataToUse);
    tEmControllerData& ControllerData;
};

class tEmOffController : public tEmController {
  public: 
    tEmOffController (tEmControllerData& ControllerData);
};

class tEmTrackController : public tEmController {
  public: 
    tEmTrackController (tEmCompleteIoSet& CompleteIoSet,
                        tEmControllerData& ControllerData);
    virtual~tEmTrackController ();
    virtual ERROR_CODE Calculate (TIME tServoTime);
    virtual ERROR_CODE Reset (TIME tServoTime);
  protected:
    tPeriodicTargetGenerator& TargetGenerator;
    tDlActuator& DelayLineActuator;
};


class tEmCompleteControllerSet {
  public: 
    tEmCompleteControllerSet (tEmCompleteIoSet& CompleteIoSetToUse,
                              const tTypeTag& DataTag);
    ~tEmCompleteControllerSet ();
    tEmOffController& GetOffController ();
    tEmTrackController& GetTrackController ();
    ERROR_CODE SetServoParameters (const tEmServoParameters& ServoParameters);
  protected:
    tEmOffController   OffController;
    tEmTrackController TrackController;
    tEmCompleteIoSet&  CompleteIoSet;
    tEmControllerData  ControllerData;
};

 

#endif emControllerh
