/* $RCSfile: emGizmo.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/emGizmo.h,v 1.2 1998/09/23 23:57:26 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#ifndef INC_emGizmoh
#define INC_emGizmoh

#include "isit.h"
#include "realTimeGizmo.h"
#include "typeTag.h"
#include "mutex.h"
#include "emTypes.h"
#include "controllerSet.h"
#include "em/emController.h"
#include "em/emIoSet.h"
#include "emGizmoInfo.h"

class ostream;
class tIsitCmd;
class tEmSetServoParametersCommand;

const ESM_OFF = 0;
const ESM_TRACK = 1;

extern const int CiEmDataBufferSize;

/****************************************************************************

tEmGizmo Black Box State

BOOL			        bInit
ERROR_CODE			ecInitError
tGizmoBlackBoxServo		Servo - (Gizmo Specification Document)
tPeriodicTargetGenerator        TargetGenerator - (see periodicTargetGen.h)
tDlActuator                     DlActuator - (see dlActuator.h)
tEmStatus                       Status
tEmData                         Data
tTypeTag                        CEmStatusTag
tTypeTag                        CEmDataTag

tEmOffController                OffController
tEmTrackController              TrackController

TASK_ID                         tidCommandTaskId
TASK_ID                         tidStatusTaskId
TASK_ID                         tidServoTaskId
TASK_ID                         tidDataTaskId

*****************************************************************************

[[D tFtGizmo::tFtGizmo_SIMPLE (const tEmGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  IsADelayLine (GizmoInfo.iDlTargetGizmoId) == FALSE ||
  ::IpcMemoryG.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 2 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 1 ||
  GizmoInfo.gtType != GT_EXTERNAL_METROLOGY ||
  CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, 
       Gizmo (GizmoInfo.iDelayLineTargetGizmoId).EmTargetId) == FALSE ||
  Unable to add periodic tasks to handle command, status and servo =>
    this->ecInitError := appropriate error code
    : *this except this->ecInitError := anything
    : console := console + appropriate error messages
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Status := default status 
    : this->CEmStatusTag := appropriate unique type tag for EM status
    : this->Servo.Mode := ESM_OFF
    : this->Servo.ControllerSet := appropriate references to controllers from 
      this->CompleteControllerSet 
    : this->CompleteControllerSet := appropriate references to i/o from 
      this->CompleteIoSet and initial servo parameters from GizmoInfo
    : this->CompleteIoSet := appropriate references to I/O channel pointers from 
      GizmoInfo and initial servo parameters from GizmoInfo
    : appropriate IPC objects for command, and open loop DL target 
      added to ::IpcMemory
    : command, status, and servo tasks added to ::PeriodicTaskSet
    : data dump task added to ::EventDrivenTaskSet
]]
 
    
[[D tEmGizmo::tEmGizmo (const tEmGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  IsADelayLine (GizmoInfo.iDlTargetGizmoId) == FALSE ||
  ::IpcMemoryG.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 2 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 1 ||
  GizmoInfo.gtType != GT_EXTERNAL_METROLOGY ||
  CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG, 
       Gizmo (GizmoInfo.iDelayLineTargetGizmoId).EmTargetId) == FALSE ||
  Unable to add periodic tasks to handle command, status and servo =>
    this->ecInitError := appropriate error code
    : *this except this->ecInitError := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->CEmStatusTag.stType := ST_STATUS
    : this->CEmStatusTag.tnName := TN_EM_STATUS
    : this->CEmStatusTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CEmStatusTag.szBlockSize := sizeof (this->Status)
    : this->Status := anything

    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U {
      IpcObject (::IpcMemoryG, GizmoInfo.iiCommandStreamId),
      IpcObject (::IpcMemoryG, Gizmo (GizmoInfo.iDelayLineTargetGizmoId).EmTargetId) }
 
    : ::PeriodicTaskSet := ::PeriodicTaskSet U 
      PeriodicTask (GizmoInfo.iGizmoId + "_CommandProcessor", GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority, GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask (GizmoInfo.iGizmoId + "_StatusRelayer", GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority, GizmoInfo.tStatusTaskOffset,
        tGizmo::ProcessStatus) U
      PeriodicTask ("0_" + GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[0].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[0].iTaskPriority, 0,
        tServo::DoServo)

    : ::EventDrivenTaskSet := ::EventDrivenTaskSet U 
       EventDrivenTask ("DataStoreDump",CiEmDataDumpPriority,
         DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream )

    : this->tidCommandTaskId := new unique periodic task id
    : this->tidStatusTaskId := new unique periodic task id
    : this->Servo.tidServoTaskIdArray [0] := 
      new unique periodic task id
    : this->tidDataTaskId := new unique event driven task id
    : this->Servo.Mode := ESM_OFF

    : this->Servo.ControllerSet := { 
      (ESM_OFF, 0, 0, &this->CompleteControllerSet.Off),
      (ESM_TRACK, 0, 0, &this->CompleteControllerSet.Track) }

    : this->CompleteControllerSet.Off.OpenLoopDl :=  
      reference to this->CompleteIoSet.OpenLoopDl
      
    : this->CompleteControllerSet.Track.TargetGenerator :=   
      reference to this->CompleteIoSet.TargetGenerator
    : this->CompleteControllerSet.Track.OpenLoopDl :=  
      reference to this->CompleteIoSet.OpenLoopDl
      
    : this->TargetGenerator := default modulation parameters
    : this->DlActuator.iDlGizmoId := Gizmo (GizmoInfo.iDelayLineTargetGizmoId).EmTargetId)
]]  


[[D tEmGizmo::~tEmGizmo (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true =>    
    *this := anything
    : ::PeriodicTaskSet := ::PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId) -
      PeriodicTask (this->Servo.tidServoTaskIdArray [0])
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet - 
      EventDrivenTask (this->tidDataTaskId)
]]

[[D tEmGizmo::Restart (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to resume  PeriodicTask (this->tidCommandTaskId) ||
  Unable to resume  PeriodicTask (this->tidStatusTaskId) ||
  Unable to resume  PeriodicTask (this->tidServoTaskId) ||
  Unable to resume  EventDrivenTask (this->tidDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) resumed
    : PeriodicTask (this->tidStatusTaskId) resumed
    : PeriodicTask (this->tidServoTaskId) resumed
    : EventDrivenTask (this->tidDataTaskId) resumed
    : this->bInit := TRUE
    : return := OK
]]

[[D tEmGizmo::Shutdown (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspended PeriodicTask (this->tidCommandTaskId) ||
  Unable to suspended PeriodicTask (this->tidStatusTaskId) ||
  Unable to suspended PeriodicTask (this->tidServoTaskId) ||
  Unable to suspended EventDrivenTask (this->tidDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) suspended
    : PeriodicTask (this->tidStatusTaskId) suspended
    : PeriodicTask (this->tidServoTaskId) suspended
    : EventDrivenTask (this->tidDataTaskId) suspended
    : this->bInit := FALSE
    : return := OK
]]

[[D tEmGizmo::IsOk(void) ]
[ invalid_pointer (this) => UNDEFINED
| this.ecInitError != OK => return := FALSE
| true => return := TRUE
]]

********************************************************************************
EM Command Specification

[[D tEmGizmo::Ping (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true =>  ::ErrorStream :=  ::ErrorStream + appropriate message 
]]

[[D tEmGizmo::Off (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->Servo.Mode := ESM_OFF
    : this->CompleteIoSet.DlActuator.Target := ZERO_TARGET
    : ::ErrorStream  := ::ErrorStream + appropriate message 
]]
	
	
[[D tEmGizmo::Track (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::DataStream.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true => 
    this->Servo.Mode := ESM_TRACK
    : this->CompleteIoSet.DlActuator.Target := appropriate target generated by 
      this->CompleteIoSet.TargetGenerator based on the current time
    : ::ErrorStream :=  ::ErrorStream + 
      appropriate message 
]]

[[D tEmGizmo::SetServoParameters (const tEmServoParams& ParamsToUse) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this.bInit==FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->CompleteIoSet.TargetGenerator.mtModType := ParamsToUse.mtModType
    : this->CompleteIoSet.TargetGenerator.dAmplitude := ParamsToUse.dAmplitude
    : this->CompleteIoSet.TargetGenerator.dFrequency := ParamsToUse.dFrequency
    : this->CompleteIoSet.TargetGenerator.dOffset := ParamsToUse.dOffset
    : this->CompleteIoSet.TargetGenerator.tSyncTime := ParamsToUse.tSyncTime	
    : ::ErrorStream := ::ErrorStream + appropriate message 
]]
	

**********************************************************************************************
Protected Specification

[[D tEmGizmo::Dispatch (const tIsitCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (Command) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| Command.Command == EM_CMD_PING => 
    ::ErrorStream := ::ErrorStream + appropriate message
    : return := OK 
| Command.Command == EM_CMD_OFF => 
    this->Servo.Mode := ESM_OFF
    : this->CompleteIoSet.DlActuator.Target := ZERO_TARGET
    : ::ErrorStream :=  ::ErrorStream + appropriate message 
    : return := OK 
| Command.Command == EM_CMD_TRACK => 
    this->Servo.Mode := ESM_TRACK
    : this->CompleteIoSet.DlActuator.Target := appropriate target generated by 
      this->TargetGenerator based on the current time
    : ::ErrorStream := ::ErrorStream + appropriate message 
    : return := OK 
| Command.Command == EM_CHNG_PARAM => 
    this->CompleteIoSet.TargetGenerator.mtModType := Command.mtModType
    : this->CompleteIoSet.TargetGenerator.dAmplitude := Command.dAmplitude
    : this->CompleteIoSet.TargetGenerator.dFrequency := Command.dFrequency
    : this->CompleteIoSet.TargetGenerator.dOffset := Command.dOffset
    : this->CompleteIoSet.TargetGenerator.tSyncTime := Command.tSyncTime	
    : ::ErrorStream := ::ErrorStream + appropriate message 
| true => return := appropriate error code
]]


[[D tEmGizmo::CollectStatus (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::StatusStream.bInit == FALSE => UNDEFINED
| IsFull (::StatusStream ) =>
    ::ErrorStream := ::ErrorStream + appropriate error code
    : return := appropriate error code
| true =>
    ::StatusStream := ::StatusStream + this->CEmStatusTag + this->Status
    : return := OK
]]  


*************************Private Specification*********************************************************************



[[D tEmGizmo::Ping (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| true =>  
    ::ErrorStream :=  ::ErrorStream + appropriate message 
    : return := OK
]]

[[D tEmGizmo::Off (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| true => 
    this->Servo.Mode := ESM_OFF
    : this->CompleteIoSet.DlActuator.Target := ZERO_TARGET
    : ::ErrorStream  := ::ErrorStream + appropriate message 
    : return := OK
]]
	
	
[[D tEmGizmo::Track (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::DataStream.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| true => 
    this->Servo.Mode := ESM_TRACK
    : this->CompleteIoSet.DlActuator.Target := appropriate target generated by 
      this->CompleteIoSet.TargetGenerator based on the current time
    : ::ErrorStream := ::ErrorStream + appropriate message 
    : return := OK
    
]]


[[D tEmGizmo::SetServoParameters (const tEmServoParams& ParamsToUse) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| true => 
    this->CompleteIoSet.TargetGenerator.mtModType := ParamsToUse.mtModType
    : this->CompleteIoSet.TargetGenerator.dAmplitude := ParamsToUse.dAmplitude
    : this->CompleteIoSet.TargetGenerator.dFrequency := ParamsToUse.dFrequency
    : this->CompleteIoSet.TargetGenerator.dOffset := ParamsToUse.dOffset
    : this->CompleteIoSet.TargetGenerator.tSyncTime := ParamsToUse.tSyncTime	
    : ::ErrorStream := ::ErrorStream + appropriate message 
    : return := OK
]]
	


****************************************************************************/


class tEmGizmo : public tRealtimeGizmo {

  public:
    tEmGizmo (const tEmGizmoInfo &GizmoInfo);
    virtual ~tEmGizmo (); 
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
    friend ostream& operator<< (ostream& stream, tEmGizmo& Gizmo);

  protected:
    virtual ERROR_CODE CollectStatus ();
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
 
  private:
    ERROR_CODE Ping ();
    ERROR_CODE Off ();
    ERROR_CODE Track ();
    ERROR_CODE SetServoParameters (tEmSetServoParametersCmd& Command);
    
    const tTypeTag CEmStatusTag;
    const tTypeTag CEmDataTag;    
    tEmCompleteIoSet CompleteIoSet; 
    tEmCompleteControllerSet CompleteControllerSet;
   
    tEmStatus Status;    
    tSimpleControllerGroup EmOffControllerGroup;
    tSimpleControllerGroup EmTrackControllerGroup;  
};

#endif INC_emGizmoh
