/* $RCSfile: emTypes.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/emTypes.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a29Aug97,rlj    written
*/

#ifndef INC_emTypesh
#define INC_emTypesh


#include "isit.h"
#include "modLib.h"

typedef enum EM_COMMAND_STATUS {
  EMCS_OFF,
  EMCS_TRACK_ACTIVE,
  EMCS_SERVO_PARAMETERS_SET} EM_COMMAND_STATUS;

typedef enum EM_SERVO_STATE {
  ESS_OFF, ESS_TRACK } EM_SERVO_STATE;

/***
[[D tEmServoParameters::tEmServoParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tEmServoParameters::tEmServoParameters_t (
      const tModulationParams& ModulationParameters) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->TrackParameters := ModulationParameters
]]

[[D tEmServoParameters::tEmServoParameters_MTdd (MOD_TYPE mtType, 
                        double dAmplitude, double dFrequency) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->TrackParameters.mtModulationType := mtType
    : this->TrackParameters.dAmplitude := dAmplitude
    : this->TrackParameters.dFrequency := dFrequency
    : this->TrackParameters.dMeanVal := 0
    : this->TrackParameters.tSyncTime := 0
]]

[[D tEmServoParameters::tEmServoParameters_MTdddt (MOD_TYPE mtType, 
                        double dAmplitude, double dFrequency, 
                        double dMeanVal, TIME tSyncTime) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->TrackParameters.mtModulationType := mtType
    : this->TrackParameters.dAmplitude := dAmplitude
    : this->TrackParameters.dFrequency := dFrequency
    : this->TrackParameters.dMeanVal := dMeanVal
    : this->TrackParameters.tSyncTime := tSyncTime
]]


[[D tEmServoParameters::GetModulationParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->TrackParameters
]]

[[D tEmStatus::tEmStatus (int iGizmoIdToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iGizmoId := iGizmoIdToUse
]]


[[D tEmStatus::UpdateCommandStatus (EM_COMMAND_STATUS ecsCommandStatus) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->ecsCommandStatus := ecsCommandStatus
]]

[[D tEmStatus::UpdateServoState (EM_SERVO_STATE essServoState) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->essServoState := essServoState
]]


****/



class tEmServoParameters  {
  public:
    tEmServoParameters () {};
    tEmServoParameters (const tModulationParams& ModulationParameters) :
        TrackParameters (ModulationParameters) {}                 
    tEmServoParameters (MOD_TYPE mtType, double dAmplitude, double dFrequency, 
                        double dMeanVal=0, TIME tSyncTime=0) : 
        TrackParameters (mtType,dAmplitude,dFrequency,dMeanVal,tSyncTime) {}                 
  
    const tModulationParams& GetModulationParameters () const { return TrackParameters; }

  protected:
    tModulationParams TrackParameters;
};


class tEmStatus_Version_1 {
  public:
    tEmStatus_Version_1 () {
      this->ecsCommandStatus = EMCS_OFF;
      this->essServoState = ESS_OFF;
    }
    int iVersion;
    int iGizmoId;
    TIME tTime;
    tModulationParams TrackParameters;
    EM_COMMAND_STATUS  ecsCommandStatus;
    EM_SERVO_STATE     essServoState;
};

class tEmData_Version_1 {
  public:
    int iVersion;
    int iGizmoId;
    TIME tTime;
    tDlTargetBase Target;
};

#define CURRENT_EM_STATUS_VERSION 1
#define CURRENT_EM_DATA_VERSION   1

#define tEmStatusCurrentVersion \
   EXPAND_AND_CONCAT(tEmStatus_Version_, CURRENT_EM_STATUS_VERSION)
#define tEmDataCurrentVersion \
   EXPAND_AND_CONCAT(tEmData_Version_, CURRENT_EM_DATA_VERSION)



class tEmStatus: public tEmStatusCurrentVersion {
  public:
    tEmStatus () { iVersion = CURRENT_EM_STATUS_VERSION; }
    tEmStatus (int iGizmoId) { iVersion = CURRENT_EM_STATUS_VERSION;}
    void UpdateCommandStatus (EM_COMMAND_STATUS ecsCommandStatus) {
       this->ecsCommandStatus = ecsCommandStatus; }
    void UpdateServoState (EM_SERVO_STATE essServoState) {
       this->essServoState = essServoState; }
    void UpdateServoParameters (const tEmServoParameters& 
				  ServoParametersToUse) { 
      this->TrackParameters = ServoParametersToUse.GetModulationParameters ();}
};

class tEmData : public tEmDataCurrentVersion {
  public:
    tEmData () { iVersion = CURRENT_EM_DATA_VERSION;}
    tEmData (int iGizmoIdToUse) { iVersion = CURRENT_EM_DATA_VERSION; 
                                  iGizmoId = iGizmoIdToUse;}
};

#endif INC_emTypesh



