/* $RCSfile: extInt.h,v $ - External Interface routines -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/extInt.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,14Aug97,ky       Initial creation
*/

/*
DESCRIPTION
  External Interface header 
*/


#include <vxWorks.h>
#include <sockLib.h>
#include <inetLib.h>
#include <taskLib.h>
#include <stdioLib.h>
#include <strLib.h>
#include <ioLib.h>
#include <fioLib.h>
#include <time.h>

#include "isit.h"
#include "tList.h"
#include "isitCommand.h"
#include "configMgr.h"


#define SERVER_PORT_NUM 5001
#define MAXIMUM_PORT_NUM 9001
#define SERVER_WORK_PRIORITY 90
#define SERVER_STACK_SIZE 32768 
#define SERVER_MAX_CONNECTIONS (5)
#define S_extInt_ALREADY_RESTART    (M_extInt | 0x1001)
#define S_extInt_ALREADY_SHUTDOWN   (M_extInt | 0x1002)


#ifndef INC_EXTINTh
#define INC_EXTINTh

#include "baseObject.h"

/****************************************************************************

tExternalInterface Black Box State

BOOL           bInit
ERROR_CODE     ecInitError
set of UI_ID   UiSet

-----------------------------------------------------------------------------

[[D tExternalInterface::tExternalInterface  (tCmdHdlr& CommandHdlrToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (CommandHdlrToUse) => UNDEFINED
| Unable to allocate or initialize resources to communicate with 
  external interfaces =>
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->CommandHdlr := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->UiSet := NULL SET 
    : this->CommandHdlr := CommandHdlrToUse
]]


[[D tExternalInterface::~tExternalInterface (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
  *this := anything
]]

[[D tExternalInterface::IsOk (void) ]
[ this is invalid => UNDEFINED
| Object is malfunctioning => return FALSE
| true => return := TRUE
]]

[[D tExternalInterface::Restart(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate messages
    : return := OK
| Unable to resume user interface communication =>
    this->ecInitError:=appropriate error code
    : console := console + appropriate messages
    : return := ERROR
| true =>
    this->bInit := TRUE 
    : return := OK 
]]

[[D tExternalInterface::Shutdown(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate messages
    : return := OK
| Unable to resume user interface communication =>
    this->ecInitError:=appropriate error code
    : console := console + appropriate messages
    : return := ERROR
| true =>
    this->bInit := FALSE 
    : return := OK 
]]

[[D tExternalInterface::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED 
| T.E element id of this->UiSet s.t. external interface cannot send data status 
  or error to or receive command from UserInterface (id)=> return := FALSE
| true => return := TRUE
]]


[[D tExternalInterface::SendError(const tTypeTag& Tag, const BYTE *pError) ]
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pError) => UNDEFINED
| this->bInit == FALSE => 
  return := ERROR
| lClientSet has Connection node with bDisconnected == TRUE => 
  Delete that Connecton node
| true =>
    Tag and pError sent to all UserInterface(id)
    : return appropriate error code
]]

[[D tExternalInterface::SendData(const tTypeTag& Tag, const BYTE *pData) ]
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pData) => UNDEFINED
| this->bInit == FALSE => 
  return := ERROR
| lClientSet has Connection node with bDisconnected == TRUE => 
  Delete that Connecton node
| true =>
    Tag and pError sent to all UserInterface(id)
    : return appropriate error code
]]

[[D tExternalInterface::SendStatus(const tTypeTag& Tag, const BYTE *pStatus) ]
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pStatus) => UNDEFINED
| this->bInit == FALSE => 
  return := ERROR
| lClientSet has Connection node with bDisconnected == TRUE => 
  Delete that Connecton node
| true =>
    Tag and pError sent to all UserInterface(id)
    : return appropriate error code
]]

******************************************************************************/

class tCommandHandler;
class tTypeTag;

class tExternalInterface : public tBaseObject
{
public:
 tExternalInterface (tCommandHandler& CommandHdlrToUse);
 ~tExternalInterface();
 BOOL       IsOk(void);
 ERROR_CODE Restart(void);
 ERROR_CODE Shutdown(void);
 ERROR_CODE SendError(const tTypeTag& Tag, const BYTE *pError);
 ERROR_CODE SendData(const tTypeTag& Tag, const BYTE *pData);
 ERROR_CODE SendStatus(const tTypeTag& Tag, const BYTE *pStatus);

protected:
 tList lClientSet;
 tCommandHandler &CommandHdlr;
 int iSockFd;
 int iPortUsed;
 int iAcceptConnTaskId;
 SEM_ID siCommandSendMutex;
 ERROR_CODE EstablishConnection(void);
 ERROR_CODE WaitForConnection(void);
};

#endif INC_EXTINTh
