/* $RCSFile$  -- structure header for feedforward.c */

/*
$Header: /home/chara/cvsroot/ople/sys/include/feedforward.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/

/*
modification history
--------------------
01a,25Jun97,beh          written
*/



#ifndef INC_feedforwardh
#define INC_feedforwardh


/***** Specification Functions ********************

double FEEDFORWARD (tFeedforward FeedForward, double x) :==
[ return Feedforward.dGain * x
]

**************** tFeedforward Stimuli **********************

[[D tFeedforward::tFeedforward (void) ]
Construct feedforward with default 0 gain.
[ invalid_pointer (this) => UNDEFINED
| true => this->dGain := 0.0
]]

[[D tFeedforward::tFeedforward_d (double dGainIn) ]
Construct feedforward with specified gain dGainIn.
[ invalid_pointer (this) => UNDEFINED
| true => this->dGain := dGainIn
]]


[[D tFeedforward::Initialize (double dGainIn) ]
Set feedforward gain to dGainIn. (for back compatibility)
[ invalid_pointer (this) => UNDEFINED
| true => this->dGain := dGainIn
]]

[[D tFeedforward::Gain (void) ]
Return gain or reference to gain
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return this->dGain
| true => return := reference to this->dGain
]]

[[D tFeedforward::Calculate (double dInputValue) ]
Return feed forward compensated value for dInputValue
[ invalid_pointer (this) => UNDEFINED
| true => return := FEEDFORWARD (*this, dInputValue)
]]

******/

class tFeedforward {
  public:
    tFeedforward () { this->dGain = 0.0; }
    tFeedforward (double dGainIn) { this->dGain = dGainIn; }
    void Initialize (double dGainIn) { this->dGain = dGainIn; }
    double Gain () const { return this->dGain; }
    double& Gain () { return this->dGain; }
    double Calculate (double dInputVal) { return dInputVal * this->dGain; }
  private:
    double dGain;
};   

#endif INC_feedforwardh

