/* $RCSFile$  -- structure header for filtLibF.c */

/*
$Header: /home/chara/cvsroot/ople/sys/include/filtLibF.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/

/*
modification history
--------------------
01a,25Jun97,eamck		adapted from Palomar filtLibF
*/



#ifndef INC_filtLibFh
#define INC_filtLibFh

/*************** Specification Functions **********************

double LOW_PASS (tFilter Filter, double x) :==
[ return := Filter.dMemoryWeight * Filter.dDecayingMemory + 
      Filter.dSampleWeight * x
]

double HIGH_PASS (tFilter Filter, double x) :==
[ return := x - Filter.dMemoryWeight * Filter.dDecayingMemory + 
      Filter.dSampleWeight * x
]

double UpdateFilter (tFilter Filter, double x) :==
[ return := tFilter with
    dDecayingMemory := Filter.dMemoryWeight * Filter.dDecayingMemory + 
      Filter.dSampleWeight * x;
]



************* tFilter Stimuli *********************************
[[D tFilter::tFilter (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tFilter::tFilter_d (double dMemoryInSamples) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dDecayingMemory := 0.0
    : this->dSampleWeight := 1.0 / dMemoryInSamples
    : this->dMemoryWeight := 1.0 - 1.0 / dMemoryInSamples
]]


[[D tFilter::~tFilter (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tFilter::Initialize (double dMemoryInSamples) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dDecayingMemory := 0.0
    : this->dSampleWeight := 1.0 / dMemoryInSamples
    : this->dMemoryWeight := 1.0 - 1.0 / dMemoryInSamples
]]


[[D tFilter::Reset (double dResetVal) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->dDecayingMemory := dResetVal
]]

[[D tFilter::LowPass (double dInputVal) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dDecayingMemory := this->dMemoryWeight * this->dDecayingMemory + 
      this->dSampleWeight * dInputVal;
    : return := this->dMemoryWeight * this->dDecayingMemory + 
      this->dSampleWeight * dInputVal
]]


[[D tFilter::HighPass (double dInputVal) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
   this->dDecayingMemory := this->dMemoryWeight * this->dDecayingMemory + 
     this->dSampleWeight * dInputVal
    : return := dInputVal - this->dMemoryWeight * this->dDecayingMemory + 
      this->dSampleWeight * dInputVal;
]]
*****/


class tFilter {
  public:
    tFilter () {}
    tFilter (double dMemoryInSamples);
    void Initialize (double dMemoryInSamples);
    void Reset (double dResetVal);
    double LowPass (double dInputVal);
    double HighPass (double dInputVal);
  private:
    double dDecayingMemory;        // Decaying memory of previous samples
    double dMemoryWeight;
    double dSampleWeight;
  
};   /* class tFilter */

#endif INC_filtLibFh

