/* $RCSfile: fringe.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/ft/fringe.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/

#ifndef INC_fringeh
#define INC_fringeh

/****
Black Box State Variables
-------------------------------------- 
typedef int[][4]  tPhotonLog     This is a record of all detector counts that 
                                 have been received since the last reset or 
                                 construction.  The abstraction is infinitely 
                                 large.  The first index is the sweep number of 
                                 the detector count, formed by dividing the time
                                 of the count by CtReadingPeriod and then by 
                                 QUADRANTS and taking the floor of the result. 
                                 The second index correspons to a dither 
                                 quadrant.
                                 
tPhotonLog        PhotonLog      
int               iCountArray[2] Last 8 detector counts, in a double buffer
int               iInputBuffer   Side of iCountArray currently receiving input
double            dDarkCurrent   Dark current expected for 1 fringe tracker period
int               iLastIndex     Last time index used to store data in PhotonLog
int               CiSampleSize   Number of samples in average of photon count or power
int               CiWobbleBins   Number of data points in wobble modulation
TIME              CtFTPeriod     Fringe tracker period (1/2 dither period)
double            WobblePositionArray[CiWobbleBins]
                                 Total of wobble positions for each bin
int               iWobbleSamplesArray[CiWobbleBins]
                                 Number of samples in each wobble bin


SPECIFICATION FUNCTIONS

//returns the total number of photons above dark current for the sweep
//specified by "index"
double Photons(int index,tPhotonLog ALog,double dark) :== 
[ return := ALog[index][0] + ALog[index][1] + ALog[index][2] + ALog[index][3] - dark

//returns the fringe power (NUM) for the sweep specified by "index"
double Power(int index,tPhotonLog ALog,double dark) :== 
[ return := (ALog[index][1] - ALog[index][3])^2 + 
            (ALog[index][0] - ALog[index][2])^2 -
            Photons(index,ALog,dark)
]

//returns the average photons above dark current over the previous "size" 
//sweeps, starting with the sweep specified by "index"
double AvgPhotons(int index,int size,tPhotonLog ALog,double dark) :== 
[ return :=  average of Photons(i,ALog,dark) for i in [index - size + 1,index]
]

//returns the average fringe power (NUM) over the previous "size" 
//sweeps, starting with the sweep specified by "index"
double AvgPower(int index,int size,tPhotonLog ALog,double dark) :== 
[ return :=  average of Power(i,ALog,dark), for (index - size + 1) <= i <= index
]

//returns the average photons above dark current over all sweeps in the
//wobble bin specified by "bin", when there are "size" bins in all
double WobbleAvgPhotons(int bin,int size,tPhotonLog ALog,double dark) :== 
[ bin < 0 || bin > size => return := anything
| true => 
   return :=  average of Photons(bin + i * size,ALog,dark) for all i
]

//returns the average fringe power (NUM) over all sweeps in the
//wobble bin specified by "bin", when there are "size" bins in all
double WobbleAvgPower(int bin,int size,tPhotonLog ALog,double dark) :== 
[ bin < 0 || bin > size => return := anything
| true => 
   return :=  average of Power(bin + i * size,ALog,dark) for all i containing data
]
   
END OF SPECIFICATION FUNCTIONS


[[D tFringe::tFringe(int iSampleSize, int iWobbleBins,TIME tFTPeriod) ]
[ invalid_pointer (this) ||
  invalid_Reference (Wobble) ||
  iSampleSize <= 0 ||
  iWobbleBins <= 0 => UNDEFINED
| true =>
   this->PhotonLog := empty 
   : this->WobblePositionArray := empty
   : this->CiSampleSize := iSampleSize
   : this->CiWobbleBins := iWobbleBins
   : this->CtFTPeriod := MAX(tFTPeriod,1)
   : this->iLastIndex := 0
   : this->dDarkCurrent := 0
   : this->WobblePositionArray[i] := 0, for 0 < i < iWobbleBins
   : this->iWobbleSamplesArray[i] := 0, for 0 < i < iWobbleBins
]]

[[D tFringe::~tFringe(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   destroy this->PhotonLog
]]

[[D tFringe::AcceptCount(TIME tTime,int iCount) ]
[ invalid_pointer (this) => UNDEFINED
| iCount < 0 => this->iCountArray := anything
| (Time / (FTPeriod / 4) - 1) % 4 != 3 =>                                                         
   this->iCountArray[iInputBuffer][(Time / (FTPeriod / 4) - 1) % 4] := iCount
| true =>
   this->iCountArray[iInputBuffer][(Time / (FTPeriod / 4) - 1) % 4] := iCount
   : iInputBuffer := !iInputBuffer
   : return := OK
]]

[[D tFringe::Calculate(TIME tTime) ]
[ invalid_pointer (this) => UNDEFINED
| Time / FTPeriod % 2 == 0 =>
   for 0 <= quadrant <= 3
      this->PhotonLog[Time / this->CtFTPeriod][quadrant] := 
         iCountArray[!iInputBuffer][quadrant];
   : iLastIndex := Time / this->CtFTPeriod
   : return := OK
| true =>                                                         
   for 0 <= quadrant <= 3
      this->PhotonLog[Time / this->CtFTPeriod][quadrant] := 
         iCountArray[!iInputBuffer][3 - quadrant];
   : iLastIndex := Time / this->CtFTPeriod
   : return := OK
]]

[[D tFringe::AcceptWobble(TIME tTime,double dPosition) ]
[ invalid_pointer (this) => UNDEFINED
| this->CiWobbleBins == 0 => return := OK
| true =>                                                         
   WobblePositionArray [Time / this->CtFTPeriod % this->CiWobbleBins] += dPosition
   : IWobbleSamplesArray[Time / this->CtFTPeriod % this->CiWobbleBins]++
   : return := OK
]]

[[D tFringe::SetDarkCurrent(double dDark) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   this->dDarkCurrent := dDark
   : return := OK
]]

[[D tFringe::Reset(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   this->PhotonLog := empty
   : this->WobblePositionArray[i] := 0, for 0 < i < CiWobbleBins
   : this->iWobbleSamplesArray[i] := 0, for 0 < i < CiWobbleBins
   : this->bDiscardCounts := TRUE
   : return := OK
]]


[[D tFringe::Photons(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := Photons(this->iLastIndex,this->PhotonLog,this->dDarkCurrent)
]]

[[D tFringe::Visibility2(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := (PI^2 / 2) * 
             Power(this->iLastIndex,this->PhotonLog,this->dDarkCurrent) /
             Photons(this->iLastIndex,this->PhotonLog,this->dDarkCurrent)^2
]]

[[D tFringe::Snr2(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
  return := 2 * 
             Power(this->iLastIndex,this->PhotonLog,this->dDarkCurrent) / 
             Photons(this->iLastIndex,this->PhotonLog,this->dDarkCurrent)
]]

[[D tFringe::Phase(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := atan2(this->PhotonLog[this->iLastIndex][0] - 
                   this-<PhotonLog[this->iLastIndex][2],
                   this->PhotonLog[this->iLastIndex][1] - 
                   this->PhotonLog[this->iLastIndex][3])
]]

[[D tFringe::EstVisibility2(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->PhotonLog empty => return := 0.0
| true =>
   return := (PI^2 / 2) * 
             AvgPower(this->iLastIndex,this->CiSampleSize,this->PhotonLog,
                      this->dDarkCurrent) / 
             AvgPhotons(this->iLastIndex,this->CiSampleSize,this->PhotonLog,
                        this->dDarkCurrent)^2
]]

[[D tFringe::EstSnr2(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->PhotonLog empty => return := 0.0
| true =>
   return := (PI^2 / 4) * 
             AvgPower(this->iLastIndex,this->CiSampleSize,this->PhotonLog,
                      this->dDarkCurrent) / 
             AvgPhotons(this->iLastIndex,this->CiSampleSize,this->PhotonLog,
                        this->dDarkCurrent)
]]

[[D tFringe::WobbleVisibility2(double Visibility2Array[],
                               double PositionArray[],
                               int iSize,
                               double* pSnr2) ]
[ invalid_pointer (this) => UNDEFINED
| iSize < CiWobbleBins => return := ERROR
| this->PhotonLog empty => 
   for 0 <= bin <= CiWobbleBins
      Visibility2Array[bin] := 0.0
   : PositionArray[bin] := 0.0   
| this->CiWobbleBins == 0 => return := OK
| true =>
   for 0 <= bin <= CiWobbleBins
      Visibility2Array[bin] := (PI^2 / 2) * 
                  WobbleAvgPower(bin,this->CiWobbleBins,this->PhotonLog,
                                 this->dDarkCurrent) / 
                  WobbleAvgPhotons(bin,this->CiWobbleBins,this->PhotonLog,
                                   this->dDarkCurrent)^2
   : PositionArray[bin] := this->WobblePositionArray[bin] / 
      this->iWobbleSamplesArray[bin]
   : this->WobblePositionArray[i] := 0, for 0 < i < iWobbleBins
   : this->iWobbleSamplesArray[i] := 0, for 0 < i < iWobbleBins
   : *pSnr2 = (PI^2 / 4) * 
      (sum over 0 < bin < iWobbleBins, 
       of WobbleAvgPower(bin,this->CiWobbleBins,this->PhotonLog,
                         this->dDarkCurrent)) / 
      (sum over 0 < bin < iWobbleBins, 
       of WobbleAvgPhotons(bin,this->CiWobbleBins,this->PhotonLog,
                           this->dDarkCurrent)^2)
   : return := OK
]]

[[D tFringe::CalculateVisibility2(double dPower,double dPhotons) ]
[ invalid_pointer (this) => UNDEFINED
| this->PhotonLog empty => return := 0.0
| true =>
   return := (PI^2 / 2) * dPower / dPhotons^2
]]

[[D tFringe::CalculateSnr2(double dPower,double dPhotons) ]
[ invalid_pointer (this) => UNDEFINED
| this->PhotonLog empty => return := 0.0
| true =>
   return := (PI^2 / 4) * dPower / dPhotons
]]

[[D tFringe::InPhaseComponent() ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->PhotonLog[this->iLastIndex][1] - 
             this->PhotonLog[this->iLastIndex][3]
]]

[[D tFringe::QuadratureComponent() ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->PhotonLog[this->iLastIndex][0] - 
             this-<PhotonLog[this->iLastIndex][2]
]]

***/

class tFringe {
   public:
      tFringe(int iSampleSize, int iWobbleBins,TIME tFTPeriod);
      ~tFringe();
      ERROR_CODE AcceptCount(TIME tTime,int iCount);//store photon count and time
      ERROR_CODE AcceptWobble(TIME tTime,double dPosition);//accepts wobble position
      ERROR_CODE tFringe::Calculate(TIME tTime);   //process last sweep
      ERROR_CODE SetDarkCurrent(double dDark);     //sets the dark current
      ERROR_CODE Reset();                          //discards all statistics
      ERROR_CODE DiscardCounts ();           //discards current counts
      ERROR_CODE StoreOnly ();               // Disables photon counting checks
      ERROR_CODE CheckCounts ();             // Enables photon counting checks  
      double Photons() const;                      //returns last total
      double A () const;
      double B () const;
      double C () const;
      double D () const;
      double Visibility2() const;                  //returns last V^2
      double Snr2() const;                         //returns last SNR^2
      double Phase() const;                        //returns last simple phase
      double EstVisibility2() const;               //returns V^2 estimate
      double EstSnr2() const;                      //returns SNR^2 estimate
      int InPhaseComponent() const;                //returns last X
      int QuadratureComponent() const;             //returns last Y
      ERROR_CODE WobbleVisibility2(
         double Visibility2Array[],                //returns Wobble estimates
         double PositionArray[],
         int iSize,
         double* pSnr2) const;
   private:
      static const int QUADRANTS = 4;  //readings per sweep (forever fixed)

      tFringe(const tFringe& f);                   //inadequate/unnecessary
      double CalculateVisibility2(double dPower,   //returns V^2
                                  double dPhotons) const;
      double CalculateSnr2(double dPower,          //returns SNR^2
                           double dPhotons) const;
                           
      unsigned long int iCountArray[2][QUADRANTS]; 
                                       //photon counts, by time, double-buffered
      unsigned long int iLastCountArray[QUADRANTS];
                                       //last photon counts, by DL position
      int iInputBuffer;                //current input buffer
      double *PhotonsArray;            //last iSampleSize Photons's
      double *PowerArray;              //last iSampleSize Power's
      double dTotalPhotons;            //total of last iSampleSize Photons's
      double dTotalPower;              //total of last iSampleSize Power's
      int iLastIndex;                  //last index to above arrays
      int iSampleCount;                //number of valid samples in above arrays
      const int CiSampleSize;          //size of above arrays
      
      unsigned long int *WobbleSamplesArray; //iWobbleBin sample sizes
      double* WobblePhotonsArray;      //iWobbleBin Photons totals
      double* WobblePowerArray;        //iWobbleBin Power totals
      unsigned long int *PositionSamplesArray;   
                                       //iWobbleBin position sample sizes
      double* WobblePositionArray;     //iWobbleBin wobble-position totals
      const int CiWobbleBins;          //size of above arrays
      
      double dDarkCurrent;             //Dark current for Photons
      const TIME CtReadingPeriod;      //time between APD readings

      TIME tLastTime;                  //time of last APD reading
      BOOL bNewData;                   //last sweep has not been processed
      BOOL bDiscardCounts;             //discard next set of photon counts
      BOOL bStoreOnly;                 //disable check for underprocessed counts
   };


#endif
