/* $RCSfile: ftFastController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/ft/ftFastController.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a7Nov97,rlj    written
*/

#ifndef ftFastControllerh
#define ftFastControllerh

#include "isit.h"
#include "ft/ftController.h"

#include "ringBuffer.h"

struct tCount {
  tCount () {};
  tCount (double d, TIME t) { count = d; time = t;}
  double count;
  TIME time;
};

/******  FT FAST CONTROLLER ***********************

[[D tFtFastController::tFtFastController (tFtCompleteIoSet& CompleteIoSet) ]
[ invalid_pointer (this) ||
  invalid_reference (CompleteIoSet) => UNDEFINED
| true =>  
    this->Apd := reference to CompleteIoSet.Apd
    : this->ApdDiagOutput := reference to CompleteIoSet.ApdDiagOutput
    : this->Fringe := reference to CompleteIoSet.Fringe
]]


[[D tFtFastController::~tFtFastController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tFtFastController::Reset (TIME tServoTime) ]
[ invalid_pointer (this) ||
  invalid_reference (this->ApdSensor) ||
  invalid_reference (this->Fringe)  => UNDEFINED
| true =>  
    this->Fringe.PhotonLog := CLEARED
    : this->Apd.iPhotonCount := 0
    : this->ApdDiagOutput.dValue := 0.0
    : return := OK
]]


************ FT FAST TRACK CONTROLLER ******************************

[[D tFtFastTrackController::tFtFastTrackController (tFtCompleteIoSet& CompleteIoSet) ]
[ invalid_pointer (this) ||
  invalid_reference (CompleteIoSet) => UNDEFINED
| true =>  
    this->Apd := reference to CompleteIoSet.Apd
    : this->ApdDiagOutput := reference to CompleteIoSet.ApdDiagChannel
    : this->Fringe := reference to CompleteIoSet.Fringe
]]


[[D tFtFastTrackController::~tFtFastTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tFtFastTrackController::Reset (TIME tServoTime) ]
[ invalid_pointer (this) ||
  invalid_reference (this->ApdSensor) ||
  invalid_reference (this->Fringe)  => UNDEFINED
| true => 
    this->Fringe.PhotonLog := empty
    : this->Apd.iPhotonCount := 0
    : this->ApdDiagOutput.dValue := 0.0
    : this->FringeWobblePositionArray[i] := 0, for 0 < i < iWobbleBins
    : this->Fringe.iWobbleSamplesArray[i] := 0, for 0 < i < iWobbleBins
    : return := OK
]]


[[D tFtFastTrackController::Calculate (tServoTime tServoTime) ]
[ invalid_pointer (this) ||
  invalid_reference (this->ApdSensor) ||
  invalid_reference (this->Fringe) ||
  tServoTime < 0  => UNDEFINED
| this->ApdSensor.dPhotonCount < 0 => 
    this->Fringe.PhotonLog := anything
| tServoTime % this->Fringe.tFTPeriod < 4 => 
    this->Apd.dPhotonCount := 0
    : this->Fringe.PhotonLog [tServoTime / this->Fringe.tFTPeriod][tServoTime / 
      this->Fringe.dFTPeriod % 4] := this->ApdSensor.dPhotonCount
    : iLastIndex := tServoTime / this->Fringe.tFTPeriod
    : return := OK
| true => 
    this->Apd.dPhotonCount := 0
    : this->ApdDiagOutput.dValue := this->Apd.dPhotonCount
    : this->Fringe.PhotonLog [tServoTime / this->Fringe.tFTPeriod][7 - 
      tServoTime / this->Fringe.tFTPeriod % 4] := this->Apd.dPhotonCount
    : iLastIndex := Time / this->Fringe.tFTPeriod
    : ec := OK
]]
****/

class tFringe;
class tIntegratorSensor;
class tActuator;
class tFtCompleteIoSet;

class tFtFastController : public tFtController {
  public: 
    tFtFastController (tFtCompleteIoSet& CompleteIoSet);
    virtual ERROR_CODE Calculate (TIME tServoTime);
  protected:
    //inputs
    tIntegratorSensor&  Apd;
   
    //outputs
    tActuator& ApdDiagOutput;
    tFringe& Fringe;
    BOOL bAudioFringe;

    tRingBuffer<tCount>&  PhotonDiag;
};

class tFtFastOffController : public tFtFastController {
  public:
    tFtFastOffController (tFtCompleteIoSet& CompleteIoSet);
    ERROR_CODE Init (TIME tServoTime);
};


class tFtFastTrackController : public tFtFastController {
  public:
    tFtFastTrackController (tFtCompleteIoSet& CompleteIoSet);
}; 

class tFtFastTrackSearchController : public tFtFastTrackController {
  public:
    tFtFastTrackSearchController (tFtCompleteIoSet& CompleteIoSet);
    virtual ERROR_CODE Init (TIME tServoTime);
}; 

class tFtFastTrackLockController : public tFtFastTrackController {
  public:
    tFtFastTrackLockController (tFtCompleteIoSet& CompleteIoSet);
    virtual ERROR_CODE Reset (TIME tServoTime);
}; 


#endif INC_ftFastControllerh
