/* $RCSfile: ftImagingController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/ft/ftImagingController.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a7Nov97,rlj    written
*/

#ifndef ftImagingControllerh
#define ftImagingControllerh

#include "ft/ftController.h"

/************
[[D tFtImagingController::tFtImagingController (tCompleteIoSet& CompleteIoSet) ]
[ invalid_pointer (this) ||
  invalid reference (CompleteIoSet) => UNDEFINED
| true => 
    this->FullFrame := reference to CompleteIoSet.FullFrame
    : this->DispersedFringe := CompleteIoSet.DispersedFringe
    : this->StarImage0 := reference to CompleteIoSet.Star0
    : this->StarImage1 := reference to CompleteIoSet.Star1
    : this->StarTracker0 := reference to CompleteIoSet.StarTracker0
    : this->StarTracker1 := reference to CompleteIoSet.StarTracker1
    : this->DispersedFringeImage := 
      reference to CompleteIoSet.DispersedFringeImage
]]

[[D tFtImagingController::~tFtImagingController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]
     
[[D tFtImagingController::Calculate (tServoTime) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Star0) ||
  invalid_reference (this->Star1) ||
  invalid_reference (this->StarTracker0) ||
  invalid_reference (this->StarTracker1) => UNDEFINED
| true => 
    this->StarTracker0.StarImage := this->Star0.Image
    : this->StarTracker1.StarImage := this->Star1.Image
    : this->StarImage0.CurrentImage := this->Star0.Image
    : this->StarImage1.CurrentImage := this->Star1.Image
    : return := OK
]]

[[D tFtImagingTrackController::tFtImagingTrackController (tFtCompleteIoSet& CompleteIoSet) ]
[ invalid_pointer (this) ||
  invalid_reference (CompleteIoSet) => UNDEFINED
| true => 
  this->FullFrameSensor := reference to CompleteIoSet.FullFrame
  : this->DispersedFringe := CompleteIoSet.DispersedFringe
  : this->StarImage0 := reference to CompleteIoSet.Star0
  : this->StarImage1 := reference to CompleteIoSet.Star1
  : this->StarTracker0 := reference to CompleteIoSet.StarTracker0
  : this->StarTracker1 := reference to CompleteIoSet.StarTracker1
  : this->DispersedFringeImage := CompleteIoSet.DispersedFringeImage
]]

[[D tFtImagingTrackController::~tFtTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tFtImagingTrackController::Init (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->DispersedFringe.Image := this->DispersedFringe.DefaultImage
    : return := OK
]]

[[D tFtImagingTrackController::Reset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->DispersedFringe.Image := this->DispersedFringe.DefaultImage
    : return := OK
]]

[[D tFtImagingTrackController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Star0) ||
  invalid_reference (this->Star1) ||
  invalid_reference (this->StarTracker0) ||
  invalid_reference (this->StarTracker1) ||
  invalid_reference (this->DispersedFringe) ||
  invalid_reference (this->DispersedFringeImage) => UNDEFINED
| true => 
    this->StarTracker0.StarImage := this->Star0.NewImage
    : this->StarTracker1.StarImage := this->Star1.NewImage
    : this->DispersedFringeImage.Image := this->DispersedFringe.Image
    : return := OK
]]


*****/

class tFtCompleteIoSet;
class tFullFrameSensor;
class tImageSensor;
class tStActuator;
class tImageStatus;

class tFtImagingController : public tFtController {
  public: 
    tFtImagingController (tFtCompleteIoSet& CompleteIoSet);
    virtual ERROR_CODE Calculate (TIME tServoTime);
    virtual ERROR_CODE Init (TIME tServoTime);
  protected:
    // inputs
    tFullFrameSensor&   FullFrame;
    tImageSensor&       Star0;
    tImageSensor&       Star1;
    tImageSensor&       DispersedFringe;
    
    // ouputs
    tStActuator&        StarTracker0;
    tStActuator&        StarTracker1;
    tImageStatus&       DispersedFringeImage;
};

class tFtImagingTrackController : public tFtImagingController {
  public: 
    tFtImagingTrackController (tFtCompleteIoSet& CompleteIoSet);
    virtual ~tFtImagingTrackController ();
    virtual ERROR_CODE Calculate (TIME tServoTime);
};

#endif INC_ftImagingControllerh
