/* $RCSfile: ftMediumController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/ft/ftMediumController.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a7Nov97,rlj    written
*/

#ifndef ftMediumControllerh
#define ftMediumControllerh

#include "ftTypes.h"
#include "ft/ftController.h"
#include "dataStore.h"
#include "ringBuffer.h"

struct dum {
  dum () {}
  dum (double d10, double d20, double d30, double d40, double d50, double d60, TIME t0) {
   d1 = d10; d2 = d20; d3 = d30 ; d4 = d40; d5 = d50; d6 = d60; t = t0; }
  double d1, d2,d3,d4,d5,d6;
  TIME t;
};
/*******
Specification Functions

QuadratureComponent (), InPhaseComponent () see "fringe.h"

*******Calculate the phase from the quadrature values

double Phase (tFringe Fringe) :==
[ return := atan2 ( 
    QuadratureComponent (Fringe.iLastIndex,Fringe.PhotonLog,Fringe.dDarkCurrent) /
    InPhaseComponent (Fringe.iLastIndex,Fringe.PhotonLog,Fringe.dDarkCurrent)
]

*******Calculate the unwrapped phase
double UnwrapPhase (double dOldUnWrappedPhase, double dNewPhase) :==
[ return := dOldUnwrappedPhase + (dNewPhase - dOldUnwrappedPhase + PI) % (2 * PI) - PI
]

double TargetVelocity (tFtControllerParameters ControllerParameters,
         double dUnwrappedPhase) :==
[ return := ControllerParameters.lvWhightLightLambda / (2 * PI) *
      (-1.0 * ControllerParameters.iPhaseSign * ControllerParameters.dFringeServoGain *
         (dUnwrappedPhase - ControllerParameters.dPhaseOffset) -
       dUnwrappedPhase )  / (ControllerParameters.tFtServoPeriod / 2.0 )
]                     

************************ STIMULI **************************************************

[[D tFtMediumController::tFtMediumController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->ClosedLoopDl :=  reference to CompleteIoSet.ClosedLoopDl
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]


[[D tFtMediumController::~tFtMediumController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtMediumOffController::tFtMediumOffController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]


[[D tFtMediumOffController::~tFtMediumOffController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtMediumOffController::Init (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl) ||
  invalid_reference (this->Fringe) => UNDEFINED
| true => 
    this->SearchTarget := RESET
    : this->ClosedLoopDl :=  ZERO_TARGET
    : return := OK    
]]


[[D tFtMediumTrackController::tFtMediumTrackController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]


[[D tFtMediumTrackController::~tFtMediumTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tFtMediumTrackController::PhaseCorrection (TIME tServoTime) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->Fringe.WobbleBins [tServoTime] := current wobble position
    : this->Fringe := this->Fringe with updated values at tServoTime  
    : this->PhaseTarget.dPosition := this->PhaseTarget.dPosition + this->PhaseTarget.dVelocity * this->tServoPeriod
    : this->PhaseTarget.dVelocity := this->dTotalGain * this->PhaseTarget.dErrorSum / this->tServoPeriod
    : this->PhaseTarget.tTime := tServoTime + this->PhaseTarget.tServoPeriod
    : this->PhaseTarget.dErrorSum := this->PhaseTarget.dErrorSum + 
      UnwrappedPhase (Phase (updated this->Fringe at tServoTime)) - 
      this->ControllerParameters.dPhaseOffset -
      this->WobbleTarget [-1]
    : return := tDlTarget with 
      dPosition = this->dPosition + this->dVelocity * this->tServoPeriod
      dRate = this->dTotalGain * this->dErrorSum / this->tServoPeriod 
      tTime = tServoTime + this->tServoPerio
]]



[[D tFtMediumTrackSearchController::tFtMediumTrackSearchController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->ClosedLoopDl :=  reference to CompleteIoSet.ClosedLoopDl
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]

[[D tFtMediumTrackSearchController::~tFtMediumTrackSearchController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtMediumTrackSearchController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl)||
  invalid_reference (this->SearchTarget) ||
  invalid_reference (this->DataStore) => UNDEFINED
| true => 
    this->ClosedLoopDl.dPosition := this->dManualTarget +
       this->dLockTarget + NextSearchValue (this->SearchTarget.iCounter)
    : this->SearchTarget.iCounter := this->SearchTarget.iCounter + 1
    : this->ClosedLoopDl.dRate := 0.0
    : this->ClosedLoopDl.tTime := tServoTime
    : this->DataStore := this->DataStore + appropriate data
    : return := OK  
]]


[[D tFtMediumTrackSemiLockController::tFtMediumTrackSemiLockController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]


[[D tFtMediumTrackSemiLockController::~tFtMediumTrackSemiLockController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtMediumTrackSemiLockController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl) ||
  invalid_reference (this->DataStore) => UNDEFINED
| true => 
    this->ClosedLoopDl.dPosition := this->dManualTarget +
       this->dLockTarget + this->SearchTarget.dCurrentTarget
    : this->dUnwrappedPhase := UnwrapPhase (this->UnWrappedPhase, 
        Phase (this->Fringe) )
    : this->ClosedLoopDl.dRate := TargetVelocity (this->ControllerParameters,
         UnwrapPhase ( Phase (this->Fringe) ) )
    : this->ClosedLoopDl.tTime := tServoTime
    : this->DataStore := this->DataStore + appropriate data
    : this->PhaseTarget := this->ControllerParameters.lvWhightLightLambda / (2 * PI) *
      this->ControllerParameters.iPhaseSign * this->ControllerParameters.dFringeServoGain *
      (dUnwrappedPhase - this->ControllerParameters.dPhaseOffset)
    : return := OK  
]]



[[D tFtMediumTrackLockController::tFtMediumTrackLockController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) ||
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->DataStore := reference to to CompleteIoSet.DataStore
    : this->Fringe := reference to CompleteIoSet.Fringe
    : this->ClosedLoopDl :=  reference to CompleteIoSet.ClosedLoopDl
    : this->dUnwrappedPhase := reference to CompleteIoSet.dUnwrappedPhase
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dManualTarget := reference to CompleteIoSet.dManualTarget
    : this->dLockTarget := reference to CompleteIoSet.dLockTarget
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
    : this->PhaseTarget := reference to CompleteIoSet.PhaseTarget
    : this->SearchTarget := reference to CompleteIoSet.SearchTarget
    : this->WobbleTarget := reference to CompleteIoSet.WobbleTarget
    : this->dServoErrorSum := reference to CompleteIoSet.dServoErrorSum
]]

[[D tFtMediumTrackLockController::~tFtMediumTrackLockController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tFtMediumTrackLockController::Init (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl) ||
  invalid_reference (this->Fringe) ||
  invalid_reference (this->dLockTarget) ||
  invalid_reference (this->dManualTarget) ||
  invalid_reference (this->dFringeCenterTarget) ||
  invalid_reference (this->SearchTarget) => UNDEFINED
| true => 
    this->dLockTarget := this->dLockTarget +
      this->dSearchTarget + this->dManualTarget
    : this->dFringeCenterTarget := 0.0
    : return := OK    
]]

[[D tFtMediumTrackLockController::Reset (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl) ||
  invalid_reference (this->Fringe) ||
  invalid_reference (this->dLockTarget) ||
  invalid_reference (this->dFringeCenterTarget) ||
  invalid_reference (this->SearchTarget) => UNDEFINED
| true => 
    this->dLockTarget += this->dFringeCenterTarget
    this->SearchTarget.iCounter := 0
    : return := OK    
]]

[[D tFtMediumTrackLockController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->ClosedLoopDl) || 
  invalid_reference (this->WobbleTarget) || 
  invalid_reference (this->Fringe) || 
  invalid_reference (this->DataStore) => UNDEFINED
| true => 
    this->ClosedLoopDl.dPosition := this->dManualTarget +
       this->dLockTarget + this->WobbleTarget[tServoTime] + 
       this->dFringeCenterTarget
    : this->dUnwrappedPhase := UnwrapPhase (this->UnWrappedPhase, 
        Phase (this->Fringe) )
    : this->ClosedLoopDl.dRate := TargetVelocity (this->ControllerParameters,
         UnwrapPhase ( Phase (this->Fringe) ) )
    : this->ClosedLoopDl.tTime := tServoTime
    : this->DataStore := this->DataStore + appropriate data
    : this->PhaseTarget := this->ControllerParameters.lvWhightLightLambda / (2 * PI) *
      this->ControllerParameters.iPhaseSign * this->ControllerParameters.dFringeServoGain *
      (dUnwrappedPhase - this->ControllerParameters.dPhaseOffset)
    : return := OK  
]]


************************************8
private method spec

[[D tFtMediumController::StoreData (TIME tServoTime, 
      FT_CONTROLLER_MODE fcmControllerMode) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Fringe) ||
  invalid_reference (this->DataStore) ||
  ::DataStream.bInit == FALSE => UNDEFINED
| IsFull (this->DataStore) =>
     ::DataStream := ::DataStream + this->DataStore.DataBag
     : this->DataStore.DataBag := (this->DataStore.DataTag, new tFtData with
       appropriate values)
| true => this->DataStore.DataBag :=  this->DataStore.DataBag +
    (this->DataStore.DataTag, new tFtData with fcmControllerMode, tServoTime, and
     other appropriate values from this->Fringe)
]]


[[D tFtMediumTrackController::UnwrapPhase (
      double dOldUnwrappedPhase, double dNewPhase) ]
[ invalid_pointer (this) => UNDEFINED
| true =>return := 
    dOldUnwrappedPhase + (dNewPhase - dOldUnwrappedPhase + PI) % (2 * PI) - PI
]]

[[D tFtMediumTrackController::TargetVelocity (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := TargetVelocity (this->ControllerParameters, 
                                    this->dUnwrappedPhase)   
]]                                  
****/

class tFtCompleteIoSet;
class tFtControllerParameters;
class tFringe;
class tDlActuator;
class tSearch;
class tPhaseTarget;
class tWobbleTarget;

class tFtMediumController : public tFtController {
  public: 
    tFtMediumController (tFtCompleteIoSet& CompleteIoSet, 
                         tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumController ();
    virtual ERROR_CODE Calculate (TIME tServoTime);

  protected:
    void StoreData (TIME tServoTime, FT_CONTROLLER_MODE fcmControllerModeToUse);
    
    const tFtControllerParameters& ControllerParameters;
    
    // inputs
    tFringe& Fringe;
    
    // outputs
    tDlActuator& ClosedLoopDl;
    tDataStore<tFtData> &DataStore;
    
    // targets
    double& dUnwrappedPhase;
    double& dManualTarget;
    double& dLockTarget;
    double& dFringeCenterTarget;
    tPhaseTarget& PhaseTarget;
    tSearch& SearchTarget;
    tWobbleTarget& WobbleTarget;
 
    tRingBuffer<dum> &TestDump;
};

class tFtMediumOffController : public tFtMediumController {
  public:
    tFtMediumOffController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumOffController ();
    virtual ERROR_CODE Init (TIME tServoTime);
}; 


class tFtMediumTrackController : public tFtMediumController {
  public:
    tFtMediumTrackController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumTrackController ();
  protected:
    tDlTarget PhaseCorrection (TIME tServoTime);
    double UnwrapPhase (double dOldUnwrappedPhase, double dNewPhase) const;
}; 
   
class tFtMediumTrackSearchController : public tFtMediumTrackController {
  public:
    tFtMediumTrackSearchController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumTrackSearchController ();
    virtual ERROR_CODE Calculate (TIME tServoTime);
}; 

class tFtMediumTrackSemiLockController : public tFtMediumTrackController {
  public:
    tFtMediumTrackSemiLockController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumTrackSemiLockController ();
    virtual ERROR_CODE Init (TIME tServoTime);
    virtual ERROR_CODE Calculate (TIME tServoTime);
}; 

class tFtMediumTrackLockController : public tFtMediumTrackController {
  public:
    tFtMediumTrackLockController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtMediumTrackLockController ();
    virtual ERROR_CODE Init (TIME tServoTime);
    virtual ERROR_CODE Reset (TIME tServoTime);
    virtual ERROR_CODE Calculate (TIME tServoTime);
}; 

#endif INC_ftMediumControllerh
