/* $RCSfile: ftSlowController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/ft/ftSlowController.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a7Nov97,rlj    written
*/

#ifndef ftSlowControllerh
#define ftSlowControllerh

#include "ft/ftController.h"

/*******

FT_LAMBA - nomimal white light wavelenght

Specification Functions

Assumes only 2 wobble bins to calculate the DVSqr/DWobblePos and compares to the
Centering  threshold.

{NEGATIVE,ZERO,POSITIVE} CenterFringeDirection (tFringe Fringe, 
    double dDeltaVDeltaWobbleToHop) :==
[ (WobbleAvgNUM (0,Fringe.iKWobbleBins,Fringe.PhotonLog,
    Fringe.dDarkCurrent)^2 -
  WobbleAvgNUM (1,Fringe.iKWobbleBins,Fringe.PhotonLog,
    Fringe.dDarkCurrent)^2 ) / 
  ( Fringe.WobblePositionArray [0] - Fringe.WobblePositionArray [1] ) > 
  dDeltaVDeltaWobbleToHop  => return := POSITIVE
| (WobbleAvgNUM (0,Fringe.iKWobbleBins,Fringe.PhotonLog,
    Fringe.dDarkCurrent)^2 -
  WobbleAvgNUM (1,Fringe.iKWobbleBins,Fringe.PhotonLog,
    Fringe.dDarkCurrent)^2 ) / 
  ( Fringe.WobblePositionArray [0] - Fringe.WobblePositionArray [1] ) < 
  -dDeltaVDeltaWobbleToHop  => return := NEGATIVE
| true => return := ZERO
] 

************************** STIMULI ***************************************

[[D tFtSlowController::tFtSlowController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) || 
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reference to ControllerParametersToUse
    : this->Fringe := reference to CompleteIoSet.Fringe   
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
]]


[[D tFtSlowController::~tFtSlowController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtSlowController::Init (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->dFringeCenterTarget) => UNDEFINED
| true => 
    this->dFringeCenterTarget := 0
    : return := OK
]]


[[D tFtSlowController::Reset (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->dFringeCenterTarget) => UNDEFINED
| true => 
    this->dFringeCenterTarget := 0
    : return := OK
]]


[[D tFtSlowTrackLockController::tFtSlowTrackLockController (tFtCompleteIoSet& CompleteIoSet,
      tFtControllerParameters& ControllerParametersToUse) ]
[ invalid_pointer (this) || 
  invalid reference (CompleteIoSet) || 
  invalid reference (ControllerParametersToUse) => UNDEFINED
| true => 
    this->ControllerParameters := reeference to ControllerParametersToUse
    : this->Fringe := reference to CompleteIoSet.Fringe    
    : this->dFringeCenterTarget := reference to CompleteIoSet.dFringeCenterTarget
]]


[[D tFtSlowTrackLockController::~tFtSlowTrackLockController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtSlowTrackLockController::Calculate (TIME tServoTime) ]
[ invalid_pointer (this) || 
  invalid_reference (this->Fringe) ||
  invalid_reference (this->dFringeCenterTarget) => UNDEFINED
| CenterFringeDirection (this->Fringe) == NEGATIVE => 
    this->dFringeCenterTarget.dPosition := 
      this->dFringeCenterTarget.dPosition -
        this->ControllerParameters.lvWhiteLightLambda
    : return := OK
| CenterFringeDirection (this->Fringe) == POSITIVE => 
    this->dFringeCenterTarget.dPosition := 
      this->dFringeCenterTarget.dPosition + 
        this->ControllerParameters.lvWhiteLightLambda
    : return := OK
| true => return := OK    
]]

*****/

class tFtCompleteIoSet;
class tFringe;
class tFtControllerParameters;
class tFtPhysicalConstants;

class tFtSlowController : public tFtController {
  public: 
    tFtSlowController (tFtCompleteIoSet& CompleteIoSet, 
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtSlowController ();
    virtual ERROR_CODE Init (TIME tServoTime);
    virtual ERROR_CODE Reset (TIME tServoTime);
  protected:
   
    tFtControllerParameters& ControllerParameters;
    
    // inputs 
    tFringe& Fringe;
    
    // outputs
    double& dFringeCenterTarget;
    
};

class tFtSlowTrackLockController : public tFtSlowController {
  public: 
    tFtSlowTrackLockController (tFtCompleteIoSet& CompleteIoSet, 
      tFtControllerParameters& ControllerParametersToUse);
    virtual ~tFtSlowTrackLockController ();
    virtual ERROR_CODE Calculate (TIME tServoTime);
};

#endif INC_ftSlowControllerh
