/* $RCSfile: ftUpdateMode.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/ft/ftUpdateMode.h,v 1.1.1.1 1998/06/11 21:36:16 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a10Nov97,rlj    written
*/

#ifndef ftUpdateModeh
#define ftUpdateModeh

#include "ftTypes.h"
#include "controllerSet.h"


#include "ringBuffer.h"

/************************************************************************
Specification Functions

N (), AvgN (), NUM (), AvgNUM () see "fringe.h"

double SNRSqr (int iIndex, tPhotoLog Log, double dDarkCurrent) :==
[ return := (PI^2/4) * NUM (iIndex,Log,dDarkCurrent) / 
    N (iIndex,Log,dDarkCurrent)
]

double EstSNRSqr (int iIndex, int iNSamples, tPhotoLog Log,
  double dDarkCurrent) :==
[ return := (PI^2/4) * AvgNUM (iIndex,iNSamples,Log,dDarkCurrent) / 
    AvgN (iIndex,iNSamplesLog,dDarkCurrent)
]

double EstVSqr (int iIndex, int iNSamples, tPhotoLog Log,
  double dDarkCurrent) :==
[ return := (PI^2/2) * AvgNUM (iIndex,iNSamples,Log,dDarkCurrent) / 
    AvgN (iIndex,iNSamplesLog,dDarkCurrent)^2
]

****************************Stimuli *******************************

[[D tFtTrackGroupUpdate::tFtTrackGroupUpdate (tFringe& FringeToUse,
      const tFtThresholds& ThresholdsToUse) ]
[ invalid_pointer (this) || 
  invalid_reference (FringeToUse) ||
  invalid_reference (ThresholdsToUse) => UNDEFINED
| true =>
    this->Fringe := reference to FringeToUse
    : this->Thresholds := ThresholdsToUse
    : iCurrentControllerMode := FTM_SEARCH
]]


[[D tFtTrackGroupUpdate::~tFtTrackGroupUpdate (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtTrackGroupUpdate::Init (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED 
| N (this->Fringe.iLastIndex, this->Fringe.PhotonLog, 
      this->Fringe.dDarkCurrent) < this->Thresholds.dThreshForSearch => 
    this->iCurrentControllerMode := FTM_SEARCH
    : ::ErrorStream := ::ErrorStream + appropriate message
    : this->tSearchStart := current system time
    : return := OK
| true => 
    this->iCurrentMode := FTM_SEARCH
    : this->tSearchStart := current system time
    : return := OK
]]


[[D tFtTrackGroupUpdate::Reset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := OK
]]

[[D tFtTrackGroupUpdate::GetNewMode_SIMPLE (TIME tUpdateTime) ]
[ invalid_pointer (this) || 
  invalid_reference (FringeToUse) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this->iCurrentControllerMode == FTM_SEARCH &&
  Total photons counted in a FT medium servo period < 
  minimum search photons =>
    ::ErrorStream := ::ErrorStream + appropriate message
    : return := this->iCurrentControllerMode
| this->iCurrentControllerMode == FTM_SEARCH &&
  Time in search > maximum time to lock =>
    ::ErrorStream := ::ErrorStream + appropriate message
    : return := this->iCurrentControllerMode
| this->iCurrentControllerMode == FTM_SEARCH &&
  SNR2 of photons read in last ft medium servo > SNR to go into semi-lock =>
    this->iCurrentControllerMode := FTM_SEMI_LOCK
    : restart semi-lock timer
    : return := FTM_SEMI_LOCK
| this->iCurrentControllerMode == FTM_SEMI_LOCK &&
  time in semi-lock > maximum semi-lock time =>
    this->iCurrentControllerMode := FTM_SEARCH
    : return := FTM_SEARCH
| this->iCurrentControllerMode == FTM_SEMI_LOCK &&
  Estimated SNR2 of photons read during last ft medium servo >= 
  SNR to go into lock && Estimated Visibility2 of photons read during last ft 
  medium servo >= Visibility2 to go into lock =>
    this->iCurrentControllerMode := FTM_LOCK
    : return := FTM_LOCK
| this->iCurrentControllerMode == FTM_LOCK &&
  Estimated SNR2 of photons read during last ft medium servo < 
  SNR to go into search =>
    this->iCurrentControllerMode := FTM_SEARCH
    : restart search timer
    : return := FTM_SEARCH
| true => return := this->iCurrentControllerMode
]]
 
[[D tFtTrackGroupUpdate::GetNewMode (TIME tUpdateTime) ]
[ invalid_pointer (this) || 
  invalid_reference (FringeToUse) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this->iCurrentControllerMode == FTM_SEARCH &&
  N (this->Fringe.iLastIndex, this->Fringe.PhotonLog, 
      this->Fringe.dDarkCurrent) < this->Thresholds.dThreshForDrop => 
    ::ErrorStream := ::ErrorStream + appropriate message
    : return := this->iCurrentControllerMode
| this->iCurrentControllerMode == FTM_SEARCH &&
  current system time - this->tSearchStart > 
      this->Thresholds.tSearchTimeOut =>
    ::ErrorStream := ::ErrorStream + appropriate message
    : return := this->iCurrentControllerMode
| this->iCurrentControllerMode == FTM_SEARCH &&
  SNRSqr (this->Fringe.iLastIndex, this->Fringe.PhotonLog,
      this->Fringe.dDarkCurrent) > this->Thresholds.dSNRThreshSearchToSemilock =>
    this->tSemiLockStartTime := current system time
    : this->iCurrentControllerMode :-= FTM_SEMI_LOCK
    : return := FTM_SEMI_LOCK
| this->iCurrentControllerMode == FTM_SEMI_LOCK &&
  current system time - this->tSemiLockStart > 
      this->Thresholds.tMaxSemiLockTime =>
    this->iCurrentControllerMode := FTM_SEARCH
    : return := FTM_SEARCH
| this->iCurrentControllerMode == FTM_SEMI_LOCK &&
  EstSNRSqr (this->Fringe.iLastIndex, this->Fringe.iKSampleSize,
    this->Fringe.PhotonLog, this->Fringe.dDarkCurrent) > 
    this->Thresholds.dSNRThreshSemilockToLock &&
  EstVSqr (this->Fringe.iLastIndex, this->Fringe.iKSampleSize,
    this->Fringe.PhotonLog, this->Fringe.dDarkCurrent) > 
      this->Thresholds.dV2ThreshSemilockToLock =>
    this->iCurrentControllerMode :-= FTM_LOCK
    : return := FTM_LOCK
| this->iCurrentControllerMode == FTM_LOCK &&
  EstSNRSqr (this->Fringe.iLastIndex, this->Fringe.iKSampleSize,
    this->Fringe.PhotonLog, this->Fringe.dDarkCurrent) < 
      this->Thresholds.dSNRLockToSearch =>
    this->tSearchStart := current system time
    : this->iCurrentControllerMode := FTM_SEARCH
    : return := FTM_SEARCH
| true => return := this->iCurrentControllerMode
]]
****************************************************************************/


class tFringe;
class tFtThresholdParameters;

class tFtTrackGroupUpdate : public tUpdateModeDecision {
  public :
    tFtTrackGroupUpdate (tFringe& FringeToUse,
                         tFtThresholdParameters& ThresholdsToUse,
                         TIME tFtMediumServoPeriodIn);
    virtual~tFtTrackGroupUpdate ();
    virtual ERROR_CODE Init ();
    virtual ERROR_CODE Reset ();
    virtual int GetNewMode (TIME tUpdateTime); 
  private:
    tFringe& Fringe;
    tFtThresholdParameters& Thresholds;
    TIME tFtMediumServoPeriod;
    TIME tSearchStart;
    TIME tSemiLockStart;

    tRingBuffer <double> SnrDiag;
    int iTrigger;
};
  

#endif INC_ftUpdateModeh
