/* $RCSfile: ftGizmo.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/ftGizmo.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $ */


/* Derek Slye -Jet Propulsion Laboratory */

#ifndef INC_ftGizmoh
#define INC_ftGizmoh

#include "isit.h"
#include "realTimeGizmo.h"
#include "typeTag.h"
#include "ftTypes.h"
#include "ft/ftIoSet.h"
#include "ft/ftControllerSet.h"

class ostream;

/****************************************************************************
[[D tFtGizmo::tFtGizmo_SIMPLE (const tFtGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  ::IpcMemory.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE ||
  ::DataStream.bInit == FALSE ||
  ::StatusStream.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 2 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 4 ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.FtTargetId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer0Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer1Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiDblBufferId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer0Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer1Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiDblBufferId) == FALSE ||
  Unable to add command, status, fast servo, medium servo, slow servo, and image servo
    periodic tasks =>
    this->ecInitError := appropriate error code
    : console := console + appropriate error messages
    : *this except this->ecInitError := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Status := default status 
    : this->CFtStatusTag := appropriate unique type tag for FT status
    : this->Servo.Mode := FSM_OFF
    : this->Servo.ControllerSet := appropriate references to controllers from 
      this->CompleteControllerSet 
    : this->CompleteControllerSet := appropriate references to i/o from 
      this->CompleteIoSet and initial servo parameters from GizmoInfo
    : this->CompleteIoSet := appropriate references to I/O channel pointers from 
      GizmoInfo and initial servo parameters from GizmoInfo
    : appropriate IPC objects for command, 2 star images, and closed loop DL target 
      added to ::IpcMemory
    : command, status, 4 servo, and track mode update tasks added to ::PeriodicTaskSet
    : data dump task added to ::EventDrivenTaskSet
]]
    

[[D tFtGizmo::tFtGizmo(const tFtGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  ::IpcMemory.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE ||
  ::DataStream.bInit == FALSE ||
  ::StatusStream.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 2 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 4 ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.FtTargetId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer0Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer1Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiDblBufferId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer0Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer1Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiDblBufferId) == FALSE ||
  Unable to add command, status, fast servo, medium servo, slow servo, and image servo
    periodic tasks =>
    this->ecInitError := appropriate error code
    : console := console + appropriate error messages
    : *this except this->ecInitError := anything
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Status := default status 
    : this->CFtStatusTag.stType := ST_STATUS
    : this->CFtStatusTag.tnName := TN_FT_STATUS
    : this->CFtStatusTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CFtStatusTag.szBlockSize := sizeof (this->Status)
    
    : ::IpcMemory.ObjectSet := ::IpcMemoryG.ObjectSet U {
      IpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) ,
      IpcObject (::IpcMemory, GizmoInfo.FtTargetId),
      IpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer0Id),
      IpcObject (::IpcMemory, GizmoInfo.iiStarImage0Id.iiBuffer1Id),
      IpcObject (::IpcMemory, GizmoInfo.iiStarTracker0ImageBufferId.iDblBufferId),
      IpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer0Id),
      IpcObject (::IpcMemory, GizmoInfo.iiStarImage1Id.iiBuffer1Id),
      IpcObject (::IpcMemory, GizmoInfo.iiStarTracker1ImageBufferId.iDblBufferId) }
      
    : ::PeriodicTaskSet := PeriodicTaskSet U {
      PeriodicTask (GizmoInfo.iGizmoId + "_CommandProcessor", GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority, GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand),
      PeriodicTask (GizmoInfo.iGizmoId + "_StatusRelayer", GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority, GizmoInfo.tStatusTaskOffset,
        this->ProcessStatus),
      PeriodicTask (FSP_FAST + "_" + GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_FAST].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_FAST].iTaskPriority, 0,
        tServo::DoServo),
      PeriodicTask (FSP_MEDIUM + "_" + GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_MEDIUM].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_MEDIUM].iTaskPriority, 0,
        tServo::DoServo),
      PeriodicTask (FSP_SLOW + "_" + GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_SLOW].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_SLOW].iTaskPriority, 0,
        tServo::DoServo),
      PeriodicTask (FSP_IMAGING + "_" + GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_IMAGING].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[FSP_IMAGING].iTaskPriority, 0,
        tServo::DoServo),
      PeriodicTask ("UpdateControllerMode",
        GizmoInfo.tTrackUpdatePeriod, GizmoInfo.tTrackUpdatePriority, 0, 
        tModedControllerGroup::PeriodicControllerModeUpdate) }
      
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet U {
      EventDrivenTask ("DataStoreDump",CiFtDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream) }
    : this->tidCommandTaskId := new unique periodic task id
    : this->tidStatusTaskId := new unique periodic task id
    : this->Servo.tidServoTaskIdArray [0..FSP_NUM_SERVO_PARTITIONS-1] := 
      new unique periodic task id
    : this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK] := new unique periodic task id
    : this->tidDataTaskId := new unique event driven task id   
    : this->Servo.Mode := FSM_OFF
  
    : this->Servo.ControllerSet := { 
      (FSM_OFF, FSP_FAST, 0, 
         &this->CompleteControllerSet.FastOff),
      (FSM_OFF, FSP_MEDIUM, 0, 
        &this->CompleteControllerSet.MediumOff),
      (FSM_OFF, FSP_SLOW, 0, 
        &this->CompleteControllerSet.SlowOffSearchSemiLock),
      (FSM_OFF, FSP_IMAGING, 0, 
        &this->CompleteControllerSet.ImagingOff), 0, 
      (FSM_TRACK, FSP_FAST, FCM_SEARCH, 
        &this->CompleteControllerSet.FastTrack),
      (FSM_TRACK, FSP_MEDIUM, FCM_SEARCH, 
        &this->CompleteControllerSet.MediumTrackSearch),
      (FSM_TRACK, FSP_SLOW, FCM_SEARCH, 
        &this->CompleteControllerSet.SlowOffSearchSemiLock),
      (FSM_TRACK, FSP_IMAGING, FCM_SEARCH, 
        &this->CompleteControllerSet.ImagingTrack),
      (FSM_TRACK, FSP_FAST, FCM_SEMI_LOCK, 
        &this->CompleteControllerSet.FastTrack),
      (FSM_TRACK, FSP_MEDIUM, FCM_SEMI_LOCK, 
        &this->CompleteControllerSet.MediumTrackSemiLock),
      (FSM_TRACK, FSP_SLOW, FCM_SEMI_LOCK, 
        &this->CompleteControllerSet.SlowOffSearchSemiLock),
      (FSM_TRACK, FSP_IMAGING, FCM_SEMI_LOCK, 
        &this->CompleteControllerSet.ImagingTrack),
      (FSM_TRACK, FSP_FAST, FCM_LOCK, 
        &this->CompleteControllerSet.FastTrack),
      (FSM_TRACK, FSP_MEDIUM, FCM_LOCK, 
        &this->CompleteControllerSet.MediumTrackLock),
      (FSM_TRACK, FSP_SLOW, FCM_LOCK, 
        &this->CompleteControllerSet.SlowTrackLock),
      (FSM_TRACK, FSP_IMAGING, FCM_LOCK, 
        &this->CompleteControllerSet.ImagingTrack) } 
     
     
    : this->CompleteControllerSet.FastOff.ApdSensor :=   
      reference to this->CompleteIoSet.ApdSensor
    : this->CompleteControllerSet.FastOff.Fringe :=  
      reference to this->CompleteIoSet.Fringe

    : this->CompleteControllerSet.FastTrack.ApdSensor := 
      reference to this->CompleteIoSet.ApdSensor
    : this->CompleteControllerSet.FastTrack.Fringe := 
      reference to this->CompleteIoSet.Fringe

    : this->CompleteControllerSet.MediumOff.ControllerParameters :=
       reference to this->CompleteControllerSet.ControllerParametersToUse
    : this->CompleteControllerSet.MediumOff.DataStore := 
      reference to this->DataStore
    : this->CompleteControllerSet.MediumOff.Fringe := 
      reference to this->CompleteIoSet.Fringe
    : this->CompleteControllerSet.MediumOff.ClosedLoopDl :=  
      reference to this->CompleteIoSet.ClosedLoopDl
    : this->CompleteControllerSet.MediumOff.dPhaseTarget :=  
      reference to this->CompleteIoSet.dPhaseTarget
    : this->CompleteControllerSet.MediumOff.dManualTarget := 
      reference to this->CompleteIoSet.dManualTarget
    : this->CompleteControllerSet.MediumOff.dLockTarget := 
      reference to this->CompleteIoSet.dLockTarget
    : this->CompleteControllerSet.MediumOff.dFringeCenterTarget := 
      reference to this->CompleteIoSet.dFringeCenterTarget
    : this->CompleteControllerSet.MediumOff.SearchTarget := 
      reference to this->CompleteIoSet.SearchTarget
    : this->CompleteControllerSet.MediumOff.WobbleTarget := 
      reference to this->CompleteIoSet.WobbleTarget

    : this->CompleteControllerSet.MediumTrackSearch.ControllerParameters := 
      reference to this->CompleteControllerSet.ControllerParameters
    : this->CompleteControllerSet.MediumTrackSearch.DataStore := 
      reference to this->CompleteIoSet.DataStore
    : this->CompleteControllerSet.MediumTrackSearch.Fringe := 
      reference to this->CompleteIoSet.Fringe
    : this->CompleteControllerSet.MediumTrackSearch.ClosedLoopDl := 
      reference to this->CompleteIoSet.ClosedLoopDl
    : this->CompleteControllerSet.MediumTrackSearch.dPhaseTarget :=  
      reference to this->CompleteIoSet.PhaseTarget
    : this->CompleteControllerSet.MediumTrackSearch.dManualTarget := 
      reference to this->CompleteIoSet.ManualTarget
    : this->CompleteControllerSet.MediumTrackSearch.dLockTarget := 
      reference to this->CompleteIoSet.LockTarget
    : this->CompleteControllerSet.MediumTrackSearch.dFringeCenterTarget := 
      reference to this->CompleteIoSet.FringeCenterTarget
    : this->CompleteControllerSet.MediumTrackSearch.SearchTarget := 
      reference to this->CompleteIoSet.SearchTarget
    : this->CompleteControllerSet.MediumTrackSearch.WobbleTarget := 
      reference to this->CompleteIoSet.WobbleTarget

    : this->CompleteControllerSet.MediumTrackSemiLock.ControllerParameters :=  
      reference to this->CompleteControllerSet.ControllerParametersToUse
    : this->CompleteControllerSet.MediumTrackSemiLock.DataStore := 
      reference to this->CompleteIoSet.DataStore
    : this->CompleteControllerSet.MediumTrackSemiLock.Fringe := 
      reference to this->CompleteIoSet.Fringe
    : this->CompleteControllerSet.MediumTrackSemiLock.ClosedLoopDl :=  
      reference to this->CompleteIoSet.ClosedLoopDl
    : this->CompleteControllerSet.MediumTrackSemiLock.dPhaseTarget :=  
      reference to this->CompleteIoSet.PhaseTarget
    : this->CompleteControllerSet.MediumTrackSemiLock.dManualTarget := 
      reference to this->CompleteIoSet.ManualTarget
    : this->CompleteControllerSet.MediumTrackSemiLock.dLockTarget := 
      reference to this->CompleteIoSet.LockTarget
    : this->CompleteControllerSet.MediumTrackSemiLock.FringeCenterTarget := 
      reference to this->CompleteIoSet.FringeCenterTarget
    : this->CompleteControllerSet.MediumTrackSemiLock.SearchTarget := 
      reference to this->CompleteIoSet.SearchTarget
    : this->CompleteControllerSet.MediumTrackSemiLock.WobbleTarget := 
      reference to this->CompleteIoSet.WobbleTarget

    : this->CompleteControllerSet.MediumTrackLock.ControllerParameters := 
      reference to this->CompleteControllerSet.ControllerParametersToUse
    : this->CompleteControllerSet.MediumTrackLock.DataStore := reference to 
      this->CompleteIoSet.DataStore
    : this->CompleteControllerSet.MediumTrackLock.Fringe := reference to 
      this->CompleteIoSet.Fringe
    : this->CompleteControllerSet.MediumTrackLock.ClosedLoopDl :=  
      reference to this->CompleteIoSet.ClosedLoopDl
    : this->CompleteControllerSet.MediumTrackLock.dPhaseTarget :=  
      reference to this->CompleteIoSet.PhaseTarget
    : this->CompleteControllerSet.MediumTrackLock.dManualTarget := 
      reference to this->CompleteIoSet.ManualTarget
    : this->CompleteControllerSet.MediumTrackLock.dLockTarget := 
      reference to this->CompleteIoSet.LockTarget
    : this->CompleteControllerSet.MediumTrackLock.dFringeCenterTarget := 
      reference to this->CompleteIoSet.FringeCenterTarget
    : this->CompleteControllerSet.MediumTrackLock.SearchTarget := 
      reference to this->CompleteIoSet.SearchTarget
    : this->CompleteControllerSet.MediumTrackLock.WobbleTarget := 
      reference to this->CompleteIoSet.WobbleTarget

    : this->CompleteControllerSet.SlowOffSearchSemiLock.Fringe := 
      reference to this->CompleteIoSet.Fringe   
    : this->CompleteControllerSet.SlowOffSearchSemiLock.dFringeCenterTarget := 
      reference to this->CompleteIoSet.dFringeCenterTarget
    : this->CompleteControllerSet.SlowTrackLock.Fringe := 
      reference to this->CompleteIoSet.Fringe   
    : this->CompleteControllerSet.SlowTrackLock.dFringeCenterTarget := 
      reference to this->CompleteIoSet.dFringeCenterTarget

    : this->CompleteControllerSet.ImagingOff.FullFrame := 
      reference to this->CompleteIoSet.FullFrame
    : this->CompleteControllerSet.ImagingOff.DispersedFringe := 
      this->CompleteIoSet.DispersedFringe
    : this->CompleteControllerSet.ImagingOff.StarImage0 := 
      reference to this->CompleteIoSet.StarImage0
    : this->CompleteControllerSet.ImagingOff.StarImage1 := 
      reference to this->CompleteIoSet.StarImage1
    : this->CompleteControllerSet.ImagingOff.StarTracker0 := 
      reference to this->CompleteIoSet.StarTracker0
    : this->CompleteControllerSet.ImagingOff.StarTracker1 := 
      reference to this->CompleteIoSet.StarTracker1
    : this->CompleteControllerSet.ImagingOff.DispersedFringeImage := 
      reference to CompleteIoSet.DispersedFringeImage

    : this->CompleteControllerSet.ImagingTrack.FullFrame := 
      reference to this->CompleteIoSet.FullFrame
    : this->CompleteControllerSet.ImagingTrack.DispersedFringe := 
      this->CompleteIoSet.DispersedFringe
    : this->CompleteControllerSet.ImagingTrack.StarImage0 := 
      reference to this->CompleteIoSet.StarImage0
    : this->CompleteControllerSet.ImagingTrack.StarImage1 := 
      reference to this->CompleteIoSet.StarImage1
    : this->CompleteControllerSet.ImagingTrack.StarTracker0 := 
      reference to this->CompleteIoSet.StarTracker0
    : this->CompleteControllerSet.ImagingTrack.StarTracker1 := 
      reference to this->CompleteIoSet.StarTracker1
    : this->CompleteControllerSet.ImagingTrack.DispersedFringeImage := 
      reference to CompleteIoSet.DispersedFringeImage
    
    : this->CompleteControllerSet.ControllerParameters := 
      GizmoInfo.ServoParameters.ControllerParameters
    : this->CompleteControllerSet.ThresholdParameters := 
      GizmoInfo.ServoParameters.ThresholdParameters
    
    : this->CompleteIoSet.Apd.pChannel := GizmoInfo.pApdChannel
    : this->CompleteIoSet.StarImage0.pFullImageChannel :=  
      GizmoInfo.pFullImageChannel
    : this->CompleteIoSet.StarImage0.ImageParameters := 
      GizmoInfo.ImageParameters.Star0Parameters
    : this->CompleteIoSet.StarImage1.pFullImageChannel := 
      GizmoInfo.pFullImageChannel
    : this->CompleteIoSet.StarImage1.ImageParameters := 
      GizmoInfo.ImageParameters.Star1Parameters

    : this->CompleteIoSet.DispersedFringe.pFullImageChannel := 
      GizmoInfo.pFullImageChannel
    : this->CompleteIoSet.DispersedFringe.ImageParameters := 
      GizmoInfo.ImageParameters.DispersedFringeParameters

    : this->CompleteIoSet.FullFrame.pFullImageChannel := 
      GizmoInfo.pFullImageChannel
   
    : this->CompleteIoSet.ClosedLoopDl.TargetId := 
      GizmoInfo.iiDelayLineTargetId
    : this->CompleteIoSet.StarTracker0.iiBufferId := 
      GizmoInfo.iiStarTracker0ImageBufferId
    : this->CompleteIoSet.StarTracker1.iiBufferId := 
      GizmoInfo.iiStarTracker1ImageBufferId
    
    : this->CompleteIoSet.DataStore.DataBag := EMPTY
    
    : this->CompleteIoSet.Fringe.PhotonLog := EMPTY
    : this->CompleteIoSet.Fringe.Parameters := 
      GizmoInfo.ServoParameters.FringeParameters

    : this->CompleteIoSet.ManualTarget := ZERO_TARGET
    : this->CompleteIoSet.LockTarget := ZERO_TARGET
    : this->CompleteIoSet.PhaseTarget := 
      GizmoInfo.ServoParameters.dPhaseTarget
    : this->CompleteIoSet.FringeCenterTarget := ZERO_TARGET
    : this->CompleteIoSet.SearchTarget.iCounter := 0
    : this->CompleteIoSet.SearchTarget.SearchParameters := 
      GizmoInfo.ServoParameters.SearchParameters
    : this->CompleteIoSet.WobbleTarget.bEnabled := TRUE
    : this->CompleteIoSet.WobbleTarget.ModParams := 
      GizmoInfo.ServoParameters.WobbleParameters

    : this->CompleteIoSet.CFullImageTag.stType := ST_STATUS
    : this->CompleteIoSet.CFullImageTag.tnName := TN_FT_FULL_IMAGE
    : this->CompleteIoSet.CFullImageTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CompleteIoSet.CFullImageTag.szBlockSize := sizeof (PIXEL) * 
      GizmoInfo.FullImage.iHSize * GizmoInfo.FullImage.iVSize

    : this->CompleteIoSet.CDispersedFringeTag.stType := ST_STATUS
    : this->CompleteIoSet.CDispersedFringeTag.tnName := TN_FT_IMAGE
    : this->CompleteIoSet.CDispersedFringeTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CompleteIoSet.CDispersedFringeTag.szBlockSize := sizeof (PIXEL) * 
      GizmoInfo.FringeImage.iHSize * GizmoInfo.FringeImage.iVSize

    : this->CompleteIoSet.CDataTag.stType := ST_DATA
    : this->CompleteIoSet.CDataTag.tnName := TN_FT_DATA
    : this->CompleteIoSet.CDispersedFringeTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CompleteIoSet.CDispersedFringeTag.szBlockSize := sizeof (tFtData) 

    : this->CompleteIoSet.FullFrameImage.iHSize := FULL_FRAME_H_SIZE
    : this->CompleteIoSet.FullFrameImage.iVSize := FULL_FRAME_V_SIZE

    : this->CompleteIoSet.DispersedFringeImage.Image.iHSize := 
      GizmoInfo.ImageParameters.iFringeImageHSize
    : this->CompleteIoSet.DispersedFringeImage.Image.iVSize := 
      GizmoInfo.ImageParameters.iFringeImageVSize    
]]  

[[D tFtGizmo::~tFtGizmo (void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to delete PeriodicTask (this->tidCommandTaskId) 
    from ::PeriodicTaskSet ||
  Unable to delete PeriodicTask (this->tidStatusTaskId) 
      from ::PeriodicTaskSet ||
  Unable to delete PeriodicTask (this->Servo.tidServoTaskIdArray 
    [0..FSP_NUM_SERVO_PARTITIONS-1] )  from ::PeriodicTaskSet ||
   Unable to delete PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK]) 
      from ::PeriodicTaskSet ||
  Unable to delete EventDrivenTask (this->tidDataTaskId) 
      from ::EventDrivenTaskSet =>
    console := console + appropriate messages
    : ::PeriodicTaskSet := any ::PeriodicTaskSet - 
      Subset of above periodic tasks
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet -
       Subset of above event driven tasks
    : *this := anything
| true => 
    *this := anything
    : ::PeriodicTaskSet := PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId) -
      PeriodicTask (this->tidServoTaskIdArray [0..FSP_NUMBER_OF_SERVO_PARTITIONS-1] ) -
      PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK])
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet - 
      EventDrivenTask (this->tidDataTaskId)
]]

[[D tFtGizmo::Restart (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to enable PeriodicTask (this->tidCommandTaskId) 
    in ::PeriodicTaskSet ||
  Unable to enable PeriodicTask (this->tidStatusTaskId) 
    in ::PeriodicTaskSet ||
  Unable to enable PeriodicTask (this->Servo.tidServoTaskIdArray
    [0..FSP_NUMBER_OF_SERVO_PARTITIONS-1]) from ::PeriodicTaskSet ||
  Unable to enable PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK]) 
    in  ::PeriodicTaskSet ||
  Unable to resume EventDrivenTask (this->tidDataTaskId) 
      in ::EventDrivenTaskSet =>
    console := console + appropriate messages
    : subset of periodic tasks above enabled
    : subset of event driven task above resumed
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) enabled
    : PeriodicTask (this->tidStatusTaskId) enabled
    : PeriodicTask (this->Servo.tidServoTaskIdArray [0..FSP_NUMBER_OF_SERVO_PARTITIONS-1] ) 
      enabled
    : PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK]) enabled
    : EventDrivenTask (this->tidDataTaskId) resumed
    : this->bInit := TRUE
    : return := OK
]]

[[D tFtGizmo::Shutdown (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to disable PeriodicTask (this->tidCommandTaskId) 
    in ::PeriodicTaskSet ||
  Unable to disable PeriodicTask (this->tidStatusTaskId) 
    in ::PeriodicTaskSet ||
  Unable to disable PeriodicTask (this->Servo.tidServoTaskIdArray
    [0..FSP_NUMBER_OF_SERVO_PARTITIONS-1]) from ::PeriodicTaskSet ||
  Unable to disable PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK]) 
    in  ::PeriodicTaskSet ||
  Unable to suspend EventDrivenTask (this->tidDataTaskId) 
      in ::EventDrivenTaskSet =>
    console := console + appropriate messages
    : subset of periodic tasks above disabled
    : subset of event driven task above suspended
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) disabled
    : PeriodicTask (this->tidStatusTaskId) disabled
    : PeriodicTask (this->Servo.tidServoTaskIdArray [0..FSP_NUMBER_OF_SERVO_PARTITIONS-1] ) 
      disabled
    : PeriodicTask (this->Servo.tidModeUpdateTaskIdArray [FSM_TRACK]) disabled
    : EventDrivenTask (this->tidDataTaskId) suspended
    : this->bInit := TRUE
    : return := OK
]]


[[D tFtGizmo::IsOk(void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this.ecInitError !=OK ||
  Unable to receive commands ||
  Unable to receive from image channels => return := FALSE
| true => return := TRUE
]]

[[D tFtGizmo::operator_extractor(ostream& stream, tFtGizmo& Gizmo) ]
[ invalid_pointer (this) ||
  invalid_reference (stream) ||
  invalid_reference (Gizmo) => UNDEFINED
| true => stream := stream + "Fringe Tracker Gizmo" followed by results of
    calling Gizmo.tRealtimeGizmo::operator_extractor(stream,Gizmo)
]]


[[D tFtGizmo::Ping (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true => ::ErrorStream := ::ErrorStream + appropriate messages
]]

[[D tFtGizmo::Off (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError !=OK => No response
| true => this->Servo.Mode := FSM_OFF
    : this->CompleteIoSet.ClosedLoopDl.Target := ZERO_TARGET
    : this->CompleteIoSet.DispersedFringe.Image := 
      this->CompleteIoSet.DispersedFringe.DefaultImage 
    : ::StatusStream := ::StatusStream + this->FtDispersedFringeTag + 
      this->CompleteIoSet.DispersedFringe.DefaultImage
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]
	
[[D tFtGizmo::Track (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE ||
  ::StatusStream.bInit == FALSE ||
  ::DataStream.bInit == FALSE => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true => this->Servo.Mode := FSM_TRACK
    : this->CompleteIoSet.ClosedLoopDl.Target := appropriate tracking target
    : this->CompleteIoSet.DispersedFringe.Image := 
      current image from frame grabber hardware from current region of interest
    : ::StatusStream := ::StatusStream + this->FtDispersedFringeTag + 
      this->CompleteIoSet.DispersedFringe.Image
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]

[[D tFtGizmo::Calibrate (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| true => No response
]]

[[D tFtGizmo::SetManualTarget (FT_TARGET_DIRECTION ftdTargetDirection) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE ||
  ftdTargetDirection not in { FTD_POSITIVE, FTD_NEGATIVE } => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| ftdTargetDirection == FTD_POSITIVE => 
    this->CompleteIoSet.dManualTarget := 
      this->CompleteIoSet.dManualTarget + WHITE_LIGHT_LAMBDA_IN_LASER_COUNTS
    : ::ErrorStream := ::ErrorStream + appropriate messages
| ftdTargetDirection == FTD_NEGATIVE => 
    this->CompleteIoSet.dManualTarget := 
      this->CompleteIoSet.dManualTarget - WHITE_LIGHT_LAMBDA_IN_LASER_COUNTS
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]
	
[[D tFtGizmo::GetImage (void) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE ||
  ::StatusStream.bInit == FALSE => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| this->Servo.Mode == FSM_OFF => 
    ::StatusStream := StatusStream +
      this->CompleteIoSet.CFtFullImageTag + 
      atomic this->CompleteIoSet.FullFrame.Image
    : ::ErrorStream := ::ErrorStream + appropriate messages
| true => 
    ::StatusStream := StatusStream +
      this->CompleteIoSet.CFtFullImageTag + 
      this->CompleteIoSet.FullFrame.Image
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]

[[D tFtGizmo::SetServoParameters (const tFtServoParameters& Parameters) ]
[ invalid_pointer (this) ||
  ::ErrorStream.bInit == FALSE ||
  invalid_reference (Parameters) => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->CompleteControllerSet.ControllerParameters := 
      Parameters.ControllerParameters
    : this->CompleteControllerSet.ThresholdParameters := 
      Parameters.Thresholds
    : this->CompleteIoSet.WobbleTarget.ModParams := 
      Parameters.WobbleParameters
    : this->CompleteIoSet.SearchTarget.SearchParemeters := 
      Parameters .SearchParameters
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]

[[D tFtGizmo::SetRegionOfInterest (const tFtImageParameters& ImageParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (ImageParameters) ||
  ::ErrorStream.bInit == FALSE  => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| invalid parameters => ::ErrorStream := ::ErrorStream + appropriate messages
| true => 
    this->CompleteIoSet.Star0.iStartX := ImageParameters.iStartXSt0
    : this->CompleteIoSet.Star0.iStartY := ImageParameters.iStartYSt0
    : this->CompleteIoSet.Star1.iStartX := ImageParameters.iStartXSt1
    : this->CompleteIoSet.Star1.iStartY := ImageParameters.iStartYSt1
    : this->CompleteIoSet.DispersedFringe.iStartX := ImageParameters.iStartYFringe
    : this->CompleteIoSet.DispersedFringe.iStartY := ImageParameters.iStartYFringe
    : ::ErrorStream := ::ErrorStream + appropriate messages
]]

********************************* Protected Member Specification ****************************

[[D tFtGizmo::CollectStatus (void) ]
[ invalid_pointer (this) ||
  ::StatusStream.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| IsFull (::StatusStream) =>
    ::ErrorStream := ::ErrorStream + appropriate error code
    : return := appropriate error code
| true =>
    ::StatusStream  := ::StatusStream (::pThisRteG) + 
      this->CFtStatusTag + this->Status + 
      this->CompleteIoSet.CFtDispersedFringeTag + 
      this->CompleteIoSet.DispersedFringeImage.Image
    : return := OK
]]  

[[D tFtGizmo::Dispatch (const tIsitCmd& Command)]
[ invalid_pointer (this) ||
  invalid_reference (Command) => UNDEFINED
| Command.Command not in { FT_CMD_PING,FT_CMD_OFF,FT_CMD_TRACK,FT_CMD_CALIBRATE,
    FT_CMD_SET_MT,FT_CMD_GET_IMAGE,FT_CMD_CHNG_PARAM,FT_CMD_SET_ROI } => 
      return := appropriate error code
| Servo not running properly && Command.Command in { FT_CMD_OFF,FT_CMD_TRACK } => 
	return := appropriate error code from Servo SetMode method
| Command.Command == FT_CMD_PING => 
  ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| Command.Command == FT_CMD_OFF && Servo running properly => 
  this->Servo.Mode := FSM_OFF
  : this->CompleteIoSet.ClosedLoopDl.Target := ZERO_TARGET
  : this->CompleteIoSet.DispersedFringe.Image := 
    this->CompleteIoSet.DispersedFringe.DefaultImage 
  : ::StatusStream := ::StatusStream + this->FtDispersedFringeTag + 
    this->CompleteIoSet.DispersedFringe.DefaultImage
  : ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| Command.Command == FT_CMD_TRACK && Servo running properly =>
    this->Servo.Mode := FSM_TRACK
    : this->CompleteIoSet.ClosedLoopDl.Target := appropriate tracking target
    : this->CompleteIoSet.DispersedFringe.Image := 
      current image from frame grabber hardware from current region of interest
    : ::StatusStream := ::StatusStream + this->FtDispersedFringeTag + 
      this->CompleteIoSet.DispersedFringe.Image
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command == FT_CMD_CALIBRATE => return := OK
| Command.Command == FT_CMD_SET_MT && Command.ftdTargetDirection == FTD_POSITIVE =>
  this->CompleteIoSet.dManualTarget := 
    this->CompleteIoSet.dManualTarget + WHITE_LIGHT_LAMBDA_IN_LASER_COUNTS
  : return := OK
| Command.Command == FT_CMD_SET_MT && Command.ftdTargetDirection == FTD_NEGATIVE =>
  this->CompleteIoSet.dManualTarget := 
    this->CompleteIoSet.dManualTarget - WHITE_LIGHT_LAMBDA_IN_LASER_COUNTS
  : return := OK
| Command.Command == FT_CMD_SET_MT && Command.ftdTargetDirection not in
    { FTD_POSITIVE,FTD_NEGATIVE } =>
  return := appropriate error code
| Command.Command == FT_CMD_GET_IMAGE && this->Servo.Mode == FSM_OFF => 
  ::StatusStream := StatusStream + this->CompleteIoSet.CFtFullImageTag + 
    atomic this->CompleteIoSet.FullFrame.Image
  : ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| Command.Command == FT_CMD_GET_IMAGE && this->Servo.Mode == FSM_TRACK => 
  ::StatusStream := StatusStream + this->CompleteIoSet.CFtFullImageTag + 
    this->CompleteIoSet.FullFrame.Image
  : ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| Command.Command == FT_CMD_CHNG_PARAM =>
  this->CompleteIoSet.WobbleParameters updated with new values
  : this->CompleteIoSet.SearchParameters updated with new values
  : this->CompleteControllerSet.Thresholds updated with new values
  : GUI-settable quantities in this->CompleteControllerSet.ControllerParameters
    updated with new values
  : ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| Command.Command == FT_CMD_SET_ROI =>
  this->CompleteIoSet.Star0.iStartX := ImageParameters.iStartXSt0
  : this->CompleteIoSet.Star0.iStartY := ImageParameters.iStartYSt0
  : this->CompleteIoSet.Star1.iStartX := ImageParameters.iStartXSt1
  : this->CompleteIoSet.Star1.iStartY := ImageParameters.iStartYSt1
  : this->CompleteIoSet.DispersedFringe.iStartX := ImageParameters.iStartYFringe
  : this->CompleteIoSet.DispersedFringe.iStartY := ImageParameters.iStartYFringe
  : ::ErrorStream := ::ErrorStream + appropriate messages
  : return := OK
| true => identity
]]

****************************************************************************/

class tFtGizmoInfo;
class ostream;
class tIsitCmd;
class tFtCompleteIoSet;
class tFtCompleteControllerSet;
class tFtSetServoParametersCommand;

class tFtGizmo : public tRealtimeGizmo {
  public:
    tFtGizmo(const tFtGizmoInfo& GizmoInfo);
    virtual ~tFtGizmo();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
    friend ostream& operator<< (ostream& stream, tFtGizmo& Gizmo);
  
  private: // these must be constructed before using in other member constructors
    const tTypeTag    CFtStatusTag;

  protected:
    virtual ERROR_CODE CollectStatus ();
    virtual ERROR_CODE Dispatch (const tIsitCmd& command);
   

  private:
    ERROR_CODE Ping ();
    ERROR_CODE Off ();
    ERROR_CODE Track();
    ERROR_CODE Calibrate ();
    ERROR_CODE SetManualTarget (const tFtSetManualTargetCmd& Command);
    ERROR_CODE GetImage ();
    ERROR_CODE SetServoParameters (const tFtSetServoParametersCmd& Command);
    ERROR_CODE SetRegionOfInterest (const tFtSetRegionOfInterestCmd& Command);

    tFtStatus                 Status;   
    tFtCompleteIoSet          CompleteIoSet; 
    tFtCompleteControllerSet  CompleteControllerSet;
    
    
};     /* class tFtGizmo */

#endif INC_ftGizmoh


