/* $RCSfile: ftGizmoInfo.h,v $ */
/* CM Info: */   
/*$Header: /home/chara/cvsroot/ople/sys/include/ftGizmoInfo.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $ */

/* Derek Slye - Jet Propulsion Laboratory */

#include "ipcid.h"
#include "realTimeGizmoInfo.h"
#include "ftTypes.h"
#include "inputChannel.h"
#include "outputChannel.h"

/************************************************************************
* Fringe tracker Gizmo information class spec
*
[[D tFtGizmoInfo::tFtGizmoInfo (int iRteId, int iGizmoId, int iNumberOfServoModes,
      tServoInfo& ServoInfo, const tFtPhysicalConstants& PhysicalConstantsToUse,
      const tFtServoParameters& ServoParametersToUse, int iClosedLoopDlIdToUse, 
      int iSt0ImageGizmoId, int iSt1ImageGizmoId,
      const tFtImageParameters& ImageParametersToUse,
      BOARD_NAME bnApdBoard, int iApdChannel, 
      BOARD_NAME bnApdDiagBoard, int iApdDiagChannel, 
      BOARD_NAME bnFrameGrabber, int iFullImageChannel) ]

[ invalid_pointer (this) ||
  invalid_reference (ServoInfo) ||
  invalid_reference (ServoParametersToUse) ||
  invalid_reference (ImageParametersToUse) ||
  invalid_reference (PhysicalConstantsToUse) ||
  invalid_pointer (pFullImageChannelToUse) ||
  IsADelayLine (iClosedLoopDlIdToUse) == FALSE ||
  IsAStarTracker (iSt0IdToUse) == FALSE ||
  IsAStarTracker (iSt1IdToUse) == FALSE => UNDEFINED
| Number of IPC_NAMES previously generated >= maximum allowable names =>
   this->ecInitError := appropriate error code
   : *this (except this->ecInitError) := anything
| true =>
   this->iRteId := iRteId
   : this->iGizmoId := iGizmoId
   : this->iiCommandStreamId := appropriate and unique tIpcFifoId
   : this->gtGizmoType := GT_FRINGE_TRACKER
   : this->iStatusTaskPriority := CiStatusTaskPriority
   : this->tStatusTaskPeriod := CtStatusTaskPeriod
   : this->tStatusTaskOffset := 0
   : this->iCommandTaskPriority := CiCommandTaskPriority
   : this->tCommandTaskPeriod := CiCommandTaskPeriod
   : this->tCommandTaskOffset := 0
   : this->iNumberOfServoModes := iNumberOfServoModes,
   : this->ServoInfo := ServoInfo 
   : this->iiCloseLoopDlTargetId = iClosedLoopDlIdToUse
   : this->iSt0Id := iSt0IIdToUse
   : this->iSt1Id := iSt1IdToUse
   : this->ServoParameters := ServoParametersToUse
   : this->ImageParameters := ImageParametersToUse
   : this->PhysicalConstants := PhysicalConstantsToUse
   : this->pApdChannel := appropriate channel pointer to Channel (bnApdBoard, iApdChannel)
   : this->pApdDiagChannel := appropriate channel pointer to Channel (bnApdDiagBoard, iApdDiagChannel)
   : this->pFullImageChannel := appropriate channel pointer to Channel (bnFrameGrabber, iFullImageChannel) 
]]


[[D tFtGizmoInfo::~tFtGizmoInfo (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtGizmoInfo::WhiteLightLambda (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->PhysicalConstants.lvWhiteLightLambda
]]


[[D tFtGizmoInfo::WobbleParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters.WobbleParameters
]]


[[D tFtGizmoInfo::SearchParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters.SearchParameters
]]


[[D tFtGizmoInfo::ControllerParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters.ControllerParameters
]]


[[D tFtGizmoInfo::Thresholds (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters.Thresholds
]]

[[D tFtGizmoInfo::NumBoxcarSamples (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters.ControllerParameters.iMemBoxcar
]]


[[D tFtGizmoInfo::NumWobbleSamples (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->PhysicalConstants.iNumberOfWobbleSamples
]]


[[D tFtGizmoInfo::ServoPeriod (FT_SERVO_PARTITION fsmPartition) ]
[ invalid_pointer (this) ||
  fsmPartition >= FSP_NUMBER_OF_SERVO_PARTITIONS => UNDEFINED
| true => return := this->ServoInfo.ServoTaskInfoArray [fsmPartition].
    tTaskPeriod
]]

[[D tFtGizmoInfo::ServoPriority (FT_SERVO_PARTITION fsmPartition) ]
[ invalid_pointer (this) ||
  fsmPartition >= FSP_NUMBER_OF_SERVO_PARTITIONS => UNDEFINED
| true => return := this->ServoInfo.ServoTaskInfoArray [fsmPartition].
    tTaskPriority
]]

[[D tFtGizmoInfo::StarImageId (int iWhichSt) ]
[ invalid_pointer(this) => UNDEFINED
| iWhichSt == 1 => return := this->St1ImageBufferId
| iWhichSt == 0 => return := StarTracker (this->iSt1Id).
    CompleteIoSet.StarImage.ImageBufferId
| true => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.ImageBufferId
]]


[[D tFtGizmoInfo::StarXStart (int iWhichSt) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichSt == 0 => return := this->ImageParameters.iStartXSt0
| iWhichSt == 1 => return := this->ImageParameters.iStartXSt1
| true => return := this->ImageParameters.iStartXSt0
]]


[[D tFtGizmoInfo::StarYStart (int iWhichSt) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichSt == 0 => return := this->ImageParameters.iStartYSt0
| iWhichSt == 1 => return := this->ImageParameters.iStartYSt1
| true => return := this->ImageParameters.iStartYSt0
]]

[[D tFtGizmoInfo::StarHSize (int iWhichSt) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichSt == 0 => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageHSize
| iWhichSt == 1 => return := StarTracker (this->iSt1Id).
    CompleteIoSet.StarImage.iImageHSize
| true => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageHSize
]]


[[D tFtGizmoInfo::StarVSize (int iWhichSt) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichSt == 0 => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageVSize
| iWhichSt == 1 => return := StarTracker (this->iSt1Id).
    CompleteIoSet.StarImage.iImageVSize
| true => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageVSize
]]



 
[[D tFtGizmoInfo::ClosedLoopDlTargetId (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := DelayLine (this->iClosedLooDlId).
    CompleteIoSet.TargetGenerator.iiFtTargetBufferId
]]


[[D tFtGizmoInfo::DispersedFringeXStart (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ImageParameters.iStartXFringeImage
]]

[[D tFtGizmoInfo::DispersedFringeYStart (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ImageParameters.iStartYFringeImage
]]


[[D tFtGizmoInfo::StarYStart (int iWhichSt) ]
[ invalid_pointer (this) => UNDEFINED
| iWhichSt == 0 => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageYStart
| iWhichSt == 1 => return := StarTracker (this->iSt1Id).
    CompleteIoSet.StarImage.iImageYStart
| true => return := StarTracker (this->iSt0Id).
    CompleteIoSet.StarImage.iImageYStart
]]


[[D tFtGizmoInfo::DispersedFringeHSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ImageParameters.FringeImageHSize
]]


[[D tFtGizmoInfo::DispersedFringeVSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ImageParameters.FringeImageVSize
]]


[[D tFtGizmoInfo::FullFrameHSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := FULL_IMAGE_HSIZE
]]


[[D tFtGizmoInfo::FullFrameVSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := FULL_IMAGE_VSIZE
]]


[[D tFtGizmoInfo::ApdScaleFactor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->PhysicalConstants.dApdScaleFactor
]]


[[D tFtGizmoInfo::ApdChannel (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->pApdChannel
]]

[[D tFtGizmoInfo::ApdDiagScaleFactor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->PhysicalConstants.dApdDiagScaleFactor
]]


[[D tFtGizmoInfo::ApdDiagChannel (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->pApdDiagChannel
]]
 
[[D tFtGizmoInfo::FullImageChannel (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->pFullImageChannel
]]

****************** Protected method specificiation****************************

[[D tFtGizmoInfo::GetStGizmoInfo (int iWhichSt) ]
[ invalid_pointer(this) => UNDEFINED
| iWhichSt out of range => return := reference to Gizmo (this->iSt1GizmoId).GizmoInfo
  : send appropriate error message
| true => return := reference to Gizmo (this->iStGizmoId(iWhichSt)).GizmoInfo
]]


****/

class tFullImageChannel;
class tModulationParams;
class tStGizmoInfo;

class tFtPhysicalConstants {
  public:
    double dApdScaleFactor;
    double dApdDiagScaleFactor;
    int iNumberOfWobbleSamples;
    LASER_VALUE lvWhiteLightLambda;
    
};
  
class tFtGizmoInfo : public tRealtimeGizmoInfo {
  public:
    tFtGizmoInfo(int iRteId, int iGizmoId, int iNumberOfServoModes,
      const tServoInfo& ServoInfo, const tFtPhysicalConstants& PhysicalConstantsToUse,
      const tFtServoParameters& ServoParametersToUse, int iClosedLoopDlIdToUse, 
      int iSt0ImageGizmoId, int iSt1ImageGizmoId,
      const tFtImageParameters& ImageParametersToUse,
      BOARD_NAME bnApdBoard, int iApdChannel, 
      BOARD_NAME bnApdDiagBoard, int iApdDiagChannel,
      BOARD_NAME bnFrameGrabber, int iFullImageChannel);
  
    virtual ~tFtGizmoInfo () {}

    LASER_VALUE WhiteLightLambda () const;
    tModulationParams WobbleParameters () const;
    tFtSearchParameters SearchParameters () const;
    tFtControllerParameters ControllerParameters () const;
    tFtThresholdParameters Thresholds () const;
    int NumBoxcarSamples () const;
    int NumWobbleSamples () const;
    TIME ServoPeriod (FT_SERVO_PARTITION fsmPartition) const;
    int  ServoPriority (FT_SERVO_PARTITION fsmPartition) const;
    tIpcDblBufferId ClosedLoopDlTargetId () const;
    tIpcIndirectionDblBufferId StarImageId (int iWhichSt) const;
    int StarXStart (int iWhichSt) const;
    int StarYStart (int iWhichSt) const; 
    int StarHSize (int iWhichSt) const;
    int StarVSize (int iWhichSt) const;
    int DispersedFringeXStart () const;
    int DispersedFringeYStart () const;
    int DispersedFringeHSize () const;
    int DispersedFringeVSize () const;
    int FullFrameHSize () const;
    int FullFrameVSize () const;
    double ApdScaleFactor () const;
    tInputChannel* ApdChannel () const;
    double ApdDiagScaleFactor () const;
    tOutputChannel* ApdDiagChannel () const;
    tFullImageChannel* FullImageChannel () const;
    TIME TrackUpdatePeriod () const;
    int TrackUpdatePriority () const;
    TIME TrackUpateOffset () const;
    
  protected: 
    const tStGizmoInfo& GetStGizmoInfo (int iGizmoId) const;
    
  private:
    tFtPhysicalConstants PhysicalConstants;
    tFtServoParameters ServoParameters;
    int iClosedLoopDlId;
    int iSt0Id;
    int iSt1Id;
    tFtImageParameters ImageParameters;
    tInputChannel* pApdChannel;
    tOutputChannel* pApdDiagChannel;
    tFullImageChannel* pFullImageChannel;
    TIME tTrackUpdatePeriod;
    int  iTrackUpdatePriority;
    TIME tTrackUpdateOffset;
};

