/* $RCSfile: ftTypes.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/ftTypes.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/
/* Richard L Johnson - Jet Propulsion Laboratory */
 
 
/*
DESCRIPTION
This file contains type defintions of externally visible fringe tracker
structures

*/

#ifndef INC_ftTypesh
#define INC_ftTypesh

#include "isit.h"
#include "modLib.h"
#include "ftError.h"

// Specific Track controller modes used by Track mode update 

typedef enum FT_TRACK_MODE { FTM_SEARCH, FTM_SEMI_LOCK, FTM_LOCK,
  FTM_NUM_MODES } FT_TRACK_MODE;

// All controller modes for FT Servo groups for use by status block etc
typedef enum FT_CONTROLLER_MODE { FCM_NONE, FCM_SEARCH, FCM_SEMI_LOCK, 
  FCM_LOCK, FCM_UNKNOWN, FCM_NUM_MODES } FT_CONTROLLER_MODE;

typedef enum FT_SERVO_MODE { FSM_OFF, FSM_TRACK, FSM_NUM_SERVO_MODES }
  FT_SERVO_MODE;

typedef enum FT_SERVO_PARTITION { FSP_FAST, FSP_MEDIUM, FSP_SLOW, FSP_IMAGING, 
  FSP_NUM_SERVO_PARTITIONS } FT_SERVO_PARTITION;

typedef enum FT_TARGET_DIRECTION { FTD_POSITIVE, FTD_NEGATIVE } FT_TARGET_DIRECTION;

typedef enum {
  FTCS_OFF, FTCS_TRACK_START, FTCS_TRACK_ACTIVE, FTCS_TRACK_ABORTED,
  FTCS_GET_IMAGE, FTCS_SET_SERVO_PARAMETERS, FTCS_SET_ROI
} FT_COMMAND_STATUS;

typedef enum SEARCH_TYPE { SPIRAL_GEOMETRIC,  SPIRAL_ARITHMETIC } SEARCH_TYPE; 

/*****************************************************************************


[[D tFtSearchParameters::tFtSearchParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtSearchParameters::tFtSearchParameters_dddST (
      double dSpiralRadiusInitialToUse,
      double dSpiralRadiusLimitToUse, double dSpiralStepToUse,
      double dProgressionConstantToUse, SEARCH_TYPE stSearchTypeToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dSpiralRadiusInitial := dSpiralRadiusInitialToUse
    : this->dSpiralRadiusLimit := dSpiralRadiusLimitToUse
    : this->dSpiralStep := dSpiralStepToUse
    : this->dProgressionConstant := dProgressionConstantToUse
    : this->stSearchType := stSearchTypeToUse
]]


[[D tFtSearchParameters::~tFtSearchParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtThresholdParameters::tFtThresholdParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtThresholdParameters::tFtThresholdParameters_ttdddddd (
      TIME tSemiLockToLockTimeoutToUse, 
      TIME tTotalTimeToLockTimeoutToUse, double dSNRThreshSearchToSemiLockToUse,
      double dSNRThreshSemiLockToLockToUse, double dV2ThreshSemiLockToLockToUse,
      double dSNRThreshLockToSearchToUse, double dThreshForDropToUse, 
      double dThreshForSearchToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->tSemiLockToLockTimeout := tSemiLockToLockTimeoutToUse
    : this->tTotalTimeToLockTimeout := tTotalTimeToLockTimeoutToUse
    : this->dSNRThreshSearchToSemiLock := dSNRThreshSearchToSemiLockToUse
    : this->dSNRThreshSemiLockToLock := dSNRThreshSemiLockToLockToUse 
    : this->dV2ThreshSemiLockToLock := dV2ThreshSemiLockToLockToUse
    : this->dSNRThreshLockToSearch := dSNRThreshLockToSearchToUse
    : this->dThreshForDrop := dThreshForDropToUse
    : this->dThreshForSearch := dThreshForSearchToUse
]]


[[D tFtThresholdParameters::~tFtThresholdParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtControllerParameters::tFtControllerParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtControllerParameters::tFtControllerParameters_tiiiddid (
      TIME tFtServoPeriodToUse, 
      LASER_VALUE lvWhiteLightLambdaToUse,
      int iNumWobbleSamplesToUse,
      int iMemBoxcarToUse, double dFringeServoGainToUse, 
      double dDeltaVDeltaWobbleToHopToUse, int iPhaseSignToUse,
      double dPhaseOffsetToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->tFtServoPeriod := tFtServoPeriodToUse
    : this->lvWhiteLightLambda := lvWhiteLightLambdaToUse
    : this->iNumWobbleSamples := iNumWobbleSamplesToUse
    : this->iMemBoxcar := iMemBoxcarToUse
    : this->dFringeServoGain := dFringeServoGainToUse
    : this->dDeltaVDeltaWobbleToHop := dDeltaVDeltaWobbleToHopToUse
    : this->iPhaseSign := iPhaseSignToUse
    : this->dPhaseOffset := dPhaseOffsetToUse
]]


[[D tFtControllerParameters::~tFtControllerParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtServoParameters::tFtServoParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtServoParameters::tFtServoParameters_tttt (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (SearchParametersToUse) ||
  invalid_reference (WobbleParametersToUse) ||
  invalid_reference (ControllerParametersToUse) ||
  invalid_reference (ThresholdsToUse) => UNDEFINED
| true => 
    this->SearchParameters := SearchParametersToUse
    : this->WobbleParameters := WobbleParametersToUse
    : this->ControllerParameters := ControllerParametersToUse
    : this->ThresholdParameters := ThresholdsToUse
]]

[[D tFtServoParameters::~tFtServoParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtImageParameters::tFtImageParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtImageParameters::~tFtImageParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tFtTargetStatus::tFtTargetStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dManual := 0.0
    : this->dSearch := 0.0
    : this->dWobble := 0.0
    : this->dLock := 0.0
    : this->dFringeCenter := 0.0
]]


[[D tFtTargetStatus::tFtTargetStatus_ddddd (double dManualToUse, 
      double dSearchToUse, double dWobbleToUse, 
      double dLockToUse, double dFringeCenterToUse,
      double dPhaseToUse ) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dManual := dManualToUse
    : this->dSearch := SearchToUse
    : this->dWobble := dWobbleToUse
    : this->dLock := dLockToUse
    : this->dFringeCenter := dFringeCenterToUse
    : this->dPhase := dPhaseToUse
]]


[[D tFtTargetStatus::~tFtTargetStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtTargetStatus::Manual (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dManual
]]


[[D tFtTargetStatus::Search (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dSearch
]]


[[D tFtTargetStatus::Wobble (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dWobble
]]


[[D tFtTargetStatus::Lock (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dLock
]]  


[[D tFtTargetStatus::FringeCenter (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dFringeCenter
]]  

[[D tFtTargetStatus::Phase (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dPhase
]]  

[[D tFtTargetStatus::Total (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dManual + this->dSearch + this->dWobble +
    this->dLock + this->dFringeCenter
]]  


[[D tFringeStatus::tFringeStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dPhotons := 0.0
    : this->dVisibility2 := 0.0
    : this->dSnr := 0.0
]]  

 
[[D tFringeStatus::tFringeStatus_ddd (double dPhotonsToUse,
      double dVisibility2ToUse, double dSnr2ToUse) ]
[ invalid_pointer (this) => UNDEFINED
| dSnr2ToUse < 0 => UNDEFINED
| true => 
    this->dPhotons := dPhotonsToUse
    : this->dVisibility2 := dVisibility2ToUse
    : this->dSnr := sqrt (dSnrToUse)
]]  


[[D tFringeStatus::Photons (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dPhotons
]]  


[[D tFringeStatus::Visibility2 (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dVisibility2
]]  


[[D tFringeStatus::Snr (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dSnr
]]  


[[D tFtStatus::tFtStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]    
    

[[D tFtStatus::~tFtStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tFtStatus::UpdateTime (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| true => this->tTimeStamp := current system time if time updating properly
    otherwize anything
]]

[[D tFtStatus::UpdateServoParameters_tttt (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (SearchParametersToUse) ||
  invalid_reference (WobbleParametersToUse) ||
  invalid_reference (ControllerParametersToUse) ||
  invalid_reference (ThresholdsToUse) => UNDEFINED
| true => 
    this->ServoParameters.SearchParameters := SearchParametersToUse
    : this->ServoParameters.WobbleParameters := WobbleParametersToUse
    : this->ServoParameters.ControllerParameters := ControllerParametersToUse
    : this->ServoParameters.ThresholdParameters := ThresholdsToUse
]]

[[D tFtStatus::UpdateServoParameters_t (
      const tFtServoParameters& ServoParametersToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (ServoParametersToUse) => UNDEFINED
| true => this->ServoParameters := ServoParametersToUse
]]

[[D tFtStatus::UpdateImageParameters (
       const tFtImageParameters& ImageParametersToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (ImageParametersToUse) => UNDEFINED
| true => this->ImageParameters := ImageParametersToUse
]]

[[D tFtStatus::UpdateFringeStatus (const tFringeStatus& FringeStatusToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (FringeStatusToUse) => UNDEFINED
| true => this->FringeStatus := FringeStatusToUse
]]


[[D tFtStatus::UpdateTargetStatus (const tFtTargetStatus& TargetStatusToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (TargetStatusToUse) => UNDEFINED
| true => this->Targets := TargetStatusToUse
]]
 
 
[[D tFtStatus::UpdateCommandStatus (FT_COMMAND_STATUS fcsCommandStatusToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->fcsCommandStatus := fcsCommandStatusToUse
]]


[[D tFtStatus::UpdateServoMode (FT_SERVO_MODE fsmServoModeToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->fsmServoMode := fsmServoModeToUse
]]


[[D tFtStatus::UpdateControllerMode (
      FT_CONTROLLER_MODE fcmControllerModeToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->fcmControllerMode := fcmControllerModeToUse
]]


[[D tFtData::tFtData (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tFtData::~tFtData (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

**********************************************/

class tFtSearchParameters {
  public:
    tFtSearchParameters () {};
    tFtSearchParameters (double dSpiralRadiusInitialToUse,
      double dSpiralRadiusLimitToUse, double dSpiralStepToUse,
      double dProgressionConstantToUse, SEARCH_TYPE stSearchTypeToUse);
      
    double        dSpiralRadiusInitial;
    double        dSpiralRadiusLimit;
    double        dSpiralStep;
    double        dProgressionConstant;
    SEARCH_TYPE   stSearchType;
};

class tFtThresholdParameters {
  public:
    tFtThresholdParameters () {};
    tFtThresholdParameters (TIME tSemiLockToLockTimeoutToUse, 
      TIME tTotalTimeToLockTimeoutToUse, double dSNRThreshSearchToSemiLockToUse,
      double dSNRThreshSemiLockToLockToUse, double dV2ThreshSemiLockToLockToUse,
      double dSNRThreshLockToSearchToUse, double dThreshForDropToUse, 
      double dThreshForSearchToUse);
      
   TIME tSemiLockToLockTimeout;
   TIME  tTotalTimeToLockTimeout;
   double dSNRThreshSearchToSemiLock;
   double dSNRThreshSemiLockToLock;
   double dV2ThreshSemiLockToLock;
   double dSNRThreshLockToSearch;
   double dThreshForDrop;
   double dThreshForSearch;
};

class tFtControllerParameters {
  public:
    tFtControllerParameters () {}
    tFtControllerParameters (TIME tFtServoPeriodToUse, 
      LASER_VALUE lvWhiteLightLambdaToUse,
      int iNumWobbleSamplesToUse, 
      int iMemBoxcarToUse, double dFringeServoGainToUse, 
      double dDeltaVDeltaWobbleToHopToUse,
      int iPhaseSignToUse, double dPhaseOffsetToUse);

    // non GUI settable parameters...
    TIME    tFtServoPeriod;
    LASER_VALUE lvWhiteLightLambda;
    int     iNumWobbleSamples;
    
    // GUI settable parameters
    int     iMemBoxcar;
    double  dFringeServoGain;
    double  dDeltaVDeltaWobbleToHop;
    int     iPhaseSign;
    double  dPhaseOffset;
   
};
    


class tFtServoParameters  {
  public:
    tFtServoParameters (void) {}
    tFtServoParameters (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse);               
      
    tModulationParams       WobbleParameters;
    tFtSearchParameters     SearchParameters;
    tFtThresholdParameters  ThresholdParameters;
    tFtControllerParameters ControllerParameters; 
};


// can't have constructor until commands aren't unioned
class tFtImageParameters {
  public:
   	int iStartXSt0, iStartYSt0;
  	int iStartXSt1, iStartYSt1;
  	int iStartXFringeImage, iStartYFringeImage;
        int iFringeImageHSize, iFringeImageVSize;
};


class tFtTargetStatus {
  public:
    tFtTargetStatus () {
      this->dManual = 0.0;
      this->dSearch = 0.0;
      this->dWobble = 0.0;
      this->dLock = 0.0;
      this->dFringeCenter = 0.0;
      this->dPhase = 0.0;
    }
    tFtTargetStatus (double dManualToUse, double dSearchToUse, 
                     double dWobbleToUse, double dLockToUse,
                     double dFringeCenterToUse, double dPhaseToUse );
    inline double Manual () const;
    inline double Search () const;
    inline double Wobble () const;
    inline double Lock () const;
    inline double FringeCenter () const;
    inline double Phase () const;
    inline double Total () const;

    double dManual;
    double dSearch;
    double dWobble;
    double dLock;
    double dFringeCenter;
    double dPhase;
};


class tFringeStatus {
  public:
    tFringeStatus () { this->dPhotons = 0.0;
      this->dVisibility2 = 0.0;
      this->dSnr = 0.0;}
    tFringeStatus (double dPhotonsToUse, double dVisibility2ToUse,
       double dSnr2ToUse);
    inline double Photons () const;
    inline double Visibility2 () const;
    inline double Snr () const;

    double dPhotons;
    double dVisibility2;
    double dSnr;  // NOTE this is not Snr2 on purpose.
};


class tFtStatus {
  public:
    void UpdateTime ();
    void UpdateServoParameters (
      const tFtSearchParameters& SearchParametersToUse,
      const tModulationParams& WobbleParametersToUse,
      const tFtControllerParameters& ControllerParametersToUse,
      const tFtThresholdParameters& ThresholdsToUse);
    void UpdateServoParameters (const tFtServoParameters& ServoParametersToUse);
    void UpdateImageParameters (const tFtImageParameters& ImageParametersToUse);
    void UpdateFringeStatus (const tFringeStatus& FringeStatusToUse);
    void UpdateTargetStatus (const tFtTargetStatus& TargetStatusToUse);
    void UpdateCommandStatus (FT_COMMAND_STATUS fcsCommandStatusToUse);
    void UpdateServoMode (FT_SERVO_MODE fsmServoModeToUse);
    void UpdateControllerMode (FT_CONTROLLER_MODE fcmControllerModeToUse);
   
 
    TIME tTimeStamp;
    tFtServoParameters ServoParameters;
    tFtImageParameters ImageParameters;
    tFtTargetStatus    Targets;
    tFringeStatus      FringeStatus;
    FT_COMMAND_STATUS  fcsCommandStatus;
    FT_SERVO_MODE fsmServoMode;
    FT_CONTROLLER_MODE fcmControllerMode;
    BOOL    bOnCenterFringe;
};



class tFtData {
  public:
    TIME tTimeStamp;
    FT_CONTROLLER_MODE fcmControllerMode;
    double dPhotons;
    double dInPhaseComponent;
    double dQuadratureComponent;
    double dUnwrappedPhase;
};



#endif  /* INC_ftTypesh */
