/* $RCSfile: gizmo.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/gizmo.h,v 1.2 1998/09/23 23:57:26 phil Exp $ */

/*
modification history
--------------------
01a,17Jun97,rlj    written
*/

#ifndef INC_gizmoh
#define INC_gizmoh

#include "isit.h"
#include "baseObject.h"
#include "commandStream.h"

//Errors 

#define S_gizmo_SHUTDOWN_ERROR    (M_gizmo | 0x1)

// Warnings
#define S_gizmo_SHUTTING_DOWN_GIZMO     (M_gizmo | 0x1001)
#define S_gizmo_ALREADY_INITIALIZED     (M_gizmo | 0x1002)
#define S_gizmo_ALREADY_SHUTDOWN        (M_gizmo | 0x1003)

// Messages

#define S_gizmo_INIT_START            (M_gizmo | 0x2001)
#define S_gizmo_INIT_COMPLETE         (M_gizmo | 0x2002)
#define S_gizmo_DESTRUCT_START        (M_gizmo | 0x2003)
#define S_gizmo_DESTRUCT_COMPLETE     (M_gizmo | 0x2004)
#define S_gizmo_RESTART_START         (M_gizmo | 0x2005)
#define S_gizmo_RESTART_COMPLETE      (M_gizmo | 0x2006)
#define S_gizmo_SHUTDOWN_START        (M_gizmo | 0x2007)
#define S_gizmo_SHUTDOWN_COMPLETE     (M_gizmo | 0x2008)




/****************************************************************************

tGizmo Black Box State

BOOL			        bInit
ERROR_CODE			ecInitError

TASK_ID                         tidCommandTaskId
TASK_ID                         tidStatusTaskId

*****************************************************************************

[[D tGizmo::tGizmo(const tGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id of ::pThisRteG ||
  GizmoInfo.iGizmoId != id of *this ||
  ::IpcMemoryG.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE =>
    this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := FALSE
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
       NewObjectSet (::IpcMemoryG,GizmoInfo.iiCommandStreamId) 
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
]]  


[[D tGizmo::~tGizmo (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>    
    *this := anything
    : ::PeriodicTaskSet := PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId)
]]

[[D tGizmo::StartTasks (const tGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  this->ecInitError != OK ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : return := OK
| Unable to add periodic tasks to handle command and status =>
    this->bInit := anything
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
    : console := console + appropriate error messages
    : return := appropriate error code
| true =>    
    (::PeriodicTaskSet) := ::PeriodicTaskSet U 
      PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
        this->ProcessStatus) U
    : this->tidCommandTaskId := new periodic task id
    : this->tidStatusTaskId := new periodic task id
    : this->bInit := TRUE
    : return := OK
]]

[[D tGizmo::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to resume  PeriodicTask (this->tidCommandTaskId) ||
  Unable to resume  PeriodicTask (this->tidStatusTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) resumed
    : PeriodicTask (this->tidStatusTaskId) resumed
    : this->bInit := TRUE
    : return := OK
]]

[[D tGizmo::Shutdown (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspended PeriodicTask (this->tidCommandTaskId) ||
  Unable to suspended PeriodicTask (this->tidStatusTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) suspended
    : PeriodicTask (this->tidStatusTaskId) suspended
    : this->bInit := FALSE
    : return := OK
]]

[[D tGizmo::IsOk(void) ]
[ invalid_pointer (this) => UNDEFINED
| this.ecInitError != OK => return := FALSE
| true => return := TRUE
]]

[[D tGizmo::operator_extractor (ostream& Stream, 
                                const tGizmo& Gizmo) ]
[ invalid_pointer (this) ||
  invalid_reference (Stream) ||
  invalid_reference (Gizmo) => UNDEFINED
| true =>
    Stream := Stream << appropriate text and Gizmo state values
    : return := Stream << appropriate text and Gizmo state values
]]

[[D tGizmo::CommandSentToGizmo (ISIT_CMD& Command) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE => no response
| Unknown command  => 
    console := console + appropriate error messages
    : ::ErrorStream (::pThisRteG) : = ::ErrorStream (::pThisRteG) + appropriate error code 
| true =>
    appropriate gizmo command executed
    : gizmo state changes appropriately
    : ::ErrorStream (::pThisRteG) : = ::ErrorStream (::pThisRteG) + appropriate message 
]]


*****************************************************************
Protected Spec Definitions

[[D tGizmo::CollectStatus (void) ]
[ Intended function since tGizmo::CollectStatus is pure virtual => Comment
| invalid_pointer (this) => UNDEFINED
| Unable to collect status from gizmo => 
    console := console + appropriate error messages
    : ErrorStream (::pThisRteG) := ErrorStream (::pThisRteG) + appropriate
      error message
    : return := appropriate error code
| true => return := OK
]]



[[D tGizmo::Dispatch (const tIsitCmd& Command) ]
[ Intended function since tGizmo::Dispatch is pure virtual => Comment
| invalid_pointer (this) => UNDEFINED
| Command is an unknown command => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    appropriate gizmo command for Command executed
    : return := OK
]]


  
[[D tGizmo::SendError_ec (ERROR_CODE ecErrCode) ]
[ this is invalid ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send error to ErrorStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ErrorStream (::pThisRteG) := ErrorStream (::pThisRteG) + ErrorInfo with
       ecErrorCode, ::pThisRteG->tSchedTime, ::pThisRteG->iRteId
    : return := OK
]]

[[D tGizmo::SendError_ect (ERROR_CODE ecErrCode, TIME tErrTime) ]
[ this is invalid ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send error to ErrorStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ErrorStream (::pThisRteG) := ErrorStream (::pThisRteG) + ErrorInfo with
       ecErrorCode, tErrTime, ::pThisRteG->iRteId
    : return := OK
]]

[[D tGizmo::SendErrorNoWait (ERROR_CODE ecErrCode, TIME tErrTime) ]
[ this is invalid ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send error immediately to ErrorStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ErrorStream (::pThisRteG) := ErrorStream (::pThisRteG) + ErrorInfo with
       ecErrorCode, tErrTime, ::pThisRteG->iRteId
    : return := OK
]]

[[D tGizmo::SendData_tp (const tTypeTag& Tag, const BYTE* pData) ]
[ this is invalid ||
  invalid_reference (Tag) ||
  invalid_pointer (pData) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send Tag and *pdata to ::DataStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ::DataStream (::pThisRteG) := ::DataStream (::pThisRteG) + Tag + *pData
    : return := OK
]]

[[D tGizmo::SendData_tt (const tTypeTag& Tag, const tBuffer& BufferIn) ]
[ this is invalid ||
  invalid_reference (Tag) ||
  invalid_reference (BufferIn) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send Tag and BufferIn.Buffer to ::DataStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ::DataStream (::pThisRteG) := ::DataStream (::pThisRteG) + Tag + BufferIn.Buffer
    : return := OK
]]




[[D tGizmo::SendStatus_tp (const tTypeTag& Tag, const BYTE* pData) ]
[ this is invalid ||
  invalid_reference (Tag) ||
  invalid_pointer (pStatus) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send Tag and *pData to ::StatusStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ::StatusStream (::pThisRteG) := ::StatusStream (::pThisRteG) + Tag + *pData
    : return := OK
]]

[[D tGizmo::SendStatus_tt (const tTypeTag& Tag, const tBuffer& BufferIn) ]
[ this is invalid ||
  invalid_reference (Tag) ||
  invalid_reference (BufferIn) ||
  invalid_pointer (::pThisRteG) => UNDEFINED
| Unable to send Tag and BufferIn.Buffer to StatusStream (::pThisRteG) => 
    console := console + appropriate error messages
    : return := appropriate error code
| true =>
    ::StatusStream (::pThisRteG) := ::StatusStream (::pThisRteG) + Tag + BufferIn.Buffer
    : return := OK
]]

********************************************************
Private Spec Definitions

[[D tGizmo::ProcessCommand (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->CommandStream is empty => no response
| this->CommandStream.NextCommand is an unknown command => 
    console := console + appropriate error messages
    : ErrorStream (::pThisRteG) := ErrorStream (::pThisRteG) + appropriate error message
| true => appropriate gizmo command for this->CommandStream.NextCommand executed
]]

[[D tGizmo::ProcessStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>  this->CollectStatus() executed to get derived gizmo to put appropriate
		Status on ::StatusStream
]]
*********/

class tIsitCmd;
class tGizmoInfo;
class tTypeTag;
class tCommandStreamReceiver;
class tBuffer;

class tGizmo : public tBaseObject {
  public:
    tGizmo (const tGizmoInfo &GizmoInfo);
    virtual ~tGizmo (void);
    ERROR_CODE Restart (void);
    virtual ERROR_CODE Shutdown (void);
    virtual BOOL IsOk (void);
 
    static ERROR_CODE SendData (const tTypeTag& Tag, const BYTE* pData);
    static ERROR_CODE SendData (const tTypeTag& Tag, const tBuffer& Buffer); 
    static ERROR_CODE SendStatus (const tTypeTag& Tag, const BYTE* pData); 
    static ERROR_CODE SendStatus (const tTypeTag& Tag, const tBuffer& Buffer); 
    static ERROR_CODE SendError (ERROR_CODE ecErrCode); 
    static ERROR_CODE SendError (ERROR_CODE ecErrCode, TIME tErrTime); 
    static ERROR_CODE SendErrorNoWait (ERROR_CODE ecErrCode, TIME tErrTime); 

  protected:
    virtual ERROR_CODE Dispatch(const tIsitCmd& Command) = 0;
    virtual ERROR_CODE CollectStatus (void) = 0;
    ERROR_CODE StartTasks (const tGizmoInfo& GizmoInfo);

  private:    
    ERROR_CODE ProcessCommand (void);
    ERROR_CODE ProcessStatus (void);

    int iStatusRelayTaskId;
    int iProcessCommandTaskId;
    
    tCommandStreamReceiver CommandStream;
    
};


#endif INC_gizmoh
 
