/* $RCSfile: gizmoInfo.h,v $ */
/* CM Info: */   

/*$Header: /home/chara/cvsroot/ople/sys/include/gizmoInfo.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $ */

/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,29Aug97,rlj    written 

*/


#ifndef INC_gizmoInfoh
#define INC_gizmoInfoh

#include "isit.h"
#include "ipcid.h"

/****
Black Box State Variables
--------------------------------------
int                iRteId
int                iGizmoId 
tIpcFifoId         iiCommandStreamId 
GIZMO_TYPE         gtGizmoType 
int                iStatusTaskPriority
TIME               tStatusTaskPeriod
TIME               tStatusTaskOffset
int                iCommandTaskPriority
TIME               tCommandTaskPeriod
TIME               tCommandTaskOffset 



[[D tGizmoInfo::tGizmoInfo (int iRteIdToUse, int iGizmoIdToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->iRteId := iRteIdToUse
    : this->iGizmoId := iGizmoIdToUse
    : this->iiCommandStreamId := appropriate and unique tIpcFifoId
    : this->gtGizmoType := GT_UNKNOWN
    : this->iStatusTaskPriority := CiStatusTaskPriority
    : this->tStatusTaskPeriod := CtStatusTaskPeriod
    : this->tStatusTaskOffset := 0
    : this->iCommandTaskPriority := CiCommandTaskPriority
    : this->tCommandTaskPeriod := CiCommandTaskPeriod
    : this->tCommandTaskOffset := 0
]]

[[D tGizmoInfo::~tGizmoInfo (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tGizmoInfo::GetGizmoId (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iGizmoId
]]


[[D tGizmoInfo::GetGizmoType (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->gtGizmoType
]]

[[D tGizmoInfo::GetCommandStreamId (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iiCommandStreamId
]]

[[D tGizmoInfo::GetStatusRelayPeriod (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->tStatusTaskPeriod 
]]


[[D tGizmoInfo::GetStatusRelayPriority (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iStatusTaskPriority 
]]

[[D tGizmoInfo::GetStatusRelayOffset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->tStatusTaskOffset 
]]


[[D tGizmoInfo::GetCommandRelayPeriod (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->tCommandTaskPeriod  
]]


[[D tGizmoInfo::GetCommandRelayPriority (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iCommandTaskPriority 
]]


[[D tGizmoInfo::GetCommandRelayOffset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iCommandTaskOffset  
]]


[[D tGizmoInfo::NewIpcName (void) ]
[ invalid_pointer (this) => UNDEFINED
| Number of IPC_NAMES previously generated == maximum allowable names =>
   return := anything
| true =>
    number of IPC_NAMES generated := number of IPC_NAMES generated + 1
    : return := unique IPC_NAME
]]

[[D tGizmoInfo::TypeOfMemory (int iMyRteNum,int iOtherGizmoId) ]
[ invalid_pointer (this) ||
  iOtherGizmoId not valid gizmo id => UNDEFINED
| iMyRteNum == iOtherGizmoId Rte number => return := MT_LOCAL
| true => return := MT_SHARED
]]


****/


class tGizmoInfo {
  public:
    tGizmoInfo (int iRteIdToUse, int iGizmoIdToUse);
    virtual ~tGizmoInfo (void) {}

    int   GetRteNumber (void) const {return iRteId;}
    int   GetGizmoId   (void) const {return iGizmoId;}
    GIZMO_TYPE GetGizmoType (void) const { return this->gtGizmoType; }
    const tIpcFifoId& GetCommandStreamId (void) const { return this->iiCommandStreamId; }
    int  GetStatusRelayPriority (void) const { return this->iStatusTaskPriority; }
    TIME GetStatusRelayPeriod (void) const { return this->tStatusTaskPeriod; }
    TIME GetStatusRelayOffset (void) const { return this->tStatusTaskOffset; }
    int GetCommandRelayPriority (void) const {return this->iCommandTaskPriority; }
    TIME GetCommandRelayPeriod (void) const { return this->tCommandTaskPeriod; }
    TIME GetCommandRelayOffset (void) const { return this->tCommandTaskOffset; }

  protected:
    IPC_NAME NewIpcName (void);
    MEM_TYPE TypeOfMemory (int iMyRteNum,int iOtherGizmoId) const;
 
    int      iRteId;
    int      iGizmoId;
    GIZMO_TYPE gtGizmoType;
    tIpcFifoId iiCommandStreamId;
    int iStatusTaskPriority;
    TIME tStatusTaskPeriod;
    TIME tStatusTaskOffset;
    int iCommandTaskPriority;
    TIME tCommandTaskPeriod;
    TIME tCommandTaskOffset;

};


#endif INC_gizmoInfoh
