/* $RCSfile: gsuDlControllerSet.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlControllerSet.h,v 1.2 1998/09/23 23:57:32 phil Exp $
*/
/* Richard Johsnon - Jet Propulsion Laboratory */
 
/*
$Log: gsuDlControllerSet.h,v $
Revision 1.2  1998/09/23 23:57:32  phil
Changes to data recording so FIFOs dont fill up

Revision 1.1.1.1  1998/06/11 21:36:16  chara
First import of RICST Code

Revision 1.8.2.1  1998/04/30 20:36:04  rbrill
commit before build_5a_008

Revision 1.8  1998/03/30 19:35:12  rich
*** empty log message ***

Revision 1.7  1998/03/12 17:59:33  rich
dl overhaul cont.

Revision 1.6  1998/03/12 01:05:10  rich
delay line overhaul cont.

Revision 1.5  1998/03/11 00:58:12  rich
dl and gsuDl overhaul

*/
 
/*
DESCRIPTION

*/

#ifndef INC_GsuDlControllerSeth
#define INC_GsuDlControllerSeth

#include "dl/dlControllerSet.h"

#include "dl/dlPztController.h"
#include "gsuDlVcController.h"
#include "gsuDlMotorController.h"
#include "gsuDlIoSet.h"
#include "controllerSet.h"
#include "gsuDlModeUpdate.h"
#include "gsuDlTypes.h"



/***************  tGsuDlCompleteControllerSet Specification ***************

[[D tGsuDlCompleteControllerSet::tGsuDlCompleteControllerSet (
      const tGsuDlGizmoInfo& GizmoInfo, tGsuDlCompleteIoSet& IoSet) ]
[ invalid_pointer (this) ||
  invalid_reference (GizmoInfo, IoSet) => UNDEFINED
| true =>
    this->ControllerData_ := tGsuControllerData with default values
    : this->ServoParameters_ := GizmoInfo.ServoParameters
    : this->OffGroupUpdate := tGsuOffGroupUpdate with default values
    : this->TrackGroupUpdate := tGsuTrackGroupUpdate with default value
    : this->OffControllerGroup_ : tModedControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions and 
      DOCM_NUM_OFF_MODES controller modes with appropriately initialized 
      controllers for each partition and controller mode, 
      default update priority and period using a tGsuDlOffGroupUpdate 
      to switch controller modes
    : this->FrontControllerGroup_ : tSimpleControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions with
      appropriately initialized controllers for each partition
    : this->BackControllerGroup_ : tSimpleControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions with
      appropriately initialized controllers for each partition
    : this->FrontCreepControllerGroup_ : tSimpleControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions with
      appropriately initialized controllers for each partition
    : this->BackCreepControllerGroup_ : tSimpleControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions with
      appropriately initialized controllers for each partition
    : this->TrackControllerGroup_ : tModedControllerGroup with 
      GizmoInfo.ServoInfo.iNumberOfPartions servo partitions and 
      DTCM_NUM_TRACK_MODES controller modes with appropriately initialized 
      controllers for each partition and controller mode, 
      default update priority and period using a tGsuDlTrackGroupUpdate 
      to switch controller modes
]]      

[[D tGsuDlCompleteControllerSet::SetServoParameters (
      const tGsuDlServoParameters& NewServoParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (ServoParameters) => UNDEFINED
| true => 
    this->ServoParameters_ := NewServoParameters
    : appropriate servo parameters sent to all controllers in all controller
      groups
    : return := OK
]]

[[D tGsuDlCompleteControllerSet::ServoParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ServoParameters_
]]

[[D tGsuDlCompleteControllerSet::ControllerData (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ControllerData_
]]
     
[[D tGsuDlCompleteControllerSet::OffControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->OffControllerGroup_
]]
    
[[D tGsuDlCompleteControllerSet::FrontControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->FrontControllerGroup_
]]

[[D tGsuDlCompleteControllerSet::BackControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->BackControllerGroup_
]]

[[D tGsuDlCompleteControllerSet::FrontCreepControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->FrontCreepControllerGroup_ 
]]

[[D tGsuDlCompleteControllerSet::BackCreepControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->BackCreepControllerGroup_
]]

[[D tGsuDlCompleteControllerSet::TrackControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->TrackControllerGroup_
]]
  
[[D tGsuDlCompleteControllerSet::User1ControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->TrackControllerGroup_
]]

[[D tGsuDlCompleteControllerSet::User2ControllerGroup (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := reference to this->TrackControllerGroup_
]]


   ***/         
class tGsuDlCompleteControllerSet : public tDlCompleteControllerSet {
  public:
    tGsuDlCompleteControllerSet (const tGsuDlGizmoInfo& GizmoInfo,
        tGsuDlCompleteIoSet& IoSet, const tTypeTag& DlDataTag,
        tGsuDlStatus& DlStatus);
  
    ERROR_CODE SetServoParameters (const tGsuDlServoParameters& NewServoParameters);
    const tGsuDlServoParameters& ServoParameters () const;
    tGsuDlControllerData& ControllerData();

    virtual tControllerGroup& OffControllerGroup (void)
      {return this->OffControllerGroup_;}
    virtual tControllerGroup& FrontControllerGroup (void)
      {return this->FrontControllerGroup_;}
    virtual tControllerGroup& BackControllerGroup (void)
    {return this->BackControllerGroup_;}
    virtual tControllerGroup& FrontCreepControllerGroup (void)
      {return this->FrontCreepControllerGroup_;}
    virtual tControllerGroup& BackCreepControllerGroup (void)
      {return this->BackCreepControllerGroup_;}
    virtual tControllerGroup& TrackControllerGroup (void)
      {return this->TrackControllerGroup_;}
    virtual tControllerGroup& User1ControllerGroup (void)
      {return TrackControllerGroup_;}
    virtual tControllerGroup& User2ControllerGroup (void)
      {return this->TrackControllerGroup_;}

  private:
    tGsuDlControllerData  ControllerData_;
    tGsuDlServoParameters ServoParameters_;

    tGsuDlOffGroupUpdate  OffGroupUpdate;
    tGsuDlTrackGroupUpdate TrackGroupUpdate;
 
    tModedControllerGroup OffControllerGroup_;
    tSimpleControllerGroup FrontControllerGroup_;
    tSimpleControllerGroup BackControllerGroup_;
    tSimpleControllerGroup FrontCreepControllerGroup_;
    tSimpleControllerGroup BackCreepControllerGroup_;
    tModedControllerGroup TrackControllerGroup_;

    tDlPztController PztIdle;
    tGsuDlVcStabilizeController VcIdle;
    tGsuDlVcStabilizeController VcStabilize;
    tDlMotorSlewController MotorDecelerate;
    
    tDlMotorSlewController MotorFront;
    tDlMotorSlewController MotorBack;
    tDlMotorSlewController MotorFrontCreep;
    tDlMotorSlewController MotorBackCreep;

    tDlMotorTrackSlewController MotorTrackSlew;

    tDlPztTrackController PztTrack;
    tGsuDlVcTrackController VcTrack;
    tGsuDlMotorTrackController MotorTrack;
    

};


#endif  INC_gsuDlControllerSeth
