/* $RCSfile: gsuDlGizmo.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlGizmo.h,v 1.2 1998/09/23 23:57:32 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
$Log: gsuDlGizmo.h,v $
Revision 1.2  1998/09/23 23:57:32  phil
Changes to data recording so FIFOs dont fill up

Revision 1.1.1.1  1998/06/11 21:36:16  chara
First import of RICST Code

Revision 1.2.2.1  1998/04/30 20:36:04  rbrill
commit before build_5a_008

Revision 1.2  1998/04/01 02:45:19  phil
Phils config changes for I5a.  Hope it compiles...

Revision 1.1  1998/04/01 01:23:29  phil
moved all gsu specific header files into gsu subdirectory

Revision 1.6  1998/03/30 19:35:05  rich
*** empty log message ***

Revision 1.5  1998/03/12 17:59:32  rich
dl overhaul cont.

Revision 1.4  1998/03/12 01:05:06  rich
delay line overhaul cont.

*/
 
/*
DESCRIPTION
  Delay line Gizmo defines and types
*/
   

#ifndef INC_GsuDlGizmoh
#define INC_GsuDlGizmoh


#include "dlGizmo.h"
#include "gsu/gsuDlIoSet.h"
#include "gsu/gsuDlControllerSet.h"
#include "gsu/gsuDlGizmoInfo.h"

/******************* tGsuDlGizmoSpecification ********************************

[[D tGsuDlGizmo::tGsuDlGizmo (const tGsuDlGizmoInfo &GizmoInfo) ]
Constructs a tGsuDlGizmo. For more details about this gizmo see the
Delay Line Component Design document.
[ invalid_pointer (this) ||
  invalid_reference (GizmoInfo) ||
  invalid_configuration (GizmoInfo) ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::IpcMemory.bInit == FALSE ||
  ::Hardware.bInit == FALSE ||
  ::StatusStream.bInit == FALSE ||
  ::DataStream.bInit == FALSE ||
  ::ErrorStream.bInit == FALSE => UNDEFINED
| Unable to allocated Ipc objects for command stream ||
  Unable to allocate Ipc objects for the four delay line target double
  buffers ||
  Unable to add periodic task to read command stream ||
  Unable to add perioic task to collect status ||
  Unable to add periodic tasks to run DSP_NUM_SERVO_PARTITIONS servo
  partitions ||
  Unable to add periodic task to switch between the off controller modes ||
  Unable to add periodic task to switch between the track controller modes ||
  Unable to add periodic task to read delay line limit switches ||
  Unable to add periodic task to read delay line target buffers =>
     console := console + appropriate error messages
     : this->ecInitError := appropriate error code
     : *this (except ecInitError) := anything
| true =>
    ::IpcMemory := ::IpcMemory U { 
       IpcObject (GizmoInfo.iiCommandStreamId),
       IpcObject (GizmoInfo.iiDelayLineTargets [TT_OPEN_LOOP]),
       IpcObject (GizmoInfo.iiDelayLineTargets [TT_MANUAL]),
       IpcObject (GizmoInfo.iiDelayLineTargets [TT_EM]),
       IpcObject (GizmoInfo.iiDelayLineTargets [TT_FT]) }
    ::PeriodicTaskSet := PeriodicTaskSet U {
       PeriodicTask ("CommandProcessor" ...),
       PeriodicTask ("StatusRelayer", ...),
       PeriodicTask ("0_DlServo", ...), ...
       PeriodicTask (DSP_NUM_SERVO_PARTITIONS + "_DlServo", ...),
       PeriodicTask ("OffGroupUpdate", ...),
       PeriodicTask ("TrackGroupUpdate", ...),
       PeriodicTask ("DlLimits", ...),
       PeriodicTask ("DlTargetGenerator", ...) }
    this->CStatusTag := tTypeTag initialized for this particular instance of
       a tGsuDlGizmo
    this->Status := tGsuDlStatus initialized to default status
    this->CompleteIoSet := tGsuDlCompleteIoSet with all i/o initialized 
       according to the configuration contained in GizmoInfo
    this->CompleteControllerSet := tGsuCompleteControllerSet with all controller
       groups initialized according to the configuration contained in GizmoInfo
       and controllers connected to appropriate i/o
]]     

*********************** Private Specification ********************************

[[D tGsuDlGizmo::Dispatch (const tIsitCmd& Command) ]
Executes appropriate delay line command specified by Commmand.
[ invalid_pointer (this) ||
  invalid_reference (Command) ||
  ::ErrorStream (::pThisRteG).bInit == FALSE => UNDEFINED
| Command.Command not in { DL_CMD_SELECT_TARGET, DL_CMD_DITHER_ON,
    DL_CMD_OFF, DL_CMD_FRONT, DL_CMD_BACK, DL_CMD_TRACK, DL_CMD_USER1,
    DL_CMD_USER2, DL_CMD_HOME_CHECK, DL_CMD_HOME_SET, DL_CMD_DITHER_CALIBRATE,
    DL_CMD_DITHER_CHECK, DL_CMD_PING, DL_CMD_CLR_LASER_CTR, DL_CMD_MODULATE,
    DL_CMD_SET_DITHER, GSU_DL_CMD_CHNG_SERVO_PARAMETERS } => 
      return := appropriate error code
| Command.Command = DL_CMD_PING =>
    ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_OFF =>
    this->iServoMode := DSM_OFF
    : appropriate delay line stops moving
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_FRONT =>
    this->iServoMode := DSM_FRONT
    : appropriate delay line moves to front of track
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_BACK =>
    this->iServoMode := DSM_BACK
    : appropriate delay line moves to back of track
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages 
    : return := OK
| Command.Command = DL_CMD_TRACK =>
    this->iServoMode := DSM_TRACK
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      contained in the IPC target buffers
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_USER1 =>
    this->iServoMode := DSM_USER1
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      contained in the IPC target buffers
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_USER2 =>
    this->iServoMode := DSM_USER2
    : this->Status := appropiate status info
    : appropriate delay line begins to track the sum of all active targets in 
      contained in the IPC target buffers
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_SELECT_TARGET =>
    Target buffer corresponding to Command.ttWhichTarget activation state toggled
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_CLR_LASER_CTR =>
    this->iServoMode := DSM_OFF
    : appropriate delay line stops moving
    : this->LaserCounter cleared
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages 
    : return := OK
| Command.Command ==  DL_CMD_HOME_SET ||
  Command.Command === DL_CMD_HOME_CHECK =>
    this->iServoMode := any of { DSM_FRONT, DSM_BACK,
      DSM_FRONT_CREEP, DSM_BACK_CREEP, DSM_OFF}
    : this->Status := appropiate status info
    : appropriate delay line moves to home sensor
    : ::ErrorStream := ::ErrorStream + appropriate
    : return := OK
| Command.Command = DL_CMD_DITHER_ON && dither is already on =>
    : appropriate delay line pzt actuator ceases dithering
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_DITHER_ON && dither is off =>
    : appropriate delay line pzt actuator dithers with current dither parameters   
    : this->Status := appropiate status info
    : ::ErrorStream := ::ErrorStream + appropriate messages
    : return := OK
| Command.Command = DL_CMD_DITHER_CALIBRATE =>
    Start delay line dither calibration
| Command.Command = DL_CMD_DITHER_CHECK =>
    Start delay line dither calibration check
| Command.Command = DL_CMD_MODULATE => Incomplete
| Command.Command = DL_CMD_SET_DITHER => Incomplete
| Command.Command = GSU_DL_CMD_CHNG_SERVO_PARAMETERS  => Incomplete
]]
  
****/


    
class tGsuDlGizmo : public tDlGizmo {
  public:
    tGsuDlGizmo (const tGsuDlGizmoInfo &GizmoInfo);

  private: 
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
    virtual ERROR_CODE CollectStatus ();
    virtual tDlCompleteIoSet& GetCompleteIoSet () {
      return this->CompleteIoSet;}
    virtual tDlCompleteControllerSet& GetCompleteControllerSet () {
      return this->CompleteControllerSet;}
    virtual tDlControllerData& GetControllerData () {
      return this->CompleteControllerSet.ControllerData();}
    virtual tDlStatus& GetStatus () { return this->Status;}
    virtual double ServoError () { return this->CompleteControllerSet.ControllerData ().ServoError (); }

    ERROR_CODE SetServoParameters (const tGsuDlSetServoParametersCmd& Parameters);
    
    const tTypeTag CGsuDlStatusTag;
    const tTypeTag CGsuDlDataTag;
    
    tGsuDlStatus                Status;
    tGsuDlCompleteIoSet         CompleteIoSet; 
    tGsuDlCompleteControllerSet CompleteControllerSet;
    
};


#define S_gsuDlGizmo_ENTERING_GSU_DL_GIZMO_CONSTRUCTOR_BODY (M_gsuDlGizmo| 0x1)
#define S_gsuDlGizmo_EXITING_GSU_DL_GIZMO_CONSTRUCTOR     (M_gsuDlGizmo | 0x2)
#define S_gsuDlGizmo_GIZMO_INIT_SUCCEEDED                 (M_gsuDlGizmo | 0x3)


#endif INC_GsuDlGizmoh


