/* $RCSfile: gsuDlIoSet.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlIoSet.h,v 1.1.1.1 1998/06/11 21:36:17 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$Log: gsuDlIoSet.h,v $
Revision 1.1.1.1  1998/06/11 21:36:17  chara
First import of RICST Code

Revision 1.7  1998/03/12 17:59:33  rich
dl overhaul cont.

*/
 
/*
DESCRIPTION

*/

#ifndef INC_gsuDlIoSeth
#define INC_gsuDlIoSeth

#include "dl/dlIoSet.h"
#include "gsuDlGizmoInfo.h"

/****************** tGsuDlCompleteIoSet Specification ****************************************


[[D tGsuDlCompleteIoSet::tGsuDlCompleteIoSet (const tGsuDlGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) || 
  invalid_reference (GizmoInfo) ||
  invalid_pointer (GizmoInfo.pLaserChannel, GizmoInfo.pDitherPztChannel,
    GizmoInfo.pServoPztReadbackChannel, pMotorReadbackChannel,
    GizmoInfo.pDiagLaserInputChannel, pDiagPztPositionInputChannel,
    GizmoInfo.pFrontHardLimitChannel, GizmoInfo.pFrontSoftLimitChannel,
    GizmoInfo.pBackHardLimitChannel, GizmoInfo.pBackSoftLimitChannel,
    GizmoInfo.pHomeSensorChannel, GizmoInfo.pServoPztChannel, 
    GizmoInfo.pServoVc1Channel, GizmoInfo.pServoMotorChannel,
    GizmoInfo.pDiagLaserOutputChannel, GizmoInfo.pDiagVc1ServoErrorOutputChannel,
    GizmoInfo.pDiagMotorServoErrorOutputChannel) ||
  ::IpcMemory.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| Unabled to allocate necessary Ipc Objects or scheudule periodic tasks =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    this->Laser := laser metrology sensor connected to *GizmoInfo.pLaserChannel
      with disabled dither waveform connected to *GizmoInfo.pDitherPztChannel
      and with disabled modulation initialized to ZERO_MODULATION and other
      parameters specified by GizmoInfo
    : this->PztPosition := tModulatedSensor connected to 
      *GizmoInfo.pPztReadbackChannel with disabled modulation initialized to
      ZERO_MODULATION and other parameters specified by GizmoInfo
    : this->MotorPosition := tModulatedSensor connected to 
      *GizmoInfo.pMotorReadbackChannel with disabled modulation initialized to
      ZERO_MODULATION and other parameters specified by GizmoInfo
    : this->LaserDiagInput := tSensor connected to 
      *GizmoInfo.pDiagLaserInputChannel and initialized with other parameters 
      specified by GizmoInfo
    : this->PztPositionDiagInput := tSensor connected to 
      *GizmoInfo.pDiagPztPositionInputChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->DlLimits := tDlLimits connected to 
      *GizmoInfo.pFrontHardLimitChannel, *GizmoInfo.pFrontSoftLimitChannel,
      *GizmoInfo.pBackHardLimitChannel, *GizmoInfo.pBackSoftLimitChannel, and
      *GizmoInfo.pHomeSensorChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Pzt := tModulatedVelocityActuator connected to 
      *GizmoInfo.pServoPztChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Vc1 := tModulatedPositionActuator connected to 
      *GizmoInfo.pServoVc1Channel and initialized with other parameters 
      specified by GizmoInfo 
    : this->Motor := tModulatedVelocityActuator connected to 
      *GizmoInfo.pServoMotorChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->WaveformActuator := tActuator connected to 
      *GizmoInfo.pDitherPztChannel and initialized with other parameters 
      specified by GizmoInfo 
    : this->LaserDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagLaserOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->ServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagLaserServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->Vc1ServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagVc1ServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->MotorServoErrorDiagOutput := tActuator connected to 
      *GizmoInfo.pDiagMotorServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->TargetGenerator := tDlTargGen connected to four IPC double buffers of 
      type tDlTarget reading these buffers with a period specified by GizmoInfo.
    : ::IpcMemory := ::IpcMemory U { 
        IpcObject (GizmoInfo.iiOpenLoopTargetBufferId),
        IpcObject (GizmoInfo.iiManualTargetBufferId),
        IpcObject (GizmoInfo.iiEmTargetBufferId),
        IpcObject (GizmoInfo.iiFtTargetBufferId) }
    : ::PeriodicTaskSet := ::PeriodicTaskSet {
         PeriodicTask ("Dl Target Generator",...),
         PeriodicTask ("Dl Limit Update", ...) }      
    : this->LaserTarget := tPositionTargetSource connected to tDlTargGen 
      connected to four IPC double buffers of type tDlTarget
    : this->PztPositionTarget := tZeroTargetSource with defalt characteristics
    : this->MotorTarget := tVelocityTargetSource connected to tDlTargGen 
      connected to four IPC double buffers of type tDlTarget
    : this->Eddy1 := tModulatedSensor connected to *GizmoInfo.pEddy1Channel and initialized 
      with other parameters specified by GizmoInfo  
    : this->Eddy2 := tModulatedSensor connected to *GizmoInfo.pEddy2Channel and initialized 
      with other parameters specified by GizmoInfo  
    : this->Eddy1DiagInput := tModulatedSensor connected to *GizmoInfo.pDiagEddy1InputChannel
      and initialized with other parameters specified by GizmoInfo  
    : this->Eddy2DiagInput := tModulatedSensor connected to *GizmoInfo.pDiagEddy2InputChannel
      and initialized with other parameters specified by GizmoInfo  
    : this->Vc2 := tModulatedAccelerationActuator connected to *GizmoInfo.pVc2Channel
      and initialized with other parameters specified by GizmoInfo  
    : this->Vc2ServoDiagOutput := tModulatedAccelerationActuator connected to 
      *GizmoInfo.pDiagVc2ServoErrorOutputChannel and initialized with other parameters 
      specified by GizmoInfo  
    : this->Eddy1Target := tZeroTargetSource with defalt characteristics 
    : this->Eddy2Target := tZeroTargetSource with defalt characteristics 
]]    
*/


class tGsuDlCompleteIoSet : public tDlCompleteIoSet {
  public:
    tGsuDlCompleteIoSet (const tGsuDlGizmoInfo& GizmoInfo);

    // Sensors
    tModulatedSensor Eddy1;
    tModulatedSensor Eddy2;

    // Diagnostic inputs for servo performance tuning
    tModulatedSensor Eddy1DiagInput;
    tModulatedSensor Eddy2DiagInput;

    // Actuators
    tModulatedAccelerationActuator Vc2;  

    // Diagnostic outputs for displaying servo errors
    tActuator Vc2ServoErrorDiagOutput;
  
    // Target Sources
    tZeroTargetSource Eddy1Target;
    tZeroTargetSource Eddy2Target;
};


#endif  /* INC_gsuDlIoSeth */
