/* $RCSfile: gsuDlModeUpdate.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlModeUpdate.h,v 1.1.1.1 1998/06/11 21:36:17 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
/*
$Log: gsuDlModeUpdate.h,v $
Revision 1.1.1.1  1998/06/11 21:36:17  chara
First import of RICST Code

Revision 1.2.2.2  1998/05/05 15:28:20  rich
Delay line full function

Revision 1.2.2.1  1998/04/23 16:51:24  rich
changes for build 7

Revision 1.2  1998/03/12 17:59:35  rich
dl overhaul cont.

Revision 1.1  1998/03/11 00:59:32  rich
dl and gsuDl overhaul

*/

#ifndef INC_gsuDlModeUpdateh
#define INC_gsuDlModeUpdateh

#include "controllerSet.h"
#include "gsuDlTypes.h"
#include "gsuDlControllerData.h"


/*************** tGsuDlOffGroupUpdate Specification **************************

[[D tGsuDlOffGroupUpdate::tGsuDlOffGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse) ]
Contructs tGsuDlOffGroupUpdate with specified thresholds and controller data.      
[ invalid_pointer (this) ||
  invalid_reference (Thresholds, ControllerDataToUse) => UNDEFINED
| true => 
    this->dDecelToIdleVelocityThreshold := Thresholds.dDecelToIdleVelocityThreshold     
    : this->iCurrentControllerMode := DOCM_DECELERATE
    : this->ControllerData := reference to ControllerDataToUse
]]



[[D tGsuDlOffGroupUpdate::~tGsuDlOffGroupUpdate (void) ]
[ invalid_pointer (this)  => UNDEFINED
| true => *this := anything
]]
 


[[D tGsuDlOffGroupUpdate::SetThresholds (const tGsuDlThresholds& Thresholds) ]
Updates thresholds.      
[ invalid_pointer (this) ||
  invalid_reference (Thresholds, ControllerDataToUse) => UNDEFINED
| true => 
    this->dDecelToIdleVelocityThreshold := Thresholds.dDecelToIdleVelocityThreshold     
    : return := OK
]]


[[D tGsuDlOffGroupUpdate::Init (void) ] = tUpdateModeDecision::Init ]]

[[D tGsuDlOffGroupUpdate::Reset (void) ] = tUpdateModeDecision::Reset ]]

[[D tGsuDlOffGroupUpdate::GetNewMode (TIME tUpdateTime) ]
Switches off controller mode from DECELERATE to IDLE if the motor velocity is
less than the threshold.  
[ invalid_pointer (this) ||
  invalid_reference (this->ControllerData) => UNDEFINED
| this->iCurrentControllerMode == DOCM_DECELERATE &&
  this->ControllerData.dMotorVelocity < this->dDecelToIdleVelocityThreshold =>
    this->iCurrentControllerMode = DOCM_IDLE
    : return := DOCM_IDLE
| true => return := DOCM_DECELERATE
]]
    
[[D tGsuDlTrackGroupUpdate::tGsuDlTrackGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse) ]
Contructs tGsuDlTrackGroupUpdate with specified thresholds and controller data.      
[ invalid_pointer (this) ||
  invalid_reference (Thresholds, ControllerDataToUse)  => UNDEFINED
| true => 
    this->dTrackToSlewErrorThreshold := Thresholds.dTrackToSlewErrorThreshold    
    : this->dTrackToSlewErrorThreshold := Thresholds.dSlewToTrackErrorThreshold    
    : this->ControllerData := reference to ControllerDataToUse
    : this->iCurrentControllerMode := DTCM_SLEW
]]



[[D tGsuDlTrackGroupUpdate::~tGsuDlTrackGroupUpdate (void) ]
[ invalid_pointer (this)  => UNDEFINED
| true => *this := anything
]]
 

[[D tGsuDlTrackGroupUpdate::~tGsuDlTrackGroupUpdate (void) ]
[ invalid_pointer (this)  => UNDEFINED
| true => *this := anything
]]
  
  
[[D tGsuDlTrackGroupUpdate::SetThresholds (const tGsuDlThresholds& Thresholds) ]
Updates thresholds.      
[ invalid_pointer (this) ||
  invalid_reference (Thresholds)  => UNDEFINED
| true => 
    this->dTrackToSlewErrorThreshold := Thresholds.dTrackToSlewErrorThreshold    
    : this->dSlewToTrackErrorThreshold := Thresholds.dSlewToTrackErrorThreshold    
    : return := OK
]]
  
[[D tGsuDlTrackGroupUpdate::Init (void) ] = tUpdateModeDecision::Init ]]

[[D tGsuDlTrackGroupUpdate::Reset (void) ] = tUpdateModeDecision::Reset ]]

[[D tGsuDlTrackGroupUpdate::GetNewMode (TIME tUpdateTime) ]
Switches track controller mode between TRACK and SLEW based on the current 
laser error (ServoError) indicated by the controller data.
[ invalid_pointer (this) ||
  invalid_reference (this->ControllerData) => UNDEFINED
| this->iCurrentControllerMode == DTCM_SLEW &&
  this->ControllerData.dServoError < this->dSlewToTrackErrorThreshold =>
    this->iCurrentControllerMode = DTCM_TRACK
    : return := DTCM_TRACK
| this->iCurrentControllerMode == DTCM_TRACK &&
  this->ControllerData.dServoError > this->dTrackToSlewErrorThreshold =>
    this->iCurrentControllerMode = DTCM_SLEW
    : return := DTCM_SLEW
| true => return := this->iCurrentControllerMode    
]]
  


********/
    
class tGsuDlOffGroupUpdate : public tUpdateModeDecision {
  public:
    tGsuDlOffGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse);
    ERROR_CODE SetThresholds (const tGsuDlThresholds& Thresholds);
    ERROR_CODE Init () { this->iCurrentControllerMode = DOCM_DECELERATE; return OK;}
    int GetNewMode (TIME tUdpateTime);
  private:
    double dDecelToIdleVelocityThreshold;
    tGsuDlControllerData& ControllerData;
};


class tGsuDlTrackGroupUpdate : public tUpdateModeDecision {
  public:
    tGsuDlTrackGroupUpdate (const tGsuDlThresholds& Thresholds,
      tGsuDlControllerData& ControllerDataToUse);
    ERROR_CODE SetThresholds (const tGsuDlThresholds& Thresholds);
    ERROR_CODE Init () { this->iCurrentControllerMode = DTCM_SLEW; return OK;}
    ERROR_CODE Reset () { this->iCurrentControllerMode = DTCM_SLEW; return OK;}
    int GetNewMode (TIME tUdpateTime);
  private:
    double dTrackToSlewErrorThreshold;
    double dSlewToTrackErrorThreshold;
    tGsuDlControllerData& ControllerData;
};


#endif INC_gsuDlModeUpdateh