/* $RCSfile: gsuDlMotorController.h,v $*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlMotorController.h,v 1.3 1998/09/13 19:43:49 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
*/
 
/*
DESCRIPTION

*/



#ifndef INC_gsuDlMotorControllerh
#define INC_gsuDlMotorControllerh

#include "dl/dlMotorController.h"
#include "gsuDlIoSet.h"
#include "gsuDlControllerData.h"
#include "pidLibF.h"

/******************* Specification Functions ****************************************

double InputValue (tInputSource, TIME) - see inputSource.h
double UpdateInput (tInputSource, TIME) - see inputSource.h
double OutputValue (tOutputDestination, double, TIME) - see outputDest.h
double UpdateOutput (tOutputDestination, double, TIME) - see outputDest.h
double TargetValue (tTargetSource, TIME) - see targetSource.h
double UpdateTarget (tTargetSource, TIME) - see targetSource.h

double PID (tPidDouble, double) - see pidLibF.h
tPidDouble UpdatePid (tPidDouble, double) - see pidLibF.h


double EddyError (tSensor Input, tSensor Diag, tTargetSource EddyTarget, TIME tTime) :==
[ invalid_sensor (Input, Diag) ||
  invalid_target (EddyTarget) => UNDEFINED
| true => return := InputValue (Input, tTime) + InputValue (Diag, tTime) - 
          TargetValue (Target, tTime) 
]

double MotorTrackVelocity (BOOL bEnabled, tTargetSource MotorTarget, tPidDouble Pid, double dEddyError) :==
[ invalid_target (MotorTarget) => UNDEFINED
| bEnabled == FALSE => return := 0
| true => return := PID (Pid, dError + TargetValue (MotorTarget) )
]

double VelocityProfile (tModululatedVelocityActuator Motor, TIME tSamplePeriod,
                        double dAggressiveness, double dError) :==
Returns the maximum velocity that will enable the motor to decelerate to zero 
when the error reaches zero.                       
[ invalid_actuator (Motor) => UNDEFINED
| StoppingDistance (Motor.dValue, Motor.dMaxAcceleration, tSamplePeriod) * 
    dAggressiveness > ABS (dError) => return := this->Motor.dMaxVelocity * -SIGN (dError)
| true => 
    return :=  (-(Motor.dMaxAcceleration * tSamplePeriod / (2 * TIME_TICKS_PER_SECOND) +
      sqrt (Motor.dMaxAcceleration^2 * (tSamplePeriod / TIME_TICS_PER_SECOND)^2 + 
            8 * Motor.dMaxAcceleration * 
            StoppingDistance (Motor.dValue, Motor.dMaxAcceleration, tSamplePeriod) *
            dAggressiveness)
       / 2
]]     



************************* tGsuDlMotorTrackController Specification ***********************  

[[D tGsuDlMotorTrackController::tGsuDlMotorTrackController (
    tGsuDlControllerData& ControllerData, tGsuDlCompleteIoSet& IoSet,
    const tDlMotorParameters& InitialParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (ControllerData, IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->bEnabled := InitialParameters.bEnabled
    : this->tMotorSamplePeriod := InitialParameters.tMotorSamplePeriod
    : this->Eddy := reference to IoSet.Eddy2
    : this->EddyDiagInput := reference to IoSet.Eddy2DiagInput
    : this->MotorServoDiagOutput := reference to IoSet.MotorServoErrorDiagOutput
    : this->EddyTarget := reference to IoSet.Eddy2Target
    : this->MotorTarget := reference to IoSet.MotorTarget
    : this->Pid := PID initialized with parameters specified in InitialParameters
    : this->dMotorServoError := reference to ControllerData.dMotorServoError
]]

[[D tGsuDlMotorTrackController::~tGsuDlMotorTrackController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tGsuDlMotorTrackController::SetControlParameters (const tGsuDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true => 
    this->tMotorSamplePeriod := NewParameters.tMotorSamplePeriod
    : this->Pid := PID initialized with parameters specified in NewParameters
]]

[[D tGsuDlMotorTrackController::Enabled (void) ] = tDlMotorController::Enabled ]]

[[D tGsuDlMotorTrackController::Velocity (void) ] = tDlMotorController::Velocity ]]


[[D tGsuDlMotorTrackController::DesiredVelocity (void) ] = tDlMotorController::DesiredVelocity ]]

[[D tGsuDlMotorTrackController::Init (TIME tServoTime) ]
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| true =>
    Motor device associated with this->Motor reset
    : this->Motor := ResetOutput (this->Motor)
    : this->MotorPid := reset tPidDouble with 0 as reset value
    : return := OK
]]

[[D tGsuDlMotorTrackController::Reset (TIME tServoTime) ] = tGsuDlMotorTrackController::Init ]]



[[D tGsuDlMotorTrackController::Calculate (TIME tServoTime) ]
Computes motor velocity to center voice coil on the eddy target while 
compensating for the known OPD target velocity.  Moves motor at computed velocity and
output servo error on diagnostic channel.
If a limit switch is tripped and the desired velocity is in the same direction
as the limit, the motor velocity is set to zero.
[ invalid_pointer (this) ||
   invalid_reference (this->Eddy, this->EddyDiagInput, this->Motor, 
     this->ServoErrorDiagOutput, this->EddyTarget, this->MotorTarget) ||
   invalid_sensor (this->Eddy, this->EddyDiagInput) ||
   invalid_actuator (this->Motor, this->ServoErrorDiagOutput) ||
   invalid_target (this->EddyTarget, this->MotorTarget) => UNDEFINED
| MovePastLimit (this->Limits, MotorTrackVelocity (this->bEnabled, this->MotorTarget, 
      this->MotorPid, EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget) ) ) => 
    this->Motor.dValue := 0
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, 0, tServoTime)
    : return := OK
| true =>
    this->Eddy := UpdateInput (this->Eddy, tServoTime)
    : this->EddyDiagInput := UpdateInput (this->EddyDiagInput, tServoTime)
    : this->EddyTarget := UpdateTarget (this->EddyTarget, tServoTime)
    : this->MotorTarget := UpdateTarget (this->MotorTarget, tServoTime)
    : this->MotorServoErrorDiagOutput := UpdateOutput (this->MotorServoErrorDiagOutput,
      EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget), tServoTime)
    : this->Motor := UpdateOuput (this->Motor, MotorTrackVelocity (this->bEnabled, 
      this->MotorTarget, this->MotorPid, EddyError (this->Eddy, this->EddyDiagInput, 
      this->EddyTarget) ), tServoTime) 
    : this->MotorPid := UpdatePid (this->MotorPid, 
      EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget) + 
      TargetValue (this->MotorTarget, tServoTime) )
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, MotorTrackVelocity (this->bEnabled, this->MotorTarget, 
      this->MotorPid, EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget) ),
      tServoTime)
    : OutputValue (this->MotorServoErrorDiagOutput, 
      EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget), tServoTime)
      sent to output associated with this->MotorServoErrorDiagOutput 
    : return := OK
]]


********************* Protected Specification *******************************************

[[D tGsuDlMotorTrackController::ServoError (TIME tServoTime) ]
Computes and returns motor servo error based on the measured voice coil position.
Sends error to diagnostic output. 
[ invalid_pointer (this) ||
  invalid_reference (this->Eddy, this->EddyDiag, this->EddyTarget, 
    this->MotorServoErrorDiagOutput) ||
  invalid_input (this->Eddy, this->EddyDiag) ||
  invalid_output (this->MotorServoErrorDiagOutput) ||
  invalid_target (this->EddyTarget) => UNDEFINED
| true => 
    this->Eddy := UpdateInput (this->Eddy, tServoTime)
    : this->EddyDiagInput := UpdateInput (this->EddyDiagInput, tServoTime)
    : this->EddyTarget := UpdateTarget (this->EddyTarget, tServoTime)
    : this->MotorServoErrorDiagOutput := UpdateOutput (this->MotorServoErrorDiagOutput,
      EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget), tServoTime)
    : OutputValue (this->MotorServoErrorDiagOutput, 
      EddyError (this->Eddy, this->EddyDiagInput, this->EddyTarget), tServoTime)
      sent to output associated with this->MotorServoErrorDiagOutput 
    : return := EddyError (this->Eddy, this->EddyDiag, this->EddyTarget, tServoTime)
]]


[[D tGsuDlMotorTrackController::MotorFeedforward (TIME tServoTime) ]
Returns motor feedforward velocity from the motor target.
[ invalid_pointer (this) ||
  invalid_reference (this->MotorTarget) ||
  invalid_target (this->MotorTarget) => UNDEFINED
| true => 
    this->MotorTarget := UpdateTarget (this->MotorTarget, tServoTime)
    : return := TargetValue (this->MotorTarget, tServoTime)
]]

[[D tGsuDlMotorTrackController::Correction (double dError) ]
Returns motor velocity to compensate for specified error using pid.
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->MotorPid := UpdatePid (this->MotorPid, dError)
    : return := PID (this->MotorPid, dError)
]]

[[D tGsuDlMotorTrackController::DecelerationDistanceInMotorUnits () ] = 
    tDlMotorController::ecelerationDistanceInMotorUnits ]]

[[D tGsuDlMotorTrackController::Actuate (double dMotorVelocity) ] = 
    tDlMotorController::Actuate ]]

[[D tGsuDlMotorTrackController::MotorToLaserUnits (double dMotorUnits) ] = 
    tDlMotorController::MotorToLaserUnits ]]

[[D tGsuDlMotorTrackController::LaserToMotorUnits (double dLaserUnits) ] = 
    tDlMotorController::LaserToMotorUnits ]]

[[D tGsuDlMotorTrackController::PhysicalToMotorUnits (double dPhysicalUnits) ] = 
    tDlMotorController::PhysicalToMotorUnits ]]

[[D tGsuDlMotorTrackController::MotorToPhysicalUnits (double dMotorUnits) ] = 
    tDlMotorController::MotorToPhysicalUnits ]]


******************* tGsuDlMotorTrackSlewController Specification *************************

[[D tGsuDlMotorTrackSlewController::tGsuDlMotorTrackSlewController (tGsuDlControllerData& ControllerData,
      tGsuDlCompleteIoSet& IoSet, const tGsuDlMotorParameters& InitialParameters,
      double dLaserPhysicalPerCount, int iLaserPasses) ]
[ invalid_pointer (this) ||
  invalid_reference (ControllerData, IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Limits := reference to IoSet.Limits
    : this->Motor := reference to IoSet.Motor
    : this->bEnabled := InitialParameters.bEnabled
    : this->tMotorSamplePeriod := InitialParameters.tMotorSamplePeriod
    : this->dMotorSlewVelocity := InitialParameters.dMotorSlewVelocity
    : this->dDecelerationAggressiveness := InitialParameters.dDecelerationAggressiveness
    : this->dLaserError := reference to ControllerData.dLaserError    
]]

[[D tGsuDlMotorTrackSlewController::~tGsuDlMotorTrackSlewController (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tGsuDlMotorTrackSlewController::SetControlParameters (const tDlMotorParameters& NewParameters) ]
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true => 
    this->tMotorSamplePeriod := NewParameters.tMotorSamplePeriod
    : this->dMotorSlewVelocity := NewParameters.dMotorSlewVelocity
    : this->dDecelerationAggressiveness := NewParameters.dDecelerationAggressiveness
    : return := OK
]]


[[D tGsuDlMotorTrackSlewController::Enabled (void) ] = tDlMotorSlewController::Enabled ]]

[[D tGsuDlMotorTrackSlewController::Velocity (void) ] = tDlMotorSlewController::Velocity ]]

[[D tGsuDlMotorTrackSlewController::Init (TIME tServoTime) ] = tDlMotorSlewController::Init ]]

[[D tGsuDlMotorTrackSlewController::Reset (TIME tServoTime) ] = tDlMotorSlewController::Reset ]]

[[D tGsuDlMotorTrackSlewController::DesiredVelocity (void) ] = tDlMotorSlewController::DesiredVelocity ]]


[[D tGsuDlMotorTrackSlewController::Calculate (TIME tServoTime) ]
Moves motor actuator so that the OPD error goes to zero using a profiled velocity
based on the maximum motor velocity, the maximum motor acceleration and the 
decelearation aggressiveness specified. 
[ invalid_pointer (this) ||
   invalid_reference (this->Motor) ||
   invalid_actuator (this->Motor) => UNDEFINED
| this->bEnabled == FALSE ||
  MovePastLimit (this->Limits, VelocityProfile (this->Motor, 
    this->dDecelerationAggressiveness, this->dLaserError) ) => 
    this->Motor.dValue := 0
    : motor actuator associated with this->Motor moves at velocity 
      OutputValue (this->Motor, 0, tServoTime)
    : return := OK
| true =>
    this->Motor := UpdateOutput (this->Motor, 
      VelocityProfile (this->Motor, this->dDecelerationAggressiveness, 
      this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT), 
      tServoTime)
    : motor actuator associated with this->Motor moves at velocity OutputValue (this->Motor, 
      VelocityProfile (this->Motor, this->dDecelerationAggressiveness, 
        this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT), 
      tServoTime)
    : return := OK
]]

********************* Protected Specification **************************************

[[D tGsuDlMotorTrackSlewController::ProfiledVelocity (void) ]
[ invalid_pointer (this) ||
  invalid_reference (this->Motor, this->dLaserError) ||
  invalid_actuator (this->Motor) => UNDEFINED 
| true => 
    return := VelocityProfile (this->Motor, this->tMotorSampleTime,
      this->dDecelerationAggressiveness, this->dLaserError * MOTOR_UNITS_PER_LASER_UNIT)
]]

********/


class tGsuDlMotorTrackController : public tDlMotorController {
  public:
    tGsuDlMotorTrackController (tGsuDlControllerData& ControllerData,
      tGsuDlCompleteIoSet& IoSet, const tGsuDlMotorParameters& InitialParameters,
      double dLaserPhysicalPerCount, int iLaserPasses);
    ERROR_CODE Init(TIME tServoTime);
    ERROR_CODE Reset(TIME tServoTime);
    ERROR_CODE SetControlParameters (const tGsuDlMotorParameters& NewParameters);
    ERROR_CODE Calculate (TIME tServoTime);
  protected:
    double ServoError (TIME tServoTime);
 //   double MotorFeedforward (TIME tServoTime);
    double Correction (double dError); 
  private:
    tSensor& Eddy;
    tModulatedSensor& EddyDiagInput;
    tActuator& MotorServoErrorDiagOutput;
    tTargetSource& EddyTarget;
    
    tPidDouble MotorPid;
    double& dMotorServoError;
};




#endif INC_gsuDlMotorControllerh
