/* $RCSfile: gsuDlTypes.h,v $*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlTypes.h,v 1.2 1998/09/13 19:43:50 phil Exp $
*/

/*
$Log: gsuDlTypes.h,v $
Revision 1.2  1998/09/13 19:43:50  phil
After Demo.  All functions work with 3 known problems (see Demo report)

Revision 1.1.1.1  1998/06/11 21:36:17  chara
First import of RICST Code

Revision 1.1.2.3  1998/05/12 20:22:47  rich
commit before 5b

Revision 1.1.2.2  1998/04/30 20:36:06  rbrill
commit before build_5a_008

Revision 1.1.2.1  1998/04/23 16:51:25  rich
changes for build 7

Revision 1.1  1998/04/01 01:23:30  phil
moved all gsu specific header files into gsu subdirectory

Revision 1.2  1998/03/12 01:05:07  rich
delay line overhaul cont.

Revision 1.1  1998/03/11 00:59:30  rich
dl and gsuDl overhaul

*/

#ifndef INC_gsuDlTypesh
#define INC_gsuDlTypesh

#include "dlTypes.h"

/****************tGsuDlVcParameters Specification*****************************


[[D tGsuDlVcParameters::tGsuDlVcParameters (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tGsuDlVcParameters::tGsuDlVcParameters (BOOL bVc1EnabledIn, BOOL bVc2EnabledIn,
      double dVc1PidGainIn, double dVc1PidTau1In, double dVc1PidTau2In, double dVc1PidTaudIn,
      double dVc2PidGainIn, double dVc2PidTau1In, double dVc2PidTau2In, double dVc2PidTaudIn,
      double dVcSamplePeriodIn, double dVc1FeedforwardGainIn, double dVc2FeedforwardGainIn,
      int iVc1FilterMemoryIn, int iVc2FilterMemoryIn, double dVcUpperLimitIn, double dVcLowerLimitIn) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->bVc1Enabled := bVc1EnabledIn
    : this->bVc2Enabled := bVc2EnabledIn

    : this->dVc1PidGain := dVc1PidGainIn
    : this->dVc1PidTau1 := dVc1PidTau1In
    : this->dVc1PidTau2 := dVc1PidTau2In
    : this->dVc1PidTaud := dVc1PidTaudIn

    : this->dVc1PidGain := dVc1PidGainIn
    : this->dVc2PidTau1 := dVc2PidTau1In
    : this->dVc2PidTau2 := dVc2PidTau2In
    : this->dVc2PidTaud := dVc2PidTaudIn

    : this->dVcSamplePeriod := dVcSamplePeriodIn

    : this->dVc1FeedforwardGain := dVc1FeedforwardGainIn
    : this->dVc2FeedforwardGain := dVc2FeedforwardGainIn
    : this->idVc1FilterMemory := iVc1FilterMemoryIn
    : this->iVc2FilterMemory := iVc2FilterMemoryIn
    : this->dVcUpperLimit := dVcUpperLimitIn
    : this->dVcLowerLimit := dVcLowerLimitIn

]]  

**/

struct tGuiGsuDlServoParameters {
  
    // Servo Rates in Hz
    
    double dPztServoRate;
    double dVcServoRate;
    double dMotorServoRate;
 
    // Servo Enables
    
    BOOL bPztServoEnabled;
    BOOL bPztLagEnabled;
    BOOL bVc1TrackServoEnabled;
    BOOL bVc1StabilizeServoEnabled;
    BOOL bVc2TrackServoEnabled;
    BOOL bVc2StabilizeServoEnabled;
    BOOL bMotorServoEnabled;

    // Pzt Control Parameters 
    
    double dPztGain;
    double dPztLagPole;
    double dPztLagZero;
 
    // VC 1 Control Parameters
    
    double dVc1StabilizePidGain;
    double dVc1StabilizePidZero1;
    double dVc1StabilizePidZero2;
    double dVc1StabilizePidPole;

    double dVc1TrackPidGain;
    double dVc1TrackPidZero1;
    double dVc1TrackPidZero2;
    double dVc1TrackPidPole;
    double dVc1TrackLaserFeedforwardGain;
    double dVc1TrackFilterMemory;

    // VC 2 Control Parameters

 
    double dVc2StabilizePidGain;
    double dVc2StabilizePidZero1;
    double dVc2StabilizePidZero2;
    double dVc2StabilizePidPole;
    double dVc2StabilizeMotorAccelerationFeedforwardGain;
 
    double dVc2TrackPidGain;
    double dVc2TrackPidZero1;
    double dVc2TrackPidZero2;
    double dVc2TrackPidPole;
    double dVc2TrackVc1ForceFeedforwardGain;
   
    // Motor Control Parameters
    
    double dMotorSlewVelocity;
    double dMotorCreepVelocity;
    double dDecelerationAggressiveness;            
    double dMotorPidGain;
    double dMotorPidZero1;
    double dMotorPidZero2;
    double dMotorPidPole;
    double dMotorFeedForwardGain;
    

    // Controller Mode Thresholds

    double dDecelToIdleVelocityThreshold;
    double dSlewToTrackThresholdInMm;
    double dTrackToSlewThresholdInMm;


};

class tGsuDlVcParameters {
  public:
    tGsuDlVcParameters () {}
    tGsuDlVcParameters (BOOL bVc1EnabledIn, BOOL bVc2EnabledIn,
      double dVc1PidGainIn, double dVc1PidTau1In, double dVc1PidTau2In, double dVc1PidTaudIn,
      double dVc2PidGainIn, double dVc2PidTau1In, double dVc2PidTau2In, double dVc2PidTaudIn,
      double dVcSamplePeriodIn) :
        bVc1Enabled (bVc1EnabledIn), bVc2Enabled (bVc2EnabledIn), dVc1PidGain (dVc1PidGainIn),
        dVc1PidTau1 (dVc1PidTau1In), dVc1PidTau2 (dVc1PidTau2In), dVc1PidTaud (dVc1PidTaudIn),
        dVc2PidGain (dVc2PidGainIn), dVc2PidTau1 (dVc2PidTau1In), dVc2PidTau2 (dVc1PidTau2In), 
        dVc2PidTaud (dVc2PidTaudIn), dVcSamplePeriod (dVcSamplePeriodIn) {}
        
  
    BOOL bVc1Enabled;
    BOOL bVc2Enabled;
    
    double dVc1PidGain;
    double dVc1PidTau1;
    double dVc1PidTau2;
    double dVc1PidTaud;
    
    double dVc2PidGain;
    double dVc2PidTau1;
    double dVc2PidTau2;
    double dVc2PidTaud;
  
    double dVcSamplePeriod;   
  
};

class tGsuDlVcStabilizeParameters : public tGsuDlVcParameters {
  public:
    tGsuDlVcStabilizeParameters () : tGsuDlVcParameters () {}
    tGsuDlVcStabilizeParameters (BOOL bVc1EnabledIn, BOOL bVc2EnabledIn,
      double dVc1PidGainIn, double dVc1PidTau1In, double dVc1PidTau2In, double dVc1PidTaudIn,
      double dVc2PidGainIn, double dVc2PidTau1In, double dVc2PidTau2In, double dVc2PidTaudIn,
      double dVcSamplePeriodIn, double dVc2FeedforwardGainIn) : tGsuDlVcParameters (
         bVc1EnabledIn, bVc2EnabledIn, dVc1PidGainIn, dVc1PidTau1In, dVc1PidTau2In, dVc1PidTaudIn,
        dVc2PidGainIn, dVc2PidTau1In, dVc1PidTau2In, dVc2PidTaudIn, dVcSamplePeriodIn), 
        dVc2FeedforwardGain (dVc2FeedforwardGainIn)
        {}
        
  
   double dVc2FeedforwardGain;
};


class tGsuDlVcTrackParameters : public tGsuDlVcParameters {
  public:
    tGsuDlVcTrackParameters () : tGsuDlVcParameters () {}
    tGsuDlVcTrackParameters (BOOL bVc1EnabledIn, BOOL bVc2EnabledIn,
      double dVc1PidGainIn, double dVc1PidTau1In, double dVc1PidTau2In, double dVc1PidTaudIn,
      double dVc2PidGainIn, double dVc2PidTau1In, double dVc2PidTau2In, double dVc2PidTaudIn,
      double dVcSamplePeriodIn, double dVc1FeedforwardGainIn, double dVc2FeedforwardGainIn,
      double dVc1FilterMemoryIn) :tGsuDlVcParameters (
         bVc1EnabledIn, bVc2EnabledIn, dVc1PidGainIn, dVc1PidTau1In, dVc1PidTau2In, dVc1PidTaudIn,
         dVc2PidGainIn, dVc2PidTau1In, dVc1PidTau2In, dVc2PidTaudIn, dVcSamplePeriodIn), 
        dVc1FeedforwardGain (dVc1FeedforwardGainIn), dVc2FeedforwardGain (dVc2FeedforwardGainIn),
        dVc1FilterMemory (dVc1FilterMemoryIn) {}
        
  
  
    double dVc1FeedforwardGain;
    double dVc2FeedforwardGain;
    
    double dVc1FilterMemory;  
};


class tGsuDlMotorParameters : public tDlMotorParameters {
   public:
    tGsuDlMotorParameters () {}
    tGsuDlMotorParameters (BOOL bEnabledIn, double dSlewVelocity, 
       double dAggressiveness, double dSamplePeriod, double dFeedforwardGain,
       double dGain, double dTau1, double dTau2, double dTaud) :
         tDlMotorParameters (bEnabledIn,
         dSlewVelocity, dAggressiveness, dSamplePeriod, dFeedforwardGain),
         dMotorGain (dGain), dMotorPidTau1 (dTau1), dMotorPidTau2 (dTau2),
         dMotorPidTaud (dTaud) {}
    double dMotorGain;
    double dMotorPidTau1;
    double dMotorPidTau2;
    double dMotorPidTaud;
};


class tGsuDlThresholds {
  public:
    tGsuDlThresholds () {}
    tGsuDlThresholds (double dDecelToIdleIn, double dTrackToSlewIn, double dSlewToTrackIn) :
      dDecelToIdleVelocityThreshold (dDecelToIdleIn),
      dTrackToSlewErrorThreshold (dTrackToSlewIn),
      dSlewToTrackErrorThreshold (dSlewToTrackIn) {}

    double dDecelToIdleVelocityThreshold;
    double dTrackToSlewErrorThreshold;
    double dSlewToTrackErrorThreshold;
};




class tGsuDlServoParameters {
  public:
     tGsuDlServoParameters () {}
     tGsuDlServoParameters (const tGuiGsuDlServoParameters& GuiParametersIn) :
       GuiParameters_ (GuiParametersIn) {}

     const tGuiGsuDlServoParameters& GuiParameters () const { return this->GuiParameters_;}

     tDlPztParameters PztParameters () const {
       return tDlPztParameters (GuiParameters_.bPztServoEnabled, GuiParameters_.bPztLagEnabled,
         GuiParameters_.dPztGain, GuiParameters_.dPztLagPole, GuiParameters_.dPztLagZero,
         1.0 / GuiParameters_.dPztServoRate);
     }
     
     tGsuDlVcStabilizeParameters VcOffParameters () const {
       return tGsuDlVcStabilizeParameters (FALSE, FALSE, 
         GuiParameters_.dVc1StabilizePidGain, GuiParameters_.dVc1StabilizePidZero1, 
         GuiParameters_.dVc1StabilizePidZero2,  GuiParameters_.dVc1StabilizePidPole, 
         GuiParameters_.dVc2StabilizePidGain, GuiParameters_.dVc2StabilizePidZero1,
         GuiParameters_.dVc2StabilizePidZero2, GuiParameters_.dVc2StabilizePidPole,
         1 / GuiParameters_.dVcServoRate,  
         GuiParameters_.dVc2StabilizeMotorAccelerationFeedforwardGain);
      }

     tGsuDlVcStabilizeParameters VcStabilizeParameters () const {
       return tGsuDlVcStabilizeParameters (GuiParameters_.bVc1StabilizeServoEnabled, 
         GuiParameters_.bVc2StabilizeServoEnabled, 
         GuiParameters_.dVc1StabilizePidGain, GuiParameters_.dVc1StabilizePidZero1, 
         GuiParameters_.dVc1StabilizePidZero2,  GuiParameters_.dVc1StabilizePidPole, 
         GuiParameters_.dVc2StabilizePidGain, GuiParameters_.dVc2StabilizePidZero1,
         GuiParameters_.dVc2StabilizePidZero2, GuiParameters_.dVc2StabilizePidPole,
         1 / GuiParameters_.dVcServoRate, GuiParameters_.dVc2StabilizeMotorAccelerationFeedforwardGain);
     }

     tGsuDlVcTrackParameters VcTrackParameters () const {
       return tGsuDlVcTrackParameters (GuiParameters_.bVc1TrackServoEnabled, 
         GuiParameters_.bVc2TrackServoEnabled, 
         GuiParameters_.dVc1TrackPidGain, GuiParameters_.dVc1TrackPidZero1, 
         GuiParameters_.dVc1TrackPidZero2,  GuiParameters_.dVc1TrackPidPole, 
         GuiParameters_.dVc2TrackPidGain, GuiParameters_.dVc2TrackPidZero1, 
         GuiParameters_.dVc2TrackPidZero2, GuiParameters_.dVc2TrackPidPole,
         1 / GuiParameters_.dVcServoRate, GuiParameters_.dVc1TrackLaserFeedforwardGain, 
         GuiParameters_.dVc2TrackVc1ForceFeedforwardGain, 
         GuiParameters_.dVc1TrackFilterMemory);
     }

     tDlMotorParameters MotorOffParameters () const {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         0, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     } 
    tDlMotorParameters MotorFrontParameters () const {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         -GuiParameters_.dMotorSlewVelocity, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     } 
         
     tDlMotorParameters MotorBackParameters () const  {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         GuiParameters_.dMotorSlewVelocity, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     }

     tDlMotorParameters MotorFrontCreepParameters () const {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         -GuiParameters_.dMotorCreepVelocity, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     }
       
     tDlMotorParameters MotorBackCreepParameters () const {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         GuiParameters_.dMotorCreepVelocity, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     }
       
     tDlMotorParameters MotorTrackSlewParameters () const  {
       return tDlMotorParameters (GuiParameters_.bMotorServoEnabled,  
         GuiParameters_.dMotorSlewVelocity, GuiParameters_.dDecelerationAggressiveness, 
         1.0 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain);
     } 
       
     tGsuDlMotorParameters MotorTrackParameters () const {
       return tGsuDlMotorParameters (GuiParameters_.bMotorServoEnabled, 
         GuiParameters_.dMotorSlewVelocity, GuiParameters_.dDecelerationAggressiveness,
         1 / GuiParameters_.dMotorServoRate, GuiParameters_.dMotorFeedForwardGain,
         GuiParameters_.dMotorPidGain,
         GuiParameters_.dMotorPidZero1, GuiParameters_.dMotorPidZero2,
         GuiParameters_.dMotorPidPole);
     }
     
     tGsuDlThresholds Thresholds () const {
       return tGsuDlThresholds (GuiParameters_.dDecelToIdleVelocityThreshold,
         GuiParameters_.dTrackToSlewThresholdInMm, 
         GuiParameters_.dSlewToTrackThresholdInMm);
     }

   private:
     tGuiGsuDlServoParameters GuiParameters_; 
};




class tGsuDlStatus: public tDlStatus {
  public:
    tGsuDlStatus () {}
    tGsuDlStatus (int iDlNumber) { Initialize (iDlNumber);}
    void UpdateTrackServoParams(const tGuiGsuDlServoParameters& NewServoParameters) {
      this->ServoParameters = NewServoParameters;
    }
  
    tGuiGsuDlServoParameters ServoParameters;
};
 
#endif
