/* $RCSfile: gsuDlVcController.h,v $*/
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/gsu/gsuDlVcController.h,v 1.2 1998/08/17 21:01:20 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
05a,29Jan98,rlj   written
*/
 
/*
DESCRIPTION
  GSU Delay Line Voice coil Controllers.
*/
  
#ifndef INC_gsuDlVcControllerh
#define INC_gsuDlVcControllerh 

#include "isit.h"
#include "controller.h"
#include "pidLibF.h"
#include "feedforward.h"
#include "filtLibF.h"

#include "gsuDlIoSet.h"
#include "gsuDlControllerData.h"


/*********** Specification Functions ****************************************

double TargetValue (tTargetSource) - see TargetSource.h (someday)

double MODULATION (tModulationParams,TIME) - see modLib.h 
double DITHER (tDitherParams,TIME) - see dither.h (someday)

double PID (tPidDouble, double) - see pidLibF.h
tPidDouble UpdatePid (tPidDouble, double) - see pidLibF.h 

double FEEDFORWARD (tFeedforward, double) - see feedforward.h 

double LOW_PASS (tFilter, double) - see filtLibF.h 
tFilter UpdateFilter (tFilter, double) - see filtLib.h 



double VcError (tSensor Input, tSensor Diag, tTargetSource Target, TIME tTime) :==
Return the servo error for specified input, diagnostic, and target at time tTime.
[ invalid_sensor (Input, Diag) ||
  invalid_target (Target) => UNDEFINED
| true =>  
    return := InputValue (Input, tTime) + InputValue (this->Diag, tTime) -
              TargetValue (this->Target, tTime)
] 


****************tGsuDlVcController Specification*****************************



[[D tGsuDlVcController::tGsuDlVcController (tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, 
        tSensor& Vc1InputIn, tSensor& Vc1DiagInputIn, 
        tSensor& Vc2InputIn, tSensor& Vc2DiagInputIn, 
        tTargetSource& Vc1TargetIn, tTargetSource& Vc2TargetIn) ]
Construct this and initializes references to IoSet, ControllerData, and initialize
controller parameters.
[ invalid_pointer (this) ||
  invalid_reference (IoSet, Vc1InputIn, Vc1DiagInputIn, Vc2InputIn, Vc2DiagInputIn,
                     Vc1TargetIn, Vc2TargetIn) => UNDEFINED
| true =>
    this->Vc1Input := reference to Vc1InputIn
    : this->Vc1DiagInput := reference to Vc1DiagInputIn
    : this->Vc2Input := reference to Vc2InputIn
    : this->Vc2DiagInput := reference to Vc2DiagInputIn
    
    : this->Vc1 := reference to IoSet.Vc1
    : this->Vc1ServoErrorDiagOutput := reference to IoSet.Vc1ServoErrorDiagOutput
    : this->Vc2 := reference to IoSet.Vc2
    : this->Vc2ServoErrorDiagOutput := reference to IoSet.Vc2ServoErrorDiagOutput

    : this->Vc1Target := reference to Vc1TargetIn
    : this->Vc2Target : reference to Vc2TargetIn

    : this->dVc1Error := reference to ControllerData.dVc1Error
    : this->dVc2Error := reference to ControllerData.dVc2Error
    
    : this->dVc1Output := reference to ControllerData.dVc1Output
    : this->dVc2Output := reference to ControllerData.dVc2Output

    : this->bVc1Enabled := TRUE
    : this->bVc2Enabled := TRUE
]]              


[[D tGsuDlVcController::~tGsuDlVcController (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]

 


[[D tGsuDlVcController::Vc1Enabled (void) ]
Returns copy or reference to VC1 enabled state.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bVc1Enabled
| true => return := reference to this->bVc1Enabled
]]

[[D tGsuDlVcController::Vc2Enabled (void) ]
Returns copy or reference to VC2 enabled state.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bVc2Enabled
| true => return := reference to this->bVc2Enabled
]]



[[D tGsuDlVcController::Reset (TIME tServoTime) ] = tController::Init ]]


[[D tGsuDlVcController::Calculate (TIME tServoTime) ] = tController::Calculate ]]


************** protected specification******************************

[[D tGsuDlVcController::Vc1ServoError (TIME tTime) ]
Computes servo error for Vc1 at time tTime.
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target) => UNDEFINED
| true =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tTime)
    : this->dVc1ServoError := InputValue (this->Vc1Input, tTime) + InputValue (this->Vc1DiagInput, tTime) - 
      TargetValue (this->Vc1Target, tTime)
    : this->Vc1ServoErrorDiagOutput := UpdateOutput (this->Vc1ServoErrorDiagOutput, 
         InputValue (this->Vc1Input, tTime) + InputValue (Diag, tTime) - TargetValue (this->Vc1Target, tTime), tTime)
    : return := InputValue (this->Vc1Input, tTime) + InputValue (this->Vc1DiagInput, tTime) - 
      TargetValue (this->Vc1Target, tTime)
]]

[[D tGsuDlVcController::Vc2ServoError (TIME tTime) ]
Computes servo error for Vc2 at time tTime.
[ invalid_pointer (this) ||
  invalid_reference (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target) => UNDEFINED
| true =>
    this->Vc2Input := UpdateInput (this->Vc2Input, tTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tTime)
    : this->dVc2ServoError := InputValue (this->Vc2Input, tTime) + InputValue (this->Vc2DiagInput, tTime) - 
      TargetValue (this->Vc2Target, tTime)
    : this->Vc2ServoErrorDiagOutput := UpdateOutput (this->Vc2ServoErrorDiagOutput, 
         InputValue (this->Vc2Input, tTime) + InputValue (Diag, tTime) - TargetValue (this->Vc2Target, tTime), tTime)
    : return := InputValue (this->Vc2Input, tTime) + InputValue (this->Vc2DiagInput, tTime) - 
      TargetValue (this->Vc2Target, tTime)
]]




[[D tGsuDlVcController::Actuate (TIME tTime, double dVc1Out, double dVc2Out) ]
Actuates VC1 to specified position and VC2 to specified acceleration.
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1, this->Vc2) ||
  invalid_actruator (this->Vc1, this->Vc2) => UNDEFINED
| true => 
    this->dVc1Output := dVc1Out
    : this->dVc2Output := dVc2Out
    : this->Vc1 := UpdateOutput (this->Vc1, dVc1Output, tTime)
    : voice coil associated with this->Vc1 actuated to dVc1Output
    : this->Vc2 := UpdateOutput (this->Vc2, dVc2Output, tTime)
    : voice coil associated with this->Vc2 actuated to dVc2Output
    : return := OK
]]


****************tGsuDlVcStabilizeController Specification*****************************

[[D tGsuDlVcStabilizeController::tGsuDlVcStabilizeController (
        tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, 
        const tGsuDlVcParameters& InitialParameters) ]
Construct this and initializes references to IoSet, ControllerData, and initialize
controller parameters.
[ invalid_pointer (this) ||
  invalid_reference (IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Vc1Input := reference to IoSet.Eddy1
    : this->Vc1DiagInput := reference to IoSet.Eddy1DiagInput
    : this->Vc2Input := reference to IoSet.Eddy2
    : this->Vc2DiagInput := reference to IoSet.Eddy2DiagInput
    
    : this->Vc1 := reference to IoSet.Vc1
    : this->Vc1ServoErrorDiagOutput := reference to IoSet.Vc1ServoErrorDiagOutput
    : this->Vc2 := reference to IoSet.Vc2
    : this->Vc2ServoErrorDiagOutput := reference to IoSet.Vc2ServoErrorDiagOutput

    : this->Vc1Target := reference to IoSet.Eddy1Target
    : this->Vc2Target : reference to IoSet.Eddy2Target
    
    : this->dVc1Error := reference to ControllerData.dVc1Error
    : this->dVc2Error := reference to ControllerData.dVc2Error
    
    : this->dVc1Output := reference to ControllerData.dVc1Output
    : this->dVc2Output := reference to ControllerData.dVc2Output

    : this->bVc1Enabled := InitialParameters.bVc1Enabled
    : this->bVc2Enabled := InitialParameters.bVc2Enabled

    : this->Vc1Pid := tPidDouble with coefiecients specified in InitialParameters
    : this->Vc2Pid := tPidDouble with coefiecients specified in InitialParameters
    : this->Vc2Feedforward := tFeedforward with coefiecients specified in InitialParameters

    : this->CdMotorAcceleration := const referenct to ControllerData.dMotorAcceleration
]]              


[[D tGsuDlVcStabilizeController::SetControlParameters (
      const tGsuDlVcStabilizeParameters& NewParameters) ]
Update controller parameters to specified values in NewParameters.    
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true =>
    this->bVc1Enabled := NewParameters.bVc1Enabled
    : this->bVc2Enabled := NewParameters.bVc2Enabled
    : this->Vc1Pid := tPidDouble with coefiecients specified in NewParameters
    : this->Vc2Pid := tPidDouble with coefiecients specified in NewParameters
    : this->Vc2Feedforward := tFeedforward with coefiecients specified in NewParameters
    : return := OK
]]              
         

[[D tGsuDlVcStabilizeController::Init (TIME tServoTime) ]
Resets PIDs to current servo errors
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, 
     this->Vc2Input, this->Vc2DiagInput, this->Vc2Target) => UNDEFINED
| true => 
    this->Vc1Pid := reset tPidDouble with InputValue (this->Vc1Input, tTime) + InputValue (this->Vc1DiagInput, tTime) - 
      TargetValue (this->Vc1Target, tTime) as reset value
    : this->Vc2Pid := reset tPidDouble with InputValue (this->Vc2Input, tTime) + InputValue (this->Vc2DiagInput, tTime) - 
      TargetValue (this->Vc2Target, tTime) as reset value
    : return := OK
]]

[[D tGsuDlVcStabilizeController::Reset (TIME tServoTime) ] = tGsuDlVcController::Reset ]]

[[D tGsuDlVcStabilizeController::Calculate (TIME tServoTime) ] 
Controls voice coils using PIDs & feedfowards.
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1Input, this->Vc1DiagInput, this->Vc2DiagInput,
    this->Vc2DiagInput, this->Vc1, this->Vc1ServoErrorDiagOutput, 
    this->Vc2, this->Vc2ServoErrorDiagOutput, this->Vc1Target, this->Vc2Target) ||
  invalid_sensor (this->Vc1Input, this->Vc1DiagInput, this->Vc2DiagInput,
    this->Vc2DiagInput) ||
  invalid_actuator (this->Vc1, this->Vc1ServoErrorDiagOutput,
                    this->Vc2, this->Vc1ServoErrorDiagOutput) ||
  invalid_target (this->Vc1Target, this->Vc2Target) => UNDEFINED
| this->bVc1Enabled == FALSE && this->bVc2Enabled == FALSE => 
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->dVc1Output := 0.0
    : this->Vc1 := UpdateOutput (this->Vc1, 0.0, tTime) 
    : voice coil associated with this->Vc1 actuated to 0.0

    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2 := UpdateOutput (this->Vc2, 0.0, tTime) 
    : voice coil associated with this->Vc2 actuated to 0.0
    : return := OK
| this->bVc2Enabled == FALSE =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->Vc1Pid := UpdatePid (this->Vc1Pid, 
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
    : this->dVc1Output := PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ) 
    : this->Vc1 := UpdateOutput (this->Vc1, PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ), tTime) 
    : voice coil associated with this->Vc1 actuated to PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
 
    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2 := UpdateOutput (this->Vc2, 0.0, tTime) 
    : voice coil associated with this->Vc2 actuated to 0.0
    : return := OK
| this->bVc1Enabled == FALSE =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->dVc1Output := 0.0 
    : voice coil associated with this->Vc1 actuated 0.0

    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2Pid := UpdatePid (this->Vc2Pid, 
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) )
    : this->dVc2Output := PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)  + 
        FEEDFORWARD (this->Vc1Feedforward, this->dVc2FeedforwardValue) )
    : this->Vc2 := UpdateOutput (this->Vc2, PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) , tTime) 
    : voice coil associated with this->Vc2 actuated to PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
       FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue)
    : return := OK
| true =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->Vc1Pid := UpdatePid (this->Vc1Pid, 
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
    : this->dVc1Output := PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ) 
    : this->Vc1 := UpdateOutput (this->Vc1, PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ), tTime) 
    : voice coil associated with this->Vc1 actuated to PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )

    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2Pid := UpdatePid (this->Vc2Pid, 
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) )
    : this->dVc2Output := PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc1Feedforward, this->dVc2FeedforwardValue) 
    : this->Vc2 := UpdateOutput (this->Vc2, PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)  + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) , tTime) 
    : voice coil associated with this->Vc2 actuated to PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
       FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue)
    : return := OK
]]   

[[D tGsuDlVcStabilizeController::Vc1Enabled (void) ] = tGsuDlVcController::Vc1Enabled ]]

[[D tGsuDlVcStabilizeController::Vc2Enabled (void) ] = tGsuDlVcController::Vc2Enabled ]]

********************** Protected Specification ********************************

[[D tGsuDlVcStabilizeController::Vc1ServoError (tInputSource& Input, tInputSource& Diag, 
    tTargetSource& Target, TIME tTime) ] = tGsuDlVcController::Vc1ServoError ]]
    
[[D tGsuDlVcStabilizeController::Vc2ServoError (tInputSource& Input, tInputSource& Diag, 
    tTargetSource& Target, TIME tTime) ] = tGsuDlVcController::Vc2ServoError ]]
    

[[D tGsuDlVcStabilizeController::Actuate (double dVc1Position, double dVc2Acceleration) ] =
    tGsuDlVcController::Actuate ]]
    
****************tGsuDlVcTrackController Specification*****************************

[[D tGsuDlVcTrackController::tGsuDlVcTrackController (
        tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, 
        const tGsuDlVcParameters& InitialParameters) ]
Construct this and initializes references to IoSet, ControllerData, and initialize
controller parameters.
[ invalid_pointer (this) ||
  invalid_reference (IoSet, InitialParameters) => UNDEFINED
| true =>
    this->Vc1Input := reference to IoSet.PztPosition
    : this->Vc1DiagInput := reference to IoSet.PztPositionDiagInput
    : this->Vc2Input := reference to IoSet.Eddy1
    : this->Vc2DiagInput := reference to IoSet.Eddy1DiagInput
    
    : this->Vc1 := reference to IoSet.Vc1
    : this->Vc1ServoErrorDiagOutput := reference to IoSet.Vc1ServoErrorDiagOutput
    : this->Vc2 := reference to IoSet.Vc2
    : this->Vc2ServoErrorDiagOutput := reference to IoSet.Vc2ServoErrorDiagOutput

    : this->Vc1Target := reference to IoSet.PztPositionTarget
    : this->Vc2Target : reference to IoSet.Eddy1Target

    
    : this->dVc1Error := reference to ControllerData.dVc1Error
    : this->dVc2Error := reference to ControllerData.dVc2Error
    
    : this->dVc1Output := reference to ControllerData.dVc1Output
    : this->dVc2Output := reference to ControllerData.dVc2Output

    : this->bVc1Enabled := InitialParameters.bVc1Enabled
    : this->bVc2Enabled := InitialParameters.bVc2Enabled

    : this->Vc1Pid := tPidDouble with coefiecients specified in InitialParameters
    : this->Vc2Pid := tPidDouble with coefiecients specified in InitialParameters
    : this->Vc1Feedforward := tFeedforward with coefiecients specified in InitialParameters
    : this->Vc2Feedforward := tFeedforward with coefiecients specified in InitialParameters
    : this->Vc1FeedforwardFilter := tFilter with coefiecients specified in InitialParameters

    : this->CdLaserError := const reference to ControllerData.dServoError
]]              


[[D tGsuDlVcTrackController::SetControlParameters (
      const tGsuDlVcTrackParameters& NewParameters) ]
Update controller parameters to specified values in NewParameters.    
[ invalid_pointer (this) ||
  invalid_reference (NewParameters) => UNDEFINED
| true =>
    this->bVc1Enabled := NewParameters.bVc1Enabled
    : this->bVc2Enabled := NewParameters.bVc2Enabled
    : this->Vc1Pid := tPidDouble with coefiecients specified in NewParameters
    : this->Vc2Pid := tPidDouble with coefiecients specified in NewParameters
    : this->Vc1Feedforward := tFeedforward with coefiecients specified in NewParameters
    : this->Vc2Feedforward := tFeedforward with coefiecients specified in NewParameters
    : this->Vc1FeedforwardFilter := tFilter with coefiecients specified in NewParameters
    : return := OK
]]              
         
[[D tGsuDlVcTrackController::Init (TIME tServoTime) ]
Resets PIDs to current servo errors
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, 
     this->Vc2Input, this->Vc2DiagInput, this->Vc2Target) => UNDEFINED
| true => 
    this->Vc1Pid := reset tPidDouble with InputValue (this->Vc1Input, tTime) + InputValue (this->Vc1iagInput, tTime) - 
      TargetValue (this->Vc1Target, tTime) as reset value
    : this->Vc1FeedforwardFilter := reset tFilter with FEEDFORWARD (this->Vc1FeedForward, this->dLaserError) as reset value
    : this->Vc2Pid := reset tPidDouble with InputValue (this->Vc2Input, tTime) + InputValue (this->Vc2DiagInput, tTime) - 
      TargetValue (this->Vc2Target, tTime) as reset value
    : return := OK
]]

[[D tGsuDlVcTrackController::Reset (TIME tServoTime) ] = tGsuDlController::Reset ]]

[[D tGsuDlVcTrackController::Calculate (TIME tServoTime) ] 
Controls voice coils using PIDs & filtered feedfowards.
[ invalid_pointer (this) ||
  invalid_reference (this->Vc1Input, this->Vc1DiagInput, this->Vc2DiagInput,
    this->Vc2DiagInput, this->Vc1, this->Vc1ServoErrorDiagOutput, 
    this->Vc2, this->Vc2ServoErrorDiagOutput, this->Vc1Target, this->Vc2Target) ||
  invalid_sensor (this->Vc1Input, this->Vc1DiagInput, this->Vc2DiagInput,
    this->Vc2DiagInput) ||
  invalid_actuator (this->Vc1, this->Vc1ServoErrorDiagOutput,
                    this->Vc2, this->Vc1ServoErrorDiagOutput) ||
  invalid_target (this->Vc1Target, this->Vc2Target) => UNDEFINED
| this->bVc1Enabled == FALSE && this->bVc2Enabled == FALSE => 
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->Vc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->dVc1Output := 0.0
    : this->Vc1 := UpdateOutput (this->Vc1, 0.0, tTime) 
    : voice coil associated with this->Vc1 actuated to 0.0

    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->Vc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2 := UpdateOutput (this->Vc2, 0.0, tTime) 
    : voice coil associated with this->Vc2 actuated to 0.0
    : return := OK
| this->bVc2Enabled == FALSE =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->Vc1Pid := UpdatePid (this->Vc1Pid, 
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
    : this->Vc1FeedforwardFilter := UpdateFilter (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) )
    : this->dVc1Output := PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range
    : this->Vc1 := UpdateOutput (this->Vc1, PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range, tTime) 
    : voice coil associated with this->Vc1 actuated to PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range
 
    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2 := UpdateOutput (this->Vc2, 0.0, tTime) 
    : voice coil associated with this->Vc2 actuated to 0.0
    : return := OK
| this->bVc1Enabled == FALSE =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->dVc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->Vc1Pid := UpdatePid (this->Vc1Pid, 
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
    : this->Vc1FeedforwardFilter := UpdateFilter (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) )
    : this->dVc1Output := PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range
    : this->Vc1 := UpdateOutput (this->Vc1, PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range, tTime) 
    : voice coil associated with this->Vc1 actuated to PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)  + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ) ) limited to half VC2 range
 
    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2Pid := UpdatePid (this->Vc2Pid, 
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) )
    : this->dVc2Output := PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc1Feedforward, this->dVc2FeedforwardValue) 
    : this->Vc2 := UpdateOutput (this->Vc2, PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) , tTime) 
    : voice coil associated with this->Vc2 actuated to PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) 
    : return := OK
| true =>
    this->Vc1Input := UpdateInput (this->Vc1Input, tServoTime)
    : this->Vc1DiagInput := UpdateInput (this->Vc1DiagInput, tServoTime)
    : this->Vc1Target := UpdateTarget (this->Vc1Target, tServoTime)
    : this->Vc1Error := VcError (this->Vc1Input, this->Vc1DiagInput, 
      this->Vc1Target, tServoTime)
    : device associated with this->Vc1ServoErrorDiagOutput actuated to
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime)
    : this->Vc1Pid := UpdatePid (this->Vc1Pid, 
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) )
    : this->Vc1FeedforwardFilter := UpdateFilter (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) )
    : this->dVc1Output := PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ) + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) )
    : this->Vc1 := UpdateOutput (this->Vc1, PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ) + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) ), tTime) 
    : voice coil associated with this->Vc1 actuated to PID (this->Vc1Pid,
      VcError (this->Vc1Input, this->Vc1DiagInput, this->Vc1Target, tServoTime) ) + 
      LOWPASS (this->Vc1FeedforwardFilter,
        FEEDFORWARD (this->Vc1Feedforward, this->dVc1FeedforwardValue) )

    : this->Vc2Input := UpdateInput (this->Vc2Input, tServoTime)
    : this->Vc2DiagInput := UpdateInput (this->Vc2DiagInput, tServoTime)
    : this->Vc2Target := UpdateTarget (this->Vc2Target, tServoTime)
    : this->dVc2Error := VcError (this->Vc2Input, this->Vc2DiagInput, 
      this->Vc2Target, tServoTime)
    : device associated with this->Vc2ServoErrorDiagOutput actuated to
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime)
    : this->Vc2Pid := UpdatePid (this->Vc2Pid, 
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) )
    : this->dVc2Output := PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc1Feedforward, this->dVc2FeedforwardValue) 
    : this->Vc2 := UpdateOutput (this->Vc2, PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) , tTime) 
    : voice coil associated with this->Vc2 actuated to PID (this->Vc2Pid,
      VcError (this->Vc2Input, this->Vc2DiagInput, this->Vc2Target, tServoTime) ) + 
        FEEDFORWARD (this->Vc2Feedforward, this->dVc2FeedforwardValue) 
    : return := OK
]]   

[[D tGsuDlVcTrackController::Vc1Enabled (void) ] = tGsuDlVcController::Vc1Enabled ]]

[[D tGsuDlVcTrackController::Vc2Enabled (void) ] = tGsuDlVcController::Vc2Enabled ]]


********************** Protected Specification ********************************

[[D tGsuDlVcTrackController::Vc1ServoError (tInputSource& Input, tInputSource& Diag, 
    tTargetSource& Target, TIME tTime) ] = tGsuDlVcController::Vc1ServoError ]]
    
[[D tGsuDlVcTrackController::Vc2ServoError (tInputSource& Input, tInputSource& Diag, 
    tTargetSource& Target, TIME tTime) ] = tGsuDlVcController::Vc2ServoError ]]
    
[[D tGsuDlVcTrackController::Actuate (TIME tTime, double dVc1Position, double dVc2Acceleration) ] =
    tGsuDlVcController::Actuate ]]

****************/


class tGsuDlVcController : public tController {
  public:
    tGsuDlVcController (tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, 
        tSensor& Vc1InputIn, tModulatedSensor& Vc1DiagInputIn, 
        tSensor& Vc2InputIn, tModulatedSensor& Vc2DiagInputIn, 
        tTargetSource& Vc1TargetIn, tTargetSource& Vc2TargetIn);
    BOOL Vc1Enabled () const { return this->bVc1Enabled;}
    BOOL& Vc1Enabled ()  { return this->bVc1Enabled;}
    BOOL Vc2Enabled () const { return this->bVc2Enabled;}
    BOOL& Vc2Enabled ()  { return this->bVc2Enabled;}
  protected:
    double Vc1ServoError (TIME tTime);
    double Vc2ServoError (TIME tTime);
    ERROR_CODE Actuate (TIME tTime, double dVc1Out, double dVc2Out);
    ERROR_CODE ResetVc1 ();
    ERROR_CODE ResetVc2 ();
    double Vc1UpperLimit () const { return this->Vc1.MaxValueInChannelUnits (); }
    double Vc1LowerLimit () const { return this->Vc1.MinValueInChannelUnits (); }
    double Vc2UpperLimit () const { return this->Vc2.MaxValueInChannelUnits (); }
    double Vc2LowerLimit () const { return this->Vc2.MinValueInChannelUnits (); }
    double Vc2CountsToVc1Counts (double dVc2Counts) const { return dVc2Counts * this->Vc1.PhysicalUnitsToCounts (1) / this->Vc2.PhysicalUnitsToCounts (1); }
    double Vc1CountsToVc2Counts (double dVc1Counts) const { return dVc1Counts * this->Vc2.PhysicalUnitsToCounts (1) / this->Vc1.PhysicalUnitsToCounts (1); }

    tSensor& Vc1Input;
    tModulatedSensor& Vc1DiagInput;
    tSensor& Vc2Input;
    tModulatedSensor& Vc2DiagInput;

    tActuator& Vc1;
    tActuator& Vc1ServoErrorDiagOutput;
    tActuator& Vc2;
    tActuator& Vc2ServoErrorDiagOutput;

  private: 
    tTargetSource& Vc1Target;
    tTargetSource& Vc2Target;
    
    double& dVc1Error;
    double& dVc2Error;

    double& dVc1Output;
    double& dVc2Output;

    BOOL bVc1Enabled;
    BOOL bVc2Enabled;
};


    
class tGsuDlVcStabilizeController : public tGsuDlVcController {
  public:
    tGsuDlVcStabilizeController (tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, const tGsuDlVcStabilizeParameters& InitialParameters);
    ERROR_CODE SetControlParameters (const tGsuDlVcStabilizeParameters& NewParameters);
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Calculate (TIME tServoTime);
        
  private:
    tPidDouble Vc1Pid;
    tPidDouble Vc2Pid;
    tFeedforward Vc2Feedforward;
    const double& CdMotorAcceleration;
    
};


class tGsuDlVcTrackController : public tGsuDlVcController {
  public:
    tGsuDlVcTrackController (tGsuDlControllerData& ControllerData,
        tGsuDlCompleteIoSet& IoSet, const tGsuDlVcTrackParameters& InitialParameters);
    ERROR_CODE SetControlParameters (const tGsuDlVcTrackParameters& NewParameters);
    ERROR_CODE Init (TIME tServoTime);
    ERROR_CODE Reset (TIME tServoTime);
    ERROR_CODE Calculate (TIME tServoTime);

  private:
    tSensor& Eddy2Input;
    
    tPidDouble Vc1Pid;
    tPidDouble Vc2Pid;

    tFeedforward Vc1Feedforward;
    tFeedforward Vc2Feedforward;
    
    tFilter Vc1FeedforwardFilter;

    const double& CdLaserError;
    
    const double CdVc1ControlUpperLimit;
    const double CdVc1ControlLowerLimit;
};

#endif INC_gsuDlVcControllerh

