/* $RCSfile: hardware.h,v $ - Hardware Initialization Header File*/
/*
$Header: /home/chara/cvsroot/ople/sys/include/hardware.h,v 1.1.1.1 1998/06/11 21:36:11 chara Exp $
*/

/*
modification history
--------------------
01b,pci,07May97      moved BOARD_NAME typedef to hwTypes.h
01a,pci,31Oct96      written
*/

#ifndef INC_hardwareh
#define INC_hardwareh

#include "hwTypes.h"
#include "board.h"
#include "hwError.h"
#include "inputChannel.h"
#include "outputChannel.h"

#ifndef INCR1
#error  "ONLY INCREMENT 1 IMPLEMENTATION EXISTS - Hardware values hard-coded"
#endif

/* OTHER HARDWARE DEFINES */
#define MIN_BUS_INT_PRIORITY              1
#define MAX_BUS_INT_PRIORITY              7
#define MIN_BUS_INT_VECTOR                0x80  // correct??
#define MAX_BUS_INT_VECTOR                0xff 
#define DT1401_DIGITAL_IO_MSB             15


/* VME_ADDRESS structure for accessing VME bus */
/*  INCREMENT 1 VERSION */
//typedef struct VME_ADDRESS {
//  int     modifier;
//  uint8_t *address;
//} VME_ADDRESS;

/**************************************************************************
*
* Forward Declarations
*/
class tChannel;
class tOutputChannel;
class tInputChannel;
class tWaveformChannel;
class tLaserMetrologyChannel;
class tReadbackChannel;


// BOARD_NAME moved to rteConfig.h, which will soon be moved to
//   configMgr.h --5/7/97, PCI
/* BOARD_NAME: used for board identification */
//typedef int BOARD_NAME;

#define NUM_CLOCK_BOARDS                  2

class tBoard;

/*****************************************************************
*
* tHardware class - class to hold pointers to hardware entities
*
*
[[D tHardware::tHardware(pBoardTypeArray, iArraySizeToUse)]
[ invalid_pointer (this) => UNDEFINED
| true => allocate memory for all tBoard pointers in system
]]

~tHardware - class destructor
[[D tHardware::~tHardware(void)]
[ invalid_pointer (this) => UNDEFINED
| true =>
    free memory for all tBoard pointers in system
    : *this := anything
]]
*
* 
HardInit - "Hard" initialize all boards; this should only be done once per
           execution of the system.  Cannot be used as a "Reinit" becuase
           it checks for prior initialization
[[D tHardware::HardInit(void)]
[ invalid_pointer(this) ||
  invalid_pointer(any board pointer) => UNDEFINED
| Any of the boards are already initialized => return := appropriate error
| true =>
  bInit for all boards := TRUE
  : initialize hardware pointers to all boards (and registers)
  : initialize registers of all boards
  : return OK
]]
*
*
SoftInit- "Soft" initialize all boards; this should be done once per Rte
           in the system.  Cannot be used as a "Reinit" becuase
           it checks for prior initialization
[[D tHardware::SoftInit(void)]
[ invalid_pointer (this) ||
  invalid_pointer (any board pointer) => UNDEFINED
| Any of the boards are already initialized => return appropriate error
| true => 
  bInit for all boards := TRUE
  : initialize hardware pointers to all boards (and registers)
  : return OK
]]
*
*
tBoard *tHardware::GetBoard(...)
[[D tHardware::GetBoard(BOARD_NAME boardName)]
[ invalid_pointer (this) => UNDEFINED
| boardName is not found in system board array => 
  return := NULL
  : send appropriate error message to console
| boardName is not a unique board identifier =>
  return := pointer to first board in array with name boardName
| true =>
    return := pointer to board with name boardName
]]
*
*
tChannel *tHardware::CreateChannel(...)
[[D tHardware::CreateChannel(BOARD_NAME boardName, int iChannelNum)]
[ invalid_pointer (this) ||
  boardName is not found in system board array => 
    return := NULL
    : send appropriate error message to console
| boardName is not a unique board identifier =>
    return := pointer to first board in array with name boardName
| true =>
    return := pointer to channel number iChannelNum that resides on
            board number boardName.
]]
*
*
[[D tHardware::GetShmemLocalAddress(void)]
[ invalid_pointer(this) || => UNDEFINED
| MM6216_BOARD_NAME is not found in system board array =>
  return := NULL
  : send appropriate error message to console
| true =>
  return := local pointer to beginning of system memory board
]]
*
*
[[D tHardware::GetShmemSize(void)]
[ invalid_pointer(this) || => UNDEFINED
| MM6216_BOARD_NAME is not found in system board array =>
  return := 0
  : send appropriate error message to console
| true =>
  return := total number of bytes on system memory board
]]
*
*
tBoard *tHardware::GetClockBoard(...)
[[D tHardware::GetClockBoard(int iRteNum)]
[ invalid_pointer (this) => UNDEFINED
| iRteNum<0 || iRteNum>= Number of RTEs in the system =>
  return := NULL
  : send appropriate error message to console
| true =>
  return := pointer to clock board for RTE #iRteNum
]]

*/
class tHardware {
  public:
    tHardware(const BOARD_TYPE *pBoardTypeArray, int iArraySizeToUse);
//    tHardware(const tHardwareInfo &hwInfo);
    ~tHardware(void);
    ERROR_CODE HardInit(void);
    ERROR_CODE SoftInit(void);
    tBoard   *GetBoard(BOARD_NAME boardName);
    tChannel *CreateChannel(BOARD_NAME boardName, int iChannelNum);
    tInterrupt *CreateInterrupt(BOARD_NAME boardName, int iIntNum);

//Get Shmem Routines
    volatile uint8_t  *GetShmemLocalAddress(void);
    unsigned int GetShmemSize(void);

//    tBoard *GetClockBoard(int iRteNum);
//    tBoard *GetVcBoard(void)    {return pPztMotorBoard;}
//    tBoard *GetLimitBoard(void) {return pDt1401Board;}
//    tBoard *GetLaserBoard(void) {return pFastLaserBoard;}
//    tBoard *GetPztBoard(void)   {return pPztMotorBoard;}
//    tBoard *GetMotorBoard(void) {return pDt1401Board;}
//    tBoard *GetDiagBoard(void)  {return pPztMotorBoard;}

  protected:
    BOOL   bSoftInit;
    tBoard **pBoardArray;
    int    iArraySize;

    tNullInputChannel  NullInputChannel;
    tNullOutputChannel NullOutputChannel;
};


#endif /* INC_hardwareh */

