/* $RCSfile: hwTypes.h,v $ - Public Hardware typedefs */

/*
$Header: /home/chara/cvsroot/ople/sys/include/hwTypes.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $
*/

/*
modification history
--------------------

01a,pci,28Apr97            written
*/

#ifndef INC_hwTypesh
#define INC_hwTypesh

#include "isit.h"

/* In increment 1, VME_ADDRESS was a struct with modifier and address,
*  now it is a class, tVmeAddress */
/**************************************************************************
*
* class tVmeAddress
*
* State Variables:
*  int iModifier
*  uint8_t *pu8Address
*
* METHODS:
[[D tVmeAddress::tVmeAddressParams(in: iModifierToUse, pu8AddressToUse;
                                inout: this;)]
[ true =>
  this->iModifier := iModifierToUse
  : this->pu8Address  := pu8AddressToUse
]] 
*
*
[[D tVmeAddress::tVmeAddressVoid(inout: this;)]
[ true =>
  this->iModifier := VME_SUP_STD_DATA
  : this->pu8Address := NULL
]]
*
*
[[D tVmeAddress::GetLocalAddress(in: this;)]
[ this is invalid => UNDEFINED
| this->pu8Address not reachable by CPU =>
    return := NULL
| true =>
  return := local address corresponding to this->iModifier,this->pu8Address
]] 
*
******/
class tVmeAddress {
public:
  tVmeAddress(int iModifierToUse, uint8_t *pu8AddressToUse)
              { this->iModifier=iModifierToUse;
                this->pu8Address=pu8AddressToUse; }
  tVmeAddress(void)
              { this->iModifier=0x0;
                this->pu8Address=NULL; }

  volatile uint8_t *GetLocalAddress(void);

  int     iModifier;
  uint8_t *pu8Address;
};


/***************************************
*
* A note on BOARD_NAME and BOARD_TYPE. 
*  Both of these types are assumed to have
*  integer-like properties later on in the
*  code.  As long as they can be treated
*  like an integer (or short, etc...), it will work.
****/
typedef int BOARD_NAME;

/*************************************** 
*  BOARD_TYPE: Used during initialization to
*              Determine what specific board to new.
******/
typedef enum BOARD_TYPE {
  BT_MPV955=0, BT_MPV906, BT_PZT_MOTOR, BT_FAST_LASER,
  BT_DT1401, BT_PC03V, BT_MM6216, BT_DVME626, BT_VIP640A,
  BT_IPQUAD, BT_IP16DAC, BT_TIM, BT_NUM_BOARD_TYPES
} BOARD_TYPE;

/* BOARD DEFINES */
/* NOTE: If there is more than one board of the same type, then the
*        BOARD_NAMEs for those boards will be numbered sequentially
*        (e.g. The first PC03V board will have a name 0x200,
*        and the second will have a name 0x201).
****/
#define NULL_BOARD_NAME                   0x0000
#define PC03V_BOARD_NAME                  0x0200
#define FAST_LASER_BOARD_NAME             0x0400
#define PZT_MOTOR_BOARD_NAME              0x0500
#define DVME626_BOARD_NAME                0x0600
#define MPV955_BOARD_NAME                 0x0700
#define MPV906_BOARD_NAME                 0x0800
#define MM6216_BOARD_NAME                 0x0900
#define VIP640A_BOARD_NAME                0x0a00
#define IPQUAD_BOARD_NAME                 0x0b00
#define IP16DAC_BOARD_NAME                0x0c00
#define TIM_BOARD_NAME                    0x0d00


/* BOARD_FUNC describes functionality of a board */
typedef int BOARD_FUNC;
#define BF_INTERRUPT                      0x0001
#define BF_CLOCK                          0x0002
#define BF_AD_CHAN                        0x0004
#define BF_DA_CHAN                        0x0008
#define BF_PZT_INTEGRATING_CHAN           0x0010
#define BF_PZT_NONINTEGRATING_CHAN        0x0020
#define BF_MOTOR_CHAN                     0x0040
#define BF_LASER_CHAN                     0x0080
#define BF_DIGITAL_IN_CHAN                0x0100
#define BF_DIGITAL_OUT_CHAN               0x0200

/* Special defines for requesting Null channels */
typedef enum NULL_CHANNEL_TYPE {
  NCT_NULL_INPUT_CHANNEL, NCT_NULL_OUTPUT_CHANNEL
} NULL_CHANNEL_TYPE;

/*************************************** 
*  CONV_CODE: Conversion code, used for various types of D/A and A/D
*             conversions.
*    CC_NO_CONV_CODE:      This channel has no conversion code.
*    CC_COMPLEMENT_BINARY: For Unipolar conversions (Ex.0V=0xFFFF, 10V=0x0)
*    CC_STRAIGHT_BINARY:   For Unipolar conversions (Ex.0V=0x0, 10V=0xFFFF)
*    CC_OFFSET_BINARY:     For Bipolar conversions (Ex.-10V=0x0, 10V=0xFFFF)
*    CC_TWOS_COMPLEMENT:   For Bipolar conversions (Ex.-10V=0x8000, 10V=0x7FFF)
******/
typedef enum CONV_CODE {
  CC_NO_CONV_CODE, CC_COMPLEMENT_BINARY, CC_STRAIGHT_BINARY, CC_OFFSET_BINARY,
  CC_TWOS_COMPLEMENT
} CONV_CODE;

#endif /* INC_hwTypes.h */
