/* $RCSfile*/

/*
$Header: /home/chara/cvsroot/ople/sys/include/image.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $
*/

#ifndef INC_imageh
#define INC_imageh

#include "buffer.h"

/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
*
* Image Header file for RICST.  This file defines the following classes:
*
* tImage : tBuffer
*
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

typedef unsigned char PIXEL;

/****************************************************************************

class tImage

DESCRIPTION:
   tImage holds images that are stored by the Image Channels.  tImage is an 
abstract data type that holds and allows manipulation of images.

BlackBox State
--------------
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
this->Image - the image

PUBLIC METHODS:
  tImage::tImage();
     Constructor
     
[[D tImage::tImage (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iHSize := 0
    : this->iVSize := 0
    : this->Image := NULL Image
]]

  tImage::tImage(tImage &Image);
     Copy Constructor

[[D tImage::tImage_t (tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_reference (Image) ||
  invalid Image.Image ||
  invalid this->Image => UNDEFINED
| true => 
    this->iHSize := Image.iHSize
    : this->iVSize := Image.iVSize
    : this->Image := Image.Image
]]

  tImage::tImage (int iHSize, int iVSize);
     Constructor for setting size of the image.

[[D tImage::tImage_ii (int iHSize, int iVSize) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iHSize := iHSize
    : this->iVSize := iVSize
    : this->Image := anything
]]

  tImage::~tImage();
     Destructor - Frees any memory used by tImage

[[D tImage::~tImage (void) ]
[ invalid_pointer (this) ||
  invalid this->Image => UNDEFINED
| true => 
    free image memory
    : *this := anything
]]

  PIXEL &tImage::GetPixel(int x, int y);
     Method to get access to image held by tImage, one pixel at a time.
    
[[D tImage::GetPixel(int x, int y) ]
[ invalid_pointer (this) ||
  invalid this->Image => UNDEFINED
| (x < 0) || (x >= this->iHSize) ||
  (y < 0) || (y >= this->iVSize) ||
  this->Image == NULL Image => return valid reference to a pixel of unknown value (BadPixel)
| const_pointer (this) => return const reference to this->Image[x][y]
| true => return := reference to this->Image[x][y]
]]

  ERROR_CODE tImage::GetDimensions (int &iHSize, int &iVSize);
     Method for getting size of the image.
     
[[D tImage::GetDimensions (int &iHSize, int &iVSize) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    iHSize := this->iHSize
    : iVSize := this->iVSize
    : return := OK
]]

  ERROR_CODE tImage::StoreImage(int iHSize, int iVSize, PIXEL *pImageArray);
     Creates a image used by tImage.  This makes a copy of the data 
     in pImageArray.  pImageArray is assumed to be an array of pointers 
     to rows of pixels of the image.

[[D tImage::StoreImage (int iHSize, int iVSize, PIXEL *pImageArray[]) ]
[ invalid_pointer (this) ||
  invalid_pointer (pImageArray) ||
  invalid_pointer (pImageArray[0..iVSize-1]) => UNDEFINED
| true => 
    this->iHSize := iHSize
    : this->iVSize := iVSize
    : this->Image := pImageArray
    : return := OK
]]

  ERROR_CODE tImage::ResizeImage (int iHSize, int iVSize);
     Creates an image of correct size.  The image that will 
     be held by tImage is unknown but storage will exist to
     hold an image.

[[D tImage::ResizeImage (int iHSize, int iVSize) ]
[ invalid_pointer (this) ||
  invalid this->Image => UNDEFINED
| true => 
    this->iHSize := iHSize
    : this->iVSize := iVSize
    : this->Image := Image Unknown Image
    : return := OK
]]

  ERROR_CODE tImage::Put (size_t szSize, const BYTE *pInBuffer);
     This method is main for receiving an image accross IPC.  This
     method converts the Image to a one dimensional array of bytes.  
     This method takes in an array of byte and puts it into the image.

[[D tImage::Put (size_t szSize, const BYTE *pInBuffer) ]
[ invalid_pointer (this) ||
  invalid_pointer (pInBuffer) ||
  invalid this->Image => UNDEFINED
| (szBufferSize != szSize) => return := appropriate error code
| true =>
    this->Image := *pInBuffer
    : return := OK
]]


  ERROR_CODE tImage::Get (size_t szSize, BYTE *pOutBuffer);
     This method is main for sending an image accross IPC.  This
     method converts the image to a one dimensional array of bytes. 
     This method takes a pointer to an array of bytes and fills it 
     with the image.

[[D tImage::Get (size_t szSize, BYTE *pOutBuffer) ]
[ invalid_pointer (this) ||
  invalid_pointer (pOutBuffer) ||
  invalid this->Image => UNDEFINED
| (szBufferSize != szSize) ||
  (this->Image == NULL Image) => return := appropriate error code
| true =>
    *pOutBuffer := this->Image
    : return := OK
]]

  BOOL IsSameSize (const tImage &Image) const;
     Method for coparing image sizes

[[D tImage::IsSameSize_t (tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_reference (Image) => UNDEFINED
| (this->iHSize != Image.iHSize) ||
  (this->iVSize != Image.iVSize) => return := FALSE
| true => return := TRUE   
]]
     
  BOOL IsSameSize (int iHSize, int iVSize) const;
     Method for coparing image sizes
  
[[D tImage::IsSameSize_ii (int iHSize, int iVSize) ]
[ invalid_pointer (this) => UNDEFINED
| (this->iHSize != iHSize) ||
  (this->iVSize != iVSize) => return := FALSE
| true => return := TRUE   
]]

  void tImage::operator= (tImage &Image);
     tImage assignment operator.

[[D tImage::operator_equals (tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_reference (Image) ||
  invalid Image.Image ||
  invalid this->Image => UNDEFINED
| true => 
    this->iHSize := Image.iHSize
    : this->iVSize := Image.iVSize
    : this->Image := Image.Image
]]

PROTECTED METHODS:
  void tImage::Copy(tImage &Image);
     This method accually does all copying of images.  All other methods
     that need to preform copy, call this method.  

[[D tImage::Copy (tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_reference (Image) ||
  invalid Image.Image ||
  invalid this->Image => UNDEFINED
| true => 
    this->iHSize := Image.iHSize
    : this->iVSize := Image.iVSize
    : this->Image := Image.Image
]]

  void tImage::Release();
     Method to accually release an image.  All other method that need to 
     release.

[[D tImage::Release (void) ]
[ invalid_pointer (this) ||
  invalid this->Image => UNDEFINED
| true => 
    free image memory
    : this->Image := NULL Image
]]

  ERROR_CODE tImage::EnsureBuffer();
     This method makes sure a buffer exist so that an Image can by stored
     
[[D tImage::EnsureBuffer (void) ]
[ invalid_pointer (this) ||
  invalid this->Image => UNDEFINED
| true => 
    this->Image := Image Unknown Image
    : return := OK
]]

****************************************************************************/
class tImage : public tBuffer {
public:
  tImage();
  tImage(tImage &Image);
  tImage(int iHSize, int iVSize);
  virtual ~tImage();
  PIXEL &GetPixel (int x, int y);
  const PIXEL& GetPixel (int x, int y) const;
  ERROR_CODE GetDimensions (int &iHSize, int &iVSize) const;
  ERROR_CODE StoreImage(int iHSize, int iVSize, const PIXEL *pImageArray[]);
  ERROR_CODE ResizeImage (int iHSize, int iVSize);
  ERROR_CODE Put (size_t szSize, const BYTE *pInBuffer);
  ERROR_CODE Get (size_t szSize, BYTE *pOutBuffer) const;
  BOOL IsSameSize (const tImage &Image) const;
  BOOL IsSameSize (int iHSize, int iVSize) const;
  void operator = (const tImage &Image);

  
protected:
  int iHSize;
  int iVSize;
  PIXEL *pImage;
  
  void Copy(const tImage &Image);
  void Release();
  ERROR_CODE EnsureBuffer();
  };

#endif
