/* $RCSfile: imageChannel.h,v $ - Header file for Image Channels */

/*
$Header: /home/chara/cvsroot/ople/sys/include/imageChannel.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $
*/

#ifndef INC_imageChannelh
#define INC_imageChannelh

#include "image.h"
#include "channel.h"

/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
*
* Image Channel Header file for RICST.  This file defines the following classes:
*
* tImageChannel : public tChannel
*   tFullImageChannel
*   tSubImageChannel
*
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

/****************************************************************************

class tImageChannel

DESCRIPTION:
  tImageChannel is a base class that provides methods for getting a image
  from the frame graber.  This class is the base class for tFullImageChannels
  and tSubImageChannel hiding wether the Image Channel being used is 
  the full image or just a small portion.
  
PRECONDITIONS:
  tImageChannel is a Pure Virtual class and can only be used as a base
  for other classes and as a pointer to any derived classes.

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image

METHODS:
  tImageChannel::tImageChannel();
  
[[D tImageChannel::tImageChannel(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := FALSE
    : this->iHSize := 0
    : this->iVSize := 0
]]

  virtual ERROR_CODE tImageChannel::GetImage(tImage &Image) = 0;
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  

[[D tImageChannel::GetImage(tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := image from frame buffer
    : return := OK
]]

  virtual ERROR_CODE tImageChannel::AtomicGetImage(tImage &Image) = 0;
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  The image is guaranteed to be from a single frame.

[[D tImageChannel::AtomicGetImage(tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true =>
    Image := an atomic image from frame buffer
    : return := OK
]]

  ERROR_CODE tImageChannel::GetDimensions(int &iHSize, int &iVSize);
     Method for getting size of the image.
     
[[D tImageChannel::GetDimensions(int &iHSize, int &iVSize) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true =>
    iHSize := this->iHSize
    : iVSize := this->iVSize
    : return := OK
]]

****************************************************************************/

class tImageChannel : public tChannel {
public:
  tImageChannel();
  virtual ERROR_CODE GetImage(tImage &Image) = 0;
  virtual ERROR_CODE AtomicGetImage(tImage &Image) = 0;
  ERROR_CODE GetDimensions(int &iHSize, int &iVSize);

protected:
  int iHSize;
  int iVSize;
  };

/****************************************************************************

tFullImageChannel

DESCRIPTION:

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image

  tFullImageChannel::tFullImageChannel();
  
[[D tFullImageChannel::tFullImageChannel(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := FALSE
    : this->iHSize := 0
    : this->iVSize := 0
]]

  ERROR_CODE tFullImageChannel::GetImage(tImage &Image);
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  

[[D tFullImageChannel::GetImage(tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := image from frame buffer
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::AtomicGetImage(tImage &Image) = 0;
     Methods to get a copy of the image.  It will return the most recent
     copy of the image.  The image is guaranteed to be from a single frame.

[[D tFullImageChannel::AtomicGetImage(tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := an atomic image from frame buffer
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::GetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image) = 0;
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.

[[D tFullImageChannel::GetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := frame from frame grabber in the region define by a box starting at
             (iStartX, iStartY) and of size (iHSize, iVSize)
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image) = 0;
     This method always accesses the frame buffer memory and returns a copy
     of the current buffer.  If the board has not yet grabbed the first frame
     or it has not yet been initialized, this method returns and error code 
     and image will contain an unknow image.  The image is guaranteed to be from 
     a single frame.

[[D tFullImageChannel::AtomicGetImageRegion(int iStartX, int iStartY, 
                                        int iHSize, int iVSize, tImage &Image) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  problem in getting image => return := appropriate error code
| true => 
    Image := an atomic frame from frame grabber  in the region define by a box 
             starting at (iStartX, iStartY) and of size (iHSize, iVSize)
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::StartCapture() = 0;
     This method initially puts the tFullImageChannel into a normal operating condition.
     This should only be called when images are going to be needed soon.  Depending on
     hardware, this method may need schedular to be initalize before this method is called.
     
[[D tFullImageChannel::StartCapture() ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true => 
    the card is put into normal operating condition.
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::Reset() = 0;
     This method puts restores the tFullImageChannel to a normal operating condition.
     
[[D tFullImageChannel::Reset() ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  capture not yet started => return := appropriate error code
| true => 
    the card is restored to normal operating condition.
    : return := OK
]]

  virtual ERROR_CODE tFullImageChannel::Update() = 0;
     This method starts and waits for an update to occure.

[[D tFullImageChannel::Update() ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  capture not yet started => return := appropriate error code
| true => 
    next frame is ready
    : return := OK
]]

****************************************************************************/
class tFullImageChannel : public tImageChannel {
public:
  ERROR_CODE GetImage(tImage &image);
  ERROR_CODE AtomicGetImage(tImage &image);
  virtual ERROR_CODE GetImageRegion(int iStartX, int iStartY, int iHSize, int iVSize, 
                                   tImage &Image) = 0;
  virtual ERROR_CODE AtomicGetImageRegion(int iStartX, int iStartY, int iHSize, int iVSize, 
                                   tImage &Image) = 0;
  virtual ERROR_CODE StartCapture() = 0;
  virtual ERROR_CODE Reset() = 0;
  virtual ERROR_CODE Update() = 0;
protected:
  };

/****************************************************************************
 
 tSubImageChannel

DESCRIPTION:

PRECONDITIONS:
  tSubImageChannel must have a valid, fully initialized tFullImageChannel with
  capture started before it can be used.  

BlackBox State
--------------
this->bInit - Variable to indicate if tImageChannel has been initalized
this->iStartX - Starting X position
this->iStartY - Starting Y position
this->iHSize - Horizontal Size of image
this->iVSize - Vertical Size of image
this->pficFullImage - pointer to the real image channel that this is a returns a part of

METHODS:
  tSubImageChannel::tSubImageChannel ();
     Base constructor.  Calls initialize (see initialize for more details).

[[D tSubImageChannel::tSubImageChannel (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := FALSE
    : this->pficFullImage := NULL
    : this->iHSize := 0
    : this->iVSize := 0
    : this->iStartX := 0
    : this->iStartY := 0
]]

  tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImage);
     Base constructor.  Calls initialize (see initialize for more details).

[[D tSubImageChannel::tSubImageChannel_t (tFullImageChannel *pficFullImage) ]
[ invalid_pointer (this) ||
  wild_pointer (pficFullImage) => UNDEFINED
| pficFullImage == NULL =>
    this->bInit := FALSE
    : this->pficFullImage := NULL
    : this->iHSize := 0
    : this->iVSize := 0
    : this->iStartX := 0
    : this->iStartY := 0
| true =>
    this->bInit := TRUE
    : this->pficFullImage := pficFullImage
    : this->iHSize := pficFullImage->iHSize
    : this->iVSize := pficFullImage->iVSize
    : this->iStartX := 0
    : this->iStartY := 0
]]

  tSubImageChannel::tSubImageChannel (tFullImageChannel *pficFullImage, 
                        int iStartX, int iStartY, int iHSize, int iVSize);
     Constructor.  Calls initialize (see initialize for more details).

[[D tSubImageChannel::tSubImageChannel_tiiii (tFullImageChannel *pficFullImage, 
                        int iStartX, int iStartY, int iHSize, int iVSize) ]
[ invalid_pointer (this) ||
  wild_pointer (pficFullImage) => UNDEFINED
| pficFullImage == NULL =>
    this->bInit := FALSE
    : this->pficFullImage := NULL
    : this->iHSize := 0
    : this->iVSize := 0
    : this->iStartX := 0
    : this->iStartY := 0
| (iStartX < 0) || (iStartX+iHSize >= pficFullImage->iHSize) ||
  (iStartY < 0) || (iStartY+iVSize >= pficFullImage->iVSize) =>
    this->bInit := TRUE
    : this->pficFullImage := pficFullImage
    : this->iHSize := pficFullImage->iHSize
    : this->iVSize := pficFullImage->iVSize
    : this->iStartX := 0
    : this->iStartY := 0
| true =>
    this->bInit := TRUE
    : this->pficFullImage := pficFullImage
    : this->iHSize := iHSize
    : this->iVSize := iVSize
    : this->iStartX := iStartX
    : this->iStartY := iStartY
]]

  ERROR_CODE tSubImageChannel::SetRegionOfInterest(int iStartX, int iStartY, 
                                                      int iHSize, int iVSize);
     This method sets the portion of the real image that it is to return.
     Returns an error code if points are out of range or if the virtual Image
     channel has not yet been initialized.  If points are out of range, the
     range of interest is unaffected.

[[D tSubImageChannel::SetRegionOfInterest(int iStartX, int iStartY, 
                                                      int iHSize, int iVSize) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pficFullImage) => UNDEFINED
| (this->bInit == FALSE) ||
  (iStartX < 0) || (iStartX+iHSize >= pficFullImage->iHSize) ||
  (iStartY < 0) || (iStartY+iVSize >= pficFullImage->iVSize) => return := appropriate error code
| true =>
    this->iHSize := iHSize
    : this->iVSize := iVSize
    : this->iStartX := iStartX
    : this->iStartY := iStartY
    : return := OK
]]

  ERROR_CODE tSubImageChannel::GetRegionOfInterest(int &iStartX, int &iStartY, 
                                                      int &iHSize, int &iVSize);
     This method returns the range of interest currently selected.  Returns error
     if the Virtual Image has not yet been initialized.

[[D tSubImageChannel::GetRegionOfInterest(int &iStartX, int &iStartY, 
                                                      int &iHSize, int &iVSize) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pficFullImage) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true =>
    iHSize := this->iHSize
    : iVSize := this->iVSize
    : iStartX := this->iStartX
    : iStartY := this->iStartY
    : return := OK
]]

  ERROR_CODE tSubImageChannel::GetImage(tImage &Image);
     Method to get a copy of the image.  It will return the most recent
     copy of the image.  
     
[[D tSubImageChannel::GetImage(tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pficFullImage) => UNDEFINED
| this->bInit == FALSE ||
  problem getting image => return := appropriate error code
| true => 
    Image := a frame from frame grabber
    : return := OK
]]

  ERROR_CODE tSubImageChannel::AtomicGetImage(tImage &Image);
     Method to get a copy of the image.  It will return the most recent
     copy of the image.  The image is guaranteed to be from a single frame.

[[D tSubImageChannel::AtomicGetImage(tImage &Image) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pficFullImage) => UNDEFINED
| this->bInit == FALSE ||
  problem getting image => return := appropriate error code
| true => 
    Image := an atomic frame from frame grabber
    : return := OK
]]

****************************************************************************/
class tSubImageChannel : public tImageChannel {
public:
  tSubImageChannel ();
  tSubImageChannel (tFullImageChannel *pficFullImage);
  tSubImageChannel (tFullImageChannel *pficFullImage, int iStartX, int iStartY, int iHSize, int iVSize);
  ERROR_CODE Initialize (tFullImageChannel *pficFullImage);
  ERROR_CODE Initialize (tFullImageChannel *pficFullImage, int iStartX, int iStartY, int iHSize, int iVSize);
  ERROR_CODE SetRegionOfInterest(int iStartX, int iStartY, int iHSize, int iVSize);
  ERROR_CODE GetRegionOfInterest(int &iStartX, int &iStartY, int &iHSize, int &iVSize) const;
  ERROR_CODE GetImage(tImage &Image);
  ERROR_CODE AtomicGetImage(tImage &Image);
  
protected:
  tFullImageChannel *pficFullImage;
  int iStartX, iStartY;
  };

#endif /* INC_imageChannelh */
