/* $RCSfile: imageStatus.h,v $ */
/* CM Info:   
$Header $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a27Aug97,rlj    written
*/

#ifndef INC_imageStatush
#define INC_imageStatush

#include "isit.h"
#include "image.h"
#include "typeTag.h"


#define S_imagestatus_INCORRECT_IMAGE_SIZE (M_imagestatus | 0x0001)

/****
[[D tImageStatus::tImageStatus (const tTypeTag& ImageTagToUse, 
    int iHSize, int iVSize) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->CImageTage := ImageTagToUse
    : this->Image.iHSize:= iHSize
    : this->Image.iVSize:= iVSize
    : this->Image.Image := CheckerBoard pattern with 4 squares 
    : this->DefaultImage->iHSize:= iHSize
    : this->DefaultImage->iVSize:= iVSize
    : this->DefaultImage->Image:= CheckerBoard pattern with 4 squares 
]]

[[D tImageStatus::~tImageStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tImageStatus::SetDefaultImage (const tImage& Image)] 
[ invalid_pointer (this) ||
  invalid_reference (Image) => UNDEFINED
| (this->DefaultImage.iHSize != Image.iHSize) ||
  (this->DefaultImage.iVSize != Image.iVSize) => 
    return := appropriate error code
| true => 
    this->DefaultImage := Image
    : return := OK
]]

[[D tImageStatus::Update_t (const tImage& ImageToUse)] 
[ invalid_pointer (this) ||
  invalid_reference (ImageToUse) => UNDEFINED
| (this->Image.iHSize != ImageToUse.iHSize) ||
  (this->Image.iVSize != ImageToUse.iVSize) => 
    return := appropriate error code
| true => 
    this->Image := ImageToUse
    : return := OK
]]

[[D tImageStatus::Update (void)] 
[ invalid_pointer (this) => UNDEFINED
| true => this->Image := DefaultImage
]]

[[D tImageStatus::GetImage (tImage& OutImage)] 
[ invalid_pointer (this) ||
  invalid_reference (OutImage) => UNDEFINED
| true => OutImage := this->Image
]]

[[D tImageStatus::SendToStatusStream (void) ]
[ invalid_pointer (this) ||
  ::StatusStream.bInit == FALSE => UNDEFINED 
| true =>
    ::StatusStream := StatusStream + this->CImageTag +
      this->Image.Image
    : return := OK
]]
  
[[D tImageStatus::GetSize (void)] 
[ invalid_pointer (this) => UNDEFINED
| true => return := this->Image->szBufferSize 
]]

****/

class tImageStatus {
  public:
    tImageStatus (const tTypeTag& ImageTagToUse, int iHSize, int iVSize);
    ~tImageStatus ();
    ERROR_CODE SetDefaultImage (const tImage& Image);
    ERROR_CODE Update (tImage& Image);
    void Update ();
    void GetImage (tImage& Image);
    ERROR_CODE SendToStatusStream ();
    size_t GetSize ();
  private:
    const tTypeTag CImageTag;
    SEM_ID siImageMutexSemaphore;
    tImage Image;
    tImage DefaultImage;
};





#endif INC_imageStatush
