/* $RCSfile: inputChannel.h,v $ - tInputChannel and decendents */
/*
$Header: /home/chara/cvsroot/ople/sys/include/inputChannel.h,v 1.2 1998/08/17 21:01:14 phil Exp $
*/

/*
modification history
--------------------
01a,7Nov97,pci           written, moved from channel.h
*/

#ifndef INC_inputChannelh
#define INC_inputChannelh

#include "hwTypes.h"
#include "channel.h"
//#include <math.h>

/*******************************************************************************
*
* class tInputChannel
*
* METHODS
* pure virtual ERROR_CODE tInputChannel::Input(int* piValue)
* Description:    Sets *piValue to corresponding converted voltage value.
[[D tInputChannel::Input(int* piValue)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
| true =>
  *piValue := integer value corresponding to physical value at input
]]
*
*
* pure virtual ERROR_CODE tInputChannel::Input_double(int* pdValue)
* Description:    Sets *pdValue to corresponding converted voltage value.
[[D tInputChannel::Input_double(double* pdValue)]
[ this is invalid || pdValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *pdValue := anything
| true =>
  *pdValue := value corresponding to physical value at input
]]
*
*
* int tInputChannel::GetInputValue(int iValueToConvert)
* Description:  Converts iValueToConvert into the appropriate value for the type
*              of input. This method should be used as follows:  Read the board
*              and call this method with the board input value as the argument.
*              As long as this->ccConvCode matches the conversion code of the board,
*              the return value will be a valid signed int representation of the
*              value at the input of the card.  This method needs to be as quick
*              as possible since it will most likey be called in high-speed servo
*              loops.
[[D tInputChannel::GetInputValue(int iValueToConvert)]
[ this is invalid || this->iBitWidth >= sizeof(int)*8 ||
  this->ccConvCode does not match conversion code of the board => UNDEFINED
| this->ccConvCode is not valid =>
  appropriate error message to console
  : return := iValueToConvert
| true =>
  return := signed int value corresponding to iValueToConvert
]]

[[D tInputChannel::Reset (void) ]
Resets channel as appropriate
[ invalid_pointer (this) => UNDEFINED
| this is not a resettable channel => return := appropriate error code
| true => channel reset appropriately
]]

*
*
********************************************************************************
*
* class tAdChannel
*
* State Variables:
*  CONV_CODE ccInputCode
*  int       iInputGain
*  double    dActualChannelValue
*
* SPEC FUNCTIONS:
* double AdResolution(tAdChannel adChannel)
[ adChannel.bInit != TRUE => UNDEFINED
| true =>
  return := ((adChannel.iInputRange/adChannel.iInputGain) /
          (double) (2^adChannel.iBitWidth))
]
*
*
* METHODS:
* pure virtual ERROR_CODE tAdChannel::Input(int* piValue)
* Description:    Sets *piValue to corresponding converted voltage value.
[[D tAdChannel::Input(int* piValue)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Related board is not polling =>
  *piValue := anything
  : return := appropriate error
| *this->ccInputCode == CC_COMPLEMENT_BINARY =>
  *piValue := 2^this->iBitWidth - 1 - ((int) this->dActualChannelValue /
                                             AdResolution(this))
  : return := OK
| *this->ccInputCode == CC_STRAIGHT_BINARY ||
  *this->ccInputCode == CC_TWOS_COMPLEMENT =>
   *piValue := ((int) this->dActualChannelValue / AdResolution(*this))
   : return := OK
| *this->ccInputCode == CC_OFFSET_BINARY =>
  *piValue :=  ((int) this->dActualChannelValue / AdResolution(*this) -
                      2^this->iBitWidth/2)
  : return := OK
| true =>
  return := S_tAdChannel_INVALID_INPUT_CODE
]]
*
*
********************************************************************************
*
* class tReadbackChannel
*
* METHODS:
* pure virtual ERROR_CODE tReadbackChannel::Input(int* piValue)
* Description:    Sets *piValue to corresponding converted voltage value.
[[D tReadbackChannel::Input(int* piValue)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
| true =>
  *piValue := integer value corresponding to physical value at input
]]
*
*
********************************************************************************
*
* class tDigitalInChannel
*
* METHODS:
* pure virtual ERROR_CODE tDigitalInChannel::Input(int* piValue)
* Description:    Sets *piValue to corresponding converted digital input value.
[[D tDigitalInChannel::Input(int* piValue)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
| true =>
  *piValue := integer value corresponding to physical value at input
]]
*
*
********************************************************************************
*
* class tLaserMetrologyChannel
*
* State Variables:
* @@ BOOL   bInit
* @@ int    iBitWidth
* LASER_CHANNEL   lcChannel
*
* METHODS:
* tLaserMetrologyChannel::tLaserMetrologyChannel(void)
* Description:    Constructor
[[D tLaserMetrologyChannel::tLaserMetrologyChannel(inout: this;) ]
[ Insufficient memory available for construction =>
   this is invalid
| true =>
  this is valid
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::ReadCounter(double* pdValue)
* Description:    Sets *pdValue to corresponding converted voltage value.
[[D tLaserMetrologyChannel::ReadCounter(double* pdValue)]
[ this is invalid || pdValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *pdValue := anything
| true =>
  *pdValue := value corresponding to entire value of this->lcChannel
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::ReadFiducial(double* pdValue)
* Description:    Sets *pdValue to corresponding converted voltage value.
[[D tLaserMetrologyChannel::ReadFiducial(double* pdValue)]
[ this is invalid || pdValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *pdValue := anything
| true =>
  *pdValue := value corresponding to fiducial part of this->lcChannel
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::SetSyncModeR(smNewSyncMode)
* Description:    Sets the syncronization mode of this->lcChannel.
* In:     this, this->bInit
[[D tLaserMetrologyChannel::SetSyncModeR(int smNewSyncMode) ]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->lcChannel.smLaserSyncModeR := smNewSyncMode
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::ClearCounter(void)
* Description:    Clears counter to the best of the board's ability immediately.
*                 (on a fast laser board, the fraction part is never cleared)
[[D tLaserMetrologyChannel::ClearCounter(inout: this;) ]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->lcChannel.dWholeCounterR := ~0
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::ClearCounterOnFiducial(void)
* Description:    Sets up board so that counter will be cleared to the best of
*                 the board's ability (on a fast laser board, the fraction part
*                 is never cleared) next time the fiducial (home) sensor is crossed.
[[D tLaserMetrologyChannel::ClearCounterOnFiducial(inout: this;) ]
[ this is invalid || this->bInit == FALSE =>
  UNDEFINED
| this->lcChannel.bOnFiducialR is never TRUE  =>
  this->lcChannel.smLaserSyncModeR := SYNC_TO_FIDUCIAL
  : this->lcChannel.bWaitingOnFiducialR := TRUE
  : return := OK
| true =>
  this->lcChannel.dWholeFiducialR := this->lcChannel.dWholeCounterR
  : this->lcChannel.dWholeCounterR := ~0 (when home sensor is crossed)
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::LoadOnFiducial(void)
* Description:    Sets up board so that counter will be cleared to the best of
*                 the board's ability (on a fast laser board, the fraction part
*                 is never cleared) next time the fiducial (home) sensor is
*                 crossed.
[[D tLaserMetrologyChannel::LoadOnFiducial(inout: this;) ]
[ this is invalid || this->bInit == FALSE =>
  UNDEFINED
  | this->lcChannel.bOnFiducialR is never TRUE after =>
  this->lcChannel.smLaserSyncModeR := SYNC_TO_FIDUCIAL
  : this->lcChannel.bWaitingOnFiducialR := TRUE
  : return := OK
| true =>
  this->lcChannel.dWholeFiducialR := this->lcChannel.dWholeCounterR
  : return := OK
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::ResetFiducialWait(void)
* Description:    Resets value of fiducial wait state so this channel is no
*                 longer waiting on the fiducial to be set.  In essence, this
*                 method cancels the effect of any previous call to
*                 LoadOnFiducial or ClearCounterOnFiducial (as long as fiducial
*                 hasn't been crossed since that call).
[[D tLaserMetrologyChannel::ResetFiducialWait(void) ]
[ this is invalid || this->bInit == FALSE =>
  UNDEFINED
| true =>
  this->lcChannel.bWaitingOnFiducialR := FALSE
  : return := OK
]]
*
*
* pure virtual BOOL tLaserMetrologyChannel::FiducialWaitState(void)
* Description:    Checks to see whether the board is waiting on a fiducial
*                 crossing and returns appropriate BOOL.
*                 This method will return TRUE if a ClearCounterOnFiducial or
*                 LoadOnFiducial was called and the fiducial input has not yet
*                 gone active nor has a ResetFiducialWait call been issued.
[[D tLaserMetrologyChannel::FiducialWaitState(void) ]
[ this is invalid || this->bInit == FALSE =>
  UNDEFINED
| true =>
  return := this->lcChannel.bWaitingOnFiducialR
]]
*
*
* pure virtual ERROR_CODE tLaserMetrologyChannel::IsFiducialOn(void)
* Description:    Checks to see whether the fiducial input is active and returns
*                 appropriate BOOL.
[[D tLaserMetrologyChannel::IsFiducialOn(inout: this;) ]
[ this is invalid || this->bInit == FALSE =>
  UNDEFINED
| true =>
  return := this->lcChannel.bOnFiducialR
]]
*
*
********************************************************************************
*
* class tFiducialInputChannel
*
* METHODS:
* tFiducialInputChannel::Initialize(...)
* Description:  Initializes class members
[[D tFiducialInputChannel::Initialize(tLaserMetrologyChannel* pMetrologyChannelToUse)
]
[ this is invalid || MetrologyChannelToUse is invalid => UNDEFINED
| true =>
  this->MetrologyChannel := MetrologyChannelToUse
]]
*
*
* pure virtual ERROR_CODE tFiducialInputChannel::Input(int* piValue)
* Description:    Sets *piValue to corresponding converted voltage value.
[[D tDigitalInChannel::Input(int* piValue)]
[ this is invalid || piValue is invalid || this->bInit == FALSE => UNDEFINED
| Board is not receiving input properly =>
  *piValue := anything
  : return := OK
| Fiducial on associated metrology channel is set =>
  *piValue := (int) TRUE
  : return := OK
| true =>
  *piValue := (int) FALSE
  : return := OK
]]
*
*
********************************************************************************
*
* class tCounterChannel
*
* METHODS:
* virtual ERROR_CODE Input (int *piValue) const
* Description:  reads current value of counter
[[D tCounterChannel::Input(int* piValue)]
[ invalid_pointer(this) ||
  invalid_pointer(piValue) || 
  tBaseObject::bInit == FALSE ||
  host board is not initialized properly => UNDEFINED
| true =>
    *piValue := value
    : return := OK
]]
*
*
* virtual ERROR_CODE Reset(void) const
* Description:  resets counter to zero
[[D tCounterChannel::Input(void)]
[ invalid_pointer(this) ||
  tBaseObject::bInit == FALSE ||
  host board is not initialized properly => UNDEFINED
| true =>
    set counter value to 0
    : return := OK
]]
*
******/




/*******************************************************************************
********************************************************************************
*
* Class Definitions
*
********************************************************************************
********************************************************************************/

class tInputChannel : public tChannel {
public:
  virtual ERROR_CODE Input(int* piValue) const = 0;
  virtual ERROR_CODE Input(double* pdValue) const;
  virtual ERROR_CODE SetGain(int iNewGainToUse) { return ERROR; }
  virtual ERROR_CODE Reset(void) const
                       { return S_inputChannel_INVALID_CALL_TO_RESET; }

protected:
  int                GetInputValue (int iValueToConvert) const;
};


class tAdChannel : public tInputChannel {
public:
  virtual ERROR_CODE Input(int* piValue) const = 0;

protected:
  int       iInputGain;
};


class tReadbackChannel : public tInputChannel {
public:
  virtual ERROR_CODE Input(int* piValue) const = 0;
};


class tNullInputChannel : public tInputChannel {
public:
  tNullInputChannel(void) { dChannelRange    = 0.0;
                             iBitWidth       = 0;
                             ccConvCode      = CC_NO_CONV_CODE;
                             SetInit(TRUE);
                           }
  ERROR_CODE Input(int* piValue) const { *piValue = 0;   return OK; }
  ERROR_CODE Input(double* pdValue) const { *pdValue = 0.0; return OK; }
};


class tDigitalInChannel : public tInputChannel {
public:
  virtual ERROR_CODE Input(int* piValue) const = 0;
};


class tLaserMetrologyChannel : public tInputChannel {
public:
  virtual ERROR_CODE Input(int* piValue) const = 0;
  virtual ERROR_CODE Input(double* pdValue) const = 0;
//  virtual ERROR_CODE ReadCounter(int64_t* p64Value) const = 0;
//  virtual ERROR_CODE ReadFiducial(int64_t* p64Value) const = 0;
  virtual ERROR_CODE ClearCounter(void) = 0;
  virtual ERROR_CODE ClearCounterOnFiducial(void) = 0;
  virtual ERROR_CODE LoadOnFiducial(void) = 0;
  virtual ERROR_CODE ResetFiducialWait(void) = 0;
  virtual BOOL       FiducialWaitState(void) = 0;
  virtual BOOL       IsFiducialOn(void) = 0;
};


class tFiducialInputChannel : public tDigitalInChannel {
public:
  tFiducialInputChannel(void) { dChannelRange    = 2.0;
                                iBitWidth        = 1;
                                ccConvCode       = CC_NO_CONV_CODE;
                                SetInit(TRUE);
                              }
  ERROR_CODE Initialize(tLaserMetrologyChannel* pMetrologyChannelToUse)
               {
                 pMetrologyChannel = pMetrologyChannelToUse;
                 return OK;
               }
  ERROR_CODE Input(int* piValue) const
                       /****
                       [[U tLaserMetrologyChannel::IsFiducialOn()]]
                       ****/
                       { *piValue = (int) (pMetrologyChannel->IsFiducialOn());
                         return OK;
                       }

protected:
  tLaserMetrologyChannel* pMetrologyChannel;
};


class tCounterChannel : public tInputChannel {
public:
  virtual ERROR_CODE Input (int *piValue) const = 0;
  virtual ERROR_CODE Reset(void) const
                       { return S_counterChannel_INVALID_CALL_TO_RESET; }
};


#endif /* INC_inputChannelh */
