/* $RCSfile: inputSource.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/inputSource.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$log$
*/
 
/*
DESCRIPTION

*/

#ifndef INC_InputSourceh
#define INC_InputSourceh


#include "isit.h"
#include "modLib.h"
#include "dlDither.h"
#include "inputChannel.h"

/**************** Specification Functions *******************************

double InputValue (tInputSource& Input, TIME tTime) :==
Return updated value for an input source.  This details the behavior of all
input sources currently defined.
[ invalid_reference (Input) ||
  invalid_input (Input) => UNDEFINED
| Input is a tSensor => return := ChannelValue (Input.Channel) + Input.dOffset
| Input is a tModulationSource && Input.bEnabled == TRUE =>
    return := MODULATION (Input.ModulationParameters, tTime) + Input.dOffset
| Input is a tModulationSource && Input.bEnabled == FALSE => return := 0.0

| Input is a tModulatedSensor && Input.bEnabled == TRUE =>
    return := ChannelValue (Input.Channel) + 
      MODULATION (Input.ModulationParameters, tTime) + Input.dOffset
| Input is a tModulatedSensor && Input.bEnabled == FALSE =>
    return := ChannelValue (Input.Channel) + Input.dOffset

| Input is a tLaserMetrologySensor && Input.bEnabled == TRUE  &&
  Input.bDitherEnabled == TRUE =>
    return := ChannelValue (Input.Channel) + 
      MODULATION (Input.ModulationParameters, tTime) +
      DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime) +
      Input.dOffset 
| Input is a tLaserMetrologySensor && Input.bEnabled == TRUE  &&
  Input.bDitherEnabled == FALSE =>
    return := ChannelValue (Input.Channel) + 
      MODULATION (Input.ModulationParameters, tTime) +
      Input.dOffset 
| Input is a tLaserMetrologySensor && Input.bEnabled == FALSE  &&
  Input.bDitherEnabled == TRUE =>
    return := ChannelValue (Input.Channel) +
      DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime) +
      Input.dOffset 
| Input is a tLaserMetrologySensor && Input.bEnabled == FALSE  &&
  Input.bDitherEnabled == FALSE =>
    return := ChannelValue (Input.Channel) + 
      DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime) +
      Input.dOffset 
    return := ChannelValue (Input.Channel) + Input.dOffset 
| true => return appropriate value for the input source at time tTime
]


tInputSource& UpdateInput (tInputSource& Input, TIME tTime) :==
Return updated state of an input source object.  
This details the behavior of all input sources currently defined.
[ invalid_reference (Input) ||
  invalid_input (Input) => UNDEFINED

| Input is a tSensor => return := copy of Input with 
    dValue = ChannelValue (Input.Channel) + Input.dOffset
| Input is a tModulationSource && Input.bEnabled == TRUE =>
    return := copy of Input with 
      dValue = MODULATION (Input.ModulationParameters, tTime) + Input.dOffset
| Input is a tModulationSource && Input.bEnabled == FALSE => return := Input
| Input is a tModulatedSensor && Input.bEnabled == TRUE =>
    return :=copy of Input with
      dValue = ChannelValue (Input.Channel) + 
        MODULATION (Input.ModulationParameters, tTime) + Input.dOffset,
      dModulation = MODULATION (Input.ModulationParameters, tTime),
| Input is a tModulatedSensor && Input.bEnabled == FALSE =>
    return := copy of Input with
      dValue = ChannelValue (Input.Channel) + Input.dOffset,
      dModulation = 0.0,

| Input is a tLaserMetrologySensor && Input.bEnabled == TRUE  &&
  Input.bDitherEnabled == TRUE =>
    return := copy of Input with
      dValue = ChannelValue (Input.Channel) + 
        MODULATION (Input.ModulationParameters, tTime) +
        DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime) +
        Input.dOffset, 
      dModulationValue = MODULATION (Input.ModulationParameters, tTime),
      dDitherValue = DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime),
| Input is a tLaserMetrologySensor && Input.bEnabled == FALSE  &&
  Input.bDitherEnabled == TRUE =>
    return := copy of Input with
      dValue = ChannelValue (Input.Channel) +
        DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime) +
        Input.dOffset, 
      dModulationValue = 0.0,
      dDitherValue = DITHER (Input.dDitherAmplitude, Input.dDitherPeriod, tTime),
| Input is a tLaserMetrologySensor && Input.bEnabled == TRUE  &&
  Input.bDitherEnabled == FALSE =>
    return := copy of Input with
      dValue = ChannelValue (Input.Channel) + 
        MODULATION (Input.ModulationParameters, tTime) + Input.dOffset, 
      dModulationValue = MODULATION (Input.ModulationParameters, tTime),
      dDitherValue = 0.0
| Input is a tLaserMetrologySensor && Input.bEnabled == FALSE  &&
  Input.bDitherEnabled == FALSE =>
    return := copy of Input with
      dValue = ChannelValue (Input.Channel) + Input.dOffset, 
      dModulationValue = 0.0,
      dDitherValue = 0.0
| true => return appropriate value for the input source at time tTime
]

****************************tInputSource ********************************

[[D tInputSource::tInputSource (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   this->dValue := 0.0
   : this->dOffset := 0.0
]]


[[D tInputSource::tInputSource_d (double dOffsetIn) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
   this->dValue := 0.0
   : this->dOffset := dOffsetIn
]]


[[D tInputSource::~tInputSource (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tInputSource::Value (void) ]
Returns the value retrieved by the last Update compensated with the offset
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dValue - this->dOffset
]]



[[D tInputSource::Offset (void) ]
Returns the offset.
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dOffset
]]


[[D tInputSource::SetValue (double dNewValue) ]
Sets value of input source to dNewValue and return dNewValue
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := dNewValue
    : return := dNewValue
]]


[[D tInputSource::SetOffset (double dNewOffset) ]
Sets offset of input source to dNewOffset and return dNewOffset
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dOffset := dNewOffset
    : return := dNewOffset
]]

[[D tInputSource::Update (TIME tUpdateTime) ]
Pure virtual method to update a new value for this->dValue
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := anything
    : return := anything
]]


****************** Protected specification ***********************************

[[D tInputSource::RawValue (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dValue 
]]



*******************  tModulationSource Black Box  ******************
*
* A tModulationSource is a type of tInputSource that encapsulates
* a tModulation object.  Each time Update is called, a modulation value for 
* the time is generated.
*


[[D tModulationSource::tModulationSource (void) ]
Constructs modulation source with default modulation parameters.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->mpParams.mtModulationType := MT_NONE
    : this->mpParams.dFrequency := 0
    : this->mpParams.dAmplitude := 0
    : this->mpParams.dMeanVal := 0
    : this->mpParams.tSyncTime := 0
    : this->mpParams.dDutyCycle := 0
    : this->bEnabled := FALSE
]]


[[D tModulationSource::tModulationSource_t (
      const tModulationParams& InitialParameters) ]
Constructs modulation source with initial modulation parameters.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->mpParams := InitialParameters
    : this->bEnabled := FALSE
]]


[[D tModulationSource::tModulationSource_td (
      const tModulationParams& InitialParameters, double dOffsetIn) ]
Constructs modulation source with initial modulation parameters.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := dOffsetIn
    : this->mpParams := InitialParameters
    : this->bEnabled := 0
]]


[[D tModulationSource::~tModulationSource (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tModulationSource::Value (void) ] = tInputSource::Value ]]

[[D tModulationSource::SetValue (double dNewValue) ] = tInputSource::SetValue ]]

[[D tModulationSource::SetOffset (double dNewOffset) ] = tInputSource::SetOffset ]]


[[D tModulationSource::SetParameters (const tModulationParams& NewParameters) ]
Set this->mpParams with specified parameters.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->mpParams := NewParameters
    : return := OK
]] 


[[D tModulationSource::GetParameters (void) ] 
[ this invalid => UNDEFINED
| true =>
    return := this->mpParams
]]


[[D tModulationSource::Enable (void) ]
Turn on modulation
[ invalid_pointer (this) => UNDEFINED
| true => this->bEnabled := TRUE 
]]


[[D tModulationSource::Disable (void) ]
Turn off modulation
[ invalid_pointer (this) => UNDEFINED
| true => this->bEnabled := FALSE 
]]





[[D tModulationSource::Update (const TIME tUpdateTime) ]
Computes and returns new value base on modulation value at time tUpdateTime
[ invalid_pointer (this) => UNDEFINED
| this->bEnabled == FALSE => return := this->dValue
| true =>
    this->dValue := MODULATION (this->mpParams, tUpdateTime)
    : return := MODULATION (this->mpParams, tUpdateTime)
]]


[[D tModulationSource::Reset (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dValue := 0.0
    : return := OK
]]

****************** protected specification *************************

[[D tModulationSource::RawValue (void) ] = tInputSource::RawValue ]]

*******************************tSensor***************************************

[[D tSensor::tSensor_td (const tInputChannel &ChannelIn,
		     double dPhysicalUnitsPerElectricalUnit) ]
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) => UNDEFINED
| true =>
    this->Channel := reference to ChannelIn
    : this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
      ChannelIn.EletricalUnitsPerCount
    : this->dValue := 0.0
    : this->dOffset := 0.0
]]


[[D tSensor::tSensor_tdd (const tInputChannel &ChannelIn,
		     double dPhysicalUnitsPerElectricalUnit,
		     double dOffsetIn) ]
[ invalid_pointer (this) ||
  invalid_reference (ChannelIn) => UNDEFINED
| true =>
    this->Channel := reference to ChannelIn
    : this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
      ChannelIn.EletricalUnitsPerCount
    : this->dValue := 0.0
    : this->dOffset := dOffsetIn
]]


[[D tSensor::~tSensor (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]


[[D tSensor::Value (void) ] = tInputSource::Value ]]

[[D tSensor::SetValue (double dNewValue) ] = tInputSource::SetValue ]]

[[D tSensor::SetOffset (double dNewOffset) ] = tInputSource::SetOffset ]]


[[D tSensor::Update(const TIME tUpdateTime) ]
Reads value from this->Channel stores it in this->dValue and returns value.
The value returned in in channel counts. The tUpdateTime argument is ignored.  
[ invalid_pointer (this) ||
  invalid_channel (this->Channel) => UNDEFINED
| true =>
    this->dValue := ChannelValue (this->Channel)
    : return := ChannelValue (this->Channel)
]]


[[D tSensor::CountsToPhysicalUnits (double dCounts) ]
Converst counts to physical value.
[ invalid_pointer (this) => UNDEFINED
| true => return := dCounts * this->dPhysicalUnitsPerCount
]]

[[D tSensor::PhysicalUnitsToCounts (double dPhysicalValue) ]
Converst physical value to channel counts.
[ invalid_pointer (this) ||
  this->dPhysicalUnitsPerCount == 0 => UNDEFINED
| true => return := dPhysicalUnits / this->dPhysicalUnitsPerCount
]]

[[D tSensor::Reset(void) ]
[ invalid_pointer (this) ||
  invalid_channel (this->Channel) => UNDEFINED
| true =>
    this->Channel := ResetChannel ();
    : this->dValue := 0.0
    : return :=  OK 
]]


****************** protected specification *************************8

[[D tSensor::RawValue (void) ] = tInputSource::RawValue ]]


****************  tModulatedSensor **********************
* A tModulatedSensor is a type of tInputSource that combines the
* behaviors of a tSensor with a tModulator, adding the effects
* of the two.  This allows a waveform to be modulated onto a
* sensor output, while maintaining the same interface as an
* ordinary tSensor has.
*


[[D tModulatedSensor::tModulatedSensor_td (const tInputChannel &ChannelIn,
			  double dPhysicalUnitsPerElectricalUnit) ]
Constructs this to use channel ChannelIn with appropriate scale factor
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->Channel := reference to ChannelIn
    : this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
      ChannelIn.EletricalUnitsPerCount
    : this->mpParams.mtModulationType := MT_NONE
    : this->mpParams.dFrequency := 0
    : this->mpParams.dAmplitude := 0
    : this->mpParams.dMeanVal := 0
    : this->mpParams.tSyncTime := 0
    : this->mpParams.dDutyCycle := 0
    : this->bEnabled := FALSE
    : this->dModulationValue := 0.0
]]


[[D tModulatedSensor::tModulatedSensor_tdt (const tInputChannel &ChannelIn,
        double dPhysicalUnitsPerElectricalUnit,
        const tModulationParams& InitialParameters) ]
Constructs this to use channel ChannelIn with appropriate scale factor, and
initial modulation parameters.
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) ||
  invalid_reference (InitialParameters) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->Channel := reference to ChannelIn
    : this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
      ChannelIn.EletricalUnitsPerCount
    : this->mpParams := InitialParameters
    : this->bEnabled := FALSE
    : this->dModulationValue := 0.0
]]

[[D tModulatedSensor::tModulatedSensor_tdtd (const tInputChannel &ChannelIn,
        double dPhysicalUnitsPerElectricalUnit,
        const tModulationParams& InitialParameters, double dOffsetIn) ]
Constructs this to use channel ChannelIn with appropriate scale factor, and
initial modulation parameters.
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) ||
  invalid_reference (InitialParameters) => UNDEFINED
| true =>
    this->dValue := 0.0
    : this->dOffset := dOffsetIn
    : this->Channel := reference to ChannelIn
    : this->dPhysicalUnitsPerCount := dPhysicalUnitsPerElectricalUnit *
      ChannelIn.EletricalUnitsPerCount
    : this->mpParams := InitialParameters
    : this->bEnabled := FALSE
    : this->dModulationValue := 0.0
]]

[[D tModulatedSensor::~tModulatedSensor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]



[[D tModulatedSensor::ModulationValue (void) ]
Gets the current value of the modulation that's being added to the raw sensor 
reading.
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dModulationValue
]]

[[D tModulatedSensor::Value (void) ] = tSensor::Value ]]

[[D tModulatedSensor::SetValue (double dNewValue) ] = tSensor::SetValue ]]

[[D tModulatedSensor::SetOffset (double dNewOffset) ] = tSensor::SetOffset ]]

[[D tModulatedSensor::CountsToPhysicalUnits (double dCounts) ] =
    tSensor::CountsToPhysicalUnits ]
   
[[D tModulatedSensor::PhysicalUnitsToCounts (double dPhysicalValue) ] =
    tSensor::PhysicalUnitsToCounts ]


[[D tModulatedSensor::SetModulationParameters (const tModulationParams& NewModParams) ]
Set this->mpParams with specified parameters.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->mpParams := NewParameters
    : return := OK
]] 



[[D tModulatedSensor::ModulationParameters (void) ] 
[ this invalid => UNDEFINED
| true =>
    return := this->mpParams
]]


[[D tModulatedSensor::EnableModulation (void) ] 
Turn on modulation
[ invalid_pointer (this) => UNDEFINED
| true => this->bEnabled := TRUE 
]]


[[D tModulatedSensor::DisableModulation (void) ]
Turn off modulation
[ invalid_pointer (this) => UNDEFINED
| true => this->bEnabled := FALSE 
]]

[[D tModulatedSensor::IsModulationEnabled (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return this->bEnabled := FALSE 
]]


[[D tModulatedSensor::Update (const TIME tUpdateTime) ]
Computes modulation value based on this->mpParams and updates this->dValue 
with sum of modulation value and value read from this->Channel. 
Returns this sum in channel counts.
[ invalid_pointer (this) ||
  invalid_channel (this->Channel) => UNDEFINED
| this->bEnabled == FALSE =>
    this->dValue := ChannelValue (this->Channel) 
    : return := ChannelValue (this->Channel) 
| true =>
    this->dModulationValue := MODULATION (this->mpParams, tUpdateTime)
    : this->dValue := ChannelValue (this->Channel) + 
      MODULATION (this->mpParams, tUpdateTime)
    : return := ChannelValue (this->Channel) + 
      MODULATION (this->mpParams, tUpdateTime)]]
]]

[[D tModulatedSensor::UnmodulatedValue (void) ]
Return unmodulated value in channel counts.
[ invalid_pointer (this) => UNDEFINED
| simultaneously call to this->Update => return := anything
| true => return := this->dValue - this->dModulationValue
]]

[[D tModulatedSensor::Reset (void) ]
[ invalid_pointer (this) => UNDEFINED
| Simulateous call to tModulatedSensor::Update =>
    this->dModulation := anything
    : this->dValue := anything
    : return := OK
| true =>
    dValue := dValue - dModulationValue
    : dModulationValue := 0.0
    : return := OK
]]

****************** protected specification *************************8

[[D tModulatedSensor::RawValue (void) ] = tSensor::RawValue ]]

*******/

  

class tInputSource {
  public:
    tInputSource(void);
    tInputSource (double dOffsetIn);
    double Value (void) const;
    double Offset (void) const;
    void SetValue (double dNewValue);
    void SetOffset (double dNewOffset); 
    virtual double Update (const TIME tUpdateTime) = 0;
  protected:
    double RawValue () const;
  private:
    double dValue;
    double dOffset;
};


class tSensor : public tInputSource {
  public:
    tSensor (const tInputChannel &ChannelIn,
            double dPhysicalUnitsPerElectricalUnit);
    tSensor (const tInputChannel &ChannelIn,
            double dPhysicalUnitsPerElectricalUnit,
            double dOffsetIn);
     double CountsToPhysicalUnits (double dCounts) const {
       return dCounts * this->dPhysicalUnitsPerCount; }
     double PhysicalUnitsToCounts (double dPhysicalUnits) const {
       return dPhysicalUnits / this->dPhysicalUnitsPerCount; }
 
    ERROR_CODE Reset (void);
    double Update (const TIME tUpdateTime);
    
  protected:
    double ChannelInput ();
 
  private:
    const tInputChannel &Channel;
    double dPhysicalUnitsPerCount;
};

class tModulationSource : public tInputSource {
  public:
    tModulationSource ();
    tModulationSource (const tModulationParams& InitialParameters);
    tModulationSource (const tModulationParams& InitialParameters, 
                       double dOffsetIn);
    ERROR_CODE SetModulationParameters (const tModulationParams& 
                                        NewParameters);
    const tModulationParams& ModulationParameters () const;
    ERROR_CODE EnableModulation ();
    ERROR_CODE DisableModulation ();
    BOOL IsModulationEnabled () const;
    ERROR_CODE Reset (void);
    double Update (TIME tUpdateTime);
  private:
    tModulation Modulation;
};



class tModulatedSensor : public tSensor {
  public:
    tModulatedSensor(const tInputChannel &ChannelIn,
          double dPhysicalUnitsPerElectricalUnit);
    tModulatedSensor(const tInputChannel &ChannelIn,
          double dPhysicalUnitsPerElectricalUnit,
          const tModulationParams& InitialParameters);
    tModulatedSensor(const tInputChannel &ChannelIn,
          double dPhysicalUnitsPerElectricalUnit,
          const tModulationParams& InitialParameters,
          double dOffsetIn);
    ERROR_CODE SetModulationParameters (const tModulationParams& 
                                        NewParameters);
    const tModulationParams& ModulationParameters () const;
    ERROR_CODE EnableModulation ();
    ERROR_CODE DisableModulation ();
    BOOL IsModulationEnabled () const;
    double ModulationValue(void) const;
    double UnmodulatedValue(void) const;
    ERROR_CODE Reset ();
    double Update(const TIME tUpdateTime);
  private:
    double dModulationValue;
    tModulation Modulation;
};


#define S_inputSource_TSENSOR_CONSTRUCTOR              (M_inputSource | 0x1001)
#define S_inputSource_TSENSOR_PHYSICAL_UNITS_PER_COUNT (M_inputSource | 0x1002)
#define S_inputSource_TSENSOR_RESET 		       (M_inputSource | 0x1003)
#define S_inputSource_TMODULATIONSOURCE_CONSTRUCTOR    (M_inputSource | 0x1004)
#define S_inputSource_TMODULATIONSOURCE_SET_MODULATION_PARAMETERS  (M_inputSource | 0x1005)
#define S_inputSource_TMODULATIONSOURCE_GET_MODULATION_PARAMETERS  (M_inputSource | 0x1006)
#define S_inputSource_TMODULATIONSOURCE_RESET          (M_inputSource | 0x1007)
#define S_inputSource_TMODULATEDSENSOR_CONSTRUCTOR     (M_inputSource | 0x1008)
#define S_inputSource_TMODULATEDSENSOR_SET_MODULATION  (M_inputSource | 0x1009)
#define S_inputSource_TMODULATEDSENSOR_GET_MODULATION  (M_inputSource | 0x100A)


#endif  /* INC_InputSourceh */