/* $RCSfile: instrumentCdsGizmo.h,v $ - Type definitions for instrument gizmo  */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/instrumentCdsGizmo.h,v 1.2 1998/07/13 22:01:16 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
02a,25Apr97,rlj      modified initialize parameter list and code for tGizmo 
                     and tInstGizmo.
                     Added CommandHandler, CommandProcessor, and ManualTarget 
                     members
01a,29Oct96,eamck    written
*/

#ifndef INC_instrumentCdsGimzoh
#define INC_instrumentCdsGimzoh

#include "isit.h"
#include "isitCommand.h"
#include "gizmo.h"
#include "commandHandler.h"
#include "extInt.h"
#include "relayer.h"


/********************

[[D tInstrumentCDSGizmo::tInstrumentCDSGizmo (const tInstGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this)  ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo does not accurately represent system configuration ||
  ::IpcMemoryG.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE ||
  ::EventDrivenTaskSet.bInit == FALSE => UNDEFINED
| For any DelayLine in GizmoInfo.GizmoSet CanAllocateIpcObject (::IpcMemoryG,
    DelayLine.iiOpenLoopTargetId) == FALSE ||
  For any DelayLine in GizmoInfo.GizmoSet CanAllocateIpcObject (::IpcMemoryG, 
    DelayLine.iiManualTargetId) == FALSE ||
  For any Rte (RteId) CanAllocateIpcObject (::IpcMemoryG,
    ::ConfigManager.iiErrorStreamId [RteId] ) == FALSE ||
  For any Rte (RteId) CanAllocateIpcObject (::IpcMemoryG,
    ::ConfigManager.iiDataStreamId [RteId] ) == FALSE ||
  For any Rte (RteId) CanAllocateIpcObject (::IpcMemoryG,
    ::ConfigManager.iiStatusStreamId [RteId] ) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE ||
  Unable to add periodic tasks to handle command, data, status and servo =>
    console := console + appropriate error messages
    : this->ecInitError := appropriate error code
    : *this except for this->ecInitError := anything
| true =>
    this->ecInitError := OK
    : console := console + appropriate messages
    : this->ExternalInterface.ecInitError := OK
    : this->Relayer.ecInitError := OK
    : this->CommandHandler.ecInitError := OK
    : this->bInit := TRUE
    : this->ExternalInterface.bInit := TRUE
    : this->Relayer.bInit := TRUE
    : this->ExternalInterface.UiSet := EMPTY_SET 
    : this->CommandHandler.bInit := TRUE
    : this->Relayer.GizmoSet := GizmoInfo.GizmoSet
    : this->CommandHandler.GizmoSet := GizmoInfo.GizmoSet
    : this->OpenLoopTargetSet := subset of GizmoInfo.GizmoSet s.t. 
      Gizmo.GizmoType == GT_DELAY_LINE
    : this->ManualTargetSet := subset GizmoInfo.GizmoSet s.t. 
      Gizmo.GizmoType == GT_DELAY_LINE
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      FA DelayLines in GizmoInfo.GizmoSet NewObjectSet (::IpcMemoryG,
        DelayLine.iiOpenLoopTargetId) U 
      FA DelayLines in GizmoInfo.GizmoSet NewObjectSet (::IpcMemoryG,
        DelayLine.iiManualTargetId) U
      FA Rte (RteId) NewObjectSet (::IpcMemoryG,
        ::ConfigManager.iiErrorStreamId [iRteId] ) U
      FA Rte (RteId) NewObjectSet (::IpcMemoryG,
        ::ConfigManager.iiDataStreamId [iRteId] ) U
      FA Rte (RteId) NewObjectSet (::IpcMemoryG,
        ::ConfigManager.iiStatusStreamId [iRteId] ) U
      NewObjectSet (::IpcMemoryG,GizmoInfo.iiCommandStreamId)
      : ::PeriodicTaskSet := ::PeriodicTaskSet U 
        PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
          GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
          tGizmo::ProcessCommand) U
        PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
          GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
          tGizmo::ProcessStatus) U
	PeriodicTask ("Relay", ::ConfigManager.tRelayTaskPeriod,
          ::ConfigManager.iRelayTaskPriority,::ConfigManager.tRelayTaskOffset,
          tRelayer::Relay)  
]]

[[D tInstrumentCDSGizmo::~tInstrumentCDSGizmo (void) ]	
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tInstrumentCDSGizmo::Restart (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : return := OK
| Unable to resume command, data, status, and error handling =>
    console := console + appropriate error messages
    : return := appropriate error code
| true => 
    this->bInit := TRUE
    : this->ExternalInterface.bInit := TRUE
    : this->Relayer.bInit := TRUE
    : this->CommandHandler.bInit := TRUE
]]

[[D tInstrumentCDSGizmo::Shutdown (void) ]	
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspend command, data, status, and error handling =>
    console := console + appropriate error messages   
    : return := appropriate error code
| true =>
    this->bInit := FALSE
    : this->ExternalInterface.bInit := FALSE 
    : this->Relayer.bInit := FALSE
    : this->CommandHandler.bInit := FALSE 
]]


[[D tInstrumentCDSGizmo::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED 
| T.E element id of this->Relayer.GizmoSet s.t. instrument  cannot receive 
    data status or error from Gizmo (id) ||
  T.E element id of this->ExternalInterface.UiSet s.t. relayer cannot send
    data status or error to UserInterface (id) ||
  T.E element id of this->CommandHandler.GizmoSet s.t. instrument cannot send 
    command to Gizmo (id) ||
  T.E element id of this->ExternalInterface.UiSet s.t. instrument cannot send 
    data status or error to or receive command from UserInterface (id) => return := FALSE
| true => return := TRUE 
]]



[[D tInstrumentCDSGizmo::ConnectionRequest (UI_ID id) ]
[ invalid_pointer (this) => UNDEFINED
| this->ExternalInterface.bInit == FALSE => no response
| true => this->ExternalInterface.UiSet := this->ExternalInterface.UiSet U id
]]

[[D tInstrumentCDSGizmo::TerminateConnection (UI_ID id) ]	
[ invalid_pointer (this) => UNDEFINED
| id not an element of this->ExternalInterface.UiSet => no response
| true => this->ExternalInterface.UiSet:= this->ExternalInterface.UiSet - {id}
]]

[[D tInstrumentCDSGizmo::CommandReceivedFromUI (UI_ID id,const tCommand& Command) ]	
[ invalid_pointer (this)  ||
  invalid_reference (Command) => UNDEFINED
| id not an element of this->ExternalInteface.UiSet || 
  this->CommandHandler.bInit == FALSE => no response
| Command.GizmoId not an element of this->CommandHandler.GizmoSet =>
    ErrorStream << appropriate error information 
| true	=> Command sent to Gizmo(Command.GizmoId)
]]

[[D InstrumentCDSGizmo::ErrorSentToInstrumentCDS (GIZMO_ID id, 
    const tTypeTag& Tag, const BYTE* pError)	]
[ invalid_pointer (this) ||
  invalid_pointer (pError) ||
  invalid_reference (Tag) => UNDEFINED
| this->Relayer.bInit == FALSE || 
  this->ExternalInterface.bInit == FALSE ||
  id not an element of this->Relayer.GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet, Tag and *pError 
    sent to UserInterface (id)
]]

[[D tInstrumentCDSGizmo::DataSentToInstrumentCDS (GIZMO_ID id, 
    const tTypeTag& Tag, const BYTE* pData) ]	
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pData) => UNDEFINED
| this->Relayer.bInit == FALSE || 
  this->ExternalInterface.bInit == FALSE ||
  id not element of this->Relayer.GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet, Tag and Data 
    sent to UserInterface (id)
]]

[[D tInstrumentCDSGizmo::StatusSentToInstrumentCDS (GIZMO_ID id, 
    const tTypeTag& Tag, const BYTE* pStatus) ]	
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pStatus) => UNDEFINED
| this->Relayer.bInit == FALSE || 
  this->ExternalInterface.bInit == FALSE ||
  id not an element of this->Relayer.GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet, Tag and Status 
    sent to UserInterface (id) 
]]

[[D tInstrumentCDSGizmo::Ping (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => ::ErrorStream (::pThisRteG) := 
    ::ErrorStream (::pThisRteG) + appropriate error code ]]

[[D tInstrumentCDSGizmo::SetOpenLoopTarget (GIZMO_ID DelayLineId, 
    const tDlTarget& DlTarget) ]
[ invalid_pointer (this) ||
  invalid_reference (DlTarget) => UNDEFINED
| DelayLineId not an element of this->OpenLoopTargetSet => 
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate error code 
| true => DlTarget sent to DelayLine (DelayLineId) 
]]

[[D tInstrumentCDSGizmo::SetManualTarget (GIZMO_ID DelayLineId, 
    const tDlTarget& DlTarget) ]
[ invalid_pointer (this) ||
  invalid_reference (DlTarget) => UNDEFINED
| DelayLineId not element of this->ManualTargetSet => ::ErrorStream (::pThisRteG) := 
   ::ErrorStream (::pThisRteG) + appropriate error code 
    sent to pThisRteG error queue
| true => DlTarget sent to DelayLine (DelayLineId)
]]

[[D tInstrumentCDSGizmo::CollectStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := OK
]]

[[D tInstrumentCDSGizmo::Dispatch (const tIsitCmd& Command) ]
[ invalid_pointer (this) ||
  invalid_reference (Command) => UNDEFINED
| Command.Command == INST_CMD_SET_OPEN_LOOP_TARGET &&
    Command.iWhichDl not valid delay line ||
  Command.Command == INST_CMD_SET_MANUAL_TARGET &&
    Command.iWhichDl not valid delay line => 
      ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate messages
      : return := OK
| Command.Command == INST_CMD_PING => 
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate messages
    : return := OK
| Command.genericCmd.Command == INST_CMD_SET_OPEN_LOOP_TARGET =>
    Command.Target set to appropriate delay line open loop target
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate messages
    : return := OK
| Command.Command == INST_SET_CMD_MANUAL_TARGET =>
    Command.Target set to appropriate delay line manual target
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate messages
    : return := OK
| true => 
    ::ErrorStream (::pThisRteG) := ::ErrorStream (::pThisRteG) + appropriate error messages
    : return := appropriate error code
]]

****/


class tInstrumentCDSGizmoInfo;
class tIsitCmd;
class tDlActuator;
class tInstSetOpenLoopTargetCmd;
class tInstSetManualTargetCmd;
class tInstSetTimeCmd;
class tClockBoard;


  
class tInstrumentCDSGizmo : public tGizmo {
  public:
    tInstrumentCDSGizmo (const tInstrumentCDSGizmoInfo &GizmoInfo);
    virtual ~tInstrumentCDSGizmo ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();

    //kludge functions for sequencer...
   
    ERROR_CODE SetSeqCallback(int iWhichDl,
                              void (* pSeqCallback)(BOOL bFrontSoft, 
						    BOOL bFrontHard, 
						    BOOL bBackSoft,
                                                    BOOL bBackHard, 
						    BOOL bHome, 
                                                    LASER_VALUE lvLaserValue));
    void DoCommand (const ISIT_CMD& Command); 

  protected:
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
    virtual ERROR_CODE CollectStatus ();

    tCommandHandler        CommandHandler; 
    tExternalInterface     ExternalInterface;
    tRelayer               Relayer;
    tDlActuator**          pManualTargetArray;
    tDlActuator**          pOpenLoopTargetArray;
    tClockBoard*           pClockBoard;
    int                    iNumberOfDelayLines;

  private:
    int        WhichDlActuator (int iGizmoId);
    ERROR_CODE Ping ();
    ERROR_CODE SetOpenLoopTarget (const tInstSetOpenLoopTargetCmd& Command);
    ERROR_CODE SetManualTarget   (const tInstSetManualTargetCmd& Command);
    ERROR_CODE SetTime           (const tInstSetTimeCmd& Command);
     
    int*                   iDlActuatorMap;

    // kludge state for Sequencer
    void (* pSeqCallback) (BOOL bFrontSoft, BOOL bFrontHard, BOOL bBackSoft,
                                 BOOL bBackHard, BOOL bHome, LASER_VALUE lvLaserValue);
    int iDlIdForCallback;

};     /* class tInstGizmo */

#endif INC_instrumentCdsGizmoh






