/* $RCSfile: interrupt.h,v $ - Hardware manager classes */
/* $Header: /home/chara/cvsroot/ople/sys/include/interrupt.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */

/* Description:
 * This file contains the interrupt class definition
 * The tInterrupt class is used almost exclusively by the scheduler
 * and is part of the harware manager library because the interrupt
 * functionality resides on a support VME board.
 */

/* $Log: interrupt.h,v $
 * Revision 1.1.1.1  1998/06/11 21:36:12  chara
 * First import of RICST Code
 *
 * Revision 1.14.2.3  1998/05/12 22:59:08  phil
 * changes before build_5b_001
 *
 * Revision 1.14.2.2  1998/05/12 20:26:22  phil
 * Interrupt and tTimBoard changes for I5b
 *
 * Revision 1.14.2.1  1998/05/08 01:20:16  phil
 * interrupt functionality for I5b
 * */

#ifndef INC_interrupth
#define INC_interrupth

#include "baseObject.h"
#include "hwTypes.h"
#include "vxworks/intLib.spc"

/************************************************************************************
*
* class tInterrupt : public tBaseObject
*
* void tInterrupt::tInterrupt(...)
* Description: Constructor - sets all default values for variables.
[[D tInterrupt::tInterrupt(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    tBaseObject::bInit := FALSE
    : tBaseObject::ecInitError := OK
    : this->bConnected := FALSE
    : this->bEnabled := FALSE
]]
*
*
* ERROR_CODE tInterrupt::Initialize(...)
[[D tInterrupt::Initialize(void) ]
Base class initialize is just a placeholder right now.  No functionality.
bInit is set to TRUE in SetIdentity.
[ invalid_pointer (this) => UNDEFINED
| true => return := OK
]]
*
*
* ERROR_CODE tInterrupt::SetIdentity(...)
[[D tInterrupt::SetIdentity(unsigned int uiVMEPriorityToUse,
                            unsigned int uiVMEVectorToUse)]
bInit is set to true in this method because setting the priority and
vector completes the initialization process.  All other initialization
routines do not change the state of bInit.
[ invalid_pointer(this) => UNDEFINED
| uiVMEPriorityToUse < MIN_BUS_INT_PRIORITY || uiVMEPriorityToUse > MAX_BUS_INT_PRIORITY =>
  return S_interrupt_INVALID_INTERRUPT_PRIORITY
| uiVMEVectorToUse < MIN_INTERRUPT_VECTOR || uiVMEPriorityToUse > MAX_INTERRUPT_VECTOR =>
  return S_interrupt_INVALID_INTERRUPT_VECTOR
| true =>
    this->uiVMEPriority := uiVMEPriorityToUse
    : this->uiVMEVector := uiVMEVectorToUse
    : this->bInit := TRUE
    : return := OK
]]
*
*
* ERROR_CODE tInterrupt::Connect(...)
* Description:
[[D tInterrupt::Connect(FUNCPTR pFunc, int iArg)]
[ invalid_pointer (this) ||
  invalid_function_pointer (pFunc) ||
  invalid_cplus_plus_function_pointer (pFunc,iArg) => UNDEFINED
| this->bInit == FALSE ||
  pFunc == NULL ||
  this->bConnected == TRUE =>
    return := appropriate error code
| true =>
    enable board interrupt source
    : connect board interrupt source to
      VME interrupt this->pvInterrupt at this->uiVMEVector
    : this->bConnected := TRUE
    : return := OK
]]
*
*
* ERROR_CODE tInterrupt::Disconnect(...)
[[D tInterrupt::Disconnect (void) ]
WARNING: VxWorks has no intDisconnect function.  This method is
provided merely for symmetry with ::Connect and ensures OS
interrupt disable as well as board-level disable (if supported).
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    return := appropriate error code
| this->bConnected == FALSE => 
    return := Ok
| true => 
    Disable interrupt source
    : this->bConnected := FALSE
    : return := OK
]]
*
*
* ERROR_CODE tInterrupt::Enable(...)
[[D tInterrupt::Enable (void) ]
Enables interrupt.  CPU will now respond to interrupt number this->uiVMEPriority
[ invalid_pointer (this) ||
  More than one CPU is connected to this->uiIntNum => UNDEFINED
| this->bInit == FALSE ||
  this->bConnected == FALSE =>
    return := appropriate error code
| this->bEnabled == TRUE =>
    return := OK
| true =>
    enable interrupt
    : Begin sending interrupt this->uiVMEVector
      to the VME bus.  The ISR on the CPU connected to this
      interrupt source will be called at a rate of
      this->irRate until a Disable command is given.
    : this->bEnabled := TRUE
    : return := OK
]]
*
*
* ERROR_CODE tInterrupt::Disable(...)
[[D tInterrupt::Disable (void) ]
Disables interrupt.  CPU ignores interrupt number this->uiVMEPriority
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    return := appropriate error code
| this->bEnabled == FALSE =>
    return OK;
| true =>
    disable interrupt
    : Stop sending interrupt number this->puwIntNum
      to the VME bus. All routines on all CPUs that are connected to
      this->puwIntNum will stop being called periodically
    : this->bEnabled := FALSE
    : return := OK
]]
*
*
* ERROR_CODE tInterrupt::Isr()
[[D tInterrupt::Isr()]
Interrupt Service Routine.
[ this->pFunc is not a pointer to valid C function &&
   !(this->pFunc is a pointer to valid C++ function &&
     this->iArg == this) => UNDEFINED
| true =>
    call this->pFunc with argument this->iArg
]]
*
*/
/************************************************************************************
*
* class tPeriodicInterrupt : public tInterrupt
*
* void tPeriodicInterrupt::tPeriodicInterrupt(...)
* Description: Constructor - sets all default values for variables.
[[D tPeriodicInterrupt::tPeriodicInterrupt(void)]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := FALSE
    : this->bConnected := FALSE
    : this->bRateSetYet := FALSE
    : this->bEnabled := FALSE
    : this->irRate := 0
    : this->irMinRate := 0
    : this->irMaxRate := 0
]]
*
*
* ERROR_CODE tPeriodicInterrupt::Initialize(...)
[[D tPeriodicInterrupt::Initialize(INT_RATE irMinRateToUse,
                                   INT_RATE irMaxRateToUse)]
Sets interrupt pointers and addresses.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->irMinRate := irMinRateToUse;
    : this->irMaxRate := irMaxRateToUse;
    : return := OK
]]
*
*
* ERROR_CODE tPeriodicInterrupt::RateSet(INT_RATE *pirRateToUse)
* Description:
[[D tPeriodicInterrupt::RateSet(INT_RATE* pirRateToUse)]
[ invalid_pointer (this) ||
  invalid_pointer (pirRateToUse) => UNDEFINED
| this->bInit == FALSE =>
    return S_interrupt_UNINITIALIZED_INTERRUPT
| No hardware-specific method exists &&
  (*pirRateToUse < this->irMinRate ||
   *pirRateToUse > this->irMaxRate) =>
    console := console || warning message about calling base class method
    : *pirRateToUse := closest limit
    : this->irRate := closest limit
    : this->bRateSetYet := TRUE
    : return := S_interrupt_W_INVALID_RATE
| No hardware-specific method exists &&
  (this->irMinRate <= *pirRateToUse <= this->irMaxRate) =>
    console := console || warning message about calling base class method
    : this->irRate := *pirRateToUse
    : this->bRateSetYet := TRUE
    : return := OK
| *pirRateToUse < this->irMinRate ||
  *pirRateToUse > this->irMaxRate =>
    *pirRateToUse := closest limit
    : this->irRate := closest limit
    : this->bRateSetYet := TRUE
    : return := S_interrupt_W_INVALID_RATE
| *pirRateToUse is not valid for this hardware =>
    *pirRateToUse := valid rate nearest and higher to *pirRateToUse
    : this->irRate := valid rate nearest and higher to *pirRateToUse
    : this->bRateSetYet := TRUE
    : return := S_interrupt_W_INVALID_RATE
| true =>  
    this->irRate := *pirRateToUse
    : this->bRateSetYet := TRUE
    : return := OK 
]]
*
*
* INT_RATE tPeriodicInterrupt::RateGet(void)
[[D tPeriodicInterrupt::RateGet(void) ]
[ invalid_pointer (this) ||
  this->bInit == FALSE => UNDEFINED 
| true => return := currently set rate for this interrupt
]]
*
*
*
*
* ERROR_CODE tPeriodicInterrupt::Enable(...)
* Description:
[[D tPeriodicInterrupt::Enable (void) ]
[ invalid_pointer (this) ||
  More than one CPU is connected to this->uiIntNum => UNDEFINED
| this->bInit == FALSE ||
  this->bRateSetYet == FALSE ||
  this->bConnected == FALSE =>
    return := appropriate error code
| this->bEnabled == TRUE =>
    return := OK
| true => 
    enable interrupt
    : Begin sending interrupt this->uiVMEVector
      to the VME bus.  The ISR on the CPU connected to this
      interrupt source will be called at a rate of
      this->pRate until a Disable command is given.
    : this->bEnabled := TRUE
    : return := OK
]]
*
*
* ERROR_CODE tPeriodicInterrupt::Disable(...)
* Description:
[[D tPeriodicInterrupt::Disable (void) ]
[ invalid_pointer (this) => UNDEFINED 
| this->bInit == FALSE =>
    return := appropriate error code
| this->bEnabled == FALSE =>
    return OK;
| true => 
    disable interrupt
    : Stop sending interrupt number this->puwIntNum
      to the VME bus. All routines on all CPUs that are connected to
      this->puwIntNum will stop being called periodically
    : this->bEnabled := FALSE 
    : return := OK
]]
*
*/
/************************************************************************************
*
* class tPeriodicTimeInterrupt : public tPeriodicInterrupt
*
*
* ERROR_CODE tPeriodicTimeInterrupt::Initialize(...)
[[D tPeriodicTimeInterrupt::Initialize(INT_RATE irMinRateToUse,
                                       INT_RATE irMaxRateToUse)]
Sets interrupt pointers and addresses.
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->irMinRate := irMinRateToUse;
    : this->irMaxRate := irMaxRateToUse;
    : return := OK
]]
*
*
* TIME tPeriodicTimeInterrupt::GetTimeLastInt(...)
[[D tPeriodicTimeInterrupt::GetTimeLastInt(void)]
Retrieve last time interrupt was fired
[ invalid_pointer(this) => UNDEFINED
| this->bInit == FALSE =>
    return S_tim_CLOCK_NOT_INITIALIZED
| true =>
    return time of last interrupt in 25us since midnight
]]
*
*/

class tInterrupt : public tBaseObject {
  public:
    tInterrupt(void);
    virtual Initialize(void);
    ERROR_CODE SetIdentity(uint8_t uiVMEPriorityToUse,
                           uint8_t uiVMEVectorToUse);
    virtual ERROR_CODE Connect(FUNCPTR pFunc,
                       int     iArg);
    virtual ERROR_CODE Disconnect(void);
    virtual ERROR_CODE Enable(void);
    virtual ERROR_CODE Disable(void);

  protected:
    unsigned int uiVMEPriority;  /* VME interrupt priority number */
    unsigned int uiVMEVector;    /* VME interrupt vector */

  private:
    virtual void Isr(void);      /* Entry point for ISR from Connect() */
    FUNCPTR  pFunc;              /* Function to call in ISR */
    int      iArg;               /* Argument to pass to pFunc (must be pointer to
                                    object that pFunc belongs to) */

    BOOL bConnected;             /* indicates function has been connected */
    BOOL bEnabled;               /* indicates interrupts are being generated */
};

class tPeriodicInterrupt : public tInterrupt {
  public:
    tPeriodicInterrupt(void);
    virtual ERROR_CODE Initialize(INT_RATE irMinRateToUse,
                                  INT_RATE irMaxRateToUse);
    virtual ERROR_CODE Enable(void);
    virtual ERROR_CODE RateSet(INT_RATE *pRate);
    virtual INT_RATE   RateGet(void) { return this->irRate; }

  protected:
    BOOL       bRateSetYet;         /* indicates rate has been set */
    INT_RATE   irRate;

  private:
    INT_RATE   irMinRate;
    INT_RATE   irMaxRate;
};

class tPeriodicTimeInterrupt : public tPeriodicInterrupt {
  public:
    virtual ERROR_CODE Initialize(INT_RATE irMinRateToUse,
                                  INT_RATE irMaxRateToUse);
    virtual TIME GetTimeLastInt(void) = 0;

  private:
};

#endif  /* INC_interrupth */
