/* $RCSfile: ipc.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipc.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */

/*
modification history
--------------------
01a,2Jun97,rlj    written
*/

#ifndef INC_ipch
#define INC_ipch

#include "isit.h"
#include "baseObject.h"

class tIpcId;
class tLocalMemory;
class tSharedMemory;
class tTokenClient;


/*** 
::IpcMemoryG gets transformed into individual tIpc resident on tRte
**/


/*********
Black Box State


tIpc:
  
tLocalMemory    *pLocalMemory 
tSharedMemory   *pSharedMemory 
tTokenClient    *pTokenClient
ERROR_CODE      ecInitError 
BOOL            bInit
-----------------------------------------------------------------------------
Other Types Used in spec

MEM_TYPE   (see ipcid.h)
tIpcId     (see ipcid.h)  
TOKEN_FLAG (see token2.h)
IPC_OBJECT_SET

Spec Functions 


set of tIpcObject ObjectSet (tIpc *pIpc, MEM_TYPE mtType) 
[ pIpc is invalid ||
  mtType not in { MT_LOCAL, MT_SHARED } => UNDEFINED
| mtType == MT_LOCAL => return pIpc->pLocalMemory->ObjectSet
| true => return := pIpc->pSharedMemory->ObjectSet
]


BOOL CanAllocateObject (tIpc *pIpc, tIpcId iiId, size_t szSize)
[ iiId.mtObjectMemType not in { MT_LOCAL,MT_SHARED } ||
  iiId.mtObjectMemType == MT_SHARED  &&
    pIpc->pSharedMemory->TokenServer does not grant token in 
    pIpc->pTokenClient->tTimeOut ticks ||
  TE. a tIpcObject x, in ObjectSet(pIpc,iiId.mtObjectMemType) st. 
    x.iiObjectId.inObjectName == iId.inObjectName && 
    ! ( (x.iiObjectId.itObjectType == iiId.itObjectType) && 
      ( x.iiObjectId.szObjectSize == szSize) ) ||
  !TE an ipc object x in ObjectSet(pIpc,iiId.mtObjectMemType) st. 
    x.iiObjectId.inObjectName == iiId.inObjectName &&
    pMemory->szHeapFree < szSize => return := FALSE
| true => return := TRUE
]


tMemory *Memory (tIpc *pIpc, MEM_TYPE mtType) 
[ invalid_pointer (pIpc) ||
  mtType not in { MT_LOCAL, MT_SHARED } => UNDEFINED
| mtType == MT_LOCAL => return := pIpc->pLocalMemory
| true => return := pIpc->pSharedMemory
]


set of tIpcObject NewObjectSet (tIpc *pIpc, tIpcId iiId )
[ TE. a tIpcObject x in ObjectSet (pIpc,iiId) st. x.iiObjectId == iiId => 
    return := NULL SET
| true => return := { tIpcObject with tIpcObject.iiId = iiId }
]
 
size_t AllocationSize (tIpc *pIpc, tIpcId iiId, size_t szSize) 
[ TE. a tIpcObject x in pIpc->Memory(pIpc,iiId.mtObjectMemType)->ObjSet 
    st. x.iiObjectId == iiId => return := 0
| true => return := szSize
]

  

------------------------------------------------------------------------------
Stimuli 

[[D tIpc::tIpc (volatile SHMEM_WORD *vswSharedMemoryStart,
          const size_t &szNumSharedMemoryBytes,
          const int &iNumRtes,
          const int &iWhichRte) ]
[ invalid_pointer (this) ||
  invalid_pointer (vswSharedMemoryStart) => UNDEFINED
|   (! (0 <= iWhichRte < CiNumRte) ) ||
    Unable to initialize local and shared memory => 
    console := console + appropriate error
    : this->ecInitError := appropriate error code
    : this->bInit := FALSE
    : this->pSharedMemory->ecInitError := appropriate errrorCode
    : this->pSharedMemory->bInit := anything
    : this->pLocalMemory->ecInitError := appropriate errorCode
    : this->pLocalMemory->bInit := anything
| true => 
      valid_pointer (this->pLocalMemory)
      : valid_pointer (this->pSharedMemory)
      : valid_pointer (this->pTokenClient)
      : this->ecInitError := OK
      : this->bInit := TRUE
      : this->pLocalMemory->bInit := TRUE
      : this->pLocalMemory->ObjectSet := NullSet
      : this->pLocalMemory->szHeapFree := appropriate size
      : this->pSharedMemory->bInit := anything
      : this->pSharedMemory->ObjectSet := anything
      : this->pSharedMemory->szHeapFree := anything
      : this->pTokenClient->bOwnToken := FALSE
      : this->pTokenClient->bInit := TRUE
      : this->pTokenClient->tTimeOut := tTimeOutToUse
      : this->pTokenClient->vtfRequestFlag := appropriate token flag location
      : this->pTokenClient->ecInitError := OK
]]

[[D tIpc::Restart (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pLocalMemory) ||
  invalid_pointer (this->pSharedMemory) ||
  invalid_pointer (this->pTokenClient) => UNDEFINED
| this->pLocalMemory->bInit == TRUE ||
  this->pSharedMemory->bInit == TRUE ||
  this->pTokenClient->bInit == TRUE => 
      appropriate warning messages printed on console
      : this->pLocalMemory->bInit := TRUE
      : this->pLocalMemory->ObjectSet := anything
      : this->pSharedMemory->bInit := anything
      : this->pSharedMemory->ObjectSet := anything
      : this->pTokenClient->bInit := TRUE
      : return := OK
| true => 
      this->pLocalMemory->bInit := TRUE
      : this->pLocalMemory->ObjectSet := anything
      : this->pSharedMemory->bInit := anything
      : this->pSharedMemory->ObjectSet := anything
      : this->pTokenClient->bInit := TRUE
      : return:=OK
]]


[[D tIpc::Shutdown  (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->pLocalMemory) ||
  invalid_pointer (this->pSharedMemory) ||
  invalid_pointer (this->pTokenClient) => UNDEFINED
| this->pLocalMemory->bInit == FALSE ||
  this->pSharedMemory->bInit == FALSE ||
  this->pTokenClient->bInit == FALSE => 
    appropriate warning message printed on console
    : this->pLocalMemory->bInit := FALSE
    : this->pTokenClient->bOwnToken := FALSE
    : this->pTokenClient->bInit := FALSE
    : return := OK
| this->pTokenClient->bOwnToken == TRUE => 
    Token returned
    : this->pLocalMemory->bInit := FALSE
    : this->pTokenClient->bOwnToken := FALSE
    : this->pTokenClient->bInit := FALSE
    : return:=OK
| true => 
    this->pLocalMemory->bInit := FALSE
    : this->pTokenClient->bInit := FALSE
    : return := OK
]]

 
[[D tIpc::Allocate (const tIpcId &iiId, const size_t &szSize, 
                    ERROR_CODE &ecErrCode) ]
[ invalid_pointer (this) ||
  piiId is invalid ||
  ecErrCode is invalid ||
  invalid_pointer (this->pLocalMemory) ||
  invalid_pointer (this->pSharedMemory) ||
  invalid_pointer (this->pTokenClient) || 
  this->pSharedMemory->bStaticInit == FALSE ||
  invalid_pointer (this->pTokenClient->vtfRequestFlag) => UNDEFINED
| CanAllocateObject (this, *piiId, szSize + overhead) == FALSE => 
    ecErrCode := appropriate error code
    : console := console + appropriate error messages
    : return := NULL
| true =>
    ecErrCode:= OK
    : IpcObjectSet(this,piiId->mtObjectMemType) := ObjectSet (this,piiId->mtObjectMemType) U 
      PossibleNewObject (this, *piiId )
    : Memory(this,piiId->mtObjectMemType).szHeapFree := 
      Memory(this,piiId->mtObjectMemType).szHeapFree - 
      AllocationSize (this, iiId, szSize)
    : return := address of appropriate tIpcObject 
]]


**************/


const int CiIpcTimeOutSeconds = 5;

class tIpc : public tBaseObject {
  public:
    tIpc (volatile SHMEM_WORD *vswSharedMemoryStart,
          const size_t &szNumSharedMemoryBytes,
          const int &iNumRtes,
          const int &iWhichRte);
    ERROR_CODE Restart ();
    ERROR_CODE Shutdown ();
    void *Allocate (const tIpcId &iiId, const size_t &szSize, 
                    ERROR_CODE &ecErrCode);   
    friend ostream &operator<< (ostream& stream,const tIpc &Ipc);
  protected:
    tLocalMemory *pLocalMemory;
    tSharedMemory *pSharedMemory;
    tTokenClient *pTokenClient;
};
    
#endif
