/* $RCSfile: ipcAtomAccessor.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcAtomAccessor.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */

#ifndef INC_ipcAtomAccessorh
#define INC_ipcAtomAccessorh

#include "isit.h"
#include "ipcid.h"
#include "ipcObjectAccessor.h"
#include "ipcElement.h"
/****************************************************************************

tIpcAtomAccessor Black Box State

BOOL        bInit
ERROR_CODE  ecInitError
tIpcAtomId  iiAtomId

-----------------------------------------------------------------------------

[[D tIpcAtomAccessor::tIpcAtomAccessor  (const tIpcAtomId &iiId) ]
[ invalid_pointer (this) || 
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiAtomId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiAtomId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]


[[D tIpcAtomAccessor::~tIpcAtomAccessor (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]

[[T tIpcAtomAccessor::Restart (void) ][
[[U tBaseObject::Restart (void) ]]
]]

[[D tIpcAtomAccessor::Get (tDataType &DataOut) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG,this->iiObjectId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, this->iiAtomId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true => 
    DataOut := GetObject (::IpcMemoryG, this->iiAtomId)->dtAtomData
    : return := OK 
]]


[[D tIpcAtomAccessor::Put (const tDataType &DataIn) ]
[ invalid_pointer (this) ||   
  this->bObjectValid == FALSE || 
  simultaneous put or get on GetObject (::IpcMemoryG,this->iiAtomId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true => 
    GetObject (::IpcMemoryG,this->iiAtomId)->dtAtomData := DataIn
    : return := OK 
]]

[[D tIpcAtomAccessor::~IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => return := this->bObjectValid
]]

********************************/


template <class tDataType> class tIpcAtomAccessor : 
    public tIpcObjectAccessor<tDataType> {
  public:
    tIpcAtomAccessor (const tIpcAtomId &iiId);
    virtual ~tIpcAtomAccessor ();
    virtual ERROR_CODE Get (tDataType &DataOut);
    virtual ERROR_CODE Put (const tDataType &DataIn);
    virtual BOOL IsOk () const;
  protected:
    volatile tIpcElement<tDataType> *vElement;
  };

/***********************************
Template method definitions
***********************************/

/**************************************************************
Representation Mapping

Black Box                            State Box
____________________________________________________________

tIpcMemory		             

::IpcMemoryG                  ::pThisRteG->Ipc
::IpcMemoryG.bInit	      ::pThisRteG->Ipc.pSharedMemory->bStaticInit

::IpcMemoryG.ObjectSet        IpcObjectSet(::pThisRteG->Ipc,iiId.mtMemType)
::IpcMemoryG.szHeapFree       Memory(::pThisRteG->Ipc,iiId.mtMemType)->szHeapFree
                                  
------------------------------------------------------------

StateBox 					    Black Box

invalid_pointer (::pThisRteG)                       ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pLocalMemory)     ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pSharedMemory)    ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pTokenClient)     ::IpcMemory.bInit := FALSE
::pThisRteG->Ipc.pSharedMemory::GlobalInit(..)      ::IpcMemory.bInit := FALSE
  not executed    

this->vAtom                        this->iiAtomId
invalid_pointer (this->vAtom)      this->bValidObject == FALSE || DoesObjectExist (..) == FALSE 
this->vAtom->dtAtomData            GetObject (...)->dtAtomData

this->ecInitError		   this->ecInitError
this->bInit		           this->bInit

		             

*****************************************************************************
			tIpcAtomAccessor Method Definitions
******************************************************************************/
  

/****
[[U tIpcAtomAccessor::tIpcAtomAccessor(iiId) ]]
****/
template <class tDataType> tIpcAtomAccessor<tDataType>::
  tIpcAtomAccessor (const tIpcAtomId &iiId) : 
  
  /****
  [[U tIpcObjectAccessor::tIpcObjectAccessor(void) ]]
  ****/  
  tIpcObjectAccessor<tDataType> () 
{
  ERROR_CODE ec;

    /****
    [[U this->vElement := tIpcElement::operator_new (appropriate_size,iiId,this->ecInitError) ]]

    [[U this->vElement->tIpcElement::tIpcAtom(void)]]
    ****/    
    this->vElement = new (iiId,ec) tIpcElement<tDataType>;
    this->SetInitError(ec);
}

/****
[[U tIpcAtomAccessor::~tIpcAtomAccessor(void) ]]
****/
template <class tDataType> tIpcAtomAccessor<tDataType>::
  ~tIpcAtomAccessor () {
  
  /****
  [[U tIpcObjectAccessor::~tIpcObjectAccessor(void) ]]
  ****/  
  
}

/****
[[U tIpcObjectAccessor::IsOk(void) ]]
****/
template <class tDataType> BOOL tIpcAtomAccessor<tDataType>::IsOk () const {
  return (this->vElement != NULL);
}


/****
[[U tIpcAtomAccessor::Get(DataOut) ]]
****/
template <class tDataType> ERROR_CODE tIpcAtomAccessor<tDataType>::
  Get(tDataType &DataOut) {

  if (tBaseObject::IsInit() == TRUE) {
    /****
    [[U return := this->vElement.tIpcElement::Get(DataOut) ]]
    ****/
   
    return this->vElement->Get(DataOut);
  }
  
  else return S_ipc_OBJECT_NOT_ENABLED;
  
}
  
/****
[[U tIpcAtomAccessor::Put(DataIn) ]]
****/
template <class tDataType> ERROR_CODE tIpcAtomAccessor<tDataType>::
  Put(const tDataType &DataIn) {
  
  if (tBaseObject::IsInit() == TRUE) {
    /****
    [[U return := this->vElement.tIpcElement::Put(DataIn) ]]
    ****/
    return this->vElement->Put(DataIn);
  }
  
  else return S_ipc_OBJECT_NOT_ENABLED;
}



#endif INC_ipcAtomAccessorh
