/* $RCSfile: ipcDblBuffer.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcDblBuffer.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */



#ifndef INC_ipcDblBufferh
#define INC_ipcDblBufferh

#include "isit.h"
#include "ipcid.h"
#include "ipc.h"
#include "ipcError.h"

/************************************************************
tIpcDblBuffer<tDataType> Black Box State

BOOL      bEmpty
tDataType dtBuffer
 
________________________________________________________________________

Specification Functions:

CanAlocateIpcObject (..) - see ipcObjectAccessor.h
NewObjectSet (...)       - see ipcObjectAccessor.h
AllocationSize (...)     - see ipcObjectAccessor.h
________________________________________________________________________

Stimuli:


[[D tIpcDblBuffer::tIpcDblBuffer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->dtBuffer := default value
    : this->bEmpty := TRUE
]]

[[D tIpcDblBuffer::Get (tDataType &DataOut) ]
[ invalid_pointer (this) => UNDEFINED
| this->bEmpty == TRUE => 
    return := S_ipc_BUFFER_EMPTY
| true =>
    DataOut := this->dtBuffer
    : return := OK
]]

[[D tIpcDblBuffer::Put (const tDataType &DataIn) ]
[ invalid_pointer (this) || 
  simultaneous put on this => UNDEFINED
| true => 
    this->dtBuffer := DataIn
    : return := OK 
]] 

[[D tIpcDblBuffer::operator_new (const tIpcDblBufferId &iiId, 
                        ERROR_CODE &ecErrCode) ]
[ invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE => 
    ecErrCode := appropriate error code
    : appropriate messages printed on console
    : return := NULL
| true =>
    ecErrCode := OK
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]

*************/


typedef enum IPC_BUFFER_SELECT {BS_EMPTY_BUFFER, BS_BUFFER1, BS_BUFFER2 } 
  IPC_BUFFER_SELECT;


template <class tDataType> class tIpcDblBuffer {
  public:
    tIpcDblBuffer ();
    ERROR_CODE Get (tDataType &DataOut) volatile;
    ERROR_CODE Put (const tDataType &DataIn) volatile;
    void *operator new (size_t szSize,const tIpcDblBufferId &iiId, 
                        ERROR_CODE &ecErrCode);
  protected:
    tDataType dtBuffer1, dtBuffer2;
    IPC_BUFFER_SELECT bsWhich; 
};

/******************************
Template Method Definitions
*******************************/


/******************************************************************************
			tDbBuffer Method Definitions
******************************************************************************/

/****
[[U tIpcDblBuffer::tIpcDblBuffer(void) ]]
****/  
template <class tDataType> tIpcDblBuffer<tDataType>::tIpcDblBuffer() {
  this->bsWhich = BS_EMPTY_BUFFER; 
}

/****
[[U tIpcDblBuffer::Get(DataOut) ]]
****/  
template <class tDataType> ERROR_CODE tIpcDblBuffer<tDataType>::
  Get(tDataType &DataOut) volatile {
    if (this->bsWhich == BS_EMPTY_BUFFER) return S_ipc_BUFFER_EMPTY;
    else if (this->bsWhich == BS_BUFFER1) DataOut = this->dtBuffer1;
    else if (this->bsWhich == BS_BUFFER2) DataOut = this->dtBuffer2;
    return OK;
}

/****
[[U tIpcDblBuffer::Put(DataIn) ]]
****/  
template <class tDataType> ERROR_CODE tIpcDblBuffer<tDataType>::
  Put(const tDataType &DataIn) volatile {
    if ((this->bsWhich == BS_EMPTY_BUFFER) || (this->bsWhich == BS_BUFFER2)) {
      this->dtBuffer1 = DataIn;
      this->bsWhich = BS_BUFFER1;
    }
    else if (bsWhich == BS_BUFFER1) {
      this->dtBuffer2 = DataIn;
      this->bsWhich = BS_BUFFER2;
    }
    return OK;
}

/****
[[U tIpcDblBuffer::operator_new(szSize,iiId,ecErrCode) ]]
****/  
template <class tDataType> void *tIpcDblBuffer<tDataType>::
  operator new (size_t szSize, const tIpcDblBufferId &iiId, 
                ERROR_CODE &ecErrCode) {

    TraceMark (S_object_CREATING_DOUBLE_BUFFER);

    /****
    [[U return := (::pThisRteG)->pIpc.tIpc::Allocate (iiId,szSize,ecErrCode) ]]
    ****/
    return GetIpc().Allocate(iiId,szSize,ecErrCode);
}



#endif INC_ipcDblBufferh
