/* $RCSfile: ipcDblBufferAccessor.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcDblBufferAccessor.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */

#ifndef INC_ipcDblBufferAccessorh
#define INC_ipcDblBufferAccessorh


#include "isit.h"
#include "ipcid.h"
#include "ipcError.h"
#include "ipcDblBuffer.h"
#include "ipcObjectAccessor.h"

/****************************************************************************

tIpcDblBufferAccessor BlackBox State

BOOL        bInit
ERROR_CODE  ecInitError
tIpcBufferId iiBufferId

______________________________________________________________________________


[[D tIpcDblBufferAccessor::tIpcDblBufferAccessor  (const tIpcDblBufferId &iiId) ]
[ invalid_pointer (this) || 
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiDblBufferId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiDblBufferId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]

[[D tIpcDblBufferAccessor::~tIpcDblBufferAccessor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tIpcDblBufferAccessor::Get (tDataType &DataOut) ]
[ invalid_pointer (this) || 
  invalid_reference (DataOut) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiDblBufferId) == FALSE => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| GetObject (::IpcMemoryG, 
    this->iiDblBufferId).bEmpty == TRUE => return := S_ipc_BUFFER_EMPTY
| true =>
    DataOut := GetObject (::IpcMemoryG, this->iiBufferId)->dtBuffer
    : return := OK
]]



[[D tIpcDblBufferAccessor::Put (const tDataType &DataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (DataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->iiDblBufferId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true => 
    GetObject (::IpcMemoryG, this->iiBufferId)->dtBuffer := DataIn
    : return := OK 
]] 
******************************************************/


template <class tDataType> class tIpcDblBufferAccessor : public tIpcObjectAccessor<tDataType> {
  public:
    tIpcDblBufferAccessor (const tIpcDblBufferId &iiId);
    virtual ~tIpcDblBufferAccessor ();
    virtual ERROR_CODE Get (tDataType &DataOut);
    virtual ERROR_CODE Put (const tDataType &DataIn);
    virtual BOOL IsOk () const;
  protected:
    volatile tIpcDblBuffer<tDataType> *vDblBuffer;
  };
  
/***********************************
Template Method definitions
************************************/  
  
/**************************************************************
Representation Mapping

tIpcDblBufferAccessor  (modeled as a simultaneous read-write atom)   

StateBox 					    Black Box

invalid_pointer (::pThisRteG)                       ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pLocalMemory)     ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pSharedMemory)    ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pTokenClient)     ::IpcMemory.bInit := FALSE
::pThisRteG->Ipc.pSharedMemory::GlobalInit(..)      ::IpcMemory.bInit := FALSE
  not executed    

this->vDblBuffer                   this->iiDblBufferId
invalid_pointer (this->vBuffer)    this->bValidObject == FALSE || DoesObjectExist (..) == FALSE 
this->vDblBuffer->dtBuffer1 or     GetObject (...)->dtBuffer 
this->vDblluffer->dtBuffer2          
this->vDblBuffer->bsWhich ==       GetObject (...)->bEmpty == TRUE
  BS_EMPTY_BUFFER 
this->vDblBuffer->bsWhich !=       GetObject (...)->bEmpty == FALSE 
  BS_EMPTY_BUFFER
  
this->ecInitError		   this->ecInitError
this->bInit		           this->bInit


		             
     

******************************************************************************
			tIpcDblBufferAccessor Method Definitions
******************************************************************************/

/****
[[U tIpcDblBufferAccessor::tIpcDblBufferAccessor(iiId) ]]
****/
template <class tDataType> tIpcDblBufferAccessor<tDataType>::
  tIpcDblBufferAccessor(const tIpcDblBufferId &iiId) :
  
   /****
   [[U tIpcObjectAccessor::tIpcObjectAccessor(void) ]]
   ****/
   tIpcObjectAccessor<tDataType> () 
{
    ERROR_CODE ec;

    /****
    [[U this->vDblBuffer := tIpcDblBuffer::operator_new (appropriate_size,iiId,this->ecInitError) ]]

    [[U this->vDblBuffer->tIpcDblBuffer::tIpcDblBuffer(void)]]
    ****/    
    this->vDblBuffer = new (iiId,ec) tIpcDblBuffer<tDataType>;
    this->SetInitError(ec);
}

/****
[[U tIpcDblBufferAccessor::~tIpcDblBufferAccessor(void) ]]
****/
template <class tDataType> tIpcDblBufferAccessor<tDataType>::
  ~tIpcDblBufferAccessor() {

   /****
   [[U tIpcObjectAccessor::~tIpcObjectAccessor(void) ]]
   ****/
  
  }


/****
[[U tIpcObjectAccessor::IsOk(void) ]]
****/
template <class tDataType> BOOL tIpcDblBufferAccessor<tDataType>::IsOk () const {
  return (this->vDblBuffer != NULL);
}

/****
[[U tIpcDblBufferAccessor::Get(DataOut) ]]
****/
template <class tDataType> ERROR_CODE tIpcDblBufferAccessor<tDataType>::
  Get(tDataType &DataOut) {

   if (tBaseObject::IsInit () == TRUE) {

    /****
    [[U return := this->vDblBuffer.tIpcDblBuffer::Get(DataOut) ]]
    ****/
    return this->vDblBuffer->Get(DataOut);
  }
  
  else return S_ipc_OBJECT_NOT_ENABLED;
}


/****
[[U tIpcDblBufferAccessor::Put(DataIn) ]]
****/
template <class tDataType> ERROR_CODE tIpcDblBufferAccessor<tDataType>::
  Put(const tDataType &DataIn) {
  
   
   if (tBaseObject::IsInit () == TRUE) {
    /****
    [[U this->vDlbBuffer.tIpcDblBuffer::Put(DataIn) ]]
    ****/
    return this->vDblBuffer->Put(DataIn);

  }
  
  else return S_ipc_OBJECT_NOT_ENABLED;
}



#endif INC_ipcDblBufferAccessorh
