/* $RCSfile: ipcFifo.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcFifo.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $ */

#ifndef INC_ipcFifoh
#define INC_ipcFifoh

#include "isit.h"
#include "ipcid.h"
#include "ipc.h"
#include "ipcError.h"



/******************************************************************
Black Box Types

Queue<tDataType> - first in first out structure with infinite capacity

Queue.Length - number of tDataType elements in queue
Queue.Front - current front element of queue
Queue.Back - current back element of queue
Queue[i] - ith element of queue

________________________________________________________________________
tIpcFifo<tDataType>

int iFifoDepth
Queue<tDataType> dtQueue

________________________________________________________________________

Specification Functions:

CanAlocateIpcObject (..) - see ipcObjectAccessor.h
NewObjectSet (...)       - see ipcObjectAccessor.h
AllocationSize (...)     - see ipcObjectAccessor.h
________________________________________________________________________

[[D tIpcFifo::tIpcFifo (int iFifoDepthToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
     this->iDepth := iFifoDepthToUse
    : this->dtQueue := Empty Queue
]]

[[D tIpcFifo::Get (tDataType &DataOut) ]
[ invalid_pointer (this) || 
  invalid_reference (DataOut) ||
  simultaneous get or GetSeveral on this => UNDEFINED
| this->iFree == this->iDepth => return := S_ipc_FIFO_EMPTY
| true =>
    DataOut := this->dtQueue.Front
    : this->dtQueue := this->dtQueue with this->dtQueue.Front removed
    : return := OK
]]
 
[[D tIpcFifo::GetSeveral (tDataType *pDataOutArray, int iNumElements) ]
[ invalid_pointer (this) || 
  invalid_pointer (pDataOutArray) ||
  simultaneous Get or GetSeveral on this => UNDEFINED
| this->dtQueue.Length < iNumElements => return := S_ipc_TOO_FEW_ELEMENTS_IN_FIFOS
| true =>
    pDataOutArray := first iNumElements elements from front of this->dtQueue
    : this->dtQueue := this->dtQueue with first iNumElements element removed
    : return := OK 
]]

   
[[D tIpcFifo::Put (const tDataType &DataIn) ]
[ invalid_pointer (this) ||
  invalid_reference (DataIn) || 
  simultaneous Put or PutSeveral or Flush on this => UNDEFINED
| this->dtQueue.Length == this->iFifoDepth => return := S_ipc_FIFO_FULL
| true => 
    this.dtQueue.Back := DataIn
    : this.dtQueue := this->dtQueue with DataIn appended to back
    : return := OK 
]]

[[D tIpcFifo::PutSeveral (const tDataType *pDataInArray, int iNumElements) ]
[ invalid_pointer (this) || 
  invalid_pointer (pDataInArray) ||
  simultaneous Put or PutSeveral or Flush on this => UNDEFINED
| this->iFifoDepth - this->dtQueue.Length < iNumElements => 
    return := S_ipc_TOO_MANY_ELEMENTS_IN_FIFO
| true => 
    this->dtQueue := this->dtQueue with iNumElements elements of pDataInArray 
      appended to back
    : return := OK 
]]


[[D tIpcFifo::Flush (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->dtQueue := Empty queue 
    : return := OK
]]

[[D tIpcFifo::IsEmpty (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->dtQueue.Length == 0 => return := TRUE
| true => return := FALSE
]]

[[D tIpcFifo::IsFull (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->dtQueue.Length == this->iDepth => return := TRUE
| true => return := FALSE
]]

[[D tIpcFifo::ElementsFree (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iDepth - this->dtQueue.Length
]]


[[D tIpcFifo::operator_new (const tIpcFifoId &iiId,
                        ERROR_CODE &ecErrCode) ]
[ invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE => 
    ecErrCode := appropriate error code
    : console := console + appropriate messages
    : return := NULL
| true =>
    ecErrCode:=OK
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
    : return := address of appropriate tIpcFifo
]]

 
******************************************/

template <class tDataType> class tIpcFifo {
  public:
    tIpcFifo (int iFifoDepthToUse);
    ERROR_CODE Get (tDataType& DataOut) volatile;
    ERROR_CODE GetSeveral (tDataType* pDataOutArray, int iNumElements) volatile;
    ERROR_CODE Put (const tDataType& DataIn) volatile;
    ERROR_CODE PutSeveral (const tDataType* pDataInArray, int iNumElements) volatile;
    ERROR_CODE Peek (tDataType& DataOut) volatile;
    ERROR_CODE PeekSeveral (tDataType* pDataOutArray, int iNumElements) volatile;
    ERROR_CODE Flush (void) volatile;
    BOOL IsEmpty (void) volatile const;
    BOOL IsFull (void) volatile const;
    int  ElementsFree () volatile const;
    int  Length () volatile const;
    void *operator new (size_t szSize, const tIpcFifoId &iiId,
                        ERROR_CODE &ecErrCode);

  protected:
    int iDepth, iFront, iBack;
    tDataType dtQueueStart;
};  


  
/******************************
Template Method Definitions
******************************/
  
/******************************************************************************
			tIpcFifo Method Definitions
******************************************************************************/

/****
[[U tIpcFifo::tIpcFifo(iFifoDepthToUse) ]]
****/  
template <class tDataType> tIpcFifo<tDataType>::tIpcFifo(int iFifoDepthToUse) 
{
  this->iFront = iFifoDepthToUse - 1;
  this->iBack = 0;
  this->iDepth = iFifoDepthToUse;
}

/****
[[U tIpcFifo::Get(DataOut) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  Get(tDataType &DataOut) volatile {
    if (this->Length () == 0) return S_ipc_FIFO_EMPTY;
    DataOut = (&this->dtQueueStart)[(this->iFront+1)% this->iDepth];
    this->iFront = (this->iFront + 1) % this->iDepth;
    return OK;
}

/****
[[U tIpcFifo::GetSeveral(pDataOutArray,iNumElements) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  GetSeveral (tDataType *pDataOutArray, int iNumElements) volatile {

    int i, iElementsInFirstBlock, iElementsRemaining;
    size_t szFirstBlock, szLastBlock;
    tDataType *pQueueStart;
    char *pCopyStart;
      
    pQueueStart = (tDataType *) &(this->dtQueueStart);
    pCopyStart = (char *) (pQueueStart + (this->iFront +1) % this->iDepth);


    if (this->Length () < iNumElements) {
      return S_ipc_TOO_FEW_ELEMENTS_IN_FIFO;
    }
    
    iElementsInFirstBlock = this->iDepth - (this->iFront+1) % this->iDepth;

    // data doesnt wrap around queue
    if (iNumElements <= iElementsInFirstBlock) { 

      memcpy (pDataOutArray,pCopyStart,sizeof (tDataType) * iNumElements);
    }

    // data wraps around queue
    else { 

      iElementsRemaining = iNumElements - iElementsInFirstBlock;
      szFirstBlock = sizeof(tDataType) * iElementsInFirstBlock;
      szLastBlock = sizeof (tDataType) * iElementsRemaining;
    
      memcpy (pDataOutArray,pCopyStart,szFirstBlock);
    
      pDataOutArray += szFirstBlock; // position next write point

      memcpy (pDataOutArray,pQueueStart, szLastBlock);
 
    }
       
    this->iFront = (this->iFront + iNumElements) % this->iDepth;
    
    return OK;
}
   

   
/****
[[U tIpcFifo::Put(DataIn) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  Put(const tDataType &DataIn) volatile {
    if (this->ElementsFree () == 0) return S_ipc_FIFO_FULL;
    (&this->dtQueueStart)[this->iBack] = DataIn;
    this->iBack = (this->iBack + 1) % this->iDepth;
    return OK;
}
  
   
/****
[[U tIpcFifo::PutSeveral(pDataInArray, iNumElements) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  PutSeveral (const tDataType *pDataInArray, int iNumElements) volatile {

    int i, iElementsInFirstBlock, iElementsRemaining;
    size_t szFirstBlock, szLastBlock;
    tDataType *pQueueStart;
    char *pCopyStart;
      
    if (this->ElementsFree () < iNumElements) 
      return S_ipc_TOO_MANY_ELEMENTS_IN_FIFO;
    
    pQueueStart = (tDataType *) &(this->dtQueueStart);
    pCopyStart = (char *) (pQueueStart + iBack);

    iElementsInFirstBlock = this->iDepth - this->iBack;
 
    if (iNumElements <= iElementsInFirstBlock) {
   
      memcpy (pCopyStart,pDataInArray,sizeof (tDataType) * iNumElements);
    }

    else {

      iElementsRemaining = iNumElements - iElementsInFirstBlock;
      szFirstBlock = sizeof(tDataType) * iElementsInFirstBlock;
      szLastBlock = sizeof(tDataType) * iElementsRemaining;

      memcpy (pCopyStart,pDataInArray, szFirstBlock);
    
      pDataInArray += szFirstBlock;

      memcpy (pQueueStart,pDataInArray, szLastBlock);

    }
            
    this->iBack = (this->iBack + iNumElements) % this->iDepth;
    
    return OK;
}
 

/****
[[U tIpcFifo::Peek (DataOut) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  Peek (tDataType& DataOut) volatile {
    if (this->Length () == 0) return S_ipc_FIFO_EMPTY;
    DataOut = (&this->dtQueueStart)[(this->iFront+1)% this->iDepth];
    return OK;
}


/****
[[U tIpcFifo::PeekSeveral(pDataOutArray,iNumElements) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  PeekSeveral (tDataType *pDataOutArray, int iNumElements) volatile {

    int i, iElementsInFirstBlock, iElementsRemaining;
    size_t szFirstBlock, szLastBlock;
    tDataType *pQueueStart;
    char *pCopyStart;
      
    pQueueStart = (tDataType *) &(this->dtQueueStart);
    pCopyStart = (char *) (pQueueStart + (this->iFront +1) % this->iDepth);


    if (this->Length () < iNumElements) {
      return S_ipc_TOO_FEW_ELEMENTS_IN_FIFO;
    }
    
    iElementsInFirstBlock = this->iDepth - (this->iFront+1) % this->iDepth;

    // data doesnt wrap around queue
    if (iNumElements <= iElementsInFirstBlock) { 

      memcpy (pDataOutArray,pCopyStart,sizeof (tDataType) * iNumElements);
    }

    // data wraps around queue
    else { 

      iElementsRemaining = iNumElements - iElementsInFirstBlock;
      szFirstBlock = sizeof(tDataType) * iElementsInFirstBlock;
      szLastBlock = sizeof (tDataType) * iElementsRemaining;
    
      memcpy (pDataOutArray,pCopyStart,szFirstBlock);
    
      pDataOutArray += szFirstBlock; // position next write point

      memcpy (pDataOutArray,pQueueStart, szLastBlock);
 
    }
           
    return OK;
}
   


  
/****
[[U tIpcFifo::Flush(void) ]]
****/  
template <class tDataType> ERROR_CODE tIpcFifo<tDataType>::
  Flush() volatile {
    iFront = this->iDepth-1;
    this->iBack = 0;
    return OK;
}

/****
[[U tIpcFifo::IsEmpty(void) ]]
****/  
template <class tDataType> BOOL tIpcFifo<tDataType>::IsEmpty () volatile const {
  return ( this->Length () == 0);
}

/****
[[U tIpcFifo::IsFull(void) ]]
****/  
template <class tDataType> BOOL tIpcFifo<tDataType>::IsFull ()  volatile const {
  return ( this->ElementsFree () == 0);
}

/****
[[U tIpcFifo::ElementsFree (void) ]]
****/  
template <class tDataType> int tIpcFifo<tDataType>::ElementsFree () volatile const 
{
  int iFree = this->iFront - this->iBack;

  if (iFree < 0) iFree = iFree + this->iDepth;

  return iFree;
}


/****
[[U tIpcFifo::Length (void) ]]
****/ 
template <class tDataType> int tIpcFifo<tDataType>::Length () volatile const
{
  int iLength = this->iBack - this->iFront;

  if (iLength < 0) iLength = iLength + this->iDepth;
  
  return iLength -1;
}



/****
[[U tIpcFifo::operator_new(szSize,iiId, ecErrCode) ]]
****/  
template <class tDataType> void *tIpcFifo<tDataType>::
  operator new (size_t szSize,const tIpcFifoId &iiId, 
                ERROR_CODE &ecErrCode) {

    TraceMark (S_object_CREATING_FIFO);

    /****
    [[U return := (::pThisRteG)->pIpc.tIpc::Allocate (iiId,szSize+sizeof(tDataType)*(iiId.Depth()-1),
                             ecErrCode) ]]
    ****/
    return GetIpc().Allocate(iiId,szSize+sizeof(tDataType)*(iiId.Depth()-1),
                             ecErrCode);
}


 
  
#endif INC_ipcFifoh
