/* $RCSfile: ipcIndirectionDblBufferAccessor.h,v $ - Header file for class tIpcIndirectionDblBufferAccessor */

/*
$Header: /home/chara/cvsroot/ople/sys/include/ipcIndirectionDblBufferAccessor.h,v 1.1.1.1 1998/06/11 21:36:12 chara Exp $
*/

#ifndef INC_ipcIndirectionDblBufferAccessorh
#define INC_ipcIndirectionDblBufferAccessorh

#include "ipcObjectAccessor.h"
#include "ipcMemPointer.h"

/** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
*
* Header file for RICST.  This file define the following classes:
*
* tIpcIndirectionDblBufferAccessor
*
** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/

// Foward declaration
class tBuffer;
class tIpcIndirectionDblBuffer;

/*****************************************************************************************
class tIpcIndirectionDblBufferAccessor

DESCRIPTION:
   tIpcIndirectionDblBufferAccessor controls access to the tIpcIndirectionDblBuffer.
tIpcIndirectionDblBuffer is a class for passing objects derived from tBuffer accross IPC.
  
Black Box State
---------------
BOOL bEmpty - TRUE if buffer has never been written to FALSE otherwise
szBufferSize - # of bytes stored in the buffer
BYTE[szBufferSize] Buffer - data array storage for indirection buffer

METHODS:
  tIpcIndirectionDblBufferAccessor::tIpcIndirectionDblBufferAccessor (tIpcIndirectionDblBufferId &iiId);
     Contructor for tIpcIndirectionDblBufferAccessor.  Constructs Buffer to store iiId.szBufferSize 
     bytes in a simultaneous read-write buffer.

[[D tIpcIndirectionDblBufferAccessor::tIpcIndirectionDblBufferAccessor (
      const tIpcIndirectionDblBufferId &iiId) ]
[ invalid_pointer (this) || 
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG,iiId.iiBuffer1Id) == FALSE ||
  CanAllocateIpcObject (::IpcMemoryG,iiId.iiBuffer2Id) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiIndirectionDblBufferId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiIndirecctionDblBufferId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId) U 
      NewObjectSet (::IpcMemoryG, iiId.iiBuffer1Id) U 
      NewObjectSet (::IpcMemoryG, iiId.iiBuffer2Id)
]]

  tIpcIndirectionDblBufferAccessor::~tIpcIndirectionDblBufferAccessor ();
     Destructor for tIpcIndirectionDblBufferAccessor.

  ERROR_CODE tIpcIndirectionDblBufferAccessor::Get (tBuffer &OutBuffer);
     This method gets data from the current read buffer in the IPC memory area and 
     puts it into the buffer class using the tBuffer method Put.  Copies contents 
     of buffer into the DataOut.Buffer if size of buffers are the same.  If buffer 
     is empty or the buffer sizes are inconsistent, Get returns an error code.

[[D tIpcIndirectionDblBufferAccessor::Get (tBuffer& DataOut) ]
[ invalid_pointer (this) || 
  invalid_reference (DataOut) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiIndirectionDblBufferId) == FALSE => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| this->bEmpty == TRUE => return := S_ipc_BUFFER_EMPTY
| this->szBufferSize != DataOut.szBufferSize => return := appropriate error code
| true =>
    DataOut.Buffer := GetObject (::IpcMemoryG, this->iiIndirectionDblBufferId)->Buffer
    : return := OK
]]

  ERROR_CODE tIpcIndirectionDblBufferAccessor::Put (const tBuffer &InBuffer);
     This method puts data into the IPC memory area that is not current set for read
     and then sets the new data to be the current read buffer.  Copies contents of 
     DataOut.Buffer into the buffer if size of buffers are the same.  If buffer sizes 
     are inconsistent, Put returns an error code.

[[D tIpcIndirectionDblBufferAccessor::Put (const tBuffer &DataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (DataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiIndirectionDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->iiIndirectionDblBufferId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| this->szBufferSize != DataIn.szBufferSize => return := appropriate error code
| true => 
    GetObject (::IpcMemoryG, this->iiIndirectionDblBufferId)->dtBuffer := DataIn.Buffer
    : return := OK 
]] 
 
*****************************************************************************************/

class tIpcIndirectionDblBufferAccessor : public tIpcObjectAccessor <tIpcMemoryBytePointer> {
  public:
    tIpcIndirectionDblBufferAccessor (const tIpcIndirectionDblBufferId &iiId);
    ~tIpcIndirectionDblBufferAccessor ();
    ERROR_CODE Get (tBuffer&) const;
    ERROR_CODE Put (const tBuffer&);
    BOOL IsOk () const;
  protected:
    ERROR_CODE Get (tIpcMemoryBytePointer& DataOut); 
    ERROR_CODE Put (const tIpcMemoryBytePointer& DataIn); 
    volatile tIpcIndirectionDblBuffer *vIndBuffer;
};

#endif
