/* $RCSfile: ipcMem.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcMem.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ */

/*
modification history
--------------------
01a,2Jun97,rlj    written
*/

#ifndef INC_ipcMemh
#define INC_ipcMemh
#include "isit.h"
#include "baseObject.h"

#define S_ipc_MEMORY_ALREADY_INITIALIZED 1
#define S_ipc_MEMORY_ALREADY_SHUTDOWN 2

/******************************************************************************

tIpcMemory Black Box State 


BOOL               bInit
set of tMemObject  ObjectSet
size_t             szHeapFree

-----------------------------------------------------------------------------

System Black Box Stimuli

[[D tIpcMemory::tIpcMemory () ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->bInit := TRUE
    : this->szHeapFree := appropriate size
    : this->ObjectSet := NULL SET
]]
                        
[[D tIpcMemory::tIpcMemory () ]
[ invalid_pointer (this) => UNDEFINED
| true =>  *this := anything
]]
   
[[D tIpcMemory::Restart () ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : this->szHeapFree := #bytes of memory free in the system
    : this->ObjectSet := NULL SETthis 
| true => 
    this->bInit := TRUE 
    : this->szHeapFree := #bytes of memory free in the system
    : this->ObjectSet := NULL SET
]]

[[D tIpcMemory::Shutdown () ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE => 
    console := console + appropriate warning messages
| true => 
    this->bInit := FALSE 
]]


*****************************************************************************/


/****************************************************************************
Black Box View of Local and Shared memory (from point of view of tIpc)

*****************************************************************************
Class: tMemory 
*****************************************************************************

Black Box State
___________________________

tMemory / tLocalMemory 

BOOL               bInit
set of tIpcObject  ObjectSet
size_t             szHeapFree


tSharedMemory

BOOL               bInit
set of tIpcObject  ObjectSet
size_t             szHeapFree
static BOOL        bStaticInit  
____________________________________________________________________________
Spec Functions

BOOL CanAllocateObject (set of tIpcObject ObjSet,IPC_NAME inName,
                        IPC_TYPE itType, size_t szSize)
[ TE. a tIpcObject x, in ObjSet st. 
    x.iiObjectId.inObjectName == inName && 
    ! ( (x.iiObjectId.itObjectType == itType) && 
      ( x.iiObjectId.szObjectSize == szSize) ) ||
  !TE an ipc object x in ObjSet st. 
    x.iiObjectId.inObjectName == inName &&
    pMemory->szHeapFree < szSize => return := FALSE
| true => return := TRUE
]

set of tIpcObject NewObjectSet (set of tIpcObject ObjSet,
              MEM_TYPE mtType, IPC_NAME inName, itType)
[ TE. a tIpcObject x in ObjSet st. x.iiId.inName == inName => 
    return := NULL SET
| true => return := { tIpcObject with tIpcObject.iiId = iiId with
                      inObjectName:=inName, itObjectType:=itType , and
                      mtMemType := mtMemType }
]
 
size_t AllocationSize (set of tIpcObject ObjSet, IPC_NAME inName,
		       size_t szSize) 
[ TE. a tIpcObject x in ObjSet st. x.iiObjectId.inName == inName => 
    return := 0
| true => return := szSize
]

____________________________________________________________________________
Stimuli

[[D tMemory::tMemory (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->bInit := TRUE
    : this->ObjectSet := NullSet
]]

[[D tMemory::~tMemory (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]  

[[D tMemory::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : this->ObjectSet := anything
    : return := OK
| true =>
    this->bInit := TRUE
    : this->ObjectSet := anything
    : return := OK
]]
  
[[D tMemory::Shutdown (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return OK
| true => 
    this->bInit := FALSE
    : return := OK 
]]
    
[[D tMemory::Allocate (IPC_NAME inName, IPC_TYPE itType, 
    		            size_t szSize,ERROR_CODE &recErrCode) ]
[ invalid_pointer (this) ||
  this is a tSharedMemory && this->bStaticInit == FALSE => UNDEFINED
| CanAllocateObject (this->ObjectSet,inName,itType,szSize) == FALSE =>
    console := console + appropriate error messages
    : *pecErrCode := appropriate error code
    : return := NULL
| true =>
    *pecErrCode := OK
    : this->ObjectSet := this->ObjectSet U 
      PossibleNewObject (this->ObjectSet,appropriate memory type,
                         inName,itType)
    : this->szHeapFree := this->szHeapFree - 
      AllocationSize (this->ObjectSet,inName,szSize)
    : return := address of appropriate tIpcObject 
]]




*******************************************************************************/



class tMemory : public tBaseObject {
  public :
    tMemory ();
    virtual ~tMemory ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual void *Allocate (IPC_NAME inName, IPC_TYPE itType, 
    		            size_t szSize,ERROR_CODE &recErrCode) = 0;
};
   


#endif 
