/* $RCSfile: ipcMemPointer.h,v $*/
/* CM Info:   

$Header: /home/chara/cvsroot/ople/sys/include/ipcMemPointer.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $

*/
/*Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
1a 07/21/97 rlj  - Created

*/

#ifndef INC_ipcMemPointerh
#define INC_ipcMemPointerh


/********
Ipc memory pointers can be used exactly like C++ pointers but unlike C++ 
pointers, can be stored in shared memory and used by any CPU as long a the CPU
using the Ipc memory pointer has a properly defined ::IpcMemoryBaseAddress

Ipc memory pointers can be used to reference both local and shared memory 
locations, since the offset is a signed number.  If CPU local memory addresses
are 0x00000 to 0x0FFFF and Shared memory is mapped from 0x10000 to 0xFFFFF.  
IpcMemoryPointers to local memory would be stored as negative offsets from
0x10000 and IpcMemoryPointers to shared memory would be stored as positive 
offsets.

::IpcMemoryBaseAddress is a static member of all shared memory pointers, and 
is used as the base address to access shared memory.  Ipc memory pointers, 
are in fact integer offsets, but the methods of shared memory pointers allow
the user to treat them as true C++ pointers. 

There are two types of shared memory pointers, byte, and word. The byte version
points to 8 bit data, the word version points to SHMEM_WORD type data.

Usage:
  
  BYTE *pYourPointer; // assume is a normal pointer to shared mem location
  tIpcMemoryBytePointer pMyPointer; // uninitialized pointer
  tIpcMemoryBytePointer pMyPointer (0);// pointer to ::IpcMemoryBassAddress
  tIpcMemoryBytePointer pMyPointer (pYourPointer) // converts normal pointer to 
                    //shared memory pointer
  ++pMyPointer; // increments pointer (only prefix versions can be overloaded)
  --pMyPointer; // decrements pointer 
  pMyPointer = pYourPointer; // same as constructor version
  pMyPointer = offset; // same as constructor version
  pYourPointer = pMyPointer; // converts from shared memory pointer to normal pointer
  pMyPointer += i // increments by i
  pMyPointer -= i; // decrements by i
  pMyPointer[i]; // indexes into pMyPointer i elements
  i = pMyPointer - pMyOtherPointer; // number of elements between pointers
  *pMyPointer; // dereference pointer
  etc...
  
  (Same operators for tIpcMemoryWordPointer)
  

Global Constants

::IpcMemoryBaseAddress - pointer to the base IPC controlled shared memory

BlackBox State
------------------------------------------
this->vByte - address of a byte in IPC contolled shared memory


[[D tIpcMemoryBytePointer::tIpcMemoryBytePointer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->vByte := anything
]]

[[D  tIpcMemoryBytePointer::tIpcMemoryBytePointer (volatile BYTE* vBytePtrToUse) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte) ||
  invalid_pointer (vBytePtrIn) => UNDEFINED
| true => this->vByte := vBytePtrToUse
]]

[[D  tIpcMemoryBytePointer::tIpcMemoryBytePointer (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte) => UNDEFINED
| true => this->vByte := ::vIpcMemoryBaseAddress + iOffset
]]

[[D tIpcMemoryBytePointer::~tIpcMemoryBytePointer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->vByte := anything
]]

[[ tIpcMemoryBytePointer::operator* (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte) => UNDEFINED
| true => return *this->vByte
]]

[[ tIpcMemoryBytePointer::operator[] (int iIndex) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte + iIndex) => UNDEFINED
| true => return this->vByte[iIndex]
]]

[[ tIpcMemoryBytePointer::operator++ (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte + 1) => UNDEFINED
| true => this->vByte := this->vByte + 1
]]

[[ tIpcMemoryBytePointer::operator-- (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte - 1) => UNDEFINED
| true => this->vByte := this->vByte - 1
]]

[[ tIpcMemoryBytePointer::operator+ (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte + iOffset) => UNDEFINED
| true => return := tIpcMemoryBytePointer with 
       vByte := this->vByte + iOffset
]]

[[ tIpcMemoryBytePointer::operator+= (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte + iOffset) => UNDEFINED
| true => this->vByte := this->vByte + iOffset
]]


[[ tIpcMemoryBytePointer::operator- (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte - iOffset) => UNDEFINED
| true => return := tIpcMemoryBytePointer with 
       vByte := this->vByte - iOffset
]]

[[ tIpcMemoryBytePointer::operator-= (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vByte - iOffset) => UNDEFINED
| true => return := this->vByte := this->vByte - iOffset
]]

[[ tIpcMemoryBytePointer::operator- (const tIpcMemoryBytePointer& PtrA,
               const tIpcMemoryBytePointer& PtrB) ]
[ invalid_pointer (PtrA.vByte) ||
  invalid_pointer (PtrB.vByte) => UNDEFINED
| true => return := Ptr.vByte - PtrB.vByte
]]


[[D tIpcMemoryWordPointer::tIpcMemoryWordPointer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->vWord := anything
]]

[[D  tIpcMemoryWordPointer::tIpcMemoryWordPointer (volatile SHMEM_WORD* vWordPtrToUse) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord) ||
  invalid_pointer (vWordPtrIn) => UNDEFINED  
| true => this->vWord = vWordPtrToUse
]]

[[D  tIpcMemoryWordPointer::tIpcMemoryWordPointer (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord) => UNDEFINED
| true => this->vWord = ::vIpcMemoryBaseAddress + iOffset
]]

[[D tIpcMemoryWordPointer::~tIpcMemoryWordPointer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->vByte := anything
]]

[[ tIpcMemoryBytePointer::operator* (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord) => UNDEFINED
| true => return *this->vWord
]]

[[ tIpcMemoryWordPointer::operator[] (int iIndex) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord + iIndex) => UNDEFINED
| true => return this->vWord[iIndex]
]]

[[ tIpcMemoryWordPointer::operator++ (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord + 1) => UNDEFINED
| true => this->vWord := this->vWord + 1
]]

[[ tIpcMemoryWordPointer::operator-- (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord - 1) => UNDEFINED
| true => this->vWord := this->vWord - 1
]]

[[ tIpcMemoryWordPointer::operator+ (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord + iOffset) => UNDEFINED
| true => return := tIpcMemoryWordPointer with 
       this->vWord := this->vWord + iOffset
]]

[[ tIpcMemoryWordPointer::operator+= (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord + iOffset) => UNDEFINED
| true => this->vWord := this->vWord + iOffset
]]


[[ tIpcMemoryBytePointer::operator- (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord - iOffset) => UNDEFINED
| true => return := tIpcMemoryWordPointer with 
       vWord := this->vWord - iOffset
]]

[[ tIpcMemoryWordPointer::operator-= (int iOffset) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->vWord - iOffset) => UNDEFINED
| true => return := this->vWord := this->vWord - iOffset
]]

[[ tIpcMemoryWordPointer::operator- (const tIpcMemoryWordPointer& PtrA,
               const tIpcMemoryWordPointer& PtrB) ]
[ invalid_pointer (PtrA.vWord) ||
  invalid_pointer (PtrB.vWord) => UNDEFINED
| true => return := Ptr.vWord - PtrB.vWord
]]
******/

class tIpcMemoryBytePointer {
  public:
    static void SetBaseAddress (volatile BYTE* vBaseAddress) {
      vIpcMemoryBaseAddress = vBaseAddress; }
    tIpcMemoryBytePointer (void) {};
    tIpcMemoryBytePointer (volatile BYTE* vBytePtrToUse ) {
      iAddressOffset = vBytePtrToUse - vIpcMemoryBaseAddress;}
    tIpcMemoryBytePointer (int iOffset) {
      iAddressOffset = iOffset;}
    ~tIpcMemoryBytePointer (void) {};
    BYTE operator* (void) {
      return *(vIpcMemoryBaseAddress + iAddressOffset); }
    BYTE operator[] (int iIndex) {
       return *(vIpcMemoryBaseAddress + iAddressOffset + iIndex); }
    void operator++ (void) { iAddressOffset++; }
    void operator-- (void) { iAddressOffset--; }
    tIpcMemoryBytePointer operator+ (int iOffset) {
      return tIpcMemoryBytePointer (iAddressOffset+iOffset); }
    operator+= (int iOffset) {
      return iAddressOffset = iAddressOffset+iOffset ; }
    tIpcMemoryBytePointer operator- (int iOffset) {
      return tIpcMemoryBytePointer (iAddressOffset-iOffset); }
    operator-= (int iOffset) {
      return iAddressOffset = iAddressOffset-iOffset ; }
    friend int operator- (
               const tIpcMemoryBytePointer& PtrA,
               const tIpcMemoryBytePointer& PtrB) {
      return PtrA.iAddressOffset - PtrB.iAddressOffset; }
    operator volatile BYTE* (void) { return vIpcMemoryBaseAddress + iAddressOffset; }
    operator BYTE* (void) const volatile { return (BYTE *)(vIpcMemoryBaseAddress + iAddressOffset); }
    
    
  protected:
    static volatile BYTE* vIpcMemoryBaseAddress;
    int iAddressOffset;
};


class tIpcMemoryWordPointer {
  public:
    static void SetBaseAddress (volatile SHMEM_WORD* vBaseAddress) {
      vIpcMemoryBaseAddress = vBaseAddress; }
    tIpcMemoryWordPointer (void) {};
    tIpcMemoryWordPointer (volatile SHMEM_WORD* vWordPtrIn ) {
      iAddressOffset = vWordPtrIn - vIpcMemoryBaseAddress;}
    tIpcMemoryWordPointer (int iOffset) {
      this->iAddressOffset = iOffset;}
    ~tIpcMemoryWordPointer (void) {};
    SHMEM_WORD operator* (void) {
      return *(vIpcMemoryBaseAddress + iAddressOffset);}
    BYTE operator[] (int iIndex) {
       return *(vIpcMemoryBaseAddress + iAddressOffset + iIndex); }
    void operator++ (void) { iAddressOffset++; }
    void operator-- (void) { iAddressOffset--; }
    tIpcMemoryWordPointer operator+ (int iOffset) {
      return tIpcMemoryWordPointer (iAddressOffset+iOffset); }
    tIpcMemoryWordPointer operator- (int iOffset) {
      return tIpcMemoryWordPointer (iAddressOffset-iOffset); }
    friend int operator- (
               const tIpcMemoryWordPointer& PtrA,
               const tIpcMemoryWordPointer& PtrB) {
      return PtrA.iAddressOffset - PtrB.iAddressOffset; }
    operator volatile SHMEM_WORD* (void) { return vIpcMemoryBaseAddress + iAddressOffset; }
    
   protected:
    static volatile SHMEM_WORD* vIpcMemoryBaseAddress;
    int iAddressOffset;
};



#endif INC_ipcMemPointerh
