/* $RCSfile: ipcMultiTypeFifoAccessor.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcMultiTypeFifoAccessor.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ */

#ifndef INC_ipcMultiTypeFifoAccessorh
#define INC_ipcMultiTypeFifoAccessorh

#include <iostream.h>
#include "isit.h"
#include "ipcid.h"
#include "ipcError.h"
#include "ipcFifoAccessor.h"

/****************************************************************************
Black Box State:

---------------------------------------------------------------------------
tIpcMultiTypeFifoAccessor 

BOOL          bInit         - TRUE if FIFO is enabled, FALSE if disabled
ERROR_CODE    ecInitError   - initialization error or OK
tIpcFifoId    iiFifoId      - id of the FIFO object stored in ::IpcMemoryG


------------------------------------------------------------------------------
other variables and types 

tIpcMemory       ::IpcMemoryG  - see Ipc Specification Document
tBuffer          BufferIn      - see buffer.h
queue<tDataType> dtQueue       - see ipcFifoAccessor.h 

------------------------------------------------------------------------------
Specfication Functions

BOOL CanAllocateIpcObject (...)      - see ipcObjectAccessor.h
set of tIpcObject NewObjectSet (...) - see ipcObjectAccessor.h
BOOL DoesObjectExist (...)           - see ipcObjectAccessor.h
tIpcObject* GetObject (...)          - see ipcObjectAccessor.h

------------------------------------------------------------------------------
Stimuli

[[D tIpcMultiTypeFifoAccessor::tIpcMultiTypeFifoAccessor (const tIpcFifoId &iiId) ]
[ invalid_pointer (this) ||
  invalid_reference (iiId) ||  
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE =>
    console := console + appropriate messages
    :this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->bObjectValid := FALSE
    : this->iiFifoId := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiFifoId := iiId
    : ::IpcMemoryG.ObjectSet := 
      ::IpcMemoryG.ObjectSet U NewObjectSet (::IpcMemoryG, iiId ) 
]]

[[D tIpcMultiTypeFifoAccessor::~tIpcMultiTypeFifoAccessor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tIpcMultiTypeFifoAccessor::Get (tTypeTag& TypeTagOut, BYTE* pDataOut) ]
[ invalid_pointer (this) || 
  invalid_reference (TypeTagOut) ||
  invalid_pointer (pDataOut) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG,this->iiFifoId) == FALSE ||
  simultaneous get on GetObject (::IpcMemoryG, this->iiFifoId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 0 => 
    return := S_ipc_FIFO_EMPTY
| true =>
    TypeTagOut := front element of GetObject (::IpcMemoryG, 
        this->iiFifoId)->dtQueue
    : *pDataOut := second element of GetObject (::IpcMemoryG, 
        this->iiFifoId)->dtQueue
    : GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue := 
        GetObject (::IpcMemoryGt, this->iiFifoId)->dtQueue 
        with front and second element removed
    : return := OK 
]]
    
[[D tIpcMultiTypeFifoAccessor::Put (const tTypeTag &TypeTagIn, const BYTE* pDataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (TypeTagIn) ||
  invalid_pointer (pDataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiFifoId) == FALSE ||
  simultaneous put or flush on GetObject (::IpcMemoryG, 
    this->iiFifoId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length + TypeTagIn.szSize +
  sizeof (TypeTagIn) > GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => 
    return := S_ipc_FIFO_FULL
| true => 
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue :=
      GetObject (::IpcMemoryG, 
        this->iiFifoId)->dtQueue with *pDataIn and TypeTagIn appended to back
    : return := OK 
]]
 
[[D tIpcMultiTypeFifoAccessor::Put (const tTypeTag &TypeTagIn, const tBuffer& BufferIn) ]
[ invalid_pointer (this) || 
  invalid_reference (TypeTagIn) ||
  invalid_reference (BufferIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiFifoId) == FALSE ||
  simultaneous put or flush on GetObject (::IpcMemoryG, 
    this->iiFifoId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length + TypeTagIn.szSize +
  sizeof (TypeTagIn) > GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => 
    return := S_ipc_FIFO_FULL
| true => 
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue :=
      GetObject (::IpcMemoryG, 
        this->iiFifoId)->dtQueue with BufferIn.Buffer and TypeTagIn appended to back
    : return := OK 
]] 
 
[[D tIpcMultiTypeFifoAccessor::Flush (void) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiFifoId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, this->iiDataFifoId) ||
  simultaneous flush or put on GetObject (::IpcMemoryG, 
    this->iiFifoId) => UNDEFINED
| this->bInit == FALSE => return := appropriate error code
| true =>
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue := Empty queue
    : return OK
]]

[[D tIpcMultiTypeFifoAccessor::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->bObjectValid 
]]

[[D tIpcMultiTypeFifoAccessor::IsEmpty (void) ]
[ invalid_pointer (this) ||
  DoesObjectExist (::IpcMemoryG, this->iiFifoId) == FALSE => UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 0 => 
    return := TRUE
| true => return := FALSE
]]

[[D tIpcMultiTypeFifoAccessor::IsFull (const tTypeTag& TypeTagIn) ]
[ invalid_pointer (this) ||
  DoesObjectExist (::IpcMemoryG, this->iiFifoId) == FALSE => UNDEFINED
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length == 
  GetObject (::IpcMemoryG, this->iiFifoId)->iDepth => return := TRUE
| true => return := FALSE
]]
**************************************************************/

class tTypeTag;
class tBuffer;

class tIpcMultiTypeFifoAccessor : public tIpcFifoAccessor<BYTE> {
  public:
    tIpcMultiTypeFifoAccessor (const tIpcFifoId &iiId);
    ERROR_CODE Get (tTypeTag& TypeTagOut, BYTE* pDataOut);
    ERROR_CODE Put (const tTypeTag &TypeTagIn, const BYTE* pDataIn);
    ERROR_CODE Put (const tTypeTag &TypeTagIn, const tBuffer& BufferIn);
    BOOL IsFull  (const tTypeTag& TypeTagIn) const;
    BOOL IsEmpty () const;
    
};


#endif INC_ipcMultiTypeFifoAccessorh


