/* $RCSfile: ipcObjectAccessor.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/ipcObjectAccessor.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ */

#ifndef INC_IPC_OBJECT_H
#define INC_IPC_OBJECT_H

#include "isit.h"
#include "ipcid.h"
#include "baseObject.h"
#include "ipcError.h"
#include "ipcUntypedObjectAccessor.h"

/****************************************************************************

tIpcObjectAccessor Black Box State

ERROR_CODE ecInitError
BOOL       bValidObject
tIpcId     iiId       (virtual data)

-----------------------------------------------------------------------------

Types used

tIpcId           (see ipcid.h)
tIpcObject       (see object.h)
tIpcAtom         (see object.h)
tIpcDblBuffer    (see object.h)
tIpcFifo         (see object.h)
tIpcMemory       (see IPC Black Box Spec document)

-------------------------------------------------------------------------
SPECIFICATION FUNCTIONS

BOOL CanAllocateIpcObject (tIpcMemory Memory, tIpcId iiId) :== 
[ iiId.mtMemType not in { MT_LOCAL,MT_SHARED } ||
  Simultaneous allocation of other objects prevent allocation of 
    tIpcObject(iiId) ||
  TE. a tIpcObject x, in pMemory->ObjectSet st. 
    x.iiId.inObjectName == inName && 
      ! ( (x.iiObjectId.itObjectType == iiId.itType) && 
        ( x.iiObjectId.szObjectSize == szSize) ) ||
  !TE an ipc object x in Memory.ObjectSet st. 
     x.iiObjectId.inObjectName == inName &&
    insuficient memory to store object => return := FALSE
| true => return := TRUE
]


The following spec function provides a clever means of only altering
tIpcMemory if the tObject that is being allocated does not already exist.
If the tObject already exists, PossibleNewObject returns a null set

set of tIpcObject NewObjectSet (tIpcMemory Memory, tIpcId iiId) :==
[ TE. a tIpcObject x in Memory.ObjSet st. x.iiObjectId == iiId => return := NULL SET
| true => 
    return := { tIpcObject with tIpcObject.iiObjectId = iiId }
]
  
tIpcObject* GetObject (tIpcMemory Memory, tIpcId iiId) :==
[ !TE. a tIpcObject x in Memory.ObjSet st. x.iiObjectId == iiId => UNDEFINED
| true => return := tIpcObject in Memory.Object st. x.iiObjectId == iiId
]

BOOL DoesObjectExist (tIpcMemory Memory, tIpcId iiId) :==
[ !TE. a tIpcObject x in Memory.ObjSet st. x.iiObjectId == iiId => return := FALSE
| truen => return := TRUE
]]


-----------------------------------------------------------------------------
STIMULI


[[D tIpcObjectAccessor::tIpcObjectAccessor_prototype (const tIpcId& iiId) ]
[ invalid_pointer (this) || 
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->(appropriate tIpcId) := anything
| true =>
    this->ecInitError := OK
    : this->bInt := TRUE
    : this->bObjectValid := TRUE
    : this->(appropriate tIpcId) := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]

[[D tIpcObjectAccessor::tIpcObjectAccessor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tIpcObjectAccessor::~tIpcObjectAccessor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tIpcObjectAccessor::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->bObjectValid
]]

[[D tIpcObjectAccessor::Get (tDataType &DataOut) ]
[ invalid_pointer (this) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->iiObjectId) == FALSE => 
    UNDEFINED
| true => 
    DataOut := GetObject (::IpcMemoryG, 
      this->iiId->iiObjectId)->(appropriate data from tIpcObject)
    : return := OK 
]]

[[D tIpcObjectAccessor::Put (const tDataType &DataIn) ]
[ invalid_pointer (this) ||   
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG,this->iiObjectId) == FALSE => UNDEFINED
| true => 
    GetObject (::IpcMemoryG, 
      this->iiObjectId )->(appropriate data from tIpcObject):= DataIn
    : return := OK 
]]



*******************************************************************************/

template <class tDataType> class tIpcObjectAccessor : 
  public tUntypedIpcObjectAccessor {
  public:
    tIpcObjectAccessor ();
    virtual ~tIpcObjectAccessor ();
    virtual ERROR_CODE Get (tDataType &DataOut) = 0;
    virtual ERROR_CODE Put (const tDataType &DataIn) = 0;
    virtual ERROR_CODE UntypedGet (void* pDataOut, size_t &szDataSize);
  };

/*****************************************************
TEMPLATE METHOD DEFINITIONS
*****************************************************/
 
/**************************************************************************

Representation Mapping

Black Box                            State Box
____________________________________________________________

tIpcMemory		             

::IpcMemoryG                  ::pThisRteG->Ipc
::IpcMemoryG.bInit	      ::pThisRteG->Ipc.pSharedMemory->bStaticInit

::IpcMemoryG.ObjectSet        IpcObjectSet(::pThisRteG->Ipc,iiId.mtMemType)
::IpcMemoryG.szHeapFree       Memory(::pThisRteG->Ipc,iiId.mtMemType)->szHeapFree
                                  
------------------------------------------------------------

StateBox 					    Black Box

invalid_pointer (::pThisRteG)                       ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pLocalMemory)     ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pSharedMemory)    ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pTokenClient)     ::IpcMemory.bInit := FALSE
::pThisRteG->Ipc.pSharedMemory::GlobalInit(..)      ::IpcMemory.bInit := FALSE
  not executed    

invalid_pointer (this->vAtom)      this->bValidObject == FALSE || DoesObjectExist (..) == FALSE 
invalid_pointer (this->vDblBuffer)	   this->bValidObject == FALSE || DoesObjectExist (..) == FALSE
invalid_pointer (this->vFifo)      this->bValidObject == FALSE || DoesObjectExist (..) == FALSE

this->ecInitError		   this->ecInitError
this->bInit		           this->bInit


******************************************************************************
			tIpcObjectAccessor Method Definitions
******************************************************************************/



/****
[[U tIpcObjectAccessor::tIpcObjectAccessor(void) ]]
****/
template <class tDataType> tIpcObjectAccessor<tDataType>::
  tIpcObjectAccessor () : 
  
  /****
  [[U tUntypedIpcObjectAccessor::tUntypedIpcObjectAccessor(void) ]]
  ****/
  tUntypedIpcObjectAccessor() {}


/****
[[U tIpcObjectAccessor::~tIpcObjectAccessor(void) ]]
****/
template <class tDataType> tIpcObjectAccessor<tDataType>::
  ~tIpcObjectAccessor ()  {
  
  /****
  [[U tUntypedIpcObjectAccessor::~tUntypedIpcObjectAccessor(void) ]]
  ****/
}

  


/****
[[U tIpcObjectAccessor::UntypedGet(pDataOut, szDataSize) ]]
****/
template <class tDataType> ERROR_CODE tIpcObjectAccessor<tDataType>::
  UntypedGet (void* pDataOut, size_t &szDataSize ) {

    ERROR_CODE ec;
   
    szDataSize = sizeof(tDataType);
    
    /****
    [[U ec := tIpcObjectAccessor::Get (*pDataOut) ]]
    ****/         
    ec = this->Get((tDataType &) *pDataOut);
    if (ec == S_ipc_BUFFER_EMPTY || ec == S_ipc_FIFO_EMPTY) 
      return ec;
    else if (ec != OK) 
      return Traceback (ec, "tIpcObjectAccessor::UntypedGet()");      
  
    return OK;   
}
 
 

#endif INC_ipcObjectAccessorh
