/* $RCSfile: isit.h,v $ - Type definitions for system-wide stuff */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/isit.h,v 1.4 1998/09/23 23:57:27 phil Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,25Sep96    written
*/
 
/*
DESCRIPTION
ISIT system-wide defines and types.
*/

#ifndef INC_isith
#define INC_isith

/******
* This file is for VxWorks compiles only.
*  This #ifdef wraps around the whole file.  We don't *really* need
*  it, but it's just a precautionary measure in case some UNIX code
*  inadvertently includes it.
******/
#ifdef VXWORKS

/******
* Do NOT include system header files when using the spec tool
******/
#ifndef SPECTOOL

#include <vxWorks.h>
#include <taskLib.h>
#include <int64.h>
#define MAXNAMELEN 64
#include <stdio.h>
#include <sys/types.h>

#endif /* SPECTOOL */

#include "rtModNum.h"
#include "isitExt.h"
#include "trace.h"

#include "tasks.h"


/************************************************
*    
*    SPECIAL GSU #defines
*
*/
//#define DO_NOT_SEND_DATA_FROM_RELAYER
#define DO_NOT_CHECK_LASER_UNKNOWNS
//#define DISABLE_LASER_TEST_MODE
//#define SERVO_TUNING_ON
/***********************************************/


typedef SHMEM_WORD HEARTBEAT;

typedef enum HEALTH { H_NOT_INIT } HEALTH;
typedef enum INIT_TYPE { IT_POWER_ON_INIT } INIT_TYPE;

typedef enum MEM_TYPE {MT_UNKNOWN,MT_LOCAL,MT_SHARED} MEM_TYPE;
typedef enum IPC_TYPE {IPC_UNKNOWN,IPC_ATOM,IPC_DBL_BUFFER,IPC_FIFO,IPC_MULTI_TYPE_FIFO,IPC_INDIRECTION_DBL_BUFFER} IPC_TYPE;
typedef enum TOKEN_FLAG {TF_IDLE,TF_REQUEST,TF_GRANTED,TF_RETURN} TOKEN_FLAG;
 
typedef struct 
{
   int		iServerRte;
} SYS_INIT_INFO;


// If the compiler supports dynamic casting, uncomment the following line
//#define DYNAMIC_CASTING_OK

#define GIZMO0_DL_NUMBER      (-1)
#define GIZMO1_FT_NUMBER      (0)
 


#define S_rte_INSUFFICIENT_MEMORY          (M_rte | 0x1)
#define S_initmgr_HB_TIMEOUT               (M_initmgr | 0x1)
#define S_initmgr_CPU_NUMBER_OUT_OF_RANGE  (M_initmgr | 0x2)
#define S_initmgr_RTE_NUMBER_OUT_OF_RANGE  (M_initmgr | 0x3)
#define S_general_NOT_IMPLEMENTED          (M_general | 0x1)


/* Error traceback prototype.  Changed from inline because of a gnu compiler bug
*  the combination of inline and variable arg list caused the compiler to create
*  an insufficient amount of space for function
*/
//ERROR_CODE Traceback(ERROR_CODE ec, char *sSubName, ...);


// Why is DL-specific class defined in isit.h?
class tDlTarget : public tDlTargetBase 
{
   public:
      tDlTarget(void);
      tDlTarget(double dInitPos, double dInitRate, TIME tInitTime);
};

/**********************************************************
*********************** Global macros *********************
***********************************************************
*/

/**********************************************************
*
   [[D ::MAX(tNumber a, tNumber b)]
   [ a<b => return := b
   | true => return := a
   ]]
*
   [[D ::MIN(tNumber a, tNumber b)]
   [ a<b => return := a
   | true => return := b
   ]]
*
*****/
#define MAX(a,b) max(a,b) 
#define MIN(a,b) min(a,b)

// *** NOTE!! FYI - vxWorks.h defines max(a,b) and min(a,b) already...  BEH


/**********************************************************
*
   [[D ::TIME_TO_VXWORKS_TICKS(TIME time)]
   [ true =>
     return := time TIME units, converted to vxWorks ticks 
   ]]
*****/
#define TIME_TO_VXWORKS_TICKS(time) \
           ( (int) ((double)time) / TIME_TICKS_PER_SECOND * sysClkRateGet() )

/**********************************************************
*
   [[D ::VXWORKS_TIME_TO_TICKS(tVxTicks Nticks)]
   [ true =>
     return := Nticks VxWorks time units, converted to TIME units 
   ]]
*****/
#define VXWORKS_TICKS_TO_TIME(ticks) \
           ( (int) ((double)ticks) / sysClkRateGet() * TIME_TICKS_PER_SECOND )


/**********************************************************
*************** Global object accessor functions **********
***********************************************************
*/

class tScheduler;
class tErrorStreamSender;
class tRelayStreamSender;
class tRte;
class tIpc;
class tGizmo;

/**********************************************************
*
   [[D ::GetScheduler(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG->Scheduler
   ]]
*****/
tScheduler& GetScheduler (void);

/**********************************************************
*
   [[D ::GetErrorStream(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG->pErrorStream
   ]]
*****/
tErrorStreamSender& GetErrorStream (void);

/**********************************************************
*
   [[D ::GetDataStream(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG->pDataStream
   ]]
*****/
tRelayStreamSender& GetDataStream (void);

/**********************************************************
*
   [[D ::GetStatusStream(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG->pStatusStream
   ]]
*****/
tRelayStreamSender& GetStatusStream (void);


/**********************************************************
*
   [[D ::GetRte(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG
   ]]
*****/
tRte& GetRte(void);

/**********************************************************
*
   [[D ::GetTime(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := ::pThisRteG.tSchedTime
   ]]
*****/
TIME GetTime(void);

/**********************************************************
*
   [[D ::GetIpc(void)]
   [ ::pThisRteG invalid => UNDEFINED
   | true => return := *::pThisRteG.pIpc
   ]]
*****/
tIpc & GetIpc(void);        

/**********************************************************
*
   [[D ::GetGizmoPointer(int iGlobalGizmoIndex) ]
   [ ::pThisRteG invalid => UNDEFINED
   | gizmo #iGlobalGizmoIndex not on this RTE => return := NULL
   | true => return := pointer to gizmo #iGlobalGizmoIndex
   ]]
*****/
tGizmo *GetGizmoPointer(int iGlobalGizmoIndex);


extern "C"
{
  void ShowIpc (void);
}

/*
* End Global object accessor functions
***********************************************************/





/*
#define INITMGR_MINIBUILD
*/

#endif  /* VXWORKS */

#endif  /* INC_isith */





