/* $RCSfile: isitExt.h,v $ -- header files for client */
/*$Header: /home/chara/cvsroot/ople/sys/include/isitExt.h,v 1.4 1998/09/23 23:57:28 phil Exp $
*/

#ifndef INC_isitExth
#define INC_isitExth

#define INCR1
#define MADE_UP
#define INCR3
#define PI (3.1415926535)

typedef int ERROR_CODE;
typedef size_t SIZE;
typedef uint32_t TIME;
typedef unsigned char BYTE;

typedef int32_t SHMEM_WORD;
typedef SHMEM_WORD IPC_NAME;  //Temporary
typedef int IPC_ID;           //Temporary

// Hardware typedefs
typedef unsigned int    INT_RATE;
typedef double          LASER_VALUE;
typedef double          PZT_DOUBLE;
typedef int             PZT_POSITION;
typedef int             PZT_VALUE;
typedef int             VC_VALUE;
typedef int             DIAG_VALUE;
typedef int16_t         MOTOR_VALUE;
typedef int             GIZMO_ID;
typedef int             EDDY_VALUE;     /* phil, what should this be? */


// These should have come from the config manager but we wont change it now
#define DL_SERVO_RATE    (5000)   /* rate of fastest servo task */
#define DL_DATA_RATE     (2)
#define DL_DATA_BUF_SIZE (DL_SERVO_RATE/DL_DATA_RATE)  /* must be integer */
#define DL_DATA_BUF_LENGTH (1)   /* how many tDlDatas are sent by a tDlDataStore */

/* The following macros allow concatenation of entities including macros */
#define CONCAT(a,b) a ## b
#define EXPAND_AND_CONCAT(a,b) CONCAT(a,b)

/* One "tick" of time in the interferometer will be defined
 * as 25 microseconds.  This gives a 40 kHz max interrupt rate.
 */

#define TIME_TICKS_PER_SECOND (40000UL)
#define TIME_TICKS_PER_DAY (3456000000UL)
#define TIME_SECONDS_PER_TICK (1.0/TIME_TICKS_PER_SECOND)


#define WHITE_LIGHT_LAMBDA_IN_M (550.0e-9)
// (663.85 nm / 512)
#define METERS_PER_LV (1.296582e-9) 
#define WHITE_LIGHT_LAMBDA_IN_LASER_UNITS ( (WHITE_LIGHT_LAMBDA_IN_M)/ (METERS_PER_LV) )


typedef enum CMD_TYPE {
    DL_CMD_SELECT_TARGET, DL_CMD_DITHER_ON, DL_CMD_DATA_RECORD_ON_OFF,
    INST_CMD_PING, INST_CMD_SET_OPEN_LOOP_TARGET, INST_CMD_SET_MANUAL_TARGET,
    INST_CMD_SET_TIME,  /* Added for GSU -- PCI -- 6/9/98 */
    DL_CMD_OFF, DL_CMD_FRONT, DL_CMD_BACK, DL_CMD_TRACK, DL_CMD_USER1,
    DL_CMD_USER2, DL_CMD_HOME_CHECK, DL_CMD_HOME_SET, DL_CMD_DITHER_CALIBRATE,
    DL_CMD_DITHER_CHECK, DL_CMD_PING, DL_CMD_CLR_LASER_CTR, DL_CMD_MODULATE,
    DL_CMD_SET_DITHER, /*DL_CMD_CHNG_SERVO_PARAM,*/
    GSUDL_CMD_CHNG_SERVO_PARAM,
    FT_CMD_OFF, FT_CMD_TRACK, FT_CMD_CHNG_PARAM, FT_CMD_PING,
    FT_CMD_GET_IMAGE, FT_CMD_SET_ROI, FT_CMD_CALIBRATE, FT_CMD_SET_MT,
    EM_CMD_OFF, EM_CMD_TRACK, EM_CMD_CHNG_PARAM, EM_CMD_PING,
    ST_CMD_OFF, ST_CMD_TRACK, ST_CMD_PING, 
    ST_CMD_HOME, ST_CMD_CALIBRATE, ST_CMD_SETSS,

    WAP_CMD_OFF, WAP_CMD_SLEW, WAP_CMD_TRACK, WAP_CMD_PING, 
    WAP_CMD_HOME, WAP_CMD_CALIBRATE, WAP_CMD_SETSS, WAP_CMD_MOVE,
    EXTINT_CMD_SET_DATA_RECORDING, /* Added for GSU -- KY -- 7/13/98 */
    DISCONNECT, RESET
} CMD_TYPE;


typedef enum GIZMO_TYPE {
   GT_UNKNOWN=0, GT_INSTRUMENT, GT_DELAY_LINE,
   GT_FRINGE_TRACKER, GT_EXTERNAL_METROLOGY, GT_DL_BB1,
   GT_DL_BB2, GT_DL_GSU, GT_STAR_TRACKER, GT_WAP, 
   GT_NUM_GIZMO_TYPES
} GIZMO_TYPE;

typedef struct {
   TIME                 tTimeStamp;
   ERROR_CODE           ecErrCode;
   int                  iWhichRte;
   int                  iWhichCpu;

} ERROR_INFO;

class tIsitCmd {
public:
   TIME                 tTimeStamp;
   int			iGizmoId;
   int                  iWhichRte;
   int                  iWhichCpu;
   CMD_TYPE             Command;
};

// needed becuase unions dont allow classes to have non default constructors
// and we need a struct for a tDlTarget as a part of target 
// Commands in ISIT_CMD

class tDlTargetBase {
   public:
      double      dPosition;
      double      dRate;
      TIME        tTime;
};

typedef enum {
   BAD, GOOD, VALID, INVALID
} VAL;

/*This structure is intended to represent a point, and is for use IPC objects 
which are parameterized by data type.  The class tPoint cannot be used for this
purpose, because it contains a pointer to the heap.
*/
struct tIpcPoint {
   double dX,dY;
};

#endif /* INC_isitExth */



