/* $RCSfile: laserMetrologySensor.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/laserMetrologySensor.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
$Log: laserMetrologySensor.h,v $
Revision 1.1.1.1  1998/06/11 21:36:13  chara
First import of RICST Code

Revision 1.2.2.3  1998/05/05 15:28:18  rich
Delay line full function

Revision 1.2.2.2  1998/04/30 20:35:55  rbrill
commit before build_5a_008

Revision 1.2.2.1  1998/04/04 00:31:53  phil
commit before build_5a_002

Revision 1.2  1998/03/30 19:35:07  rich
*** empty log message ***

Revision 1.1  1998/03/12 01:05:08  rich
delay line overhaul cont.

*/

#ifndef INC_laserMetrologySensorh
#define INC_laserMetrologySensorh


#include "isit.h"
#include "inputSource.h"
#include "outputDest.h"
#include "outputChannel.h"
#include "channelBank.h"


/******************************************************************
*
* NOTE: A laser metrology sensor measures Optical Path Delay,
* which is twice any physical motion.  Our convention is that
* the PhysicalUnitsPerCount function must return *physical* motion,
* so the scale factor must account for the factor of two (or four,
* for a double-pass system).  The function OpdUnitsPerCount is
* provided in the metrology sensor classes for those cases
* where you want to know OPD rather than physical.
*
* Note that the constructor here takes OPD / metrology wave as
* its input, not physical motion of the target / metrology wave.
*

****************  tLaserMetrologySensor Black Box  *****************

* This is just a tModulatedSensor, except that it also includes
* a conversion factor to convert from physical pathlength to optical
* pathlength.  This is usually a factor of 4 for a double-pass metrology
* system.  It would be a factor of 2 for a single-pass system.
*
* Note that starlight path in a delay line is also a factor of 2
* greater than physical path.  So the conversion from Opd to stellar
* OPD always requires dividing by 2.
*

[[D tLaserMetrologySensor::tLaserMetrologySensor_tditTTd (tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave, int iNumLaserPassesIn, int iLaserSignIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, 
         TIME tServoPeriodIn, TIME tDitherPeriod, double dDitherAmplitude) ]
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) ||
  invalid_channel (DitherPztChannel) ||
  invalid_channelbank (ChannelBank (DitherPztChannel)) ||
  iNumPassesIn < 1 => UNDEFINED
| ChannelBank (DitherPztChannel).dSampOutFreq <= 0 => appropriate action
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->Channel := reference to ChannelIn
    : this->dElectricalToPhysicalScaleFactor :=
      dElectricalToOpdScaleFactorIn / iNumPassesIn
    : this->ModulationParameters.mtModulationType := MT_NONE
    : this->ModulationParameters.dFrequency := 0
    : this->ModulationParameters.dAmplitude := 0
    : this->ModulationParameters.dMeanVal := 0
    : this->ModulationParameters.tSyncTime := 0
    : this->ModulationParameters.dDutyCycle := 0
    : this->bModulationEnabled := FALSE
    : this->dModulationValue := 0.0
    : this->iNumPasses := iNumPassesIn
    : this->dDitherValue := 0
    : this->pDitherBank := ChannelBank (DitherPztChannel)
    : this->pDitherChannel := &DitherPztChannel;
    : this->pWaveformActuator = &WaveformActuatorIn;
    : this->tDlServoTaskPeriod := tDlServoPeriodToUSe;
    : this->dDitherChannelUnits := (DitherPztChannel.CountsPerElectricalUnit / dDitherPhysicalUnitsPerElectricalUnits) / 
          (ChannelIn.CountsPerEletricalUnit / dLaserOpdPerWave) / iNumLaserPassesIn
    : this->bDoingCheckOrCalib := FALSE
    : this->bDithering := FALSE
    : this->sdDitherSamplePeriod := TIME_TICKS_PER_SECOND / 
      ChannelBank (DitherPztChannel).dSampOutFreq
    : this->siMaxDitherBufSize = ChannelBank (DitherPztChannel).MemSize
    : this->siDoubleMaxDitherBufSize = tChannelBank (DitherPztChannel).MemSize * 2
    : this->bCalibrating := FALSE
    : this->bWasDitherOn := FALSE
    : this->bStoreDither := FALSE
    : additional other junk that needs to be abstracted
]]

[[D tLaserMetrologySensor::tLaserMetrologySensor_tditTTdt (
         tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave, int iNumLaserPassesIn, int iLaserSignIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, 
         TIME tServoPeriodIn, TIME tDitherPeriod, double dDitherAmplitude, 
     const tModulationParams& InitialParameters) ]
[ invalid_pointer (this) ||
  invalid_channel (ChannelIn) ||
  invalid_channel (DitherPztChannel) ||
  invalid_channelbank (ChannelBank (DitherPztChannel)) ||
  iNumPassesIn < 1 => UNDEFINED
| ChannelBank (DitherPztChannel).dSampOutFreq <= 0 => appropriate action
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->Channel := reference to ChannelIn
    : this->dElectricalToPhysicalScaleFactor :=
      dElectricalToOpdScaleFactorIn / iNumPassesIn
    : this->bModulationEnabled := FALSE
    : this->dModulationValue := 0.0
    : this->iNumPasses := iNumPassesIn
    : this->dDitherValue := 0
    : this->pDitherBank := ChannelBank (DitherPztChannel)
    : this->pDitherChannel := &DitherPztChannel;
    : this->pWaveformActuator = &WaveformActuatorIn;
    : this->tDlServoTaskPeriod := tDlServoPeriodToUSe;
    : this->dDitherChannelUnits := (DitherPztChannel.CountsPerElectricalUnit / dDitherPhysicalUnitsPerElectricalUnits) / 
          (ChannelIn.CountsPerEletricalUnit / dLaserOpdPerWave) / iNumLaserPassesIn
    : this->bDoingCheckOrCalib := FALSE
    : this->bDithering := FALSE
    : this->sdDitherSamplePeriod := TIME_TICKS_PER_SECOND / 
      ChannelBank (DitherPztChannel).dSampOutFreq
    : this->siMaxDitherBufSize = ChannelBank (DitherPztChannel).MemSize
    : this->siDoubleMaxDitherBufSize = tChannelBank (DitherPztChannel).MemSize * 2
    : this->bCalibrating := FALSE
    : this->bWasDitherOn := FALSE
    : this->bStoreDither := FALSE
    : additional other junk that needs to be abstracted
]]


[[D tLaserMetrologySensor::~tLaserMetrologySensor (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]




[[D tLaserMetrologySensor::UnmodulatedValue (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dValue - this->dModulationValue
]]

[[D tLaserMetrologySensor::GetUnmodulatedUnditherCompensatedValue(void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    return := this->dValue - this->dModulationValue - this->dDitherValue
]]



[[D tLaserMetrologySensor::Value (void) ] = tModulatedSensor::Value ]]

[[D tLaserMetrologySensor::SetValue (double dNewValue) ] = tModulatedSensor::SetValue ]]

[[D tLaserMetrologySensor::SetOffset (double dNewOffset) ] = tModulatedSensor::SetOffset ]]

[[D tLaserMetrologySensor::PhysicalUnitsPerCount (void) ] = tModulatedSensor::PhysicalUnitsPerCount ]]

[[D tLaserMetrologySensor::SetParameters (const tModulationParams& NewModParams) ] =
    tModulation::SetParameters_t ]]

[[D tLaserMetrologySensor::GetParameters (void) ] = tModulation::GetParameters ]]

[[D tLaserMetrologySensor::Enable (void) ] = tModulation::Enable ]]

[[D tLaserMetrologySensor::Disable (void) ] = tModulation::Disable ]]


[[D tLaserMetrologySensor::ModulationValue (void) ] = 
    tModulatedSensor::ModulationValue ]]

    
[[D tLaserMetrologySensor::Reset (void) ] = tModulatedSensor::Reset ]]



[[D tLaserMetrologySensor::CalibrateDither (void) ] = tDlDither::CalibrateDither ]]

[[D tLaserMetrologySensor::CheckDither (void) ] = tDlDither::CheckDither ]]


[[D tLaserMetrologySensor::DitherCalibrationInfo (void) ] = tDlDither::GetCalibInfo ]]

[[D tLaserMetrologySensor::DoingCheckOrCalib (void) ] = 
    tDlDither::DoingCheckOrCalib ]]
    
[[D tLaserMetrologySensor::SetDitherParameters (double dNewAmplitude, TIME tInitPeriod) ] = 
    tDlDither::SetDitherParameters ]]

[[D tLaserMetrologySensor::UnditherCompensatedValue () ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dValue - this->dDitherValue
]]


[[D tLaserMetrologySensor::UnmodulatedUnditherCompensatedValue () ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dValue - this->dModulationValue - this->dDitherValue
]]

[[D tLaserMetrologySensor::NumberOfPasses () ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iNumPasses
]]

[[D tLaserMetrologySensor::DitherPeriod () ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->tDitherPeriod
]]

[[D tLaserMetrologySensor::DitherAmplitude () ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->dDitherAmplitude
]]

    
[[D tLaserMetrologySensor::DisableDither (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->bDitherEnabled := FALSE
    : return := OK
]]

[[D tLaserMetrologySensor::EnableDither (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->bDitherEnabled := TRUE
    : return := OK
]]


[[D tLaserMetrologySensor::IsDitherEnabled (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->bDitherEnabled
]]

[[D tLaserMetrologySensor::Update (const TIME tUpdateTime) ]
[ invalid_pointer (this) ||
  invalid_channel (this->Channel) ||
  dither undefines => UNDEFINED
| this->bDoingCheckOrCalib == TRUE => Dither stuff happens
| this->bEnabled == FALSE && this->bDithering == FALSE =>
    this->dValue := ChannelValue (this->Channel) 
    : return := ChannelValue (this->Channel) 
| this->bEnabled == FALSE && this->bDithering == TRUE =>
    this->dValue := ChannelValue (this->Channel) + DITHER (tUpdateTime)
    : this->dDitherValue := DITHER (tUpdateTime)
    : return := ChannelValue (this->Channel) + DITHER (tUpdateTime)
| this->bEnabled == TRUE && this->bDithering == FALSE =>
    this->dValue := ChannelValue (this->Channel) + MODULATION (this->mpParams, tUpdateTime)
    : this->dModulationValue := MODULATION (this->mpParams, tUpdateTime)
    : return := ChannelValue (this->Channel) + MODULATION (this->mpParams, tUpdateTime)
| true =>
    this->dValue := ChannelValue (this->Channel) + 
      Modulation (this->mpParams, tUpdateTime) + DITHER (tUpdateTime)
    : this->dModulationValue := MODULATION (this->mpParams, tUpdateTime)
    : this->dDitherValue := DITHER (tUpdateTime)
    : return := ChannelValue (this->Channel) + 
      MODULATION (this->mpParams, tUpdateTime) + Dither (tUpdateTime)
]]

****************** protected specification *************************

[[D tLaserMetrologySensor::RawValue (void) ] = tModulatedSensor::RawValue ]]

*/




class tLaserMetrologySensor : public tModulatedSensor {
  public:
    tLaserMetrologySensor (tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave, int iNumLaserPassesIn, int iLaserSignIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, 
         TIME tServoPeriodIn, TIME tDitherPeriod, double dDitherAmplitude);
    tLaserMetrologySensor (tLaserMetrologyChannel &ChannelIn,
         double dLaserOpdPerWave,
         int iLaserSignIn, int iNumLaserPassesIn, 
         tWaveformChannel& DitherPztChannel, 
         double dDitherPhysicalUnitsPerElectricalUnits, TIME tServoPeriodIn, 
         TIME tDitherPeriod, double dDitherAmplitude, 
         const tModulationParams& InitialParameters);
    ERROR_CODE EnableDither ();
    ERROR_CODE DisableDither ();
    BOOL IsDitherEnabled () const;
    ERROR_CODE SetDitherParameters (double dNewAmplitude, TIME tNewPeriod);
    double DitherAmplitude () const;
    TIME DitherPeriod () const;
    ERROR_CODE CalibrateDither ();
    ERROR_CODE CheckDither ();
    BOOL DoingCheckOrCalib () const;
    ERROR_CODE SpoutMeasurementDiags (void) { return this->Dither.SpoutMeasurementDiags ();}    
    tDitherCalibInfo DitherCalibrationInfo () const;
    int NumberOfLaserPasses () const { return this->iNumLaserPasses;}
    
    double LaserOpdToLaserCounts (double dLaserOpd) const {
      return this->PhysicalUnitsToCounts (dLaserOpd); }
      
    double LaserCountsToLaserOpd (double dLaserCounts) const {
      return this->CountsToPhysicalUnits (dLaserCounts);}
      
    
    double UnditherCompensatedValue () const;
    double UnmodulatedUnditherCompensatedValue () const;
    virtual double Update (const TIME tUpdateTime);
    ERROR_CODE ClearCounterOnFiducial () { 
      return this->LaserChannel.ClearCounterOnFiducial ();}
    ERROR_CODE ResetFiducialWait () { return this->LaserChannel.ResetFiducialWait ();}
    ERROR_CODE LoadOnFiducial () { return this->LaserChannel.LoadOnFiducial ();}
    ERROR_CODE CleanUpDither (ERROR_CODE ec) { return this->Dither.CleanUp (ec);}
    ERROR_CODE ClearCounter () { return this->LaserChannel.ClearCounter (); }

    

  private:
    const int iNumLaserPasses;        // Usually 2 or 4.  Default is 4.
    int iLaserSign;
    double dDitherValue;
    tActuator WaveformActuator;
    tDlDither Dither;
    tLaserMetrologyChannel& LaserChannel;
};


#define S_inputSource_LASERMETROLOGYSENSOR_OPD_UNITS_PER_COUNT   (M_inputSource | 0x100D)


#endif INC_laserMetrologySensorh
