/* $RCSfile: memHeader.h,v $  */
/*$Header: /home/chara/cvsroot/ople/sys/include/memHeader.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $*/

/*
modification history
--------------------
01a,16Jun97,rlj    written
*/

#ifndef INC_memHeaderh
#define INC_memHeaderh
 
#include "isit.h"
#include "tList.h"
#include <iostream.h>
/*****************************************************************************
Class: ObjectHeader 
Description: generic header storage for IPC Object name,type,and size
Users:  tLocalMemory, tSharedMemory and decendents.
*****************************************************************************

Black Box State

tObjectHeader 

IPC_NAME inObjectName
IPC_TYPE itObjectType
size_t   szObjectSize
_______________________________
tLocalHeader

IPC_NAME inObjectName
IPC_TYPE itObjectType
size_t   szObjectSize
void *   pObjectLoc
_______________________________
tSharedHeader

IPC_NAME inObjectName
IPC_TYPE itObjectType
size_t   szObjectSize
int      iObjectLoc
______________________________________________________________________________
Stimuli

[[D tObjectHeader::tObjectHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->inObjectName := anything
    : this->itObjectType := anything
    : this->szObjectSize := anything 
]]

[[D tObjectHeader::tObjectHeader_INITsz (IPC_NAME,IPC_TYPE, size_t) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
  this->inObjectName := inName
  : this->itObjectType := itType
  : this->szObjectSize := szSize 
]]

[[D tObjectHeader::~tObjectHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]  

[[D tObjectHeader::IsName (IPC_NAME inName) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := (this->inObjectName == inName) 
]]

[[D tObjectHeader::IsType (IPC_TYPE itType) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := (this.itObjectType == itType) 
]]

[[D tObjectHeader::IsSize (size_t szSize) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := (this.szObjectSize == szSize)
]]

*****************************************************************************
Class: tLocalHeader 
Description: header storage for Local IPC Object name,type,and size and location
Users:  tLocalMemory and decendents only.
*****************************************************************************

[[D tLocalHeader::tLocalHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->inObjectName := anything
    : this->itObjectType := anything
    : this->szObjectSize := anything 
    : this->pObjectLoc := anything 
]]    

[[D tLocalHeader::tLocalHeader_INITszp (IPC_NAME inName,IPC_TYPE itType, 
    size_t szSize, vod *pLoc) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->inObjectName := inName
    : this->itObjectType := itType
    : this->szObjectSize := szSize 
    : this->pObjectLoc := pLoc 
]]

[[D tLocalHeader::~tLocalHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]  

[[D tLocalHeader::GetObjectReference (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->pObjectLoc 
]]

*****************************************************************************
Class: tSharedHeader 
Ancestors: tObjectHeader
Description: header storage for Shared IPC Object name,type,and size and location
Users:  tSharedMemory and decendents only.
*****************************************************************************

[[D tSharedHeader::tSharedHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->inObjectName := anything
    : this->itObjectType := anything
    : this->szObjectSize := anything 
    : this->iObjectLoc := anything 
]]    

[[D tSharedHeader::tSharedHeader_INITszi(IPC_NAME inName,IPC_TYPE 
                     itType, size_t szSize, int iLoc) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->inObjectName := inName
    : this->itObjectType := itType
    : this->szObjectSize := szSize 
    : this->iObjectLoc := iOffset 
]]

[[D tSharedHeader::~tSharedHeader (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]  

[[D tSharedHeader::GetObjectReference (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iObjectLoc 
]]

*****************/

class tObjectHeader {
  public:
    tObjectHeader ();
    tObjectHeader (IPC_NAME,IPC_TYPE, size_t);
    BOOL IsName (IPC_NAME inName) const volatile { return  (inObjectName == inName);};
    BOOL IsType (IPC_TYPE itType) const volatile { return  (itObjectType == itType);};
    BOOL IsSize (size_t szSize) const volatile { return (szObjectSize == szSize);}; 
    friend ostream& operator<< (ostream& stream, const tObjectHeader& ObjHeader);
  protected:
    IPC_NAME inObjectName;
    IPC_TYPE itObjectType;
    size_t szObjectSize;
};

class tLocalHeader : public tNode, public tObjectHeader {
  public:
    tLocalHeader ();
    tLocalHeader (IPC_NAME inName,IPC_TYPE itType, size_t szSize, void *pLoc);
    void *GetObjectReference () const volatile { return  pObjectLoc;};
    friend ostream& operator<< (ostream& stream, const tLocalHeader& LocHeader);
  protected:
    void *pObjectLoc;   
};

class tSharedHeader : public tObjectHeader {
  public:
    tSharedHeader ();
    tSharedHeader (IPC_NAME inName,IPC_TYPE 
                     itType, size_t szSize, int iLoc);
    int GetObjectReference () const volatile {return iObjectLoc;}
    friend ostream& operator<< (ostream& stream,const tSharedHeader& SharedHeader);
  protected:
   int iObjectLoc; 
};

#endif INC_memHeaderh
