/* $RCSfile: multiActuator.h,v $ */
/* CM Info:$
*/

#ifndef INC_multiActuatorh
#define INC_multiActuatorh

#include "isitExt.h"
#include "matrix.h"
#include "outputChannel.h"
#include "outputDest.h"
#include "stCommands.h"

/*************************************************************************
************************** tMultiActiuator *******************************
**************************************************************************

Black Box State Variables
-------------------------------------- 
const int            CiActuators          Number of actuators
tModulatedActuator   pActuatorArray[CiActuators]
                                          Positions or dithers actuators
tMatrix              XformDetToAct(CiActuators,2)   
                                          Transform matrix, detector-to-actuators
tMatrix              XformDetToAct(CiActuators,2)   
                                          Transform matrix, actuators-to-detectors
tVector              Home(CiActuators)    Home position
Bool                 bValidXform          TRUE if XformDetToAct has been initialized

[[D tMultiActuator::tMultiActuator(tOutputChannel *const ChannelArray[],
                                   int iActuators,double dScaleFactor) ]
constructor
[  invalid_pointer(this) ||
   invalid_pointer(ChannelArray) => UNDEFINED
|  for 0 <= i < iActuators
      invalid_pointer(ChannelArray[i]) => UNDEFINED
| true =>
      allocate memory for pActuatorArray and initialize as follows:
         for 0 <= i < iActuators
         : this->pActuatorArray[i]->ModulationSource := 
               this->pActuatorArray[i]->ModulationSource set for zero modulation
         : this->pActuatorArray[i]->Channel := a reference to *ChannelArray[i]
         : this->pActuatorArray[i]->dElectricalToPhysicalScaleFactor := 
            dScaleFactor
         : Appropriate hardware output := 0.0
         : this-> bValidXform := FALSE
      : this->CiActuators := iActuators
      : this->XformActToDet := zero (2 x iAcutuator) matrix
]]

[[D tMultiActuator::~tMultiActuator() ]
destructor
[  invalid_pointer(this) => UNDEFINED
| true =>
      deallocate memory for pActuatorArray 
]]

[[D tMultiActuator::SetXform(const tMatrix& Value) ]
Assign a value to XformDetToAct
[  invalid_pointer(this) ||
   invalid_reference(Value) => UNDEFINED
|  Value not (this->CiActuators x 2) => this->XformDetToAct := anything
|  this->CiActuators != 2 =>
      this->XformDetToAct := Value
      : this->ValidXform := TRUE
|  true =>
     this->XformActToDet := inverse of Value
   : this->XformDetToAct := Value
   : this->ValidXform := TRUE

]]

[[D tMultiActuator::SetActuatorModulation(int iActuator,
                                          const tModulationParams& Params) ]
Change modulation paramters
[  invalid_pointer(this) ||
   invalid_reference(Params) => UNDEFINED
|  true=>
      this->CalibrationActuator := iActuator
      : this->pActuatorArray[iActuator]->Frequency := Params.Frequency
      : this->pActuatorArray[iActuator]->Amplitude := 
         Params.Amplitude *  COUNTS_PER_PHYSICAL_UNIT
      : this->pActuatorArray[iActuator]->Time := 0
      : this->pActuatorArray[iActuator]->Type := MT_TRIANGLE
]]

[[D tMultiActuator::Move(TIME tTime,const tPoint& Position) ]
Move the actuators to the specified tip/tilt, relative to the home position
[  invalid_pointer(this) ||
   invalid_reference(Position) => UNDEFINED
|  invalid(this->XformDetToFsm) => UNDEFINED
|  !this->ValidXform || !this->ValidHome => return := appropriate error
|  any computation overflow => UNDEFINED
|  true =>
      for 0 <= i <= this->CiActuators
         command PZT for this->pActuatorArray[i] to i-th component of 
            (this->XformDetToAct * Position + this->Home) * 
            COUNTS_PER_PHYSICAL_UNIT
      : return := OK
]]

[[D tMultiActuator::Dither(TIME tTime) ]
Set dither position for specified time
[  invalid_pointer(this) => UNDEFINED
|  this->CalibrationActuator < 0 || this->CalibrationActuator >= 
      this->CiActuators ||
   this->pActuatorArray[this->CalibrationActuator]->DitherFrequency < 0 ||
   this->pActuatorArray[this->CalibrationActuator]->DitherAmplitude < 0 => UNDEFINED
|  true =>
   move this->pActuatorArray[this->iCalibrationActuator] to 
      modulation position for tTime
   : return := OK 
]]

[[D tMultiActuator::XformActToDet(void) ]
returns transform matrix, actuator to detector
[  invalid_pointer(this) => UNDEFINED
| true =>
   return := this->XformActToDet_
]]

**************************************************************************
************************** tPositionMultiActiuator ***********************
**************************************************************************

Black Box State Variables
-------------------------------------- 
const int            CiActuators          Number of actuators
tModulatedActuator   pActuatorArray[CiActuators]
                                          Positions or dithers actuators
tMatrix              XformDetToAct(CiActuators,2)   
                                          Transform matrix, detector-to-actuators
tVector              Home(CiActuators)    Home position
Bool                 bValidXform          TRUE if XformDetToAct has been initialized
Bool                 bValidHome           TRUE if Home has been initialized

[[D tPositionMultiActuator::tPositionMultiActuator(tOutputChannel *const ChannelArray[],
                                   int iActuators,double dScaleFactor) ]
constructor
[  invalid_pointer(this) ||
   invalid_pointer(ChannelArray) => UNDEFINED
|  for 0 <= i < iActuators
      invalid_pointer(ChannelArray[i]) => UNDEFINED
| true =>
      allocate memory for pActuatorArray and initialize as follows:
         for 0 <= i < iActuators
         : this->pActuatorArray[i]->ModulationSource := 
               this->pActuatorArray[i]->ModulationSource set for zero modulation
         : this->pActuatorArray[i]->Channel := a reference to *ChannelArray[i]
         : this->pActuatorArray[i]->dElectricalToPhysicalScaleFactor := 
            dScaleFactor
         : Appropriate hardware output := 0.0
         : this-> bValidXform := FALSE
      : this-> bValidHome := FALSE  
      : this->CiActuators := iActuators
]]

[[D tPositionMultiActuator::~tPositionMultiActuator() ]
destructor
[  invalid_pointer(this) => UNDEFINED
| true =>
      deallocate memory for pActuatorArray 
]]

[[D tPositionMultiActuator::SetXform(const tMatrix& Value) ]
Assign a value to XformDetToAct
[  invalid_pointer(this) ||
   invalid_reference(Value) => UNDEFINED
|  Value not (this->CiActuators x 2) => this->XformDetToAct := anything
|  true=>
      this->XformDetToAct := Value
      : this->ValidXform := TRUE
]]

[[D tPositionMultiActuator::SetHome(const tVector& Voltage) ]
Assign a value to HomePosition
[  invalid_pointer(this) ||
   invalid_reference(Voltage) => UNDEFINED
|  Voltage not length this->CiActuators => this->HomePosition := anything
|  true=>
      for 0 <= i <= this->CiActuators
         this-pActuatorArray[i]->Offset := Voltage[i] * COUNTS_PER_PHYSICAL_UNIT
      : this->ValidHome := TRUE
      : return := OK
]]

[[D tPositionMultiActuator::Move(TIME tTime,const tPoint& Position) ]
Move the actuators to the specified tip/tilt, relative to the home position
[  invalid_pointer(this) ||
   invalid_reference(Position) => UNDEFINED
|  invalid(this->HomePosition) || invalid(this->XformDetToFsm) => UNDEFINED
|  !this->ValidXform || !this->ValidHome => return := appropriate error
|  any computation overflow => UNDEFINED
|  true =>
      for 0 <= i <= this->CiActuators
         command PZT for this->pActuatorArray[i] to i-th component of 
            (this->XformDetToAct * Position + this->Home) * 
            COUNTS_PER_PHYSICAL_UNIT
      : return := OK
]]

[[D tPositionMultiActuator::Dither(TIME tTime) ]
Set dither position for specified time
[  invalid_pointer(this) => UNDEFINED
|  this->CalibrationActuator < 0 || this->CalibrationActuator >= 
      this->CiActuators ||
   this->pActuatorArray[this->CalibrationActuator]->DitherFrequency < 0 ||
   this->pActuatorArray[this->CalibrationActuator]->DitherAmplitude < 0 => UNDEFINED
|  true =>
   move this->pActuatorArray[this->iCalibrationActuator] to 
      modulation position for tTime
   : return := OK 
]]

[[D tPositionMultiActuator::Home(void) ]
Move the actuators to the home position
[  invalid_pointer(this) => UNDEFINED
|  invalid(this->HomePosition) || invalid(this->XformDetToFsm) => UNDEFINED
|  !this->ValidHome => return := appropriate error
|  true =>
      for 0 <= i <= this->CiActuators
         command PZT for this->pActuatorArray[i] to this->Home[i] * 
         COUNTS_PER_PHYSICAL_UNIT
      : return := OK
]]


****/

class tMultiActuator {
   public:
      tMultiActuator(tOutputChannel *const ChannelToUse[],int iActuators,
                     double dScaleFactor);
      virtual ~tMultiActuator();
      void SetXform(const tMatrix& MatrixToUse);
      ERROR_CODE SetActuatorModulation(int iActuator,
                                       const tModulationParams& Params);
      ERROR_CODE Dither(TIME tTime);      
      ERROR_CODE Move(TIME tTime,const tVector& Value);     
      virtual ERROR_CODE Home() {return OK;}      
      const tMatrix& XformActToDet() const {return XformActToDet_;}
   private:
      tMultiActuator(const tMultiActuator&);              //not allowed
      tMultiActuator& operator=(const tMultiActuator&);   //not allowed
      tMatrix XformDetToAct_;
      tMatrix XformActToDet_;
      BOOL bXformValid;
      int iCalibrationActuator;  
   protected:
      int CiActuators;
      tModulatedActuator **pActuatorArray;
   
};


/*
tPositionMultiActuator adds the following functionality:
1) It defines SetHome(), which puts the specified values into 
pActuatorArray[i]->Offset.
2) It overrides Home() to move the actuators to the positions defined in (1).
*/
class tPositionMultiActuator : public tMultiActuator {
   public:
      tPositionMultiActuator(tOutputChannel *const ChannelToUse[],
                             int iActuators,double dScaleFactor);
      void SetHome(const tVector& VectorToUse);
      ERROR_CODE Move(TIME tTime,const tPoint& Position);     
      virtual ERROR_CODE Home();      

   private:
      tPositionMultiActuator(const tPositionMultiActuator&);              //not allowed
      tPositionMultiActuator& operator=(const tPositionMultiActuator&);   //not allowed
      BOOL bHomeValid;
};



#endif
