/* $RCSfile: outputChannel.h,v $ - tOutputChannel and decendents */
/*
$Header: /home/chara/cvsroot/ople/sys/include/outputChannel.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $
*/

/*
modification history
--------------------
01a,7Nov97,pci           written, moved from channel.h
*/

#ifndef INC_outputChannelh
#define INC_outputChannelh

#include "hwTypes.h"
#include "inputChannel.h"
//#include <math.h>


/*******************************************************************************
*
* class tOutputChannel
*
* State Variables:
*  int    iResetValue
*  tInputChannel* pReadbackChannel
*  double dActualChannelValue
*
* SPEC FUNCTIONS:
* BOOL ConvCodeToPolarity(CONV_CODE ccCode)
* Description: Returns appropriate value for a bBipolar variable depending
*              on value of ccCode.
[ ccCode == CC_STRAIGHT_BINARY || CC_COMPLEMENT_BINARY =>
  return := FALSE
| ccCode == CC_OFFSET_BINARY || ccCode == CC_TWOS_COMPLEMENT =>
  return := TRUE
| true => UNDEFINED
]
*
*
* int ABS(int iInValue)
* Description: Returns absolute value of iInValue.
[ iInValue < 0 =>
  return := -iInValue
| true =>
  return := iInValue
]
*
*
* unsigned int Unsigned(int iValue, int iBitWidth)
* Description: Returns unsigned value for iValue to demonstrate channel output
*              behavior when a negative value is passed in to an output
*              function.
[ iBitWidth <= 0 =>
  UNDEFINED
| true =>
  return := (unsigned int) (iValue & ((2^iBitWidth) - 1))
]
*
*
* double DaResolution(int iOutputRange, int iBitWidth)
* Description: Returns resolution of D/A converter (in V/count).
[ (iOutputRange < 0) || (iBitWidth < 0) => UNDEFINED
| true =>
  return := (iOutputRange / (2^iBitWidth))
]
*
*
* METHODS:
* pure virtual ERROR_CODE tOutputChannel::Output(int)
* Description:    Sends output appropriate for specific channel
[[D tOutputChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->1dActualChannelValue := channel value corresponding to iValue
]]
*
*
* pure virtual ERROR_CODE tOutputChannel::Output(double)
* Description:    Sends output appropriate for specific channel
[[D tOutputChannel::Output_double(int dValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(dValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->1dActualChannelValue := channel value corresponding to dValue
]]
*
*
* ERROR_CODE tOutputChannel::Reset(void)
* Description:  Set channel to this->iResetValue.  On most channels this is 0V.
*               On some unipolar channels, the reset value is midway between
*               0V and the maximum value.
[[D tOutputChannel::Reset (void)]
[ true => Needs update..
]]
*
*
* pure virtual ERROR_CODE tOutputChannel::SetResetValue(int iNewResetValue)
* Description:   Sets the reset value for this channel
[[D tOutputChannel::SetResetValue(int iNewResetValue)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE =>
  return := S_tOutputChannel_CHANNEL_NOT_INITIALIZED
| true =>
  this->iResetValue := iNewResetValue
  : return := OK
]]
*
*
* BOOL tOutputChannel::HasReadback(void)
* Description:  Returns TRUE if the channel has an associated channel for
*               reading back the current value.
[[D tOutputChannel::HasReadback(void)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| this->pReadbackChannel == NULL =>
  return := FALSE
| true =>
  return := TRUE
]]
*
*
* int tOutputChannel::GetOutputValue(...)
* Description: Converts iValueToConvert into the appropriate value for the type
*              of output.
[[D tOutputChannel::GetOutputValue(int iValueToConvert)]
[ this is invalid || this->iBitWidth >= sizeof(int)*8 ||
  this->ccConvCode does not match conversion code of the board => UNDEFINED
| this->ccConvCode is not valid =>
  appropriate error message to console
  : return := iValueToConvert
| this->ccConvCode == CC_NO_CONV_CODE ||
  this->ccConvCode == CC_TWOS_COMPLEMENT =>
  return := iValueToConvert
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  return := (2^this->iBitWidth - 1 - Unsigned(iValueToConvert, this->iBitWidth))
| this->ccConvCode == CC_STRAIGHT_BINARY =>
  return := Unsigned(iValueToConvert, this->iBitWidth)
| this->ccConvCode == CC_OFFSET_BINARY =>
  return := Unsigned(iValueToConvert, this->iBitWidth) - (2^this->iBitWidth/2)
| true => UNDEFINED
]]
*
******/
/*******************************************************************************
*
* class tDaChannel
*
* State Variables:
*  @ double     dActualChannelValue
*  int        iOutputRange
*
* METHODS:
* pure virtual ERROR_CODE tDaChannel::Output(int iValue)
* Description:    Sends iValue to the D/A converter for voltage output
[[D tDaChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := any voltage (voltage will reflect relevant bits only)
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  this->dActualChannelValue := (2^this->iBitWidth - 1 -
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth))
  : return := OK
| this->ccConvCode == CC_STRAIGHT_BINARY =>
  this->dActualChannelValue :=
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth)
  : return := OK
| this->ccConvCode == CC_OFFSET_BINARY =>
  this->dActualChannelValue :=
                          Unsigned(iValue, this->iBitWidth) *
                          DaResolution(this->iOutputRange, this->iBitWidth) -
                          (2^this->iBitWidth/2)
  : return := OK
| this->ccConvCode == CC_TWOS_COMPLEMENT =>
  this->dActualChannelValue := iValue *
                               DaResolution(this->iOutputRange, this->iBitWidth)
| true => UNDEFINED
]]
*
******/
/*******************************************************************************
*
* class tDigitalOutChannel
*
* METHODS:
* pure virtual ERROR_CODE tDigitalOutChannel::Output(int iValue)
* Description:    Sends digital output to this channel
[[D tDigitalOutChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->dActualChannelValue := channel value corresponding to iValue
]]
*
******/
/*******************************************************************************
*
* class tWaveformChannel
*
*  The tWaveformChannel refers to channels such as those on the Pentland
*   MPV955 board.  These channels are related to D/A channels except instead
*   of converting the values on-the-fly, the values are first stored in memory
*   for output at a later time.  With this in mind, the dActualChannelValue
*   variable (inherited from tOutputChannel) really means the voltage value
*   output by the channel if and when the output is turned on.
*
* StateVariables:
*  @ int    iResetValue;
*  @ tInputChannel* pReadbackChannel;
*  @ double dActualChannelValue
*  int       iOutputRange
*
*
* METHODS:
* pure virtual ERROR_CODE tWaveformChannel::Output(int iValue)
* Description:    Sends output appropriate for specific channel

[[D tWaveformChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  this->dActualChannelValue := (2^this->iBitWidth - 1 -
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth))
  : return := OK
| this->ccConvCode == CC_STRAIGHT_BINARY =>
  this->dActualChannelValue :=
                           Unsigned(iValue, this->iBitWidth) *
                           DaResolution(this->iOutputRange, this->iBitWidth)
  : return := OK
| this->ccConvCode == CC_OFFSET_BINARY =>
  this->dActualChannelValue :=
                          Unsigned(iValue, this->iBitWidth) *
                          DaResolution(this->iOutputRange, this->iBitWidth) -
                          (2^this->iBitWidth/2)
  : return := OK
| this->ccConvCode == CC_TWOS_COMPLEMENT =>
  this->dActualChannelValue := iValue *
                               DaResolution(this->iOutputRange, this->iBitWidth)
| true => UNDEFINED
]]
*
*
* pure virtual ERROR_CODE tWaveformChannel::LoadMemory(int* piValues)
* Description:    Loads memory for this channel
[[D tWaveformChannel::LoadMemory(int* piValues)]
[ this is invalid => UNDEFINED
| this->bInit == FALSE => return := S_tWaveformChannel_CHANNEL_NOT_INITIALIZED
| ABS(piValues[0]..piValues[current size of waveform]) >= 2^this->iBitWidth =>
  this->dActualChannelValues := anything (loaded values will reflect relevant bits)
| this->ccOuputCode == CC_COMPLEMENT_BINARY =>
  this->dActualChannelValues := (2^this->iBitWidth - 1 -
                        Unsigned(piValues[0..waveform size], this->iBitWidth) *
                        DaResolution(this->iOutputRange, this->iBitWidth))
  : return := OK
| this->ccConvCode == CC_STRAIGHT_BINARY =>
  this->dActualChannelValues :=
                        Unsigned(piValues[0..waveform size], this->iBitWidth) *
                        DaResolution(this->iOutputRange, this->iBitWidth)
  : return := OK
| this->ccConvCode == CC_OFFSET_BINARY =>
  this->dActualChannelValues :=
                       Unsigned(piValues[0..waveform size], this->iBitWidth) *
                       DaResolution(this->iOutputRange, this->iBitWidth) -
                          (2^this->iBitWidth/2)
  : return := OK
| this->ccConvCode == CC_TWOS_COMPLEMENT =>
  this->dActualChannelValues := piValues[0..waveform size] *
                       DaResolution(this->iOutputRange, this->iBitWidth)
| true =>
  this->dActualChannelValues := piValues[0..waveform size]
]]
*
*
* pure virtual int tWaveformChannel::GetMemSize(void)
* Description:    returns available memory for this channel
[[D tWaveformChannel::GetMemSize( void )]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  return := amount of memory available to this channel
]]
*
* eamck 8/19/97
*
* pure virtual int tWaveformChannel::GetChannelBank(void)
* Description:    returns channel bank pointer for this channel
[[D tWaveformChannel::GetChannelBank( void )]
[ this is invalid => UNDEFINED
| true =>
    [[U tMpv955WaveformChannel::GetChannelBank()]]
]]
*
*
******/
/*******************************************************************************
*
* class tStepMotorChannel
*
* METHODS:
* pure virtual ERROR_CODE tStepMotorChannel::Output(int iValue)
* Description:    Sends output appropriate for specific channel
[[D tStepMotorChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->dActualChannelValue := pulse train corresponding to iValue (pulses per sec)
]]
*
*
* pure virtual ERROR_CODE tStepMotorChannel::Reset(void)
* Description:    resets channel output to midrange.
[[D tStepMotorChannel::Reset(void)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->dActualChannelValue := midrange voltage value
]]
*
******/
/*******************************************************************************
*
* class tRampChannel
*
* METHODS
* pure virtual ERROR_CODE tRampChannel::Output(int iValue)
* Description:    Sends voltage ramp relative to iValue to corresponding channel
[[D tRampChannel::Output(int iValue)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| ABS(iValue) >= 2^this->iBitWidth =>
  this->dActualChannelValue := anything (output will reflect relevant bits only)
| true =>
  this->dActualChannelValue := voltage ramp corresponding to iValue
]]
*
*
* pure virtual ERROR_CODE tRampChannel::Reset(void)
* Description:    resets channel output to midrange.
[[D tRampChannel::Reset(void)]
[ this is invalid || this->bInit == FALSE => UNDEFINED
| true =>
  this->dActualChannelValue := midrange voltage value
]]
*
******/



/*******************************************************************************
**************************** Class Definitions *********************************
*******************************************************************************/

class tOutputChannel : public tChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
  virtual ERROR_CODE Output(double dValue) { return (Output((int) dValue)); }
  virtual ERROR_CODE Reset(void);
  ERROR_CODE SetResetValue(int iNewResetValue);
  BOOL       HasReadback(void);
  int        GetOutputValue (int iValueToConvert);

protected:
  int           iResetValue;
  tInputChannel* pReadbackChannel;
};


class tNullOutputChannel : public tOutputChannel {
public:
  tNullOutputChannel(void) { iResetValue      = 0;
                             pReadbackChannel = NULL;
                             iBitWidth        = 0;
                             dChannelRange    = 0.0;
                             ccConvCode       = CC_NO_CONV_CODE;
                             SetInit(TRUE);
                           }
  ERROR_CODE Output(int iValue)    { return OK; }
  ERROR_CODE Output(double dValue) { return OK; }
  ERROR_CODE Reset(void)           { return OK; }
};


class tDaChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
};


class tDigitalOutChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
};


class tWaveformChannelBank;

class tWaveformChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
  virtual ERROR_CODE LoadMemory(int* piValues) = 0;
  virtual ERROR_CODE LoadMemory(double* piValues) = 0;
  virtual int        GetMemSize(void) = 0;
  virtual tWaveformChannelBank* GetChannelBank(void) = 0;
};


class tStepMotorChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
  virtual ERROR_CODE Reset(void) = 0;
};


class tRampChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
};


class tClockChannel : public tOutputChannel {
public:
  virtual ERROR_CODE Output(int iValue) = 0;
};


#endif /* INC_outputChannelh */
