/* $RCSfile*/
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/periodicTargetGen.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a2Sept97,rlj    written
*/

#ifndef INC_periodicTargetGenh
#define INC_periodicTargetGenh

#include "isit.h"
#include "inputSource.h"

/****
Black Box State Variables
____________________________

  tModulationParams ModulationParameters
  tDlTarget         Target


[[D tPeriodicTargetGenerator::tPeriodicTargetGenerator (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tPeriodicTargetGenerator::SetModulationParameters_MTdd (MOD_TYPE mtType, 
                          double dFrequency, double dAmplitude) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->ModulationParameters.mtModulationType := mtType
    : this->ModulationParameters.dFrequency := dFrequency
    : this->ModulationParameters.dAmplitude := dAmplitude
    : this->ModulationParameters.dMeanVal := 0
    : this->ModulationParameters.tSyncTime := 0
    : this->Target := anything
]] 	             

[[D tPeriodicTargetGenerator::SetModulationParameters_MTdddt (MOD_TYPE mtType, 
                               double dFrequency, double dAmplitude, 
                               double dMeanVal, TIME tSyncTime) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->ModulationParameters.mtModulationType := mtType
    : this->ModulationParameters.dFrequency := dFrequency
    : this->ModulationParameters.dAmplitude := dAmplitude
    : this->ModulationParameters.dMeanVal := dMeanVal
    : this->ModulationParameters.tSyncTime := tSyncTime
    : this->Target := anything
]] 	             

	           
[[D tPeriodicTargetGenerator::SetModulationParameters (
      const tModulationParams& NewModParams) ]
[ invalid_pointer (this) => UNDEFINED
| true =>
    this->ModulationParameters := NewModParams
    : this->Target := anything
]]


Note that currently we don't actually support MT_SIN, MT_TRIANGLE, and MT_RANDOM	           

[[D tPeriodicTargetGenerator::Update (TIME tUpdateTime) ]
[ invalid_pointer (this) => UNDEFINED
| this->mtModulationType == MT_NONE =>
    this->Target.dPosition := 0
    : this->Target.dRate := 0
    : this->Target.tTime := tUpdateTime
    : return := 0
| this->mtModulationType == MT_SQUARE =>
    this->Target.dPosition := this->dAmplitude * SIGN ( sin (2 * PI *this->dFrequency * 
      (tUpdateTime-this->tSyncTime) ) ) + this->dOffset
    : this->Target.dRate := 0
    : this->Target.tTime := tUpdateTime
    : return := this->dAmplitude * SIGN ( sin (2 * PI *this->dFrequency * 
      (tUpdateTime-this->tSyncTime) ) ) + this->dOffset
| this->mtModulationType == MT_SIN =>
    this->Target.dPosition := this->dAmplitude * sin (2 * PI *this->dFrequency * 
      (tUpdateTime-this->tSyncTime) ) + this->dOffset
    : this->Target.dRate := 0
    : this->Target.tTime := tUpdateTime
    : return := this->dAmplitude * sin (2 * PI *this->dFrequency * 
      (tUpdateTime-this->tSyncTime) ) + this->dOffset
| this->mtModulationType == MT_TRIANGLE =>
    this->Target.dPosition := Integral (this->dAmplitude * 
      Sign (sin (2 * PI *this->dFrequency * (t-this->tSyncTime) ) ),
      t,0,tUpdateTime) * this->dFrequency / 2 - this->dAmplitude/2 + 
      this->dOffset
    : this->Target.dRate := 0
    : this->Target.tTime := tUpdateTime
    : return := Integral (this->dAmplitude * 
      Sign (sin (2 * PI *this->dFrequency * (t-this->tSyncTime) ) ),
      t,0,tUpdateTime) * this->dFrequency / 2 - this->dAmplitude/2 + 
      this->dOffset
| this->mtModulationType == MT_RANDOM =>
    this->Target.dPosition := UniformRandom (-1 * this->dAmplitude,
      this->dAmplitude) + this->dOffset
    : this->Target.dRate := 0
    : this->Target.tTime := tUpdateTime
    : return := UniformRandom (-1 * this->dAmplitude,
      this->dAmplitude) + this->dOffset
]]

	           
[[D tPeriodicTargetGenerator::GetTarget (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->Target
]]
*****/

class tPeriodicTargetGenerator : public tModulationSource {
  public:
    tPeriodicTargetGenerator ();
    double Update(const TIME tUpdateTime);
    tDlTarget GetTarget ();
  protected:
    tDlTarget Target;
};
    
#endif INC_periodicTargetGenh
