/*  ControlLaw.h   */
/* Bob Grogan - Jet Propulsion Laboratory */

#ifndef INC_ControlLawh
#define INC_ControlLawh


/*************************************************************/
/* Define number of states in controller */ 
#define NUM_PZT_STATES		8
#define NUM_VC_STATES		7
#define NUM_MOTOR_STATES	5
#define NUM_HPFILTER_STATES	1
/*************************************************************/
/*************************************************************/ 
/* Define State Coefficient Matrix for Controller State Space */ 
typedef struct PZT_CONTROL
{
double A[NUM_PZT_STATES][NUM_PZT_STATES];
double B[NUM_PZT_STATES];
double C[NUM_PZT_STATES];
double D;

};
/*************************************************************/
/*************************************************************/ 
/* Define State Coefficient Matrix for Controller State Space  */ 
typedef struct VC_CONTROL
{
double A[NUM_VC_STATES][NUM_VC_STATES];
double B[NUM_VC_STATES];
double C[NUM_VC_STATES];
double D;

};
/*************************************************************/
/*************************************************************/ 
/* Define State Coefficient Matrix for Controller State Space  */ 
typedef struct MOTOR_CONTROL
{
double A[NUM_MOTOR_STATES][NUM_MOTOR_STATES];
double B[NUM_MOTOR_STATES];
double C[NUM_MOTOR_STATES];
double D;

};
/*************************************************************/  
/*************************************************************/ 
/* Define State Coefficient Matrix for Controller State Space  */ 
typedef struct HPFILTER_CONTROL
{
double A[NUM_HPFILTER_STATES][NUM_HPFILTER_STATES];
double B[NUM_HPFILTER_STATES];
double C[NUM_HPFILTER_STATES];
double D;

};
/*************************************************************/  
/*************************************************************/ 

/* Comment */ 
class tPztController
{
protected:
  PZT_CONTROL coeffs;
  double State[NUM_PZT_STATES];
  double NewState[NUM_PZT_STATES];
  double dPztGain;

   
public:
  ERROR_CODE Initialize(double dPztGainToUse);
  double Calculate( double dLaserError);
};


/*************************************************************/
/*************************************************************/  
/* Comment */ 
class tVcController
{
protected:
  VC_CONTROL coeffs;
  double State[NUM_VC_STATES];
  double NewState[NUM_VC_STATES];
  double dVcGain, dVcErrorGain;


  // High pass filter declarations 
  HPFILTER_CONTROL highpass;
  double HPState[NUM_HPFILTER_STATES];
  double NewHPState[NUM_HPFILTER_STATES];
  
  // Error Ramp Stuff
  /*
    double SampleRate;
    double RampTime;
    double Ramp;
    double NextRamp;
    int    Count;
    int    NextCount;
  */

public:
  ERROR_CODE Initialize(double dVcGainToUse, double dVcErrorGainToUse);
  double Calculate( double dVcError, double dLaserError);
};


/*************************************************************/
/*************************************************************/ 
/* Comment */ 
class tMotorController
{
protected:
  MOTOR_CONTROL coeffs;
  double State[NUM_MOTOR_STATES];
  double NewState[NUM_MOTOR_STATES];
  double dMotorGain, dMotorTargetGain, dMotorErrorGain;

  
public:
  ERROR_CODE Initialize(double dMotorGainToUse, double dMotorErrorGainToUse,
                        double dMotorTargetGainToUse);
  double Calculate( double dMotorError, double dLaserError, double dTargetVelocity);
};


#endif /* INC_ControlLawh */
