/* $RCSfile: quicDlTrackController.h,v $ - implements Quic-specific servo and controller functionality */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/quic/quicDlTrackController.h,v 1.1.1.1 1998/06/11 21:36:17 chara Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,25Apr97    written
*/
 
/*
DESCRIPTION

*/

#ifndef INC_QuicDlTrackControllerh
#define INC_QuicDlTrackControllerh


#include "isit.h"
#include "quicDlController.h"
//#include "dl/dlTrackController.h"


/*******************************************************************
*
* tQuicDlFastTrackSlewController Black Box
*
* Variable                                  Initial Value
* -------------------------------------------------------
* tQuicDlFastTrackInputSet  &InputSet
* tQuicDlFastTrackOutputSet &OutputSet
* tQuicDlFastTrackTargetSet &TargetSet
*
*******************************************************************
*
* tQuicDlFastTrackSlewController::tQuicDlFastTrackSlewController
*
* DESCRIPTION: Consructor
*
* RETURNS: Nothing
*

[[D tQuicDlFastTrackSlewController::tQuicDlFastTrackSlewController (
    in: InputSetToUse, OutputSetToUse, TargetSetToUse, ControllerDataToUse;
    inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
     this->InputSet := InputSetToUse
     : this->OutputSet := OutputSetToUse
     : this->TargetSet := TargetSetToUse
     : this->ControllerData := ControllerDataToUse
]]

*
*******************************************************************
*
* tQuicDlFastTrackSlewController::Calculate
*
* DESCRIPTION: Simply computes the laser error
*
* RETURNS: ERROR_CODE
*

[[D tQuicDlFastTrackSlewController::Calculate (in: this;)]
  [ this invalid => UNDEFINED
  | true =>
     return := OK
]]

*
*/



/*******************************************************************
*
* tQuicDlFastTrackLinearController Black Box
*
* Variable                                  Initial Value
* -------------------------------------------------------
* tQuicDlFastTrackInputSet  &InputSet
* tQuicDlFastTrackOutputSet &OutputSet
* tQuicDlFastTrackTargetSet &TargetSet
*
*******************************************************************
*
* tQuicDlFastTrackLinearController::tQuicDlFastTrackLinearController
*
* DESCRIPTION: Consructor
*
* RETURNS: Nothing
*

[[D tQuicDlFastTrackLinearController::tQuicDlFastTrackLinearController (
    in: InputSetToUse, OutputSetToUse, TargetSetToUse, ControllerDataToUse;
    inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
     this->InputSet := InputSetToUse
     : this->OutputSet := OutputSetToUse
     : this->TargetSet := TargetSetToUse
     : this->ControllerData := ControllerDataToUse
]]

*

*******************************************************************
*
* tQuicDlFastTrackLinearController::Calculate
*
* DESCRIPTION: Simply computes the laser error
*
* RETURNS: ERROR_CODE
*

[[D tQuicDlFastTrackLinearController::Calculate (in: this;)]
  [ this invalid => UNDEFINED
  | true =>
     return := OK
]]

*
*/


/*******************************************************************
*
* tQuicDlFastTrackController Black Box
*
* Variable                                  Initial Value
* -------------------------------------------------------
* tQuicDlFastTrackInputSet  &InputSet
* tQuicDlFastTrackOutputSet &OutputSet
* tQuicDlFastTrackTargetSet &TargetSet
*
*******************************************************************
*
* tQuicDlFastTrackController::tQuicDlFastTrackController
*
* DESCRIPTION: Consructor
*
* RETURNS: Nothing
*

[[D tQuicDlFastTrackController::tQuicDlFastTrackController (
    in: InputSetToUse, OutputSetToUse, TargetSetToUse, ControllerDataToUse;
    inout: this;)]
  [ this invalid => UNDEFINED
  | true =>
     this->InputSet := InputSetToUse
     : this->OutputSet := OutputSetToUse
     : this->TargetSet := TargetSetToUse
     : this->ControllerData := ControllerDataToUse
]]

*
*******************************************************************
*
* tQuicDlFastTrackController::Calculate
*
* DESCRIPTION: Simply computes the laser error
*
* RETURNS: ERROR_CODE
*

[[D tQuicDlFastTrackController::Calculate (in: this;)]
  [ this invalid => UNDEFINED
  | true =>
return := OK
]]

*
*/


/*******************************************************************/
/*******************************************************************/
/********************                     **************************/
/********************  Class Definitions  **************************/
/********************                     **************************/
/*******************************************************************/
/*******************************************************************/

/*******************************************************************/
/******************  tQuicDlFastTrackLinearController *************/
/*******************************************************************/

class tQuicDlCompleteIoSet;
class tQuicDlControllerSet;

class tQuicDlFastTrackLinearController : public tDlController {
  public:
    tQuicDlFastTrackLinearController ( 
       tQuicDlCompleteIoSet& CompleteIoSet,
       tQuicDlControllerData& ControllerDataToUse);
  
    ERROR_CODE Calculate (TIME tServoTime);
  
  protected:

    //inputs
    tSensor&                    LaserDiagInputSensor; 
    tLaserMetrologySensor&      LaserInput;

    //outputs
    tModulatedVelocityActuator& Pzt;
    tActuator&                  LaserDiagOutput;
    tActuator&                  ServoErrorDiagOutput;
  
    //targets
    tTargetSource&              LaserTarget;
};


/*******************************************************************/
/******************  tQuicDlMediumTrackLinearController *************/
/*******************************************************************/

class tQuicDlMediumTrackLinearController : public tDlController {
  public:
    tQuicDlMediumTrackLinearController (tQuicDlCompleteIoSet& CompleteIoSet,
      tQuicDlControllerData &ControllerDataToUse);
  
    ERROR_CODE Calculate (TIME tServoTime);
  
  protected:

    //inputs
    tSensor&                    Vc1DiagInputSensor;
    tModulatedSensor&           PztInputPosition;
    tSensor&                    DiagPztInputSensor;

    //outputs
    tModulatedPositionActuator& Vc1;
    tActuator&                  Vc1ServoErrorDiagOutput;

    //targets
    tTargetSource&              PztTargetPosition;
};


/*******************************************************************/
/******************  tQuicDlSlowTrackLinearController ***************/
/*******************************************************************/

class tQuicDlSlowTrackLinearController : public tDlController {
  public:
    tQuicDlSlowTrackLinearController (tQuicDlCompleteIoSet& CompleteIoSet,
                              tQuicDlControllerData &ControllerDataToUse);
  
    ERROR_CODE Calculate (TIME tServoTime);
  
  protected:
 
    //inputs
    tSensor&                    MotorDiagInputSensor;

    //outputs
    tModulatedVelocityActuator& Motor;
    tActuator&                  MotorServoErrorDiagOutput;

    //targets
    tTargetSource&              MotorVelocityFeedforward;
};


#endif  /* INC_QuicDlTrackControllerh */
