/* $RCSfile: realTimeGizmo.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/realTimeGizmo.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ */

/*
modification history
--------------------
01a,17Jun97,rlj    written
*/

#ifndef INC_realTimeGizmoh
#define INC_realTimeGizmoh

#include "gizmo.h"
#include "controllerSet.h"
#include "servo.h"


//Errors

//Warnings

#define S_realtime_gizmo_SHUTTING_DOWN_GIZMO  (M_rtgizmo | 0x1001)
#define S_realtime_gizmo_ALREADY_SHUTDOWN     (M_rtgizmo | 0x1002)
#define S_realtime_gizmo_ALREADY_RUNNING      (M_rtgizmo | 0x1003)

//Messages 

#define S_realtime_gizmo_CONSTRUCTOR_START    (M_rtgizmo | 0x2001)
#define S_realtime_gizmo_CONTRUCTOR_COMPLETE  (M_rtgizmo | 0x2002)
#define S_realtime_gizmo_START_TASKS_COMPLETE (M_rtgizmo | 0x2003)
#define S_realtime_gizmo_DESTRUCTOR_START     (M_rtgizmo | 0x2004)
#define S_realtime_gizmo_DESTRUCTOR_COMPLETE  (M_rtgizmo | 0x2005)
#define S_realtime_gizmo_RESTART_START        (M_rtgizmo | 0x2006)
#define S_realtime_gizmo_RESTART_COMPLETE     (M_rtgizmo | 0x2007)
#define S_realtime_gizmo_SHUTDOWN_START       (M_rtgizmo | 0x2008)
#define S_realtime_gizmo_SHUTDOWN_COMPLETE    (M_rtgizmo | 0x2009)


/****************************************************************************

tRealtimeGizmo Black Box State

BOOL			        bInit
ERROR_CODE			ecInitError
tServo			      	Servo - (see servo.h)

TASK_ID                         tidCommandTaskId
TASK_ID                         tidStatusTaskId
ARRAY <TASK_ID>                 tidServoTaskIdArray

*****************************************************************************

[[D tRealtimeGizmo::tRealtimeGizmo(const tRealtimeGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id of ::pThisRteG ||
  GizmoInfo.iGizmoId != id of *this ||
  ::IpcMemoryG.bInit == FALSE ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG,GizmoInfo.iiCommandStreamId) == FALSE ||
  Unable to add periodic tasks to handle command, status and servo =>
    this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->Servo := anything
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
    : this->tidServoTaskIdArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions] := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := FALSE
    : this->Servo.ControllerArrayArray := EMPTY ARRAY [GizmoInfo.iNumberOfModes,
                                                 GizmoInfo.ServoInfo.iNumberOfPartitions]
    : this->Servo.Mode := anything
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
       NewObjectSet (::IpcMemoryG,GizmoInfo.iiCommandStreamId)
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
    : this->tidServoTaskIdArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions] := anything
]]  


[[D tRealtimeGizmo::~tRealtimeGizmo (void) ]
[ invalid_pointer (this) => UNDEFINED
| true =>    
    *this := anything
    : (::PeriodicTaskSet) := ::PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId) -
      PeriodicTask (this->tidServoTaskId)
]]


[[D tRealtimeGizmo::StartTasks (const tRealtimeGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  ::PeriodicTaskSet.bInit == FALSE => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : return := OK
| Unable to add periodic tasks to handle command and status =>
    this->bInit := anything
    : this->tidCommandTaskId := anything
    : this->tidStatusTaskId := anything
    : this->tidServoTaskIdArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions] := anything
    : console := console + appropriate error messages
    : return := appropriate error code
| true =>    
    (::PeriodicTaskSet) := ::PeriodicTaskSet U 
      PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
        this->CollectStatus) U
      PeriodicTask (GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions].iTaskPriority, 0,
        this->Servo.ServoTaskArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions].DoServo 
          [0..GizmoInfo.ServoInfo.iNumberOfPartitions])
    : this->tidCommandTaskId := new periodic task id
    : this->tidStatusTaskId := new periodic task id
    : this->tidServoTaskIdArray [0..GizmoInfo.ServoInfo.iNumberOfPartitions] := new periodic task id
    : this->bInit := TRUE
    : return := OK
]]

[[D tRealtimeGizmo::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to resume  PeriodicTask (this->tidCommandTaskId) ||
  Unable to resume  PeriodicTask (this->tidStatusTaskId) ||
  Unable to resume  PeriodicTask (this->tidServoTaskIdArray 
   [0..GizmoInfo.ServoInfo.iNumberOfPartitions] ) =>
     console := console + appropriate error messages
     : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) resumed
    : PeriodicTask (this->tidStatusTaskId) resumed
    : PeriodicTask (this->tidServoTaskIdArray 
      [0..GizmoInfo.ServoInfo.iNumberOfPartitions]) resumed
    : this->bInit := TRUE
    : return := OK
]]

[[D tRealtimeGizmo::Shutdown (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspended PeriodicTask (this->tidCommandTaskId) ||
  Unable to suspended PeriodicTask (this->tidStatusTaskId) ||
  Unable to suspended PeriodicTask (this->tidServoTaskIdArray 
   [0..GizmoInfo.ServoInfo.iNumberOfPartitions]) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) suspended
    : PeriodicTask (this->tidStatusTaskId) suspended
    : PeriodicTask (this->tidServoTaskIdArray 
      [0..GizmoInfo.ServoInfo.iNumberOfPartitions]) suspended
    : this->bInit := FALSE
    : return := OK
]]

[[D tRealtimeGizmo::IsOk(void) ]
[ invalid_pointer (this) => UNDEFINED
| this.ecInitError != OK => return := FALSE
| true => return := TRUE
]]


[[D tRealtimeGizmo::operator_extractor (ostream& Stream, 
                                        const tRealtimeGizmo& Gizmo) ]
[ invalid_pointer (this) ||
  invalid_reference (Stream) ||
  invalid_reference (Gizmo) => UNDEFINED
| true =>
    Stream := Stream << appropriate text and Gizmo state values
    : return := Stream << appropriate text and Gizmo state values
]]

*********/

class tRealtimeGizmoInfo;

class tRealtimeGizmo : public tGizmo {
  public:
    tRealtimeGizmo (const tRealtimeGizmoInfo &GizmoInfo);
    virtual ~tRealtimeGizmo ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
  protected:
    ERROR_CODE StartTasks (const tRealtimeGizmoInfo& GizmoInfo);
    tControllerSet ControllerSet;
    tServo Servo;
};

#endif INC_realTimeGizmoh
