/* $RCSfile: relayStream.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/relayStream.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ */

/*
modification history
--------------------
01a,17Jun97,rlj    written
*/

#ifndef INC_relayStreamh
#define INC_relayStreamh

#include "isit.h"
#include "baseObject.h"
#include "ipcid.h"
#include "ipcMultiTypeFifoAccessor.h"
#include <msgQLib.h>

// ERRORS

#define S_relayStream_SENDER_NOT_INITIALIZED       (M_relayStream | 0x1)
#define S_relayStream_RECEIVER_NOT_INITIALIZED     (M_relayStream | 0x2)

// WARNINGS
#define S_relayStream_ALREADY_INITIALIZED          (M_relayStream | 0x1000)
#define S_relayStream_ALREADY_SHUTDOWN             (M_relayStream | 0x1001)
#define S_relayStream_SENDER_ALREADY_INITIALIZED   (M_relayStream | 0x1002)
#define S_relayStream_SENDER_ALREADY_SHUTDOWN      (M_relayStream | 0x1003)
#define S_relayStream_RECEIVER_ALREADY_INITIALIZED (M_relayStream | 0x1004)
#define S_relayStream_RECEIVER_ALREADY_SHUTDOWN    (M_relayStream | 0x1005)
#define S_relayStream_SHUTTING_DOWN                (M_relayStream | 0x1006)
#define S_relayStream_SHUTTING_DOWN_SENDER         (M_relayStream | 0x1007)
#define S_relayStream_SHUTTING_DOWN_RECEIVER       (M_relayStream | 0x1008)



// INFO
#define S_relayStream_INIT_COMPLETE                (M_relayStream | 0x2001)
#define S_relayStream_REINIT_COMPLETE              (M_relayStream | 0x2002)
#define S_relayStream_SHUTDOWN_COMPLETE            (M_relayStream | 0x2003)

#define S_relayStream_SENDER_INIT_COMPLETE         (M_relayStream | 0x2004)
#define S_relayStream_SENDER_REINIT_COMPLETE       (M_relayStream | 0x2005)
#define S_relayStream_SENDER_SHUTDOWN_COMPLETE     (M_relayStream | 0x2006)

#define S_relayStream_RECEIVER_INIT_COMPLETE       (M_relayStream | 0x2007)
#define S_relayStream_RECEIVER_REINIT_COMPLETE     (M_relayStream | 0x2008)
#define S_relayStream_RECEIVER_SHUTDOWN_COMPLETE   (M_relayStream | 0x2009)



#define RELAY_STREAM_SEND_MAX_WAIT (300)


/************

[[D tRelayStreamSender::tRelayStreamSender (const tIpcFifoId& iiId, 
      int iMutexSemaphoreOptions) ]
[ invalid_pointer (this) ||
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE ||
  Unable to create mutex semaphore =>
    console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiFifoId := anything
    : this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->siPutMutexSemaphore := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiFifoId := iiId
    : ::IpcMemoryG.ObjectSet := 
      ::IpcMemoryG.ObjectSet U NewObjectSet (::IpcMemoryG, iiId ) 
    : vxWorks mutex semaphore created with options iMutexSemaphoreOptions
    : this->siPutMutexSemaphore := new vxWorks mutex semaphore id
]]

[[D tRelayStreamSender::~tRelayStreamSender(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning message
    : mutex semaphore this->siPutMutexSemaphore deleted
    : *this := anything
| true =>
    mutex semaphore this->siPutMutexSemaphore deleted
    : *this := anything
]]

[[D tRelayStreamSender::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning message
    : return := OK
| true =>
   this->bInit := TRUE
   : return OK
   
]]

[[D tRelayStreamSender::Shutdown (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning message
    : return := OK
| true =>
   this->bInit := FALSE
   : return OK   
]]

[[D tRelayStreamSender::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := TRUE   
]]


[[D tRelayStreamSender::Put (const tTypeTag& TypeTagIn, const BYTE* pDataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (TypeTagIn) ||
  invalid_pointer (pDataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, this->iiFifoId) == FALSE ||
  simultaneous put or flush on GetObject (::IpcMemoryG, 
    this->iiFifoId) => UNDEFINED
| this->bInit == FALSE ||
  invalid_semaphore (this->siPutMutexSemaphore) ||
  semaphore_time_out (this->siPutMutexSemaphore,RELAY_STREAM_SEND_MAX_WAIT) => 
    console := console + appropriate error message
    : return := appropriate error code
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length + TypeTagIn.szSize +
  sizeof (TypeTagIn) > GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => 
    return := S_ipc_FIFO_FULL
| true => 
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue :=
      GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue with *pDataIn and 
      TypeTagIn appended to back
    : return := OK 
]]




[[D tRelayStreamSender::PutNoWait(const tTypeTag& TagIn, const BYTE* pDataIn) ]
[ invalid_pointer (this) || 
  invalid_reference (TypeTagIn) ||
  invalid_pointer (pDataIn) ||
  this->bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, this->iiFifoId) == FALSE ||
  simultaneous put or flush on GetObject (::IpcMemoryG, 
    this->RelayStream.iiFifoId) => UNDEFINED
| this->bInit == FALSE ||
  invalid_semaphore (this->siPutMutexSemaphore) ||
  semaphore_time_out (this->siPutMutexSemaphore,NO_WAIT) => 
    console := console + appropriate error message
    : return := appropriate error code
| GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue.Length + TypeTagIn.szSize +
  sizeof (TypeTagIn) > GetObject (::IpcMemoryG, this->iiFifoId)->iFifoDepth => 
    return := S_ipc_FIFO_FULL
| true => 
    GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue :=
      GetObject (::IpcMemoryG, this->iiFifoId)->dtQueue with *pDataIn and 
      TypeTagIn appended to back
    : return := OK 
]]



[[D tRelayStreamReceiver::tRelayStreamReceiver (const tIpcFifoId& iiId) ]
[ invalid_pointer (this) ||
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE =>
    console := console + appropriate messages
    : this->bObjectValid := FALSE
    : this->iiFifoId := anything
    : this->ecInitError := appropriate error code
    : this->bInit := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->bObjectValid := TRUE
    : this->iiFifoId := iiId
    : ::IpcMemoryG.ObjectSet := 
      ::IpcMemoryG.ObjectSet U NewObjectSet (::IpcMemoryG, iiId ) 
]]


[[D tRelayStreamReceiver::~tRelayStreamReceiver (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tRelayStreamReceiver::Restart (void) ]
[ invalid_pointer (this)  => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning message
    : return := OK
| true =>
   this->bInit := TRUE
   : return OK
   
]]

[[D tRelayStreamReceiver::Shutdown (void) ]
[ invalid_pointer (this)  => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning message
    : return := OK
| true =>
   this->bInit := FALSE
   : return OK
]] 


[[D tRelayStreamReceiver::Get (tTypeTag& TypeTagOut, BYTE* pDataOut) ]
[ invalid_pointer (this) ||
  invalid_reference (TypeTagOut) ||
  invalid_pointer (pDataOut) ||
  this->RelayStream.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, this->RelayStream.iiObjectId) == FALSE ||
  simultaneous get on GetObject (::IpcMemoryG, this->iiFifoId)=>  UNDEFINED
| GetObject (::IpcMemoryG, this->RelayStream.iiFifoId)->dtQueue.Length == 0 => 
    return := S_ipc_FIFO_EMPTY
| true =>
    TypeTagOut := front element of GetObject (::IpcMemoryG, 
        this->RelayStream.iiFifoId)->dtQueue
    : *pDataOut := second element of GetObject (::IpcMemoryG, 
        this->RelayStream.iiFifoId)->dtQueue
    : GetObject (::IpcMemoryG, this->RelayStream.iiFifoId)->dtQueue := 
        GetObject (::IpcMemoryGt, this->RelayStream.iiFifoId)->dtQueue 
        with front and second element removed
    : return := OK 
]]



************/

class tTypeTag;

class tRelayStreamSender : protected tIpcMultiTypeFifoAccessor {
  public:
    tRelayStreamSender (const tIpcFifoId& iiId, int iMutexSemaphoreOptions);
    virtual ~tRelayStreamSender ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
    ERROR_CODE Put (const tTypeTag& TagIn, const BYTE* DataIn);
    ERROR_CODE Put (const tTypeTag& TagIn, const tBuffer& BufferIn);
    ERROR_CODE PutNoWait (const tTypeTag& TagIn, const BYTE* DataIn); 
    ERROR_CODE PutNoWait (const tTypeTag& TagIn, const tBuffer& BufferIn);
    tIpcMultiTypeFifoAccessor::IsFull;
    tBaseObject::GetInitError;  
  protected:
    SEM_ID siPutMutexSemaphore;
  
};

class tRelayStreamReceiver: protected tIpcMultiTypeFifoAccessor {
  public:
    tRelayStreamReceiver (const tIpcFifoId& iiId);
    virtual ~tRelayStreamReceiver ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    tIpcMultiTypeFifoAccessor::IsOk;
    tIpcMultiTypeFifoAccessor::Length;
    tIpcMultiTypeFifoAccessor::IsEmpty;
    tBaseObject::GetInitError;  
    ERROR_CODE Get (tTypeTag& TagOut, BYTE* DataOut);
}
;


#endif INC_relayStreamh

