/* $RCSfile: relayer.h,v $ - Type definitions for relayer stream stuff */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/relayer.h,v 1.1.1.1 1998/06/11 21:36:13 chara Exp $ayer 
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,20Aug97,rlj      updated relayer for new instrument cds
03a,25June97,rlj     converted to new ipc 
02a,25Apr97,rlj      added increment 2 status and data stream servers
01f,28Oct96,eamck    split Relayer.h into Relayer.h and relayStreams.h
01e,17Oct96,eamck    changed ERROR_INFO, added two new Puts and a new PutNoWait
01d,16Oct96,eamck    put DL-specific stuff in separate file
01c,09Oct96,eamck    deleted tRelayStream, added client and server streams
01b,07Oct96,eamck    made tRelayStream more general, added data and status streams
01a,02Oct96,eamck    written 
*/
 
/*
DESCRIPTION
  Relayer stream defines and types
*/
#ifndef INC_relayerh
#define INC_relayerh
   
#include "baseObject.h"

//Errors
#define S_relayer_CANT_ALLOCATE_ERROR_STREAMS           (M_relayer | 0x0001)
#define S_relayer_CANT_ALLOCATE_DATA_STREAMS            (M_relayer | 0x0002)
#define S_relayer_CANT_ALLOCATE_STATUS_STREAMS          (M_relayer | 0x0003)
#define S_relayer_CANT_ALLOCATE_DATA_HEADERS            (M_relayer | 0x0004)
#define S_relayer_CANT_ALLOCATE_STATUS_HEADERS          (M_relayer | 0x0005)
#define S_relayer_UI_NOT_ATTACHED	                (M_relayer | 0x0006)
#define S_relayer_INITIALIZED_ERROR_DATA_AND_STATUS_STREAMS (M_relayer | 0x0007)
#define S_relayer_INTERNAL_ERROR                        (M_relayer | 0x0008)
#define S_setcallback_INVALID_GIZMO                        (M_relayer | 0x0009)
//Warnings

#define S_relayer_ALREADY_INITIALIZED                   (M_relayer | 0x1001)
#define S_relayer_ALREADY_SHUTDOWN                      (M_relayer | 0x1002)
#define  S_relayer_NOT_SHUTDOWN                         (M_relayer | 0x1003)
//Info
#define S_relayer_SENDING_ERROR_TO_UI                   (M_relayer | 0x2001)
#define S_relayer_SENDING_DATA_TO_UI                    (M_relayer | 0x2002)
#define S_relayer_SENDING_STATUS_TO_UI                  (M_relayer | 0x2003)
#define S_relayer_ENTERING_RELAYER_INIT                 (M_relayer | 0x2004)
#define S_relayer_INITIALIZED_ERROR_STREAMS             (M_relayer | 0x2005)
#define S_relayer_INITIALIZED_DATA_AND_STATUS_STREAMS   (M_relayer | 0x2006)
#define S_relayer_CREATED_MSGQS_AND_SPAWNED_MSGQ_TASKS  (M_relayer | 0x2007)
#define S_relayer_LEAVING_RELAYER_INIT                  (M_relayer | 0x2008)
#define S_relayer_TRELAYER_RELAY                        (M_relayer | 0x2009)



/****

[[D tRelayer::tRelayer (tExternalInteface& ExternalInterfaceToUse) ]
[ invalid_pointer (this)  ||
  invalid_reference (ExternalInterfaceToUse) => UNDEFINED
| Unable to allocate or initialize resources to handle data status and error =>
    console := console + appropriate error messages
    : this->ecInitError := appropriate error code
    : this->bInit := anything
    : this->GizmoSet := anything
    : this->ExternalInteface := anything
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Relayer.GizmoSet := {ids of every gizmo}
    : this->ExternalInterface := ExternalInterfaceToUse
]]

[[D tRelayer::~tRelayer (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]


[[D tRelayer::Restart (void)]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE => 
    console := console + appropriate warning messages
    : return := OK
| Unable to resume command, data, status, and error handling =>
    console := console + appropriate error messages 
    : return := appropriate error code
| true => this->bInit := TRUE
]]

[[D tRelayer::Shutdown (void)]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to suspend command, data, status, and error handling =>
    console := console + appropriate error messages
    : return := appropriate error code
| true => this->bInit := FALSE
]]

[[D tRelayer::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED 
| T.E element id of this->GizmoSet s.t. relayer cannot receive data status 
  or error from Gizmo (id) ||
  T.E element id of this->ExternalInterface.UiSet s.t. relayer cannot send
  data status or error to UserInterface (id) => return := FALSE
| true => return := TRUE
]]

	
[[D tRelayer::ErrorSentToRelayer (GIZMO_ID id ERROR_INFO eiError)]
[ invalid_pointer (this) => UNDEFINED
| this->bInit ==  FALSE || 
  this->ExternalInterface.bInit == FALSE||
  id not an element of this->GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet, 
    eiError sent to UserInterface (id)
]]

[[D tRelayer::DataSentToRelayer (GIZMO_ID id, const tTypeTag& Tag, const BYTE* pData)]
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pData) => UNDEFINED
| this->bInit == FALSE || 
  this->ExternalInterface.bInit == FALSE||
  id not an element of this->GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet  Tag and *pData 
    sent to UserInterface (id)
]]

[[D tRelayer::StatusSentToRelayer (GIZMO_ID id, const tTypeTag& Tag, const BYTE* pStatus)]
[ invalid_pointer (this) ||
  invalid_reference (Tag) ||
  invalid_pointer (pStatus) => UNDEFINED
| this->bInit ==  FALSE || 
  this->ExternalInterface.bInit == FALSE ||
  id not an element of this->GizmoSet => no response
| true => For all id elements of this->ExternalInterface.UiSet, Tag and *pStatus 
    sent to UserInterface (id) 
]]

[[D tRelayer::Relay (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_pointer_array (this->pErrorStreamArray) ||  
  invalid_pointer_array (this->pDataStreamArray) ||
  invalid_pointer_array (this->pStatusStreamArray) ||
  invalid_pointer (this->pRelayBuffer) ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, 
    this->pDataStreamArray[0..this->iNumberOfRtes]->
    RelayStream.iiObjectId) == FALSE ||
  simultaneous get on GetObject (::IpcMemoryG, 
    this->pDataStreamArray[0..this->iNumberOfRtes]->iiFifoId) ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, 
    this->pStatusStreamArray[0..this->iNumberOfRtes]->
    RelayStream.iiObjectId) == FALSE ||
  simultaneous get on GetObject (::IpcMemoryG, 
    this->pStatusStreamArray[0..this->iNumberOfRtes]->iiFifoId) ||
  DoesObjectExist (::IpcMemoryG.ObjectSet, 
    this->pErrorStreamArray[0..this->iNumberOfRtes]->
    RelayStream.iiObjectId) == FALSE ||
  simultaneous get on GetObject (::IpcMemoryG, 
    this->pErrorStreamArray[0..this->iNumberOfRtes]->iiFifoId) => UNDEFINED
| true =>
    Contents of ::pThisRteG->ErrorStream sent to all connected user interfaces
    : Contents of ::pThisRteG->DataStream sent to all connected user interfaces
    : Contents of ::pThisRteG->StatusStream sent to all connected user interfaces
]]

****/   


/******************************************************************************
*
* Type definitions
*/

class tExternalInterface;
class tErrorStreamReceiver;
class tRelayStreamReceiver;

class tRelayer : public tBaseObject {
   public:
     tRelayer (tExternalInterface& ExternalInterfaceToUse);
     virtual ~tRelayer ();
     virtual ERROR_CODE Restart ();
     virtual ERROR_CODE Shutdown ();
     virtual BOOL IsOk ();

// kludge for sequencer

     ERROR_CODE SetSeqCallback (int iWhichDl, void (* pSeqCallback) (BOOL bFrontSoft,
                                  BOOL bFrontHard, BOOL bBackSoft, BOOL bBackHard,
                                  BOOL bHome, LASER_VALUE lvLaserValue) );
   protected:
     int iNumberOfRtes;
     int iNumberOfGizmos;
     tExternalInterface& ExternalInterface;
     
   private:
     ERROR_CODE Relay (void );
     tErrorStreamReceiver      **pErrorStreamArray;
     tRelayStreamReceiver      **pStatusStreamArray;
     tRelayStreamReceiver      **pDataStreamArray;
     int                       iRelayTaskId;
     BYTE*                     pRelayBuffer;

// kludge for sequencer
     int iDlIdForCallback;
     void (* pSeqCallback) (BOOL bFrontSoft,
                            BOOL bFrontHard, BOOL bBackSoft, BOOL bBackHard,
                            BOOL bHome, LASER_VALUE lvLaserValue); 
}; 	/* class tRelayer */

#endif INC_relayerh

