/* $RCSfile: servo.h,v $ - implements generic servo and controller functionality */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/servo.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Richard Johsnon - Jet Propulsion Laboratory */
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,28Aug97    re-architectured to eliminate monolithic and partition servo
                   classes
01a,beh,23Apr97    written
*/
 
/*
DESCRIPTION

*/

#include "servoInfo.h"

#ifndef INC_servoh
#define INC_servoh

//Errors
 
#define S_servo_COULD_NOT_ALLOCATE_SERVO_TASK_ARRAY      (M_servo | 0x1)
#define S_servo_INVALID_SERVO_MODE_REQUESTED             (M_servo | 0x2)
#define S_servo_REQUESTED_TASK_PERIOD_MODIFIED           (M_servo | 0x3)
#define S_servo_UNABLE_TO_ALLOCATE_SERVO_TASK_INFO_ARRAY (M_servo | 0x4)
#define S_servo_INVALID_PARTITION                        (M_servo | 0x5)

//Warnings
#define S_tservotask_TASK_ALREADY_RUNNING                (M_servo | 0x1001)
#define S_tservotask_TASK_ALREADY_SHUTDOWN               (M_servo | 0x1002)
#define S_tservo_ALREADY_RUNNING                         (M_servo | 0x1003)
#define S_tservo_ALREADY_SHUTDOWN                        (M_servo | 0x1004)


//Messages

#define S_servotask_CONSTRUCTOR                          (M_servo | 0x2001)
#define S_servotask_ADDED_SERVO_TASK                     (M_servo | 0x2002)
#define S_servotask_DESTRUCTOR                           (M_servo | 0x2003)
#define S_servotask_DOSERVO                              (M_servo | 0x2004)
#define S_servotask_EXITING_DOSERVO                      (M_servo | 0x2005)
#define S_servo_CONSTRUCTOR_BEGIN                        (M_servo | 0x2006)
#define S_servo_TASK_ARRAY_CONSTRUCTOR_COMPLETE          (M_servo | 0x2007)
#define S_servo_SELECT_MODE                              (M_servo | 0x2008)
#define S_servotask_DELETED_SERVO_TASK                   (M_servo | 0x2009)
#define S_servo_TASK_ARRAY_ALLOCATED                     (M_servo | 0x2008)

/****
Black Box State Variables
______________________________
tServoTask

int             iPartitionNumber
int             iServoTaskId
tServo&         Servo
tServoTaskInfo& ServoTaskInfo

tServo

tServoInfo&       ServoInfo
tControllerSet&   ControllerSet
Array<tServoTask> ServoTaskArray
___________________________________________

Stimuli


[[D tServoTask::tServoTask (char *sTaskName, tServoTaskInfo& TaskInfoToUse, 
                            int iPartitionNumberToUse, 
                            tControllerSet& ControllerSetToUse) ]
[ invalid_pointer (this) ||
  invalid_string (sTaskName) ||
  invalid_reference (ControllerSetToUse) ||
  invalid_reference (TaskInfoToUse) || 
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| Unable to add periodic task to ::PeriodicTaskSetG =>
    console := console + appropriate error messages
    : this->ecInitError := appropriate error code
    : *this := anything
| TaskInfoToUse.tTaskPeriod not even multiple of the base periodic task period =>
    console := console + appropriate warning messages 
    : ::PeriodicTaskSetG := ::PeriodicTaskSetG U NewPeriodicTask (sTaskName, 
      this->DoServo,NULL,NULL,TaskInfoToUse.iTaskPriority,
      modified period,0,TEB_DISABLE, TRUE) 
    : this->iServoTaskId := appropriate periodic task id
    : this->ServoTaskInfo := ServoTaskInfoToUse
    : this->ServoTaskInfoToUse.tTaskPeriod := modified to an even multiple of 
      the base task period
    : this->ControllerSet := ControllerSetToUse
    : this->iPartitionNumber := iPartitionNumberToUse
    : this->ecInitError := OK
    : this->bInit := TRUE
| true =>
    ::PeriodicTaskSetG := ::PeriodicTaskSetG U NewPeriodicTask (sTaskName, 
      this->DoServo,NULL,NULL,TaskInfoToUse.iTaskPriority,
      TaskInfoToUse.tTaskPeriod,0,TEB_DISABLE, TRUE)
    : this->iServoTaskId := appropriate periodic task id
    : this->ServoTaskInfo := ServoTaskInfoToUse
    : this->ControllerSet := ControllerSetToUse
    : this->iPartitionNumber := iPartitionNumberToUse
    : this->ecInitError := OK
    : this->bInit := TRUE
]]

[[D tServoTask::~tServoTask (void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to delete PeriodicTask (this->iServoTaskId) from ::PeriodicTaskSetG =>
    console := console + appropriate error messages
    : *this := anything
| true =>
    ::PeriodicTaskSetG := ::PeriodicTaskSetG - PeriodicTask (this->iServoTaskId)
    : *this := anything
]]

[[D tServoTask::Restart (void) ]
[ invalid_pointer (this) ||
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate error messages
    : return := OK
| Unable to resume PeriodicTask (this->iServoTaskId)  =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    PeriodicTask (this->iServoTaskId) resumed 
    : this->bInit := TRUE
]]

[[D tServoTask::Shutdown (void) ]
[ invalid_pointer (this) ||
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate error messages
    : return := OK
| Unable to suspend PeriodicTask (this->iServoTaskId)  =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    PeriodicTask (this->iServoTaskId) suspended
    : this->bInit := FALSE
]]

[[D tServoTask::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->ecInitError != OK => return := FALSE
| true => return := TRUE
]]

[[D tServoTask::DoServo (TIME tServoTime) ]
[ invalid_pointer (this) => UNDEFINED
|  control law calculation error  =>
    *this := anything
    : return appropriate error code
| true =>
    this->ControllerSet.pCurrentControllerGroup->ControllerArray 
      [this->iPartitionNumber].OutputSet := 
      ControlLaw (this->ControllerSet.PreCalculateController.InputSet,
        this->ControllerSet.PreCalculateController.OutputSet,
        this->ControllerSet.PreCalculateController.TargetSet) +
      ControlLaw (this->ControllerSet.pCurrentControllerGroup->
        ControllerArray[this->iPartitionNumber].InputSet updated appropriately with
        tServoTime,
        this->ControllerSet.pCurrentControllerGroup->ControllerArray 
        [this->iPartitionNumber].OutputSet updated appropriately with tServoTime,
        this->ControllerSet.pCurrentControllerGroup->ControllerArray 
        [this->iPartitionNumber].TargetSet updated appropriately with tServoTime)  
        updated appropriately with tServoTime +
      ControlLaw (this->ControllerSet.PostCalculateController.InputSet,
        this->ControllerSet.PostCalculateController.OutputSet,
        this->ControllerSet.PostCalculateController.TargetSet)
    : return := OK
]]


[[D tServo::tServo (tServoInfo& ServoInfoToUse,
                    tControllerSet& ControllerSetToUse) ]
[ invalid_pointer (this) ||
  invalid_reference (ServoInfoToUse) ||
  invalid_reference (ControllerSetToUse) => UNDEFINED 
| true =>
    this->ServoInfo := ServoInfoToUse
    : this->ControllerSet := ControllerSetToUse
    : this->ecInitError := OK
]]

[[D tServo::~tServo (void) ]
[ invalid_pointer (this) => UNDEFINED 
| Unable to delete PeriodicTask (this->ServoTaskArray[0..this->iServoInfo.
  iNumberOfPartitions].iServoTaskId ) =>
    console := console + appropriate error messages
    : *this := anything
| true =>
    ::PeriodicTaskSetG := ::PeriodicTaskSetG - 
      PeriodicTask (this->ServoTaskArray[0..this->iServoInfo.
      iNumberOfPartitions].iServoTaskId )
    : *this := anything
]]


[[D tServo::Restart (void) ]
[ invalid_pointer (this) ||
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate error messages
    : return := OK
| Unable to resume PeriodicTask (this->ServoTaskArray[0..GizmoInfo.iNumberOfPartitions].
  iServoTaskId)  =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    PeriodicTask (this->ServoTaskArray[0..GizmoInfo.iNumberOfPartitions].
      iServoTaskId) resumed 
    : this->bInit := TRUE
]]

[[D tServo::Shutdown (void) ]
[ invalid_pointer (this) ||
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate error messages
    : return := OK
| Unable to suspend PeriodicTask (this->ServoTaskArray[0..GizmoInfo.iNumberOfPartitions].
      iServoTaskIdt) =>
    console := console + appropriate error messages
    : *this := anything
| true => 
    PeriodicTask (this->ServoTaskArray[0..GizmoInfo.iNumberOfPartitions].
      iServoTaskId) suspended
    : this->bInit := FALSE
]]

[[D tServoTask::IsOk (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->ecInitError != OK => return := FALSE
| true => return := TRUE
]]
 
[[D tServo::StartServo (void) ]
[ invalid_pointer (this)||
  invalid_string (this->ServoInfo.sTaskName) ||
  ::PeriodicTaskSetG.bInit == FALSE => UNDEFINED
| Unable to dimension this->ServoTaskArray with 
  this->ServoInfo.iNumberOfPartitions elements =>
    return := appropriate error code
| Unable to add this->ServoInfo.iNumberOfPartitions periodic tasks to 
  ::PeriodicTaskSetG =>
    console := console + appropriate error messages
    : *this := anything
    : ::PeriodicTaskSetG := ::PeriodicTaskSetG U anything
    : return := appropriate error messages
| this->ServoInfo.TaskInfoArray[0..this->ServoInfo.iNumberOfPartitions].
  TaskInfoToUse.tTaskPeriod not even multiple of the base periodic 
  task period =>
    console := console + appropriate warning messages 
    : ::PeriodicTaskSetG := ::PeriodicTaskSetG U 
      NewPeriodicTask (this->ServoInfo.sTaskName, 
      appropriate tServoTask::DoServo,NULL,NULL,
      this->ServoInfo.TaskInfoArray [0..this->ServoInfo.iNumberOfPartitions].
      TaskInfo.iTaskPriority,
      modified periods, 0, TEB_DISABLE, TRUE) 
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].
      iServoTaskId := appropriate periodic task id
    : this->ServoTaskInfoToUse.tTaskPeriod := modified to an even multiple of 
      the base task period
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].Servo := 
      ServoToUse
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].
      iPartitionNumber := [0..this->ServoInfo.iNumberOfPartitions]
    : this->bInit == TRUE
| true =>
    ::PeriodicTaskSetG := ::PeriodicTaskSetG U 
      NewPeriodicTask (this->ServoInfo.sTaskName, 
      appropriate tServoTask::DoServo,NULL,NULL,
      this->ServoInfo.TaskInfoArray [0..this->ServoInfo.iNumberOfPartitions].
      TaskInfo.iTaskPriority,
      this->ServoInfo.TaskInfoArray [0..this->ServoInfo.iNumberOfPartitions].
      TaskInfo.tTaskPeriod,0,TEB_DISABLE, TRUE)
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].
      iServoTaskId := appropriate periodic task id
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].Servo := 
      ServoToUse
    : this->ServoTaskArray[0..this->ServoInfo.iNumberOfPartitions].
      iPartitionNumber := [0..this->ServoInfo.iNumberOfPartitions]
    : this->bInit == TRUE
]]                

[[D tServo::SelectMode (int iWhichServoMode) ]
[ invalid_pointer (this) => UNDEFINED
| InvalidIndex (this->ControllerSet.ControllerGroupArray,iWhichServoMode) =>
    return := appropriate error code
| true =>
    this->ControllerSet.iCurrentControllerGroup := iWhichServoMode
    : return := OK
]]

[[D tServo::GetMode (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ControllerSet.iCurrentControllerGroup
]]


[[D tServo::ControllerMode (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->ControllerSet.
    ControllerGroupArray [iCurrentControllerGroup].iCurrentControllerMode
]]

[[D tServo::GetControllerGroup (void) ]
[ invalid_pointer (this) ||
  invalid_pointer (this->ControllerSet.pCurrentControllerGroup) => UNDEFINED
| true => return := *this->ControllerSet.pCurrentControllerGroup
]]
*****/

#include "baseObject.h"

class tControllerGroup;
class tControllerSet;
class tServo;



class tServoTask : public tBaseObject {
  public:
    tServoTask(char *sTaskName, const  tServoTaskInfo& TaskInfoToUse,
               int iPartitionNumberToUse,tControllerSet& ControllerSetToUse);
    virtual ~tServoTask(void);
    ERROR_CODE Restart ();
    ERROR_CODE Shutdown ();
    BOOL IsOk ();
    ERROR_CODE DoServo(TIME tServoTime);
  
  protected:
    tServoTaskInfo ServoTaskInfo;
    int iPartitionNumber;
    tControllerSet& ControllerSet;
  private:
    int iServoTaskId;
};


class tServo : public tBaseObject {
  public:
    tServo (const tServoInfo& ServoInfoToUse,
            tControllerSet& ControllerSetToUse);
    ~tServo ();
    ERROR_CODE Restart ();
    ERROR_CODE Shutdown ();
    BOOL IsOk ();
    ERROR_CODE SelectMode (int iServoMode);
    int GetMode () const;
    int ControllerMode () const;
    ERROR_CODE StartServo ();
  protected:
    tServoInfo ServoInfo;
    tControllerSet &ControllerSet;
    tServoTask **pServoTaskArray;
};



#endif  /* INC_servoh */


