/* $RCSfile: servoInfo.h,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/include/servoInfo.h,v 1.2 1998/08/17 21:01:17 phil Exp $
*/
/* Richard Johsnon - Jet Propulsion Laboratory */
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,2Sept97    written
                   
*/

#ifndef INC_servoInfoh
#define INC_servoInfoh

#include "isit.h"

#define S_servoinfo_UNABLE_TO_ALLOCATE_SERVO_TASK_INFO_ARRAY  (M_servoinfo | 0x1)
#define S_servoinfo_INVALID_PARTITION                         (M_servoinfo | 0x2)

/****
[[D tServoTaskInfo::tServoTaskInfo (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tServoTaskInfo::tServoTaskInfo_ti (TIME tPeriod, int iPriority) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->tTaskPeriod := tPeriod
    : this->iTaskPriority := iPriority
]]


[[D tServoInfo::tServoInfo (char* sServoNameToUse, int iNumberOfPartitonsToUse) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->sServoName := sServoNameToUse
    : this->iNumberOfPartitions := iNumberOfPartitionsToUse
    : this->TaskInfoArray := ARRAY [iNumberOfPartitionsToUse]
]]

[[D tServoInfo::AssignServoTaskInfo (int iPartitionNumber,TIME tPeriod, int iPriority) ]
[ invalid_pointer (this) => UNDEFINED
| InvalidIndex (iPartitionNumber,this->TaskInfoArray) => 
    console := console + appropriate error code
    : return := appropriate error code
| true => 
    this->TaskInfoArray [iPartitionNumber].tTaskPeriod := tPeriod
    : this->TaskInfoArray [iPartitionNumber].iTaskPriority := iPriority
    : return := OK
]]

[[D tServoInfo::GetServoTaskInfo (int iWhichPartition) ]
[ invalid_pointer (this) || 
  InvalidIndex (iWhichPartition,this->TaskInfoArray) => UNDEFINED
| true => return := this->TaskInfoArray [iWhichPartition]
]]

[[D tServoInfo::GetNumberOfPartitions (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => return := this->iNumberOfPartitions
]]


[[D tServoInfo::GetServoName (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => return := this->sServoName
]]

******/

class tServoTaskInfo {
  public:
    tServoTaskInfo ();
    tServoTaskInfo (TIME tPeriod, TIME tOffset, int iPriority);

    int  iTaskRate;
    TIME tTaskPeriod;
    TIME tOffset;
    int  iTaskPriority;
};


class tServoInfo {
  public:
    tServoInfo (const char* sServoNameToUse, int iNumberOfPartitonsToUse);
    tServoInfo (const tServoInfo& ServoInfo);
    ~tServoInfo ();
    ERROR_CODE AssignServoTaskInfo (int iWhichPartition,
                                    TIME tPeriod, int iPriority );
    ERROR_CODE AssignServoTaskInfo (int iWhichPartition,
                                    TIME tPeriod, TIME tOffset, int iPriority );
    const tServoTaskInfo& GetServoTaskInfo (int iWhichPartition) const;
    int GetNumberOfPartitions () const;
    char* GetServoName () const;
  protected:
    int iNumberOfPartitions;
    char* sServoName;
    tServoTaskInfo* ServoTaskInfoArray;
};      


#endif INC_servoInfoh
