/* $RCSfile: configShare.cc,v $ - rteConfigShare.cc */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/share/configShare.cc,v 1.3 1998/08/17 21:01:21 phil Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b,29May97,rlj - Configured for first I2 build
01a,13May97,ky - creation
*/

/*
DESCRIPTION
  code share between rte and GUI
*/


#include "share/configShare.h"


/*******************************************************************/
/*******************  Shared System Config Info ********************/
/*******************************************************************/


const int CiNumInstruments = 1;
const int CiNumFringeTrackers = 0;
const int CiNumExtMets = 0;
const int CiNumStarTrackers = 0;

const int CiStarImageHSize = 0;
const int CiStarImageVSize = 0;
const int CiFringeImageHSize = 0;
const int CiFringeImageVSize = 0;

// SYSTEM GIZMO INDICES - every gizmo has a unique integer identifier that
// corresponds to the position in the system gizmo info array of a particular
// gizmo`s configuration information  IMPORTANT:: THESE NUMBERS MUST MATCH THE
// INDEX OF THE CORRECT GIZMO INFO IN THE SYSTEM GIZMO ARRAY

const int INVALID_GIZMO = -1;
const int CiInst = 0;
const int CiDlGsu0= 1;
const int CiDlGsu1= 2;
const int CiDlGsu2= 3;
const int CiDlGsu3= 4;
const int CiDlGsu4= 5;
//const int CiDlBrassBoard2 = 6;
//const int CiFt0 = 2;
//const int CiEm0 = 3;
//const int CiSt0 = 4;
//const int CiSt1 = 5;
//const int CiDlBrassBoard1 = 7;
//const int CiFt1 = 8;
//const int CiEm1 = 9;
//const int CiFt2 = 10;
//const int CiEm2 = 11;
//const int CiWap0 = 12;
//const int CiWap1 = 13;

// GUI GIZMOs - the GUI numbers gizmos by type so that
// if there are n delay lines DL0 = 0 and DLn = n etc
// These gui indexes are mapped into system wide gizmo id numbers using the
// constant arrays below


const int CiGuiDls[] = {CiDlGsu0, CiDlGsu1, CiDlGsu2, CiDlGsu3, CiDlGsu4};
const int CiGuiFts[] = { };
const int CiGuiEms[] = { };
const int CiGuiSts[] = { };

// RTE GIZMO CONFIGURATION

const int CiRte0Gizmos [NUM_RTE0_GIZMOS] = { CiInst, CiDlGsu0 };
const int CiRte1Gizmos [NUM_RTE1_GIZMOS] = { CiDlGsu1, CiDlGsu2 };
const int CiRte2Gizmos [NUM_RTE2_GIZMOS] = { CiDlGsu3, CiDlGsu4 };
const int CiRte3Gizmos [NUM_RTE3_GIZMOS] = { };
const int CiRte4Gizmos [NUM_RTE4_GIZMOS] = { };

const GIZMO_TYPE CgtSystemGizmoTypes[NUM_SYS_GIZMOS] = {
  GT_INSTRUMENT, GT_DL_GSU, GT_DL_GSU, GT_DL_GSU, GT_DL_GSU, GT_DL_GSU
};

const int CiGizmoSubindexArray[NUM_SYS_GIZMOS] = {
  0, 0, 1, 2, 3, 4
};

GIZMO_TYPE PubGetGizmoType (int iGizmoId) {
  if ( (iGizmoId < 0) || (iGizmoId >= NUM_SYS_GIZMOS) ) return GT_UNKNOWN;
  else return CgtSystemGizmoTypes[iGizmoId];
}


void GetEmbeddedSystemGizmoInfo (GIZMO_TYPE gtGenericType, int iGuiIndex,
                                GIZMO_TYPE& gtSpecificType, int& iGizmoId) {

  switch (gtGenericType) {
    case GT_DELAY_LINE : gtSpecificType = PubGetGizmoType (CiGuiDls[iGuiIndex]);
                         iGizmoId = CiGuiDls [iGuiIndex];
                         break;

    case GT_FRINGE_TRACKER : gtSpecificType = PubGetGizmoType (CiGuiFts[iGuiIndex]);
                             iGizmoId = CiGuiFts [iGuiIndex];
                             break; 

    case GT_EXTERNAL_METROLOGY : gtSpecificType = PubGetGizmoType (CiGuiEms[iGuiIndex]);
                         iGizmoId = CiGuiEms [iGuiIndex];
                         break;
    
    case GT_STAR_TRACKER : gtSpecificType = PubGetGizmoType (CiGuiSts[iGuiIndex]);
                         iGizmoId = CiGuiSts [iGuiIndex];
                         break;

    case GT_INSTRUMENT : gtSpecificType = GT_INSTRUMENT;
                         iGizmoId = CiInst;
                         break;

    default : gtSpecificType = GT_UNKNOWN;
              iGizmoId = INVALID_GIZMO;
  };

}

