/* $RCSfile: sharedMem.h,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/include/sharedMem.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $ */

/*
modification history
--------------------
01a,2Jun97,rlj    written
*/

#ifndef INC_sharedMemh
#define INC_sharedMemh

#include "isit.h"
#include "ipcMem.h"
#include "memHeader.h"


#define S_ipc_TOKEN_SERVER_COULD_NOT_BE_CREATED 1

/******************************************************************************
Class: tSharedMemory 
*****************************************************************************



[[D tSharedMemory::tSharedMemory (volatile SHMEM_WORD *vswSharedMemoryStart, 
                   const int &iNumSharedMemoryBytes, const int &iNumRtes ) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->bInit := TRUE
    : this->ObjectSet := anything
    : this->szHeapFree := anything
]]


[[D tSharedMemory::~tSharedMemory (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]  

[[D tSharedMemory::Allocate (IPC_NAME inName, IPC_TYPE itType, 
    			    size_t szSize, ERROR_CODE &ecErrCode) ]
[ invalid_pointer (this) ||
  this->bStaticInit == FALSE => UNDEFINED
|  CanAllocateObject (mtMemType, inName,itType,szSize + overhead) == FALSE =>
    ecErrCode := appropriate error code
    : return := NULL
| true =>
    ecErrCode := OK
    : this->ObjectSet := this->ObjectSet U 
      NewObjectSet (this->ObjectSet,MT_SHARED,inName,itType)
    : this->szHeapFree := this->szHeapFree - 
      AllocationSize (this->ObjectSet,inName,szSize)
    : return := address of appropriate tIpcObject
]]

[[D tSharedMemory::GlobalInit (int iServerTimeOutInSeconds) ]
[ invalid_pointer (this) ||
  this->bStaticInit == TRUE => UNDEFINED
| cannot spawn token server task => return := appropriate error code
| true =>
    this->bStaticInit := TRUE
    : this->ObjectSet := NULL
    : this->szHeapFree := appropriate size
    : token server task spawned
    : return := OK
]]

[[D tSharedMemory::GlobalShutdown (void) ]
[ invalid_pointer (this) ||
  this->bStaticInit == FALSE => UNDEFINED
| cannot delete token server task => 
    console := console + appropriate error code
    : return := appropriate error code
| true =>
    this->bStaticInit := FALSE
    : token server task deleted
    : return := OK
]]


[[D tSharedMemory::GetTokenArea (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->vtfTokenArray
]]

********************************************************/

const int CiTokenServerTimeOutInSeconds = 5;


class tSharedMemory : public tMemory {
  public:
    tSharedMemory (volatile SHMEM_WORD *vswSharedMemoryStart, 
                   const int &iNumSharedMemoryBytes, const int &iNumRtes );
    virtual ~tSharedMemory ();
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    volatile TOKEN_FLAG *GetTokenArea () { return vtfTokenArray;}
    virtual void *Allocate (IPC_NAME inName, IPC_TYPE itType, 
    			    size_t szSize, ERROR_CODE &ecErrCode);
    ERROR_CODE GlobalInit (int iServerTimeOutInSeconds );
    ERROR_CODE GlobalShutdown ();
    friend ostream &operator<< (ostream& stream,const tSharedMemory &ShareMem);

  protected:
    BOOL EnoughMemory (size_t szSize);
    int SizeInShmemWords (size_t SizeInBytes);
    int iNumRtes;
    int iNumSharedMemoryBytes;
//    int iTokenTaskId;  		//PCI 7/8/97
    volatile SHMEM_WORD *vswSharedMemory;
    volatile int *viNumObjects;
    volatile size_t *vszNumBytesFree;
    volatile int *viLastObjectOffset;
    volatile TOKEN_FLAG *vtfTokenArray;
    volatile tSharedHeader *vHeaderList;
    volatile SHMEM_WORD *vswObjectList;
};    

#endif INC_sharedMemh
