/* $RCSfile: stController.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/st/stController.h,v 1.1.1.1 1998/06/11 21:36:17 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  27Aug97,rlj    written
*/

#ifndef stControllerh
#define stControllerh

#include "isit.h"
#include "stTypes.h"
#include "controller.h"
#include "controllerSet.h"
#include "dataStore.h"
#include "matrix.h"
#include "multiActuator.h"
#include "pidLibF.h"
#include "lagLib.h"

/*******
Black Box State Variables
--------------------------

tStControllerData 

tStController
tStTrackController
tStCompleteControllerSet

------------------------------------------------
Spec Fns


One-dimensional summing function:

   Sum( func(a), a, min_a, max_a ) 
         is equivalent to

   for (a = min_a, Sum = 0; a < max_a; a++) 
      Sum += func(a);


Two-dimensional summing function:

   Sum( func(a,b), a, min_a, max_a, b, min_b, max_b ) 
         is equivalent to

   for (a = min_a, Sum = 0; a < max_a; a++) 
      for (b = min_b; b < max_b; b++) 
    Sum += func(a,b);


------------------------------------------------
Stimuli

******************************************************************************
**********************  class tStControllerData  *****************************
******************************************************************************

[[D tStControllerData::tStControllerData(tImageStatus& StarImageStatusToUse,
                     			 const tTypeTag& StarPosDataTag,
                     			 const tTypeTag& FsmPosDataTag,
                								  const tTypeTag& StarImageDataTag) ] 
[ invalid_pointer(this) ||
    invalid_reference(StarImageStatusToUse) ||
       invalid_reference (StarPosDataTag) ||
          invalid_reference (StarImageDataTag) => UNDEFINED
| CiStDataDumpPriority < priority st. 
       (Time needed to empty this->StarPositionData.DataBag) < 
          ( CiStDataBufferSize / (avg rate at which tDataStore::StoreData called) ) ||
    CiStDataDumpPriority < priority st. 
       (Time needed to empty this->StarImageData.DataBag) < 
          ( CiStDataBufferSize / (avg rate at which tDataStore::StoreData called) ) =>
     ::ErrorStream := ::ErrorStream + appropriate error 
                messages whenever storing data would cause this->StarPositionData.DataBag 
                        or this->StarImageData.DataBag to overflow
| true =>
     this->StarPositionData.DataBag := EMPTY
     : this->StarPositionData.iMaxItems := CiStDataBufferSize
     : this->StarPositionData.iDataDumpPriority := CiStDataDumpPriority
     : this->StarPositionData.DataTag := StarPosDataTag 
     : this->FsmPositionData.iMaxItems := CiStDataBufferSize
     : this->FsmPositionData.iDataDumpPriority := CiStDataDumpPriority
     : this->FsmPositionData.DataTag := FsmPosDataTag 
     : this->StarImageData.DataBag := EMPTY
     : this->StarImageData.iMaxItems := CiStDataBufferSize
     : this->StarImageData.iDataDumpPriority := CiStDataDumpPriority
     : this->StarImageData.DataTag := StarImageDataTag 
     : this->StarImageStatus := StarImageStatusToUse 
]]


******************************************************************************
**********************  class tStController  *********************************
******************************************************************************

[[D tStController::tStController(tStCompleteIoSet& CompleteIoSet,
                                 tStControllerData& ControllerDataToUse) ]
[ invalid_pointer(this) ||
  invalid_reference(ControllerDataToUse) ||
  invalid_reference(CompleteIoSet) => UNDEFINED
| true => 
   this->ControllerData := reference to ControllerDataToUse
   : this->StarSensor := reference tp ControllerIoSet.StarSensor
]]

[[D tStController::~tStController(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => *this := anything
]]

[[D tStController::Calculate(TIME tServoTime) ]
calculate centroid and write to data stream
[ invalid_pointer(this) ||
  invalid_reference(this->StarSensor.StarImage) ||
  this->StarSensor.StarImageBuffer.bObjectValid == FALSE ||
  DoesObjectExist(::IpcMemory,
        this->StarSensor.StarImageBuffer.iiIndirectionDblBufferId) == FALSE ||
  invalid_pointer (&this->ControllerData.StarImageStatus) => UNDEFINED

| (this->StarSensor.StarImageBuffer.bInit == FALSE) ||
      (this->StarSensor.StarImageBuffer.szBufferSize !=
                 this->StarSensor.StarImage.szBufferSize) =>
    appropriate error code sent to RTE error stream
    : return := appropriate error code 

| dimensions of this->ControllerData.StarImageStatus.Image != 
                                                   dimensions of StarImage =>
    image pointed to by this->StarSensor.StarImage.pImage := 
   GetObject(::IpcMemory,
            this->StarSensor.StarImageBuffer.iiIndirectionDblBufferId)->Buffer
    : this->ControllerData.StarPosition := coords of intensity centroid of 
                        this->StarSensor.StarImage.pImage 
    : return := appropriate error code

| (dimensions of this->ControllerData.StarImageStatus.Image != dimensions 
         of StarImage) &&  (this->StarSensor.StarImageBuffer.bEmpty == TRUE) =>
    image pointed to by this->StarSensor.StarImage.pImage := default image
    : this->ControllerData.StarPosition := coords of intensity centroid of 
                                                       default image 
    : return := appropriate error code

| this->StarSensor.StarImageBuffer.bEmpty == TRUE => 
    image pointed to by this->StarSensor.StarImage.pImage := default image
    : this->ControllerData.StarPosition := coords of intensity centroid of 
                                                       default image 
    : this->ControllerData.StarImageStatus.Image:= default image
    : return := OK 

| true =>
    image pointed to by this->StarSensor.StarImage.pImage := 
      GetObject(::IpcMemory,
            this->StarSensor.StarImageBuffer.iiIndirectionDblBufferId)->Buffer
    : this->ControllerData.StarPosition := coords of intensity centroid of 
      GetObject(::IpcMemory,
            this->StarSensor.StarImageBuffer.iiIndirectionDblBufferId)->Buffer                         
    : this->ControllerData.StarImageStatus.Image:= StarImage
    : return := OK 
]]

[[D tStController::CalculateCentroid(const tImage&  TheImage,
                                     tPoint& StarPosition ) ]
[ invalid_pointer(this) ||
    invalid_reference(TheImage) ||
      invalid_reference(StarPosition) => UNDEFINED
| sum of all pixel values in image <= 0.0 =>
    StarPosition.dXPosition := 0.0
    : StarPostion.dYPosition := 0.0
| true =>
    StarPosition.dXPosition := 
       Sum( x * TheImage.Image[x,y], x, 0, TheImage.iHSize, 
                                          y, 0, TheImage.iVSize ) / 
       Sum( TheImage.Image[x,y], x, 0, TheImage.iHSize,
                  y, 0, TheImage.iVSize )
    : StarPostion.dYPosition := 
       Sum( y * TheImage.Image[x,y], x, 0, TheImage.iHSize, 
                                          y, 0, TheImage.iVSize ) / 
       Sum( TheImage.Image[x,y], x, 0, TheImage.iHSize,
                  y, 0, TheImage.iVSize )
]]

[[D tStController::StarPosition(void) ]
return last star position
[invalid_pinter(this) => UNDEFINED
| true => return := this->StarPosition_
]]


******************************************************************************
**********************  class tStTrackController  ****************************
******************************************************************************

[[D tStTrackController::tStTrackController(tStCompleteIoSet& CompleteIoSet,
                                           tStControllerData& ControllerDataToUse,
                                           TIME tServoPeriod) ]
[ invalid_pointer(this) ||
    invalid_reference(CompleteIoSet) ||
    invalid_reference(ControllerDataToUse) => UNDEFINED
| true => 
   this->ControllerData := reference to ControllerDataToUse 
   : this->StarSensor := reference tp ControllerIoSet.StarSensor
   : this->Fsm := reference to CompleteIoSet.Fsm
   : this->CServoPeriod := tServoPeriod
   : this->bLagEnabled := FALSE 
]]

[[D tStTrackController::~tStTrackController(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
     *this := anything
]]

[[D tStTrackController::Calculate(TIME tServoTime) ]
[ invalid_pointer(this) ||
  invalid_actuator(this->Fsm) => UNDEFINED
| this->bLagEnabled =>
   this->ControllerData.StarPosition := coords of intensity centroid of 
                        this->StarSensor.StarImage.pImage 
   : this->ControllerData.StarImageStatus.Image:= StarImage
   : this->Lag := UpdateLag(this->Lag,
      Centroid (this->StarSensor.StarImage,this->StarPosition) - this->StarTarget)
   : this->Pid := UpdatePid(this->Pid,LAG(this->Lag, 
      Centroid(this->StarSensor.StarImage,this->StarPosition) - this->StarTarget))
   : this->Fsm.Position = PID(this->Pid,LAG(this->Lag,
      Centroid(this->StarSensor.StarImage,this->StarPosition) - this->StarTarget))
   : DataStream := DataStream + Centroid(this->StarSensor.StarImage,this->StarPosition) + 
      this->StarImage + PID(this->Pid,LAG(this->Lag,
      Centroid(this->StarSensor.StarImage,this->StarPosition) - this->StarTarget))                      
   : return := OK
| true =>
   this->ControllerData.StarPosition := coords of intensity centroid of 
                        this->StarSensor.StarImage.pImage 
   :  this->ControllerData.StarImageStatus.Image:= StarImage
   : this->Pid := UpdatePid(this->Pid,
      Centroid(this->StarSensor.StarImage,this->StarPosition) - this->StarTarget)
   : this->Fsm.Position = PID(this->Pid,
      Centroid(this->StarSensor.StarImage,this->StarPosition) - this->StarTarget)
   : DataStream := DataStream + 
      Centroid(this->StarSensor.StarImage,this->StarPosition) + 
      this->StarImage + 
      PID(this->Pid,Centroid(this->StarSensor.StarImage,this->StarPosition) - 
            this->StarTarget)                        
   : return := OK
]]


[[DtStTrackController::Init(TIME tServoTime) ]
reset controller memory
[ invalid_pointer(this) => UNDEFINED
| true =>
   this->Lag := (0,0)
   : this->Pid := (0,0)
   : return := OK
]]


[[DtStTrackController::Reset(TIME tServoTime) ]
Send FSM (0,0) to WAP
[ invalid_pointer(this) => UNDEFINED
| true =>
   Wap := Wap + (0,0)
   : return := OK
]]


[[DtStTrackController::SetControllerParams(const tStControllerParams& Params) ]
set controller parameters
[ invalid_pointer(this) => UNDEFINED
| invalid_reference(Params) => UNDEFINED
| true =>
   initialize this->Lag with Params
   : initialize this->Pid with Params
   : this->bLagEnabled := Params.LagEnabled
]]

******************************************************************************
**********************  class tStCalibrateController  ************************
******************************************************************************



[[D tStCalibrateController::tStCalibrateController(tStCompleteIoSet& CompleteIoSet,
                                                   tStControllerData& ControllerDataToUse) ]
constructor
[  invalid_pointer(this) ||
   invalid_reference(CompleteIoSet) ||
   invalid_reference(ControllerData) => UNDEFINED
| true =>
   this->ControllerData := reference to ControllerDataToUse
   : this->StarSensor := reference tp ControllerIoSet.StarSensor
   : this->Fsm := reference to CompleteIoSet.Fsm
]]

[[D tStCalibrateController::Calculate(TIME tServoTime) ]
Move FSM to new calibration position
[  invalid_pointer(this) ||
   invalid_actuator(this->Fsm) ||
   invalid_reference(this->StarSensor) => UNDEFINED
|  true =>
   DataStream := DataStream + 
      Centroid(this->StarSensor.StarImage,this->StarPosition )
   : DataStream := DataStream + StarImage
   : move actuator this->iCalibrationActuator to
      MODULATION(this->pActuatorArray[this->iCalibrationActuator]->
         Modulation.mpParams,tServoTime)
   : return := OK 
]]

******************************************************************************
**********************  class tStCompleteControllerSet  **********************
******************************************************************************

[[D tStCompleteControllerSet::tStCompleteControllerSet(
                                                 tStCompleteIoSet& CompleteIoSetToUse, 
                                                 tImageStatus& ImageStatusToUse,
                                                 const tTypeTag& StarPosDataTag, 
                                                 const tTypeTag& FsmPosDataTag, 
                                                 const tTypeTag& StarImageDataTag,
                                                 TIME tServoPeriod) ]
[ invalid_pointer(this) ||
    invalid_reference(CompleteIoSetToUse) ||
    invalid_reference(ImageStatusToUse) ||
    invalid_reference(StarPosDataTag) ||
    invalid_reference(StarImageDataTag) ||
    invalid_reference(FsmPosDataTag) => UNDEFINED

| CiStDataDumpPriority < VxWorks priority needed to empty ::DataStream(::pThisRteG) 
      before this->ControllerData.StarPositionData.DataBag or 
          this->ControllerData.StarImageData.DataBag overflows =>
    ::ErrorStream := ::ErrorStream + 
           appropriate error messages whenever time to store data would cause
               this->ControllerData.StarPositionData.DataBag
                    or this->ControllerData.StarImageData.DataBag to overflow
    : other assignments in "true" case probably work
| true =>
    this->ControllerData.StarPositionData.DataBag := EMPTY
    : this->ControllerData.StarPositionData.iMaxItems := CiStDataBufferSize
    : this->ControllerData.StarPositionData.iDataDumpPriority := CiStDataDumpPriority
    : this->ControllerData.StarPositionData.DataTag := StarPosDataTag
    : this->ControllerData.FsmPositionData.DataTag := FsmPosDataTag
    : this->ControllerData.StarImageData.DataBag := EMPTY
    : this->ControllerData.StarImageData.iMaxItems := CiStDataBufferSize
    : this->ControllerData.StarImageData.iDataDumpPriority := CiStDataDumpPriority
    : this->ControllerData.StarImageData.DataTag := StarImageDataTag
    : this->ControllerData.StarImageStatus := ImageStatusToUse 
    : this->OffController.ControllerData := ControllerData 
    : this->TrackController.ControllerData := ControllerData
    : this->TrackController.StarSensor := CompleteIoSetToUse.Sensor 
    : this->CompleteIoSet := CompleteIoSetToUse
    : this->StOffControllerGroup.pControllerArray[0] := &this->OffController 
    : this->StTrackControllerGroup.pControllerArray[0] := &this->TrackController 
    : this->StCalibrateControllerGroup.pControllerArray[0] := &this->CalibrateController 
]]


[[D tStCompleteControllerSet::~tStCompleteControllerSet(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
     *this := anything
]]

[[D tStCompleteControllerSet::OffControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->StOffControllerGroup
]]

[[D tStCompleteControllerSet::TrackControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->StTrackControllerGroup
]]

[[D tStCompleteControllerSet::CalibrateControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->StCalibrateControllerGroup
]]

[[D tStCompleteControllerSet::GetStarPosition(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    return := this->ControllerData.StarPosition
]]

[[D tStCompleteControllerSet::SetControllerParams(const tStControllerParams& Params) ]
sets the controller parameters as specified
[ invalid_pointer(this) => UNDEFINED
| invalid_reference(Params) => UNDEFINED
| true => 
    initialize this->TrackController.Lag with Params
    : initialize this->TrackController.Pid with Params
    : this->TrackController.bLagEnabled := Params.LagEnabled
    : this->TrackController.StarTarget := Params.StarTarget
]] 

*****/

class tStarSensor;
class tStCompleteIoSet;
class tStCompleteControllerSet;
class tStServoParameters;
class tServoInfo;
class tImage;
class tImageStatus;


class tStControllerData {
  public:
    tStControllerData(tImageStatus& StarImageStatusToUse,
                       const tTypeTag& StarPosDataTag, 
                       const tTypeTag& FsmPosDataTag, 
                       const tTypeTag& StarImageDataTag);

    tDataStore<tStData> StarPositionData;
    tDataStore<tIpcPoint> FsmPositionData;
    tImageDataStore     StarImageData;
    tPoint              StarPosition;
    tPoint              FsmPosition;
    tImageStatus&       StarImageStatus;
};

class tStController : public tController {
  public:
    tStController(tStCompleteIoSet& CompleteIoSet,
                  tStControllerData& ControllerDataToUse);
    virtual ~tStController(void);
    virtual ERROR_CODE Calculate(TIME tServoTime);
  protected:
    tStControllerData& ControllerData;
    tPoint StarPosition(void) {return this->ControllerData.StarPosition;}
    
  private:
    void CalculateCentroid(const tImage& Image, tPoint& StarPosition);
    tStarSensor& StarSensor;
};


class tStTrackController : public tStController {
  public: 
    tStTrackController(tStCompleteIoSet& CompleteIoSet,
                       tStControllerData& ControllerDataToUse,
                       TIME tServoPeriod);
    virtual ~tStTrackController(void);
    virtual ERROR_CODE Calculate(TIME tServoTime);
    virtual ERROR_CODE Init(TIME tServoTime);
    virtual ERROR_CODE Reset(TIME tServoTime);
    void SetControllerParams(const tStControllerParams& Params);
  private:
    enum {X,Y};

    tPositionMultiActuator& Fsm;
    tLagCompensator Lag[2];
    tPidDouble Pid[2];     
    tPoint StarTarget;
    BOOL bLagEnabled;
    const TIME CtServoPeriod;    
    ///tIpcDblBufferAccessor<tIpcPoint>& Wap;  FOR 5B
};

class tStCalibrateController : public tStController {
   public:
      tStCalibrateController(tStCompleteIoSet& CompleteIoSet,
                             tStControllerData& ControllerDataToUse);
      ERROR_CODE Calculate(TIME tServoTime);
   private:
      tPositionMultiActuator& Fsm;
};

class tStCompleteControllerSet {
  public: 
    tStCompleteControllerSet(tStCompleteIoSet& CompleteIoSetToUse, 
                              tImageStatus& ImageStatusToUse,
                              const tTypeTag& StarPosDataTag, 
                              const tTypeTag& FsmPosDataTag, 
                              const tTypeTag& StarImageDataTag,
                              TIME tServoPeriod);
    ~tStCompleteControllerSet(void);
    tSimpleControllerGroup& OffControllerGroup() 
      {return StOffControllerGroup;}
    tSimpleControllerGroup& TrackControllerGroup() 
      {return StTrackControllerGroup;}
    tSimpleControllerGroup& CalibrateControllerGroup() 
      {return StCalibrateControllerGroup;}
    tPoint GetStarPosition(void) 
      {return this->ControllerData.StarPosition;}
    tPoint GetFsmPosition(void) 
      {return this->ControllerData.FsmPosition;}
    void SetControllerParams(const tStControllerParams& Params)
      {this->TrackController.SetControllerParams(Params);}
    
  private:
    tStControllerData  ControllerData; 
    tStController      OffController;
    tStTrackController TrackController;
    tStCalibrateController CalibrateController;
    tSimpleControllerGroup StOffControllerGroup;
    tSimpleControllerGroup StTrackControllerGroup;  
    tSimpleControllerGroup StCalibrateControllerGroup;  
};

 

#endif stControllerh
