/* $RCSfile: starSensor.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/st/starSensor.h,v 1.1.1.1 1998/06/11 21:36:17 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  4Sept97,rlj    written

*/


#ifndef INC_starSensorh
#define INC_starSensorh

#include "inputSource.h"
#include "image.h"
#include "ipcIndirectionDblBufferAccessor.h"

/*****

State Variables
-------------------
int iImageHSize
int iImageVSize
tImage StarImage
tIpcIpcIndirectionDblBufferAccessor StarImageBuffer



[[D tStarSensor::tStarSensor(const tIpcIndirectionDblBufferId& iiId, 
                             int  iHSize, int  iVSize) ]
[ invalid_pointer(this) || 
        invalid_reference(iiId) ||
        ::IpcMemory.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject(::IpcMemory,iiId) == FALSE ||
        CanAllocateIpcObject(::IpcMemory,iiId.iiBuffer1Id) == FALSE ||
        CanAllocateIpcObject(::IpcMemory,iiId.iiBuffer2Id) == FALSE =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->StarImageBuffer.ecInitError := appropriate error code
    : this->StarImageBuffer.bInit := anything
    : console := console + appropriate messages
    : this->StarImageBuffer.bObjectValid := FALSE
    : this->StarImageBuffer.iiIndirectionDblBufferId := anything
    : this->StarImage.iHSize := iHSize
    : this->StarImage.iVSize := iVSize
    : this->StarImage.Image := NULL Image 
| true =>
    this->dValue := 0.0
    : this->dOffset := 0.0
    : this->StarImageBuffer.ecInitError := OK
    : this->StarImageBuffer.bInit := TRUE
    : this->StarImageBuffer.bObjectValid := TRUE
    : this->StarImageBuffer.iiIndirecctionDblBufferId := iiId
    : ::IpcMemory.ObjectSet := ::IpcMemory.ObjectSet U
          NewObjectSet(::IpcMemory, iiId) U
          NewObjectSet(::IpcMemory, iiId.iiBuffer1Id) U
          NewObjectSet(::IpcMemory, iiId.iiBuffer2Id) 
    : this->StarImage.iHSize := iHSize
    : this->StarImage.iVSize := iVSize
    : this->StarImage.Image := NULL Image 
]]


[[D tStarSensor::~tStarSensor(void) ]
[ this is invalid => UNDEFINED
| true => * this := anything
]]

[[D tStarSensor::GetStarImage(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => return := reference to this->StarImage
]] 


[[D tStarSensor::Update(TIME tUpdateTime) ]
[ invalid_pointer(this) ||
     this->StarImageBuffer.bObjectValid == FALSE ||
        DoesObjectExist(::IpcMemory,
             this->StarImageBuffer.iiIndirectionDblBufferId) == FALSE =>
    UNDEFINED
| (this->StarImageBuffer.bInit == FALSE) || 
       (this->StarImageBuffer.szBufferSize != this->StarImage.szBufferSize) =>
    appropriate error code sent to RTE error stream
    : return := -1.0
| this->StarImageBuffer.bEmpty == TRUE => 
    return := 0.0
| true =>
    image pointed to by this->StarImage.pImage := 
             GetObject(::IpcMemory,
                     this->StarImageBuffer.iiIndirectionDblBufferId)->Buffer
    : return := 0.0
]] 

******/


class tIpcIndirectionDblBufferId;

class tStarSensor : public tInputSource {
  public:
    tStarSensor(const tIpcIndirectionDblBufferId& iiId, int iHSize, int iVSize);
    tImage& GetStarImage(void);
    virtual double Update(const TIME tUpdateTime);
  protected:
    tIpcIndirectionDblBufferAccessor StarImageBuffer;
    tImage StarImage;
};
    
#endif INC_starSensorh
