/* $RCSfile: stActuator.h,v $  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/stActuator.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Derek Slye - Jet Propulsion Laboratory */


#ifndef INC_stActuatorh
#define INC_stActuatorh

#include "isit.h"
#include "outputDest.h"
#include "ipcid.h"
#include "ipcIndirectionDblBufferAccessor.h"
#include "image.h"

#define S_stactuator_INVALID_STAR_IMAGE_SIZE      (M_stactuator | 0x01)

/*****
Black Box State Variables
______________________________

tDlTarget  Target
tIpcDblBufferAccessor<tDlTarget> TargetBuffer

______________________________________________________________-
Stimuli

[[D tDlActuator::tDlActuator (const tIpcDblBufferId& iiId) ]
[ invalid_pointer (this) || 
  invalid_reference (iiId) ||
  ::IpcMemoryG.bInit == FALSE => UNDEFINED
| CanAllocateIpcObject (::IpcMemoryG, iiId) == FALSE => 
    this->ecInitError:=appropriate error code
    : this->bInit := anything
    : console := console + appropriate messages
    : this->TargetBuffer.bObjectValid := FALSE
    : this->TargetBuffer.iiDblBufferId := anything
    : this->TargetBuffer.bInit := anything
| true =>
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->TargetBuffer.ecInitError := OK
    : this->TargetBuffer.bInit := TRUE
    : this->TargetBuffer.bObjectValid := TRUE
    : this->TargetBuffer.iiDblBufferId := iiId
    : ::IpcMemoryG.ObjectSet := ::IpcMemoryG.ObjectSet U 
      NewObjectSet (::IpcMemoryG, iiId)
]]


[[D tDlActuator::~tDlActuator (void) ]
[ this is invalid => UNDEFINED
| true => * this := anything
]]

[[D tDlActuator::SetTargetValue (const tDlTarget& NewTarget) ]
[ invalid_pointer (this) || 
  invalid_reference (Target) => UNDEFINED
| true => this->Target := NewTarget
]] 


[[D tDlActuator::Output (TIME tUpdateTime) ]
[ invalid_pointer (this) || 
  this->TargetBuffer.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->TargetBuffer.iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->TargetBuffer.iiDblBufferId) => UNDEFINED
| true => 
    GetObject (::IpcMemoryG, this->TargetBuffer.iiBufferId)->dtBuffer := Target
    : return := OK 
]] 

[[D tDlActuator::Reset (void) ]
[ invalid_pointer (this) || 
  this->TargetBuffer.bObjectValid == FALSE ||
  DoesObjectExist (::IpcMemoryG, this->TargetBuffer.iiDblBufferId) == FALSE ||
  simultaneous put on GetObject (::IpcMemoryG, 
    this->TargetBuffer.iiDblBufferId) => UNDEFINED
| true =>
    this->Target := ZERO_TARGET 
    : GetObject (::IpcMemoryG, this->TargetBuffer.iiBufferId)->dtBuffer := 
      ZERO_TARGET
    : return := OK 
]] 
*/   

class tStActuator : public tOutputDestination {
  public:
    tStActuator (int iHSize, int iVSize,
                 const tIpcIndirectionDblBufferId& iiId);
    virtual ~tStActuator () {}
    void SetImage (const tImage& NewImage);
    virtual ERROR_CODE Output (TIME tUpdateTime);
    virtual ERROR_CODE Reset();
  protected:
    tImage Image;
    tImage DefaultImage;
    tIpcIndirectionDblBufferAccessor ImageBuffer;
};

#endif INC_stActuatorh
