/* $RCSfile: stGizmo.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/stGizmo.h,v 1.2 1998/09/23 23:57:29 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  27Aug97,rlj    written
*/

#ifndef INC_stGizmoh
#define INC_stGizmoh

#include "isit.h"
#include "realTimeGizmo.h"
#include "typeTag.h"
#include "stTypes.h"
#include "imageStatus.h"
#include "stCommands.h"
#include "st/stIoSet.h"
#include "st/stController.h"

const SSM_OFF = 0;
const SSM_TRACK = 1;
const SSM_CALIBRATE = 2;

extern const int CiStDataBufferSize;

/****************************************************************************
tStGizmo Black Box State

BOOL                    bInit
ERROR_CODE              ecInitError
tGizmoBlackBoxServo     Servo - (see Gizmo Specification Document)
tStStatus                       Status
tStarSensor                     StarSensor
tImageStatus                    ImageStatus
tStData                         Data
tImage                          ImageData
tTypeTag                        CStStatusTag
tTypeTag                        CStStarImageStatusTag
tTypeTag                        CStStarPosDataTag
tTypeTag                        CStStarImageDataTag

tStController                   OffController
tStTrackController              TrackController
tStCalibrateController          CalibrateController

TASK_ID                         tidCommandTaskId
TASK_ID                         tidStatusTaskId
TASK_ID                         tidServoTaskId
TASK_ID                         tidStarPosDataTaskId
TASK_ID                         tidStarImageDataTaskId
*****************************************************************************

[[D tStGizmo::tStGizmo(const tStGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  IsAFringeTracker (GizmoInfo.iFtGizmoId) == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 3 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 1 ||
  GizmoInfo.gtGizmoType != GT_STAR_TRACKER ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiImageId) == FALSE ||
  Unable to add periodic tasks to handle command, status and servo =>
    this->ecInitError := appropriate error code
    : *this except this->ecInitError := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Servo.ControllerArrayArray := appropriate controllers
    : this->Servo.Mode := ST_OFF
    : this->StarSensor.iFtGizmoId := GizmoInfo.iFtGizmoId
    : this->Status := anything
    : this->Data := anything
    : this->CStStatusTag.stType := ST_STATUS
    : this->CStStatusTag.tnName := TN_ST_STATUS
    : this->CStStatusTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CStStatusTag.szBlockSize := sizeof (this->Status)
    : this->CStImageStatusTag.stType := ST_STATUS
    : this->CStImageStatusTag.tnName := TN_ST_STAR_IMAGE
    : this->CStImageStatusTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CStImageStatusTag.szBlockSize := sizeof (PIXEL) * 
        GizmoInfo.iImageHSize * GizmoInfo.iImageVSize
    : this->CStStarPosDataTag.stType := ST_DATA
    : this->CStStarPosDataTag.tnName := TN_ST_DATA
    : this->CStStarPosDataTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CStStarPosDataTag.szBlockSize := sizeof (this->Data)
    : this->CStStarImageDataTag.stType := ST_DATA
    : this->CStStarImageDataTag.tnName := TN_ST_STAR_IMAGE
    : this->CStStarImageDataTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CStStarImageDataTag.szBlockSize := sizeof (PIXEL) * 
        GizmoInfo.iImageHSize * GizmoInfo.iImageVSize
     : ::IpcMemory.ObjectSet := ::IpcMemory.ObjectSet U 
       NewObjectSet (::IpcMemory,GizmoInfo.iiCommandStreamId) U
       NewObjectSet (::IpcMemory,GizmoInfo.iiImageId)
    : ::PeriodicTaskSet := PeriodicTaskSet U 
      PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
        this->ProcessStatus) U
      PeriodicTask (GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[0].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[0].iTaskPriority, 0,
        tServo.DoServo)
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet U 
      EventDrivenTask ("DataStoreDump",CiEmDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream) U
      EventDrivenTask ("DataStoreDump",CiEmDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream)
    : this->tidCommandTaskId := new unique periodic task id
    : this->tidStatusTaskId := new unique periodic task id
    : this->tidServoTaskId := new unique periodic task id
    : this->tidStarPosDataTaskId := new unique event driven task id   
    : this->tidStarImageDataTaskId := new unique event driven task id   
]]  


[[D tStGizmo::~tStGizmo (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => 
    *this := anything
    : ::PeriodicTaskSet := PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId) -
      PeriodicTask (this->tidServoTaskId)
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet - 
      EventDrivenTask (this->tidStarPosDataTaskId) -
      EventDrivenTask (this->tidStarImageDataTaskId)
]]

[[D tStGizmo::Restart (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to enable  PeriodicTask (this->tidCommandTaskId) ||
  Unable to enable  PeriodicTask (this->tidStatusTaskId) ||
  Unable to enable  PeriodicTask (this->tidServoTaskId) ||
  Unable to enable  EventDrivenTask (this->tidStarPosDataTaskId) ||
  Unable to enable  EventDrivenTask (this->tidStarImageDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) enabled
    : PeriodicTask (this->tidStatusTaskId) enabled
    : PeriodicTask (this->tidServoTaskId) enabled
    : EventDrivenTask (this->tidStarPosDataTaskId) enabled
    : EventDrivenTask (this->tidStarImageDataTaskId) enabled
    : this->bInit := TRUE
    : return := OK
]]

[[D tStGizmo::Shutdown (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to disable PeriodicTask (this->tidCommandTaskId) ||
  Unable to disable PeriodicTask (this->tidStatusTaskId) ||
  Unable to disable PeriodicTask (this->tidServoTaskId) ||
  Unable to disable EventDrivenTask (this->tidStarPosDataTaskId) ||
  Unable to disable EventDrivenTask (this->tidStarImageDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) disabled
    : PeriodicTask (this->tidStatusTaskId) disabled
    : PeriodicTask (this->tidServoTaskId) disabled
    : EventDrivenTask (this->tidStarPosDataTaskId) disabled
    : EventDrivenTask (this->tidStarImageDataTaskId) disabled
    : this->bInit := FALSE
    : return := OK
]]


[[D tStGizmo::IsOk(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this.ecInitError !=OK ||
  Unable to receive commands ||
  Unable to receive from image channels => return := FALSE
| true => return := TRUE
]]

--------------------------------------------------------------------------
Star Tracker Command Specification

[[D tStGizmo::Ping (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true =>  ErrorStream :=  ErrorStream + 
     appropriate message 
]]

[[D tStGizmo::Off (void) ]
[ invalid_pointer (this) => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->Servo.Mode := ST_OFF
    : ErrorStream :=  ErrorStream + 
      appropriate message 
]]
   
   
[[D tStGizmo::Track(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true => 
    this->Servo.Mode := ST_TRACK
    : DataStream (::pThisRteG) := DataStream (::pThisRteG) + 
      this->CStStarPosDataTag + this->Data + this->CStStarImageDataTag +
      this->StarImageData as appropriate 
    : ErrorStream :=  ErrorStream + 
      appropriate message 
]]

[[D tStGizmo::Calibrate(const tFtCalibrateCmd& icNewCmd) ]
implements CALIBRATE command
[invalid_pointer(this) => UNDEFINED
| invalid_reference(icNewCmd) => UNDEFINED
| true =>
    this->Servo.Mode := ST_CALIBRATE
    : this->CompleteIoSet.Fsm.DitherParameters := icNewCmd.Parameters
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
]]

[[D tStGizmo::Home(void) ]
implements HOME command
[invalid_pointer(this) => UNDEFINED
| true =>
    this->Servo.Mode := ST_OFF
    : move this->CompleteIoSet.Fsm to home position
    : ErrorStream () :=  ErrorStream () + appropriate message 
    : return := OK 
]]

[[D tStGizmo::SetServoParams(const tStSetServoParametersCmd& icNewCmd) ]
implements SETSERVOPARAMETERS command
[invalid_pointer(this) => UNDEFINED
| invalid_reference(icNewCmd) => UNDEFINED
| true =>
    this->CompleteControllerSet.TrackController.Parameters := icNewCmd.ControllerParameters
    : this->CompleteIoSet.Fsm.Transform := icNewCmd.Transform
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
]]

[[D tStGizmo::ChangeServoParams(const tStServoParameters& Params) ]
changes the servo parameters as specified
[invalid_pointer(this) ||
  invalid_reference(Params) => UNDEFINED
| true =>
    this->CompleteControllerSet.TrackController.Parameters := Params.ControllerParameters
    : this->CompleteIoSet.Fsm.Transform := Params.Transform
    : return := OK 
]]

--------------------------------------------
Protected Specification

[[D tStGizmo::Dispatch (const tIsitCmd& icNewCmd) ]
[ invalid_pointer (this) ||
  invalid_reference (icNewCmd) => UNDEFINED
| icNewCmd.Command == ST_CMD_PING => 
    ErrorStream :=  ErrorStream + appropriate message
    : return := OK 
| icNewCmd.Command == ST_CMD_OFF => 
    this->Servo.Mode := ST_OFF
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == ST_CMD_TRACK => 
    this->Servo.Mode := ST_TRACK
    : DataStream (::pThisRteG) := DataStream (::pThisRteG) + 
      this->CStStarPosDataTag + this->Data + this->CStStarImageDataTag +
      this->StarImageData as appropriate 
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == ST_CMD_HOME => 
    this->Servo.Mode := ST_OFF
    : move FSM to home position
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == ST_CMD_CALIBRATE => 
    this->Servo.Mode := ST_CALIBRATE
    : Fsm.DitherParameters := icNewCmd.Parameters
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == ST_CMD_SETSP => 
    TrackController.Parameters := icNewCmd.ControllerParameters
    : Fsm.Transform := icNewCmd.Transform
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| true => return := appropriate error code
]]


[[D tStGizmo::CollectStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| IsFull (::StatusStream (::pThisRteG) ) =>
    ::ErrorStream := ::ErrorStream + 
      appropriate error code
    : return := appropriate error code
| true =>
    ::StatusStream (::pThisRteG) := ::StatusStream (::pThisRteG) + 
       this->CStStatusTag + this->Status + this->CStImageStatusTag +
       this->ImageStatus
    : return := OK
]]  

[[D tStGizmo::operator_extractor (ostream& Stream, const tStGizmo& Gizmo) ]
[ invalid_pointer (this) ||
  invalid_reference (Stream) ||
  invalid_reference (Gizmo) => UNDEFINED
| true =>
    Stream := Stream << appropriate text and Gizmo state values
    : return := Stream << appropriate text and Gizmo state values
]]

****************************************************************************/

class tStGizmoInfo;
class ostream;
class tIsitCmd;
class tStSetServoParametersCommand;


class tStGizmo : public tRealtimeGizmo {

  public:
    tStGizmo (const tStGizmoInfo &GizmoInfo);
    virtual ~tStGizmo (); 
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
    friend ostream& operator<< (ostream& stream, tStGizmo& Gizmo);

  private: // these must be constructed before using in constructors of other members 
    const tTypeTag CStStatusTag;
    const tTypeTag CStStarImageStatusTag; 
    const tTypeTag CStStarPosDataTag;
    const tTypeTag CStFsmPosDataTag;
    const tTypeTag CStStarImageDataTag; 
 	 
  protected:
    virtual ERROR_CODE CollectStatus ();
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
 
    tStStatus Status;    
    tImageStatus StarImageStatus;

  private:
    enum {X,Y};

    ERROR_CODE Ping ();
    ERROR_CODE Off ();
    ERROR_CODE Track ();
    ERROR_CODE Calibrate(const tStCalibrateCmd& icNewCmd);
    ERROR_CODE SetServoParams(const tStSetServoParamsCmd& icNewCmd);
    ERROR_CODE Home();
    void ChangeServoParams(const tStServoParams& Params);
    tStCompleteIoSet CompleteIoSet; 
    tStCompleteControllerSet CompleteControllerSet;
};

#endif INC_stGizmoh
