/* $RCSfile: stGizmoInfo.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/stGizmoInfo.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  27Aug97,rlj    written
*/

#ifndef INC_stGizmoInfoh
#define INC_stGizmoInfoh

#include "isit.h"
#include "realTimeGizmoInfo.h"
#include "servoInfo.h"
#include "ipcid.h"
#include "hardware.h"
#include "stTypes.h"
#include "stCommands.h"

/****
Black Box State Variables

int        iRteId
int        iGizmoId
int        iNumberOfServoModes
tServoInfo ServoInfo
GIZMO_TYPE gtType
int        iImageHSize 
int        iImageVSize 
int        iFtGizmoId 
tIpcIndirectionDblBufferId iiStarImageId
__________________________
Stimuli



[[D tStGizmoInfo::tStGizmoInfo(int iRteId, int iGizmoId, 
      const tStPhysicalConstants& PhysicalConstants,
      const tStServoParams& ServoParams,
      int iNumberOfServoModes, const tServoInfo& ServoInfo, 
      int iFtGizmoId,int iWapGizmoId, int iImageHSize, int iImageVSize,
      BOARD_NAME bnFsmBoardArray[NUM_FSM_CHANNELS], int iFsmChannelArray[NUM_FSM_CHANNELS]) ]
     [ invalid_pointer (this) ||
       invalid_reference (ServoInfo) ||
       iFtGizmoId not valid gizmo id ||
       iWapGizmoId not valid gizmo id ||
       invalid_pointer(bnFsmBoardArray) ||
       invalid_pointer(iFsmChannelArray) => UNDEFINED
    | Number of IPC_NAMES previously generated + NUM_FSM_CHANNELS >= maximum allowable names =>
          this->ecInitError := appropriate error code
          : *this (except this->ecInitError) := anything
     | true =>
          this->iRteId := iRteId
          : this->iGizmoId := iGizmoId
          : this->iiCommandStreamId := appropriate and unique tIpcFifoId
          : this->gtGizmoType := GT_UNKNOWN
          : this->iStatusTaskPriority := CiStatusTaskPriority
          : this->tStatusTaskPeriod := CtStatusTaskPeriod
          : this->tStatusTaskOffset := 0
          : this->iCommandTaskPriority := CiCommandTaskPriority
          : this->tCommandTaskPeriod := CiCommandTaskPeriod
          : this->tCommandTaskOffset := 0
          : this->iNumberOfServoModes := iNumberOfServoModes,
          : this->ServoInfo := ServoInfo 
          : number of IPC_NAMES generated := number of IPC_NAMES generated + NUM_FSM_CHANNELS
          : this->iiStarImageId.inObjectName := unique IPC_NAME
          : this->iiStarImageId.inStarBuffer1 := unique IPC_NAME
          : this->iiStarImageId.inStarBuffer2 := unique IPC_NAME
          : this->iiStarImageId.mtObjectMemType := 
               ( if (iRteId == iFtGizmoId Rte number) then MT_LOCAL else MT_SHARED )
          : this->iiStarImageId.itObjectType := IPC_INDIRECTION_DBL_BUFFER
          : this->iiStarImageId.szBufferSize := iImageHSize*iImageVSize
          : this->iStarImageHSize := iImageHSize
          : this->iStarImageVSize := iImageVSize
          : this->iFtGizmoId := iFtGizmoId
          : this->iWapGizmoId := iWapGizmoId
          : for 0 <= actuator < NUM_FSM_CHANNELS
               pFsmChannelArray[actuator] := tOutputChannel corresponding to
                  (bnFsmBoardArray[actuator],iFsmChannelArray[actuator])
]]


[[D tStGizmoInfo::~tStGizmoInfo(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => *this := anything
]]


[[D tStGizmoInfo::FsmChannels(void) ]
returns FSM channels
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->pFsmChannelArray
]]


[[D tStGizmoInfo::WapGizmoId(void) ]
returns WAP gizmo id
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->CiWapGizmoId
]]

[[D tStGizmoInfo::PhysicalConstants(void) ]
returns physical constants 
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->PhysicalConstants_
]]

[[D tStGizmoInfo::ServoParams(void) ]
returns servo parameters
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->ServoParams_
]]

[[D tStGizmoInfo::GetStarImageId(void) ]
returns star image ID
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->iiStarImageId
]]


***********/

//N.B., FsmChannels() should be declared (tOutputChannel const *const *()),
//but gcc has trouble interpreting that declaration.

class tStGizmoInfo : public tRealtimeGizmoInfo {
  public:
    tStGizmoInfo(int iRteId, int iGizmoId, 
        const tStPhysicalConstants& PhysicalConstants,
        const tStServoParams& ServoParams, int iNumberOfServoModes,
        const tServoInfo& ServoInfo, int iFtGizmoId,int iWapGizmoId, 
        int iImageHSize, int iImageVSize,
        const BOARD_NAME bnFsmBoardArray[NUM_FSM_CHANNELS],
        const int iFsmChannelArray[NUM_FSM_CHANNELS]);
    const tIpcIndirectionDblBufferId& GetStarImageId(void) const;
    int GetStarImageHSize(void) const;
    int GetStarImageVSize(void) const;
    tOutputChannel *const *FsmChannels() const {return pFsmChannelArray;}
    int WapGizmoId() {return CiWapGizmoId;}
    const tStPhysicalConstants& PhysicalConstants() const 
      {return PhysicalConstants_;}
    const tStServoParams& ServoParams() const 
      {return ServoParams_;}
  protected:
    tIpcIndirectionDblBufferId iiStarImageId;
    int iStarImageHSize;
    int iStarImageVSize;
    int iFtGizmoId;
    const tStPhysicalConstants& PhysicalConstants_;
    const tStServoParams& ServoParams_;
    const int CiWapGizmoId;
    tOutputChannel *pFsmChannelArray[NUM_FSM_CHANNELS];   
};

#endif INC_stGizmoInfoh

