/* $RCSfile: stTypes.h,v $ */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/include/stTypes.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01b  22Sep97,eamck  taken over
01a  29Aug97,rlj    written
*/

#ifndef INC_stTypesh
#define INC_stTypesh


#include "isit.h"
#include "modLib.h"
#include "tVector.h"

#define NUM_FSM_CHANNELS 3

typedef enum ST_COMMAND_STATUS {
  STCS_OFF, STCS_TRACK_ACTIVE, STCS_CALIBRATE_ACTIVE,
  } ST_COMMAND_STATUS;

typedef enum ST_SERVO_STATE {
  SSS_OFF, SSS_TRACK,SSS_CALIBRATE,SSS_NUM_SERVO_MODES
} ST_SERVO_STATE;

typedef enum ST_PARTITION {
   STP_BASIC,STP_NUM_PARTITIONS
} ST_PARTITION;

/*****
[[D tStStatus::tStStatus (int iGizmoId) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iGizmoId := iGizmoId
]]

[[D tStStatus::UpdateCommandStatus (ST_COMMAND_STATUS scsNewCmdStatus) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->scsCommandStatus := scsNewCmdStatus
]]

[[D tStStatus::UpdateServoState (ST_SERVO_STATE sssServoState) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->sssServoState := sssServoState
]]

[[D tStStatus::UpdateStarPosition (const tStarPosition& StarPosition) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->StarPosition := StarPosition
]]


[[D tStarPosition::tStarPosition(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    this->dXPosition := 0.0 
    : this->dYPosition := 0.0 
]]

[[D tStarPosition::tStarPosition_dd(double dX, double dY) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    this->dXPosition := dX
    : this->dYPosition := dY
]]

****/

struct tStPhysicalConstants {
   double dActuatorScaleFactor;
};

struct tStControllerParams {
   double StarTarget[2];//Target position for star centroid (pixels)
   double LagTauPole;	// (sec)
   double LagTauZero;	// (sec)
   BOOL LagEnabled;
   double PidTauZero1;	// (sec)
   double PidTauZero2;	// (sec)
   double PidTauPole;	// (sec)
   double Gain;
};

struct tStServoParams {
      tStControllerParams ControllerParams;
      double XformDetToFsm[2][NUM_FSM_CHANNELS];// detector to FSM transform (V/pixel)
      double FsmHome[NUM_FSM_CHANNELS];         //FSM home position (V, relative)
};

class tStarPosition {
  public:
    double dXPosition,dYPosition;
    tStarPosition () { this->dXPosition = 0.0; this->dYPosition = 0.0; }
    tStarPosition (double dX, double dY) 
      {this->dXPosition = dX; this->dYPosition = dY;}
};

class tStStatus_Version_1 {
  public:
    int iVersion;
    int iGizmoId;
    TIME tTime;
    ST_COMMAND_STATUS  scsCommandStatus;
    ST_SERVO_STATE     sssServoState;
    tStarPosition StarPosition;
    tStServoParams ServoParams;
    double FsmPosition[2];
};

class tStData_Version_1 {
  public:
    int iVersion;
    int iGizmoId;
    TIME tTime;
    tStarPosition StarPosition;
};

#define CURRENT_ST_STATUS_VERSION 1
#define CURRENT_ST_DATA_VERSION   1

#define tStStatusCurrentVersion \
   EXPAND_AND_CONCAT(tStStatus_Version_, CURRENT_ST_STATUS_VERSION)
#define tStDataCurrentVersion \
   EXPAND_AND_CONCAT(tStData_Version_, CURRENT_ST_DATA_VERSION)


class tStStatusUnix : public tStStatus_Version_1 {
  public:
    tStStatusUnix () {
      iVersion = CURRENT_ST_STATUS_VERSION;
    }
    tStStatusUnix (int iGizmoId) {
      iVersion = CURRENT_ST_STATUS_VERSION;
      this->iGizmoId = iGizmoId;
    }
};

class tStDataUnix: public tStData_Version_1 {
  public:
    tStDataUnix () {
      iVersion = CURRENT_ST_DATA_VERSION;
     }
    tStDataUnix (int iGizmoId) {
      iVersion = CURRENT_ST_DATA_VERSION;
      this->iGizmoId = iGizmoId;
     }
};

class tStStatus: public tStStatusUnix {
  public:
    tStStatus () : tStStatusUnix() {
      this->scsCommandStatus = STCS_OFF;
      this->sssServoState = SSS_OFF;
    }
    tStStatus (int iGizmoId) : tStStatusUnix (iGizmoId) {
      this->scsCommandStatus = STCS_OFF;
      this->sssServoState = SSS_OFF;
    }
    void UpdateCommandStatus (ST_COMMAND_STATUS scsNewCmdStatus) {
       this->scsCommandStatus = scsNewCmdStatus; }
    void UpdateServoState (ST_SERVO_STATE sssServoState) {
       this->sssServoState = sssServoState; }
    void UpdateStarPosition (const tStarPosition& StarPosition) {
       this->StarPosition = StarPosition; }
 
};

class tStData : public tStDataUnix {
  public:
    tStData () : tStDataUnix () {};
    tStData (int iGizmoId) : tStDataUnix (iGizmoId) {}
};


#endif INC_stTypesh



