/* $RCSfile: tList.h,v $  */
/*$Header: /home/chara/cvsroot/ople/sys/include/tList.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $*/
/*
modification history
--------------------
01a,2Jun97,rlj    written
*/



#ifndef INC_LISTh
#define INC_LISTh

#include "lstLib.h"

/***
[[D tList::tList (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iLength := 0
    : this->pHead := NULL
    : this->pTail := NULL
]]

[[D tList::~tList (void) ]
[ invalid_pointer (this) || 
  a list element externally deleted => UNDEFINED
| true =>
   *this := anything
   : this := NULL
]]

[[D tList::Add (tNode* pNode) ]
[ invalid_pointer (this) ||
  a list element externally deleted ||
  invalid_pointer (pNode) => UNDEFINED
| true =>
    this->Element[this->iLength] := pNode
    : this->iLength := this->iLength + 1
    : this->pTail := pNode
]]

[[D tList::Concat (tList* pDstList, tList* pAddList) ]
[ a list element externally deleted from pDstList or pAddList||
  invalid_pointer (ppDstList) ||
  invalid_pointer (pAddList)  => UNDEFINED
| true =>
    *pDstList := *pDstList concat *pAddList
    : pDstList->iLnegth := pDstList->iLength + pAddList.iLength
    : pDstList->pTail := pAddList->pTail
]]

[[D tList::Count (void) ]
[ invalid_pointer (this) ||
  a list element externally deleted from pDstList or pAddList => UNDEFINED
| true => return := this->iLength
]]

[[D tList::Delete (tNode* pNode) ]
[ invalid_pointer (this) ||
  a list element externally deleted from pDstList or pAddList ||
  invalid_pointer (pNode) ||
  pNode not in this->Elements => UNDEFINED
| true =>
    this->Elements := this->Elements with pNode removed
    : this->iLength := this->iLength - 1
    : this->pHead := set appropriately
    : this->pTail := set appropriately
]]

[[D tList::Extract (tList* pSrcList, tNode* pStartNode,tNode* pEndNode, 
                         tList* pExtractList) ]
[ a list element externally deleted from pSrcList ||
  pSrcList is invalid ||
  invalid_pointer (pStartNode)|| 
  invalid_pointer (pEndNode) ||
  pStartNode is not in this->Elements ||
  pEndNode is not in this->Elements ||
  pStartNode not ordinally before pEndNode in this->Elements ||
  pExtractList is invalid => UNDEFINED
| true =>
    pExtractList->Elements := pSrcList->Elements with sublist from pStartNode to pEndNode
    : pExtractList->iLength := #nodes in sublist from pStartNode to pEndNode
    : pExtractList->pHead := pStartNode
    : pExtractList->pTail := pEndNode
]]
        
[[D tList::First (void) ]
[ invalid_pointer (this) || 
  a list element deleted externally => UNDEFINED
| true => return := this->pHead
]]
  
[[D tList::Last (void) ]
[ invalid_pointer (this) || 
  a list element deleted externally => UNDEFINED
| true => return := this->pTail
]]
  
[[D tList::Next (tNode* pNode) ]
[ invalid_pointer (this) || 
  a list element deleted externally ||
  invalid_pointer (pNode) => UNDEFINED
| pNode == this->pTail => return := NULL
| true => return := list element sequentially following pNode
]]
  
[[D tList::Nth (int iNodeNum) ]
[ invalid_pointer (this) || 
  a list element deleted externally => UNDEFINED
| iNodeNum > this->iLength ||
  iNodeNum < 1 => return NULL
| true => return := this->Element[iNodeNum]
]]
  
[[D tList::Prev (tNode* pNode) ]
[ invalid_pointer (this) || 
  a list element deleted externally ||
  invalid_pointer (pNode) => UNDEFINED
| pNode == this->pHead => return := NULL
| true => return := list element sequentially before pNode
]]

[[D tList::NStep (tNode* pNode, int iNStep) ]
[ invalid_pointer (this) || 
  a list element deleted externally ||
  invalid_pointer (pNode) ||
  pNode not in this->Elements => UNDEFINED 
| iNStep node away from pNode does not exist => return := NULL
| true => return := list element iNStep nodes away from pNode
]]

[[D tList::Find (tNode* pNode) ]
[ invalid_pointer (this) || 
  a list element deleted externally ||
  invalid_pointer (pNode) => UNDEFINED
| pNode not in this->Elements => return := ERROR
| true => return := ordinal value of pNode in this->Elements
]]

[[D tList::Free (void) ]
[ invalid_pointer (this) ||
  a list element deleted externally => UNDEFINED
| true =>
   this->Elements := Empty List
   : this->iLength := 0
   : this->pHead := NULL
   : this->pTail := NULL
   : sizeof (list elements) * this->iLength bytes freed to heap
]]

***/


class tNode : public NODE {
  public:
    tNode () {};
    virtual ~tNode () {};
}; 

class tList {
  public:
    tList ();
    ~tList ();
    void Add (tNode *pNode);
    friend void Concat (tList *pDstList, tList *pAddList);
    int Count ();
    void Delete (tNode *pNode);
    friend void Extract (tList *pSrcList, tNode *pStartNode,tNode *pEndNode, 
                         tList *pExtractList);
    tNode *First();
    tNode *Last();
    tNode *Next (tNode *pNode);
    tNode *Nth (int iNodeNum);
    tNode *Previous (tNode *pNode);
    tNode *NStep (tNode *pNode, int iNStep);
    int Find (tNode *pNode);
    void Free ();
  
  protected:
    LIST lListDescriptor;
};

#endif INC_LISTh
