/* $RCSfile: tVector.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/tVector.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
$Log: tVector.h,v $
Revision 1.1.1.1  1998/06/11 21:36:14  chara
First import of RICST Code

Revision 1.2  1998/03/25 16:58:27  rbrill
added tVelocity

Revision 1.1  1998/03/16 16:04:12  rbrill
filename changed from vector.h

Revision 1.6  1998/03/03 20:38:26  rbrill
spec errors fixed

Revision 1.5  1998/02/25 19:54:29  rbrill
first compile, increment 5

Revision 1.4  1998/02/24 19:07:02  rbrill
compile errors fixed

Revision 1.3  1998/02/19 15:38:19  rbrill
testing dollar-Log-dollar

Revision 1.2  1998/02/19 15:35:37  rbrill
modified various type qualifiers to match vector.cc

*/

#ifndef INC_tvectorh
#define INC_tvectorh

#include "vtError.h"

//if NUMBER is changed, then change DBL_EPSILON in matrix.h
typedef double NUMBER;
typedef NUMBER type;

/****

The class tVector represents a vector, and supports vector addition 
and subtraction, and multiplication of a vector by a double.  The dot
product is generated by the asterisk: 

   d = v1 * v2;
   
where d is a double and v1, v2, and v3 are vectors.  The cross 
product is generated by global function cross():
   
   v1 = cross(v2,v3);
   
A vector element is referenced by

   v[i]
   
where "v" is a tVector, and "i" is an zero-based integer, and the
result is a NUMBER (double).  That expression may be used on either 
side of the equal sign for a non-const tVector, but only on the right
for a const tVector.

Other operations are also supported.  The class tPoint is just a
tVector of length 2.  Whenever a size problem is detected a call is
made to TraceMsg().  That behavior is localized to inline function,
error(), in order that it may be easily modified.  

This class was derived (in the English sense) from a vector class by 
Brad Hines.



class tVector

Black Box State Variables
-------------------------------------- 
iSize             int                        number of data elements
pData             type*                      pointer to data
pDefaultFormat    static const char *const   default output format string
pFmt              static char*               output format string



[[D tVector::tVector(void) ]
default constructor
[ invalid_pointer (this) => UNDEFINED
| true =>
   create a vector of undefined length
]]

[[D tVector::tVector_i(int iNumElements) ]
constructor with size
[ invalid_pointer (this) => UNDEFINED
| iNumElements < 0 => UNDEFINED
| true =>
   create a vector if length iNumElements
]]

[[D tVector::tVector_t(const tVector& Vector) ]
copy constructor
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := deep copy of *this
]]

[[D tVector::~tVector(void) ]
destructor
[ invalid_pointer (this) => UNDEFINED
| true =>
   delete[] pData
]]

[[D tVector::addelt(void) ]
add a zero element
[ invalid_pointer (this) => UNDEFINED
| true =>
   add a zero element to the end of the vector
]]

[[D tVector::addelt_d(type x) ]
add the specified element
[ invalid_pointer (this) => UNDEFINED
| true =>
   add the specified element to the end of the vector
]]

[[D tVector::operator= (const tVector& Vector) ]
assignment operator
[ invalid_pointer (this) => UNDEFINED
| invalid_reference (Vector) => UNDEFINED
| true =>
   make this vector a copy of Vector
]]

[[D tVector::print(void) ]
formats and writes the elements to stdout
[ invalid_pointer (this) => UNDEFINED
| invalid_pointer(this->pFmt) => UNDEFINED
| true =>
   format and write the elements to stdout
]]

[[D tVector::input(void) ]
reads the elements from stdin
[ invalid_pointer (this) => UNDEFINED
| true =>
   read the elements from stdin, with prompts
]]

[[D tVector::clear(void) ]
zeroes all elements
[ invalid_pointer (this) => UNDEFINED
| true =>
   zeroes all elements
]]

[[D tVector::operator[] (const int element) ]
index operator for non-const objects
[ invalid_pointer (this) => UNDEFINED
| element < 0 || element >= this->iSize => ERROR
| true =>
   return := a const reference to the specified row
]]

[[D tVector::operator[]_c (const int element) ]
index operator for const objects
[ invalid_pointer (this) => UNDEFINED
| element < 0 || element >= this->iSize => ERROR
| true =>
   return := a reference to the specified row
]]

[[D tVector::mag(void) ]
returns the magnitude of the vector
[ invalid_pointer (this) => UNDEFINED
| this->iSize < 1 => ERROR
| true =>
   return := the magnitude of the vector
]]

[[D tVector::sum(void) ]
returns the sum of the elements
[ invalid_pointer (this) => UNDEFINED
| this->iSize < 1 => ERROR
| true =>
   return := the sum of the elements
]]

[[D tVector::avg(void) ]
returns the average of the elements
[ invalid_pointer (this) => UNDEFINED
| this->iSize < 1 => ERROR
| true =>
   return := the average of the elements
]]

[[D tVector::abs(void) ]
returns a vector of the absolute values of the elements 
[ invalid_pointer (this) => UNDEFINED
| this->iSize < 1 => ERROR
| true =>
   return := a vector of the absolute values of the elements
]]

[[D tVector::sort(void) ]
returns a vector of sorted elements
[ invalid_pointer (this) => UNDEFINED
| this->iSize < 1 => ERROR
| true =>
   return := a vector of sorted elements (low-to-high)
]]

[[D tVector::operator* (const tVector& Vector) ]
returns the dot product of the vectors
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   return := the dot product of *this and Vector
]]

[[D tVector::operator+ (const tVector& Vector) ]
returns the sum of the vectors
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   return := *this + Vector
]]

[[D tVector::operator+= (const tVector& Vector) ]
adds the specified vector to the *this
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   *this := *this + Vector
   : return := *this
]]

[[D tVector::operator- (const tVector& Vector) ]
returns the difference of the vectors
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   return := *this - Vector
]]

[[D tVector::operator-= (const tVector& Vector) ]
subtracts the specified vector from *this
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   *this := *this - Vector 
   : return := *this
]]

[[D tVector:: operator- (void)]
returns a vector of the negatives of the elements
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| true =>
   *this := -(*this)
   : return := *this
]]

[[D tVector::operator+(const NUMBER r) ]
returns a vector with the specified number added to each element
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := a vector with r added to each element
]]

[[D tVector::operator-(const NUMBER r) ]
returns a vector with the specified number subtracted from each element
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := a vector with r subtracted from each number
]]

[[D tVector::operator*(const NUMBER r) ]
returns a vector with each element multiplied by the specified number
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := a vector with each number multiplied by r
]]

[[D tVector::operator/(const NUMBER r) ]
returns a vector with each element divided by the specified number
[ invalid_pointer (this) => UNDEFINED
| r == 0 => UNDEFINED
| true =>
   return := a vector with each number divided by r
]]

[[D tVector::operator+=(const NUMBER r) ]
adds the specified number to each element
[ invalid_pointer (this) => UNDEFINED
| true =>
   add r to each element
   : return := *this
]]

[[D tVector::operator-=(const NUMBER r) ]
subtracts the specified number from each element
[ invalid_pointer (this) => UNDEFINED
| true =>
   subtract r from each element
   : return := *this
]]

[[D tVector::operator*=(const NUMBER r) ]
multiplies each element by the specified number
[ invalid_pointer (this) => UNDEFINED
| true =>
   multiply each element by r
   : return := *this
]]

[[D tVector::operator/=(const NUMBER r) ]
divides each element by the specified number
[ invalid_pointer (this) => UNDEFINED
| r == 0 => UNDEFINED
| true =>
   divide each element by r
   : return := *this
]]

[[D tVector::Dimension(void) ]
returns the number of elements
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->iSize
]]

[[D tVector::SetFormat(char *fmt) ]
uses the specified format string (not a copy) for method print()
[ invalid_pointer (this) => UNDEFINED
| invalid_pointer(fmt) => UNDEFINED
| true =>
   pFmt := fmt
]]

[[D tVector::DefaultFormat(void) ]
uses the default format string for method print()
[ invalid_pointer (this) => UNDEFINED
| true =>
   pFmt := pDefaultFormat
]]

[[D ::operator*(const NUMBER r, const tVector& Vector) ]
returns the product of the number and the vector
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(Vector) => UNDEFINED
| true =>
   return := r * Vector
]]

[[D ::cross(const tVector& v1, const tVector& v2) ]
returns the cross product of the vectors
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(v1) => UNDEFINED
| invalid_reference(v2) => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| this->iSize != 3 => ERROR
| true =>
   return := v1 x v2
]]

[[D ::angle(const tVector& v1, const tVector& v2) ]
returns the angle between the vectors, in radians
[ invalid_pointer (this) => UNDEFINED
| invalid_reference(v1) => UNDEFINED
| invalid_reference(v2) => UNDEFINED
| v1.mag() == 0 || v2.mag() == 0 => UNDEFINED
| this->iSize != Vector.iSize => ERROR
| this->iSize != 3 => ERROR
| true =>
   return := angle between v1 and v2
]]

************************

class tPoint

Black Box State Variables
-------------------------------------- 
x,y         NUMBER         data
[inherits format strings from tVector]


[[D tPoint::tPoint(void) ]
default constructor
[ invalid_pointer (this) => UNDEFINED
| true =>
   create (0,0)
]]

[[D tPoint::tPoint(tVector& Vector) ]
type conversion
[ invalid_pointer (this) => UNDEFINED
| invalid reference(Vector) => UNDEFINED
| Vector.iSize != 2 => ERROR
| true =>
   convert a vector to a point
]]

[[D tPoint::x(void) ]
returns reference to x for non-const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->x
]]

[[D tPoint::y(void) ]
returns reference to y for non-const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->y
]]

[[D tPoint::x_c(void) ]
returns const reference to x for const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->x
]]

[[D tPoint::y_c(void) ]
returns const reference to y for const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->y
]]


************************

class tVelocity

Black Box State Variables
-------------------------------------- 
xDot,yDot         NUMBER         data
[inherits format strings from tVector]


[[D tVelocity::tVelocity(void) ]
default constructor
[ invalid_pointer (this) => UNDEFINED
| true =>
   create (0,0)
]]

[[D tVelocity::tVelocity(tVector& Vector) ]
type conversion
[ invalid_pointer (this) => UNDEFINED
| invalid reference(Vector) => UNDEFINED
| Vector.iSize != 2 => ERROR
| true =>
   convert a vector to a point
]]

[[D tVelocity::xDot(void) ]
returns reference to x component for non-const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->xDot
]]

[[D tVelocity::yDot(void) ]
returns reference to y component for non-const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->yDot
]]

[[D tVelocity::xDot_c(void) ]
returns const reference to x component for const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->xDot
]]

[[D tVelocity::y_c(void) ]
returns const reference to y component for const object
[ invalid_pointer (this) => UNDEFINED
| true =>
   return := this->yDot
]]

****/

class tVector {
   public:
     tVector(void);
     explicit tVector(const int NumElements);
     tVector(const tVector& Vector);
     ~tVector();
     tVector& addelt(void);
     tVector& addelt(type x);
     tVector& operator= (const tVector&Vector);
     void print(void) const;
     void input(void);
     void clear(void);
     type& operator[] (const int element);
     const type&operator[] (const int element) const;
     NUMBER mag(void) const;
     NUMBER sum(void) const;
     NUMBER avg(void) const;
     tVector abs(void) const;
     tVector sort(void);
     NUMBER operator* (const tVector& Vector) const;
     tVector& operator+= (const tVector& Vector);
     tVector& operator-= (const tVector& Vector);
     tVector operator- ();   
     tVector operator- (const tVector& Vector) const;
     tVector operator+ (const tVector& Vector) const;
     tVector operator+(const NUMBER r) const;
     tVector operator-(const NUMBER r) const;
     tVector operator*(const NUMBER r) const;
     tVector operator/(const NUMBER r) const;
     tVector& operator+=(const NUMBER r);  
     tVector& operator-=(const NUMBER r);
     tVector& operator*=(const NUMBER r);   
     tVector& operator/=(const NUMBER r);
     int Dimension(void) const { return size;}
     static void SetFormat(char *fmt) {}
     static void DefaultFormat(void) {pFmt = defaultFormat;}
   
   private:
     int size;
     type *pointer;
     static const char *const defaultFormat;
     static const char *pFmt;
};


tVector operator*(const NUMBER r, const tVector& Vector);
tVector cross(const tVector& v1, const tVector& v2);
NUMBER angle(const tVector& v1, const tVector& v2);


class tPoint : public tVector {
   public:
      tPoint() : tVector(2) {}
      tPoint(double xinit,double yinit) : tVector(2) 
         {x() = xinit; y() = yinit;}
      tPoint(const tVector& v);
      NUMBER& x() {return (*this)[0];}
      NUMBER& y() {return (*this)[1];}
      const NUMBER& x() const {return (*this)[0];}
      const NUMBER& y() const {return (*this)[1];}
};


class tVelocity : public tVector {
   public:
      tVelocity() : tVector(2) {}
      tVelocity(double xinit,double yinit) : tVector(2) 
         {xDot() = xinit; yDot() = yinit;}
      tVelocity(const tVector& v);
      NUMBER& xDot() {return (*this)[0];}
      NUMBER& yDot() {return (*this)[1];}
      const NUMBER& xDot() const {return (*this)[0];}
      const NUMBER& yDot() const {return (*this)[1];}
};


#endif

