/* $RCSfile: targetSource.h,v $ - Implements abstract target classes */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/targetSource.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Brad Hines - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a,beh,27Apr97    written
*/
 
/*
DESCRIPTION

*/


#ifndef INC_TargetSourceh
#define INC_TargetSourceh

#include "inputSource.h"


/*******************************************************************
*
* tTargetSource Black Box
*
* A tTargetSource is a thing which has a value and a method for
* updating that value.  Derived classes can overload the
*
* Variable                   Initial Value
* ----------------------------------------
* double dValue              anything
*
*******************************************************************
*
* tTargetSource::tTargetSource
*
* DESCRIPTION: Compiler-supplied consructor
*
* RETURNS: Nothing
*

[[D tTargetSource::tTargetSource (void)]
  [ this invalid => UNDEFINED
  | true => this->dValue := anything
]]

*
*******************************************************************
*
* tTargetSource::GetValue
*
* DESCRIPTION: Returns the value retrieved by the last Update
*
* RETURNS: double
*

[[D tTargetSource::GetValue (in: this;)]
  [ this invalid => UNDEFINED
  | true => return := this->dValue
]]

*
*******************************************************************
*
* tTargetSource::Update {pure virtual}
*
* DESCRIPTION: Uses an implementation-specific method to get a new
* value for dValue.
*
* RETURNS: double
*

[[D tTargetSource::Update (TIME tUpdateTime)]
[ invalid_poiner (this) => UNDEFINED
| true =>
    this->dValue := Appropriate new value for time tUpdateTime
    : return := Appropriate new value for time tUpdateTime
]]

*
*/


/*******************************************************************
***********************  tTargetSet Black Box  **********************
********************************************************************
*
* A tTargetSet is an entity which owns a group of tInputSources
* and can Update them all on command
*
* BB State Variables
*
* Variable                         Initial Value
* ----------------------------------------------
* TargetSet: set of tInputSources   Set at construction
*
*
*******************************************************************
*
* tTargetSet::Update
*
* DESCRIPTION: Updates all its individual input sources
*
* RETURNS: OK, or an ERROR_CODE if something goes wrong
*

[[D tTargetSet::Update(in: this; inout: this->TargetSet;)]
  [ this or this->TargetSet invalid => UNDEFINED
  | some entry in this->TargetSet cannot be Upated =>
      this->TargetSet.*.dValue := anything
      : return := appropriate error code
  | true =>
      this->TargetSet.*.dValue := appropriate new value
      : return := OK
]]

*
*/




/*******************************************************************/
/*******************************************************************/
/********************                     **************************/
/********************  Class Definitions  **************************/
/********************                     **************************/
/*******************************************************************/
/*******************************************************************/

/*******************************************************************/
/*************************  tTargetSource    ***********************/
/*******************************************************************/

class tTargGen;


class tTargetSource : public tInputSource {
};



/*******************************************************************/
/*********************  tZeroTargetSource    ***********************/
/*******************************************************************/

class tZeroTargetSource : public tTargetSource {
public:
  double Update(TIME tTargetTime) { this->SetValue (0.0); return 0.0; }
};

/*******************************************************************/
/********************  tLaserPositionTargetSource  *****************/
/*******************************************************************/

class tPositionTargetSource : public tTargetSource {
public:
  tPositionTargetSource (tTargGen &TheTargetGenerator);
  double Update(TIME tTargetTime);
protected:
  tTargGen &TargetGenerator;
};


/*******************************************************************/
/*****************  tLaserVelocityTargetSource  ********************/
/*******************************************************************/

class tVelocityTargetSource : public tTargetSource {
public:
  tVelocityTargetSource(tTargGen &TheTargetGenerator);
  double Update(TIME tTargetTime);
protected:
  tTargGen &TargetGenerator;
};



#endif  /* INC_TargetSourceh */
