/* $RCSfile: telemetryItem.h,v $*/
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/telemetryItem.h,v 1.1.1.1 1998/06/11 21:36:14 chara Exp $
*/
/* Richard L. Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
01a, 3Mar98,  rj      written
01a, 10Mar98, blw     updated
 */



/*
DESCRIPTION

Class definition for telemetry items.
*/


#ifndef INC_telemetryItemh
#define INC_telemetryItemh

//  includes 
#ifdef UNIX
#define Traceback(a, b, c, d)  printf(b, c, d)
#define FALSE (0)
#define TRUE  (1)
#define MAX(a,b) ((a>b)?(a):(b))
#include <isitUnix.h>
#else
#include <isit.h>
#include <trace.h>
#endif

#include <string.h>
#include <objectId.h>
#include <configMgr.h>
#include <telemetryManager.h>
#include <telemetryEvent.h>
#include <ringBuffer.h>


/***************** tTelemetryItemBase Specification **************************

[[D tTelemetryItemBase::Selected (void) ]
Returns selection state or reference to selection state of telemetry item.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->bSelected
| true => return := reference to this->bSelected
]]


[[D tTelemetryItemBase::Priority (void) ]
Returns priority or reference to priority of telemetry item.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->iPriority
| true => return := reference to this->iPriority
]]

[[D tTelemetryItemBase::Type (void) ]
Pure virtual: returns telemetry type.
Must be overridden by subclass.
[ true => return := appropriate telemetry type
]] 

[[D tTelemetryItemBase::Extract (BYTE*   pDestination, 
                                 size_t  szDestination, 
                                 size_t& szSource) ]
Extracts telemtry value from this into a buffer of size szDestination returning the
actual size of the value in szSource.
Pure virtual: Extracts data into a buffer.
Must be overridden by subclass.
[ true => extracts the data, depending on the specific subclass
]] 

[[D tTelemetryItemBase::Id() const ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => return := uId
]]

======================== Protected Specification ========================

[[D tTelemetryItemBase::tTelemetryItemBase (const char*      sItemName,
                                        const tObjectId* Parent) ]
 [ invalid_pointer (this) => UNDEFINED
 | true => 
   call tObjectId parent class constructor to create Id for this object
   : bSelected := False
   : uId := extracted from tObjectId baseclass
   : iPriority := tTelemetryItemBase::iNORMAL_PRIORITY
]]

[[D virtual tTelemetryItemBase::~tTelemetryItemBase (void) 
 [ invalid_pointer (this) => UNDEFINED
 | true => bSelected := False
]]

[[D void tTelemetryItemBase::ReportError(const char *msg,
                     ERROR_CODE  code=0) ]
 [ invalid_pointer (this) => UNDEFINED
 | NULL msg pointer => do nothing.
 | ID obtainable => Use Traceback to put out an error message with the code
                    (optionally -- Traceback will ignore it if 0), the string
                    "TelemetryItem", the ID, and the msg string.
 | true => Use Traceback to put out an error message with the code
           (optionally -- Traceback will ignore it if 0), the string
           "TelemetryItem", and the msg string.

]]

*******************tTelemetryItem Specification *****************************


[[D tTelemetryItem<T>::operator=_1 (const tTelemetryItem& rhs) ]
Assignment operator between two telemetry items copies only the value.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | this != &rhs => Assign rhs's value to *this's value using operator=_2()
   : return := const reference to *this
 | true =>
   return := const reference to *this
]]

[[D tTelemetryItem<T>::operator=_2 (const T& rhs) ]
Assignment operator from T to this.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | true =>
   this->Value_:= rhs 
   : return := const reference to *this
]]


[[D tTelemetryItem<T>::operator==_1 (const tTelemetryItem& rhs) ]
Equality comparison operator between telemety item and raw value.
[ invalid_pointer (this) ||
  invalid_reference (rhs) => UNDEFINED
| rhs's value equals *this's value => return := TRUE
| true => return := FALSE
]]


[[D tTelemetryItem<T>::operator!=_1 (const tTelemetryItem& rhs) ]
Inequality comparison operator between telemety item and raw value.
[ invalid_pointer (this) ||
  invalid_reference (rhs) => UNDEFINED
| rhs's value equals *this's value => return := FALSE
| true => return := TRUE
]]


[[D tTelemetryItem<T>::operatorT (void) ]
Casts this into a T.
[ invalid_pointer (this) => UNDEFINED
| true => return := this->Value
]]


[[D tTelemetryItem<T>::Extract (BYTE* pDestination, size_t szDestination, 
      size_t& szSource) ]
Extracts telemtry value from this into a buffer of size szDestination returning the
actual size of the value in szSource.
[ invalid_pointer (this) => UNDEFINED
| valid_pointer (pDestination) && szDestination >= sizeof(T) => 
    pDestination [0..sizeof(T)-1] := telemetry value 
    : pDestination [sizeof(T).. szDestination-1] := anything 
    : szSource := sizeof(T)
| true => 
    pDestination [0..sizeof (this->Value)-1] := telemetry value
    pDestination [0..szDestination-1] := anything 
    : szSource := 0
]]


======================== Protected Specification ========================

[[D tTelemetryItem<T>::Value (void) ]
Returns const reference to value or referece to value of the telemetry item.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := const reference to this->Value_
| true = >return := reference to this->Value_
]]

[[D tTelemetryItem<T>::tTelemetryItem (void) ]
Constructor
 [ invalid_pointer (this) => UNDEFINED
 | true => constructs tTelemetryItem
]]

[[D tTelemetryItem<T>::tTelemetryItem (const T& InitialValue) ]
Constructor with initial value
 [ invalid_pointer (this) => UNDEFINED
 | true => constructs tTelemetryItem
   : Assign InitialValue to *this's value using operator=_2()
]]

[[D virtual tTelemetryItem<T>::~tTelemetryItem(void) ]
 [ invalid_pointer (this) => UNDEFINED
 | true => destroys *this
]]

******************tEventDrivenTelemetryItemBase Specification***************

[[D tEventDrivenTelemetryItemBase::ManualEvent (void) ]
Notifies telemetry manager to collect telemetry from this item.  This is used
as a manual stimulus as oposed to automatic event signaling that may 
occur during an assignment.
 [ invalid_pointer (this) => UNDEFINED
 | true => Trigger the telemetry manager
]]

[[D tEventDrevenTelemetryItemBase::Type (void) ]
 [ invalid_pointer (this) => UNDEFINED
 | true => return := TT_EVENT_DRIVEN
]]

[[D tEventDrivenTelemetryItemBase::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tEventDrivenTelemetryItemBase::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]

=======================Protected Specification=======================

[[D tEventDrivenTelemetryItemBase::tEventDrivenTelemetryItemBase(
      const char*      sItemName,
      const tObjectId* Parent) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true =>
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
]]

[[D virtual tEventDrivenTelemetryItemBase::~tEventDrivenTelemetryItemBase(void)]
 [ invalid_pointer (this)  => UNDEFINED
 | true => destroys *this
]]

[[D void tEventDrivenTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]

*******************tEventDrivenTelemetryItem  Specification******************


[[D tEventDrivenTelemetryItem<T>::tEventDrivenTelemetryItem(
                              const char*      sItemName,
                              const tObjectId* Parent) ]
Contructs event driven telemetry item with specified name, lineage, and 
priority and registers item with the telemetry manager.
[ invalid_pointer (this) || 
  invalid_reference (Parent) => UNDEFINED
 | true =>
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : this->Value_:= anything
]]

[[D tEventDrivenTelemetryItem<T>::tEventDrivenTelemetryItem(
                               const char*      sItemName,
                               const tObjectId* Parent,
                               const T&         InitialValue) ]
Contructs event driven telemetry item with specified name, lineage, 
priority and initial value, and registers item with the telemetry manager.
[ invalid_pointer (this) || 
  invalid_reference (Parent) => UNDEFINED
 | true =>
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Assign InitialValue to *this's value using this->operator=_2()
]]


[[D tEventDrivenTelemetryItem<T>::operator=_1 (const tTelemetryItem& rhs) ]
Assignment operator between two telemetry items copies only the value.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | this != &Item => 
   : Assign rhs's value to *this's value using this->operator=_2()
   : return := const reference to *this
 | true =>
   return := const reference to *this
]]

[[D tEventDrivenTelemetryItem<T>::operator=_2 (const T& rhs) ]
Assignment operator from T to this.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | true =>
   : Assign rhs's value to *this's value using tTelemetryItem<T>::operator=_2()
   : Notify telemetry manager to collect telemetry from this item.
   : return := const reference to *this
]]

[[D void tEventDrivenTelemetryItem<T>::Extract(BYTE*   pDestination,
                                           size_t  szDestination,
                                           size_t& szSource) ]
  call parent class's extract 
[ invalid_pointer (this)  => UNDEFINED
| valid_pointer (pDestination) && szDestination >= sizeof(T) => 
    pDestination [0..sizeof(T)-1] := telemetry value 
    : pDestination [sizeof(T).. szDestination-1] := anything 
    : szSource := sizeof(T)
| true => 
    pDestination [0..sizeof (this->Value)-1] := telemetry value
    pDestination [0..szDestination-1] := anything 
    : szSource := 0
]]

[[D virtual tEventDrivenTelemetryItem<T>::~tEventDrivenTelemetryItem(void) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => destroys *this
]]

[[D tEventDrivenTelemetryItem<T>::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tEventDrivenTelemetryItem<T>::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]

[[D tEventDrivenTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ]]

[[D tEventDrivenTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ]]

[[D tEventDrivenTelemetryItem<T>::operatorT (void) ] =
 [  tTelemetryItem<T>::operatorT (void) ]]


=======================Protected Specification=======================


[[D void tEventDrivenTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]

[[D tEventDrivenTelemetryItem<T>::Value (void) ] = 
 [  tTelemetryItem<T>::Value (void) ]]

***************** tBufferedTelemetryItemBase Specification  *****************

[[D tBufferedTelemetryItemBase::ManualEvent (void) ]
Notifies telemetry manager to collect telemetry from this item.  This is used
as a manual stimulus as oposed to automatic event signaling that may 
occur during an assignment  It is also called by the object itself
when it's buffer is full.
 [ invalid_pointer (this) => UNDEFINED
 | true => Send event trigger to telemetry manager
]]

[[D tBufferedTelemetryItemBase::Type (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := TT_BUFFERED
]]

[[D tTelemetryItemBase::Period (void) ]
Returns sampling period or reference to sampling period.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->tPeriod
| true => return := reference to this->tPeriod
]]

[[D tBufferedTelemetryItemBase::BufferSize (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->uBufferSize
]]

[[D tBufferedTelemetryItemBase::NumValues (void) ]
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->uNumValues
| true => UNDEFINED
]] 

[[D tBufferedTelemetryItemBase::GetNumValues (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->uNumValues
]] 

[[D tBufferedTelemetryItemBase::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tBufferedTelemetryItemBase::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]

====================== Protected Specification ======================

[[D tBufferedTelemetryItemBase::NumValues (void) ]
[ invalid_pointer (this) || const_pointer(this) => UNDEFINED
| true => return := reference to this->uNumValues
]] 

[[D tBufferedTelemetryItemBase::tBufferedTelemetryItemBase(
      const char*      sItemName,
      const tObjectId* Parent,
      unsigned         uBufferSizeIn,
      TIME             tSamplePeriodIn) ]
 [ invalid_pointer (this)  => UNDEFINED
 | uBufferSizeIn==0 => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   : Report the following error -
      "Cannot create a buffered telemetry item with 0 length buffer"
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
]]

[[D virtual tBufferedTelemetryItemBase::~tBufferedTelemetryItemBase (void) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => 
   : this->uNumValues := 0
   : this->uBufferSize := 0
   : destroys *this
]]

[[D void tBufferedTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]


*******************tBufferedTelemetryItem  Specification******************

[[D tBufferedTelemetryItem<T>::tBufferedTelemetryItem(
                            const char*      sItemName,
                            const tObjectId* Parent,
                            unsigned         uBufferSizeIn,
                            TIME             tSamplePeriodIn=APERIODIC_TIME) ]
Contructs buffered telemetry item with specified name, lineage,  
priority, and buffer size and registers item with the telemetry manager.
 [ invalid_pointer (this)  => UNDEFINED
 | uBufferSizeIn==0 => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   : Report the following error -
      "Cannot create a buffered telemetry item with 0 length buffer"
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : this->Buffer := uBufferSizeIn *
              tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER)
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   :   this->Buffer := uBufferSizeIn *
              tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER)
]]

[[D tBufferedTelemetryItem<T>::tBufferedTelemetryItem(
                            const char*      sItemName,
                            const tObjectId* Parent,
                            unsigned         uBufferSizeIn,
                            TIME             tSamplePeriodIn,
                            const T&         InitialValue)
Contructs buffered telemetry item with specified name, lineage,  
priority, buffer size and initial value,  and registers item with 
the telemetry manager.
 [ invalid_pointer (this)  => UNDEFINED
 | uBufferSizeIn==0 => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   : Report the following error -
      "Cannot create a buffered telemetry item with 0 length buffer"
   : this->Buffer := uBufferSizeIn *
              tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER)
   : Assign InitialValue to *this's value using this->operator=_2()
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->uBufferSize := uBufferSizeIn
   : this->uNumValues := 0
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : Event := value retrieved from tTelemetryItemBase::TelemetryManager
   : uId := default value assigned by tObjectBase class.
   : this->Buffer := uBufferSizeIn *
              tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER)
   : Assign InitialValue to *this's value using this->operator=_2()
   : Assign Initial Value Ring Buffer
]]


[[D tBufferedTelemetryItem<T>::operator=_1 (const tTelemetryItem& rhs) ]
Assignment operator between two telemetry items copies only the value.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | this != &rhs => 
   : Assign rhs's value to *this's value using this->operator=_2()
   : return := const reference to *this
 | true =>
   return := const reference to *this
]]

[[D tBufferedTelemetryItem<T>::operator=_2 (const T& rhs) ]
Assignment operator from T to this.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | this->NumValues(void)==this->BufferSize(void)-1 =>
   : Assign rhs's value to *this's value using tTelemetryItem<T>::operator=_2()
   : append rhs to ring buffer
   : increment uNumValues
   : Notify telemetry manager to collect telemetry from this item.
   : return := const reference to *this
 | uNumValues > uBufferSize - 1 =>
   : Assign rhs's value to *this's value using tTelemetryItem<T>::operator=_2()
   : append rhs to ring buffer
   : Notify telemetry manager to collect telemetry from this item.
   : return := const reference to *this
 | true =>
   : Assign rhs's value to *this's value using tTelemetryItem<T>::operator=_2()
   : append rhs to ring buffer
   : increment uNumValues
   : return := const reference to *this
]]

[[D void tBufferedTelemetryItem<T>::Extract(BYTE*   pDestination,
                                           size_t  szDestination,
                                           size_t& szSource) ]
 extract the number of items, then the items themselves
 [ invalid_pointer (this)  => UNDEFINED
 | invalid_pointer (pDestination) || (szDestination<sizeof(unsigned)) =>
   szSource := 0
 | szDestination not large enough to hold one unsigned + uNumValues data
 => pDestination[0..sizeof(unsigned)-1] := number of values able to write
                                           given the constricted space
 :  pDestination[sizeof(unsigned)..szDestination-1)] := as much of the bufferd
    data as will fit, starting with the least recent, going forwards in time.
 | true => 
 => pDestination[0..sizeof(unsigned)-1] := this->NumValues(void)
 :  pDestination[
     sizeof(unsigned)..uNumValues*sizeof(T)+sizeof(unsigned)-1]
       := the bufferd data, starting with the least recent, and
          going forwards in time.
]]

[[D virtual tBufferedTelemetryItem<T>::~tBufferedTelemetryItem(void) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => destroys *this
]]

[[D tBufferedTelemetryItem<T>::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tBufferedTelemetryItem<T>::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]

[[D tBufferedTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ]]

[[D tBufferedTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ]]

[[D tBufferedTelemetryItem<T>::operatorT (void) ] =
 [  tTelemetryItem<T>::operatorT (void) ]]


=======================Protected Specification=======================


[[D void tBufferedTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]

[[D tBufferedTelemetryItem<T>::Value (void) ] = 
 [  tTelemetryItem<T>::Value (void) ]]


***************** tSampledTelemetryItemBase Specification  *****************

[[D tSampledTelemetryItemBase::Type (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := TT_SAMPLED
]]

[[D tTelemetryItemBase::Period (void) ]
Returns sampling period or reference to sampling period.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->tPeriod
| true => return := reference to this->tPeriod
]]

[[D tTelemetryItemBase::Offset (void) ]
Returns sampling offset or reference to sampling period.
[ invalid_pointer (this) => UNDEFINED
| const_pointer (this) => return := this->tOffset
| true => return := reference to this->tOffset
]]

[[D tSampledTelemetryItemBase::IsTimeToSample (TIME tTime) ]
Returns TRUE if it is time to sample item.
[ invalid_pointer (this) => UNDEFINED
| tTime % this->tPeriod - this->tOffset == 0 => return TRUE
| true => return := FALSE
]]


[[D tBufferedTelemetryItemBase::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tBufferedTelemetryItemBase::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]


=======================Protected Specification=======================

[[D tSampledTelemetryItemBase::tSampledTelemetryItemBase(
                               const char*      sItemName,
                               const tObjectId* Parent,
                               TIME             tSamplePeriodIn,
                               TIME             tSampleOffsetIn) ]
 [ invalid_pointer (this)  => UNDEFINED
 | tSamplePeriodIn==tTelemetryItemBase::INVALID_TIME =>
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : Report error in sample time.
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
]]

[[D virtual tSampledTelemetryItemBase::~tSampledTelemetryItemBase(void) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => destroys *this
]]


[[D void tBufferedTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]


*******************tSampledTelemetryItem Specification ******************


[[D tSampledTelemetryItem<T>::tSampledTelemetryItem(
                            const char*      sItemName,
                            const tObjectId* Parent,
                            TIME             tSamplePeriodIn,
                            TIME             tSampleOffsetIn) ]
Contructs sampled telemetry item with specified name, lineage,  
priority, period and offset and registers item with the telemetry manager.
 [ invalid_pointer (this)  => UNDEFINED
 | tSamplePeriodIn==tTelemetryItemBase::INVALID_TIME =>
   : Report error in sample time.
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
]]


[[D tSampledTelemetryItem<T>::tSampledTelemetryItem(
                            const char*      sItemName,
                            const tObjectId* Parent,
                            TIME             tSamplePeriodIn,
                            TIME             tSampleOffsetIn,
                            const T&         InitialValue)
Contructs sampled telemetry item with specified name, lineage,  
priority, period, offset and initial values and registers item 
with the telemetry manager.
 [ invalid_pointer (this)  => UNDEFINED
 | tSamplePeriodIn==tTelemetryItemBase::INVALID_TIME =>
   : Report error in sample time.
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : uId := default value assigned by tObjectBase class.
 | true => 
   : this->tSamplePeriod := tSamplePeriodIn
   : this->tSampleOffset := tSampleOffsetIn
   : iPriorityIn := default value assigned by tTelemetryItemBase class.
   : bSelected := default value assigned by tTelemetryItemBase class.
   : Assign InitialValue to *this's value using this->operator=_2()
   : uId := default value assigned by tObjectBase class.
]]

[[D tSampledTelemetryItem<T>::operator=_1 (const tTelemetryItem& rhs) ]
Assignment operator between two telemetry items copies only the value.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | this != &rhs => 
   : Assign rhs's to *this's value using this->operator=_2()
   : return := const reference to *this
 | true =>
   return := const reference to *this
]]

[[D tSampledTelemetryItem<T>::operator=_2 (const T& rhs) ]
Assignment operator from T to this.
 [ invalid_pointer (this) ||
   invalid_reference (rhs) => UNDEFINED
 | true =>
   : Assign rhs to *this's value using tTelemetryItem<T>::operator=_2()
   : return := const reference to *this
]]

[[D void tSampledTelemetryItem<T>::Extract(BYTE*   pDestination,
                                           size_t  szDestination,
                                           size_t& szSource) ]
  call parent class's extract 
[ invalid_pointer (this)  => UNDEFINED
| valid_pointer (pDestination) && szDestination >= sizeof(T) => 
    pDestination [0..sizeof(T)-1] := telemetry value 
    : pDestination [sizeof(T).. szDestination-1] := anything 
    : szSource := sizeof(T)
| true => 
    pDestination [0..sizeof (this->Value)-1] := telemetry value
    pDestination [0..szDestination-1] := anything 
    : szSource := 0
]]

[[D virtual tSampledTelemetryItem<T>::~tSampledTelemetryItem(void) ]
 [ invalid_pointer (this)  => UNDEFINED
 | true => destroys *this
]]

[[D tSampledTelemetryItem<T>::Selected (void) ] = 
 [ tTelemetryItemBase::Selected (void) ]]

[[D tSampledTelemetryItem<T>::Priority (void) ] = 
 [ tTelemetryItemBase::Priority (void) ]]

[[D tSampledTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator==_1 (const tTelemetryItem& Item) ]]

[[D tSampledTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ] = 
 [  tTelemetryItem<T>::operator!=_1 (const tTelemetryItem& Item) ]]

[[D tSampledTelemetryItem<T>::operatorT (void) ] =
 [  tTelemetryItem<T>::operatorT (void) ]]


=======================Protected Specification=======================


[[D void tSampledTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ] = 
 [  void tTelemetryItemBase::ReportError
    (const char *msg, ERROR_CODE code=0) ]]

[[D tSampledTelemetryItem<T>::Value (void) ] = 
 [  tTelemetryItem<T>::Value (void) ]]


*************/

// TYPES of Telemetry Ids
typedef enum 
  { TT_UKNOWN, TT_EVENT_DRIVEN, TT_BUFFERED, TT_SAMPLED } TELEMETRY_TYPE ;

//**************************************************************************//

class tTelemetryItemBase : public tObjectId 
{
  public:
    //  constants
    static const int      iNORMAL_PRIORITY ;
    static const TIME     INVALID_TIME ;
    static const TIME     APERIODIC_TIME ;

    //  accessors
    BOOL  Selected () const { return this->bSelected; }    //  inspector
    BOOL& Selected ()       { return this->bSelected; }    //  mutator

    int   Priority () const { return this->iPriority; }    //  inspector
    int&  Priority ()       { return this->iPriority; }    //  mutator

    virtual TELEMETRY_TYPE Type () const = 0;

    virtual void Extract (BYTE*   pDestination, 
                          size_t  szDestination, 
                          size_t& szSource) = 0;

    unsigned Id() const { return uId ; }

  protected:
    //  Constructor
    tTelemetryItemBase (const char*      sItemName, 
                        const tObjectId* Parent) ;

    //  Destructor
    virtual ~tTelemetryItemBase () { Selected() = FALSE ; }

    //  reports errors
    void ReportError(const char *msg,
                     ERROR_CODE  code=0) ;

    //  Singleton Telemetry Manager
    static tTelemetryManager TelemetryManager ;
    
  private:
    BOOL       bSelected;
    int        iPriority;
    unsigned   uId ;

};        //  tTelemetryItemBase

//==========================================================================//

template <class T> class tTelemetryItem  
{
  public:
    //  Assignment operators
    const tTelemetryItem<T>& operator= (const T&);
    const tTelemetryItem<T>& operator= (const tTelemetryItem<T>&) ;

    //  Equality checkers
    //    the following is not needed because of operator T() below
    //    int operator== (const tTelemetryItem<T> rhs) const
    //      { return (this->Value() == rhs.Value()) ; } 
    int operator== (const T& rhs) const 
      { return (this->Value() == rhs) ; } 
    int operator!= (const T& rhs) const { return !(operator==(rhs)) ; }

    operator T() const { return this->Value() ; }

    void Extract (BYTE*   pDestination, 
                  size_t  szDestination, 
                  size_t& szSource) ;

  protected:
    //  Value extractors/modifiers
    const T& Value() const { return this->Value_; }        // inspector
          T& Value()       { return this->Value_; }        // mutator

    //  Constructor 
    tTelemetryItem () {} ;

    //  Constructor with initial value
    tTelemetryItem (const T& InitialValue) { *this=InitialValue ; }

    //  Destructor
    virtual ~tTelemetryItem() {}

  private:
    //  the actual value!
    T Value_;

};        //  tTelemetryItem

//**************************************************************************//

class tEventDrivenTelemetryItemBase : public tTelemetryItemBase 
{
  public:
    //  Force an update to the telemetry manager 
    virtual void ManualEvent () ;

    TELEMETRY_TYPE Type () const { return TT_EVENT_DRIVEN; }

  protected:
    //  Constructor 
    tEventDrivenTelemetryItemBase (const char*      sItemName, 
                                   const tObjectId* Parent) ;

    //  Destructor 
    virtual ~tEventDrivenTelemetryItemBase () {}

  private:
    tTelemetryEvent Event;

};        //  tEventDrivenTelemetryItemBase


//==========================================================================//

     
template <class T> class tEventDrivenTelemetryItem : 
    public tEventDrivenTelemetryItemBase, public tTelemetryItem<T> 
{
  public:
    //  Constructors, one with initial value
    tEventDrivenTelemetryItem (const char*      sItemName,
                               const tObjectId* Parent) :
      /*****
       [U tEventDrivenTelemetryItemBase::tEventDrivenTelemetryItemBase(
            sItemName, Parent) ]]
       *****/
      tEventDrivenTelemetryItemBase(sItemName, Parent),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>() {}

    tEventDrivenTelemetryItem (const char*      sItemName,
                               const tObjectId* Parent,
                               const T&         InitialValue) :
      /*****
       [[U tEventDrivenTelemetryItemBase::tEventDrivenTelemetryItemBase(
              sItemName, Parent) ]]
       *****/
      tEventDrivenTelemetryItemBase(sItemName, Parent),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>()
      {
        /*****
         [[U tEventDrivenTelemetryItem<T>::operator=_2(InitialValue)
         *****/
        *this = InitialValue ;  // calls operator=(const T&)
      } 

    //  Assignment operators
    const tEventDrivenTelemetryItem<T>& operator= (const T&)  ;
    const tEventDrivenTelemetryItem<T>& operator=  
                                        (const tEventDrivenTelemetryItem<T>&) ;

    //  for extracting the data for transmission
    void Extract (BYTE*   pDestination, 
                  size_t  szDestination, 
                  size_t& szSource) ;

    //  Destructor
    virtual ~tEventDrivenTelemetryItem() {}

};        //  tEventDrivenTelemetryItem


//**************************************************************************//


class tBufferedTelemetryItemBase : public tTelemetryItemBase 
{
  public:

    //  forces an emptying of the buffer
    virtual void ManualEvent () ;

    //  accessors
    TELEMETRY_TYPE Type () const { return TT_BUFFERED; }        // inspector
    TIME  Period () const        { return this->tSamplePeriod;} // inspector
    TIME& Period ()              { return this->tSamplePeriod;} // mutator
    unsigned BufferSize () const { return this->uBufferSize; }  // inspector
    unsigned NumValues  () const { return this->uNumValues;}    // inspector

    //  the following is for chunks of code where the compiler
    //  thinks you want the protected NumValues() and not the
    //  public one.  I can't think of an elegant way around this.
    unsigned GetNumValues() const { return this->uNumValues;}   // inspector

  protected:
    static const unsigned uRING_BUFFER_MULTIPLIER ;

    unsigned& NumValues ()       { return this->uNumValues;}    // mutator

    //  Constructor
    tBufferedTelemetryItemBase (const char*     sItemName,
                                const tObjectId*Parent,
                                unsigned        uBufferSizeIn,
                                TIME            tSamplePeriodIn);

    //  Destructor
    virtual ~tBufferedTelemetryItemBase()
    {
      // make sure this is unusable as it's being destroyed
      NumValues() = 0 ;
      uBufferSize = 0 ;
    }


  private:
    TIME tSamplePeriod;         //  may be undefined for aperiodic buffers

    unsigned uBufferSize ;
    unsigned uNumValues ;

    tTelemetryEvent Event;

};        //  tBufferedTelemetryItemBase


//==========================================================================//
     
template <class T> class tBufferedTelemetryItem : 
    public tBufferedTelemetryItemBase, public tTelemetryItem<T> 
{
  public:
    //  Constructors, one with an initial value
    tBufferedTelemetryItem (const char*      sItemName,
                            const tObjectId* Parent,
                            unsigned         uBufferSizeIn,
                            TIME             tSamplePeriodIn=APERIODIC_TIME) :
      /*****
       [[U tBufferedTelemetryItemBase::tBufferedTelemetryItemBase
                    (sItemName, Parent, uBufferSizeIn, 
                     tSamplePeriodIn) ]]
       *****/
      tBufferedTelemetryItemBase(sItemName, Parent, uBufferSizeIn,
                                  tSamplePeriodIn),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>(),
      Buffer(uBufferSizeIn * 
             tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER) 
      {
      }

    tBufferedTelemetryItem (const char*      sItemName,
                            const tObjectId* Parent,
                            unsigned         uBufferSizeIn,
                            TIME             tSamplePeriodIn,
                            const T&         InitialValue) :
      /*****
       [[U tBufferedTelemetryItemBase::tBufferedTelemetryItemBase
               (sItemName, Parent, uBufferSizeIn, 
         tSamplePeriodIn) ]]
       *****/
      tBufferedTelemetryItemBase(sItemName, Parent, uBufferSizeIn,
                                 tSamplePeriodIn),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>(),
      Buffer(uBufferSizeIn * 
             tBufferedTelemetryItemBase::uRING_BUFFER_MULTIPLIER)
      { 
        /*****
         [[U tBufferedTelemetryItem<T>::operator=_2(InitialValue) ]]
         *****/
         *this = InitialValue ; //  calls operator=
      } 

    //  Assignment operator
    const tBufferedTelemetryItem<T>& operator= (const T&);
    const tBufferedTelemetryItem<T>& operator=  
                                     (const tBufferedTelemetryItem<T>&) ;

    //  for extracting the data for transmission
    void Extract (BYTE*   pDestination, 
                  size_t  szDestination, 
                  size_t& szSource) ;

    //  Destructor
    virtual ~tBufferedTelemetryItem() {}

  private:
    tRingBuffer<T> Buffer;        //  storage for buffered data
    
};        //  tBufferedTelemetryItem



//**************************************************************************//


class tSampledTelemetryItemBase : public tTelemetryItemBase 
{
  public:
    BOOL IsTimeToSample (TIME tTime) const ;

    //  Accessors
    TELEMETRY_TYPE Type () const { return TT_SAMPLED; }
    TIME  Period () const { return this->tSamplePeriod;}    // inspector
    TIME& Period ()       { return this->tSamplePeriod;}    // mutator
    TIME  Offset () const { return this->tSampleOffset;}    // inspector
    TIME& Offset ()       { return this->tSampleOffset;}    // mutator

  protected:
    //  Constructor
    tSampledTelemetryItemBase (const char*      sItemName,
                               const tObjectId* Parent,
                               TIME             tSamplePeriodIn, 
                               TIME             tSampleOffsetIn) ;

    //  Destructor
    virtual ~tSampledTelemetryItemBase() {}

  private:
    TIME tSamplePeriod; 
    TIME tSampleOffset;

} ;        //  tSampledTelemetryItemBase
 
   

//==========================================================================//

  
template <class T> class tSampledTelemetryItem : 
    public tSampledTelemetryItemBase, public tTelemetryItem<T> 
{

  public:
    //  Constructors, one with an initial value
    tSampledTelemetryItem (const char*      sItemName,
                           const tObjectId* Parent,
                           TIME             tSamplePeriodIn, 
                           TIME             tSampleOffsetIn) :
      /*****
       [[U tSampledTelemetryItemBase::tSampledTelemetryItemBase(
                                sItemName, Parent, tSamplePeriodIn,
                                tSampleOffsetIn) ]]
       *****/
      tSampledTelemetryItemBase(sItemName, Parent, tSamplePeriodIn,
                                            tSampleOffsetIn),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>() {}


    tSampledTelemetryItem (const char*      sItemName,
                           const tObjectId* Parent,
                           TIME             tSamplePeriodIn, 
                           TIME             tSampleOffsetIn,
                           const T&         InitialValue) :
      /*****
       [[U tSampledTelemetryItemBase::tSampledTelemetryItemBase(
                                      sItemName, Parent, tSamplePeriodIn,
                                      tSampleOffsetIn) ]]
       *****/
      tSampledTelemetryItemBase(sItemName, Parent, tSamplePeriodIn,
                                tSampleOffsetIn),
      /*****
       [[U tTelemetryItem<T>::tTelemetryItem(void) ]]
       *****/
      tTelemetryItem<T>()
    {
      /*****
       [[U tTelemetryItem<T>::operator=_2(InitialValue) ]]
       *****/
      *this = InitialValue ;   // calls operator=(const T&)
    }



    //  Assignment operator
    const tSampledTelemetryItem<T>& operator= (const T&);
    const tSampledTelemetryItem<T>& operator= (const tSampledTelemetryItem<T>&);

    //  for extracting the data for transmission
    void Extract (BYTE*   pDestination, 
                  size_t  szDestination, 
                  size_t& szSource) ;

    //  Destructor
    virtual ~tSampledTelemetryItem() {}

};        // tSampledTelemetryItem
    

//**************************************************************************//

//  Following are the inline template member functions, which
//  have to be inline 'cause Gnu says so.

//==========================================================================//

//  Assignment operator
template <class T> inline
const tTelemetryItem<T>& tTelemetryItem<T>::operator= (const T& rhs)
{
  //  Assign the value from the right hand side to this.
  /*****
   [[U this->Value() = rhs ]]
   *****/
  this->Value() = rhs ;

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tTelemetryItem assignment operator with class Right Hand Side

//--------------------------------------------------------------------------//


//  Assignment operator
template <class T> inline
const tTelemetryItem<T>& tTelemetryItem<T>::operator=
                                             (const tTelemetryItem<T>& rhs)
{
  //  Make sure we're not being reflexive
  if (this != &rhs)
  {
    //  Assign the value from the right hand side to this.
    //  by calling the simpler operator=

    /*****
     [[U this->operator=_2(rhs.Value()) ]]
     *****/
    *this = rhs.Value() ;
  }

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tTelemetryItem assignment operator with class Right Hand Side

//--------------------------------------------------------------------------//


template <class T> inline
void tTelemetryItem<T>::Extract(BYTE*   pDestination,
                                size_t  szDestination,
                                size_t& szSource) 
{
  szSource = sizeof(T) ;

  //  Check for valid input
  if (pDestination && szDestination>=szSource)
  /*****
   [[U this->Value(void) ]]
   *****/
    memcpy(pDestination, &(this->Value()), szSource) ;
  else
    szSource = 0 ;        // to indicate that nothing was extracted

}        //  tTelemetryItem::Extract


//==========================================================================//


//  Assignment operator
template <class T> inline
const tEventDrivenTelemetryItem<T>&
    tEventDrivenTelemetryItem<T>::operator= (const T& rhs)
{
  /*****
   [[U tTelemetryItem<T>::operator=_2(rhs) ]]
   *****/
  //  Let the base class do the Value_ assignment
  tTelemetryItem<T>::operator=(rhs) ;

  /*****
   [[U this->ManualEvent(void) ]]
   *****/
  //  Tell the manager that the value has changed
  this->ManualEvent() ;

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tEventDrivenTelemetryItem assignment operator


//--------------------------------------------------------------------------//


//  Assignment operator
template <class T> inline
const tEventDrivenTelemetryItem<T>&
    tEventDrivenTelemetryItem<T>::operator=
      (const tEventDrivenTelemetryItem<T>& rhs)
{
  //  Make sure we're not being reflexive
  if (this!=&rhs)
  {
  /*****
   [[U tEventDrivenTelemetryItem<T>::operator=_2(rhs.Value()) ]]
   *****/
    //  Let the simple value assignment operator do the work
    *this = rhs.Value() ;  //  calls operator=(const t&)
  }

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tEventDrivenTelemetryItem assignment operator


//--------------------------------------------------------------------------//

//  for extracting the data for transmission
template <class T> inline
void tEventDrivenTelemetryItem<T>::Extract(BYTE*   pDestination,
                                           size_t  szDestination,
                                           size_t& szSource) 
{
  /*****
   [[U tTelemetryItem<T>::Extract(pDestination, szDestination, szSource) ]]
   *****/

  //  Parent class's extract is good enough
  tTelemetryItem<T>::Extract(pDestination, szDestination, szSource) ;

}        //  tEventDrivenTelemetryItem::Extract


//==========================================================================//

//  Assignment operator
template <class T> inline
const tBufferedTelemetryItem<T>& 
    tBufferedTelemetryItem<T>::operator= (const T& rhs)
{
  /*****
   [[U tTelemetryItem<T>::operator=(rhs) ]]
   *****/

  //  Let the base class do the Value_ assignment
  tTelemetryItem<T>::operator=(rhs) ;

  //  Add the value to the ring buffer
  /*****
   [[U this->Buffer.Append(rhs) ]]
   *****/

  if (BufferSize())
    Buffer.Append(rhs) ;

  /*****
   [[U this->NumValues(void) ]]
   *****/
  /*****
   [[U this->BufferSize(void) ]]
   *****/
  if (this->NumValues() < this->BufferSize())
    this->NumValues() = this->NumValues() + 1 ;

  //  Check for a full buffer
  if (this->NumValues() >= this->BufferSize())
    //  send an event to tell the telemetry manager
  /*****
   [[U this->ManualEvent(void) ]]
   *****/
    this->ManualEvent() ;

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tBufferedTelemetryItem assignment operator


//--------------------------------------------------------------------------//

//  Assignment operator
template <class T> inline
const tBufferedTelemetryItem<T>& 
    tBufferedTelemetryItem<T>::operator= 
      (const tBufferedTelemetryItem<T>& rhs)
{
  //  Make sure we're not being reflexive
  /*****
   [[U rhs.NumValues(void) ]]
   *****/

  if (this!=&rhs && rhs.NumValues()>0)
  {
  /*****
   [[U this->operator=_2(rhs.Value()) ]]
   *****/

    //  Let the simple value assignment operator do the work
    *this = rhs.Value() ;  //  calls operator=(const t&)
  }

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tBufferedTelemetryItem assignment operator


//--------------------------------------------------------------------------//



//  for extracting the data for transmission
template <class T> inline
void tBufferedTelemetryItem<T>::Extract (BYTE*   pDestination,
                                         size_t  szDestination,
                                         size_t& szSource)
{
  BYTE*     dest = pDestination ;
        szSource = 0 ;

  //  Check for valid input
  if (!pDestination || sizeof(T)==0 || szDestination < sizeof(unsigned)) 
  {
  /*****
   [[U ReportError("Invalid value passed to tBufferedTelemetryItem::Extract") ]]
   *****/

    ReportError("Invalid value passed to tBufferedTelemetryItem::Extract") ;
    return ;
  }

  //  make room for number of items
  /*****
   [[U this->GetNumValues(void) ]]
   *****/
  szSource = sizeof(unsigned) ;
  unsigned numvals = MIN(this->NumValues(), 
                         (szDestination-sizeof(unsigned))/sizeof(T)) ;
  memcpy(dest, &numvals, szSource) ;
  dest += szSource ;
  
  for (unsigned n=0 ; n<numvals ; n++)
  {
    /*****
     [[U this->Buffer.operator[] ]]
     *****/
    BYTE*   src = (BYTE*) &(Buffer[(int) (n - numvals + 1)]) ;


    //  extract the data for the element
    memcpy(dest, src, sizeof(T)) ;

    //  update the counters
    szSource += sizeof(T) ;
    dest += sizeof(T) ;
  }

  //  update the number of items left unread (typically zero)
  /*****
   [[U this->GetNumValues(void) ]]
   *****/
  this->NumValues() = this->NumValues() - numvals ;

}        //  tBufferedTelemetryItem::Extract


//==========================================================================//


//  Assignment operator
template <class T> inline
const tSampledTelemetryItem<T>& 
     tSampledTelemetryItem<T>::operator= (const T& rhs)
{
  //  Let the base class do the Value_ assignment
  /*****
   [[U tTelemetryItem<T>::operator=(rhs) ]]
   *****/
  tTelemetryItem<T>::operator=(rhs) ;

  //  ... and that's really all that need be done; it's up
  //  to the Telemetry Manager to poll for a sample when it's
  //  time.

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tSampledTelemetryItem assignment operator

//--------------------------------------------------------------------------//

//  Assignment operator
template <class T> inline
const tSampledTelemetryItem<T>&
     tSampledTelemetryItem<T>::operator= (const tSampledTelemetryItem<T>& rhs)
{
  //  Make sure we're not being reflexive
  if (this!=&rhs)
  {
    //  Let the simple value assignment operator do the work
  /*****
   [[U this->operator=_2(rhs.Value()) ]]
   *****/
    *this = rhs.Value() ;  //  calls operator=(const t&)
  }

  //  return a pointer to myself 'cause that's what you do in C++
  return *this ;

}        //  tSampledTelemetryItem assignment operator


//--------------------------------------------------------------------------//

//  for extracting the data for transmission
template <class T> inline
void tSampledTelemetryItem<T>::Extract(BYTE*   pDestination,
                                       size_t  szDestination,
                                       size_t& szSource) 
{
  //  Parent class's extract is good enough
  /*****
   [[U tTelemetryItem<T>::Extract(pDestination, szDestination, szSource) ]]
   *****/

  tTelemetryItem<T>::Extract(pDestination, szDestination, szSource) ;

}        //  tSampledTelemetryItem::Extract


//**************************************************************************//

#endif INC_telemetryItemh
