/* $RCSfile: trace.h,v $ - Traceback header file */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/trace.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Michael Deck - Cleanroom Software Engineering, Inc. */

/* 
modification history
--------------------
01a,mdd,14Apr97    written
02a,mdd,17Apr97    rewritten
*/

/*
DESCRIPTION

*/

#ifndef INC_traceh
#define INC_traceh

#ifndef SPECTOOL

#include <msgQLib.h>
#include <taskLib.h>
#include <sysLib.h>
#include <string.h>
#include <stdarg.h>
#endif

#include "isitExt.h"

#define S_tracedump_QUEUE_ERROR         (M_trace | 1)
#define TRACEBACK_QUEUE_TIMEOUT_SECS    2
#define TRACEBACK_TASK_PRIO             (98)
#define TRACEBACK_TASK_OPTIONS          (VX_FP_TASK)
#define TRACEBACK_TASK_STACKSIZE        (16384)

// TBD: rename to naming convention 
#define CiMaxTraceMarks 100000
#define CiMaxTraceStrings 500

//TBD rename TMT_xxxx 
typedef enum TRACE_MSG_TYPE 
   {tmt_INFO, tmt_WARN, tmt_ERROR} 
   TRACE_MSG_TYPE;

typedef TIME (*PTIMEFUNC)(void);  //BEH

/****************************************************************
*
* tTrace Black Box
*
* BB State Variables
* ------------------
*   this->bInit : true iff properly constructed 
*   this->fastbuf : ring buffer of up to CiMaxTraceMarks integers
*     .length : integer current number of items in fastbuf
*     .newest : integer most recently adde item in fastbuf
*   this->bFrozen : boolean determines whether fastbuf adds are accepted
*   this->msgQId : integer "pointer to" message queue
*   this->iTraceTaskId : identifier of msgQ draining task 
*   this->bTraceMsgUsesLogMsg : true iff tracemsg should use logmsg 
* 

  Specification Functions
  -----------------------
  
  tracedump_log(int iCount, int iFromNewest, ring buffer fastbuf) ==
    returns a string containing max(iCount,fastbuf.length-iFromNewest)
    messages, nicely formatted. The messages represent elements of 
    fastbuf beginning with the iFromNewest'th most recent  and with 
    older messages following
   
  traceback_message (TRACE_MSG_TYPE tmtMsg, ERROR_CODE ec, string subrname) ==
    returns a string containing ec and subrname, nicely formatted to 
    indicate the message type tmtMsg
    
    
*****************************************************************
*
* tTrace::tTrace(void) - constructor
*
* Description
*
* construct *this to have a new mark buffer and a new message queue 
*  and spawn a task to drain the message queue to the console 

[[D tTrace::tTrace (void) ]
[ a message queue cannot be created to contain CiMaxTraceStrings 
  elements each sizeof(char*) || 
  a new task cannot be created having the appropriate characteristics => 
    this->bInit := false
  : this->bTraceMsgUsesLogMsg := false
  : OS.Task.errno := appropriate error code
  : *this := anything 

| true => 
  this->bInit := true 
  : this->bTraceMsgUsesLogMsg := false
  : this->msgQId := id of a new message queue that contains 
      CiMaxTraceStrings elements each sizeof(char*) 
  : this->iTraceTaskId := id of a new task w/appropriate characteristics 
  : this->fastbuf := newly-allocated empty ring buffer
  : this->bFrozen := FALSE
]]
* 
* RETURNS: void
*
*
*****************************************************************
*
* tTrace::~tTrace(void) - destructor
*
* Description
*
* clear the mark buffer, free the message queue, and 
* delete the drain task 

[[D tTrace::~tTrace (void) ]
 [ invalid_pointer(this) => UNDEFINED

 | true => free message queue this->msgQId
    : delete task this->iTraceTaskId
    : *this := anything
]]       
* 
* RETURNS: void
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceMark(int iStatus)
*
* Description
*
* add a trace mark to this->mark buffer 

[[D tTrace::TraceMark (int iStatus)]
[ invalid_pointer(this) => UNDEFINED
| this->bFrozen => return := OK
| this->fastbuf.length == CiMaxTraceMarks =>
    this->fastbuf := this->fastbuf with oldest element 
    removed and iStatus added as newest element
  : return := OK
| true => 
    this->fastbuf := this->fastbuf with iStatus added
    as newest element
  : this->fastbuf.length := this->fastbuf.length+1  
  : return := OK
]]
* 
* RETURNS: OK
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceDump(int iCount, int iFromNewest=0)      
*
* Description
*
* copy up to iCount elements in the mark buffer (beginning with
* FromNewest'th newest, then next newest, etc.) to  
* console via message queue with appropriate accompanying text
    
[[D tTrace::TraceDump(int iCount, int iFromNewest) ]
[ invalid_pointer(this) => UNDEFINED 
| message queue this->msgQId cannot be written => 
      logmsg "ERROR: Tracedump message queue is full"
    : printf message
    : return := S_tracedump_QUEUE_TIMEOUT
| task iTraceTaskId is not running or cannot write
    iCount items to console via message queue this->msgQId => No response??
| true => console := console + tracedump_log(iCount,iFromNewest,this->fastbuf)
      via message queue this->msgQId
    : return := OK
]]
*
* 
* RETURNS: OK, S_tracedump_QUEUE_TIMEOUT
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceDumpText(int iCount, int iFromNewest=0)
*
* Description
*
* copy up to iCount elements in the mark buffer (beginning with
* FromNewest'th newest, then next newest, etc.) to  
* console via message queue with appropriate accompanying text

[[D tTrace::TraceDumpText(void)]   
[ invalid_pointer(this) => UNDEFINED
| message queue this->msgQId cannot be written =>
      logmsg "ERROR: Tracedump message queue is full"
    : printf message
    : return := S_tracedump_QUEUE_TIMEOUT
| task iTraceTaskId is not running or cannot write
    iCount items to console via message queue this->msgQId => No response??
| true => console := console + tracedump_log(iCount,iFromNewest,this->fastbuf)
      via message queue this->msgQId
    : return := OK
]]
*
*
* RETURNS: OK, S_tracedump_QUEUE_TIMEOUT
*
*
*****************************************************************
*
* void tTrace::TraceClear(void)    
*
* Description
*
* clear this->mark buffer 

[[D tTrace::TraceClear(void)]
[ invalid_pointer(this) => UNDEFINED
| true => this->fastbuf.length := 0
]]    
*
* 
* RETURNS: void
*
*
*****************************************************************
*
* void tTrace::TraceFreeze(void)    
*
* Description
*
* freeze additions to mark buffer

[[D tTrace::TraceFreeze(void)]
[ invalid_pointer(this) => UNDEFINED
| true => this->bFrozen := TRUE
]]    
*
* 
* RETURNS: void
*
*
*****************************************************************
*
* void tTrace::TraceUnfreeze(void)    
*
* Description
*
* unfreeze additions to mark buffer 

[[D tTrace::TraceUnfreeze(void)]
[ invalid_pointer(this) => UNDEFINED
| true => this->bFrozen := FALSE
]]    
*
* 
* RETURNS: void
*
*
*****************************************************************
*
* ERROR_CODE tTrace::Traceback(int ec, const char *fmt, ...)     
*
* Description
*
* write a diagnostic message to console via msg queue,
* with message level INFO if ec==0 and ERROR otherwise 

[[D tTrace::Traceback(int ec, const char *fmt, va_args args) ]
[ invalid_pointer(this) => UNDEFINED 
| insufficient memory to allocate print string => UNDEFINED
| (DIAGS_NOINFO is #defined || DIAGS_NOWARN is #defined) && ec==0 => identity
    : return := ec
| bTraceMsgUsesLogMsg => 
   console := console + traceback_message(tmt_INFO, ec, fmt, args) via logMsg
 : return := ec
| message queue this->msgQId returns error on send && ec == 0 => 
      logmsg "ERROR: Traceback message queue error" 
    : printf "ERROR: Traceback message queue error" then 
         traceback_message(tmt_INFO, ec, fmt, args)
    : return := ec      
| msgQ *(this->msgQId) returns error on send => 
      logmsg "ERROR: Traceback message queue error" 
    : printf "ERROR: Traceback message queue error" then 
         traceback_message(tmt_ERROR, ec, fmt, args)
    : return := ec      
| task this->iTraceTaskId not running or cannot write
    message to console via message queue this->msgQId => No response?? 
| ec == 0 => 
   console := console + traceback_message(tmt_INFO, ec, fmt, args)
      via msgQ this->msgQId
    : return := ec
| true =>  
   console := console + traceback_message(tmt_INFO, ec, fmt, args) 
      via msgQ this->msgQId
 : return := ec
]]
* 
* RETURNS: ERROR_CODE ec
*
*
*****************************************************************
*
* ERROR_CODE tTrace::TraceMsg(TRACE_MSG_TYPE mtMsgType, 
*                               int ec, const char *fmt, va_args args)     
*
* Description
*
* write a diagnostic message to console via msg queue,
* with message level mtMsgType 
  
[[D tTrace::TraceMsg(TRACE_MSG_TYPE mtMsgType, 
                     int ec, const char *fmt, va_args args) ]
[ invalid_pointer(this) => UNDEFINED 
| insufficient memory to allocate print string => UNDEFINED

| DIAGS_NOINFO is #defined && mtMsgType==tmt_INFO => identity
   : return := ec
| DIAGS_NOWARN is #defined && 
    (mtMsgType==tmt_INFO || mtMsgType==tmt_WARN) => identity
   : return := ec
| bTraceMsgUsesLogMsg => append tracedump_logsb(iCount,imarkbufferarray,
           imarkbufferlength,imarkbuffernewest) to console via logMsg
  : return := ec                  
| message queue this->msgQId returns error on send => 
      logmsg "ERROR: Traceback message queue error" 
    : printf "ERROR: Traceback message queue error" then 
         traceback_message(mtMsgType, ec, fmt, args)
    : return := ec      
| task this->iTraceTaskId not running or cannot write
    message to console via message queue this->msgQId => No response?? 
| true => append tracedump_logsb(iCount,imarkbufferarray,
                  imarkbufferlength,imarkbuffernewest) to msgQ this->iMsgQId
  : return := ec                  
]]
* 
* RETURNS: ERROR_CODE ec
*
*
*****************************************************************
*
* tTrace::Okay(void)
*
* Description
*
* future will return OK if this->state good, false otherwise
*  currently returns OK  

[[D tTrace::Okay(void) ]
[ invalid_pointer(this) but not null => UNDEFINED
| true => return := OK
]]
* 
* RETURNS: OK
*
*
*****************************************************************
*****************************************************************
*
* tTrace::TraceMsgOut(void)
*
* Description
*
* drain message queue and relay to console 


[[D tTrace::TraceMsgOut(void) ]
 [ invalid_pointer(this) || method called from ISR => UNDEFINED
 | unable to receive message from this->msgQId =>
     OS.Task.errno := appropriate error code
 | true => remove all messages from message queue this->msgQId 
     and display them on the console
]]
* 
* RETURNS: OK
*
*
*****************************************************************/

/*****************************************************************
* The following functions are dependent on the declared type of 
* the static item theTrace
*
[[D main::Traceback (const int ec, const char *fmt, va_list args) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::Traceback(ec, fmt,
args)]]
| true => UNDEFINED ]]

[[D main::TraceMark (const int iStatus) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceMark(iStatus)]]
| true => UNDEFINED ]]

[[D main::TraceDump (const int iCount, const int iFromNewest) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceDump(iCount,iFromNewest)]] 
| true => UNDEFINED ]]

[[D main::TraceDumpText (const int iCount, const int iFromNewest) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceDumpText(iCount,iFromNewest)]]
| true => UNDEFINED ]]

[[D main::TraceClear (void) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceClear(void)]]
| true => UNDEFINED ]]
    
[[D main::TraceFreeze (void) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceFreeze(void)]]
| true => UNDEFINED ]]

[[D main::TraceUnfreeze (void) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceUnfreeze(void)]]
| true => UNDEFINED ]]

[[D main::TraceMsg (const TRACE_MSG_TYPE tmtMsgType, const int ec, 
                    const char *fmt, va_list args) ]
[ theTrace instanceof tNoTrace => identity
| theTrace instanceof tTrace => [[U theTrace.tTrace::TraceMsg(tmtMsgType,ec,fmt,args)]]
| true => UNDEFINED ]]

**********************************************************************/

class tTrace {
  protected:
    int imarkbufferarray[CiMaxTraceMarks];
    MSG_Q_ID msgQId;
    int iTraceTaskId;
    int imarkbufferlength;
    int imarkbuffernewest;
    BOOL bFrozen;
    PTIMEFUNC pGetTimeFunc;    // BEH
  public: 
    tTrace(void);
    ~tTrace(void);
    void TraceFreeze(void);
    void TraceUnfreeze(void);
    ERROR_CODE TraceMsgCommon (const TRACE_MSG_TYPE mtMsgType, 
                               const int ec, const char* fmt, va_list arglist);
    ERROR_CODE TraceMark(const int iStatus);
    ERROR_CODE TraceDump(const int iCount, const int iFromNewest=0);      
    ERROR_CODE TraceDumpText(const int iCount, const int iFromNewest=0);      
    void TraceClear(void);    
    ERROR_CODE Traceback (const int ec, const char *fmt, ...);    
    ERROR_CODE TraceMsg(const TRACE_MSG_TYPE tmtMsgType, 
                        const int ec, const char *fmt, ...);    
    ERROR_CODE Okay(void);
    void TraceMsgOut (void);
    void SetGetTimeFunc(PTIMEFUNC pNewGetTimeFunc);  //BEH
    
    BOOL bTraceMsgUsesLogMsg;
};


class tNoTrace {
  protected:
  public: 
    tNoTrace(void){}
    ~tNoTrace(void){}
    void TraceFreeze(void){}
    void TraceUnfreeze(void){}
    ERROR_CODE TraceMark(const int iStatus){return OK;}
    ERROR_CODE TraceDump(const int iCount, const int iFromNewest=0){return OK;}
    ERROR_CODE TraceDumpText(const int iCount, const int iFromNewest=0){return OK;}
    void TraceClear(void){}    
    ERROR_CODE Traceback (const int ec, const char *fmt, ...){return ec;}
    ERROR_CODE TraceMsg(const TRACE_MSG_TYPE tmtMsgType, 
                        const int ec, const char *fmt, ...){return OK;}
    ERROR_CODE Okay(void){return OK;}
    void TraceMsgOut (void){} 
    void SetGetTimeFunc(PTIMEFUNC pNewGetTimeFunc) { }  //BEH

    BOOL bTraceMsgUsesLogMsg;
};

extern "C" {
  ERROR_CODE Traceback(const int ec, const char *fmt, ...);
  ERROR_CODE TraceMark(const int iStatus);
  ERROR_CODE TraceDump(const int iCount, const int iFromNewest=0);      
  ERROR_CODE TraceDumpText(const int iCount, const int iFromNewest=0);      
  void TraceClear(void);    
  void TraceFreeze(void);
  void TraceUnfreeze(void);
  ERROR_CODE TraceMsg(const TRACE_MSG_TYPE mtMsgType, 
                               const int ec, const char *fmt, ...);    
  void TraceUseLogMsg(const BOOL bNewValue);
  void TraceSetGetTimeFunc(PTIMEFUNC pNewGetTimeFunc);   //BEH
}                              
#endif


