/* $RCSfile: typeTag.h,v $ */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/include/typeTag.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
04a,rlj,Sept2,97    writen
*/


#ifndef INC_typeTagh
#define INC_typeTagh

#include <stddef.h>

/****************************************************************************
Black Box State:


--------------------------------------------------------------------------
tTypeTag

STREAM_TYPE   stType        - enum identifying type of stream (error,data,etc.)
TAG_NAME      tnName        - enum that describes type of data
int           iGizmoId      - System Id of the gizmo sending information
size_t        szSize        - number of bytes 
 
---------------------------------------------------------------------------
Stimuli

[[D tTypeTag::tTypeTag (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]

[[D tTypeTag::tTypeTag_STTNisz (STREAM_TYPE stNameToUse,TAG_NAME tnNameToUse, 
               int iGizmoIdToUse, size_t szBlockSizeToUse) ]
[ invalid_pointer (this) => UNDEFINED 
| true => 
    this->stType := stTypeToUse
    : this->tnName := tnNameToUse
    : this->iGizmoId := iGizmoIdToUse
    : this->szSize := szBlockSizeToUse
]]

[[D tTypeTag::~tTypeTag (void) ]
[ invalid_pointer (this) => UNDEFINED 
| true => *this := anything
]]

[[D tTypeTag::StreamType (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->stType
]]

[[D tTypeTag::TagName (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->tnName
]]

[[D tTypeTag::GizmoId (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := this->iGizmoId
]]

[[D tTypeTag::Size (void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := szBlockSize
]]  
*/


// Every new additon to STREAM_TYPE, and TAG_NAME must also have a case in the
// respective operator<< in order to display correct text string

typedef enum STREAM_TYPE {ST_UNKNOWN, ST_ERROR, ST_DATA, 
                          ST_COMMAND, ST_STATUS} STREAM_TYPE;
                          
typedef enum TAG_NAME {TN_UNKNOWN,TN_ERROR_INFO,
                       TN_QUIC_DL_STATUS,
                       TN_GSU_DL_STATUS, TN_DL_DATA,
                       TN_EM_STATUS, TN_EM_DATA,
                       TN_ST_STATUS, TN_ST_IMAGE,
                       TN_ST_STAR_POSITION,
                       TN_ST_FSM_POSITION,
                       TN_WAP_STATUS,
                       TN_WAP_STAR_POSITION,
                       TN_WAP_COLLECTOR_POSITION,
                       TN_FT_STATUS, TN_FT_DATA,
                       TN_FT_IMAGE, TN_FT_FULL_IMAGE } TAG_NAME;

class ostream;

ostream &operator<< (ostream& stream,const STREAM_TYPE &stType);
ostream &operator<< (ostream& stream,const TAG_NAME &tnName);

// inlined functions to allow gui code to use methods.

class tTypeTag {
   public:
     tTypeTag () {
       this->stType = ST_UNKNOWN;
       this->tnName = TN_UNKNOWN;
       this->iGizmoId = -1;
       this->szBlockSize = 0;
     } 

     tTypeTag (STREAM_TYPE stNameToUse,TAG_NAME tnNameToUse, 
               int iGizmoIdToUse, size_t szBlockSizeToUse) {
       this->stType = stNameToUse;
       this->tnName = tnNameToUse;
       this->iGizmoId = iGizmoIdToUse;
       this->szBlockSize = szBlockSizeToUse;
     } 

     STREAM_TYPE StreamType () const { return this->stType; }
     TAG_NAME TagName () const { return this->tnName; }
     int GizmoId () const { return this->iGizmoId; }
     size_t Size () const { return this->szBlockSize; }
     friend ostream &operator<< (ostream& stream,const tTypeTag &TypeTag);

   protected:
     STREAM_TYPE stType;
     TAG_NAME tnName;
     int iGizmoId;
     size_t szBlockSize;
};

#endif typeTagh
