/*****************************************************************************
*
* intLib specs - specifications for VxWorks functions:
*     intConnect()
*     sysIntEnable()
*     sysIntDisable()
*
*
******************************************************************************
*
[[D main::intConnect(VOIDFUNCPTR *vector, VOIDFUNCPTR routine, int parameter)]
This routine connects a specified C routine to a specified interrupt vector. The 
address of routine is stored at vector so that routine is called with parameter
when the interrupt occurs.
[ invalid_pointer(vector) ||
  invalid_pointer(routine) ||
  vector is not a valid VxWorks Vector number ||
  (routine is not a pointer to valid C function &&
   !(routine is a pointer to valid C++ function &&
     parameter == this)) => UNDEFINED
| interrupt handler cannot be built =>
    return := appropriate error code
| true =>
    Set up Vector table so routine is called with parameter when Vector is executed
    : return := OK
]]
*
*
[[D main::sysIntEnable(int intLevel)]
Enables interrupt number intLevel.  CPU starts responding to this interrupt.  A
call to sysIntDisable() will cause the CPU to ignore the interrupt.  intConnect
must be called before this function or else
[ vector assigned to intLevel on slave board is invalid ||
  intConnect has not been called to configure the vector => UNDEFINED
| intLevel < MIN_BUS_INT_PRIORITY ||
  intLevel > MAX_BUS_INT_PRIORITY =>
    return := appropriate error code
| true =>
    CPU begins responding to interrupt number intLevel.
]]
*
*
[[D main::sysIntDisable(int intLevel)]
Disables interrupt number intLevel.  CPU stops responding to this interrupt.  A
call to sysIntEnable() will cause the CPU to begin responding to the interrupt again.
[ intLevel < MIN_BUS_INT_PRIORITY ||
  intLevel > MAX_BUS_INT_PRIORITY =>
    return := appropriate error code
| true =>
    CPU stops responding to interrupt number intLevel.
]]
*
*
*/

