/*$RCSfile: semLib.spec,v $*/
/*$Header: /home/chara/cvsroot/ople/sys/include/vxworks/semLib.spec,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/***
Brad Hines
Created ~10/15/96

***/

/**********************************************************************
SemLib Types
------------
typedef enum { SEM_BINARY, SEM_COUNTING, SEM_MUTEX } SEM_TYPE;
typedef enum { SEM_EMPTY, SEM_FULL } SEM_B_STATE;
Semaphore :== {
  SEM_TYPE     Type
  SEM_ID       Id
  VwTaskList   PendedTaskList   // An ordered list of tasks waiting on the sem
}
**********************************************************************/

/*** The ordering of the task list depends on the type of semaphore and
 *** how it was created.
  
BinarySemaphore :== public Semaphore {
  int          QueuingMethod
  SEM_B_STATE  State
}

 *** For binary semaphores, the PendedTaskList is ordered according to
 *** task priority if QueuingMethod == SEM_Q_PRIORITY, or by the order
 *** in which the tasks called semTake if QueuingMethod == SEM_Q_FIFO.
 ***/



  

/**********************************************************************
SemLib State Variables
----------------------

OS.SemaphoreSet :==  Set of semaphores


SemLib Spec functions
---------------------
SEM_ID NewBinarySemaphore(int QueuingMethod, SEM_B_STATE initialState) :==
  [ QueuingMethod not in { SEM_Q_FIFO, SEM_Q_PRIORITY } -> UNDEFINED
  | true ->
       return a new binary semaphore with
          .Type          := SEM_BINARY
          .QueuingMethod := QueuingMethod
          .State         := initialState
          .Id            := a unique semaphore id
  ]
   
Semaphore operator*(SEM_ID) :==
  [ return the semaphore represented by SEM_ID ]

  Example:  *siSemaphore

BOOL SemAvailable(VwTask ThisTask, SEM_ID semId) :==
  [ *semId.Type == SEM_BINARY ->
        ....
  ]


 This function returns true if the semaphore can be taken within the
 timeout specified by iTimeoutInTicks.

 BOOL CanTakeSemaphoreWithinSpecifiedTimeout(SEM_ID siSemaphore,
                                               int iTimeoutInTicks) :==
       [ iTimeoutInTicks != WAIT_FOREVER &&
            (iTimeoutInTicks == NO_WAIT &&
               ! SemAvailable(siSemaphore) ||
               SemAvailable(siSemaphore) not true within
                                     iTimeoutInTicks ticks) ->
           return FALSE
       | true ->
           return TRUE
       ]
***********************************************************************/ 


/*************************************************************************
*
SEM_ID semBCreate(
                  int options,                semaphore options       
                  SEM_B_STATE initialState    initial semaphore state 
                  )
*
* DESCRIPTION 
*         This routine allocates and initializes a binary semaphore. The 
*         semaphore is initialized to the initialState of either SEM_FULL 
*         (1) or SEM_EMPTY (0).
*
*
*         The options parameter specifies the queuing style for blocked 
*         tasks. Tasks can be queued on a priority basis or a 
*         first-in-first-out basis. These options are SEM_Q_PRIORITY 
*         (0x1) and SEM_Q_FIFO (0x0), respectively. 
*
* RETURNS 
*   The semaphore ID, or NULL if memory cannot be allocated. 
*
* IN:
*   options
*   initialState
*
* INOUT:
*   OS.SemaphoreSet
*
* OUT:
*   
* SPEC:
*
    [[D ::semBCreate(in: options, initialState; out: return;)]
    [ memory for semaphore cannot be allocated => 
       return := NULL
    | true =>
       OS.SemaphoreSet := OS.SemaphoreSet U
                                { NewBinarySemaphore(options, initialState) }
       : return := NewBinarySemaphore(options, initialState).Id
                             (the same semaphore that is appended to the set)
    ]]
*/



/*************************************************************************
*
STATUS semTake(
              SEM_ID  semId,    semaphore ID to take 
              int     timeout   timeout in ticks     
              )
*
* DESCRIPTION 
*         This routine performs the take operation on a specified 
*         semaphore. Depending on the type of semaphore, the state of 
*         the semaphore and the calling task may be affected. The 
*         behavior of semTake( ) is discussed fully in the library 
*         description of the specific semaphore type being used.
*
*         A timeout in ticks may be specified. If a task times out, 
*         semTake( ) will return ERROR. Timeouts of WAIT_FOREVER 
*         (-1) and NO_WAIT (0) indicate to wait indefinitely or not to wait 
*         at all.
*
*         When semTake( ) returns due to timeout, it sets the errno to 
*         S_objLib_OBJ_TIMEOUT (defined in objLib.h).
*
*         The semTake( ) routine is not callable from interrupt service 
*         routines.
*
* RETURNS 
*          OK, or ERROR if the semaphore ID is invalid or the task timed
*          out. 
*
* IN:
*   semId
*   timeout
*
* INOUT:
*   *semId
*   OS.Task.errno
*
* OUT:
*   
* SPEC:
*
    [[D ::semTake(in: semId, timeout; out: return;)]
    [ called from interrupt level => UNDEFINED
    | semId invalid =>
        OS.Task.errno := S_objLib_OBJ_ID_ERROR
        : return := ERROR
    | timeout != WAIT_FOREVER &&
            (timeout == NO_WAIT && !SemAvailable(semId) ||
             SemAvailable(semId) not true within timeout ticks) =>
        OS.Task.errno := S_objLib_OBJ_TIMEOUT
        : return := ERROR
           (prior to timeout, the task is inserted into
            *semId.PendedTaskList as specified by *semId.QueuingMethod)
    | true =>
        Calling task is blocked until SemAvailable(semId) is true
           (while blocked, the task is inserted into
            *semId.PendedTaskList as specified by *semId.QueuingMethod)
        : [ *semId.Type == SEM_BINARY =>
              *semId.State := SEM_EMPTY
          | true =>
              unfinished at this time (11/13/96)
          ]
        : return := OK
    ]]
*/



/*************************************************************************
*
STATUS semGive(
              SEM_ID  semId,    semaphore ID to give 
              )
*
* DESCRIPTION 
*          This routine performs the give operation on a specified 
*          semaphore. Depending on the type of semaphore, the state of 
*          the semaphore and of the pending tasks may be affected. The 
*          behavior of semGive( ) is discussed fully in the library 
*          description of the specific semaphore type being used. 
*
*
* RETURNS 
*          OK, or ERROR if the semaphore ID is invalid.
*
* IN:
*   semId
*   timeout
*
* INOUT:
*   *semId
*   OS.Task.errno
*
* OUT:
*   
* SPEC:
*
    [[D ::semGive(in: semId; out: return;)]
    [ semId invalid =>
        OS.Task.errno := S_objLib_OBJ_ID_ERROR
        : return := ERROR
    | *semId.PendedTaskList not empty =>
        First task in *semId.PendedTaskList unblocks
        : First task removed from *semId.PendedTaskList
        : return := OK
    | true =>
        [ *semId.Type == SEM_BINARY =>
            *semId.State := SEM_FULL
        | true =>
            unfinished at this time (11/13/96)
        ]
        : return := OK
    ]]
*/


/*************************************************************************
*
     STATUS semDelete
         (
         SEM_ID  semId   semaphore ID to delete 
         )
*
* DESCRIPTION
*      This routine terminates and deallocates any  memory  associ-
*      ated  with  a  specified  semaphore.   Any pended tasks will
*      unblock and return ERROR.
* 
* RETURNS
*      OK, or ERROR if the semaphore ID is invalid.
* 
* IN:
*   semId
* INOUT:
*   OS.SemaphoreSet
*
* OUT:
*   
* SPEC:
*
    [[D ::semDelete(in: semId; out: return;)]
    [ semId invalid =>
        OS.Task.errno := S_objLib_OBJ_ID_ERROR
        : return ERROR
    | true =>
        All tasks in *semId.PendedTaskList unblock
        : OS.SemaphoreSet := OS.SemaphoreSet with *semId removed
        : semId := invalid
        : return OK
    ]]
*/

