/*
$RCSfile: taskLib.spc,v $
$Header: /home/chara/cvsroot/ople/sys/include/vxworks/taskLib.spc,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/*************************************************************************
*
     int taskSpawn
         (
         char     *name,       name of new task (stored at pStackBase)   
         int      priority,    priority of new task                      
         int      options,     task option word                          
         int      stackSize,   size (bytes) of stack needed plus name    
         FUNCPTR  entryPt,     entry point of new task                   
         int      arg1,        1st of 10 req'd task args to pass to func 
         int      arg2,
         int      arg3,
         int      arg4,
         int      arg5,
         int      arg6,
         int      arg7,
         int      arg8,
         int      arg9,
         int      arg10
         )
*
* DESCRIPTION
*      This routine creates and activates a new task with a  speci-
*      fied  priority and options and returns a system-assigned ID.
*      See taskInit() and taskActivate() for the building blocks of
*      this routine.
* 
*      A task may be assigned a name as a debugging aid.  This name
*      will appear in displays generated by various system informa-
*      tion facilities such as i().  The name may be  of  arbitrary
*      length and content, but the current VxWorks convention is to
*      limit task names to ten characters and prefix  them  with  a
*      "t".   If <name> is specified as NULL, an ASCII name will be
*      assigned to the task of the form  "t<n>"  where  <n>  is  an
*      integer which increments as new tasks are spawned.
* 
*      The only resource allocated to a spawned task is a stack  of
*      a  specified  size  <stackSize>, which is allocated from the
*      system memory partition.   Stack  size  should  be  an  even
*      integer.   A  task  control  block  (TCB) is carved from the
*      stack, as well as any memory required by the task name.  The
*      remaining  memory  is  the  tasks  stack  and every byte is
*      filled with the value 0xEE for  the  checkStack()  facility.
*      See  the manual entry for checkStack() for stack-size check-
*      ing aids.
* 
*      The entry address <entryPt> is the  address  of  the  "main"
*      routine  of the task.  The routine will be called once the C
*      environment has been set up.  The specified routine will  be
*      called  with  the ten given arguments.  Should the specified
*      main routine return, a call to exit() will automatically  be
*      made.
* 
*      Note that ten (and only ten) arguments must  be  passed  for
*      the spawned function.
* 
*      Bits in the options argument may be set to run with the fol-
*      lowing modes:
* 
*      VX_FP_TASK          - execute with floating-point  coproces-
*                            sor support.
* 
*      VX_PRIVATE_ENV      - include  private  environment  support
*                            (see envLib).
* 
*      VX_NO_STACK_FILL    - do not  fill  the  stack  for  use  by
*                            checkstack().
* 
*      See the definitions in taskLib.h.
* 
* RETURNS
*      The task ID, or ERROR if memory is insufficient or the  task
*      cannot be created.
*
*
* SPEC:
*
[[D taskLib::taskSpawn 
     (in: name,priority,options,stackSize,entryPt,arg1,arg2,arg3,
          arg4,arg5,arg6,arg7,arg8,arg9,arg10;
      inout: OS;
      out: return, OS.Task.errno;)]
[ ! (entryPt is a pointer to valid C function ||
      (entryPt is a valid pointer to a non-virtual C++ method &&
       arg1 is a valid pointer to a C++ object with method *entryPt)) =>
                                             UNDEFINED
| Task cannot be created || Insufficient memory to spawn task =>
       OS.Task.errno := VxWorks errno (taskSpawn)
       : return := ERROR
| entryPt is a pointer to a C++ method =>
    OS.SpawnedTasks := OS.SpawnedTasks U
                       { (arg1->(*entryPt))(arg2, ...), with specified
                         name, priority, options, and stack size }
    : return := new task ID from VxWorks
| true =>
    OS.SpawnedTasks := OS.SpawnedTasks U
                       { (*entryPt)(arg1, ...), with specified
                         name, priority, options, and stack size }
    : return := new task ID from VxWorks
]]
*
*/




/*************************************************************************
*
     STATUS taskDelete
         (
         int  tid   task ID of task to delete 
         )

* DESCRIPTION
*      This routine causes a specified task to cease to  exist  and
*      deallocates  the  stack and WIND_TCB memory resources.  Upon
*      deletion, all routines specified by taskDeleteHookAdd() will
*      be called in the context of the deleting task.  This routine
*      is the companion routine to taskSpawn().
* 
* RETURNS
*      OK, or ERROR if the task cannot be deleted.
*
* SPEC:
*
[[D taskLib::taskDelete (in: tid; out: return;)]
[ tid not a valid task id =>
    OS.Task.errno := S_objLib_OBJ_ID_ERROR
    : return :=ERROR
| task cannot be deleted =>
    OS.Task.errno := VxWorks errno (taskDelete)
    : return := ERROR
| true =>
    OS.SpawnedTasks := OS.SpawnedTasks with the task with id tid removed
    : return := OK
]]
*
*/

