/* $RCSfile: stepperMultiActuator.h,v $ */
/* CM Info:$Header: /home/chara/cvsroot/ople/sys/include/wap/stepperMultiActuator.h,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/

#ifndef INC_stepperMultiActuatorh
#define INC_stepperMultiActuatorh

#include "multiActuator.h"
#include "inputSource.h"

/****
Black Box State Variables
-------------------------------------- 
const int            CiActuators          Number of actuators
tModulatedActuator   pActuatorArray[CiActuators]
                                          Stepper actuators
tSensor              pSensorArray[CiActuators]
                                          Position registers
tMatrix              XformDetToAct(2,CiActuators)   
                                          Transform matrix, detector-to-actuators
tVector              Home(CiActuators)    Home position
Bool                 bValidXform          TRUE if XformDetToAct has been initialized
Bool                 bValidHome           TRUE if Home has been initialized
tVector              Position             Position from actuator registers


[[D tStepperMultiActuator::tStepperMultiActuator(int iActuators,
                                   InputChannel *const InputChannelArray[],
                                   double dInputScaleFactor,
                                   tOutputChannel *const OutputChannelArray[],
                                   double dOutputScaleFactor) ]
constructor
[  invalid_pointer(this) ||
   invalid_pointer(ChannelArray) => UNDEFINED
|  for 0 <= i < iActuators
      invalid_pointer(ChannelArray[i]) => UNDEFINED
| true =>
      allocate memory for pActuatorArray and pSensorArray and initialize as 
         follows:
         for 0 <= i < iActuators
         : this->pSensorArray[i]->Channel := a reference to *InputChannelArray[i]
         : this->pSensorArray[i]->dElectricalToPhysicalScaleFactor := 
            dInputScaleFactor
         : this->pActuatorArray[i]->ModulationSource := 
               this->pActuatorArray[i]->ModulationSource set for zero modulation
         : this->pActuatorArray[i]->Channel := a reference to *OutputChannelArray[i]
         : this->pActuatorArray[i]->dElectricalToPhysicalScaleFactor := 
            dOutputScaleFactor
         : Appropriate hardware output := 0.0
         : this-> bValidXform := FALSE
      : this->CiActuators := iActuators
]]

[[D tStepperMultiActuator::~tStepperMultiActuator() ]
destructor
[  invalid_pointer(this) => UNDEFINED
| true =>
      deallocate memory for pStepperArray 
]]

[[D tStepperMultiActuator::SetHome(const tPoint& Position) ]
Assign a value to HomePosition
[  invalid_pointer(this) ||
   invalid_reference(Position) => UNDEFINED
|  Position not length this->CiActuators => this->HomePosition := anything
|  true=>
      this->HomePosition := Position
      : return := OK
]]

[[D tStepperMultiActuator::SetHomeParams(tVector& NewHardStop,
                                         TIME tNewHardStopTime,TIME tNewHomeTime) ]
Assign values to Home parameters
[  invalid_pointer(this) ||
   invalid_reference(HardStop) ||
   length of tVector != this->CiActuators => UNDEFINED
|  true=>
        HardStop := NewHardStop
      : HardStopTime := NewHardStopTime
      : HomeTime := NewHomeTime
      : return := OK
]]


[[D tStepperMultiActuator::Home(void) ]
Move the FSM to the home position
[  invalid_pointer(this) => UNDEFINED
|  invalid(this->HomePosition) || invalid(this->XformDetToFsm) => UNDEFINED
|  !this->ValidHome => return := appropriate error
|  true =>
     mirror moves to hard stops and then to home position
   : for 0 <= i < this->CiActuators
        this->tMultiActuator[i].Offset := -HomeConstants.HardStop[i]
      : this->tMultiActuator[i].Value := 0
      : this->pSensorArray[i].Value := 0
   : return := OK
]]

[[D tStepperMultiActuator::Position(void) ]
return position from hardware register, in detector coordinates
[  invalid_pointer(this) => UNDEFINED
|  for 0 <= i < this->CiActuators
   invalid_sensor(pSensorArray[i]) => UNDEFINED
| true => 
   return := this->XformActToDet * (vector of values read from pSensorArray)
]]

****/

/*
This derived class adds the concept of home:
1) SetHome() specifies the distance (in radians) between the hard stop and the home
position.
2) SetHomeParams() specifies the other parameters needed to get to the hardstop
and then to home.
3) Home() move the actuators to the hard stops and then to home, and it resets
the hardware position register.

It also addes the hardware position registers (see GetPosition()).
*/

class tStepperMultiActuator : public tMultiActuator {
   public:
      tStepperMultiActuator(int iActuators,
                            tInputChannel *const InputChannelArray[2],
                            double dInputScaleFactor,
                            tOutputChannel *const OutputChannelArray[2],
                            double dOutputScaleFactor);
      virtual ~tStepperMultiActuator();
      ERROR_CODE Move(TIME tTime,const tVelocity& Velocity)
         {return tMultiActuator::Move(tTime,Velocity);}     
      virtual ERROR_CODE Home();
      void SetHome(const tVector& NewHomePosition);
      void SetHomeParams(const tVector& NewHardStop,
                         double dNewHardStopTime,double dNewHomeTime);
      tPoint Position(void) const;      
   private:
      enum {X,Y};

      tStepperMultiActuator(const tStepperMultiActuator&); //not allowed
      tStepperMultiActuator& operator=(const tStepperMultiActuator&); //not allowed
      tSensor **pSensorArray;
      tVector HomePosition;
      tVector HardStop;
      double dHardStopTime;
      double dHomeTime;
};




#endif
