/* $RCSfile: wapController.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/wap/wapController.h,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */


#ifndef wapControllerh
#define wapControllerh

#include "isit.h"
#include "pidLibF.h"
#include "ipcDblBufferAccessor.h"
#include "wapTypes.h"
#include "wapCommands.h"
#include "controller.h"
#include "controllerSet.h"
#include "dataStore.h"
#include "matrix.h"
#include "wap/stepperMultiActuator.h"
#include "paddle.h"

/*******
Black Box State Variables
--------------------------

tWapControllerData 

tWapController
tWapTrackController
tWapCompleteControllerSet


------------------------------------------------
Stimuli


******************************************************************************
**********************  class tWapController  *********************************
******************************************************************************

[[D tWapController::tWapController(tWapCompleteIoSet& CompleteIoSet,
                                 tWapControllerData& ControllerDataToUse) ]
[ invalid_pointer(this) ||
  invalid_reference(ControllerDataToUse) ||
  invalid_reference(CompleteIoSet) => UNDEFINED
| true => 
   this->ControllerData := reference to ControllerDataToUse
   : this->Collector := reference to ControllerIoSet.Collector
]]

[[D tWapController::~tWapController(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => *this := anything
]]

[[D tWapController::Reset(TIME tServoTime) ]
stops the Collector
[ invalid_pointer(this) ||
  invalid_reference(this->Collector) => UNDEFINED
| true =>
   stop this->Collector
]]

[[D tWapController::SendPosition(void) ]
puts the actuator position into the data stream
[ invalid_pointer(this) ||
  invalid_reference(this->Collector) => UNDEFINED
| true =>
   DataStream := DataStream + CollectorPosition.Value                     
]]

[[D tWapController::Calculate(TIME tServoTime) ]
puts the actuator position into the data stream
[ invalid_pointer(this) ||
  invalid_reference(this->Collector) => UNDEFINED
| true =>
   DataStream := DataStream + CollectorPosition.Value                     
]]

******************************************************************************
**********************  class tWapSlewController   ****************************
******************************************************************************

[[D tWapSlewController::tWapSlewController(tWapCompleteIoSet& CompleteIoSet,
                                   tWapControllerData& ControllerDataToUse) ]
[ invalid_pointer(this) ||
  invalid_reference(ControllerDataToUse) ||
  invalid_reference(CompleteIoSet) ||
  invalid_actuator(this->Collector) => UNDEFINED
| true => 
   this->ControllerData := reference to ControllerDataToUse
   : this->Collector := reference to ControllerIoSet.Collector
   : this->Paddle := reference to ControllerIoSet.Paddle
]]

[[D tWapSlewController::~tWapSlewController(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => *this := anything
]]

[[D tWapSlewController::Calculate(TIME tServoTime) ]
move the collector at the velocity specified by the paddle
[ invalid_pointer(this) => UNDEFINED
| true =>
    multiActuator.actuator[0]->Value := Paddle.Velocity[0]
  : multiActuator.actuator[1]->Value := Paddle.Velocity[1]
  : DataStream := DataStream + CollectorPosition.Value                     
  : return := OK 
]]


******************************************************************************
**********************  class tWapTrackController  ****************************
******************************************************************************

[[D tWapTrackController::tWapTrackController(tWapCompleteIoSet& CompleteIoSet,
                                           tWapControllerData& ControllerDataToUse,
                                           TIME tServoPeriod) ]
[ invalid_pointer(this) ||
    invalid_reference(CompleteIoSet) ||
    invalid_reference(ControllerDataToUse) ||
    invalid_actuator(this->Collector) => UNDEFINED
| true => 
   this->ControllerData := reference to ControllerDataToUse 
   : this->Collector := reference to CompleteIoSet.Collector
   : this->CServoPeriod := tServoPeriod
]]

[[D tWapTrackController::~tWapTrackController(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
     *this := anything
]]

[[D tWapTrackController::Calculate(TIME tServoTime) ]
moves the collector at a velocity proportional to the FSM error
[ invalid_pointer(this) ||
  invalid_actuator(this->Collector) => UNDEFINED
| CtServoPeriod < DBL_EPSILON => return := appropriate error
| true =>
     this->Collector.Value = -2 * PI * this->dGain * this->Fsm.Position / 
      this->CtServoPeriod * this->Collector.XformDetToAct
   : DataStream := DataStream + CollectorPosition.Value                     
   : return := OK
]]



[[DtWapTrackController::SetControllerParams(const tWapControllerParams& Params) ]
set controller parameters
[ invalid_pointer(this) => UNDEFINED
| invalid_reference(Params) => UNDEFINED
| true =>
   initialize this->Pid with Params
]]

******************************************************************************
**********************  class tWapCalibrateController  ************************
******************************************************************************



[[D tWapCalibrateController::tWapCalibrateController(tWapCompleteIoSet& CompleteIoSet,
                                                   tWapControllerData& ControllerDataToUse) ]
constructor
[  invalid_pointer(this) ||
    invalid_reference(CompleteIoSet) ||
      invalid_reference(ControllerDataToUse) => UNDEFINED
| true =>
   this->ControllerData := reference to ControllerDataToUse
   : this->Collector := a reference to CompleteIoSet.Collector
]]

[[D tWapCalibrateController::Calculate(TIME tServoTime) ]
Move Collector to new calibration position
[  invalid_pointer(this) ||
   invalid_actuator(this->Collector) => UNDEFINED
|  true =>
     move actuator this->iCalibrationActuator to
      MODULATION(this->pActuatorArray[this->iCalibrationActuator]->
         Modulation.mpParams,tServoTime)
   : DataStream := DataStream + CollectorPosition.Value                     
   : return := OK 
]]

*****/

class tWapCompleteIoSet;
class tWapCompleteControllerSet;
class tWapControllerData;
class tWapServoParameters;
class tWapControllerParams;
class tServoInfo;
class tImage;


class tWapController : public tController {
  public:
    tWapController(tWapCompleteIoSet& CompleteIoSet,
                  tWapControllerData& ControllerDataToUse);
    virtual ~tWapController(void);
    virtual ERROR_CODE Calculate(TIME tServoTime);
    virtual ERROR_CODE Reset(TIME tServoTime);
  protected:
    enum {X,Y};

    void SendPosition();
    tWapControllerData& ControllerData;
    tStepperMultiActuator& Collector;
};

class tWapSlewController : public tWapController {
  public:
    tWapSlewController(tWapCompleteIoSet& CompleteIoSet,
                      tWapControllerData& ControllerDataToUse);
    virtual ~tWapSlewController(void);
    virtual ERROR_CODE Calculate(TIME tServoTime);
  private:
      const tPaddle& Paddle;
};


class tWapTrackController : public tWapController {
  public: 
    tWapTrackController(tWapCompleteIoSet& CompleteIoSet,
                       tWapControllerData& ControllerDataToUse,
                       TIME tServoPeriod);
    virtual ~tWapTrackController(void);
    virtual ERROR_CODE Calculate(TIME tServoTime);
    void SetControllerParams(const tWapControllerParams& Params) 
      {dGain = Params.dGain;}
  private:
    double dGain;
    const double CdServoPeriod; //(sec)    
    tIpcDblBufferAccessor<tIpcPoint>& Fsm;
};

class tWapCalibrateController : public tWapController {
   public:
      tWapCalibrateController(tWapCompleteIoSet& CompleteIoSet,
                             tWapControllerData& ControllerDataToUse);
      ERROR_CODE Calculate(TIME tServoTime);
   private:
};


 

#endif wapControllerh
