/* $RCSfile: wapControllerSet.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/wap/wapControllerSet.h,v 1.1.1.1 1998/06/11 21:36:18 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */

#ifndef wapControllerSeth
#define wapControllerSeth

#include "isit.h"
#include "wapTypes.h"
#include "controller.h"
#include "controllerSet.h"
#include "dataStore.h"
#include "matrix.h"
#include "wapController.h"

/*******
Black Box State Variables
--------------------------

tWapControllerData 

tWapController
tWapTrackController
tWapCompleteControllerSet


------------------------------------------------
Stimuli

******************************************************************************
**********************  class tWapControllerData  *****************************
******************************************************************************

[[D tWapControllerData::tWapControllerData(const tTypeTag& StarPosDataTag,
                                           const tTypeTag& CollectorPosDataTag) ] 
[ invalid_pointer(this) ||
  invalid_reference (StarPosDataTag) ||
  invalid_reference (CollectorrPosDataTag)  => UNDEFINED
| CiWapDataDumpPriority < priority st. 
       (Time needed to empty this->StarPositionData.DataBag) < 
          ( CiWapDataBufferSize / (avg rate at which tDataStore::StoreData called) ) ||
    CiWapDataDumpPriority < priority st. 
       (Time needed to empty this->StarImageData.DataBag) < 
          ( CiWapDataBufferSize / (avg rate at which tDataStore::StoreData called) ) =>
     ::ErrorStream := ::ErrorStream + appropriate error 
                messages whenever storing data would cause this->StarPositionData.DataBag 
                        or this->StarImageData.DataBag to overflow
| true =>
     this->StarPositionData.DataBag := EMPTY
     : this->StarPositionData.iMaxItems := CiWapDataBufferSize
     : this->StarPositionData.iDataDumpPriority := CiWapDataDumpPriority
     : this->StarPositionData.DataTag := StarPosDataTag 
     : this->CollectorPositionData.iMaxItems := CiWapDataBufferSize
     : this->CollectorPositionData.iDataDumpPriority := CiWapDataDumpPriority
     : this->CollectorPositionData.DataTag := CollectorPosDataTag 
     : this->StarImageData.DataBag := EMPTY
     : this->StarImageData.iMaxItems := CiWapDataBufferSize
     : this->StarImageData.iDataDumpPriority := CiWapDataDumpPriority
]]


******************************************************************************
**********************  class tWapCompleteControllerSet  **********************
******************************************************************************

[[D tWapCompleteControllerSet::tWapCompleteControllerSet(
                                                 tWapCompleteIoSet& CompleteIoSetToUse, 
                                                 const tTypeTag& WapStatusDataTag, 
                                                 const tTypeTag& StarPosDataTag, 
                                                 const tTypeTag& CollectorPosDataTag, 
                                                 TIME tServoPeriod) ]
[ invalid_pointer(this) ||
  invalid_reference(CompleteIoSetToUse) ||
  invalid_reference(StarPosDataTag) ||
  invalid_reference(CollectorPosDataTag) => UNDEFINED

| CiWapDataDumpPriority < VxWorks priority needed to empty ::DataStream(::pThisRteG) 
      before this->ControllerData.StarPositionData.DataBag or 
          this->ControllerData.StarImageData.DataBag overflows =>
    ::ErrorStream := ::ErrorStream + 
           appropriate error messages whenever time to store data would cause
               this->ControllerData.StarPositionData.DataBag
                    or this->ControllerData.StarImageData.DataBag to overflow
    : other assignments in "true" case probably work
| true =>
    this->ControllerData.StarPositionData.DataBag := EMPTY
    : this->ControllerData.StarPositionData.iMaxItems := CiWapDataBufferSize
    : this->ControllerData.StarPositionData.iDataDumpPriority := CiWapDataDumpPriority
    : this->ControllerData.StarPositionData.DataTag := StarPosDataTag
    : this->ControllerData.CollectorPositionData.DataTag := CollectorPosDataTag
    : this->ControllerData.StarImageData.DataBag := EMPTY
    : this->ControllerData.StarImageData.iMaxItems := CiWapDataBufferSize
    : this->ControllerData.StarImageData.iDataDumpPriority := CiWapDataDumpPriority
    : this->OffController.ControllerData := ControllerData 
    : this->SlewController.ControllerData := ControllerData 
    : this->TrackController.ControllerData := ControllerData
    : this->TrackController.AcquisitionSensor := CompleteIoSetToUse.Sensor 
    : this->CompleteIoSet := CompleteIoSetToUse 
    : this->WapOffControllerGroup.pControllerArray[0] := &this->OffController 
    : this->WapSlewControllerGroup.pControllerArray[0] := &this->SlewController 
    : this->WapTrackControllerGroup.pControllerArray[0] := &this->TrackController 
    : this->WapCalibrateControllerGroup.pControllerArray[0] := &this->CalibrateController 
]]


[[D tWapCompleteControllerSet::~tWapCompleteControllerSet(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
     *this := anything
]]

[[D tWapCompleteControllerSet::OffControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->WapOffControllerGroup
]]

[[D tWapCompleteControllerSet::SlewControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->WapSlewControllerGroup
]]

[[D tWapCompleteControllerSet::TrackControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->WapTrackControllerGroup
]]

[[D tWapCompleteControllerSet::CalibrateControllerGroup(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
      return := reference to this->WapCalibrateControllerGroup
]]

[[D tWapCompleteControllerSet::GetStarPosition(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    return := this->ControllerData.StarPosition
]]

[[D tWapCompleteControllerSet::SetControllerParams(const tWapControllerParams& Params) ]
sets the controller parameters as specified
[ invalid_pointer(this) => UNDEFINED
| invalid_reference(Params) => UNDEFINED
| true => 
    initialize this->TrackController.Lag with Params
    : initialize this->TrackController.Pid with Params
]] 

*****/

class tStarSensor;
class tWapCompleteIoSet;
class tWapCompleteControllerSet;
class tWapServoParameters;
class tServoInfo;
class tImage;


class tWapControllerData {
  public:
    tWapControllerData(const tTypeTag& StarPosDataTag, 
                       const tTypeTag& CollectorPosDataTag);

    tDataStore<tWapData>   StarPositionData;
    tDataStore<tIpcPoint>  CollectorPositionData;
    tPoint                 StarPosition;
};


class tWapCompleteControllerSet {
  public: 
    tWapCompleteControllerSet(tWapCompleteIoSet& CompleteIoSetToUse, 
                              const tTypeTag& WapStatusDataTag, 
                              const tTypeTag& StarPosDataTag, 
                              const tTypeTag& CollectorPosDataTag, 
                              TIME tServoPeriod);
    ~tWapCompleteControllerSet(void);
    tSimpleControllerGroup& OffControllerGroup() 
      {return WapOffControllerGroup;}
    tSimpleControllerGroup& SlewControllerGroup() 
      {return WapSlewControllerGroup;}
    tSimpleControllerGroup& TrackControllerGroup() 
      {return WapTrackControllerGroup;}
    tSimpleControllerGroup& CalibrateControllerGroup() 
      {return WapCalibrateControllerGroup;}
    tPoint GetStarPosition(void) 
      {return this->ControllerData.StarPosition;}
    void SetControllerParams(const tWapControllerParams& Params)
      {this->TrackController.SetControllerParams(Params);}
    
  private:
    tWapControllerData      ControllerData; 
    tWapController          OffController;
    tWapSlewController      SlewController;
    tWapTrackController     TrackController;
    tWapCalibrateController CalibrateController;
    tSimpleControllerGroup WapOffControllerGroup;
    tSimpleControllerGroup WapSlewControllerGroup;
    tSimpleControllerGroup WapTrackControllerGroup;  
    tSimpleControllerGroup WapCalibrateControllerGroup;  
};

 

#endif stControllerh
