/* $RCSfile: wapGizmo.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/wapGizmo.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */


#ifndef INC_wapGizmoh
#define INC_wapGizmoh

#include "isit.h"
#include "realTimeGizmo.h"
#include "typeTag.h"
#include "mutex.h"
#include "wapTypes.h"
#include "imageStatus.h"
#include "wapCommands.h"
#include "wap/wapIoSet.h"
#include "wap/wapControllerSet.h"

const WSM_OFF = 0;
const WSM_SLEW = 1;
const WSM_TRACK = 2;
const WSM_CALIBRATE = 3;

/****************************************************************************
twapCmdEngine Black Box State

tMutexUser     base class     controls tWapGizmo::Collector Mutex
*****************************************************************************

[[D tWapCmdEngine::tWapCmdEngine(const tWapGizmo *pWapGizmo,const char* sTaskName) ]
constructs the command engine
[invalid_pointer(this) ||
 invalid_pointer(pWapGizmo) ||
 invalid_pointer(sTaskName) => UNDEFINED
| true =>
   this->pMutex := address of pWapGizmo->CollectorMutex
   : this->sTaskName := sTaskName
]]

****************************************************************************
twapHomeCmdEngine Black Box State

tMutexUser        base class     controls tWapGizmo::Collector Mutex
const tWapGizmo   *pWapGizmo     pointer to controlling tWapGizmo

*****************************************************************************

[[D tWapHomeCmdEngine::tWapHomeCmdEngine(const tWapGizmo *pWapGizmo) ]
constructs the HOME command engine
[invalid_pointer(this) ||
 invalid_pointer(pWapGizmo) => UNDEFINED
| true =>
   this->pMutex := address of pWapGizmo->CollectorMutex
 : this->sTaskName := "WapHomeCmdEngine"
 : this->pWapGizmo := pWapGizmo
]]

[[D tWapHomeCmdEngine::ExecuteMethod(void) ]
[invalid_pointer(this) ||
  invalid_pointer(WapGizmo) ||
  bKillCurrentOwnerIfTimeout != FALSE ||
  this.pMutex invalid ||
  somebody deletes this object while ExecuteMethod is running ||
  MUTEX_DEFAULT_TIMEOUT ticks elapse ||
  this->CollectorMutex taken again => UNDEFINED
| true =>
     take this->CollectorMutex
   : mirror moves to hard stops and then to home position
   : this->WapGizmo.CompleteIoSet.Collector[0].Offset := 
            -WapGizmo.WapGizmoInfo.ServoConstants.HardStop[0]
   : this->WapGizmo.CompleteIoSet.Collector[1].Offset := 
            -WapGizmo.WapGizmoInfo.ServoConstants.HardStop[1]
   : this->WapGizmo.CompleteIoSet.Collector[0].Value := 0
   : this->WapGizmo.CompleteIoSet.Collector[1].Value := 0
]]  

****************************************************************************
twapMoveCmdEngine Black Box State

tMutexUser        base class     controls tWapGizmo::Collector Mutex
const tWapGizmo   *pWapGizmo     pointer to controlling tWapGizmo

*****************************************************************************

[[D tWapMoveCmdEngine::tWapMoveCmdEngine(const tWapGizmo *pWapGizmo,
                                         const tPoint& DeltaToUse) ]
constructs the HOME command engine
[invalid_pointer(this) ||
 invalid_pointer(pWapGizmo) => UNDEFINED
| true =>
   this->pMutex := address of pWapGizmo->CollectorMutex
 : this->sTaskName := "WapMoveCmdEngine"
 : this->pWapGizmo := pWapGizmo
 : this->Delta := DeltaToUse
]]

[[D tWapMoveCmdEngine::ExecuteMethod(void) ]
[invalid_pointer(this) ||
  invalid_pointer(WapGizmo) ||
  bKillCurrentOwnerIfTimeout != FALSE ||
  this.pMutex invalid ||
  somebody deletes this object while ExecuteMethod is running ||
  MUTEX_DEFAULT_TIMEOUT ticks elapse ||
  this->CollectorMutex taken again => UNDEFINED
| true =>
     take this->CollectorMutex
   : move this->WapGizmo.CompleteIoSet.Collector by Delta
]]  


****************************************************************************
twapGizmo Black Box State

BOOL                bInit
ERROR_CODE          ecInitError
tGizmoBlackBoxServo     Servo - (see Gizmo Specification Document)
tWapStatus                      Status
tStarSensor                     AcquisitionSensor
tWapData                        Data
tImage                          ImageData
tTypeTag                        CWapStatusTag
tTypeTag                        CWapStarPosDataTag

tWapController                   SlewController
tWapTrackController              TrackController
tWapCalibrateController          CalibrateController

TASK_ID                         tidCommandTaskId
TASK_ID                         tidStatusTaskId
TASK_ID                         tidServoTaskId
TASK_ID                         tidStarPosDataTaskId
TASK_ID                         tidStarImageDataTaskId
*****************************************************************************

[[D tWapGizmo::tWapGizmo(const tWapGizmoInfo& GizmoInfo) ]
[ invalid_pointer (this) ||
  invalid_pointer (::pThisRteG) ||
  invalid_reference (GizmoInfo) ||
  GizmoInfo.iRteId != id associated with ::pThisRteG ||
  GizmoInfo.iGizmoId != id associated with this gizmo ||
  IsAFringeTracker (GizmoInfo.iFtGizmoId) == FALSE => UNDEFINED
| GizmoInfo.iNumberOfServoModes != 3 ||
  GizmoInfo.ServoInfo.iNumberOfServoPartitions != 1 ||
  GizmoInfo.gtGizmoType != GT_WAP ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiCommandStreamId) == FALSE ||
  CanAllocateIpcObject (::IpcMemory,GizmoInfo.iiImageId) == FALSE ||
  Unable to add periodic tasks to handle command, status and servo =>
    this->ecInitError := appropriate error code
    : *this except this->ecInitError := anything
    : console := console + appropriate error messages
| true => 
    this->ecInitError := OK
    : this->bInit := TRUE
    : this->Servo.ControllerArrayArray := appropriate controllers
    : this->Servo.Mode := WAP_OFF
    : this->AcquisitionSensor.iFtGizmoId := GizmoInfo.iFtGizmoId
    : this->Status := anything
    : this->Data := anything
    : this->CWapStatusTag.stType := WAP_STATUS
    : this->CWapStatusTag.tnName := TN_WAP_STATUS
    : this->CWapStatusTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CWapStatusTag.szBlockSize := sizeof (this->Status)
    : this->CWapStarPosDataTag.stType := WAP_DATA
    : this->CWapStarPosDataTag.tnName := TN_WAP_DATA
    : this->WapStarPosDataTag.iGizmoId := GizmoInfo.iGizmoId
    : this->CWapStarPosDataTag.szBlockSize := sizeof (this->Data)
     : ::IpcMemory.ObjectSet := ::IpcMemory.ObjectSet U 
       NewObjectSet (::IpcMemory,GizmoInfo.iiCommandStreamId) U
       NewObjectSet (::IpcMemory,GizmoInfo.iiImageId)
    : ::PeriodicTaskSet := PeriodicTaskSet U 
      PeriodicTask ("CommandProcessor",GizmoInfo.tCommandTaskPeriod, 
        GizmoInfo.iCommandTaskPriority,GizmoInfo.tCommandTaskOffset,
        tGizmo::ProcessCommand) U
      PeriodicTask ("StatusRelayer",GizmoInfo.tStatusTaskPeriod, 
        GizmoInfo.iStatusTaskPriority,GizmoInfo.tStatusTaskOffset,
        this->ProcessStatus) U
      PeriodicTask (GizmoInfo.ServoInfo.sServoName,
        GizmoInfo.ServoInfo.TaskInfoArray[0].tTaskPeriod, 
        GizmoInfo.ServoInfo.TaskInfoArray[0].iTaskPriority, 0,
        tServo.DoServo)
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet U 
      EventDrivenTask ("DataStoreDump",CiEmDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream) U
      EventDrivenTask ("DataStoreDump",CiEmDataDumpPriority,
        DATA_DUMP_STACK_SIZE, DATA_DUMP_OPTIONS, tDataStore::SendToDataStream)
    : this->tidCommandTaskId := new unique periodic task id
    : this->tidStatusTaskId := new unique periodic task id
    : this->tidServoTaskId := new unique periodic task id
    : this->tidStarPosDataTaskId := new unique event driven task id   
    : this->tidStarImageDataTaskId := new unique event driven task id   
    : this->WapGizmoInfo := reference to GizmoInfo
]]  


[[D tWapGizmo::~tWapGizmo (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| true => 
    *this := anything
    : ::PeriodicTaskSet := PeriodicTaskSet - 
      PeriodicTask (this->tidCommandTaskId) -
      PeriodicTask (this->tidStatusTaskId) -
      PeriodicTask (this->tidServoTaskId)
    : ::EventDrivenTaskSet := ::EventDrivenTaskSet - 
      EventDrivenTask (this->tidStarPosDataTaskId) -
      EventDrivenTask (this->tidStarImageDataTaskId)
]]

[[D tWapGizmo::Restart (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == TRUE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to enable  PeriodicTask (this->tidCommandTaskId) ||
  Unable to enable  PeriodicTask (this->tidStatusTaskId) ||
  Unable to enable  PeriodicTask (this->tidServoTaskId) ||
  Unable to enable  EventDrivenTask (this->tidStarPosDataTaskId) ||
  Unable to enable  EventDrivenTask (this->tidStarImageDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) enabled
    : PeriodicTask (this->tidStatusTaskId) enabled
    : PeriodicTask (this->tidServoTaskId) enabled
    : EventDrivenTask (this->tidStarPosDataTaskId) enabled
    : EventDrivenTask (this->tidStarImageDataTaskId) enabled
    : this->bInit := TRUE
    : return := OK
]]

[[D tWapGizmo::Shutdown (void) ]
[ true => UNDEFINED
| invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning messages
    : return := OK
| Unable to disable PeriodicTask (this->tidCommandTaskId) ||
  Unable to disable PeriodicTask (this->tidStatusTaskId) ||
  Unable to disable PeriodicTask (this->tidServoTaskId) ||
  Unable to disable EventDrivenTask (this->tidStarPosDataTaskId) ||
  Unable to disable EventDrivenTask (this->tidStarImageDataTaskId) =>
    console := console + appropriate warning messages
    : return := appropriate error code
| true => 
    PeriodicTask (this->tidCommandTaskId) disabled
    : PeriodicTask (this->tidStatusTaskId) disabled
    : PeriodicTask (this->tidServoTaskId) disabled
    : EventDrivenTask (this->tidStarPosDataTaskId) disabled
    : EventDrivenTask (this->tidStarImageDataTaskId) disabled
    : this->bInit := FALSE
    : return := OK
]]


[[D tWapGizmo::IsOk(void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this.ecInitError !=OK ||
  Unable to receive commands ||
  Unable to receive from image channels => return := FALSE
| true => return := TRUE
]]

--------------------------------------------------------------------------
Star Tracker Command Specification

[[D tWapGizmo::Ping (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true =>  ErrorStream (::pThisRteG) :=  ErrorStream (::pThisRteG) + 
     appropriate message 
]]

[[D tWapGizmo::Off (void) ]
[ invalid_pointer (this) => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->Servo.Mode := WAP_OFF
    : ErrorStream (::pThisRteG) :=  ErrorStream (::pThisRteG) + 
      appropriate message 
]]
  [[D tWapGizmo::Slew (void) ]
[ invalid_pointer (this) => UNDEFINED
| this.bInit == FALSE ||
  this.ecInitError != OK => No response
| true => 
    this->Servo.Mode := WAP_OFF
    : take this->CollectorMutex
    : ErrorStream (::pThisRteG) :=  ErrorStream (::pThisRteG) + 
      appropriate message 
]]
   
   
[[D tWapGizmo::Track (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE ||
  this->ecInitError != OK => No response
| true => 
    this->Servo.Mode := WAP_TRACK
    : take this->CollectorMutex
    : ErrorStream (::pThisRteG) :=  ErrorStream (::pThisRteG) + 
      appropriate message 
]]

[[D tWapGizmo::Calibrate(const tFtCalibrateCmd& icNewCmd) ]
implements CALIBRATE command
[invalid_pointer(this) ||
 invalid_reference(icNewCmd) => UNDEFINED
| true =>
    this->Servo.Mode := WAP_CALIBRATE
    : take this->CollectorMutex
    : this->CompleteIoSet.Collector.DitherParameters := icNewCmd.Parameters
    : ErrorStream (::pThisRteG) :=  ErrorStream (::pThisRteG) + 
      appropriate message 
    : return := OK 
]]

[[D tWapGizmo::Home(void) ]
implements HOME command
[invalid_pointer(this) ||
 invalid_reference(icNewCmd) ||
 MUTEX_DEFAULT_TIMEOUT ticks elapse ||
 this->CollectorMutex taken again => UNDEFINED
| true =>
     this->Servo.Mode := WAP_OFF
   : take this->CollectorMutex
   : mirror moves to hard stops and then to home position
   : this->WapGizmo.CompleteIoSet.Collector[0].Offset := 
            -WapGizmo.WapGizmoInfo.ServoConstants.HardStop[0]
   : this->WapGizmo.CompleteIoSet.Collector[1].Offset := 
            -WapGizmo.WapGizmoInfo.ServoConstants.HardStop[1]
   : this->WapGizmo.CompleteIoSet.Collector[0].Value := 0
   : this->WapGizmo.CompleteIoSet.Collector[1].Value := 0
   : ErrorStream () :=  ErrorStream () + appropriate message 
   : return := OK 
]]

[[D tWapGizmo::SetServoParams(const tWapSetServoParametersCmd& icNewCmd) ]
implements SETSERVOPARAMETERS command
[invalid_pointer(this) ||
 invalid_reference(icNewCmd) => UNDEFINED
| true =>
      this->CompleteControllerSet.TrackController.Parameters := icNewCmd.ControllerParameters
    : this->CompleteIoSet.Collector.Transform := icNewCmd.Transform
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
]]

[[D tWapGizmo::Acquire(const tWapAcuireCmd& icNewCmd) ]
implements ACQUIRE command
[
true => THIS SPEC NOT COMPLETE
]]

[[D tWapGizmo::Move(const tWapMoveCmd& icNewCmd) ]
implements ACQUIRE command
[invalid_pointer(this) ||
 invalid_reference(icNewCmd) => UNDEFINED
| true =>
   take this->CollectorMutex
   : this->WapGizmo.CompleteIoSet.Collector[0].Value := icNewCmd.Position[0]
   : this->WapGizmo.CompleteIoSet.Collector[1].Value := icNewCmd.Position[1]
   : return := OK 
]]

--------------------------------------------
Protected Specification

[[D tWapGizmo::Dispatch (const tIsitCmd& icNewCmd) ]
[ invalid_pointer (this) ||
  invalid_reference (icNewCmd) => UNDEFINED
| icNewCmd.Command == WAP_CMD_PING => 
    ErrorStream :=  ErrorStream + appropriate message
    : return := OK 
| icNewCmd.Command == WAP_CMD_OFF => 
    this->Servo.Mode := WAP_OFF
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == WAP_CMD_TRACK => 
    this->Servo.Mode := WAP_TRACK
    : DataStream (::pThisRteG) := DataStream (::pThisRteG) + 
      this->CWapStarPosDataTag + this->Data 
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == WAP_CMD_HOME => 
    this->Servo.Mode := WAP_OFF
    : move Collector to home position
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == WAP_CMD_CALIBRATE => 
    this->Servo.Mode := WAP_CALIBRATE
    : Collector.DitherParameters := icNewCmd.Parameters
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| icNewCmd.Command == WAP_CMD_SETSP => 
      TrackController.Parameters := icNewCmd.ControllerParameters
    : Fsm.Transform := icNewCmd.Transform
    : ErrorStream :=  ErrorStream + 
      appropriate message 
    : return := OK 
| true => return := appropriate error code
]]


[[D tWapGizmo::CollectStatus (void) ]
[ invalid_pointer (this) => UNDEFINED
| IsFull (::StatusStream (::pThisRteG) ) =>
    ::ErrorStream := ::ErrorStream + 
      appropriate error code
    : return := appropriate error code
| true =>
    ::StatusStream (::pThisRteG) := ::StatusStream (::pThisRteG) + 
       this->CWapStatusTag + this->Status 
    : return := OK
]]  

[[D tWapGizmo::operator_extractor (ostream& Stream, const tWapGizmo& Gizmo) ]
[ invalid_pointer (this) ||
  invalid_reference (Stream) ||
  invalid_reference (Gizmo) => UNDEFINED
| true =>
    Stream := Stream << appropriate text and Gizmo state values
    : return := Stream << appropriate text and Gizmo state values
]]

--------------------------------------------
Private Specification

[[D tWapGizmo::ChangeServoParams(const tWapServoParams& Params) ]
change servo parameters
[invalid_pointer(this) ||
 invalid_reference(Params) => UNDEFINED
| true =>
      this->CompleteControllerSet.TrackController.Parameters := 
         Params.ControllerParameters
    : this->CompleteIoSet.Collector.Transform := Params.Transform
    : return := OK 
]]

****************************************************************************/

class tWapGizmoInfo;
class ostream;
class tIsitCmd;
class tWapSetServoParametersCommand;
class tWapGizmo;

class tWapCmdEngine : public tMutexUser {
   public:
      tWapCmdEngine(tWapGizmo *pGizmoToUse,char* sTaskName);      
};

class tWapHomeCmdEngine : public tWapCmdEngine {
   public:
      tWapHomeCmdEngine(tWapGizmo *pGizmoToUse);
      ERROR_CODE ExecuteMethod();
   private:
      tWapGizmo *pWapGizmo;
};

class tWapMoveCmdEngine : public tWapCmdEngine {
   public:
      tWapMoveCmdEngine(tWapGizmo *pGizmoToUse,const tPoint& DeltaToUse);        
      ERROR_CODE ExecuteMethod();
   private:
      tWapGizmo *pWapGizmo;
      const tPoint Delta;
};

class tWapGizmo : public tRealtimeGizmo {
  friend class tWapCmdEngine;
  friend class tWapHomeCmdEngine;
  friend class tWapMoveCmdEngine;

  public:
    tWapGizmo (const tWapGizmoInfo &GizmoInfo);
    virtual ~tWapGizmo (); 
    virtual ERROR_CODE Restart ();
    virtual ERROR_CODE Shutdown ();
    virtual BOOL IsOk ();
    friend ostream& operator<< (ostream& stream, tWapGizmo& Gizmo);
  private: // these must be constructed before using in constructors of other members 
    const tTypeTag CWapStatusTag;
    const tTypeTag CWapStarPosDataTag;
    const tTypeTag CWapCollectorPosDataTag;
    
  protected:
    virtual ERROR_CODE CollectStatus ();
    virtual ERROR_CODE Dispatch (const tIsitCmd& Command);
 
    tWapStatus Status;    

  private:
    enum {X,Y};

    ERROR_CODE Ping ();
    ERROR_CODE Off ();
    ERROR_CODE Slew ();
    ERROR_CODE Track ();
    ERROR_CODE Calibrate(const tWapCalibrateCmd& icNewCmd);
    ERROR_CODE SetServoParams(const tWapSetServoParamsCmd& icNewCmd);
    ERROR_CODE Home();
    ERROR_CODE Move(const tWapMoveCmd& icNewCmd);
    void ChangeServoParams(const tWapServoParams& Params);

    tWapCompleteIoSet CompleteIoSet; 
    tWapCompleteControllerSet CompleteControllerSet;
    const tWapGizmoInfo& WapGizmoInfo;
    tMutex CollectorMutex;
};

#endif INC_stGizmoh
