/* $RCSfile: wapGizmoInfo.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/wapGizmoInfo.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */


#ifndef INC_wapGizmoInfoh
#define INC_wapGizmoInfoh

#include "isit.h"
#include "realTimeGizmoInfo.h"
#include "servoInfo.h"
#include "ipcid.h"
#include "hardware.h"
#include "wapTypes.h"
#include "wapCommands.h"
#include "paddle.h"

/****
Black Box State Variables

int        iRteId
int        iGizmoId
int        iNumberOfServoModes
tServoInfo ServoInfo
GIZMO_TYPE gtType
int        iImageHSize 
int        iImageVSize 
int        iFtGizmoId 
tIpcIndirectionDblBufferId iiStarImageId
__________________________
Stimuli



[[D tWapGizmoInfo::tWapGizmoInfo(int iRteId, int iGizmoId, 
      const tWapPhysicalConstants PhysicalConstants,
      const tWapServoParams& ServoParams,
      int iNumberOfServoModes, const tServoInfo& ServoInfo, 
      int iFtGizmoId,int iWapGizmoId, int iImageHSize, int iImageVSize,
      BOARD_NAME bnInputBoardArray[COLLECTOR_CHANNELS],
      int iInputChannelArray[COLLECTOR_CHANNELS],
      BOARD_NAME bnOutputBoardArray[COLLECTOR_CHANNELS],
      int iOutputChannelArray[COLLECTOR_CHANNELS],
      BOARD_NAME bnPaddleBoardArray[PADDLE_CHANNELS],
      int iPaddleChannelArray[PADDLE_CHANNELS],) ]
      BOARD_NAME bnPaddleOutputBoard,
      int iPaddleOutputChannel) ]
     [ invalid_pointer (this) ||
       invalid_reference (ServoInfo) ||
       iFtGizmoId not valid gizmo id ||
       iStGizmoId not valid gizmo id ||
       invalid_pointer(bnInputBoardArray) ||
       invalid_pointer(iInputChannelArray) ||
       invalid_pointer(bnOutputBoardArray) ||
       invalid_pointer(iOutputChannelArray) => UNDEFINED
     | Number of IPC_NAMES previously generated + 3 >= maximum allowable names =>
          this->ecInitError := appropriate error code
          : *this (except this->ecInitError) := anything
     | true =>
          this->iRteId := iRteId
          : this->iGizmoId := iGizmoId
          : this->iiCommandStreamId := appropriate and unique tIpcFifoId
          : this->gtGizmoType := GT_UNKNOWN
          : this->iStatusTaskPriority := CiStatusTaskPriority
          : this->tStatusTaskPeriod := CtStatusTaskPeriod
          : this->tStatusTaskOffset := 0
          : this->iCommandTaskPriority := CiCommandTaskPriority
          : this->tCommandTaskPeriod := CiCommandTaskPeriod
          : this->tCommandTaskOffset := 0
          : this->iNumberOfServoModes := iNumberOfServoModes,
          : this->ServoInfo := ServoInfo 
          : number of IPC_NAMES generated := number of IPC_NAMES generated + 3
          : this->iiStarImageId.inObjectName := unique IPC_NAME
          : this->iiStarImageId.inStarBuffer1 := unique IPC_NAME
          : this->iiStarImageId.inStarBuffer2 := unique IPC_NAME
          : this->iiStarImageId.mtObjectMemType := 
               ( if (iRteId == iFtGizmoId Rte number) then MT_LOCAL else MT_SHARED )
          : this->iiStarImageId.itObjectType := IPC_INDIRECTION_DBL_BUFFER
          : this->iiStarImageId.szBufferSize := iImageHSize*iImageVSize
          : this->iStarImageHSize := iImageHSize
          : this->iStarImageVSize := iImageVSize
          : this->iFtGizmoId := iFtGizmoId
          : this->iStGizmoId := iStGizmoId
          : for 0 <= actuator < 3
               pInputChannelArray[actuator] := tInputChannel corresponding to
                  (bnInputBoardArray[actuator],iInputChannelArray[actuator])
             : pOutputChannelArray[actuator] := tOutputChannel corresponding to
                  (bnOutputBoardArray[actuator],iOutputChannelArray[actuator])
]]


[[D tWapGizmoInfo::~tWapGizmoInfo(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => *this := anything
]]


[[D tWapGizmoInfo::InputChannels(void) ]
returns Collector input channels
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := pInputChannelArray
]]

[[D tWapGizmoInfo::OutputChannels(void) ]
returns Collector output channels
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := pOutputChannelArray
]]


[[D tWapGizmoInfo::WapGizmoId(void) ]
returns WAP gizmo id
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := CiWapGizmoId
]]

[[D tWapGizmoInfo::GetStBufferId(void) ]
returns Star Tracker buffer id
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := StBufferId
]]

[[D tWapGizmoInfo::GetStarImageId(void) ]
returns star image ID
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->iiStarImageId
]]

[[D tWapGizmoInfo::PhysicalConstants(void) ]
returns reference to physical constants
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->PhysicalConstants_
]]

[[D tWapGizmoInfo::ServoParams(void) ]
returns reference to default servo parameters
[ invalid_pointer(this) => UNDEFINED
| true =>
   return := this->ServoParams_
]]



***********/

//N.B., SidChannels() should be declared (tOutputChannel const *const *()),
//but gcc has trouble interpreting that declaration.

class tWapGizmoInfo : public tRealtimeGizmoInfo {
  public:
    tWapGizmoInfo(int iRteId, int iGizmoId,
        const tWapPhysicalConstants& PhysicalConstants, 
        const tWapServoParams& ServoParams, int iNumberOfServoModes,
        const tServoInfo& ServoInfo, int iFtGizmoId,int iStGizmoId, 
        int iImageHSize, int iImageVSize,
        BOARD_NAME bnInputBoardArray[COLLECTOR_CHANNELS],
        int iCInputChannelArray[COLLECTOR_CHANNELS],
        BOARD_NAME bnOutputBoardArray[COLLECTOR_CHANNELS],
        int iCOutputChannelArray[COLLECTOR_CHANNELS],
        BOARD_NAME bnPaddleBoardArray[PADDLE_CHANNELS],
        int iPaddleChannelArray[PADDLE_CHANNELS],
        BOARD_NAME bnPaddleOutputBoard,
        int iPaddleOutputChannel);
    const tIpcIndirectionDblBufferId& GetStarImageId(void) const;
    const tIpcDblBufferId& GetStBufferId(void) const;
    int GetStarImageHSize(void) const;
    int GetStarImageVSize(void) const;
    tInputChannel *const *InputChannels() const 
      {return pInputChannelArray;}
    tOutputChannel *const *OutputChannels() const 
      {return pOutputChannelArray;}
    tInputChannel *const *PaddleChannels() const 
      {return pPaddleChannelArray;}
    const tOutputChannel *PaddleOutputChannel() const 
      {return pPaddleOutputChannel;}
    const tWapPhysicalConstants& PhysicalConstants() const
      {return PhysicalConstants_;}
    const tWapServoParams& ServoParams() const 
      {return ServoParams_;}
  protected:
    tIpcIndirectionDblBufferId iiStarImageId;
    tIpcDblBufferId StBufferId;
    int iStarImageHSize;
    int iStarImageVSize;
    int iFtGizmoId;
    const tWapPhysicalConstants& PhysicalConstants_;
    const tWapServoParams& ServoParams_;
    tInputChannel *pInputChannelArray[COLLECTOR_CHANNELS];   
    tOutputChannel *pOutputChannelArray[COLLECTOR_CHANNELS];   
    tInputChannel *pPaddleChannelArray[PADDLE_CHANNELS];   
    tOutputChannel *pPaddleOutputChannel;
};

#endif INC_wapGizmoInfoh

