/* $RCSfile: wapTypes.h,v $ */
/* CM Info: $Header: /home/chara/cvsroot/ople/sys/include/wapTypes.h,v 1.1.1.1 1998/06/11 21:36:15 chara Exp $
*/
/* Russ Brill - Jet Propulsion Laboratory */


#ifndef INC_wapTypesh
#define INC_wapTypesh


#include "isit.h"
#include "tVector.h"
#include "stTypes.h" //for tStarPosition

const int COLLECTOR_CHANNELS = 2;

typedef enum WAP_COMMAND_STATUS {
  WAPCS_OFF,WAPCS_SLEW_ACTIVE, WAPCS_TRACK_ACTIVE, WAPCS_CALIBRATE_ACTIVE,
  } WAP_COMMAND_STATUS;

typedef enum WAP_SERVO_STATE {
  WSS_OFF,WSS_SLEW, WSS_TRACK,WSS_CALIBRATE,WSS_NUM_SERVO_MODES
} WAP_SERVO_STATE;

typedef enum WAP_PARTITION {
   WP_BASIC,WP_NUM_PARTITIONS
} WAP_PARTITION;

/*****
[[D tWapStatus::tWapStatus (int iGizmoId) ]
[ invalid_pointer (this) => UNDEFINED
| true => 
    this->iGizmoId := iGizmoId
]]

[[D tWapStatus::UpdateCommandStatus (WAP_COMMAND_STATUS scsNewCmdStatus) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->scsCommandStatus := scsNewCmdStatus
]]

[[D tWapStatus::UpdateServoState (WAP_SERVO_STATE sssServoState) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->sssServoState := sssServoState
]]

[[D tWapStatus::UpdateStarPosition (const tStarPosition& StarPosition) ]
[ invalid_pointer (this) => UNDEFINED
| true => this->StarPosition := StarPosition
]]


[[D tStarPosition::tStarPosition(void) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    this->dXPosition := 0.0 
    : this->dYPosition := 0.0 
]]

[[D tStarPosition::tStarPosition_dd(double dX, double dY) ]
[ invalid_pointer(this) => UNDEFINED
| true => 
    this->dXPosition := dX
    : this->dYPosition := dY
]]

****/

class tWapStatus_Version_1 {
  public:
    int iVersion;
    int iGizmoId;
    TIME tTime;
    WAP_COMMAND_STATUS  scsCommandStatus;
    WAP_SERVO_STATE     sssServoState;
    tStarPosition StarPosition;
};

class tWapData_Version_1 {
  public:
    int iVersion;
    int iGizmoId;
    TIME tTime;
    tStarPosition StarPosition;
};

#define CURRENT_WAP_STATUS_VERSION 1
#define CURRENT_WAP_DATA_VERSION   1

#define tWapStatusCurrentVersion \
   EXPAND_AND_CONCAT(tWapStatus_Version_, CURRENT_WAP_STATUS_VERSION)
#define tWapDataCurrentVersion \
   EXPAND_AND_CONCAT(tWapData_Version_, CURRENT_WAP_DATA_VERSION)

class tWapStatusUnix : public tWapStatus_Version_1 {
  public:
    tWapStatusUnix () {
      iVersion = CURRENT_WAP_STATUS_VERSION;
    }
    tWapStatusUnix (int iGizmoId) {
      iVersion = CURRENT_WAP_STATUS_VERSION;
      this->iGizmoId = iGizmoId;
    }
};

class tWapDataUnix: public tWapData_Version_1 {
  public:
    tWapDataUnix () {
      iVersion = CURRENT_WAP_DATA_VERSION;
     }
    tWapDataUnix (int iGizmoId) {
      iVersion = CURRENT_WAP_DATA_VERSION;
      this->iGizmoId = iGizmoId;
     }
};

class tWapStatus: public tWapStatusUnix {
  public:
    tWapStatus () : tWapStatusUnix() {
      this->scsCommandStatus =WAPCS_SLEW_ACTIVE;
      this->sssServoState = WSS_SLEW;
    }
    tWapStatus (int iGizmoId) : tWapStatusUnix (iGizmoId) {
      this->scsCommandStatus = WAPCS_SLEW_ACTIVE;
      this->sssServoState = WSS_SLEW;
    }
    void UpdateCommandStatus (WAP_COMMAND_STATUS scsNewCmdStatus) {
       this->scsCommandStatus = scsNewCmdStatus; }
    void UpdateServoState (WAP_SERVO_STATE sssServoState) {
       this->sssServoState = sssServoState; }
    void UpdateStarPosition (const tStarPosition& StarPosition) {
       this->StarPosition = StarPosition; }
 
};

class tWapData : public tWapDataUnix {
  public:
    tWapData () : tWapDataUnix () {};
    tWapData (int iGizmoId) : tWapDataUnix (iGizmoId) {}
};

struct tWapPhysicalConstants {
      double dHardStopDuration;       //duration of hard stop move (sec)
      double dHomeDuration;           //duration of home move (sec)
      double dHardStopPosition[2];    //distance to hard stop (pixels)
      double dMoveDuration;           //duration of move command (sec)
      double dPaddleScaleFactor;      //converts to pixels/sec
      double dInputScaleFactor;       //converts to steps
      double dOutputScaleFactor;      //converts from steps/sec
      double dPaddleSpeeds[3];        //speed for each button (pixels/sec)
};

#endif INC_wapTypesh



