/* $RCSfile: commandHandler.cc,v $ */

/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/inst/cmdhdlr/commandHandler.cc,v 1.1.1.1 1998/06/11 21:36:36 chara Exp $
*/

/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,20Aug97,rlj      updated for new instrument gizmo
01a,16Oct96,eamck    written 
*/
      
/*
DESCRIPTION
  ISIT Command Stream code
*/

#include "isit.h"
#include "ipcError.h"
#include "configMgr.h"
#include "commandHandler.h"
#include "errorStream.h"
#include "commandStream.h"




/*****
[[U tCommandHandler::tCommandHandler (void) ]]
*****/
tCommandHandler::tCommandHandler () :

  /****
  [[U tBaseObject::tBaseObject (void) ]]
  ****/
  tBaseObject()  {
  
  TraceMark(S_cmdhdlr_ENTERING_CMDHDLR_CONSTRUCTOR);

  int iGizmoId;
  
  const tGizmoInfo *pGizmoInfo;

  tIpcFifoId iiId;
  
  /****
  [[U this->iNumberOfCommandStreams := ConfigManager.tConfig::NumberOfSystemGizmos (void) ]]
  ****/
  this->iNumberOfCommandStreams = ConfigManager.NumberOfSystemGizmos(); 
 
  this->pCommandStreamArray = new (tCommandStreamSender *) [this->iNumberOfCommandStreams];

  if (this->pCommandStreamArray == NULL) {

      TraceMsg (tmt_ERROR,S_cmdhdlr_CANT_ALLOCATE_COMMAND_STREAMS,
                "tCommandHandler::tCommandHandler()");
                
      tBaseObject::SetInitError(S_cmdhdlr_CANT_ALLOCATE_COMMAND_STREAMS);
      return;
    }    
  
  for (iGizmoId = 0; iGizmoId < this->iNumberOfCommandStreams; iGizmoId++) {
   
    /****
    [[U pGizmoInfo := ConfigManager.tConfig::GetGizmoInfo_i (iGizmoId) ]]
    ****/
    pGizmoInfo = ConfigManager.GetGizmoInfo(iGizmoId);

    /****
    [[U iiId := pGizmoInfo->tGizmoInfo::GetCommandStreamId (void) ]]
    ****/
    iiId = pGizmoInfo->GetCommandStreamId();


    /****
    [[U tCommandStreamSender::tCommandStreamSender(iiId)]]
    ****/
    this->pCommandStreamArray[iGizmoId] = new tCommandStreamSender (iiId);
      
    if (this->pCommandStreamArray[iGizmoId] == NULL ) {
      TraceMsg (tmt_ERROR,S_cmdhdlr_CANT_ALLOCATE_COMMAND_STREAMS,
                "tCommandHandler::tCommandHandler()");
                
      tBaseObject::SetInitError(S_cmdhdlr_CANT_ALLOCATE_COMMAND_STREAMS);
      return;
    } // if
  } // for
  
  TraceMark(S_cmdHdlr_LEAVING_CMDHDLR_INIT);

}

/*****
[[U tCommandHandler::~tCommandHandler (void) ]]
*****/
tCommandHandler::~tCommandHandler () {

  int i;
  
  TraceMark(S_cmdHdlr_ENTERING_CMDHDLR_DESTRUCTOR);

  for (i = 0; i < this->iNumberOfCommandStreams; i++) {
  
      /****
      [[U this->pCommandStreamArray[i]->tCommandStreamSender::~tCommandStreamSender()]]
      ****/
    delete this->pCommandStreamArray[i];
    
  }  

  delete[] this->pCommandStreamArray;
    
  TraceMark(S_cmdHdlr_LEAVING_CMDHDLR_DESTRUCTOR);

}

ERROR_CODE tCommandHandler::Restart () {
 
 TraceMark (S_cmdHdlr_RESTART);

 if (tBaseObject::IsInit () == TRUE) {
     TraceMsg (tmt_WARN,S_cmdhdlr_ALREADY_INITIALIZED,"tCommandHandler::Restart");
     return OK;
  }
  
  tBaseObject::SetInit (TRUE);

  
  return OK;

}

ERROR_CODE tCommandHandler::Shutdown () {
 
  TraceMark (S_cmdHdlr_SHUTDOWN);

  if (tBaseObject::IsInit () == FALSE ) {
     TraceMsg (tmt_WARN,S_cmdhdlr_ALREADY_SHUTDOWN,"tCommandHandler::Shutdown");
     return OK;
  }
  
  tBaseObject::SetInit (FALSE);  
  
  return OK;

}

BOOL tCommandHandler::IsOk () {
  
  int i;

  TraceMark (S_cmdHdlr_ISOK);
  
  for (i = 0; i < this-> iNumberOfCommandStreams; i++ ) {

      /****
      [[U this->pCommandStreamArray[i]->tBaseObject::IsOk (void) ]]
      ****/  
    if (this->pCommandStreamArray[i]->IsOk () == FALSE) return FALSE;
  } //for
  
    
  /****
  [[U return := this->tBaseObject::IsOk (void) ]]
  ****/  

  return tBaseObject::IsOk ();
}
  

/***** 
[[U tCommandHandler::Put (Command) ]] 
*****/
ERROR_CODE tCommandHandler::Put (const ISIT_CMD& Command) {

  int iGizmoId = Command.genericCmd.iGizmoId;

  TraceMark(S_cmdHdlr_PUT);

  if ( iGizmoId < 0 || iGizmoId >= this->iNumberOfCommandStreams) 
    return Traceback (S_cmdhdlr_INVALID_GIZMO_ID,"tCommandHandler::Put");

  /****
  [[U return := this->pCommandStreamArray[iGizmoId]->
                     tCommandStreamSender::Put (Command) ]]
  ****/      
  return this->pCommandStreamArray[iGizmoId]->Put (Command);

}

