/* $RCSfile: connection.cc,v $ - External Interface routines -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/inst/extint/connection.cc,v 1.3 1998/09/23 23:57:33 phil Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,14Aug97,ky       Initial creation
*/

/*
DESCRIPTION
  External Interface code 
*/

#include "commandHandler.h"
#include "connection.h"
#include "connectionP.h"
#include "typeTag.h"


/****
[[U tConnection::~tConnection() ]]
****/
tConnection::~tConnection()
{
   if (taskIdVerify(this->iCommandLoopTaskId) != ERROR)
      taskDelete(this->iCommandLoopTaskId);
}

/****
[[U tConnection::tConnection(sockfd, CommandHdlrToUse, siCommandSendMutex) ]]
****/
tConnection::tConnection(int sockfd, tCommandHandler& CommandHdlrToUse,
                         SEM_ID siCommandSendMutex) :

  CommandHandler (CommandHdlrToUse)
{

    this->bDisconnected = FALSE;
    this->iSocketInUse = sockfd;
    this->siCommandSendMutex = siCommandSendMutex;
    this->iCommandLoopTaskId = taskSpawn("CommandLoop", 
           CONNECTION_WORK_PRIORITY, (VX_FP_TASK), CONNECTION_STACK_SIZE, 
           (FUNCPTR) &(this->CommandLoop), (int)this,0,0,0,0,0,0,0,0,0);
    if (this->iCommandLoopTaskId == ERROR)
    {
       this->bDisconnected = TRUE;
       close(this->iSocketInUse);
    }
    else
       signal(SIGPIPE,SIG_IGN);

}

/****
[[U tConnection::CommandLoop() ]]
****/
void tConnection::CommandLoop(void)
{
    int n;
    ERROR_CODE rc;
    ISIT_CMD isitCmd;
    
    FOREVER 
    {
       /****
       [[U tConnection::readn(iSocketInUse,&isitCmd,sizeof(ISIT_CMD)) ]]
       ****/

       if ( (n = readn(this->iSocketInUse, (char *)&isitCmd, 
                       sizeof(ISIT_CMD))) < 0)
       { 
             this->bDisconnected = TRUE;
             close(this->iSocketInUse);
             break;
       }
       else if ( (n == 0) || (n != sizeof(ISIT_CMD)) )
       {
             this->bDisconnected = TRUE;
             close(this->iSocketInUse);
             break;
       }
       else if (n == sizeof(ISIT_CMD))
       {
          if ( (isitCmd.genericCmd.Command == DISCONNECT) ||
               (isitCmd.genericCmd.Command == RESET) )
          {

             this->bDisconnected = TRUE;
             close(this->iSocketInUse);
             break;
          }
          else
          {
             semTake(this->siCommandSendMutex, WAIT_FOREVER);
             rc = this->CommandHandler.Put(isitCmd);
             semGive(this->siCommandSendMutex);
          }
       }
       taskDelay(1);
    }
    exit(0);
}


/****
[[U tConnection::SendError(Tag, pError) ]]
****/
ERROR_CODE tConnection::SendError(const tTypeTag& Tag, const BYTE *pError)
{
    int n;
   
    /****
    [[U tConnection::writen(iSocketInUse,&Tag,sizeof(Tag)) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)&Tag, sizeof(Tag))) <= 0)
       return(errno); 

    /****
    [[U tConnection::writen(iSocketInUse,pError,Tag.Size()) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)pError, Tag.Size())) <= 0)
       return(errno); 
    return( OK );
}

/****
[[U tConnection::SendData(Tag, pData) ]]
****/
ERROR_CODE tConnection::SendData(const tTypeTag& Tag, const BYTE *pData)
{
    int n;
   
    /****
    [[U tConnection::writen(iSocketInUse,&Tag,sizeof(Tag)) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)&Tag, sizeof(Tag))) <= 0)
       return(errno); 

    /****
    [[U tConnection::writen(iSocketInUse,pData,Tag.Size()) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)pData, Tag.Size()) ) <= 0) 
       return(errno); 
    
    return ( OK );
}

/****
[[U tConnection::SendStatus(Tag, pStatus) ]]
****/
ERROR_CODE tConnection::SendStatus(const tTypeTag& Tag, const BYTE *pStatus)
{
    int n;
   
    /****
    [[U tConnection::writen(iSocketInUse,&Tag,sizeof(Tag)) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)&Tag, sizeof(Tag))) <= 0)
       return(errno); 

    /****
    [[U tConnection::writen(iSocketInUse,pStatus,Tag.Size()) ]]
    ****/

    if ( (n = writen(this->iSocketInUse, (char *)pStatus, Tag.Size()) ) <= 0)
       return(errno); 
    
    return ( OK );
}

/****
[[U tConnection::readn(sd, ptr, size) ]]
****/
int tConnection::readn(int sd,char *ptr,int size)
{    
   int no_left,no_read;

   no_left = size;
   while (no_left > 0) 
   { 
      no_read = read(sd,ptr,no_left);
      if ( (no_read == -1) && (errno == EINTR) ) 
         continue;
      if(no_read <0)  
         return(no_read);
      if (no_read == 0) 
         break;
      no_left -= no_read;
      ptr += no_read;
   }
   return(size - no_left);
}

/****
[[U tConnection::writen(sd, ptr, size) ]]
****/
int tConnection::writen(int sd,char *ptr,int size)
{         
   STATUS rc;
   int no_left,no_written;

   no_left = size;
   while (no_left > 0) 
   {
      no_written = write(sd,ptr,no_left);
      if(no_written <=0)
      {

         this->bDisconnected = TRUE;
         close(sd);
         if (( rc = taskIdVerify(this->iCommandLoopTaskId)) != ERROR)
            rc = taskDelete(this->iCommandLoopTaskId);

         return(no_written);
      }
      no_left -= no_written;
      ptr += no_written;
   }
   return(size - no_left);
}

/****
[[U tConnection::Restart(void) ]]
****/
ERROR_CODE tConnection::Restart(void)
{
   STATUS rc;

   rc = taskResume(this->iCommandLoopTaskId);
   if (rc == ERROR)
   {
      return (errno);
   }
   else
      return (OK);

}

/****
[[U tConnection::Shutdown(void) ]]
****/
ERROR_CODE tConnection::Shutdown(void)
{
   STATUS rc;

   rc = taskSuspend(this->iCommandLoopTaskId);
   if (rc == ERROR)
   {
      return (errno);
   }
   else
      return (OK);


}

