/* $RCSfile: extInt.cc,v $ - External Interface routines -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/inst/extint/extInt.cc,v 1.1.1.1 1998/06/11 21:36:36 chara Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,14Aug97,ky       Initial creation
*/

/*
DESCRIPTION
  External Interface code 
*/

#include "commandHandler.h"
#include "connection.h"
#include "extInt.h"
#include "extIntP.h"

/****
[[U tExternalInterface::~tExternalInterface() ]]
****/
tExternalInterface::~tExternalInterface()
{
   // Delete All tasks associated with External Interface
   if (taskIdVerify(this->iAcceptConnTaskId) != ERROR)
      taskDelete(this->iAcceptConnTaskId);

   /****
   [[U lClientSet.tList::Free(void)]]
   ****/

   lClientSet.Free();
}

/****
[[U tExternalInterface::tExternalInterface(CommandHdlrToUse) ]]
****/
tExternalInterface::tExternalInterface(tCommandHandler& CommandHdlrToUse) :

  /****
  [[U tBaseObject::tBaseObject () ]]
  ****/
  tBaseObject (),
  CommandHdlr (CommandHdlrToUse)

{

   ERROR_CODE ecTemp;

   SetInitError(OK);

   SetInit(TRUE);

   this->siCommandSendMutex = semBCreate(SEM_Q_FIFO, SEM_FULL);
   if ( this->siCommandSendMutex == NULL)
   {

      TraceMsg (tmt_ERROR,errno,"tExternalInterface::tExternalInterface()");

      SetInitError(errno);

      SetInit(FALSE);


   }
   /****
   [[U tExternalInterface::EstablishConnection() ]]
   ****/
   else if ( (ecTemp = this->EstablishConnection() ) != OK)
   {

      TraceMsg (tmt_ERROR,ecTemp,"tExternalInterface::tExternalInterface()");

      SetInitError(ecTemp);

      SetInit(FALSE);

   }
   else
   {
      /****
      [[U tExternalInterface::WaitForConnection() ]]
      ****/
      this->iAcceptConnTaskId = taskSpawn("WaitForConnection",
             SERVER_WORK_PRIORITY, (VX_FP_TASK),
             SERVER_STACK_SIZE, (FUNCPTR) &(this->WaitForConnection),
             (int)this,0,0,0,0,0,0,0,0,0);
      if (this->iAcceptConnTaskId == ERROR)
      {

         TraceMsg (tmt_ERROR,errno,"tExternalInterface::tExternalInterface()");

         SetInitError(errno);

         SetInit(FALSE);

      }
   }
}

/****
[[U tExternalInterface::EstablishConnection() ]]
****/
ERROR_CODE tExternalInterface::EstablishConnection(void)
{
   int    sockfd;
   struct sockaddr_in serv_addr;
   int port;
   int buff_size;
   BOOL bGotAPort;

   port = SERVER_PORT_NUM;
   bGotAPort = FALSE;
   buff_size = 16384; 

   while (bGotAPort == FALSE)
   {
      /* create a TCP-based socket */

      sockfd = socket(AF_INET, SOCK_STREAM, 0);
      if (sockfd  == ERROR)
      {
         TraceMsg (tmt_ERROR,errno,"tExternalInterface::EstablishConnection()");

         return(errno);
      }
   

      memset((char *) &serv_addr, 0, sizeof(serv_addr));
      serv_addr.sin_family = AF_INET;
      serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
      serv_addr.sin_port = htons(port);
   
      /* bind socket to local address */
      if (bind(sockfd,(struct sockaddr *)&serv_addr,sizeof(serv_addr)) == ERROR)
      {
         // Try the next available port number
         if (errno == EADDRINUSE)
         {
            close(sockfd);
            if (port < MAXIMUM_PORT_NUM)
            {
               port++;
            }
            else
               port = SERVER_PORT_NUM;
            
         }
         else
            return(errno);
         
      }
      else
         bGotAPort = TRUE;

      taskDelay(1);
   } 
   /* create queue for client connection requests */

   if ( listen(sockfd, SERVER_MAX_CONNECTIONS)  == ERROR)
   {
      TraceMsg (tmt_ERROR,errno,"tExternalInterface::EstablishConnection()");
      close(sockfd);
      return(errno);
   }

   this->iPortUsed = port;
   this->iSockFd = sockfd;

   /* set socket option send and receive buffer size to 16384 */
   if (setsockopt(this->iSockFd, SOL_SOCKET, SO_SNDBUF, 
                     (char *) &buff_size, sizeof(buff_size)) == ERROR)
         TraceMsg (tmt_ERROR,errno,"tExternalInterface::EstablishConnection()");

   if (setsockopt(this->iSockFd, SOL_SOCKET, SO_RCVBUF, 
                     (char *) &buff_size, sizeof(buff_size)) == ERROR)
         TraceMsg (tmt_ERROR,errno,"tExternalInterface::EstablishConnection()");

   return(OK);
}

/****
[[U tExternalInterface::WaitForConnection() ]]
****/
ERROR_CODE tExternalInterface::WaitForConnection(void)
{
   int    clilen, newsockfd;
   struct sockaddr_in cli_addr;
   tConnection *pt=NULL;
   BOOL bConnectionAccepted;

   FOREVER
   {
      bConnectionAccepted = FALSE;
      while(bConnectionAccepted  == FALSE)
      {
         clilen = sizeof(cli_addr);
         newsockfd = accept(iSockFd, (struct sockaddr *) &cli_addr, &clilen);

         if (newsockfd == ERROR)
         {
            if (errno != EINTR)
            {
               close(iSockFd);
               return(errno);
            }
         }
         else
            bConnectionAccepted = TRUE;

      } 

      /****
      [[U tConnection::tConnection(newsockfd,CommandHdlr,siCommandSendMutex) ]]
      ****/

      pt = new tConnection(newsockfd, CommandHdlr, siCommandSendMutex);
      if (pt)
      {
         /****
         [[U listHead := lClientSet.tList::Add(pt)]]
         ****/

         lClientSet.Add(pt);
      }

      taskDelay(1);

   }

}

/****
[[U tExternalInterface::Restart(void) ]]
****/
ERROR_CODE tExternalInterface::Restart(void)
{
   STATUS rc;
   ERROR_CODE ecTemp;
   tConnection *listHead, *listNext=0;

   if (this->IsInit() == TRUE)
   {
      TraceMsg (tmt_WARN,S_extInt_ALREADY_RESTART,"tExternalInterface::Restart()");

   }
   else
   {
      if ((rc = taskResume(this->iAcceptConnTaskId)) == OK)
      {
         
         this->SetInit(TRUE);
         /****
         [[U listHead := lClientSet.tList::First(void)]]
         ****/

         for (listHead = (tConnection *)lClientSet.First(); listHead; listHead = listNext)
         {
            /****
            [[U listNext := lClientSet.tList::Next(listHead)]]
            ****/

            listNext = (tConnection *)lClientSet.Next(listHead);
            if ((ecTemp = listHead->Restart()) != OK)
            {
               this->SetInit(FALSE);

               this->SetInitError(ecTemp);

               return(ecTemp);
            }
            
         }
      }
      else
         return(errno);
   }
   return(OK);
}

/****
[[U tExternalInterface::Shutdown() ]]
****/
ERROR_CODE tExternalInterface::Shutdown(void)
{
   STATUS rc;
   ERROR_CODE ecTemp;
   tConnection *listHead, *listNext=0;

   /****
   [[U this->tBaseObject::IsInit(void)]]
   ****/

   if (this->IsInit() == FALSE)
   {
      TraceMsg (tmt_WARN,S_extInt_ALREADY_SHUTDOWN,"tExternalInterface::Shutdown()");
   }
   else
   {
      if ((rc = taskSuspend(this->iAcceptConnTaskId)) == OK)
      {

         this->SetInit(FALSE);


         /****
         [[U listHead := lClientSet.tList::First(void)]]
         ****/

         for (listHead = (tConnection *)lClientSet.First(); listHead; listHead = listNext)
         {
            /****
            [[U listNext := lClientSet.tList::Next(listHead)]]
            ****/

            listNext = (tConnection *)lClientSet.Next(listHead);

            if ((ecTemp = listHead->Shutdown()) != OK)
            {
               this->SetInit(TRUE);

               this->SetInitError(rc);

               return(ecTemp);
            }

         }
      }
      else
         return(errno);
   }
   return(OK);
}

/****
[[U tExternalInterface::SendError(Tag, pError) ]]
****/
ERROR_CODE tExternalInterface::SendError(const tTypeTag& Tag, 
                                         const BYTE *pError)
{
   ERROR_CODE ecTemp;
   tConnection *listHead, *listNext=0;

   if (this->IsInit() == FALSE)
      return(ERROR);

   /****
   [[U listHead := lClientSet.tList::First(void)]]
   ****/

   for (listHead = (tConnection *)lClientSet.First(); listHead; listHead = listNext)
   {
      /****
      [[U listNext := lClientSet.tList::Next(listHead)]]
      ****/

      listNext = (tConnection *)lClientSet.Next(listHead);
      // connection is disconnected, delete the node.  Otherwise
      // SendError to that connection
      
      /****
      [[U listHead->tConnection::getConnectionFlag(void)]]
      ****/

      if (listHead->getConnectionFlag() == TRUE)
      {
         /****
         [[U lClientSet.tList::Delete(listHead)]]
         ****/

         lClientSet.Delete(listHead);
      }
      else
        /****
        [[U listHead->tConnection::SendError(Tag, pError)]]
        ****/

         if ( (ecTemp = listHead->SendError(Tag, pError)) != OK)
            return(ecTemp);

   }
   return(OK);
}

/****
[[U tExternalInterface::SendData(Tag, pData) ]]
****/
ERROR_CODE tExternalInterface::SendData(const tTypeTag& Tag,
                                         const BYTE *pData)
{
   ERROR_CODE ecTemp;
   tConnection *listHead, *listNext=0;

   if (this->IsInit() == FALSE)
      return(ERROR);

   /****
   [[U listHead := lClientSet.tList::First(void)]]
   ****/

   for (listHead = (tConnection *)lClientSet.First(); listHead; listHead = listNext)
   {
      /****
      [[U listNext := lClientSet.tList::Next(listHead)]]
      ****/

      listNext = (tConnection *)lClientSet.Next(listHead);
      // connection is disconnected, delete the node.  Otherwise
      // SendData to that connection

      /****
      [[U listHead->tConnection::getConnectionFlag(void)]]
      ****/

      if (listHead->getConnectionFlag() == TRUE)
      {
         /****
         [[U lClientSet.tList::Delete(listHead)]]
         ****/

         lClientSet.Delete(listHead);
      }
      else
        /****
        [[U listHead->tConnection::SendData(Tag, pData)]]
        ****/

         if ( (ecTemp = listHead->SendData(Tag, pData)) != OK)
            return(ecTemp);

   }
   return(OK);
}

/****
[[U tExternalInterface::SendStatus(Tag, pStatus) ]]
****/
ERROR_CODE tExternalInterface::SendStatus(const tTypeTag& Tag,
                                         const BYTE *pStatus)
{
   ERROR_CODE ecTemp;
   tConnection *listHead, *listNext=0;

   if (this->IsInit() == FALSE)
      return(ERROR);

   /****
   [[U listHead := lClientSet.tList::First(void)]]
   ****/

   for (listHead = (tConnection *)lClientSet.First(); listHead; listHead = listNext)
   {
      /****
      [[U listNext := lClientSet.tList::Next(listHead)]]
      ****/

      listNext = (tConnection *)lClientSet.Next(listHead);
      // connection is disconnected, delete the node.  Otherwise
      // SendStatus to that connection

      /****
      [[U listHead->tConnection::getConnectionFlag(void)]]
      ****/

      if (listHead->getConnectionFlag() == TRUE)
      {
         /****
         [[U lClientSet.tList::Delete(listHead)]]
         ****/

         lClientSet.Delete(listHead);
      }
      else
        /****
        [[U listHead->tConnection::SendStatus(Tag, pStatus)]]
        ****/

         if ( (ecTemp  = listHead->SendStatus(Tag, pStatus)) != OK)
            return(ecTemp);

   }
   return(OK);
}

/****
[[U tExternalInterface::IsOk(void) ]]
****/
BOOL tExternalInterface::IsOk(void)
{
   return (taskIdVerify(this->iAcceptConnTaskId) == OK);
}

