/* $RCSfile: connection.h,v $ - External Interface routines -  */
/* CM Info:
$Header: /home/chara/cvsroot/ople/sys/inst/include/connection.h,v 1.3 1998/09/23 23:57:34 phil Exp $
*/
/* Kevin Young - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,14Aug97,ky       Initial creation
*/

/*
DESCRIPTION
  External Interface header 
*/

#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include "semLib.h"
#include "semSmLib.h"
#include "smNameLib.h"

#include "ipcMultiTypeFifoAccessor.h"
#include "tList.h"

#ifndef INC_CONNECTIONh
#define INC_CONNECTIONh

typedef void (*sighnd) (...);
#define CONNECTION_WORK_PRIORITY 90
#define CONNECTION_STACK_SIZE 32768 


/****************************************************************************

[[D tConnection::tConnection  (int sockfd, tCmdHdlr& CommandHdlrToUse,
                SEM_ID siCommandSendMutex) ]
[ invalid_pointer (this) ||
  invalid_reference (CommandHandler) ||
  invalid sockfd => UNDEFINED
| Unable to spawn task iCommandLoopTaskId successfully =>
    this->bDisconnected := TRUE;
    : close socket this->iSocketInUse
| true =>
    this->bDisconnected := FALSE;
    : this->iSocketInUse := sockfd;
    : this->CommandHandler := CommandHdlrToUse;
    : this->siCommandSendMutex := siCommandSendMutex;
    : this->iCommandLoopTaskId := appropriate spawned task id
    : set up signal for broken pipe
]]

[[D tConnection::~tConnection  (void) ]
[ invalid_pointer (this) => UNDEFINED
| Task id iCommandLoopTaskId does not exist =>
  *this := anything
| true =>
  *this := anything
  : Command Loop Task deleted
]]

[[D tConnection::SendError(const tTypeTag& Tag, const BYTE *pError) ]
[ invalid_pointer (this) ||
  invalid_reference(Tag) ||
  invalid_pointer (pError) ||
  invalid iSocketInUse => UNDEFINED
| number of bytes return by write() on Tag <= 0  =>
    return := number of bytes written
| true =>
    OS perform write() on pError 
    : return number of bytes written
]]

[[D tConnection::SendData(const tTypeTag& Tag, const BYTE *pData) ]
[ invalid_pointer (this) ||
  invalid_reference(Tag) ||
  invalid_pointer (pData) ||
  invalid iSocketInUse => UNDEFINED
| number of bytes return by write() on Tag <= 0  =>
    return := number of bytes written 
| true =>
    OS perform write() on pData 
    : return number of bytes written
]]

[[D tConnection::SendStatus(const tTypeTag& Tag, const BYTE *pStatus) ]
[ invalid_pointer (this) ||
  invalid_reference(Tag) ||
  invalid_pointer (pStatus) ||
  invalid iSocketInUse => UNDEFINED
| number of bytes return by write() on Tag <= 0  =>
    return := number of bytes written 
| true =>
    OS perform write() on pStatus 
    : return number of bytes written
]]

[[D tConnection::Restart(void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to resume connection => return := appropriate error code 
| true => return := OK
]]

[[D tConnection::Shutdown(void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to suspend connection => return := appropriate error code 
| true => return := OK
]]

[[D tConnection::getTaskId(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := iCommandLoopTaskId
]]

[[D tConnection::getSocket(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := iSocketInUse 
]]

[[D tConnection::getConnectionFlag(void) ]
[ invalid_pointer (this) => UNDEFINED
| true => return := bDisconnected
]]

****************************************************************************/

class tConnection : public tNode {
  public:
    tConnection(int sock,
                tCommandHandler& CommandHdlrToUse,
                SEM_ID siCommandSendMutex);
    ~tConnection();
    ERROR_CODE SendError(const tTypeTag& Tag, const BYTE *pError);
    ERROR_CODE SendData(const tTypeTag& Tag, const BYTE *pData);
    ERROR_CODE SendStatus(const tTypeTag& Tag, const BYTE *pStatus);
    ERROR_CODE Restart(void);
    ERROR_CODE Shutdown(void);
    int getTaskId(void) const { return iCommandLoopTaskId; }
    int getSocket(void) const { return iSocketInUse; }
    BOOL getConnectionFlag(void) const { return bDisconnected; }
  protected:
    void CommandLoop(void);
    int readn(int sd,char *ptr,int size);
    int writen(int sd,char *ptr,int size);
    void TerminateConnection(int signo);
    BOOL bDisconnected;
    int iCommandLoopTaskId;
    int iSocketInUse;
    tCommandHandler &CommandHandler;
    SEM_ID siCommandSendMutex;
};

#endif INC_CONNECTIONh

