/* $RCSfile: configInfo.cc,v $ - tConfigInfo method definitions */

/*
$Header: /home/chara/cvsroot/ople/sys/inst/initMgr/configInfo.cc,v 1.1.1.1 1998/06/11 21:36:36 chara Exp $
*/

/* Phil Irwin - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,26Aug97,pci    written

*/

#include <iostream.h>
#include "isit.h"
#include "configInfo.h"


/****************************************************************************
*
* tConfigInfo
* -----------
* Black Box/State Box Mapping = unity
*
****************************************************************************/


/****************************************************************************
*
* Global Variables
*/
istream isNullInputStream(NULL);



/****************************************************************************
*
*  main::TruncateAndCopy(const char *, int)
*
*  [[U main::TruncateAndCopy()]]
*
*/
char* TruncateAndCopy(const char *s, int len)
{
  char* sNew;

  if(!s) 
    return NULL;
  else {
    sNew = new char[len + 1];
    if(strlen(s) > len)
      strncpy(sNew, s, len);
    else
      strcpy(sNew, s);
  }

  return sNew;
}


/****************************************************************************
*
*  tConfigInfo::tConfigInfo(void)
*
[[U tConfigInfo::tConfigInfo()]]
*
*/

tConfigInfo::tConfigInfo(void) : 

  /****
  [[U tBaseObject::tBaseObject()]]
  ****/
{
  this->sInfoFilename = NULL;
  this->sWhichSection = NULL;
}


/****************************************************************************
*
*  tConfigInfo::tConfigInfo(const char* sFilename)
*
* DESCRIPTION: Constructor.  This constructor sets the filename string and 
*  reads from the file into the info stream.  It assumes the user
*  wants to read the entire file; the section string is set accordingly.
*  The object will be ready to use after this constructor executes.
*
[[U tConfigInfo::tConfigInfo_s()]]
*
*/

tConfigInfo::tConfigInfo(const char* sFilename) :

  /****
  [[U tBaseObject::tBaseObject()]]
  ****/
{
  /****
  [[U this->ecInitError = tConfigInfo::SetFilename(sFilename)]]
  ****/
  this->ecInitError = this->SetFilename(sFilename);
  if(this->ecInitError != OK) {
    TraceMsg(TMT_ERR, S_configInfo_SET_FILENAME_FAILED,
                      "Filename, %s, not set", sFilename);
    return;
  }

  this->sWhichSection = NULL;

  /****
  [[U tBaseObject::ecInitError = tConfigInfo::ReadInfoFromFile()]]
  ****/
  this->ecInitError = this->ReadInfoFromFile();
  if(this->ecInitError != OK) {
    TraceMsg(TMT_ERR, S_configInfo_READ_INFO_FAILED,
                      "ReadInfo failed for file, %s", sFilename);
    return;
  }
}


/****************************************************************************
*
*  tConfigInfo::tConfigInfo(const char* sFilename, const char* sSection)
*
* DESCRIPTION: Constructor.  This constructor sets the filename and section
*  strings and reads information into the info stream.  The object will be
*  ready to use after this constructor executes.
*
[[U tConfigInfo::tConfigInfo_ss()]]
*
*/

tConfigInfo::tConfigInfo(const char* sFilename, const char* sSection) :

  /****
  [[U tBaseObject::tBaseObject()]]
  ****/
{
  /****
  [[U this->ecInitError = tConfigInfo::SetFilename(sFilename)]]
  ****/
  this->ecInitError = this->SetFilename(sFilename);
  if(this->ecInitError != OK) {
    TraceMsg(TMT_ERR, S_configInfo_SET_FILENAME_FAILED,
                      "Filename, %s, not set", sFilename);
    return;
  }

  /****
  [[U this->ecInitError = tConfigInfo::SetSection(sSection)]]
  ****/
  this->ecInitError = this->SetSection(sSection);
  if(this->ecInitError != OK) {
    TraceMsg(TMT_ERR, S_configInfo_SET_SECTION_FAILED,
                      "Section, %s, not set", sSection);
    return;
  }

  /****
  [[U tBaseObject::ecInitError = tConfigInfo::ReadInfoFromFile(sFilename)]]
  ****/
  this->ecInitError = this->ReadInfoFromFile();
  if(this->ecInitError != OK) {
    TraceMsg(TMT_ERR, S_configInfo_READ_INFO_FAILED,
                      "ReadInfo failed for file, %s; section %s",
                      sFilename, sSection);
    return;
  }
}


/****************************************************************************
*
*  tConfigInfo::SetFilename(const char* sFilename)
*
* DESCRIPTION: Sets the filename for future reading.
*
[[U tConfigInfo::SetFilename()]]
*
*/

ERROR_CODE tConfigInfo::SetFilename(const char* sFilename)
{
  /****
  [[U this->sInfoFilename := main::TruncateAndCopy(sFilename, CiMaxFileNameLength)]]
  ****/
  this->sInfoFilename = ::TruncateAndCopy(sFilename, CiMaxFileNameLength);

  return OK;
}


/****************************************************************************
*
*  tConfigInfo::SetSection(const char* sSection)
*
* DESCRIPTION: Sets the section name to read.  The section string is used if
*  the specific configInfo class only uses one section of the input file.
*  If this method (or constructor with two args) is not called, the entire
*  file will be read in.
*
[[U tConfigInfo::SetSection()]]
*
*/
ERROR_CODE tConfigInfo::SetSection(const char* sSection)
{
  /****
  [[U this->sWhichSection := main::TruncateAndCopy(sSection, CiMaxSectionLength)]]
  ****/
  this->sWhichSection = ::TruncateAndCopy(sSection, CiMaxSectionLength);

  return OK;
}


/****************************************************************************
*
*  tConfigInfo::ReadInfoFromFile(void)
*
* DESCRIPTION: Reads configuration information from appropriate location.
*  Uses internal state to determine file to open/section to read.
*
[[U tConfigInfo::ReadInfoFromFile()]]
*
*/
ERROR_CODE tConfigInfo::ReadInfoFromFile(void)
{
  ERROR_CODE ec;
  
  if (this->bInit == FALSE) {
    return Traceback(S_configInfo_OBJECT_NOT_INITIALIZED,
           "tConfigInfo::ReadInfoFromFile: Unitialized object.  ecInitError = %d",
           this->ecInitError);
  }

  if(!(this->isInfoFile.open(sInfoFilename))) {
    return Traceback(S_configInfo_CANNOT_OPEN_FILE,
                "tConfigInfo::ReadInfoFromFile: unable to open file, %s",
                this->sInfoFilename);
  }


  while (this->isInfoFile) {
    /****
    [[U ec := tConfigInfo::FindSection(this->sWhichSection)]]
    ****/
    ec = this->FindSection(this->sWhichSection);
    if (ec != OK) {
      if (ec == S_configInfo_EOF_REACHED_OK) {
        return OK;
      }
      return Traceback(S_configInfo_ERROR_FINDING_SECTION,
                  "tConfigInfo::ReadInfoFromFile: Error Finding Section %s",
                  this->sWhichSection);
    }
  
    // Section is found.  Now Read all tags in section
    /****
    [[U ec := tConfigInfo::LoadTagsToInfoStream(void)]]
    ****/
    if (ec != OK) {
      if (ec == S_configInfo_EOF_REACHED_OK) {
        return OK;
      }
      return Traceback(S_configInfo_ERROR_FINDING_TAGS,
                  "tConfigInfo::ReadInfoFromFile: Error Reading Tags in Section %s",
                  sCurrentSection);
    }

    // If we are only supposed to read one section, get out of loop
    if (this->sWhichSection)  break;
  }
}


/****************************************************************************
*
* tConfigInfo::FindSection(const char* s)
*
* DESCRIPTION: Finds section whose name matches the string pointed to by s.
*  *exactly*.  Positions file stream immediately after "]"
*
[[U tConfigInfo::FindSection()]]
*
*/
ERROR_CODE tConfigInfo::FindSection(const char* s)
{
  char inChar,
       sSection;
  int  iCharCount;
  BOOL bBeginningOfSectionFound = FALSE,
       bSectionFound = FALSE,
       bSectionComplete = FALSE;

  while(this->isInfoFile.get(&inChar)) {
    if(inChar == BEGIN_SECTION) bBeginningOfSectionFound = TRUE;
    break;
  }

  if(!bBeginningOfSectionFound) {
    if (s) { /* if s has a value, we're looking for a specific section */
      return Traceback(S_configInfo_EOF_REACHED_NO_SECTION_FOUND,
                  "tConfigInfo::FindSection: No Section, %s, found", s);
    }
    else {  /* s is null, not finding a section is OK */
      return S_configInfo_EOF_REACHED_OK;
    }
  }
  else {  /* a section was found */
    sSection = new char[CiMaxSectionNameLength + 1];

    for (iCharCount=0; this->isInfoFile.get(&inChar); iCharCount++) {
      if(inChar == END_SECTION) {
        bSectionComplete = TRUE;
        break;
      }

      if(iCharCount >= CiMaxSectionNameLength) {
        TraceMsg(TMT_WARN, S_configInfo_MAX_SECTION_NAME_EXCEEDED,
                "Maximum Section Name exceeded, %s > %d characters",
                sSection, CiMaxSectionNameLength);
        break;
      }
       
      if(!isspace(inChar))  sSection[iCharCount] = inChar;
    }
  }

  /*
  * At this point, we know that either EOF was reached, the section was
  * successfully read, or the name length was succeeded
