/* $RCSfile: configMgr.cc,v $ */

/*
$Header: /home/chara/cvsroot/ople/sys/inst/initMgr/configMgr.cc,v 1.8 1999/07/14 17:31:31 theo Exp $
*/

/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
01a,25Apr97,rlj    written 
*/

// This file is compiled by both embedded system and GUI therefore it cannot
// include vxworks specific files/definitions.


#include "isit.h"
#include "ftTypes.h"
#include "dlTypes.h"
#include "hwTypes.h"
#include "configMgr.h" 
#include "configError.h"
#include "share/configShare.h"

#include "servoInfo.h"
#include "instrumentCdsGizmoInfo.h"
#include "dlGizmoInfo.h"
#include "quic/quicDlGizmoInfo.h"
#include "gsu/gsuDlGizmoInfo.h"
#include "emGizmoInfo.h"
#include "ftGizmoInfo.h"
#include "stGizmoInfo.h"

#include <logLib.h>


const size_t CszMaxEmbeddedSystemDataSize = 0x80000; // 0.5 MB

class tOutputChannel;
class tInputChannel;

tConfig ConfigManager;
 

/***********************************************************************/
/****************** DL servo param class constructors ******************/
/***********************************************************************/

/*******************************************************************/
/*************************  tDlServoParams *************************/
/*******************************************************************/

/******************************************************************
*
* tDlServoParams::tDlServoParams - constructor
*
* DESCRIPTION: Initializes the structure
*
* RETURNS: Nothing
*
  [[U tDlServoParams::tDlServoParams_params()]]
*
*/

tDlServoParams::tDlServoParams(
                 double dPztGain, double dPztLagPole,
                 double dPztLagZero, double dPztServoRate,
                 BOOL bLagEnabled, BOOL bPztServoEnabled,

                 double dSlewToTrackThresholdInMm,
                 double dTrackToSlewThresholdInMm,
  
                 double dVc1TrackPidGain, double dVc1TrackPidZero1,
                 double dVc1TrackPidZero2, double dVc1TrackPidPole,
                 double dVc1LaserFeedforwardGain,
                 double dVc1LaserFeedforwardFilterMemory,
                 double dVc1TrackServoRate, BOOL bVc1TrackServoEnabled,

                 double dVc1SlewPidGain, double dVc1SlewPidZero1,
                 double dVc1SlewPidZero2, double dVc1SlewPidPole,
                 double dVc1SlewServoRate, BOOL bVc1SlewServoEnabled,

                 double dMotorGain, double dMotorFeedForwardGain,
                 double dMotorServoRate, BOOL bMotorServoEnabled)
{
  this->dPztGain                 = dPztGain;
  this->dPztLagPole              = dPztLagPole;
  this->dPztLagZero              = dPztLagZero;
  this->dPztServoRate            = dPztServoRate;
  this->bLagEnabled              = (double) bLagEnabled;
  this->bPztServoEnabled         = (double) bPztServoEnabled;
  
  this->dSlewToTrackThresholdInMm= dSlewToTrackThresholdInMm;
  this->dTrackToSlewThresholdInMm= dTrackToSlewThresholdInMm;
  
  this->dVc1TrackPidGain         = dVc1TrackPidGain;
  this->dVc1TrackPidZero1        = dVc1TrackPidZero1;
  this->dVc1TrackPidZero2        = dVc1TrackPidZero2;
  this->dVc1TrackPidPole         = dVc1TrackPidPole;
  this->dVc1LaserFeedforwardGain = dVc1LaserFeedforwardGain;
  this->dVc1LaserFeedforwardFilterMemory
                                 = dVc1LaserFeedforwardFilterMemory;
  this->dVc1TrackServoRate       = dVc1TrackServoRate;
  this->bVc1TrackServoEnabled    = (double) bVc1TrackServoEnabled;

  this->dVc1SlewPidGain          = dVc1SlewPidGain;
  this->dVc1SlewPidZero1         = dVc1SlewPidZero1;
  this->dVc1SlewPidZero2         = dVc1SlewPidZero2;
  this->dVc1SlewPidPole          = dVc1SlewPidPole;
  this->dVc1SlewServoRate        = dVc1SlewServoRate;
  this->bVc1SlewServoEnabled     = (double) bVc1SlewServoEnabled;

  this->dMotorGain               = dMotorGain;
  this->dMotorFeedForwardGain    = dMotorFeedForwardGain;
  this->dMotorServoRate          = dMotorServoRate;
  this->bMotorServoEnabled       = (double) bMotorServoEnabled;

}   /* tDlServoParams::tDlServoParams() */


/*******************************************************************/
/************************  tQuicDlServoParams ***********************/
/*******************************************************************/

/******************************************************************
*
* tQuicDlServoParams::tQuicDlServoParams - constructor
*
* DESCRIPTION: Initializes the structure
*
* RETURNS: Nothing
*
  [[U tQuicDlServoParams::tQuicDlServoParams_params()]]
*
*/

tQuicDlServoParams::tQuicDlServoParams(
                 double dPztGain, double dPztLagPole,
                 double dPztLagZero, double dPztServoRate,
                 BOOL bLagEnabled, BOOL bPztServoEnabled,

                 double dSlewToTrackThresholdInMm,
                 double dTrackToSlewThresholdInMm,
  
                 double dVc1TrackPidGain, double dVc1TrackPidZero1,
                 double dVc1TrackPidZero2, double dVc1TrackPidPole,
                 double dVc1LaserFeedforwardGain,
                 double dVc1LaserFeedforwardFilterMemory,
                 double dVc1TrackServoRate, BOOL bVc1TrackServoEnabled,

                 double dVc1SlewPidGain, double dVc1SlewPidZero1,
                 double dVc1SlewPidZero2, double dVc1SlewPidPole,
                 double dVc1SlewServoRate, BOOL bVc1SlewServoEnabled,

                 double dMotorGain, double dMotorFeedForwardGain,
                 double dMotorServoRate, BOOL bMotorServoEnabled,
                 double dMotorErrorGain, double dDiagLaserScale) :

  /****
    [[U tDlServoParams::tDlServoParams_params()]]
  ****/

  tDlServoParams( dPztGain, dPztLagPole,
                  dPztLagZero, dPztServoRate,
                  bLagEnabled,  bPztServoEnabled,
                  
                  dSlewToTrackThresholdInMm, dTrackToSlewThresholdInMm,
                  
                  dVc1TrackPidGain,  dVc1TrackPidZero1,
                  dVc1TrackPidZero2,  dVc1TrackPidPole,
                  dVc1LaserFeedforwardGain,
                  dVc1LaserFeedforwardFilterMemory,
                  dVc1TrackServoRate,  bVc1TrackServoEnabled,

                  dVc1SlewPidGain,  dVc1SlewPidZero1,
                  dVc1SlewPidZero2,  dVc1SlewPidPole,
                  dVc1SlewServoRate,  bVc1SlewServoEnabled,

                  dMotorGain,  dMotorFeedForwardGain,
                  dMotorServoRate, bMotorServoEnabled)

{}  /* tQuicDlServoParams::tQuicDlServoParams() */



// Board array: CiBoardArraySize *MUST* equal the number of elements in
//              the CbtBoardArray.
const int CiBoardArraySize = 8;
const BOARD_TYPE CbtBoardArray[CiBoardArraySize] = {
  BT_MM6216, BT_TIM, BT_PZT_MOTOR, BT_MPV906,
  BT_DVME626, BT_DVME626, BT_FAST_LASER, BT_MPV955
};


/*********************************************************
*
* tConfig:: tConfig
*/

tConfig::tConfig(void) :
  
  Hardware(CbtBoardArray, CiBoardArraySize)
{

  int i,iGizmoId;

  TraceMark(S_configMgr_ENTERING_CONFIG_MANAGER_CONSTRUCTOR);
  
  int iInstRte = OnWhichRte (CiInst);
  IPC_NAME inErrorName, inDataName, inStatusName;
  MEM_TYPE mtMemType;

//  tFtPhysicalConstants FtPhysicalConstants;
  
//  FtPhysicalConstants.lvWhiteLightLambda = WHITE_LIGHT_LAMBDA_IN_LASER_UNITS;
//  FtPhysicalConstants.dApdScaleFactor = 1;
//  FtPhysicalConstants.dApdDiagScaleFactor = 1;
//  FtPhysicalConstants.iNumberOfWobbleSamples = 4;
  
  #ifndef MADE_UP
  #error Made Up values
  #endif

  /****
  [[U FtDefaultThresholds.tFtThresholdParameters::tFtThresholdParameters_ttdddddd  (
    (int) (TIME_TICKS_PER_SECOND * 0.01), TIME_TICKS_PER_SECOND * 60,
    0.0, 0.0, 0.0, 0.0, 0.0, 0.0) ]]
  ****/
//  tFtThresholdParameters FtDefaultThresholds (
//    (int) (TIME_TICKS_PER_SECOND * 0.01),
//    TIME_TICKS_PER_SECOND * 60,
//    0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

  /****
  [[U FtDefaultControllerParameters.tFtControllerParameters::
    tFtControllerParameters_tiiiddid (
    CtFtMediumServoPeriod, WHITE_LIGHT_LAMBDA_IN_LASER_UNITS, 4, 15,
    1.0, 1.0, 1, 0.0) ]]
  ****/
//  tFtControllerParameters FtDefaultControllerParameters (
//    CtFtMediumServoPeriod, WHITE_LIGHT_LAMBDA_IN_LASER_UNITS, 4, 15,
//    0.2, 1.0, 1, 0.0);
    
  // end made up
  
  /****
  [[U FtDefaultSearchParmeters.tFtSearchParameters::tFtSearchParameters_dddST  (
    (100 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS), 
    (100000 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS),
    (2 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS),
     2, SPIRAL_GEOMETRIC ) ]]
  ****/
//  tFtSearchParameters FtDefaultSearchParameters (
//    (50 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS), 
//    (10000 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS),
//    (2 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS),
//    2, SPIRAL_GEOMETRIC );
     
     
  /****
  [[U FtDefaultWobbleParameters.tModulationParams::tModulationParams_MTdd (
     MT_WOBBLE,
     (0.5 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS), 
     ( 4.0 * TIME_TICKS_PER_SECOND / (double) (CtFtMediumServoPeriod ) ) ]]
  ****/
//  tModulationParams FtDefaultWobbleParameters (
//     MT_WOBBLE,
//     ( 4.0 / (double) CtFtMediumServoPeriod),
//     0.5 * WHITE_LIGHT_LAMBDA_IN_LASER_UNITS );
     
  
  /****
  [[U FtDefaultServoParameters.tFtServoParameters::tFtServoParameters_tttt (
     FtDefaultSearchParameters,
     FtDefaultWobbleParameters, FtDefaultControllerParameters,
     FtDefaultThresholds) ]]
  ****/
//  tFtServoParameters FtDefaultServoParameters (FtDefaultSearchParameters,
//     FtDefaultWobbleParameters, FtDefaultControllerParameters,
//     FtDefaultThresholds);

//  tFtImageParameters FtDefaultImageParameters;
//  FtDefaultImageParameters.iStartXSt0 = 0;
//  FtDefaultImageParameters.iStartYSt0 = 0;
//  FtDefaultImageParameters.iStartXSt0 = 0;
//  FtDefaultImageParameters.iStartXSt1 = 0;
//  FtDefaultImageParameters.iStartYSt1 = 0;
//  FtDefaultImageParameters.iStartXFringeImage = 0;
//  FtDefaultImageParameters.iStartYFringeImage = 0;
//  FtDefaultImageParameters.iFringeImageHSize = CiFringeImageHSize;
//  FtDefaultImageParameters.iFringeImageVSize = CiFringeImageVSize;


  /****
  [[U StServoInfo.tServoInfo::tServoInfo ("StServo",1) ]]
  ****/
//  tServoInfo StServoInfo ("StServo",1);

 /****
  [[U EmServoInfo.tServoInfo::tServoInfo ("EmServo",1) ]]
  ****/
//  tServoInfo EmServoInfo ("EmServo",1);


  /****
  [[U FtServoInfo.tServoInfo::tServoInfo ("FtServo",FSP_NUM_SERVO_PARTITIONS) ]]
  ****/
//  tServoInfo FtServoInfo ("FtServo",FSP_NUM_SERVO_PARTITIONS);

  /****
  [[U DlServoInfo.tServoInfo::tServoInfo ("DlServo",3) ]]
  ****/
  tServoInfo DlServoInfo ("DlServo",3);
  

  /****
  [[U EmServoInfo.tServoInfo::AssignServoTaskInfo (0,CtEmServoPeriod,CiEmServoPriority) ]]
  ****/
//  EmServoInfo.AssignServoTaskInfo (0,CtEmServoPeriod,CiEmServoPriority);

  /****
  [[U StServoInfo.tServoInfo::AssignServoTaskInfo (0, CtStServoPeriod,CiStServoPriority) ]]
  ****/
//  StServoInfo.AssignServoTaskInfo (0,CtStServoPeriod, CiStServoPriority);

  /****
  [[U FtServoInfo.tServoInfo::AssignServoTaskInfo (FSP_FAST, CtFtFastServoPeriod,
      CiFtFastServoPriority) ]]
  ****/
//  FtServoInfo.AssignServoTaskInfo (FSP_FAST, CtFtFastServoPeriod,
//      CiFtFastServoPriority);

  /****
  [[U FtServoInfo.tServoInfo::AssignServoTaskInfo (FSP_FAST, CtFtMediumServoPeriod,
      CiFtMediumServoPriority) ]]
  ****/
//  FtServoInfo.AssignServoTaskInfo (FSP_MEDIUM, CtFtMediumServoPeriod,
//      CiFtMediumServoPriority);

  /****
  [[U FtServoInfo.tServoInfo::AssignServoTaskInfo (FSP_SLOW, CtFtSlowServoPeriod,
      CiFtSlowServoPriority) ]]
  ****/
//  FtServoInfo.AssignServoTaskInfo (FSP_SLOW, CtFtSlowServoPeriod,
//      CiFtSlowServoPriority);

  /****
  [[U FtServoInfo.tServoInfo::AssignServoTaskInfo (FSP_IMAGING, CtFtImagingServoPeriod,
      CiFtImagingServoPriority) ]]
  ****/
//  FtServoInfo.AssignServoTaskInfo (FSP_IMAGING, CtFtImagingServoPeriod,
//      CiFtImagingServoPriority);



  /****
  [[U DlServoInfo.tServoInfo::AssignServoTaskInfo (DSP_FAST,
                         CtDlFastServoPeriod,CiDlFastServoPriority) ]]
  ****/
  DlServoInfo.AssignServoTaskInfo (DSP_FAST,CtDlFastServoPeriod,
                                   CiDlFastServoPriority);

  /****
  [[U DlServo.tServoInfo::AssignServoTaskInfo (DSP_MEDIUM, 
        CtDlMedServoPeriod,CiDlMedServoPriority) ]]
  ****/
  DlServoInfo.AssignServoTaskInfo (DSP_MEDIUM,CtDlMedServoPeriod,
                                   CiDlMedServoPriority);

  /****
  [[U DlServo.tServoInfo::AssignServoTaskInfo (DSP_SLOW,
        CtDlSlowServoPeriod,CiDlSlowServoPriority) ]]
  ****/
  DlServoInfo.AssignServoTaskInfo (DSP_SLOW,CtDlSlowServoPeriod,
                                   CiDlSlowServoPriority);
  
  
  
  
  
  
  
  for (i = 0; i < NRTES; i++) {
    inErrorName  = (NUM_SYS_GIZMOS << 16) | (3*i + 1); // calc unique IPC_NAME
    inDataName   = (NUM_SYS_GIZMOS << 16) | (3*i + 2); // calc unique IPC_NAME
    inStatusName = (NUM_SYS_GIZMOS << 16) | (3*i + 3); // calc unique IPC_NAME
    
    if (i == iInstRte) mtMemType = MT_LOCAL; // Instrument RTE stream
    else mtMemType = MT_SHARED;
    
    /****
    [[U  this->iiRteErrorStreamIdArray[i] := 
      tIpcFifoId::tIpcFifoId_INMTi (inErrorName,mtMemType,CiRteErrorFifoDepth)]]
    ****/
    this->iiRteErrorStreamIdArray[i] = 
      tIpcFifoId (inErrorName,mtMemType,CiRteErrorFifoDepth);

    /****
    [[U  this->iiRteDataStreamIdArray[i] := 
      tIpcFifoId::tIpcFifoId_INMTi (inDataName,mtMemType,CiDataFifoDepth)]]
    ****/
    this->iiRteDataStreamIdArray[i] = 
      tIpcFifoId (inDataName,mtMemType,CiDataFifoDepth);

    /****
    [[U  this->iiRteStatusStreamIdArray[i] := 
      tIpcFifoId::tIpcFifoId_INMTi (inErrorName,mtMemType,CiStatusFifoDepth)]]
    ****/
    this->iiRteStatusStreamIdArray[i] = 
      tIpcFifoId (inStatusName,mtMemType,CiStatusFifoDepth);
  }   
      

  int iRteId, iGizmoSubindex;
  
  for (iGizmoId=0; iGizmoId<NUM_SYS_GIZMOS; iGizmoId++) {
    iRteId = OnWhichRte (iGizmoId);
    iGizmoSubindex = GetGizmoSubindex (iGizmoId);

    switch(CgtSystemGizmoTypes[iGizmoId]) {
     case GT_INSTRUMENT:
       TraceMark(S_configMgr_CONSTRUCTING_INST_GIZMO_INFO);

       /*****
       [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tInstrumentCDSGizmoInfo::
         tInstrumentCDSGizmoInfo (iRteId,iGimzoId)]]
       ****/
       this->pCgiSystemGizmoInfoArray[iGizmoId] = 
               new tInstrumentCDSGizmoInfo (iRteId, iGizmoId, TIM_BOARD_NAME);
       break;

#if 0  /* No Quic DL in GSU build */
     case GT_DL_BB1:
       TraceMark(S_configMgr_CONSTRUCTING_BB1_DL_GIZMO);

       /*****
       [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tQuicDlGizmoInfo::
                 tQuicDlGizmoInfo_params(void)]]
       ****/
       this->pCgiSystemGizmoInfoArray[iGizmoId] = new tQuicDlGizmoInfo
             (iRteId,iGizmoId,
              GT_DL_BB1,
              CiDlBb1TargetSourceArray, 
              CtBb1DitherPeriod,
              CpcBb1PhysConst, CiBb1NumServoModes,
              DlServoInfo,
              CspBb1ServoParams,
              DVME626_BOARD_NAME, 2,
              PZT_MOTOR_BOARD_NAME, 1,
              PZT_MOTOR_BOARD_NAME, 7,
              MPV955_BOARD_NAME, 1,
              PZT_MOTOR_BOARD_NAME, 29,
              PZT_MOTOR_BOARD_NAME, 31,
              PZT_MOTOR_BOARD_NAME, 30,
              PZT_MOTOR_BOARD_NAME, 32,
              FAST_LASER_BOARD_NAME, 1,
              FAST_LASER_BOARD_NAME, 13,
              PZT_MOTOR_BOARD_NAME, 13,
              PZT_MOTOR_BOARD_NAME, 19,
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,
              DVME626_BOARD_NAME, 3,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,

              MPV906_BOARD_NAME, 4           // Diag VcPos In
              );
       break;
        
     case GT_DL_BB2:
       TraceMark(S_configMgr_CONSTRUCTING_BB2_DL_GIZMO);

       /*****
       [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tQuicDlGizmoInfo::
           tQuicDlGizmoInfo_params()]]
       ****/
       this->pCgiSystemGizmoInfoArray[iGizmoId] = new tQuicDlGizmoInfo 
             (iRteId,iGizmoId,
              GT_DL_BB2,
              CiDlBb2TargetSourceArray, 
              CtBb2DitherPeriod,
              CpcBb2PhysConst, CiBb2NumServoModes,
              DlServoInfo,
              CspBb2ServoParams,
              DVME626_BOARD_NAME, 2,
              PZT_MOTOR_BOARD_NAME, 1,
              PZT_MOTOR_BOARD_NAME, 7,
              MPV955_BOARD_NAME, 1,
              PZT_MOTOR_BOARD_NAME, 29,
              PZT_MOTOR_BOARD_NAME, 31,
              PZT_MOTOR_BOARD_NAME, 30,
              PZT_MOTOR_BOARD_NAME, 32,
              FAST_LASER_BOARD_NAME, 0,
              FAST_LASER_BOARD_NAME, 12,
              PZT_MOTOR_BOARD_NAME, 13,
              PZT_MOTOR_BOARD_NAME, 19,
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,

              MPV906_BOARD_NAME, 4           // Diag VcPos In
              );
       break;

#endif  /* Removing Quic DL from Build */



/*****************************************************************************
*
*    FOR CONFIGURING GSU DELAY LINES
*
*****************************************************************************/

     case GT_DL_GSU:
       TraceMark(S_configMgr_CONSTRUCTING_GSU_DL_GIZMO);

	/**********************************************************/
	/* New addition by THEO.				  */
	/* Make sure we have the correct iLaserSign for each cart */
	/**********************************************************/

        DlPhysicalConstants GsuPhysConst(CpcGsuPhysConst);

        switch (iGizmoSubindex) {

        case 0 :  /* For DL #0 (S2) */
		GsuPhysConst.iLaserSign = 1;
		break;

        case 1 :  /* For DL #1 (W1) */
		GsuPhysConst.iLaserSign = 1;
		break;

        case 2 :  /* For DL #2 (S1) */
		GsuPhysConst.iLaserSign = 1;
		break;

        case 3 :  /* For DL #3 (E1) */
		GsuPhysConst.iLaserSign = 1;
		break;

        case 4 :  /* For DL #4 (E2) */
		GsuPhysConst.iLaserSign = 1;
		break;

        default:
          TraceMsg(tmt_ERROR, S_configMgr_UNKNOWN_GSU_DL_GIZMO_NUMBER,
                   "Invalid DL #%d", iGizmoSubindex);
	}
	
       switch (iGizmoSubindex) {

        case 0 :  /* For DL #0 */
	  
          this->pCgiSystemGizmoInfoArray[iGizmoId] = new tGsuDlGizmoInfo
             (iRteId, iGizmoId,
              CiDlGsuTargetSourceArray, 
              CtGsuDitherPeriod,
              GsuPhysConst, CiGsuNumServoModes,
              DlServoInfo,
              CspGsuServoParams,
              DVME626_BOARD_NAME, 0,     // VC1
              DVME626_BOARD_NAME, 1,     // VC2
              PZT_MOTOR_BOARD_NAME, 0,   // PZT Channel
              PZT_MOTOR_BOARD_NAME, 6,   // Motor channel
              MPV955_BOARD_NAME, 0,      // Dither channel
              MPV906_BOARD_NAME, 0,      // Eddy sensor 1
              MPV906_BOARD_NAME, 1,      // Eddy sensor 2
              PZT_MOTOR_BOARD_NAME, 24,  // Front Soft Limit
              PZT_MOTOR_BOARD_NAME, 26,  // Front Hard Limit
              PZT_MOTOR_BOARD_NAME, 25,  // Back Soft Limit
              PZT_MOTOR_BOARD_NAME, 27,  // Back Hard Limit
              FAST_LASER_BOARD_NAME, 1,                   // Laser
              FAST_LASER_BOARD_NAME, 13,                  // Home
              PZT_MOTOR_BOARD_NAME, 12,                   // PZT readback
              PZT_MOTOR_BOARD_NAME, 18,                   // Motor readback
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag laser in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag pzt pos in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy1 in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy2 in
		/* USE THESE FOR PZT TUNING */
              //DVME626_BOARD_NAME+1, 5,                    // diag laser err
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,  
              DVME626_BOARD_NAME+1, 5,                      // diag laser out
              //NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,  
		/* USE THIS FOR VC1 TUNING */
              DVME626_BOARD_NAME+1, 4,                      // diag vc1 pos
              //NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
		/* USE THIS FOR VC2 TUNING */
              //DVME626_BOARD_NAME+1, 5,                      // diag vc2 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5);                     // diag motor error
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL);
        break; /* case GT_DL_GSU #1 */

        case 1 :  /* For DL #1 */
          this->pCgiSystemGizmoInfoArray[iGizmoId] = new tGsuDlGizmoInfo
             (iRteId, iGizmoId,
              CiDlGsuTargetSourceArray, 
              CtGsuDitherPeriod,
              GsuPhysConst, CiGsuNumServoModes,
              DlServoInfo,
              CspGsuServoParams,
              DVME626_BOARD_NAME, 2,     // VC1
              DVME626_BOARD_NAME, 3,     // VC2
              PZT_MOTOR_BOARD_NAME, 1,   // PZT Channel
              PZT_MOTOR_BOARD_NAME, 7,   // Motor channel
              MPV955_BOARD_NAME, 1,      // Dither channel
              MPV906_BOARD_NAME, 2,      // Eddy sensor 1
              MPV906_BOARD_NAME, 3,      // Eddy sensor 2
              PZT_MOTOR_BOARD_NAME, 29,  // Front Soft Limit
              PZT_MOTOR_BOARD_NAME, 31,  // Front Hard Limit
              PZT_MOTOR_BOARD_NAME, 30,  // Back Soft Limit
              PZT_MOTOR_BOARD_NAME, 32,  // Back Hard Limit
              FAST_LASER_BOARD_NAME, 3,                   // Laser
              FAST_LASER_BOARD_NAME, 15,                  // Home
              PZT_MOTOR_BOARD_NAME, 13,                   // PZT readback
              PZT_MOTOR_BOARD_NAME, 19,                   // Motor readback
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag laser in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag pzt pos in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy1 in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy2 in
             //DVME626_BOARD_NAME, 3,                      // diag laser out
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 4,                      // diag laser err
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc1 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc2 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5);                     // diag motor error
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL);
        break; /* case GT_DL_GSU  #1 */

        case 2 :  /* For DL #2 */
          this->pCgiSystemGizmoInfoArray[iGizmoId] = new tGsuDlGizmoInfo
             (iRteId, iGizmoId,
              CiDlGsuTargetSourceArray, 
              CtGsuDitherPeriod,
              GsuPhysConst, CiGsuNumServoModes,
              DlServoInfo,
              CspGsuServoParams,
              DVME626_BOARD_NAME, 4,     // VC1
              DVME626_BOARD_NAME, 5,     // VC2
              PZT_MOTOR_BOARD_NAME, 2,   // PZT Channel
              PZT_MOTOR_BOARD_NAME, 8,   // Motor channel
              MPV955_BOARD_NAME, 2,      // Dither channel
              MPV906_BOARD_NAME, 4,      // Eddy sensor 1
              MPV906_BOARD_NAME, 5,      // Eddy sensor 2
              PZT_MOTOR_BOARD_NAME, 34,
              PZT_MOTOR_BOARD_NAME, 36,
              PZT_MOTOR_BOARD_NAME, 35,
              PZT_MOTOR_BOARD_NAME, 37,
              FAST_LASER_BOARD_NAME, 5,                   // Laser
              FAST_LASER_BOARD_NAME, 17,                  // Home
              PZT_MOTOR_BOARD_NAME, 14,                   // PZT readback
              PZT_MOTOR_BOARD_NAME, 20,                   // Motor readback
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag laser in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag pzt pos in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy1 in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy2 in
             //DVME626_BOARD_NAME, 3,                      // diag laser out
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 4,                      // diag laser err
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc1 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc2 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5);                     // diag motor error
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL);
        break; /* case GT_DL_GSU #2 */

        case 3 :  /* For DL #3 */
          this->pCgiSystemGizmoInfoArray[iGizmoId] = new tGsuDlGizmoInfo
             (iRteId, iGizmoId,
              CiDlGsuTargetSourceArray, 
              CtGsuDitherPeriod,
              GsuPhysConst, CiGsuNumServoModes,
              DlServoInfo,
              CspGsuServoParams,
              DVME626_BOARD_NAME+1, 0,     // VC1
              DVME626_BOARD_NAME+1, 1,     // VC2
              PZT_MOTOR_BOARD_NAME, 3,   // PZT Channel
              PZT_MOTOR_BOARD_NAME, 9,   // Motor channel
              MPV955_BOARD_NAME, 3,      // Dither channel
              MPV906_BOARD_NAME, 6,      // Eddy sensor 1
              MPV906_BOARD_NAME, 7,      // Eddy sensor 2
              PZT_MOTOR_BOARD_NAME, 39,
              PZT_MOTOR_BOARD_NAME, 41,
              PZT_MOTOR_BOARD_NAME, 40,
              PZT_MOTOR_BOARD_NAME, 42,
              FAST_LASER_BOARD_NAME, 7,                   // Laser
              FAST_LASER_BOARD_NAME, 19,                  // Home
              PZT_MOTOR_BOARD_NAME, 15,                   // PZT readback
              PZT_MOTOR_BOARD_NAME, 21,                   // Motor readback
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag laser in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag pzt pos in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy1 in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy2 in
             //DVME626_BOARD_NAME, 3,                      // diag laser out
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 4,                      // diag laser err
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc1 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc2 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5);                     // diag motor error
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL);
        break; /* case GT_DL_GSU  #3 */

        case 4 :  /* For DL #4 */
          this->pCgiSystemGizmoInfoArray[iGizmoId] = new tGsuDlGizmoInfo
             (iRteId, iGizmoId,
              CiDlGsuTargetSourceArray, 
              CtGsuDitherPeriod,
              GsuPhysConst, CiGsuNumServoModes,
              DlServoInfo,
              CspGsuServoParams,
              DVME626_BOARD_NAME+1, 2,     // VC1
              DVME626_BOARD_NAME+1, 3,     // VC2
              PZT_MOTOR_BOARD_NAME, 4,   // PZT Channel
              PZT_MOTOR_BOARD_NAME, 10,  // Motor channel
              MPV955_BOARD_NAME, 4,      // Dither channel
              MPV906_BOARD_NAME, 8,      // Eddy sensor 1
              MPV906_BOARD_NAME, 9,      // Eddy sensor 2
              PZT_MOTOR_BOARD_NAME, 44,
              PZT_MOTOR_BOARD_NAME, 46,
              PZT_MOTOR_BOARD_NAME, 45,
              PZT_MOTOR_BOARD_NAME, 47,
              FAST_LASER_BOARD_NAME, 9,                   // Laser
              FAST_LASER_BOARD_NAME, 21,                  // Home
              PZT_MOTOR_BOARD_NAME, 16,                   // PZT readback
              PZT_MOTOR_BOARD_NAME, 22,                   // Motor readback
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag laser in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag pzt pos in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy1 in
              NULL_BOARD_NAME, NCT_NULL_INPUT_CHANNEL,    // diag eddy2 in
             //DVME626_BOARD_NAME, 3,                      // diag laser out
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 4,                      // diag laser err
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc1 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5,                      // diag vc2 pos
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL,    
             //DVME626_BOARD_NAME, 5);                     // diag motor error
              NULL_BOARD_NAME, NCT_NULL_OUTPUT_CHANNEL);
        break; /* case GT_DL_GSU #4 */
  
        default:
          TraceMsg(tmt_ERROR, S_configMgr_UNKNOWN_GSU_DL_GIZMO_NUMBER,
                   "Invalid DL #%d", iGizmoSubindex);
     }
     break;

#if 0  /* No FT, EM, or ST in GSU build */

     case GT_FRINGE_TRACKER:
       TraceMark(S_configMgr_CONSTRUCTING_FT_GIZMO);

       /*****
       [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tFtGizmoInfo::
           tFtGizmoInfo (iRteId, iGizmoId, FSM_NUM_SERVO_MODES,
             FtServoInfo, CiDlGsu, CiSt0, CiSt1, FtDefaultServoParameters,
             FtDefaultImageParameters, FtPhysicalConstants,
             VIP640A_BOARD_NAME, 0, 
             DVME626_BOARD_NAME, 5,
             IPQUAD_BOARD_NAME, 0) ]]
       ****/
        this->pCgiSystemGizmoInfoArray[iGizmoId] = 
          new tFtGizmoInfo (iRteId, iGizmoId, FSM_NUM_SERVO_MODES,
            FtServoInfo, FtPhysicalConstants, FtDefaultServoParameters,
            CiDlGsu, CiSt0, CiSt1, FtDefaultImageParameters,
            IPQUAD_BOARD_NAME, 0, DVME626_BOARD_NAME, 5,
            VIP640A_BOARD_NAME, 0);
       break; /* case GT_FRINGE_TRACKER */

     case GT_EXTERNAL_METROLOGY:
       TraceMark(S_configMgr_CONSTRUCTING_EM_GIZMO);

       /*****
       [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tEmGizmoInfo::
           tEmGizmoInfo (iRteId,iGizmoId, 2, EmServoInfo, CiDlGsu) ]]
       ****/
       this->pCgiSystemGizmoInfoArray[iGizmoId] = 
                new tEmGizmoInfo (iRteId,iGizmoId, 2, EmServoInfo,
                                   CiDlGsu );
       break; /* case GT_EXTERNAL_METROLOGY */

    case GT_STAR_TRACKER:
       TraceMark(S_configMgr_CONSTRUCTING_ST_GIZMO);
       switch (iGizmoSubindex) {
         case 0 :  /* For StarTracker #0 */
           /*****
           [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tStGizmoInfo::
               tStGizmoInfo (iRteId, iGizmoId,
                             CstPhysConst, CstServoParams, 2, StServoInfo, CiFt0,
                             0, CiStarImageHSize, CiStarImageVSize,
                             FsmBoardArray, FsmChannelArray) ]]
           ****/
           this->pCgiSystemGizmoInfoArray[iGizmoId] = 
                new tStGizmoInfo (iRteId,
                                  iGizmoId,
                                  Cst0PhysConst,
                                  Cst0ServoParams,
                                  SSS_NUM_SERVO_MODES,
                                  StServoInfo,
                                  CiFt0, /* FT Gizmo Id */
                                  0, /* WAP Gizmo Id, not used in I5a */
                                  CiStarImageHSize,
                                  CiStarImageVSize,
                                  CbnSt0FsmBoardArray, /* FSM output board(s) */
                                  CiSt0FsmChannelArray /* FSM output channels */
                                  );
           break; /*Star Tracker 0 */

         case 1 :   /* For Star Tracker #1 */
           /*****
           [[U this->pCgiSystemGizmoInfoArray[iGizmoId]->tStGizmoInfo::
               tStGizmoInfo (iRteId, iGizmoId,
                             CstPhysConst, CstServoParams, 2, StServoInfo, CiFt0,
                             0, CiStarImageHSize, CiStarImageVSize,
                             FsmBoardArray, FsmChannelArray) ]]
           ****/
           this->pCgiSystemGizmoInfoArray[iGizmoId] = 
                new tStGizmoInfo (iRteId,
                                  iGizmoId,
                                  Cst1PhysConst,
                                  Cst1ServoParams,
                                  SSS_NUM_SERVO_MODES,
                                  StServoInfo,
                                  CiFt0, /* FT Gizmo Id */
                                  0, /* WAP Gizmo Id, not used in I5a */
                                  CiStarImageHSize,
                                  CiStarImageVSize,
                                  CbnSt1FsmBoardArray, /* FSM output board(s) */
                                  CiSt1FsmChannelArray /* FSM output channels */
                                  );
           break; /*Star Tracker 1 */
       }
       break; /* case GT_STAR_TRACKER */
#endif  /* Excluding FT, ST, and EM */

    default:
      this->SetInitError(S_configMgr_INVALID_GIZMO_TYPE);
      TraceMsg(tmt_ERROR, S_configMgr_INVALID_GIZMO_TYPE,
               "Invalid GizmoType on Gizmo %d", iGizmoId);
      return;
    }    
  }

  TraceMark(S_configMgr_DONE_CONSTRUCTING_CONFIG_MANAGER);
}


/*********** Utility Function ***************************/




int OnWhichRte (int iGizmoId) {

  int i;
  
  for (i = 0; i < NUM_RTE0_GIZMOS; i++) 
    if (CiRte0Gizmos[i] == iGizmoId) return 0;
  for (i = 0; i < NUM_RTE1_GIZMOS; i++) 
    if (CiRte1Gizmos[i] == iGizmoId) return 1;
  for (i = 0; i < NUM_RTE2_GIZMOS; i++) 
    if (CiRte2Gizmos[i] == iGizmoId) return 2;
  for (i = 0; i < NUM_RTE3_GIZMOS; i++) 
    if (CiRte3Gizmos[i] == iGizmoId) return 3;
  for (i = 0; i < NUM_RTE4_GIZMOS; i++) 
    if (CiRte4Gizmos[i] == iGizmoId) return 4;
  return -1;
}

int GetGizmoSubindex (int iGizmoId)
{
  return CiGizmoSubindexArray[iGizmoId];
}

int RteIndexToGizmoId (int iRteId, int iWhichGizmo)
{
  if (iWhichGizmo < 0) return INVALID_GIZMO;
  
  switch (iRteId) {
    case 0 : if (iWhichGizmo < NUM_RTE0_GIZMOS) return CiRte0Gizmos [iWhichGizmo];
             break;
    case 1 : if (iWhichGizmo < NUM_RTE1_GIZMOS) return CiRte1Gizmos [iWhichGizmo];
             break;
    case 2 : if (iWhichGizmo < NUM_RTE2_GIZMOS) return CiRte2Gizmos [iWhichGizmo];
             break;
    case 3 : if (iWhichGizmo < NUM_RTE3_GIZMOS) return CiRte3Gizmos [iWhichGizmo];
             break;
    case 4 : if (iWhichGizmo < NUM_RTE4_GIZMOS) return CiRte4Gizmos [iWhichGizmo];
             break;
    default : return INVALID_GIZMO;
  }
  return INVALID_GIZMO;
}


/****
[[U tConfig::GetGizmoInfo_i (iGizmoId) ]]
****/
const tGizmoInfo* tConfig::GetGizmoInfo (int iGizmoId) const {

  if ( (iGizmoId < 0) || (iGizmoId >= NUM_SYS_GIZMOS) ) return NULL;
  return this->pCgiSystemGizmoInfoArray[iGizmoId];
}

/****
[[U tConfig::NumberOfGizmosOnRte  (iWhichRte)]]
****/
int tConfig::NumberOfGizmosOnRte (int iWhichRte) const {

  switch (iWhichRte) {
    case 0  : return NUM_RTE0_GIZMOS;break;
    case 1  : return NUM_RTE1_GIZMOS;break;
    case 2  : return NUM_RTE2_GIZMOS;break;
    case 3  : return NUM_RTE3_GIZMOS;break;
    case 4  : return NUM_RTE4_GIZMOS;break;
    default : 
      TraceMsg(tmt_ERROR, S_configMgr_INVALID_RTE_NUM,
               "tConfig::NumberOfGizmosOnRte");
      return -1;
  };
}
       
int tConfig::GetGizmoRteNumber (int iGizmoId) const {
  const tGizmoInfo* pGizmoInfo = this->GetGizmoInfo (iGizmoId);
  
  return pGizmoInfo->GetRteNumber ();
}

size_t tConfig::GetRelayBufferSize () const 
{

  return CszMaxEmbeddedSystemDataSize;
}

