/* $RCSfile: initMgr.cc,v $ - Init Manager code */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/inst/initMgr/initMgr.cc,v 1.2 1998/08/17 21:01:25 phil Exp $*/
/* Beth McKenney - Jet Propulsion Laboratory */
 
/*
modification history
--------------------
03a,19Jun97,rlj      modified for new IPC
01b,18Oct96,eamck    added iWhichCPU to Init arg list
01b,03Oct96,eamck    fixed bug in WaitForHB
01a,25Sep96,eamck    written (PowerOn only)
*/
  
/*
DESCRIPTION
  ISIT Power On Init code
*/

#define INC_initMgrcc

#include "isit.h"
#include "initMgr.h"
#include "sharedMem.h"
#include "ipcMemPointer.h"
#include "configMgr.h"
#include <stdioLib.h>
#include <sysLib.h>
#include <symLib.h>
#include <sysSymTbl.h>
#include <cplusLib.h>
#include <usrLib.h>
#include <errnoLib.h>
#include "vxworks/taskLib.spc"
#include <new.h>

/**************************************************************************
*
* Global variables
*/

volatile SYS_INIT_INFO *vSysInitInfoG;
const TIME CiTokenServerTimeOut = 1000;

// static members of IpcPointer objects
volatile BYTE* tIpcMemoryBytePointer::vIpcMemoryBaseAddress;
volatile SHMEM_WORD *tIpcMemoryWordPointer::vIpcMemoryBaseAddress;


/*************************************************************************
*
Spec Functions Defined in initMgr.h:

int CPUNumber (int)
volatile HEARTBEAT* HeartBeat (volatile BYTE*,int)
volatile HEALTH* Health (volatile BYTE*,int)
volatile INIT_TYPE* InitType (volatile BYTE*, int)
*/

/*******************************************************************************
*
* Function Name: FreeStoreException (void)
* 
* 
  [[U ::FreeStoreException (void) ]]
*/
void FreeStoreException(...)
{
   fprintf(stderr,"Operator new failed  -- aborting task\n");
   abort();
}


/*******************************************************************************
*
* Function Name: PowerOn (void)
* 
* 
  [[U ::PowerOn (void) ]]
*/

ERROR_CODE PowerOn ()
{
   int                  iShmemSize;
   int                  iWhichCPU;
   int                  i, iInitHeaderSize, iIpcBytes;
   volatile SHMEM_WORD  *vswIpcShmemStart;
   volatile BYTE        *vIpcShmem;
   volatile BYTE        *vShmemStart;
   volatile HEARTBEAT   *vHbLocation;
   volatile HEALTH      *vRteHealth;
   volatile INIT_TYPE   *vRteInitType;
   
   tSharedMemory        *pSharedMemory;

   ERROR_CODE           ec = OK;                 /* [ ec := OK ] */


   TraceMark(S_initmgr_ENTERING_POWERON);

   set_new_handler(FreeStoreException);

   /****
   [ iWhichCPU := VxWorks CPU number stored in EEPROM ]
   ****/   
   iWhichCPU = sysProcNumGet();
   
   if (iWhichCPU == INIT_SERVER_CPU)      /* Are we the server CPU? */
   {                
      /*****
      [[U ec := (::ConfigMangager).tHardware::HardInit (void) ]]
      *****/
      ec = ::ConfigManager.Hardware.HardInit();
      if (ec != OK)
      {
        fprintf(stderr, "PowerOn: ConfigManager.Hardware.HardInit() error\n");
        return ec;
      }

      TraceMark(S_initmgr_HARDWARE_HARDINIT_SUCCESSFUL);

      /*****
      [[U iShmemSize := (::ConfigMananger).tHardware::GetShmemSize (void)]]
      *****/
      iShmemSize = ::ConfigManager.Hardware.GetShmemSize();

      if (iShmemSize <= 0) return Traceback (S_initmgr_SHMEM_BOARD_HAS_NO_MEMORY,
        "PowerOn");

       /*****
      [[U vShmemStart := (::ConfigManager).tHardware::GetShmemLocalAddress (void)]]
      *****/
      vShmemStart = ConfigManager.Hardware.GetShmemLocalAddress();

      if (vShmemStart == NULL) return Traceback (S_initmgr_INVALID_SHMEM_START_ADDRESS,
        "PowerOn");

      printf("Shared memory local address is 0x%x\n",(unsigned int)vShmemStart);

           /* Initialize pointers to InitMgr shmem structures */
      vSysInitInfoG = (SYS_INIT_INFO *) vShmemStart;
      vHbLocation   = (volatile HEARTBEAT *) 
				  (vShmemStart + sizeof(SYS_INIT_INFO));
      vRteHealth    = (volatile HEALTH *) (vHbLocation + NRTES);       
      vRteInitType  = (volatile INIT_TYPE *) (vRteHealth + NRTES);

          /* Initialize init server portion of SYS_INIT_INFO */
      vSysInitInfoG->iServerRte = INIT_SERVER_RTE;

      iInitHeaderSize =  sizeof(SYS_INIT_INFO)
            + NRTES * sizeof(HEARTBEAT)
            + NRTES * sizeof(HEALTH)
            + NRTES * sizeof(INIT_TYPE);

      vIpcShmem = ((BYTE *) vShmemStart) + iInitHeaderSize;   
                               /* token area after init hdr  */

      tIpcMemoryBytePointer::SetBaseAddress ((volatile BYTE*) vIpcShmem);
      tIpcMemoryWordPointer::SetBaseAddress ((volatile SHMEM_WORD* ) vIpcShmem);
      
      iIpcBytes = iShmemSize - iInitHeaderSize;

      if (iShmemSize < iInitHeaderSize) return Traceback (
        S_initmgr_INSUFFICIENT_SHMEM_FOR_INIT_HEADERS,"PowerOn");

      /****
      [ Insufficient memory to allocate a tSharedMemory => pSharedMemory := NULL
      | true => pSharedMemory := address of new tSharedMemory
      ]
      
      [[U pSharedMemory->tSharedMemory::tSharedMemory (vIpcShmem, 
          iIpcBytes, NRTES) ]]
      ****/      
      pSharedMemory = new tSharedMemory ( (volatile SHMEM_WORD *) vIpcShmem, 
         iIpcBytes, NRTES);
      
      ERROR_CODE ecTemp;
      if (pSharedMemory == NULL) return Traceback (S_initmgr_CANT_CREATE_SHARED_MEMORY,"PowerOn");
      
      ecTemp = pSharedMemory->GetInitError(); 
      if (ecTemp != OK )  return Traceback (ecTemp,"PowerOn");

      printf("Init Server CPU - initializing shared memory... Size = %d\n",
               iShmemSize);

      for (i=0; i< NRTES; i++)
      {
         vHbLocation[i] = 0;
         vRteHealth[i] = H_NOT_INIT;
         vRteInitType[i] = IT_POWER_ON_INIT;
      }

      /****
      [[U pSharedMemory->tSharedMemory::GlobalInit (CiTokenServerTimeOut) ]]
      ****/
      pSharedMemory->GlobalInit (CiTokenServerTimeOut);

      TraceMark(S_initmgr_TOKEN_SERVER_STARTED);

   }

   else      /****** i.e. if we're not the Init Server CPU ******/

   {
      /*****
      [[U ec := (::ConfigManager).tHardware::SoftInit (void)]]
      *****/
      ec = ConfigManager.Hardware.SoftInit();
      if (ec != OK)
      {
        fprintf(stderr, "PowerOn: ConfigManager.Hardware.SoftInit() error\n");
        return ec;
      }

      TraceMark(S_initmgr_HARDWARE_SOFTINIT_SUCCESSFUL);

      /*****
      [[U iShmemSize := (::ConfigManager).tHardware::GetShmemSize (void)]]
      *****/
      iShmemSize = ::ConfigManager.Hardware.GetShmemSize ();

      if (iShmemSize <= 0) return Traceback (S_initmgr_SHMEM_BOARD_HAS_NO_MEMORY,
        "PowerOn");

      /*****
      [[U vShmemStart := (::ConfigManager).tHardware::GetShmemLocalAddress (void)]]
      *****/
      vShmemStart = ConfigManager.Hardware.GetShmemLocalAddress();

      if (vShmemStart == NULL) return Traceback (S_initmgr_INVALID_SHMEM_START_ADDRESS,
        "PowerOn");

      printf("Shared memory local address is 0x%x\n",(unsigned int)vShmemStart);

           /* Initialize pointers to InitMgr shmem structures */
      vSysInitInfoG = (SYS_INIT_INFO *) vShmemStart;
      vHbLocation   = (volatile HEARTBEAT *) 
				  (vShmemStart + sizeof(SYS_INIT_INFO));
      vRteHealth    = (volatile HEALTH *) (vHbLocation + NRTES);       
      vRteInitType  = (volatile INIT_TYPE *) (vRteHealth + NRTES);

          /* Initialize init server portion of SYS_INIT_INFO */
      vSysInitInfoG->iServerRte = INIT_SERVER_RTE;

      iInitHeaderSize =  sizeof(SYS_INIT_INFO)
            + NRTES * sizeof(HEARTBEAT)
            + NRTES * sizeof(HEALTH)
            + NRTES * sizeof(INIT_TYPE);

      vIpcShmem = ((BYTE *) vShmemStart) + iInitHeaderSize;   
  
      tIpcMemoryBytePointer::SetBaseAddress ((volatile BYTE*) vIpcShmem);
      tIpcMemoryWordPointer::SetBaseAddress ((volatile SHMEM_WORD* ) vIpcShmem);
                             
      iIpcBytes = iShmemSize - iInitHeaderSize;

      if (iShmemSize < iInitHeaderSize) return Traceback (
        S_initmgr_INSUFFICIENT_SHMEM_FOR_INIT_HEADERS,"PowerOn");

      TraceMark(S_initmgr_ABOUT_TO_CALL_WAITFORHB);

      printf("Init Client - Waiting for Server heartbeat\n");

      /****
      [[U ec := ::WaitForHB (&vHbLocation[vSysInitInfoG->iServerRte],
                            IM_HB_WAIT_LIMIT) ]]
      ****/
      ec = WaitForHB(&vHbLocation[vSysInitInfoG->iServerRte],IM_HB_WAIT_LIMIT);
      if (ec != OK) 
      {
         fprintf(stderr, "PowerOn: WaitForHB() timed out\n");
         return ec;
      }
   }

   /****** InitMgr Client code (executed on all CPUs): ******/

   TraceMark(S_initmgr_ABOUT_TO_CALL_CLIENTPOWERON);

   /****
   [[U ec := ::ClientPowerOn (vIpcShmem, iIpcBytes, vHbLocation, 
                              vRteHealth, vRteInitType ) ]]
   ****/                              
   ec = ClientPowerOn( (volatile SHMEM_WORD *) vIpcShmem, 
                       iIpcBytes, vHbLocation, 
                       vRteHealth, vRteInitType );
   if (ec != OK)
   {
      fprintf(stderr, "ClientPowerOn did not succeed, ec=0x%x \n",ec);
      return ec;
   }

   TraceMark(S_initmgr_POWERON_COMPLETED_SUCCESSFULLY);

   return OK;

}               /* PowerOn() */

/*****************************************************************************
*
* Function Name: WaitForHB ( volatile HEARTBEAT *vHbLoc, int iTimeLimit )
*
* DESCRIPTION:
  Loops for iTimeLimit seconds, checking the value at vHbLoc; if the
  value increases before the time limit is up, WaitForHB returns OK.
  If the time limit is reached without the value increasing, it returns
  S_initmgr_HB_TIMEOUT
*
  [[U ::WaitForHB (vHbLoc, iTimeLimit) ]]
*/

ERROR_CODE WaitForHB (volatile HEARTBEAT* vHbLoc, int iTimeLimit )
{
   int          iMaxLoops;
   HEARTBEAT    hbStartValue;

   BOOL         bHbPresent = FALSE,
                bTimeOut = FALSE;

   iMaxLoops = iTimeLimit * sysClkRateGet();    /* iTimeLimit in seconds */
   hbStartValue = *vHbLoc;

   while ( !bHbPresent && !bTimeOut )
   {
      taskDelay(1);     /* check out spec on this fn - what about partial ticks? */

      if (*vHbLoc > hbStartValue)
         bHbPresent = TRUE;
      else if ( --iMaxLoops < 0 ) 
         bTimeOut = TRUE;

      hbStartValue = *vHbLoc;

   }

   if (bHbPresent) 
      return OK;
   else
      return Traceback(S_initmgr_HB_TIMEOUT, "WaitForHB", vHbLoc, iTimeLimit);

}       /* WaitForHB() */


/*****************************************************************************
*
* Function Name: ClientPowerOn (volatile SHMEM_WORD* vswIpcShmemStart, 
*                               int iIpcBytes, volatile HEARTBEAT* vHbLocation, 
*                               volatile HEALTH* vRteHealth, 
*                               volatile INIT_TYPE* vRteInitType )
*
* DESCRIPTION:
  Loops for iTimeLimit seconds, checking the value at vHbLoc; if the
  value increases before the time limit is up, WaitForHB returns OK.
  If the time limit is reached without the value increasing, it returns
  S_initmgr_HB_TIMEOUT
*
  [[U ::ClientPowerOn (vswIpcShmemStart, iIpcBytes, vHbLocation, 
                       vRteHealth, vRteInitType ) ]]
*
*/

ERROR_CODE ClientPowerOn( volatile SHMEM_WORD   *vswIpcShmemStart, 
                          int                   iIpcBytes,
                          volatile HEARTBEAT    *vHbLocation, 
                          volatile HEALTH       *vRteHealth, 
                          volatile INIT_TYPE    *vRteInitType )
{
   int iWhichCpu,
       iWhichRte;
   ERROR_CODE ec;
  
   TraceMark(S_initmgr_ENTERING_CLIENTPOWERON);

   /****
   [ iWhichCpu := VxWorks CPU number stored in EEPROM  ]
   ****/     
   iWhichCpu = sysProcNumGet();        

   /* NRTES_ON_CPUx and RteListForCPUx will be ConfigMgr calls later; 
             globals for now */

   // Temporary kludge for CPU0 until kernel is fixed.
   if(iWhichCpu == INIT_SERVER_CPU)   iWhichRte = 0;
   else                               iWhichRte = iWhichCpu;

   /****
   [[U ec := ::LoadAndInitRtes (iWhichRte, NRTES_ON_CPU1, RteListForCPU1,
               vswIpcShmemStart, iIpcBytes,vHbLocation, vRteHealth,
               vRteInitType) ]]
   ****/
   ec = LoadAndInitRtes (iWhichRte, vswIpcShmemStart, iIpcBytes,
                         vHbLocation, vRteHealth, vRteInitType);

   
   TraceMark(S_initmgr_EXITING_CLIENTPOWERON);

   return ec;

}       /* ClientPowerOn() */




/******************************************************************************
*
* Function Name: LoadAndInitRtes (int iWhichCPU, 
*                                 volatile SHMEM_WORD*  vswIpcShmemStart, 
*                                 int iIpcBytes,
*                                 volatile HEARTBEAT* vHbLocation, 
*                                 volatile HEALTH* vRteHealth, 
*                                 volatile INIT_TYPE* vRteInitType)
*
  [[U ::LoadAndInitRtes (iWhichCPU, vswIpcShmemStart, 
                         iIpcBytes, vHbLocation, vRteHealth, 
                         vRteInitType) ]]
*
*/


ERROR_CODE   LoadAndInitRtes ( int iWhichCPU,
                               volatile SHMEM_WORD     *vswIpcShmemStart, 
                               int                     iIpcBytes,
                               volatile HEARTBEAT      *vHbLocation, 
                               volatile HEALTH         *vRteHealth, 
                               volatile INIT_TYPE      *vRteInitType )
{
   MODULE_ID miId;
   ERROR_CODE ec;
   char taskName[256];
   char symName[] = "RteInit";

   TraceMark(S_initmgr_ENTERING_LOADANDINITRTES);


   #ifndef CROSSWIND

      TraceMark(S_initmgr_LOADING_RTE_EXECUTABLE);

      printf("Loading software...");
      miId = ld( 0, FALSE, sRteExecFile [iWhichCPU] );
      if (miId == NULL) 
         return Traceback (errnoGet(), "LoadAndInitRtes", iWhichCPU,
                           vswIpcShmemStart, iIpcBytes, vHbLocation, 
                           vRteHealth, vRteInitType );
      printf("Done!\n");
   #endif /* CROSSWIND */

      TraceMark(S_initmgr_SPAWNING_RTEINIT_TASK);

      printf("Spawning Init Task.\n");
      sprintf(taskName,"Rte[%d].Init()",iWhichCPU);

      /****
      NOTE: iWhichCPU is passed in twice because iRteId = iWhichCPU...
            eventually we will do away with having both numbers.
                 
      [[U ec := ::doSpawn (taskName, RTE_INIT_PRIO, RTE_INIT_OPTS, 
                        RTE_INIT_STKSZ, symName,  iWhichCPU, iWhichCPU,  
                        (int)vswIpcShmemStart, iIpcBytes, (int)vHbLocation, 
                        (int)vRteHealth, (int)vRteInitType, 0, 0, 0) ]]
      ****/                       
      ec = doSpawn (taskName, RTE_INIT_PRIO, RTE_INIT_OPTS, RTE_INIT_STKSZ, 
                        symName,  iWhichCPU, iWhichCPU,  
                        (int)vswIpcShmemStart, iIpcBytes, (int)vHbLocation, 
                        (int)vRteHealth, (int)vRteInitType, 0, 0, 0); 
      if (ec != OK) 
         return Traceback (ec, "LoadAndInitRtes", iWhichCPU, 
                          vswIpcShmemStart,iIpcBytes,
                          vHbLocation, vRteHealth, vRteInitType);


   return OK;

}       /* LoadAndInitRtes() */



/****************************************************************************************
*
* Function Name: doSpawn (char* taskName, int priority, int options, 
*                         int stackSize, char* funcName, int arg1, int arg2, 
*                         int arg3, int arg4, int arg5, int arg6, int arg7, 
*                         int arg8, int arg9, int arg10)
*
* DESCRIPTION:
  Looks up the function whose name is in the string FuncName,
  then calls taskSpawn with the pointer to that function and
  the rest of the arguments to doSpawn.
* 
  [[U ::doSpawn (taskName,priority,options,stackSize,funcName,arg1,arg2, 
                 arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10)  ]]
*
*/

ERROR_CODE doSpawn(char *taskName, int priority, int options, int stackSize, 
                   char *funcName, int arg1, int arg2, int arg3, int arg4, 
                   int arg5, int arg6, int arg7, int arg8, int arg9, int arg10)
{
   ERROR_CODE ec;
   int iTaskId;
   FUNCPTR pInitFunc = NULL; 
   SYM_TYPE stType;

   TraceMark(S_initmgr_ENTERING_DOSPAWN);

   /*****
      [ sysSymTbl is an invalid symbol table ID -> 
          return VxWorks errno (symFindByName)
      | funcName not in sysSymTbl -> return VxWorks errno (symFindByNName)
      | true->
          pInitFunc := pointer to entity whose symbol is funcName
          : ec := OK
      ]
   *****/
   #ifdef CROSSWIND
   pInitFunc = (FUNCPTR) RteInit;
   #else
   ec = symFindByName( sysSymTbl, funcName, (char **)&pInitFunc, &stType );
   if (ec != OK) 
      return Traceback(errnoGet(),"doSpawn", taskName, priority, options,
                       stackSize, funcName, arg1, arg2, arg3, arg4, arg5,
                       arg6, arg7, arg8, arg9, arg10);
   #endif
  

   /*****
   [[U iTaskId := taskLib::taskSpawn (taskName, priority, options, stackSize,
         pInitFunc, arg1,arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, 
         arg10) ]]
   *****/
   iTaskId = taskSpawn (taskName, priority, options, stackSize, pInitFunc, arg1,
                        arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
   if (iTaskId == ERROR) 
      return Traceback(errnoGet(),"doSpawn",taskName,priority,options,stackSize,
                  funcName,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10);
   
   TraceMark(S_initmgr_DOSPAWN_COMPLETED_SUCCESSFULLY);

   return OK;

}               /* doSpawn() */

