/* $RCSfile: instrumentCdsGizmo.cc,v $ - Type definitions for Instrument Gizmo stuff  */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/inst/instGizmo/instrumentCdsGizmo.cc,v 1.2 1998/07/13 22:01:00 phil Exp $
*/
/* Richard Johnson  - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,20Aug97,rlj      addapted to new gizmo architecture
02a,25Apr97,rlj      changed initialize method to take fewer parameters
                     Added calls to command handler initialize and instrument 
                     command processor.
01a,29Oct96,eamck    written
*/
 
/*
DESCRIPTION
  Instrument Gizmo init code, etc.
*/
   

#include "isit.h"
#include "rte.h"
#include "ipcid.h"

#include "configMgr.h"
#include "instrumentCdsGizmoInfo.h"
#include "instrumentCdsGizmo.h"
#include "instCommands.h"
#include "errorStream.h"
#include "dlActuator.h"
#include "board.h"




/****
[[U tInstrumentCDSGizmo::tInstrumentCDSGizmo (GizmoInfo) ]] 
****/
tInstrumentCDSGizmo::tInstrumentCDSGizmo (const tInstrumentCDSGizmoInfo &GizmoInfo) : 

    /****
    [[U tGizmo::tGizmo (GizmoInfo) ]]
    ****/
    tGizmo (GizmoInfo),

    /****
    [[U this->CommandHandler.tCommandHandler::tCommandHandler (void) ]]
    ****/
    CommandHandler (),
       
    
    /****
    [[U this->ExternalInterface.tExternalInterface::
          tExternalInterface (this->CommandHandler) ]]
    ****/
    
    ExternalInterface (CommandHandler),

    /****
    [[U this->Relayer.tRelayer::tRelayer (this->ExternalInterface) ]]
    ****/
   
    Relayer (ExternalInterface),

    pClockBoard (GizmoInfo.pClockBoard) {
   
  ERROR_CODE ecTemp;
  int iDlNum;
  tIpcDblBufferId iiTargetId;
  GIZMO_TYPE gtType;
 
  // Check initialization of base classes and reference variables.   
   
   if (tBaseObject::GetInitError () != OK) return;
   if (this->CommandHandler.GetInitError () != OK) {
      tBaseObject::SetInitError (this->CommandHandler.GetInitError() );
      return;
   }

   if (this->ExternalInterface.GetInitError () != OK) {
      tBaseObject::SetInitError (this->ExternalInterface.GetInitError() );
      return;
   }

   if (this->Relayer.GetInitError () != OK) {
      tBaseObject::SetInitError (this->Relayer.GetInitError() );
      return;
   }

   // Continue construction of Instrument CDS

   /****
   [[U this->iNumberOfDelayLines := ConfigManager.tConfig::NumberOfDelayLines (void) ]]
   ****/
   this->iNumberOfDelayLines = ConfigManager.NumberOfDelayLines ();


   // Initialize manual and open loop targets

   this->pManualTargetArray = new (tDlActuator *) [iNumberOfDelayLines];
   
   if (this->pManualTargetArray == NULL) {
     TraceMsg (tmt_ERROR,S_instcds_CANT_ALLOCATE_TARGET_BUFFERS,
               "tInstrumentCDSGizmo::tInstrumentCDSGizmo ()" );
     SetInitError (S_instcds_CANT_ALLOCATE_TARGET_BUFFERS);
     return;
   }

   this->pOpenLoopTargetArray = new (tDlActuator *) [iNumberOfDelayLines];
   
   if (this->pOpenLoopTargetArray == NULL) {
     TraceMsg (tmt_ERROR,S_instcds_CANT_ALLOCATE_TARGET_BUFFERS,
               "tInstrumementCDSGizmo::tInstrumentCDSGizmo ()" );
     tBaseObject::SetInitError (S_instcds_CANT_ALLOCATE_TARGET_BUFFERS);
     return;
   }

   this->iDlActuatorMap = new int [iNumberOfDelayLines];
 
   for (iDlNum = 0; iDlNum < this->iNumberOfDelayLines; iDlNum++) {

    /**** 
    [[U iiTargetId := GizmoInfo.tInstrumentCDSGizmoInfo::
          GetManualTargetId (iDlNum) ]]
    ****/
    iiTargetId = GizmoInfo.GetManualTargetId (iDlNum);

    /****
    [ cannot allocate new tDlTargetActuator => 
        console := console + appropriate error message
        : this->ecInitError := appropriate error message
    | true =>  this->pManualTargetArray[iDlNum] := address of new tDlTargetActuator
    ]
    
    [[U this->pManualTargetArray[iDlNum]->tDlActuator::
           tDlActuator (iiTargetId)]]
    ****/
    this->pManualTargetArray[iDlNum] = new tDlActuator (iiTargetId);
    
    if (this->pManualTargetArray[iDlNum] == NULL) {
      TraceMsg (tmt_ERROR,S_instcds_CANT_ALLOCATE_TARGET_BUFFERS,
                "tInstrumentCDSGizmo::tInstrumentCDSGizmo ()" );
      this->SetInitError(S_instcds_CANT_ALLOCATE_TARGET_BUFFERS);
      return;

    }
      
    /**** 
    [[U iiTargetId := GizmoInfo.tInstrumentCDSGizmoInfo::
            GetOpenLoopTargetId (iDlNum) ]]
    ****/
    iiTargetId = GizmoInfo.GetOpenLoopTargetId (iDlNum);
    
    /****
    [ cannot allocate new tDlTargetActuator => 
        console := console + appropriate error message
        : this->ecInitError := appropriate error message
    | true =>  this->pOpenLoopTargetArray[iDlNum] := address of new tDlTargetActuator
    ]
    
    [[U this->pOpenLoopTargetArray[iDlNum]->tDlActuator::tDlActuator (iiTargetId)]]
    ****/

    this->pOpenLoopTargetArray[iDlNum] = new tDlActuator (iiTargetId);
    
    if (this->pOpenLoopTargetArray[iDlNum] == NULL) {
      TraceMsg (tmt_ERROR,S_instcds_CANT_ALLOCATE_TARGET_BUFFERS,
                "tInstGizmo::tInstGizmo ()" );
      this->SetInitError(S_instcds_CANT_ALLOCATE_TARGET_BUFFERS);
      return;
    }

    /****
    [[U ::GetEmbeddedSystemGizmoInfo (GT_DELAY_LINE,iDlNum,
                                   this->iDlActuatorMap [iDlNum]) ]]
    ****/
    GetEmbeddedSystemGizmoInfo (GT_DELAY_LINE,iDlNum,gtType,
                                   this->iDlActuatorMap [iDlNum]);

  } // for all delay lines
  
   /****
   [[U tGizmo::StartTasks (void) ]]
   ****/
   tGizmo::StartTasks (GizmoInfo);

 
   //Kludge stuff for Sequencer

   this->pSeqCallback = NULL;
   this->iDlIdForCallback = INVALID_GIZMO;



   printf("Instrument Gizmo Construction complete.\n");

}	/* tInstGizmo::Initialize() */


/****
[[U tInstrumentCDSGizmo::~tInstrumentCDSGizmo (void) ]]
****/
tInstrumentCDSGizmo::~tInstrumentCDSGizmo () {

  int iDlNum;

  if (tBaseObject::IsInit() ) {
    TraceMsg (tmt_WARN,S_instcds_GIZMO_NOT_SHUT_DOWN,
              "tInstrumentCDSGizmo::~tInstrumentCDSGizmo");

    /****
    [[U this->tInstrumentCDSGizmo::Shutdown (void) ]]
    ****/
    this->Shutdown ();
  }

  for (iDlNum = 0; iDlNum < this->iNumberOfDelayLines; iDlNum++) {

       /****
       [[U this->pOpenLoopTargetArray[iDlNum]->tDlActuator::~tDlActuator (void) ]]
       ****/
     delete this->pOpenLoopTargetArray[iDlNum];

       /****
       [[U this->pManualTargetArray[iDlNum]->tDlActuator::~tDlActuator (void) ]]
       ****/
     delete this->pManualTargetArray[iDlNum];
  }

  delete[] this->pOpenLoopTargetArray;
  delete[] this->pManualTargetArray;
  delete[] this->iDlActuatorMap;

  /****
  [[U this->Relayer.tRelayer::~tRelayer (void) ]]
  [[U this->ExternalInterface.tExternalInterface::~tExternalInterface (void) ]]
  [[U this->CommandHandler.tCommandHandler::~tCommandHandler (void) ]]
  [[U this->tGizmo::~tGizmo (void) ]]
  ****/ 

}

/****
[[U tInstrumentCDSGizmo::Restart (void) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::Restart () {


  // IMPROVEMENT :: DO WE WANT TO BAIL IF A SINGLE ELEMENT WON'T RESTART
  
  ERROR_CODE ecTemp;

  if (tBaseObject::IsInit() ) {
    TraceMsg (tmt_WARN,S_instcds_GIZMO_ALREADY_INITIALIZED,"tInstrumentCDSGizmo::Restart");
    return OK;
  }

  /****
  [[U ecTemp := this->CommandHandler.tCommandHandler::Restart (void) ]]
  ****/
  ecTemp = this->CommandHandler.Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Restart");

  /****
  [[U ecTemp := this->ExternalInterface.tExternalInterface::Restart (void) ]]
  ****/
  ecTemp = this->ExternalInterface.Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Restart");

  /****
  [[U ecTemp := this->Relayer.tRelayer::Restart (void) ]]
  ****/
  ecTemp = this->Relayer.Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Restart");


  /****
  [[U ecTemp := this->tGizmo::Restart (void) ]]
  ****/
  ecTemp = tGizmo::Restart ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Restart");
  
  tBaseObject::SetInit (TRUE);
  return OK;
}
  

/****
[[U tInstrumentCDSGizmo::Shutdown (void) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::Shutdown () {
  
  // IMPROVEMENT :: DO WE WANT TO BAIL IF A SINGLE ELEMENT WON'T SHUTDOWN

  ERROR_CODE ecTemp;

  if (!tBaseObject::IsInit() ) {
    TraceMsg (tmt_WARN,S_instcds_GIZMO_ALREADY_SHUTDOWN,"tInstrumentCDSGizmo::Shutdown");
    return OK;
  }

  /****
  [[U ecTemp := this->tGizmo::Shutdown (void) ]]
  ****/
  ecTemp = tGizmo::Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Shutdown");

  /****
  [[U ecTemp := this->CommandHandler.tCommandHandler::Shutdown (void) ]]
  ****/
  ecTemp = this->CommandHandler.Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Shutdown");

  /****
  [[U ecTemp := this->ExternalInterface.tExternalInterface::Shutdown (void) ]]
  ****/
  ecTemp = this->ExternalInterface.Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Shutdown");

  /****
  [[U ecTemp := this->Relayer.tRelayer::Shutdown (void) ]]
  ****/
  ecTemp = this->Relayer.Shutdown ();

  if (ecTemp != OK) return Traceback (ecTemp,"tInstrumentCDSGizmo::Shutdown");



  tBaseObject::SetInit (FALSE);

  return OK;
}


/****
[[U tInstrumentCDSGizmo::IsOk (void) ]]
****/
BOOL tInstrumentCDSGizmo::IsOk () {


  
    /****
    [[U this->CommandHandler.tCommandHandler::IsOk (void) ]]
    ****/
  if (this->CommandHandler.IsOk() == FALSE) return FALSE;

    /****
    [[U this->Relayer.tRelayer::IsOk (void) ]]
    ****/
  if (this->Relayer.IsOk() == FALSE) return FALSE;

    /****
    [[U this->ExternalInterface.tExternalInterface::IsOk (void) ]]
    ****/
  if (this->ExternalInterface.IsOk() == FALSE) return FALSE;

  return TRUE;
}



/****
[[U tInstrumentCDSGizmo::Dispatch (Command) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::Dispatch (const tIsitCmd& Command) { 

  ERROR_CODE ecTemp;

  switch (Command.Command) {

    case INST_CMD_PING : 
 
      /****
      [[U ecTemp := this->tInstrumentCDSGizmo::Ping (void) ]]
      ****/
      ecTemp = this->Ping();break;

    case INST_CMD_SET_MANUAL_TARGET : 

      /****
      [[U ecTemp := this->tInstrumentCDSGizmo::SetManualTarget (Command.instsmtCmd.iWhichDl,
          Command.instsmtCommand.Target) ]]
      ****/      
      ecTemp = this->SetManualTarget ((tInstSetManualTargetCmd&) Command);break; 

    case INST_CMD_SET_OPEN_LOOP_TARGET : 

      /****
      [[U ecTemp := this->tInstrumentCDSGizmo::SetOpenLoopTarget (Command.instsoltCmd.iWhichDl,
          Command.instsoltCommand.Target) ]]
      ****/
      ecTemp = this->SetOpenLoopTarget ((tInstSetOpenLoopTargetCmd&) Command);break;

    case INST_CMD_SET_TIME :

      /****
      [[U ecTemp := this->tInstrumentCDSGizmo::SetTime (Command)]]
      ****/
      ecTemp = this->SetTime ((tInstSetTimeCmd&) Command);  break;

    default : ecTemp = S_instcds_UNRECOGNIZED_COMMAND;
  };
  

  return ecTemp; 
}

/****
[[U tInstrumentCDSGizmo::CollectStatus (void) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::CollectStatus () {
  return OK;
}


ERROR_CODE tInstrumentCDSGizmo::Ping () {

  /****
  [[U return := tGizmo::SendError (S_ping_INSTRUMENT_CDS_ALIVE) ]]
  ****/  
  return tGizmo::SendError (S_ping_INSTRUMENT_CDS_ALIVE);
}


/****
[[U tInstrumentCDSGizmo::SetOpenLoopTarget (Command.genericCmd.iWhichDl,
  Command.genericCmd.Target) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::SetOpenLoopTarget (
  const tInstSetOpenLoopTargetCmd& Command) {

  ERROR_CODE ecTemp;
  int iDlNum;

  iDlNum = WhichDlActuator (Command.iDlGizmoId);


  if (iDlNum == INVALID_GIZMO)
    return Traceback (S_instcds_INVALID_DELAY_LINE_ID, 
                      "tInstrumentCDSGizmo::SetOpenLoopTarget"); 
 
  this->pOpenLoopTargetArray[iDlNum]->SetTargetValue ( (tDlTarget &) Command.Target);  


  /****
  [[U ecTemp := this->pOpenLoopTargetArray[Command->iWhichDl]->
      tDlActuator::Output ((::pThisRteG)->SchedTime  ) ]]
  ****/
  ecTemp = this->pOpenLoopTargetArray [iDlNum]->Output (::GetTime ());  

  if (ecTemp != OK)
    return Traceback (ecTemp,"tInstrumentCDSGizmo::OpenLoopTargetCommand");

  /****
  [[U return := tGizmo::SendError (S_instcds_OPEN_LOOP_TARGET_SET) ]]
  ****/
  return tGizmo::SendError (S_instcds_OPEN_LOOP_TARGET_SET);

}  


/****
[[U tInstrumentCDSGizmo::SetTime (Command.genericCmd.iWhichDl,
  Command.genericCmd.Target) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::SetTime (const tInstSetTimeCmd& Command) {

  ERROR_CODE ecTemp;

  ecTemp = this->pClockBoard->SetTime(Command.iSecondsSinceZero);

  /****
  [[U return := tGizmo::SendError (S_instcds_TIME_SET) ]]
  ****/
  return tGizmo::SendError (S_instcds_TIME_SET);

}


/****
[[U tInstrumentCDSGizmo::SetManualTarget (Command.genericCmd.iWhichDl,
  Command.genericCmd.Target) ]]
****/
ERROR_CODE tInstrumentCDSGizmo::SetManualTarget (
  const tInstSetManualTargetCmd& Command) {

  ERROR_CODE ecTemp;
  int iDlNum;

  iDlNum = WhichDlActuator (Command.iDlGizmoId);

  if (iDlNum == INVALID_GIZMO)
    return Traceback (S_instcds_INVALID_DELAY_LINE_ID, 
                      "tInstrumentCDSGizmo::SetManualTarget"); 

  this->pManualTargetArray [iDlNum]->SetTargetValue ( (tDlTarget &) Command.Target);  

  /****
  [[U ecTemp := this->pManualTargetArray [iDlNum]->
          tDlActuator::Output ((::pThisRteG)->SchedTime) ]]
  ****/
  ecTemp = this->pManualTargetArray [iDlNum]->Output (::GetTime ());  

  if (ecTemp != OK)
    return Traceback (ecTemp,"tInstrumentCDSGizmo::ManualTargetCommand");
 
  /****
  [[U return := tGizmo::SendError (S_instcds_MANUAL_TARGET_SET) ]]
  ****/
  return tGizmo::SendError (S_instcds_MANUAL_TARGET_SET);

}  


// Determine which actuator goes with which delay line gizmo id

int tInstrumentCDSGizmo::WhichDlActuator (int iGizmoId) {

  int i = 0;

  while (this->iDlActuatorMap [i] != iGizmoId )  {
    if (i == this->iNumberOfDelayLines) return INVALID_GIZMO;
    i++;
  }

  return i;
} 


// Kludge for Sequncer

ERROR_CODE tInstrumentCDSGizmo::SetSeqCallback (int iWhichDl, 
         void (* pSeqCallback) (BOOL bFrontSoft, BOOL bFrontHard, 
                                BOOL bBackSoft, BOOL bBackHard, BOOL bHome, 
                                LASER_VALUE lvLaserValue) )
{
  return this->Relayer.SetSeqCallback (iWhichDl,pSeqCallback);
}


void tInstrumentCDSGizmo::DoCommand (const ISIT_CMD& Command) 
{
  this->CommandHandler.Put (Command);
}

/********************************************************************
    "Roll Your Own" RICST commands -- helper functions below:
 ********************************************************************/

/********************************************************************
* ERROR_CODE MakeRicstCmd(...) -- assembles RICST command and sends it
*                                      directly to CmdHdlr
*
*  (for today, it can only handle commands with up to one argument;
*        later, it will be able to do any valid RICST command)
*
*
*****/
ERROR_CODE MakeRicstCmd(CMD_TYPE ctWhatCommand, GIZMO_TYPE gtWhatGizmo, 
			       int iGuiGizmoIndex, void *pArgument)
{
   ERROR_CODE ec;
   ISIT_CMD icTheCommand;
   int iGizmoId;
   GIZMO_TYPE gtType;

   tInstrumentCDSGizmo* pInstGizmo =  
     ( tInstrumentCDSGizmo *) GetGizmoPointer(CiInst);

   GetEmbeddedSystemGizmoInfo(gtWhatGizmo,iGuiGizmoIndex,gtType,iGizmoId);

   if (pInstGizmo==NULL)
   {
      TraceMsg(tmt_ERROR,ERROR,"No InstGizmo on this RTE");
      return ERROR;
   }

   switch (ctWhatCommand)
   {
      case DL_CMD_PING:
         icTheCommand.dlpCmd.Command = ctWhatCommand;
         icTheCommand.dlpCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_OFF:
         icTheCommand.dloCmd.Command = ctWhatCommand;
         icTheCommand.dloCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_FRONT:
         icTheCommand.dlfCmd.Command = ctWhatCommand;
         icTheCommand.dlfCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_BACK:
         icTheCommand.dlbCmd.Command = ctWhatCommand;
         icTheCommand.dlbCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_TRACK:
         icTheCommand.dltCmd.Command = ctWhatCommand;
         icTheCommand.dltCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_HOME_SET:
         icTheCommand.dlhsCmd.Command = ctWhatCommand;
         icTheCommand.dlhsCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_CLR_LASER_CTR:
         icTheCommand.dlclcCmd.Command = ctWhatCommand;
         icTheCommand.dlclcCmd.iGizmoId = iGizmoId;
	 break;

      case DL_CMD_SELECT_TARGET:  // note this can only turn a target ON 
         icTheCommand.dlsatCmd.Command = ctWhatCommand;
         icTheCommand.dlsatCmd.iGizmoId = iGizmoId;
         icTheCommand.dlsatCmd.ttWhichTarget = *((TARGET_TYPE *)pArgument);
         icTheCommand.dlsatCmd.bTargetSetActive = TRUE;
	 break;

      case INST_CMD_SET_OPEN_LOOP_TARGET:
         icTheCommand.instsoltCmd.Command = ctWhatCommand;
         icTheCommand.instsoltCmd.iGizmoId = iGizmoId;
         icTheCommand.instsoltCmd.iDlGizmoId = iGuiGizmoIndex;
         icTheCommand.instsoltCmd.Target = *((tDlTargetBase *)pArgument);
	 break;

      default:
         TraceMsg(tmt_ERROR,ERROR,
	    "MakeRicstCmd: Cannot handle command (yet): %d", ctWhatCommand);
         return ERROR;
   }

   pInstGizmo->DoCommand(icTheCommand);
   return OK;
  
}  /* MakeRicstCmd() */


extern "C"
{
   ERROR_CODE SetCallbackFunction( int iWhichDl, 
			           void (*pFunc)(BOOL bFrontSoft, 
		                                 BOOL bFrontHard, 
					         BOOL bBackSoft, 
					         BOOL bBackHard, 
					         BOOL bHome, 
					         LASER_VALUE lvValue) );
   void DlOff(int iWhichDl);
   void DlFront(int iWhichDl);
   void DlBack(int iWhichDl);
   void DlTrack(int iWhichDl);
   void DlHomeSet(int iWhichDl);
   void DlClrLaser(int iWhichDl);
   void DlActivateOpenLoopTarget(int iWhichDl);
   void DlSetOpenLoopTarget(int iWhichDl, double dInitPos, 
					  double dRampRate, 
					  TIME tInitTime);
}

/********************************************************************
* ERROR_CODE SetCallbackFunction( int iWhichDl,
			          void (*pFunc)(BOOL bFrontSoft,
					        BOOL bFrontHard, 
					        BOOL bBackSoft, 
					        BOOL bBackHard,
					        BOOL bHome, 
					        LASER_VALUE lvValue) )
*
*                 -- sets DL# and function pointer so that when
*                      relayer gets the right DL status block it
*                      calls the function with the proper arguments
*
*****/
ERROR_CODE SetCallbackFunction(int iWhichDl, 
                               void (*pFunc)(BOOL bFrontSoft,
                                             BOOL bFrontHard,
                                             BOOL bBackSoft,
                                             BOOL bBackHard,
                                             BOOL bHome,
                                             LASER_VALUE lvValue) )
{
   ERROR_CODE ec = OK;
   tInstrumentCDSGizmo* pInstGizmo =  
     ( tInstrumentCDSGizmo *) GetGizmoPointer(CiInst);

   if (pInstGizmo==NULL)
   {
      TraceMsg(tmt_ERROR,ERROR,
                "SetCallbackFunction: No InstGizmo on this RTE");
      return ERROR;
   }

   ec = pInstGizmo->SetSeqCallback(iWhichDl,pFunc);

   if (ec !=OK)
      return Traceback(ec,"SetCallbackFunction",iWhichDl,pFunc);

   return OK;
   
}  /* SetCallbackFunction() */

/********************************************************************
* void DlOff(int iWhichDl) -- calls MakeRicstCmd to send OFF
*                                  command to DL #iWhichDl
*
*****/
void DlOff(int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_OFF,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlOff() */

/********************************************************************
* void DlFront(int iWhichDl) -- calls MakeRicstCmd to send FRONT
*                                  command to DL #iWhichDl
*
*****/
void DlFront(int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_FRONT,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlFront() */

/********************************************************************
* void DlBack(int iWhichDl) -- calls MakeRicstCmd to send BACK
*                                  command to DL #iWhichDl
*
*****/
void DlBack(int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_BACK,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlBack() */

/********************************************************************
* void DlTrack(int iWhichDl) -- calls MakeRicstCmd to send TRACK
*                                  command to DL #iWhichDl
*
*****/
void DlTrack(int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_TRACK,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlTrack() */

/********************************************************************
* void DlHomeSet(int iWhichDl) -- calls MakeRicstCmd to send Home Set 
*                                  command to DL #iWhichDl
*
*****/
void DlHomeSet (int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_HOME_SET,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlHomeSet() */

/********************************************************************
* void DlClrLaser(int iWhichDl) -- calls MakeRicstCmd to send ClrLaserCtr
*                                  command to DL #iWhichDl
*
*****/
void DlClrLaser(int iWhichDl)
{
   ERROR_CODE ec = MakeRicstCmd(DL_CMD_CLR_LASER_CTR,GT_DELAY_LINE,iWhichDl,NULL);   
   
}  /* DlClrLaser() */

/********************************************************************
* void DlActivateOpenLoopTarget(int iWhichDl) -- calls MakeRicstCmd 
*               to activate the OpenLoop Target for DL #iWhichDl
*
*****/
void DlActivateOpenLoopTarget(int iWhichDl)
{
   ERROR_CODE ec = OK;
   TARGET_TYPE ttTargetType = TT_OPEN_LOOP;

   ec = MakeRicstCmd(DL_CMD_SELECT_TARGET,GT_DELAY_LINE,iWhichDl,
					      &ttTargetType);
   
}  /* DlActivateOpenLoopTarget() */

/********************************************************************
* void DlSetOpenLoopTarget(int iWhichDl, 
			   double dInitPos, double dRampRate, TIME tInitTime) 
*
*       -- calls MakeRicstCmd to send SetOpenLoopCommand command to 
*		    DL #iWhichDl with target (dInitPos, dRampRate, tInitTime) 
*
*****/
void DlSetOpenLoopTarget(int iWhichDl,
			 double dInitPos, double dRampRate, TIME tInitTime) 
{
   ERROR_CODE ec = OK;
   tDlTargetBase tbTheTarget;

   tbTheTarget.dPosition = dInitPos;
   tbTheTarget.dRate = dRampRate;
   tbTheTarget.tTime = tInitTime;
   
   ec = MakeRicstCmd(INST_CMD_SET_OPEN_LOOP_TARGET,GT_DELAY_LINE,iWhichDl,
							  &tbTheTarget);
   
}  /* DlSetOpenLoopTarget() */

