/* $RCSfile: relayer.cc,v $ - Relayer routines */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/inst/relayer/relayer.cc,v 1.4 1998/09/23 23:57:35 phil Exp $
*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
04a,20Aug97,rlj      updated relayer for new instrument cds
02b,28May97,rlj      fixed relay to attach data to packet header inside the 
	              while loop
02a,26Apr97,rlj      modified to handle increment 2 gizmos.  This involved :
                     using loops to create all error fifos for all rtes, 
                     data and status fifos/buffers for all
                     types of gizmos,and modifying the SendData/Status/ToUI 
                     so that handles more than just delay line gizmos.
01f,28Oct96,eamck    split Relayer.cc into Relayer.cc and relayStreams.cc
01e,17Oct96,eamck    changed ERROR_INFO, added two new Puts and a new PutNoWait
01d,16Oct96,eamck    put DL-specific stuff in separate file
01c,09Oct96,eamck    added client and server streams
01b,07Oct96,eamck    added data & status streams
01a,02Oct96,eamck    written 
*/
      
/*
DESCRIPTION
  ISIT Relayer code
*/


#include "isit.h"
#include "relayer.h"
#include "configMgr.h"
#include "Scheduler.h"
#include "ipcid.h"
#include "extInt.h"
#include "errorStream.h"
#include "relayStream.h"
#include "ipcError.h"
#include "typeTag.h"


/******************************************************************************
*
* Global variable declarations
*/

extern BOOL bUIAttached;




/****
[[U tRelayer::tRelayer (ExternalInterface)]]
****/
tRelayer::tRelayer (tExternalInterface& ExternalInterfaceToUse) :

   /****
   [[U tBaseObject::tBaseObject (void) ]]
   ****/
   tBaseObject (),
   
   ExternalInterface (ExternalInterfaceToUse) {

   ERROR_CODE ecTemp;
   TIME tPeriod,tOffset;
   int iRteNum;
   size_t szRelayBufferSize;
   
   tIpcFifoId iiErrorStreamId;
   tIpcFifoId iiDataStreamId;
   tIpcFifoId iiStatusStreamId;
   
   TraceMark(S_relayer_ENTERING_RELAYER_INIT);

   if (tBaseObject::GetInitError  () != OK) return;

   /****
   [[U this->iNumberOfRtes := ConfigManager.tConfig::NumberOfRtes (void) ]]
   ****/
   this->iNumberOfRtes = ConfigManager.NumberOfRtes ();     

   
   // Allocate array of error stream pointers
 
   this->pErrorStreamArray = new (tErrorStreamReceiver *) [this->iNumberOfRtes];
   
   if (this->pErrorStreamArray == NULL) {
     tBaseObject::SetInitError (S_relayer_CANT_ALLOCATE_ERROR_STREAMS);
     TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_ERROR_STREAMS,
               "tRelayer::tRelayer()");
     return;
   } 
     
   // Allocate array of data stream pointers
   
   this->pDataStreamArray = new (tRelayStreamReceiver *) [this->iNumberOfRtes];
 
   if (this->pDataStreamArray == NULL) {
     tBaseObject::SetInitError (S_relayer_CANT_ALLOCATE_DATA_STREAMS);
     TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_DATA_STREAMS,
               "tRelayer::tRelayer()");
     return;
   } 
 

   // Allocate array of status stream pointers

   this->pStatusStreamArray = new (tRelayStreamReceiver *) [this->iNumberOfRtes];
   
   if (this->pStatusStreamArray  == NULL) {
     this->SetInitError (S_relayer_CANT_ALLOCATE_STATUS_STREAMS);
     TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_STATUS_STREAMS,
               "tRelayer::tRelayer()");
     return;
   }

   for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {

     /****
     [[U iiErrorStreamId := ConfigManager.tConfig::GetErrorStreamId (iRteNum) ]]
     ****/
     iiErrorStreamId = ConfigManager.GetErrorStreamId (iRteNum);
     
     /****
     [[U this->pErrorStreamArray[iRteNum]->tErrorStreamReceiver::tErrorStreamReceiver (iiErrorStreamId) ]]
     ****/
     this->pErrorStreamArray[iRteNum] = 
       new tErrorStreamReceiver (iiErrorStreamId);
     
     if (this->pErrorStreamArray[iRteNum] == NULL) {
       this->SetInitError (S_relayer_CANT_ALLOCATE_ERROR_STREAMS);
       TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_ERROR_STREAMS,
                 "tRelayer::tRelayer()");
       return;
     }
     
     ecTemp = this->pErrorStreamArray[iRteNum]->GetInitError ();
     if (ecTemp != OK ) {
       this->SetInitError (ecTemp);
       TraceMsg (tmt_ERROR,ecTemp,"tRelayer::tRelayer()");
       return;
     }

     /****
     [[U iiDataStreamId := ConfigManager.tConfig::GetDataStreamId (iRteNum) ]]
     ****/
     iiDataStreamId = ConfigManager.GetDataStreamId (iRteNum);
     
     /****
     [[U this->pDataStreamArray[iRteNum]->tRelayStreamReceiver::tRelayStreamReceiver (iiDataStreamId) ]]
     ****/
     this->pDataStreamArray[iRteNum] = new tRelayStreamReceiver (iiDataStreamId);
     
     if (this->pDataStreamArray[iRteNum] == NULL) {
       this->SetInitError (S_relayer_CANT_ALLOCATE_DATA_STREAMS);
       TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_DATA_STREAMS,
                 "tRelayer::tRelayer()");
       return;
     }
     
     ecTemp = this->pDataStreamArray[iRteNum]->GetInitError ();
     if (ecTemp != OK ) {
       this->SetInitError (ecTemp);
       TraceMsg (tmt_ERROR,ecTemp,"tRelayer::tRelayer()");
       return;
     }
   
     /****
     [[U iiStatusStreamId := ConfigManager.tConfig::GetStatusStreamId (iRteNum) ]]
     ****/
     iiStatusStreamId = ConfigManager.GetStatusStreamId (iRteNum);
     
     /****
     [[U this->pStatusStreamArray[iRteNum]->tRelayStreamReceiver::tRelayStreamReceiver (iiStatusStreamId) ]]
     ****/
     this->pStatusStreamArray[iRteNum] = new tRelayStreamReceiver (iiStatusStreamId);
     
     if (this->pStatusStreamArray[iRteNum] == NULL) {
       this->SetInitError (S_relayer_CANT_ALLOCATE_STATUS_STREAMS);
       TraceMsg (tmt_ERROR,S_relayer_CANT_ALLOCATE_STATUS_STREAMS,
                 "tRelayer::tRelayer()");
       return;
     }
  
     ecTemp = this->pStatusStreamArray[iRteNum]->GetInitError ();
     if (ecTemp != OK ) {
       this->SetInitError (ecTemp);
       TraceMsg (tmt_ERROR,ecTemp,"tRelayer::tRelayer()");
       return;
     }   
   }  // for all Rtes
   
   TraceMark(S_relayer_INITIALIZED_ERROR_DATA_AND_STATUS_STREAMS);

   szRelayBufferSize = ConfigManager.GetRelayBufferSize ();
   
   this->pRelayBuffer = new BYTE [szRelayBufferSize];
   
   if (this->pRelayBuffer == NULL) {
       this->SetInitError (S_relayer_INTERNAL_ERROR);
       TraceMsg (tmt_ERROR,S_relayer_INTERNAL_ERROR,"tRelayer::tRelayer()");
       return;
   }
                 
   /****
   [[U (::pThisRteG)->Scheduler.tScheduler::AddTask (&this->iRelayerTaskId, "Relayer",
                            this->Relay, NULL,  NULL, this,
                            CiRelayerPriority, &tPeriod, &tOffset,
                            TEB_CONTINUE, TRUE ) ]]
   *****/

   tPeriod = CtRelayerPeriod;
   tOffset = CtRelayerOffset;
   ecTemp = ::GetScheduler().AddTask(&this->iRelayTaskId, "Relayer",
                                 (FUNCPTR) &(this->Relay),
                                 (FUNCPTR)NULL,  (FUNCPTR)NULL, (void *) this,
                                 CiRelayerPriority, &tPeriod, &tOffset,
                                 TEB_CONTINUE, TRUE );
   

   if (ecTemp != OK) {
       this->SetInitError (ecTemp);
       TraceMsg (tmt_ERROR,ecTemp,"tRelayer::tRelayer()");
       return;
     }
   
   
  
  TraceMark(S_relayer_LEAVING_RELAYER_INIT);

  
}     

/****
[[U tRelayer::~tRelayer (void) ]]
****/
tRelayer::~tRelayer () {

  int iRteNum;

  if (tBaseObject::IsInit() == TRUE) {
    TraceMsg (tmt_WARN,S_relayer_NOT_SHUTDOWN,"tRelayer::~tRelayer");
  
    /****
    [[U this->tRelayer::Shutdown (void) ]]
    ****/
    this->Shutdown ();
  }
          
  /****
  [[U (::pThisRteG)->Scheduler.tScheduler::DeleteTask (this->iRelayTaskId) ]]
  ****/
  ::GetScheduler().DeleteTask (this->iRelayTaskId);
  
  delete[] this->pRelayBuffer;
  
  for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {
    
      /****
      [[U this->pErrorStreamArray[iRteNum]->tErrorStreamReceiver::~tErrorStreamReceiver (void) ]]
      ****/
    delete this->pErrorStreamArray[iRteNum];

      /****
      [[U this->pStatusStreamArray[iRteNum]->tRelayStreamReceiver::~tRelayStreamReceiver (void) ]]
      ****/
    delete this->pStatusStreamArray[iRteNum];
 
      /****
      [[U this->pDataStreamArray[iRteNum]->tRelayStreamReceiver::~tRelayStreamReceiver (void) ]]
      ****/
    delete this->pDataStreamArray[iRteNum];
  } 
  
  delete[] this->pErrorStreamArray;
  delete[] this->pStatusStreamArray;
  delete[] this->pDataStreamArray; 

  /****
  [[U tBaseObject::~tBaseObject (void)]]
  ****/
}


ERROR_CODE tRelayer::Restart () { 

  ERROR_CODE ecTemp;
  int iRteNum;
  
  if (tBaseObject::IsInit() == TRUE) {
    TraceMsg (tmt_WARN,S_relayer_ALREADY_INITIALIZED,"tRelayer::~tRelayer");
    return OK;
  }
  
  tBaseObject::Restart ();
  
  for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {
    
    /****
    [[U ecTemp := this->pErrorStreamArray[iRteNum]->tRelayStreamReceiver::Restart (void) ]]
    ****/
    ecTemp = this->pErrorStreamArray[iRteNum]->Restart ();

    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Restart");
    
    /****
    [[U ecTemp := this->pStatusStreamArray[iRteNum]->tRelayStreamReceiver::Restart (void) ]]
    ****/
    ecTemp = this->pStatusStreamArray[iRteNum]->Restart ();
 
    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Restart");
    
    /****
    [[U ecTemp := this->pDataStreamArray[iRteNum]->tRelayStreamReceiver::Restart (void) ]]
    ****/
    ecTemp = this->pDataStreamArray[iRteNum]->Restart ();

    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Restart");
  } 
   
  /****
  [[U ecTemp := (::pThisRteG)->Scheduler.tScheduler::EnableTask (this->iRelayTaskId) ]]
  ****/ 
  ecTemp = ::GetScheduler().EnableTask (this->iRelayTaskId);
    
  if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Restart");

  return OK;

}

ERROR_CODE tRelayer::Shutdown () { 

  ERROR_CODE ecTemp;
  int iRteNum;
  
  if (tBaseObject::IsInit() == FALSE) {
    TraceMsg (tmt_WARN,S_relayer_ALREADY_SHUTDOWN,"tRelayer::~tRelayer");
    return OK;
  }
  
  tBaseObject::Shutdown ();
  
  /****
  [[U ecTemp := (::pThisRteG)->Scheduler.tScheduler::DisableTask (this->iRelayTaskId) ]]
  ****/ 
  ecTemp = ::GetScheduler().DisableTask (this->iRelayTaskId);
    
  if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Shutdown");


  for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {
    
    /****
    [[U ecTemp := this->pErrorStreamArray[iRteNum]->tRelayStreamReceiver::Shutdown (void) ]]
    ****/
    ecTemp = this->pErrorStreamArray[iRteNum]->Shutdown ();

    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Shutdown");
    
    /****
    [[U ecTemp := this->pStatusStreamArray[iRteNum]->tRelayStreamReceiver::Shutdown (void) ]]
    ****/
    ecTemp = this->pStatusStreamArray[iRteNum]->Shutdown ();
 
    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Shutdown");
    
    /****
    [[U ecTemp := this->pDataStreamArray[iRteNum]->tRelayStreamReceiver::Shutdown (void) ]]
    ****/
    ecTemp = this->pDataStreamArray[iRteNum]->Shutdown ();

    if (ecTemp != OK) return Traceback (ecTemp,"tRelayer::Shutdown");
  } 
   

  return OK;

}

BOOL tRelayer::IsOk () { 

  int iRteNum;

  if (tBaseObject::IsOk() == FALSE) return FALSE;
  
  for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {
    
    /****
    [[U ecTemp := this->pErrorStreamArray[iRteNum]->tIpcObjectAccessor::IsOk (void) ]]
    ****/
    if (this->pErrorStreamArray[iRteNum]->IsOk () == FALSE) return FALSE;

    /****
    [[U ecTemp := this->pStatusStreamArray[iRteNum]->tIpcObjectAccessor::IsOk (void) ]]
    ****/
    if (this->pStatusStreamArray[iRteNum]->IsOk () == FALSE) return FALSE;
 
    /****
    [[U ecTemp := this->pDataStreamArray[iRteNum]->tIpcObjectAccessor::IsOk (void) ]]
    ****/
    if (this->pDataStreamArray[iRteNum]->IsOk () == FALSE) return FALSE;

   } 
   

  return TRUE;

}


/****
This spec currently ignores the loops 
[[U tRelayer::Relay () ]]
****/
ERROR_CODE tRelayer::Relay (void)
{
   ERROR_CODE ecTemp;
   int iRteNum;
   tTypeTag Tag;

   int iNumDataBlocks;
   int iNumStatusBlocks;
   int iNumErrorBlocks;
   int i;
   tDlStatus SeqStatus;

   ULONG uiNumTicks;
      

   TraceMark(S_relayer_TRELAYER_RELAY);

// Read streams

   for (iRteNum = 0; iRteNum < this->iNumberOfRtes; iRteNum++) {
      
#ifndef DO_NOT_SEND_DATA_FROM_RELAYER
     while ( ! this->pDataStreamArray[iRteNum]->IsEmpty () ) {

       /****
       [[U ecTemp := this->pDataStreamArray[iRteNum]->tRelayStreamReceiver::
                Get (Tag,this->pRelayBuffer) ]]
       ****/
       ecTemp = this->pDataStreamArray[iRteNum]->Get(Tag,this->pRelayBuffer);

       if ( (ecTemp != OK) && (ecTemp != S_ipc_FIFO_EMPTY) )  {
          TraceMsg (tmt_ERROR,ecTemp, "tRelayer::Relay DataStream, GizmoId = %d",
                    Tag.GizmoId());
          continue;
       }

       /****
       [[U this->ExternalInterface.tExternalInterface::
            SendData (Tag,this->pRelayBuffer) ]]
       ****/
      this->ExternalInterface.SendData (Tag,this->pRelayBuffer);
     }       
#endif /* DO_NOT_SEND_DATA_FROM_RELAYER */

 
     while ( ! this->pStatusStreamArray[iRteNum]->IsEmpty () ) {

       /****
       [[U ecTemp := this->pStatusStreamArray[iRteNum]->tRelayStreamReceiver::
              Get (Tag,this->pRelayBuffer) ]]
       ****/
       ecTemp = this->pStatusStreamArray[iRteNum]->Get(Tag,this->pRelayBuffer);

       if ( (ecTemp != OK) && (ecTemp != S_ipc_FIFO_EMPTY) )  {
          TraceMsg (tmt_ERROR,ecTemp, "tRelayer::Relay StatusStream, GizmoId = %d", Tag.GizmoId());
          continue;
       }

       /****
       [[U this->ExternalInterface.tExternalInterface::SendStatus (Tag,this->pRelayBuffer) ]]
       ****/
       this->ExternalInterface.SendStatus (Tag,this->pRelayBuffer);
     
       // kludge for sequencer
       // if right gizmo.. do call back.. with right info...

       if (Tag.GizmoId () == this->iDlIdForCallback) { 
	 
         SeqStatus = (tDlStatus) *pRelayBuffer;
         this->pSeqCallback (SeqStatus.bLimitFrontSoft,SeqStatus.bLimitFrontHard,
                             SeqStatus.bLimitBackSoft,SeqStatus.bLimitBackHard,
                             SeqStatus.bOnHomeSensor, SeqStatus.laLaser);
       }
     }
       
     while ( ! this->pErrorStreamArray[iRteNum]->IsEmpty () ) {

       /****
       [[U ecTemp := this->pErrorStreamArray[iRteNum]->tRelayStreamReceiver::
          Get (Tag,this->pRelayBuffer) ]]
       ****/
       ecTemp = this->pErrorStreamArray[iRteNum]->Get(Tag,this->pRelayBuffer);

       if ( (ecTemp != OK) && (ecTemp != S_ipc_FIFO_EMPTY) )  {
          TraceMsg (tmt_ERROR,ecTemp, "tRelayer::Relay ErrorStream, GizmoId = %d", Tag.GizmoId());
          continue;
       }
  
       /****
       [[U this->ExternalInterface.tExternalInterface::
         SendError (Tag,this->pRelayBuffer) ]]
       ****/
       this->ExternalInterface.SendError (Tag, this->pRelayBuffer);
       
     }   

  } // For each RTE
  

  return OK;

}       /* tRelayer::Relay() */


// Kludge for sequencer

ERROR_CODE tRelayer::SetSeqCallback (int iWhichDl, 
         void (* pSeqCallback) (BOOL bFrontSoft, BOOL bFrontHard, 
                                BOOL bBackSoft, BOOL bBackHard, BOOL bHome, 
                                LASER_VALUE lvLaserValue) )
{
  GIZMO_TYPE temp;

  this->pSeqCallback = pSeqCallback;

  GetEmbeddedSystemGizmoInfo (GT_DELAY_LINE,iWhichDl,
                              temp,this->iDlIdForCallback);  
  if (this->iDlIdForCallback == INVALID_GIZMO)   
    return S_setcallback_INVALID_GIZMO;
  return OK;
}
 
