/* $RCSfile: rte.cc,v $ - Generic RTE initialization code */
/* CM Info:   
$Header: /home/chara/cvsroot/ople/sys/inst/rte/rte.cc,v 1.2 1998/08/17 21:01:28 phil Exp $*/
/* Richard Johnson - Jet Propulsion Laboratory */

/*
modification history
--------------------
02b,5,May97,rlj      Static class tConfig now used to configure gizmos and 
	             initialize them.
02a,25Apr97,rlj      Initialize now calls a psuedo config manager to create 
                     all gizmos. Rtes with various gizmos are specified in 
                     the config manager. tRte0,and tRte1 still specify
                     scheduling individual priorities but this may be
                     subject to change.
01d,24Oct96,eamck    updated init calls and their args to current definitions 
01c,17Oct96,eamck    added iWhichCPU, tSchedTime members 
01b,25Sep96,beh      fixed VxWorks/C++ stuff 
01a,25Sep96,eamck    written 
*/
   
/*
DESCRIPTION
  ISIT generic RTE routines
*/

#define INC_rtecc
 
#include "isit.h"
#include "configMgr.h"
#include "rte.h"
#include "hardware.h"
#include "board.h"

#include "gizmoInfo.h"
#include "gizmo.h"

#include "instrumentCdsGizmoInfo.h"
#include "instrumentCdsGizmo.h"

#include "emGizmoInfo.h"
#include "emGizmo.h"

#include "stGizmoInfo.h"
#include "stGizmo.h"

#include "ftGizmoInfo.h"
#include "ftGizmo.h"

#include "gsu/gsuDlGizmoInfo.h"
#include "gsu/gsuDlGizmo.h"

//#include "quic/quicDlGizmoInfo.h"
//#include "quic/quicDlGizmo.h"

#include "ipcid.h"
#include "ipc.h"
#include "Scheduler.h"
#include "errorStream.h"

#include <usrLib.h>

/******************************************************************************
*
* Global variable declarations
*/

tRte ThisRteG;
tRte *pThisRteG = &ThisRteG;




/****
[[U tRte::tRte (void) ]]
****/
tRte::tRte () :

  /****
  [[U tBaseObject::tBaseObject (void)]]
  ****/
  tBaseObject ()
  
  /****
  [[U RteClock.tClock::tClock (void)]]
  ****/
  // tClock removed - PCI 5/8/98
  //RteClock () 

{ }


  
/****
[[U tRte::Init (iWhichCPU,iWhichRTE,vswIpcShmemStart,iIpcBytes,
                vHbLocation,vRteHealth,vRteInitType )]]
****/
ERROR_CODE tRte::Init(int                      iWhichCPU, 
                      int                      iWhichRTE, 
                       volatile SHMEM_WORD      *vswIpcShmemStart, 
                       int                      iIpcBytes,
                       volatile HEARTBEAT       *vHbLocation, 
                       volatile HEALTH          *vRteHealth, 
		       volatile INIT_TYPE       *vRteInitType,
                       BOARD_NAME               bnInterruptBoard,
                       int                      iInterruptPriority ) {


   ERROR_CODE ec;
   TIME tPeriod, tOffset;
   int iGizmoCount;
   int iSchedPriority, iSchedMaxTasks, iSchedRate;
   tIpcFifoId iiErrorStreamId, iiDataStreamId, iiStatusStreamId;
   int iGizmoId;
   
   TraceMark(S_rte_ENTERING_TRTE_INIT);

   // Set up new timestamp function for tTrace
   TraceSetGetTimeFunc(&GetTime);    // BEH
  
   //taskSuspend(0);  // uncomment this line only if you need to use debugger
 

   /****
   [ not enough memory to allocate tIpc => this->pIpc := NULL
   | true => valid_pointer (this->pIpc)
   ]

   [[U this->pIpc.tIpc::tIpc (vswSharedMemoryStart=vswIpcShmemStart,
                  szNumSharedMemoryBytes=iIpcBytes,
                  iNumRtes=NRTES,iWhichRte=iWhichRTE) ]]
   ****/
   this->pIpc = new tIpc(vswIpcShmemStart,iIpcBytes,NRTES,iWhichRTE);

   if (this->pIpc == NULL) 
       return Traceback (S_rte_UNABLE_TO_CREATE_IPC,"tRte::Init");

   /****
   [[U ec := this->pIpc.tBaseObject::GetInitError(void) ]]
   ****/
   ec = this->pIpc->GetInitError();
   if (ec != OK) 
     return Traceback(ec, "tRte::Init" );
     
              
   TraceMark(S_rte_IPC_INITIALIZED);

   /****
   [[U iiErrorStreamId := ConfigManager.tConfig::GetErrorStreamId (iWhichRTE) ]]
   ****/
   iiErrorStreamId =  ConfigManager.GetErrorStreamId (iWhichRTE);
   
   /*****
   [ not enough memory to allocate tErrorStreamSender => 
       this->pErrorStreamSender := NULL
   | true => valid_pointer (this->pErrorStreamSender)
   ]
   [[U tErrorStreamSender::tErrorStreamSender(iiErrorStreamId) ]]
   *****/
   this->pErrorStream = new tErrorStreamSender(iiErrorStreamId);

   if (this->pErrorStream == NULL) 
     return Traceback (S_rte_UNABLE_TO_CREATE_ERROR_STREAM,"tRte::Init");


   /****
   [[U ec := this->pErrorStream.tBaseObject::GetInitError(void) ]]
   ****/
   ec =  this->pErrorStream->GetInitError();
   if (ec != OK) return Traceback (ec,"tRte::Init");

   TraceMsg (tmt_WARN,S_rte_ERROR_STREAM_INITIALIZED,"Error Stream created");
                 
   TraceMark(S_rte_ERROR_STREAM_INITIALIZED);
   
   /****
   [[U iiDataStreamId := ConfigManager.tConfig::GetDataStreamId (iWhichRTE) ]]
   ****/
   iiDataStreamId =  ConfigManager.GetDataStreamId (iWhichRTE);
   
   /*****
   [ not enough memory to allocate tErrorStreamSender => 
       this->pDataStreamSender := NULL
   | true => valid_pointer (this->pDataStreamSender)
   ]
   [[U this->pDataStream->tRelayStreamSender::tRelayStreamSender(iiDataStreamId,SEM_Q_FIFO) ]]
   *****/
   this->pDataStream = new tRelayStreamSender(iiDataStreamId,SEM_Q_FIFO);

   if (this->pDataStream == NULL) 
     return Traceback (S_rte_UNABLE_TO_CREATE_DATA_STREAM,"tRte::Init");


   ec =  this->pDataStream->GetInitError();
   if (ec != OK) return Traceback (ec,"tRte::Init");

                          
   TraceMsg (tmt_WARN,S_rte_DATA_STREAM_INITIALIZED,"Data Stream created");

   TraceMark(S_rte_DATA_STREAM_INITIALIZED);
   
   
   /****
   [[U iiStatusStreamId := ConfigManager.tConfig::GetStatusStreamId (iWhichRTE) ]]
   ****/
   iiStatusStreamId =  ConfigManager.GetStatusStreamId (iWhichRTE);
   
   /*****
   [ not enough memory to allocate tRelayStreamSender => 
       this->pStatusStreamSender := NULL
   | true => valid_pointer (this->pStatusStreamSender)
   ]
   [[U this->pStatusStream->tRelayStreamSender::tRelayStreamSender(     
        iiStatusStreamId, SEM_Q_FIFO) ]]
   *****/
   this->pStatusStream = new tRelayStreamSender(iiStatusStreamId,SEM_Q_FIFO);

   if (this->pStatusStream == NULL) 
     return Traceback (S_rte_UNABLE_TO_CREATE_STATUS_STREAM,"tRte::Init");


   ec =  this->pStatusStream->GetInitError();
   if (ec != OK) 
     return Traceback (ec,"tRte::Init");
                          
   TraceMsg (tmt_WARN,S_rte_STATUS_STREAM_INITIALIZED,
              "Status Stream created");

   TraceMark(S_rte_STATUS_STREAM_INITIALIZED);
   
   
   /*****
   [ not enough memory to allocate tScheduler => 
       this->pScheduler := NULL
   | true => valid_pointer (this->pScheduler)
   ]
   [[U this->pScheduler.tScheduler::tScheduler (void) ]]
   ****/
   this->pScheduler = new tScheduler(); 

   if (this->pScheduler == NULL) 
     return Traceback (S_rte_UNABLE_TO_CREATE_SCHEDULER,"tRte::Initialize");

   /*****
   [[U this->pRteInterruptServer :=
            ConfigManager.Hardware.tHardware::CreateInterrupt(
            bnInterruptBoard, iInterruptPriority) ]]
   *****/
   this->pRteInterruptServer = D_CAST(tPeriodicTimeInterrupt *)
                             ConfigManager.Hardware.CreateInterrupt(
                             bnInterruptBoard, iInterruptPriority); 
   if (this->pRteInterruptServer == NULL) 
     return Traceback (ERROR, "tRte::Init" );

   /*****
   [[U ec := this->pRteInterruptServer->tInterrupt::Initialize ( 
                RteIntNum[iWhichRTE], RteIntVec[iWhichRTE]) ]]
   *****/
   ec = this->pRteInterruptServer->SetIdentity(RteIntNum[iWhichRTE],
                                               RteIntVec[iWhichRTE]);  
   if (ec != OK) 
     return Traceback (ec, "tRte::Init" );

                          
#if 0   // I don't think we need this anymore - PCI 5/8/98
   /*****
   [[U ec := this->RteClock.tClock::Initialize (this->pRteClockBoard) ]]
   *****/
   ec = this->RteClock.Initialize (this->pRteClockBoard );		
   if (ec != OK)  return Traceback (ec, "tRte::Init" );
#endif

   TraceMark(S_rte_INTERRUPT_INITIALIZED);
     
 
// Initialize the heartbeat and health locations

   this->vHeartbeat = vHbLocation;
   this->vHealth = vRteHealth;
   this->iMyRteNum = iWhichRTE;
   this->iMyCpuNum = iWhichCPU;
     
   /****
   [[U iSchedPriority := ConfigManager.tConfig::GetRteSchedPriority (iWhichRTE) ]]
   ****/
   iSchedPriority = ConfigManager.GetRteSchedPriority (iWhichRTE);

   /****
   [[U iSchedMaxTasks := ConfigManager.tConfig::GetRteSchedMaxTasks (iWhichRTE) ]]
   ****/   
   iSchedMaxTasks = ConfigManager.GetRteSchedMaxTasks (iWhichRTE);

   /****
   [[U ConfigManager.tConfig::GetRteSchedRate (iWhichRTE) ]]
   ****/
   iSchedRate = ConfigManager.GetRteSchedRate (iWhichRTE);


   /****
   [[U ec := this->pScheduler.tScheduler::Initialize (iSchedPriority, 
                                   iSchedMaxTasks, iSchedRate,
                                   &(this->tSchedTime), 
			           this->pRteInterruptServer) ]]
   ****/
   ec = this->pScheduler->Initialize(iSchedPriority, 
                                   iSchedMaxTasks, 
			           iSchedRate,
                                   &(this->tSchedTime), 
			           this->pRteInterruptServer );   
   if (ec != OK) 
     return Traceback (ec, "tRte::Init" );
   

   TraceMark(S_rte_SCHEDULER_INITIALIZED);

   tPeriod = CtEkgPeriod;
   tOffset = CtEkgOffset;

   /****
   [[U ec := this->pScheduler.tScheduler::AddTask ( &(this->tidEkgTask),
                                 EKG_TASK_NAME, &(this->EKG), 
                                 NULL,NULL,this,CiEkgPriority, 
                                 &tPeriod,&tOffset,TEB_CONTINUE, 
                                 TRUE) ]]
   ****/   
   ec = this->pScheduler->AddTask( &(this->tidEkgTask),
                                 EKG_TASK_NAME,
                                 (FUNCPTR)&(this->EKG), 
                                 (FUNCPTR)NULL,  
                                 (FUNCPTR)NULL, 
                                 (void *)this, 
                                 CiEkgPriority, 
                                 &tPeriod, 
                                 &tOffset, 
                                 TEB_CONTINUE, 
                                 TRUE );   
   if (ec != OK) 
     return Traceback (ec, "tRte::Init" );

   TraceMsg (tmt_INFO,OK,"Added Ekg");

   tPeriod = CtHbgenPeriod;
   tOffset = CtHbgenOffset;
   
   /****
   [[U ec := this->pScheduler.tScheduler::AddTask ( &(this->tidHbTask),
                                 HB_TASK_NAME, &(this->HbGen), 
                                 NULL,NULL,this,CiHbgenPriority, 
                                 &tPeriod,&tOffset,TEB_CONTINUE, 
                                 TRUE) ]]
   ****/   
   ec = this->pScheduler->AddTask( &(this->tidHbTask),
                                 HB_TASK_NAME,
                                 (FUNCPTR)&(this->HbGen),
                                 (FUNCPTR)NULL,  
                                 (FUNCPTR)NULL, 
                                 (void *)this, 
                                 CiHbgenPriority,  
                                 &tPeriod, 
                                 &tOffset, 
                                 TEB_CONTINUE, 
                                 TRUE );   
   if (ec != OK) 
     return Traceback (ec, "tRte::Init" );


   TraceMsg (tmt_INFO,OK,"Added Heartbeat");

   tPeriod = CtEmtPeriod;
   tOffset = CtEmtOffset;

   /****
   [[U ec := this->pScheduler.tScheduler::AddTask ( &(this->tidEmtTask),
                                 EMT_TASK_NAME, &(this->EMT), 
                                 NULL,NULL,this,CiEmtPriority, 
                                 &tPeriod,&tOffset,TEB_CONTINUE, 
                                 TRUE) ]]
   ****/   
   ec = this->pScheduler->AddTask( &(this->tidEmtTask),
                                 EMT_TASK_NAME,
                                 (FUNCPTR)&(this->EMT), 
                                 (FUNCPTR)NULL,  
                                 (FUNCPTR)NULL, 
                                 (void *)this, 
                                 CiEmtPriority, 
                                 &tPeriod, 
                                 &tOffset, 
                                 TEB_CONTINUE, 
                                 TRUE );  
   if (ec != OK) 
     return Traceback (ec, "tRte::Init" );

   TraceMsg (tmt_INFO,OK,"Added Heartbeat");

   TraceMark(S_rte_HB_EKG_EMT_TASKS_SCHEDULED);


         /*** Now create gizmos that belong on this RTE: ***/

   /****
   [[U iNumberOfGizmos := ConfigManager.tConfig::NumberOfGizmosOnRte (iWhichRTE) ]]
   ****/
   iNumberOfGizmos = ConfigManager.NumberOfGizmosOnRte(iWhichRTE);  // returns # of gizmos

   // allocate iNumberOfGizmos, tGizmo pointers
   /*****
   [ not enough memory to allocate iNumberOfGizmos tGizmo* => 
       this->pGizmoArray := NULL
   | true => valid_pointer (this->pGizmoArray)
   ]
   ****/
   this->pGizmoArray = new (tGizmo *) [iNumberOfGizmos] ;
   
   // for each gizmo pointer on this RTE create appropriate kind of gizmo
   for (iGizmoCount = 0; iGizmoCount < iNumberOfGizmos; iGizmoCount++ ) 
   {
      TraceMark(S_rte_CREATING_NEW_GIZMO);

      iGizmoId = ::RteIndexToGizmoId (iWhichRTE,iGizmoCount);
 
      if (iGizmoId == INVALID_GIZMO) return Traceback (S_rte_INVALID_RTE_GIZMO_INDEX,
          "tRte::Init");
    
      /****
      [[U this->pGizmoArray[iGizmoCount] := tRte::CreateGizmo (iGizmoId) ]] 
      ****/
      this->pGizmoArray[iGizmoCount] = this->CreateGizmo(iGizmoId);

      if (this->pGizmoArray[iGizmoCount] == NULL )  
        return Traceback (S_rte_GIZMO_CREATION_ERROR, "tRte::Init: Gizmo #%d",
                          iGizmoId);

      else 
      {
          /****
          [[U this->pGizmoArray[iGizmoCount].tGizmo::IsOk(void) ]]
          ****/
         if (! this->pGizmoArray[iGizmoCount]->IsOk())
            return 
              Traceback (this->pGizmoArray[iGizmoCount]->GetInitError(),
               "tRte::Init" );

      } // else
   } // for


   /****
   [[U tBaseObject::SetInitError(OK)]]
   ****/
   this->SetInitError(OK);

   TraceMark(S_rte_TRTE_INIT_COMPLETED);

   return OK;
   
}       /* tRte::Init() */


/****
[[U tRte::~tRte (void) ]]
****/
tRte::~tRte () {

  int i;
  
  for (i = 0; i < this->iNumberOfGizmos; i++) {
  
      /****
      [[U tGizmo::~tGizmo (void)]]
      ****/
    delete this->pGizmoArray[i];
  }
  
  delete[] this->pGizmoArray;    

  /****
  [[U pRteInterruptServer->tInterrupt::~tInterrupt (void)]]
  [[U tBaseObject::~tBaseObject (void)]]
  ****/
}

ERROR_CODE tRte::Restart () 
{  
  return this->tBaseObject::Restart ();
}

ERROR_CODE tRte::Shutdown () 
{  
  return this->tBaseObject::Shutdown ();
}

BOOL tRte::IsOk () 
{  
  return this->tBaseObject::IsOk ();
}


ostream& operator<< (ostream& stream, tRte& Rte) 
{
  return stream << "NOT IMPLEMENTED YET";
}

int tRte::WhatsMyCPU () {  return this->iMyCpuNum; }
        
int tRte::WhatsMyRTE() { return this->iMyRteNum; }

TIME tRte::LastIntTime() { return this->tSchedTime; }



/* Global variable to suspend rteInit for debugging purposes */
BOOL bSuspendRteInit = FALSE;


ERROR_CODE RteInit( int                      iWhichCPU,
                    int                      iWhichRTE, 
                    volatile SHMEM_WORD      *vswIpcShmemStart, 
                    int                      iIpcBytes,
                    volatile HEARTBEAT       *vHbLocation, 
                    volatile HEALTH          *vRteHealth, 
                    volatile INIT_TYPE       *vRteInitType )
{
   ERROR_CODE ec;

   TraceMark(S_rte_ENTERING_RTEINIT);

   if (bSuspendRteInit) 
     taskSuspend (0);

   /* now we just turn around and construct the Rte */
   printf ("About to create an RTE\n");

   ec = pThisRteG->Init (iWhichCPU, iWhichRTE, 
			 vswIpcShmemStart, iIpcBytes, vHbLocation, 
           	         vRteHealth, vRteInitType, 
                         CbnInterruptBoardArray[iWhichRTE],
                         CiBoardInterruptArray[iWhichRTE] );
   if (ec != OK)
   {
         #ifndef CROSSWIND
             printErrno(ec);   // makes tornado unhappy  :(
         #endif CROSSWIND

       fprintf(stderr,"Rte #%d Init failed, error 0x%x\n", iWhichRTE, ec);
       return Traceback (ec,"RteInit()");
      }

   printf ("Created an RTE\n");

   TraceMark(S_rte_EXITING_RTEINIT);

   return OK;

}       /* RteInit() */


/************************************************************************
*
* tRte::HbGen - tick this rte's heartbeat
*
*     Called at every Scheduler interrupt; increments value at given pointer
*     so as to indicate health of RTE corresponding to that Heartbeat pointer.
*     (possibly also write "HEALTHY" flag into local RTE health)  
*
* IN:
*   tRte      *this
*
* INOUT :  
*   Heartbeat *this.vHeartbeat
*
* OUT :
* 
* SPEC: 
* 
   [ this invalid || *this.vHeartbeat invalid -> UNDEFINED
   | *this.iMyRteNum >= dimension of *this.vHeartbeat array -> UNDEFINED
   | true ->
      *this.vHeartbeat[*this.iMyRteNum] := *this.vHeartbeat[*this.iMyRteNum] + 1
      : print message
      : return OK
   ]
*
* RETURNS:
*  OK
*/

ERROR_CODE tRte::HbGen()
{

   this->vHeartbeat[this->iMyRteNum]++;

   TraceMark(S_rte_HEARTBEAT_TICKED);

   // printf("Ticking heartbeat on RTE #%d...\n",this->iMyRteNum);

   return OK;
}		/* HBgen() */


ERROR_CODE tRte::EKG(void)
{
   
   /* For each RTE in system:				*/
   /* get value of local copy of RTE health 		*/
   /* put value in global RTE health location 		*/
   /* put "UNHEALTHY" flag in local RTE health location */
   
   // printf("Hello from EKG()\n");
   TraceMark(S_rte_EKG_TICKED);
   return OK;
}		/* EKG() */








ERROR_CODE tRte::EMT(void)
{
   
   /* For each RTE, get value of global copy of RTE health */
   /* If not HEALTHY, then deal with it appropriately.     */
   
   // printf("Hello from EMT()\n");
   TraceMark(S_rte_EMT_TICKED);
   return OK;
}		/* EMT() */



/****
[[U tRte::CreateGizmo (iGizmoId) ]]
****/
tGizmo* tRte::CreateGizmo (int iGizmoId) {

  GIZMO_TYPE gtType;
  const tGizmoInfo *pGizmoInfo;
  
  /*****
  [[U gtType := ConfigManager.tConfig::GetGizmoType (iGizmoId) ]] 
  ****/
  gtType = ConfigManager.GetGizmoType (iGizmoId);

  /*****
  [[U pGizmoInfo := ConfigManager.tConfig::GetGizmoInfo_i (iGizmoId) ]] 
  ****/
  pGizmoInfo = ConfigManager.GetGizmoInfo (iGizmoId);
  
  
  switch (gtType ) {

    case GT_INSTRUMENT : 
      TraceMark (S_rteConfig_CREATE_NEW_INSTRUMENT);
      
      /****
      [ Not enough memory to allocate new tInstrumentCDSGizmo => pTemp := NULL
      | true => pTemp = address of new tInstrumentCDSGizmo
      ]

      [[U pTemp->tInstrumentCDSGizmo::tInstrumentCDSGizmo (*pGizmoInfo) ]]
      
      [ return := pTemp ]
      ****/
      return new tInstrumentCDSGizmo ((tInstrumentCDSGizmoInfo &) (*pGizmoInfo));

   case GT_DL_GSU : 
      TraceMark (S_rteConfig_CREATE_NEW_GSU_DELAY_LINE);

      /****
      [ Not enough memory to allocate new tGsuDlGizmo => pTemp := NULL
      | true => pTemp = address of new tInstrumentCDSGsuDlGizmo
      ]

      [[U tGsuDlGizmo::tGsuDlGizmo (pGizmoInfo) ]]
      
      [ return := pTemp ]
      ****/
      return new tGsuDlGizmo ((tGsuDlGizmoInfo &) (*pGizmoInfo));

#if 0  /* No Quic, FT, ST, or EM in GSU build */
    case GT_DL_BB1: 
    case GT_DL_BB2 :
      TraceMark (S_rteConfig_CREATE_NEW_QUIC_DELAY_LINE);

      /****
      [ Not enough memory to allocate new tQuicDlGizmo => pTemp := NULL
      | true => pTemp = address of new tQuicDlGizmo
      ]

      [[U tQuicDlGizmo::tQuicDlGizmo (pGizmoInfo) ]]
      
      [ return := pTemp ]
      ****/
      return new tQuicDlGizmo ((tQuicDlGizmoInfo &) (*pGizmoInfo));
      
   case GT_FRINGE_TRACKER :

      TraceMark (S_rteConfig_CREATE_NEW_FRINGE_TRACKER);

       /****
       [ Not enough memory to allocate new tFtGizmo => pTemp := NULL
       | true => pTemp = address of new tFtGizmo
       ]

       [[U tFtGizmo::tFtGizmo (pGizmoInfo) ]]
      
       [ return := pTemp ]
       ****/
       return new tFtGizmo ((tFtGizmoInfo &) (*pGizmoInfo));


    case GT_EXTERNAL_METROLOGY : 

      TraceMark (S_rteConfig_CREATE_NEW_EXTERNAL_METROLOGY);

      /****
      [ Not enough memory to allocate new tEmGizmo => pTemp := NULL
      | true => pTemp = address of new tEmGizmo
      ]

      [[U tEmGizmo::tEmGizmo (pGizmoInfo) ]]
      
      [ return := pTemp ]
      ****/
      return new tEmGizmo ((tEmGizmoInfo &) (*pGizmoInfo));

    case GT_STAR_TRACKER :

      TraceMark (S_rteConfig_CREATE_NEW_STAR_TRACKER);

      /****
      [ Not enough memory to allocate new tStGizmo => pTemp := NULL
      | true => pTemp = address of new tStGizmo
      ]

      [[U tStGizmo::tStGizmo (pGizmoInfo) ]]
      
      [ return := pTemp ]
      ****/
      return new tStGizmo ((tStGizmoInfo &) (*pGizmoInfo));

#endif /* Removing FT, ST, and EM */

      default : 
        TraceMsg(tmt_ERROR, S_rte_GIZMO_CREATION_ERROR,
                 "tRte::CreateGizmo: Invalid Gizmo Type, %d", gtType);
        return NULL;
   };
}

/*****
* helper function
*/

ERROR_CODE construct(void)
{
  ConfigManager.tConfig();
//  ThisRteG.tRte();

  return OK;
}


tGizmo *GetGizmoPointer(int iGlobalGizmoIndex)
{ 
   char sMsgOut[256];

   if (pThisRteG->WhatsMyRTE()==0) {
      for (int i=0; i<pThisRteG->iNumberOfGizmos; i++)
 	 if (CiRte0Gizmos[i]==iGlobalGizmoIndex)
	    return pThisRteG->pGizmoArray[i];
   }
   else {
     for (int i=0; i<pThisRteG->iNumberOfGizmos; i++) {
         printf ("i = %d  RTE = %d\n",i, pThisRteG->WhatsMyRTE ());
 	 if (CiRte1Gizmos[i]==iGlobalGizmoIndex)
	    return pThisRteG->pGizmoArray[i];
     }
   }

   if (iGlobalGizmoIndex<0)    /* e.g. INVALID_GIZMO */
   {
      TraceMsg(tmt_WARN,S_rte_INVALID_GIZMO_NUMBER,"::GetGizmoPointer");
      return (tGizmo *)NULL;  
   }
   
   sprintf(sMsgOut,"::GetGizmoPointer: Gizmo #%d not on RTE #%d",
                                      iGlobalGizmoIndex, pThisRteG->WhatsMyRTE());

   TraceMsg(tmt_WARN,S_rte_GIZMO_NOT_ON_THIS_RTE,sMsgOut);
   return (tGizmo *)NULL;  /* if we get here, didn't find it */
} 


tErrorStreamSender& GetErrorStream(void) { 
  return *pThisRteG->pErrorStream; 
}

tRelayStreamSender& GetDataStream(void) { 
  return *pThisRteG->pDataStream; 
}

tRelayStreamSender& GetStatusStream(void) { 
  return *pThisRteG->pStatusStream; 
}

tScheduler& GetScheduler(void) { 
  return *pThisRteG->pScheduler; 
}

TIME GetTime(void) { 
  return pThisRteG->tSchedTime; 
}

tRte& GetRte(void)  { 
  return *pThisRteG; 
}

tIpc& GetIpc(void)  {
  return *pThisRteG->pIpc; 
}  

void ShowIpc () {
  cout << ::GetIpc();
}
