/*$RCSfile: token.h,v $ */
/*$Header: /home/chara/cvsroot/ople/sys/ipc/include/token.h,v 1.1.1.1 1998/06/11 21:36:37 chara Exp $*/

/* Richard Johnson - Jet Propulsion Laboratory */


#ifndef INC_tokenh
#define INC_tokenh


#include "isit.h"
#include "baseObject.h"


/****
[[D tTokenClient::tTokenClient (volatile TOKEN_FLAG *vtfTokenArray, int iClientId,
                  int iTimeOutInSeconds)]
[ invalid_pointer (this) ||
  vtfTokenArray is invalid ||
  iClientId > CiNumRte => UNDEFINED
| true =>
    this->bOwnToken := FALSE
    : this->bInit := TRUE
    : this->tTimeOut := tTimeOutToUse
    : this->vtfRequestFlag := vtfTokenArray[iClientId]
    : this->ecInitError := OK
]]

[[D tTokenClient::~tTokenClient (void)]
[ invalid_pointer (this) => UNDEFINED
| true => *this := anything
]]

[[D tTokenClient::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED 
| this->bInit == TRUE => 
    console := console + appropriate warning message 
    : return := OK
| true =>
    this->bInit := TRUE
    : return := OK
]]

[[D tTokenClient::Shutdown (void) ]
[ invalid_pointer (this) || 
  this->vtfRequestFlag is invalid => UNDEFINED
| this->bInit == FALSE => 
    console := console + appropriate warning message 
    : return OK
| this->bOwnToken == TRUE =>
    Token returned
    : this->bOwnToken := FALSE
    : this->bInit := FALSE
    : return := OK
| true =>
    this->bInit := FALSE
    : return := OK
]]


[[D tTokenClient::Request (void) ]
[ invalid_pointer (this) || 
  this->vtfRequestFlag is invalid => UNDEFINED
| Server does not grant token in this->tTimeOut ticks => 
    console := console + appropriate error message 
    : return := appropriate error code
| true => 
    Token given to client from appropriate token server
    : this->bOwnToken := TRUE
    : return := OK
]]


[[D tTokenClient::Return (void) ]
[ invalid_pointer (this) || 
  this->vtfRequestFlag is invalid => UNDEFINED
| this->bOwnToken == FALSE => 
    console := console + appropriate error message 
    : return := appropriate error code
| true => 
    token returned to appropriate token server
    : this->bOwnToken := FALSE
    : return := OK
]]


[[D tTokenServer::tTokenServer (volatile TOKEN_FLAG *vtfTokenArrayToUse,
                  int iNumClientsToUse, 
                  int iServerTimeOutInSeconds) ]
[ invalid_pointer (this) ||
  vtfTokenArrayToUse is invalid ||
  iNumClientsToUse < 0 ||
  iServerTimeOutInSeconds < 0 => UNDEFINED
| Unable to spawn Poll task => 
    console := console + appropriate error message 
    : this->ecInitError := appropriate error message
| true =>
    this->vtfTokenArray := vtfTokenArrayToUse
    : vtfTokenArrayToUse[i] := TF_IDLE for 0 <= i <= iNumClientsToUse
    : this->tServerTimeOut := tServerTimeOutToUse
    : this->bInit := TRUE
    : this->bOwnToken := TRUE
    : this->iNumClients := iNumClientsToUse
    : Poll task spawned
    : this->ecInitError := OK 
]]

[[D tTokenServer::~tTokenServer (void) ]
[ invalid_pointer (this) => UNDEFINED
| Unable to delete Poll task =>
    console := console + appropriate error message 
    : return := appropriate error code
| true =>
    Poll task deleted
    : *this := anything
]]

[[D tTokenServer::Restart (void) ]
[ invalid_pointer (this) => UNDEFINED 
| this->bInit == TRUE =>
    console := console + appropriate warning message 
    : return := OK
| Unable to resume Poll task =>
    console := console + appropriate error message 
    : return := appropriate error message
| true =>
    this->bInit := TRUE
    : Poll task resumed
    : return := OK
]]

[[D tTokenServer::Shutdown (void) ]
[ invalid_pointer (this) => UNDEFINED
| this->bInit == FALSE =>
    console := console + appropriate warning message 
    : return := OK
| unable to suspend Poll task =>
    appropriate message printed on console
    : return := appropriate error code
| true =>
    this->bInit := FALSE
    : Poll task suspended
    : return := OK
]]

[[D tTokenServer::TokenClientRequestsToken(void) ]
[ invalid_pointer (this) || 
  vtfTokenArray is invalid => UNDEFINED
| this->bInit == FALSE ||
  Server can't grant token => no response
| true =>
    token granted to appropriate token client
    : this->bOwnToken == FALSE
]]

[[D tToknServer::TokenGrantedToClient (void)]
[ invalid_pointer (this) ||
  vtfTokenArray is invalid => UNDEFINED
| this->bInit == FALSE => no response
| Token not returned within this->tServerTimeOut ticks =>
    appropriate message printed on console
    : this->bOwnToken := TRUE
| true => this->bOwnToken := TRUE
]]

********/


class tTokenClient : public tBaseObject { 
  public:
    tTokenClient (volatile TOKEN_FLAG *vtfTokenArray, int iClientId,
                  int iTimeOutInSeconds);
    ERROR_CODE Restart ();
    ERROR_CODE Shutdown ();
    ERROR_CODE Request ();
    ERROR_CODE Return ();
    friend ostream& operator<< (ostream& stream, const tTokenClient& TokenClient);
  protected:
    int iTimeOutInTicks;
    BOOL bOwnToken;
    volatile TOKEN_FLAG *vtfRequestFlag;
};

class tTokenServer : public tBaseObject {
  public:
    tTokenServer (volatile TOKEN_FLAG *vtfTokenArrayToUse,
                  int iNumClientsToUse, 
                  int iServerTimeOutInSeconds);
    ~tTokenServer ();
    ERROR_CODE Restart ();
    ERROR_CODE Shutdown ();
    ERROR_CODE Poll ();
    friend ostream& operator<< (ostream& stream, const tTokenServer& TokenServer);
    
  protected:
    BOOL CheckRequest ();
    void GrantRequest ();
    BOOL CheckReturn ();
    void AcceptReturn ();
    void Steal ();
    void NextClient (); 

    int tidPollTaskId;
    volatile TOKEN_FLAG *vtfTokenArray;
    int iNumClients;
    int iCurrentClient;
    int iServerTimeOutInTicks;
    BOOL bInit;
    BOOL bOwnToken;
};


#endif







