/* $RCSfile: ipc.cc,v $  */
/*
$Header: /home/chara/cvsroot/ople/sys/ipc/ipc/ipc.cc,v 1.1.1.1 1998/06/11 21:36:37 chara Exp $
*/
/*
modification history
--------------------
01a,2Jun97,rlj    written
*/


#include "ipcid.h"
#include "ipc.h"
#include "token.h"
#include "localMem.h"
#include "sharedMem.h"
#include "ipcError.h"


/****
[[U tIpc::tIpc (vswSharedMemoryStart,szNumSharedMemoryBytes,
          iNumRtes,iWhichRte) ]]
****/
tIpc::tIpc (volatile SHMEM_WORD *vswSharedMemoryStart,
          const size_t &szNumSharedMemoryBytes,
          const int &iNumRtes,
          const int &iWhichRte) {

  volatile TOKEN_FLAG *vtfTokenArea;

            
  /****
  [ Unable to allocate new tLocalMemory object => this->pLocalMemory := NULL
  | true =>valid_pointer (this->pLocalMemory) 
  ]
  
  [[U *this->pLocalMemory := tLocalMemory::tLocalMemory (void) ]]
  ****/
  this->pLocalMemory = new tLocalMemory ();

  if (this->pLocalMemory == NULL) {
    tBaseObject::SetInitError (S_ipc_CANT_CREATE_LOCAL_MEMORY_POOL);
    Traceback (S_ipc_CANT_CREATE_LOCAL_MEMORY_POOL,"tIpc::tIpc()");
    return;
  }

  /****
  [[U tBaseObject::GetInitError (void) ]]
  ****/
  if (this->pLocalMemory->GetInitError() != OK) {
    tBaseObject::SetInitError (S_ipc_CANT_CREATE_LOCAL_MEMORY_POOL);
    Traceback (S_ipc_CANT_CREATE_LOCAL_MEMORY_POOL,"tIpc::tIpc()");
    return;
  }
  
  /****
  [ Unable to allocate new tSharedMemory object => this->pSharedMemory := NULL
  | true =>valid_pointer (this->pSharedMemory) 
  ]
  
  [[U *this->pSharedMemory := tSharedMemory::tSharedMemory (vswSharedMemoryStart,
                              szNumSharedMemoryBytes,iNumRtes) ]]
  ****/
  this->pSharedMemory = new tSharedMemory (vswSharedMemoryStart,
                              szNumSharedMemoryBytes,iNumRtes);

  if (this->pSharedMemory == NULL) {
    tBaseObject::SetInitError (S_ipc_CANT_CREATE_SHARED_MEMORY_POOL);
    Traceback (S_ipc_CANT_CREATE_SHARED_MEMORY_POOL,"tIpc::tIpc()");
    return;
  }
  
  
  /****
  [[U vTokenArea := tSharedMemory::GetTokenArea (void) ]]
  ****/
  vtfTokenArea = this->pSharedMemory->GetTokenArea ();
  
  /****
  [ Unable to allocate new tTokenClient object => this->pTokenClient := NULL
  | true =>valid_pointer (this->pTokenClient) 
  ]
  [[U this->pTokenClient := tTokenClient::tTokenClient (vtfTokenArea,iWhichRte, 
                            CiIpcTimeOutSeconds) ]]
  ****/
  this->pTokenClient = new tTokenClient (vtfTokenArea,iWhichRte,
                                         CiIpcTimeOutSeconds);

  if (this->pTokenClient == NULL) {
 
    tBaseObject::SetInitError (S_ipc_CANT_CREATE_TOKEN_CLIENT);
    Traceback (S_ipc_CANT_CREATE_TOKEN_CLIENT,"tIpc::tIpc()");
    return;
  }
  
  /****
  [[U this->pTokenClient.tBaseObject::GetInitError (void) ]]
  ****/
  if (this->pTokenClient->GetInitError() != OK) {
    this->SetInitError(S_ipc_CANT_CREATE_TOKEN_CLIENT);
    Traceback (S_ipc_CANT_CREATE_TOKEN_CLIENT,"tIpc::tIpc()");
    return;
  }
 
}   

/****
[[U tIpc::Restart (void) ]]
****/
ERROR_CODE tIpc::Restart () {
 
  ERROR_CODE ecErrCode;
  /****
  [[U ecErrCode := this->pLocalMemory.tMemory::Restart (void) ]]
  ****/
  ecErrCode = this->pLocalMemory->Restart ();

  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Restart()");
  
  /****
  [[U ecErrCode := this->pSharedMemory.tMemory::Restart (void) ]]
  ****/
  ecErrCode = this->pSharedMemory->Restart ();
  
  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Restart()");

  /****
  [[U ecErrCode := this->pTokenClient.tTokenClient::Restart (void) ]]
  ****/
  ecErrCode = this->pTokenClient->Restart ();

  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Restart()");

  return OK;
}
  
/****
[[U tIpc::Shutdown (void) ]]
****/
ERROR_CODE tIpc::Shutdown () {
  
  ERROR_CODE ecErrCode;

  /****
  [[U ecErrCode := this->pTokenClient.tTokenClient::Shutdown (void) ]]
  ****/
  ecErrCode = this->pTokenClient->Shutdown ();

  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Shutdown()");

  /****
  [[U ecErrCode := this->pLocalMemory.tMemory::Shutdown (void) ]]
  ****/
  ecErrCode = this->pLocalMemory->Shutdown ();

  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Shutdown()");
      
  /****  
  [[U ecErrCode := this->pSharedMemory.tMemory::Shutdown (void) ]]
  ****/
  ecErrCode = this->pSharedMemory->Shutdown ();

  if (ecErrCode != OK) return Traceback (ecErrCode,"tIpc::Shutdown()");


  return OK;
}



/****
[[U tIpc::Allocate (iiId, szSize, recErrCode) ]] 
***/
void *tIpc::Allocate (const tIpcId &iiId, const size_t &szSize, 
                      ERROR_CODE &recErrCode) {

  void *pObjectTemp;
  IPC_NAME tempName;
  IPC_TYPE tempType;


 /****
 [[U tempName := iiId.tIpcId::Name(void) ]]
 ****/
 tempName = iiId.Name ();
 
 /****
 [[U tempType := iiId.tIpcId::Name(void) ]]
 ****/
 tempType = iiId.Type ();
 
 
 /****
 [[U tIpcId::Memory (void) ]]
 ****/
  switch (iiId.Memory () ) {
   
    case MT_LOCAL : 

      TraceMark (S_ipc_LOCAL_OBJECT_ALLOCATE_START);

      /****
      [[U pObjectTemp := this->pLocalMemory.tLocalMemory::Allocate (tempName ,
                              tempType , szSize, recErrCode ) ]]
      ****/

      tempName = iiId.Name();
      tempType = iiId.Type();
      pObjectTemp = this->pLocalMemory->Allocate (tempName,
                              tempType, szSize, recErrCode );
      if (recErrCode != OK) {
        recErrCode = S_ipc_CANT_ALLOCATE_LOCAL_OBJECT;
        Traceback (recErrCode,"tIpc::Allocate ()");
      }
      TraceMark (S_ipc_LOCAL_OBJECT_ALLOCATE_DONE);
      return pObjectTemp;

    case MT_SHARED : 
    
      TraceMark (S_ipc_SHARED_OBJECT_ALLOCATE_START);

      /****
      [[U recErrCode := this->pTokenClient.tTokenClient::Request (void) ]]
      ****/
      recErrCode = this->pTokenClient->Request ();   
      
      if (recErrCode != OK) {
         recErrCode = S_ipc_CANT_ALLOCATE_SHARED_MEMORY_OBJECT;
         Traceback (recErrCode,"tIpc::Allocate()" );
         return NULL;
      }
      
      /****
      [[U pObjectTemp := this->pSharedMemory.tSharedMemory::Allocate (inNameToUse,
                              itTypeToUse, szSize, recErrCode ) ]]
      ****/
      tempName = iiId.Name();
      tempType = iiId.Type();
      pObjectTemp = this->pSharedMemory->Allocate (tempName,
                              tempType, szSize, recErrCode );
      if (recErrCode != OK) {
         recErrCode = S_ipc_CANT_ALLOCATE_SHARED_MEMORY_OBJECT;
         this->pTokenClient->Return ();
         Traceback (recErrCode,"tIpc::Allocate()" );
      }
      
      /****
      [[U recErrCode := this->pTokenClient.tTokenClient::Return (void) ]]
      ****/
      this->pTokenClient->Return ();
      
      TraceMark (S_ipc_SHARED_OBJECT_ALLOCATION_DONE);
       
      return pObjectTemp;
 
    default :
      recErrCode = S_ipc_UNKNOWN_MEMORY_TYPE;
      Traceback (recErrCode,"tIpc::Allocate()" );
      return NULL;
  }
  return NULL;
}                          


ostream &operator<< (ostream& stream,const tIpc& Ipc) {
  
  stream << "Local Memory State =" << endl << (*Ipc.pLocalMemory) << endl <<
            "Shared Memory State = " << endl << (*Ipc.pSharedMemory) << endl <<
            "TokenClient State = " << endl << (*Ipc.pTokenClient) << endl;
  return stream;
}

