
#include "isit.h"
#include "ipcid.h"
#include "iostream.h"

/**********************************************************************
Representation Mapping:

State Box                                 BlackBox

(tIpcObjectId,tIpcAtomId,tIpcDblBufferId)

this->inObjectName                        inObjectName
this->itObjectType                        this->Type()
this->mtObjectMemType	                  this->mtObjectMemType

(tIpcFifoId)

this->iFifoDepth			  this->iFifoDepth

**********************************************************************/

/****
[[U tIpcId::tIpcId (void) ]]
****/
tIpcId::tIpcId () {
  this->inObjectName = IPC_NULL_NAME;
  // this->mtObjectMemType = MT_UNKNOWN;  messed up in merge 7/21 PCI
}

/****
[[U tIpcId::tIpcId_INMT (inName, mtMemType) ]]
****/
tIpcId::tIpcId (IPC_NAME inName, MEM_TYPE mtMemType) {
  this->inObjectName = inName;
  this->mtObjectMemType =mtMemType;
}
 
/****
[[U tIpcId::Name (void) ]]
****/
IPC_NAME tIpcId::Name() const { 
  return this->inObjectName; 
}

/****
[[U tIpcId::Memory (void) ]]
****/
MEM_TYPE tIpcId::Memory() const {
  return this->mtObjectMemType;
}

/****
[[U tIpcAtomId::tIpcAtomId (void) ]]
****/
tIpcAtomId::tIpcAtomId () : 
  
  /****
  [[U tIpcId::tIpcId (void) ]]
  ****/
  tIpcId () {}

ostream &operator<< (ostream& stream,const tIpcId& iiId) {

  stream << "IpcId" << endl << "Name = " << iiId.Name() << endl <<
    "Type = " << iiId.Type () << endl << "Memory = " << iiId.Memory() << endl;
  return stream;
}

ostream &operator<< (ostream& stream,const tIpcFifoId& iiId) {

  stream << (tIpcId) iiId << "Fifo Depth = " << iiId.Depth() << endl;
  return stream;
}

ostream &operator<< (ostream& stream,const tIpcIndirectionDblBufferId& iiId) {

  stream << (tIpcId) iiId << "Buffer 1 Name = " << iiId.inBuf1Name << endl
         << "Buffer 2 Name = " << iiId.inBuf2Name << endl
         << "Buffer Size = " << iiId.BufferSize() << endl;
  return stream;
}

/****
[[U tIpcAtomId::tIpcAtomId_INMT (inName,  mtMemType) ]]
****/
tIpcAtomId::tIpcAtomId (IPC_NAME inName,MEM_TYPE mtMemType): 

  /****
  [[U tIpcId::tIpcId_INMT (inName,mtMemType) ]]
  ****/
  tIpcId(inName,mtMemType) {}

/****
[[U tIpcAtomId::Type (void) ]]
****/
IPC_TYPE tIpcAtomId::Type() const { 
  return IPC_ATOM; 
}


/****
[[U tIpcDblBufferId::tIpcDblBufferId (void) ]]
****/
tIpcDblBufferId::tIpcDblBufferId () : 
 
 
  /****
  [[U tIpcId::tIpcId (void) ]]
  ****/
  tIpcId () {}

/****
[[U tIpcDblBufferId::tIpcDblBufferId_INMT( inName, mtMemType) ]]
****/
tIpcDblBufferId::tIpcDblBufferId (IPC_NAME inName, MEM_TYPE mtMemType):

  /****
  [[U tIpcId::tIpcId_INMT (inName,mtMemType) ]]
  ****/
  tIpcId(inName,mtMemType) {}
  

/****
[[U tIpcDblBufferId::Type(void) ]]
****/  
IPC_TYPE tIpcDblBufferId::Type() const {
  return IPC_DBL_BUFFER;
}


/****
[[U tIpcFifoId::tIpcFifoId (void) ]]
****/  
tIpcFifoId::tIpcFifoId () : 

  /****
  [[U tIpcId::tIpcId (void) ]]
  ****/  
  tIpcId () {}
    
/****
[[U tIpcFifoId::tIpcFifoId_INMTi (inName, mtMemType, iDepth) ]]
****/    
tIpcFifoId::tIpcFifoId (IPC_NAME inName, MEM_TYPE mtMemType, int iDepth) :
  
  /****
  [[U tIpcId::tIpcId_INMT (inName,mtMemType)]]
  ****/
  tIpcId (inName,mtMemType) {
  
  this->iFifoDepth = iDepth;
}


/****
[[U tIpcFifoId::Depth(void) ]]
****/
int tIpcFifoId::Depth () const {
  return this->iFifoDepth;
}

/****
[[U tIpcFifoId::Type(void) ]]
****/
IPC_TYPE tIpcFifoId::Type() const {
  return IPC_FIFO;
}


/****
[[U tIpcIndirectionDblBufferId::tIpcIndirectionDblBufferId (void) ]]
****/  
tIpcIndirectionDblBufferId::tIpcIndirectionDblBufferId () : 

  /****
  [[U tIpcId::tIpcId (void) ]]
  ****/  
  tIpcId () {

  this->inBuf1Name = IPC_NULL_NAME;
  this->inBuf2Name = IPC_NULL_NAME;
  this->szBufferSize = 0;
}
    
/****
[[U tIpcIndirectionDblBufferId::tIpcIndirectionDblBufferId_InInInMtSz (inName, 
                                inBuf1Name, inBuf2Name, mtMemType, szBufSize) ]]
****/    
tIpcIndirectionDblBufferId::tIpcIndirectionDblBufferId (IPC_NAME inName, 
                                IPC_NAME inBuf1Name, IPC_NAME inBuf2Name, 
                                MEM_TYPE mtMemType, size_t szBufferSizeToUse) :
  
  /****
  [[U tIpcId::tIpcId_INMT (inName,mtMemType)]]
  ****/
  tIpcId (inName,mtMemType) {
  
  this->inBuf1Name = inBuf1Name;
  this->inBuf2Name = inBuf2Name;
  this->szBufferSize = szBufferSizeToUse;
}


/****
[[U tIpcIndirectionDblBufferId::BufferSize(void) ]]
****/
size_t tIpcIndirectionDblBufferId::BufferSize () const {
  return this->szBufferSize;
}

/****
[[U tIpcIndirectionDblBufferId::Type(void) ]]
****/
IPC_TYPE tIpcIndirectionDblBufferId::Type() const {
  return IPC_INDIRECTION_DBL_BUFFER;
}

/****
[[U tIpcIndirectionDblBufferId::Buf1Id(void) ]]
****/
tIpcAtomId tIpcIndirectionDblBufferId::Buf1Id() const {
  /****
  [[U tIpcAtomId::tIpcAtomId_INMT (this->inBuf1Name, this->mtObjectMemType) ]]
  ****/
  return tIpcAtomId(this->inBuf1Name, this->mtObjectMemType);
}
  
/****
[[U tIpcIndirectionDblBufferId::Buf2Id(void) ]]
****/
tIpcAtomId tIpcIndirectionDblBufferId::Buf2Id() const {
  /****
  [[U tIpcAtomId::tIpcAtomId_INMT (this->inBuf2Name, this->mtObjectMemType) ]]
  ****/
  return tIpcAtomId(this->inBuf2Name, this->mtObjectMemType);
}
  
