/* $RCSfile: localMem.cc,v $  */
/*
$Header: /home/chara/cvsroot/ople/sys/ipc/memory/localMem.cc,v 1.1.1.1 1998/06/11 21:36:38 chara Exp $
*/
/*
modification history
--------------------
01a,2Jun97,rlj    written
*/

#include "localMem.h"
#include "ipcError.h"
    
#include "errnoLib.h"

/****
[[U tLocalMemory::tLocalMemory (void) ]]
****/
tLocalMemory::tLocalMemory () :

  /****
  [[U tMemory::tMemory (void) ]]
  ****/
  tMemory (),
  
  /****
  [[U tList::tList (void) ]]
  ****/
  lHeaderList ()
  
{
   ERROR_CODE ecTemp;
   
   this->siAllocationMutex = semMCreate(SEM_Q_FIFO);
   
   if (this->siAllocationMutex == 0) {
     ecTemp = Traceback (errnoGet(),"tLocalMemory::tLocalMemory()");
     
     /****
     [[U tBaseObject::SetInitError(void)]]
     ****/
     tBaseObject::SetInitError (ecTemp);
   }

}

/****
[[U tLocalMemory::~tLocalMemory (void) ]]
****/
tLocalMemory::~tLocalMemory () {

  /****
  [[U iListLength := this->lHeaderList.tList::Count (void) ]]
  ****/
  int iListLength = this->lHeaderList.Count();
  

  semDelete (this->siAllocationMutex);
  
  // delete all local memory objects
  
  int i;
 
  for (i=1;i <= iListLength; i++)
           /****
           [[U this->lHeaderList.tList::Nth(i) ]]
           ****/
    delete this->lHeaderList.Nth(i);
  
  // delete all local memory object headers 
  /****
  [[U tList::~tList (void) ]]
  ****/

  /****
  [[U tMemory::~tMemory (void) ]]
  ****/
    
}


/****
[[U tMemory::Restart (void) ]]
****/
ERROR_CODE tLocalMemory::Restart ()  {

  /****
  [[U return := tMemory::Restart (void) ]]
  ****/
  return tMemory::Restart () ;
}
 
/****
[[U tMemory::Shutdown (void) ]]
****/
ERROR_CODE tLocalMemory::Shutdown () {


  /****
  [[U return := tMemory::Shutdown (void) ]]
  ****/
  return tMemory::Shutdown ();
}


/****
[[U tLocalMemory::Allocate (inName,itType,Size,recErrCode) ]]
****/
void *tLocalMemory::Allocate (IPC_NAME inName,IPC_TYPE itType,
			      size_t Size, ERROR_CODE &recErrCode) {
 
    
    int i,iListLength; 
    tLocalHeader *pHeader;
    recErrCode = OK;
    void *pObject;
    
    int iTimeOutinTicks = CiAllocationTimeOutInSeconds * sysClkRateGet ();
    recErrCode = semTake (this->siAllocationMutex,iTimeOutinTicks); // Begin critical region
    
    if (recErrCode == ERROR)  {
      recErrCode = errnoGet();
      TraceMsg (tmt_ERROR,recErrCode,"tLocalMemory::Allocate()");
      return NULL;
    }
   
    /****
    [[U pHeader := this->lHeaderList.tList::First (void) ]]
    ****/
    pHeader = (tLocalHeader *) this->lHeaderList.First();
   
    /****
    [[U iListLength := this->lHeaderList.tList::Count (void) ]]
    ****/
    iListLength = this->lHeaderList.Count();
        
    for (i = 0; i < iListLength; i++) {

      /****
      [[U tObjectHeader::IsName (inName) ]]
      ****/
      if (pHeader->IsName(inName) ) 
 
        /****
        [[U tObjectHeader::IsType (itType) ]]
        [[U tObjectHeader::IsSize (Size) ]]
        ****/
        if ((pHeader->IsType(itType)) && (pHeader->IsSize (Size))) {
        
          semGive (this->siAllocationMutex); // end critical region
        
          /****
          [[U return := pHeader.tLocalHeader::GetObjectReference (void) ]]
          ****/
          return pHeader->GetObjectReference();    
        }
        else {
         
          semGive (this->siAllocationMutex); // end critical region
          recErrCode = S_ipc_OBJECT_MISMATCH;
          return NULL;
        }
 
       /****
       [[U pHeader := this->lHeaderList.tList::Next (pHeader) ]]
       ****/
       pHeader = (tLocalHeader *) this->lHeaderList.Next(pHeader);
    }
    
    /****
    [ Unabled to allocate new char[Size] => pObject := NULL
    | true => valid_pointer (pObject)
    ]
    ****/
    pObject = new unsigned char[Size]; 
   
    if (pObject == NULL) recErrCode = S_ipc_INSUFFICIENT_LOCAL_MEMORY;
 
    /****
    [[U pTempNode := tLocalHeader::tLocalHeader_INITszp (inName,itType,Size,pObject) ]]
    [[U this->lHeaderList.tList::Add (pTempNode) ]]
    ****/
    else this->lHeaderList.Add (new tLocalHeader (inName,itType,Size,pObject) );
    
    semGive (this->siAllocationMutex); // end critical region
    return pObject;
}    
    
ostream &operator<< (ostream &stream, tLocalMemory &LocalMem) {

  int i;
  //  tLocalMemory LocalMemTemp = LocalMem;
  int iNumObjects = LocalMem.lHeaderList.Count();
  
  stream << "Local Memory Contents:" << endl <<
            "Number of Objects = " << iNumObjects << endl <<
            "Object Info : " << endl;            
  
  
  for (i = 1; i <= iNumObjects; i++ )
    stream << *(tLocalHeader *) (LocalMem.lHeaderList.Nth(i)) << endl;
  
  stream << "End of Local Memory Contents" << endl;
  return stream;
}
      				  
 
  
