/* $RCSfile: sharedMem.cc,v $  */
/*
$Header: /home/chara/cvsroot/ople/sys/ipc/memory/sharedMem.cc,v 1.2 1998/08/17 21:01:30 phil Exp $
*/
/*
modification history
--------------------
01a,2Jun97,rlj    written
*/




#include "sharedMem.h"
#include "ipcMemPrivate.h"
#include "ipcError.h"
#include <errnoLib.h>
#include <taskLib.h>
#include "token.h"
#include "memHeader.h"

/****
[[U tSharedMemory::tSharedMemory (vswSharedMemoryStart, 
                   	      iNumSharedMemoryBytes, 
                   	      iNumRtes) ]]
****/
tSharedMemory::tSharedMemory (volatile SHMEM_WORD *vswSharedMemoryStart, 
                   	      const int &iNumSharedMemoryBytes, 
                   	      const int &iNumRtes) :

  /****
  [[U tMemory::tMemory (void) ]]
  ****/
  tMemory ()
  
{
  size_t temp = sizeof(SHMEM_WORD);
  volatile BYTE *pTempPtr;
  this->iNumRtes = iNumRtes;
  this->iNumSharedMemoryBytes = (iNumSharedMemoryBytes/temp) * temp;

  this->vswSharedMemory = vswSharedMemoryStart;
  this->vswObjectList = vswSharedMemoryStart + 
    iNumSharedMemoryBytes/temp;  

  pTempPtr = (volatile BYTE *) vswSharedMemoryStart;

  this->viNumObjects = (volatile int *) pTempPtr;
  pTempPtr += sizeof (*this->viNumObjects);

  this->vszNumBytesFree = (volatile size_t *) pTempPtr;
  pTempPtr += sizeof (*this->vszNumBytesFree);

  this->viLastObjectOffset = (volatile int *) pTempPtr;
  pTempPtr += sizeof (*this->viLastObjectOffset);

  this->vtfTokenArray = (volatile TOKEN_FLAG *) pTempPtr; 
  pTempPtr += iNumRtes * sizeof (*this->vtfTokenArray);

  this->vHeaderList = (volatile tSharedHeader *) pTempPtr;

}

/****
[[U tSharedMemory::~tSharedMemory (void) ]]
****/
tSharedMemory::~tSharedMemory () 

  /****
  [[U tMemory::~tMemory (void) ]]
  ****/

{}


/****
[[U tSharedMemory::GlobalInit (iServerTimeOutInSeconds ) ]]
****/
ERROR_CODE tSharedMemory::GlobalInit (int iServerTimeOutInSeconds ) {

  int i;
  tTokenServer *pTokenServer;
 
  *(this->viNumObjects) = 0;
  *(this->vszNumBytesFree) =  this->iNumSharedMemoryBytes - sizeof (*this->viNumObjects) -
    sizeof (*this->vszNumBytesFree) - sizeof (*this->viLastObjectOffset) - 
    sizeof (*this->vtfTokenArray) * this->iNumRtes;
  *(this->viLastObjectOffset) = 0;
  
  for (i=0;i<this->iNumRtes;i++) {
     this->vtfTokenArray[i] = TF_IDLE;
  }
  
  /****
  [ Unable to allocate new tTokenServer => pTokenServer := NULL
  | true => valid_pointer (pTokenServer)
  ]
  [[U pTokenServer := tTokenServer::tTokenServer (this->vtfTokenArray,this->iNumRtes,
                                   iServerTimeOutInSeconds) ]]
  ****/                                 
  pTokenServer = new tTokenServer (this->vtfTokenArray,this->iNumRtes,
                                   iServerTimeOutInSeconds);
  
  if (pTokenServer == NULL) 
    return Traceback (S_ipc_TOKEN_SERVER_COULD_NOT_BE_CREATED,
                      "tTokenServer:GlobalInit()");
                      
  return OK;
}
     
/****
[[U tSharedMemory::GlobalShutdown (void) ]]
****/
ERROR_CODE tSharedMemory::GlobalShutdown () {
  
  ERROR_CODE ec;
  
  return OK;
}  
    
     
/****
[[U tMemory::Restart (void) ]]
****/
ERROR_CODE tSharedMemory::Restart ()  {

  /****
  [[U tMemory::Restart (void) ]]
  ****/
  return tMemory::Restart (); 
}
 
/****
[[U tMemory::Shutdown (void) ]]
****/
ERROR_CODE tSharedMemory::Shutdown () {

  /****
  [[U tMemory::Shutdown (void) ]]
  ****/
  return tMemory::Shutdown ();
}

  

/****
[[U tSharedMemory::Allocate (inName,itType,Size,recErrCode) ]]
****/
void *tSharedMemory::Allocate (IPC_NAME inName,IPC_TYPE itType,
			      size_t Size, ERROR_CODE &recErrCode) {
 
    
    recErrCode = OK;
    int i;
    int iListLength;
    int iSizeInShmemWords;

    iListLength = *(this->viNumObjects);
    
    for (i = 0; i < iListLength; i++) {

      /****
      [[U this->vHeaderList[i].tObjectHeader::IsName (inName) ]]
      ****/
      if (this->vHeaderList[i].IsName(inName) ) 
 
        /****
        [[U this->vHeaderList[i].tObjectHeader::IsType (itType) ]]
        [[U this->vHeaderList[i].tObjectHeader::IsSize (Size) ]]
        ****/
        if ( (this->vHeaderList[i].IsType(itType)) && 
             (this->vHeaderList[i].IsSize (Size) ) )

          /****
          [[U this->vHeaderList[i].tSharedHeader::GetObjectReference (void) ]]
          ****/
          return (void *) (this->vswObjectList - 
            this->vHeaderList[i].GetObjectReference() ) ;
        else {
          recErrCode = S_ipc_OBJECT_MISMATCH;
          TraceMsg (tmt_ERROR,recErrCode,"tSharedMemory::Allocate");
          return NULL;
        }
 
     }

    /****
    [[U tSharedMemory::EnoughMemory (Size + sizeof (tSharedHeader) ) ]]
    ****/
    if ( EnoughMemory (Size + sizeof (tSharedHeader) ) == FALSE ) {
      recErrCode = S_ipc_INSUFFICIENT_SHARED_MEMORY;
      TraceMsg (tmt_ERROR,recErrCode,
                "tSharedMemory::Allocate: Can't allocate %d B of memory. %d left",
                (Size + sizeof (tSharedHeader)), *this->vszNumBytesFree);
      return NULL;
    }
    

      /****
      [[U tSharedMemory::SizeInShmemWords (this=this,SizeInBytes = Size ) ]]
      ****/ 
      iSizeInShmemWords = SizeInShmemWords (Size);
      *this->viLastObjectOffset += iSizeInShmemWords; 
     
      /****
      [[U temp := tSharedMemory::SizeInShmemWords (Size) ]]
      [ *this->viLastObjectOffset := *this->viLastObjectOffset + temp ]
      ****/

    *this->viLastObjectOffset += SizeInShmemWords (Size);
     
    /****
    [[U this->vHeaderList[iListLength] := tSharedHeader::tSharedHeader_INITszi (inName,itType,Size,*this->viLastObjectOffset) ]]
    ****/
    this->vHeaderList[iListLength] = tSharedHeader (inName,itType,
    					   Size,*this->viLastObjectOffset);

    *this->viNumObjects = *this->viNumObjects + 1;
    *this->vszNumBytesFree =  *this->vszNumBytesFree 
                             -  iSizeInShmemWords * sizeof (SHMEM_WORD) 
                             -  sizeof (tSharedHeader);
    
    return (void *) (this->vswObjectList - *(this->viLastObjectOffset)); 
    
}    

/****
[[U tSharedMemory::EnoughMemory (Size) ]]
****/
BOOL tSharedMemory::EnoughMemory (size_t SizePassedIn) {
  if (SizePassedIn > *(this->vszNumBytesFree)) {
    printf("Returning FALSE from tSharedMemory::EnoughMemory() because 0x%x > 0x%x.\n",
            SizePassedIn, *(this->vszNumBytesFree));
    return FALSE;
  }
  else return TRUE;
}

/****
[[U tSharedMemory::SizeInShmemWords (SizeInBytes) ]]
****/
int tSharedMemory::SizeInShmemWords (size_t SizeInBytes) {

 if ( SizeInBytes % sizeof(SHMEM_WORD) == 0) 
   return SizeInBytes/sizeof(SHMEM_WORD);

 return SizeInBytes/sizeof(SHMEM_WORD) + 1;
}


ostream& operator<< (ostream& stream,const tSharedMemory &SharedMem) {

  stream << "Number of Rtes = " << SharedMem.iNumRtes << endl <<
    "Size of Shared Memory = " << SharedMem.iNumSharedMemoryBytes << endl  <<
    "Local address of Shared Memory = " << (void *)SharedMem.vswSharedMemory << endl <<
    "Number of IPC objects in Shared Memory = " << *SharedMem.viNumObjects <<
     endl << "Number of bytes free = " << *SharedMem.vszNumBytesFree << endl <<
    "Last object allocated offset location = " <<  *SharedMem.viLastObjectOffset << endl <<
    "Shared Memory Object Info : " << endl;
    
    int i;
    for ( i = 0; i < *(SharedMem.viNumObjects); i++)
      stream << (tSharedHeader& ) SharedMem.vHeaderList[i] << endl;
     
    stream << "End of Shared Memory Info" << endl;
    return stream;
}
  


