/* $RCSfile: ipcMultiTypeFifoAccessor.cc,v $ */
/* $Header: /home/chara/cvsroot/ople/sys/ipc/objectAccessor/ipcMultiTypeFifoAccessor.cc,v 1.1.1.1 1998/06/11 21:36:38 chara Exp $ */

#include "ipcMultiTypeFifoAccessor.h"
#include "ipcError.h"
#include "ipc.h"
#include "buffer.h"
#include "iostream.h"
#include "typeTag.h"


/**************************************************************
Representation Mapping

StateBox 					    Black Box

invalid_pointer (::pThisRteG)                       ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pLocalMemory)     ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pSharedMemory)    ::IpcMemory.bInit := FALSE
invalid_pointer (::pThisRteG->Ipc.pTokenClient)     ::IpcMemory.bInit := FALSE
::pThisRteG->Ipc.pSharedMemory::GlobalInit(..)      ::IpcMemory.bInit := FALSE
  not executed    

this->vFifo                      this->iiFifoId
invalid_pointer (this->Fifo )    this->bValidObject == FALSE || DoesObjectExist (..) == FALSE 
this->vFifo->iFifoDepth          GetObject (...)->iFifoDepth
this->vFifo->iFree               GetObject (...)->iFifoDepth - GetObject (...)->dtQueue.Length
this->vFifo->iFront              front element of GetObject (...)->dtQueue
this->vFifo->iBack               back of GetObject (...)->dtQueue
this->vFifo->dtQueueStart	 GetObject (...)->dtQueue
this->ecInitError		 this->ecInitError
this->bInit		         this->bInit

****************************************************************************/


/****
[[U tIpcMultiTypeFifoAccessor::tIpcMultiTypeFifoAccessor (iiId) ]]
****/
tIpcMultiTypeFifoAccessor::
  tIpcMultiTypeFifoAccessor (const tIpcFifoId &iiId) :

    /****
    [[U tIpcFifoAccessor::tIpcFifoAccessor (iiId) ]]
    ****/
    tIpcFifoAccessor<BYTE> (iiId) 
{

} 



/****
[[U tIpcMultiTypeFifoAccessor::Get (TypeTagOut, DataOut)  ]]
****/
ERROR_CODE tIpcMultiTypeFifoAccessor::Get (tTypeTag& TypeTagOut, BYTE* pDataOut) {
  
 ERROR_CODE ec;
 size_t szDataSize;
  

  // Make sure that there is both tag and data 
 
  /****
  [[U bTemp := tIpcMultiTypeFifoAccessor::IsEmpty (void) ]]
  
  [ bTemp == TRUE => return := S_ipc_FIFO_EMPTY ]
  ****/
  if (this->IsEmpty () == TRUE) return S_ipc_FIFO_EMPTY;

  // Both Tag and Data are there...



  /*****
  [[U this->vFifo.tIpcFifo::GetSeveral (TypeTagOut, sizeof (TypeTagOut)) ]]
  ****/  
  tIpcFifoAccessor<BYTE>::GetSeveral ((BYTE *) &TypeTagOut, sizeof(TypeTagOut) );


  /****
  [[U szDataSize := TypeTagOut.tTypeTag::Size(void) ]]
  ****/
  szDataSize = TypeTagOut.Size ();

  /*****
  [[U ec := this->vFifo.tIpcFifo::GetSeveral (pDataOut,szDataSize) ]]
  ****/  
  ec = tIpcFifoAccessor<BYTE>::GetSeveral (pDataOut,szDataSize);

     
  if (ec != OK) {
    /****
    [[U tIpcFifo::Flush (void) ]]
    ****/
    this->Flush ();
    return Traceback (S_ipc_MULTI_TYPE_FIFO_GET_NO_DATA,
       "tIpcMultiTypeFifoAccessor::Get () Fifo Flushed,  GizmoId = %d,  Size = %d\n",
        TypeTagOut.GizmoId(), szDataSize);
  }
  return OK;
 
}
 

/****
[[U tIpcMultiTypeFifoAccessor::Put (TypeTagIn,pDataIn)  ]]
****/
ERROR_CODE tIpcMultiTypeFifoAccessor::Put (const tTypeTag &TypeTagIn,
                                   const BYTE* pDataIn) {
  
 ERROR_CODE ec;
 size_t szDataSize;
  
  /****
  [[ szDataSize := TypeTagIn.tTypeTag::Size (void) ]]
  ****/
  szDataSize = TypeTagIn.Size();

  if (tIpcFifoAccessor<BYTE>::ElementsFree () < sizeof (TypeTagIn) + TypeTagIn.Size() ) 
    return S_ipc_FIFO_FULL;
  
  /*****
  [[U ec := this->vFifo.tIpcFifo::PutSeveral (TypeTagIn,sizeof(TypeTagIn)) ]]
  ****/  
  ec = this->tIpcFifoAccessor<BYTE>::PutSeveral ((BYTE *)&TypeTagIn,sizeof(TypeTagIn) );

  if (ec != OK) return S_ipc_FIFO_FULL; 
   
  /*****
  [[U ec := this->vFifo.tIpcFifo::PutSeveral (pDataIn, szDataSize ) ]]
  ****/  
  ec = tIpcFifoAccessor<BYTE>::PutSeveral (pDataIn, szDataSize );
 
  if (ec != OK) return Traceback (S_ipc_FIFO_FULL,"tIpcMuliTypeFifo::Put");
 
  return OK;
 
}

/****
[[U tIpcMultiTypeFifoAccessor::Put (TypeTagIn,BufferIn)  ]]
****/
ERROR_CODE tIpcMultiTypeFifoAccessor::Put (const tTypeTag &TypeTagIn,
                                   const tBuffer& BufferIn) {
  
 ERROR_CODE ec;
 size_t szDataSize;
 BYTE* pBuffer;
  
  /****
  [[U szDataSize := TypeTagIn.tTypeTag::Size (void) ]]
  ****/
  szDataSize = TypeTagIn.Size();
  
  /****
  [[U tIpcFifoAccessor::ElementsFree (void)]]
  ****/
  if (tIpcFifoAccessor<BYTE>::ElementsFree () < sizeof (TypeTagIn) + TypeTagIn.Size() ) 
    return S_ipc_FIFO_FULL;

  pBuffer = new BYTE [szDataSize];
  if (pBuffer == NULL) return Traceback (S_ipc_INTERNAL_PUT_ERROR,
      "tIpcMultiTypeFifoAccessor::Put ()"); 
  
  /****
  [[U ec := BufferIn.tBuffer::Get (szDataSize,pBuffer) ]]
  ****/
  ec = BufferIn.Get (szDataSize,pBuffer);
  
  if (ec != OK) return ec;
  
  /*****
  [[U ec := this->vFifo.tIpcFifo::PutSeveral (TypeTagIn,sizeof(TypeTagIn)) ]]
  ****/  
  ec = this->tIpcFifoAccessor<BYTE>::PutSeveral ((BYTE *)&TypeTagIn,sizeof(TypeTagIn) );

  if (ec != OK) return S_ipc_FIFO_FULL; 
   
  /*****
  [[U ec := this->vFifo.tIpcFifo::PutSeveral (pBuffer, szDataSize ) ]]
  ****/  
  ec = tIpcFifoAccessor<BYTE>::PutSeveral (pBuffer, szDataSize );
 
  delete[] pBuffer;
  
  if (ec != OK) return S_ipc_FIFO_FULL;
 

  return OK;
 
}

/****
[[U tIpcMultiTypeFifoAccessor::IsFull (TagIn)  ]]
****/
BOOL tIpcMultiTypeFifoAccessor::IsFull (const tTypeTag& TagIn) const
{
  size_t szDataSize;

  /*****
  [[U iFree := this->vFifo.tIpcFifo::ElementsFree (void) ]]
  
  [[U szDataSize := TagIn.tTypeTag.Size (void) ]]
 
  [ iFree < sizeof (TagIn) + szDataSize => return := FALSE ]
  ****/  
  if (this->ElementsFree () < sizeof (TagIn) + TagIn.Size ()) return TRUE;

  return FALSE;
}



/****
[[U tIpcMultiTypeFifoAccessor::IsEmpty (void)  ]]
****/
BOOL tIpcMultiTypeFifoAccessor::IsEmpty (void) const
{
  tTypeTag Tag;
  size_t szDataSize;

  /*****
  [[U this->vFifo.tIpcFifo::PeekSeveral (Tag, sizeof (Tag)) ]]
  ****/  
  this->vFifo->PeekSeveral ((BYTE *) &Tag, sizeof (Tag) );

  /****
  [[U szDataSize := Tag.tTypeTag::Size(void) ]]
  ****/
  szDataSize = Tag.Size ();

  /****
  [[U iLength := tFifio::Length (void) ]]
 
  [ iLength < sizeof (TypeTagOut) + szDataSize => return := TRUE ]  
  ****/
  if ( (size_t) this->Length () < sizeof (Tag) + szDataSize)  return TRUE;

  return FALSE;
}

