/* $RCSfile: relayStream.cc,v $  */
/* $Header: /home/chara/cvsroot/ople/sys/ipc/stream/relayStream.cc,v 1.2 1999/05/17 23:32:44 theo Exp $ */
/*
modification history
--------------------
01a,17Jun97,rlj    written
*/

#include "buffer.h"
#include "ipcError.h"
#include "relayStream.h"
#include <errnoLib.h>
#include "vxworks/taskLib.spc"
#include "vxworks/msgQLib.spc"


/****
[[U tRelayStreamSender::tRelayStreamSender (iiId, iMutexSemaphoreOptions) ]]
****/
tRelayStreamSender::tRelayStreamSender (const tIpcFifoId& iiId, int iMutexSemaphoreOptions ) :
  
  /****
  [[U tIpcMultiTypeFifoAccessor::tIpcMultiTypeFifoAccessor (iiId) ]]
  ****/
  tIpcMultiTypeFifoAccessor (iiId) {
  
  ERROR_CODE ecTemp;

  if (tBaseObject::GetInitError() != OK) {
    Traceback (tBaseObject::GetInitError(),"tRelayStreamSender::tRelayStreamSender");
    return;
  }
  
  /****
  [[U this->siPutMutexSemaphore := ::semMCreate (iMutexSemaphoreOptions) ]]
  ****/
  this->siPutMutexSemaphore = semMCreate (iMutexSemaphoreOptions);
  if (this->siPutMutexSemaphore == NULL) {
      ecTemp = Traceback (errnoGet(),"tRelayStreamSender::tRelayStreamSender()");
      tBaseObject::SetInitError (ecTemp);
    return;
  }
   
 TraceMark (S_relayStream_SENDER_INIT_COMPLETE);
 
}



/****
[[U tRelayStreamSender::~tRelayStreamSender (void) ]]
****/
tRelayStreamSender::~tRelayStreamSender () {

  if (tBaseObject::IsInit() ) {
    TraceMsg (tmt_WARN,S_relayStream_SHUTTING_DOWN_SENDER,
              "tRelayStreamSender::~tRelayStreamSender() ");
    /****
    [[U tRelayStreamSender::Shutdown (void)]]
    ****/
    this->Shutdown();
  }
 
 
  /****
  [[U ::semDelete (this->siPutMutexSemaphore) ]]
  ****/ 
  semDelete (this->siPutMutexSemaphore);

  /****
  [[U tIpcMultiTypeFifoAccessor::~tIpcMultiTypeFifoAccessor (void) ]]
  ****/
}


/****
[[U tRelayStreamSender::Restart (void) ]]
****/
ERROR_CODE tRelayStreamSender::Restart () {
  
  ERROR_CODE ecTemp;
  
  if (tBaseObject::IsInit() )  
    TraceMsg (tmt_WARN,S_relayStream_SENDER_ALREADY_INITIALIZED,
              "tRelayStreamSender::Restart()");
 
  else {

   /****
   [[U ecTemp := this->tIpcMultiTypeFifoAccessor::Restart (void) ]]
   ****/
   ecTemp = tIpcMultiTypeFifoAccessor::Restart();
   if (ecTemp != OK) return Traceback (ecTemp,"tRelayStreamSender::Restart()");
  }

  TraceMark (S_relayStream_SENDER_REINIT_COMPLETE);
    
  return OK;
}
  
/****
[[U tRelayStreamSender::Shutdown (void) ]]
****/
ERROR_CODE tRelayStreamSender::Shutdown () {

  ERROR_CODE ecTemp;
  
  if (!tBaseObject::IsInit() )  
    TraceMsg (tmt_WARN,S_relayStream_SENDER_ALREADY_SHUTDOWN,
              "tRelayStreamSender::Shutdown()");

  else { 
    /****
    [[U ecTemp := this->tIpcMultiTypeFifoAccessor::Shutdown (void) ]]
    ****/
    ecTemp = tIpcMultiTypeFifoAccessor::Shutdown ();
    if (ecTemp != OK) return Traceback (ecTemp,"tRelayStreamSender::Shutdown()");

  }
 
  TraceMark (S_relayStream_SENDER_SHUTDOWN_COMPLETE);
  return OK;
} 

/****
[[U tRelayStreamSender::IsOk (void) ]]
****/
BOOL tRelayStreamSender::IsOk () {
 
  ERROR_CODE ecErrCode;
  
  if (this->siPutMutexSemaphore == NULL) return FALSE;
  
  /****
  [[U return := tIpcMultiTypeFifoAccessor::IsOk(void)]]
  ****/
  return tIpcMultiTypeFifoAccessor::IsOk();

}
    
/****
[[U tRelayStreamSender::Put (TypeTagIn, pDataIn) ]]
****/ 
ERROR_CODE tRelayStreamSender::Put (const tTypeTag& TypeTagIn, const BYTE* pDataIn) {
  
  ERROR_CODE ecErrCode;
  
  if (!tBaseObject::IsInit()) 
    return Traceback (S_relayStream_SENDER_NOT_INITIALIZED,"tRelayStreamSender::Put()");

  /****
  [[U ecErrCode := ::semTake (this->siPutMutexSemaphore,RELAY_STREAM_SEND_MAX_WAIT) ]]
  ****/
  ecErrCode = semTake (this->siPutMutexSemaphore,RELAY_STREAM_SEND_MAX_WAIT);
    
  if (ecErrCode == ERROR) 
    return Traceback (errnoGet(), "tRelayStreamSender::Put()");                             
  
  /****
  [[U ecErrCode := this->tIpcMultiTypeFifoAccessor::Put (TagTypeIn,pDataIn) ]]
  ****/
  ecErrCode = tIpcMultiTypeFifoAccessor::Put (TypeTagIn,pDataIn);
  
  semGive (this->siPutMutexSemaphore);

  if  (ecErrCode == OK
    || ecErrCode == S_ipc_FIFO_FULL 
    //||  ecErrCode == S_ipc_TOO_MANY_ELEMENTS_IN_FIFO 
    )
    return ecErrCode;

  else return Traceback (ecErrCode,"tRelayStreamSender::Put()");  

}

/****
[[U tRelayStreamSender::Put (TypeTagIn, BufferIn) ]]
****/ 
ERROR_CODE tRelayStreamSender::Put (const tTypeTag& TypeTagIn, const tBuffer& BufferIn) {
  
  ERROR_CODE ecErrCode;
  
  if (!tBaseObject::IsInit()) 
    return Traceback (S_relayStream_SENDER_NOT_INITIALIZED,"tRelayStreamSender::Put()");

  /****
  [[U ecErrCode := ::semTake (this->siPutMutexSemaphore,RELAY_STREAM_SEND_MAX_WAIT) ]]
  ****/
  ecErrCode = semTake (this->siPutMutexSemaphore,RELAY_STREAM_SEND_MAX_WAIT);
    
  if (ecErrCode == ERROR) 
    return Traceback (errnoGet(), "tRelayStreamSender::Put()");                             
  
  /****
  [[U ecErrCode := this->tIpcMultiTypeFifoAccessor::Put (TagTypeIn,pDataIn) ]]
  ****/
  ecErrCode = tIpcMultiTypeFifoAccessor::Put (TypeTagIn,BufferIn);
  
  semGive (this->siPutMutexSemaphore);

  if (ecErrCode == OK || ecErrCode == S_ipc_FIFO_FULL || 
      ecErrCode == S_ipc_TOO_MANY_ELEMENTS_IN_FIFO ) return ecErrCode;
 
  else return Traceback (ecErrCode,"tRelayStreamSender::Put()");  

}


/****
[[U tRelayStreamSender::PutNoWait (TypeTagIn, pDataIn) ]]
****/ 
ERROR_CODE tRelayStreamSender::PutNoWait (const tTypeTag& TypeTagIn, const BYTE* pDataIn) {
  
  ERROR_CODE ecErrCode;

  if (!tBaseObject::IsInit()) 
    return Traceback (S_relayStream_SENDER_NOT_INITIALIZED,
                      "tRelayStreamSender::PutNoWait()");
  
  /****
  [[U ecErrCode := ::semTake (this->siPutMutexSemaphore,NO_WAIT) ]]
  ****/
  ecErrCode = semTake (this->siPutMutexSemaphore,NO_WAIT);
    
  if (ecErrCode == ERROR) 
    return Traceback (errnoGet(), "tRelayStreamSender::Put()");                             
  
  /****
  [[U ecErrCode := this->tIpcMultiTypeFifoAccessor::Put (TypeTagIn,pDataIn) ]]
  ****/
  ecErrCode = tIpcMultiTypeFifoAccessor::Put (TypeTagIn,pDataIn);
  
  /****
  [[U ::semGive (this->siPutMutexSemaphore) ]]
  ****/
  semGive (this->siPutMutexSemaphore);

  if (ecErrCode == OK || ecErrCode == S_ipc_FIFO_FULL || 
      ecErrCode == S_ipc_TOO_MANY_ELEMENTS_IN_FIFO ) return ecErrCode;

  else return Traceback (ecErrCode,"tRelayStreamSender::Put()");  

}



/****
[[U tRelayStreamReceiver::tRelayStreamReceiver (rIpcObjectAccessorToUse) ]]
****/
tRelayStreamReceiver::tRelayStreamReceiver (const tIpcFifoId& iiId) :
  
  /****
  [[U tIpcMultiTypeFifo::tIpcMultiTypeFifo (iiId) ]]
  ****/
  tIpcMultiTypeFifoAccessor (iiId) {
  
  /****
  [[U tBaseObject::GetInitError (void) ]]
  ****/
  if (tBaseObject::GetInitError() != OK) {
    TraceMsg (tmt_ERROR,tBaseObject::GetInitError(),"tRelayStreamReceiver::tRelayStreamReceiver");
    return;
  }
  
 TraceMark (S_relayStream_RECEIVER_INIT_COMPLETE);
 
}
  
  
/****
[[U tRelayStreamReceiver::~tRelayStreamReceiver (void) ]]
****/
tRelayStreamReceiver::~tRelayStreamReceiver () {

  if (tBaseObject::IsInit() ) {
    TraceMsg (tmt_WARN,S_relayStream_SHUTTING_DOWN_RECEIVER,
              "tRelayStreamReceiver::~tRelayStreamReceiver() ");
    /****
    [[U tRelayStreamReceiver::Shutdown (void)]]
    ****/
    this->Shutdown();
  }
  
  /****
  [[U tIpcMultiTypeFifoAccessor::~tIpcMultiTypeFifoAccessor (void) ]]
  ****/
}
  
  

/****
[[U tRelayStreamReceiver::Restart (void) ]]
****/
ERROR_CODE tRelayStreamReceiver::Restart () {
  
  ERROR_CODE ecTemp;

  if (tBaseObject::IsInit() )  {
    ecTemp = S_relayStream_RECEIVER_ALREADY_INITIALIZED;
    TraceMsg (tmt_WARN, ecTemp, "tRelayStreamReceiver::Restart()");
  }
 
  /****
  [[U ecTemp := this->tIpcMultiTypeFifoAccessor::Restart (void) ]]
  ****/
  else  ecTemp = tIpcMultiTypeFifoAccessor::Restart();
 
  if (ecTemp != OK) return Traceback (ecTemp,"tRelayStreamReceiver::Restart()");
  TraceMark (S_relayStream_RECEIVER_REINIT_COMPLETE);
  
  return OK;
}
  
/****
[[U tRelayStreamReceiver::Shutdown (void) ]]
****/
ERROR_CODE tRelayStreamReceiver::Shutdown () {

  ERROR_CODE ecTemp;
  
  if (!tBaseObject::IsInit() )  
    TraceMsg (tmt_WARN,S_relayStream_RECEIVER_ALREADY_SHUTDOWN,
              "tRelayStreamReceiver::Shutdown()");
 
 
  /****
  [[U ecTemp := this->tIpcMultiTypeFifoAccessor::Shutdown (void) ]]
  ****/
  else ecTemp = tIpcMultiTypeFifoAccessor::Shutdown();
  
  TraceMark (S_relayStream_RECEIVER_SHUTDOWN_COMPLETE);
  
  return OK;
}  
  

/****
[[U tRelayStreamReceiver::Get (TypeTagIn, pDataOut) ]]
****/
ERROR_CODE tRelayStreamReceiver::Get (tTypeTag& TypeTagIn, BYTE* pDataOut) {
  
  if (!tBaseObject::IsInit()) 
    return Traceback (S_relayStream_RECEIVER_NOT_INITIALIZED,
                      "tRelayStreamReceiver::Get()");
  
  /****
  [[U return := this->tIpcMultiTypeFifoAccessor::Get (TypeTagIn,pDataOut) ]]
  ****/                               
  else return tIpcMultiTypeFifoAccessor::Get(TypeTagIn,pDataOut);

}

