/* $RCSfile: token.cc,v $ Token Server and Client code */

/* CM Info: 
$Header: /home/chara/cvsroot/ople/sys/ipc/token/token.cc,v 1.1.1.1 1998/06/11 21:36:39 chara Exp $*/

/* Richard Johnson - Jet Propulsion Laboratory */



#include "token.h"
#include "tokenPrivate.h"
#include <errnoLib.h>
#include "tokenError.h"


const CiTokenServerPriority = 75;
const CiTokenServerOptions = VX_FP_TASK;
const CiTokenServerStackSize = 32768;


ostream& operator<< (ostream& stream,const TOKEN_FLAG& Flag) {

  switch (Flag) {
    case TF_IDLE : stream << "TF_IDLE";break;
    case TF_REQUEST : stream << "TF_REQUEST";break;
    case TF_GRANTED : stream << "TF_GRANTED";break;
    case TF_RETURN : stream << "TF_RETURN";break;
    default : stream << "UNKNOWN"; break;
  }
  return stream;
}

/****
[[U tTokenServer::tTokenServer (volatile TOKEN_FLAG *vtfTokenArrayToUse,
		             int iNumClientsToUse, 
		             int iServerTimeOutInSeconds)]]
****/
tTokenServer::tTokenServer ( volatile TOKEN_FLAG *vtfTokenArrayToUse,
		             int iNumClientsToUse, 
		             int iServerTimeOutInSeconds) :
    /****
    [[U tBaseObject::tBaseObject(void)]]
    ****/		             
    tBaseObject() {
    
  int i;
  
  this->SetInitError(OK);
  this->SetInit(TRUE);
  this->vtfTokenArray = vtfTokenArrayToUse;
  for (i = 0; i < iNumClientsToUse;i++) this->vtfTokenArray[i] = TF_IDLE;
  this->iNumClients = iNumClientsToUse;
  this->iServerTimeOutInTicks = sysClkRateGet () * 
                                iServerTimeOutInSeconds;
  this->bOwnToken = TRUE;

  this->iCurrentClient = 0;                  // eamck 7/8/97

  /****
  [[U this->tidPollTaskId := ::taskSpawn ("Poll", CiTokenServerPriority, 
					  CiTokenServerOptions,
			                  CiTokenServerStackSize, 
					  (FUNCPTR) &(this->Poll), 
			       (int) this, 0, 0, 0, 0, 0, 0, 0, 0, 0 )]]
  ****/
  this->tidPollTaskId = taskSpawn("Poll", CiTokenServerPriority, 
					  CiTokenServerOptions,
			                  CiTokenServerStackSize, 
					  (FUNCPTR) &(this->Poll), 
			       (int) this, 0, 0, 0, 0, 0, 0, 0, 0, 0 );
  if (this->tidPollTaskId == ERROR)
    this->SetInitError(Traceback(errnoGet(), "tTokenServer::tTokenServer"));

  TraceMark(S_ts_SUCCESSFULLY_COMPLETED_SERVER_INIT);


}

/****
[[U tTokenServer::~tTokenServer (void) ]]
****/
tTokenServer::~tTokenServer () {

  ERROR_CODE ecErrCode;

  /****
  [[U tTokenServer::Shutdown(void) ]]
  ****/
  if (this->IsInit() == TRUE) ecErrCode = this->Shutdown (); 
  
  /****
  [[U mecErrCode := ::taskDelete (this->tidPollTaskId)]]
  ****/
  ecErrCode = taskDelete (this->tidPollTaskId);
  
  if (ecErrCode == ERROR) 
    Traceback (errnoGet(),"tTokenServer::~tTokenServer()");

  TraceMark(S_ts_SUCCESSFULLY_COMPLETED_SERVER_DESTRUCTOR);
 
  /****
  [[ tBaseObject::~tBaseObject (void)]]
  ****/
} 

/****
[[U tTokenServer::Restart (void)]]
****/
ERROR_CODE tTokenServer::Restart () {
 
  ERROR_CODE ecErrCode = OK;
  
  if (this->IsInit() == TRUE) 
    TraceMsg (tmt_WARN,S_token_SERVER_ALREADY_RUNNING,"tTokenServer::Restart()");
  
  /****
  [[U ecErrCode := ::taskResume (this->tidPollTaskId)]]
  ****/
  else ecErrCode = taskResume (this->tidPollTaskId);
  
 if (ecErrCode == ERROR) 
   return Traceback (errnoGet(),"tTokenServer::Restart()");
 
 this->SetInit(TRUE);
 
 TraceMark(S_ts_SUCCESSFULLY_COMPLETED_SERVER_REINIT);

 return OK;  
}


/****
[[U tTokenServer::Shutdown (void) ]]
****/
ERROR_CODE tTokenServer::Shutdown () {


 ERROR_CODE ecErrCode = OK;
 
 if ( this->IsInit() == FALSE) 
    TraceMsg (tmt_WARN,S_token_SERVER_NOT_RUNNING,"tTokenServer::Restart()");

 /****
 [[U ecErrCode := ::taskSuspend (this->tidPollTaskId) ]]
 ****/
 else ecErrCode = taskSuspend (this->tidPollTaskId);
 
 if (ecErrCode == ERROR) 
   return Traceback (errnoGet(),"tTokenServer::Shutdown()");

 this->SetInit(FALSE);
 
 TraceMark(S_ts_SUCCESSFULLY_COMPLETED_SERVER_SHUTDOWN);
 return OK;  
}


/****
[[U tTokenServer::TokenClientRequestsToken (void) ]]

[[U tToknServer::TokenGrantedToClient (void) ]]
****/

ERROR_CODE tTokenServer::Poll () {
  
  int i = 0;
  TIME tCountDown; 

  TraceMark(S_ts_START_OF_POLL);
  
  FOREVER {

  tCountDown = this->iServerTimeOutInTicks;
  
  if (this->IsInit() == FALSE) 
     Traceback (S_token_SERVER_NOT_RUNNING,"tTokenServer::Poll()");

  /****
  [[U tTokenServer::CheckRequest(void)]]
  [[U tTokenServer::NextClient (void)]]
  ****/
  while ( this->CheckRequest() == FALSE ) {
     this->NextClient();
     //   taskDelay (1);
  }
 
  /****
  [[U tTokenServer::GrantRequest(void)]]
  ****/
  this->GrantRequest ();

  /****
  [[U tTokenServer::CheckReturn(void)]]
  ****/ 
  while ( (tCountDown > 0) && ( this->CheckReturn() == FALSE ) ) {
    tCountDown--;
    taskDelay (1);
   }    

  /****
  [[U tTokenServer::Steal (void) ]]
  ****/
  if ( tCountDown == 0)  this->Steal ();
  
  /****
  [[U tTokenServer::AcceptReturn (void) ]]
  ****/
  else this->AcceptReturn();
  
  /****
  [[U tTokenServer::NextClient (void) ]]
  ****/
  this->NextClient();

  }

 TraceMark(S_ts_END_OF_POLL);
     
}


/****
[[U tTokenServer::NextClient (void) ]]
****/
void tTokenServer::NextClient () {

  if (++this->iCurrentClient >= this->iNumClients) 
  {
     if (this->iCurrentClient > this->iNumClients)          // eamck 7/8/97
     {
        TraceMsg(tmt_ERROR,S_token_CLIENT_OUT_OF_RANGE,
					  "tTokenServer::NextClient");
     }
     this->iCurrentClient = 0;
     taskDelay (1); // Allows lower priority tasks to run by 
                    // delaying once/token request cycle.
  }
}

/****
[[U tTokenServer::CheckRequest (void) ]]
****/
BOOL tTokenServer::CheckRequest () {

  return ( this->vtfTokenArray[this->iCurrentClient] == TF_REQUEST );
}

/****
[[U tTokenServer::GrantRequest (void) ]]
****/
void tTokenServer::GrantRequest () {

  this->vtfTokenArray[this->iCurrentClient] = TF_GRANTED;
  this->bOwnToken = FALSE;
  TraceMark(S_ts_REQUEST_GRANTED);

}
    
/****
[[U tTokenServer::CheckReturn (void) ]]
****/
BOOL tTokenServer::CheckReturn () {

  return ( this->vtfTokenArray[this->iCurrentClient] == TF_RETURN );
}

/****
[[U tTokenServer::AcceptReturn  (void) ]]
****/
void tTokenServer::AcceptReturn () {

  this->vtfTokenArray[this->iCurrentClient] = TF_IDLE;
  this->bOwnToken = TRUE;
  TraceMark(S_ts_RETURN_ACCEPTED);
  
}

/****
[[U tTokenServer::Steal (void) ]]
****/
void tTokenServer::Steal () {

  this->vtfTokenArray[this->iCurrentClient] = TF_IDLE;
  this->bOwnToken = TRUE;
  TraceMsg (tmt_ERROR,S_token_SERVER_TIME_OUT,"TokenServer timeout waiting for return of token #%d\n",
					      this->iCurrentClient);

}

ostream& operator<< (ostream& stream, const tTokenServer& TokenServer) {

  int i;
  
  stream << "Token Server Contents" << endl <<
            (tBaseObject) TokenServer << endl <<
            "Number of Clients = " << TokenServer.iNumClients << endl <<
            "Server On ? " << TokenServer.bInit << endl <<
            "Server Owns Token ? " << TokenServer.bOwnToken << endl <<
            "Poll Task Id = " << TokenServer.tidPollTaskId << endl <<
            "Time out = " << TokenServer.iServerTimeOutInTicks << " ticks" << endl;
                     
  for (i = 0; i < TokenServer.iNumClients; i++ ) 
    stream << "Token Flag " << i << " " <<  TokenServer.vtfTokenArray[i] << endl;
 
  stream << "End of TokenServer Contents" << endl;
   
  return stream;
}
    
    
    



/****
[[U tTokenClient::tTokenClient (volatile TOKEN_FLAG *vtfTokenArray, 
                             int iClientId, int iTimeOutInSeconds ) ]]
****/
tTokenClient::tTokenClient (volatile TOKEN_FLAG *vtfTokenArray, 
                             int iClientId, int iTimeOutInSeconds ) :
                             
  /****
  [[U tBaseObject::tBaseObject(void) ]]
  ****/                              
  tBaseObject() {

  this->iTimeOutInTicks = sysClkRateGet () * iTimeOutInSeconds;
  this->vtfRequestFlag = &vtfTokenArray[iClientId];
  this->bOwnToken = FALSE;
  TraceMark(S_tc_SUCCESSFULLY_COMPLETED_CLIENT_INIT);
  
}

/****
[[U tTokenClient::~tTokenClient (void) ]]
****/
tTokenClient::~tTokenClient () {

  /****
  [[U tTokenClient::Shutdown (void) ]]
  ****/
  if (this->IsInit ()) this->Shutdown ();
  
  /****
  [[U tBaseObject::~tBaseObject(void) ]]
  ****/                              

}  


/****
[[U tTokenClient::Restart (void) ]]
****/
ERROR_CODE tTokenClient::Restart () {

  if (this->IsInit()) TraceMsg (tmt_WARN, S_token_CLIENT_ALREADY_INITIALIZED, "tTokenClient::Restart()");
  else tBaseObject::Restart ();
  TraceMark(S_tc_SUCCESSFULLY_COMPLETED_CLIENT_REINIT);
  return OK;
}

/****
[[U tTokenClient::Shutdown ()]]
****/
ERROR_CODE tTokenClient::Shutdown () {

  if (! tBaseObject::IsInit ()) TraceMsg (tmt_WARN,S_token_CLIENT_ALREADY_SHUTDOWN, "tTokenClient::Restart()" );
  else tBaseObject::Shutdown ();

  /****
  [[U tTokenClient::Return (void)]]
  ****/
  if (this->bOwnToken) this->Return();
  
  TraceMark(S_tc_SUCCESSFULLY_COMPLETED_CLIENT_SHUTDOWN);
  return OK;
}


/****
[[U tTokenClient::Request (void) ]]
****/
ERROR_CODE tTokenClient::Request () {

  TIME tTickCounter = this->iTimeOutInTicks;

  if (this->IsInit() == FALSE ) return Traceback (S_token_CLIENT_NOT_INITIALIZED,
          "tTokenClient::Request()");
  if (this->bOwnToken)  return Traceback (S_token_CLIENT_ALREADY_OWNS_TOKEN,
          "tTokenClient::Request()"); 

  /* wait for token server to make token ready first */
  while ( (*vtfRequestFlag != TF_IDLE) && (tTickCounter > 0) )
  {
      taskDelay(1);
      tTickCounter--;
  }

  if (tTickCounter == 0)
    return Traceback (S_token_CLIENT_TIME_OUT,"tTokenClient::Request()");

  if (tTickCounter > 0)
  {
    /* make token request */
    *vtfRequestFlag = TF_REQUEST;
  
    /* wait for server to reply */
    while ( (*this->vtfRequestFlag != TF_GRANTED)  && 
            (tTickCounter > 0) )  {      
      taskDelay (1);
      tTickCounter--;
    }
  TraceMark(S_tc_SUCCESSFULLY_COMPLETED_TOKEN_REQUEST);
    
  }
  
    /*
    [ *this.*vtfRequestFlag != TF_GRANTED within 
      *this.iClientWait ticks  -> 
        : *this.vtfRequestFlag := TF_IDLE
	: return S_tc_TC_TIME_OUT ]
    */
  if (tTickCounter == 0) {
    *this->vtfRequestFlag = TF_IDLE;
    return Traceback (S_token_CLIENT_TIME_OUT,"tTokenClient::Request()");
  }
    /*
    [ *this.bOwnToken := TRUE
      : return OK ]
    */
  else {
    this->bOwnToken = TRUE;
    return OK;
  }
}


/****
[[U tTokenClient::Return(void)]]
****/
ERROR_CODE tTokenClient::Return () {

  if (this->IsInit() == FALSE ) return Traceback (S_token_CLIENT_NOT_INITIALIZED,
 	"tTokenClient::Return()" );
 	
  if (!bOwnToken) return Traceback (S_token_CLIENT_DOESNT_OWN_TOKEN,
 	"tTokenClient::Return()" );

  *this->vtfRequestFlag = TF_RETURN;
  this->bOwnToken = FALSE;
  TraceMark(S_tc_SUCCESSFULLY_COMPLETED_TOKEN_RETURN);
  return OK;
}

ostream& operator<< (ostream& stream, const tTokenClient& TokenClient) {
 
   stream << "Token Client Contents:" << endl <<
            (tBaseObject) TokenClient << endl <<
            "Time out = " << TokenClient.iTimeOutInTicks << " ticks " << endl <<
            "Client owns token ? " << TokenClient.bOwnToken << endl <<
            "Token Flag = " << *TokenClient.vtfRequestFlag << endl <<
            "End of TokenClient Contents " << endl;
   return stream;
}
