/* dt1401Lib.c - routines to access dt1401 i/o card */
/*		sccs info:	@(#)dt1401Lib.c	1.5  9/1/94  */

/*
modification history
--------------------
01k,01Sep94,mmc    added dt1401IsPresent
01j,20Jan94,hs     changed second digital port to be an input
01i,19nov93,mmc    added vxMemProbe check
01h,11Oct93,mmc    added global dt1401ADData
01g,14Sep93,beh    made compatible with VxWorks 5.1
01f,31Aug91,mmc    Added some includes, commented out some unused vars,
				added VOIDFUNCPTR casts
01f,16Jul91,grw    Added dt1401FullInit function
01e,08May90,beh    Added prototypes in dt1401Lib.h, WORD etc., range check
01d,29Jan90,beh	Moved the various defines off to dt1401Config.h
01c,20Dec89,mmc	most user routines now return STATUS, unitialized
				global pointers now point somewhere. 
01b,01Dec89,mmc	changed a/d gain
01a,26Oct89,mmc	written	
*/

/*
DESCRIPTION
This module contains subroutines to interface with the data
translation dt1401 analog/digital i/o card.
*/

#include <vxWorks.h>
#include <vme.h>
#include <vxLib.h>
#include <stdio.h>

#ifdef VW_502
#include <68k/iv.h>
#include <vxProto.h>
#else
#include <iv.h>
#endif  /* VW_502 */

#include <sysLib.h>
#include <intLib.h>

#include "rtTypes.h"
#include "dt1401Lib.h"
#include "dt1401Config.h"

/* pointers to various registers on dt1401 card.
   Initialized by dt1401Init */

static WORD dt1401Tmp; /* initial destination of pointers */
volatile UWORD *pdt1401adcsr = 		(UWORD *)&dt1401Tmp;
volatile UWORD *pdt1401adchanreg = 	(UWORD *)&dt1401Tmp;
volatile UWORD *pdt1401paclk = 		(UWORD *)&dt1401Tmp;
volatile WORD  *pdt1401addata = 			&dt1401Tmp;
volatile WORD  *pdt1401dadata0 = 			&dt1401Tmp;
volatile WORD  *pdt1401dadata1 = 			&dt1401Tmp;
volatile UWORD *pdt1401dioreg = 	(UWORD *)&dt1401Tmp;

/* global AD value read on clock interrupt */
int dt1401ADData;

/*********************************************
*
* dt1401FullInit - initialize dt1401 card and library
*
* DESCRIPTION
* Perform address translation and init hardware
*
* RETURNS:
*   Echos error status of dt1401Init
*/

STATUS dt1401FullInit (void)
{
	if (ERROR == dt1401Init())
		return ERROR;

	/* set up a/d control/status register */
	{
		/*UWORD  adcsr_strt=			0x0001;*/
		/*UWORD  adcsr_to=				0x0002;*/
		/*UWORD  adcsr_inc_ena= 		0x0004;*/
		  UWORD  adcsr_hard_trg_ena=	0x0008;
		/*UWORD  adcsr_eos_int_ena=		0x0010;*/
		/*UWORD  adcsr_eos=				0x0020;*/
		  UWORD  adcsr_done_int_ena=	0x0040;
		/*UWORD  adcsr_done=			0x0080;*/
		  UWORD  adcsr_do0=				0x0100;
       	/*  UWORD  adcsr_do1=				0x0200; */
		/*UWORD  adcsr_scan_ena=		0x0400;*/
		/*UWORD  adcsr_not_used=		0x0800;*/
		  UWORD  adcsr_init_clr=		0x1000;
		/*UWORD  adcsr_err_clr= 		0x2000;*/
		/*UWORD  adcsr_err_int_ena=		0x4000;*/
		/*UWORD  adcsr_err=				0x8000;*/

		/* check for board */
		/* *pdt1401adcsr =   adcsr_init_clr; *//* initial clear */
		if (ERROR == vxMemProbe((char *)pdt1401adcsr, VX_WRITE,
				sizeof(UWORD), (char *)&adcsr_init_clr)) {
			printErr("dt1401FullInit: configuration error or board not present \n");
			return ERROR;
		}

		*pdt1401adcsr =	  adcsr_do0			/* set dio as output */
						| adcsr_hard_trg_ena /* enable hardware trigger */
						| adcsr_done_int_ena; /* generate int. on done */
	}

	/* set up a/d channel register */
 	{
		/* g1    g0    Gain */
		/*  0     0     1   */
		/*  0     1     2   */
		/*  1     0     4   */
		/*  1     1     8   */
		UWORD  adchanreg_g0=	0;
		UWORD  adchanreg_g1=	0;

		*pdt1401adchanreg = 0;	/* clear (gain=1; start, end channel=0) */
		*pdt1401adchanreg = (adchanreg_g0<<14) | (adchanreg_g1<<15);
	}

	/* set up pacer clk register */
	{
		  UWORD  paclk_ctl0=	0x0001;
		  UWORD  paclk_ctl1=	0x0002;
		/*UWORD  paclk_ctl2=	0x0004;*/
		/*UWORD  paclk_ctl3=	0x0008;*/
		  UWORD  paclk_ctl4=	0x0010;
		/*UWORD  paclk_ctl5=	0x0020;*/
		/*UWORD  paclk_res= 	0x0040;*/
		/*UWORD  paclk_cse=		0x0080;*/

		*pdt1401paclk = 0;	/* clear */
		*pdt1401paclk =   paclk_ctl4	/* set rate to 1kHz */
						| paclk_ctl1
						| paclk_ctl0;
	}

	dt1401DA0((WORD) 0);
	dt1401DA1((WORD) 1);
	return OK;
}

/*********************************************
*
* dt1401Init - initialize dt1401 library
*
* DESCRIPTION
* Performs address tranlation
*
* RETURNS:
*   ERROR if problems converting bus address; OK otherwise.
*/

STATUS dt1401Init (void)
{
	char *localAdrs;

	/* convert bus to local addresses */
	if ( sysBusToLocalAdrs ((int) DT1401_AM, (char *)DT1401_ADRS, &localAdrs)
						 == ERROR)
		return ERROR;
	/* printf ("busAdrs: %lxH, localAdrs: %lxH \n", DT1401_ADRS, localAdrs); */

	pdt1401adcsr = 		(UWORD *)	( localAdrs + 0);
	pdt1401adchanreg = 	(UWORD *)	( localAdrs + 2);
	pdt1401paclk = 		(UWORD *)	( localAdrs + 4);
	pdt1401addata = 	(WORD *)	( localAdrs + 6);
	pdt1401dadata0 = 	(WORD *)	( localAdrs + 8);
	pdt1401dadata1 = 	(WORD *)	( localAdrs + 10);
	pdt1401dioreg = 	(UWORD *)	( localAdrs + 12);

	return OK;
} 

/*********************************************
*
* dt1401IsPresent - checks for presence of board using vxMemProbe
*
* DESCRIPTION
* Performs address tranlation
*
* RETURNS:
*   TRUE if board present, FALSE otherwise or if error converting address
*/

BOOL dt1401IsPresent (void) {

	char foo[2];

	/* translate address */
	if (ERROR == dt1401Init())
		return FALSE;

	/* check for board */
	if (ERROR == vxMemProbe((char *)pdt1401adcsr, VX_READ, sizeof(UWORD), foo))
		return FALSE;
	else
		return TRUE;
}

/*********************************************************
*
* dt1401DA0 - output a value on d/a converter 0
*
*/

void dt1401DA0 (WORD value)
{
    *pdt1401dadata0 = value;
}

/*********************************************************
*
* dt1401DA1 - output a value on d/a converter 1
*
*/

void dt1401DA1 (WORD value)
{
    *pdt1401dadata1 = value;
}

/*********************************************************
*
* dt1401DA - output a value on d/a converter specified
*
*/

void dt1401DA (int channel, WORD value)
{
  switch(channel)
  {
   case(0):
    *pdt1401dadata0 = value;
    break;

   case(1):
    *pdt1401dadata1 = value;
    break;

   default:
#ifdef DEBUG
    errnoSet(S_dt1407_INVALID_CHANNEL);
#endif
  }
}

/*********************************************************
*
* dt1401AD - input a value from the a/d converter
*
*/

void dt1401AD (WORD *pvalue)
{
		*pvalue = *pdt1401addata;
}

/***********************************************************
*
* dt1401DI - input a value on the digital input lines
*
*/
UWORD dt1401DI(void)
{
  UWORD value;
  value = *pdt1401dioreg;
  
  return(value);
}

/***********************************************************
*
* dt1401DO - output a value on the digital output lines
*
*/
void dt1401DO(UWORD value)
{
	*pdt1401dioreg = value;
}

/*************************************************************
*
* dt1401ClkRateSet - set frequency of pacer clock
*
*	RETURNS:
*		closest rate to desired rate (Hz)
*
*/
int dt1401ClkRateSet(int rate)
{
	/* scale factors for 3 mantissa and 3 exponent bits */
	static int mantissa[]={1,4,2,6,10,5,3,12};
	static int exponent[]={1,10000,100,1000000,10,100000,1000,10000000}; 
	int freq0=600000;	/* pacer clock freq */
	int mi,ei,omi=0,oei=0,delta,oldDelta;

#ifdef DEBUG
	/* Check for error */
	if (rate < 0)
	{
		errnoSet(S_dt1401_NEGATIVE_RATE);
		return(ERROR);
	}
#endif
	/* find mantissa and exponent bits closest to desired rate */
	oldDelta=freq0;
	for(mi=0;mi<=7;++mi){
		for (ei=0;ei<=7;++ei){
			delta = freq0/(mantissa[mi]*exponent[ei]) - rate;
			delta = (delta >= 0) ? delta : -delta;
			if( delta < oldDelta) {
				omi=mi;
				oei=ei;
				oldDelta=delta;
			}
		}
	}
	/* replace lowest 6 bits of paclk with new rate */
	*pdt1401paclk &= (UWORD) ~077;
	*pdt1401paclk |= (UWORD) (8*oei+omi);

	/* return closest frequency */
	return (freq0/(mantissa[omi]*exponent[oei]));
}

/****************************************************************
*
* dt1401ClkRateGet - get rate of pacer clock
*
*	RETURNS:
*		rate of clock
*
*/
int dt1401ClkRateGet(void)
{
	/* see comments in dt1401ClkRateSet */
	static int mantissa[]={1,4,2,6,10,5,3,12};
	static int exponent[]={1,10000,100,1000000,10,100000,1000,10000000};
	int freq0=600000;
	int omi,oei,tmp;

	tmp = *pdt1401paclk & 077;
	oei=tmp/8;
	omi=tmp-8*oei;
	return (freq0/(mantissa[omi]*exponent[oei]));
}

/******************************************************************
*
* dt1401ClkEnable - enable pacer clock
*
*/
STATUS dt1401ClkEnable(void)
{
	UWORD  paclk_res= 	0x0040;
	*pdt1401paclk |= paclk_res; /* set reset(enable) bit in paclk */
	return OK;
}

/****************************************************************
*
* dt1401ClkDisable - disable pacer clock
*
*/
STATUS dt1401ClkDisable(void)
{
	UWORD  paclk_res= 	0x0040;
	*pdt1401paclk &= ~paclk_res; /* clear reset(enable) bit in paclk */
	return OK; 
}






FUNCPTR dt1401ClkRoutine;
int dt1401ClkArg;

/****************************************************************
*
* dt1401ClkInt - handle a dt1401 interrupt
*
*/
void dt1401ClkInt(void)
{
	dt1401ADData = *pdt1401addata; /* acknowlege interrupt */
	(*dt1401ClkRoutine)(dt1401ClkArg); /* call user routine */
}

/****************************************************************
*
* dt1401ClkConnect - connect a user routine to the dt1401 clock interrupt
*
*/
STATUS dt1401ClkConnect(FUNCPTR routine, int arg)
{
	STATUS status;

#ifdef DEBUG
	if( ((int)routine == 0) || ((int)routine == -1) ) /* check for invalid address */
	{
		errnoSet(S_dt1401_NULL_FUNC_POINTER);
		return ERROR;
	}
#endif
	dt1401ClkRoutine = routine; /* set clock interrupt routine */
	dt1401ClkArg = arg;
	status = intConnect ((VOIDFUNCPTR *)INUM_TO_IVEC(DT1401_VECTOR),
								(VOIDFUNCPTR) dt1401ClkInt, 0);
	if(status == ERROR)
		return ERROR;
	return (sysIntEnable ((int) DT1401_INT)); /*enable this bus interrupt level */
}

/****************************************************************
*
* dt1401ClkDisconnect - disconnect a routine from the clk interrupt
*
*/
STATUS dt1401ClkDisconnect(void)
{
	return sysIntDisable ((int) DT1401_INT);
}


/****************************************************************
*
* dt1401IntHand - sample interrupt handler
*
* use dt1401ClkConnect(dt1401IntHand)
*
*/
void dt1401IntHand(void)
{
	*pdt1401dadata1=1024;	/* toggle da1 for scope display */
	*pdt1401dadata1=0;	/* toggle da1 for scope display */
}

/****************************************************************
*
* dt1401BackTask - sample background task
*
* from shell,use sp dt1401BackTask
* for uniform int response time, disable sysClk (sysClkDisable)
*
*/
void dt1401BackTask(void)
{
	for(;;){
		*pdt1401dadata0=1024;	/* toggle da0 for scope display */
		*pdt1401dadata0=0;	/* toggle da0 for scope display */
	}
}
