/* laserLib.h  -  header file for laserLib.c library */
/* sccs info:       @(#)laserLib.h	1.1  11/4/93  */

/*
modification history
--------------------
01f,beh,23May90     Added 64-bit stuff, locking, laserFullInit
01e,beh,09May90     Added function prototypes, changed from int to LWORD
01d,beh,05May90     Added LASER_ prefix to identifiers
01c,beh,19Feb90     Moved user-configuration info off to laserConfig.h
01b,beh,19Feb90     Put source code in standard format
01a,beh,24Jan90     written, debugged
*/

#ifndef INC_laserLibh
#define INC_laserLibh

#include "rtModNum.h"
/* #include "laserRead64.h" */

/* Number of counters on the laser board & half that number */
/* These have to do with the architecture and remain the same even
 * if you have an underpopulated board.
 */
#define LASER_NUM_COUNTERS 12
#define LASER_HALF_COUNTERS 6

/***************************************************************/
/* Pointers to various addresses & registers on the laser card */
/* The various laser counters                      */
#define LASER_TEST_MODULE     0x00
#define LASER_COUNTER(N)      (N << 4)
#define LASER_STATUS_MODULE   0xf0

/* The Current Position Registers & Fiducial Position Registers */
/* Read from these addresses		 */
#define LASER_CPR_MSW 0x0
#define LASER_CPR_LSW 0x4
#define LASER_FPR_MSW 0x8
#define LASER_FPR_LSW 0xc

/* The Mode and Command Words for each counter */
/* Write to these addresses                    */
#define LASER_MODE_WORD    0x0
#define LASER_COMMAND_WORD 0x8

/* Test and status module registers */
/* Status registers: For reading    */
#define LASER_TEST_SIGNAL_STATUS_INT      0x4 + LASER_TEST_MODULE
#define LASER_UNKNOWN_STATUS              0x6 + LASER_TEST_MODULE
#define LASER_TEST_SIGNAL_STATUS_FRACINT  0xc + LASER_TEST_MODULE
#define LASER_CLOCK_STATUS                0xe + LASER_TEST_MODULE

#define LASER_SYNC_MODE_HI                0x4 + LASER_STATUS_MODULE
#define LASER_FID_WAIT_STATES             0x6 + LASER_STATUS_MODULE
#define LASER_SYNC_MODE_LO                0xc + LASER_STATUS_MODULE
#define LASER_FID_STATES                  0xe + LASER_STATUS_MODULE

/* Test module control registers: for writing */
#define LASER_TEST_CNT_HI                 0x6 + LASER_STATUS_MODULE
#define LASER_TEST_CNT_LO                 0xe + LASER_STATUS_MODULE
#define LASER_TEST_SIG_STATUS_HI          0x4 + LASER_TEST_MODULE
#define LASER_TEST_SIG_STATUS_LO          0xc + LASER_TEST_MODULE

   
/***********************************************************/
/* Various values for command and control bits             */
/* Mode word:    */
#define LASER_MODE_CPR_FLOWTHROUGH      0x0
#define LASER_MODE_CPR_SYNC_TO_REF      0x1
#define LASER_MODE_CPR_SYNC_TO_EXT      0x2
#define LASER_MODE_FPR_LOAD_ON_FIDUCIAL 0x3

/* Command Word  */
#define LASER_CMD_LEAVE_TEST_MODE       0x0
#define LASER_CMD_ENTER_TEST_MODE       0x1
#define LASER_CMD_CLEAR_COUNTER         0x2
#define LASER_CMD_RESET_WAIT_STATES     0x3

/* Clear counter command -- meaning of various modes for clearing
 * the counters				*/
#define LASER_CLEAR_IMMED       LASER_MODE_CPR_FLOWTHROUGH
#define LASER_CLEAR_ON_REF      LASER_MODE_CPR_SYNC_TO_REF
#define LASER_CLEAR_ON_EXT      LASER_MODE_CPR_SYNC_TO_EXT
#define LASER_CLEAR_ON_FIDUCIAL LASER_MODE_FPR_LOAD_ON_FIDUCIAL

#define LASER_HEALTHY_REF		0x01
#define LASER_HEALTHY_2_MHZ		0x02
#define LASER_HEALTHY_16_MHZ	0x04
#define LASER_HEALTHY_64_MHZ	0x10
#define LASER_HEALTHY_EXT		0x20
#define LASER_HEALTHY_PLL		0x40

#ifdef LASER_NO_EXT_SYNC
#define LASER_HEALTHY_CLOCKS    (LASER_HEALTHY_REF | LASER_HEALTHY_2_MHZ | LASER_HEALTHY_16_MHZ | LASER_HEALTHY_PLL)
#else
#define LASER_HEALTHY_CLOCKS    (LASER_HEALTHY_REF | LASER_HEALTHY_2_MHZ | LASER_HEALTHY_16_MHZ | LASER_HEALTHY_EXT | LASER_HEALTHY_PLL)
#endif

#define LASER_LONG_TEST_POS 0x05a5a5a5
#define LASER_LONG_TEST_NEG 0xfa5a5a5a
#define LASER_FAST_TEST_POS 0x000005a5
#define LASER_FAST_TEST_NEG 0xfffffa5a

#define LASER_TEST_FRAC_BITS 0x7

#define LASER_OFFSET_FRAC 0x2

/***********************************************************/

/* Error codes returned by laserBoardInit */
#define S_laser_INVALID_COUNTER        (M_laserLib | 1)
#define S_laser_NO_REFERENCE           (M_laserLib | 2)
#define S_laser_NO_64_MHZ              (M_laserLib | 3)
#define S_laser_NO_16_MHZ              (M_laserLib | 4)
#define S_laser_NO_2_MHZ               (M_laserLib | 5)
#define S_laser_NO_EXTERNAL_CLOCK      (M_laserLib | 6)
#define S_laser_UNKNOWN_CLOCK_ERROR    (M_laserLib | 7)
#define S_laser_UNKNOWN_NOT_PRESENT    (M_laserLib | 8)
#define S_laser_INIT_INVALID_ARGUMENT  (M_laserLib | 9)
#define S_laser_INVALID_SYNC_MODE      (M_laserLib | 10)
#define S_laser_COUNTER_FAILED_POS_SLEW_TEST       (M_laserLib | 11)
#define S_laser_COUNTER_FAILED_RET_TO_ZERO_TEST    (M_laserLib | 12)
#define S_laser_COUNTER_FAILED_NEG_SLEW_TEST       (M_laserLib | 13)
#define S_laser_COUNTER_FAILED_CLEAR_TEST          (M_laserLib | 14)
#define S_laser_COUNTER_FAILED_FRACTION_TEST       (M_laserLib | 15)
#define S_laser_INVALID_BOOL_VALUE     (M_laserLib | 16)
#define S_laser_BOARD_NOT_INITIALIZED  (M_laserLib | 17)
#define S_laser_NULL_POINTER_ARGUMENT  (M_laserLib | 18)
#define S_laser_PLL_OUT_OF_LOCK        (M_laserLib | 19)


STATUS laserInit(int numCounters);
STATUS laserFullInit(int numCounters);
STATUS laserTestModeEnable(int counter);
STATUS laserTestModeDisable(int counter);
STATUS laserSetSyncMode(int counter, int newMode);
STATUS laserClearCounter(int counter);
STATUS laserCheckClocks(void);
void laserRead64Unlocked(volatile LWORD *fromAddress, LWORD *p64bitNum);
volatile LWORD *laserCprAddr(int counter);
volatile LWORD *laserFprAddr(int counter);
STATUS laserCpr(int counter, QWORD *contents);
STATUS laserCprHi(int counter,LWORD *high);
STATUS laserCprLo(int counter,LWORD *low);
STATUS laserCprInt(int counter, LWORD *ival);
STATUS laserCprFrac(int counter,LWORD *fval);
STATUS laserFpr(int counter, QWORD *contents);
STATUS laserFprHi(int counter, LWORD *high);
STATUS laserFprLo(int counter, LWORD *low);
STATUS laserFprInt(int counter, LWORD *ival);
STATUS laserFprFrac(int counter,LWORD *fval);
STATUS laserLoadFprOnFiducial(int counter);
STATUS laserClearCounterOnFiducial(int counter);
STATUS laserResetFiducialWait(int counter);
int laserTestSignalStatus(void);
BOOL laserTestDone(void);
void laserSetTestCount(int numCounts);
int laserSyncMode(int counter);
BOOL laserFiducialWaitState(int counter);
BOOL laserFiducialState(int counter);
STATUS laserUnknownStatus(int counter);
void laserTestWait(void);
STATUS laserFastTest(int counter);
STATUS laserLongTest(int counter);
STATUS laserFastTestAll(void);
STATUS laserLongTestAll(void);


#endif /* INC_laserLibh */
