/* mpv955_test.cc */

#include "mpv955_test.h"
#include "rtTypes.h"
#include <vme.h>
#include <stdio.h>

/* register address pointers */
static volatile UWORD *pmpv955CntrlStatusReg;
static volatile UWORD *pmpv955StartReg;
static volatile UWORD *pmpv955BaseDataAdr;
static volatile UWORD *pmpv955StartAdrReg;
static volatile UWORD *pmpv955StopAdrReg;
static volatile UWORD *pmpv955RateTimerReg;
static char *localAdrs;	/* convert bus to local addresses */

#define DATA_MEMORY_SIZE 16384       /* size of data memory */

/* Control Register */
static UWORD  internal_trigger   = 	0x0000;
static UWORD  ext_trigger_selct   =     0x0001;
static UWORD  event_trigger       = 	0x0002;
static UWORD  continuous_mode_select =  0x0000;
static UWORD  one_shot_mode_select=	0x0004;
static UWORD  timeout_enable         =  0x0000;
static UWORD  timeout_disable     =	0x0008;

#define NUM_CHANNEL 0 
/* number of the channel to be driven by mpv955 */


UWORD RateTimerConv(int delay) /* conversion of delay in us to reg format */
{
if (delay > 127) return(0xFF00);
else if (delay < 2) return(0xFFFC);
     else return(0xFFFF - (2 * delay)); 
}


void start(void)
    {
    *pmpv955StartReg = 0;
    }


STATUS MPV955Init (void)
{
  if ( sysBusToLocalAdrs((int) MPV955_AM, (char *)MPV955_ADRS, &localAdrs) == ERROR)
    return ERROR;

  printf ("busAdrs: %xH, localAdrs: %lxH \n", MPV955_ADRS, localAdrs);

  pmpv955BaseDataAdr = 	  (UWORD *)	( localAdrs + 0);
  pmpv955CntrlStatusReg = (UWORD *)	( localAdrs + 0x8000);
  pmpv955StartAdrReg =    (UWORD *)     ( localAdrs + 0x8002);
  pmpv955StopAdrReg =     (UWORD *)     ( localAdrs + 0x8004);
  pmpv955RateTimerReg =   (UWORD *)     ( localAdrs + 0x8008);
  pmpv955StartReg =       (UWORD *)	( localAdrs + 0xc002);
  
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNEL) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | continuous_mode_select  /* continuously cycle thru input data */
        | internal_trigger;       /* determine output rate of dac with
				       onboard time rate reg */
  printf("control status reg: %xH\n", *pmpv955CntrlStatusReg);

  *pmpv955StartAdrReg = 0x0;
  *pmpv955BaseDataAdr = 0x8000; 
  *pmpv955StopAdrReg = 0x0;
  *pmpv955RateTimerReg = 0xFF00; 
  
  printf("MPV955 Init done. \n");
  
  return OK;
}


STATUS tri_wave(void)
{
  int i;
  static volatile UWORD *addressPointer;
    
  *pmpv955StartAdrReg = 0x0;
  addressPointer = pmpv955BaseDataAdr;
  
  for (i = 0; i < DATA_MEMORY_SIZE-1; i++) 
    {
   if (i< DATA_MEMORY_SIZE/2)  *addressPointer++ = (UWORD)(8*i);
   else *addressPointer++ = (UWORD)(8 * (DATA_MEMORY_SIZE - i - 1)); 
    }

  *pmpv955StopAdrReg = (--i);
  *pmpv955RateTimerReg = RateTimerConv(10);  /* 1 us clock */
  
  start();
  
  return OK;
}


STATUS square_wave(int period)
{
  static volatile UWORD *addressPointer;
  
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNEL) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | continuous_mode_select   /*  */
	| event_trigger;           /*  */

  *pmpv955StartAdrReg = 0x0;
  addressPointer = pmpv955BaseDataAdr;
  *addressPointer++ = 0xffff;
  *addressPointer = 0xe000;
  *pmpv955StopAdrReg = 0x1;
  *pmpv955RateTimerReg = RateTimerConv(period);  /* 1us clock */
  
  start();
  
  return OK;
}

STATUS square_wave2(int period)
{
  static volatile UWORD *addressPointer;
  int i;
  
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNEL) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | continuous_mode_select   /*  */
	| event_trigger;           /*  */

  *pmpv955StartAdrReg = 0;
  addressPointer = pmpv955BaseDataAdr;
  for (i=0;i<period;i++) *addressPointer++ = 0xffff;
  for (i=0;i<period;i++) *addressPointer++ = 0xe000;
  *pmpv955StopAdrReg = 2*period-1;
  *pmpv955RateTimerReg = RateTimerConv(2);  /* 2us clock */
  
  start();
  
  return OK;
}


STATUS square_pulse(int period)
{
  static volatile UWORD *addressPointer;
  
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNEL) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | one_shot_mode_select   /*  */
	| event_trigger;           /*  */
  
  *pmpv955StartAdrReg = 0x0;
  addressPointer = pmpv955BaseDataAdr;
  *addressPointer = 0xc000;
  *(++addressPointer) = 0xFFFF;  
  *(++addressPointer) = 0xFFFF;
  *pmpv955StopAdrReg = 0x2;
  *pmpv955RateTimerReg = RateTimerConv(period);  /* 1us clock */
  
  start();
  
  return OK;
}


STATUS tri_pulse(void)
{
  int i;
  static volatile UWORD *addressPointer;
    
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNEL) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | one_shot_mode_select   /*  */
	| event_trigger;           /*  */
	  
  *pmpv955StartAdrReg = 0x0;
  addressPointer = pmpv955BaseDataAdr;
  
  for (i = 0; i < DATA_MEMORY_SIZE-1; i++) 
    {
   if (i< DATA_MEMORY_SIZE/2)  *addressPointer = (UWORD)(8*i);
   else *addressPointer = (UWORD)(1 + 8 * (DATA_MEMORY_SIZE - i - 1)); 
   addressPointer++;
    }

  *pmpv955StopAdrReg = (--i);
  *pmpv955RateTimerReg = RateTimerConv(10);  /* 1 us clock */
  start();
  
  return OK;
}


/*****************/
/* test function */

  void test(void)
    {
    MPV955Init();
    tri_wave();
    }

