/* mpv955_test.cc */

#include "mpv955_test.h"
#include "rtTypes.h"
#include "/opt/wind/target/h/vme.h"
#include <stdio.h>

/* register address pointers */
static volatile UWORD *pmpv955CntrlStatusReg;
static volatile UWORD *pmpv955StartReg;
static volatile UWORD *pmpv955BaseDataAdr;
static volatile UWORD *pmpv955StartAdrReg;
static volatile UWORD *pmpv955StopAdrReg;
static volatile UWORD *pmpv955RateTimerReg;
static volatile UWORD *addressPointer;
static char *localAdrs;	/* convert bus to local addresses */

static UWORD  one_dac_channel     =    0x0000;
static UWORD  start_wave_gen        =  0x0000;
#define DATA_MEMORY_SIZE 16384       /* size of data memory */

/* Control Register */
static UWORD  internal_trigger   = 	0x0000;
static UWORD  ext_trigger_selct   =     0x0001;
static UWORD  normal_trigger     =      0x0000;
static UWORD  event_trigger       = 	0x0002;
static UWORD  continuous_mode_select =  0x0000;
static UWORD  one_shot_mode_select=	0x0004;
static UWORD  timeout_enable         =  0x0000;
static UWORD  timeout_disable     =	0x0008;
#define NUM_CHANNELS 1 
/* number of d to a channels to be driven by mpv955 */


UWORD RateTimerConv(int delay) /* conversion of delay in us to reg format */
{
if (delay > 127) return(0xFF00);
else if (delay < 2) return(0xFFFC);
     else return(0xFFFF - (2 * delay)); 
}


STATUS MPV955Init (void)
{
  int i;
  
  if ( sysBusToLocalAdrs((int) MPV955_AM, (char *)MPV955_ADRS, &localAdrs) == ERROR)
    return ERROR;

  printf ("busAdrs: %lxH, localAdrs: %lxH \n", MPV955_ADRS, localAdrs);

  pmpv955BaseDataAdr = 	  (UWORD *)	( localAdrs + 0);
  pmpv955CntrlStatusReg = (UWORD *)	( localAdrs + 0x8000);
  pmpv955StartAdrReg =    (UWORD *)     ( localAdrs + 0x8002);
  pmpv955StopAdrReg =     (UWORD *)     ( localAdrs + 0x8004);
  pmpv955RateTimerReg =   (UWORD *)     ( localAdrs + 0x8008);
  pmpv955StartReg =       (UWORD *)	( localAdrs + 0xc002);
  
  *pmpv955CntrlStatusReg =  (UWORD) ((NUM_CHANNELS - 1) << 4)   
    | timeout_disable         /* disable onboard watchdog timer */
      | continuous_mode_select  /* continuously cycle thru input data */
	| normal_trigger           /* use start reg to start stroke */
	  | internal_trigger;       /* determine output rate of dac with
				       onboard time rate reg */
  printf("control status reg: %xH\n", *pmpv955CntrlStatusReg);

  *pmpv955StartAdrReg = 0x0;
  addressPointer = pmpv955BaseDataAdr;
  for (i = 0; i < DATA_MEMORY_SIZE; i++) 
    {
    *addressPointer++ = (UWORD)(4*i); 
    }
  *pmpv955StopAdrReg = (UWORD) (--addressPointer);
  
  *pmpv955RateTimerReg = RateTimerConv(10); /* 100 us clock */
  
  return OK;
}


/*****************/
/* test function */


  void test(void)
    {
    MPV955Init();
    printf("MPV955 Init done. \n");
    *pmpv955StartReg = 0;
    }


