/* recItpDline.c - records data from the ITP delay line through 
   laser card
*/

#include <vxWorks.h>
#include <iv.h>
#include <stdioLib.h>
#include <sysLib.h>
#include <intLib.h>
#include <taskLib.h>
#include <rtTypes.h> 
#include <recLibB.h>
#include <dt1401Lib.h>
#include <dt1401Config.h>
#include <irigLib.h>
#include <laserLib.h>
#include <pztMotorLib.h>
#include "recItpDlineF.h"

/* DEFINES (variables for vxWorks debugging) */
#define NUM_LASER_RETRYS 1000

int LASER_COUNTER_NUM = 1;
double FILTER_WIDTH = 1000.0;
int NUM_PZTS = 4;
int NUM_MOTORS = 2;
int PZT_COUNTER_NUM = 4;

/* Variables */
QWORD qLaserPos;		/* holds entire 40-bit word from laser card */
LWORD lLaserPosHi;	/* holds data from hi part of laser card */
LWORD lLaserPosLo;	/* holds data from lo part of laser card */
int dispScaleM = 1;	/* mult. scale factor for displaying to D/A */
int dispScaleD = 16;	/* div. scale factor for displaying to D/A */
double avg = 0.0;	/* avg value for low-pass filter for D/A display */
BOOL analogOn = TRUE;	/* TRUE: turns on analog output */


/*********************************************
*
* recItpDlineInit - Initializes clock (either
*                   irig or dt1401 to
*                   read from delay line
*
* This routine initializes the slow laser card,
* and sets up irig clock to read data from 
* the slow laser card counter at clkRate. After
* initializing, type recOpen(<filename>) to start recording.
* type recClose() to end recording.
*
* RETURNS:  OK or ERROR
*
*********************************************/

STATUS recItpDlineInit(int clkRate)
{
   int i, actClkRate;
/*
   STATUS status;
*/

   printf("Initializing...\n");

   if(irigFullInit() == ERROR)
/*
   if(dt1401FullInit() == ERROR)
*/
   {
      printf("Error initializing clock card\n");
      return(ERROR);
   }

   /* set clock rate */
   if((actClkRate=irigClkRateSet(clkRate)) == ERROR)
/*
   if((actClkRate=dt1401ClkRateSet(clkRate)) == ERROR)
*/
   {
      printf("Error setting clock rate\n");
      return(ERROR);
   }
   printf("Sample rate is %d Hz\n",actClkRate);

   /* get ready to read by connecting the interrupt to itpLaserRead */
   if(irigClkConnect(itpLaserRead,0) == ERROR)
/*
   if(dt1401ClkConnect(itpLaserRead,0) == ERROR)
*/
   {
      printf("Error Connecting clock to interrupt function\n");
      return(ERROR);
   }

   /* enable clock */
   if(irigClkEnable() == ERROR)
/*
   if(dt1401ClkEnable() == ERROR)
*/
   {
      printf("Error enabling clock\n");
      return(ERROR);
   }

   if (pztMotorFullInit(NUM_PZTS, NUM_MOTORS) != OK)
   {
      printErr("Error initializing the pztMotor card\n");
      return ERROR;
   }
   pztReset(PZT_COUNTER_NUM);

   for (i=0 ; (i<=NUM_LASER_RETRYS && laserFullInit(1)!=OK) ; i++) {
     if (i == NUM_LASER_RETRYS)
     {
       printErr("Error initializing the laser card\n");
       return ERROR;
     }
   }
   laserClearCounter(LASER_COUNTER_NUM);


   printf("Done Initializing\n\n");

   return OK;
}  


/*********************************************
*
* itpLaserRead - interrupt fnct to read laser card
*
*
* RETURNS:  OK
*
*********************************************/

STATUS itpLaserRead(void)
{

   /*  FOR TESTING */
/*
   static WORD philstemp = 0;
   philstemp += 256;
   dt1401DA0(philstemp);
   printf("%5d",philstemp);
*/

   /* Read laser card, output to recInt.		       *
   *  Average data and output to D/A if analogOn is TRUE       */

   laserCpr(LASER_COUNTER_NUM, &qLaserPos);

   lLaserPosHi = qLaserPos.high;
   lLaserPosLo = qLaserPos.low;

   /* sign-extend lLaserPosHi */
/*
   if(lLaserPosHi & 0x00000080)
   {
      lLaserPosHi |= 0xffffff00;
   }
   else
   {
      lLaserPosHi &= 0x000000ff;
   }
*/

   /* record data via recInt 
    *  to record LS32-bits, comment out
    *  recInt(lLaserPosHi)  */

   /* recInt(lLaserPosHi); */
   recInt(lLaserPosLo);

   if (analogOn)
   {
/*
      avg = (avg * (FILTER_WIDTH - 1.0) + lLaserPosLo) / FILTER_WIDTH;
      dt1401DA0(dispScaleM*(lLaserPosLo - avg)/dispScaleD);
      pztVelocitySet(PZT_COUNTER_NUM, dispScaleM*(lLaserPosLo-avg)/dispScaleD);
*/
      pztVelocitySet(PZT_COUNTER_NUM, dispScaleM*(lLaserPosLo)/dispScaleD);
/*
      pztVelocitySet(PZT_COUNTER_NUM, 0);
      pztVelocitySet(PZT_COUNTER_NUM, 0x4000);
*/
   }

   return OK;
}


void recmotor (char *, int, int);

/****************
* 
* recmotor - Quickie test script for BobG.
*
*/
void recmotor(char *sFilename, int numSeconds, int motorCommand)
{
  motorVelocitySet(2, motorCommand);
  taskDelay(1);
  recOpen(sFilename);
  taskDelay(sysClkRateGet()*numSeconds);
  recClose();
  motorReset(2);
}
