/* rtModNum.h - Module number definition for use with makeStatTbl */
/*		sccs info:	@(#)rtModNum.h	1.22	29 Sep 1994	*/

/*
modification history
--------------------
01n,25Jan94,grw         added definition for filtWheelLib
01m,15Oct93,grw         added definition for M_st
01l,05Aug93,grw         added gpibLib
01k,23Jul93,grw         added photonLib
01j,28Jun93,beh         added siderostat, other palomar stuff
01i,29Apr93,grw         added CCD Timing Board
01h,16Apr92,beh         added lots of new stuff
01g,03Oct90,dlp         added pztMotorLib
01f,11May90,beh         Added #ifndef INC...
01e,08May90,beh         Added dt1401Lib and irigLib
01d,08Mar90,arg         added adcLib and dioLib
01c,07Mar90,arg         added dacLib
01b,19Feb90,beh         added laserLib
01a,03Jan90,beh		written
*/

/*
DESCRIPTION
These module numbers are used by makeStatTbl when it makes the file statTbl.c,
which is then compiled into a special symbol table which can be used by VxWorks
for the printErrno function.  The symbol table, statSymTbl, is created by
running: makeStatTbl hdir >statTbl.c, where hdir is the directory containing
this file.  For the VxWorks files in vw/h, this is normally done automatically
by the vw/config makefile.  In order to include these updates, we must remake
VxWorks, including our new symbol table.  See vwman makeStatTbl for more info.
*/

#ifndef INCrtModNumh
#define INCrtModNumh

/* I've started our module numbers at 1024 (1-500 are reserved for VxWorks) */
#define M_schedLib      (1024 << 16)
#define M_sockio        (1025 << 16)
#define M_laserLib      (1026 << 16)
#define M_dacLib        (1027 << 16)    /* Datel DVME-626 dac drivers */
#define M_adcLib        (1028 << 16)    /* Burr-Brown MPV906 adc drivers */
#define M_dioLib        (1029 << 16)    /* Acromag 9460 digital i/o drivers */
#define M_dt1401Lib     (1030 << 16)    /* Data translation A/D / D/A card */
#define M_irigLib       (1031 << 16)    /* Irig time code reader card */
#define M_pztMotorLib   (1032 << 16)    /* PZT/Motor board drivers */
#define M_ccdtbLib      (1033 << 16)    /* CCD Timing Board */
#define M_photonLib     (1034 << 16)    /* Photon Binning Board */
#define M_gpibLib       (1035 << 16)    /* gpib-1014 Board */
#define M_filtWheelLib  (1036 << 16)    /* Filter Wheel controller */

/* Delay line modules */
#define M_dlStatus      (1088 << 16)    /* delay line status reporting module */
#define M_pztServo      (1089 << 16)    /* delay line pzt servo module */
#define M_tasks         (1090 << 16)    /* tasks.c servo task scheduler */
#define M_vc1Servo      (1091 << 16)    /* voice coil 1 servo */
#define M_vc2Servo      (1092 << 16)    /* voice coil 2 servo */
#define M_modeSwitch    (1093 << 16)    /* servo mode finite state machine */
#define M_motorServo    (1094 << 16)    /* motor servo */
#define M_servoDoCommCmds (1095 << 16)  /* servo's Comm CPU command processor */
#define M_dlDataSend    (1096 << 16)    /* Data recording task */
#define M_dlDoCommands  (1097 << 16)    /* Comm CPU command processor */
#define M_dispatch      (1098 << 16)    /* dispatch CPU code */
#define M_mpRngLib      (1099 << 16)    /* Ring buffer library */

#define M_genPrivate    (1100 << 16)    /* Palomar generic interface stuff */
#define M_sid           (1101 << 16)    /* Siderostat subsystem errors */
#define M_acq           (1102 << 16)    /* Acquisition subsystem errors */
#define M_track         (1103 << 16)    /* Star tracker subsystem errors */
#define M_dl            (1104 << 16)    /* Delay line subsystem errors */
#define M_ft            (1105 << 16)    /* Fringe tracker subsystem errors */
#define M_seq           (1106 << 16)    /* Sequencer subsystem errors */
#define M_ipc           (1107 << 16)    /* InterProcessor Communications lib */
#define M_ini           (1108 << 16)    /* INI-file access library */
#define M_st            (1109 << 16)    /* Star tracker subsystem errors */
#define M_globals       (1110 << 16)    /* global variable init/manip file */

/* Star tracker modules */
#define M_stStateMachine (1200 << 16)   /* Star tracker state machine */
#define M_stActions      (1201 << 16)   /* Star tracker actions       */


#endif INCrtModNumh
