/* rtTypes.h - realtime type definitions */
/* sccs info:        @(#)rtTypes.h	1.4  05 Jul 1995  */

/*
modification history
--------------------
01e,05Jul95,beh     Added macros to convert QWORDs to/from doubles/long longs
01d,26Aug91,beh     Changed (U)LWORD from int to long
01c,23May90,beh     added QWORD
01b,11May90,beh     changed LONG to LWORD, added #ifndef INC...
01a,21feb90,arg     written.
*/

/*
DESCRIPTION
This module defines types to be used by realtime programs running under VxWorks.
In particular, all memory/address references should be defined using these
types.  This will allow the types to be changed for use with other compilers
which may define char, short, and int differently.
*/

#ifndef INCrtTypesh
#define INCrtTypesh

typedef char BYTE;			/*   signed  8 bit integer */
typedef unsigned char UBYTE;		/* unsigned  8 bit integer */
typedef short WORD;			/*   signed 16 bit integer */
typedef unsigned short UWORD;		/* unsigned 16 bit integer */
typedef long LWORD;			/*   signed 32 bit integer */
typedef unsigned long ULWORD;		/* unsigned 32 bit integer */
typedef struct { LWORD high; LWORD low; } QWORD;  /* 64-bit integer */
typedef struct { ULWORD high; ULWORD low; } UQWORD;  /* unsgn 64-bit integer */

#define QWORD_TO_LONG_LONG(qword, llong) \
  llong = * (long long *) &qword;

#define LONG_LONG_TO_QWORD(llong, qword) \
  qword.high = * (LWORD *) &llong; \
  qword.low  = * ( 1 + (LWORD *) &llong );

#define QWORD_TO_DOUBLE(qword, dbl) \
  { \
    long long ll; \
    QWORD_TO_LONG_LONG(qword, ll)  \
    dbl = (double) ll;  \
  }

#define DOUBLE_TO_QWORD(dbl, qword) \
  { \
    long long ll = (long long) dbl; \
    LONG_LONG_TO_QWORD(ll, qword)  \
  }

/* The following is stolen from the gcc docs.  This is a way to eliminate
 * warnings about unused variables or functions. */
#define USE(var) \
            static void * use_##var = (&use_##var, (void *) &var)
          
#endif INCrtTypesh
