/* testItp.c - records data from the ITP delay line through 
   laser card
*/

#include <vxWorks.h>
#include <stdioLib.h>
#include <sysLib.h>
#include <taskLib.h>
#include <rtTypes.h> 
/* #include <recLibB.h> */
#include <dt1401Lib.h>
#include <irigLib.h>
#include <laserLib.h>
#include <pztMotorLib.h>
#include "recItpDlineF.h"

/* DEFINES (variables for vxWorks debugging) */
int COUNTER_NUM = 1;
double FILTER_WIDTH = 1000;
int NUM_PZTS = 4;
int NUM_MOTORS = 0;
int PZT_COUNTER_NUM = 4;

/* Variables */
QWORD qLaserPos;		/* holds entire 40-bit word from laser card */
LWORD lLaserPosHi;	/* holds data from hi part of laser card */
LWORD lLaserPosLo;	/* holds data from lo part of laser card */
int dispScaleM = 1600;	/* mult. scale factor for displaying to D/A */
int dispScaleD = 1;	/* div. scale factor for displaying to D/A */
double avg = 0.0;	/* avg value for low-pass filter for D/A display */
BOOL analogOn = TRUE;	/* TRUE: turns on analog output */

/*********************************************
*
* recItpDlineInit - Initializes clock to
*                   read from delay line
*
* This routine initializes the slow laser card,
* and sets up clock to read data from 
* the slow laser card counter at clkRate. After
* initializing, type recOpen(<filename>) to start recording.
* type recClose() to end recording.
*
* RETURNS:  OK or ERROR
*
*********************************************/

STATUS recItpDlineInit(int clkRate)
{
   int actClkRate;

   printf("Initializing...\n");

   /* initialize dt1401 card */
   if(dt1401FullInit() == ERROR)
   {
      printf("Error initializing dt1401 card\n");
      return(ERROR);
   }
   
   /* initialize irig Card */
   if(irigFullInit() == ERROR)
   {
      printf("Error initializing irig card\n");
      return(ERROR);
   }

   /* set irig clock rate */
   if((actClkRate=irigClkRateSet(clkRate)) == ERROR)
   {
      printf("Error setting irig clock rate\n");
      return(ERROR);
   }
   printf("Sample rate is %d Hz\n",actClkRate);

   /* get ready to read by connecting the interrupt to itpLaserRead */
   if(irigClkConnect(itpLaserRead,0) == ERROR)
   {
      printf("Error Connecting irig clock to interrupt function\n");
      return(ERROR);
   }

   /* enable irig interrupt clock */
   if(irigClkEnable() == ERROR)
   {
      printf("Error enabling irig Clock\n");
      return(ERROR);
   }

   if (pztMotorFullInit(NUM_PZTS, NUM_MOTORS) != OK)
   {
      printErr("Error initializing the pztMotor card\n");
      return ERROR;
   }
   pztReset(PZT_COUNTER_NUM);


   if (laserFullInit(1) != OK)
   {
      printErr("Error initializing the laser card\n");
      return ERROR;
   }
   laserClearCounter(1);


   printf("Done Initializing\n\n");

   return OK;
}  


/*********************************************
*
* itpLaserRead - interrupt fnct to read laser card
*
*
* RETURNS:  OK
*
*********************************************/

STATUS itpLaserRead(void)
{

   /*  FOR TESTING */
/*
   static WORD philstemp = 0;
   philstemp += 256;
   dt1401DA0(philstemp);
   printf("%5d",philstemp);
*/

   dt1401DA1(0x7ff);

   /* Read laser card, output to recInt.		       *
   *  Average data and output to D/A if analogOn is TRUE       */

   laserCpr(COUNTER_NUM, &qLaserPos);

   lLaserPosHi = qLaserPos.high;
   lLaserPosLo = qLaserPos.low;

   /* sign-extend lLaserPosHi */
/*
   if(lLaserPosHi & 0x00000080)
   {
      lLaserPosHi |= 0xffffff00;
   }
   else
   {
      lLaserPosHi &= 0x000000ff;
   }
*/

   /* record data via recInt 
    *  to record LS32-bits, comment out
    *  recInt(lLaserPosHi)  */

/*   recInt(lLaserPosHi);
   recInt(lLaserPosLo);
*/

   if (analogOn)
   {
/*
      avg = (avg * (FILTER_WIDTH - 1.0) + lLaserPosLo) / FILTER_WIDTH;
      pztPositionSet(4,dispScaleM*(lLaserPosLo - avg)/dispScaleD);
*/
      dt1401DA0(dispScaleM*(lLaserPosLo)/dispScaleD);
   }

   dt1401DA1(0x0);

   return OK;
}
