/* timingTest.c - a combination of dt1401Lib and recItpDlineF for testing the
                  interrupt problems seen with the laser card.  When a second
                  intConnect is done, the delay disappears.
*/

/* dt1401Lib.c - routines to access dt1401 i/o card */
/*		sccs info:	@(#)dt1401Lib.c	1.5  9/1/94  */

/*
modification history
--------------------
01k,01Sep94,mmc    added dt1401IsPresent
01j,20Jan94,hs     changed second digital port to be an input
01i,19nov93,mmc    added vxMemProbe check
01h,11Oct93,mmc    added global dt1401ADData
01g,14Sep93,beh    made compatible with VxWorks 5.1
01f,31Aug91,mmc    Added some includes, commented out some unused vars,
				added VOIDFUNCPTR casts
01f,16Jul91,grw    Added dt1401FullInit function
01e,08May90,beh    Added prototypes in dt1401Lib.h, WORD etc., range check
01d,29Jan90,beh	Moved the various defines off to dt1401Config.h
01c,20Dec89,mmc	most user routines now return STATUS, unitialized
				global pointers now point somewhere. 
01b,01Dec89,mmc	changed a/d gain
01a,26Oct89,mmc	written	
*/

/*
DESCRIPTION
This module contains subroutines to interface with the data
translation dt1401 analog/digital i/o card.
*/

#include <vxWorks.h>
#include <taskLib.h>
#include <vme.h>
#include <vxLib.h>
#include <stdio.h>
#include <sysLib.h>
#include <intLib.h>

#include "rtTypes.h"
#include "laserLib.h"

/* Variables */
QWORD qLaserPos;		/* holds entire 40-bit word from laser card */
LWORD lLaserPosLo;	/* holds data from lo part of laser card */

/* Function Prototypes */
STATUS recItpDlineInit(void);
void itpLaserRead(void);

/*********************************************
*
* recItpDlineInit - Initialized irig clock to
*                   read from delay line
*
*
*********************************************/

STATUS recItpDlineInit(void)
{
   printf("Initializing...\n");

   /* set dt1401adcsr */
   *((WORD *)0xefff7f00) = 0x148;
   *((WORD *)0xefff7f04) = 0;
   *((WORD *)0xefff7f04) = 0x0013;

   /* connect and enable interrupt */
   intConnect((VOIDFUNCPTR *) 0xc0, (VOIDFUNCPTR) itpLaserRead, 0);
   intConnect((VOIDFUNCPTR *) 0xc0, (VOIDFUNCPTR) itpLaserRead, 0);
   sysIntEnable(4); /*enable this bus interrupt level */

   /* enable dt1401 clock */
   *((WORD *)0xefff7f04) |= 0x0040;

   taskDelay(120);

   if (laserFullInit(1) != OK)
   {
      printErr("Error initializing the laser card\n");
      return ERROR;
   }
   laserClearCounter(1);


   printf("Done Initializing\n\n");

   return OK;
}  


/*********************************************
*
* itpLaserRead - interrupt fnct to read laser card
*
*
* RETURNS:  OK
*
*********************************************/

void itpLaserRead(void)
{
  WORD dummy;

   /* acknowlege interrupt */
   dummy = (WORD) *((WORD *)0xefff7f06);


   laserCpr(1, &qLaserPos);
   lLaserPosLo = qLaserPos.low;
   *((WORD *)0xefff7f0a) = lLaserPosLo;
}
