#include <vxWorks.h>
#include <intLib.h>
#include <semLib.h>
#include <sysLib.h>
#include <stdio.h>
#include <taskLib.h>
#include <iv.h>

int iVecA = 0xa0;
int iLevelA=7;
int iVecB = 0xa1;
int iLevelB=1;
SEM_ID sSemA, sSemB;

int iChanA = 0;
int iChanB = 2;

unsigned int *pBoard    = (unsigned int *)0x81220000;

unsigned int *pCsr      = (unsigned int *)0x81220000;
unsigned int *pRollover = (unsigned int *)0x81220004;
unsigned int *pTime     = (unsigned int *)0x81220008;
unsigned int *pTimediv  = (unsigned int *)0x8122000c;
unsigned int *pWatchdog = (unsigned int *)0x81220010;

STATUS setClock(int iWhichClk);
STATUS showCsr(void);
STATUS showTime(int iNumTimes);
STATUS setWatch(unsigned int uWatchTime);
STATUS syncOff(void);
STATUS syncOn(void);
STATUS syncNow(void);
STATUS syncNext(void);
STATUS syncNext2(void);
STATUS setTick(unsigned int uTickSize);
STATUS setRover(unsigned int uRollover);
STATUS timSetup(void);

#define SYS 0
#define EXT 1

STATUS setClock(int iWhichClk)
{
   switch (iWhichClk)
   {
      case SYS:
         printf("Setting Clk to SYSCLK\n");
	 *pCsr &= 0xfffffffe;   // clear bit 0
         return OK;
      case EXT:
         printf("Setting Clk to EXT_CLK\n");
	 *pCsr |= 0x1;          // set bit 0
         return OK;
      default :
	 printf("Invalid Clk to Set\n");
	 return ERROR;
   }
}

STATUS showCsr(void)
{
   printf("CSR: 0x%08x\n",*pCsr);
   return OK;
}

STATUS showTime(int iNumTimes)
{
   int i;

   for (i=0; i<iNumTimes; i++)
   {
      printf("Time: 0x%08x\n",*pTime);
      taskDelay(60);
   }
   return OK;
}

STATUS setWatch(unsigned int uWatchTime)
{
   printf("Setting watchdog timer to 0x%x\n",uWatchTime);
   *pWatchdog = uWatchTime;
   return OK;
}

STATUS syncOff(void)
{
   printf("Syncing: off...\n");
   *pCsr &= 0xfffffffd;
   return OK;
}

STATUS syncOn(void)
{
   printf("Syncing: on...\n");
   *pCsr |= 0x2;
   return OK;
}

STATUS syncNow(void)
{
   printf("Syncing now...\n");
   *pCsr |= 0x4;
   return OK;
}

STATUS syncNext(void)
{
   printf("Syncing on next EXT_SYNC...\n");
   *pCsr |= 0x6;
   return OK;
}

STATUS syncNext2(void)
{
   printf("Syncing on next EXT_SYNC...(2-step)\n");
   syncOn();
   syncNext();
   return OK;
}

STATUS setTick(unsigned int uTickSize)
{
   printf("Setting tick size to 0x%x...\n",uTickSize);
   *pTimediv = uTickSize;
   return OK;
}

STATUS setRover(unsigned int uRollover)
{
   printf("Setting time rollover value to 0x%x...\n",uRollover);
   *pRollover = uRollover;
   return OK;
}


STATUS timSetup(void)
{
   printf("Setting up chip...\n");
   taskDelay(60);
 
   *pCsr = 0x10;
   *(pBoard + 0x20 + iChanA) = 0x8000001fU;
   *pTimediv = 0x0;
   *pRollover = 0x0;

   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   
   *pRollover = 0x100;
   taskDelay(60);

   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
 
   *pTimediv = 0x00ffffff;
   taskDelay(60);

   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
   taskDelay(60);
   printf("Time is 0x%X\n",*pTime);
 

   return OK;
}

